/*
Midbar
Distributed under the MIT License
© Copyright Maxim Bortnikov 2023
For more information please visit
https://sourceforge.net/projects/midbar/
https://osdn.net/projects/midbar/
https://github.com/Northstrix/Midbar
Required libraries:
https://github.com/zhouyangchao/AES
https://github.com/peterferrie/serpent
https://github.com/ddokkaebi/Blowfish
https://github.com/Northstrix/DES_and_3DES_Library_for_MCUs
https://github.com/ulwanski/sha512
https://github.com/moononournation/Arduino_GFX
https://github.com/techpaul/PS2KeyAdvanced
https://github.com/intrbiz/arduino-crypto
https://github.com/GyverLibs/EncButton
*/
/*******************************************************************************
 * Start of Arduino_GFX setting
 *
 * Arduino_GFX try to find the settings depends on selected board in Arduino IDE
 * Or you can define the display dev kit not in the board list
 * Defalult pin list for non display dev kit:
 * Arduino Nano, Micro and more: CS:  9, DC:  8, RST:  7, BL:  6, SCK: 13, MOSI: 11, MISO: 12
 * ESP32 various dev board     : CS:  5, DC: 27, RST: 33, BL: 22, SCK: 18, MOSI: 23, MISO: nil
 * ESP32-C3 various dev board  : CS:  7, DC:  2, RST:  1, BL:  3, SCK:  4, MOSI:  6, MISO: nil
 * ESP32-S2 various dev board  : CS: 34, DC: 35, RST: 33, BL: 21, SCK: 36, MOSI: 35, MISO: nil
 * ESP32-S3 various dev board  : CS: 40, DC: 41, RST: 42, BL: 48, SCK: 36, MOSI: 35, MISO: nil
 * ESP8266 various dev board   : CS: 15, DC:  4, RST:  2, BL:  5, SCK: 14, MOSI: 13, MISO: 12
 * Raspberry Pi Pico dev board : CS: 17, DC: 27, RST: 26, BL: 28, SCK: 18, MOSI: 19, MISO: 16
 * RTL8720 BW16 old patch core : CS: 18, DC: 17, RST:  2, BL: 23, SCK: 19, MOSI: 21, MISO: 20
 * RTL8720_BW16 Official core  : CS:  9, DC:  8, RST:  6, BL:  3, SCK: 10, MOSI: 12, MISO: 11
 * RTL8722 dev board           : CS: 18, DC: 17, RST: 22, BL: 23, SCK: 13, MOSI: 11, MISO: 12
 * RTL8722_mini dev board      : CS: 12, DC: 14, RST: 15, BL: 13, SCK: 11, MOSI:  9, MISO: 10
 * Seeeduino XIAO dev board    : CS:  3, DC:  2, RST:  1, BL:  0, SCK:  8, MOSI: 10, MISO:  9
 * Teensy 4.1 dev board        : CS: 39, DC: 41, RST: 40, BL: 22, SCK: 13, MOSI: 11, MISO: 12
 ******************************************************************************/

#include <Arduino_GFX_Library.h>

#define GFX_BL DF_GFX_BL // default backlight pin, you may replace DF_GFX_BL to actual backlight pin

/* More dev device declaration: https://github.com/moononournation/Arduino_GFX/wiki/Dev-Device-Declaration */
#if defined(DISPLAY_DEV_KIT)
Arduino_GFX *gfx = create_default_Arduino_GFX();
#else /* !defined(DISPLAY_DEV_KIT) */

/* More data bus class: https://github.com/moononournation/Arduino_GFX/wiki/Data-Bus-Class */
Arduino_DataBus *bus = create_default_Arduino_DataBus();

/* More display class: https://github.com/moononournation/Arduino_GFX/wiki/Display-Class */
Arduino_GFX *gfx = new Arduino_ILI9341(bus, DF_GFX_RST, 0 /* rotation */, false /* IPS */);

#endif /* !defined(DISPLAY_DEV_KIT) */
/*******************************************************************************
 * End of Arduino_GFX setting
 ******************************************************************************/

/* more fonts at: https://github.com/moononournation/ArduinoFreeFontFile.git */

const uint16_t Dallas PROGMEM [320][241] = {
{23900,25916,25947,27996,27996,27996,28028,28028,30076,30108,30108,30108,30108,30108,30108,30108,30108,30140,30140,28092,30108,30140,30140,30172,30140,32220,32220,30172,30173,32188,34269,34301,34301,34301,34333,34301,32221,32252,32253,34301,38428,42588,46684,48764,48796,48796,48764,48797,48829,48829,46749,42653,44701,40573,38493,38493,40541,44636,44636,44636,42588,44603,46683,50844,52956,50908,50877,52989,55101,57149,57149,57149,57149,55069,57117,53021,46813,44733,44733,44733,46781,46781,44765,46781,48861,50941,53021,50941,48893,46813,48861,48861,48861,50940,50908,50876,50876,50908,52988,59197,59196,57084,55003,52955,50875,50843,50843,52923,52923,52956,52956,52923,57115,52923,50843,48763,48795,48795,46715,42554,38425,30104,23830,13265,4814,4749,4717,2604,2506,2505,2441,2441,327,2407,295,294,262,262,294,2343,2375,2374,4422,2374,2341,2342,4422,2342,2373,6567,8615,6535,10696,4423,4390,4423,8648,12938,10826,15051,17066,19211,19179,17066,14986,14953,17002,19212,19212,19114,21259,12775,8614,8613,8581,14888,10662,21291,17066,46517,44271,31790,48664,46584,46616,27502,17132,31727,14888,16968,35952,25389,21163,23243,23276,25486,38099,25454,17035,19147,38099,52627,33318,33286,33318,35366,35367,42195,25062,12482,12548,35986,57116,56984,44372,33808,38131,33775,25518,44404,29679,36050,33839,23470,40276,33807,29711,48663,48369,10760,12871,10660,4354,4322,2210,6566,19180,21163,21228,16741,59},
{25916,25916,25948,25948,25948,27996,30076,28028,30076,30108,32188,32188,30108,30140,30140,30108,30140,30140,30108,30140,30140,30140,30140,30140,32188,30172,32221,30172,30172,32220,32252,32252,34300,34301,34301,34301,32220,32253,30205,34301,34333,38460,42556,46684,46716,48796,48796,48797,48829,48829,46781,44701,42653,38493,38493,38493,40541,44636,44636,42589,42588,44636,46683,50844,52924,50876,50876,52989,55069,57149,57149,57149,57149,57117,57149,52989,46813,44733,44733,44733,46813,44733,44765,46813,48893,50941,52989,50941,48893,48893,48861,48861,48828,50908,50876,50876,50876,48795,50908,57148,59228,57116,52955,52923,50843,50843,50843,50875,52923,50875,50875,50843,57116,50875,48795,40474,32153,25879,23766,21782,23895,21782,19702,23797,15182,6731,6699,2570,2472,2440,2439,359,2407,2407,358,358,2375,2342,2407,2407,2439,2439,4519,4454,2406,4454,4454,4486,4486,4519,6632,8680,8680,4519,4454,4454,6600,6600,4422,2342,4486,10825,10825,4389,2244,2276,2341,6600,4519,10793,10761,8681,10793,8680,6599,8647,4356,8614,12841,48663,48531,31821,48664,46584,46616,27533,19147,38066,25354,29679,27534,21195,21195,23308,23276,27501,33873,21097,12711,23308,44503,56887,33318,33286,33286,33351,33351,37676,22884,16610,35790,50711,57083,57017,46452,44241,42259,40015,31597,46485,35789,37968,39982,27370,40146,39917,29451,42389,50514,18918,12741,6434,2273,4289,4290,4323,6436,12776,19017,12579,124},
{23899,23900,25948,25947,25979,28028,28028,30108,30108,30108,32188,32188,30140,30140,30108,30140,30140,30140,30140,30108,30140,30140,30140,30140,32188,32220,32220,32220,30140,34301,32252,32220,36349,36381,36349,36381,32253,32252,32253,34301,32285,36413,42556,44636,46716,48764,48829,48829,48829,48829,46781,46749,44701,38493,38493,38461,40541,44636,44636,42589,42589,44636,46683,50844,52956,50876,48828,52957,55069,57149,57149,55101,57149,57149,55069,50941,46813,44733,44733,46781,46781,44733,44765,46813,48893,50941,50941,50941,48893,46813,48861,48828,50876,48860,52956,52956,50876,50876,50875,55036,59228,59196,55035,50843,50843,52923,50843,50843,52923,50843,46715,38394,36345,27991,21686,19605,19605,19638,19637,21717,25943,21750,21750,21750,23700,10924,6698,424,2473,2408,2407,391,359,326,326,2375,2374,294,2343,327,327,2407,2407,2406,2374,2406,2373,4518,2406,4519,4519,6568,8648,2374,6535,2374,4519,6600,2342,2309,4454,8713,10761,2276,164,2276,2276,6600,6568,10794,10794,8745,10794,6600,6567,10727,4389,8647,14986,44469,50644,31788,48696,46584,46584,27533,29712,19310,19211,40146,25388,21195,25356,27502,25356,27501,33873,23275,14759,23341,48729,59065,35430,33286,33286,33318,33319,33318,33351,33449,54938,57083,55067,59131,46452,44372,40211,42259,33970,42356,42259,40244,46484,40211,40276,46452,40211,27631,35985,21226,10690,4385,2273,4289,4257,2145,8452,8387,8386,10531,126},
{23899,23867,25948,25948,25980,28028,30076,34236,32156,32156,34268,34236,32188,32156,30108,30140,30140,30140,30140,30140,30140,30140,30140,32220,32220,32220,32220,32220,32220,32252,32252,32252,34301,36381,38429,36381,32253,32252,32253,34301,34301,34333,40508,44636,46684,48764,48796,48797,48829,48829,48829,46781,44701,40541,38493,38493,38493,42589,42588,40573,42589,42620,46683,48763,50876,50876,50844,52956,55069,57149,57149,57149,57149,57149,55069,48861,44733,44733,44733,46781,48829,48829,50909,50909,48861,50941,50941,48893,48893,48861,48861,50876,50876,50876,50908,52956,48828,48795,50843,55068,59196,59196,57083,52923,50843,52923,52891,50875,52924,50843,34135,17557,19637,17525,17557,17557,19637,19638,19637,21782,25943,21749,19734,25943,23830,21619,8843,2473,392,2407,391,359,2374,2342,2342,294,2342,2310,2310,294,326,2407,2407,2374,2406,2374,2373,2373,326,6567,6535,6535,4520,2342,4454,2309,4454,6567,2309,2309,4422,6567,4454,164,164,2244,2276,6600,6503,8681,10793,10793,10761,2309,2342,6567,2341,10791,12872,46583,50710,35917,46616,44535,46584,38098,19148,12841,25453,38033,23276,23243,25388,31727,23308,25453,35953,23243,16839,19115,55003,61275,35496,31271,29157,31206,31238,31206,31272,31532,57083,52890,46518,57083,46452,44339,33775,44306,18854,29711,33676,19082,44339,16838,19115,42226,10692,6467,6468,4353,6497,6401,2241,10694,4290,2177,6403,8386,6306,18951,121},
{21851,25916,25948,25948,25979,28028,30140,36316,34236,34268,32220,32188,32188,32188,30140,32188,30140,30140,30140,30140,30140,32188,32188,32220,32220,32220,34268,32220,30172,32220,32220,32252,34301,34301,38429,38429,32253,34301,32253,32253,34301,34333,36413,42556,46684,46684,46716,46716,46749,46781,46781,46749,44701,40541,38493,36445,38493,40541,42621,40573,40573,42621,46716,48763,50876,50876,48828,52956,55069,57117,57149,57149,57149,57149,50973,46781,44733,44733,46781,52989,55037,55069,55068,52956,48861,50909,50908,48860,50876,48828,48828,50876,50876,50876,48828,50876,48795,48763,50843,55036,59164,59196,57083,52923,50843,52923,50843,50875,50875,50843,32055,17557,19605,19605,19605,19605,17557,19605,19605,19701,23862,19669,19637,19669,21718,17557,2440,230,197,229,229,229,197,197,196,196,196,197,197,197,196,197,197,197,196,228,196,229,229,229,229,229,229,2341,326,2309,2342,2342,261,2342,2342,2309,2276,2245,2212,2276,4421,6568,6535,6600,6601,6632,4487,196,196,2341,2308,4420,12938,36084,44469,40078,44535,44503,46616,21260,12678,10630,38099,29582,21195,23276,33808,31695,25356,27469,29549,25258,12678,16969,59196,61275,31270,29223,27013,29093,29093,27013,29093,29452,55003,52922,46518,55067,50613,46517,46517,46485,46419,46485,48500,46452,46517,46387,46387,44305,12706,6434,4353,4353,6433,4321,4289,4257,129,2241,2274,8484,6274,10467,78},
{23899,23899,25948,25948,25948,28028,28060,32156,32156,32188,32220,32188,30140,32188,32188,32188,30140,30140,30140,30140,30140,30140,30140,32221,32220,32220,34301,32221,32220,32221,32220,32253,32253,34301,36381,36381,32253,32253,32253,32253,34301,34333,36413,40541,44636,46683,46716,46716,46749,48829,46781,46781,44701,38493,38493,38493,38493,40541,42589,40573,40573,40573,46716,48764,50844,50876,48828,52957,55069,57149,57149,57149,57149,55069,50941,46781,44733,46781,50909,55036,57149,57149,55069,52956,50908,50876,50876,50876,50876,50876,50876,50876,50844,48796,48795,50843,50843,50843,48763,52956,59196,59196,57116,52923,48763,50843,50843,50843,50875,48795,32086,17557,17557,19637,19637,15509,15509,19637,17524,17589,19669,19605,19605,21685,21685,6797,229,196,197,196,196,196,196,196,196,164,164,164,164,164,164,164,164,132,132,164,164,164,164,164,164,164,196,2341,261,2309,2309,261,261,261,2277,2277,229,2277,164,4422,6567,4455,6535,8680,8648,2276,163,131,163,2276,2276,8677,8646,8679,21390,44337,44535,46583,31857,17001,14888,14791,38099,25389,23243,25356,33840,21163,23276,27469,27371,23210,12646,19212,59261,57016,31205,27143,27045,27012,27012,27012,27012,27144,48697,52792,29679,38197,54871,44372,31694,38098,27436,17100,44372,19114,29809,29613,19180,33871,12706,6434,4353,10563,4353,4321,4289,2241,2241,4321,2210,6469,6273,8322,64},
{23899,23867,25916,25947,25980,25980,30076,30076,30108,30108,32188,32188,32188,32188,30140,30140,32188,30140,30140,30140,30140,30140,32220,32220,32220,32220,34300,34300,32220,32220,32220,34269,34301,34301,36413,38429,34301,32253,34301,32253,32253,34333,36413,42589,46749,46748,46716,46716,46749,48829,48829,46781,42653,38493,38493,38493,38493,40541,40541,40573,38525,40573,44668,48764,48796,48828,50876,52957,55069,57149,55101,57149,55101,53021,48861,46781,50877,52989,57116,57117,57149,55069,55036,50908,48828,48796,48828,50876,50876,52956,52924,50876,50876,48795,48763,48795,50843,50843,48763,50843,55036,57116,57084,52923,48763,50843,50875,50843,50843,48795,32086,19573,17525,15509,19605,17557,17589,17589,15444,17557,21717,19604,19604,19605,17524,2472,2342,229,2245,229,2245,197,196,196,196,196,2212,196,196,2244,164,196,196,164,2244,2212,196,228,164,164,2277,2277,2309,2309,2309,2309,2277,229,2309,2309,2277,229,229,228,164,2309,2309,4422,4454,10728,8648,4356,131,131,163,4323,4324,6468,6468,4356,12840,38130,44503,38164,21227,14953,14856,23178,31760,25421,23243,25389,29549,21130,25388,25388,27370,25258,14693,16937,42390,54740,31205,27111,27012,27012,26980,24932,26980,27013,42260,48565,23145,42324,52856,44372,44241,44307,42128,37902,46517,37837,38065,42127,31596,31595,10594,6434,4290,12742,6433,6434,4256,2209,2273,2273,2209,2210,6273,10402,82},
{23900,25948,23900,25948,27996,28028,28028,28028,28060,30108,32156,32188,32188,32188,30140,32188,32188,30140,30140,32188,32188,30140,32220,32220,32220,32220,34300,32220,32220,32220,32220,32253,32220,34301,36381,38429,32285,32253,32253,32253,34301,34333,36413,40541,48797,50909,48829,48829,48829,48861,46781,46749,42621,38493,36445,38493,38493,40541,40541,40573,38525,40573,44669,46716,48764,50876,52957,52989,55069,55101,57149,57149,55101,53021,46813,48829,55037,57117,59197,57149,55069,52989,52956,50876,48796,48796,50876,52956,52956,52924,50876,50876,50876,48795,48763,50843,50843,50843,50811,48763,52956,57116,57084,52923,48763,50843,50843,50843,48795,48795,34134,17589,17525,17556,17557,19605,17557,19605,17524,21717,23862,19604,19604,19605,2506,2374,2342,229,2277,196,197,2276,196,196,2276,2244,164,196,2244,2244,196,2244,2244,2244,2212,2244,228,2309,163,2244,2277,2309,2309,4390,4357,2277,2276,2309,2309,229,2245,4325,2276,2244,2244,6469,2341,4422,6567,8680,2406,2244,131,2243,4452,6499,6500,8548,10660,4387,17033,35919,38229,27599,25452,21291,23210,35921,25356,23276,23308,29614,21065,25389,23308,25389,29613,35888,21064,14823,52857,58933,41736,25030,27044,27110,27044,22949,29157,22949,48632,56952,44209,42291,50711,46387,40211,36082,46452,31857,42357,40178,36082,44404,33970,36017,29514,10626,4321,6402,6466,6466,6434,6433,6433,4321,2209,4290,16675,14562,79},
{23867,25916,25948,25948,25948,25980,25980,28028,28028,30108,30140,32188,32188,30172,32188,32188,32188,32188,32220,32220,34268,32220,34268,34300,34300,34300,34301,34301,32221,32252,32220,30173,32252,32253,36381,36413,34333,32253,32253,34301,32253,34333,36413,42589,46749,48861,46781,46781,48829,48861,44733,44701,42621,38493,38493,36445,38493,38493,38493,38493,40541,38525,42621,44668,48764,50909,52989,53021,55069,55101,57117,55101,55069,50941,46781,48829,55069,59197,57149,55069,52988,52956,48828,48796,48796,50876,50876,52956,52956,50876,52923,50876,50844,50843,48795,52891,50843,50875,52891,50811,50843,55036,55035,50843,50811,52923,50843,50843,48795,50843,32022,17525,17589,17557,17557,17557,17557,17557,17556,21781,21750,17556,19605,6894,2407,2375,294,261,2309,2277,229,2277,2245,2276,2276,2277,2244,2276,2244,2244,196,228,4292,2276,196,2244,2244,2276,2243,2276,2276,2276,2308,4325,2277,2277,2277,4389,2341,2309,2277,6437,4324,2276,2244,4421,6534,6535,8648,8680,2276,131,163,2307,6531,6499,6499,2275,4419,6500,10726,14918,8711,10760,12905,12906,42389,40179,25356,23276,25388,35921,21163,17002,25389,25421,27436,27469,14758,14856,59228,65469,54675,29223,27143,25290,25127,14727,29385,12646,55036,55003,50710,33873,48630,48499,38001,29614,46452,20999,33905,37902,19081,42291,16838,14855,23240,10594,6402,12708,10626,8546,6401,8578,8514,4353,2241,16968,41967,24802,70},
{23868,23900,25948,25948,25948,25980,25980,28028,30108,30108,32188,32188,30140,30140,32188,32188,32188,32220,32220,34268,32220,32220,34300,36380,36348,36348,36349,34301,32220,32252,30204,32252,32253,34301,36381,38461,34333,32253,34301,34301,34333,34333,36413,40541,46749,48861,46781,44733,46781,46781,44701,42653,40573,38493,36445,36445,36445,38493,38493,38493,38525,40573,40573,44668,46748,50876,53021,55069,55069,55101,55101,55101,53021,50909,46781,48829,52989,55101,55069,52989,50908,50908,46813,46781,48796,48796,48796,48796,48796,48796,50876,50876,50843,50843,48795,50843,50843,50843,52891,50843,50843,52955,55036,50843,50843,50843,50843,50843,50843,50843,32054,17557,15509,19573,17557,17524,17524,15509,17556,19669,21717,21653,15347,2440,359,326,326,2342,2276,229,261,2277,229,229,228,2276,2244,2276,2276,2244,2244,228,2276,2276,2244,196,2212,196,196,2244,229,2277,228,228,2276,2277,2309,4390,2342,2309,2277,2309,4324,4357,2244,4421,4453,6567,8680,8680,2276,163,163,4420,6499,4419,6467,2275,6499,6499,4452,8644,6499,8580,6436,8583,40245,29647,21195,23243,25388,25421,8485,10696,27469,27502,27371,25290,12613,23341,63421,65469,61079,33384,24965,25193,25128,16839,29386,21065,57148,46486,38099,44274,48598,50678,48532,48565,48598,50580,46518,48565,48532,46517,46386,44305,33772,21093,10659,6467,8579,8611,6467,12708,8482,10563,8482,19048,33578,35333,92},
{23867,23899,23900,25947,25948,25948,28028,28028,30076,30108,32188,30140,32188,32188,32188,30140,32188,34268,34268,32220,34268,34300,36348,38428,38428,36381,36381,34301,32220,32220,32220,32252,32253,34300,36381,38461,36381,32253,32253,34301,34301,34333,36413,40541,46781,48861,44702,44701,44701,44702,42653,42621,40573,38493,36445,38493,38493,38493,38525,38525,38525,40573,40573,44669,48764,50908,53021,53021,55069,55069,55069,55069,53021,48861,46813,44733,46813,48861,48861,48829,50909,48861,46813,46813,48829,48828,48829,48861,48828,48796,50876,52924,50876,50844,50844,50843,50843,50811,50875,50843,50843,50875,52956,52891,50843,50843,50843,50843,50875,52923,32054,17525,17524,17492,15476,17525,15509,15477,17524,19637,23927,23895,6796,4617,6665,4552,6567,6600,4455,2439,2439,2406,358,2374,326,2406,4454,4454,6503,6534,6535,4422,4486,6534,4454,4390,4357,4421,4357,4357,2374,4422,4422,2309,2374,4454,4487,4552,4551,4422,2374,4422,4422,6567,2276,4486,4420,4487,6632,6567,196,195,2211,2372,6500,6436,2339,4387,6500,16966,6531,10724,6467,8580,19048,29484,44340,25388,21163,21195,25389,27436,23080,19049,27501,27469,27371,25290,12645,33905,63389,65501,65403,35528,29288,29320,31368,27241,48598,56985,59196,38001,36019,54937,52792,50743,44470,38131,42389,44437,38099,44437,46485,40179,44437,44405,38066,31658,10691,6466,8579,8546,8579,10661,18982,8418,10691,23305,39949,31301,82},
{23867,25915,25916,25948,25948,25948,25980,28028,28028,30108,32188,32188,32188,30140,30140,30140,32188,32220,34268,32220,34300,36381,36381,40509,42557,38461,36413,34301,32221,32220,34301,32252,32252,34301,36380,38461,36381,34301,32285,32253,34333,34301,36413,40541,46749,48861,42653,42621,44669,42653,42621,40573,40573,38493,36445,38493,38494,38493,38525,40573,40573,40573,40573,44701,46716,52957,52989,55069,55069,55069,55069,53021,50941,48861,46813,44733,44733,46781,44733,46813,50941,48893,46814,46813,46813,46813,46813,48861,48861,50908,52924,52956,50876,50843,50843,50843,50843,48763,50843,50843,50843,50843,50843,50875,50843,50875,50843,50843,50908,52923,34135,17460,15444,17524,17525,17525,15476,17524,19573,19669,23895,9105,8876,8876,8844,4618,4650,2569,2537,2504,2504,2471,2503,423,391,2438,2503,4519,6600,6665,6633,4519,6632,8680,6632,4519,6599,6632,4486,4454,2406,4454,6632,2374,4519,6632,2471,4552,2536,2439,6632,4487,2439,2438,2341,8744,14822,8583,8615,4389,163,163,163,163,4453,6533,4420,4452,10694,23274,8643,10724,6467,6467,17001,31793,35986,23373,21195,21163,25356,29484,29549,27534,25388,27469,29549,27371,21032,57050,61308,63421,65501,41867,29322,23080,33709,16839,48730,57083,59196,35888,40245,54970,46419,25421,23242,18984,23210,23210,19049,21162,23275,19049,23210,23242,16969,19015,10659,8546,4322,6498,8579,8547,17000,8418,14659,33350,33480,27236,64},
{25915,25916,25948,25948,25948,25980,25980,28028,25980,30076,30108,32188,32188,30140,30140,30140,30140,30140,32220,32220,36349,40509,38461,40541,44669,42621,40509,36381,32220,32220,32252,34268,34269,32252,36381,36413,36381,34301,34301,32285,32285,34333,36413,40509,46749,48829,42653,42621,44669,42653,40573,40541,40541,36445,36445,38493,38493,36445,38493,38525,38525,40573,40573,44701,48797,50941,50941,53021,55069,55069,53021,50941,50941,48861,48861,44733,44733,44733,44765,48861,52989,48893,44765,46813,46813,48893,48861,48861,48861,52956,52956,50876,48795,50843,50843,50875,50843,48763,50875,50875,52923,50843,50843,50875,50875,52923,50875,50875,50876,52924,34134,17427,15476,15476,15477,17525,17524,19573,17557,17589,13363,6829,8941,8941,10989,4715,587,2602,2569,2569,4584,6665,2536,2504,2472,2504,2536,4584,6729,6729,4552,4552,6600,6697,6664,4519,6567,4584,4519,4519,4486,6567,6697,2438,4551,4551,2439,2504,2503,423,4584,2438,391,4519,4551,19245,23338,8615,4519,8615,2243,163,131,2309,8647,2308,6501,4388,12872,35820,16900,10756,8515,6403,23274,33808,25389,21195,21163,23243,25389,29581,35888,27404,25388,27501,33873,29549,31760,63389,54937,59261,65501,46125,29224,23112,29517,18985,55003,57051,61309,42129,44437,55002,52693,37902,40015,37902,37902,37902,35789,35822,33709,33644,33677,31531,25289,25255,12739,6466,6402,8547,6466,6466,6435,8386,22916,16773,33872,23238,52},
{25916,25915,23900,25947,27996,25948,25980,28028,25980,28028,30108,32188,32188,32188,30140,30140,30140,32188,32220,34300,38428,40509,42589,42589,44669,44669,40541,36381,34268,34269,34301,34301,34300,32252,36349,36413,36381,36349,34301,34301,34333,34333,34365,38493,44701,46781,42621,42621,42621,42621,40573,38493,38493,36445,36445,36445,38493,38493,38493,38525,38525,40573,40573,44701,50909,50909,50941,52989,53021,53053,53021,48893,48861,48861,48861,42685,44733,44733,46781,48893,53021,48893,44765,46813,48861,48861,50909,52957,52956,52956,52956,50876,48763,50811,50843,50876,50843,48763,48763,48763,50843,50843,50875,52923,52923,50843,50843,50875,50875,52955,32054,15380,15476,13428,13428,13428,15476,17524,17556,17589,6894,6828,8876,8876,6828,4715,554,2537,2537,456,456,2504,2472,2440,391,2439,2472,2471,4551,2472,4487,4487,2439,4487,6535,4487,4454,4487,4454,4454,2374,2406,2472,2374,2439,4519,325,2471,2471,2439,4584,2341,2406,4552,2568,19374,21226,4454,6600,8744,4389,2211,163,229,6600,2308,6534,6564,12839,31791,33771,12772,8547,8548,14985,21260,23276,21195,25356,23243,25388,33808,40179,23276,27469,27502,33841,31695,44470,54969,46550,57115,65501,52464,29321,29419,31369,33743,57116,54872,50711,50612,44372,52857,54904,31792,29549,21130,23275,25355,21162,23276,25388,23243,23308,25388,23307,25354,12739,6466,8546,6531,6465,6466,8548,10467,12514,14563,16935,12803,44},
{23900,23899,21852,25948,23900,25948,25980,25980,25980,28028,30108,32188,32188,32188,32188,30140,32188,30140,32188,34300,38429,40509,42589,42589,44669,44669,42621,38461,34301,34268,32220,34301,34301,32253,36381,38429,36381,36381,34333,32285,32285,34333,36381,38493,44701,46781,42621,42621,42621,38525,40573,38493,38494,36445,38493,38494,38493,38493,38525,40573,40573,40573,40605,44701,50942,50941,50909,50941,52989,53021,50941,48861,48893,48861,46813,44734,42685,44765,44765,50941,50941,48893,46813,46813,48861,50909,52988,57117,57116,55004,52924,50876,50811,50811,50843,50875,50843,48763,50811,50843,52923,50843,50843,52923,52923,50875,50843,48795,48795,50875,32022,17395,15476,15444,17492,15444,15444,15477,17524,9073,6796,6796,6796,6731,4651,4715,2506,424,424,392,392,360,391,359,327,326,359,359,4487,2407,2374,2374,2374,2374,2374,2374,2374,2374,2374,2341,261,261,326,2342,2439,4552,2342,2374,2471,2471,4616,293,2439,4551,2438,25584,19112,6535,6535,10760,4421,195,2211,196,2309,2243,4484,6532,8646,16969,14920,8644,8580,12773,10628,19147,23243,21195,31695,27436,25421,33873,27534,21228,23308,29582,33841,31760,52825,54937,50776,50744,61373,56787,31369,31305,33351,33581,57149,56918,50645,52660,46452,52857,57017,31432,31531,29385,29418,29417,27272,25256,27305,25160,25192,25192,23079,23111,16900,12771,10626,4353,6466,6466,4322,6306,4193,10466,14787,12771,39},
{25916,25947,23900,25948,23900,25948,25948,25980,28028,28028,30108,30140,32188,32188,30140,30140,30108,30140,32188,34268,36381,40509,42589,42621,44669,44669,42589,36413,34301,34268,32252,34301,32252,34301,34333,38428,40508,36381,34333,32285,34333,34333,36413,38493,44701,44733,42653,40541,40541,40541,38493,40541,38493,38461,36445,38493,38494,38493,38525,40573,38525,40573,40605,44733,52989,52989,48861,50941,50941,48893,50941,48861,48893,48893,46813,44733,42685,44733,44765,48861,50941,46814,46814,48861,52989,52989,57117,59197,57149,55036,52956,50876,50844,48763,48763,50843,50843,50811,50843,50875,50843,52891,48795,52923,52923,52923,50843,50843,48795,50844,32023,17427,15444,15476,17524,17492,17492,17492,15379,4716,8876,8876,8876,4780,8876,2635,2570,2537,2569,2504,424,4520,2472,2439,391,391,2439,4487,4519,2439,4454,2406,2406,4454,4454,4422,4422,4422,4454,4421,2341,2309,293,2341,4487,6600,4422,2309,2439,4519,4617,2374,2472,6600,2308,23535,21161,6535,6536,10793,6566,195,2243,195,195,163,4419,6563,6500,17001,10693,6499,8580,10692,8581,19114,21163,23275,42292,27436,25389,29614,25356,25388,23341,27501,33906,33938,59195,56919,52857,56952,52922,61144,31336,29321,25192,33710,44569,56951,50612,52693,42193,46550,54969,48630,42291,31694,27598,33807,31727,29679,33808,33807,31727,33840,33807,33775,23240,10691,10658,6434,8514,6465,6434,12644,6306,8386,10724,10691,32},
{23868,23900,25947,25948,25948,25948,25948,25948,28028,28028,30108,30140,32188,32188,30140,30140,30140,30140,30140,34268,36349,38460,42589,44669,44669,44669,42589,36413,34301,32252,32252,34301,34300,36348,36381,40476,42556,38461,34333,34333,34333,34333,34365,38493,44669,42653,42621,42589,40541,38493,38493,38493,36445,38493,36445,36445,38493,38493,38493,40573,40573,40574,40573,46782,50941,50941,48861,50909,50941,48861,46813,46781,48861,48861,46813,44733,44733,44733,44765,46813,48893,46814,46813,52989,59197,57149,57149,59229,57149,55036,55036,52924,50844,50843,48763,48763,48763,48763,50843,50843,50843,50843,50843,55004,57084,52923,50875,50843,50843,50875,32055,17395,15412,15444,15444,17492,17460,19508,8942,4779,8908,6893,8908,8909,6860,2602,4618,4618,521,2504,456,424,2504,2439,391,424,2471,4519,4551,2439,2374,2406,2374,4486,4454,2406,4486,2407,4487,4454,325,261,260,2374,2406,2439,326,4519,4519,2439,6664,4454,2406,2342,2341,23470,21226,12776,4487,8745,6567,195,195,163,163,131,195,8644,10725,18984,8579,6499,6531,8547,6468,14953,21163,25421,42292,23243,25356,29582,23308,25388,25389,27469,35953,35986,63389,56951,55001,63355,48663,65436,35528,27143,21032,33710,19115,56951,52693,52693,44339,50711,57050,39917,19016,25192,20999,25192,25192,20999,23112,23144,18919,21064,23112,18886,16901,8546,10658,8546,4353,4353,6402,12709,12546,8354,10659,10690,32},
{25916,23900,25947,25947,25948,25948,25948,27996,28028,28028,30108,30140,32188,30140,30140,30140,30140,30140,30172,34268,36348,40476,42589,46717,46717,44669,40541,36413,32253,32253,34301,34300,36380,36380,36381,40476,42556,38461,36381,34333,36381,34333,36413,38493,42621,42654,40573,40573,40541,38493,38493,38493,38493,36445,36445,38493,38493,38493,38493,40573,40573,40573,42653,46781,50942,50941,48861,48861,48861,46813,46813,46781,48861,48861,46813,46813,44733,44733,46814,46813,48861,46814,52989,59229,61309,59261,57149,57149,55101,55068,55036,52924,50876,50843,50843,50843,48763,50811,50811,48795,50811,50843,50843,55036,57116,55003,52891,50843,50843,50843,32022,17459,13396,15444,15444,15444,17492,11121,4748,4812,6860,6893,4813,6893,8973,2667,2602,2570,2570,2505,2504,456,2504,2472,2472,423,4552,4551,4486,2438,4486,2406,2406,4454,2438,4519,2406,2374,2406,2406,2374,2374,8647,6567,4519,4551,4519,2374,2406,2406,2503,2406,2374,2373,4453,17165,19178,10695,4487,8745,10793,2308,196,195,2243,163,2276,4355,10694,23209,12739,6499,6531,8547,6467,14953,23275,29615,27566,25324,23340,23276,27469,25356,27437,29550,29647,38099,61341,54936,63356,63323,44469,63453,46060,29158,27176,29354,27371,56984,54773,46223,25453,42324,52890,54839,50645,42193,35920,38001,38001,37968,38000,38000,35919,37968,37936,37903,33740,14820,10627,10659,8546,8482,8547,23273,23013,8386,10659,10658,35},
{25947,23900,25947,25948,25948,25948,25980,25980,28028,28028,30108,32156,30140,32188,32188,30140,30140,30140,32220,32220,36349,40477,42589,44669,46749,44669,40541,36381,34301,34269,34269,34300,34300,34301,34333,40508,42556,38461,36413,34333,34333,34333,36413,38493,42621,42621,40573,40541,38493,38493,38493,38493,38493,38493,38493,38493,38493,38525,40573,42621,40573,40574,42621,46813,48861,50941,48861,48861,48893,46813,46781,44733,48861,48893,46813,46813,44765,46781,44734,46814,48861,52989,57149,61309,59261,59261,57181,57148,57117,57116,55036,52956,52924,52923,50875,50843,48763,48763,48763,50811,50811,52891,50843,52956,52988,55003,52923,50843,52891,50843,34070,17460,15444,13396,15412,15444,13332,490,425,489,522,457,425,489,490,425,359,359,327,294,294,326,294,261,262,261,262,2342,261,229,229,228,229,228,228,2471,261,2309,228,228,229,261,4649,326,326,326,2374,196,2406,2374,293,2342,2439,2341,2341,2341,14984,6437,2374,6599,10793,6534,4356,2276,2275,163,2275,6532,8613,25420,14790,8644,6531,8547,8547,14888,23243,31760,17035,19147,23308,25421,27469,25356,29582,29582,33841,36018,57082,52888,65501,65403,48533,61341,61079,35431,33319,33351,33449,56952,56853,54674,42030,46452,57050,52790,18984,25289,18919,21064,21097,18919,21064,23145,18919,19016,21065,16871,19014,10659,10659,8514,6434,6402,6435,8582,27337,8419,14788,12803,45},
{25915,25948,25948,25948,23900,23900,27996,25948,28028,28028,30108,32156,32188,32188,32188,30140,30141,30140,32188,34268,36349,40508,42589,44669,46749,44669,40541,36381,36349,32221,34301,34300,36381,34301,38428,42556,44636,40509,36413,34333,34301,34333,36413,40541,42589,42621,40541,40541,38493,38493,38493,38493,38461,36445,38493,38493,38493,38525,40573,42621,42621,40574,42621,46781,48861,50941,48861,48861,48861,46781,46781,44733,46781,48861,46813,46813,44765,44733,46814,48861,57149,61310,61341,61342,59261,57181,57149,57116,57117,57149,57084,55004,52956,52923,50875,50843,48763,48763,48763,50843,50811,52923,50843,50843,50875,52955,50875,52923,55003,52923,32022,15412,13396,13364,15412,15444,4717,294,327,359,423,391,359,359,359,326,294,326,326,294,261,293,326,2374,2406,2374,326,293,293,325,2406,261,261,261,293,358,261,261,293,261,260,2341,2439,2374,2308,2308,229,2276,2341,2341,2309,2309,2341,2308,2276,2308,21193,6404,8518,8680,10793,10793,12905,10793,8679,6501,4421,4355,8581,21292,12711,19048,8547,8579,10660,23242,23244,25389,14823,17034,23276,27502,25356,25388,27502,27501,23275,44372,57017,50776,65501,65469,56984,57180,65502,44144,33351,33318,33352,52726,56886,52725,52692,44404,57050,56985,44175,42161,40015,37999,40015,37902,37902,37902,35789,35789,33709,33644,27401,14852,14819,12707,10595,8483,8482,4225,12612,8419,31498,18979,43},
{23899,25948,25948,25948,25948,23900,25948,27996,28028,28028,30108,32156,32188,32188,30140,30140,30140,30140,30172,34268,36349,40477,44637,44669,44669,44669,40541,36413,34301,32221,34300,38396,40476,36380,40476,44604,44636,40509,38461,34333,34301,34333,36413,40541,40541,40573,40541,40541,38494,36413,38493,38493,36445,36445,36445,38493,38525,38525,40573,42621,42621,40573,42621,44702,48861,50909,48861,48861,48829,46748,46749,44733,46781,48861,46813,46813,46813,46813,46813,53021,59229,63422,63422,61342,59261,59229,59229,57149,57117,57149,59197,52988,52956,52956,52923,52923,50843,48763,48731,50875,52923,52923,50843,50875,48795,52923,52923,52923,55003,52955,32054,15444,15444,15444,15444,11153,327,294,327,391,391,391,359,359,359,294,294,293,294,294,262,326,326,293,294,294,293,261,261,326,2439,293,261,261,261,2439,196,196,261,261,261,293,2439,2503,358,2309,2244,164,163,260,2406,196,196,228,131,2276,23306,10760,16969,4519,8713,8713,10793,10761,8647,4453,8614,10724,10660,8613,10598,31660,23109,16932,14854,31661,23276,27469,27371,19114,23308,27534,25388,27501,31694,25388,23210,55002,61178,46518,65502,63389,56985,61374,54937,40146,35561,33318,33351,46420,58999,46288,33807,35985,54970,57017,38001,29614,23177,19049,23210,23242,19082,23243,23275,21162,23275,25323,19048,16901,14884,10659,10627,10596,12677,22949,22883,22884,33514,44141,79},
{23900,23900,25948,25948,25948,25948,25948,27996,25980,28028,30108,30140,30140,32188,30140,30141,30140,30140,30140,34268,36349,40509,44669,44669,44669,42621,42589,36413,34301,32253,34300,38396,42556,40508,40508,44636,44636,40508,36413,36381,34333,34333,36413,38493,40541,40541,40541,40541,38493,38461,36445,36445,36446,36445,36445,38493,38525,38525,38525,42621,42621,40573,42621,42653,48862,48861,48861,48861,46748,46716,46748,46781,44733,46813,46813,46813,46813,48861,48861,57149,61342,63454,63454,63422,61342,61309,59261,57181,57149,57181,59229,55036,52956,52956,52923,52923,50843,48763,48763,50843,52955,52923,50843,52924,50843,50875,52923,50843,52923,52955,32022,13364,17460,15444,13332,425,294,294,326,391,2471,423,359,391,391,359,326,326,2439,2438,2406,2438,2439,2406,326,2374,2374,2374,293,2406,2471,326,293,2341,2374,2471,2341,293,293,2373,2341,293,2438,423,390,293,2308,2309,196,2341,2373,2276,228,196,195,2340,36050,14953,17001,2342,4487,6600,8713,6632,2341,163,4420,10757,14919,12741,16903,35920,42127,35786,27531,37967,23276,27469,29581,29549,23275,23308,27469,29582,29614,29679,25420,61276,61275,48631,65501,59163,52791,52889,40146,42292,41900,33318,33351,42129,59064,46254,27435,29614,52856,55002,42062,31531,35626,31498,31530,31530,29385,29418,29418,25192,27273,27272,23079,23111,23141,16835,14755,14692,24964,20738,14627,18690,29223,48532,154},
{23899,25948,25948,23900,25947,25980,25948,25980,28028,28028,28060,30140,30140,30140,32188,30140,30140,32188,32220,34268,36381,40509,44669,44669,42621,44669,42621,38461,34301,34268,32253,38396,44604,42556,44604,44636,44668,40541,36413,36381,34333,34333,36413,38493,40541,40541,38493,38493,36445,36413,38461,38494,36445,36445,38493,38493,40541,40573,40573,42621,42621,42621,42621,44701,48861,48861,48893,48829,46716,46716,46748,46781,44733,48861,46813,46814,46813,48861,52989,59262,61342,63422,63422,63422,63422,63422,61310,57181,59197,59229,59229,55069,52956,52956,52955,52923,50843,48763,48763,50843,52955,52923,50843,52923,55003,52923,50875,50843,52923,50875,32022,17460,17460,15412,6797,294,294,262,326,327,391,391,359,359,2439,359,390,2438,2439,2438,358,358,391,2406,325,293,325,326,2373,2406,2471,293,293,293,326,455,2438,390,2439,2342,293,326,2406,2503,2438,293,2341,4486,293,261,260,2341,2341,293,228,4453,25583,10728,4552,4422,4487,4487,6632,6567,4389,163,195,2339,8646,27434,37901,46418,42256,40142,33935,40048,23308,27469,38001,33743,25389,25388,29582,31695,29647,31695,31760,65501,59162,54970,63389,52823,59130,57017,56919,52693,46223,33383,37642,42096,61176,44044,29483,33839,50743,57050,44438,46648,61242,37968,25552,44502,40049,29745,44470,40080,29744,44437,42161,31825,42323,39982,25354,25387,31269,31563,35788,31269,41640,48467,142},
{23900,23900,25947,25948,25948,25948,25948,25980,28028,28028,30108,32156,30140,30140,30140,30140,30140,30140,32220,34268,36380,40509,44669,44669,44669,44669,42621,38461,34301,34269,34268,38396,44604,44604,44636,44636,44668,40541,38461,36381,34333,34333,36413,38493,38493,38493,38493,38493,36413,36413,38461,38493,36445,38493,36445,38526,40574,40573,40573,42621,44669,44669,44669,44669,48861,48861,48861,46748,46716,46716,46748,46781,44733,46813,46813,46813,46813,48861,50941,57181,61342,63454,63422,63422,63454,63422,61342,59229,59229,59229,59229,57117,55036,55004,52956,52923,52923,50811,48763,50843,52955,52923,48795,52956,57116,55003,50843,50843,50875,52923,32022,17460,17460,11154,295,262,262,262,294,326,359,359,294,327,4584,326,2503,4584,326,294,293,326,326,261,261,261,261,261,2438,2374,293,294,2277,229,229,261,293,293,261,293,293,261,293,326,293,261,2277,293,2438,2276,196,228,2309,228,196,4388,12807,131,131,131,131,163,228,2308,2276,2275,4356,131,14857,35920,50612,46385,33707,33772,33902,40048,23308,25421,48599,25420,25388,25389,29615,27502,33873,23178,21293,63389,46517,57148,56984,44470,59130,57115,65501,63323,56820,31336,33514,35726,50743,48401,39983,46419,48565,57018,27272,12808,55035,37771,2178,23471,37771,4291,25551,35691,2178,21391,35723,2178,19310,37738,2146,8615,23013,18821,21032,24997,39656,48402,141},
{23900,23900,25948,25948,25948,25948,27996,25980,28028,28028,28060,32188,32188,30140,30140,30140,30140,32188,34268,34300,38429,42589,44669,46717,46749,44669,42621,38493,34301,34301,32253,40476,44636,44636,46684,44668,44669,40541,40509,38461,34333,34333,36413,38461,38493,38493,38493,36445,36445,36413,36445,36445,36445,36445,38493,38525,40573,40573,42621,44669,44668,44668,44668,44668,48829,46781,46781,46716,46716,48796,48796,46749,44733,46813,46813,46813,46813,46813,48861,55069,59261,63454,63422,63422,63454,63454,63422,61342,59229,59229,59229,57117,52988,55004,55004,52956,52923,50843,48763,50843,52923,50875,50843,52955,55036,55035,52923,50843,50843,50875,32022,15444,13363,2506,262,294,262,196,2374,326,294,262,261,2342,327,294,2342,294,2342,229,261,2341,261,229,229,229,229,196,294,2309,2277,229,2244,2277,4389,164,196,228,2245,229,196,196,2244,196,196,2244,2276,2277,2341,2276,2244,196,196,195,2244,8646,10694,130,99,131,99,163,2275,227,2307,195,2244,2179,27502,35952,48467,46354,52659,40078,33902,37967,23276,27567,40211,23340,25356,27501,25421,31695,31663,16904,23373,63389,57017,63389,44339,40309,63323,55002,65502,65501,54740,25095,27110,25160,48565,42194,31662,48564,42259,54969,35723,10662,48794,44143,4193,14954,44143,4193,17034,44176,2081,10793,46256,2080,8713,46256,2048,4324,23112,4193,2146,33643,47880,48370,157},
{25948,25948,25948,25948,25980,25948,25948,25980,28028,28028,30108,30140,30140,30140,30140,30140,32188,32188,34268,36349,38429,42589,44669,46749,46749,44669,42589,40541,34333,34301,34301,40476,44636,44636,44636,44668,44668,42589,40541,38461,36381,34333,36413,38493,38493,36445,38493,36445,36413,36413,36445,36445,36445,38494,38493,38493,40573,40605,42653,44701,44668,44636,46684,46716,46748,48797,48796,48763,48764,50843,50876,46781,44733,46781,46813,46813,46813,44766,52989,57149,59261,63422,63422,63422,63454,65502,63422,63422,59261,59229,59229,59197,55036,52956,52956,52955,52923,50843,48763,48763,52923,50843,50843,55003,57116,57084,52923,52923,50843,52923,32022,15476,6895,262,262,262,262,164,4422,293,2471,261,2309,2341,228,229,2277,2277,2310,229,229,2309,2277,196,196,2244,229,196,2277,4357,2276,228,2244,4324,2277,164,164,2244,2244,2244,164,2244,4324,196,164,2276,2309,4325,164,2244,2276,2244,164,2211,4356,6598,8549,130,98,131,131,163,2275,2275,2275,2243,2211,6404,31727,35920,50677,54805,44338,33805,23273,31695,23275,29679,25421,23308,25388,29582,29549,27534,19049,16838,14922,59195,59196,65501,35920,34035,65469,59098,59228,65501,48369,25030,27241,27307,52824,44339,33839,46419,35953,54937,25224,6436,50809,35756,4193,21195,35821,4225,19082,37870,2081,14921,39983,2080,12808,40015,2080,8517,23144,0,65,38064,52041,46289,165},
{25948,23900,25948,23932,23932,27996,25980,25980,28028,28028,30108,30140,30140,30140,30140,30140,32188,32220,34269,36349,38461,42589,44669,46749,46749,44701,42589,40541,36381,34333,34301,42524,44636,44636,44636,44636,44668,44669,42589,38461,36381,36381,36413,38493,38493,36445,36445,36445,36413,36413,36445,36445,36445,36445,36445,38493,40573,42653,44701,44701,44701,44668,46716,46716,46748,48796,46716,48763,48795,48828,50876,46781,44733,44765,46813,46813,46813,46813,50941,57181,57181,61342,63422,63422,63454,63454,63454,63422,61342,61309,61309,61277,55036,55004,52956,52955,52923,50875,48763,48763,50843,50843,50875,52955,57084,57116,55003,52923,52923,50875,32022,13267,392,262,262,262,262,197,294,293,326,261,261,261,2277,262,2277,229,229,229,229,197,197,196,197,197,196,196,196,196,2244,196,196,2244,196,164,2211,196,2244,2212,164,164,196,196,164,2276,2244,2244,164,2212,164,2244,164,2211,2276,6565,4356,99,130,98,131,163,195,2243,4387,10628,10660,12806,21260,31727,50644,50644,27464,21160,21161,31727,23243,27502,25389,25356,25421,27534,25291,27436,16806,16839,16904,59261,65502,63324,29582,29745,63422,61145,52922,54970,46224,29126,25062,25129,38066,27436,27403,29548,33807,48663,20933,14823,57082,23079,2112,29679,25224,4193,29647,27272,33,27534,29385,2081,23373,29417,2081,12744,16773,4128,4258,50644,50057,44177,169},
{23900,23900,25948,23900,23932,25948,25980,25980,25980,28028,28060,30140,30140,30140,30140,30140,30141,32220,34268,36349,38461,42589,46717,46749,46749,44669,42621,40541,36381,36381,32285,38428,44636,44636,44636,46684,44669,44637,42557,38461,34333,34333,36413,36445,38493,36413,38493,36445,36413,36413,36445,36445,36445,36445,36445,38493,40573,44669,44701,44702,42654,42621,44668,46716,46716,48796,48764,48764,48796,50844,50876,48829,46813,46813,46813,46813,46814,46814,48893,57181,59261,57181,59294,61342,63422,63454,63422,63390,61309,61309,61309,61309,57116,55036,52956,52923,52924,50843,48763,48763,50843,50843,52923,52955,55035,57084,55004,52923,52923,50875,29974,2603,360,294,262,262,294,294,294,294,295,294,262,262,294,294,262,294,262,262,229,229,229,229,197,197,197,196,197,197,228,196,196,196,196,164,164,164,164,164,164,164,196,196,164,164,164,164,163,163,163,131,132,163,131,10696,2211,99,99,99,131,131,163,2275,2339,10757,12804,21161,21228,33904,52725,33770,16996,21159,21096,35952,23308,25421,27501,25388,29582,29614,29581,25291,18983,21064,23210,55067,63454,54871,19082,12842,55035,63356,52727,46421,37903,25031,27208,27338,42193,25257,27305,27337,29418,29483,27337,27403,35985,25158,20966,31694,23046,18886,31726,23046,16773,31695,23046,16741,31726,23046,14660,18983,20998,23111,21096,52659,50090,39918,169},
{23900,23900,23900,25948,25948,25948,25980,25980,28028,28060,28060,30140,30140,30140,30140,30140,30141,32221,32220,36349,38461,42557,44669,46749,46749,44669,42621,40509,38429,36381,34333,38429,42589,44637,44636,44668,44669,44637,40541,38461,36381,34333,36413,36445,36445,36445,36445,36413,36413,36413,36445,36445,36445,36445,36445,38494,42621,44669,42653,42654,40574,42621,42621,46683,48763,48764,48764,50844,50843,50876,52924,50909,48861,48861,46814,46813,46814,46814,48861,57181,57181,55101,50973,57149,63422,65502,65502,63422,61342,61342,61309,61309,59261,55036,52956,52924,52923,50843,48763,48763,50843,55003,55003,55003,55035,57084,55003,52923,50875,57084,33938,8615,6503,4422,4422,4454,4423,2342,2342,2342,2310,2310,2309,2310,2309,2278,2309,229,262,229,2277,2245,228,196,196,2244,196,2212,196,2244,196,2244,196,2212,2180,2179,2179,2180,2179,131,131,2179,2211,163,163,131,163,131,131,131,131,131,131,131,99,164,131,99,98,131,163,131,163,2243,6467,10724,10724,19112,12808,35985,52758,27431,12835,21257,29646,33808,27501,25421,25389,25421,29646,29582,33808,19016,21162,27501,25356,52954,61308,50612,21064,16904,48696,65469,50613,57017,27241,23015,27208,40114,44307,23176,25224,25225,25257,25257,25257,25257,25257,25256,25224,25224,25224,25192,25224,25224,25224,25192,25223,25224,25225,25192,25192,25192,25224,25224,25192,48270,48140,39853,164},
{23900,25948,25948,25947,25948,25948,27996,25980,25980,28028,28060,30108,30108,30140,30108,30140,30140,32189,32220,36349,38429,40541,44669,46749,46749,44669,40573,38461,38429,36381,36381,38461,42589,42589,44669,44668,44669,42589,40509,38461,36413,34365,36413,36445,38493,38493,36445,36413,36413,36445,36446,36445,36445,36445,36445,38494,42621,44669,42653,42621,42622,40573,44669,46683,48763,48764,48796,50844,50844,52924,52956,50908,48861,48861,46813,44765,46814,46814,46814,50941,55101,50941,48861,55101,63422,63454,63454,63454,61342,63390,63390,61309,61341,59229,55036,52923,52891,50843,48731,48763,50875,55035,55036,55004,55004,52955,52955,50875,50843,55036,52923,50777,46518,40114,33807,25355,18952,10630,8550,8582,6437,12679,10695,10695,10695,10663,12743,6502,6502,8550,8582,8582,8550,8582,8582,10630,8582,8582,8582,10662,10662,8582,8614,12808,14758,14758,14758,14725,14725,14725,14726,12678,12678,12677,14725,12677,12677,12677,12645,12645,12645,12645,12678,12710,2211,163,131,131,131,131,163,131,131,6403,12836,14949,12837,17065,17034,35985,50644,31721,14916,19014,27402,33807,42227,31695,25421,27437,27534,29614,29582,16904,21065,25290,33808,54904,59196,52725,35953,33840,52922,65469,56984,56951,27143,23015,25160,40147,38000,21063,21031,21031,23079,21031,21031,21031,21031,25290,29483,21063,21031,21031,21031,21031,20999,20999,21031,21031,21031,21031,23079,21031,25192,23079,23111,46124,48206,37772,130},
{23900,25948,25948,25948,25980,23932,25980,28028,25980,25980,28060,30108,30108,32188,32188,30140,30140,32220,32220,36381,38429,40541,44669,48797,46749,44669,40541,36413,36381,36381,36381,40509,44669,42589,44701,46749,44669,42588,40541,38461,38429,34333,36413,36413,36413,36413,36445,36413,36413,36445,36446,36445,36445,36446,36477,38493,40573,44669,44669,44669,42653,42621,44668,46683,46716,48764,48796,48796,50844,52924,52956,50908,50909,48861,46813,46814,46814,46814,46814,46814,48861,48893,46813,48893,61310,63422,63422,61342,61310,63390,61342,63390,61341,59261,57116,55004,50843,48763,48730,48763,52923,55036,52956,55003,55003,52955,50875,48795,50875,52955,50843,48795,50875,50843,50843,50810,50842,48697,44438,42357,42325,40245,40277,42325,40244,29713,27633,33775,19212,19147,17099,17067,17066,21227,27501,14889,12840,14921,10727,25420,25355,17001,21194,25420,19049,19049,19016,12743,6501,6469,6469,6501,8550,12710,12678,12742,8549,10662,23176,29482,29482,27369,14823,12776,4292,2211,2211,2211,2179,2211,2243,2211,2211,10757,15013,14917,12837,12805,14888,35985,48531,23270,19143,21161,27402,42291,38066,29614,25389,29582,29614,31760,21097,16904,23243,21162,33939,57050,55035,54839,33873,27534,52955,63388,54970,52595,25063,22982,25030,33743,35789,21031,21031,21031,23079,21031,20999,21031,21031,27468,29548,21031,21031,18951,20999,18918,18918,20998,20999,20999,18951,18950,18918,20999,25225,20998,23177,52496,48369,35659,100},
{23900,23900,25948,25948,25980,23932,25980,28028,28060,28060,28060,28060,30108,30140,30140,32188,32188,32220,34301,36381,38461,40541,44701,48797,46749,42621,38493,36381,34333,36381,38429,40541,44669,42589,44669,46749,46717,44636,40541,38461,38461,36413,36413,36413,34365,36445,36445,36413,36413,36445,36445,36445,36445,38494,38494,38493,42621,44636,46684,46716,42621,42653,44668,46715,46715,46716,48764,50844,50876,52924,52956,52956,50909,50909,48861,46814,46814,44766,44766,44766,44766,46814,46814,46846,50974,55134,61342,61342,61309,61342,61309,61342,61342,61309,59229,57116,50876,48763,46682,48763,50875,55004,55036,52956,52923,52955,50843,50843,52923,55035,50843,48795,50875,50843,50875,50843,48795,48795,46715,46715,46715,44635,44635,46715,44634,32022,29877,34004,21423,21390,19278,19278,19245,19245,33906,21228,14954,17066,12841,25454,25421,27534,27566,31824,19083,19050,23275,6534,2341,2309,2277,4357,4389,12712,6438,10728,4420,8583,31727,42193,42194,42193,29516,23210,4357,4357,4325,4356,4324,4325,4357,4324,6405,14950,17061,14949,10756,12805,14918,21258,19146,12837,14853,10596,27435,44405,29647,25388,25389,29647,25323,25355,21000,16839,18984,16871,34004,59065,48663,56952,33840,27502,52954,63323,57018,46224,22949,27176,25030,33776,27370,18919,18919,18951,18951,18951,18951,18951,18919,21096,18951,18951,18951,18918,18918,18951,18918,18919,18951,20999,20999,18951,18950,20998,21031,18950,23242,50579,46419,33578,79},
{23900,23900,23900,25980,25980,28028,28028,28028,28060,28060,30108,30108,30140,30108,30140,30140,30172,32220,34301,36381,38429,40541,46717,46781,46749,44669,38493,34333,34301,36381,36413,40541,42621,42589,46717,46749,44669,42588,40541,38461,36413,36413,34365,36413,36413,36413,36413,36413,36413,36445,36445,36445,36446,36445,38493,38493,44668,46716,46716,46716,44669,42653,44668,44668,46715,48764,48796,50844,50876,50876,52956,52956,48861,48893,48861,44765,46813,46814,46814,46814,46814,46814,46814,46846,46846,48894,55133,61342,61310,61342,61342,61309,61309,61309,61309,57149,50875,50811,48730,48763,50843,55004,55036,55036,52955,52923,50843,50843,52923,55036,50875,48795,50875,50875,52955,50843,50843,48795,46715,46715,46715,46683,46715,46715,44634,31991,31925,31891,21455,21423,21358,23406,31792,23405,31793,19148,12938,38032,17002,27566,31759,29646,25453,29646,21163,21130,25388,10695,8583,2341,2341,6437,8518,10632,4357,6535,4389,8583,31727,42194,42226,42194,46452,42128,2147,99,67,99,98,99,66,98,2275,14981,14916,8644,6499,10724,10724,12804,8580,8548,29515,10564,25290,31727,29647,25388,21195,29646,27501,18984,16871,16806,16903,12743,29843,61243,48696,59065,35921,31727,57115,65469,65469,39820,20869,25029,22949,29516,20999,16838,18886,18918,18886,16838,18886,18886,18886,16837,16838,16838,16805,16805,16805,16805,16805,16805,16806,16806,16806,16806,16805,18950,27271,10466,17034,48498,42324,27337,76},
{23900,25947,25948,23900,23900,25980,25980,28060,28060,28028,30108,30108,30108,30140,30140,30140,32188,32220,34301,36381,38429,40509,46717,46749,48797,44669,38493,34333,32285,36381,38429,40541,40541,44637,46749,46749,42621,42556,40508,38461,36413,36413,34365,36413,34365,36413,34365,34365,36413,36413,36445,36446,36446,36446,38494,40542,46716,46716,48764,46716,46716,44700,44668,46716,46716,46748,48796,50844,48828,50908,52956,50908,48861,48893,48861,46813,44766,44766,44766,44766,46814,46814,46814,46846,46846,48893,55069,59229,61310,61309,59229,59229,59229,61309,59229,57116,52923,50811,48762,48730,50843,52955,52955,55036,52955,52923,50875,50843,50875,52955,50875,50843,50875,50843,52923,50875,50875,48763,46715,48763,46715,46715,46715,46715,44635,31991,27797,31891,21455,21422,21390,21357,42292,23438,29712,19180,15018,31792,25454,27598,29647,25454,29614,33840,21196,21163,23276,12808,4390,261,8615,6470,10663,8518,4325,6470,4388,8583,33808,40146,40145,40146,46485,42161,2179,131,131,131,98,131,98,99,6500,14948,10724,8579,6499,10692,10692,10692,8548,6500,21128,10564,31498,29451,25389,23308,23340,29614,31695,18984,25355,29581,21065,16969,33938,61178,46584,59065,35953,38164,63389,65502,65468,31368,22950,25160,22950,27403,22981,20966,23014,33709,37836,23143,29450,33546,12579,14725,27337,12612,14725,25224,10564,14693,23111,12644,14757,23079,12645,14757,18886,14757,35983,12481,16968,44241,27534,18951,90},
{23900,25948,25948,25948,25980,25948,28028,28060,28060,28060,28060,28060,30108,30140,30140,30140,32220,34268,34301,36349,38429,40509,44669,46749,48829,44669,36413,34333,34301,36381,36381,40541,40541,44669,44701,46749,42621,40540,42557,40509,36413,34365,34365,34365,36414,36413,34365,34365,36413,36445,36445,36446,36446,36445,38493,40541,46716,50844,50844,48796,48796,48764,46748,46716,44668,46748,48796,50844,48796,52924,52956,50908,48861,48893,48893,46814,44766,44766,44766,44766,46814,46814,46845,46846,48894,55069,59261,59229,59229,59229,59229,59229,59229,59229,57149,55036,52923,48763,48731,48731,52923,52923,52956,55035,52987,52955,52923,50875,52955,55004,52923,50843,50875,48795,46682,46682,44634,44602,44602,42554,42522,42554,42522,44602,42522,29910,29812,36084,25519,25551,23438,25486,40212,27566,38099,27599,17099,31760,19147,31825,27598,21228,27567,36018,21228,25421,17066,17034,10662,6437,8583,8550,14857,8550,4357,8582,6501,8550,33840,31728,42292,35921,40244,44273,2179,131,163,131,99,195,131,163,6563,12835,10820,10724,8612,8580,10693,10692,10692,8612,29548,18984,35723,31564,23308,25389,25421,31760,23243,16871,19017,21065,14758,19049,36181,61210,52825,59065,33873,38164,61243,57147,63258,27142,22982,25063,20902,25095,24997,25095,24997,31728,38033,33904,36017,40210,40015,37968,40113,35821,40080,42194,35757,40048,44209,33708,37967,42128,33611,35789,35821,33675,21192,23306,44305,40015,18984,19016,83},
{23900,25948,25948,25980,25980,27996,28028,28028,28060,28060,28060,28060,30108,32156,32188,30140,32220,34269,34300,36381,36381,40509,44669,46749,46781,44669,36413,34301,34333,34333,36381,40541,40541,42621,46749,46749,42621,42588,42557,40541,36413,34365,34365,36413,36413,36413,36413,34365,36413,36445,36413,36446,36445,36446,38493,40541,46716,50876,50876,50876,50844,48796,48796,46748,44701,46749,48796,48796,50844,50876,52956,50908,48861,48861,48894,46813,44766,44766,44766,44798,46814,46846,46846,46846,46845,55069,61310,61310,59229,59261,59261,59229,59229,57148,55068,52956,50875,48763,48730,48730,52923,52955,50907,55036,52955,52955,52923,50843,52923,52956,52923,48795,48794,46682,38165,36085,36117,38165,38230,40278,38197,40278,40245,40278,40245,40245,40212,40245,38132,36051,33939,33971,36051,36051,36052,33939,27567,19213,17100,19181,12972,19148,21261,23438,15019,14955,17035,14987,14954,17003,17034,17035,17003,14922,14889,14921,6502,8583,33840,31695,44471,31662,29744,46386,2179,131,163,131,130,195,163,2275,8612,10756,12836,10724,8580,4420,10724,10725,10692,8580,21128,23209,37803,29483,23308,27469,31695,31695,16904,21194,27501,21129,16904,14791,31988,57115,54905,61210,35953,38196,56985,61308,56853,25062,27111,25030,22982,25094,24997,25128,24964,23113,22982,16805,16903,27533,39982,23046,20966,27207,27239,27239,25191,27239,25094,23111,29319,25126,25224,33707,31694,33675,10658,8612,12871,33937,33676,18951,70},
{23900,25948,25980,25980,25980,28028,28028,28028,30108,28060,30108,28060,28092,30108,32188,32188,34268,32220,34301,34300,36381,40541,44669,46749,46749,42653,36381,34301,34333,34333,36381,40541,42621,44669,46749,46749,42621,42588,44636,40509,36413,34365,34365,36413,34365,34365,34365,34365,36413,36445,36445,36414,36445,38494,38493,42621,46716,50844,52924,52924,50875,50843,48796,46748,44701,44701,48796,48796,48796,50876,50876,50877,48861,48861,48893,46813,46814,44765,46813,44766,46814,46814,46846,46846,46845,52989,59261,61342,61309,61309,61309,61309,59229,57149,55068,52956,50843,48763,48730,48730,52923,52924,50843,52956,52955,52955,52923,50843,50875,52955,52891,46650,42488,42455,40310,40278,38229,38230,40278,40310,38198,38262,38230,38230,40278,40310,38262,40310,36117,34036,34004,36117,34004,31924,31923,33971,31793,25454,19213,19115,17068,21261,19245,19213,14987,14955,19116,17068,17035,17067,12842,14922,14890,14922,14922,12808,6437,8583,33840,33808,40277,35887,27599,46353,2179,131,163,2244,163,228,163,2243,8579,10756,10756,8579,6532,6499,10756,10692,8612,10725,31595,27501,35723,33611,27437,29582,25356,23210,16871,14758,16872,12743,19050,10597,34003,57082,61275,63324,38001,46582,59097,50841,50481,22982,24997,22949,20901,24997,22917,24997,22884,22949,18756,14661,20966,31726,42160,27206,27238,29286,31431,29254,27271,25126,29221,20933,29286,27173,25126,33674,35854,29480,10594,8579,8482,8614,21161,14758,31},
{25948,25948,23932,25980,25980,25980,28060,28060,30108,28060,30108,28060,28060,30140,30140,32220,32220,32220,34268,34301,36381,40509,44669,44669,46749,42621,36381,34301,34333,36381,36413,40541,42621,44701,46749,46781,42621,44636,44636,40509,36413,34365,34333,34333,34365,34365,36413,34365,36413,36446,36446,36445,36446,40541,44669,46749,48796,50844,50876,52956,52924,52924,50876,46748,44701,44701,46748,48796,48796,50876,48828,48829,48861,46813,48861,46846,44766,44766,46814,46814,46814,46814,46814,46846,46846,50941,59261,61342,61341,61309,61309,61309,59229,57149,55036,52923,50843,48763,48762,48763,52956,50876,50875,52956,52955,52955,52923,50843,50875,52955,52923,42456,42456,42456,40342,38262,40310,38230,38262,40343,42358,40310,40278,40278,38230,40310,38230,38262,36117,34037,31924,33972,36084,34004,36085,36117,36019,31826,25454,21228,17035,17035,15019,17099,17068,14955,17100,19148,17068,15019,14955,12842,12842,12874,12938,12874,6534,8583,29679,31727,40245,44307,33872,44241,2179,131,131,2276,196,228,4323,4419,10724,8644,8643,8612,6531,4419,10756,10691,10692,10693,27532,31759,37836,37836,29581,29549,29582,18919,16839,18919,16871,17002,16936,10598,48696,63356,57050,63388,38033,52889,54969,54970,44110,20869,25128,22917,23047,25030,23015,25062,20869,23112,18691,14790,23013,27207,29288,27304,27238,29384,29286,29351,25191,25191,31432,25126,29286,29286,29255,31594,33741,23173,10627,8547,10596,10693,12710,6371,26},
{25948,25948,25980,25980,25980,28028,28060,30108,30108,28060,30108,30140,30108,30140,30140,32188,32220,32220,34300,36349,36381,40509,42589,44669,44701,42621,36381,34301,36381,36381,38461,40541,44669,48829,46781,46749,42621,44636,44636,40541,36413,34333,34365,34365,36413,34365,34365,34365,36413,36445,36445,36446,40541,46717,50877,52988,52956,52956,52924,52956,52956,52956,50876,48828,44733,44701,46748,46748,48796,48828,48829,48861,48861,46813,46813,46846,44766,44766,46813,46814,46846,46846,46846,46845,46845,48893,57182,61342,61342,61342,61309,59261,59229,57117,55036,55004,50843,48763,48731,46715,50875,50876,50875,52956,55035,52955,50843,50843,52955,52987,50843,42456,42456,44504,40343,40278,38231,40343,40343,40343,38263,38263,38230,38230,36150,36182,36150,34069,36117,38165,36085,38197,38198,36149,38198,36085,36052,36117,31924,31826,29680,27567,19180,19148,19180,17035,19148,19116,15019,17035,14922,14890,14890,14922,14954,12809,4421,8615,27567,33776,42390,46453,25453,46321,2178,131,164,228,228,228,4421,6500,8676,12836,10724,6499,6467,4387,8643,10692,10724,12772,14822,31825,37836,37803,33709,31727,27501,18984,23210,25323,16839,19049,8517,10663,57148,63421,48598,63389,40114,57083,46485,61308,35626,18821,25095,20869,25160,24964,20934,24965,20901,20934,14595,23208,25093,29221,29254,29352,27173,29351,29253,29254,29384,29352,29384,23045,27206,29351,29352,29481,23208,14819,10595,8547,8547,10627,10564,6403,47},
{25948,25948,25980,25980,28028,28028,30108,30108,30140,30140,30140,30140,30108,28092,30140,32189,32220,32221,34301,36349,36381,40541,42589,44669,44669,40573,36413,34333,36381,36413,38461,42589,44701,48829,48829,46781,42621,44636,44636,40541,36413,34333,34365,34365,34365,34365,34365,36413,36414,38493,42621,40573,44669,50877,55037,52989,55036,55036,55004,52956,52956,52956,50876,48828,44701,44701,46749,46716,48764,48796,50877,50877,48829,46813,46813,46813,44765,44766,44766,44766,46846,46813,46846,46846,48894,48894,50973,59262,61342,61342,61310,59229,57116,55036,52956,52923,50843,48763,48763,46715,50843,50875,50875,55004,55003,52923,50843,50843,55003,52955,50843,42456,42456,42456,36182,36117,36085,36150,36182,38295,38230,38230,38262,38198,38198,38230,42423,42391,38295,40375,38230,40343,36085,36117,36149,34036,34037,31924,31892,31891,29843,27730,27632,27568,23407,17003,14922,17100,17068,15019,17035,17035,17035,12874,8648,8680,6501,6566,25422,33840,40245,48598,27501,46386,2179,131,131,228,196,2276,4356,2275,2275,10724,12868,8611,8580,6435,10692,10724,10692,16965,35919,33906,44208,37803,35887,33808,23209,23210,25323,21097,14725,27338,18886,31596,57148,63421,52791,63421,44339,61243,44405,63388,27175,22949,23015,20869,25062,24964,22950,22884,22950,20901,18788,23242,25126,25093,27206,27239,27239,29384,29351,29319,27304,25223,29351,29351,29351,29384,31464,27270,14755,10562,10562,8546,10627,10723,8546,4257,56},
{25948,25980,25948,25980,27996,28028,30108,32188,30140,32188,32188,32188,30140,30140,30140,32220,32188,32188,32220,34301,38429,40541,42589,44669,42621,40541,36413,34333,36413,36413,38461,42621,46749,48861,48829,46781,42621,44636,44636,40541,36413,34333,34333,34365,34365,36413,36413,34365,40541,44669,50877,50877,50877,52989,55037,55037,55069,55069,55036,55036,52956,52956,52956,48796,44701,46749,44701,46748,48796,50844,50876,50876,48829,46813,46813,46814,46814,44765,44766,44766,46846,48893,48893,48894,48894,48894,48893,55102,59230,61310,61309,59229,57116,55036,52924,50875,50843,48763,48763,48763,48763,50875,52923,55036,55035,52923,50843,52923,57116,55003,48763,40343,42424,42391,40311,38263,40310,38263,36182,38230,38230,38262,40343,38263,38230,38197,38263,40310,34036,36150,31956,38230,42391,40342,38262,38165,36117,36084,31891,31891,31891,31923,31924,29843,27730,29778,27633,27697,25584,19278,19246,17133,19180,14955,14890,12874,6534,4420,17067,33808,40245,50711,31759,48466,4324,4357,2308,8550,4422,4388,4388,2275,6500,10756,12804,8611,8579,4387,8612,10692,10661,19112,40112,33938,48467,37804,35756,29548,23209,40080,31595,23176,12677,35886,40048,50678,57115,61308,50710,61373,50679,63356,48631,59064,23014,25095,20934,20934,25030,22884,22982,22852,20902,20902,33742,29548,14724,16805,20966,23078,23013,27206,29286,29254,27271,42257,44141,42093,42094,42093,42061,39981,39980,39948,35722,14787,12740,10691,8514,10596,114},
{23900,23932,25948,28028,27996,28060,30108,32220,32188,32220,32188,30140,30140,30141,30140,32220,30140,30140,32220,34301,36413,40541,42589,44669,44670,40541,38461,36413,36413,38461,40509,42621,48829,48861,48829,46781,42621,44636,44636,40541,36413,34333,34333,34333,34365,34365,34365,36414,40541,48829,55037,55037,55069,55069,55069,57117,57149,55101,55068,55036,55004,52956,52956,48828,46749,46749,44700,46748,48796,48796,50876,50876,48861,46813,46813,46814,46813,44765,44766,46814,50941,53021,50942,48894,48893,48894,48894,48894,53021,57181,59229,59229,57148,55036,52956,52923,50843,50843,48763,48763,48763,48795,52955,55004,55035,52955,50875,52955,59196,55036,48762,44536,42488,44536,38230,38198,38198,38263,38230,38198,36182,36150,34070,34037,36117,34069,38198,36117,36117,36117,36149,38197,38263,34069,34069,36150,36117,36149,36117,34004,33971,27632,17133,15085,23309,23309,23309,23342,23277,23276,21163,21163,19050,16969,16904,14823,12677,10564,23275,33775,38132,50744,25356,44338,6404,4389,4454,4421,2341,10727,4420,6468,4387,8645,10757,10660,8580,6468,6435,14918,12772,25386,44272,36019,52790,35723,31529,18949,21193,44371,42192,27434,21129,46419,48565,48565,57115,57115,59162,59196,57017,56984,54969,52660,22949,25063,22917,23014,25062,22949,22982,22884,23047,18821,27468,29743,25193,16871,21031,23013,25125,27206,31399,31334,31530,48597,44174,42126,44174,42093,42094,40013,40014,40013,39980,25189,14820,10627,8482,4322,57},
{25948,25948,25980,27996,25980,28060,30108,32220,34268,34268,32220,30140,30140,30140,30140,32189,30141,30173,30173,34300,38461,40541,42589,44669,44701,40541,38461,38461,38461,38493,40541,44669,48829,50941,48861,46749,42621,42589,42588,40541,36413,34365,34365,34365,34365,36413,34365,34365,36445,46717,57117,57182,57181,55101,55069,59197,59229,57149,57116,57116,55036,55036,52956,50876,46748,46748,46748,46748,48829,48828,52892,50908,48861,48861,46813,46813,46814,46814,46814,48893,52989,55069,53021,50941,48893,48894,48894,48894,50941,55069,55101,57148,57148,57116,55036,55004,52923,50843,48763,48763,48763,48763,52923,55036,55036,52955,50843,52955,57116,57116,50843,34168,32087,30007,30006,32054,31990,29941,29973,29941,29941,31989,29941,31989,32022,34070,34037,31957,25748,25683,25716,23667,23635,23634,21554,25683,23635,25715,23635,25650,25682,25650,23504,15019,35724,46322,35887,27305,39852,41899,41899,41900,41900,39851,41868,39819,39786,39754,39819,39852,39885,39951,35723,46386,6404,6502,6567,4356,163,4389,259,4485,6563,8612,8644,8612,6467,4387,6500,10757,12805,35951,46320,33873,54904,39917,33577,18949,19048,42226,35919,25354,31694,46452,48565,46485,57115,57018,61243,63388,61210,52824,55034,46256,20902,25160,20837,25193,25030,20902,22981,20869,20998,16675,14823,27598,35790,21096,31595,22981,27239,31367,33545,33512,37902,42193,44206,42094,44142,42094,42093,40013,40013,40013,40014,31528,10626,8579,8514,6402,41},
{25948,23900,25980,25980,25980,28060,34236,36349,38429,36381,32220,30140,30140,30141,30140,30140,30173,30140,32221,32253,38461,40541,42589,44637,44701,42589,40541,38493,38493,40541,40541,44701,48861,50942,50942,46749,40573,42589,42588,40541,36413,34365,34365,34365,34365,34365,34365,36414,36414,40541,52989,59230,57181,57149,57149,59229,59229,59229,59229,57149,57116,55068,55004,50876,48796,46716,46716,48764,48796,48796,50876,50908,48861,46814,46813,46814,46814,44766,44766,48893,55069,55069,55101,53021,50941,48894,48894,48926,55069,57117,55069,57116,55068,57148,57116,55036,55004,50875,50843,48795,48763,48763,50875,55036,52956,50875,50875,55036,55035,55068,44602,8942,2538,4715,8941,11054,11054,11054,11054,13102,11054,8974,9006,11022,6828,2635,8876,4684,2570,4553,2504,2504,6601,4520,2472,4552,2472,2472,4520,4520,2472,4520,2472,2407,10663,8778,8648,6501,10630,8582,12710,10694,14887,17000,16968,16903,14790,12710,12710,14823,14822,16838,14823,17066,6437,2309,4422,2276,2243,4388,2275,4388,4419,4420,6499,8579,6467,6467,8548,4452,4387,33904,35853,31825,52857,39982,27369,18983,23339,40146,23308,33709,37935,42193,40145,42259,52922,59195,63356,65501,65468,48630,61276,39885,20836,24998,20837,24998,22884,22885,22884,22884,20901,14691,16806,25323,35789,27370,31661,29254,25158,29319,33513,31431,42128,35789,44272,42126,42094,42094,42093,40013,40013,39981,39981,25287,10659,8546,4386,6466,63},
{25980,23900,25948,28028,25980,30108,34268,40509,42589,40509,34268,30140,30140,32189,30141,30140,30141,30140,32221,36349,40509,40541,42589,42589,44669,42589,40541,38493,40541,40541,40573,44701,48861,52990,52990,46749,40541,40509,42589,38493,36413,34333,34365,34365,34365,34365,36414,36414,36446,38493,46749,57149,57181,59229,59229,59261,61310,59262,59262,59229,57116,57084,55036,52924,48796,48763,48763,50811,48796,48828,50876,50909,46813,46813,46813,46814,46814,46814,46814,48861,52989,55069,57117,55069,53021,48926,48926,48926,55069,57149,57116,55068,55036,55068,57116,55068,55036,52955,50843,50843,50811,48731,48795,52955,52923,50843,50875,52988,46715,21522,2538,2472,4487,2439,4553,6763,8908,13069,11021,11021,11053,10989,11021,11021,8843,4585,6730,4650,4553,4520,6535,4487,4519,4487,4487,6535,4487,4487,4455,4487,2439,6503,4487,2341,2373,2373,4357,4357,4389,2309,4389,4454,8615,12840,8647,6567,4389,2276,4324,6470,6469,4356,6404,4357,2276,2244,2276,228,2276,2308,2275,6469,2243,2307,8612,6532,8580,4452,4387,4323,4387,27565,29450,21228,44405,37869,25257,14790,16935,35919,25323,48566,48499,48565,46452,52759,55035,61309,61308,65501,65468,50645,63356,31400,31336,20902,18789,25127,20803,22950,22851,22917,20901,14659,14724,25323,31596,35789,35855,31464,29384,31367,33512,31399,44241,35756,44305,42094,42094,42094,42093,40013,40013,39981,37835,16835,10595,8482,6466,8578,45},
{25980,23932,25948,25980,28028,30108,36316,40509,44669,40541,34301,30140,30140,30140,30140,30172,30140,32189,34301,34301,38461,42589,40541,40573,42621,40573,42589,40541,42589,40541,42621,44701,48861,52990,52990,44733,40509,40541,42589,38493,34365,34333,34333,34365,34365,34365,36413,36414,36446,38493,46749,55069,57181,59262,61310,61310,61342,61342,61310,59229,57149,57116,55036,52956,50875,48795,48795,50843,50844,50844,50876,48861,46813,46813,46814,44766,46814,46814,46814,46814,52989,55069,57117,57117,53021,48894,48894,50973,55069,57149,57149,57116,55036,52956,52988,52956,52955,52923,50843,50843,50843,48763,48763,50875,52923,50875,46747,23700,2571,2472,4488,4488,4487,4487,2440,2440,2536,6763,8909,11021,11053,11021,11021,8973,8811,4585,8746,4617,4552,4520,4487,2407,2439,2439,4455,6503,4487,4455,4455,4455,4454,4454,4486,4421,2341,4421,4389,4389,2341,2309,2341,4421,8647,10760,8615,6502,4357,4325,4324,4421,6470,4324,4389,4389,2308,2276,2276,2276,2276,2308,2308,6469,195,2307,6596,6499,8548,6532,8580,4355,4355,14920,16935,23308,48664,40015,31628,25224,16901,33838,42161,54905,50678,50613,50678,52791,54970,63389,54871,63421,54936,65436,63290,27273,48272,18789,14628,25257,14498,14563,20803,20707,16642,20803,22884,21097,31660,42160,38000,27337,23209,23078,25126,23112,46386,37868,44273,42126,42094,42094,42094,42061,40013,39981,37900,21092,8514,6466,8578,8546,36},
{25980,25948,25980,25980,28028,32156,36348,42557,44669,40541,34301,30172,30140,32188,30141,30140,30141,32220,34269,36381,38461,42621,40573,40541,42621,40573,40541,40541,40541,40541,42589,46717,48830,52990,50942,44702,38461,40541,40541,38461,34365,34365,34365,34365,34365,34365,36413,34365,36446,38494,50877,57117,59229,59262,61310,61342,61342,61342,61310,59229,57149,57116,55036,52956,52924,50875,50843,48795,50844,50876,50876,48861,46813,46813,46814,46814,46814,46814,44766,48861,52989,55069,55069,55101,53021,48893,48926,55069,57149,57149,57149,57148,55036,52956,52956,52956,52923,50875,50843,50875,50843,48763,48763,50875,48795,29974,6764,2440,2440,4456,4455,4487,4455,2407,2407,2407,358,2439,2505,6763,10956,10989,10989,10989,8876,4585,8746,6633,4520,2471,4487,4455,2439,2439,2439,4487,4487,4455,4455,6502,6470,6534,6567,4454,4421,4421,4389,4421,2341,2341,4389,4389,8615,6632,4519,4389,4324,2244,2276,4389,4421,4324,6405,4357,2276,2276,2308,2275,2276,4356,2275,6469,2211,163,6467,8644,6500,4420,6467,4355,4388,14854,8580,19147,40245,35887,33709,29449,21094,33870,48565,54904,50711,52726,50711,52758,52857,61341,57016,55002,52889,65501,56918,33710,54675,22917,14693,25290,16610,12515,18789,20739,14596,23014,25192,27403,25388,42225,29613,18853,12580,12677,20868,16904,44240,35788,40112,42126,42126,42094,42094,42061,39981,39948,25287,14852,10659,8514,8545,8578,35},
{28028,27996,25948,25980,30108,34268,36381,42589,44669,40541,34301,32220,30140,30140,30140,30140,30140,30172,32252,34333,38461,42589,42621,40541,42589,40573,42621,40573,40541,40541,42621,46749,48861,50942,50942,42654,36413,38461,42589,36413,34365,34365,34365,34365,34365,34365,36414,34398,36446,46749,52957,57181,59230,61310,61310,61342,61342,61342,61310,59229,59197,57116,55036,52956,52956,52923,50843,50843,50844,50876,50908,50909,46813,46813,46813,46814,46814,46814,44798,48893,52989,55069,55069,55101,53021,48894,48894,55101,59262,59229,59229,59229,57116,55036,52923,52923,50875,50843,50811,50843,50811,48730,46715,32055,6829,2472,2440,4455,4455,2407,4423,2406,2342,2374,4422,2374,326,2375,294,2407,2505,4682,8876,8908,6763,6730,4618,6633,4519,4487,2439,359,4455,4487,4455,4455,4487,2407,4455,6535,4454,4519,8712,4454,4422,6469,4389,4389,4389,2308,4389,2341,6567,6600,6534,4357,4324,4292,2276,6437,4389,4324,4356,2341,2276,2275,2276,2243,2243,4324,4291,6470,2211,163,4420,10660,6467,6532,6435,6468,8548,10692,8516,19082,33938,35886,29483,23144,19046,31790,50744,50710,50743,54904,50711,52758,52857,61341,59032,52791,52824,61340,50514,50712,58966,22917,16611,20935,22916,18691,18723,24835,18691,22983,29516,27436,29646,42193,21096,16741,10468,14757,18788,25421,42062,35788,37967,44175,42094,42094,42094,42093,40013,39980,23173,12739,8514,6402,6434,8546,49},
{28028,25980,25980,28028,32156,38396,40509,42589,44669,38493,34301,32220,30140,30140,30140,30140,30140,30173,32221,34301,38461,40541,42621,40541,40541,42589,42621,42621,40541,40541,42621,46750,48861,50942,48894,38525,34333,36381,40541,36413,34365,34333,34333,34365,34365,34365,36413,34397,38493,52989,57150,57181,59230,59262,61342,61342,61342,61342,61310,59229,57149,57116,55036,52956,52956,52955,52891,50843,48796,50876,50876,48861,46813,46813,46814,46814,46814,46814,44766,48861,52989,55101,55069,55069,53021,48893,48894,53021,59229,61310,61309,59229,57148,55036,52955,52923,50843,50811,48763,48795,46715,34200,13200,2505,4520,4488,4455,4455,4455,4455,4455,4487,4455,4455,4455,4455,2407,2407,2407,2374,326,2374,4584,6730,2569,4682,2569,4552,4552,4487,2407,358,2406,4454,4455,4455,2407,4422,4422,6470,2406,4519,6599,4454,4421,4421,4389,4389,4389,260,4356,2308,6535,8615,6502,4389,4357,4357,4324,6437,4389,4324,4357,2439,2276,2244,4324,2243,2243,6437,6470,6502,2211,163,8613,10692,4388,8644,8612,10725,12773,8580,6436,12872,19146,10662,14790,18853,10693,31791,38099,46485,46485,46517,48597,52724,52824,61309,63388,57049,52856,61308,46288,59164,61242,25029,16676,12678,27175,10434,12580,18756,10435,14726,29548,33741,40211,44338,23144,18853,10499,20901,18755,35952,42094,37868,31661,44272,42094,42094,42093,42094,42061,37900,25221,10658,8514,10594,8482,8546,62},
{28028,28060,28060,30108,34268,38429,40509,42589,42621,38461,34301,32220,30140,28092,30140,30140,32188,30173,32220,34301,36381,40509,42589,38493,40509,42589,44669,42621,40541,42621,44669,46750,46781,50942,46814,38493,34333,34333,36413,36413,34333,34365,34365,36413,34365,36413,34365,34397,38494,50909,55102,57149,59229,59262,61342,61342,61342,61310,61310,59229,57149,57116,55036,52955,55004,52956,50875,50843,48828,48828,50876,48861,46814,46814,46814,46813,46814,44766,46814,50941,55069,57149,57149,55101,50973,48925,48926,50973,55101,61310,61310,59229,57149,57116,55036,52955,50843,48763,48731,40474,19507,8908,6666,4584,2504,4552,6600,4552,4520,4520,4488,4519,4520,4487,4488,4488,4487,4487,4519,2406,2407,2374,2375,2472,4552,4585,6665,4552,4552,2440,4487,390,4487,4455,4487,4487,4454,4454,6503,6535,2406,4487,4487,4454,2373,4421,4357,4421,4389,2341,2309,2341,6535,8583,4390,6437,4325,4324,4324,4324,4324,4324,4357,4423,4357,4323,2276,2276,2244,4325,8615,6534,2243,2243,6532,8612,4419,8644,12773,10693,10693,8580,8549,12773,8516,8516,12709,16870,4355,27565,25323,46485,42194,44372,48564,52725,48663,61276,61275,56984,48631,63421,48467,61276,61210,27142,18724,18724,25062,14531,16643,18690,16578,18724,27338,31628,38033,44372,27370,21032,16805,22948,16772,44338,42127,35755,31563,44273,42094,42094,42094,42094,40013,35754,19012,14851,10690,10658,6401,4321,28},
{32124,32188,32188,36316,40477,44637,42589,44669,42621,38461,34301,32220,28124,28092,30140,30140,30140,30141,32221,34269,36381,40509,40541,38493,40509,42589,44669,40573,38493,42621,44670,44669,46781,50942,44734,36445,34333,34333,36413,34365,34333,34333,34365,36381,34365,34365,34398,36446,36446,44701,50941,55101,57181,59230,61342,61342,61342,61310,61309,59229,57149,57116,55036,52955,55004,55004,52923,50875,48796,48828,50876,48861,46813,46814,46814,46814,46814,44766,46814,52989,55101,57149,57149,55101,50973,48893,48925,50941,50973,57181,59229,59197,57149,57148,57116,52955,50843,42554,21587,8941,8876,6730,4585,4552,4552,6600,6600,4552,4552,4552,2472,6600,6568,4520,4552,4488,4520,4520,4519,4519,4552,2439,2439,2439,2374,4519,6665,6665,6600,2439,2439,2407,2439,4455,4455,4455,2407,2439,4487,6535,2406,4455,6502,4422,2373,6502,6469,4454,4421,4421,4357,2341,6567,8647,4389,6437,4325,4324,4324,4324,4324,4357,6469,4390,4357,4324,4324,2276,2244,2276,4357,6470,2211,195,6532,6533,4387,6595,12805,8580,8612,6532,10660,8580,6468,6467,4323,4258,4323,31791,50678,52824,50678,54904,50677,52692,42324,59228,65437,56984,46486,50744,44372,59163,59130,31499,14595,10500,27305,12418,12515,20836,18658,16676,25193,29516,38001,44404,23046,14660,14726,22949,16838,48466,44207,35755,33675,42225,44174,42094,42094,42094,42061,35722,16964,14818,10689,10657,8513,6369,24},
{32188,36348,38396,40509,44637,46717,48829,46749,42621,38461,34301,32220,30141,30140,30140,30140,30141,30141,30141,32221,36349,40509,40541,40541,38461,40541,44670,40541,40541,40573,42654,44670,46781,48861,44701,36413,34333,34333,34365,34365,34365,34365,34333,34365,34365,34365,36413,36414,36446,38493,40573,52989,57149,59229,61310,61342,61310,61342,61310,59229,57149,57116,55036,55004,55004,55004,52955,52923,50844,48828,48861,46813,46814,46814,46814,46814,44766,44766,48862,52989,55101,57181,59229,55101,50941,48893,50941,50909,50908,53021,57181,59229,59197,59197,55068,48828,29974,8941,10891,10891,4617,4585,6665,6665,8778,8778,6633,4552,6633,6665,6665,8713,10793,8713,8745,6600,4584,6632,6632,4616,4584,2504,4552,4519,4487,2439,2439,6665,8745,6665,4519,391,2439,4487,4487,2439,4454,4454,2406,2407,4422,4454,4454,4389,4421,4421,4454,4422,4421,2341,2341,2308,6502,8647,4421,4389,4324,2276,4324,4324,4325,4389,4422,4421,4357,2276,4324,2276,4292,2276,2277,6502,4323,2275,8612,6532,6532,6564,10693,10660,10693,8612,8612,8580,6436,6468,6370,2145,4291,14889,35985,52791,50678,48565,48597,48532,40114,55068,63389,48565,50612,29418,40147,54937,56853,29451,14595,12515,25257,14498,12482,20869,20707,14628,25225,33740,44273,44306,18853,12547,14790,18756,25421,48433,48400,37868,35820,42225,42093,42094,42094,42094,42093,37769,18980,21124,12769,8545,10626,6433,20},
{36348,40477,44637,46717,46749,48829,48829,46750,42589,38461,34301,32220,30141,30140,28092,28092,30141,30141,30141,30173,34301,38461,40541,40509,40509,40541,44670,38493,38461,40541,42622,44669,46750,46781,42621,36413,34333,32285,36413,38493,36413,34365,34365,34365,34365,34365,34398,36446,36446,36446,38526,50909,55069,57181,61310,59262,61310,61342,61310,59197,57149,57116,57084,55004,55035,52956,52955,52923,50876,48828,48861,46813,46814,46814,46814,46814,44798,44798,50941,55069,55101,59229,59229,57149,52989,50941,50941,50908,50908,50908,52988,57149,59229,57149,38393,19474,17295,17294,13068,15149,13036,10891,10923,10956,10988,10924,10923,8842,10955,13036,13003,8842,10955,10923,10890,10891,10890,10890,8842,10923,12939,8810,8778,10826,6665,2439,4519,2407,2440,6697,8778,8713,4551,4519,4487,4487,4455,4454,4422,2406,4422,4422,4422,4389,4421,4453,6501,4454,4454,2341,2341,2309,6502,8615,4454,8582,4292,2276,4292,4324,4389,4422,6502,4421,4357,2276,2276,2243,2243,2244,2244,6470,4324,6500,8612,6500,6500,6531,12773,16934,12741,8676,8579,8580,6435,6436,6403,4258,6372,23210,48565,40210,33970,48598,44307,44274,21163,44569,63324,52824,46321,25127,40147,54936,61177,35692,16644,14531,23145,16578,14562,18723,22787,18691,25095,35689,46419,42127,23079,14660,20869,20836,33807,46353,48400,42094,37901,42193,42126,42094,42094,42094,42093,39948,31560,19077,14850,10626,8578,10626,35},
{36348,38428,44637,48797,48829,46749,46781,44701,42589,38461,34269,32220,30140,28092,28092,30140,30141,28093,30141,32221,36349,38461,40541,38493,38461,40541,42621,38493,38461,42589,42621,42621,44701,44734,40573,36413,34333,34333,42589,44668,42589,38493,36413,34365,34365,34365,36446,36445,38494,38494,36446,46749,55069,57149,59230,59230,59262,61310,59262,59229,57149,57116,57116,55036,55036,52987,52955,52923,50876,48828,48861,46813,46814,46814,46814,46814,46814,46846,50941,55069,57149,59229,59229,57181,53021,50941,50941,50940,52956,50908,50876,55069,44667,21619,15279,19440,17359,17327,13101,15214,15247,15214,15181,13101,13069,15117,15149,15117,13101,13133,13069,13036,13101,15149,15117,11021,10988,10988,8940,10988,8908,8843,10956,13036,8843,2439,4649,6600,2375,2472,4552,8713,8745,8712,6535,4455,4455,4455,4454,4454,2374,4422,4422,4421,4454,4453,4453,4454,4453,2341,2341,2309,4422,6502,6502,8582,4324,4324,2276,4324,4389,4422,4454,6470,2341,4356,2276,2243,2243,2276,2244,6470,2243,2307,8613,10660,6499,6499,12773,12774,10693,8644,8580,8579,6468,2242,2178,12645,6405,44437,52823,25288,14888,38132,48532,27305,16937,44535,61275,54936,33611,25127,38066,54936,63291,50547,18756,10435,16969,18755,10402,16709,22786,12515,23079,35722,46483,40112,29417,18918,16707,16708,38064,44272,46287,44239,39949,37967,44272,42094,42094,42094,42094,40013,33640,16963,12770,12770,8513,8513,29},
{36316,40476,44637,46781,48829,48797,46781,46750,38461,38461,34269,30173,30141,28093,28092,30141,30140,30140,28125,32220,34301,38461,40541,38493,38461,40509,42589,38461,38461,40541,40573,40541,44669,42653,38493,34365,34333,36381,38461,44636,46716,44669,40541,38461,36413,36414,36414,36446,38494,36446,36446,46749,52989,55101,59229,59230,59230,59262,59262,59229,57149,57116,57116,55068,55036,55035,52955,52923,50876,50876,48861,46814,46814,46814,46814,46846,46846,48862,50941,55037,55069,57149,59229,57181,55069,50973,50941,50941,50941,50908,42587,23700,15247,17327,15247,19440,19440,17359,17262,15214,15214,17262,15182,13101,13102,15150,15150,15150,15182,13134,13134,13101,13101,13134,13133,13069,11053,13101,11021,11021,13069,10956,13037,10956,8876,6632,6730,4585,327,6632,2439,359,6632,6698,6697,8680,6535,6503,6535,4454,2374,4422,6470,4389,4421,4421,4453,4421,6502,6502,6437,4324,2308,6502,6502,4389,4324,2276,4324,4292,4389,4421,6502,4422,4389,4324,2244,2212,2211,2276,2243,6469,4323,131,6501,17000,10725,8612,12773,12741,10692,8612,8612,8579,8548,4291,2178,14791,10662,48629,52791,31529,27468,40179,44372,18853,14792,40277,63421,59032,25160,25127,44470,59097,63356,56820,20804,16611,18756,22949,18659,18692,22819,20804,31596,35821,42291,40080,25159,14725,20868,12645,42225,44240,44174,44207,42061,35854,42095,37900,37900,37901,37900,35820,33674,21124,12770,12801,8577,8513,37},
{36348,42557,46717,46749,46749,48829,48829,44669,38461,36381,32220,30140,30140,30141,28061,28093,28093,30141,30173,30173,36349,36413,38461,40541,38461,38461,40541,36413,38461,38493,40541,40541,42621,42621,38493,36381,34333,34333,38461,44636,46716,48797,48797,46716,40573,38493,36446,36446,38493,36446,36446,42653,50941,55069,57149,57181,59229,59262,59229,59261,59229,57149,57148,55068,55068,55036,52955,50875,50875,48828,48861,46814,46813,46814,46814,46846,46846,46846,50941,55069,55069,57149,59229,57181,55101,55069,53021,50973,46780,27894,13200,17327,17327,15246,15247,17327,17327,17359,17294,15214,15215,15214,15214,11086,13134,13102,13101,13101,13134,13102,13134,15182,13134,13102,13133,13069,13101,15149,11053,13101,11053,11021,10989,10956,8876,6698,6698,2472,2406,6665,2504,358,2439,2439,6665,8745,8745,6633,6600,4487,4422,4389,4390,4389,4389,4421,4389,4421,6566,8582,6469,4357,6404,6503,8615,4357,4324,4324,4324,6372,6502,4422,6502,4454,2341,2308,2276,2212,2243,2276,2276,6470,4324,4323,8580,8612,6531,2339,10725,16934,10692,8579,8579,6499,8581,8484,8484,19017,27534,50677,52824,54839,48500,40179,40114,44306,35725,35953,63390,46321,25127,20999,52922,63421,65469,65436,27044,12483,8452,27240,10370,12515,22883,16611,27533,35854,40178,23177,23144,18821,22916,25290,42159,44304,44174,42159,44207,40047,37900,37835,37868,37900,37868,35788,33673,25253,14818,12769,10690,8545,40},
{38429,42589,46749,46749,46749,48798,48830,42621,38461,36381,30173,30140,30141,30140,28061,30140,28093,28092,30140,32221,34301,36413,38461,40541,38461,38461,40541,36413,36381,38461,38493,38461,42621,40573,36413,34365,34333,32285,40509,44668,46716,48797,50876,48796,46716,42589,38493,38494,38494,38493,36446,38526,44733,48861,53021,57181,57181,59229,59230,59262,59261,57149,57116,55068,55036,55036,55003,52923,50875,48828,48861,48861,46814,46814,46814,46846,46846,48894,50941,55069,57117,57149,59261,57181,55101,55101,55069,53021,40441,25584,15182,15247,15246,15214,13198,17294,15279,17295,17294,15246,15246,15182,15182,13102,13101,13134,13134,13101,15150,13134,13134,13134,13102,13101,13133,13069,13069,15149,13101,11021,13069,11021,8908,10988,10956,4586,6633,4552,359,6698,6632,2407,4455,4487,4487,2374,4552,6697,8745,6632,6503,4421,4389,4357,4389,4389,4357,4389,4454,8647,6469,4357,4357,6502,8550,6405,4324,4324,4292,4324,6470,4390,4454,4454,4389,4357,2276,2244,2244,6502,4388,8517,4324,6469,12806,12804,8547,4387,10726,23240,10692,8612,6467,8547,6435,12678,21129,31694,37968,46452,44306,48565,42259,40212,31629,44503,42194,42292,63389,33579,25127,21097,61341,63422,65501,65501,35463,14563,14596,27304,14498,14531,22884,18659,23243,35952,38000,16773,10467,14725,18756,25453,40046,44272,42126,42094,42159,40047,37868,37868,37868,37868,37836,35755,33674,23206,12739,10690,10690,8514,36},
{38396,42589,46749,48798,46749,48829,46750,42589,36381,36381,32220,30140,28093,28061,28093,28092,28092,28092,30141,30173,34333,36413,38461,40541,36413,38461,38493,36413,36413,38461,38493,38493,40573,38493,36413,32285,34333,36413,42588,48764,50845,48796,48796,46748,44668,44668,42621,40573,38494,38494,38494,38494,38526,40573,42685,48861,55069,59197,59229,59229,59229,59197,57149,57116,55068,57116,55004,52923,50843,48796,48829,48861,46813,46814,46814,46846,46846,48894,50941,53021,55069,57149,57181,57149,55069,52989,55069,52989,48795,34036,27696,21423,13166,15247,15246,15246,15247,17327,17294,17294,15246,17294,15214,15182,15182,15182,15181,15182,15182,13133,13134,13134,11053,13134,13101,11053,13101,15149,13101,13101,13069,10988,10956,13036,10923,2472,6698,2537,2407,6698,4553,2407,4455,2439,2439,294,294,359,4584,6697,8713,6632,4454,4389,4357,4389,4389,4389,4422,8615,6470,4389,2309,6502,6502,4389,4324,4324,4324,2277,4389,6438,6470,4422,4357,2309,2309,2244,2244,2309,4389,6470,4356,6502,12808,17032,12773,8579,12872,29545,12772,8579,6467,8516,12677,6372,8484,25419,23371,33905,33871,35986,42259,40212,31662,42356,36019,46551,52824,23112,23112,25388,63422,65470,65502,65501,41866,14563,12515,25289,14498,14530,20771,20706,21065,35952,31628,16773,8387,18885,16708,33872,39981,40046,44175,44175,40014,40047,37900,37900,37900,37900,37836,35755,33609,16867,8482,10626,8578,10594,40},
{36348,44637,46717,48829,48829,48830,44701,40509,38429,36381,32188,28093,28092,30108,30140,28092,28093,30141,30141,30173,34301,38429,38461,40541,38461,38461,38461,38461,38461,38461,36413,38461,38493,38493,36381,34333,34333,38460,46684,48796,50844,48796,44668,46716,44668,44668,44669,42621,38493,38494,38494,38526,38526,38526,38526,44701,53021,57149,59230,59229,59229,59197,57148,55068,55036,57116,55036,52956,50843,48796,46749,48861,46845,46846,46846,46846,48894,48894,52989,52989,55037,57149,57149,57149,55068,52988,52988,52989,52988,36084,29777,34036,27697,17262,17262,15311,17327,17327,17294,17327,15247,15246,15214,15150,15182,15214,15182,15182,15182,13134,13134,13133,13101,13134,13102,13069,13101,13101,11053,13069,11021,10989,10989,10988,10923,2504,4585,4552,2407,4585,4520,4455,4487,4488,4487,2406,261,229,2310,4487,6665,6697,8713,8616,6502,4389,4357,2341,4486,6502,6470,4389,4389,8550,4454,6437,4324,4324,6437,6470,4421,4454,6534,4422,4389,4357,2309,2276,2244,4389,18983,21064,18919,21097,23112,18951,21095,18982,25386,42192,25286,16932,16868,21128,31595,16805,21064,23271,16999,23275,21094,12873,40211,40179,44307,25486,42325,40178,48467,33677,37935,44307,63454,63389,65502,65501,48270,14563,8387,25322,12449,10402,18788,20738,18919,33807,25225,18886,14693,23013,18789,40144,39981,37965,44239,42127,40014,42062,37868,37868,37900,37868,35755,35755,35722,18980,12738,8546,6434,12707,48},
{38429,42589,46717,48829,48830,46782,44702,40509,38429,36381,32220,30141,28092,28092,28060,28093,28093,30141,30140,30173,34301,38461,40509,40541,38461,36413,36413,38461,38461,38461,36413,38461,38493,36413,34365,34333,36381,42556,46716,48796,50876,50876,46716,44668,42589,42621,44669,44669,42621,38494,38494,38526,38526,38526,38526,42621,55069,59197,59229,59229,59229,57149,57116,57116,55036,55068,55036,52955,50875,48796,48796,48861,46814,46814,46813,48894,48894,50942,52989,52989,55069,55069,57149,57117,55069,55068,55036,52988,55036,48697,36051,31923,31956,31859,25552,15182,15246,17327,15246,17327,15247,17327,15246,15214,15182,15150,15182,15214,13166,11086,13133,15149,13101,13102,13101,13102,13069,13101,13069,13069,13069,10989,13037,13036,10956,4617,4585,4552,2440,4552,4520,4487,4487,4488,2407,2406,2341,2309,2309,2309,2342,4519,8713,8713,8680,6599,6470,4389,6502,4454,4389,6405,6405,6470,4454,6405,6340,4324,8615,8614,4454,4454,6502,4421,4389,4357,2277,2276,2244,19115,54937,54937,54905,54904,54904,54904,54872,54872,54871,54872,54871,52790,54806,52791,52791,52758,52758,33738,21225,40145,29512,12775,38098,38098,40178,21097,25486,42291,52627,44112,40179,54970,65502,65469,65502,65502,54674,18691,20739,25127,18723,18691,20803,22818,22982,33807,21031,18951,16805,22948,21032,42127,40013,35852,40047,42126,40079,37868,37868,37868,37868,37835,35755,35723,35722,21091,12770,8514,6401,10626,48},
{36348,42557,46749,46749,46717,46750,44701,40541,38429,36349,30172,30141,28092,28060,28092,28093,28093,30141,30173,30173,34301,38461,40509,40541,38461,36413,38461,40509,40541,38461,38493,38461,38461,36413,32285,34333,34333,40508,44636,48796,50876,50876,48796,44668,42621,42621,44669,44669,44701,40574,40574,40574,38526,38526,40574,46749,55037,57149,59229,59229,59229,57149,57116,55068,55036,55036,55068,55035,52923,48796,46780,48861,48861,46813,46846,46845,48894,48926,50941,52989,55069,55069,57149,59229,57149,57117,52988,52956,52956,52956,50843,44503,34003,31891,29811,29810,23503,19278,17294,17327,15214,17327,17295,17294,15214,15214,15182,15214,13134,13134,15182,15182,13101,13101,13101,13101,13101,13101,13069,11053,13069,10989,10988,10988,8907,2537,4617,4553,2440,4520,4552,4487,2439,4487,2439,2406,2342,261,2309,2309,2309,2309,294,4520,8713,6664,6600,6567,6567,6502,4357,6405,4422,6534,6470,4389,4324,4357,8681,8614,4422,4421,6502,4421,4389,2309,2309,2244,2244,23406,54970,57051,50745,44406,52793,52793,50745,52793,52793,52825,52793,52825,52825,52825,52825,52825,54971,42159,31725,57016,23176,21129,40211,42292,44273,37968,35854,38032,48272,44046,42259,59196,65534,63421,65502,65502,50481,31334,31269,25159,14758,20999,23046,22981,25127,31661,16740,12612,14790,20836,23308,40014,42094,35788,35821,40046,37999,37868,37868,37868,37868,37803,35722,33674,25222,16867,8546,8546,8514,8578,43},
{34268,40477,44669,46717,46717,44670,44701,40541,36381,36349,32220,30140,28092,28092,28061,28061,28093,28093,30140,30173,34301,38461,40541,40541,36445,36413,38461,38461,40541,38461,38461,38461,36413,34365,34333,34333,34365,38461,40541,46748,48828,50876,50876,48796,46717,42621,44669,46716,44701,42621,40574,40574,38526,38526,40574,48829,53021,57117,57181,57181,57181,57149,57116,55068,55036,55036,57116,55036,52956,48828,48829,46813,46813,46814,46846,48894,48894,48926,50942,55037,57149,59229,61310,61342,61310,59229,55036,55036,55036,52956,52924,50875,50810,42390,31891,29843,29843,31891,21423,15214,17295,17295,17295,17294,17295,13166,15214,15214,15182,13134,15182,13102,11053,13101,11054,11053,13101,11053,13101,13101,13069,10989,11020,10988,10956,2536,6665,4552,2407,2504,4520,4487,4487,4487,4455,2406,2342,261,2309,2277,261,261,229,261,2374,6600,8713,6600,6568,6502,6470,6437,4390,6502,6438,4389,4357,4389,6535,8615,4422,4389,6470,4390,4389,2309,2244,163,6372,25356,44437,54970,50680,40148,52793,52793,50745,52793,52793,50713,48600,44407,42327,42327,44407,44440,52890,57082,59130,59162,46354,48466,48533,44340,48597,44273,42160,40080,27404,42031,42292,61341,65502,63389,65502,65502,46125,33415,31335,27306,16871,23113,23113,23048,23145,27403,16740,10532,18918,18853,25355,25322,42158,39981,42094,39981,40047,37868,37868,37868,35787,35723,35690,33642,29383,25255,14787,12739,8481,8578,48},
{32220,36348,42589,46749,44701,44637,44669,40509,36349,34301,30172,30141,28092,26044,26044,28061,28093,28093,30140,30173,32253,38429,40541,42621,38461,36413,36413,38461,40541,38461,36413,36413,38461,34333,34333,34333,36381,36413,38461,46716,48828,48796,48796,48796,48796,46748,46716,46716,46748,44701,40574,40606,38526,38526,40606,46781,52989,55069,55101,57149,57149,57149,55068,55036,55036,55036,57116,55068,52956,48828,46781,48893,46845,46813,46846,48894,48894,50974,55069,61310,63422,61342,61342,61310,59261,59229,57149,55068,52956,55036,55036,50875,50876,52924,48730,38197,31955,31956,31956,29810,19342,15182,17294,17295,17294,15214,13166,15214,15214,15214,13133,13101,13069,13101,11053,13101,13101,13101,13101,11053,13069,13069,10989,11020,8876,2537,4618,2537,4487,4552,6568,2439,4487,4487,2407,4422,2342,2342,2309,2309,2309,2309,2309,2277,261,2309,2407,6600,6600,6535,4518,4422,4422,4389,6437,4389,6437,4357,4422,6502,4421,4390,6502,4389,4324,2244,2211,2211,12711,23177,44371,54970,50648,44374,52793,52793,52793,52793,42229,31828,31795,27537,25425,25424,25425,27505,46617,57083,59163,59195,52791,50645,38066,44307,48565,48434,40047,16903,29549,46354,40179,61342,63421,63389,65502,65469,37705,31334,31269,29254,25094,25094,22981,25028,27077,27273,20999,20901,20901,25160,31596,25290,40112,42226,48498,39981,40078,39916,37868,37868,35787,35722,33642,33610,31496,31561,18980,8546,14819,10658,44},
{32188,32220,38428,44669,44702,44669,42589,38461,34301,34301,30141,28092,28061,28061,26044,28092,28092,28093,30173,32221,32253,36381,42589,42621,38461,38461,36413,36413,40509,38461,36413,36381,38461,34333,34333,34333,36381,36413,36413,40541,44668,46748,46748,46716,48764,48796,48764,46716,48764,46749,42654,42654,38558,40574,40606,42654,48829,55037,55069,55069,57117,57117,55068,55036,55036,55036,57116,55068,52956,48829,48861,48893,48893,46814,46846,48894,50942,55102,61310,63454,63422,63422,63422,61342,59261,59229,57149,55036,52956,57116,59197,55036,52924,50875,50843,50843,46616,36149,34004,31956,34036,27729,19278,15214,17294,17262,13134,15214,13166,15182,13134,13133,13101,13101,13101,13101,13101,15149,13101,11021,13069,11021,10989,8940,10923,4585,4617,4552,4519,4584,6568,4487,2407,4487,4455,4454,2342,2342,4358,2342,2309,2309,2309,2309,261,2341,2341,2374,4486,6568,6567,4422,4390,6502,6469,6469,4357,4357,6535,6502,6470,4422,6470,4389,2309,2276,2244,2276,16936,21097,50645,54970,52793,44373,52793,52793,52793,52793,40116,27473,38005,38037,37973,35892,35892,37940,48697,57083,59163,59162,54871,40014,27566,40081,48565,46418,37934,16870,50743,46387,31695,57148,63389,63389,65502,63290,33415,33382,31302,29222,25094,25062,22981,27141,27076,27110,27109,25029,25029,31498,35854,31562,33806,42160,48466,42127,39981,39948,39916,37868,37835,37835,39915,35722,18884,23143,21060,10626,12803,10658,42},
{30140,32188,38428,44637,44669,42621,40509,36381,34301,34301,30141,28092,28060,28061,28093,28060,28092,28093,30173,32221,32253,36381,42589,42622,38493,36413,36413,36413,36413,38461,36413,36381,36413,36381,32317,34333,36413,36413,36413,36413,36413,46748,46748,46716,48764,48796,48796,46748,48796,48829,44702,42654,40606,40574,40606,42654,42654,48829,52957,55037,55037,55068,55068,55036,52956,55036,55068,55068,52956,48861,48861,48861,48894,46846,46846,48894,50974,59262,63422,63455,63422,65502,63454,63422,61342,59229,57149,55036,52956,59197,61309,57149,52956,52923,50875,48795,48763,48730,48665,38165,34004,31988,33971,27697,17229,15182,13166,15214,13134,13134,15182,13101,11053,13101,13101,13134,13069,15149,11021,11021,13069,10989,10988,6827,8843,4682,4617,4584,4487,4584,4520,4487,4455,4487,4455,4422,2342,2342,2341,2342,2309,2309,2309,2309,2309,2309,2341,293,4389,4422,6502,4454,4422,4422,6502,10761,8615,4357,4454,6535,4422,4390,4422,4421,2309,4324,2244,10662,23177,23242,52757,54970,52793,44341,52793,52793,52793,50681,27471,25394,31700,35893,37941,35925,35925,37973,48697,57083,54969,57082,54872,42062,27533,38033,46453,40048,27436,16935,50776,33807,29647,50842,65469,65469,65502,56886,31366,33383,29222,27142,27142,25094,25029,27109,27076,27141,27109,27109,27142,33611,50676,42028,25420,42258,44239,42160,39948,39948,39981,35756,46353,39981,40013,39981,39948,37867,18948,14851,10690,10690,41},
{30140,32220,36349,40509,42621,40541,38461,36349,34301,32253,30140,28092,28060,28092,28093,28093,28093,28093,30173,32221,32253,36413,42589,44670,38493,38461,36413,36381,36413,36413,36413,34365,34333,34333,34333,34365,36413,34365,36413,34365,38493,44669,46748,46716,48796,48796,48796,46749,48829,48829,48830,42654,40606,40606,40606,40638,40606,46749,50877,52989,55037,55036,55036,52956,52956,55036,55068,55036,52956,48861,46813,48861,48862,46846,46846,48894,53022,61310,63422,63454,65503,65503,65503,63454,61342,59261,57149,55068,55036,57149,61309,57116,52988,52956,50875,50843,48763,48763,55004,50843,44504,36116,34004,34036,31923,25584,15117,13166,13134,15181,15182,13134,13101,11053,13101,11053,13101,13101,13069,11020,13069,11021,8940,8875,8843,6698,6633,6633,4519,4584,4520,4487,4455,2439,2439,4454,2374,294,2342,2310,2309,2309,2309,4357,2309,4358,4357,2341,4390,4389,4357,4422,6502,6470,6470,6567,12841,10728,8648,6502,4422,4390,4422,4421,2341,12776,23145,16936,33709,12710,50645,54970,52793,46454,52793,50745,52793,46487,12712,19119,29587,25362,23217,21136,21137,23217,44472,57083,59163,59162,54936,50612,27468,40081,48597,37934,31661,16902,42324,44274,40113,52922,65502,65502,65502,52562,31334,31302,29221,27142,27109,25093,27174,29222,27141,29190,27141,27141,29222,37903,50742,35853,31725,44240,42158,42159,41997,33643,33805,37998,50579,42029,40013,40013,42061,39948,27334,19043,12770,10690,39},
{30140,30140,34268,36381,38461,38461,36381,36349,34301,34269,30141,28061,28092,26044,28093,28093,30141,30173,30173,34301,34301,38429,42621,44702,40541,40541,36413,36413,38461,38461,36413,34333,34333,34333,34333,32285,36413,36413,36413,34397,38493,42621,46748,48796,48796,48796,48796,48829,48861,48829,48861,44734,40606,40606,40606,42654,40606,44701,48829,52956,55004,55036,55036,55036,52956,55036,55036,52988,50908,48861,46813,48861,48893,46846,46846,48894,53022,59262,63422,63454,65535,65535,65503,65503,63422,61310,59229,57149,57116,59229,61309,59229,55036,52956,50876,48795,48763,50843,59229,52923,50843,50842,44503,36084,34036,34036,31955,23471,15149,15182,13166,13102,11053,13101,13102,13102,13101,13101,13069,10988,13069,11021,10956,8875,10891,6698,6665,6633,2439,4584,6568,4487,2439,4487,2439,2407,2374,2342,2341,2341,4357,2341,2309,4389,2309,4358,4390,4389,4390,4390,2310,4422,4454,2373,6470,6470,6503,10793,10793,10760,6469,8582,10663,8550,6470,2341,10696,21097,33709,10533,50645,54970,50712,46454,52793,52793,52793,38003,8486,16974,19023,16911,18958,23119,31473,35666,48600,57083,59163,61211,33807,42324,29515,38032,54904,44273,33807,16902,44404,52791,52758,57083,65502,65502,65502,41964,29221,29189,29189,27141,27142,25061,29222,27207,27142,27206,29222,27142,27174,33742,40178,27370,25354,42159,42094,44207,42029,29450,37967,42225,52659,42028,42061,42093,42126,42126,39981,27365,14850,8577,39},
{28060,28060,30108,32220,34300,36380,36381,34301,34300,34268,30140,30109,26045,28060,28092,28093,28092,30141,32253,34333,36381,38429,42621,44702,42621,40573,38493,36381,38461,38461,36413,34365,32285,34333,32285,36413,42589,38493,38493,40541,42621,44669,46748,48796,48796,50844,48828,50909,48861,48829,48829,46781,42686,42654,42654,42654,40606,42654,48829,52956,55036,55069,55036,55036,55036,55036,55036,52956,50908,48861,46813,48894,48894,48894,46846,48926,53022,53021,57181,63422,65503,65535,65535,65535,63455,63422,61309,59229,59229,61309,61309,57149,57116,55036,52956,50843,48763,48763,57116,52956,52923,57116,55036,50810,42390,34036,34068,34036,31890,23438,17197,15150,11021,13102,13102,13102,13101,13101,13101,10988,13069,11021,8940,8875,10923,4618,6697,6633,2471,6632,4520,4487,4487,4487,4487,4454,4423,4390,4390,6503,4422,4422,4422,4422,4454,6470,4390,4357,4390,4422,4357,4454,4422,2309,2341,4422,4422,8616,10793,12841,10630,12776,14823,10630,8582,4357,12678,25323,35789,8452,50645,54937,52793,52761,52761,52793,48632,23245,10568,16877,16877,25198,39761,43922,39730,37617,46488,57082,57082,59163,29549,44404,31562,38001,57082,48467,33774,16935,44437,54872,52759,50809,63454,65502,65469,35560,29156,29189,29221,27173,27142,25061,27108,25028,27174,27141,27109,27142,29255,33643,38034,29548,27499,44240,42160,42093,42029,27370,46386,48499,52626,44142,42061,42094,42126,44173,37835,35720,19010,12738,37},
{28028,28028,28060,28060,30140,32188,36381,34301,32221,32221,30140,28093,26044,30109,28093,28093,30141,30173,32253,34333,36413,36413,40541,44702,42654,42621,38493,36381,38461,38461,38461,36413,34333,34333,32285,36413,44669,44669,42621,44669,48797,46749,46749,46748,48796,50844,50876,48861,48862,48861,48829,48861,44734,42654,42654,42654,42654,42654,48829,52957,55069,55069,55068,55036,55004,55036,55036,52956,50909,46813,46814,48862,48894,48894,48894,48926,50974,55101,57149,61342,65502,65535,65535,65535,65502,63422,61342,59229,57181,59229,59229,59229,57148,55100,52956,50843,48763,48731,52956,55068,55036,59196,55036,52923,48795,48697,42390,34068,34036,34036,31858,21325,13101,13101,13101,13101,15149,13101,13102,11021,13069,13069,11021,8843,10859,8778,6730,4585,2471,6633,6600,4519,4487,4519,4487,6471,4455,4454,4422,2374,4422,6535,6567,4454,4486,6534,4422,4390,4422,4422,4390,6503,4454,2309,2309,2310,2309,4390,8648,14921,10663,12743,12711,8582,6437,2244,31727,33709,39982,8451,46452,54938,52825,52793,52793,52793,46487,12711,12682,18925,35568,43890,43826,41713,35505,29264,42359,57115,59131,54970,57018,57049,33740,44405,59162,48500,33709,12742,46582,54904,46485,55003,63454,65502,63291,29189,27108,27109,27142,27174,25093,27109,27076,25028,25061,27141,27109,27109,29320,25355,44372,31727,44273,46320,42192,42061,39916,29548,54707,52691,50513,46319,40013,42094,42126,42126,39948,42126,21026,10658,47},
{28028,28028,26012,28028,28060,30108,32252,34301,32221,32220,30140,28092,28092,28092,28093,28093,30141,32221,34301,38461,38493,38461,40509,44702,44702,42653,38493,36413,40509,42589,44669,42589,40541,34365,34333,36413,44669,46749,46749,48797,50877,50877,48829,46749,48797,48796,48796,48829,48861,48861,50910,48861,44734,42686,40606,40606,40606,42654,48829,52989,55069,55069,55069,55036,52956,55004,55036,52956,50909,48861,46814,48894,48894,48894,48894,50942,55102,59230,59262,61342,65535,65535,65535,65535,65534,63454,61342,59261,55101,57149,59229,59229,55068,50875,50876,52891,48763,46682,52923,55068,59196,59196,55068,50907,50843,50875,55004,48729,38229,34068,34068,34036,23471,19246,19311,19310,19343,17262,19310,19310,19310,17230,17262,19278,17197,12972,8778,8713,6600,6632,2406,262,261,2341,8615,2341,4389,2341,2341,2341,2309,2309,4389,4389,2341,4357,6438,4390,4357,4389,4357,4422,4422,4357,4357,4357,4357,6437,12711,10695,10662,12744,10631,8550,4324,6404,33905,50678,42127,6371,50612,54938,52825,52793,52793,52793,35890,14794,16876,35633,45971,48147,48116,45971,39762,33521,42327,57083,54937,55003,59196,59195,35788,46518,61210,46387,37934,14757,46583,42161,29711,29810,61406,65534,56918,29156,27109,27109,27141,27174,25061,25061,25061,22981,27141,27109,27110,27142,29319,21163,44438,36051,46517,44272,44240,42061,35723,35886,54673,54674,42094,48433,42061,42093,42093,42093,35722,27498,23140,12738,43},
{25980,25980,25980,28060,28028,30108,30172,34301,32221,30172,30140,28093,28061,28093,30109,28093,30141,32221,36381,38429,38493,36413,38461,44670,46750,46749,46717,44701,46749,48829,48829,46749,46749,42589,42589,44637,46749,48797,50877,50877,48828,50908,48829,48797,46749,48764,48796,48796,48829,48861,48894,48894,48862,44734,40606,40606,40606,40606,46781,50941,55069,57149,57149,55068,55036,55004,52988,52956,48861,48861,48861,48894,48894,48894,50942,55069,59229,57181,61310,61342,65535,65502,65535,65535,65502,63454,61342,61310,57149,55036,57149,59229,52988,50875,50843,50875,48763,46682,50843,55036,59228,59228,55068,52923,50843,50875,50876,50908,52955,48696,38197,34068,23536,19278,21359,19343,21390,17262,19278,21359,19310,17263,19278,19278,15182,10923,8811,10762,6632,4552,4389,2245,2244,2309,8550,4389,4357,6437,4389,4389,2308,4357,6405,6405,4357,4357,4389,4389,6405,4356,4357,4357,4357,4389,4324,4357,4357,8550,14856,12743,10630,8583,8551,8518,6405,6437,19244,35985,40047,14757,52693,54970,52825,52793,52793,52793,37872,14827,27343,43955,48147,43921,37616,33424,27184,23087,40247,52889,52857,55002,57083,57083,33740,42357,59130,46354,38000,14791,40179,25257,16967,6533,55101,65534,52659,27173,27174,31334,29222,25159,29287,27142,25061,27174,25093,29221,29190,27207,31564,19082,29549,38197,38131,38032,44207,42029,31530,42159,50545,54805,29482,46418,39981,42093,42093,44141,44141,35721,21060,10658,38},
{25980,25980,25980,25980,28060,30108,32188,34269,32221,30173,30141,30109,28061,28092,28093,30141,32221,34301,34301,36381,36413,36381,36381,42621,46750,48829,50909,50909,52989,52989,48861,48797,48829,46781,48829,48829,48829,50876,52956,52957,50908,50876,48829,48797,46748,48796,48828,48796,48829,48861,50910,50942,48894,46782,42654,40606,42654,40606,42654,44733,52957,57149,57149,55068,55004,52956,52988,52988,48861,48861,46813,48894,48894,48894,50974,59229,59261,57181,61310,61342,63422,65502,65535,65535,65502,63454,63422,63390,59229,52956,55036,55036,52956,50876,50843,50875,50843,46682,48795,55035,59228,59196,57116,52955,50875,50875,50876,50875,50875,52988,50843,46616,29745,23406,23406,23471,23471,21390,21390,23439,21391,21358,21358,21358,19246,15020,10826,10794,8680,8680,6470,6372,4356,4421,6534,4389,6437,6437,6405,6405,6404,6404,6405,6437,6404,6405,6405,6437,6404,6404,6405,6405,6404,6404,6404,6405,6436,6534,16936,19082,12710,4357,6372,6372,6404,8517,4356,27533,40015,37934,54837,54969,54873,50745,50681,48633,46487,37970,35826,35826,35794,35793,35761,33681,31633,31633,44471,57051,54938,59164,59196,59163,33739,40211,46485,37935,25355,14791,25128,18854,12740,6500,46714,65534,46223,27174,29352,33512,29286,29320,33545,27207,27174,29287,22981,29190,29221,23046,38066,19082,27304,33840,25355,40178,44174,42029,27370,48432,50513,52759,25257,48530,42028,42093,42094,42126,42125,39947,21124,12770,36},
{25980,28028,25980,25980,28060,30109,30140,32221,32221,30173,28092,28061,28093,28093,30141,30173,34301,36381,34301,36381,36381,36413,36413,42589,46749,48829,50909,55037,55101,55069,53021,52989,52989,50909,50909,55037,55069,52989,55036,52988,52956,50876,50876,50844,48796,48796,48828,48796,48796,48829,50909,48861,50942,46814,42686,40606,42654,40606,40606,42654,48829,53021,55069,55036,52988,52956,52956,52956,48861,48861,48861,48894,48894,48894,50974,57181,59262,59262,59262,57182,59262,63454,65503,65503,63454,63454,63422,63422,61342,52988,52988,55036,52988,52924,50843,52923,50843,48731,50843,57116,59228,57116,55035,52923,50875,50875,50875,50876,50875,52956,52956,50907,55035,52923,52891,52891,48762,48762,48762,48762,50810,50810,48730,46681,40376,36085,31923,21391,23471,21325,21389,29647,27534,15018,15018,19147,14953,29451,33513,33513,33513,35593,35593,35561,35561,35561,35561,35561,33512,33480,33480,33481,31433,31400,31400,31400,31400,31401,31497,33578,31401,27239,27239,29255,18821,6404,18951,33774,46353,54837,54805,54937,52825,46520,46487,27569,31860,27505,23281,33748,33748,33716,33715,31635,29555,29587,42392,57051,52857,54970,57116,55068,33772,38098,61210,37935,27468,23210,35726,29418,10626,8612,40439,65502,39786,33513,33544,33512,31432,33512,31432,31367,29319,29287,29286,29221,29254,33579,44373,23243,27337,35952,40112,48531,42060,37868,29515,50512,52593,50547,37835,46385,44142,40013,42094,42125,42125,42028,25286,12738,40},
{25980,28028,25980,25980,28060,28060,30141,32189,32221,32221,28092,28093,28061,28092,28092,30173,32253,36381,34333,34333,34333,36381,36413,40541,46750,48829,52957,57150,57182,57182,57150,57150,57149,57150,57150,57149,57149,57149,55069,55069,55036,52956,52924,50876,50844,50844,48796,48796,48796,48828,48829,48861,50942,48862,42654,40606,42654,40606,40606,42654,44734,50909,52989,52988,52956,52956,52956,52956,50909,48861,48861,48894,48894,50942,50942,53021,55101,57181,59261,57182,59229,63422,63454,63454,63454,63454,63454,63422,61342,55036,55068,55068,55036,52956,50875,50875,52923,46683,48763,59196,59228,57116,52956,50875,50875,50875,50875,50875,50876,50875,50875,52923,52956,50907,50908,50876,50875,48795,50875,48827,50875,50875,48827,48795,42522,36183,34037,27698,25584,21423,21422,31857,29712,15116,17164,31824,21227,35724,23112,20999,20999,18886,18853,20999,20999,20999,20999,18854,20999,21031,23079,21032,18886,20999,23080,23079,23112,20999,20966,23112,23112,25160,23112,23047,20966,14758,27403,35855,52724,56950,52725,54970,52825,44407,46455,25293,23279,29519,21134,31700,35828,35796,35796,33748,31636,29588,42360,55002,52856,57050,42259,25551,23241,31792,59129,27402,27469,31596,44274,27369,10594,8644,32020,63355,35592,33545,35625,31432,33512,33545,33512,31431,31431,31335,29221,29254,29189,29484,48663,27469,23176,16903,21162,46418,42029,35723,37999,52592,54871,44241,46254,44239,46319,42061,42093,42093,42093,39915,21060,12706,38},
{25980,25980,25980,25980,28060,28060,30108,32188,32221,32221,28092,28093,28093,26045,30141,30172,32221,34301,34333,34333,34333,36381,36413,40541,44702,46781,52957,57150,59262,59262,59262,59262,59262,59262,59262,59262,59262,57182,57149,55069,55037,52988,52956,52924,50876,50844,50844,50844,48796,48796,48828,50941,53022,48894,44702,40606,42654,42654,42654,42686,44734,48861,50909,52956,52956,52956,52924,50908,50909,48861,48861,48894,48894,50942,50974,53022,53021,55069,57181,57149,59230,61342,65535,65535,63454,63454,63422,63422,61342,57149,55068,55068,55068,55036,52923,50843,50875,48731,46714,55068,59196,57116,55004,50875,52923,50875,50875,50875,50876,50875,50876,52956,52988,50907,50908,50876,50875,48795,48827,48795,48827,48827,48827,48827,42522,36183,31957,27698,23536,21391,21423,36019,33873,21356,19243,27599,29613,39982,23112,18919,16871,16806,14725,18887,18887,18919,18887,14693,16839,16871,16871,16903,14693,16806,18919,18919,18919,16806,16741,18887,18951,18919,16839,14693,21031,14789,25290,35822,50644,56886,52724,54970,52825,48600,46520,46455,44343,46488,40117,40086,35796,33684,31636,29587,29556,29555,42360,55002,52824,57115,37934,29646,23207,25453,42226,25323,27436,33773,35886,35820,21093,10755,25648,56951,33480,35626,35625,31464,33513,31432,31400,29286,29287,31367,29221,29254,29221,25193,40277,31629,23209,20932,18950,40079,35820,31595,42192,52560,52659,48499,48334,46319,48432,42061,42093,42093,42061,39915,29514,42192,161},
{25980,23932,25980,28028,28060,28092,30108,30140,32221,32221,30141,32188,30141,28093,30140,30173,32221,34301,36381,34333,36381,36381,38461,40541,46750,46782,52957,57118,59230,61310,61342,61342,61342,61375,61374,61342,59294,59262,57181,57149,55069,55036,52956,52956,52924,50876,50876,50843,48795,50843,50876,50941,52990,48894,44734,42654,42654,42686,40638,42686,44734,48861,50909,50908,52956,52924,52924,50876,50909,48893,48893,48894,50942,50974,50974,53022,50974,53021,55069,55101,57182,61342,63454,59294,63422,63454,63422,63422,63390,57181,57116,57117,55068,55036,52923,50875,50843,48762,48763,52956,55036,55036,55004,50875,50875,52924,52955,52923,52923,52955,50875,52955,52956,50875,50908,50875,50875,48795,46747,48795,50875,48827,48795,48795,42522,36151,29876,23569,21456,17230,21390,31890,33873,21324,17163,27664,38066,40015,31433,29320,29320,29255,27175,29288,29288,29288,27207,25094,27207,27240,27240,27240,23046,25127,27208,27240,25160,23047,22982,23047,25127,23047,23014,20869,23111,16806,27403,35823,50645,56853,54805,54937,52793,50680,44375,44408,44440,44407,42263,40150,39989,35796,33716,33684,31635,31603,40215,54971,50679,48696,52823,52856,42223,35951,42128,31629,29548,40079,25288,27533,23303,12771,17228,50547,27239,29352,33512,31432,29352,31464,27239,27207,27207,25126,27141,27141,25062,21032,33840,35855,21194,23078,23046,18917,18950,29547,52593,52559,54706,54739,46286,50512,50513,42061,42093,42094,42094,42127,56951,57017,219},
{25980,25980,25980,28028,28060,30140,28092,30140,32221,30173,30140,32220,30173,28093,32189,32221,32253,34333,36381,34333,34333,34333,36381,40541,46750,46782,48829,50941,57150,61342,63423,63423,63423,63423,63423,61343,61342,59262,59230,57149,57149,55068,55036,52956,52956,50876,50876,50844,50843,50875,50876,50909,50909,50942,44734,44734,44702,42686,42686,42686,44766,48829,48861,50909,50876,50876,50876,50908,50909,50942,50942,48894,50942,50974,50974,53054,53054,50974,53021,55102,55102,53021,55069,55069,59261,63454,63454,63422,61342,59261,55101,57149,57148,55036,52956,50876,50875,48763,48763,50875,52988,52956,52956,50875,50875,52956,52956,52956,52923,52988,52955,50907,52956,52924,50876,50907,52956,48795,46747,50875,50875,48827,48827,48795,44570,34103,27828,23536,21455,19343,21422,29713,35986,17164,19211,25486,31695,35691,39787,41834,43914,43914,43914,43914,43946,43914,43946,43914,43946,43947,43947,43946,43946,43947,43947,41866,43914,43914,43914,41866,41866,43913,43914,43946,29287,10500,29483,33742,50645,56918,54838,54970,54873,52761,46456,44408,46488,44343,40150,35925,35861,35861,35829,35829,35828,31603,40182,55003,52791,50776,57083,57083,46449,35951,37935,33774,35789,46386,23177,10628,14981,12803,12872,42094,27207,29384,33513,29352,27239,31432,25093,27174,27174,20965,29287,31367,23046,21065,35986,42128,16936,18951,12644,18918,16870,40048,52724,50479,54739,54641,48366,52593,50513,39948,40013,42094,42094,46385,52791,56985,218},
{25980,25980,25980,28028,28060,30140,28093,32188,32221,30173,30141,32221,30173,30173,32221,34301,34301,34333,34333,34333,34333,34333,36381,40541,46782,46782,46781,50909,57150,61310,63423,63455,63455,63455,63455,61375,61342,59262,59262,57181,57149,57117,55069,55036,52956,52956,52924,50875,50875,50875,50876,50876,50909,50909,46782,44734,44734,44734,42686,42686,44766,46813,48829,48829,50844,50876,50876,50908,50941,50973,50974,50974,50974,50974,50974,53054,53054,50974,53022,53054,53021,55037,55069,52988,57181,63422,63422,63422,63422,61342,57149,57149,57148,55036,52956,50876,50875,48763,48763,50875,52988,52956,50876,50876,50876,50876,55036,57116,55004,55036,52956,50875,52956,52955,50876,50907,52956,48795,46747,50875,50875,48827,48827,48795,42522,32055,27796,23568,21455,17230,19310,25552,33905,19180,27599,27567,25453,33676,39786,41866,43914,43914,45994,43946,43946,43946,43946,43946,43946,43946,43947,43946,43946,43946,43914,43946,43946,43946,43914,43946,43914,43914,43946,43946,29286,10533,29483,35822,50612,56918,52692,54938,52825,52793,48600,46488,46520,44407,42263,40086,35893,31700,29587,29555,27507,29555,40247,55003,52857,55002,57083,52890,46514,35918,44339,50646,52692,52726,33709,21127,12869,12771,6499,12741,12677,12676,12612,14725,18918,33644,39852,35626,37772,29352,40048,48401,44142,23144,29582,37968,21162,21129,14758,18951,12710,42192,50644,50447,54739,50480,48399,52593,48432,42061,42093,42093,42093,44272,44404,54904,217},
{25980,25980,23932,25980,28028,30141,28093,32188,32221,30173,28124,32253,32253,32221,32221,32253,36381,34333,34333,34333,34333,34333,36381,40541,46782,46782,44702,48797,57150,59262,61374,63455,63455,63455,63455,63423,61374,61342,61310,59261,59229,57181,57117,55068,55036,52956,52955,52955,52923,52892,50876,50876,50909,50941,48862,46814,44766,44734,44734,46782,46781,46814,48861,48829,50876,48828,48828,50941,50973,50973,53021,50974,50974,50974,53022,53054,53054,50974,50974,55102,53053,55069,55069,55037,55069,61310,63422,61342,61342,61342,61309,57149,57149,55068,52956,52923,50875,48763,48763,50875,52956,52988,50908,50875,50875,50875,52955,55036,55068,55036,52955,52956,52956,52956,50875,50875,50876,48795,48795,50875,48827,48827,48827,48795,42522,31990,27796,23568,21456,15150,19278,29745,29713,17164,23439,27599,23372,35724,39786,41866,43914,43914,43946,43946,43946,43946,43946,43946,43946,43946,43946,43946,43946,43946,43946,43946,43946,43914,43946,43947,43946,43946,43946,43978,31335,12613,31564,40047,50644,58998,56918,54970,52825,52793,50713,44440,46488,46488,44408,42263,40118,37973,35861,33781,33748,31668,40280,55003,52857,52857,55002,42390,44370,25450,44307,44339,31693,21194,27663,23207,12804,12803,8547,6434,6404,6404,6307,4259,14856,37935,42063,37870,42129,31595,40179,50678,50579,27239,25192,27469,25354,27338,23177,29483,27402,46352,54641,50480,52626,46253,48399,52592,50513,46287,40013,42061,40045,35788,40244,56983,216},
{25980,25980,25980,25980,26012,30140,30141,28125,32221,30173,30140,32253,36381,32253,36381,36381,36381,36381,36381,34333,34333,36381,36381,40541,46750,46782,44701,46749,55069,61342,61374,63455,63455,63455,63455,63423,61374,61374,61342,61310,59261,59261,57149,57148,55036,55036,52956,52956,52923,50843,50875,50908,52989,52989,50942,46814,44734,46814,44734,46782,46781,46814,48894,48861,48829,48828,50877,50941,53022,53022,53054,53054,53054,51006,53054,53054,53054,51006,50974,55102,55134,57149,55101,55069,55068,57181,59261,59261,61309,61310,61310,59229,55068,55068,55004,52923,50875,50843,48731,50843,52956,52988,52988,50876,50875,50876,52956,55036,52956,50908,50875,52956,52956,50908,50875,50875,50876,48795,48795,50875,48828,48827,50875,50875,44570,29942,25716,21488,19376,17198,21390,27664,23439,17197,21325,25454,21292,40016,27305,25192,25192,25192,25127,25192,27272,27304,27272,25159,27272,27305,29353,29353,27240,27273,29353,29353,29385,27272,27240,29353,29385,29385,29385,29352,42030,44241,48500,42259,44371,54805,54870,54970,52825,52793,52793,46520,44408,46488,44408,44343,42231,40086,38005,35893,35861,33812,42360,55003,52856,52824,38064,21194,33870,25483,48565,33740,10628,6403,12873,33936,14885,12803,8547,6434,2210,16903,14692,4291,19147,44306,39917,39983,44273,35789,38098,50678,50677,31432,23013,40146,48498,52626,52627,48467,44273,52625,54705,50479,50545,44108,48399,52560,50448,46352,39981,42093,44240,35658,33873,56951,215},
{25980,25980,23932,26012,26012,28060,30173,30172,32221,30173,30172,32221,38429,34333,36381,38461,36381,34333,34333,36381,34333,34333,36413,40541,44702,48829,52990,57182,59262,61343,61375,63455,63455,63455,63455,63423,61375,61342,61342,61342,59262,59262,57181,57149,57116,55036,52955,52956,52955,50843,50843,50876,52957,52989,50942,48894,46814,44766,44766,46814,46814,46814,48894,48894,48861,48829,48861,50942,53022,53022,53054,55102,55102,53054,53054,53054,53054,51006,51006,55102,55134,55134,55069,55036,55068,55068,55069,57149,59261,59261,59261,59229,55068,55036,55036,50875,50875,50843,48731,50843,52956,50908,52988,50908,50876,50876,52956,52956,52956,50876,50875,52924,50876,50875,52923,50875,50908,48795,48795,50843,48827,48795,48827,48827,42522,29943,25748,23537,21455,17198,17230,27632,25519,17164,29744,25453,27566,42161,21032,16904,16904,16904,16839,14791,16903,16903,16904,16871,14759,16872,16903,16904,16872,14759,14823,16872,16904,16872,14759,14759,16871,16904,16904,16871,48598,52758,50580,37967,35854,54869,54805,54970,52825,52793,52793,46455,35957,46520,44408,42327,40247,40182,40150,38070,38038,38005,42392,54971,52856,52825,42127,33904,46514,29709,52790,19013,6467,4322,21162,19079,8547,12803,8546,4387,12742,16936,16904,14824,19050,27500,39885,42064,46354,31596,38065,50678,50677,25159,10565,42290,48498,54673,54705,52658,52659,54706,54737,52560,44206,42060,46319,50479,48334,48465,42028,42061,44306,46223,31694,56984,218},
{25980,25980,25980,25980,26012,28060,30140,30140,32221,30173,30173,32253,36381,36381,38429,40541,38461,34333,34365,36381,34301,34333,36413,40541,44702,46782,55070,61310,61375,61375,63423,63455,63455,63455,63455,63423,63423,61374,61342,61342,61342,61310,59262,59229,57149,55036,55036,52988,52955,50843,50843,50876,50908,52989,53022,50942,48894,46814,44766,46814,46814,46814,46846,50942,50942,48893,48861,50942,53022,53054,55102,57182,55134,55102,53054,53054,53054,51006,51006,53054,55134,55134,55101,55101,55068,55068,55069,57149,57181,59229,61309,59229,55068,55036,52956,50875,50875,50843,48763,50843,52956,52956,52988,52988,50908,52956,55004,52956,52924,50875,50876,52923,52955,50907,52956,50875,50907,48827,48795,48827,48827,46747,48827,48827,42522,31991,27828,23568,21455,17263,17230,25584,23438,23438,33905,29646,31792,37999,19080,31563,23177,17034,27436,14888,23275,14856,19017,29516,31629,14791,14824,19017,21097,23178,23177,23177,16935,16903,16936,21129,18984,21032,21065,21065,46550,52758,50645,52823,52692,56918,54838,54969,52825,52793,52793,52793,50680,50681,44375,42197,40117,40149,42198,42198,42230,42198,46520,48664,54905,54970,57082,55035,48595,31821,50677,12675,8547,6371,16967,8546,10660,6531,10690,4354,6468,6404,6372,8518,29581,37967,41933,42162,48533,46386,48532,48565,50645,37870,16970,42192,48498,54705,54705,54674,54706,54771,56753,52560,39980,44205,44173,48399,46254,52659,42028,42061,42127,50579,40113,54936,221},
{23932,23932,28028,28028,26012,28060,32188,30141,32221,30173,30141,32253,34301,36381,40509,40541,38494,36381,36413,36381,36381,34333,36413,40542,44702,46782,57118,61342,63423,63455,63455,63455,63455,63455,61407,61375,63423,61375,61342,61342,61342,61342,59294,59261,59229,57149,57116,52988,52955,50843,50875,50876,50876,50909,52989,50974,48894,48862,46814,46814,46814,46814,46814,48894,50974,48926,50942,50974,53054,55102,55102,57182,57182,55134,53054,55102,53054,53054,50974,53054,55134,55134,55101,55101,57117,57116,55069,55069,57117,57181,57149,57181,55036,50876,50876,50843,50843,50843,48795,50875,50908,50908,50876,50908,50876,52923,55036,52956,50876,44602,40408,42456,42489,42521,42521,42521,42521,42521,40441,42521,42521,42521,42554,44602,38328,27829,23635,21456,19311,15118,17197,25583,23438,21357,23503,21357,29745,38098,21226,33774,23274,11051,29646,21227,27534,6534,17001,33905,38099,10662,8549,16936,19114,33840,35985,42226,21192,12805,17032,35985,33872,46517,48533,42226,48663,52758,50645,52856,52723,56918,54969,57082,52857,52793,52793,52793,52793,52793,52793,52793,52793,50745,50745,52761,50713,50713,52825,48696,54937,50776,57116,57083,44401,12804,21290,8547,4323,6436,27531,10627,8546,6434,10691,8546,4290,6403,12742,21097,35953,40113,41998,44275,48565,48499,48532,42291,48598,39982,23340,42094,48531,52625,54705,54705,54705,54739,54705,52593,39948,46286,35722,42225,50512,54739,44108,42061,40046,48434,46321,52823,225},
{25980,23932,25980,28028,25980,28028,30141,30173,32221,32221,30141,32221,32253,34301,38461,40541,40541,36381,34333,36413,36413,36381,36413,40541,46750,50910,59230,63391,63455,63455,63455,63455,63455,63455,63455,61375,61375,61342,61342,61342,61342,61342,61342,61310,59261,57181,57149,55036,52955,50875,50875,52924,50876,50908,50941,50942,48894,48894,46814,46814,48862,46814,46814,48894,50974,50974,50974,53022,53054,55102,55134,57182,57182,57182,55134,55134,53054,53054,53054,53054,55134,55102,57149,57181,59229,57149,55068,55068,55068,55101,55068,55068,55036,52924,50875,50843,50843,52923,52923,52956,50876,50876,50876,50876,50876,50875,52923,46649,31957,21294,17002,8648,10728,10729,10729,10696,8583,8551,6504,6536,8584,8584,8584,6536,4455,6503,6503,8551,8550,6471,6502,6470,6502,6502,8551,8550,6502,8551,6469,6470,8550,8551,10599,8582,8583,6437,6470,8583,10664,10598,8518,8518,8550,8550,12776,19049,10662,10789,19242,38130,29647,40178,38065,29645,42389,44372,44372,42259,44371,56951,54969,55002,54970,54970,54970,54970,54970,54970,54970,54970,54970,54970,54970,54970,54938,54938,54970,54938,54937,50646,40179,40146,21158,6498,6499,8579,4354,6468,14917,10627,6434,6434,4386,8548,8484,6372,19081,31628,31693,35821,40015,42226,46419,40081,37967,38033,44437,48466,42225,42126,48532,48498,54706,54673,56786,54706,54706,52560,42061,46221,31432,31825,50578,54739,44173,39949,37933,46354,44239,48565,228},
{25980,23932,25980,25980,26012,28060,28060,32189,32221,30173,30141,32189,32253,34301,38429,40541,42590,36381,36413,38461,38461,36413,38461,42589,52958,57150,61342,63423,63455,63455,63455,63455,63455,63423,61374,61342,61342,61342,59294,59262,61310,61310,61310,61310,59262,57181,57148,55036,52955,50875,50875,52924,50876,50908,50909,52990,50942,48894,46846,46846,48894,46814,48862,48894,50974,50974,53054,53054,53054,55102,55134,57182,57182,57182,55134,55134,53054,51006,53054,55102,55134,55101,59229,59262,59261,59229,57149,55069,55068,55068,55036,52988,52988,52956,50875,50843,52923,55004,55004,52956,50876,50876,50876,50876,50875,46649,38066,38066,16937,16905,16969,12744,14824,14824,14824,12744,10631,10663,10631,10631,10663,12711,10663,10631,10598,10599,10630,10631,10630,10598,8550,8518,10598,8550,10598,10598,10598,10566,8518,8518,8518,10566,10598,8550,8518,8518,8486,10565,10566,10598,10598,10566,8518,8518,8550,14856,12742,14949,25484,38098,29615,40146,33838,16968,19179,23307,23275,29646,48564,56918,29581,23243,23275,25388,27534,27501,27501,27534,27501,27501,27534,27502,27502,29582,27534,27501,27502,27469,36050,42096,33775,35886,16932,4322,6467,8580,6435,8579,12803,6466,6434,10628,16935,14822,8452,6339,25420,29514,21259,39950,40049,42194,48499,48500,42128,14791,36051,50710,50579,44206,44306,48498,52658,54705,52724,52593,52659,50447,44207,46156,27239,27534,50579,54739,39948,37900,37900,44338,42094,44339,228},
{25980,23932,25980,25980,26012,28060,30109,32189,32221,32221,30173,30173,32221,32253,38429,42589,44670,38461,38493,40541,40542,36381,42621,52957,59230,61343,63423,63423,63455,63455,63455,63455,61375,61342,61342,61342,61342,61310,59262,59262,59262,61310,61310,61310,59262,59229,57149,55036,52955,52923,50875,52924,50876,50908,53021,53021,50974,48894,48894,46814,48894,48894,48894,48894,50974,53054,55102,55102,55134,55134,55134,57214,57182,57214,57182,57182,53086,51006,51006,53054,55102,57149,59229,61310,61309,59229,59229,57149,55068,55068,55036,52988,52956,52956,50876,50843,52923,55004,52956,52956,50876,50876,50876,50876,44537,27633,23308,23308,12777,16937,19018,10696,12776,12776,14824,12744,10631,8583,8550,8551,8583,10631,10631,8583,8518,8518,8518,10598,10598,10598,8550,8486,8518,8518,8550,8550,10598,8518,8518,8518,10598,8550,8518,8550,8517,8517,8518,8518,8550,10566,10566,10598,10566,10566,10598,14889,12773,12868,23372,35985,29647,38097,27499,12774,12806,10661,8549,25421,50580,50547,23145,18984,16871,21194,23308,23340,21227,23308,21195,23275,23308,25356,23211,25389,23308,21195,21162,18952,14824,17033,35855,35886,18980,4354,6499,4386,6467,8611,10722,8547,4322,6403,8451,12677,6371,6372,21161,18981,17001,35757,42129,44339,48532,50613,46322,12646,42324,50743,48497,44272,44239,50578,52657,54705,44273,50544,50579,46254,44239,46221,29319,23275,50513,50513,39948,37900,37900,42225,42094,40112,226},
{25980,25980,25980,25980,25980,28060,30108,32189,34269,32221,30173,30173,32221,32221,36381,40541,44702,42590,42622,40574,40542,38461,50877,57182,61342,63423,63423,63455,63423,63423,61375,61343,61342,61342,61374,61342,61310,59262,59262,59262,59262,61310,59261,59261,59229,59229,57117,55036,55036,52923,52923,52924,50876,52956,53021,53022,53054,48894,48894,46814,48894,48894,48894,50942,50974,53022,53054,55134,57182,57182,57182,57182,57182,57182,57182,57182,55134,53054,53054,53054,55101,57149,57181,59229,59261,59261,59229,57149,57149,55068,55036,52988,52956,52956,50843,50843,52956,52956,52955,50875,48795,50843,50876,44536,35953,12744,12744,12744,10664,16937,16970,12744,14824,12776,14824,16937,12744,12712,10663,12711,12743,12744,12711,12711,10631,10631,10631,10663,12711,10631,10599,8550,8550,10599,10598,10599,10599,10566,8518,10598,10598,10630,10598,8550,8550,10598,10598,10598,10598,10566,10566,10566,8550,10566,10566,14888,10693,12837,12742,33905,31792,38098,21193,12774,12806,12807,10662,31662,54773,33644,21032,16871,19049,23308,21195,23373,23340,23308,21163,21228,21195,21163,21227,19115,21227,19114,14791,14791,19016,33707,37967,27468,31627,12675,6466,6402,4386,6466,8642,6498,6434,4290,10629,10532,4258,2211,21195,14755,23274,35724,42129,44339,48532,50645,46418,29516,44404,52823,46352,44304,44207,48563,52625,54706,37934,52593,50580,42093,48367,48334,35624,25355,50546,46287,37900,37900,39948,40079,46353,40015,213},
{26012,23964,25980,25980,25980,28028,28092,30140,32220,32221,30173,32221,34301,34301,36381,40541,46750,42654,42621,42622,40574,38461,46749,57182,61343,63423,63455,63455,63455,61375,59262,59262,61342,61342,61342,61342,59262,59262,59262,59262,61310,61310,59262,59261,59229,57149,57117,55068,55036,52955,50875,50875,50908,52989,53021,55102,55102,50974,48862,48894,48894,48894,48894,48926,48926,50974,55102,55134,57182,57182,57182,59230,59262,59262,57214,57182,57182,53054,53054,55102,55101,57149,57181,57149,57149,59229,59229,57181,59197,57117,55036,55036,52956,52956,50875,50843,55004,52956,52956,50843,50843,50844,44472,35953,33840,12744,10664,10632,10664,16937,16969,12712,12744,12712,12744,21163,14824,12711,10631,12711,10664,12712,10663,12711,8583,8486,8518,8519,8518,8518,8518,6437,8485,8485,8485,8486,6437,6405,6373,6405,6437,6437,6405,6405,4325,4356,6405,6437,6405,6405,4324,4324,6405,6405,8453,12743,10660,10757,10725,38195,27501,19147,12774,27467,31660,29580,29581,42194,46322,21064,16871,14823,19114,23308,21228,21260,21227,21227,21195,21195,21227,19082,21227,21195,17001,14856,12709,12710,40112,40014,46418,42161,46452,27432,6466,4353,2274,6402,8643,10690,6466,4289,4258,4291,2178,2178,21227,14723,25420,37805,42226,27436,42291,48499,46418,40113,38098,50677,44207,46384,46352,48532,52658,50480,31595,52658,46319,46286,50480,50447,41897,21128,50611,42029,37868,39948,39948,40047,50579,42127,207},
{25980,25980,25980,25980,26012,26012,28060,30173,32221,32221,30173,32221,34301,36349,38429,44670,46749,44702,42622,42621,40574,38461,42589,55069,61343,61375,63423,63455,63455,61375,59230,59262,61342,61342,61342,61342,61342,61310,59262,59262,59262,59262,59262,59261,59229,57149,55068,57116,55036,52955,50875,50875,52924,52988,55101,55102,55102,53054,48894,48894,48894,50942,50974,48894,48894,50974,55102,57182,57214,57214,59262,57182,59262,59262,59262,57214,57182,55134,53054,55102,57149,57149,57149,57181,57149,59229,59229,59197,57149,57116,55036,55036,52988,52956,50843,50843,52956,52956,52923,50843,48795,42391,38066,38001,33840,14856,10664,10664,8583,12712,19017,19050,12744,12744,12744,12744,12744,12743,12711,10663,12743,12744,12744,12744,10663,10631,10631,10631,10631,8583,8583,8518,10599,10598,10599,10599,10599,8518,8550,8550,10598,8550,8550,8518,8518,8517,8518,8518,10566,8518,8518,10598,8518,8518,8518,12776,10693,10756,12837,29711,14921,12840,17032,46550,52856,54936,33840,38033,31596,18951,16903,19016,21163,23275,21194,21163,21227,23308,23275,21130,21130,23275,25324,18952,8483,6435,6435,8451,16902,35919,46419,46484,35919,31627,8547,4353,6402,8612,8643,8610,8546,6370,4322,6435,2146,4291,23307,10563,23275,35758,42226,27436,38066,50612,48532,44372,25453,48564,44239,48432,48498,48499,52626,46222,29417,48531,42027,44207,50511,52560,44043,21064,46418,39916,39916,39948,37867,37933,52692,44240,202},
{25980,28028,25980,25980,26012,26012,28092,30141,32253,34301,30173,32189,34301,34301,38429,44669,48830,46750,44670,42622,42622,38461,40541,46781,55070,59262,61375,63455,63455,63455,59262,59262,61342,61342,61342,61342,61342,59294,59262,59262,59262,61310,59261,59261,59261,57149,55068,57116,55036,52955,50875,52923,52956,52988,55101,55102,55102,55102,50974,48894,50942,50942,50942,48894,48926,53022,53086,57182,57214,59262,59262,57214,59262,59262,59262,59262,57182,55134,53086,55102,57149,57181,59229,59229,57181,59229,59229,59229,57148,57116,57116,55068,55036,52955,50843,50875,55036,55036,52955,48795,42390,38066,38066,35921,33808,14857,10664,8584,10664,10663,19050,19018,12744,12744,12744,12744,12744,12743,10663,12744,12744,12743,12711,12712,10663,10631,8518,8550,10598,8550,8518,8486,8485,8518,8518,8518,8486,6437,6405,6405,8485,8485,8486,6437,6405,6405,6437,8485,8453,6437,6437,14823,8485,8485,8453,14791,8517,12773,12837,17063,14887,12807,27533,50776,52824,50743,35888,27371,29549,33775,33840,35920,40146,40178,35921,38066,42292,40179,38034,29549,33677,33645,29386,18886,20869,18789,18853,23145,21064,35887,37934,35853,12708,10628,14853,10594,6402,10724,10659,8579,4322,2210,2241,4291,4259,19017,21129,16936,25323,23210,29581,31629,42291,50612,48532,33676,14986,48465,44239,46352,50611,50546,50545,41931,29416,46386,41963,42094,50479,52560,44075,18950,40111,37868,39948,37900,37868,39948,48499,42127,188},
{25980,26012,25980,25980,26012,26012,28060,30141,32253,34333,30173,32221,34301,34301,40509,44669,48862,46782,44702,42622,40574,38494,40541,44702,46782,52990,57182,63423,63455,63455,59230,59262,61342,61374,61374,61342,61342,61342,59262,61310,59262,59261,59261,59229,59229,57149,57116,57116,55036,52956,50843,50875,52956,52956,55069,55101,55102,55102,53022,48894,48926,50974,50974,48894,50974,53054,55102,57182,59262,59262,59262,59262,59262,59262,59262,59262,57214,55134,55134,55134,55133,57181,59261,59261,59229,57149,59229,57149,57148,57116,57116,55068,55036,52956,48795,50875,57116,55068,52955,44438,40146,38065,38065,37969,33840,16905,10664,8583,10632,10664,19050,19050,10696,12712,12743,12744,10695,10663,10663,10663,10664,12743,12711,10663,10631,10663,8583,8583,10631,10631,8550,8518,8518,8518,10598,10598,10598,8518,6438,8518,10598,25388,10566,8518,10566,10566,12679,12743,18986,19050,25356,23275,33808,23243,25389,29582,29614,25419,21225,21193,21031,18951,31695,48565,42129,42161,42226,31629,36019,42260,33808,31727,38131,40211,33840,29614,38099,40147,38033,23210,27339,27274,27338,14725,20902,18756,18853,18952,19081,42258,44306,40046,8546,4322,10692,8481,6435,12804,8579,6563,6434,2242,2145,2146,4226,27468,25255,17001,35756,33709,40113,46419,48565,50612,48532,37837,21260,44239,44272,46319,50579,50546,52626,33576,31495,42192,41995,42093,50512,52527,39947,29417,40046,37868,37900,39916,39948,39948,44338,46255,124},
{28028,26012,26012,25980,26012,26012,30108,32189,34269,36381,32253,34269,34301,34333,40509,46782,48862,48862,46750,42654,40573,38493,40541,42621,44702,48830,52958,63391,65503,63455,59230,59262,61342,61342,61342,61342,61342,61342,61342,61310,59262,59261,59230,61309,59262,59197,57149,57116,55036,52955,52923,52923,52956,52956,52989,53021,55102,55102,53054,48894,48894,50974,50974,50942,50974,53054,55134,57182,59262,59262,59262,59262,59262,59262,59262,59262,57214,55134,55134,55134,57182,59229,59262,59261,57181,57181,59229,57149,57149,57116,57148,57148,55036,52955,50843,50875,57116,55003,55003,42260,40146,40114,38034,35953,33807,14856,10664,10664,10664,12776,19050,16937,12776,12776,12776,14856,12744,12744,12743,12712,12743,12743,12744,12744,12744,14791,12679,14791,16904,16937,19049,19018,21131,23243,23243,25356,25356,23276,14792,14759,14759,16904,14759,14792,19017,16872,16905,16905,29647,19147,23340,21163,38099,23276,23341,25389,31792,21193,16967,14886,16902,23242,35791,42259,33709,37871,38000,23177,35953,33840,27533,27501,35986,35985,29614,27501,33905,38002,27534,25356,27403,29419,16837,10596,16773,14660,18985,44405,29580,44339,44305,31691,8514,4322,8547,6433,6434,12804,8612,6435,6467,4322,2178,65,4259,31660,16835,14854,29482,33742,40113,46419,48597,50612,48564,42128,33840,46287,46352,44206,46385,48465,46384,31463,31496,37999,44108,44238,50512,50414,37835,42127,39916,37868,37868,39916,39948,39916,44404,50578,107},
{28028,25980,25980,25981,28060,26012,28093,30141,34269,36381,32253,34301,36381,40541,44670,48862,48862,48862,46782,42622,38493,38493,40541,40573,42622,46782,57150,61343,63455,63487,61375,61374,61342,61342,61375,61375,61374,61342,61342,61342,59262,59262,59261,59261,59261,57181,57149,57116,55036,52955,50875,50875,52923,52956,52956,52989,55069,57150,55102,48894,48894,50974,50974,50942,50974,53054,55102,57182,59262,59262,59262,59294,59294,59262,59262,59262,57214,55134,55134,55134,59230,61310,59229,59229,59229,57181,57149,57149,57149,55068,55068,55068,55036,52955,50843,50875,57116,57148,55004,46551,38066,40114,38034,38001,33807,14824,10664,10664,10664,16937,16970,10696,10696,12744,12744,21195,14856,18921,16872,16905,16872,18952,18952,18952,16872,18952,16840,21131,25421,25421,19147,19082,23308,23341,21162,25389,23244,19050,10566,12646,12679,12679,12646,12679,16904,10566,12679,12711,21228,12744,21163,10631,31793,18984,23340,16937,23373,23307,21225,12806,21159,29613,39919,44340,35855,37936,40146,42226,46453,44274,40081,40081,42292,44372,48565,42097,42292,46419,14855,29679,37935,37836,10659,10629,16805,16838,29582,50777,31628,42225,33806,14853,8546,8547,6434,4290,6565,16998,8579,6403,2210,4323,4291,4226,10695,33774,14756,29613,29384,29515,37968,44339,46485,50612,50645,44273,44371,46320,46352,42094,44305,48433,42094,31463,31496,35853,48399,46286,50480,46221,35787,42160,39916,39948,39948,39948,39948,39948,44339,48498,119},
{25980,25980,25980,25980,26013,26012,28093,30141,34269,38429,34333,34333,38461,42621,48830,50942,48862,46814,46782,42622,38493,38461,38493,40541,40574,44701,48861,59230,63423,65503,65535,63487,61342,61342,61375,63423,61375,61374,61342,61342,59262,59262,59261,59229,59229,57149,57149,57116,55036,52955,52891,52891,52923,52956,52956,52989,55102,55102,53053,48861,48894,50974,50974,50974,51006,53054,55102,57182,59262,59262,59262,59294,59294,59262,59262,59262,57214,57182,61310,59262,61342,59261,59229,59229,59229,57149,57149,57148,57149,57116,55068,55068,55036,52956,50875,50875,57148,57148,52988,52923,48664,38033,38066,35953,31727,12712,12744,10696,10664,16905,19018,12744,12776,12776,12776,16969,14857,14824,16905,16905,16904,18985,19017,19050,19018,19050,16937,23308,27535,29647,27534,29615,31760,29615,31727,29582,31727,23275,16904,16839,18952,16839,16872,14759,19017,12678,16872,14791,23308,12743,21195,12743,33841,16871,23373,19017,25486,23307,19048,12839,23273,31694,40016,50711,40080,42162,44307,46451,46518,48532,44338,42226,44405,46485,57017,50580,48631,48532,19015,25453,42160,44273,16901,29646,25159,25290,33775,33840,23306,31660,10628,8546,8578,6466,6466,2209,6599,14884,10660,6467,4258,4290,4324,40013,14790,10596,6403,35854,35692,35822,33677,46419,46452,50612,50645,48532,48531,46384,44207,42093,44337,48433,35722,27237,8419,8614,50578,48367,50479,44141,37835,42127,37868,39949,39949,39948,39947,40014,48435,46418,112},
{23932,26012,26012,26012,28061,28061,28061,30141,34269,38461,36381,36381,38461,42621,46782,48862,48894,46782,46782,42622,36445,36413,36414,36445,38493,42621,42654,50877,61342,63455,65535,63487,61343,61310,61374,63423,61375,61375,61342,61342,61310,61310,59262,59229,59229,57149,57117,57116,55036,52955,52891,52891,52923,52956,52956,52988,55069,55102,53021,50909,48894,48894,50974,50974,53054,53054,55134,57214,59262,59262,59262,59262,61342,59294,61310,59262,59262,59230,61374,61342,61342,59229,59229,59229,59229,57181,57149,57116,57116,55068,55036,55068,52988,52955,50875,52923,59229,59229,55036,52924,55036,54970,35986,35953,31728,12711,12743,12744,8583,16905,19018,12744,14792,12776,12776,12744,12776,12744,12744,12744,12744,12743,12744,12744,12712,10663,10630,10663,10631,8583,10663,8583,10663,10728,14856,16970,14889,16969,12678,12711,14824,16872,14824,16872,21130,19017,19050,21227,33905,25453,29615,23340,40212,31662,25388,27469,29712,29613,27532,16935,25322,27436,38033,54937,40080,42161,44340,44372,46517,50678,48532,40113,44340,48631,54936,52823,54937,54904,40113,31759,44241,46419,37870,57083,50483,23080,27306,27403,16935,12804,6499,6466,6498,6466,4354,2209,2307,10757,8612,6435,4322,2177,12841,54836,10530,4322,4291,29548,35725,25257,37967,44338,46452,48564,50612,50678,46385,50644,40046,42061,48465,44239,31464,20867,2113,6536,52724,50480,52527,39947,42159,39981,37868,39948,39948,39948,31528,35952,44207,46451,155},
{25980,26012,26012,28060,28060,28060,28093,30141,32221,38461,38461,36381,38461,42589,44701,48862,48862,46782,44734,42622,38493,36413,36413,36413,38462,40573,44702,50877,61342,63455,65535,65535,61343,61310,61374,63423,63423,63423,61342,61342,61310,59262,59262,59261,57181,57149,55100,55036,55035,52923,52923,52891,52923,52956,52956,52989,55101,55069,50941,50941,48894,48894,50974,53022,53054,55102,55102,57214,59262,59262,61310,59262,61342,61342,59262,59262,59262,59262,61342,61342,63422,59261,61310,61309,61309,59261,57181,57149,55100,55068,55068,55068,55036,52955,50875,50843,59229,59229,52956,52955,52956,57116,54971,35986,31760,21130,21098,21162,10663,12744,12744,6438,6470,8550,10599,10631,10631,10599,10599,10630,10631,10630,10631,10631,10598,10599,10599,10631,10631,10631,10631,10631,10631,10631,10631,10663,10631,10631,10598,10630,10630,10631,12679,12679,10631,10631,10598,10663,14889,14922,16970,14921,29647,21163,21195,23308,21260,27533,27598,21064,25160,35791,50679,54872,37805,35822,42194,44307,48566,50776,50711,50711,50679,50712,50711,54937,52889,54936,35920,29581,39983,40081,42096,40213,42097,20968,27338,23210,14821,8611,8547,8546,6466,4418,4322,6434,6467,8580,4419,4355,4290,2178,21325,29514,6370,4258,97,16903,35790,20999,37967,44306,46452,48564,50612,50710,46255,46418,46320,40046,48465,37933,33642,35755,20998,25388,44371,52560,52561,39948,44305,37867,37900,39948,39948,39948,18818,33905,44207,46418,146},
{28060,26012,26012,28061,28061,28093,30141,30140,34301,40509,40509,38429,40509,44670,44702,48862,48862,48862,46782,42622,38461,36413,36413,36413,38494,40541,44702,55038,61342,63455,65535,65535,61375,59262,61342,61375,63423,61374,61374,61342,61342,59262,59262,59229,57181,57149,55068,55036,55036,52923,50875,50875,52923,52956,52956,52989,53021,55101,50941,52989,48894,48894,50974,53054,53054,55102,55134,57214,59262,59294,59294,61310,61342,61342,61342,59262,59262,59262,61342,63422,61342,61310,61342,61342,61342,61342,61309,59229,57148,55068,55068,55068,55036,52956,50875,50843,59196,57148,52956,52955,50876,55036,57116,55003,38099,29582,38033,38033,12744,16905,16937,10663,12711,12712,12712,12711,12712,12712,12711,12712,12711,10663,10663,12711,10631,10631,8550,10599,10598,8550,10598,8550,10566,10566,8550,10598,10566,10566,8517,8485,8518,8518,10598,8518,8518,8485,8485,8518,8518,8518,8518,8518,6437,8485,8518,14824,8518,12710,10758,21063,29451,50744,59196,57050,50613,48500,48532,46387,50711,48598,46517,48565,50743,50776,50711,54969,54970,52824,42258,35887,35757,37903,40015,31531,27176,27241,25128,23145,12740,10660,10658,8578,6530,4386,4290,8612,8547,4354,6434,4290,2241,2275,27599,16836,4322,4258,6435,8515,27500,21031,14823,40145,46452,48532,50612,52725,44206,48465,46352,42191,46320,35755,35821,44273,35755,33675,29646,50513,50447,37868,44240,39916,39916,39949,39980,37802,14659,42258,48433,48433,166},
{26012,28028,28028,28061,28061,28061,30141,32221,38429,44670,42622,38461,40541,44670,46750,48862,48862,48862,46782,42622,38462,36413,36413,36413,36445,38493,44670,52989,61343,63455,63455,63455,61375,59262,61342,61374,63422,63423,61374,61342,61310,61310,59262,59262,59229,57181,55100,55036,55036,52955,50875,50875,52923,52956,52956,52988,55037,55102,55069,55069,50974,50974,50974,53054,53022,55102,55134,57182,59262,59262,59262,61310,61342,61342,61342,59262,59262,57214,61342,63455,63423,61342,63422,63390,61374,61342,61342,61309,57181,55068,55068,55036,55068,55004,50875,50843,59196,59197,55004,52956,50843,52988,55036,55036,52923,42325,38065,35953,16970,16937,18985,12743,12744,12744,12712,12744,12744,12744,12712,12712,12711,12744,12711,12711,10631,10631,10631,10598,8550,8518,8518,6469,8485,8518,6470,8518,8518,8486,6405,6437,8485,8486,8517,10566,6437,8453,8453,8486,8518,8518,8518,8485,6405,6437,8517,14856,10598,8613,14886,46419,33709,29713,59228,61308,50776,52824,52823,50678,52824,52824,52824,52824,52824,50678,52823,54904,54936,29678,33838,25355,37870,31662,31563,37902,35724,33677,27272,29482,10627,14853,10626,8546,6466,4354,2210,10692,6499,4355,8579,6370,2177,4323,25616,27303,8514,6402,8579,6434,12839,16870,8485,29711,46419,46484,50612,50611,46352,48465,48432,46385,44174,33610,35854,42160,35821,35788,25453,52625,50414,37901,44175,37900,39948,39948,39948,39915,40080,46353,48400,48400,200},
{26012,23964,26012,28060,28060,28093,32189,34301,40477,44670,44670,40509,42590,46750,46782,48862,48894,48862,46782,40574,36413,38429,36413,36413,36413,38494,44670,52989,61342,63423,63455,63455,59295,59262,61342,63390,61374,63390,61342,61342,59262,61310,59262,59261,59229,57149,57148,55068,55036,52955,52923,52923,52923,52955,52956,52988,55037,57149,57182,55102,53022,50974,53054,55102,53054,55102,55134,57214,59262,61310,59262,61342,61342,61342,61342,59262,59262,59262,63422,63454,65535,63390,63422,63422,63422,63390,63422,61342,59261,57149,55068,55068,55068,55036,50875,48795,52923,59229,55036,52956,50875,50908,52956,52956,52956,46584,27502,21228,16970,23308,23276,19082,14922,17002,17002,17003,14857,14824,12776,12744,14824,14824,12776,12744,12743,12744,10663,10663,10663,10663,10663,12679,10631,10663,10663,12711,10663,10631,10631,10599,10631,10631,10663,12711,12678,12679,10631,10631,10599,10598,10630,10631,10599,10630,10663,16936,10598,14823,25387,56951,33709,31793,57148,61276,52824,52824,52824,48630,52823,52823,52824,52856,52791,48598,54904,54936,48531,14853,23306,21161,37934,21194,37966,50613,37967,35920,33643,38000,21094,16966,14884,10659,6498,8610,8547,12772,6499,6435,6531,8482,2210,8451,25584,23141,10723,6434,8546,8546,4354,4291,6372,14921,42291,46484,52725,42127,35886,48497,48432,48400,42094,31463,35822,42193,35787,35756,16936,52690,50414,40047,39949,39948,39948,39948,39980,39980,46451,42094,44239,48335,195},
{28060,26012,26012,28060,30108,30141,32221,36381,38461,42621,42622,40542,42621,46750,46750,48862,50942,48862,46782,40574,36413,36381,36381,36414,36414,38494,46750,50910,57182,61343,63423,63455,61343,59262,61342,61342,61342,61374,61342,61342,59262,61310,59262,59261,59229,57181,57149,57084,55036,52955,52955,52923,52955,52955,52956,52988,52989,57149,59230,57182,53054,53054,55102,55134,53054,55102,55134,57214,59262,59262,61342,61342,61342,61342,61342,59294,59294,59262,63423,63454,63455,63422,63422,63422,63454,63422,63422,63390,61309,59229,57116,55036,55068,52988,50875,48763,50843,59229,57084,55003,50876,52956,52988,55036,52956,52956,52923,48729,46552,33873,27501,23276,19050,19050,19050,19050,14857,12776,12744,12776,12776,14824,12776,12744,10696,10632,8551,6503,8551,8550,8518,6438,6438,6438,8518,6470,6470,8486,6438,6405,6405,6438,6470,8486,8486,6438,6438,8453,6405,6405,6405,6405,6405,6373,12711,16936,12678,16904,40113,54871,35822,29680,59196,59196,52824,52824,52824,50678,52823,52823,52824,52823,52791,48597,56984,54904,38031,19048,31725,23273,40048,25420,44405,48565,31693,38032,29417,44372,27401,14853,12772,8578,6530,10754,8579,10691,8547,4322,6499,4354,4323,12644,6436,12804,8611,4354,8546,6433,2209,2210,6371,6502,40211,46484,50710,37901,27468,50611,48433,48433,44239,31463,29581,42160,33643,29449,10630,50675,50414,42127,37868,37900,39980,39980,37900,39949,48499,46352,48400,48432,206},
{28060,26012,28061,28061,30141,32189,34301,36349,38429,42622,40542,38461,40541,44670,46782,48862,50942,48862,44734,40542,38461,36381,36413,34365,36414,38493,42621,50910,55070,59262,63423,61375,59262,61342,61342,61342,61342,61342,61342,61342,61310,61342,61342,59261,59229,59229,57149,55068,55036,52955,52923,52955,52955,55004,52956,52956,52989,55101,59229,59262,55134,55134,55134,55134,53054,55102,57182,57214,59262,61342,61310,61342,61342,61342,61342,61342,59294,61374,61342,61342,61342,63422,63422,63422,63422,61342,61342,63422,61342,59229,55068,55036,55036,52956,52923,48763,50843,59196,59196,55036,52956,52956,55068,55036,52956,52955,50876,50876,50843,52923,57116,57116,52923,55003,52923,50843,50875,50842,50842,48762,48762,55002,44405,33938,17133,15053,17069,15021,10795,10827,10794,19148,27534,14923,8584,6503,8648,8584,8616,8616,8616,8551,10697,8584,10664,10664,8551,8551,14760,25128,14758,21065,20935,21097,25388,19082,12776,19016,46419,50645,33741,23406,55002,61244,52824,52824,52824,50678,52824,52823,52824,52824,52791,42259,52823,40112,14886,16936,42258,31726,50613,38033,40178,50743,59196,52693,35690,40178,23240,12771,8611,10658,10658,10722,8611,10692,6499,4322,2210,4387,10597,16805,6402,12803,8579,6435,6466,6498,4289,6468,18951,8582,38098,44372,50645,40014,23274,52724,50578,48464,39981,27270,10597,25355,31693,31596,29483,48563,50447,42127,37868,39948,39948,39980,39948,42127,46386,46385,48400,50512,145},
{28060,26012,28060,28060,30140,32189,34301,36381,38429,40541,40509,40509,40509,44670,46750,46782,50942,48894,44734,40541,38461,34333,36381,36413,36413,40541,42621,48829,55070,57150,61342,61375,59294,61342,61342,61342,61342,61342,61342,61374,61342,61342,61342,59262,59229,57181,57149,55068,55036,55003,52955,52956,52988,55004,52956,52956,52988,57149,59230,59262,57182,57182,57182,55134,55102,55134,57182,57214,59262,59294,59294,59294,61342,61342,61342,61342,61374,63455,61342,63422,63422,63422,61374,61342,61342,61342,61342,61342,61342,59229,55068,55036,52956,52955,52923,48795,50843,59196,57148,55068,52956,52956,52988,57117,52988,52956,52956,52956,50876,50875,52956,52988,52955,55036,52987,50875,52956,50908,50876,50876,48827,57115,50809,29746,17134,13038,15021,12941,10828,10827,8747,27632,44438,19214,17003,12809,12808,12841,12841,12841,10760,12841,14921,14889,12841,12809,12841,14889,16969,19017,18985,16969,16969,14889,14888,12776,12776,23307,44371,54838,33709,23341,59228,63356,52823,52823,52824,50678,52824,52824,52824,52823,52791,35887,44404,25320,10661,19016,44339,31662,52791,42226,50678,54904,59228,56984,39916,35887,29580,17029,12804,10723,8578,10722,8643,10724,8547,4354,2178,98,8516,38065,18850,8546,8579,6434,4322,6402,8547,10728,40015,10662,31791,46452,50643,40047,21096,46450,52691,46351,33641,25190,12645,10695,36017,38031,39981,44370,52593,42093,37868,39948,37900,39980,39981,46419,39981,37868,46319,50480,101},
{28060,26013,28061,28061,30141,32188,34301,36381,38429,38493,38461,40509,40542,44670,46750,46782,50942,50942,44734,40542,36413,34333,34333,34366,38461,40542,40574,48829,55069,57182,61310,61343,59262,59262,61342,61342,61342,61342,61342,61342,61374,61342,61342,61342,59261,59229,57149,57116,55036,55036,52956,52956,55036,55036,52956,52956,52988,55069,59230,59262,57182,57182,57214,57214,55134,55134,57214,57214,59262,59294,59294,59294,61342,61342,61342,59294,63423,65535,61375,61374,63422,63422,59262,61342,63422,61342,61342,61342,59261,57181,55068,52956,50875,50875,52955,50843,48763,57116,57116,55036,52955,52956,52956,52956,52988,55036,52956,52956,50876,50876,50875,52956,52956,52956,52988,50876,50908,50876,50876,50876,48827,57115,46583,27535,19215,12974,12941,10893,10860,8747,10795,31858,44438,19213,14889,6502,8485,10631,8614,8582,4389,8550,8582,10663,6502,4389,10631,10631,8614,6469,8550,10630,8582,8582,4357,8518,10630,14953,48564,50612,31661,23309,63390,63356,52824,52824,52824,50678,52824,52824,52824,52824,50710,38033,52790,31627,10661,21194,29581,33873,50710,40081,52791,42226,61374,59129,39916,35887,29579,19109,14852,8611,6498,8675,8611,10724,8547,6435,2242,6404,8582,50808,18883,4322,4354,4354,4322,12806,8483,4422,40113,10564,25421,46485,48497,29450,21161,50578,52691,46287,31431,23046,10597,15018,40145,44272,37868,40177,52626,39948,39948,39948,39948,37868,23078,40113,44207,44142,48367,46319,165},
{30108,28060,28060,30109,30141,32189,34301,36349,36381,40541,38461,40509,40541,44669,46750,48830,50942,50942,44734,40542,36381,36381,34333,34333,36413,38493,40574,46749,55070,59230,61342,61375,61342,61342,61310,61342,61342,61342,61342,61342,61342,61342,61342,61310,59262,59229,57181,57116,55068,55036,52956,52955,55036,55036,55036,52956,55036,57117,57213,59294,57214,57182,57214,57214,55134,55134,57214,59262,59262,61342,61342,61342,61342,61342,61342,61342,61342,63455,61374,61374,63422,57214,52988,59229,61342,61342,63390,61342,59261,57148,55068,55036,50875,50875,52923,48795,48763,57116,55068,55036,52956,55036,52955,52955,52956,55068,55036,55004,50876,48827,50876,52956,52956,52956,52956,50875,50908,50875,50908,50875,48827,55035,50744,36019,17102,17134,15053,12940,12908,12875,10794,31858,44438,29680,19114,10727,10663,14857,14888,14857,12775,12808,14856,14888,12808,10727,12808,14856,12808,10728,12776,14856,14889,12808,10663,12808,12808,17034,48565,37935,31661,19083,61374,63356,54904,52856,52856,50678,52824,52824,52824,52823,48565,31694,50710,48564,23241,35887,42160,38065,52791,50678,44339,48564,52890,57049,39916,40081,29580,27464,14883,12771,10658,6466,6530,12836,10659,4354,4354,2177,8647,54968,14691,6402,8482,6370,2177,6435,10627,6403,14823,16838,23372,48597,39948,19015,29645,50544,52691,44141,29351,18885,14790,23405,42225,44240,35820,38032,48465,39948,39949,39948,39948,35754,16870,44370,42061,46287,48367,46319,199},
{28060,26012,28060,30141,30141,32221,34301,34301,38429,40509,38461,38461,40541,44669,46750,46782,50910,50942,46782,40542,36381,34333,34333,34333,34365,38493,38493,44701,55069,57182,59262,61375,63423,63423,63422,61342,61342,61342,61342,61342,61342,61310,61310,59262,59262,59229,57181,57149,55068,55036,55036,52955,55004,55068,55036,55036,55036,57149,59262,59262,57214,57182,57214,59262,57214,55134,57214,59262,61342,61342,61342,61342,61342,61342,61342,61342,61342,63455,63454,61342,59262,52956,50876,52956,55036,59229,63422,61342,59261,57149,55068,52956,52923,50843,50875,48763,48763,55036,55036,55036,52955,55036,52956,52956,52956,55068,55036,52956,52988,50876,52956,55036,55036,55036,52956,50875,52955,50876,50876,50876,48827,50842,52922,48696,42292,40244,40245,40244,42293,42293,42325,46551,44438,25584,16969,8517,8485,8582,8614,8615,6437,8550,10631,10630,8550,6437,8582,10631,8582,6437,8517,8582,8582,8550,4357,8550,12808,14921,44306,31661,35822,14922,61309,61243,54904,52856,52856,50678,52824,52855,52856,52856,54904,50678,54904,50743,46386,46418,42226,48598,52823,42161,37967,52856,48435,50710,33610,40113,23274,29579,25319,21159,12771,10691,17063,27466,27466,23273,23209,21063,25322,44273,25255,23110,21061,18917,16869,21061,21125,18981,21063,29482,25355,44371,37835,18983,38032,50545,48465,39915,27270,14725,12742,27696,44272,46320,33708,33806,46320,37868,39948,37900,37900,29414,19242,44272,44109,46319,48335,50480,200},
{28060,28060,28061,30141,28093,32221,34301,34301,38429,38461,38461,38461,40541,44670,44702,46750,48862,50974,46782,38526,36413,34333,34333,34333,36413,36446,38493,42622,52989,57150,59262,61342,63423,63455,63455,61375,61342,61310,61310,59294,59262,61310,59230,59261,59261,59229,57149,57117,55068,55036,55068,55036,55004,55068,55036,55036,55036,57149,59262,61310,59262,57182,57214,59262,57214,57182,59262,59262,61342,61342,61342,61342,61342,61342,61342,61342,61342,65535,65535,63423,59262,55037,52956,50876,50908,57117,61342,61342,59261,57149,55036,52988,52923,50875,52923,48795,48763,55068,55068,55036,52956,55036,52956,52956,52924,55068,55036,52956,52988,52924,52956,55036,55036,55068,52956,50876,50908,50876,50876,50908,48827,46682,48729,50809,50777,48696,48696,48696,48696,48696,46616,48729,42390,21325,17034,10728,10696,12841,12808,12840,10696,10727,12808,12840,12808,10728,14856,14856,12808,10695,12776,14857,12841,12808,12744,14888,19147,19114,40047,21130,33774,16937,59196,61179,54904,52856,52856,50710,52824,52824,52824,54904,54969,44404,42292,42226,44273,48532,48565,50711,52824,42226,52791,57115,63356,54903,37771,42161,16903,35951,31659,19045,10658,12804,38163,37803,35755,35755,35755,37803,37803,37803,37803,35755,35755,35755,35754,35722,35722,33642,35690,33642,33642,33741,44142,18951,46450,46385,44239,35624,25158,14757,12776,36017,46384,44272,31497,33773,16772,10629,33675,31562,14820,10594,27663,44239,48367,50512,46287,50480,201},
{28060,28060,28092,28092,30141,30172,32221,36349,38429,38461,36413,38493,42589,42590,42622,44702,48862,53022,46814,40574,34365,34333,34333,34333,36413,36413,42622,46750,52990,55102,59230,61342,63423,63455,63423,61375,61342,61342,61310,59261,59262,59262,59229,57181,59197,57181,57181,57149,57117,57116,55068,55036,55036,55036,55036,55036,55036,55101,59261,61310,59294,57214,57214,59294,59262,57214,59262,59294,61342,61342,61342,61342,61342,61342,61342,61342,63390,65535,65535,65535,57149,55069,52988,50908,52988,57149,61342,61342,59261,57148,55036,55004,52923,52923,52923,48763,48763,55068,55068,55036,52956,55036,55068,55036,52956,52956,55036,52988,52956,52955,55036,55036,55068,55036,50908,50876,52956,50876,50908,50908,50876,48827,48763,46714,46649,38198,34037,34003,38131,33905,33873,44406,42390,25519,19082,8583,6502,10663,8615,8615,6470,8582,10663,8615,8583,6502,8615,8583,8583,6470,6502,8615,8615,8583,6437,12873,19146,14921,42127,19016,33741,12744,57116,63324,52824,52856,52824,50710,52824,52824,52824,52856,52791,25323,35985,46354,48500,48565,46452,48598,52823,44371,54904,55002,63454,61243,48401,40048,14790,38032,37998,27464,14852,38064,48629,37835,37803,37802,35755,37803,37835,37803,37803,37803,37803,37803,37802,37803,35722,35690,33642,35690,33609,31595,48465,35821,50578,44305,44207,31430,27271,25191,25388,37999,46385,44240,31529,33741,14627,14822,31529,25256,6433,4418,36016,46287,50480,50480,46254,48400,196},
{28093,30140,26044,28093,30141,30173,34269,36381,38429,38461,38461,42622,44670,44701,46750,46782,50910,53023,48862,40574,34365,34333,34333,34333,36413,40541,50910,55070,55102,57182,57182,59262,61342,61375,61342,61374,61342,61342,59262,59230,59230,59261,59229,57181,57149,57149,57149,57149,57149,55068,55068,55036,55036,55036,55036,55036,55036,55069,59229,61310,59294,57214,59262,59294,59294,59262,59262,59294,59294,61342,61342,61342,61342,61342,61342,61342,63422,65535,65535,65535,59262,55069,55036,52956,52956,55036,59229,61342,59261,57148,55036,52987,52923,52955,52955,48763,48763,55036,55036,55036,52956,55036,55068,57116,52956,52956,52955,52956,52988,52955,55036,52988,55036,55036,50876,50876,52956,50908,50908,52956,50876,48828,48828,48795,44634,40278,29876,40244,38099,33841,33840,44405,42357,21293,21227,12841,10728,12841,12841,12809,10728,12808,14889,12873,12841,14954,14954,12841,12841,10728,12808,12841,12841,12841,12776,17066,19147,17066,40080,19017,33741,12711,52955,61276,52824,52856,52824,50678,52792,52791,52823,52791,50710,27338,38032,48565,52823,46451,44339,46420,48565,40146,54937,57017,61341,63454,59097,35822,16871,38000,40079,23207,10726,48662,54838,39948,37803,37803,37803,37835,37835,37835,37835,37803,37803,37803,35754,35754,35722,35722,35722,35690,33609,31562,46450,48465,52724,40047,40013,29350,29350,25190,29614,42225,44305,40046,35756,27369,21063,25288,27336,21062,6433,10824,42160,48368,50480,50447,48367,48400,194},
{28093,28093,28092,30141,30141,32221,34333,36381,38461,40541,40541,44702,46750,48830,48862,50910,53022,53055,48862,40542,34366,34333,34333,34333,34365,38461,48829,55102,57150,59262,61342,59262,59262,61342,61342,61342,61342,59294,59230,59230,59229,59230,59229,57181,57149,57149,57149,57149,57149,55101,55068,55069,55036,55036,52988,55036,55036,55068,59229,61342,61342,59262,59262,59262,59294,59262,59262,59294,61342,61342,61342,61342,61342,61342,61342,61342,63422,65503,63455,65535,63423,55069,55069,55037,52956,52956,57149,63422,61342,59229,57148,52988,52923,52923,52923,48763,46683,52956,52988,55068,52956,55036,52956,55036,52956,55036,52956,52956,52988,52956,52988,52956,55036,52988,50908,50876,55036,52956,50908,50908,50876,48828,48828,48795,44602,34070,27796,36083,35986,31727,33808,44405,44438,23438,21227,10728,6535,10695,8648,8616,6502,10663,10695,10728,8615,8615,12808,10696,10663,6470,10630,10695,8615,8615,6470,17034,19082,12840,40080,18984,35854,16937,52923,54937,42324,44405,44404,42259,42324,35984,25453,25485,27533,35920,46451,48598,48565,48564,42193,38000,46387,42194,54969,54936,50744,55002,46452,38000,25289,38032,31691,10659,15050,52823,54838,44109,35754,37803,37803,37835,37835,35787,37835,37803,37802,37803,35755,35787,35755,35722,35690,35690,33610,33610,44338,48432,50611,37901,39915,27302,29383,23078,38098,42258,46385,37901,29449,27337,0,16968,14787,6433,4385,27663,44241,48368,50481,48335,48367,46319,190},
{28093,28092,30141,30141,32189,34269,34333,36381,40509,42589,42621,46750,48830,48862,50942,53022,53022,55103,48894,40542,36414,34333,34333,34333,34365,38461,44669,50910,55102,59262,61343,61375,61375,59262,61310,61342,59294,59262,57149,57149,57149,57181,59197,57181,57181,59229,59229,59229,59229,57149,55069,55069,55036,55036,55036,55036,55036,55068,59230,61310,61342,59294,59262,59262,59294,59294,59262,59294,61342,61342,61342,61342,61342,61342,61342,61342,63422,63455,65535,65535,63455,57149,55069,55101,52988,52924,55036,61374,61342,59261,57148,55036,52955,52923,52923,48763,48730,52988,55036,55068,55036,55036,52956,50876,52956,57148,52988,52956,52956,50908,55036,52956,52956,52988,52956,50908,55036,52956,50876,50876,50876,48828,48828,46748,42522,34070,27829,31890,35953,33808,33808,42357,42358,21325,19180,14921,12808,12841,12841,12841,10695,12808,14889,14921,12841,12808,14889,14889,14889,12808,14889,14921,12873,12809,12776,17066,17001,17002,40080,29515,35822,16904,48729,50679,35920,35920,31694,31727,25387,8580,4355,4323,10662,21195,46452,44371,48532,48498,40113,44306,42226,44307,57017,56985,52725,46452,37836,40113,27370,29580,16901,12773,36083,54903,52790,46222,35754,35755,37803,35755,35787,37835,37804,37835,37803,35755,37803,35755,35754,35754,35690,35690,35626,33609,42225,46319,46319,31628,33576,27270,29351,21031,40178,42257,46353,37836,29450,27304,6306,19014,6465,4353,4354,31856,48466,48399,50480,48399,50480,46287,190},
{30141,28093,30140,30141,32221,34301,36381,38461,40541,42621,42621,44670,48830,50910,53022,55070,55102,55102,48894,40574,34333,34333,34333,34365,34365,36414,38494,42622,50910,59230,61342,63423,63455,63455,59294,59262,59262,57214,55069,57149,57149,57149,57181,57181,57181,59229,59229,59262,59197,57149,57148,55068,55036,55068,55036,55036,55036,57149,59262,61310,61342,59294,59262,59294,59294,59262,59262,59262,61342,61342,61342,61342,61342,61342,61374,61342,63422,65503,65535,65535,61374,57117,57149,55069,52956,52923,52988,61342,61342,59261,57149,55036,52955,52955,52923,50811,48730,55036,55036,55036,55036,57084,52956,50875,52956,57148,57116,52956,52956,50908,55036,52988,52956,52988,50908,52956,55068,52988,52956,50876,50876,48828,48828,46748,42522,34070,27796,31858,35985,33840,33840,42325,44438,29712,17034,10728,8615,10696,8648,8647,6502,10695,10696,12776,8615,8582,10696,10727,10663,8615,10663,10663,10695,10663,10663,17034,12807,14888,42193,54838,35822,14824,42423,38033,35888,33840,31662,31727,27499,8580,4355,6436,12645,25453,50710,52758,48499,21127,17033,23305,29613,38000,54969,57083,65502,54937,59097,35822,21129,29547,10660,19114,38230,54936,52790,46287,35755,35755,35755,35755,37835,37835,37835,37835,37803,35755,37803,37802,37802,35754,35690,33642,33609,33610,40080,48400,48367,35820,27269,25157,25157,23274,40145,44337,46352,37933,29482,27336,25256,21093,6433,2273,4453,44403,52626,44142,48400,50480,50481,48433,190},
{30141,30141,30173,32221,32253,36381,38493,40541,42621,44669,42622,42621,48830,50942,53022,55102,55102,55103,48862,38526,34333,34333,34333,34333,36381,36413,40542,42622,48797,57150,61310,61342,63423,63423,63455,63423,61375,59262,55101,57181,57182,57149,57149,59229,59229,59229,59229,59262,59229,57149,57149,55068,57116,55068,55068,55068,55068,57181,59262,61342,61342,59294,59262,61310,59294,61342,59262,59294,61342,61342,61342,61374,61342,61342,61374,61342,63455,63455,65535,63455,57181,57149,57149,55069,55004,52956,52955,57149,61342,61309,57149,55036,52955,52955,52923,50843,46682,50876,52956,55068,55068,57116,52956,50875,52956,57148,57116,55036,52988,52956,55036,52988,52956,55036,52956,52956,57116,52956,50908,50876,48828,48828,48828,46779,42554,34070,29876,27729,35986,31760,35921,42325,42390,31857,17066,10793,8648,12809,12841,12841,8680,12809,12841,12841,12840,12808,14921,14889,12841,12808,12841,14921,14889,12841,14888,17001,17034,17034,48565,46419,31629,12678,25486,31695,29581,27468,29646,29614,31694,21063,8484,19049,29548,40146,52758,50579,52790,27433,10821,14853,12807,33774,52856,54937,63422,52791,50776,37967,29613,16902,12708,12709,19244,54969,52789,48433,35754,35723,35755,35755,37835,37835,37835,37835,35723,35755,35754,35754,35722,35723,35690,33610,33609,33610,40047,50545,48432,37867,25157,25157,20965,27469,42192,44272,44240,31561,31660,10434,2178,25287,6433,4353,17228,44273,50513,42093,48400,48399,48400,44207,185},
{30140,30173,32221,34301,36349,38429,40541,42621,44670,44670,44702,44701,48830,50942,50974,55070,55102,55102,46846,38494,34365,34333,34333,34333,34333,38462,40574,42622,44702,53022,59230,61342,63423,63423,63423,63423,59294,59262,57181,57150,57181,57149,57149,57149,57149,59229,59229,59229,59229,59229,57149,57116,57117,57148,55068,55068,57117,57181,59262,61310,61342,61342,59294,59294,61342,59294,59262,59294,61342,61342,61342,61342,61374,61374,63422,61342,63454,61375,61374,59262,57181,59229,59229,57149,55036,52988,52955,55036,61309,61342,59229,55068,55036,52955,52923,50843,46682,50844,52956,55036,55036,55068,52956,50843,50876,57116,55036,55068,55036,55036,55036,52988,52956,55036,52956,52956,57116,52956,50908,50908,48828,48828,48828,46747,44602,34102,27796,23569,36018,35986,38066,40179,33938,27663,17034,8680,8551,10728,10696,8648,4487,8647,10728,10760,8647,8582,10696,10696,12744,8614,8615,10695,10663,10663,10663,14986,17001,12808,44307,29386,20966,18854,18918,20966,20966,18919,16871,18984,29581,39853,37803,37771,46223,48368,48498,46321,54904,33739,10756,12805,8581,29549,50775,54904,61341,61243,35822,40113,37966,12740,10628,12742,19147,52888,52757,50546,35754,35755,35723,37803,37835,37835,37835,37835,37803,35755,37803,35755,35755,35723,35722,33642,33642,33610,42258,50546,46320,44175,29384,27303,25158,29548,44272,44239,40047,31496,35789,6240,6371,14819,6465,6497,27727,46385,50448,44207,46320,50480,48335,44207,193},
{32220,34301,34301,36381,36381,40509,42589,44669,44669,44702,46749,46750,48830,50942,53022,55070,55102,55103,48894,38494,36414,34333,34333,34365,36413,38494,42622,42654,44702,52990,57150,61310,61342,61375,63423,63455,61375,59262,59230,55101,55069,57117,57149,55069,57149,59229,57149,57149,57149,57149,57117,57117,57116,57116,57117,55068,57117,59229,61310,61342,61342,61342,59294,59294,61310,59294,59262,59262,61342,61342,61342,61342,61342,61374,61374,61342,61342,61374,59262,57149,59229,59261,59229,57149,55068,55036,52956,52956,61342,61342,61309,57148,55036,52955,52923,50843,48730,50875,52956,55068,52956,55004,52956,50843,52924,55036,52956,57116,55068,55036,55068,55036,52956,52956,52956,52956,57116,55036,52956,50876,48828,48828,48796,46748,46714,34102,27828,23569,29745,27600,27632,29680,14922,21292,15018,12841,10728,10728,12809,12841,10696,12841,14921,14921,14921,14888,14921,14921,14921,12808,12841,14889,14889,14921,14889,17034,17034,14954,40016,25192,18919,10532,12580,14661,12613,12613,16806,14726,42193,35855,39916,39819,48172,48172,44207,42160,48630,37998,14886,27466,12805,29613,46550,54936,50711,46485,46451,50645,40079,14853,10628,21194,46616,59163,52789,50643,35754,35722,29384,27369,33675,33642,31561,35722,35690,31529,33610,35722,31529,31529,33641,33609,31529,33642,48564,52692,48466,44241,33642,31497,31497,31530,42192,46352,42095,35788,29482,25288,21029,6433,4353,6563,38130,50546,50480,48434,42094,50481,48367,46287,196},
{34269,36349,36349,36413,38461,42557,44637,44669,44702,46750,46750,48830,50910,50942,50942,53022,55134,55103,48894,38526,34365,34333,34333,34365,36413,38493,42622,42622,44702,50942,57150,59262,61342,61374,63423,63455,61375,59294,59230,55101,52989,55069,55069,57117,57149,57149,57149,55101,57149,57149,57117,55068,55068,55068,55068,55068,55101,59229,61310,61342,61342,61342,61342,59294,61342,59294,59262,59262,61342,61342,61342,61342,61342,61374,61374,61310,59262,57181,59262,57181,59229,59261,59261,57181,57116,55036,52956,55036,63390,61374,61309,57148,55036,52955,52923,50843,48763,50875,52956,55036,50908,52956,50908,48795,50876,50876,52956,57116,57116,52956,57148,52988,50908,52956,55036,52956,55036,52956,50908,50876,50876,48828,48796,46747,46650,34102,27796,25649,25617,21391,21391,21293,23308,25454,19114,8680,8615,10696,10696,8680,8583,10696,10728,10728,8680,6534,12743,10695,10663,8583,8647,10695,8615,10761,10728,17033,14953,10760,35855,20999,10533,8420,8452,10533,8484,6372,6371,14823,25420,21227,37804,39884,44142,48172,46319,44240,44339,48597,46451,44338,19047,31661,44372,56984,46419,33807,31661,31792,35886,14853,10661,25485,57181,59195,52757,40047,35755,33674,18885,8453,27434,23078,8517,19049,31464,6339,10630,33674,12547,4258,23208,25125,10597,19114,42225,46353,37935,42225,37835,33610,33609,33610,42225,48432,40013,29449,29417,2145,8514,6401,4354,12968,40145,48465,46353,42193,42062,50513,46287,48432,188},
{36349,38429,38429,38461,40509,42557,44637,42621,44702,46750,46750,48830,48862,50910,50942,52990,55102,55102,48894,40542,34365,34334,36381,36414,36414,38494,40574,42622,44702,50942,55102,59230,61342,61374,63423,63423,63423,61375,61342,57182,55101,55101,57149,57149,57149,57149,55101,55101,55069,55069,55068,55036,57116,57116,55068,55068,55101,59229,61310,61342,61342,61342,59294,59294,59294,59294,59262,59262,61342,61342,61342,61342,61374,61374,59294,55069,55069,55069,59262,59262,59229,59262,59261,59229,57149,52988,52955,55036,63390,63390,61309,59197,55036,52955,52955,50843,48763,50875,52924,52988,52956,52956,50876,48795,50843,48795,50876,57148,57116,55036,57148,55036,50876,52956,55068,52956,52955,52924,50908,50876,48828,48828,46747,46747,44570,32022,27828,25617,27665,23471,23439,21358,33906,33874,17066,12841,12808,12776,10728,10728,10728,10792,10760,10761,12808,10760,12776,12776,10728,10728,10760,10728,10696,12873,14954,14986,17066,21162,25258,12581,6340,6339,8420,10532,4388,2243,10597,8580,6501,33807,42029,41932,46092,48204,46319,44273,50711,54904,52823,27465,12742,33774,40179,52791,44306,31694,23275,14856,14790,12774,38098,44437,61342,59228,52757,42160,37836,35723,20965,8518,25321,27271,8518,16936,33609,6339,10598,31594,16741,4292,19048,29320,8518,14921,38031,37900,33774,44338,35722,33642,33642,33642,44338,50513,37901,29449,25192,0,8482,4321,4353,27629,18852,10564,8484,10662,33773,48465,48401,50546,194},
{36381,38429,38429,40509,40509,42589,44637,42589,44669,46749,48830,48862,48830,50942,50942,50942,55102,55103,48894,40574,36413,34333,34333,36414,36414,38494,40542,42622,44702,52990,55102,59230,61342,61342,63423,63423,63423,61375,61342,59262,57149,57181,57181,59229,59229,59229,57149,57149,55069,57117,55068,55068,57116,57148,55068,55068,55069,59229,59262,61342,61342,61342,59294,59294,59294,59294,59262,57214,61342,61342,61342,61342,63422,63422,61342,55101,52988,55069,59261,57214,57149,59229,61309,59261,57148,55036,52956,57116,61342,63390,61309,57149,55036,55036,52955,50843,48763,52924,52956,55036,52956,50876,50875,50843,48795,48763,50875,57148,57116,55036,57116,52956,50876,50876,50876,52956,52956,52955,50908,50876,48827,48795,46747,46747,44602,32022,31956,17132,23341,25454,23373,23341,27567,31760,12840,10760,8648,6535,6535,6535,6535,6535,4486,6535,6502,6535,6502,6470,6470,6502,6502,6502,6502,6632,10695,10727,10662,16903,25323,8453,6372,8484,10565,6468,6468,4323,6436,10692,19015,25289,41997,39787,48172,50285,48368,44240,50711,54904,54904,33707,16935,35888,38066,29580,46419,35920,37967,40081,25257,14952,50776,54970,61342,61309,52757,50644,39980,35755,25191,8485,21129,31464,10598,14856,35722,10467,8550,29514,20933,4291,16935,31496,10533,12743,37999,31594,35919,46353,35690,33610,33609,33675,46418,48433,37901,31562,29449,25256,19013,8515,6468,40242,23012,4258,4258,4324,23306,44240,44208,44306,186},
{36413,38429,40477,42589,42589,44669,44669,44669,44669,48797,50910,48862,48862,50910,50942,50974,53022,55134,50942,42622,36413,34333,34365,34365,38494,38494,40574,42622,46750,50942,57150,59262,61342,61343,61342,61342,61342,61343,59262,59294,61342,61310,61310,61310,61342,61310,61310,59229,57181,57149,57149,55068,57149,55068,55036,55036,55069,59229,61310,61342,61342,61342,59294,59294,59294,59262,59262,57182,61342,61342,61342,61374,61374,63390,61374,55101,52956,55069,55069,55069,55101,59229,61310,59229,57148,55036,52988,57117,61309,61342,61309,57149,55068,55068,55003,50875,48763,52923,52955,55036,55036,52956,50875,48795,48763,48763,48795,55068,57148,55036,55068,52956,50907,50876,50843,50876,52956,50875,50876,50908,48827,48796,48795,46748,44602,31990,29811,16937,21162,25323,27436,27468,27468,27468,17001,14889,14889,14921,14889,14889,14888,14888,14889,14856,14888,14856,12841,12808,14856,12808,12808,12808,12776,14921,14856,14888,12808,16936,21097,19016,25355,18984,8580,4420,4388,12710,19082,27466,44306,25256,48270,41867,46157,46125,48367,31661,52823,54969,54937,50645,48532,44306,42258,31629,50743,40113,48630,48564,31595,23341,52825,57083,59261,59196,52790,42192,40013,37803,35691,31530,33643,35722,31530,31563,35755,31497,29482,33675,33577,29417,29481,35689,29417,29417,37999,33708,40080,44239,33610,33610,33609,33740,48498,46352,33642,31595,14659,8546,10594,6402,10759,50644,33609,10629,4258,4291,21194,44272,44208,42225,177},
{40509,40509,40509,42589,44669,46717,44669,44637,44669,46749,48862,50942,48862,50910,50942,53022,53022,55102,50974,42654,36413,34365,34333,34365,36446,38494,40574,42622,46782,52990,57150,59262,61343,61343,61342,61342,61342,61342,59294,61310,61342,61342,61342,61342,61342,61342,61342,59262,59262,59229,57181,57149,57117,55068,55068,55036,57117,57181,61310,61342,61310,61342,59294,59294,59262,59262,57182,57182,61342,61342,61374,61342,63422,63422,61342,57181,52988,52988,52956,50876,52956,57117,59261,59229,57148,55036,52956,57117,61309,61342,61341,57181,55068,55036,55004,52923,48763,50875,52955,52988,55036,52956,50876,48795,48763,46715,46715,52956,55068,52956,55036,55036,52955,50876,50875,48827,52956,50876,44536,12874,23341,23373,25421,25422,23373,23309,21228,16937,21162,25323,27404,27436,27468,27469,27404,25356,25323,25323,25388,27468,25388,27436,27468,25356,25323,25356,25388,23275,35919,27599,21260,25486,19147,27567,29710,31790,31758,35887,29580,40146,27467,10693,6500,6468,8549,19080,23307,19047,44372,25224,44109,46125,48205,46157,42061,31661,52856,54936,54904,42225,21128,31726,38066,61308,44338,23307,46550,48597,33708,33873,52824,48598,46518,52823,54871,52822,46287,37835,37835,37803,37803,37835,37835,37867,37835,37835,37835,37835,37803,37803,37835,37802,35754,35754,37966,40047,42192,44175,33609,33609,33609,35821,48497,44304,31497,33740,8417,8481,16836,14658,21357,50513,44468,29579,4258,2178,12808,40047,48400,46287,152},
{42589,42589,42589,42621,46717,46749,44669,44637,46717,46749,48862,50942,48862,48862,50942,52990,55070,55102,50974,42654,38462,36381,34333,36413,38493,40542,42590,40574,44702,52990,55102,59262,61343,61375,61375,61342,59262,61342,61342,61342,61342,61342,61342,61342,61342,61342,61342,61342,61310,59261,59229,57149,57148,55068,55036,55036,55101,57181,61310,59294,59262,59262,59262,59262,59294,59262,55101,57182,59262,61342,61342,63390,61374,61374,61342,59262,52988,52956,52956,50876,50876,55069,59229,59229,57149,55036,52956,55068,57149,61309,59229,57149,55068,55036,55036,52923,48763,50876,52956,55036,55036,55036,50875,48795,48763,46715,46683,50843,55068,52956,52956,52956,52956,50908,50876,50875,52988,50908,42358,8517,23243,23275,25323,25324,23243,23275,23243,16937,21130,23243,25323,25356,27468,27501,27468,27436,27404,27468,27469,29549,29581,29549,27501,27469,27469,27468,27469,27436,40178,33938,25421,29712,21325,31857,33969,38161,33871,29548,27469,42259,19047,8581,8580,6468,6468,8612,8613,16967,31727,29482,33610,39819,50253,46092,46255,38000,50743,54937,50644,21158,14854,29613,33839,61374,54838,19016,48663,50645,42193,44340,46551,48630,46550,50743,54903,54870,48400,37803,37835,37835,37803,37835,37835,37835,37835,37835,37835,37835,37835,37835,37834,37802,37770,37803,40047,44240,46385,42094,33609,31561,33609,35854,48432,44240,35788,31595,40113,48499,46189,18723,33969,48433,38161,23338,4226,2178,12710,35853,50513,46222,159},
{42589,44670,44669,44701,46749,46717,44669,44669,46717,48829,48862,50942,48862,50910,50942,52990,53022,53054,50974,42654,36413,34365,34333,34365,38494,40542,40574,42654,44702,48862,55070,57182,61342,61375,61375,61374,61342,61342,61342,61342,61342,61342,61342,61342,61342,61342,61342,61342,61342,61310,59229,59229,57149,57116,55036,55036,57149,59229,59262,59294,59262,59262,59262,59262,59294,59262,55101,55101,59262,61342,61342,61374,61374,61374,61342,57182,55037,52989,52956,52924,50908,55036,57181,59229,57149,55036,52956,52988,57117,59229,59229,57149,55068,57116,57084,52956,50843,50876,52956,55036,57116,55036,50875,48795,48763,23374,12777,14890,14857,14922,17002,17002,14954,17035,17035,17035,17067,17067,17002,10631,12743,14792,12711,12679,12679,10630,10598,10598,12679,10631,10630,12711,12743,12678,12679,14759,12679,10598,12711,14791,12743,14791,19050,18985,12711,14791,16872,14791,21130,17002,14856,16969,12776,17034,14921,21226,19049,21130,10630,16937,12710,8517,8484,8484,8517,10629,8549,23308,31727,23242,35723,39786,50285,50318,48400,42128,54904,54904,35884,12804,8547,25453,31693,55002,50678,33774,50743,46451,33807,48598,52890,48630,48630,48662,52855,52822,50545,35754,37803,37835,37803,37835,37835,37835,37835,37803,37803,35787,37835,37835,37835,35754,37770,37802,37934,46352,48466,39948,33609,33577,31529,37999,48433,42127,31562,29450,21193,25386,41963,18821,46451,48434,31497,10565,6339,6404,31629,42158,48433,52560,175},
{42621,44702,46750,46750,46749,46717,46717,44669,46717,48797,48862,50942,50942,50910,50942,52990,55070,53022,53054,42654,36414,34333,34333,36413,38494,38526,40574,42654,44702,48862,53022,55070,59230,61343,61343,61342,61375,61342,61342,61342,61342,61342,61342,61342,61342,61342,61342,61374,61342,61342,59262,59229,57149,55069,55036,55036,55069,57181,59230,59262,59294,59262,59262,59262,59262,57214,55069,55101,59262,61342,61342,61374,63422,63422,61374,55101,55037,52989,52956,52924,50908,55036,57149,59229,59197,55068,52956,55036,57117,59229,59229,57149,57116,57148,57116,52956,50843,50876,52956,55036,57116,55036,50875,50843,48763,21164,10663,12712,12679,12712,12743,14823,12743,14792,12744,12744,12743,12743,12711,12711,12711,12711,10631,10598,10599,8550,10566,8485,10566,8486,6405,8518,10566,6405,8453,8518,6437,6372,8485,8518,6405,8452,14792,12711,8453,8485,10533,6373,6372,10534,10566,10663,16904,21163,16969,16937,21098,19082,14856,16936,19050,16936,14823,18984,16936,14823,14823,27534,33775,23210,35691,43980,48172,48173,50546,48433,54937,54936,25352,10724,8548,17034,29580,54969,52791,31726,50743,48564,35919,50711,55035,48630,48630,48597,52855,52822,50611,37802,37803,35754,35755,37835,37835,37835,37835,37835,37802,35787,37835,37835,37834,35722,35722,37802,37901,46385,48465,37835,31529,29515,29548,40112,46352,39981,31529,27337,64,6433,14755,21259,48401,44240,16739,6404,6404,10663,44272,42126,50513,50513,181},
{46750,46750,46750,46750,46749,46749,46717,44669,44701,46749,48829,50942,50942,50942,50942,50974,53054,55070,53054,44734,38461,34365,34333,36414,38462,38494,40574,42654,44702,48862,50974,53022,57182,59230,59262,59294,61342,61342,61343,61342,61342,61342,61342,61342,63422,63390,63422,63422,63422,61342,61310,59229,57149,57148,55068,52988,55068,57149,57182,59262,61342,59294,59262,59262,59262,59262,55101,55069,59262,61342,63390,63422,61374,61342,59294,57149,52956,52956,52956,52956,52988,55037,57149,57149,57149,55068,52956,52956,57117,59261,59229,57181,57149,57148,55068,52956,52923,52924,52956,55036,57116,55068,52923,50843,48763,19083,12711,12712,12711,12744,12743,12743,12744,14791,12743,12743,12743,12743,12743,12711,12711,12711,10630,10598,10599,8518,10566,8518,10598,8518,6405,8518,8517,6405,6405,8518,6437,6372,8486,8517,6373,8453,10566,8485,6372,8517,10565,6405,8452,8486,8453,19050,21195,31793,29712,31825,29711,31824,29679,29679,31857,31857,33905,31792,31824,31824,29646,35953,29516,33676,33610,37738,46092,46092,50611,42094,52856,54903,19013,12773,16935,21227,38000,54937,52726,31629,50710,44339,42094,46517,59261,48630,48598,48630,52823,52789,52724,37802,35754,35755,37803,37835,37867,37835,37835,37803,37803,37835,37835,37803,37835,35754,35722,37802,37836,44305,48433,35690,31529,29516,29581,42224,46320,39981,31594,29449,18949,8482,6401,29776,48368,44109,10467,6372,6372,21227,46222,42061,48400,46352,185},
{46749,46750,46749,46750,46749,46749,44701,44701,46749,46749,48861,50942,50942,50942,50942,50942,53022,55102,55102,44766,38494,34365,36381,38461,38494,38526,40574,42654,46782,48862,52990,53022,55102,55102,57182,57182,59262,59262,61310,59262,61342,61342,61374,61374,63422,63422,63422,63422,63422,63422,61342,61310,59229,57149,55068,55036,55068,59229,59262,59262,61310,61342,59294,59294,59262,59262,55069,55069,59230,61342,61342,63422,61374,61342,55069,55037,52988,52956,52924,52956,55069,55069,57149,57149,57116,55068,52956,52956,57149,59229,59229,57149,57116,57116,55036,52956,52956,52923,52956,55036,57116,55036,50875,50843,48763,16970,12679,12711,12711,12711,12711,12711,12711,14759,12711,12711,12711,12711,12743,12711,10663,12711,12679,10599,10598,8517,8485,8485,10566,8486,6405,10566,8485,6405,6405,8486,8453,6372,8485,8485,6373,8453,8518,8485,6373,8485,8517,6373,8453,8485,8485,21163,25421,36051,31825,33905,36018,33905,29679,33873,36018,38000,40145,33873,31792,29711,38098,36017,33774,35790,31563,37738,46092,46157,54838,37966,54936,50611,14884,16934,29613,23243,44339,50743,50613,35854,40112,42291,50513,46484,55002,48663,48630,46582,52856,52790,54837,37835,37835,37803,37835,37867,37835,37835,37867,37835,37803,37835,37867,37803,37803,37770,37770,37770,35787,42224,46320,33609,33577,31561,33643,44304,44240,35788,33643,20966,12644,27369,27337,48596,46353,35624,6371,6372,6372,29645,48335,46255,50480,48368,186},
{46782,46782,44702,46750,46749,46749,46749,46749,46749,48829,48861,50942,50942,50942,50942,50942,53022,55102,55103,46814,38494,36413,34365,36414,38494,40542,42622,42654,46782,48862,50942,53022,55102,55102,57183,55134,57182,57182,59262,59262,61310,61342,61342,61374,63391,63422,63422,63422,63422,63422,63390,61310,59229,57149,57116,55068,55036,57149,59262,59262,61342,61342,59294,59262,59262,59262,55101,52989,57181,61342,61342,63422,63422,61374,57149,52988,52988,52988,52956,55069,53021,55101,55069,57149,57148,55068,52956,55036,57148,59229,59229,59229,57116,55036,52988,52956,52956,52956,55004,55004,55036,52956,50843,50875,48731,14857,10631,10663,10631,12711,12711,12711,12711,12711,14759,12711,12711,12711,12711,12711,12711,12679,10630,10598,8518,10566,8485,8486,8518,8485,6405,8518,8486,6373,6437,8517,6405,6404,8517,8485,6373,8453,10565,8453,8453,8517,8485,6372,8453,8485,10598,27534,25421,36018,29712,33905,38098,33970,31760,35953,35986,40080,37934,16870,29614,18951,33840,29515,29581,33644,27436,37869,27338,39982,40113,23177,42226,38032,23143,37999,29678,29450,40178,31694,42160,42226,25256,44469,54772,48500,54938,48663,48630,46485,46583,52790,54869,41996,39883,37803,37835,37835,37803,37803,37803,37835,37803,37803,37835,37803,35754,37802,37802,37802,37835,42193,44207,31529,33577,31529,33707,46320,40079,33610,35788,8321,4291,35919,46289,48531,46352,22980,6372,4324,8518,40079,46254,48466,48399,42062,191},
{46749,46750,44702,44702,46750,46749,46749,46749,48830,48829,50909,50942,50974,50942,52990,50942,53022,55102,55102,46846,38526,36413,34365,36414,38494,40574,42622,44702,46782,50942,52990,53022,53055,55102,55134,57182,59230,59230,59262,59262,59262,61342,61342,61374,61374,61374,63422,63422,61374,63422,63390,61342,59261,57181,55069,55068,55036,57117,59262,61310,61342,61342,59294,59294,59262,57182,55069,52989,57149,61342,61342,61374,61374,61374,61342,57181,55069,55069,55069,57149,52956,55069,55069,55068,57117,55068,52956,57116,61342,59197,57181,57181,57149,55036,55036,52956,52956,52956,55004,55004,55036,52923,50843,50843,46650,14857,12711,12711,10631,12711,14791,12711,12711,12711,14791,12711,12711,12711,12711,12679,12711,12711,10598,8582,10566,10566,8518,8485,10566,6438,6405,8550,8486,6405,8485,8518,8485,6404,8518,8485,6372,8453,8517,8453,8453,10565,8485,6372,10533,14823,35953,35953,23275,31760,27534,27534,31792,29711,27534,31759,31792,42128,42030,16805,21194,12644,25355,27337,25290,31563,21097,33708,21064,33708,33742,21030,35854,27500,27304,37933,23209,29449,37967,21063,37868,38032,23078,44405,59030,42160,52857,50743,48663,48598,38132,52823,54870,44141,39884,37835,37803,35723,35722,35690,35754,37803,37803,37803,37803,37802,37802,35754,37803,37835,37835,42160,42061,31497,35821,35821,35821,29449,29514,29448,31562,14692,16935,37967,50645,44208,46254,10466,6372,4323,14953,46319,48400,50513,48432,44206,197},
{46750,46750,44702,44670,44702,46749,46749,48798,48829,50910,52958,53022,50974,50942,50942,52990,53022,53022,55102,48895,40574,36414,36413,38462,40542,42590,42622,44702,48862,50942,50974,53022,53022,55102,55102,57182,59230,59262,59294,59262,59262,61342,61342,61342,63422,61374,63422,63422,63422,63422,63390,61342,59261,57181,57117,57116,55036,57116,59261,61342,61342,61342,59294,59262,59262,57182,55069,55037,57181,61342,61342,63390,63422,63422,61374,59262,57182,59229,57149,55069,52956,55037,59230,55069,55036,55036,52956,59197,63422,61342,57181,57149,57148,55068,55068,55036,52956,52956,55036,55036,52956,52956,50843,50844,44569,14825,12679,12711,12679,12711,12711,12711,12711,12711,12744,10663,12711,12711,12711,12679,12711,12711,12679,10598,10598,10534,8518,8550,10566,6405,6405,8517,6437,6373,8453,8486,6372,6405,8486,6405,6404,6437,8485,6373,6405,8485,8453,8453,14824,57115,54871,44208,37706,35659,35594,35626,35658,35691,33578,35659,33643,46288,40047,10499,14726,16837,21063,29449,23111,29449,23111,31497,23143,29482,25256,29384,31498,25223,31529,31497,25223,29449,29449,29384,33642,31497,33610,46517,61143,46386,50678,50776,59196,59163,48631,50742,54838,46255,39916,37835,37835,37803,35723,35723,35754,37835,37835,37803,37835,37803,37802,35722,37803,37835,37835,40048,37835,31529,35821,37967,35821,25256,21064,31562,31562,18917,14790,38098,54838,46353,37769,8419,6372,6372,27598,46319,50513,50480,48400,46287,197},
{46750,46750,46750,46718,46750,48797,46782,48829,48861,50942,52990,50974,50974,50942,50942,50974,53022,53022,55102,48895,42622,38462,36413,38462,40542,42622,42622,44702,48830,50942,53022,55070,55070,57150,57182,59262,59262,59262,57214,59262,59262,59262,61342,61342,61374,63423,63422,63422,63422,63422,63422,61342,59261,57181,57149,57149,55068,55068,59229,61342,61342,61342,61342,61310,59262,57150,55102,55069,57149,61342,61342,63422,63422,63422,61374,57214,57149,59262,57182,52988,52956,55069,59262,59262,57181,55069,55069,59229,61342,63422,59229,57149,55069,55068,55036,55068,52956,52956,55036,55036,55004,52924,50843,50875,44504,12744,10631,10631,12679,10630,12678,12679,12711,10663,12711,10663,12679,12679,12711,12679,12711,12711,12679,10598,10598,8518,6437,8518,10566,6405,6437,8486,6437,6372,8485,8485,6405,6372,8517,6405,6372,8485,8485,6373,6405,8485,8452,6372,19049,59195,56984,46158,41899,41867,41866,41834,41867,41867,41867,41866,41899,48401,42160,10499,14757,23144,21030,29417,23046,27369,23078,31530,29384,37803,39948,39947,39948,39915,39948,39947,39882,39915,42028,42060,44108,46189,31562,42259,59129,46418,48565,52857,61341,61309,54970,52856,54903,48367,37835,37835,37835,37803,35755,35722,35754,37835,37835,37835,37835,37803,37802,35754,37803,37835,37835,39982,33609,33610,35821,42192,37901,29482,16870,31627,31594,29448,23209,46582,52659,48401,25125,8452,8452,10598,40079,37901,50513,44142,46287,48399,186},
{46782,46782,46750,46750,46750,48829,46782,48829,50942,53022,53022,53022,50942,50942,50942,52990,52990,52990,55102,48894,40574,36446,36413,38494,40542,40574,42654,44702,48830,50942,53022,55102,55102,57182,59230,59262,59262,59262,59262,59262,59262,59230,59262,61342,61375,63423,63455,63455,63423,63423,63422,61342,61310,59229,57149,57148,55068,55069,59229,61342,61342,61342,61342,61342,59294,55133,55102,55069,57149,61342,63390,61374,63422,61342,59230,57149,57181,57149,57181,52989,52988,57149,59262,61342,59262,59262,59262,59261,59229,59262,59261,57149,55068,55068,55036,55068,55036,52924,55068,55036,55036,52956,50843,50875,42424,14824,10631,10631,12679,12743,12711,12711,12711,12711,12711,12711,12711,12711,12711,12711,12711,12711,10630,10630,10630,8518,8485,10598,10598,6437,6437,8518,6437,6373,8485,10565,6372,8421,8485,6404,6372,8485,8485,6404,8453,18985,12711,10598,27533,57082,54936,46190,41867,43914,41866,41866,41867,43947,43915,41866,44012,52660,44208,16772,14725,18950,27336,33642,31497,33642,31465,33642,31561,35755,33642,35755,37835,35722,37867,37835,35722,37835,39915,39947,42028,42028,35756,35952,59162,44305,50678,52824,59229,61341,57083,54937,48630,48432,37835,37835,37835,37803,35755,35722,35754,37803,37835,37803,37803,37802,35754,37803,35755,37835,37835,37835,35722,35756,35854,44273,35690,31531,2080,31725,33643,27337,12743,52889,48400,50480,42029,42029,39981,39981,46319,44207,50513,46288,48400,44141,173},
{48862,46782,46782,46782,48830,48830,48862,50910,52990,55070,55102,53022,50942,48862,50942,52990,50974,50942,53054,48894,40574,36446,36414,38462,40542,40574,44702,44702,48862,50942,50974,55070,57150,57182,57182,59262,61310,61310,61342,61310,59262,59294,59262,61310,63422,63423,63455,63455,63455,63422,63422,61342,59262,59229,57149,57116,55068,55069,59229,59262,61342,61342,61342,61342,59294,59262,55102,55101,57149,59262,61342,61374,61374,57214,57182,59229,57181,57182,57181,55101,55069,59229,61310,61342,61342,61374,61310,61310,57149,57181,57149,57149,55069,55036,55036,55068,55068,50876,57116,55036,55036,52956,50843,50875,42358,12744,10663,10631,12711,12711,12711,12711,10695,10663,12679,12711,10663,12711,12711,10663,12711,12679,10598,10630,10598,10599,8518,10598,10566,6405,6405,8486,6405,6372,8485,8486,6372,8453,8485,6404,6372,8485,8453,6372,8485,12711,10533,10565,38066,52561,54805,37804,37674,37673,37641,37673,37706,39753,37673,39722,44013,56853,44208,18885,10564,16870,21063,29482,25191,31530,23079,29482,25191,29514,25223,29481,27401,25223,31562,27337,29416,31595,27336,33674,33707,29416,37900,37966,61177,42192,50710,52824,59196,61342,57083,50678,44372,46418,37835,37803,37803,37803,35722,35722,35722,35754,35755,37770,37770,35754,35754,37770,35722,42258,48565,48532,46419,46386,37933,46386,40014,33643,25224,33708,35723,33643,27468,54903,50513,50512,48367,50480,48367,50513,48367,48400,48400,48531,50480,48367,176},
{50942,46814,48862,48862,48830,48862,50942,50942,50942,55102,55102,53054,50974,50942,50942,50942,50974,52990,55102,48894,42622,36445,36414,36446,38494,40574,44702,48830,48862,50942,53022,53054,55102,57182,59262,59262,61342,61342,61342,61342,61342,61342,61342,61342,63422,63423,63455,63455,63454,63422,63422,61342,59262,59229,57149,57149,55068,57117,59229,59262,61342,61342,61342,61342,59294,59262,57182,55101,57182,61310,61342,61374,61374,59230,59294,59262,57181,57181,59229,57182,55069,59229,61342,61342,61342,63422,61374,61310,59229,59229,59229,57149,55069,55068,55036,55036,52956,50876,55036,52988,55036,55036,50843,52924,38165,12711,10631,10631,12678,12679,10630,12711,10663,12679,14727,10663,10630,10663,12711,10630,10630,10630,10598,10598,10630,8550,8518,8550,10566,6405,6373,8485,8485,6372,8485,8485,6372,6404,8485,6372,6372,8485,6404,6372,10533,10598,8452,10565,27468,52529,52627,27338,37673,37674,37641,37673,37673,37674,37673,37673,41964,56885,46321,16772,12677,21064,23111,29482,25191,29482,25191,31530,25223,29482,29416,31561,31529,29416,33642,29416,31497,33642,29416,35690,35723,31528,35755,27402,59130,44273,54871,50711,57148,61309,57082,40145,35887,42192,37803,37802,37835,37803,35722,35722,35722,35722,37803,35754,35754,35754,35722,35754,37770,44372,52856,52791,50645,48564,37934,46386,35787,31562,10531,31725,41900,31303,35661,50579,48432,50480,50513,50479,46287,50513,48335,50513,46386,42226,48400,48367,179},
{53022,50942,48862,48862,50910,50910,48862,50910,52990,55102,55134,53055,50974,50942,48894,50942,52990,52990,55102,48894,42654,36414,36413,38462,38494,40574,46782,48862,48862,50942,53022,53055,55102,55102,59230,59262,61342,61342,61342,61342,61342,63422,61343,61342,63423,63423,63455,63455,63454,63422,61374,61342,61309,59229,57149,57148,55068,57117,59229,61342,61342,61342,59294,59294,59294,59262,59230,57181,57181,59262,61342,63390,61342,59262,61342,59262,57181,57149,59230,55102,55069,59229,61342,61342,61342,63422,63422,63422,61342,63422,63422,59262,55069,55036,55004,55036,55036,52956,55004,52956,55036,55036,50843,50875,38133,12711,12679,10631,12679,12711,12711,12679,12711,12679,14759,12711,10630,12711,12679,10663,10630,12711,10630,12679,10631,10598,8485,8518,10598,6405,8453,8518,8453,6372,8518,8485,6372,8453,8517,6372,6404,10566,6404,6372,10566,12679,10565,10598,19050,37804,42160,33676,37673,37673,37641,37673,37673,39722,37673,39722,41965,56886,46354,27402,14692,21031,23078,29416,25158,29417,25158,29417,29417,35722,39947,39915,39883,39915,39947,39915,39882,39882,41995,42060,44108,44140,39949,27370,57048,46353,54871,50744,57116,61342,61244,42193,38065,48597,37835,37802,37835,37803,35722,35722,35690,37802,37803,35722,35754,37770,35722,35722,37802,40145,52791,52759,46288,46418,37934,44273,33610,31466,2145,35985,41900,33287,37807,44241,48400,50480,50480,50480,46287,50480,48400,50480,44175,40080,50480,48400,194},
{53022,53022,50910,50910,50942,48862,48862,50910,53022,55134,55134,55102,50974,50942,50942,52990,53022,50974,55102,50974,40606,36414,36413,38462,40542,42622,46782,48894,50942,50942,53022,55102,55103,55102,57150,59230,59262,59294,61310,61310,61342,61342,61374,63422,63423,63423,63455,63455,63455,63422,61342,61342,59262,59229,57149,55101,55068,55068,59229,61342,61342,61342,59294,59294,59294,59294,59262,57149,59262,59262,61342,61374,59262,61310,61342,59262,57181,57182,57181,57149,55069,57149,59262,61342,63422,61374,63422,63422,63422,63422,63422,61310,57117,55068,55036,57116,55068,55036,55036,55004,55036,52956,50843,50875,36051,14824,12711,10631,14759,12711,12711,12711,12711,12711,14759,12711,12711,12711,12711,12711,12679,14759,10598,12678,12711,10598,8518,10631,10566,6405,6405,8517,6405,6372,8517,8485,6372,6405,8517,6372,6404,10533,6372,4292,8485,8453,6340,10501,10565,23144,42161,35789,37674,37673,37673,37641,39721,37673,37673,39721,41965,56918,46321,44240,20998,18983,29514,29449,39982,29417,35821,29449,33708,33610,33675,37868,37901,31529,42127,35788,35722,40046,31529,39981,37901,35689,42094,27403,52725,42128,54904,50679,57115,61342,61309,44306,35919,38098,37835,37835,37835,37803,35722,35722,35690,35722,37802,35722,35754,35722,35690,35722,35722,37935,50743,50645,48466,44273,40047,42127,39982,29449,27370,35821,37740,33320,40082,44240,44174,50513,50480,52593,50415,50513,50546,44174,37836,44272,48400,48400,211},
{53022,53022,50942,50942,50942,50942,50942,50942,55070,55134,55135,55103,50974,50942,50942,53022,50974,53022,55102,50974,42654,36414,36413,36414,40541,44702,46782,48894,50942,50942,53022,55102,55102,55134,57150,57182,57182,59262,59262,59262,61310,61342,61342,63423,63423,63423,63423,63455,63423,63422,61342,61342,59262,59229,59197,57149,55068,55068,59261,61342,61342,61342,59294,61342,61342,61342,59262,57182,59262,61310,61342,61374,59262,61310,61342,59262,57181,57181,55101,55101,55069,57149,59261,61342,63454,63422,61342,63422,61341,63390,61374,59262,57149,57149,57116,57149,57116,55036,55036,52956,52988,52923,48795,50876,38100,14824,12743,14791,14824,14824,14823,14791,16872,14824,14792,16872,14791,14791,14791,14791,12743,14791,12679,12711,12711,12679,10598,12711,12679,8486,8485,10598,8518,6405,10598,8518,6405,8485,10598,6405,8485,10566,6437,6404,10598,10565,8453,10533,10533,6404,42258,44208,37673,37641,37673,37673,37673,37673,37673,37673,41964,56918,48434,44207,29450,27402,42095,44175,44175,35690,37868,37802,37835,37803,37803,37835,37835,35754,37868,37836,37835,37868,35722,39916,37868,37835,39981,37868,42029,33676,52856,46453,57116,61342,61341,48565,25355,29678,39948,37835,37803,37803,35722,35722,35690,35722,35722,35722,35722,35723,35722,35722,35722,37869,52824,52758,48531,46387,42160,37836,31562,16772,12742,35659,35660,33417,44372,50611,46319,50480,50480,50513,52561,50513,50481,46287,39948,35885,44206,48433,222},
{53022,53022,50974,50942,50974,50974,50942,53022,55103,57183,57183,55103,53022,53022,50974,53022,52990,53022,55102,53055,42654,36414,36413,36446,42622,44702,46782,50942,48894,50942,53023,55102,55102,55102,57182,57182,57182,57182,59262,59262,59262,61310,61342,63423,63423,63423,63423,63423,63422,61342,61342,61342,61310,59229,57149,57116,55068,55036,59229,59262,59262,59294,59294,59294,61342,59294,57182,59230,59262,61342,61342,61342,59262,61342,61342,59262,57181,57149,55069,55069,55069,57149,59229,61310,61374,63422,61342,61342,59229,59229,57181,59261,59261,55101,57149,59229,57116,57084,55036,52956,52956,52956,48795,50875,36019,14824,14824,14792,16840,14824,14823,14791,14824,14824,14792,16872,14823,14792,14824,14791,14791,14791,12711,12679,14791,12679,10598,12711,12678,10566,8485,10598,8485,6372,10566,8518,6372,8485,10566,6373,8485,10598,6405,8453,10566,8485,6404,10533,10598,8452,40113,39950,37641,37641,37641,37641,37673,37673,37673,37673,39851,56918,48434,44240,29483,23308,38033,38065,44175,35722,39947,39915,39883,39883,37835,37835,39915,39915,39883,39915,39915,39915,37802,39883,39915,39915,39947,41995,41963,37901,50776,46452,57149,61342,61341,52791,46517,50776,44142,37802,37802,37802,35722,35722,35690,35690,35690,35722,35722,35722,35722,35754,35722,35756,48663,52823,50646,48598,48597,42161,37967,33708,33773,29223,33579,37709,50644,50545,50545,50480,50513,50545,52593,52593,44141,48432,46221,44239,46287,48532,227},
{53022,53022,53022,50974,53022,53022,50942,53022,55103,57182,55135,55135,53054,53022,50974,53022,50974,53022,55102,50975,42654,36414,38462,38494,40574,44702,46782,48894,48894,50942,53022,55134,57182,57215,59263,59262,59263,57150,59230,59262,59262,59262,61342,61374,63423,63423,63423,63423,63422,63390,61310,59294,59262,59261,57181,57116,55036,55036,55069,57149,59262,59294,59262,59262,61342,59262,57182,59229,59262,61342,61342,59294,59262,61342,61342,59261,57181,55101,55069,55069,55037,55069,57149,59261,61342,63422,63422,61342,57117,57149,55068,59261,63390,61342,59229,59197,57148,55036,44471,40245,42325,27534,23374,25454,31695,14824,12743,12743,16840,16872,14824,16872,14824,14824,14792,14791,14791,14791,14823,12743,10663,14791,12711,10662,14791,12679,10565,12678,12711,8518,8485,10598,8453,6372,10598,10565,6373,8486,10566,6405,8453,10598,8485,8452,12646,10533,8453,10565,12646,6372,35920,39917,37641,35561,37641,37673,37673,37674,37674,37673,39819,56853,42128,42193,29515,21163,27404,29549,42095,35690,39915,39915,39915,39915,37867,37834,39915,39915,39883,39883,39915,39915,37801,37834,39915,41963,41995,41995,41963,39949,48631,48598,57149,63422,61341,50743,48663,57082,48367,37834,35754,35754,35722,35722,35722,35722,35690,35722,35722,37770,35722,35722,35690,35722,40079,42160,42160,42193,42193,42160,40080,37966,42095,24965,29386,44209,48465,48400,50512,48399,50513,50579,52593,52593,46222,48432,48335,50481,50513,52889,219},
{52990,53022,53022,53022,53022,53022,52990,55070,55102,57182,55135,55134,55103,55103,53022,53022,50942,53022,55102,51007,42654,36446,38462,38494,40574,44702,46782,48862,50942,50942,55102,55134,57182,59263,59263,59294,59263,59263,57150,59230,59230,59262,61310,61342,63423,63423,63423,63423,63422,63422,61342,61310,59261,59229,59197,57148,55068,52956,52988,55069,57149,57182,57182,59262,59294,57182,57182,59230,59262,59262,61342,61342,59262,61310,59262,59229,61342,57182,57149,55069,55069,55069,57149,59229,59262,61342,61374,57149,55068,55069,52988,59229,63422,63422,61342,57149,57149,42359,25455,35922,46454,21196,19050,19050,27469,14824,14791,14791,14791,16904,14824,16904,14856,14824,14792,14791,14792,12743,14792,14791,12743,14791,14791,12678,14792,12678,10566,12710,12711,10598,8453,10566,8485,8421,10598,8517,6372,8485,10598,6405,8453,10598,8485,8453,10598,10566,8453,12645,12646,8420,25421,41965,37641,37641,37641,37673,37641,37641,37641,37641,39819,56854,33742,38000,31531,16840,25226,29484,39950,35690,39915,39883,37835,39883,37835,39883,39883,39883,39883,39883,39915,39915,37802,39882,39915,39915,39915,39947,41995,41996,48598,50678,57148,61342,61308,46517,46583,54969,48433,37803,35755,37802,35754,35722,35722,35690,35690,35722,35722,37770,35722,35722,35722,35689,35722,35722,35722,33642,33609,31529,31496,29416,37804,24966,25161,48532,48465,48432,46319,50480,52593,50513,52593,50512,50447,46287,48400,50513,46353,57115,199},
{52990,53022,53022,53022,53022,53022,50974,53054,55102,57182,57183,55134,55103,55103,53022,53022,50942,53022,55103,53055,42654,36446,36414,38493,42622,44702,46782,48862,48894,53022,55102,57183,57183,57214,59294,59295,59295,61342,61343,61342,57182,59230,59262,61342,63422,63423,63423,63422,63422,63422,61342,61310,59261,57181,59197,55100,55068,55036,52988,55069,55101,55101,57150,57181,59262,57150,55101,57181,59262,59262,61342,61374,59262,61310,59262,59262,63422,61374,61374,57214,55069,55069,55037,55069,57181,59229,59261,55069,55036,55036,55036,61309,63422,63454,63422,61342,61277,40278,23374,33809,46518,21164,19083,16970,25388,14824,14791,12711,14823,16872,14856,14824,16904,14824,14824,14791,12743,14791,14792,14824,12743,14791,14791,12711,14791,12679,10598,10598,12679,10598,8453,10566,8485,8421,10598,8485,6372,8485,10598,6404,8485,10566,8485,6405,10565,10533,8420,10565,10598,6339,14824,37771,37641,35593,37641,37641,37641,37641,37641,37641,37674,54676,31694,35952,31628,18985,27371,29452,37902,35690,39915,39915,39883,37835,37835,39882,39882,39883,37835,37835,39915,39915,37802,37802,39915,39947,39947,39947,41995,42028,52791,50678,57116,61342,59195,46517,52889,54970,52692,37835,37803,37803,37803,35754,35754,35690,35722,37770,37770,35722,35722,35690,35722,35690,35722,35722,35722,35690,33641,31529,31496,31496,35820,27046,25194,48531,46319,48433,46320,50512,52593,52593,52625,50480,50512,46222,48400,52593,48499,59195,166},
{53022,53022,53022,53023,53022,55103,55103,55103,55134,57183,55135,55135,55103,55103,53022,50942,50942,53022,55135,53023,42654,36446,36414,38494,40574,44702,46782,48862,48862,53022,55102,57182,57183,57215,59262,59263,59295,61342,61375,61375,59262,57150,59230,61310,61342,63423,63422,63422,63422,61374,61342,61342,59261,59229,57149,57116,55068,55036,55068,55069,55102,55069,55069,55069,55069,55069,55069,57149,59262,59262,61342,61342,61342,61374,61374,61374,63422,61374,61374,59262,55069,57117,55069,55036,55036,55069,55069,52956,52956,55036,55036,61309,63422,63422,63422,61341,61309,40278,23342,33809,44406,21163,19051,16970,25388,14824,14791,12743,14823,16904,14824,14824,16872,14791,14792,14791,14792,14791,14791,14791,12711,14791,12743,12678,12711,10631,10598,12679,10630,10598,8453,12679,8485,6405,10598,8517,6372,8485,10598,6372,8453,10566,8485,8453,10565,8485,6404,10533,10598,6340,12646,31629,31564,35691,35692,35724,37773,37772,37773,37772,37805,52627,35887,35887,33709,21228,29582,35888,42160,35722,39915,37835,39851,37803,37835,39882,39850,39882,39850,37803,39915,39915,39818,37802,39915,39948,39947,39947,41995,42027,50677,48500,52890,61342,54969,44437,50744,55003,54837,37835,37835,37803,37803,35754,35755,35690,35723,35754,37802,35722,35722,35690,35722,35722,35722,35722,35722,35690,35690,33609,40047,46419,48565,44210,46387,40013,35789,44207,44239,50480,52593,50480,52626,50480,50480,46254,46287,50480,50711,56951,142},
{53022,53022,53022,53023,53022,55103,55135,55135,57183,57183,55135,55135,55135,55135,50974,48894,48894,53022,55103,53055,44702,36446,36414,38494,40574,44702,46782,48862,48862,50942,53054,55134,57182,57214,59262,59263,59294,61342,61374,63423,61375,57182,57182,59262,61342,63390,63422,63422,61342,61342,61342,61342,59262,59229,57148,57116,55036,55036,57117,57182,57182,55102,55069,55069,53021,52989,55069,57182,59262,61342,61342,61342,61374,61374,61374,61374,63422,63422,61374,61374,55069,55069,57149,52989,52988,52988,52956,52956,52988,52956,55036,61342,63422,63422,63422,61341,61309,42358,23342,31728,42325,21132,19051,19050,27404,14824,14791,12744,14792,14824,14824,14824,14791,12743,14792,14791,14791,14791,14824,14791,12743,14792,14759,10630,12711,10631,10598,10598,10598,8518,8453,10598,6437,6372,10598,8518,6405,8518,8518,6372,6405,10565,8485,8452,12678,8518,8453,10533,10566,6372,6372,10565,6372,6372,10565,8452,6371,10501,8453,6340,8484,48499,35888,35887,37837,23275,31694,38033,42192,37771,39915,37867,37834,37803,37802,37835,37802,37802,35722,35722,35722,35722,35657,35689,37802,37802,37802,37802,39882,39915,46419,46419,52857,61341,54904,35920,33840,59261,56983,37835,37834,37834,37835,35755,35755,35722,35722,35754,35754,35722,37802,35690,35722,35722,35722,35722,35722,35689,35690,35789,57148,57049,52824,50645,50677,44306,44338,48432,46352,50512,50512,50480,52626,50447,50448,48335,42061,48498,57115,50513,128},
{55102,55103,55103,53054,53055,55103,55134,57183,57183,57183,55135,55135,55134,55135,50974,48894,50942,53022,55103,50975,42654,36414,36414,38494,40574,44702,46782,48862,48862,50942,55102,55134,57182,57214,59262,59295,61342,61342,61374,63423,63422,61375,57150,59230,61342,61342,63423,63422,61374,61342,61342,61310,59261,59229,57149,55068,55068,55036,55036,55069,57182,55069,55069,55037,55069,53021,55069,57182,59262,61342,61342,61342,61342,61374,61374,61374,61374,63422,61374,61374,59262,59229,59229,55101,55069,52989,52956,52956,52956,55036,55068,61342,63422,63422,63390,61309,59229,40246,23309,33809,42325,19084,19051,19018,25356,16904,14792,12744,14824,14824,14824,14824,14823,12743,14792,14791,14791,14792,14792,12743,12743,14791,14791,12678,14759,10630,10598,10598,10598,8517,8453,10598,8485,6404,10597,8518,6405,8518,10566,6372,6405,8517,8485,8452,12678,8517,6372,8485,8485,6372,6372,8485,6372,6340,8485,6404,6339,8453,8453,6340,8452,42094,37935,40080,35755,16838,29548,29449,35789,33643,27369,35788,25223,35756,25191,29482,21030,25256,23111,21030,25256,18918,23143,21063,21030,25256,21031,25223,25256,23110,25355,33775,48696,61342,52791,52921,55002,57051,54904,37868,37868,37835,37835,37835,35723,33642,33674,35722,35754,35690,35722,35690,35722,35722,35722,35722,35690,33641,35690,42226,55002,57049,57017,50677,50677,50709,46451,50578,48401,50513,50480,52594,50578,50448,50481,50448,48400,50645,59163,42093,121},
{57182,55134,55102,55102,55103,55103,55135,55135,55135,57183,55135,55135,55103,55103,50974,48862,50942,53022,55103,53023,42654,36414,36414,38494,40574,44702,46782,46814,48894,50974,53054,55134,55134,57214,59263,59295,61342,61374,61374,63423,63423,53020,42424,42424,44471,44504,44504,44503,44536,44536,46583,46616,46584,44503,44536,44503,44536,44536,44504,44569,44536,44536,44536,44536,44536,44569,44568,46681,48729,50841,50842,50810,50842,52890,52890,50842,50809,50810,50810,52890,52890,52890,50809,48761,48729,46649,44569,46617,46617,46649,48762,54970,55003,55003,55003,55003,54938,36085,23406,33809,42292,21196,21196,21196,27501,14824,14824,12744,14856,14824,12743,14824,14824,12743,14792,14824,12744,14824,14824,12743,10695,14792,12711,10631,12711,12679,8550,10598,10631,8517,8485,10630,8517,8453,10566,10566,6405,8485,10598,6405,6437,8518,8485,6404,10565,8517,6372,8485,8485,6372,8453,8485,6372,6372,8485,6372,6340,8485,8453,6372,8517,42126,39916,42226,27434,2178,19082,6306,14791,23144,6371,25290,4226,21162,4194,19082,6339,19049,10695,12645,21162,6339,18983,12776,8451,19049,8485,18983,14921,12612,14856,23244,38164,59228,46484,48663,59196,61276,59163,42161,40113,38032,40080,44273,40079,37935,40080,40079,37966,37966,40079,37966,37966,40047,40047,37934,37966,40046,37966,44339,50744,59130,54902,50577,56982,50708,46385,46352,48400,50513,52593,52593,50513,50480,50480,50480,50545,54904,56950,35820,114},
{57183,55134,55134,55103,55103,55103,55134,55135,55135,55135,55135,55103,55135,55102,50974,48894,50974,53023,55103,53023,42654,36414,36414,38494,40574,44702,46782,46814,48862,53022,55102,55102,57150,57215,59295,61343,61342,61375,61374,63423,61375,38393,29844,27829,29844,29876,27796,31892,27764,31924,29876,31956,31859,34036,31988,34004,34036,34004,34068,34036,33971,36052,33971,31923,31890,31923,33971,34004,31890,36084,36019,31890,31923,31858,31890,31858,29777,31891,31858,31826,29810,31826,29777,27697,27697,27665,27633,27665,29745,29713,27697,25584,29713,25552,25552,29745,27632,29745,29745,31890,31825,31825,29777,29778,31858,31825,29745,27730,27697,29745,27665,27697,25617,27633,29713,27665,27664,25617,25584,27664,25552,25520,25552,23472,21293,21326,14987,25422,17067,17067,23341,17002,14954,17034,8615,17067,10760,12808,19180,10728,14889,19115,12841,19147,14888,12776,19147,12775,16969,23340,21162,21163,19082,19050,21130,12646,16937,19082,19017,19082,19049,25289,29516,25290,19049,19049,17001,19049,21227,23242,21195,23275,14888,21195,8485,17034,14856,16936,17034,12743,16937,14888,12678,14953,10565,14856,19115,25388,27534,23243,16871,19082,19212,55035,42324,52922,59228,61341,57017,40048,31660,4225,4291,35919,14724,4258,23340,27369,4225,10630,35821,8418,8483,25452,25256,6306,8549,35886,12676,38065,46550,59162,56983,44273,46450,37866,19015,50513,50513,50546,52626,52593,48400,52593,50512,50480,50579,59163,50513,33675,122},
{57215,57215,57183,55103,55134,55103,55102,55135,55102,55102,55103,55135,55135,53055,50974,50942,50974,53023,55103,53023,42654,36413,36413,38494,40574,42654,46750,46782,48862,50942,53022,53055,57150,59262,59294,61342,61342,61343,61374,63423,61374,36280,27795,27797,29812,27764,25684,29812,25683,31891,29843,29843,31891,34069,34004,34004,34004,33971,34003,36019,36116,34004,31922,29842,31891,29842,31923,33939,31923,33939,31858,31923,29810,29778,29810,29778,31826,29810,29809,29810,27697,29746,27697,25616,27633,25585,27665,27633,27665,29713,25617,25552,27665,23472,25520,25520,25584,29745,29745,29810,29745,29777,29777,29777,31858,31858,29810,27762,27729,29810,27697,29746,27730,25617,27697,25650,27697,27697,27665,27665,25617,23537,27664,23504,21293,21391,15052,23406,17100,17100,23406,14986,12874,19115,4487,12906,14922,8647,21260,10727,10760,21260,12808,14921,19114,10727,19180,12839,14888,27502,25355,21195,25453,17002,21195,16871,16936,25356,19082,17066,25388,12743,16969,16968,6469,19115,12743,16969,25388,21129,23308,25486,23308,23340,12678,6437,17001,8452,12743,21228,31759,29647,23340,8485,14857,8517,21227,29679,38130,33872,18951,10630,17002,48696,38065,38131,59261,61341,52758,38000,25256,4226,10695,35887,8354,6371,27598,21030,4258,19179,27369,6306,8516,31693,20966,6371,17002,33707,8386,25420,40244,59195,56951,48531,48531,44272,27402,50513,50514,50546,52593,52593,50545,52625,50513,52625,52791,59162,42029,31595,121},
{57215,57215,57183,55135,55102,53055,55103,53054,53022,53022,53054,55103,55103,55103,53022,50942,52990,53023,55103,53023,40606,36414,36414,38494,40574,44702,46750,46814,48862,46781,42457,44569,44569,46585,46617,46617,48697,46649,48664,48697,46616,29909,25683,25651,25650,23603,25650,29810,27763,29843,27795,29843,31859,31891,33939,33939,34003,31891,33970,36083,34003,31891,34004,31923,29810,31923,31891,31890,34036,31891,33971,29843,31826,31891,29810,29778,33971,29810,31826,29810,27697,29810,29810,29777,31858,29778,31825,33971,29810,27665,23537,27632,27633,23504,25584,23472,27665,29745,29810,31858,29778,31858,29809,29778,29810,31826,29810,27729,29745,27697,25617,29745,25649,27665,25617,25617,29745,27665,25617,27665,23537,21391,25519,21392,17133,21358,12940,21293,19213,14987,23406,14986,12873,19148,4487,10760,19115,6599,17099,14953,10727,21260,10759,12808,21259,10727,14953,19146,12807,21260,21195,17001,23373,12743,12808,21195,14856,19082,19114,16969,23308,16936,10695,23308,10662,12808,21195,14888,21195,19049,10727,19212,23308,17001,23341,8517,8583,12711,4291,12808,19114,27598,27501,8452,8518,16904,8452,12808,23373,27534,16936,12612,10598,31792,36018,50809,59229,61341,48532,42193,18950,14856,31759,37967,25355,27533,40145,29515,25355,35952,35789,25388,29613,40080,27435,25388,35920,35789,25355,27468,31792,59195,57016,46419,48564,46450,40014,50513,50480,52626,52593,52561,50545,52625,50481,52593,52856,56918,37868,29547,123},
{59263,57215,57183,55135,55103,53055,53022,53022,53023,53022,53055,55103,55135,55103,53022,50942,52990,53022,55103,50975,42654,36414,36414,36413,17395,15085,21294,34037,31990,29844,29811,29843,29778,31859,27731,27763,27763,27698,31923,31957,29844,27795,25780,29811,27763,25748,29811,27795,27796,29811,27795,29843,31859,34004,33971,31858,34004,31890,33971,31923,31890,34003,33971,33939,31891,36051,31891,38164,34004,34004,34036,31891,33971,31923,31891,31891,33971,29810,29810,29810,31858,29811,29809,31825,29810,31890,31858,33971,31858,31858,27665,29745,27665,25584,29777,23504,29745,27633,29745,31826,31826,31858,31858,29810,29778,31858,31858,29778,31826,29778,29778,29778,27730,29745,29745,27698,31826,29746,27697,27697,25617,23439,25551,23472,19213,21358,15020,21326,21326,17132,23438,19115,14986,23406,12873,10793,21261,10793,17066,19147,12840,19212,17034,14921,23373,14920,14888,23340,17001,19114,25420,17001,21260,23308,14888,23341,14856,14921,23373,19049,17002,21228,14856,19115,21195,14791,19115,16904,14856,21195,10630,14889,25486,27631,21293,23276,12776,17002,10598,8517,17034,14856,14823,16937,8453,12743,14791,8517,16937,14823,10565,16904,8484,12776,27632,57083,57115,61308,42193,38000,8419,4226,25453,25256,4226,14953,35789,6306,8484,35887,18853,8451,23308,31594,6273,8452,31726,20966,6371,21129,38065,59195,57049,54904,54903,52789,46320,50480,50481,52593,50480,50513,52625,52625,50512,52659,59164,50480,33707,31628,131},
{59263,57215,57183,55135,55102,53022,53023,53022,50974,53022,53023,55103,55102,55103,50974,50942,50942,53023,55103,53055,42686,36414,36414,36446,8910,10728,21130,29811,29811,31891,29811,29778,29876,29844,27763,27828,25617,27697,31923,31989,29876,29844,25748,31891,29876,27796,29811,29876,29875,29843,27828,31923,31891,34037,33971,33971,34068,33971,36052,31955,33938,36084,33971,33938,31923,36084,31922,36116,33971,36051,31890,31858,33971,31891,31891,31924,31858,31858,31891,31826,31858,29810,29777,29778,29778,31858,29810,29810,31858,31859,27697,31826,25585,23504,27697,23472,29745,27633,31890,31858,31826,31858,33906,31858,31858,31858,29842,29778,31890,29778,29809,29778,27730,29778,29778,27698,31858,27698,27697,29778,25585,21358,27632,23471,17165,23407,12939,17165,21294,17100,23438,17067,12873,23406,12809,10728,23309,8648,12841,21325,12840,15018,19115,10759,23341,12839,8647,23373,12840,14888,25453,19049,19082,27501,14888,21195,21195,14921,23373,19114,16969,25421,14823,14888,25453,14823,10662,21163,8452,12808,16936,10630,23341,27566,19114,23374,8614,8582,21195,4291,6502,19082,4292,14922,21195,14824,25422,25355,12775,17034,6339,10630,16937,6404,19148,55003,57115,59130,40080,35821,4225,4291,35952,16772,6339,23372,27369,6306,14856,37934,8386,8484,27565,25191,4259,10662,37966,10434,8452,29777,42128,59163,57017,54936,54904,56983,46320,48367,48401,50480,52593,46255,52593,48431,52593,52790,61242,42094,31627,31660,131},
{59263,57215,57183,55103,55102,53022,53022,53022,50974,53022,53022,53023,55103,55103,50974,50942,50974,53055,55103,53055,44734,36414,36414,34333,8780,10728,23275,27763,29811,31956,29811,31826,31956,29844,29843,29877,29843,29811,31924,29942,29876,29843,27796,31891,29876,27763,29843,29876,29843,29843,29876,31923,31923,34068,31923,31923,31955,33971,36084,34003,33971,34036,34003,31891,31923,34003,31923,36149,34003,34003,33971,33971,31923,31891,33938,31923,29778,31890,29842,29810,31890,29778,31825,27729,29810,31858,29810,29810,31858,31891,29778,29810,23537,25552,27698,23504,29745,25585,31858,33906,31826,29777,31890,31858,29777,31890,29810,29778,31858,31890,31826,29810,27730,29778,27698,25617,31858,27730,27730,29746,25585,19278,25584,21456,17197,23406,12939,19213,21326,17132,25487,17034,12809,23406,10760,10695,23373,8647,12840,21293,10727,17066,19114,12807,21260,12840,8679,21325,10759,14921,27566,21163,17001,25453,19082,19082,21195,17001,23373,21194,19049,25454,16969,14856,25422,14791,8517,23275,8452,10663,19049,8452,17003,14856,12809,23373,10662,8550,19147,4227,4357,19082,4291,14922,23340,14888,25486,25388,10662,17002,4258,4389,16936,2210,10793,59294,63454,56951,38032,33709,23275,29646,44338,29580,29582,40145,35854,29549,35952,40047,27468,29581,40145,33708,27467,33871,40014,27403,27500,44437,40015,55002,57049,54936,54936,57016,46320,46286,48433,50513,52593,46287,52561,52593,52626,54937,56918,37869,29579,31660,132},
{59295,57215,57183,55103,53055,53022,53023,53022,53022,53022,53022,53023,53055,53055,50942,50942,50974,55103,55103,53055,42654,36413,42589,30073,8746,14888,27534,25520,31891,31989,31892,31891,31957,29876,31924,27861,29876,29811,31924,29941,29843,31892,29844,29843,29876,27763,29843,31924,27795,29843,29876,31891,31891,34036,31923,31891,33972,31923,34036,34003,33971,34004,34003,31891,31923,34003,31922,36117,36051,34003,31923,33971,34036,33971,33971,31955,31923,31923,29843,31858,31890,27729,31858,27697,29810,33938,33971,31890,33906,31891,31858,31858,25585,25552,27665,25552,29745,25552,31858,31890,29778,27697,31858,29778,29745,29778,29778,29745,31858,31858,29778,29778,27698,29778,27698,25649,29842,27698,27697,29777,25584,21326,25584,21423,19213,21326,12907,21228,21294,17100,23438,17067,12808,21325,10760,8648,23373,8680,12840,23340,10695,17034,21195,12840,21228,14888,10727,23373,12807,14953,27534,21162,19082,27534,17002,19082,23308,19082,23341,21195,16969,27567,16969,14921,25454,16968,10597,21227,8420,10598,21130,6404,14922,14889,8680,23341,12743,10630,21195,6339,4356,19114,6404,19148,29647,17034,33970,29646,10630,17034,8516,10598,19050,8517,10728,57181,63422,52693,38000,29548,27468,35920,40080,27500,33807,44305,31660,31661,40145,37934,31661,35952,42192,31660,31693,40145,37998,31693,35919,44306,42193,52889,59130,50612,44175,46287,42062,46319,50546,48466,50545,48367,50513,50512,52626,59196,50480,33708,33741,35821,139},
{57215,57215,57183,55135,53023,53022,53022,53054,53022,53022,53022,53022,53022,53054,50974,50942,50974,55103,55135,51007,42654,36413,40508,25815,8681,17034,29712,27568,31924,31958,29909,31924,29876,29876,29876,31924,29843,29843,31957,29909,31924,29876,27796,29843,27829,29811,29843,31924,29843,29843,29908,31891,31923,34004,33971,33971,34004,31923,34036,33971,33971,33972,34003,31890,31923,36084,31923,34036,34003,34003,33971,33971,36116,36084,33971,31923,31890,33971,31923,33970,31890,27730,31858,27730,31890,33971,31923,31922,31890,31891,29778,31858,25585,25584,27665,25552,29745,23504,29810,31858,27697,27697,33906,29778,29713,31858,29810,29745,31858,29778,29778,29778,27697,29745,27697,25617,31858,27698,27697,29778,23537,23439,25584,21391,17165,19278,12939,21261,21326,15019,23438,17035,10793,23373,10728,8648,21325,10728,10792,23341,8647,14954,19179,12808,21228,16969,8679,21325,12807,17002,27599,23307,19082,27566,16970,19082,25421,19082,25389,23275,19049,25453,19049,16969,27566,21161,12645,21195,6372,8517,21162,6404,17002,19147,10664,21294,17002,14824,21228,8420,6404,19115,8550,21261,29680,23373,33938,31727,10630,14922,6403,8517,19017,10499,12678,46616,57050,42128,37967,14627,6339,25388,27337,6339,19114,35789,8386,8517,35920,18885,10532,25453,29482,4193,8517,35919,16772,6371,25388,29547,42225,50743,61211,52757,46321,46224,46288,50513,48432,48400,52593,50480,50513,50480,54838,61210,42093,31595,31627,33741,137},
{59263,57215,57183,55135,53022,50974,50942,53022,53022,50942,50974,53023,53022,53054,53023,50974,53054,55103,55135,51007,42654,36414,36413,19574,10728,19082,29778,27665,29876,31990,31924,29844,29843,29877,27829,31924,29844,31891,31957,29909,29843,27860,27796,31891,29877,29843,29811,31924,29843,31891,29875,31891,31891,33971,33939,34004,31956,31923,34036,33971,33971,34036,36084,31891,33971,36084,34003,36148,36083,36116,34003,33971,36116,36084,33971,34004,33971,33971,31923,33939,31891,27730,31858,29778,31890,33971,31923,33971,31890,34003,31858,31858,25585,25584,27665,25552,29777,23504,29810,31890,27697,27697,31858,29811,29746,31890,29810,29777,31858,29778,29777,27730,25649,29778,27697,25617,31858,27698,27697,29778,25617,23472,27664,21391,19213,21359,15052,19245,23374,17035,23438,17067,10793,23374,10696,10728,23374,10727,12840,23373,8647,14953,21227,12840,19179,17001,10727,23405,12807,19082,27567,19082,17002,29615,14856,17034,25421,19049,23340,25453,23275,25421,21162,17002,27567,21162,12645,23276,8451,8485,23243,8452,14922,27599,21260,23439,23373,17001,21260,10500,8485,21195,12808,25519,31825,29744,25453,27566,10630,12841,10532,19180,25094,22917,20868,22949,22981,22916,22916,22884,22981,25159,25127,25159,25127,25159,23079,25160,25192,25159,25160,25192,23111,23080,20999,23111,18918,18919,21031,23080,38000,48598,61243,56984,54937,57050,52725,46352,50447,48368,52593,50512,46352,50513,54937,56918,35821,31628,33741,35821,152},
{57215,57183,57183,55103,53022,50974,50942,50942,50942,50942,52990,55071,53055,55103,53055,53022,55103,55102,55135,51007,42654,36413,36381,15283,8713,19114,23504,31826,29876,29909,31924,31892,31891,29877,29909,29843,29811,29844,29908,27796,29843,29876,27763,31891,27828,29811,31891,29876,29810,31891,29875,31891,29810,31891,31891,31955,31923,33971,34036,31891,31923,34004,34036,31923,36084,36084,36084,36149,38132,40309,36149,36084,38197,36116,34036,36116,33971,34003,31923,33939,31890,27730,31858,29810,31890,31890,34003,34003,31891,31955,29810,31858,25584,27665,27633,25552,31825,25552,31858,31890,29810,29777,31858,31923,29810,29810,29810,29777,31826,29778,29810,27730,25649,29777,27698,25649,29810,27697,25649,29810,27697,25552,29745,23472,19278,21391,15085,19245,23374,17100,23438,14987,10793,23406,10728,8680,23373,8680,10760,23341,8679,12873,21260,12839,19147,17033,12839,25453,19114,21227,27566,23275,19082,29647,17001,19114,25421,19049,25420,25421,23307,25389,21195,16969,29614,23274,12645,23276,8452,8485,21195,8452,12808,29679,27566,25486,27566,19049,23276,12645,10566,21195,17002,29680,31825,36084,23307,25453,10662,10728,12678,31923,35725,27174,22948,24997,24996,22948,22916,22916,22916,23047,27208,25257,29385,25160,27305,27240,27338,29353,25192,27305,25160,23145,25192,23111,23209,23112,21032,23112,38033,46485,59195,57049,54904,57018,54871,46353,44173,48400,50513,50447,46320,52692,59196,50513,35788,31660,33773,33805,151},
{57215,57183,57183,55103,53022,50942,50910,50910,50942,48894,50942,55102,55103,55103,53055,53023,53055,55102,55135,50975,42654,36445,36414,8976,10761,23308,23472,31858,29844,29877,29843,29811,31892,29877,29844,29843,27731,31891,29908,29843,31891,29876,29843,29843,29844,29811,29844,31892,29811,29843,29810,29842,29810,29843,31891,31923,31891,33971,31956,31922,33971,31923,34003,33971,36116,36084,38196,38197,36084,38197,36084,38132,36117,36084,36084,38197,33938,36083,34003,33939,31891,27730,31858,29810,31890,33971,33971,33971,33971,34003,29810,31890,27665,27665,25584,25552,31858,23504,31890,33938,31890,31890,33939,33971,31858,31858,27698,29778,29778,27697,29778,27730,25650,31826,27698,25649,29810,27697,27697,31858,25649,25584,29778,23537,19278,27600,17101,21326,23406,14987,25486,17035,10793,23406,10760,8680,23405,8712,10695,23373,6567,12841,21260,12839,19179,19114,12839,21292,21227,19114,29615,21260,14921,29615,21195,23275,27534,19082,25388,27501,21194,19179,23308,19082,27534,23243,12677,23308,10532,8517,21228,8452,10695,25454,25453,25454,29679,16936,19082,12678,8517,21293,19115,33905,36019,42423,27501,23405,12710,10695,14921,44601,46452,27174,22949,22948,24997,22948,22916,24964,22916,22982,22949,8321,16740,20836,8387,25029,12482,16708,22884,6274,24996,10401,16708,20803,6306,20836,10369,16708,35855,44306,59196,57017,54904,57017,57017,44207,40013,48400,48368,50480,46287,52726,59162,44142,31627,31628,31692,35854,147},
{59263,57215,57183,55135,50974,48894,48862,48862,50942,50942,50974,53023,55103,55135,53055,53022,53022,55103,55135,48894,42654,36413,34333,6765,12809,27501,25585,31891,31957,29877,29875,29811,31923,31957,29876,31892,27763,31891,31989,31891,29843,29843,29843,31891,29876,31891,31924,31892,31890,31891,29811,29810,31858,31890,31858,29843,31891,31923,31956,31891,31923,31923,36019,33971,36116,34003,36148,38196,36084,38229,38132,38164,38197,36084,36084,36116,33970,36051,33971,33971,31891,29778,31858,29778,31858,31890,31955,33970,33971,33971,31858,31858,25585,29745,27600,25584,31858,25584,31857,33971,33971,31890,31891,31890,31858,29810,27698,29745,29810,27698,29778,27730,27697,29810,27697,27697,31859,27730,25650,31858,25649,25585,29778,21424,21326,25552,15052,21294,21326,17100,23406,14987,10761,25421,6600,8712,23373,10760,8647,23373,8614,10792,21260,12840,17099,19146,12839,21293,19146,17066,27567,23308,14888,27567,21227,21195,27534,19114,23340,27534,21130,19147,25388,21162,25486,23307,16838,23308,10532,10533,23276,8452,8550,27567,27566,25486,29679,16904,16969,14791,8517,21261,21260,31793,38131,44503,27500,27599,14791,10663,19180,55068,54937,31399,24997,22916,24997,22948,22948,22948,22916,22950,22981,10402,14660,22884,4193,25029,14529,12579,22916,4161,22948,12450,12547,22883,4161,20836,12449,12548,35888,42128,59196,57017,54904,57017,57049,46353,42028,44174,48400,50448,48400,54937,56918,37901,31595,33708,33741,37966,138},
{59263,57215,57215,55135,50974,48894,48862,48862,50942,50942,53022,53023,55103,55135,53055,53022,53055,55103,55103,48894,42654,36446,30205,6699,12809,29679,25585,31892,31989,31957,31924,29844,31924,29941,29844,29876,29811,31892,31957,31891,31891,31957,29843,31891,29876,31890,31956,29843,29810,34004,29842,31826,29811,31859,31891,31923,33939,31923,31923,31891,31923,31923,33971,31923,34068,34003,38229,38197,36116,38261,36084,36084,36117,36084,36084,36116,36051,34003,31923,33971,31891,29778,31858,29778,31858,31890,31891,31890,31890,31858,31826,29778,23504,27665,25552,25552,31858,27697,31890,33906,33938,33939,33971,33971,29810,29810,27730,27729,31858,27730,29778,27730,25649,29810,27730,27697,31859,27730,27697,31858,25649,25617,31826,21424,19278,23472,15020,21294,21326,19180,23406,12906,8680,21325,8680,8680,23373,10760,8648,23341,6534,8712,21292,10759,17066,19146,12807,23373,19146,17001,27599,19147,14889,27566,21227,21195,27534,21195,23308,27534,21195,17066,25420,19049,25486,25420,14758,21228,10532,8485,23276,10532,10598,27534,27566,25421,31728,14791,12776,16969,8484,19115,19050,23438,36051,44503,27533,25518,16903,10663,27665,59228,63356,35625,24997,24996,24996,24996,22916,22916,22916,22949,22981,14595,14693,22917,4161,22949,16610,10467,22916,4161,20869,14562,8387,22916,4160,18788,14562,8419,44438,42128,55002,57049,54937,57050,57082,48433,44142,48433,39916,46320,44207,59196,50448,33708,33709,35821,35886,37999,137},
{59263,57215,57215,55135,53022,48894,48830,48862,48862,50942,53022,53054,55103,55135,55103,53055,55103,55103,55103,50943,42654,36446,25914,8713,14921,31857,27567,33972,31990,29909,31923,29844,31956,29941,31892,29876,29812,31924,31956,31923,31891,31924,31891,34004,29843,31859,31956,29875,29842,31956,31859,31858,29811,29811,31858,31891,31891,31923,29875,33971,31955,34003,34003,36116,36148,36051,40277,38196,38196,38261,36051,36116,38197,36084,36084,34036,33971,34036,33971,33971,31923,29778,31858,29778,31858,31890,31891,31858,31858,27730,29778,29778,23505,27697,25552,25552,31826,27698,29777,33906,33938,33971,33971,33971,29777,33938,29778,27729,29778,27665,29810,27698,27665,29810,25649,27665,31858,27730,27729,31859,27730,25585,29810,21456,19278,25584,15020,19245,21358,17132,21358,14986,6600,21325,10761,10760,23373,8679,10728,23373,6566,8680,21292,8679,17066,19179,12839,23373,17033,17033,27566,21195,14889,25486,19082,21195,27534,17034,21227,27534,23308,19115,25421,14888,25453,25420,12678,19116,10565,8517,21260,10565,10566,27567,27566,23340,29647,14791,10631,21130,6404,17034,21195,21261,36051,42390,29646,23438,16968,10695,34069,61276,65501,41965,24997,24996,24996,22948,22917,22948,22916,22949,25127,20966,21096,27143,18952,27272,23014,23177,27175,16806,23079,23014,19016,25062,16806,25127,22982,18984,42293,37902,46616,59130,54936,57050,57017,48466,46319,48433,42030,46320,50645,59162,42094,33708,33773,35854,40080,40079,143},
{59263,59263,57215,55135,53054,50910,46782,48862,48862,48862,50942,55103,55103,55135,55135,53055,53055,55103,55135,50942,42654,36381,21687,8713,25387,21359,27567,31956,29941,31925,29876,29876,31957,29941,31859,29844,29843,31956,31956,31923,31924,29908,31891,34036,31923,31891,34036,31923,31891,31956,31890,31859,29843,31891,29810,31891,31923,31923,31923,33971,31956,34068,34003,36052,36149,36052,40309,38196,38164,38229,36083,36116,36149,36084,36084,34036,34003,34036,31923,33971,31923,29778,31858,29810,29745,29810,29810,31826,31858,29810,31826,27698,23472,27665,23472,25552,29777,25584,29810,31858,31890,31890,33939,33971,29777,31858,27730,29778,29810,27697,29810,27697,27665,31826,27697,25617,31858,27730,25682,31891,27698,23536,29810,21489,21359,23504,12972,19278,21359,10858,21358,17099,8680,21325,8712,8712,23373,8680,10728,23373,8647,10760,23340,10759,17034,21259,10791,19211,17033,17033,25486,23308,17002,27566,19114,21195,27534,19114,23307,27566,23308,21195,27501,19082,27534,29581,10629,17035,14791,10630,21261,10597,10598,25454,23341,23308,29647,14823,10598,21162,6372,16970,21162,25486,36084,42390,33872,25454,21162,10695,40343,61308,65501,46158,25029,24997,25029,24996,22917,22916,22916,22916,23046,18723,8419,22917,10402,18853,20836,8354,22916,10402,12547,18723,8386,22916,12482,16708,20836,6338,35921,35757,40310,59163,54936,57049,57017,50579,50513,50513,42094,50546,54969,56885,37901,31660,33773,35821,37999,35821,132},
{59263,59263,57214,55103,53054,50910,46782,48830,48862,48862,50942,55070,55103,55135,55135,53055,53022,55103,55135,48926,42654,38429,17396,10761,23275,19245,29712,31956,29974,31924,29876,31924,31956,27763,31858,29844,31891,29908,31956,31923,31956,31956,31891,34036,31955,31891,31988,29875,33939,31924,31858,29810,27763,31891,29811,29811,33939,34004,31956,33971,34036,34068,33971,36084,38164,36051,40277,36116,38196,38229,36051,38164,36149,36083,36149,34004,33971,36084,34003,34003,31923,29746,31858,29810,29777,31858,29810,29745,31858,29778,29810,27697,23504,29745,23440,25552,31826,27697,29777,33906,33971,31890,33939,33971,29810,33938,25682,29745,27697,25617,29810,27697,25617,29778,25649,25617,29810,27762,25650,31890,27697,25552,29778,23537,19311,25585,15020,21294,23374,15019,23406,17099,10760,21326,12841,10760,23373,10792,10760,23405,10728,10728,23373,10792,17034,21259,10791,19179,19146,14985,25486,23275,14921,27599,21194,19082,29615,19114,23340,29647,23275,21195,27566,29613,27534,29581,12710,19180,16969,12775,23341,14758,14823,25487,23340,23308,27566,14790,8485,23276,10597,14824,27501,29745,38132,40309,38098,25486,21194,10760,50875,61308,63356,46256,25029,25028,24996,24997,22916,22916,22916,20869,23014,20804,8387,22917,10369,16708,22916,4161,20868,12449,10467,20803,4161,20836,14498,10467,22883,4161,29582,33611,38099,59195,54969,54969,57017,50611,48466,48466,48400,50611,59195,48400,33676,31660,35821,35886,37966,33708,131},
{59295,57215,57183,55103,53054,50910,48862,48830,48862,50910,50942,53023,55103,55103,55135,53055,53022,55071,55103,48894,42589,36348,15217,10761,17066,10794,29680,31957,31990,31924,31924,31924,34037,33972,29713,31891,31891,31956,34004,31923,34036,34004,31923,36149,34004,34003,34037,31923,33971,31923,31891,29843,29811,31923,29811,31891,33939,31956,34004,31923,34068,34036,33971,36117,36148,36084,38261,38164,38196,36148,36083,38164,38196,36084,36149,34003,33971,36084,34003,34003,33971,29778,31858,29810,29777,29778,27698,31826,31891,29810,31858,27697,23504,29745,23472,25584,31826,27665,31826,31890,31890,31890,33971,31890,29745,33939,27698,27729,27730,25617,29810,27697,27665,29810,25649,25617,31858,27697,27697,31858,25617,23504,29778,23537,19246,25520,15020,21294,21294,15019,23438,17099,10760,21325,10760,10760,23406,10760,10792,23373,8615,10728,23373,10727,14953,23340,12871,19179,19146,14920,27534,23308,17066,27567,21162,19114,27567,19114,23340,29647,23307,21227,29647,29646,27533,27534,12775,17099,16936,10695,23373,16936,14823,25519,21228,17067,27567,12710,8485,23308,10597,14856,29646,31890,38131,40277,40211,23373,23242,17067,57180,63389,65502,54740,27077,24996,24965,22917,22948,22917,22916,22916,20966,22884,8387,22949,14530,10499,24964,4129,20836,16610,8386,22916,6209,18788,16610,8354,22916,6208,23243,42127,40178,59195,57049,52823,50677,48629,50710,48630,52594,52823,59129,42061,29547,31660,33740,35918,37901,33741,135},
{59295,59263,57183,55103,50974,48894,48862,48862,48862,50942,50942,53022,53023,55135,55135,53055,53023,53022,53023,48862,40573,36380,10925,10761,21227,19279,31858,31957,29909,31924,31892,31891,34037,29811,31858,29876,31924,31956,31924,31923,34036,34004,33971,36149,34004,34036,34069,31923,31923,31891,31891,29843,29811,31891,29811,31891,31891,31956,31955,31923,34036,34068,34003,36117,36116,36116,38229,36116,38229,36149,36083,38197,36149,36051,38229,34036,34003,34004,33971,33971,31955,29810,29810,29842,29809,29778,27730,31826,29810,27698,29810,27698,23504,29745,21424,27633,29778,25585,29777,29810,31890,31890,33971,31890,31826,33970,27698,29777,27697,25617,31858,27665,27665,31859,27697,25616,31858,27698,27697,31890,25617,25584,31858,23504,21359,23472,15020,21293,21326,15019,23406,15019,8712,21358,10760,8712,23406,10728,10760,23374,10695,10695,23341,10695,14953,23372,12839,17099,21195,12840,25454,25421,19114,27534,19082,16969,27534,23275,23307,29615,23307,17034,27566,29646,31726,31727,8582,10761,16936,8550,19147,16936,12678,25487,21293,19147,25486,14758,8485,23276,10565,10631,29647,27697,38164,40245,40211,21228,23275,25519,59228,65501,65502,59031,27110,24997,20772,20836,24965,22916,22916,22916,23046,25094,18984,27239,20933,21064,27175,16806,25127,22981,16838,25062,14693,23014,22981,16838,27142,16773,23210,50711,46452,59196,56984,46385,52756,54935,50742,48532,50546,57050,56853,35788,31628,33740,33741,37966,35821,35886,145},
{59295,59263,57215,55103,50974,48862,48862,48862,48862,48894,50942,53022,53022,55103,55135,53055,50974,53022,55103,48894,42622,32220,6666,10760,25389,29812,31892,29909,29877,31892,29844,31923,29941,29876,31956,29843,31891,31924,33972,31891,34004,31924,33971,36117,34004,36116,34068,34004,34004,29843,31891,31859,31826,31891,29843,29843,31890,31956,34004,33971,36117,34036,34003,36117,36084,36148,38229,36084,36149,36116,36051,36148,36116,36083,36148,34035,36051,34036,33971,36051,34003,29810,31890,29842,29778,29810,27730,31858,29778,27697,29778,27697,23504,29746,21424,25584,31826,25552,29745,31826,33938,31890,33939,31890,31857,29810,27730,29778,27697,27697,31858,27697,25649,31858,27698,25617,31858,27697,25617,31890,25649,25552,29810,23537,21359,23472,15020,21294,21326,19148,23374,17067,8713,21358,10760,8680,23406,10728,10760,23373,8647,8647,23341,8647,14953,23404,12839,17066,21227,12872,25454,23308,17001,25486,19114,21162,27566,19017,23274,27566,21194,14888,27566,27533,31693,33774,6371,6535,17001,6469,17002,16936,12743,23406,23438,21228,25454,14758,10565,23276,10597,8485,25486,23406,36051,38131,40212,23340,25453,31825,52889,65502,65501,61144,27109,24996,24997,22948,24997,25028,25029,22948,23014,22948,10434,18821,20804,12580,22981,12547,18821,20836,10499,20901,14627,16773,22948,12547,22981,14627,16839,52824,42193,59195,57017,56917,50643,54870,44305,40079,50547,59196,48367,31628,31628,31692,35854,37966,33741,35854,143},
{59295,59263,57215,55103,50974,48862,48862,48862,48862,50910,50942,53022,53023,55102,55134,53055,50974,53022,53055,48862,42589,27994,10761,10761,27534,29844,31924,31990,31957,31924,31924,31924,32022,31924,31924,31891,31891,31956,34004,29843,34004,33971,36084,36149,34004,36116,34036,34004,34004,31891,33971,31859,31858,31891,29843,31891,31923,34036,34003,34003,36149,36084,34036,34069,36084,36116,36148,36084,38197,36084,36051,36148,36084,36083,36116,36084,34004,34068,36051,36052,31923,31858,29842,31890,31826,29810,27698,29745,29778,27665,29778,27697,25584,29746,23472,27633,29778,23504,27697,31858,33939,31890,33971,31890,31890,29842,27698,29778,27730,27665,31858,27697,27697,31858,27698,27665,31858,27730,27698,31890,27698,23503,29745,23504,21327,23472,12972,21326,23406,19147,23374,15019,10761,23374,10760,8648,23373,8680,10728,23406,10727,10728,23373,10759,12904,23372,12839,17066,21259,12840,23373,23340,14953,27567,19049,19082,27534,16936,19081,27567,21194,16968,23373,21194,31661,31726,6371,4422,19082,6469,12809,18984,10598,21326,25486,23308,25454,14790,8485,21228,8485,10598,23341,21293,36051,38132,40276,19114,23341,36051,52889,65501,65501,63290,29189,24996,24996,24996,22948,24997,24997,22916,20966,22949,8289,14660,20803,4193,22949,12449,12547,22916,4161,20869,14530,10467,22916,4161,20901,16610,10500,46583,44240,54970,57017,50579,35918,40111,27335,29612,46485,59096,39981,33676,33773,33773,37966,35821,33741,35853,143},
{59295,59263,57215,55103,53022,48894,48862,48862,48862,50910,50942,53022,50974,55102,55103,53055,50974,53022,53022,48894,42654,23800,10794,12873,31760,29844,33972,31990,34005,31956,31891,31924,32022,31956,31956,31891,29811,31892,31859,29811,31924,33971,36052,34036,33971,36084,34004,33971,31924,31890,33939,29843,29810,31858,29810,31890,31891,31988,34003,33971,36084,34003,36084,34036,36083,38229,38197,36084,38196,36084,36051,36116,36084,36084,36149,36116,36052,36116,34003,34003,31955,31858,31891,31858,31858,29778,29746,31793,27730,27665,31826,27698,27632,31794,25520,27633,29713,23504,27697,31858,33970,31858,31890,29810,31858,31891,27698,29810,27730,27666,31858,27697,27697,29810,27730,25649,31858,27698,25650,31858,27698,25584,27697,19279,19246,23504,15053,23406,23374,17099,23406,15019,8713,21325,10761,10696,23374,8648,10760,23373,10759,8679,23373,10759,12872,23372,10759,14985,21259,12840,21227,21260,19114,25486,19082,16969,25454,14888,17033,27566,19049,19081,29680,23307,29580,31695,6339,4389,21162,6372,10663,19017,6469,21293,27567,23373,23374,16839,8485,21260,8517,8485,21293,19213,38131,38132,38131,17001,25486,46616,54970,65502,65502,63389,31302,24996,22948,24996,24996,24996,24997,22916,20934,22981,10369,12580,20836,4161,20901,14562,8419,24996,4161,18821,16642,8386,22916,6208,18789,18723,6339,42293,44208,48631,54904,46451,46484,44403,42192,31660,46616,54772,35788,33709,35821,33805,40046,33740,33773,35853,145},
{59295,59295,57215,55135,53023,50942,48862,48862,48862,48862,50942,50975,50974,53022,55135,53055,50974,50942,50974,48861,42621,34235,12939,14986,27566,31858,33972,36118,34037,31956,31924,34004,34070,31924,31956,29811,17133,12875,14987,15020,10827,17035,12906,10826,17100,12906,10826,19181,10826,10859,19213,12907,12907,19148,12907,17068,19181,15019,21293,19180,17132,23373,19212,21293,25454,21260,21293,25487,19180,19180,23406,19212,21292,23438,21325,23373,25486,21357,23406,23406,21325,23405,23406,21325,23406,19213,19213,23373,17165,19213,23438,19245,19213,25487,19213,21293,25487,21326,21357,25518,25486,25486,25519,21358,25519,25487,23438,25552,23438,21358,27632,23439,23374,27632,21358,21293,27600,21325,21325,27599,21293,21293,23471,12907,17100,19246,12907,19181,21293,10761,21293,12873,10729,23374,10696,8648,23373,8648,12809,23373,8647,10728,23341,10663,12840,23340,10791,14953,23308,14921,19147,23340,17001,25453,21162,17033,25486,19114,17001,27534,19049,17001,27534,31694,25387,29615,6404,6502,21195,4356,10663,19082,12841,23374,29680,25454,23373,12677,10565,21228,10565,10598,23309,19213,36051,38131,38099,14888,27697,61341,59163,65502,65501,65469,33480,24996,24964,24996,24996,24996,24996,22916,22949,23014,14595,12580,22948,6306,20934,18723,8451,22949,8322,18821,20803,8387,22949,8321,16741,20836,6339,35986,44208,46518,59162,54935,40111,33806,31660,38032,57115,46255,33708,33708,33741,35918,40014,35789,35854,37934,154},
{59295,59263,57215,55103,53054,50942,48862,48862,48862,48862,50942,53023,53022,53022,55103,53055,50974,50974,53022,50974,44702,40541,17264,14986,23340,31890,33971,36084,34036,31890,33972,34036,34101,31924,34037,23602,15020,8811,10826,13036,8779,12875,12972,8811,15020,12940,8779,17101,10827,10859,17132,8811,10827,17067,15019,12939,17100,12939,17067,15052,15019,17100,15052,15019,19212,17067,15019,21261,12939,12906,19212,12906,12906,17132,12939,14986,17132,12939,17035,17100,12939,14986,12906,10826,17035,10794,10793,14987,10793,10826,17099,10826,12874,17067,12874,12874,17068,12874,12874,15019,12841,14954,14954,10793,17035,12874,12841,17067,12841,12873,17067,12841,10696,17034,8615,8549,14921,6437,6502,14921,4356,8517,10727,4356,10663,10696,4357,12776,8582,4356,14921,6437,6405,16970,4324,4356,17002,2275,4356,14921,4324,6437,14921,4291,6469,14856,2243,8647,12743,4324,14856,10598,6404,17002,8517,4356,17034,6404,6437,19114,6404,8582,19115,10662,12808,21195,8582,12776,16970,6470,12841,14856,6437,17002,16937,19114,21260,10565,6372,19083,6339,6405,19082,6404,10630,19049,8517,10695,34036,63421,63356,65469,63388,63421,37738,24996,24964,24964,22916,22916,22916,22916,22949,25095,23014,23144,25062,21031,25159,25062,23145,25094,20999,23111,25095,21064,25095,23015,23143,25094,18983,33808,46354,44405,59163,54969,44208,16903,21194,48598,56983,39981,31660,33740,33773,37998,35854,35853,37967,40079,168},
{61343,57215,57183,55103,53055,50974,48862,48862,48862,48862,50910,53022,53055,53023,55103,53055,50974,50974,53023,50974,46782,42654,23735,14954,19082,31857,33939,36116,36117,33971,33938,33971,34068,34004,34003,27827,15118,8747,10794,17100,8746,12874,15020,8747,14987,12940,6699,17101,8746,8746,19213,8746,8746,15052,8778,12874,17100,12907,17067,15084,14987,17100,17100,14987,21260,15019,12939,23341,12907,12906,21261,12938,12906,19213,12939,14987,19245,15051,14986,19213,12939,17067,15019,10794,17068,12906,10793,19180,10794,10825,19180,10794,10793,17132,10794,12841,17100,12874,12873,17100,12842,14954,14987,10793,17067,12906,12841,19147,12841,10826,17099,10826,8647,17034,10695,6469,14921,6469,4421,14954,4389,6469,12776,4324,10663,12743,4356,12776,6469,4324,14889,6405,4356,14921,4324,6404,14889,4324,4356,14889,2275,6437,14856,2244,6502,10663,4324,10695,10598,4324,12808,6437,4356,14921,6372,4388,14889,4324,4356,14889,4324,8582,17002,8583,10695,16969,8550,10728,12775,6470,12841,10597,4357,14954,14823,17034,21227,6371,4292,14856,2178,4292,12776,2210,4389,10630,4259,8615,40375,65501,65469,50546,42127,57082,41963,24997,22948,22948,22916,22916,22916,22949,22949,23014,18723,6274,22949,10369,14627,20835,4161,20868,12449,10467,22884,4161,20836,14530,8386,22884,4160,25356,48532,44340,59195,57050,57017,52856,50710,50776,54771,35820,33708,33708,33773,37966,33740,35821,35886,37966,161},
{59295,57215,57183,55103,55103,50975,50910,50942,48862,48862,48862,53022,53023,53022,55103,53055,53022,53022,53023,50974,46814,44734,34299,14988,17034,29679,31890,34003,38197,34004,31825,33971,38197,36149,34003,34068,21489,17198,19246,19311,17166,21326,21326,17166,21359,17198,15085,23407,15084,13005,23407,15020,15052,19246,15053,17132,21326,17068,21261,17165,15019,19213,17100,17068,23406,15020,15052,23406,12939,12906,23373,12906,12906,21293,12906,14986,19245,12938,17035,17100,10793,17035,14987,10826,19180,10794,8746,21261,8713,10793,19213,10761,10793,21261,12841,12841,19212,12874,14921,19180,10793,17034,17067,10793,17067,12874,10793,19147,10793,12873,19180,10793,8647,19147,6535,6437,17034,4389,4422,14954,2276,6469,14889,4292,8615,12743,4324,10760,8550,4324,17001,4324,4324,17034,4291,4324,17002,2243,4356,17001,2243,4357,16969,2243,8550,12776,2243,10696,10630,4291,14889,6437,4324,14954,6371,4324,17002,4291,4324,17002,4324,8550,17034,6502,10663,16969,6437,10729,12776,6437,12841,10597,6405,17002,10695,19114,23275,4259,4292,14921,2178,4324,14856,2178,6437,10630,2178,12776,40244,52758,52758,42095,48565,57083,46190,24997,24996,22916,22948,22916,22916,22916,24997,25062,20803,2113,22949,12449,12547,22916,4128,20868,16578,8387,22884,6209,18756,16642,6306,22916,8289,16905,52791,42194,59196,57050,57050,57016,57017,59163,48367,35821,35821,35853,37967,40014,35789,35854,37966,40112,157},
{61343,57215,55135,55103,55103,50974,50942,50942,50942,48862,48862,53022,53023,53022,53055,53023,53022,53055,53055,53022,48829,46749,42621,17199,17035,27501,31858,33971,38229,34036,31858,31890,36116,34068,33970,36051,33971,34036,34004,33971,36149,33971,36083,36117,31923,36084,31987,31890,34003,31891,29810,31891,31891,29745,29875,31891,31858,34036,33971,29842,34068,31923,33971,31956,33939,34004,31923,31891,31955,31923,31858,33971,31923,31858,31955,33971,31890,31956,31890,31890,29810,31858,29810,29778,29745,27697,25617,29745,25617,23504,27665,23472,23439,27633,21327,23439,25552,23471,25552,25552,27664,25552,27632,25584,27664,23471,21391,27632,23471,23439,27632,23439,21359,25584,19278,19246,25552,19246,19245,25519,17165,17132,21358,12939,17100,17100,8681,17068,14955,10663,19180,10728,6567,19212,6502,6535,19180,6534,8647,19147,6502,8582,21163,6469,10695,17034,6469,12841,14888,6502,17035,12775,8582,21228,12743,8550,21228,8517,6502,21228,8582,12776,21260,12807,12776,21227,6437,10728,16937,6437,14922,12743,6502,19180,17034,17034,21260,6372,4357,19115,4291,6437,17066,6470,10761,17002,8485,23373,33839,35920,40113,33774,44273,48664,50417,25029,22948,22916,24996,22916,22916,22948,24997,23014,22916,2081,20869,14562,10467,24964,6208,18788,18690,6306,22916,8289,16708,20771,4194,22916,10369,12646,50743,40146,59196,54937,55002,54937,57050,56951,42160,37967,37967,37967,37999,37967,37967,40047,40079,42193,162},
{61343,57215,57183,55103,53055,53023,50942,50942,50942,48862,48862,52990,53023,53022,53055,53055,53023,53022,53054,50942,48829,48797,46749,23637,17034,21227,31858,33970,38197,36117,33971,33938,34003,31923,29680,31825,33971,36148,34004,31890,34003,31858,31890,31955,31858,29810,29810,29778,31825,29778,27697,29745,27697,29712,27697,27697,29745,29810,27729,29745,31858,29745,29745,27730,29777,29713,27697,29777,27697,27697,29713,27665,27665,27665,27697,27664,27665,27665,27632,27664,25616,25616,27665,27697,29745,27697,27697,29745,29745,27697,29745,27698,29713,29745,27697,27664,29745,27697,25584,27664,27664,27632,27632,27633,27632,25584,25584,27633,25552,25552,27664,27600,25584,27632,27633,25584,27664,25584,27632,25616,23537,25552,27632,23472,23407,21326,12972,19213,17100,12874,21293,14955,10793,21293,10760,10793,21293,14921,14889,21261,12841,12841,23341,14889,14921,21195,12808,17066,19114,12808,19179,19114,17033,23341,21130,16936,23308,16936,16968,23308,14855,12743,21228,16871,8582,21195,4324,6502,16937,8485,12776,14856,10695,19148,19050,10598,19115,10597,10598,19115,12711,12743,21227,23341,29712,27566,21162,27534,33807,33839,40113,35822,39982,40277,54740,25029,22916,22917,22948,22948,22916,24964,24964,23047,27175,18984,25159,23047,21096,25094,18984,27240,23014,18984,25094,20967,23112,25094,18952,25127,20934,23178,50777,48598,59163,54970,54970,54969,54969,54937,54937,54969,54969,54937,52857,54970,54937,54970,54969,54969,220},
{59295,57215,57183,55135,53054,53023,50942,50942,48894,48862,48862,53022,53055,53055,55103,53055,53023,53022,53055,53023,50943,48862,50910,34202,14954,19082,29744,31890,38164,36182,33971,33971,36051,38229,34003,31889,33938,36083,34004,31858,31922,31890,31858,33971,29842,29810,29842,29778,27697,29778,29777,27729,27729,29777,27697,27697,29777,29842,29810,29778,31890,29778,29777,27762,29778,27729,29810,31825,27697,29745,31826,27665,27665,27633,27697,25584,29680,27665,25584,27697,25585,25552,27632,27665,27664,25616,25616,27665,27665,27665,29745,27664,27665,27600,27664,25584,27632,25585,25551,27632,27632,25584,25584,25584,27632,25584,25552,27632,23471,21424,27632,25552,23471,27632,25552,25552,27632,25584,25552,27632,23504,23503,27600,23504,23407,19246,12907,19213,14987,8746,21261,10793,10761,19180,8648,10728,23373,14889,12808,21260,12808,12808,23340,17001,14954,19082,10728,19147,16969,12776,21195,19017,19016,23275,19015,19048,25388,18984,19049,25453,14823,10598,16969,4324,8550,14856,6404,10727,14856,6437,12841,12678,6437,16970,8485,6405,17002,6404,6437,16969,4291,4389,16970,4324,8550,14856,6404,29679,31694,33807,38032,33774,37869,33937,56886,25029,24996,24996,24996,22948,22916,22917,22916,23046,22948,8321,18788,20836,8386,24997,12514,16708,22884,6273,22949,14594,14628,22916,8354,20868,18723,12580,42358,57115,57050,55002,54970,54970,54969,54969,55002,54969,54969,54937,54969,55002,54969,55002,54969,57050,218},
{61343,59263,57183,55103,55103,53055,50974,50942,48894,48830,48862,53022,53055,53055,53055,53055,53055,53023,53055,53055,53022,50942,50942,44635,12972,17001,27533,31858,33971,36116,34003,31858,34003,38229,36117,31890,33938,36051,36116,33939,33971,31891,31858,34003,31923,29810,31891,29778,29745,29810,29746,27729,27762,31858,27697,29778,31858,31858,29842,29778,29810,29778,29745,27730,27697,27665,27729,27697,27665,27697,29777,25584,25617,27665,25617,25584,27632,25584,25552,27665,25585,27632,27664,27665,27665,29713,27697,29713,27697,29745,29745,27664,29713,27632,29713,25584,27665,27665,25584,27665,25617,27632,27632,27633,27664,25584,25552,27632,23472,23472,29680,25552,23472,27632,25552,23504,27600,25617,25584,27632,23504,23503,27632,23504,23439,19246,12939,19213,12906,8714,19213,10761,10760,19212,8647,10761,21293,14921,12808,21260,10728,12841,23308,16969,17034,19082,14921,21228,16968,12743,21228,14823,19016,23308,16936,16968,23308,16935,16968,27534,14791,12743,16936,4324,10663,12743,4357,12841,10630,6405,14954,8485,4389,14986,8485,4421,14954,6372,6437,14889,4324,6503,14824,4292,8648,12710,8517,31824,33807,33839,37968,33807,37869,31695,59064,27109,24996,22948,22948,24965,24964,22917,22948,22982,24997,8289,14660,22884,4161,25029,14530,10467,22949,4161,18821,16642,8387,22916,6208,16740,20771,6306,38067,57083,54937,54970,54969,54970,54970,54970,55002,54970,54970,54937,54970,57018,54970,54970,54969,57050,217},
{59295,57215,57183,55134,55103,53055,50974,50974,50942,48862,48862,52990,55103,53055,53054,55103,55103,53023,55103,53055,53023,50974,48894,48862,19475,14953,21195,31793,33939,36116,36052,31858,33938,36083,34068,33971,31890,33938,36116,33971,31858,33939,31858,33938,31890,29778,29810,27729,27729,29778,27698,27697,27730,29745,27697,29778,31825,29777,29810,29778,29810,27762,29778,29778,29778,27697,29778,27729,27697,29777,29745,27665,25617,27665,27664,25584,27665,25584,25584,27664,25617,25617,27697,25649,27697,27697,29810,29745,29745,29778,29745,27697,27697,29745,29745,27632,29745,27665,25584,27664,27664,25584,27665,27632,25584,25584,23504,27600,23472,23471,27632,25520,25519,27632,25552,23503,27632,25584,23504,25584,23472,23471,25584,23471,21358,19246,12907,19213,12874,8713,19213,8713,8680,19180,8648,10793,21293,14889,12808,21228,10728,12873,19180,14921,19179,21195,19114,23341,16904,12743,23275,14791,16936,23276,16904,16904,23275,16936,17001,25421,12743,14824,12743,6404,12809,8517,6405,17034,6436,6437,17002,4324,4389,16969,4324,6470,16969,4324,8582,12711,4324,10728,10598,6437,14889,8485,10631,33840,33774,29613,35920,33807,37901,27501,61242,27109,22949,22916,22917,24964,24964,22916,24964,22981,25029,10369,10499,22916,4129,22949,18690,8387,22949,6209,16741,18723,6274,22917,10369,14660,22884,4194,33809,46583,52857,57083,59196,59196,59196,54970,54970,50743,44404,42291,42292,44372,44372,44340,48663,52824,201},
{59295,59263,57183,55134,55103,53055,50974,50974,50942,48862,48862,52990,55103,53055,53023,53055,53055,53023,55103,55135,53055,50975,50974,50942,27993,12874,17002,29679,31858,36084,34036,33939,33938,33971,36084,33939,31825,31890,34003,31923,31858,33971,31858,31826,31890,29778,29777,29778,29777,29778,29777,29745,27730,29745,29778,29778,29778,29778,29842,29778,31858,29810,29778,29810,29810,31826,31891,29810,29778,29810,29745,29713,27730,27697,27665,27665,27665,25584,25584,27665,25617,27665,27697,27697,27697,27697,29810,27665,29745,29777,29713,27665,27665,29745,27664,25552,29712,25584,23504,27632,25584,25520,27632,25584,25584,27632,25552,25552,23504,23503,27633,25520,23471,27632,25552,23471,27664,25584,23504,27632,23472,23439,25584,23471,23407,19213,12907,19213,12874,10793,19245,8713,8681,19181,8647,10761,21293,14954,12841,19148,10728,17034,19115,14889,19180,19082,21162,25421,16904,14823,23276,16936,14855,23275,16903,16936,23242,16903,19081,23275,12678,14889,8517,4324,14922,6404,6405,17002,6405,6437,17002,4356,6470,16937,6437,8615,12743,4324,10760,8517,4324,14890,6404,6405,14889,6437,19148,27469,25355,21194,31726,27468,37869,38033,63356,31302,22949,22916,24997,24964,24964,22916,22916,22949,25094,16675,12613,25030,8387,23014,20836,10532,22982,10435,18854,22916,6339,25030,14563,14692,24997,6274,29582,46583,52890,59196,61276,59196,61276,57018,52890,50743,42292,40178,38098,40178,40178,40178,44470,50744,190},
{59295,59295,57183,55103,55103,53055,50974,50974,48894,48862,50910,53022,53055,53055,53055,55103,53055,53055,55135,55135,53055,53055,53023,50942,42653,10924,14954,27501,31890,33971,36084,31859,31857,31890,34036,31923,31825,31825,31890,31890,31858,31923,31858,31826,31923,29810,29810,29811,29778,29810,29810,29810,29778,29778,31858,31891,31858,31826,31890,31858,31858,31891,29778,29778,29810,31826,31858,29810,29810,29778,27697,27697,27697,27665,27665,25585,27665,25584,23504,27665,25585,25584,27665,27697,29713,29713,29778,27697,29745,29777,29713,29745,27665,27665,27664,25552,27665,25584,25584,27665,25617,25584,27665,25584,25552,27632,23504,23536,25552,23472,27632,25552,25520,27632,23536,25552,27665,25552,23504,25584,23504,23503,27632,23504,21358,21326,10827,19213,12907,10793,21261,8713,8713,21229,8680,10793,21261,14954,14921,17067,10728,19115,17002,12840,21260,14889,19082,27534,16904,14855,23276,14855,14888,21195,16903,16969,21162,16935,21130,19049,14823,17034,6404,6372,17002,6372,6405,17002,6405,6438,16937,4324,8583,12775,4356,12809,10598,6405,14954,6404,4292,14922,6437,6437,14921,12907,46518,23242,29646,33775,35887,35887,35789,52791,61308,33447,24965,22916,24964,24996,22916,22948,22948,22981,25126,22981,18951,25094,18919,25127,25094,18983,25094,20999,23079,25062,18951,25094,20933,21031,25094,18918,25324,48696,55003,61276,61276,52661,61276,57050,52922,52856,44404,42259,40179,40211,40211,40211,42422,50777,196},
{59295,59295,57183,55135,55103,55103,53023,50942,50942,48862,48862,53022,55103,55103,55103,55103,53055,53055,55103,55135,55135,53055,50974,50974,53055,44635,46650,46650,36247,31858,34036,31890,31858,33971,34004,34003,31923,33906,33938,34003,31891,31923,31891,31858,31923,29811,31826,31858,29777,27729,27729,27729,27729,27730,31858,29778,29810,29777,29810,29778,29777,31858,29778,29777,29810,29778,31825,31858,31858,29778,27698,29745,27729,27697,29745,27665,27697,27697,25585,27665,27665,27698,27600,29777,29745,29745,31890,29778,31826,31890,29745,29745,27697,29745,27665,25584,27665,27632,25584,27697,27665,27632,27633,25584,25552,27632,23504,25552,23504,23472,27632,25551,25551,27632,25552,23503,27664,25552,23472,27633,21456,23472,25552,21423,21358,19246,10827,19213,12907,10793,21261,8713,10760,21228,6599,10761,21228,14921,14954,19115,12776,21228,16937,14824,21228,14856,19082,25453,12710,14823,21227,12742,16936,21194,14822,21162,19081,16935,23243,16904,14856,17034,6339,4324,16970,6372,6438,16969,6372,8615,14824,4324,12809,10598,4357,14954,6404,4357,14921,4324,4357,16970,6404,8550,12775,19180,44372,33807,31727,33775,35887,37968,35789,52726,57115,37673,24997,24996,24996,24964,22916,22948,22948,22981,23014,18723,4193,22917,10369,14628,22916,4129,22916,14562,8386,22884,6208,18788,18690,6274,22916,10369,14759,48697,57116,61276,61276,57050,61276,59131,52922,54970,46485,42292,42292,42292,42259,40211,42357,52857,199},
{61343,59295,57183,55135,55103,55103,53055,50942,48894,50942,48862,52990,53055,55103,55103,55103,53055,53055,55103,55135,55103,53023,50975,50974,53054,57183,57215,57215,55134,46683,44635,44603,46682,46714,46682,46682,44634,36150,31858,31923,29842,29810,31891,29778,27762,27730,29778,29778,29778,29778,29778,29778,27729,27730,29810,29778,29778,29778,29810,29778,27762,29842,29810,29810,31923,29810,29810,29874,29778,29778,29778,31793,29810,27698,29746,29745,27633,27665,27665,27697,27697,27730,27632,29777,31826,29745,31826,29777,29777,29810,29745,29745,27729,27697,27665,25616,27665,25584,25584,29713,27632,25584,27664,25584,23504,27600,25552,23504,25552,23472,25584,25552,25552,27632,23504,23503,27633,25552,23504,27632,23504,23504,25584,21423,21326,17165,12907,21294,12939,10793,21261,8713,8712,19212,6600,12841,19147,12841,17035,19082,12808,21227,17001,12743,21228,12808,17066,25421,10663,10662,19082,12678,19017,19049,14790,21163,16936,16903,23308,16903,12807,17002,4291,6405,16969,6372,8583,14856,4356,10761,10598,4356,14954,6404,4389,14954,4356,4389,14889,4292,6503,14824,4324,10728,8550,31760,38098,25355,33840,31694,33807,38033,33741,46419,52890,39851,24997,24996,24996,24964,22948,24996,22948,24997,23014,20835,2113,22917,14530,12515,24964,4128,18788,18690,6274,22916,8289,16708,20771,4193,22916,12449,12613,46551,59228,61276,61276,61276,61276,59195,54970,54970,46550,44404,44372,42324,42292,42292,40277,50809,200},
{61343,59295,57215,55135,57183,55135,53055,53022,50942,50942,48862,50942,55103,55103,55103,55103,55103,55103,55103,55103,55103,53054,50974,50974,53055,55103,57183,59263,57215,55135,53055,53054,57215,57215,59263,57215,57215,46780,31826,31890,31858,29810,31890,29778,29777,29810,29778,29778,27730,29778,29810,29810,29778,29810,29842,31859,31891,31859,29810,29842,29810,31891,31891,31858,34003,31923,29810,31955,31858,29810,29810,29777,27697,27729,27697,27665,27664,29745,25617,27665,27665,27697,27632,27697,29810,29745,29778,29745,27632,29778,29745,29745,27697,27665,27665,25584,27633,25584,25617,27697,27632,25584,27665,25552,25552,25552,23504,23472,25552,25519,25552,23504,23471,27632,23504,23503,27665,25584,25552,27633,23536,25552,25584,21391,21358,17165,12907,19246,12906,10793,21261,8681,8712,19180,8648,12906,17099,14889,19115,17034,14857,19180,12775,8615,19212,10727,17001,23308,12743,14856,19082,12710,19082,16936,12743,17002,8516,10662,21195,14790,12711,16969,4291,8518,14856,6372,10728,10630,6437,14922,8485,6405,14922,4324,4389,14921,4324,6503,14792,4324,10728,10597,6437,12874,6437,29646,33807,23307,40113,33807,33808,40146,38066,59164,61309,44109,24997,22949,24997,22948,22916,24996,24964,24997,23014,22916,2081,20901,16642,8387,24965,8257,16708,20803,4193,22948,12417,14596,22884,4161,20868,14530,10500,42326,59228,61276,61276,61276,61308,61276,54970,54971,48663,44405,44372,42324,42324,42324,42325,50809,204},
{59295,59295,57215,57183,57183,55103,55103,53023,50974,50942,48894,50942,55103,55103,55103,55103,55103,55103,57183,55103,53055,53023,50974,53022,53023,55103,57183,59295,57215,55135,53055,55103,57215,57215,57215,57215,57183,53087,29811,29810,29810,29778,29842,29810,31826,31891,31858,31826,29811,31826,29778,29810,31826,29810,29842,31858,31891,31890,29777,29874,29810,29810,29842,29810,31923,31890,31858,33971,31859,31890,29811,27730,27665,27698,29745,27665,27665,27697,25617,27665,27697,27697,27664,27697,29777,29777,29777,29778,29713,29778,27729,27697,27730,27697,29745,27665,27633,27633,27665,27697,27697,27665,27665,25585,25552,27632,25584,25552,27600,23472,25552,25552,23471,27632,23504,23504,27664,23504,23504,25584,23536,25552,25584,23439,23407,19181,12907,21326,12906,8746,19213,8713,8712,19180,6632,14986,19147,12874,21293,19115,17002,21260,12776,10695,21228,12775,17001,21227,14824,16969,19081,14822,21195,16872,19017,17067,8485,10694,21195,10597,8550,14889,4356,10664,12710,6404,12841,8485,6437,16970,4324,6437,14921,4324,6503,14856,6437,10729,10598,4357,14922,8452,6470,14889,4356,19081,27533,27534,42226,35920,35888,40179,46615,63390,63422,50449,25029,22949,22949,24996,24996,25028,24997,24997,23047,25127,21097,27240,23014,23177,27175,21032,25225,25095,23145,27175,20967,23112,25126,18951,25127,23014,21064,40180,59196,61276,61276,61276,61308,61276,55002,55003,52824,44437,44373,42356,44405,44404,42325,48729,205},
{59295,59263,57215,57183,55103,55103,55103,53055,53023,53023,50942,53023,55103,55103,55103,55103,55103,55103,55135,55103,53055,50974,53055,53023,50975,53023,55135,59263,57215,55167,53087,55103,57215,57215,57183,55135,57215,57215,34103,29810,31923,31858,31890,29811,31858,31923,31891,31858,31891,31858,29810,29810,31826,29778,29810,29778,29843,29842,29778,29811,31858,29778,31858,29778,29810,29810,29778,31922,29810,31858,29842,27729,27697,27730,27697,27665,25584,29745,27665,25649,27665,29746,27665,29745,31858,29777,29745,29778,29777,29778,29778,29745,29778,27697,29745,27665,27697,27664,27665,29745,27665,27665,27665,25616,25584,27664,25584,25552,27600,23504,25552,25520,23472,27632,23472,23472,27632,25552,23504,27633,23536,25552,25552,19278,23439,15020,12907,21326,12907,10793,19213,8713,8712,19148,8647,14987,17034,12874,21293,19082,14921,21228,10695,10695,19147,10662,17001,19147,12743,19082,19016,14823,21227,14823,25355,19114,6437,12743,21195,10629,12744,14856,6404,14921,10565,6437,16970,8453,6469,17034,6404,6502,16936,6405,10728,12678,6405,14922,8453,6437,14921,6372,6470,16937,6404,10695,21162,29647,40178,38065,38065,42291,52922,63389,63422,50514,27241,31597,31564,27208,22916,25029,22949,24996,23014,24997,8289,16708,20803,6274,22949,12482,12547,22917,6241,20869,18691,10434,22916,8321,16740,20836,8354,35889,57115,61276,61276,61276,61308,61276,57051,55003,52856,46518,44438,44405,44405,44405,44405,46648,207},
{61343,59263,57215,55135,55103,55103,55102,53055,53023,53023,53022,53022,55103,55103,53055,53087,55103,55103,55103,55103,53055,53023,53055,53055,50975,53055,55135,57215,57215,57215,55135,55103,57215,55135,55135,55135,57214,59263,55134,55069,53021,55069,55069,55069,55036,53020,52988,52956,52956,52956,50875,50875,50843,50843,50843,46682,44602,42489,42456,42489,44504,42424,42456,42424,42489,46649,46681,46682,46681,46649,46681,44569,44504,44569,42456,42423,36116,33937,27664,27632,25519,25487,25519,27599,27664,29680,27631,25584,27631,27664,25551,27632,27632,25552,27599,25519,25519,25552,25551,27632,25552,27632,27632,25551,23471,25519,25520,25487,27600,23439,25520,23471,23503,25551,23439,23471,25584,23471,23471,25551,21455,23471,25551,21391,21358,15052,10859,19246,10826,10794,19212,8680,10793,19147,8647,17034,17034,14987,23341,23340,19082,21228,10695,10728,19147,8582,17001,19082,14824,19114,16904,14823,23275,14823,19113,19082,6469,10695,16969,8484,12776,10598,6404,14921,6372,4324,14921,4323,6437,14920,4324,8615,12710,6372,14921,8452,6404,14921,6371,6405,14889,4291,8550,12743,8615,23243,19081,31792,35920,38032,42291,46451,54970,63389,63389,54740,33678,38034,31630,27240,22917,25029,24997,24964,23014,25029,8289,14595,22883,4161,22949,14562,8387,24997,6209,18788,20771,6242,22916,10369,12548,22884,4161,31663,57083,61276,61276,61276,61308,61308,57083,55003,54937,48663,48631,46551,46551,46551,46551,48696,207},
{61343,59263,57183,55135,55103,55103,55135,55103,53023,53022,53022,55103,55103,55103,55103,55135,55103,55135,55135,53055,53055,53055,55103,53055,53023,53023,55102,57215,57215,57215,55135,55135,57215,55135,55103,55135,57215,59263,59295,59294,59295,61343,61375,61375,61375,61375,61375,61343,59295,61343,59294,59294,59262,59294,61342,59294,55102,52956,50876,50876,52924,50876,50876,50876,55037,59230,61342,61374,61374,61374,61374,61342,61342,63422,61310,59229,46551,40178,29679,31825,15053,15019,19179,25519,23438,23406,23405,23374,23341,21325,19115,23373,21293,21293,21293,19213,21293,21293,19147,23405,23405,27566,23405,25518,23406,23406,19212,23373,23438,21325,17100,21358,23406,21357,19147,21326,21293,21325,21325,23373,21293,21325,17034,14987,17099,15019,14954,14922,12841,14922,12808,10728,12841,12840,12808,12775,10727,10727,12775,10694,12775,12775,14856,12775,14888,14856,14856,16969,16969,14888,14823,14823,14824,12710,12743,12743,12743,12743,12775,12710,12775,10695,10695,12776,10630,6469,10598,8485,10630,10662,10598,8583,10598,8550,14856,12743,10695,12742,8485,6469,10597,6404,10695,12809,23439,23242,14823,12710,21194,27468,27402,29482,44437,59130,59162,56918,27142,27305,27273,25095,22917,24997,22949,22916,23014,25029,12449,10499,22916,4128,20869,18691,6306,24997,8289,14660,22883,4161,22917,14530,10467,24996,4161,23243,57115,61276,61308,61276,61309,61308,59163,57051,55003,55003,55035,55003,55003,55003,55003,54971,211},
{61343,59263,57183,55103,55102,55103,57151,55071,53022,53023,53023,53055,55103,55103,55103,55103,55135,55135,55103,53023,53055,55103,55103,53055,53023,53022,55102,57182,57182,57215,55135,55102,57183,55135,55135,55135,57183,59263,59295,59295,59295,61343,61343,61375,61375,61375,61375,61343,61343,61343,59295,57182,57182,59230,59262,61310,57182,52989,50876,50876,50876,52924,52924,50908,55036,61310,61374,63422,63423,63423,61342,61342,63390,61374,52922,44406,42390,31825,31760,31792,15053,15019,17099,17100,23405,23405,23373,21325,21293,21261,19115,21293,21260,19180,21261,19213,21260,23341,19147,25454,25421,25486,25485,25518,23373,17100,19115,19148,25519,19180,17132,17132,23470,19180,17067,19180,21293,21325,21325,23373,21293,21325,17002,12906,15019,15019,12874,12873,10761,12841,10728,8615,10728,8648,10663,8582,8583,6566,8582,8582,8614,8582,10695,10662,12775,12743,12743,12775,14856,12743,12711,10598,10598,10565,10597,8550,10630,8550,10598,8517,8549,8517,10630,12743,8517,6372,6340,6372,8485,4356,6404,4324,4356,6404,10630,12743,10630,6404,4324,2242,4291,2210,6437,10663,21326,31694,29646,23243,25356,29516,25289,31564,44372,48597,61275,61209,29222,24997,24997,24997,22949,25029,22949,24997,22982,25160,18853,16838,25063,12613,23080,22982,10597,25127,16741,16838,25094,10532,23047,18821,14726,25094,12580,21130,55035,61276,61308,61276,61308,61308,59164,55003,55003,55003,55003,55003,55003,55003,55003,54971,212},
{61343,59295,57183,55103,55103,55103,55135,53055,53023,53023,53023,55103,55103,55103,55103,55103,55135,55135,53055,53023,53055,55103,55103,53023,53022,53022,53022,57150,57183,57215,55135,55135,57215,55103,55135,55103,57183,59263,59295,59295,59295,61343,61375,61375,61343,61343,61375,61343,61343,61375,61375,55102,55101,57149,57181,59262,57214,55102,50908,50876,50876,50876,50876,52956,55036,61310,63422,63454,63423,63422,61342,59262,61342,61342,38165,27534,40244,31761,27567,25486,13037,15084,17099,15020,19180,19147,19148,17099,19115,17035,17067,17067,17067,17035,17035,17067,17035,15018,14986,19147,19147,21228,19147,19179,17132,15019,15019,12906,19180,15019,17099,12939,17099,15019,15019,14987,17099,17035,19115,17067,17034,17067,14954,10793,12873,12874,10760,10728,10728,10696,8680,8583,8615,6534,8582,6469,8582,6502,8582,8549,8582,6501,10630,8581,10663,10662,12711,12711,12711,10630,10630,8517,10565,8485,8549,8517,10598,8517,8517,8485,8517,4356,8550,8550,8517,6339,6372,4324,6404,4324,6404,4292,4324,6437,10662,12710,10630,6437,6404,4291,4291,2211,6437,8517,17067,33710,31694,25355,27469,33839,27338,33644,50743,61276,65501,63322,27173,20836,20835,20836,20836,22916,22916,22916,20869,23176,27370,27337,27337,27337,27370,29418,29483,29483,29483,29482,29418,27337,27305,27337,29450,29450,27402,27403,50874,61276,61308,61308,61308,61309,59196,52922,52890,52890,52890,52890,52890,52890,52890,54970,213},
{59295,59263,57215,55135,55103,55103,55135,55103,53022,53022,53055,55103,55103,55103,55103,55103,55135,55135,55135,53055,55103,55135,53055,53022,53022,52990,53022,57150,57183,57215,57183,57183,55135,53055,53055,55103,57183,59263,59295,59263,59295,59295,61343,61375,59327,61343,61375,61375,59294,61375,61375,59294,55102,55069,57150,59262,57214,53021,52956,50844,50876,50876,52956,52956,52988,59229,63455,63455,63455,61342,59262,59262,61342,61342,33971,29614,46518,36050,15149,10988,13005,17067,21260,19278,23406,23406,23405,19277,23341,19213,19147,19245,21293,21293,19212,17132,21260,21293,17067,23373,23341,25454,21293,17165,25487,19213,17067,15052,23406,23373,21325,21326,21325,15052,15019,15052,23406,17100,21260,21293,19213,21293,14954,14955,15019,15019,14954,12841,12873,10793,10760,8647,10696,8615,10663,6534,8614,8614,8614,8582,10662,8614,10694,10662,12775,12775,12743,12775,12743,12743,10630,10663,10662,8549,8550,10598,8550,8549,8549,6404,8549,6405,8550,8550,6437,6404,6372,8452,6372,4291,4324,4324,6436,6437,10630,12710,8549,8517,8549,6404,4291,4291,6404,8485,14889,27435,33840,25356,27501,31661,27371,35789,46419,61341,65502,63289,20836,12515,10434,12515,14595,14595,14595,14595,14595,12579,12579,12579,14659,12579,14627,12547,12547,12547,12515,12547,14660,16740,12547,12547,12547,14595,12547,12612,42423,61308,61308,61308,61308,61309,57050,52890,50777,46550,44470,46518,46551,46551,46551,50809,216},
{59327,59295,57215,55135,55103,55103,55135,55103,53023,50974,53055,53055,55103,55103,55135,55135,55135,55103,55103,53055,55103,55103,53023,53022,50942,52990,55102,55102,57182,57215,57183,55135,55103,53055,53054,53055,57183,57215,59263,59263,59295,59295,61343,61343,59295,61343,61343,59295,59295,61375,61374,61374,57182,53021,57150,59294,59262,55070,50908,50844,50844,50876,52924,52956,52989,57149,63423,65535,65535,63423,61342,59294,55003,54970,31858,27567,31825,29744,29745,25649,13005,19116,21260,21293,23406,23406,23405,21326,21293,19212,19180,23341,21293,17165,19213,19213,19180,19180,17067,21325,23373,23373,23373,23438,23406,23406,19179,23406,25486,23373,23374,23406,23405,23373,19147,21325,21325,21326,21293,21293,21261,21293,17034,12939,17067,15019,14954,12841,12873,10793,10728,8647,8647,6567,8615,6534,8582,8582,8582,8582,10662,8614,10694,10662,12775,10694,12743,12775,12775,12776,10662,10662,10630,8517,8517,10565,8517,8517,8517,8485,6436,6404,8550,10598,6404,6372,6372,6404,6372,4323,4291,4323,4324,8517,10662,10630,8517,6404,8485,4323,2243,4291,2211,6404,10728,25388,38098,29581,29646,33742,33807,52758,44273,61308,63421,61177,23046,16676,18756,20869,22916,24996,22949,22949,24965,22949,24997,22949,22948,22917,24997,24996,24997,22949,22948,22948,24997,24997,24964,24996,22949,24997,22949,25029,42293,61277,61276,61309,61308,61276,52890,52890,50744,46583,46583,46583,46583,46583,48664,52987,219},
{61343,59263,57215,55135,55135,55135,55135,55103,53054,50975,55103,53055,55135,55135,55135,55103,55103,55103,55103,53023,53055,55103,53023,50942,50941,52990,55070,55102,57182,57215,57183,53055,53055,53022,55103,55103,57183,57215,57215,59295,59295,59295,59295,61343,61343,61343,61343,59295,59295,61375,61374,59294,55102,52989,57149,61342,59262,55102,50908,48795,50843,50876,52956,52989,53021,57149,57051,61309,65502,63422,61342,59262,33906,27599,13069,17197,25617,25552,19245,17165,15085,17164,15052,14987,19180,19180,19147,17068,19115,17035,17067,17067,19147,17067,17067,17099,12939,12874,12907,14955,19212,14986,21228,21260,21260,19212,17099,21260,21260,21260,21260,19212,21260,21260,17099,19212,19180,19212,19180,12907,19213,12939,12906,8713,14954,12906,12841,10760,12841,10761,8647,8647,8648,8647,8615,6534,8614,8614,8582,8614,10662,8614,10694,10662,12743,12743,14823,14824,12743,14791,12711,10663,10630,8517,6438,8517,8583,6437,8550,8517,6404,6436,8549,10598,8484,6404,6372,6404,6405,6404,4324,4323,4292,4324,6372,6372,6372,6372,6404,4291,2243,4291,4291,4291,8550,27501,38034,31694,33807,35855,35855,42095,48532,63421,65502,61243,31498,25062,22884,22916,22948,24996,22916,24997,24997,22949,24997,24997,24964,24964,24996,24996,24996,24996,24996,24996,22948,24996,24997,24997,24996,24996,24964,24997,38034,59228,61277,61309,61308,59131,52890,52890,48631,46583,46583,46583,46583,48631,48729,55068,221},
{61375,59295,57215,55135,55102,57183,57183,55103,53055,53023,55103,53023,55135,55135,55135,55135,55103,53055,55103,53023,50974,53023,50974,50910,50909,52989,52990,53022,57182,57215,55135,53055,53022,53055,53055,55102,57183,57182,57183,59295,59295,59295,61343,61343,61343,61343,61375,61343,61343,61375,61375,61375,57182,55069,55101,57214,59294,55102,48828,46650,36051,46617,44503,50810,50810,46649,38099,50777,42357,42259,50777,40277,31826,33939,21456,25519,36051,29712,29680,25519,15053,17099,15052,17133,23438,21326,21325,21293,21261,19180,17099,19212,19212,19180,17035,12907,15117,12907,10858,12939,21293,21325,21293,21325,21292,21293,17132,21293,19212,21293,21260,21293,21260,21261,17099,19212,19180,19180,19148,17164,17067,12875,12906,8713,14987,8649,10729,10728,10760,10728,8647,8615,8615,8582,6534,6534,8550,6502,8582,6501,8582,6501,8582,8581,8614,10663,12775,12775,12743,10663,10630,8550,8550,8517,6438,8517,8550,8517,8517,6469,6372,6404,8517,8517,6404,6372,6372,6404,8550,6436,4292,4291,2243,4323,4324,4291,2243,4291,4292,4323,4292,4291,4323,4291,4356,21195,29647,38001,38033,42226,38000,40079,48662,65501,65502,61243,33544,25029,22917,22948,22948,22949,22917,24996,24997,22949,24997,24996,24964,24996,24996,24997,24997,24997,24997,24997,24997,24997,24996,24997,24997,24996,24997,24997,33808,59196,61277,61309,61308,54970,52890,50777,48631,46583,46583,46551,46583,48631,52923,57116,224},
{61375,59295,57215,57183,57183,57215,57183,55103,53055,53023,55103,53055,55135,55135,55135,55135,55135,55135,55103,53055,48894,48862,50942,50910,50909,50941,52990,53021,55134,57215,55135,50975,50974,53023,53055,55134,55134,55134,55135,59295,59295,59295,61375,61343,61343,61343,61343,61343,59295,61375,61375,61374,59295,55102,57149,59262,59295,57182,48828,29972,19309,40244,31825,42423,36084,44536,55002,61310,50810,57083,59229,44568,27632,33938,25486,31759,38164,35986,33905,27633,15021,19147,23406,23438,25518,21325,23406,21325,21293,19180,19147,19245,21293,19180,19181,19213,19180,19181,17067,21358,23373,25454,23405,23438,23406,21325,19180,23406,21325,21325,23373,23406,21325,23405,19147,21325,21293,21293,19245,21325,19245,19181,14987,12874,14987,12906,12874,10793,12841,12841,10696,10695,8679,8615,8615,6567,8583,8615,8549,8582,8582,8614,10694,8614,10694,12743,12775,12775,14856,12743,12743,12743,10695,10662,8517,8518,8582,10630,10663,10630,6404,6436,8484,8485,6372,6372,4291,6404,14856,6437,4291,4291,2211,4291,4259,2211,4259,2243,4292,4291,2243,2211,2243,2211,4356,29646,23308,31662,29581,33742,29548,38065,55002,65501,65502,65435,29254,24997,24997,22981,22949,24997,22948,24996,24997,24997,25029,24997,24997,24997,25029,27077,25029,24997,25029,24997,25029,25029,25029,25029,24997,24997,25029,25029,29517,59196,61308,61308,59196,52890,52890,50712,46551,46551,46551,46519,46551,48664,55068,57116,229},
{61375,59295,57215,57183,57215,57215,57215,55103,53055,53022,55071,53055,55135,55135,55135,57183,57183,57183,55135,53055,50943,48894,50942,50942,50909,50909,50909,53021,57182,57183,55135,50975,50942,50942,53023,55102,55135,55103,57183,59262,59295,61343,61343,61343,61343,59295,59295,59295,59294,61342,61342,61374,59294,55102,57182,59294,59262,59262,53022,36280,29712,48730,42424,50875,44536,44537,40179,55002,50744,46485,57051,48696,27633,38067,23373,33873,36050,35954,35986,23471,13037,19147,17100,19180,23406,23341,25422,23373,17068,14955,14955,17100,17133,14955,19180,19212,19180,21228,17067,21325,21325,23405,23373,23373,23405,23373,21228,23406,23373,23373,21325,23373,21325,21325,17099,21325,21325,21293,21293,21325,21293,19213,14987,10826,14954,14987,12874,8680,12906,8648,8615,6535,8680,10696,8615,6534,8614,6534,8582,8582,8582,8614,10694,8613,12743,10662,10695,12776,12808,12743,12711,12743,12743,12743,10630,10630,12711,10630,12743,10630,6404,8484,8484,8484,6372,6372,6339,6404,16936,8484,4291,4291,2211,4291,2211,4259,4259,2243,4291,4291,2211,2211,4291,4291,4324,33872,29581,31629,29581,31694,25323,40178,61341,65502,65534,56886,22949,20836,20836,20868,20836,20804,20836,20836,20869,20901,20869,20869,20901,20869,20901,20901,20869,20901,20869,20869,20901,20869,20901,20869,20869,20869,18820,18853,21097,57148,61308,61276,57051,52890,52890,50777,50777,50777,50777,50809,50810,52923,57084,57116,238},
{61375,59295,59263,57215,57215,57215,57215,55103,53023,50974,55070,53055,55135,57183,55135,57183,57183,57183,57183,55135,53023,50974,50942,50941,50909,50909,52957,53021,55102,57183,53054,50975,48894,50942,53023,53055,55103,55103,55135,59263,59295,59295,61343,61343,61343,59295,59295,59295,57214,59230,59294,61342,59262,55101,57182,59294,59294,57214,55134,52989,46683,50908,46683,52989,50908,48730,46584,52955,42423,50777,52923,42423,31891,38131,29713,33873,36051,31858,31792,23471,15085,15084,15020,17068,21228,19180,14987,17068,14987,14955,12939,12907,14987,14955,15019,14987,14954,14987,14986,17067,19115,19147,17099,19147,17099,17067,17067,19147,17099,19147,17099,19115,19115,19147,17067,17067,17067,17067,17035,17099,17067,17067,12874,10826,12873,14922,10728,6535,10760,6535,8648,4454,8615,6534,6534,6534,8582,6502,6502,6501,8549,8582,10630,8581,10630,6502,10695,10695,10695,10630,10598,10630,10630,10630,8550,10598,10630,10630,12743,10630,6404,6404,6404,6404,6404,6372,4323,6436,12743,6436,4291,2243,2211,2243,2211,2243,2211,4259,2211,4292,2243,2211,2275,2211,2276,25420,19016,12710,10565,31792,44372,57050,63421,65502,65502,58998,27109,22981,25029,25029,25029,25029,25029,25029,23079,29483,29418,27338,29451,27371,31564,29451,27403,29482,27370,29451,29418,27338,29450,29418,27435,31499,27370,31563,29516,55068,61277,61276,54938,52890,52890,52890,52890,52890,52922,52923,55003,55035,57116,59229,240},
{61375,59295,59295,57215,59263,57215,57215,53055,53023,50974,53022,55103,55135,57183,57183,57183,57215,57215,57215,55135,55103,53055,50942,50942,50909,50909,52989,53022,57182,57183,53055,50975,48862,48862,50942,50974,53055,55103,55134,57215,59295,59295,59295,61343,61343,61343,59295,59295,55102,59262,59262,59294,55102,57182,59294,59294,59262,57214,57215,55102,52989,53022,53022,55102,55102,55134,59262,59262,61374,63455,63455,61407,31988,27598,31824,33905,38132,31793,38066,23504,15085,19179,21358,21326,25518,23406,19213,21294,17197,17035,15052,15052,19213,19180,19213,21293,21229,21261,17099,21325,23373,23374,21357,23406,23405,23374,21260,23405,23373,23373,21325,23373,21325,21357,19147,21325,21325,19245,21293,19245,19245,19213,12906,15019,14954,10793,10793,8713,10793,6600,8648,6502,10760,8615,6567,8615,8614,8679,8582,8582,8614,8582,10630,6501,10662,8647,10662,8582,8614,12743,10663,10662,12743,12743,10662,10662,12743,12775,12775,10630,6404,8485,6404,8485,8420,6372,6404,8485,10663,8485,4324,4324,2243,4291,2211,4291,4259,4291,4291,4324,4291,4291,4324,4291,4324,25453,25388,23274,14856,25387,46551,63421,63421,65502,65502,46157,27110,25029,25029,25029,24997,25029,24997,25029,25127,25029,12449,23047,25062,10402,16708,14530,12547,24964,8289,18820,18722,6306,22948,12417,16708,20835,6273,20868,16741,46649,61277,59196,52858,52890,52890,52890,52890,52890,52922,54971,55035,57084,57148,61341,239},
{61375,59327,59295,59295,59263,57215,57183,53055,50975,50974,53022,55103,55135,57215,57183,57183,57215,59263,57247,57215,57183,55103,52990,52990,52990,50909,52989,55070,57182,57183,55135,50975,46815,46814,48894,50974,50974,53055,55134,57215,59295,59295,61343,61343,61343,59295,59295,59295,55101,59262,59294,61342,57214,57182,59294,59294,57214,59262,57215,55102,53022,53022,53022,53054,53054,55135,55134,55134,59262,61342,63423,55101,27923,25648,31889,31825,40245,33938,36083,21456,17133,19115,23406,23406,23438,21325,25518,23374,21325,17067,19213,19212,19212,19180,17132,19213,21228,21228,19179,23373,23373,23373,23373,23405,23373,21326,21228,23406,23373,21357,21325,21325,23373,21325,19147,21325,21293,19277,21293,21293,21293,19245,14986,15019,14954,12809,10761,8681,12809,10761,10696,8647,8647,10695,8615,8615,8647,8615,8582,6502,8582,8582,10662,8614,10694,10662,12743,12743,10695,12775,12743,12711,14823,12743,10630,10630,12710,12710,12743,10695,6404,6404,6372,8452,6340,6371,6339,8485,12743,6437,4259,4291,2179,2243,2178,2211,2146,2211,131,4291,2179,2211,2211,4291,4291,27566,38065,33840,33872,35985,52823,61308,65469,65501,65501,35560,27110,25029,25029,25029,25029,25029,24997,25030,25095,24997,10467,22981,18788,10500,20901,12515,18789,20836,8419,22917,14627,14660,22884,8387,20868,18723,10532,20868,12515,40343,61277,57050,52890,52890,52890,52890,52890,52922,52923,55003,55035,57116,59196,63389,239},
{61375,61343,59295,59295,59263,57215,57183,53055,50974,50974,50942,55103,55135,57215,57215,57215,57215,57215,59295,59263,57215,55135,53022,53022,52990,52989,53022,55102,57182,57183,55135,51007,46815,46782,48895,50975,50975,53055,55135,57215,59295,59295,59295,59295,59295,59295,59295,57214,55102,61310,59294,59262,61342,59262,59294,59294,57214,57182,55134,55134,53054,53054,53054,53086,53086,55103,55134,55134,57214,59262,61342,61375,50940,31922,38229,33873,42390,33938,40212,23536,17197,19245,17165,21261,17068,19147,21260,21227,19147,17035,17099,19115,17034,14986,17067,17099,17067,19115,17067,19179,19147,19180,19179,21228,21228,21228,19148,21260,21260,21260,21228,21228,21260,21260,19147,19180,21260,19179,19212,19212,21228,19212,14986,14954,12874,10761,10761,10729,12809,12809,8680,8647,10696,8615,8647,6502,10695,6502,6469,6469,8615,6501,6501,8614,12743,10662,12743,12743,12743,12743,12710,12711,14823,12743,12710,10630,12710,12710,10663,12711,6372,6404,6372,6372,4291,6404,6372,6404,10630,6469,2243,4291,2179,4291,2179,2243,2178,2243,2211,4292,2211,4324,2243,4324,4291,14823,23210,27501,38099,33872,61243,61308,65502,65501,63225,31302,27110,25029,25030,25029,25029,25029,25029,23014,25094,25061,14660,25030,20837,18854,25029,14693,23014,22916,16806,25029,16740,22982,22949,14758,22981,18788,20966,22949,16774,40245,59228,52890,50842,52890,52890,52890,52890,52922,52923,55003,55035,57148,59261,63389,235},
{61375,61375,59295,59295,59263,57215,57215,55135,53023,50975,50974,55103,55103,57215,59263,57215,59263,59263,59295,59295,59263,55102,53054,55102,50941,53021,53022,55102,57183,57183,55135,53055,46815,46814,50943,50974,50975,53055,55135,57215,59295,59295,59295,59295,59295,59295,59295,57182,57182,59230,59262,59262,61342,61342,59294,59294,57214,57182,55134,55134,55102,53054,53022,53054,53054,53086,55135,55134,57214,59262,59262,61342,61342,59262,42424,29647,38131,35986,31825,23406,19244,19277,21325,23438,23374,23406,23470,25486,25518,23406,23406,23405,23373,23374,23373,23374,23341,21293,21293,23341,21260,21293,21260,21293,21260,21292,19212,21293,21260,21260,19212,21228,19180,19212,19180,19180,19179,19180,19147,19180,19147,19148,15019,17035,14954,14922,14921,12809,14889,12808,12841,10728,10728,10695,10695,10696,12776,10727,10695,10695,10695,10695,8647,10662,10663,10727,12808,12775,12743,10695,12743,12743,12775,12775,12775,12743,12743,12711,10663,12711,8550,8550,8550,8550,8518,8518,8550,10598,12711,10630,6469,8518,8485,8517,6437,6469,6436,6469,6469,8517,6437,6470,6469,8517,6469,8517,6469,19082,42422,33676,50678,63421,65502,65501,52626,27142,25029,25029,25030,25029,25029,24996,25029,25127,25029,20803,6306,22949,12417,14660,22851,6241,22917,14530,10499,22884,6241,18821,16642,8387,22916,8289,16740,18723,6339,33874,59131,52890,52890,52890,52890,52890,52890,52922,54970,55003,57116,57148,63422,61308,233},
{61375,61375,59295,59295,59263,59263,57215,55135,55103,53023,53022,55103,57183,57215,59263,59263,59263,59295,59295,59295,59295,55102,53021,55070,50941,52989,55070,55102,57215,57215,55135,53055,46815,48862,48894,48894,50975,53054,57183,59263,59295,59295,59295,61343,59295,61343,59263,59262,57214,57182,57182,59262,61342,61342,59294,59262,57214,57182,57182,57182,55102,53054,53054,55103,55103,55134,55134,57182,57182,59262,59230,59229,59262,59262,55101,48730,36149,27664,27665,17165,17067,19180,21325,25518,25487,27567,27566,27599,27599,23405,25518,25486,27534,23405,23406,23405,23406,21293,21293,23374,23373,21325,21325,21357,21325,21293,19180,21293,21293,21261,21261,21261,21293,21293,19147,21293,21260,21293,21261,19213,21260,21260,17034,19148,17067,15019,14954,12808,12808,10695,10630,10630,10598,8549,8549,8485,10597,8517,6436,6404,8485,6404,6404,6404,6436,8485,6404,10630,10597,10597,8549,8485,8485,8484,8484,8517,8484,8484,8452,8484,8452,8452,8452,8452,6404,8484,8452,8484,8484,8484,8517,8484,8484,8517,8485,8485,6436,8517,8484,8517,8517,8517,6469,10597,8517,10597,6436,19049,38261,59196,61308,63421,65502,65501,44076,25062,25030,25029,25029,25029,25029,25029,25029,25159,27109,22982,21032,25029,18887,23014,20901,18951,25029,18854,20998,22981,16871,22981,18854,20966,22981,16839,22949,18821,18919,31663,55002,52890,52890,52890,52858,52890,52890,52890,54970,55003,57116,59229,63389,61276,225},
{61375,61375,59295,59295,59263,59263,57215,57183,55135,53055,53023,55103,57183,59263,59295,59295,59295,59295,59295,59295,59295,55102,52989,52989,52957,52989,55069,57182,59295,59263,55135,53055,48895,46815,46815,48895,50942,53055,55135,59263,59295,59295,59295,59295,59295,59295,59295,59294,59294,55102,57182,59262,59262,59230,57149,57149,57182,55102,55102,55102,53054,53054,53054,53054,55102,55134,55134,57214,57182,57181,59230,59229,59261,61310,59261,57149,46649,31858,31760,21390,17100,17132,19180,21293,23374,25454,23373,25454,23341,21260,21260,21292,23341,21228,23373,23373,21292,21260,21260,21293,21292,21292,21260,21260,21260,21260,19147,21260,19180,19180,19180,19180,19180,19180,17099,19180,19180,21228,21195,19180,19180,19147,14986,19115,17035,12906,12873,10728,12776,8583,8615,6535,10663,8550,10630,8550,10598,8550,8517,6437,6469,6437,6437,4356,6437,8485,8485,10565,8485,10597,10597,8452,6404,6404,6404,6436,6404,6404,6404,6404,6404,6372,6372,6404,6404,6372,6372,6404,6404,8517,8517,6404,8484,8485,8485,8484,6436,8517,8484,8485,8485,8485,8484,8516,8485,8517,10598,16968,36052,65502,65501,63421,65502,65436,33479,25062,25061,25061,27110,27110,27078,25029,25094,25126,27077,14530,14627,22948,6241,22949,18722,10499,22916,8289,20869,18723,8386,22948,10369,16740,20771,6306,20868,12449,16740,20901,21031,29385,23079,25225,27305,21064,29418,23111,23209,27500,57148,61341,63357,61243,212},
{61343,61343,61343,59263,59263,59263,59263,57215,57183,55103,53055,55103,57183,59263,59263,59295,59295,59295,59295,61343,59327,57215,55070,53022,50909,52989,55102,57182,59262,59295,55167,53055,48895,46814,46814,46846,50943,53023,55135,57215,59295,59295,59295,59295,59263,59295,59295,61343,59262,55069,57181,59262,55101,55069,52989,55037,55069,53021,53021,53021,53054,53022,53022,53054,55102,55134,55134,57214,59262,59262,59262,59230,59230,59262,57181,55101,50710,46484,33905,25584,15020,19148,21293,25454,25487,27599,25550,27599,27599,25519,25486,25518,25486,23373,25454,25454,23406,23373,21260,23405,23373,23374,21325,21325,23341,21325,19180,21325,21293,21293,21293,21293,21293,21261,19147,21293,21293,21293,21260,21261,19212,19180,14987,19180,19147,14987,12874,8648,12841,6567,8615,8583,10695,8615,10663,8614,10662,10662,8549,8517,8549,8517,8518,8517,10566,8550,8549,6437,8485,6404,8484,6404,6404,6404,6404,8452,6404,6404,6404,6404,6404,6404,6404,6404,6404,6404,6372,6404,6436,10630,10597,6404,8484,8484,8485,8484,8484,8484,8484,8484,6436,8484,8484,8452,8484,8484,14856,14791,29843,65502,65501,65501,65501,61111,29222,25061,25029,27077,25061,27077,27077,27109,25127,25061,27077,10434,20869,20836,10500,24997,12515,18821,22916,10468,22949,14595,14693,20836,8386,20869,14595,12612,20836,8354,20868,16643,12645,20869,10402,18789,16643,10532,20836,10562,8514,8645,50907,63389,61308,59130,191},
{61375,61375,59295,59295,59295,59295,59295,57215,55135,55135,53055,55103,57215,59263,59295,59295,59295,61343,61375,61375,61375,61375,59295,55102,52957,52989,55070,57182,59262,59295,57215,51007,48894,46815,46814,48862,50975,53055,55103,57215,59295,59295,59295,59295,59263,59295,61343,59295,55101,52989,59262,57182,52989,50908,50908,50908,52989,53021,55069,53021,53022,53021,52989,53022,55102,55102,55134,57182,59262,59262,59294,59262,59230,57181,57149,57149,50745,42324,33938,31793,17100,19180,19180,19180,23374,25486,25486,25518,25486,25486,25421,25454,25453,23373,23341,21293,21293,19148,17067,17067,21293,19212,21260,21261,21260,21261,19147,19244,21228,21260,21228,19180,21228,21228,19115,21228,19180,21260,19180,19180,17131,17035,14954,17035,17035,10793,12841,10761,12809,10728,8679,8647,10695,8614,10695,8614,10663,8582,10630,10630,12711,12711,12711,10662,12743,12743,10662,8549,8485,6404,8485,6372,6437,6404,6436,6404,6436,6372,6404,6404,6404,6372,6404,6372,4356,6371,6372,4324,4356,8517,10598,8484,6436,6404,8485,8484,8485,8452,8485,6404,8517,6404,6436,8484,8517,8484,16969,12678,27729,65502,65534,65501,65501,52626,27142,25061,25061,27077,25061,27109,25029,25062,25127,27142,27110,16773,23014,20933,18886,25029,14725,23046,22949,16871,25029,16773,23046,20901,16838,22949,16741,23014,20868,16806,22949,16774,20998,22917,14726,20901,16708,20966,18884,12739,10658,10758,50907,61309,61275,56985,172},
{61343,61375,59295,59295,59295,59295,59295,57215,55135,57183,55135,55135,57215,59263,59295,59295,59295,61343,61375,61375,61375,61375,59327,55102,55070,55102,57150,57182,59294,61375,57215,53023,48894,46815,46815,48894,50974,55103,55135,57215,59295,59295,59295,59263,57215,59295,61343,55102,50908,55037,57214,55102,50908,50876,50876,52956,50908,52989,53021,52989,52989,50909,52989,53021,55102,55134,55134,57214,59262,59262,61342,61374,61342,57149,55069,57149,52923,42390,40309,42423,15020,19148,21293,23406,25486,27631,25550,27663,27566,27631,25453,27599,25518,23438,25454,21390,21325,21293,19148,21293,21325,21293,23341,23374,23373,23373,19180,21293,21293,21293,21293,23373,21293,21293,19147,21293,21260,21260,19212,17132,19245,17100,14986,17067,17067,12874,12841,10761,12841,10728,10728,10663,10695,8615,10663,8615,10663,8582,10630,12776,12775,14824,14856,14856,16969,16936,12743,12711,10598,8485,8517,6404,8485,6404,6469,6372,8485,4324,6437,6404,6404,6372,8484,6372,6404,6371,8452,4292,6404,8550,10565,8484,8484,8452,8484,6404,8517,6404,8485,6404,8517,8484,8517,8484,8549,8517,19017,10630,23439,63454,65502,65501,65501,44012,27142,25062,25029,25029,27109,27109,25029,25062,25094,27077,22916,6242,22949,14530,12580,20836,6242,22949,14530,12548,22884,6241,20869,16578,10499,22884,6241,20869,16610,10499,22884,6241,18821,16610,8387,20836,6241,16772,14690,8578,6465,8612,40277,59196,59130,52759,160},
{61375,61343,61343,59295,59295,59295,59295,59263,57183,57183,57183,57183,57215,59295,61343,59295,59295,61375,61375,61375,61375,61375,61407,59295,59262,59263,59295,61343,61375,61375,57215,53055,48895,46815,46815,48895,53023,55134,55135,57215,59295,59295,59295,59295,57215,59295,61343,55102,52957,55037,57182,55069,50908,50876,50844,52956,50941,52989,52989,52957,50908,50876,52957,53021,53054,55134,55134,59262,59262,59262,59294,63422,63455,57182,52923,50809,50745,48632,42390,46584,19279,19180,19180,19180,23373,25518,25454,27566,25486,25518,23372,25454,21325,23373,21293,21325,21293,21293,19179,21293,21292,21293,21260,19148,21261,17067,17067,17100,21260,21228,21228,19212,19180,17067,17067,17067,21260,19180,19179,17132,19148,17132,14987,17099,14986,12874,14922,10760,12809,10696,10696,10663,10695,8615,8647,8615,10662,8582,8582,12743,12743,14824,14824,14856,16936,16936,14824,12743,10663,8550,6469,6437,6469,6404,6469,6404,6437,6404,6436,6372,6436,6372,6404,6371,6404,4324,6404,4324,4324,8517,8484,6404,8484,6403,8484,8452,8485,6437,8484,6436,8484,8485,8517,8517,8549,8582,16936,10663,34004,63454,65502,65501,65435,35528,25062,25062,25061,25029,27077,25029,25061,25094,27142,27109,22981,18919,25030,16838,23014,20901,18952,25030,18886,23015,22949,16871,22981,18821,20966,22949,16871,20901,16773,20966,22917,18951,22949,16774,18918,20868,16839,22948,14723,8578,6466,6466,21454,57082,57017,46420,160},
{61375,61375,61375,61343,59295,61343,59295,57215,57183,57183,57183,57183,59263,61343,61343,61343,61343,61375,61375,63423,61375,61375,63423,63455,61343,61343,61375,61375,63423,61375,57247,53055,48895,46814,46814,48894,53055,55135,57183,57215,59295,59295,59295,59295,57215,59295,61343,57182,53021,52989,55101,55069,52956,50876,50876,52956,52957,52957,52956,50876,50908,50908,53021,53022,55102,55102,57182,59262,59262,59262,59262,61342,61342,55036,34035,33872,44471,44406,44503,40244,23472,19180,21293,25454,25487,27631,27599,29679,25518,29679,23373,27566,23406,25486,23406,23438,21357,23405,21260,23406,23373,23373,21325,23373,21293,19213,17099,19213,23373,19180,17100,19213,21293,19180,17067,17100,21261,21293,19212,21261,19213,19245,17067,17132,14987,10826,14922,10761,10793,8680,8647,10663,8647,8614,8615,10663,10662,8582,10630,10663,14824,14856,14856,14856,16969,19082,14888,14823,12808,12743,12743,10663,12743,8550,8485,6404,8485,8452,8484,6404,6404,4324,6404,6372,6404,4291,6372,4324,6469,8517,6404,6436,8484,8484,8516,8484,8485,8485,8484,8517,8517,8485,8581,8613,8582,12743,14791,12841,29712,63422,65502,65469,58999,29222,25062,25029,25029,25029,27077,25029,25029,25127,25029,27077,16642,10467,22948,8289,20901,18722,10467,22916,8289,18821,18690,10467,22916,8321,18820,18690,8386,22916,8289,18820,18690,10435,20836,8289,18788,16610,8387,20867,10594,6498,6402,6434,12970,54936,52758,23045,84},
{61375,61375,61375,61343,61343,61343,59295,59295,57215,57215,57215,57215,59263,59295,61343,59327,61343,61375,61375,61375,61375,61375,63423,63455,61375,61375,61375,61407,63423,61375,57247,53087,48927,48894,48895,50943,53055,55135,57183,57215,59295,59295,59295,59295,57247,59295,61343,59262,55069,52989,52957,52989,52957,50908,50876,52956,52956,52956,50908,50908,50908,52956,53021,53054,55102,55102,57182,59230,59262,59262,59262,59262,57181,44666,29875,36019,42423,44535,44536,42390,29876,19212,19212,21261,23341,25454,25486,25486,25454,25518,21292,25454,23373,23373,23373,23373,23341,21293,19180,21293,21293,23309,21260,21228,21293,17067,19147,17100,19213,17100,19180,19180,21261,17100,17035,17035,21228,19212,19180,19148,19180,19180,17035,17100,14955,10793,14890,10761,12809,10696,10696,8615,10695,6567,8615,8614,10663,8582,8582,10663,12775,12808,12776,16904,14888,17001,16937,14856,14888,12776,14856,14856,12776,12743,8550,8517,8582,8550,6404,4356,6372,6372,6372,6372,6372,4323,6372,4324,6404,6404,6372,6436,8485,8516,10565,8517,8517,8517,8517,10565,8517,10565,10630,10662,8582,14889,12678,19245,23308,61341,65502,65469,50448,27142,25030,25030,25030,25030,25029,25029,25062,25127,27110,27110,14660,18885,22949,12613,22949,14628,18853,22916,12581,22949,14628,16741,20836,10532,22949,14596,16741,20804,10500,20868,14595,16773,20804,10500,22916,12515,16741,18756,10660,8578,6498,6498,6534,48663,42158,12611,38},
{61375,61375,61375,61343,61343,61343,59295,59295,57215,57215,57215,57215,59263,59295,61343,61343,61375,61375,61375,61375,63423,61375,63455,63455,63455,63455,63423,63455,63455,61375,59295,55135,50975,48895,48895,50943,53055,55135,57183,57215,59295,59295,59295,59295,59263,59295,57214,55069,53021,52989,52989,52989,55037,52956,52956,52956,52956,52956,50876,50876,50876,52924,52989,53021,55102,55102,55134,57182,57214,57214,59262,59294,59262,57182,46682,42391,46616,48664,50810,48697,36149,17100,19181,21326,25486,27567,27567,27599,27599,27599,23405,27534,25454,25486,23406,25454,23406,21325,19180,21293,23373,21293,23341,21325,23405,21293,19180,21293,21325,21325,23341,21326,21293,21261,17067,17132,21261,17132,19212,21261,19213,19213,17067,19180,17067,12874,14922,10729,12808,10696,10696,8615,10663,8582,8615,8614,10663,8582,8550,12775,12775,14856,12743,14824,16969,19049,16969,14888,14888,14888,14856,14824,14856,14823,12743,10630,14856,12743,8484,6404,4324,6372,6372,4324,6372,4323,6372,4356,10694,6372,6404,8484,8484,8484,10533,8517,8485,8517,8516,8517,8517,10565,8516,8517,8517,19049,12743,25486,16969,57181,65501,65468,41899,25062,25030,25029,27078,27110,25029,25029,25095,27175,27110,27110,14628,23014,22948,14693,25029,14628,23014,22916,14725,22981,14628,20934,20869,14757,22948,14661,23014,20804,16773,22916,12580,20934,18756,14693,22916,12547,20901,18788,14789,10690,6466,8578,4452,44502,29414,8482,14},
{61375,61375,61375,61343,61343,61343,59295,59295,59263,57215,59263,59295,59295,61343,61343,61343,61375,61375,61375,61375,63423,63423,63455,63455,63455,63455,63455,63455,63455,63423,59327,55167,51007,50975,50975,50975,55103,55135,57215,57215,59295,59295,61343,59295,59295,59294,57214,55070,55069,52989,52989,55069,55069,55037,52988,52988,52956,50908,50908,50876,50876,50876,52956,52989,53053,55134,55102,57182,57214,59262,59262,59294,59262,59262,55101,52989,52956,55037,57149,59229,42521,17133,19212,21261,23373,25486,25486,25486,25486,25486,23373,23373,23373,23405,23373,21228,21293,17100,17100,17067,21260,19180,21260,21293,21293,23340,19180,21293,21293,21293,21260,19212,19212,21261,19147,19212,19180,19212,19148,17068,17132,14987,14954,14954,17035,12841,14921,12777,10761,10728,10696,8647,10695,8615,10663,8614,10663,8582,10630,12743,12743,12743,10695,12776,16936,16969,16936,14889,14888,14823,12743,12743,12743,12711,12710,10598,12743,12743,8485,6437,6372,6404,6372,4324,4324,4324,4324,4324,10597,6404,6372,6404,6372,6404,6371,6404,6372,6404,6404,8484,6404,6436,6404,6436,6469,19017,10598,27566,17002,55068,63388,63322,31334,25062,25029,25029,25029,27110,25029,25029,25127,27110,27077,24996,6274,22982,16643,12580,22916,8322,22949,14562,14660,20836,6306,20901,12482,14660,20836,8387,20869,12450,14660,18755,6306,20869,10401,14660,20771,8354,20868,10402,14692,14787,6498,8611,8579,10627,6434,4385,11},
{61375,61375,61375,61343,61343,61343,59295,59295,59263,57215,59263,59295,59295,61343,61343,61375,61375,61375,61375,61375,63423,63455,63455,63455,63455,63455,63455,63455,63455,63455,61375,57247,53087,53055,53023,53055,55103,55135,57215,57215,59295,59295,59295,59295,59295,59295,59263,55069,55069,52989,55069,55069,55069,55069,55036,55036,52956,52956,52956,52956,50876,50876,50876,52957,55070,55134,55134,57182,57182,59262,59262,59294,59262,59262,57182,57149,52988,55036,55068,57149,50907,15053,21260,23341,23374,25518,27567,27567,27567,25454,23373,23373,23406,23405,23373,23406,23406,23373,19212,21293,23341,23373,23341,23373,23373,23374,19212,23374,23373,23373,23341,23373,21325,21325,19179,21292,21261,21293,19213,19245,19213,19180,17067,21293,25486,23373,23373,21293,21292,19179,21195,19179,19147,19114,19114,19146,19114,17034,19114,19114,19114,19146,19114,21227,21227,23275,21227,21227,21162,19082,17001,19049,17002,17001,17001,14889,17002,19114,14888,14856,12775,12775,12743,12808,10695,12743,10663,12743,10695,10695,10662,10695,10663,10695,10662,12711,10662,10662,10663,12775,12743,12807,14855,14856,17001,21194,21163,29614,23308,50841,59130,56917,29190,25029,25029,25029,25029,27077,25029,25030,25160,27142,27110,25029,18887,22982,18821,20934,22981,18919,25030,18886,23046,22949,18951,25029,16838,23046,22949,18886,22949,14693,22982,20869,18919,24997,14725,22981,18788,18886,22917,14724,12578,10562,8514,6530,6466,4321,2241,4386,21},
{61375,61375,61343,59295,61343,59295,59295,59295,59263,57215,57215,59263,59295,61343,61375,61375,61375,61375,61375,61375,63455,63455,63455,63455,63455,63455,63455,63455,63455,63455,61407,59327,57215,55135,53055,53055,55135,55135,57215,57215,59295,59295,61343,59295,61343,59295,59295,55102,55069,55069,55069,55101,55101,55069,55069,55036,52956,52956,52956,52956,52924,50876,50876,52957,55070,55134,55134,55102,57150,57182,59262,59294,59262,59262,57181,55069,52988,52988,55036,57116,55101,17197,19181,19245,21326,25454,25486,23406,23374,23406,21293,21261,21261,23374,21325,21326,21325,23341,19212,21292,21293,21261,21260,21292,21292,21293,19212,21293,21292,23340,21293,21293,21260,19212,19148,19212,19180,19180,17132,19212,19180,19180,17099,29678,33839,33839,33871,35919,35887,21064,6338,27435,8419,21129,10564,16936,14757,16871,18984,12678,23177,10596,25322,10532,25355,10532,25322,10532,23242,12645,23275,14757,21162,16902,19016,21031,14791,21128,12710,25290,12677,25355,12645,25387,12645,27467,12645,25354,14757,21162,16870,21129,21031,19016,21128,16903,23177,14790,25354,14758,27435,14725,27467,14725,27467,14725,27403,16838,23242,38098,57083,46222,25062,25029,25029,25029,27078,25029,25029,25062,25127,27110,27077,20803,12548,29158,10369,14596,14562,10467,22949,8321,20869,16610,10499,22916,6274,20901,14530,12580,22884,6274,22917,14562,16741,20836,8387,22916,12482,18788,20803,10564,12643,8514,10630,12774,12872,14952,10694,8678,36},
{61375,61375,61343,61343,61343,59295,59295,59295,59263,57215,57215,59263,59295,61343,61343,61375,61375,61375,61375,61375,63455,63423,63455,63455,63455,63455,63455,63455,63455,63455,61407,61375,57247,55135,55103,55103,55103,55135,57183,57215,59295,59295,59295,59295,59295,61343,59295,57183,55069,55069,57149,57181,57149,57149,55069,55069,52989,55004,52988,52956,52956,52924,50876,52989,55102,55102,55102,55102,57150,57181,57182,59294,59262,59262,57182,55101,53021,52988,52956,55036,57149,21392,19181,21293,23406,25486,23406,25518,25551,25486,25519,23373,25519,25486,25486,23439,23406,23406,21293,23340,21325,21293,23373,23373,23373,23373,21228,23373,23373,23373,21325,23374,23341,21325,19147,21261,21261,19245,19212,19245,21261,21293,19180,33807,35887,35920,38000,35951,35919,25355,16936,29581,16936,25387,19016,23307,23177,21162,25290,19081,27435,19017,27467,16936,27500,16936,27468,16936,27468,18984,25420,21097,25355,23209,21194,25322,19049,25355,19017,27468,16936,29548,16904,27500,16904,27500,16935,25388,18984,23275,19048,21194,23209,19082,25322,17001,25322,16904,27468,14856,29548,14855,29548,16903,25452,18984,25420,21096,23307,33774,44437,35594,25062,24997,24997,25029,27110,25029,25029,25127,25094,27110,27142,22982,23080,27142,14758,23014,20934,23047,25030,18952,25030,18854,20966,22949,16839,25029,16773,22982,20868,16838,25029,16774,22982,20934,18951,25029,16838,22949,20869,18886,20932,10628,31859,40277,38131,42357,44470,42259,145},
{61375,61375,61375,61343,59295,59295,59295,59295,59263,57215,57215,57215,59263,61343,61375,61375,61375,61375,61375,61375,61407,63455,63455,63455,63455,63455,63455,63455,63455,63455,63455,61375,59295,57215,55135,55135,55135,55135,57183,57215,59295,59295,61343,61343,59295,61343,61375,57215,55070,55069,57149,59229,57181,57149,57149,55069,55069,52988,52988,52956,52956,52956,52956,53021,53021,55101,55102,55102,55102,57181,59262,61310,59294,59262,61342,57214,55069,52988,52956,55036,57116,50908,48827,48795,48763,44601,42521,40408,40409,44569,44569,42456,42456,42456,44569,44568,31826,10662,14824,12710,12711,10630,12743,10662,12743,12711,12743,14823,12743,14824,12743,14856,14791,12775,14824,12743,14856,12775,14888,14856,14824,16936,14856,31694,35919,35920,35920,35919,35887,23210,8550,27501,10630,23242,12710,19049,16903,14823,19016,10630,23242,8517,23242,6437,25388,8452,25354,8484,21194,10564,21129,12677,16904,14823,12710,19016,8485,23209,6372,25322,4291,27403,4259,25355,6306,23210,8419,19049,10564,14855,14758,12711,18950,8517,23177,6404,23241,4291,27403,4258,25355,6338,25355,8419,21194,12612,17001,16838,14888,27370,46485,29254,25062,25029,24997,25029,27109,25029,27078,25128,27110,27110,27109,25061,27109,22948,8386,25029,12482,18821,20836,10467,24997,10402,20901,18723,12547,24964,8354,22949,16642,14628,22884,8354,22948,14595,18820,22851,16772,24964,10401,20900,24996,23275,38132,40276,35888,35888,40211,37707,95},
{61375,61375,61375,61343,59295,59295,59295,59295,59263,57215,57215,57215,59263,61343,61375,61375,61375,61375,61407,61375,61375,63455,63455,63455,63455,63455,63455,63455,63455,63455,63455,61375,59327,57215,55135,55135,55135,55135,57183,57215,59295,59295,61343,61343,59295,59295,61343,57214,57150,57117,57181,59229,59229,57149,57149,57149,55069,55036,52988,52956,52956,52956,52956,52956,50876,52988,55069,55069,55101,57182,59230,59262,59262,59262,59262,59262,55037,52956,52956,55068,57149,59229,59229,57149,57116,52956,50843,48795,50875,55036,55036,52955,52923,52955,57116,59229,33873,4356,8452,6372,4292,4260,4291,4291,6340,4292,4292,4292,4291,4292,4259,4292,4259,4259,4291,4259,4292,4259,4291,2211,4259,4291,6339,29581,35920,35920,35920,35887,35887,23177,10532,27468,10564,21129,12677,19048,18983,16903,21096,14790,25322,12677,23274,12677,27435,12677,25355,12677,23274,14790,23242,18983,21129,21096,16968,25289,16903,25322,14823,27435,14790,29516,14790,27468,14822,25387,18951,23274,21063,21161,23177,19049,25289,16904,27435,16871,27435,14823,29548,16903,29548,16870,27467,18951,23306,21063,23210,23176,19049,25289,33742,27109,25030,25029,25030,27110,25062,25061,27110,25160,27142,27142,27110,25062,25094,18853,14758,25029,12612,23014,18788,16805,25029,12645,22981,14660,18886,20868,10565,22981,12548,20934,20837,12645,22981,12580,23014,18789,16806,22949,12613,22981,25062,31728,40147,40212,35888,23374,31793,37838,86},
{61375,61375,61343,61343,61343,59295,59295,59295,59295,57215,57215,59263,59263,61343,61375,61375,61375,61375,61375,61375,63423,63455,63455,63455,63455,63455,63455,63455,63455,63455,63455,61375,59295,57215,55103,57183,57215,57215,57215,57215,59295,61343,59295,59295,59295,57214,59262,57182,57150,57150,59230,59230,59229,59229,57149,57149,55068,55036,55004,52956,52956,52956,52956,50876,50844,50876,52989,52989,55069,57150,57181,59262,59262,57214,59262,59262,52989,52956,55036,55068,59229,59261,59229,57149,55068,52955,50875,50843,55004,57148,55036,52956,52923,52923,57116,59196,31793,6404,10533,6372,4324,4260,4292,4259,6340,4259,4292,4292,4292,4292,2211,4292,4259,4291,4259,4259,4291,2179,4259,4259,4259,4259,4324,29613,38000,38131,40179,33806,35919,27435,16968,29613,19016,25387,19081,23307,23242,21162,25322,19082,27436,16969,27436,16968,29581,16936,27468,16936,25388,19049,25387,21161,21227,23242,19081,25355,16969,27436,16937,29549,16904,27533,14855,27500,16936,25420,18984,21226,21129,19114,23210,17001,25322,14856,27435,14824,27467,12775,29580,14823,27500,16903,25420,19016,21259,21161,21194,25322,19113,27402,23242,29451,27305,29483,25225,29385,29451,31499,33709,29515,27370,31660,33742,27305,21163,33906,33839,18981,16965,16868,16934,16901,18981,19014,14692,14758,19017,42324,46483,25127,21162,40048,33482,23014,18952,21064,23275,42193,37967,40081,31629,42194,42194,38066,42325,40180,35953,40212,44406,42227,137},
{61375,61375,61375,61343,59295,59295,59295,59295,59263,57215,57215,59263,59295,61343,61375,61375,61375,61375,61375,61375,63423,63423,63455,63455,63455,63455,63455,63455,63455,63455,63455,63455,61375,57215,55135,55103,57215,57215,57215,57215,59295,61343,59295,59295,61343,57182,55069,57150,57182,59230,59262,59230,59230,59229,59197,57149,55068,55036,52956,52956,52956,52956,52956,50876,50876,50844,52924,52957,55069,55102,57182,59262,59262,57214,59262,57182,55069,52956,52988,55068,59197,59261,59261,57149,55068,52956,52923,50875,55036,57149,55068,52956,52923,52923,55036,57116,23341,6405,8485,6372,4291,4259,4260,2211,4260,2211,4292,4292,2211,4292,2179,4292,2211,2211,4292,2179,4260,2211,4259,4259,2211,2243,4324,29581,35984,44406,36018,33806,35919,23209,6371,27436,8484,19016,12678,16903,14823,10629,19016,8517,23242,6371,23210,6339,25355,6339,23242,6371,21097,10565,16936,14757,12710,18951,8516,21129,6339,25290,4258,25355,4226,25355,4258,21194,8419,16968,12612,12774,16806,10629,18951,8452,23176,6307,25322,6339,27403,6339,25355,8386,23306,10499,19049,14725,14823,18983,10629,23144,8484,25257,6371,27435,8419,25355,8419,8419,8451,10532,25355,23176,6402,25418,33774,27337,10793,36051,27499,14852,12772,12739,12772,12772,10692,12772,6370,12710,25421,48630,52659,16806,42291,46420,31305,25161,10500,8550,25485,46354,42128,44241,27436,44307,44340,38066,46485,40212,38033,36052,42325,44438,167},
{61375,61375,61375,61343,59295,59295,59295,59295,59263,57215,57215,59263,59295,61343,61375,61375,61375,61375,61375,61375,63423,63423,63455,63455,63455,63487,63455,63455,63487,63455,63455,63455,61375,59263,55135,55135,57183,57215,57215,57215,59295,59295,59295,59295,59327,57215,55069,57117,59230,59230,59262,59230,59229,59229,57149,57117,55036,55036,52988,52988,55036,55036,52956,50908,50876,50843,50876,52956,55069,57149,57182,57182,57182,57214,57214,57182,55069,52924,52956,55036,57148,61341,61309,59229,55068,52956,52923,52924,55036,57149,57116,55036,52956,52923,55035,50810,19115,4259,4259,2212,2211,2244,2244,4292,2212,2243,2211,4259,2211,4259,2179,4292,2243,2211,4292,2179,4292,2211,4260,17034,17067,14888,31792,33839,35952,40179,40179,33774,35919,25290,12709,29580,14790,23242,16935,21129,21096,19016,23241,16903,27435,16871,27435,16839,29580,14790,25387,16903,25354,19016,23242,21128,21130,25289,19016,27435,16936,27499,18952,29580,16903,29548,16903,27467,19015,25355,23177,21162,25289,19049,27370,16968,27467,16935,29548,16903,29547,16903,27500,21031,27500,21096,23306,23209,21161,25322,21097,29483,19016,29483,16903,29548,18951,27467,18918,18951,21064,23144,29581,27369,10595,21161,31693,29482,8615,36051,42258,16997,8610,6401,10691,10595,10629,25353,8451,10662,40210,52725,54772,31596,44338,42162,29452,35823,21032,33872,42291,46419,42159,42193,35888,40178,42259,35888,27469,40212,40146,33840,38033,42390,156},
{61375,61375,61343,61343,59295,59263,59295,59295,59263,57215,57215,59295,61343,61343,61375,61375,61375,61375,61375,61375,63423,63423,63455,63455,63455,63455,65503,63455,63487,63455,63455,63455,61375,57247,55135,55135,57183,59263,57215,57215,59295,61343,59295,61343,59327,57182,55102,57149,57182,59230,59230,59230,59229,59229,57149,55069,55068,55036,55036,55068,55036,55036,52956,50908,50876,50876,50844,52924,55069,55069,57149,57182,57182,57214,57182,59262,55101,52956,52956,55036,57116,61309,61310,59229,57116,55036,52956,55004,55036,57149,57116,55036,52956,52923,55004,55036,27600,4324,4292,2212,2211,2212,2244,2244,2212,4260,2211,2211,2211,2211,2212,4292,4259,2179,4259,4227,2211,2211,2211,14822,25290,21162,37933,33741,35920,38066,40147,31694,35919,27435,16936,29581,16936,23307,17001,21194,23210,19049,25322,16969,27436,14889,27468,14856,29581,14856,25420,16904,23307,19049,21194,21162,19081,25322,14888,27435,14823,27468,14824,27500,14791,25420,14823,23307,16935,21194,21096,17001,23242,14856,25322,12743,27435,10662,27468,12710,25388,12710,25387,16871,21195,18984,19082,21128,16969,25290,14856,27435,14856,27468,14855,29548,16936,27500,19016,16936,16969,16968,23340,31595,10628,19113,31693,31563,8517,29678,48531,16964,8611,4322,10724,8547,17001,21031,4193,14889,42193,42095,37868,48531,48499,44438,44209,35821,21195,44371,46451,48433,33675,42192,40114,40211,38034,40081,23276,36051,42260,27501,23373,31890,159},
{61375,61343,61343,61343,59295,59263,59295,59295,59263,57215,57215,59263,59295,61343,61375,63423,61375,61375,61375,63423,63423,63455,63455,63455,63455,63455,63455,65503,63455,63455,63455,63455,61375,57215,57183,57215,57215,59263,57215,57215,59295,61343,59295,61343,59295,57182,57150,57181,59230,59262,59230,59230,59229,59229,57149,55101,55068,55036,55036,55069,55004,52956,50875,50876,52924,50876,50876,50876,55069,55101,57181,59230,57182,57182,57214,59262,53021,52956,52924,55036,57116,61277,61309,59229,57116,55036,52923,52923,55036,57149,57116,55036,52955,52955,55036,57116,33939,25553,23472,23472,21392,21392,21392,21392,21327,21359,19279,19247,17166,19214,19214,19246,17133,17101,17101,17100,15020,15020,14988,31792,27501,29647,25322,33742,35919,36018,33807,31726,35919,21129,4226,25355,6371,16936,10565,14758,16871,8517,18984,6339,23242,4258,23210,4258,25355,6306,21162,8451,16936,12645,14791,16838,10597,21096,6371,23242,6339,25354,6338,27435,6338,23242,10499,19017,14693,14823,18918,12646,21064,8516,25289,6371,25322,6370,27435,8451,25323,10564,21161,14692,16968,16837,14790,21063,8517,23209,8419,25321,6338,25322,6338,23274,8419,21129,12612,6338,8419,8451,16968,31595,8483,12774,31693,33709,8484,14855,25354,29580,21224,8611,10659,8515,12676,12579,2178,19179,48566,29483,8451,44371,46321,37871,46289,44306,48499,44371,48530,33511,6274,27533,42161,42161,35920,38066,29582,38131,42324,35987,42325,44438,167},
{61375,61375,61375,59295,59295,59295,59263,59295,59263,57215,57215,59263,59295,61343,61375,61375,61375,61375,61375,61375,61407,63423,63455,63455,63455,63455,63455,65503,63455,63455,63455,63455,61375,59263,59263,57215,57215,57215,59295,57247,59295,59295,59295,59295,59295,57182,57182,57149,59230,59230,59230,57181,57149,59197,57149,57117,55069,55036,55036,55069,52956,50876,50844,52924,50908,50876,50876,52988,55101,55102,57149,55069,55069,55101,57149,57182,55069,52956,50876,52956,55036,59229,59261,57149,55068,55036,52955,52956,55004,57149,57148,55036,52955,52955,55036,55068,35921,27468,27500,27500,29548,29581,29581,31662,29614,31662,29614,31694,31694,31726,33775,33775,33775,31727,33807,33807,33839,35888,35888,52725,50644,42291,54870,35919,35919,35921,40212,31695,35920,25355,16903,29581,16935,25322,21064,23210,23242,19049,25322,19016,27435,16936,27468,16936,29581,18984,25419,19048,25355,23209,23242,25322,21161,27467,19016,29548,19016,29548,18984,29580,18984,27468,21096,25355,23209,21195,25290,19081,25354,16968,27467,16936,29548,16903,29548,18984,27468,19016,25355,23209,23274,25289,21162,27402,19049,29547,19016,31628,21064,29548,19016,27500,21096,25388,23208,19016,21129,23209,27467,33773,14821,19081,29581,33773,10531,12741,21194,33774,35918,14820,8611,6466,4258,4258,4356,23405,31564,31629,8453,38098,48531,39626,42161,42225,48467,44371,50579,31464,4259,21226,27467,29614,29679,33840,35954,40147,42292,44405,44405,46518,175},
{61375,61375,61343,59295,59295,59295,59295,59295,59263,57215,59263,59263,59295,61343,61375,61375,61375,61375,61375,61375,61375,63455,63455,63455,63455,63455,63455,63455,63455,63455,63455,61407,61375,59295,59295,59295,59295,57215,59295,59295,59295,61343,59295,59295,61343,59295,57182,57150,57150,59230,59230,59230,59229,59197,57149,57117,55068,55037,55036,55036,52956,50876,52924,52924,52956,50908,50908,53021,57182,57182,55037,50908,52956,55037,57149,57182,55069,52956,50876,52924,55036,59197,57148,57116,55036,52956,52956,52956,55004,59196,59197,57116,52956,52955,55036,55068,33971,10598,10599,10630,10631,10598,8550,10631,8550,10598,8550,8550,8518,10598,8550,8550,10598,10598,10598,8518,8550,8550,10630,38000,42160,33708,46287,37967,35919,38099,40179,33807,37967,25387,14824,29581,14856,23275,19017,19082,21162,14888,23274,12776,25387,12743,25420,10695,27500,12743,23307,14856,21162,19017,16969,21129,12775,25355,10630,27435,10598,25355,10598,25387,12678,21226,14823,19049,18983,14856,23177,10630,23242,8549,25355,8517,25387,8484,25355,12645,21162,14758,17001,18951,12808,23176,10695,25322,10630,27435,10597,27468,12710,25387,14758,23307,19016,21195,21128,12743,14823,14823,16936,33806,14853,19113,27532,31661,12708,10629,21260,31694,44306,25385,10660,8547,12806,6371,21325,31725,8484,40210,42259,48565,48531,46290,48467,42225,48500,48531,42060,25256,33839,25257,44306,44339,44274,35986,42259,40146,38131,44372,29581,31727,132},
{61375,61375,61375,61343,59295,59295,59295,59295,59295,59263,59263,57215,59295,61343,61343,61375,61375,61375,61375,61375,63423,63423,63455,63455,63455,63455,63487,63487,63455,63455,63423,61375,61375,59295,59263,59295,59295,59295,59295,59295,59295,61343,59295,59295,61343,59295,57150,57182,59230,59230,59230,59262,59262,59261,57181,57117,57116,55036,52988,55036,52988,52956,52924,52956,52956,52956,52989,55102,57182,55102,52957,52956,52956,55037,55069,55036,52956,52956,50876,52923,55036,55068,55036,55068,55036,52956,52956,52956,55036,57149,57148,57116,55036,55004,55068,57116,31727,21227,21227,21227,21227,21228,21260,19180,19148,21196,21196,19180,21195,21196,19115,19147,19147,19147,19180,19148,19148,19180,33971,44339,48500,44405,42226,35887,35919,35953,38066,33807,35919,21097,6274,25355,8452,18984,12645,14790,16903,10564,21096,8451,25322,8419,23242,8419,25355,10499,21129,12677,16936,16870,14823,21095,10597,25322,10532,27435,10532,27467,10531,25355,12644,21162,18919,19016,18983,14823,23209,12677,25322,12645,25354,12612,27435,12612,25355,14757,21162,18918,19048,23143,16871,25257,12709,27402,10564,27435,10532,27435,12579,23242,12612,19081,16837,16903,21031,6371,10500,12612,14790,35854,10531,14920,33807,31628,16903,14887,33839,42226,50547,42258,31660,10627,12773,10565,38097,35722,19114,48532,52692,50612,50546,46421,52660,44339,48532,48432,18851,10596,35853,8451,42258,44307,40081,38066,40081,40146,36018,44405,23340,23371,100},
{61375,61375,61375,61343,59295,59295,59295,59295,59263,59263,57215,59263,59263,59295,61343,61375,61375,61375,61375,61375,61375,63455,63455,63455,63455,63455,63455,63455,63455,63455,61407,61375,61375,59295,59295,59295,59295,59295,59295,59295,59295,61343,61343,59295,59327,57215,57150,59230,59262,59262,59262,59230,59229,59229,59229,57149,55069,55036,52988,52956,52988,52956,52924,52924,52956,52956,53021,55102,57150,55069,52957,52956,50876,52924,50876,50908,52956,52956,52924,50876,52956,55036,55036,55068,55036,52956,52956,52956,55036,59229,57149,57116,55036,52956,55036,52955,25487,10598,10630,10598,8550,8550,10631,10598,8550,8550,8550,8550,8517,10598,8518,8518,8518,8518,10630,8518,8550,10598,35953,42260,42324,54937,52758,35919,37968,40212,42292,33774,38000,27435,18984,29580,19016,25355,21129,23275,23274,21161,27468,19049,29580,16968,27500,16968,29580,19016,25420,21129,23307,23274,21194,25354,19081,29548,19016,29580,19016,29580,19016,27500,19048,23339,23209,21226,25322,21130,27403,16969,27467,16936,27500,16936,27468,16903,25388,19016,23307,23177,21194,25322,19081,27435,16969,29548,16936,29581,19016,27500,19048,25420,21128,23307,25289,23242,27435,19016,21129,23241,25355,33806,23209,25355,40113,31628,19150,21227,31661,40113,46322,44274,37868,14854,27598,25322,50578,27207,29646,54773,52660,46386,48466,44211,52725,46452,48531,42060,10562,10661,23241,6371,42291,44339,46517,38066,40146,35822,33873,42325,42390,38098,125},
{61375,61375,61375,61343,59295,59295,59295,59295,59263,59263,57215,59263,59263,59295,61343,61375,61375,61375,61375,61375,61375,63455,63455,63455,63455,63455,63455,63455,63455,63455,63423,61375,61375,59295,59327,59295,59327,59295,59295,59295,59295,61343,61375,59295,59295,57214,57182,59230,59230,59262,59262,59262,59229,59229,59229,57149,55069,55036,52988,52956,52956,52924,50876,52924,52924,52956,53021,55102,55101,52989,52988,52956,50876,50876,50876,50908,52956,52956,50876,50875,52924,52956,55036,55068,55036,55036,52956,52956,57116,61309,59229,57149,55068,52956,52956,48763,23342,21228,21228,19179,19147,19147,19180,19180,19148,19180,17132,19180,19180,19180,17100,17100,17132,19148,19148,19116,19148,19148,42357,48664,48728,52889,48630,38000,35920,40179,35986,33774,38000,23275,8517,27436,12711,19049,14856,14888,19017,10662,25323,8550,27468,6469,25356,8484,23307,10565,19081,14759,14855,16936,10662,21129,8452,27403,6372,25387,6339,23274,6371,21162,10564,16936,16870,10662,21032,8484,23209,4291,25322,4291,25322,6339,21194,8419,19049,12645,14791,14758,10597,21064,6404,25322,4291,27434,6371,25387,8452,21194,12645,16968,16838,14855,21064,10662,25289,8516,8517,10565,10597,29645,21064,14888,38000,27433,14919,27533,35855,44372,44339,42128,18948,12741,35887,14756,48531,40048,38097,52724,50547,33709,48499,39724,37935,46484,50611,48432,12674,16968,37966,10596,44371,44339,50743,38066,37968,31629,29614,42358,44470,42324,151},
{61375,61375,61375,61343,61343,61343,59295,59295,59263,59263,57215,59263,59263,59295,61343,61375,61375,61375,61375,61375,63455,63455,63455,63455,63455,63455,63455,65503,63455,63455,63423,61375,61375,61375,61375,61375,61375,59295,59295,61343,61343,61375,61375,59295,59295,59295,57215,57182,59230,59230,59262,59262,59261,59261,59229,57149,55101,55036,52988,52956,52956,52924,50876,50876,50876,52956,55069,55102,55069,52989,52989,52924,50876,50876,52956,52989,52989,52956,52956,50876,50844,52924,55036,55068,55068,55036,52956,52956,55036,61309,61309,59229,55068,55036,52924,48763,27729,10598,10598,10630,10598,8550,10630,10598,10598,10630,8550,10630,10598,10663,8550,8550,10598,10565,10630,10597,10630,10598,38098,50776,50743,46517,42258,38000,35919,40113,40146,33807,37967,23209,10532,27435,12677,21097,16871,18983,21064,14758,25257,14726,29516,12678,27467,14725,27436,16838,23210,16903,19048,21129,18984,25322,14822,29516,14790,29548,16838,29516,16870,25355,19015,23242,23209,19048,25321,16935,27468,14823,29515,14822,27468,16870,25355,18950,23242,23144,21129,23176,16968,27402,16903,29516,16903,29548,16870,27468,18950,25355,18983,21097,21063,16935,25289,14790,27402,12645,14757,16838,18951,31693,23111,14757,37934,25256,16967,23307,40212,48533,40113,35789,12707,8548,19015,10532,46484,46385,54837,52692,46387,33742,48531,42031,40146,48499,50612,50611,27368,10693,44272,14790,42323,46452,52823,38033,38065,31662,27469,40245,35887,27501,129},
{61375,61375,61375,61343,59295,59295,59295,59295,59263,59263,57215,57215,59263,59295,61343,61375,61375,61375,61375,61375,61375,63455,63455,63455,63455,63455,63455,63455,63455,63455,61375,61375,61375,61375,61375,61375,61375,59295,59295,59295,61343,61375,61375,59295,59262,59295,61343,57182,57150,57150,59230,59262,59229,59230,59197,57149,57149,55068,55036,52956,52924,50875,50875,52924,50908,52989,55069,55101,55101,52989,52956,50876,50876,52923,52956,52956,55037,52989,52956,50876,50844,50875,52956,55068,55068,52988,52956,52956,55036,59229,59229,57149,57116,55036,50875,48795,38328,19245,19180,19148,19148,19148,19181,19148,19148,19180,19148,19148,17100,19180,17067,17067,17067,17035,19115,17035,15019,17035,21163,23275,23242,29548,31629,38000,38000,38032,38032,38000,38000,27436,16936,29581,19049,23275,21162,21194,23274,19081,27467,19049,29581,16969,27533,16968,27501,19049,25356,21162,21162,23275,19082,25387,16969,29549,16936,29581,16936,27500,18985,25387,19081,21227,23275,19049,25355,16937,29548,14856,29548,14856,27468,16936,23340,16936,21194,21129,17001,23209,14888,27467,12808,29580,14855,27500,16903,27500,19048,23340,23177,19146,25289,19081,27467,19049,29580,16936,16969,17001,21161,31694,27435,16871,21129,16934,21194,29581,42290,33741,42226,35822,12675,10628,25420,16870,44273,27337,46418,50644,48532,35855,48499,50579,48532,48531,50612,46253,18980,23208,29417,18982,27400,27336,37999,35953,35920,31694,23243,38165,35953,21293,110},
{61375,61375,61375,59295,59295,59295,59263,59295,59263,57215,57183,57183,59263,59295,61343,61375,61375,61343,61375,61375,61375,63455,63455,63455,63455,63455,63455,63455,63455,63455,61375,61375,61375,61375,61375,61375,61375,59295,59295,59295,61343,61375,61375,59263,59262,59295,59295,59295,55134,57150,57181,59230,59229,59229,59229,57149,57149,57117,55036,52956,52956,50876,50876,52956,52989,55069,55069,55069,55069,52957,50876,50876,50875,50875,52956,52956,55037,55037,52989,52956,50876,50876,52956,55036,55036,52956,52955,52923,55036,59197,57117,55068,57116,55036,50876,52956,55036,12808,10598,10565,10566,10598,10631,10598,10598,10630,10598,12710,10630,12710,10630,10630,10663,10630,12743,10630,10662,12743,10663,25388,25453,27468,29549,33742,35951,35920,37968,38000,38000,21129,4258,25323,8452,16871,12710,10597,16903,6371,21129,4258,25355,4226,23242,4258,23242,8452,14823,12677,10597,16903,8484,23242,6306,27403,6307,25355,6306,21129,8419,16936,14725,12710,18984,8484,23177,6339,27403,6339,25322,8419,23242,10531,19048,12612,14791,18918,10565,21063,6371,27402,6339,27435,6306,25355,8419,21129,14692,16903,18918,8549,21064,6404,27370,6339,27435,6338,4258,6339,8451,25355,25257,6469,25420,23243,27501,33708,29449,18951,35920,31595,12675,6434,14887,23143,35919,25258,33806,50645,50612,48631,40015,50612,48532,50579,50611,33542,10594,14756,21062,14787,10627,10595,6370,23340,38098,33905,31661,36051,42357,42357,164},
{61375,61375,61375,61343,59295,59295,59295,59295,59295,57215,57183,57183,59263,59295,61343,61343,61375,61375,61375,61375,63423,63455,63455,63455,63455,63455,63455,63455,63455,61407,61375,61375,61375,61375,61375,61375,61375,59327,59295,59295,61343,61375,59327,57214,57182,59262,59295,59295,59295,57214,57150,59229,59261,59230,59229,57149,57149,55068,55068,55036,52956,52956,50876,52956,55069,55101,57117,55069,55037,52956,50876,50876,50844,50876,52956,52956,52956,55037,55037,52956,52956,50876,52923,52956,55036,55036,52924,52923,57149,57181,57149,57117,57116,52988,50876,52956,55036,23406,19148,19115,19115,19115,19148,15020,15051,17067,15019,17067,15019,17035,17002,14987,14922,14922,14922,12842,12874,10794,10762,16969,23210,29614,27469,31726,37967,38000,38000,38000,37999,27403,16903,27500,18983,23274,23177,21161,25290,19016,27467,18984,29516,16936,29581,18984,27468,21096,23275,23210,21194,27370,21129,29548,19016,31628,18984,29581,21064,27468,21096,23307,23209,21194,27435,21129,27467,19048,29580,18984,27500,16936,27435,21064,23275,23177,21162,25322,17001,27402,16968,29580,18984,29613,19016,29580,21096,25387,25257,23242,27402,19081,27434,19016,29548,18983,29580,18951,21063,21096,23176,29581,31595,21096,25322,31529,35887,31433,12678,17002,38000,40145,16900,8547,12904,29515,25453,40081,40113,50613,50612,52791,42160,50612,48532,50612,50611,25253,8514,10692,10626,4353,6466,8514,4289,15019,38099,35953,40048,33971,44438,46518,177},
{61375,61375,61375,61343,61343,59295,59295,59295,59295,57215,57215,57183,59263,59295,61343,61343,61375,61375,61375,61375,63423,63455,63455,63455,63455,63455,65503,63455,63455,63423,61375,61375,61375,61375,61375,61375,61375,59327,59295,59295,61343,61375,59327,57182,55102,57182,59294,59295,61343,59295,55102,57149,59197,59197,57149,57117,57117,55069,55068,55036,52956,52956,52924,52956,55069,55101,55101,55101,52989,52956,50876,50876,52924,50875,50876,50908,52989,55069,55037,52956,52924,52956,52924,52956,52956,55004,52923,52924,59229,63422,59262,57116,55036,52956,50875,50876,52956,23342,12776,12743,14824,14856,14889,12808,14888,14856,14856,14889,14856,16937,16936,16937,16937,16937,19050,19017,17002,19050,17002,19082,19050,27436,23275,35919,35887,35919,38000,38000,35952,27468,14889,27533,17002,21259,23243,19115,23275,14921,27501,12808,27501,12808,27501,12808,23308,14921,19147,21130,14921,23242,12808,25420,10695,29549,10663,25420,12743,23243,14823,17033,19049,14856,25323,10695,27435,8550,27500,10630,25355,10630,21162,14823,17001,18983,12743,23177,8550,23242,8518,27468,8517,27436,10630,23307,16871,17034,21097,14888,25322,12808,25387,12743,27500,14856,27500,16936,14856,14856,14888,23307,31661,16968,19080,23306,37902,35856,19083,44338,37967,48564,27335,8547,17066,31563,29580,52693,48565,50612,50612,48532,40145,48564,50612,50612,52659,25253,6466,10691,8546,2242,8547,4355,4388,27762,38099,31694,40048,35986,44437,29549,113},
{61375,61375,61375,61343,61343,59295,59295,59295,59263,57215,57183,57183,59263,59295,61343,61343,61375,61375,61375,61375,63455,63455,63455,63455,63455,63455,63455,63455,63455,63455,63423,61375,61375,61375,61375,63423,61375,61375,61343,59327,61343,61375,59295,57183,57182,57182,61343,59295,61343,59295,57215,55102,55102,57117,55069,57117,57149,57117,55069,55036,52956,52956,52956,52989,55069,55069,57149,55101,52989,52956,50876,52924,52924,50876,50876,52956,52989,55069,55036,52956,50876,52956,52956,52956,52956,55004,50875,52924,59229,63390,57117,55036,52956,52923,50875,50876,52956,31826,21195,21163,21195,21195,21195,21163,21228,19147,19147,19147,19147,19148,19115,21163,19147,21163,21163,21163,19115,21163,19115,19147,19115,21195,23242,33774,35919,37967,38032,38000,37968,33773,29547,33806,31628,33773,33741,33773,35822,33741,35886,31693,35854,31660,35886,33741,35886,33773,35854,35854,33806,35887,33806,35887,33773,35887,33773,35887,33774,35887,33806,33806,35854,33806,35886,33806,35887,33806,35919,33774,35886,35854,35886,33806,33806,35854,33806,35854,33806,35854,33774,35886,35854,35886,35887,35887,35886,35887,35887,35919,37999,37999,35919,37967,37967,37967,35886,35919,37967,37967,37967,35918,35918,35853,21062,27531,25256,35855,25388,35789,27435,44338,35656,10692,23405,40014,27434,50645,50613,50612,50612,48499,44339,48532,50612,52692,50612,37900,10659,8610,8546,4322,6402,14921,6633,29679,40244,31597,29549,38034,44438,27501,99},
{61375,61375,61375,61343,59295,59295,59295,59295,59263,57215,57183,57183,57215,59263,61343,61343,61343,61375,61375,61375,63455,63455,63455,63455,63455,63455,63455,63455,63455,63455,63423,61407,61375,61375,61375,63423,63423,61375,61343,59327,61375,61343,59262,57214,57182,57150,59262,59295,59295,59295,59295,57214,55134,55102,55069,55069,57117,55068,55069,55036,55036,52956,52956,52957,55069,55101,57181,57150,52989,50908,52924,52956,52956,50908,52956,55036,57149,55101,55069,52988,52924,52924,52956,52956,52956,52956,50875,52924,59197,59197,55036,52956,52923,50875,50875,50876,52956,33971,19115,19115,19115,19115,21163,19147,19148,19115,21195,19147,19115,19115,17035,19115,17067,19083,21163,19083,19115,19083,17067,19083,17034,23243,25388,35919,37999,38000,38032,38032,38000,27435,19048,31661,18984,27468,21129,25355,25290,21161,27435,19049,29580,16936,31629,18951,29580,18983,27435,21129,25322,25322,21129,27435,19048,29548,18951,31628,18983,29580,19015,27467,21096,23242,25289,21161,27435,21064,29515,16935,29547,18951,27500,18951,25387,21064,25290,23176,21129,25289,19016,29515,19016,31628,21064,31628,23144,29548,25289,29515,29450,27435,29515,25290,31627,25257,31660,27337,29483,31660,33741,37966,35853,29484,25322,23271,27401,23180,29615,42161,23209,44436,45994,16868,23340,40047,14757,40146,52692,50612,52659,50547,48532,50580,50579,50546,50612,50579,25254,10658,6465,2273,4322,8549,25552,14791,36018,31662,29548,29647,44406,40213,155},
{61375,61375,61375,61375,59295,59295,59295,59295,59263,57215,57183,57183,57215,59263,59295,61343,61343,61343,61375,61375,63423,63423,63455,63455,63455,63455,63455,63455,63455,63423,63455,61375,61375,61375,61375,63423,61375,61375,61343,61343,61375,59295,59263,59295,57182,55102,57182,59262,59295,59295,59294,57215,55134,55134,55102,55037,55036,55036,55036,52988,55004,52956,52956,52957,55069,55101,57149,57149,52989,52956,52956,52956,52956,52956,52924,55037,59229,57149,55069,52956,50876,50876,52956,55036,55036,52956,50875,52924,57116,55068,52988,52956,50876,50843,50875,50876,52988,38133,14921,14889,16937,14889,14889,14889,16937,14857,14857,12808,12776,14857,12776,14857,12808,14856,14888,14856,14856,12808,12776,14857,21195,25356,31695,35887,37968,38000,38032,40112,38033,27467,23242,31726,21097,29581,21162,29548,25323,25355,27468,23243,29580,21162,29613,19049,31661,19048,29581,21129,27500,25290,25355,25354,21194,29515,19081,29548,19049,31661,19016,29581,18984,27468,21129,25355,25290,21227,27370,17001,27435,16968,29548,16936,29548,16936,25387,18951,23307,21096,23275,25290,21162,29483,21096,29547,16935,31628,19016,29580,23177,25452,25257,25355,27370,23275,29515,23210,27435,29581,31693,37967,35854,23276,21194,19045,10659,8581,19082,40080,18984,50711,44176,23240,19146,40048,8420,35986,48532,50612,50611,44175,42161,50611,42092,33672,37934,46319,12674,8546,6466,6434,4354,2307,29777,29614,31760,31629,27435,27567,42325,46518,173},
{61375,61375,61375,61343,61343,61343,59295,59295,59263,57183,57183,57183,59263,59263,61343,61343,61343,61375,61375,61375,63423,63423,63455,63455,63455,63455,63455,63455,63455,63455,61407,61375,61375,61375,63423,63423,61375,61375,61343,59295,61343,61375,59295,59295,57183,55070,57182,59262,59295,59262,59295,57215,57183,55102,55070,53021,52957,52988,52957,52989,52989,50941,52957,52989,55069,57149,57149,57149,53021,52956,52956,52956,52956,52956,52956,55036,59229,59230,55069,55037,50876,50876,55004,55068,55068,52988,50875,52955,55068,55068,55036,52956,50843,50843,50843,52924,55036,40343,16970,14922,17034,17067,19082,19082,19115,17002,19083,19050,19114,19082,19050,19115,19082,17002,21163,19082,19115,21131,19083,19147,23308,25388,33807,38000,38032,38033,38032,40113,40081,27403,21130,33742,19016,29580,21129,27500,23242,25355,27435,25323,29548,21194,29581,21161,31629,21129,31661,21130,29613,23242,27500,25323,25388,27435,21195,29548,21194,29613,21162,31693,21129,29581,21129,27501,23242,25420,25290,23275,27370,21194,29483,21130,29548,19049,29548,19017,27468,19016,27500,23177,25420,25289,23307,27402,21162,31563,19048,29580,19016,29580,19015,27500,21096,25419,25289,21129,25355,31628,33773,37934,27370,29581,38129,25319,8578,6434,19114,40080,21129,44304,40049,44404,31824,46452,12710,40243,48565,52692,50579,42095,42193,50546,31624,10593,8482,19012,10626,8546,6466,6401,4322,8679,36019,46518,33808,33775,29549,29712,42325,40146,120},
{61375,61375,61375,61343,61343,61343,59263,59295,59263,57183,57183,57183,57215,59295,61343,61343,61375,61375,61375,61375,63423,63423,63455,63455,63455,63455,63455,63455,63455,63455,61407,63423,61375,61375,63423,63423,61375,61375,61375,61343,61375,61375,59295,59295,57247,55134,57182,59263,59263,57214,59262,57215,55134,55134,55103,53054,53022,50974,50942,50942,50974,53022,55102,55069,57149,55101,57149,57149,55069,52989,52956,52988,52956,52988,52956,55036,57181,59262,57117,55069,52988,50876,52988,55036,55068,52988,50875,52955,55036,55068,55036,52956,50876,50843,50876,52956,52956,48795,36183,36248,34167,34135,34071,31957,25552,25552,25520,23406,23374,23341,23342,23374,23374,21293,21293,21261,21261,21261,21261,21293,23374,23275,31662,35920,38000,40081,40113,40113,40112,23274,19048,31726,16936,29548,19015,27468,21129,25355,23242,23242,27403,19081,29516,16968,29549,16968,29581,16936,29581,19016,29516,21097,23307,23209,21162,25354,21129,29516,19049,29580,18984,31629,18984,29549,19016,27468,21096,25355,23176,23242,25289,21162,27402,19048,29483,16936,27467,16936,29580,21096,29580,23209,27501,25289,25420,27402,23275,27434,23242,29547,19081,31628,21064,29612,21063,23210,29516,31693,35886,33707,23210,33872,52756,31625,10627,6467,21260,31595,21226,40012,38065,50743,46452,50646,35919,44371,50613,52691,50644,48532,46320,33673,27398,10658,6434,10659,8578,6434,4321,6498,2241,6665,38098,50678,31695,31726,31760,33840,42325,42260,103},
{61375,61375,61375,61343,61343,59295,59263,59263,57215,57183,57183,57215,59263,59295,59295,61343,61343,61375,61375,61375,63423,63423,63455,63455,63455,63455,63455,63455,63455,63455,61375,61375,61375,61375,63423,61375,61375,61375,61375,61343,61375,61343,59295,59295,59263,57215,57182,57182,57214,59294,59263,57183,55134,55102,55102,53054,53055,50974,48894,50974,50974,53054,57182,57150,57182,57182,57149,57149,55101,52989,52956,55036,55036,52988,52956,52988,57181,59229,57117,55069,55069,55036,55068,55069,55036,52988,50875,52924,55068,55069,55068,52956,50875,50843,52956,52956,52956,52956,50843,55036,55036,52955,50843,44569,42390,17100,17067,23309,25390,25390,23277,21228,23276,23277,23309,23309,21228,21228,21261,27599,27567,27534,33775,38000,40080,40080,38032,38032,38032,25322,23274,31725,19048,31629,19048,29580,21129,27500,23210,25387,25322,23243,27435,21129,29548,19049,29548,19016,31629,18984,29580,18984,27468,21096,25323,23177,23275,27370,21130,27435,18984,29516,18984,29548,16903,29548,16903,27435,18951,25355,21064,23274,23176,21162,25257,19049,27370,16936,29547,16935,31628,18984,29548,21064,29548,23176,27436,25289,25387,27402,23307,29515,23243,29580,21129,25355,29580,33741,37967,33708,25388,40178,44239,18980,6466,4322,10597,23176,27369,33705,46451,46354,44371,50678,42192,44338,48532,50579,50644,48532,48465,31593,12739,10658,10659,12771,10659,8515,6500,8515,4356,23602,40146,39950,27469,33840,31761,40016,38164,44470,172},
{61375,63423,61375,61343,59295,59295,59263,59263,57215,57183,57183,57183,59263,59295,61343,61343,61343,61343,61375,63423,63455,63455,63423,63455,63455,63455,63455,63455,63455,61407,61375,61375,61375,63423,63423,63423,61375,61375,61375,61375,61375,61343,59295,59263,59263,57215,57214,57183,57182,59294,57214,57182,55134,55103,55134,53054,53054,51006,48894,48894,50974,53022,59230,59262,59230,59262,57181,57149,57149,55037,52956,52988,55036,55004,52924,52989,57181,55069,55069,55069,57116,57117,57116,57116,55068,52988,52923,52923,55068,57149,55068,55004,52923,50875,55036,55036,55004,52956,50843,55004,55036,55004,50843,44569,42390,14955,14987,23308,23341,23341,21261,21261,21229,21228,23341,23341,21229,21196,21228,27535,27535,29647,33840,38032,38032,38032,40048,38000,38032,25322,21194,31694,21162,31661,21129,29613,21162,29613,25322,27500,25355,27436,27436,23275,29516,23242,31596,21130,31661,21129,31628,21129,31661,21097,29548,23209,27468,25290,25388,25354,23275,27435,23242,29516,21129,29548,19016,29515,19016,29548,19016,27500,21096,27435,23176,23307,23209,21227,27402,21194,29515,19082,29548,19016,31660,18983,29580,18984,29548,21064,25387,25257,25323,27401,23241,27435,31628,33741,37967,23144,25289,42159,23239,10659,4322,4322,2210,10660,12643,25351,44339,37902,44305,48565,42095,44240,48467,48499,50580,48499,46385,33738,21191,12771,8578,6531,21291,33872,38195,27631,34003,46616,50613,31596,25421,35986,27502,42194,40180,46551,185},
{61375,61375,61343,61343,59295,61311,59263,59263,57215,57183,57183,57215,59263,59263,61343,61343,61343,61375,61375,63423,63423,63423,63455,63455,63455,63455,63455,63455,63455,63423,63423,61375,63423,63423,63423,63423,61375,61375,61375,61375,61375,61343,59295,59263,59263,57215,59262,57215,57150,55102,57118,55102,53054,53054,53055,53054,53054,50974,48894,48894,50974,53022,57150,57182,57181,59229,57149,55069,55069,55069,52989,55036,55036,52956,52956,55068,57149,55101,55069,57117,57116,57116,57116,55068,55068,52988,50875,52923,55068,57116,57116,52988,52956,50876,52956,55036,55004,55004,50843,52923,55003,52955,50843,46617,42390,12972,12972,23277,23341,23342,23309,21261,21228,21228,23309,23309,21196,21196,21164,21196,23277,27534,31662,38000,38032,38032,38032,38064,40112,21129,21129,31661,16936,29548,16936,29581,18984,29548,19016,27435,23145,25355,25290,23210,25290,21129,27435,19049,29548,19016,29548,18984,29580,18983,29548,21064,27468,21096,25387,21160,25323,25289,23242,27402,21161,27435,19081,27467,19048,29515,19016,29548,19015,27499,19016,27468,21096,25420,25257,25387,25321,25355,27402,23274,29515,21194,31596,21130,31629,19016,29548,21032,27468,21096,21129,25354,31628,33806,35886,21064,19081,38031,19045,12772,4323,6403,4355,4323,4290,19047,29515,23207,21158,40178,46386,35919,48532,50612,48532,50611,50611,31559,16898,8546,12772,19112,40112,48467,48565,48630,50679,50776,46354,40146,29613,33873,27404,37935,33873,46518,137},
{61375,61375,61375,59295,59295,59263,59263,59263,57215,57183,57215,59263,59263,59263,59295,61343,61343,61375,61375,61375,63423,63423,63455,63455,63455,63455,63455,63455,63455,61375,61375,61375,63423,63423,63423,63455,63423,61375,61375,61375,61375,59295,59295,59263,57215,57182,57214,55102,52989,52957,55037,53022,53022,53022,53055,53055,53055,50974,48894,50942,50974,50974,53054,55102,57150,55101,55069,55069,55069,55069,52989,52988,55004,52956,52924,55069,57181,55101,57117,57149,57117,57116,55068,55036,55068,55036,52924,52923,57116,55068,55068,55036,52956,52924,52988,55036,55036,55004,50843,50875,52923,50843,48763,46585,42390,12972,12939,21261,23309,23342,21229,21228,21228,21228,23277,23309,21197,21196,19115,19116,21196,25453,31759,37967,37999,40080,38000,38032,40080,23210,23275,31661,19081,29548,19049,31661,19049,31661,19017,29549,21096,27468,23209,25323,23209,23242,25290,21129,27403,19048,27435,19016,29515,16936,29548,18983,29548,18983,27468,18983,25387,19015,23275,21128,23274,23209,19081,25289,19048,27370,16968,27434,16903,27434,16903,27467,14855,27500,18951,27468,21064,27468,23144,25355,25257,23307,27402,25355,29483,23242,29547,21129,31628,21096,23274,27467,31628,35886,31661,25291,27532,31658,19046,12772,10628,10628,4323,6403,6403,19079,27336,21159,14917,27598,46419,33805,48500,50612,48532,50644,50644,42124,12770,10626,10660,25322,33707,39982,42259,50743,48565,50743,42129,40146,35921,31825,33742,35887,31760,44438,135},
{61375,61375,61375,59295,59295,59263,59263,57215,57183,57183,57215,57215,59263,59295,59295,59295,61343,61375,61375,61375,63423,63455,63455,63455,63455,63455,63455,63455,63455,61375,61375,61375,61375,63423,63423,61407,63423,61375,61375,61375,61375,59295,59295,57215,57182,55070,57182,55070,50876,50876,52989,53021,53022,53022,53055,53055,53055,50974,48894,48894,50974,50974,53023,55102,57150,55069,55069,55069,55069,55069,55036,55036,55036,52956,52924,55069,59229,57149,57149,57149,57149,57149,57117,55036,55036,52988,52924,50875,57116,57116,57116,55068,55004,52956,55036,55036,55036,55004,50843,50843,52923,48763,48763,44536,42358,15052,12907,23309,23341,23341,21261,21229,21229,21229,23309,21261,21229,21196,21196,19148,21196,23308,31695,38000,38032,40080,40080,40113,40080,23242,23307,29613,21162,29581,21130,31661,21130,31662,21130,29581,21129,29581,23209,27500,25290,25387,25289,23275,25322,23242,27403,21162,27467,21162,29548,21129,29580,21129,29580,19048,27500,19016,27468,21128,27468,21128,25355,23177,23307,25289,23275,25289,21129,25290,19049,27402,19048,29548,19049,31628,19016,29581,18983,29548,21064,27467,21096,25388,25225,25355,25289,23274,27402,21129,25355,29548,31693,35853,31628,21230,42225,23206,12804,10659,6499,10692,4355,4323,2178,17000,23175,27370,27597,44502,48565,40078,48532,48532,50580,50612,50578,29413,12770,10756,14918,23176,16805,8484,44470,52759,46485,52725,35822,33775,40179,35953,35887,31727,27534,42358,171},
{61375,61375,61343,59295,59263,59263,57215,57215,57215,57215,59263,59263,59263,59295,59295,59295,61343,61375,61375,63423,63423,63423,63455,63455,63455,63455,63455,63455,63423,61375,63423,63423,61375,61375,63423,63423,63423,61375,61375,61375,61375,59295,59263,57215,55102,55070,55069,50909,50844,50844,55037,53022,53022,50974,53022,53055,50974,48926,48894,50942,50974,50975,53055,55102,55102,55069,55101,57149,57149,55069,55068,55036,55004,52923,50876,55068,59197,57149,57149,57149,57149,57117,57116,55036,55036,52956,52956,50875,55068,57148,55100,55068,55036,52955,55036,55036,55036,55004,50843,50875,50875,48763,48763,44536,40310,12939,12874,23276,23309,23309,21228,21229,21228,19148,21228,21261,21196,21196,21163,19116,21163,21130,31662,37967,40080,40080,40080,40081,38032,21064,23242,29516,19049,27500,19016,29548,18984,29581,16935,29580,16936,29548,18984,27467,19016,27403,21096,23275,21161,23274,23241,21162,25354,19081,25355,19049,29515,17000,29548,16936,29548,16903,27468,16935,29516,16935,27467,19016,25388,21096,25355,21096,23274,21128,21194,25257,21162,27402,21162,29483,23210,29548,19081,29580,21129,29580,21097,29548,21064,29548,21064,27468,23144,21129,25355,29580,33806,27337,23209,29646,48464,16900,8547,12773,12772,8579,4354,2242,2178,19113,18917,10629,33903,44371,50645,37999,46484,48532,48498,48497,42059,14785,6433,10659,21193,31531,8484,12939,48663,50678,48566,37966,21098,33807,31662,33873,38000,29549,27502,42325,175},
{61375,61375,61343,59295,59263,59263,57215,57183,57215,57183,57215,59263,59263,59295,61343,59295,61343,61375,61375,61375,61407,63455,63455,63455,63455,63455,63455,63455,63423,61375,61375,61375,61375,61375,63455,63423,63423,61375,61375,61375,61343,59295,57215,57215,55135,55135,52957,50908,50876,50844,52957,52989,53022,50974,53022,53054,50974,48926,48894,50974,50974,50974,53054,55102,55102,55101,57117,57149,57149,57149,55069,55036,52956,50875,50876,55069,59229,57181,57149,57149,55101,57117,57116,55068,55036,55004,52956,52923,55068,59229,57148,55068,55036,52956,55036,55036,55036,52956,50843,50875,50908,48763,48763,44471,42358,10794,10826,21196,21229,23309,21228,19115,19148,19116,19148,21228,21164,19083,19083,17035,19115,16969,23275,35919,38000,40080,40080,40080,38032,23210,27468,29548,23275,29548,23242,29580,21130,31629,21129,31661,21097,31661,19016,29549,21064,29548,19048,27468,21129,27435,21129,25323,23209,23242,25289,23210,27402,21129,25354,19048,27435,16936,27435,16936,29516,14855,27467,14855,27436,16871,27467,16903,25387,16935,25355,18983,23307,23177,23275,25257,23242,25321,21162,27402,21129,29483,23210,29547,21162,31628,21129,29580,21129,23306,27467,31628,33773,21063,25290,38097,39914,16998,8580,10660,8579,6499,4386,8578,4354,16934,8418,8549,21255,31725,50645,37966,35851,29479,33707,25286,14786,19144,6498,6434,25484,46386,27534,38164,52791,50646,48631,40145,29679,31760,29614,33873,40146,29613,23340,40278,160},
{61375,61375,61375,59295,59263,59263,57215,57215,57215,57215,57215,57215,59263,59295,59295,59295,61343,61375,61375,61375,61375,61407,63455,63455,63455,63455,63455,63455,63423,61375,61375,61375,63423,61375,63423,63455,63423,61375,61343,61375,61343,59295,59263,57215,57183,57183,52989,52956,50876,50876,50876,52957,52990,50974,53022,53055,50974,48894,48894,50974,50974,50974,53055,55134,55102,55069,57117,57149,57181,57149,57149,55036,52956,50876,50876,55069,59229,59230,57149,57149,57149,57149,57116,55068,55036,55036,52956,52924,57149,59229,57149,57116,55036,52988,55068,55036,55036,52988,50843,52956,52956,50843,48763,44537,42325,10794,10826,19148,21229,23277,21196,19115,19115,19115,21196,21196,21164,19083,19083,19083,21163,29615,23307,38000,40080,40112,40080,40113,38032,19049,25355,27468,23242,29516,21162,29548,21162,31628,21130,31661,21129,31661,21097,29549,21097,29548,21065,29548,21097,29548,21097,27468,23209,25387,23209,25355,25289,25355,25289,23242,25322,21162,27370,21161,27435,19049,27402,19017,29483,19016,27435,19016,27467,18984,27467,18983,27500,21096,29581,21096,27500,21096,27467,21128,23339,23209,23274,27337,21194,27369,21129,27402,19016,23274,27435,29548,35821,23177,25355,44338,31495,14853,10692,10724,12773,12771,12834,10721,12866,16965,6306,8580,19143,21224,40243,39982,21158,16933,16966,18949,12806,35984,25352,8547,31791,50645,42226,50775,50710,50678,48598,44339,35953,31759,33775,35985,35952,31694,27501,38165,169},
{61375,61375,61343,59295,59263,59295,59263,59263,57215,57215,59263,59263,59263,59295,59295,59295,61343,61375,61343,61375,61375,61375,63455,63455,63455,63455,63455,63455,63423,61375,61375,61375,63423,63423,63423,63423,63423,61375,61375,61375,59295,59263,57215,57183,55135,57183,52990,50908,52924,50876,50876,50876,50909,50942,50974,53054,50974,48895,48894,50942,50974,50974,53055,55102,55102,55069,55069,57149,59197,57149,57117,55036,52956,52924,52956,57149,57181,59230,57181,57149,59197,57149,57117,57117,55036,55036,52956,55036,57149,59229,59197,55068,55036,55068,57117,57116,55036,52956,50843,52956,52956,50875,48763,44504,40245,8746,12906,21196,21229,23309,21196,19115,19115,19115,21196,21229,19148,19115,19115,19083,19083,23243,23210,40080,38032,40112,40112,40113,37999,17000,25387,27435,21194,25355,21129,27435,19049,27468,19017,29548,16968,29548,16936,27468,16904,29548,16903,29516,18951,27468,18951,27467,18952,25355,18951,25355,21064,23274,19016,23242,23144,21161,23177,21129,23209,19049,23209,19016,25289,16936,25289,16903,25321,14855,27370,14823,29515,16936,29548,19016,29548,21064,29548,21096,29516,21096,27468,23145,27468,23144,25355,23176,19081,25322,27435,31661,33676,23145,27501,42258,23206,10660,10660,14952,29548,16997,10786,10721,14882,16964,8451,14985,33870,33969,50776,40014,19044,12771,10659,10594,14917,38032,35722,10597,35919,50645,48533,50646,42094,37935,44373,40049,21326,33840,25453,40179,38066,31694,25323,36051,174},
{61375,61343,61343,59295,59295,59263,57215,57215,57215,57215,59263,59263,59263,59295,59295,59295,61343,61343,61343,61343,61375,61375,63455,63455,63455,63455,63455,63455,63455,63423,61375,61375,63423,63423,63423,63423,63423,61375,61343,59295,59230,57182,57182,57182,57183,55135,52990,50876,50876,50876,50876,50876,50877,50942,50942,53022,48894,48894,48894,50942,50974,50974,53055,53055,55134,55102,55069,57149,57149,57149,57117,55036,52956,52924,55037,59262,59294,59262,57181,57149,57149,57149,57117,55036,55036,52956,52956,55068,59197,59229,57149,55068,55036,57116,57149,57116,57116,52956,50843,52956,52956,52923,48763,44504,40245,10826,12874,21196,23277,23309,21228,19115,19115,19115,21196,21228,19116,19115,17035,17035,19083,23275,21162,38000,40080,40080,40080,40112,38032,21129,27468,27435,23307,27435,25355,27468,23274,27500,21227,29548,21194,29548,21130,29548,21129,27468,19081,29548,19049,27468,21129,29580,18984,27435,18984,27468,21064,27436,21032,25387,21064,25355,21064,23274,21096,23242,21096,23242,21096,21161,23144,21161,23176,19081,23176,19049,25322,19081,27402,19016,27402,16904,29483,16903,27435,16903,27467,19016,29547,21064,25355,21096,23242,27435,29548,33806,29450,21097,33872,52691,27400,10691,10693,25485,27336,21093,14882,8577,10755,8514,6338,17032,52757,50645,52791,44305,16995,10690,6466,12740,10627,23208,27272,12612,33742,46386,50711,52791,48565,46485,46584,46420,13005,42227,35888,38033,35986,31629,25323,36051,174},
{61375,61375,61343,59295,59295,59263,59263,57215,57215,57215,59263,57215,59263,59263,59295,59295,61343,61343,61343,61343,61375,61375,63423,63455,63455,63455,63455,63455,63455,63423,61375,61375,63423,63423,63423,61375,61374,61343,61342,57182,55102,55102,55102,57183,55134,55167,53054,52957,50844,50844,50844,50844,50876,50909,50942,53022,48894,48894,46846,50942,53023,50974,53054,55134,55134,55069,55069,57149,57149,57117,55068,55004,52924,52956,55069,61342,61342,59229,59229,57149,57117,57117,57117,55036,55004,52956,52956,57117,59230,59197,57149,55036,55036,57116,57149,57116,55036,52988,50875,52923,52956,52924,48762,44471,40245,10826,8810,21196,21229,21229,21228,19116,19115,19083,19148,21196,21164,21163,19083,17035,19083,21097,23242,38032,38032,40112,40112,40113,37967,18984,27436,23274,23274,25322,23210,25322,21162,25387,21161,27435,19081,27435,19016,29516,19049,27403,16968,27435,16936,27435,16968,27468,16903,27435,16904,27468,16903,27467,16935,27435,18951,27403,18951,25387,18952,25355,18984,25323,18983,25322,21031,23274,21031,23242,21064,23275,23176,25355,25257,23274,25289,21194,25289,23210,25289,21129,25322,19049,27370,16936,25290,16903,23209,27435,29548,35886,27435,16904,31791,46254,21094,12741,12839,25352,8514,12739,10658,8546,10658,6466,2177,10629,50644,46452,52759,42191,14882,12802,8610,6466,6401,6370,8419,10532,14498,29614,50711,42063,29548,46517,48663,50645,10762,46518,35855,35920,33873,29516,25388,31858,177},
{61375,61375,61343,59295,59295,59263,59263,57215,59263,59263,59263,57215,59263,59263,59263,59295,61343,61343,61343,61343,61343,61375,63455,63423,63455,63455,63455,63455,63455,63423,61375,61375,61375,63423,63423,61375,61342,59262,59262,57150,55102,55102,55102,57183,55135,57215,55135,53022,50876,50811,50844,50844,50876,50909,50942,53022,50942,46846,46846,48894,50974,50974,53054,55134,55102,55069,57117,57149,55069,55069,55036,52956,52924,55037,59262,63422,61342,59262,59229,57149,57149,57117,57117,55036,52956,52956,52955,57148,61342,59229,57117,55068,55036,57084,57116,57116,55036,55004,50875,52923,52956,52924,48730,46551,40277,10859,8811,19180,21229,23309,21228,19116,19116,21164,21196,21228,19148,19115,17035,19083,21196,21228,21162,38000,40080,40080,40080,38032,35887,21162,31629,25323,27468,25322,25387,25322,25355,25387,25323,25354,25322,27403,21162,27403,21129,25322,19049,25322,19048,25354,19113,25355,16935,27403,16936,27435,16935,27435,16903,27435,16871,27435,16871,27435,16871,27403,16838,27403,16871,25355,16838,25387,16871,25355,16870,25387,18951,27436,18950,25355,18983,25323,21064,23275,23144,23242,23176,23274,23176,21194,23176,19049,25322,27435,29580,35854,25355,27501,31661,29481,12708,12708,19113,21030,8580,12803,8610,10658,10691,6434,4290,6403,42225,48498,37966,42191,19075,10721,12802,8610,6434,6466,8452,18886,16580,34003,50710,33644,21162,44437,48631,54839,10729,42390,42259,38034,35953,29549,31661,40244,172},
{61375,61375,61343,59295,59295,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59295,61343,61343,61343,61343,61343,61375,63423,63423,63455,63455,63455,63455,63455,63423,61375,61375,61375,61342,61342,61343,59262,59294,59262,57150,57182,57183,55134,57183,57183,57183,57183,53087,50877,48796,50844,48796,50909,50910,50942,50942,48894,46814,46846,50942,50974,50974,53054,55102,55102,55069,55069,57117,55069,55037,55036,52956,52924,57149,61342,63423,63422,61342,59261,59229,57149,55068,55068,55036,52956,52956,52955,59196,61342,59229,59229,57149,55068,55004,57116,55068,55035,55003,52923,52923,52956,52955,50843,46585,40245,10826,12907,21229,21229,23309,21228,21164,19116,19115,19148,21229,21196,19115,17067,19083,21261,23373,19049,38000,40080,40112,38032,38032,35855,19017,29548,21161,25387,23210,25323,23210,25355,23242,25323,25290,23275,25290,23242,25322,23242,25290,21162,25290,21162,25322,21194,25290,19049,25322,19049,27402,19049,27402,19049,27403,19016,27402,19016,27402,16936,25354,16968,27402,16936,27402,16904,27402,16936,25354,16935,29515,21097,29547,19016,29515,18984,27435,16903,27435,18950,27403,16871,25354,16870,23306,16870,16968,23242,27403,31661,29482,27338,42259,35853,16901,8515,8516,14822,23209,16966,10693,6467,8547,8547,4322,4322,8580,36017,37933,14820,25450,17059,10754,10657,8546,8578,8514,10596,10565,18986,46582,50645,38000,29712,48532,46453,54936,21130,40277,44340,38066,38001,31629,31793,33775,166},
{61375,61343,61343,61343,59295,59263,59263,57215,59263,59263,59263,57215,59263,59263,59295,59295,61343,61343,61343,61343,61343,61375,63423,63423,63423,63455,63455,63455,63455,63423,61375,61375,61343,57182,59230,61342,59262,61310,59262,59262,59295,59263,55135,55135,55135,57183,55135,55103,50909,48828,48796,48828,50909,50942,52990,50941,48894,46814,48894,48894,50975,50974,53054,55102,55102,55069,55069,57117,55069,55037,52988,52956,52924,59197,63455,63455,63422,61342,61310,59229,57149,55069,55037,52956,52956,52956,52956,57116,61277,59229,59229,59229,57116,55036,57116,55068,55004,52923,52923,52924,52956,52923,50843,46682,40343,19442,21588,27764,27731,27699,25651,23538,25586,25618,23570,25586,25586,23505,21425,21425,23538,25487,14921,29548,33708,31660,33708,31628,31596,25355,29516,25323,27467,25323,27403,25291,25355,25291,25323,25290,25323,23242,25323,25322,25322,23242,23242,25290,25388,21063,21032,27501,18984,18919,19049,21130,23308,27534,23372,23275,14822,19016,14822,21031,16871,18950,14855,18919,16903,21064,21227,27436,23307,29516,25355,31661,25387,31628,25323,29580,21226,29548,23307,29581,19049,27436,21129,27402,21097,27402,19016,23210,25290,25322,29547,21031,21097,44371,31627,8515,4290,4290,6371,21096,14724,8580,6499,6434,6434,6435,4354,12773,35984,50645,29645,31821,16994,12865,14948,10722,10690,6371,8516,8517,34003,48630,48565,50678,48565,48401,42129,52889,31695,38196,48598,35888,35888,33775,38131,31629,163},
{61343,61343,61343,61343,59263,59263,57215,57215,59263,59263,57215,57215,59263,59263,59263,59295,61343,61343,61343,61343,61375,61375,63423,63423,63423,63423,63455,63455,63455,63455,61407,61375,61343,57182,57182,59262,57182,59230,59262,59262,61343,59295,57183,57183,55135,55135,55135,53055,50942,50909,48796,50844,50877,50910,50942,50942,48894,46814,48894,50974,50974,50974,53054,57150,57182,55069,55069,55069,55037,55037,52988,52956,52988,59229,65502,63454,63422,61342,61310,59229,57149,57149,55101,52989,52956,52956,52956,55036,59197,59229,59229,59229,57149,52988,55068,55036,55036,52955,52923,52923,52956,52956,50843,50843,48763,46683,50875,52956,48827,46715,44667,44635,44603,48795,46715,44667,48795,48827,46747,46715,44667,29778,19180,31693,37999,38000,40080,40080,40080,38000,40047,40080,40048,40016,40015,37967,37967,37967,37935,37967,37967,37967,37935,37967,37967,37935,37935,37967,35920,16871,14726,33807,12711,8486,12841,12842,21228,36019,27696,23340,6404,12743,6339,8485,6437,4291,8549,6339,10565,8484,19016,33873,37935,40047,40079,40047,40047,42127,42127,40079,40111,42160,48598,48532,19180,21326,19115,19017,19049,21097,19049,19049,19017,21129,27403,25322,25387,44273,35788,10596,4355,4323,6435,21161,8451,8548,6532,4322,2178,14790,4289,10725,33902,54839,50645,39980,14914,12801,10690,12739,12773,25290,21129,14889,44470,48630,44372,50646,50613,39951,31532,44340,29647,38131,48565,33775,35888,35920,29613,25258,161},
{61375,61375,61343,61343,59295,59263,57215,59231,59263,59263,59263,59263,59263,59263,59263,59263,59295,59295,59295,59295,59295,61343,61343,61375,63423,63423,63455,63455,63455,63423,61375,61375,59295,59295,57182,59230,55102,55102,57150,59262,61342,59295,57183,55135,55135,55135,55103,53055,53022,50909,48828,48828,50909,50942,50942,50942,48894,46814,48894,50974,50974,50974,53054,57182,57182,55102,55069,55069,55036,55037,55069,55037,55069,61342,65503,63454,63422,61342,61342,59261,57181,57181,59230,57181,55037,52956,55036,55068,59229,59229,61309,61309,57181,55036,55036,57116,55036,52956,52923,52923,52956,52955,52923,50843,48763,46682,48763,52955,50843,48763,46683,44635,44603,48763,48763,46683,48795,50875,48795,46715,48795,42489,34004,33774,40047,40047,42127,40080,40079,40047,42095,42128,42127,40047,40047,40015,40047,40047,40014,40047,40047,40015,40047,40047,40015,40014,40015,40047,42161,14693,12613,31759,14824,8518,14922,17002,19115,36051,40179,27469,8550,14823,6405,10598,8582,6404,10630,6404,10630,6436,14823,33741,39981,37933,37934,39981,39981,40014,42062,42062,42094,42094,44240,59064,19180,19310,21260,19082,19082,19081,19049,19050,16969,21194,27435,27436,29647,33840,33709,33807,12806,6435,4387,10628,8581,14853,10627,6402,2210,12708,8579,10756,29675,54838,44174,21059,12834,12801,6465,6466,10823,31596,16969,29745,46549,54970,42194,46517,50712,46420,21262,25487,27697,36018,48435,29582,29549,33808,27403,21163,147},
{61343,61343,61375,61343,61311,59263,57215,59231,59263,59263,59263,59263,59263,59263,59263,59263,59295,59295,61343,59295,59295,59295,61343,61375,63423,63423,63455,63455,63423,63423,61375,61343,61375,61343,57214,57182,59262,59295,59262,61310,61343,59263,57183,55135,55135,55103,55103,53055,53022,50877,48828,50908,50877,50909,50942,50942,48894,46814,50942,50974,50974,50974,53054,57182,59230,59230,57182,55101,52989,55036,55069,55101,57182,63423,65535,63455,63422,61374,63422,61342,59294,61342,61342,61310,57181,55069,55036,57116,59197,61277,61341,61309,59197,55036,55068,57116,55036,52955,52923,52923,52956,52923,52924,50875,50843,48763,48795,55036,50875,48763,46683,44635,44635,48763,46715,46715,48795,52923,48795,46715,48795,48763,42390,33774,40015,40015,42095,40047,40047,40015,42063,42127,42095,40047,40015,40014,40047,40015,40014,40014,40046,40014,40015,40015,40015,40015,40014,40047,35920,10565,16970,33808,25323,16903,25256,25256,25256,33676,29384,27304,23176,25289,25224,25257,23143,25256,23176,23143,25256,23176,25224,27304,29383,31463,29383,29383,29416,31463,29384,29319,31431,29384,31562,54805,21292,19343,21358,21293,21294,23374,23341,21293,19180,21227,27435,29516,48564,52724,44241,46419,31660,8483,6436,8547,8547,12772,10659,6402,4323,6435,10627,12772,19111,54837,31527,14852,12770,12769,8578,8580,27532,44208,23340,44338,39950,46485,40146,48564,46289,29451,23275,50743,44307,27435,31564,29548,23243,33840,27469,40147,130},
{61375,61375,61343,59295,59263,59263,59263,59263,57215,59263,59263,59263,59263,59263,59263,59263,59263,59263,61343,59295,59295,59263,61343,61343,63423,63423,63455,63423,63423,63423,61343,61310,61343,61375,61343,59295,61343,61375,61343,61343,59295,57215,57182,55135,55135,55103,53055,55103,53022,50909,50876,50876,50876,50877,50942,50942,46814,48862,50942,50974,50942,50974,53054,57150,61310,61342,59262,57182,52957,52956,55037,57181,61342,65503,65535,65535,63422,63454,63423,63422,63422,63390,63422,61342,63422,59261,55069,55036,59197,59261,61309,61309,59197,55036,57116,57148,55036,55003,52923,52956,52956,52923,50875,50875,50843,48763,48795,55036,52955,46715,44635,44603,44635,48763,46715,46715,48795,52923,48763,46715,48763,48763,42358,29516,31629,31661,33709,33709,33709,33677,33709,33709,33709,33709,31628,31628,31628,31628,31628,31628,31628,31628,31628,31628,31596,31596,31596,31628,29548,31628,31694,33840,27434,16903,25224,27336,25224,31596,27304,27304,27304,27336,27303,27304,23143,27336,25256,27271,27336,25256,27336,25256,27303,29416,27272,29384,27272,27303,27336,25191,29383,27304,27369,29678,21293,21391,23536,25649,27697,27729,27697,25648,23503,27566,25290,31661,46452,46353,37805,37901,21029,6435,6467,6466,6467,6467,6467,4387,4355,6403,4257,8579,10660,50578,23172,10657,8546,8610,6466,8546,29645,40080,38066,42031,27047,46518,48532,48499,37739,29516,27566,54935,44241,19049,23341,25258,23275,35953,29582,40015,115},
{61375,61375,61343,59295,59295,59263,59263,57215,57215,59263,59263,59263,59263,59263,59263,59263,59263,61311,59295,59263,59263,59263,59295,61343,63423,63423,63423,63423,63455,61375,59263,59262,61342,61343,61375,61375,61375,61375,61343,61343,59295,57214,55134,55135,55135,55103,53055,55103,53055,53022,50877,50877,50877,50877,50942,50942,46814,48862,50941,50942,48926,50974,53054,57182,61310,61342,61342,61342,55134,55069,55069,59230,61342,63422,65535,65535,63455,63455,63422,61309,61277,63390,63422,63422,63422,59262,57149,55069,57117,59229,61309,61277,59197,55036,57149,57148,57084,55003,52955,55003,55003,52923,50875,52923,50843,48763,48762,55003,55003,48763,44634,44602,44634,48763,46683,46683,48795,50843,48763,46683,48763,48762,40310,33742,37967,40047,42127,40047,40047,40047,40047,40080,42128,40047,40047,40015,40047,40047,40015,40014,40047,40047,40047,40047,37999,40015,37999,40015,33774,31595,33708,33774,27403,16839,21063,27304,23111,31595,27304,25224,27239,25224,25191,25224,23046,27336,25224,25159,25288,23143,29416,27304,25223,27336,25191,29351,25159,27271,27303,23077,29384,25224,27337,48663,21227,21391,23536,25617,25681,29810,29777,27729,25649,27632,29647,35953,50677,50482,31563,14820,8515,6500,8579,8547,4354,4355,6499,4386,6370,2145,2210,4322,6467,37996,16929,8545,4353,4321,4321,12803,27533,38065,46451,42161,33744,48565,50580,41964,29386,50743,56950,56950,44339,31727,31662,23210,19049,33906,33742,25290,104},
{61375,61375,61343,61343,59295,59295,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59295,59295,59263,59263,59263,59295,61343,61375,61375,61375,63423,63423,63423,63423,59295,57214,61310,61342,61375,61343,61342,61343,61343,61310,59263,57182,55135,55135,55135,55103,53055,53055,55103,53055,50942,50909,50909,50909,50941,48894,46814,50910,50941,50942,48927,50974,55102,57182,59230,59262,61342,61342,59262,59262,59262,59262,61342,63422,65503,65535,63455,63422,61374,57149,57149,63422,63422,63390,61309,59197,57117,57116,57116,59229,61309,59229,59197,55036,59229,59197,35921,29549,29581,29550,25258,25259,29581,29582,29614,29614,29646,33743,33743,31727,31695,31727,31760,31760,31760,31760,33808,33840,33808,31761,33840,33873,35986,33773,40015,40015,42095,40047,40047,42095,40079,42128,42127,42095,40047,40015,40047,40015,40015,40015,42095,40015,40015,40047,40047,40015,39982,35756,33676,27435,23210,21161,27435,16871,25224,27336,23143,31596,27304,23176,27239,27337,27271,27272,23111,27369,25225,27239,27337,25191,29416,25256,27271,27304,25191,29384,23111,27303,27336,25158,29384,25223,27336,54839,21260,21391,25617,25649,25681,29777,29777,29777,27729,29778,29777,36050,52725,41996,14755,10595,8579,8547,6499,6499,6499,6467,8547,6402,6435,2210,2210,2210,6499,35980,14913,10689,6497,2241,2242,16967,25420,42227,40078,27401,44404,42226,50579,23046,27468,59063,56918,56918,50645,46453,35920,27403,23341,36019,35920,31727,133},
{61375,61375,61343,61343,59295,59263,59263,59263,59263,59263,59263,59263,59263,57215,59263,59263,59295,59263,59295,59263,59263,61343,61343,61343,61375,61375,63423,63423,63423,63423,61375,59262,59262,61343,61374,61343,61343,61343,57182,59230,59262,57182,55135,55135,55135,53055,53023,53055,53055,53055,53023,50909,48861,50877,50941,48862,46814,50942,52989,50942,48926,50974,55102,57182,59262,59262,61310,61310,59262,59229,59261,59262,61374,63423,65503,65535,63455,63423,63422,55069,55004,61310,63422,59229,55069,55036,55036,55036,55036,57116,59196,59229,57148,55036,59197,55003,23341,19180,21228,19148,6470,12809,19180,19148,19181,19180,19213,19181,19180,21229,21261,21261,21293,21260,21261,21228,19180,21228,19148,19148,19148,21294,31825,27403,31531,31530,31530,31563,31562,31562,31563,33611,33643,33643,31563,31595,33643,33611,33611,33611,33643,33643,33643,33643,33643,33644,27337,25289,25290,29581,23210,18983,29515,16903,23177,27336,21063,31596,27304,23144,25191,25289,25223,23176,21063,27369,23176,25223,25289,23111,27368,25224,27271,25224,23110,27336,21031,27271,25256,23111,27336,23176,27337,52823,31727,21358,23439,23503,25584,29744,29712,29712,27664,23276,25487,33905,46288,18949,6466,4354,8547,12805,6467,6532,10725,8515,8547,4355,4322,4355,2242,2242,12837,35948,19138,10753,8609,4386,19147,35985,27469,46452,27270,14854,35887,38033,31498,16741,29516,56917,56918,56918,54838,48598,44307,29452,27567,40146,42292,40081,113},
{61375,61375,61343,61343,59295,59263,59263,57215,57215,59263,59263,59263,59263,57215,57215,59295,59295,59295,59295,59263,59263,59295,61343,61343,61343,61375,61375,61375,63423,63423,61375,59262,61342,61343,61375,61374,61375,59295,57183,57182,59262,57215,55135,55103,55135,53055,53023,53023,53055,53055,53023,50941,50909,50877,50909,48861,48862,50909,52989,50942,50974,50974,55134,57182,59262,61310,59262,59262,59262,59229,59229,61310,63390,63455,65535,65535,63423,63390,59262,52989,52956,61342,63455,59262,57149,55069,52988,52988,55004,55036,57116,57116,57116,57084,57116,52890,21326,19213,19181,19148,4390,12842,21228,19181,23309,21261,21261,21229,21228,21261,21261,19213,21261,21260,19213,21261,19213,19180,19180,19181,19148,21326,27599,16969,14857,16904,12743,12711,12776,14856,14856,14824,14824,14824,14856,14824,16904,16904,16904,16872,16904,14824,14824,14824,14823,16872,14856,21097,23243,16838,12645,10566,27403,16903,25289,29417,23143,31596,27337,25224,27271,25289,27271,25256,25191,27336,25224,27271,27337,25223,29384,25191,27304,25256,25191,27336,25192,27303,27304,27271,27336,25256,29417,50678,25355,12775,17034,14856,14823,16936,14855,16904,14823,16969,16904,25420,25224,6467,6434,4354,6402,8483,10660,10693,10660,6435,4322,6403,4355,4355,2210,4419,12709,33805,40046,23369,23468,25550,48630,54838,42226,44241,20998,14854,16966,31726,35691,14726,40046,54805,54870,56950,56918,52825,44176,31630,36018,38066,42260,35855,110},
{61375,61343,61343,61343,59295,59263,59263,59263,59263,59263,59263,59263,59263,57215,59263,59295,59295,59295,59295,59295,59263,61311,59295,61343,61343,61375,61375,63423,63423,63423,61375,59262,61310,61343,61375,61343,61343,61343,59295,59263,59263,59263,55135,55103,55135,53055,53023,53023,53055,53055,53023,50942,50909,50877,50909,48862,48862,52957,52989,50974,48894,50974,55134,59230,59262,59262,61310,59262,61310,59230,59229,59230,61310,63455,65535,65535,63455,63423,61342,52989,52988,61310,63422,61374,57149,55069,52956,52988,55036,55068,55068,55036,55036,55036,55036,48730,23374,21229,21261,19181,6503,14955,19213,19213,21261,21261,21261,19181,19213,19212,21229,21228,19213,19213,19213,19180,19212,17132,19180,17132,15051,36084,33906,14889,12776,12744,10663,10630,12743,12743,12743,12776,12776,14824,12743,10695,12744,14792,14856,12743,12743,12744,12744,12743,12711,12775,12744,12711,19082,8485,6372,8485,25290,18952,23209,25225,14758,25257,16837,23210,18950,16903,18950,18984,16838,19015,16838,18983,18951,16838,18983,14725,18951,16806,16805,21031,14757,18950,16871,16837,21031,16805,21063,40015,21163,8647,10662,4291,2211,4291,4226,4291,4323,10661,12710,33904,21031,8484,6467,4322,4322,12774,14853,10660,6467,8483,4323,6435,4290,4323,2210,8580,8548,27531,50645,48532,48565,48499,46416,46285,37931,40080,25385,19080,19113,35919,31335,16872,42128,54870,56917,58998,56950,52857,44177,23210,29614,38034,40081,29387,97},
{61343,61343,61343,59295,59263,59263,57215,57215,57215,59263,59263,59263,57215,59263,59263,59295,59295,59295,59263,59263,59263,59263,59295,59295,61343,61375,61375,61375,63423,63423,61343,59263,61342,61374,61343,61343,61343,61343,61343,61343,59263,57215,55135,55103,55103,53023,50975,53023,53023,53023,53055,53022,50909,50877,50909,48894,50909,52957,52989,50942,50974,53054,55134,57182,59262,61310,61342,61342,61310,59262,59230,59230,61310,65503,65502,65502,65503,65535,61374,55037,52988,61310,63422,63422,57149,57117,52956,52956,55036,57116,57148,57148,55036,55004,55004,48697,23341,19181,19181,19148,6438,14923,19212,21293,21293,21293,21293,19213,19213,19212,21228,19180,21261,19212,19213,19212,19213,19180,19180,19180,21293,38132,27567,14824,12776,14824,12711,12711,12743,12776,12711,12744,12744,12743,12744,12776,12744,12776,14792,12744,12776,12744,14824,12710,8484,14790,10564,12645,12677,10597,12710,10565,16838,12612,16838,12612,14823,12645,16871,12645,14790,12677,12677,16871,12677,14823,12645,14790,10564,14791,12612,14791,12645,12710,12678,12645,14790,12645,14790,10564,14790,10532,16871,14692,16903,12677,12678,12677,8517,12742,8484,10629,8484,12742,10564,18918,21096,8484,8547,6402,4290,12740,14853,8547,6467,6435,4387,6467,2242,4355,6370,6435,6370,21290,52790,50645,48565,37931,29544,23237,19012,31758,38032,25387,31825,27304,16741,23243,46451,56951,52657,59030,56950,52825,44143,25355,17001,35986,35921,27437,129},
{61343,61343,59295,59295,59263,59263,59231,59263,59263,59295,59295,59263,57215,59263,59263,59295,59295,59263,59263,59263,59263,59295,59295,59295,61343,61343,61375,61343,63423,63423,61343,59262,61310,61342,61342,61342,61343,61343,61343,61343,59263,57215,55102,55102,55103,53055,53023,50974,53055,50974,50974,53022,50910,50909,50941,50942,52957,52957,52989,48894,50974,53054,55134,57182,59230,61310,61342,61342,61310,59262,59262,59262,61310,61342,61342,63422,65502,65502,61374,55037,57149,61342,61341,61309,55036,52956,50876,52924,55004,57116,59197,57148,57116,55068,55004,46616,23341,19213,19181,19148,4390,14955,21260,21293,21261,21261,21261,19212,19181,19212,19180,19180,21260,21228,19181,19181,19213,19180,19181,17100,21325,42325,38131,14856,10663,12743,12711,14792,10663,12711,12711,12744,12744,12711,12744,12744,12744,12744,12776,12711,12743,12744,14823,10532,4258,8452,6339,6371,8387,4258,8419,4227,8451,2146,10500,4194,8452,4259,8452,6339,6339,8419,4258,10500,6372,10532,4226,8452,4226,10500,6371,10532,8387,6339,8451,4226,10532,4226,10532,4226,8451,6274,10532,6306,8452,6339,4259,10467,4258,12580,4226,8451,4258,10532,6371,10564,23177,8484,8547,4354,4387,10661,16967,14950,8580,8580,6499,8548,6467,4322,4258,8579,8482,14886,48499,50645,46485,40077,25417,23335,19077,25483,44306,31726,40113,18852,18887,29516,54870,59030,52689,56949,56918,52759,42096,21162,21227,40179,38067,40147,141},
{61343,61343,59295,59295,59263,59263,57215,57215,59263,59263,59263,59263,57215,59263,59263,59295,59263,59263,59263,59263,59263,59295,61311,59295,61343,61343,61343,61343,61343,61343,57150,57149,59230,61342,61343,61342,61343,59295,59294,61343,59263,57215,55103,55102,55103,51007,50975,50975,53023,50974,50974,50974,50942,50909,50909,50909,50909,52957,50941,50942,50974,53055,55134,57150,59230,59262,61310,61310,59229,59229,59229,59230,59262,61342,61342,63390,63422,63422,61342,55101,61309,61310,57116,57084,52956,52924,50876,50875,52923,57084,59197,57116,57116,57116,55036,44504,21293,19213,19180,19148,4390,16971,21260,21261,23341,21293,21261,19212,19213,19181,19213,21229,21228,19180,19180,19180,21229,19180,21229,19148,19147,42259,40146,14856,12743,12711,12743,12743,12711,12711,12743,12776,12743,12744,12743,12776,12744,12744,14792,10663,12743,14824,14824,6339,4259,4259,4291,6371,4258,2178,4258,4258,4259,2178,4258,2178,4259,4259,4259,4226,2178,4258,2146,4258,8452,4258,2146,4227,2178,4226,6339,8419,4226,2178,4226,2178,4258,2146,4258,2178,4226,2146,4226,2178,4258,2178,2178,4226,4259,6339,4258,4226,4258,6338,8452,8452,18983,8482,6467,6499,8515,18949,8418,14852,10692,10692,6435,12741,6435,6467,8579,10692,10594,10629,38065,52791,50710,42158,27497,25384,23302,14852,29514,33840,31597,21031,25192,40145,54837,56950,54770,56917,56917,52792,46322,27501,31728,44405,40245,48533,104},
{61375,61343,61343,59295,59295,59263,57215,59231,59263,59263,59263,59263,59263,59263,59263,59295,59263,59263,59263,59263,59263,59263,59295,59295,61343,61343,61343,61375,61343,59295,55134,52989,57149,59262,61343,61342,61343,59295,59295,61343,59263,57183,55103,55103,53055,53023,53023,50975,50975,50943,48894,50974,50942,50909,48894,48861,50909,52957,50942,50942,53022,53054,57182,59230,61310,61310,61310,59230,59229,59197,59197,59262,61374,61342,63423,63423,65503,63455,61374,59262,59262,55037,52988,52956,52988,52956,50843,50843,52923,55036,57149,55068,57084,57084,55036,42423,21293,19213,19180,17036,4390,14955,19213,21261,23341,21293,21260,19180,19213,19213,19180,21228,21229,19180,19148,19180,19213,19180,19180,19180,17067,21227,14791,12743,8614,10663,10663,12711,10631,10631,10663,10663,10663,10663,10663,12711,10631,10663,12711,10663,12711,21097,16936,6339,4258,4291,4226,4226,4259,2178,4259,4226,4259,2178,4259,2146,4226,4259,4259,4226,2146,4259,2178,4259,8452,4259,2114,4226,2146,2178,6339,6371,2178,4226,4226,2146,4258,98,4226,2146,4226,2146,2178,4226,4258,4226,4226,4226,4226,4291,4226,4226,2146,4226,8451,8451,16935,8579,6531,4386,4289,4290,2210,10693,10659,10693,10660,14756,8580,12771,10658,10594,10595,14921,38065,52791,50677,37965,29643,25449,27497,16998,19177,42258,25127,31531,37902,48596,54837,56885,58931,50609,56885,52791,48533,31792,48631,44373,38099,46420,118},
{61375,61343,61343,59295,59263,59263,57215,57183,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59295,59263,61343,61343,61343,61343,61343,61375,61343,59295,55102,57118,59262,61343,61342,61343,61343,59295,59295,59263,57183,55103,55103,53055,53022,53023,50975,50975,48895,48862,50974,50942,50910,50942,50941,50909,52957,50942,48894,50974,55102,57182,59262,61310,61310,59262,59229,59229,59230,59230,61310,61374,63423,65503,65503,65535,65535,63454,61374,61342,55069,52989,55069,52988,52956,50844,48763,50843,55004,57116,55036,55036,55004,55004,42391,21293,19213,19213,17035,6470,17035,21261,19213,21293,21261,21228,19180,19212,19213,19181,21229,21261,19180,19180,19180,19213,17100,19180,19148,17035,12744,10598,10664,10631,10663,10663,10695,10663,10631,10696,10663,10663,10631,10663,12679,10631,10663,10663,10663,10695,25323,6339,6339,4291,4291,4259,4259,4291,4226,4259,4258,4291,4259,6339,4227,4259,4259,4291,4259,2178,4259,4259,4259,6404,6339,4226,4259,4226,4259,6307,6371,4226,2178,4259,2178,4259,2178,4258,4226,4259,2178,2211,4226,4226,4258,4226,4258,4226,6339,4259,4258,2178,4258,6339,6371,12677,10659,8611,8547,6435,2242,2210,21160,14852,19047,16868,8579,10754,14914,12706,6369,8483,25290,33904,52823,48468,40178,38129,31787,25350,21160,27663,48499,29386,27402,39982,52757,54805,54772,54706,46384,58998,52758,52759,42325,50743,50711,40114,42324,174},
{61343,61343,61343,59295,59263,57215,57215,57183,59263,59263,59263,59263,59263,59263,59263,59295,59263,59263,59263,59263,59263,59263,59263,61343,61343,61343,61343,61343,63391,61375,61343,59263,57182,61310,61343,61343,61343,59294,61343,59295,59263,55135,55103,55103,53022,53023,50975,50975,50975,48894,48862,50974,50941,50941,50910,50910,50909,52957,50942,48894,50974,55102,59230,59262,61310,59262,59230,59230,59262,59262,61342,61342,63390,63422,65502,65503,65535,65535,65535,63422,61342,57181,55069,55069,55036,52956,50843,48763,50843,52923,55036,55036,55003,52955,52955,40278,21293,19213,19180,14987,2342,17067,21260,21260,21293,21261,21261,21228,21260,19180,19212,19212,21261,19212,19180,19180,19181,19180,19181,19148,12906,4422,6405,6372,6372,6372,4324,4324,6372,6372,6372,6372,6372,6372,6372,6372,6372,6372,6372,6372,12678,27436,6339,6339,4259,4259,4258,4258,4258,2178,4259,4226,4259,4226,4259,4226,4258,4259,4291,4258,2178,4259,4226,4259,8452,4291,2146,4226,4226,4226,4291,6371,4258,4227,4259,4226,4258,2178,4226,4226,2178,4258,4226,4226,4226,4259,4226,4226,4259,4259,4258,4258,4259,4259,4291,6371,10628,8547,6466,6467,6434,4322,8581,12741,8482,8483,8611,12866,10690,10657,10626,14821,6370,23209,33774,52758,44274,48598,55035,50674,29543,25386,40113,46452,44241,37836,46386,54773,54805,54837,58997,48463,58998,54806,54937,50743,46452,52791,42195,44372,152},
{61343,61343,61343,59295,59263,59263,57183,57183,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59295,61343,61343,61375,61343,61375,61375,61375,59295,57182,61310,61343,61343,61343,61342,59263,59263,59263,55134,55103,55103,53023,53023,50975,50975,50975,48894,46814,50942,50941,50941,50942,50910,50909,50909,50941,50942,53022,55102,59230,61310,59262,59262,59230,59262,63455,63487,65535,63455,61342,61342,63422,65503,65503,65503,65535,63423,61310,57149,55069,57084,55036,52924,50843,50843,50843,50843,55004,55036,52923,52891,52923,36052,14857,14856,14856,10663,4324,12776,14889,16936,14888,16937,14889,14888,16937,16969,16937,16969,16969,16969,16969,16937,16937,16937,14889,14889,12808,4389,4356,2244,2244,2212,2212,2212,4259,4259,2211,2212,2211,4259,4227,2211,2211,2211,2211,2211,12710,29516,6339,4291,4258,2210,4226,2146,4258,2178,4291,2178,4259,2178,4258,2178,2178,4259,4291,4226,2146,4226,2146,4258,6404,6339,2146,4226,2178,2178,6339,6371,4258,4227,4259,6404,4259,2146,4226,4226,4226,4226,2146,4226,2146,4226,2146,4227,4259,6339,2146,4226,4258,6339,4259,8451,8580,6434,6467,6434,6435,6402,16902,10563,4354,4387,6499,12802,8546,6466,8547,16968,8483,8549,33839,50613,44307,50712,63422,52529,31627,35822,14692,42291,44142,37933,50579,50644,54837,54804,56917,48398,58998,56918,52824,50613,38034,46421,42292,38066,155},
{61343,61343,59295,59263,59263,57183,57183,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59295,59295,61311,59295,61343,61343,61343,61343,63423,61375,61343,59262,61311,61342,61342,61343,59295,59263,59263,59263,55134,55103,53055,50975,50975,50975,50975,50943,48862,46814,50943,48862,50910,48862,50910,50909,52957,50942,50974,53022,55102,59230,59262,59262,59262,57182,61343,65535,65535,65503,63422,61342,61342,63422,65502,65503,65535,65535,65503,57117,57117,57117,55069,55036,52924,50843,48763,50843,50843,55004,55068,52956,50875,50843,34037,8648,8616,6567,4454,2212,6470,6535,6503,6503,8551,6502,6471,6470,4422,6470,6503,6470,6470,4422,6470,4390,4357,4325,2277,2276,2276,2276,2244,2244,4292,2244,2244,2244,4260,4292,4260,2211,2211,2211,2211,2211,2179,2179,2211,23243,31564,6339,4291,6404,6372,4259,2179,4259,2178,4259,2211,4259,2179,4258,2178,2178,6372,8485,4226,2146,4226,2146,4258,6404,6339,4291,4291,2178,2178,4259,6404,4258,2211,4258,2145,4258,2146,4226,2178,4226,4226,2146,2178,98,4226,2178,2178,6339,6339,2178,4291,4226,2178,4259,8483,8548,10628,6499,6467,4322,4290,6402,6467,6402,4322,4419,4354,8579,6466,6435,14789,14756,8452,42258,50580,46550,50776,65436,33513,33775,35757,6403,35886,27303,40112,39947,42223,48399,46254,54771,37769,48499,56951,52824,48533,38034,46518,42357,38098,143},
{61343,61343,59295,59263,59263,57183,57183,57215,59263,59263,59263,59263,59263,59263,59263,59263,59263,57215,57215,59263,59263,59295,59295,59295,59295,61343,61375,61375,61375,61375,61375,61343,59262,61343,61343,61343,61343,59295,59263,59263,57215,55135,53055,53054,50974,53023,53022,50975,48927,46815,46815,50943,48862,48862,48862,50942,50909,52957,50942,50974,53054,57182,59230,59262,59262,57214,55134,61342,65535,65503,63390,61342,61342,63390,63422,65503,65535,65535,65535,63423,57117,55069,57117,57117,55036,52956,50843,48763,50843,50843,55004,57149,55036,52955,50843,36085,12776,10695,10695,8583,2244,10631,12744,12808,12776,12808,10728,10760,12776,12808,10728,12776,10728,12776,10728,10728,10728,12744,10696,10696,8583,4325,4357,2244,2244,2244,2244,2244,2212,2212,2244,2212,2211,2212,2179,163,2211,2211,2211,4259,14823,29451,6371,6371,6404,6372,4258,2179,4259,2179,4291,4259,4291,4259,4259,4259,4259,6371,8484,4259,2178,4259,2178,4291,8452,6339,6372,6372,4259,4259,4291,8484,4258,4291,4259,2178,4258,2178,2178,4226,4226,4258,2178,4226,2178,4227,2178,4258,4291,6339,4226,4259,2210,2178,4259,6403,6371,14854,8612,6434,4290,2274,4354,4386,8579,6467,4419,6435,8579,6467,4354,2210,4355,6469,48628,52693,50711,52791,54772,25355,33840,35787,12739,19013,14852,29512,20995,19014,23077,14722,37998,29286,35951,56950,52824,42161,35954,46518,44438,38067,161},
{61343,61343,61343,59263,59263,57215,57183,57215,57215,59263,59263,59263,59263,59263,59263,59263,59263,57215,57215,59263,59263,59263,59263,59295,61343,61375,61375,61375,61375,61375,61375,59295,59262,61343,61343,61343,59294,59263,59263,59263,57215,55134,53055,53055,50943,53023,53023,53023,50943,46814,46814,48894,48862,48862,48862,48893,50941,52957,50942,50974,53054,57182,59230,59262,59262,57182,55134,57182,65470,63455,61374,61342,61342,61342,63422,65502,65535,65535,65535,63422,55069,55036,57117,55068,55036,52956,50843,48763,50843,50843,52988,57148,55068,52956,52924,38198,10794,10729,8648,6536,197,6535,8680,8616,8648,8616,6568,6568,8616,6568,6536,8583,6535,6535,6503,6503,6502,4422,4390,4389,4325,2277,4324,2244,2244,2244,2212,2212,2244,2212,4292,2212,2211,2211,2179,2179,2211,2211,2179,2211,14888,27403,6371,10597,8484,10597,8485,8517,8517,8517,10598,8485,8549,8484,10630,8452,10598,8484,10597,8517,8485,10565,6404,10597,8452,10565,6404,10597,6404,8484,8484,8485,8485,6404,10598,6372,8549,6372,8517,8420,6436,8484,6404,8484,6339,8485,4323,8517,6371,8517,6371,6436,6404,4356,8484,8452,10565,16967,10627,2209,4354,6435,8515,4354,6435,8579,2241,4354,8547,4354,4322,4419,8644,21324,56917,56886,52758,52759,50646,44372,44404,37964,12803,14852,12771,14819,8449,10659,6401,4290,25484,27238,27565,56918,52791,42193,38034,40211,44372,35986,179},
{61343,61343,59295,59295,59263,57215,57183,57215,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59295,59263,59295,61343,61343,61343,61343,61375,61375,61375,61375,61343,61310,61342,61343,61343,59294,59263,59263,59263,57182,55134,55103,53055,50942,50942,53023,53023,48895,46814,46782,48862,48862,48862,48862,50909,50909,50910,50942,50974,55102,57182,57214,59262,59262,57182,57183,55134,63391,63455,61374,63390,61342,61342,63390,65502,65535,65535,65535,63422,55036,52956,55036,57084,55036,52956,52891,48763,50843,50843,55036,57148,57116,55004,52955,38198,12744,10631,10663,8551,2244,8550,10695,10663,10663,10695,10663,10663,10695,10695,10695,10663,10695,12743,10695,10664,10695,10663,10663,10663,6502,4324,4325,2244,2244,4292,4292,2244,2244,4292,2244,2212,2211,2212,2212,2212,2211,2211,2211,2243,23243,31596,6371,14758,6371,8420,10565,6338,12678,4258,14790,4258,10597,4258,12678,8419,10564,12645,8484,12710,4258,14790,4226,14790,6339,12678,6339,8516,10564,6339,14758,4258,14758,4226,16871,6307,14791,6306,10597,10532,4291,14757,6371,18919,2146,12710,6339,14823,8419,12678,10532,8452,12645,2178,14757,4194,16935,23240,8546,2274,4322,6402,6370,6403,6467,6499,4354,2241,4354,4354,6467,10789,14983,35918,46287,54806,52726,50646,50678,46387,46485,48465,23336,19242,10626,8579,8546,8578,6369,4322,21257,25158,23372,56853,50678,50581,42228,44372,40114,35986,176},
{61343,61343,61343,59295,59263,59263,57183,57183,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,59263,61343,61343,61375,61375,61375,63423,61375,61342,61342,61343,61343,61311,59295,59262,59262,59263,57182,55103,55103,53023,50942,50942,50974,50974,48894,46782,46782,48862,46782,48830,48862,50909,52957,50942,50942,50974,55102,57182,57182,57214,57182,57182,55134,53087,59262,61310,61342,61342,61342,61342,63390,63422,65502,63390,63389,61277,52956,52956,55036,55004,55036,52956,52923,50811,50843,50843,55036,57148,55036,55036,52955,38230,12874,12842,10762,8681,2277,10697,12842,10761,8713,8713,8713,8681,8681,8681,8680,8648,8648,8648,8648,8616,8616,6535,6535,6503,2342,2277,2276,2244,2244,2212,2212,2244,2244,2212,2212,2211,2211,2211,2179,2179,2211,2211,2179,2211,19081,33709,6371,8451,6404,6404,6339,2178,8451,8517,8452,4226,6371,4259,6371,6307,4259,8452,8485,8419,6404,8451,2210,8419,6307,6371,4258,4259,8419,6372,8452,4292,8419,2178,8419,6339,8452,6307,4258,8419,6372,8419,4226,8419,4226,6371,6371,10532,8387,6339,6339,4258,8419,4258,6371,2146,10596,10659,8547,6499,6434,4322,4322,4354,6467,4322,4323,2177,2177,2274,8611,12740,19145,40013,31628,48500,48533,46452,48500,46355,46452,48334,38064,42225,12771,10691,8546,10627,6466,6434,27662,33610,19114,40114,40179,40147,38067,38066,40244,40245,181},
{61375,61375,61343,61311,59263,59263,57183,59231,59231,59263,59263,59263,59263,59263,59263,59263,57183,57215,59263,59263,59295,59263,59263,59295,61343,61343,61375,61375,61375,63423,61375,61342,61310,61342,61343,61343,59295,59263,59262,57215,57183,55103,53054,53022,48894,50942,50975,50974,48894,46814,46782,48862,46782,48862,50910,50909,50909,50941,50974,53022,55102,57182,57214,59262,57182,57182,57182,59263,59294,61342,61310,61342,61310,61310,61342,63422,63422,59229,57149,55069,52924,52956,55036,52956,52956,52924,50843,50811,50844,50843,55068,57116,55036,55004,52923,38198,10663,10761,12777,8551,6438,10598,10631,10631,10631,10631,10631,10663,10631,10631,10663,10631,8583,10663,10663,10631,10631,10631,10663,10663,8583,6470,4390,4357,4357,4357,4357,4325,4356,4325,4325,4325,4324,4324,4324,4292,4292,2276,4292,4292,21130,31661,6371,6339,4259,4259,4226,2146,4259,10597,6307,2146,4258,4258,4258,2178,4226,6371,10564,4259,8484,4258,2178,4226,4259,4259,2146,2146,4259,6371,4259,6371,4226,2178,4226,6371,6339,4226,2146,4259,6339,4259,2146,4226,2145,2178,8452,8452,6371,4291,4226,2146,4258,4258,4226,2145,8483,10628,8547,6435,6402,4290,2210,2242,4355,4387,2210,2242,6467,6499,8579,8547,21226,37835,33839,46452,44339,46419,46387,48500,48499,33512,42193,35724,8515,19144,17031,14852,12836,10789,36113,46288,31823,52790,48598,42228,33841,38001,31661,38066,195},
{61343,61343,61343,61343,59263,59263,57183,57183,59231,59263,57215,59263,59263,59263,59263,59263,57183,57215,59263,59263,59263,59263,59263,59263,61343,61343,61343,61375,61375,63423,63423,61343,59262,59294,61342,61343,59294,59262,59262,57215,55134,55102,55102,53022,48894,50942,50974,50974,48894,44734,46782,46814,44734,48829,50909,50909,50909,50942,50974,53054,55134,57182,59262,59262,57182,57182,59229,61310,61342,61342,61310,61310,61310,61310,61310,63390,61310,57116,55036,55004,50876,52956,57149,57182,52956,50876,50843,50811,50844,50875,57148,59197,57116,55036,52923,42424,8615,19179,14987,8551,4325,8518,8551,8551,8550,6503,6503,8550,8551,8550,6502,6503,6502,6502,8550,8550,6502,6470,4454,6502,6470,4422,6438,6437,4389,4389,4357,4357,6405,6405,4357,4357,4324,4357,4356,4324,4325,4325,6372,4324,16968,33709,6372,6371,6371,6371,4259,2178,6339,10565,6307,2178,4258,4259,4259,4259,4259,6339,8484,4258,8484,4259,4226,4258,2210,4258,2178,2146,4259,8452,4258,6404,4259,2178,4226,6371,6371,4226,4226,4259,6404,4258,2146,4226,2178,2178,6404,8451,4291,4291,4226,130,2178,4226,2178,2178,4290,8612,8611,6467,2242,2210,130,2242,4387,19179,14822,8579,10756,10755,10723,8580,33806,39981,42291,46452,44372,48565,48598,48500,48465,42062,40015,35756,10627,25383,29512,16867,14820,38097,50710,56885,46417,59030,54805,42195,27469,33775,21064,40212,204},
{61343,61343,61343,59295,59263,59263,59231,57183,57183,57215,59263,59263,59263,59263,57215,59263,57183,57183,59263,57215,59263,59263,61311,59263,59295,61343,61343,61375,61375,63423,63423,61375,61343,61343,61343,61343,59262,59262,59263,57183,55103,55103,53054,50943,48862,50942,50974,50975,48894,46782,46782,46781,44734,48829,48861,48862,50909,50942,53022,53054,55134,59262,59262,57182,57182,55069,55004,61278,61342,61342,61342,61342,61342,61342,61310,61309,59229,55069,52956,52956,50876,52956,59197,61342,55069,50876,50843,48795,50876,52923,59197,59229,57149,55036,52955,46650,8680,19083,19148,8551,4357,6470,6502,8550,8550,6502,8582,6503,8550,8550,6502,6502,6502,6502,6502,8551,8518,6502,6502,8518,6470,4422,6438,6437,4357,6405,4357,4325,4325,6405,4325,4324,6372,4324,4324,4324,4324,4325,4324,4324,12710,31661,10629,10532,4259,4259,6339,6404,4258,4226,6307,4259,4259,4258,4258,4259,4259,4291,4258,4259,4259,4259,4258,4259,4259,4258,4259,6339,4258,4226,4259,4259,4259,4259,4258,4258,4226,4226,2178,4259,6371,4258,4226,4226,4291,6306,4259,4291,4290,4258,4226,2178,4226,2146,4226,4226,4291,8612,6467,8547,4355,4387,162,2210,6435,31693,16836,8611,12835,10722,10723,12741,25256,33741,48533,44339,46485,48533,50711,44274,44206,42029,31626,21125,14883,10658,8513,8546,6370,33839,52725,56950,52657,59030,54838,42227,31728,35790,19017,40212,193},
{61375,61343,61343,59295,59263,57183,57183,57183,57183,57183,59263,59263,59263,57215,59263,59263,57215,57183,57215,57215,59263,59263,59263,61311,61343,61343,61343,61375,61375,61375,63423,61375,61343,61343,61343,59295,59263,59263,59263,57215,55103,53054,53055,48894,48894,50942,50974,53022,48862,46781,48829,46781,46782,48829,48829,50909,50909,50942,53022,55102,57182,59262,59262,57149,55102,52989,52956,57117,61342,63390,63422,61342,63422,61342,61342,59262,59229,57149,52956,52956,52924,55036,59197,61309,57181,50876,48795,48795,50876,50843,59165,61309,59229,57149,52956,50843,10761,14856,19017,8551,4389,6437,8550,8550,8550,8550,8550,8550,8550,8550,6502,8550,8519,8518,8550,8550,8550,8551,8550,8518,6470,6502,6470,6437,6437,6437,4389,4357,4357,6405,6405,4357,4357,4357,4357,4357,4325,6405,4357,6470,19147,31661,8419,6339,4226,2178,4291,8484,4259,2146,6307,2146,4258,2178,2178,4258,2146,4259,2146,4259,2178,4258,4226,4226,2178,2178,4258,4291,4258,2145,4226,2178,4226,2146,2178,4258,4259,2146,65,4226,4259,4226,2178,4226,4259,4258,4259,4259,4258,2146,4226,98,4226,2178,2146,2145,4355,8547,6467,4290,4322,6435,4322,2274,4355,35853,12643,6466,10690,10658,12805,21129,16903,42291,48500,44306,44372,48532,50645,39983,31496,31562,21124,12738,12770,6497,6465,12837,16869,23307,52692,56950,52657,59030,56918,46485,31564,31629,16936,44470,169},
{61375,61343,59295,59295,59263,57215,57183,57183,57183,57183,57215,57215,59231,59263,59263,59263,57215,57183,57215,57215,59263,59263,59263,59295,61343,61343,61343,61375,61375,61375,63423,61375,61343,61343,61343,59294,59262,59263,59263,57183,55102,53054,53055,48894,48894,50974,50974,50974,48862,46781,46781,46782,46782,48829,48829,50877,50909,50942,53022,55102,57182,57182,57149,52989,52989,52989,50876,52956,59229,63390,63422,63422,63422,63422,61342,61310,59230,59262,55037,52956,52956,55036,61310,61309,59229,52956,50843,50811,52956,50875,57116,61309,61277,57149,55004,50843,12842,6437,8485,8486,4357,6405,8518,8518,8550,8517,8518,8518,8518,6438,8485,8517,8485,6469,8517,8485,8486,8485,8517,8517,6438,6437,6437,6405,6405,8485,8485,6437,6405,8453,6405,6437,6405,6437,6437,6405,6405,6437,8485,12777,31727,29548,8452,6339,2178,2178,6339,8484,4259,2179,4259,2210,4259,4226,4226,4259,4258,4258,2146,4259,4291,4259,4259,2179,2178,2178,4259,6404,4258,2146,4258,2178,2210,6372,4291,4291,6371,4226,66,4226,2211,2178,2178,2146,4291,4291,4258,4259,4258,2146,4226,97,2178,2146,2146,2178,2242,8612,8579,4322,6467,4322,2210,4323,4290,29449,10595,6434,6434,4322,8549,18950,14823,37870,37935,44274,38001,40081,42128,40180,42127,33740,19110,10722,10690,6497,10754,12838,20997,19178,52725,56918,48366,33513,52725,46420,31598,29549,19017,38000,175},
{61343,61343,61343,59263,59263,57215,57183,57183,57183,57183,57183,59263,57183,59263,57215,59263,57215,57183,57183,57215,57215,59263,59263,59295,61343,61343,61375,61375,63423,61375,63391,61375,61374,61342,61343,59295,59263,59263,59263,57183,55102,53054,53022,48894,48862,50942,50974,50974,44766,44702,42654,44734,46782,48829,48829,48796,50876,50942,53022,55102,59230,57182,55069,52989,52956,52956,50876,50876,57117,61310,63422,63422,63422,63422,63390,61342,61310,61342,59262,52956,50876,57117,61309,61309,59229,52988,48795,50843,52988,52956,57116,61309,59229,57148,55004,50843,19148,4358,4390,8616,6470,6438,10631,10631,8582,8550,10599,10631,10599,8550,10630,8550,8550,8550,10598,10566,8518,8550,8550,8550,8518,8550,8550,6437,6437,8486,8517,6437,6437,8518,8485,6405,8485,6469,8485,8453,6437,8485,6437,21196,31727,27436,6404,6339,4226,2178,6307,8452,4259,6437,4291,6404,6371,4258,4259,4258,2178,4258,2210,4259,6339,4258,6404,4259,4226,2178,4259,6404,4258,2146,4226,4226,2210,6371,6339,4291,6371,4226,2178,4226,6404,4258,4291,4258,4291,6339,4258,4258,4259,4291,4226,2178,4226,2146,2146,4258,4323,12806,10692,4290,4386,6467,2274,4322,6436,25321,8515,8611,6466,4355,25452,18918,16936,14725,25486,42161,40145,31561,33774,52987,46321,31597,35951,19143,12802,6433,8674,12805,12611,17066,52757,54870,46222,21098,50677,48565,33644,27469,29451,35986,200},
{61343,61343,59295,59263,59263,59263,57183,57183,57183,57183,57183,59263,57183,57183,59263,59263,57215,57183,57183,57183,57183,59263,59295,59295,61343,61343,61375,61375,61375,61375,63423,61375,61343,61343,61343,61343,59262,59263,59263,57182,55102,53054,53022,48862,50910,50942,53022,50943,44734,40606,42687,42686,46782,48829,48829,48797,50877,50909,53022,55102,57182,57182,55069,53021,52956,50876,50844,52924,57149,61342,63422,63423,63422,63422,61342,63390,61342,61310,61310,55069,52924,55036,61277,61309,59197,52924,50843,52891,59229,57149,59197,59229,59197,57116,52956,50843,29910,15282,17395,23603,19443,19411,19410,19410,19377,19345,19345,19312,17264,15184,15119,13071,13039,13039,13038,13039,10958,13006,13005,10958,10925,10925,10924,8780,8682,8682,8649,8584,8583,8583,8583,6502,6502,8648,10631,8550,8583,10631,8551,25454,27469,21130,8517,8484,4259,2210,4291,6372,4291,8452,6339,6404,6371,4291,4259,4291,4259,4259,4259,4259,4259,4259,6339,4259,4259,4259,4259,6339,4258,4226,4259,4259,4258,6339,6339,4259,4291,4258,4227,4226,6371,6307,4259,4258,4258,6339,4259,6339,4259,4226,4226,2178,4226,2178,2178,4258,4258,8516,14821,8579,6435,10660,6467,4322,10628,21095,8547,12708,8514,8646,44272,19016,35985,21163,35984,29448,25256,14755,19212,61374,52660,27434,37900,46516,14883,8674,10818,10755,14887,14886,33705,48498,50415,27565,56918,50645,27338,27501,25258,38131,194},
{61343,61343,59295,59295,59263,59263,57183,57183,57183,57183,57215,57215,57183,59231,59263,59263,57215,57183,57183,57183,57183,59263,59263,61343,61343,61343,61343,61375,61375,63423,61375,61343,61343,61343,61343,59263,59263,59262,59263,57182,55102,53054,50942,48830,48829,50942,53022,50942,44734,42654,42686,44734,46782,48830,48829,50844,50876,50909,52989,55102,57182,55102,57182,55102,52957,50876,50876,52956,61310,63423,63422,63422,63422,63422,63390,63390,63390,61310,61342,59262,52956,55036,59229,61309,57117,50876,50844,50843,57116,59196,59261,59229,57116,55036,52923,50843,46683,44602,46682,48763,48763,50843,50875,50875,50875,52923,52956,50875,50875,46715,42587,42555,42555,42555,42555,42587,44635,46715,46715,44667,44667,46715,44667,40442,29943,25748,25683,25585,19375,17262,19310,17197,17164,36084,37968,35887,33807,33872,29679,31759,27501,23242,8485,8484,4226,2146,6339,10565,4259,8517,6339,8485,6372,6339,4226,6339,8485,4258,2178,4259,6404,6339,6339,6339,6339,6404,4226,2146,4226,2114,4226,2146,4226,4226,4226,4226,2178,4259,8485,4258,2146,4226,2146,2146,2178,2146,4259,4258,6339,4226,4226,4291,6307,4259,4258,2178,2145,8549,19047,6402,6467,8514,4354,4290,6402,23208,10595,10627,8547,21291,52593,27468,46452,44339,31595,21159,10627,8548,30006,65434,54706,23144,25158,35887,27434,21222,17060,12835,14820,6433,10593,25418,48334,29548,54771,50645,31531,33872,40179,35920,180},
{61343,61343,59295,59263,59263,59231,57183,57183,57183,57183,57215,57215,57215,59263,59263,59263,57215,55103,55103,57183,57183,57182,57182,59263,61343,61343,61375,61375,63423,63423,61375,61343,61343,61343,61343,59295,59295,59263,59263,57183,55103,53022,50942,48830,48829,50942,53022,50942,46781,46781,48829,46782,44734,46782,48797,48796,50844,50877,50941,55070,55102,55070,57182,57182,53021,52924,52924,52956,61310,65535,63423,63422,63422,63390,61342,63390,61310,59229,61277,57149,52924,55036,59197,59229,57117,52924,50876,50876,52956,55036,59197,59229,57149,55036,52923,50843,48763,46650,46650,48763,50811,50843,52891,52891,52923,52955,55003,52923,50875,48763,44504,44472,44504,44536,44536,46616,46616,48697,48665,46649,48697,48697,48729,42424,38197,29811,25618,23504,21422,21390,19277,19276,21293,40178,42063,42063,42030,40080,35822,25355,25290,23242,8550,8452,4226,2146,4259,8549,4291,8518,6339,8485,6404,4259,4226,6307,10565,4259,2146,4258,6371,4259,6339,6371,4291,8484,4226,2146,4226,2114,4226,2146,2146,4258,4259,4226,2146,4258,8485,4258,2146,2178,2146,2146,4226,2146,4259,4259,6307,2146,2178,6307,4226,4259,4258,2177,97,6403,14820,6499,10627,4321,4354,2274,8547,14788,10595,8579,12901,19048,50480,42226,46420,42095,16741,16967,12773,10857,57213,61109,56951,21096,14790,33775,25288,23206,19077,19077,10626,6498,4385,16999,44109,10499,25290,50645,25290,35985,44405,42324,196},
{61343,61343,59263,59263,59263,57183,57183,57183,57183,57183,57183,57215,57183,59231,59263,57215,57183,55103,55103,55102,55102,55102,59230,59263,61343,61343,61343,61375,61375,63423,63423,61375,61343,61343,59295,59295,59263,59263,59262,57182,55103,53022,50942,48862,48861,50910,50942,50909,50876,50877,48829,44734,42686,44734,48797,50812,50844,50908,52989,53021,55102,55102,57182,57182,52989,52924,50875,52956,59229,63390,63390,63390,63390,61342,61342,61310,61277,57117,57117,57181,52988,55004,57117,59229,55069,52924,50876,50876,52891,52923,57116,59196,57116,55004,50875,50843,48763,46650,46650,50810,50811,50843,50843,50843,50843,54971,55035,52923,50843,48698,23210,23178,23179,23178,23178,23210,23210,25226,25226,25258,23210,25226,25258,23178,21065,21033,19017,19017,18984,18984,16936,18952,18952,23112,23079,23047,25127,23080,20999,23242,23112,21195,25585,12776,6404,6437,10598,10597,12678,10565,12710,8517,12710,8484,8517,10565,10597,10630,6372,10630,8484,10630,6436,10597,8517,10565,10565,4291,8517,4291,8582,4291,8517,6404,6404,8485,4291,8517,8484,10630,4291,8517,6339,6404,6404,6371,8517,6371,10597,4291,8517,6371,8452,6371,8452,8452,4291,10694,10692,8547,6434,6499,6467,6467,8611,8514,4386,6466,23337,16805,46288,46420,46419,31466,10533,19081,37967,36148,65401,54739,56984,23112,10597,27501,33904,21225,14949,16997,10692,12803,6465,10790,42160,12677,31792,52757,31628,38031,42193,50678,145},
{61343,61343,59295,59263,59263,57183,57183,57183,57183,57183,57215,57215,57215,59263,59263,59263,57183,55135,55103,55102,55135,57215,57183,59263,61343,61343,61375,61343,61375,63423,61375,61343,61342,61343,59295,59263,59263,59263,59263,57183,55102,53054,50974,48862,48862,50942,50942,50909,50877,50877,48861,44734,44734,44734,48829,48796,50844,50909,52989,55070,55102,55102,57182,57150,52989,50876,50876,52924,57117,59229,59229,61310,63390,61342,61310,61309,59197,57117,57117,57149,57149,55069,57149,59229,55069,50908,52924,52924,52924,52923,55036,57116,57084,55004,50875,50843,50843,46650,46650,48763,48730,50843,50843,50843,50811,50843,54971,52891,50811,46617,19082,19082,19051,19050,19050,19050,17034,19050,19050,19050,19083,19050,19017,16937,14889,14857,14857,14857,12776,14824,14792,12744,12744,14792,14791,12743,12743,12743,12711,25323,20999,19083,38132,21162,8452,4226,14758,6371,16871,4291,14758,8451,12645,10532,6306,14758,4226,16871,4194,14758,4226,10597,8452,8419,12677,2178,12710,2113,12710,4226,12710,6339,6404,10565,4291,14791,2146,14759,4259,14824,4259,10662,8452,4324,14758,2211,16904,4259,16903,6339,12743,6339,6405,10565,4259,14790,2178,16967,10628,6500,6468,8547,8612,8612,8612,6467,6499,4387,16903,14693,42226,46387,44241,16773,8486,10564,33709,52889,65401,52723,57017,29418,6306,14888,40179,38032,27563,23271,16966,14915,12772,17195,54869,27467,50742,54837,33671,23267,23303,35817,53},
{61343,61343,59263,59263,59263,57183,57183,57183,57183,57183,57215,57215,59231,59263,59263,57215,57183,55103,55103,57151,57183,57215,55102,57150,61343,61343,61343,61375,61375,61375,61375,61375,61343,61343,59295,59263,59263,59263,59263,57183,55103,53055,50974,48862,48862,52990,50942,48862,50877,50877,50909,48830,44734,46782,48829,48829,50844,50909,52957,52989,53021,55037,55069,55037,52924,50844,50844,52924,55036,55037,57149,61342,63390,61342,61310,61277,59229,55069,57117,59197,59197,57117,57149,59197,59197,57149,55036,52956,52924,50876,52955,55004,55036,52956,50875,50843,50843,46650,46650,48763,48730,50843,50843,50811,48763,48730,37968,37935,35888,31695,19083,19083,17035,19051,19050,17003,17035,19051,19050,17003,17003,17002,18986,16970,14792,14792,14791,12711,12678,14759,14727,12646,14726,14758,14727,12646,14726,14726,12678,23243,16871,21228,23276,14726,6339,4258,8419,6372,8451,6339,8419,6372,8420,8419,4291,8452,4324,10500,4259,8419,6339,6371,8387,6307,8419,4291,8451,4292,8451,4291,16969,21194,21228,29778,27698,33939,27697,31826,27665,31891,31891,31924,31923,31924,36052,31892,36052,31892,36052,31892,34004,31891,29843,33972,31892,36052,31891,36116,34036,34004,34036,34036,34036,34036,34035,34036,34036,34036,36083,33840,29450,31465,29352,16871,16903,16870,27337,50644,59030,61173,56983,33676,8452,6339,33905,54904,37901,44435,38031,29643,23305,38227,56918,44303,59030,52624,25282,19138,21186,19105,41},
{61343,59295,59263,59263,59263,59231,57183,57183,57183,57183,57215,57183,57183,57215,59263,57215,57183,55103,55103,55103,57183,57183,55102,57182,59263,59262,61343,61343,61375,61375,61375,61375,61375,61343,59295,59263,59263,59263,57215,55167,55103,53054,50942,48862,50910,52990,50942,50910,50877,50877,50877,46782,44734,46781,48829,50877,48829,50909,52925,52957,53021,52989,52989,52989,50876,50844,50843,50876,52956,52924,57084,61310,63390,61342,61310,59229,59229,57117,55036,57116,57116,57084,59197,59197,57149,57149,57116,52956,52924,50875,52955,52923,55004,52956,52923,50843,50843,46650,46650,48731,46682,48763,52923,50843,48763,44504,23211,21130,23211,21130,14889,16937,14889,16937,16937,16937,14889,14889,16905,14889,12809,14857,16905,14889,10664,12712,12744,10598,8550,12710,12679,8518,10598,10630,10599,6470,10598,10630,10598,21162,14822,19115,29714,12711,4259,4226,4291,8517,6339,4226,4258,6404,6404,4258,4258,6307,6372,4258,4259,4259,6372,4291,4226,2178,4258,6372,2210,8484,6339,6470,29549,33676,25159,16707,18788,20932,27337,35724,20901,14627,18820,18852,18820,18852,18821,18788,18821,18853,18788,18820,18820,18788,18820,18788,18821,16772,16740,16772,16772,16708,18852,16772,16707,16772,16708,16707,16740,14659,16740,23079,25225,23047,12547,14693,16805,12643,14820,52588,56984,48498,59028,39980,10563,4355,23338,44108,23140,31560,40078,44141,35951,52757,56918,50544,59030,50478,21185,21251,19104,19137,45},
{59295,59263,59263,59263,57215,57183,57183,57183,57183,57215,57183,57183,57183,57215,57215,57183,57183,55103,55103,55103,57183,57215,57215,57183,57182,55037,59230,61343,61343,61343,61343,61375,61343,61343,59295,59263,59263,59263,57215,57183,55103,53054,50942,48862,48862,50942,50942,48862,50877,50877,50877,46781,46782,48830,50877,50877,50876,50876,50876,50876,52989,53021,55037,52989,52956,50876,50844,50876,52956,50844,54972,59165,61310,63390,61342,59261,59197,55069,55036,55036,55004,55036,57117,57149,57148,57148,55068,52956,52924,52923,52924,52923,52955,52956,52923,50843,48763,46650,46650,48730,46682,48730,42194,42194,40146,33840,19083,19083,19083,17002,19083,19051,17035,17003,19050,17003,17003,17003,17002,17002,16970,16970,16970,14954,12744,14792,14759,12647,10598,12711,12679,10598,10630,12678,12679,10566,12646,12678,10630,25323,14823,21195,35987,16904,4226,2146,4259,8517,6339,4226,4259,6372,8452,4258,4291,4259,8452,4259,8484,6339,6404,6372,4226,2146,4259,8485,4259,8485,6339,16904,35822,33676,27371,19214,19246,19246,27600,38001,21261,12940,12972,15052,15021,15020,15052,15020,15053,15053,15020,12972,15020,15020,15085,15053,15085,15085,15053,15052,15053,15053,15117,17165,17101,15085,15085,15053,17165,15117,17165,23439,27338,25160,12580,14726,12676,8547,25476,31459,54937,48564,59130,48401,20995,29508,25350,18979,8481,6401,19145,39948,31758,54772,56918,50544,56950,54771,27460,25411,16992,21249,52},
{61343,59263,59263,59231,57183,57183,57183,57183,57183,57183,57183,57183,57183,57183,57183,57183,57183,55103,55103,57151,57183,59263,57215,57183,55102,52957,57150,61343,61343,61343,61343,61375,61343,59295,59263,59263,59263,59263,57150,57150,55102,53022,50942,48862,48862,50942,50942,48862,48830,48829,50909,50909,48829,50909,48829,50877,50876,50876,50876,52924,52989,55070,55070,52989,52956,52924,50844,50875,50844,50844,52924,55036,61277,63390,61310,61309,57149,55069,52988,52956,52956,55036,57149,57117,57116,57148,57116,52956,52924,52924,52923,52923,52923,52923,52923,50843,48762,46650,46682,48730,46650,44439,23211,21163,23211,21130,21130,19082,19083,19050,21130,21098,19082,19050,19082,21130,19050,19050,19050,19050,19050,19050,19018,19018,16904,18952,16904,14824,16872,16904,14824,12743,14823,14824,14792,14791,14791,16839,14823,27436,14791,19017,25356,12645,4259,2178,4291,8550,6339,6404,4323,6404,8484,4259,6371,4291,10565,4291,8484,6371,6371,6404,4259,2179,4259,8484,4259,8485,8419,14694,31531,35757,29451,33643,33675,35724,37902,42096,35756,31596,35822,37902,37935,35854,35854,35854,37902,37902,35822,35854,35822,35790,35822,35822,35822,35822,35790,33709,33709,33709,35789,35789,33677,33709,33709,33676,33709,33644,33709,40015,31466,25225,14628,14693,14724,10564,35848,35754,46517,48532,59162,54741,12643,14850,14818,10626,6434,6466,14854,31464,25419,52692,54772,48432,54869,56917,42222,23331,16993,14944,72},
{61343,59263,59263,57215,57183,57183,57183,57183,57183,57183,57183,57215,57183,57215,57183,57183,55103,55103,55103,55103,57183,59263,57215,55102,53022,52990,57150,61343,61343,61343,61343,61375,61375,61343,59295,61311,59231,55102,52989,55038,53022,50974,48862,46814,48894,53022,50942,48862,46781,48829,48861,50909,50909,50909,50877,52989,50909,50876,50876,52956,52989,55069,55070,52989,52956,52924,50843,50843,50843,50843,52924,55036,59229,61310,61310,61310,59229,57149,55036,52924,52892,55036,55036,55036,57116,57117,55036,52956,50875,52956,52923,52923,50843,50811,50843,50843,46682,46682,48731,48731,46682,35986,21163,19083,21163,21131,17003,17034,17067,14987,17002,17003,17003,14955,17002,17003,17003,14922,16970,16970,14922,16970,14889,17002,8518,12678,12678,10566,10566,10630,10598,6437,10566,12646,10598,8485,10566,10566,10566,23210,14791,23275,19082,10565,4259,2146,4291,10630,6371,6404,4291,6404,8517,4259,6404,4291,6339,4259,8517,8419,6404,8484,4259,6404,4259,6404,4259,8484,10500,31565,33579,35724,27273,6372,4292,8453,16904,33677,16838,2147,2147,4259,4259,4259,4259,4227,4259,4259,2179,4259,4259,4227,4292,4259,4259,4291,4259,4292,4292,4292,4324,4356,4259,4356,4324,4292,6469,6405,8518,21130,25257,27241,16774,16806,14725,16838,29447,52426,42226,48598,52791,54807,16772,10692,10691,8578,8546,8578,6468,10532,23339,52756,54772,56884,52723,56917,50644,33901,21251,17025,91},
{59263,59263,59263,57183,57183,57183,57183,57183,57183,57183,57183,57183,57183,57215,57183,57183,55103,55071,53055,55135,57183,57215,57183,57183,55070,55070,57150,59230,61343,61343,61343,61375,61343,61343,59295,59263,57182,52957,52989,55070,53022,50942,48862,48862,50910,52990,50910,46814,46781,46781,50877,52957,52957,52957,52957,52989,52957,50909,50908,52956,52989,55069,55070,55037,52957,50908,50876,50843,48795,50843,50876,55069,59229,61309,63390,61342,61277,57117,55036,50876,52924,55036,55036,55036,59164,57116,55004,52956,52924,52956,52924,52956,50843,48763,48730,48730,46650,46682,48730,48697,46617,25324,21163,19050,19082,19050,16938,16937,16970,16970,16938,16937,14890,16937,16937,16969,14889,14857,16905,16937,14857,16937,16905,16937,12711,12743,14791,12711,12711,12743,12711,12711,12711,12711,10663,10630,12711,12711,12679,23210,16903,18984,21261,12678,6339,4259,6339,6339,6339,6307,6339,6339,6339,4259,6339,6339,4259,4259,4291,6339,6339,6404,4259,4259,4259,6339,6307,6339,10533,20967,33482,35724,33645,40211,42324,40212,42260,42259,40179,38197,38262,40310,42423,40310,38230,40310,38262,40342,40342,42423,42423,42390,42422,40374,42422,42390,40310,40309,40341,42422,42455,42422,42390,42422,42358,42357,42358,42357,42357,44437,35757,27273,18886,23112,23144,23112,25224,52590,56947,52889,59195,54904,20999,16936,12772,12773,10658,10659,12871,29645,29580,56884,52724,58964,50511,54641,52626,48530,29539,19138,110},
{59263,59263,59231,57183,57183,57183,57150,57183,57151,57183,57183,57183,59231,57215,57183,55135,55103,55071,55071,55135,57183,57215,59231,57183,55102,55070,57182,59230,61343,61343,61343,61343,61343,59295,59262,59263,55102,52957,52989,52989,52990,50942,48862,48862,50909,52957,48862,48829,48829,48829,48829,50877,52957,52957,52989,52989,52989,52989,52957,52989,52989,55037,55069,53021,52956,52956,52892,50843,48763,50843,52956,55069,59229,59229,61341,61277,61277,57117,55068,52956,52956,52956,55036,55036,57116,57116,55036,52956,52924,52923,52955,55004,52923,50843,48763,48730,46650,48730,38067,27404,25291,19018,19083,19082,17034,17035,17035,19050,19083,19083,17034,19050,17035,16971,17002,17035,14955,14922,19018,17002,16970,16970,19017,16970,10663,14759,12711,12679,10630,12711,12711,10598,10631,12710,10630,10566,12679,12679,10598,21064,18951,18984,31793,16937,4259,4259,4259,2178,4259,4258,4258,4258,4259,4259,2178,6307,4259,4291,2178,4259,6371,8484,4259,2178,4259,4226,4227,2178,17035,14856,29386,31531,29386,12482,12482,16675,20998,35724,23047,10370,10402,12547,12547,12514,12547,12514,12547,12547,12482,14595,12546,12482,14595,12547,12482,14595,12514,12482,12546,12514,12514,12546,12482,12514,12514,10402,14595,14659,10402,14628,23112,25193,16741,12580,14660,16741,20934,46387,56918,58999,59032,54937,25160,14758,10662,38161,14819,12772,25582,52724,37998,54805,54740,46155,20898,12545,27369,50709,33831,21218,63},
{59263,59263,59231,57183,57183,57183,57183,57183,57183,57183,57183,57215,59263,57215,57183,57151,53055,53023,55071,55103,57183,59263,59231,57215,55102,55038,55102,59262,61343,61343,61343,61343,61343,59262,59230,59263,53022,50876,48796,50876,50909,50909,50877,50909,50877,50909,50909,50877,48829,50877,50909,50909,52957,55005,55069,55069,52989,52989,52989,52989,52957,55037,55069,52989,52956,52924,52892,50843,50811,50811,52956,57149,59230,59229,61310,59229,57116,55068,57116,55037,52956,52956,52956,55004,55036,57116,57148,55068,52956,52924,55004,55004,52923,50843,50843,48763,46650,48698,29615,21195,19051,21066,19116,21131,19051,19083,19083,21099,19051,21099,19050,19082,19082,19051,19050,19050,17002,17002,19018,19018,19018,19050,19017,19018,16839,16839,16871,16839,14758,16839,16839,14759,16839,16839,14791,14726,16839,16871,16806,16806,16838,16871,36019,21162,4259,6339,6339,2178,4259,4259,4291,4259,2178,4259,2178,4259,8485,6339,4226,4258,6372,10532,6339,4259,4259,2146,4258,4226,27436,23309,31532,33612,31564,21391,23505,23504,27665,38034,31761,19311,17231,19311,19311,19279,19343,19311,19311,19311,19311,21392,19343,19343,19344,21391,21391,21424,19343,21392,21424,21424,21424,23472,21424,23504,25585,23440,25553,27665,23473,25585,29549,25225,18885,12645,16773,16773,20967,44307,52791,48566,57115,54905,27273,8517,10662,46288,20739,10662,46484,44077,37834,39850,41929,14657,6434,4258,6435,44470,52724,21126,39},
{59263,59263,59263,57183,57151,57151,57151,57151,57183,57183,57183,57183,57183,57215,57183,57151,53023,53023,53023,57151,57183,59263,59263,59263,57215,57183,57150,59230,59262,61311,61342,61343,61343,57215,57150,57150,55038,50877,50844,50844,50876,50876,50876,50876,50876,50877,48861,48829,48829,50909,50909,52957,52957,55037,55069,55069,55037,55037,52989,52989,55037,55037,52989,52956,52956,52924,52892,50843,48763,48763,50843,57117,59197,59229,61310,61277,55036,55036,57117,55036,52956,52956,50843,52924,55004,55036,57148,57116,55036,52955,55003,55003,52956,50875,50843,50811,46682,46617,23276,21195,19018,16937,12809,16937,14857,12809,16937,16905,14857,12776,14857,14857,14857,12809,16905,14889,12809,12777,14824,14857,12777,14857,14824,12776,8518,10631,10631,8518,6437,8550,10598,6405,8518,10598,8518,4324,10566,10598,6437,6372,10566,10598,38099,23275,8452,8485,6437,6372,6404,6372,8485,6372,6404,6436,6404,6404,8517,8485,6371,8485,8452,10565,8485,10565,6372,4259,4324,8419,37968,38067,37870,35725,31563,27304,31433,31497,33611,37935,33676,27304,31499,33643,31563,33643,33676,33643,31563,31563,33643,33643,33611,33579,31563,31563,31530,31562,29450,31466,31530,31498,31498,31498,31498,31498,31530,31498,31498,31530,29417,31530,31531,25225,20934,14661,14693,16773,18918,40015,52692,46452,54838,52824,31563,12710,12677,23047,23213,42357,40047,21064,18919,14758,18917,8514,6434,6402,4387,42422,56886,37869,67},
{59263,59263,57215,57183,57151,55103,55103,57183,57183,57183,57183,57183,57183,57183,57183,55103,53023,52991,53023,55103,57183,57215,57215,59263,59263,59263,57183,57150,59262,59262,59262,59263,61311,59263,55070,52957,52957,50876,50844,50844,52924,52924,50876,50876,50876,50876,50877,50877,50876,50908,52957,52957,55037,55069,55069,55069,57117,55037,55037,52988,52989,52989,52956,52956,52956,52924,50876,50843,48763,48731,50843,55036,57149,59229,61309,59229,55036,52988,57117,55004,55036,52956,50843,52923,55004,55036,57116,57116,55036,55004,52956,52923,52923,52923,52891,50811,48698,40278,21163,21196,19018,16872,14759,16807,16839,16840,14791,16872,16872,16807,16839,16839,16839,16839,16840,16839,16839,16839,16839,16871,14759,16871,16839,16871,16807,16871,16872,14791,14758,14823,16839,14758,16839,16839,14791,14759,16839,16839,14791,16839,16839,16839,25257,25323,12645,8452,16903,4291,12710,6404,10630,10597,8452,12710,6339,14823,4291,12742,6372,10565,10565,6371,14758,4259,12710,4259,12710,14693,42194,38001,39886,33612,31531,8517,6470,8551,12744,33710,25291,2244,2212,4325,4325,4293,4357,4357,4325,4325,4357,4357,4357,4325,4357,4357,4357,4389,4357,4325,4422,6437,4358,4422,4390,6406,8615,6470,8551,12744,6470,8551,14889,25225,20966,14661,14726,14725,18886,23143,21029,27501,52792,50712,37869,12645,16838,12613,40310,50548,42258,29450,16903,19114,18884,6434,4322,4290,6402,29712,54839,42128,73},
{59263,59263,57215,57183,55103,55103,55102,57151,57183,57183,57183,57183,57183,57183,57183,57151,53023,52991,52991,55103,57183,59231,59263,59263,59263,59263,57215,57150,59198,57182,57150,59230,59262,59263,52957,52924,52925,50844,50876,52924,50876,50876,52956,52956,50876,50876,50876,50876,50876,52956,52957,55037,55069,55102,57149,55101,57117,55069,55037,55037,55037,55036,52988,52956,52956,52956,52924,50843,50811,48730,48763,55037,57149,59197,61277,59229,55069,55037,55004,55036,55036,52988,52956,52956,55004,57084,57116,57116,57084,55036,55004,52923,52923,52891,50843,50811,46650,33874,21163,23244,16970,12744,4455,10599,10631,8551,8518,10631,10631,6470,10599,10631,8551,6470,10599,12647,8518,8518,10598,10599,6438,10598,10598,10599,6437,8550,10566,8486,6405,8518,8518,4324,8485,10566,8486,6372,10534,10565,8485,10631,19116,17068,21227,23210,16871,16871,18984,12743,16969,14791,16969,16871,14888,16936,14824,16968,12743,16968,12711,16936,16937,14856,16871,14888,27533,27631,29647,33840,44341,31695,33579,33644,33644,35920,42259,42292,42259,42259,44340,38163,42357,42390,44503,44503,44503,44502,42390,42422,44535,44470,42389,42389,42390,44438,42422,44470,42357,42357,42324,42356,42357,44405,44405,42292,42292,42292,44372,42324,42291,40211,40145,27273,23079,18886,23080,23112,27272,29451,42389,36050,52922,54905,42096,12645,16806,8484,33872,33643,52725,20966,12776,29614,14724,6402,4322,4290,6403,23406,61243,59162,86},
{59263,59263,57215,57151,55103,55071,55103,55103,57151,55135,57183,57183,57183,57183,55135,55103,53023,50943,53023,55103,57183,57183,57183,59263,59230,57150,57182,57215,55102,57150,55102,57150,59198,57182,50909,50876,50876,50844,50876,50876,52924,52924,52924,52956,50908,50876,50876,50877,50876,52957,52989,55037,57149,57149,57149,57117,57149,57117,55037,55037,55037,55037,52988,52956,52956,52956,52924,50844,48763,48730,50811,55037,59262,61310,59229,61310,59229,57149,52988,55036,55036,57117,55069,55036,55036,55068,57116,57116,57116,57084,55036,52955,52923,52891,50843,50811,46650,29616,21131,21162,21130,19050,19083,19050,19050,19050,16970,16970,19017,16970,16938,16937,16937,16937,16937,16937,16937,14889,16905,16937,16905,14857,16936,14856,14856,14824,14824,14824,14824,14824,14824,14792,14792,16872,14824,12744,14824,14824,14792,25356,46451,46484,44273,25225,12710,14726,14758,12678,14726,12678,14758,12678,14726,14758,14823,14758,10565,12645,10597,12677,25259,14726,12678,23210,40080,42226,44274,44306,40016,33744,20967,31531,31564,10434,10369,12514,14659,31596,29385,8289,8257,10402,10402,10369,10434,10402,10369,10434,10402,8354,10402,8354,10402,10434,8321,8354,10402,8321,10434,10434,8321,10402,8354,8353,10498,8354,8386,14692,10370,8354,12612,25192,23079,10499,12612,14692,18853,29484,54839,50613,50744,50711,46322,14790,18951,31825,27468,33775,46321,35952,23305,12707,14724,6434,4289,4290,16772,17033,52987,65469,112},
{59263,59263,57183,57183,55071,55071,55103,55103,55103,55102,57183,57183,57183,57183,55135,55103,53023,50943,52991,55103,57182,57183,57183,55070,55037,55037,57182,59263,57183,57182,55134,55102,55069,55102,50941,52956,50908,50876,50876,52924,52924,50844,52924,52956,52956,52956,52924,50876,52892,50876,52957,55037,57149,57149,57149,57149,57149,57117,55037,55037,55037,55069,55037,52988,52956,55004,52956,52892,48763,48698,50811,57117,63422,63422,61374,59262,59229,59197,57149,55036,57117,57117,57149,55069,55069,57149,57148,57116,57116,55036,55036,52956,52923,52923,50843,50811,46650,29713,21131,21163,21163,21163,23243,21130,21163,21163,19050,19082,21130,21131,21130,21098,21130,21130,21098,21130,19082,21130,21098,19082,19082,19050,19050,19017,16937,18985,18985,16905,16905,18985,18985,18985,19017,18985,18984,16904,16937,19017,18985,27468,46419,48499,44273,20998,12678,12677,12677,10597,12678,12709,12710,12678,12677,12677,14790,12710,10597,10597,10598,10565,12645,10564,10565,25290,40048,44273,44274,44274,33645,23179,16839,31563,31532,25487,27698,27698,27698,36019,35986,21392,21392,23537,23537,23537,23537,23537,23505,23505,23537,23537,23537,21456,23505,21457,23537,23570,25617,23537,25650,23537,25617,25682,23569,25585,27698,27698,25682,29810,27698,25617,27665,27338,25160,12612,14725,16773,18854,27370,54805,48500,48696,59130,48532,21063,18919,33775,27436,35888,33774,52790,33641,12739,12676,6402,4322,6370,4323,23666,63487,61112,100},
{59263,59263,57215,57183,55071,55071,55071,55103,55103,55102,57183,57183,57183,57183,55103,55071,52991,50942,52991,55103,57183,57183,55102,52990,53022,55070,57150,59231,59231,57150,57150,55102,52989,52989,52989,52925,50876,50876,52924,52924,52924,52924,52956,55037,52957,52956,52956,50876,50876,52924,52924,55004,57117,57149,57149,57149,57149,55069,55069,55069,55037,55037,55036,55004,52956,55036,52956,52923,48763,48730,48731,55036,61310,63390,63422,63455,59262,59197,57149,55068,57117,57117,57117,57149,59229,59229,57149,57149,57116,57116,55036,52955,52923,52923,52891,50811,46650,31891,21163,21163,21163,21163,23243,21131,21131,21163,21130,19082,21131,21130,21130,21130,21130,21131,21130,21130,21130,21130,19082,21130,21131,21098,19050,18985,16937,16969,18985,16937,16937,16969,19017,19017,19017,18985,16937,16936,19017,21097,21130,27501,46419,48532,44241,16838,12677,12677,12678,12678,12710,12742,12710,12710,12710,12678,14823,14790,10597,12645,12677,10597,10597,10565,12678,25258,37968,44242,44241,44274,29516,14759,16806,31563,31564,25159,27207,29287,29353,35789,35724,23079,27273,31498,31530,31466,31498,29450,29418,29450,29450,29386,31466,27337,27337,29385,27337,29385,29385,27304,29385,29385,27272,29450,29385,27240,29384,27304,27272,29385,25159,25191,29385,27305,25192,14660,14725,16773,16806,20998,37835,44339,48631,61178,52725,27239,18918,27436,27370,16903,33871,44075,25188,8514,10596,6434,6402,6370,13036,55166,65534,65502,238},
{59263,59263,57215,57151,53023,55071,55071,55103,55103,55103,55103,57151,57151,57183,55103,53023,50943,50942,52991,57151,57183,55134,53022,52990,55070,55102,57182,57150,57150,55102,55102,55102,52990,52990,50909,52924,50877,50876,52892,52924,50876,52924,55037,55069,55037,52956,52924,50876,50844,50844,52924,52989,55037,57149,57181,57149,57149,57117,57117,55069,55037,55037,55036,55036,55036,55036,52956,52891,50811,46682,48731,55004,59197,59229,63390,63454,59230,57117,55069,55036,55036,57084,57149,57149,59197,61309,59229,59229,59197,57116,57084,55004,52923,52923,50843,50811,46650,36216,27764,27765,27732,23439,19082,19050,19050,19018,19018,19018,19050,19050,19018,19050,19050,19050,19050,19050,19050,19050,19050,19050,19050,19050,19017,16969,16969,16936,16905,16905,16904,18985,18985,18985,18985,16937,16936,16904,16936,21097,21065,31629,46419,48531,42161,14758,10630,12678,12710,12710,12678,12710,12678,12677,12677,10597,14823,14790,10598,10597,12678,10598,10629,10597,16936,23145,40048,44274,44274,44274,31597,10663,23210,31499,31564,14759,8583,8616,10728,27501,31597,6438,2277,4390,6438,4357,6470,6470,4358,6470,4390,4358,6470,6470,4422,6502,6502,6470,6502,6470,6503,8583,6503,10729,12776,6503,8615,8615,6535,10728,6568,6568,12809,25258,29418,14692,16773,16773,16806,18918,23307,46518,46550,59097,50580,27239,16806,21097,35854,23373,46484,14753,6465,6465,6402,6434,6402,8711,38393,65535,65534,65502,254},
{59263,59263,57215,57183,53022,53023,53022,55071,55103,55103,55103,55103,57150,57151,55103,53022,50943,50910,52991,55102,57183,57150,55070,55102,55070,57150,57182,55102,55069,55070,55069,57150,55102,52989,50909,50877,50876,50844,50844,52924,52924,52956,55037,57085,55037,52988,52956,50876,50844,50844,50844,52956,52956,57117,59198,57149,57149,57117,57117,55069,55069,55037,55036,55036,55036,55004,52956,52891,50811,48730,48731,52924,57117,59229,63390,63422,59229,55004,57149,55037,52956,55036,61310,61342,59197,59228,61277,59229,57149,57148,57084,55036,52923,52923,52891,50811,46682,44602,46650,46651,46650,36052,23276,23243,21196,21163,21163,21163,21195,21163,21195,21163,21163,21163,21163,21163,21163,19114,21130,19083,19115,19083,19082,19050,19050,16970,16970,16938,14889,19017,16969,19018,16969,16937,16904,16872,16904,16936,18985,27469,42161,44338,44273,25485,21325,25486,21325,25486,21228,23372,23372,25453,19081,17001,23242,19049,21161,19017,21161,19016,19081,19048,23274,21163,38000,44274,44274,44273,33742,25585,37870,33579,33644,33743,40146,42259,42291,42259,42227,38066,40244,44438,42357,42325,42357,44437,42389,42325,42325,42325,42357,44438,44437,44437,44437,42292,40244,40244,44373,42292,42291,42292,42291,42259,42260,42324,42292,42291,42291,40211,40178,33644,27241,16773,20966,20999,23112,23047,37903,42095,44306,52693,50775,31497,18951,14823,38000,27467,33641,8481,6498,6433,4321,4354,6434,10791,61407,65535,65535,65534,253},
{59263,59263,57183,55103,53022,53023,53023,53023,55071,55103,55103,55103,55103,55103,55071,52991,48863,50910,50943,57150,57183,57183,55070,55102,57150,57118,55070,52989,52989,52989,55037,55102,55102,55102,52989,52957,50844,50844,50876,50876,52956,52956,55005,55005,55004,52956,52924,52924,50844,50844,50876,50876,55005,57117,57149,57149,57149,57117,55069,57085,55069,55036,55036,55036,55036,55036,52956,50843,50811,48731,48730,50843,57117,61277,63390,63390,55037,55069,57181,57181,52956,55036,61310,63422,59229,57116,59197,59229,57149,57116,57084,55004,54971,52923,52891,50810,46650,46618,46650,48698,46682,42424,27699,27699,27699,25585,21130,21130,21130,21130,21130,21130,21130,21130,21130,21130,21130,21130,21131,21131,21162,21130,21130,21098,19050,18985,19017,19017,18985,19017,19017,19017,16904,14791,12678,10598,10631,14792,14824,27468,42193,44306,46418,46419,46419,46419,46419,46451,42225,44337,48531,52724,27369,29449,27303,29449,29449,27336,29384,31497,29417,29384,33644,23276,37967,44274,44274,44274,42226,40178,27175,31498,33644,16708,8321,8354,14595,27338,33643,10434,6209,10402,10434,10369,10402,10402,8289,10402,8354,8321,10434,10402,8321,10434,8354,8321,8386,8321,8354,12547,10434,10500,16773,10402,10467,10499,8354,8354,8419,8386,8386,21064,25160,12515,4161,4193,8420,10532,29549,40211,46484,40114,52857,31465,29384,21063,29481,14787,10593,6433,8513,6434,4289,2209,4321,8613,59294,65535,65535,65534,254},
{59263,59263,57183,55103,52991,52991,53023,52991,53022,53023,55103,55103,57151,55103,53023,50943,50911,48862,50942,57150,57183,57183,55134,55102,55038,52957,52989,52989,52989,52957,55037,55070,55069,55038,52957,50876,50844,50844,50876,50876,52956,52957,52956,52956,52956,52924,52956,52924,52892,52892,50844,50876,55005,57117,59230,57182,57149,57149,55069,55069,55069,55036,55036,55004,55004,55036,52956,50843,48763,46682,48731,50844,57117,61278,63390,61342,55069,57117,57149,59261,52988,55036,61277,63389,61277,57084,59197,57149,59197,57148,57116,55036,54971,52923,50843,48762,46682,46618,46650,48730,48730,48763,50875,48795,50843,46683,23276,21195,21196,21195,23243,21195,21195,23243,21163,21163,21163,21195,21163,21163,21163,21163,21163,19083,19050,16970,16969,16937,14889,16937,16969,19017,14857,12744,10663,10631,10663,12744,17002,23308,40112,42225,46418,46418,46418,46418,48466,48498,46385,46385,50546,52691,29449,29449,29449,31529,31530,29449,31529,31529,29449,29449,29549,33873,40048,44274,44274,44274,42128,18953,16774,31466,33644,25422,27731,27698,27730,36019,40179,25617,23537,23505,23505,23505,23505,23505,23505,23570,25618,23537,25618,25618,23538,23570,23570,25617,25618,25650,25649,27730,25682,25618,31891,27698,25650,27762,27698,25650,25682,27730,27730,29582,23112,14628,4226,4226,6339,10532,25388,52758,50612,40277,52857,31499,25191,25158,23206,10626,6465,6466,8513,6401,4321,4321,6402,6435,50907,65534,65535,65534,253},
{59263,59263,57183,53055,53023,52991,52990,52991,53022,53023,55071,55103,55103,55103,53022,50942,50942,48862,52990,55102,57183,57183,57182,57183,52989,52989,55070,52989,52957,52989,52989,52989,52957,52957,52957,50909,50844,50844,50876,52924,52956,52957,55005,52924,52956,52924,52924,52924,50844,52892,50844,50844,52924,55004,59197,57149,59197,57149,55069,57117,55069,55037,55036,55036,55004,55004,52956,50843,50811,48730,48763,50843,59229,61342,63390,63390,59262,55036,57084,59261,57214,57181,57116,59164,55004,55004,59197,59229,57116,57148,57116,55036,55003,52923,52891,48762,46682,46618,46682,48730,48730,50843,50875,50875,50843,48763,25455,23342,23375,23342,23342,23342,23342,23310,23310,21262,21261,21261,23277,23277,21261,21229,21196,21164,21163,19051,19082,19050,19050,21098,19050,21130,21098,19017,19049,16937,19018,16937,16969,23242,40080,40112,46418,46418,46386,46386,46386,46418,46385,46385,50546,50481,29385,27402,29417,31530,29482,27369,29417,31497,29385,29449,29548,35985,37967,44274,44274,44306,37902,17069,23243,31434,31596,25224,27239,27239,29352,35692,37869,27271,27272,29353,29418,29385,29386,29386,29353,29418,31466,29385,29385,29385,29353,29385,29353,29352,29385,29385,27272,29417,27304,25224,29417,27272,27304,29385,25159,29450,33611,25224,27272,29386,23112,14628,2145,2146,4259,10532,21130,50677,44306,57115,52825,35757,16805,23079,19013,8545,6497,8513,4289,2241,2273,4354,4353,2210,44568,65535,65535,65534,254},
{59263,57215,57183,55103,53023,50943,52990,50942,52990,55039,55071,55103,53055,53055,50942,50910,48862,48862,50942,55071,57183,57183,57183,57150,52925,52957,55070,52989,52957,55037,55037,52989,52989,52989,52989,50909,50877,50844,50844,50876,52924,55005,52957,52956,52956,52924,52924,52924,50876,50876,50844,50844,50843,50843,54972,57117,57149,57149,57117,57117,57117,57117,55036,55036,55004,55004,52923,52891,50811,48730,48762,50843,57084,59197,59196,61309,59229,52956,52923,59229,63422,61309,57149,55068,50843,55036,61277,61277,59196,57148,57116,55036,55003,52923,50843,48763,48730,46618,48730,50811,50843,50843,52923,52923,52923,50811,46683,46715,46715,44667,46683,42587,40475,40474,42522,42554,40506,42586,42554,40474,40474,40474,40474,34104,27764,25684,23506,19279,19246,21326,21357,19212,27567,27566,25355,25258,29516,29581,25290,21161,42160,42161,46386,46386,46354,46386,46386,46418,46385,46385,50546,48400,27304,27369,27304,29417,29417,25191,27304,29416,27304,27369,29581,35888,37903,44274,44274,44274,37902,14824,29418,31466,33645,21065,8584,8583,10696,21196,33677,14759,4357,4358,6439,6438,6438,6470,4390,6438,6438,4358,6470,6470,6438,6503,6470,4390,6503,6503,6503,12776,8551,8583,12777,6535,8584,10696,6503,10728,23211,8616,10729,16969,23145,16741,4259,4291,4324,12678,19049,50678,46420,57115,54904,39983,10597,14725,14852,6465,4385,4321,4289,2241,161,97,2209,4258,36149,65502,65535,65534,254},
{59263,59263,57183,55103,53023,53023,50942,50942,50942,52991,53022,55103,53023,53023,50942,48830,50910,48862,50942,55070,57183,57183,55070,52989,50876,52957,52989,52989,52989,52989,52989,52957,52989,52957,52989,52989,50909,50876,50876,50876,52924,52924,52957,52956,52956,52956,52956,52924,52924,52924,52892,50844,50844,50844,52924,55004,57117,59165,57117,57116,55068,57116,57116,55069,55036,55004,52923,52923,50843,50811,48731,50843,50875,52924,57117,59229,59197,52956,52924,61277,63422,61277,59130,54806,46322,46354,50450,48402,48370,48402,46354,46322,46322,46354,44307,44274,42194,40179,44437,46420,46387,46387,46387,48467,46452,46420,44340,44372,46453,46421,44372,44372,42293,42325,42325,44405,44373,44437,46485,44437,44438,44438,44438,36084,29778,29745,25585,21391,19278,23438,21389,19244,19180,23340,27371,29386,31499,35854,35886,35919,44306,44306,46418,46418,46418,46418,46418,48466,46386,48433,50513,48433,27337,27304,29352,29417,29385,25191,27304,29384,27272,29417,31630,29582,35823,44274,44274,44241,37967,14823,29582,33644,33644,31597,42259,42259,42259,44340,44339,42226,40212,42358,44470,44438,44470,44470,44470,42357,44405,42357,42357,42357,42357,44437,40276,42357,44437,44405,44405,44405,42325,44373,42325,42324,44373,44372,42292,44372,44372,44340,44372,40049,25225,20967,16806,20967,20999,25160,27305,50646,50613,57050,54872,46322,12677,12677,12739,8546,6433,4321,2209,4321,2274,2242,4322,4323,36181,63422,65535,65534,254},
{59263,59263,57183,55103,53023,50942,50942,50910,50942,53023,53023,55071,53023,53023,48895,48830,48862,46782,50910,55070,57183,55135,53022,50877,52957,55037,53022,52957,52989,52989,52989,52989,52989,52957,52957,52989,52957,52957,50877,50876,50876,52924,52957,52956,52956,52924,52956,52924,52924,52892,52892,50844,50844,50844,52924,52957,55037,55004,55037,57084,55037,55036,55069,55068,55036,55036,55004,52924,50844,50811,48763,50843,50876,52956,57117,52891,52891,50876,52956,59229,63390,52694,37869,27272,23112,35757,33677,33709,33710,33677,33677,33677,33677,33709,33710,33677,33710,46321,40049,35823,25257,29484,27403,23145,27339,29484,25290,25257,29451,29419,25225,27370,29451,25290,25257,29419,27370,25225,27338,29419,27305,27305,29483,27371,23177,25323,25355,23210,23242,27403,25323,21129,25355,27371,25225,25258,27403,27338,31661,38032,44305,44306,48499,46418,46418,46418,46418,48498,46353,46353,48465,52658,29449,27336,29384,29417,29416,27337,29417,29417,27336,31465,35822,31695,35854,44274,44274,42226,40048,12776,52825,35692,33612,23015,10401,10370,12547,21031,33644,18788,6241,8289,12514,12449,10402,12514,10402,10402,12482,10369,10402,10434,8322,10402,10402,8321,10434,12514,10402,14660,12547,10434,14660,12514,10402,12547,10402,10434,16740,10434,12580,14693,25257,20967,6209,4161,4161,12613,12678,48532,50645,50679,50776,48401,14726,12645,12738,8546,6401,4353,4386,4321,2177,2241,4289,6436,40276,65535,65534,65534,253},
{59263,57215,57183,53023,53023,50942,50910,50910,50910,52991,53023,53023,53023,53023,48862,46750,48862,46782,50878,55070,57183,57183,52990,50877,55005,52989,52989,52957,52957,52990,52989,52989,52957,52989,52957,52957,52957,50909,50877,50876,52924,52956,52956,52956,52924,52956,52956,52956,52924,50876,50844,50844,50844,50844,52924,52957,52989,52924,55004,55037,55037,55069,55036,55036,55036,55036,55036,52956,52923,50843,50843,50843,50843,54972,55036,50876,48763,50843,52924,59165,61342,44406,29614,21130,21097,27436,29517,29549,29549,27436,27501,29517,27436,29549,29581,29549,27436,31662,29648,31695,21129,27404,27404,21098,25291,29484,25291,23210,27404,27404,23210,25323,27436,23211,23178,25323,25323,21130,25291,27371,23210,23210,27404,25291,21097,25291,25323,23178,23210,25323,23243,19017,25291,27371,23178,23210,25323,23210,31596,40014,46386,46451,46451,48499,48499,48499,48498,48597,42193,44174,44142,46254,38000,23143,29548,35853,35886,33804,35886,33708,31692,29482,27403,38000,35822,42226,44306,44306,42161,23308,48533,33579,33644,27436,25617,25618,27698,29745,38034,27632,21392,21424,21489,21489,21457,21457,21457,21457,21457,21424,21489,21457,21425,21457,23505,21457,23569,25618,25618,29746,27697,25585,27698,25617,25618,27730,27730,27666,31858,29778,29843,29778,25290,23080,2081,97,65,8452,8452,40046,48532,48696,59162,31367,12677,12677,10626,6433,4353,6466,6466,4354,2242,2241,2241,2242,31857,65535,65534,65535,249},
{59263,57183,55103,55103,53023,50910,50910,48862,50910,50942,53023,53023,52991,53023,48862,46782,48862,46782,48862,55070,57151,57183,52990,50877,52957,52989,52990,52990,52990,53022,52990,52957,52957,52989,52957,52957,52957,52957,50876,50876,50876,52924,52924,50876,52924,52924,52924,52924,50876,50876,50844,50844,50844,50844,50876,52957,52956,52924,52957,55037,55037,55069,55036,55036,55036,55036,55036,52956,52956,52892,50843,50843,48763,50843,52956,52956,50876,48763,50811,54972,61309,42390,31727,21130,21097,27469,27469,29549,29549,27436,27469,29549,27436,27469,31597,29549,27436,31695,27600,31695,21130,27436,27436,23178,25291,27436,25323,23210,27404,27404,23210,25323,27436,23211,21130,27404,25323,21130,25291,27403,23210,23210,27404,25323,21098,25323,25323,21130,23210,27371,23242,21065,25291,25323,21097,23210,27403,23242,23177,29483,44338,46353,48499,48466,48499,48531,48564,57050,40113,40013,35821,40046,44273,25224,35887,48399,50578,48431,50545,46318,44174,29417,25224,35854,35822,44242,44306,44306,44307,40081,48435,35627,35725,29418,29320,31400,31465,31563,35789,31466,25192,29418,33610,33611,31530,31563,31530,31498,31530,31466,31498,31498,31466,29450,31498,29417,31498,31498,31465,31498,31498,29385,31498,31465,31433,33611,31498,29353,29418,31465,31466,31498,27305,23112,4193,2178,65,6372,8516,19047,38098,48663,54740,23209,12677,10627,10658,8546,4321,6466,6466,2241,4386,4354,2177,4355,21195,63422,65534,65535,233},
{59263,57183,55103,55070,52990,50910,48862,50910,50910,50943,52991,53023,52991,52990,48862,46750,46814,46782,48830,53022,57151,57183,55103,52990,52957,52989,52990,55070,53022,55070,55070,52989,52957,52989,52957,52957,52957,50908,50876,50876,50876,50844,52892,50876,52924,52924,52924,52924,52924,50877,48797,50844,48796,50844,50876,52924,50876,52956,52988,55037,55069,55069,55036,55036,55036,55004,55036,52956,52956,52923,50875,50843,48763,48763,52956,55069,50843,48763,48763,52923,61277,44438,29614,19050,19049,29614,27469,29549,27469,25388,25388,27468,27404,27436,29517,29484,25356,29581,29648,33775,23178,27404,29484,23178,25291,27436,25323,23178,27404,27436,23210,25322,29484,25258,23209,27403,25323,21129,23243,25355,23210,23178,27403,25291,21097,25323,27403,21097,23178,25323,23243,21065,25291,25323,21097,23177,27371,23178,23145,27436,46386,44306,48531,46418,48531,50611,52758,61244,40145,40046,35853,40046,44273,25224,35854,48431,50578,48366,44141,42029,37901,35853,25322,33741,33742,44242,46354,46354,46322,37903,29582,37903,35757,27306,6470,4390,8551,14889,31629,18984,2245,2212,4357,6373,4324,4357,4325,4324,6437,4325,4325,4357,2276,4357,4389,4324,4389,6437,4357,6502,8551,4390,8551,6470,4357,12776,10631,6470,10696,8615,8583,14857,25257,23112,8387,6339,4259,8517,16773,37935,50743,50580,50645,31529,12675,6433,8546,10690,6466,4385,6434,4322,4354,4321,2145,6468,17165,52856,63389,65436,199},
{57183,55103,55071,53022,50942,50910,50878,50910,50910,52958,52991,52991,52991,50942,48830,46750,48830,46750,48830,53022,57150,57183,57183,55103,55070,55102,55070,55070,55102,55102,55102,53022,52990,52989,52989,52957,52956,52924,50876,50876,50876,50876,50844,50876,50876,52892,52892,52924,50876,50876,48797,50844,48796,48797,48797,50877,52924,52989,55037,55037,55037,55069,55036,55036,55004,55004,55004,55004,52956,52923,50843,50843,48763,48731,52924,55004,48763,48763,48763,52924,59197,42293,29582,19050,21130,29549,27468,27501,29549,27436,27436,27468,25356,25356,23211,21130,19050,16937,23309,31630,21098,25323,27371,21162,23210,27404,25291,21130,25323,27403,23178,23210,27371,23210,21130,25291,23243,23210,23242,25323,23210,23210,25323,23210,19017,25258,25291,21097,21097,25291,23210,21064,25290,25290,21065,23178,25291,23178,21065,29549,46386,48531,50579,50611,50611,50611,54904,61211,46452,52856,40014,40046,46354,25224,35853,48398,50546,42061,33642,37900,37900,39981,35888,46583,40049,42193,44273,46322,46354,40048,42456,42193,35757,33677,40179,42259,40211,42292,44372,42292,40245,42422,44503,44471,42423,42423,44535,42422,42390,42390,42358,42358,40310,44503,44503,44502,44503,42390,42358,44503,44470,42422,44470,44437,42390,44438,44470,42357,42325,44437,44405,44405,31564,25257,16773,23079,21064,23112,25159,40048,52693,39949,48532,33709,10562,6402,6369,10626,8546,6466,6434,2209,4322,2209,2209,4356,48826,42159,57083,65469,206},
{57151,55103,52991,50943,50910,50910,48862,48862,48862,50910,52991,52991,52991,50943,46750,44702,46782,44702,46782,53022,55103,57182,57183,59263,59263,59263,57215,57183,55103,55102,55070,55070,55102,53022,52990,52957,50876,52924,50876,50844,50844,50844,50844,50844,50844,50844,50844,50844,50876,50845,48797,46749,46749,44702,46749,48797,50876,52957,55037,55037,55037,55069,55036,55036,55004,55004,55036,55004,52956,52923,50843,50844,48763,48731,52924,50844,46683,48763,50844,52924,57117,36019,29582,25356,29516,29517,25388,27469,29550,27469,27469,27468,27469,27501,27501,27502,27469,25389,27469,27468,27436,27469,27469,27469,27437,27469,27437,27469,27469,27469,27469,27469,27437,27469,27469,27469,27436,27469,27436,27404,25388,25388,25290,25290,23243,25258,25291,25291,25291,25291,25291,23243,25291,25291,23243,25291,25291,25290,25323,29549,46386,48531,50579,50579,50579,50579,57017,57017,50711,63422,37967,42094,48531,23143,33773,48398,50513,39917,25257,35788,37868,39949,35921,52825,40081,42129,42193,42194,42193,38000,59228,44274,35725,29418,12482,12450,14594,18853,35692,27207,10369,10337,12514,12482,12450,14562,12482,12450,12482,10402,12450,12514,10402,12482,12514,12450,10434,14627,10434,14595,14627,10434,12547,16740,12514,16772,16707,12482,16740,16740,10434,10466,23112,27337,14628,12580,14660,16741,18853,42161,50645,37965,42291,44208,10562,6402,4354,6466,8546,6466,4354,2177,4354,4322,2210,2243,57148,44207,52889,65470,224},
{55102,55071,52991,50943,50910,48830,50878,48862,48862,50910,50942,52991,52990,48862,46750,44702,46782,44702,46782,53022,55135,57183,59231,59263,59263,59263,59263,59263,57215,55103,55070,52989,55070,55102,55102,52990,50877,52924,52924,50844,50844,50844,50812,50844,50844,50844,50844,50845,50845,48797,48797,46750,44702,44734,46782,48829,50877,52957,55005,55037,55037,55037,57085,55036,55036,55004,55036,55004,52956,52924,50844,50843,48763,48731,50843,48731,48731,48763,48763,50843,55004,33742,19017,16970,25356,27469,25454,23406,25422,23374,23373,23373,25422,23374,23374,25454,23373,21228,21293,21261,21261,21261,23341,23341,23341,21325,23373,23341,25389,23341,23309,21293,21293,23341,23309,23309,23309,23309,23308,23309,21228,21228,14791,12679,12646,12646,12679,12678,10566,12678,12646,12646,12646,14726,12646,12645,14726,12678,18984,31629,46386,48498,48466,46353,48466,46385,52692,54838,52856,65535,46452,42094,48466,21063,35821,48366,44239,33741,33741,37901,35821,35789,42227,52825,42161,44274,44306,46387,46387,40212,52890,46387,35725,31564,19311,19344,23504,27633,38034,33809,17166,17166,19279,19279,17166,17166,19246,17166,17199,17166,17199,19311,19279,17231,19311,19312,19344,21457,21392,21456,21424,19344,21424,27633,21424,25617,27633,21424,25585,25585,21424,21391,25356,25160,16773,14693,14725,16773,18853,35788,35754,40046,44371,44240,10594,4354,6466,6466,6498,6434,4322,2210,4322,6402,4289,2210,50775,46321,57181,65535,248},
{55135,53023,50942,52990,50878,48830,48862,48830,48862,48862,50942,52991,50942,48862,44734,44670,46750,44702,46782,53022,57183,57183,59231,59263,59263,59263,59263,59263,59263,57215,55135,52990,55037,55070,55070,55102,52990,50877,50877,50876,50876,50845,50812,50812,48796,50845,50845,46781,46749,46749,46749,44702,44702,48830,52957,50877,50877,50876,52924,52956,52956,55004,55037,55037,55036,55036,55036,55004,52956,52924,52892,50843,48763,48731,48763,46683,46683,48699,48731,48698,44373,16840,17002,16970,27436,27501,25454,23406,23374,25422,23374,23341,25422,23374,23406,25454,23373,19212,23342,21325,23341,21261,23373,23341,23341,23373,23341,23341,23341,23341,21261,21261,21293,21294,21261,23309,21261,23309,21261,23309,21260,19148,12743,10631,10631,8550,10631,10630,8518,10631,10598,10599,10598,10631,8550,8518,12679,8550,10597,23274,46352,48400,48400,44207,46320,44206,46287,46255,50546,65534,48499,33675,48434,18886,31661,35787,46418,27435,40014,42127,37901,35853,38033,54905,40113,44273,44306,46386,46419,55002,59261,48402,35724,31564,29450,31530,33643,35723,39983,37837,29385,31564,35789,37869,35757,35757,35757,33676,33708,33676,35725,35789,35757,33644,35724,35724,35724,35724,33644,35724,33676,33644,35724,35757,33643,35724,35756,33611,33611,33643,31498,31531,31498,25192,18853,12612,14693,14725,16773,21061,21062,18949,38000,48499,14723,6466,8546,6497,6466,4354,4386,2241,4322,4321,4289,2209,14659,38099,65535,65534,227},
{55103,55071,50943,50943,48862,48830,48830,48830,48830,50910,50943,50942,50942,48862,44702,42622,46750,44670,46750,53022,57151,57183,59231,59263,59263,59263,59263,59263,59263,59231,59231,55135,52989,55102,57182,57182,53022,50909,50877,50877,48829,48797,48796,48764,48797,48797,48797,46749,46749,44702,44702,44702,44702,48861,52989,52957,52957,52956,52956,55004,55004,55036,55069,55069,57085,55036,55036,55036,55004,52924,52924,50843,48763,48731,46651,48731,46651,46618,46651,44570,29583,12777,21260,19083,25356,25388,25486,25454,23374,25422,23373,23341,23342,23341,25421,25454,23373,19181,21261,21261,21293,21261,23373,23341,23341,23341,23341,21293,23341,23341,21260,21261,21293,23341,23341,21293,21261,21261,21228,23341,21261,19148,12744,10631,10663,8583,10663,10631,10599,10663,8583,10599,10598,10631,10598,8550,10631,8582,10598,12710,18918,18886,20934,18919,18885,18885,20966,18886,20998,56919,44241,33741,46321,16870,35855,40047,48465,38031,42292,40047,35756,37935,35888,46387,42194,42193,44306,46387,48500,52791,50613,44242,35692,33579,6372,2212,4324,8485,31564,27338,4227,2146,4259,4259,4227,4259,4259,4227,4323,4291,4227,4324,4260,4227,2243,4259,4259,4292,4260,4292,4324,4292,6405,10597,6372,8485,8484,6372,8485,8517,4292,6405,16872,25160,18854,12580,14693,14725,10628,10626,12740,8483,23372,42161,18982,8578,8577,8481,6466,6498,4386,2209,97,2209,2209,2145,2210,25583,65502,65502,232},
{55102,53023,50943,50942,48830,48830,48830,48830,48830,48862,50943,50942,50942,48830,44702,42622,46750,44702,46750,53023,57151,57183,59263,59263,59263,59263,59263,59263,59263,59263,57183,57183,52990,55037,57150,57182,53022,50910,48829,48829,46781,46750,46718,46718,46750,46750,46749,46749,44702,44702,42654,42654,44702,46782,50877,52989,55069,52989,52956,52956,55004,55037,57117,57117,57117,57116,57116,55036,52956,52924,50876,48763,48763,46683,46651,46683,44603,44570,46618,44538,23342,14857,23276,19050,27468,27469,25487,25486,23374,25422,23341,23373,23341,21261,21260,23341,23341,21229,21229,21261,21261,23341,25421,23373,23341,23341,23341,23309,23341,23341,21261,23309,21293,21293,21293,21261,21293,21261,19213,21293,21261,19116,14922,12842,10794,10729,12776,10696,10696,10696,10696,10663,10663,10663,10631,10630,10696,8647,10662,14922,12775,10630,10631,12711,10662,10598,12711,8615,10631,27305,35789,40080,39950,16838,35821,42093,46320,46353,57017,44241,27371,29484,35920,31662,42226,44241,44306,46355,46387,39983,29451,33709,35725,33644,33906,40245,38164,40245,42325,42325,38197,38294,38327,38262,38262,40407,40375,38294,36214,38262,38262,38294,40375,38295,38262,36214,38294,40375,40407,42455,42423,42423,42391,42390,40310,42423,42422,42422,42422,42390,42390,42390,40146,25225,20999,18886,23112,16869,8547,4354,19276,25452,25453,48565,27434,19081,8578,6465,6466,6498,6466,2177,65,65,32,32,2178,17099,65502,65534,251},
{57151,55071,52991,50910,48830,48798,48830,46750,48830,48862,50910,50943,50910,46782,44670,42622,44702,44702,46750,53023,57183,59231,59231,59263,59263,59231,59263,59263,59263,59231,57215,57182,55102,57150,57183,57183,55103,50942,48862,48830,46749,44702,40606,40574,42654,44702,44702,44702,42654,44702,42654,42654,40606,44734,44702,52957,55037,55036,55004,52956,55004,55069,55069,57117,57117,57117,57084,55036,52956,52924,50844,48763,48731,48731,46651,46651,46651,44571,44570,44538,29747,12745,23243,16969,27436,25421,27566,25486,23406,23374,23341,23309,23341,21261,21261,23341,23341,19181,21228,23309,23309,23309,23373,25421,23341,21293,23373,21293,23341,23341,23373,21261,23341,23341,21293,23341,21261,21261,19213,23341,21293,19148,17099,15019,14955,12842,16970,12809,12809,12841,10729,10728,12809,12776,12776,12808,15019,19212,14921,27599,19017,16969,27501,25420,29646,29646,29581,27500,27500,27468,31628,35788,33676,12581,14660,20998,40013,46288,52824,44241,23210,29451,40146,33905,42227,42193,44273,44274,46355,42194,25291,33775,33644,33644,18788,14594,18723,20836,31531,33579,12482,12482,16707,16739,16675,18756,16707,16675,16707,16707,16675,18788,16707,14595,16707,14627,14595,16708,16675,14627,16739,16707,16675,16740,14562,14595,14595,14595,16707,16708,14627,16773,20966,25192,20967,12548,16708,14725,10660,8645,40243,46453,31693,48565,35755,27468,19079,10658,8546,8578,4321,97,32,33,64,2210,2146,10760,63455,65535,253},
{55103,53023,52991,50910,48830,46750,46750,46750,48830,48830,50910,50911,48862,46782,44670,42590,44702,44702,46782,53022,57183,59230,59231,59263,57215,57183,59231,59231,59231,57183,57215,59263,59263,59263,59263,57215,55102,50942,48862,48798,46750,44702,40574,40574,40574,44702,42654,42622,42654,42654,40606,42654,42654,42687,44734,50941,55037,55037,55005,55005,55036,57085,57117,57149,57149,57117,57084,55036,52924,50843,50843,50811,48731,48731,46650,46651,46651,44570,44570,44570,40279,17101,16937,19018,27436,25389,25486,25486,25454,25454,23341,21261,23341,23341,21261,21293,21293,19180,21196,21261,21228,21261,23341,25389,23341,23341,23373,23341,21261,21325,23341,21293,23341,23341,23341,21293,21261,23341,19213,23341,21260,19083,19115,17068,14922,14889,16970,14889,12777,14857,12808,14922,27534,21130,27501,29614,35919,29645,25323,40080,21097,16904,23177,25257,25290,31596,23144,16806,18951,16838,16806,16871,25257,25289,27370,33708,42061,39981,54904,37968,23275,33514,37870,40146,44274,42161,44274,44306,46355,44209,23210,31662,33612,33644,19115,15053,15053,17133,31728,33840,12875,10795,10827,12908,10859,12908,12908,12875,12940,12940,12907,12940,12972,12907,10924,12940,12908,12972,12940,12940,15085,12972,12972,15053,15052,15053,15085,15053,15085,17198,17100,34036,27469,23145,18984,14823,16904,16904,16968,21162,42095,46321,33546,39949,25190,21031,16836,12675,8515,10595,8483,6403,6371,6371,6403,16838,14921,25583,61374,65534,251},
{55102,53023,50910,48862,46782,46750,46750,46750,48830,48830,50910,50910,48862,46750,44670,42590,44670,46750,48830,55070,57183,57183,57183,59263,57183,57182,59231,59231,59231,57182,57182,59263,59263,59263,59263,57182,55070,50942,48862,44702,42622,40574,38526,40542,42622,44702,42622,42654,42654,42622,40606,40606,42654,42686,44734,50909,55037,55037,55037,57117,55069,57117,57117,57117,57117,57149,57084,55004,52923,50844,50844,50843,48731,46651,46650,46650,46650,46650,44570,44570,40376,31857,23406,19115,29549,27501,25486,25486,25422,25454,23341,21293,23341,23309,21261,21261,21261,21228,21228,21261,21261,21261,21293,25389,23341,23340,23309,21261,21261,21293,21260,21260,21260,21260,21261,21261,21260,21260,21260,23309,21228,19147,17035,14955,14922,12906,16970,17002,14890,14857,12841,23308,29549,21130,25388,29516,31692,29580,23210,29582,21097,16936,19016,25322,25291,31629,19082,12808,19081,14888,16969,16936,25355,29385,29418,31531,40013,44207,52725,33743,25323,25258,29483,37968,40015,42160,44274,44274,44274,42128,27404,29516,33643,33676,33677,37869,37870,37902,42129,42161,35790,37968,40080,40080,40080,42161,40113,40080,40081,40080,40048,38000,40048,37968,37968,38000,37935,37967,37968,40016,40016,37935,37935,37968,40015,39983,39983,37902,39983,42194,44405,50678,37968,29516,31726,40145,40145,40145,40144,40113,42193,42160,21064,18951,18983,18951,16903,18951,18984,18984,18984,18984,19016,21096,21096,21097,31726,38000,59229,65534,252},
{55102,53022,50910,48830,46750,44702,46750,46750,46782,48830,48862,50910,48862,46750,42622,42590,44670,46750,48798,55070,57183,57183,57183,59231,57183,57151,57183,57183,57183,57183,57182,59263,59263,59263,59263,57182,55102,50942,48862,42654,40574,40574,38526,38494,42622,42654,40606,40606,40606,40574,40607,42654,42654,42686,44734,52989,55037,55037,57117,57149,57149,57149,57117,57117,57117,57117,55036,54972,52923,50844,50844,50844,48763,48731,46651,46651,46650,46650,44570,44570,40376,33970,29614,27436,29483,31629,29581,29581,29582,29582,29581,29581,29582,27534,27501,27534,27533,27534,27534,25453,27502,27501,25453,27501,27501,27501,25453,25454,25453,23340,23341,23308,23308,23340,25389,23341,25421,25454,25421,25421,23373,21228,19147,19180,19180,19147,16937,19082,19050,19050,16937,23275,29582,21162,27468,29549,27467,29580,23242,40015,23177,21065,27403,31596,31628,35854,33741,31628,33741,29548,31694,31628,31628,27338,29450,27402,37966,46385,44274,27404,27436,25290,29451,40114,31596,42160,44242,44274,33644,33743,33710,29548,33644,33676,12580,6242,6242,8387,27338,33644,8355,4162,6242,6274,6242,6274,6274,6242,6306,8387,4194,8419,12515,6274,6307,6307,6242,6307,6274,4194,6339,6306,4226,6339,6306,6307,6307,6307,6339,31890,44307,38001,40081,14628,21064,25159,27272,29450,27239,29483,25223,29482,29515,23111,27434,29483,23144,29483,29515,25322,29483,29516,23177,29450,29516,25322,33774,29516,55035,65534,249},
{55103,52991,50910,48830,46750,44670,44670,46750,46750,48830,50878,48862,48830,44702,42622,42590,44670,44670,48830,55070,57183,59231,59231,59231,57183,55135,57182,57183,57183,57183,57183,59263,59295,59263,59263,57215,55103,50942,46782,40574,38558,40542,38494,38526,40574,42654,40574,40606,40606,40607,40606,42654,42654,42686,46814,52957,55005,55037,57117,57149,57149,57149,57149,57117,57117,55068,55004,52923,52923,50844,50844,50844,50843,48763,46683,46650,46650,46651,46650,44570,40376,33970,27566,19050,33677,31596,31564,27403,29484,31564,29483,29451,31564,29516,27403,29516,33677,38000,31662,31629,29516,29451,33645,31597,29516,31564,31596,29451,29451,33644,29483,29451,31597,31596,31532,31564,31596,31531,31531,33677,33644,31564,31596,31564,29451,27306,23080,23145,23275,25388,17002,17002,31695,23210,29581,31661,25386,29580,23242,31661,23178,19017,21064,27402,27371,33742,29484,29483,29516,25289,33807,29483,29450,29418,31531,31595,35821,39949,31595,23275,27501,25225,25258,40147,38000,37967,42193,44274,33840,31760,25355,27468,33645,33677,29647,31956,33972,34004,40212,42260,31891,27795,29876,29876,29876,27796,29876,29908,29908,29908,29876,31956,31924,29876,31988,31989,29908,31956,31988,29908,29876,29876,27828,31989,31988,31988,34036,34036,31988,44503,46453,33775,42194,14661,14790,6371,8517,25258,14628,21097,16806,21162,27403,10532,23275,27403,12645,25355,27435,21195,27468,27468,19114,29515,29548,19146,33839,27403,50744,65534,245},
{55070,52990,50910,48830,46750,44670,44670,44702,46750,48830,48862,48830,48830,44702,42590,42590,44638,44702,48830,55070,57183,57183,57183,57183,57183,55103,57183,57182,59231,57183,57183,59263,59295,59263,59263,57183,55135,53023,46782,40574,40574,38526,36446,38495,40574,42654,40574,40574,40606,40606,40606,42655,42654,44734,50909,52957,52989,55036,57117,59197,57149,59197,59197,57117,57084,55004,52924,52892,50843,50843,50844,50844,48795,48763,46683,46651,46651,46651,46650,44570,38295,33970,25486,19114,27403,29516,27403,25291,27404,27435,23242,25291,27403,27370,25258,29451,29483,35953,29647,29549,25355,23210,29484,29484,25323,25355,27404,23210,23178,27436,25290,23145,27403,27403,21097,25290,27403,23177,23177,27403,25323,23145,27404,25323,21032,27209,24967,25258,33872,25388,23340,23308,29549,27468,33741,31693,29774,29613,23242,29614,23178,19049,23275,29549,29581,35855,29613,31661,31693,27436,35953,31563,31563,27337,29482,31563,37901,33676,31531,35920,40113,38000,31694,46484,37999,40080,44274,44242,44274,42226,29516,29549,31596,33644,20966,22949,25094,25127,31531,35724,22982,20933,25094,25127,25127,25127,27240,27175,25127,25191,25095,25159,25127,25094,27239,27271,23046,25127,25127,23014,23046,23078,20934,25094,23014,23014,25159,25127,23014,31662,48566,29549,44307,16839,16904,19114,16969,27436,21130,23275,23275,21260,29516,21195,25420,29516,21194,27500,29548,27468,31629,29581,27501,31596,31628,27500,33839,35724,44307,65437,244},
{53022,52991,50910,48830,46750,44638,44638,44670,46718,46782,48830,48830,46782,42622,42590,42590,42590,46750,50878,55070,57183,57183,57183,57183,57183,57151,57183,57182,57182,57183,57183,59263,59263,59263,59263,57182,55103,50943,44703,40574,40574,38494,36446,38526,40574,40606,40574,40606,40574,40606,40606,42655,44734,50942,52989,52924,52956,55036,57084,59197,57117,59197,59165,57117,55036,52924,52924,52891,50876,50844,50876,50843,48795,48763,48731,46683,46651,46650,46618,44538,40311,33905,25453,19050,29549,31630,29549,29549,29549,29549,27468,27468,29548,27468,25388,29516,29549,35953,29647,29517,25324,23178,27436,27436,25291,25323,27404,23210,23178,29484,25291,21097,27404,27371,21097,25291,27436,23178,23178,27404,25291,21097,27404,25355,23146,25291,27371,23178,27403,25355,25388,25356,29581,23177,27371,31758,36048,29677,23242,33709,23210,19017,27338,31563,31563,35854,33676,35756,33709,31563,38001,31563,33644,27370,31530,31563,31563,33611,27370,37935,42161,40113,35856,50679,52857,40080,44242,44274,44274,44242,27371,29484,33644,33644,14824,6536,6536,12809,23276,33678,12744,4390,4358,4423,4423,4390,6535,6471,6471,6535,4391,6471,6503,6471,6535,8616,4423,6535,6503,4423,6535,6567,4455,6568,8616,8616,12809,12841,8681,21293,48599,38066,46420,29646,25453,38066,38098,38098,38066,38065,38065,40210,29418,20901,20901,20901,20900,20901,20901,20901,20901,18820,18788,20868,23079,33774,33741,40048,44242,39851,229},
{52990,52990,50910,48830,44670,42590,42590,44670,44670,46782,48830,48830,46750,44638,40542,42590,44638,46750,48862,55102,57183,57183,57183,57183,57183,57151,57183,57150,57182,57183,57215,59263,59263,59263,57215,57183,55103,50943,42654,40574,38526,36446,36446,38526,40574,40574,40574,40606,42654,40606,42654,42686,50910,55070,52957,52924,52924,55004,55004,57117,57149,57117,57116,57117,55036,52956,52924,50876,50876,52924,52924,50844,50811,48763,46683,46650,46651,46651,44602,44570,40310,31793,23341,18985,29550,31630,29549,29549,29549,29549,29517,27468,29549,27468,27404,29549,29549,33905,27567,27436,25323,21130,27404,27404,23210,25323,27404,25290,23178,27436,25323,23177,27404,27403,23178,25291,27436,23210,23178,27436,25323,21129,25355,27404,21129,25323,27404,23210,23210,27371,23243,25291,31629,21130,27436,31790,36048,31725,23210,33742,25258,19017,21129,27403,23210,33742,25323,29548,27403,27371,35921,31564,33709,29450,31562,31563,33741,25192,27403,35887,40113,40113,33908,46519,52791,40048,44241,42226,44306,44242,27403,27403,33644,33676,33743,42259,42259,42259,44339,44307,42227,42325,42357,42389,44470,42390,44405,44438,44470,44437,44405,42357,42389,44437,42325,42292,42324,44438,42324,42324,44405,42324,42324,44405,42324,42259,42259,42259,40211,42325,48599,46550,48565,31596,21031,29417,33675,35887,33643,27240,25159,25224,23143,21097,16871,18919,18919,18951,18984,20967,18984,21096,18951,21129,33709,44240,25159,31597,42129,27305,131},
{52991,50942,50910,46782,44670,42590,42590,44638,46718,48798,48830,46782,44670,42590,40542,42590,42590,46750,50910,55102,57183,57183,57183,57182,57151,57150,57182,57150,55102,57183,59263,59263,61311,59263,57183,57183,55103,48895,42654,40574,38526,36446,38494,38526,40606,40574,40574,40606,42622,40606,42654,44734,50877,55005,52956,50844,52924,52924,55004,57084,57084,57117,57116,57116,57084,55004,52924,52892,50844,52924,52924,50843,50811,48731,46650,46683,46650,46651,44570,44570,40278,29744,23308,19017,29549,31630,31629,29581,29581,29549,29549,27469,29549,29517,27404,27469,29517,31825,27567,27437,27371,23178,25355,27436,23210,25323,27404,25291,23210,27404,27372,21129,27404,27436,23178,25258,27436,23210,21130,25388,25291,21129,25323,27403,21097,25291,27371,23210,23210,27371,23210,21065,27371,25291,23112,31757,36048,31757,23242,35920,23242,21129,29548,31629,33741,37967,33774,35854,33774,33774,38001,35822,37902,31563,33578,33578,40048,18984,29549,35887,42194,42194,34005,44439,37935,40047,42161,44242,44274,44274,29452,27404,31564,33644,18853,10369,12450,14595,23112,35724,18821,8289,8289,12515,12450,10370,16707,12514,10401,12514,10402,10402,12515,10402,10402,12515,10369,10402,10434,8321,10402,16772,10369,10402,10434,8289,8354,8354,8354,21195,40049,54937,44045,14693,19016,23144,29515,35855,37870,10499,29614,23373,25290,31727,21194,21064,25290,14856,23145,27404,19017,25257,27468,27468,39949,42127,23111,21163,38033,31531,177},
{53022,52990,50910,46782,44670,42590,42590,42590,44702,46750,48830,46782,44670,42590,40542,42590,42590,46750,50910,55103,57182,57183,57183,57183,57151,57150,57182,57151,55103,57183,59263,59263,59263,59263,57183,55135,53055,48895,40606,40574,38526,36478,38494,38526,40574,40574,40574,40606,42654,42654,44702,44702,46750,52957,52924,50844,52892,52924,55004,55036,55036,55036,57084,57084,55036,52956,52924,52924,50876,50876,52924,50843,48763,46651,46650,46618,46650,46650,46618,44570,40245,31857,21260,21130,29581,31630,31630,29582,29582,31629,29549,29517,29549,29517,25356,27436,33839,44404,35888,29549,29549,27436,29549,31629,27469,29516,31630,29517,27436,31597,29549,27436,29549,29549,27436,27468,29549,27437,25356,29549,29517,25356,27501,29549,25356,27436,29549,27436,25356,27469,25388,23275,27469,27469,25388,31758,33935,31757,23210,29548,25290,21097,21129,29483,27403,35822,27370,33677,31564,27338,29484,33643,35822,31563,31530,31562,35822,21064,27404,35855,44274,44274,36021,42326,35855,40015,42161,42193,44274,44274,29549,27404,31564,33644,23276,21424,23504,23537,29745,38034,25552,17231,17231,19312,19279,17199,21391,21359,19279,19376,19344,19344,19376,19344,19344,21424,19344,19312,19376,19376,19344,23569,21424,21392,21392,19343,19376,19408,21456,31956,40081,44176,14562,16871,42226,42095,42127,42160,40048,10531,29614,25454,27403,33774,16871,23144,29484,10662,23177,31629,14888,23177,31662,19049,42062,39982,21064,19082,38066,29319,176},
{53022,52990,48830,46750,42622,42590,42590,42590,44670,46782,46782,46750,44670,42590,40510,40510,42622,46750,50910,55103,57150,57183,57183,57151,57151,55103,57183,55103,55103,57183,59263,59263,59263,57215,57183,55103,53023,48863,42622,40574,38494,38494,38494,38494,40574,40574,40574,40606,42654,44702,46749,46749,48829,52957,50876,50844,50876,52924,55004,55004,55036,55036,55036,55036,52956,52956,52891,50876,50844,52892,52924,50843,48731,46683,46651,46618,46650,46618,46650,46618,38165,33938,23406,23244,33742,33710,33710,33742,33742,33710,33742,33710,33710,33710,31662,31662,37837,31466,29483,31596,31564,29484,31564,31628,29516,29548,31629,33645,31564,33709,33677,31564,33677,33678,31596,31596,33677,31597,29517,31661,31629,29516,31629,33709,29516,29548,31629,29516,29516,31596,31629,29516,31596,31596,31564,33903,31855,31758,23242,27436,25290,21097,23210,29516,29549,35854,27468,31629,27436,27467,29549,23144,31694,31531,31498,31498,37935,18951,27436,38000,46355,46354,29716,38101,35791,40048,37902,37869,44274,44306,31596,29516,33709,33741,33775,46485,48565,48565,48566,48565,46485,46485,46486,46518,46518,46518,46486,44438,46486,46485,46486,46486,46486,46486,46486,46486,46486,46486,46486,46486,46486,46518,46518,46486,46454,46454,46454,46453,44242,44372,44307,35822,10466,14726,44307,46255,44174,44208,42193,10499,27468,21228,29484,31661,14823,23176,29484,8517,25290,31629,12743,25225,31629,23340,42062,37869,27371,16904,33905,33447,156},
{52990,50910,48830,46750,42590,42590,42590,44638,46718,46782,46782,44702,44670,42590,40510,40510,42590,48798,50910,55071,57150,57183,57183,57183,55103,55102,57151,55102,55102,57183,59263,59263,59263,57215,55135,55102,53023,46782,40574,38526,38494,38494,38494,38526,40574,40606,42622,42654,44702,46749,48764,48764,48764,52925,50876,50844,52892,52924,52924,52956,55004,55036,55036,55036,55004,52924,50844,50843,50843,50811,50844,50811,48763,46683,46650,46650,46650,46618,46618,46618,48598,44241,37935,31662,31564,33678,33776,33808,33776,31727,33776,31727,33744,33743,33743,33743,31695,31695,31663,31663,31663,31695,33742,33742,33743,33711,31695,33743,33743,33743,33743,35791,35791,33743,33743,33743,33743,33743,33710,33710,33742,33742,33742,33710,33678,33677,33677,31629,31597,33645,31597,31629,31629,31597,31629,33870,34000,31790,23242,40081,25290,21097,29450,33677,35757,37935,35822,35822,31628,33709,31693,16936,29581,29418,29417,29353,35788,18983,27468,37901,44208,37869,31861,27537,44307,44242,27403,27468,46387,46387,33742,31629,37870,37869,25192,29386,33547,33611,35692,35725,27404,29614,38002,40147,42227,42260,42227,42196,42260,40147,42228,42260,42260,42228,42260,42228,42228,44308,42260,42260,44308,42260,42260,42260,42260,42260,42260,42260,35789,25290,46420,33742,10499,12613,40146,46256,39883,42095,46354,23242,25289,16773,25257,29516,14693,23209,29515,12644,25257,29515,14757,25290,31563,16805,35788,37837,35822,27534,38164,33480,124},
{50942,50942,48830,46718,44638,44638,42590,42590,44670,46782,48798,44702,42622,40510,40510,40510,42590,46782,50942,55103,57151,57151,57151,55103,55102,55103,55103,55102,55103,57183,59263,59263,59263,57215,57183,55103,53023,46782,40574,38494,38494,38494,38494,38526,40574,40574,42622,46750,48797,48797,48764,48764,48763,50844,50876,50876,50844,52892,52956,55036,57117,57117,55036,55036,55004,52924,52891,50843,48763,50811,50811,50811,48763,48731,46651,46651,46683,46650,44570,44538,33842,19019,25357,25357,27404,29549,31761,33841,31761,31760,31760,29713,29713,29712,29712,29680,29680,31728,29648,29680,29648,29680,29648,29648,29680,29648,29680,29648,27632,29680,29680,29680,29712,31760,31793,29680,29680,29680,29680,29680,29680,31760,29648,27535,27470,23276,23308,23244,23276,23243,21195,25356,27469,29549,25388,31789,36080,31791,23210,27468,25290,21097,19016,27435,27435,35855,29516,33709,27371,29516,29516,25355,31695,29451,31563,31564,31563,29547,31595,31562,33611,29483,38101,31827,44306,46386,44339,44339,48467,42062,29450,33742,37870,33611,23145,29418,31499,31563,33644,33677,29647,35986,40180,42260,42260,42260,42260,42261,42293,42261,42228,42260,42260,42292,42260,42260,42260,42260,40180,42260,42228,42228,42228,42228,42228,42228,42228,46486,29386,23210,44340,35822,14693,14791,35823,44175,37869,46288,46354,18820,23177,19114,27469,29516,14726,23209,29581,16871,25322,29613,16903,23209,31629,21162,35789,40047,27369,25453,52758,25062,89},
{50942,50910,48830,46750,44670,42622,42590,42590,44670,46750,46750,44702,42590,38462,38461,38462,40542,48830,50942,55070,57151,57183,57151,55103,55102,55103,55103,55103,55071,57151,59263,59263,59263,57183,55103,55103,50975,44734,42622,38494,36446,36446,38494,38494,38526,40574,44702,46750,48797,48797,48764,48796,50811,50843,50844,50876,50844,52924,52956,57117,57117,57117,55036,55036,55004,52924,52892,50843,50811,48763,48763,50811,48763,48731,48796,50844,48796,48763,46683,44570,42522,36314,29942,29845,31892,31860,27699,29779,29779,29779,29779,29779,29778,27730,27698,27666,27633,27633,25585,25553,27633,27633,27600,25552,25552,25552,27600,25552,25519,25520,25520,25520,25520,27568,27568,25519,25519,25519,25519,25519,25519,25519,23406,23374,23342,21229,21228,21228,21228,19148,21196,21228,25388,25389,21227,27597,36016,31790,23242,40146,27371,21129,25356,29549,29549,35854,29613,33774,29548,31661,33710,29516,33807,31563,33644,33644,33643,33676,33644,31595,33644,29483,44307,44242,42193,46386,46386,46386,48434,39983,31694,35822,37903,29386,27306,29451,31499,33644,35725,27404,27535,33841,38034,38067,40180,40147,40147,40180,40147,40147,40147,40147,40147,40180,40147,38067,38067,40115,38066,38066,38067,40147,40179,40147,40147,40147,40179,46519,27338,23209,37903,35757,19115,25323,37935,39916,44208,46287,48467,18854,23242,23275,29614,29516,16970,23275,29548,19049,23275,29549,16839,21129,31629,21195,35822,35789,25355,38130,35528,21032,75}
};

const uint16_t Tel_Aviv PROGMEM [320][241] = {
{32090,32122,34170,34170,32090,32058,32090,32090,32090,32058,32090,34170,34138,32090,32090,32090,32090,32090,32090,32122,32154,32122,32122,34202,34202,34170,34137,34137,34137,32057,32056,32024,29976,29976,32056,34104,32056,32024,32024,29976,29976,29976,27895,27895,27895,27863,27863,27863,27863,27863,27862,29878,29910,27830,27830,29910,29910,29910,29910,29910,29910,29910,29878,29877,29877,29877,29877,29877,29877,29845,31925,31924,44438,52824,44372,46550,36018,48631,44372,46485,50743,50743,48598,48631,40146,38098,31759,48598,57050,31759,31759,46517,54970,46550,21195,14921,19082,19050,16969,29647,25454,23341,23373,46551,44471,40179,31825,36018,40212,38066,38099,40244,40179,38067,33905,25871,46517,54969,50612,42127,44240,48498,42160,40179,36309,32343,50808,46485,46452,40179,25454,29712,31760,38033,40179,42324,38099,40212,40243,34068,48598,35888,14825,48597,40178,31760,50677,35920,27502,31663,23277,23341,21228,19115,19082,19115,23309,25455,29680,31793,31661,40080,36018,38132,40244,23307,23468,36050,27565,25452,27597,27564,29613,38032,40146,35952,29581,31629,46419,48532,46484,46451,46419,44306,38033,57148,57116,57148,57148,57116,46583,46550,59196,54970,54969,52856,42226,35953,35954,27567,23309,29679,38034,35921,31727,29647,38066,40211,38034,33872,38097,35952,23306,12806,19079,23305,25384,18982,21128,27435,25388,27468,31661,29549,19016,23340,29647,25454,19147,21228,19148,19148,21228,25454,27600,27599,29647,29679,35986,29712,112},
{32090,34138,34170,34170,32090,32058,32090,32090,30009,32058,32090,34170,34138,32090,32090,32090,32090,32090,32122,32122,32154,32154,32122,34202,34170,34137,34137,34105,34137,32025,30009,29976,29976,29976,32056,34104,32056,32024,32024,29976,29976,29943,27895,27895,27863,27863,27863,27863,27863,27863,27862,29910,29910,27830,27830,29910,29910,29910,29910,29878,29910,29909,29877,29877,29877,29877,29877,29877,29877,29845,29845,29844,44471,52857,52857,48663,44372,54905,50744,59195,59163,57050,52824,61276,57082,59131,46485,52889,57083,42260,48631,54936,57050,31759,21163,16969,23243,19049,21163,38098,25420,19114,17001,44535,46583,40180,27534,25389,31760,29679,21196,23373,38066,25324,29548,31595,40048,52758,48466,42095,44241,50580,48467,40114,52922,52986,48631,48565,46485,31728,23309,19115,23341,38066,29712,42324,42292,40211,40211,34035,48630,35920,14890,48597,40114,27567,50678,35953,33873,40212,31794,23474,17101,4357,4325,19083,12777,4358,8583,44438,42291,44437,44438,42357,42358,29647,25453,33938,38099,31726,29645,31758,31758,35919,35920,31694,29548,29548,46419,48532,48565,48565,46419,44338,38033,59228,55068,57148,57148,54970,42292,46583,29615,31759,48630,40178,31759,23373,48697,33906,33841,14857,29647,38066,44438,33905,40146,42292,19083,23276,40146,29644,21256,19144,19079,21159,25352,19013,31626,23208,25322,25323,21097,23275,19081,25453,29647,29615,19147,21260,19148,19148,21261,29647,25454,25486,29647,29647,29647,29679,110},
{32090,34170,34170,34170,32090,32058,32058,32058,30009,32058,32090,34170,34138,32090,32090,32090,32090,32090,30042,32154,30106,32154,34202,34202,36218,34137,34137,34137,32057,29976,30009,29976,29976,29976,32056,34104,34104,32024,32024,29943,29944,29943,27895,27895,27863,27863,27863,27863,27863,27863,27862,27830,29910,27830,27830,29910,29910,29910,29910,29910,29910,29877,29877,29877,29877,29877,29877,29877,27797,29845,27764,27699,44438,48599,42325,48598,50744,50776,46518,46550,44470,36019,55002,59195,54969,57082,50744,59196,57115,48631,46485,52824,55002,50744,38066,23243,21162,19082,21227,29711,19146,14953,14889,44535,46616,40212,17133,17197,33938,29679,23471,31792,40212,25519,25518,42095,37967,56919,44208,42095,44273,50645,46419,46453,63454,61309,50743,48565,46485,17035,17035,19116,29680,38132,25454,40244,40244,40211,40211,38164,50646,35888,16970,48564,35921,31825,50711,44373,46551,40179,31859,23473,17069,4325,6405,10631,6406,6438,8584,19148,35985,46517,38066,19180,31760,33873,19146,33937,38066,31727,31759,33871,35919,29613,19049,29549,27403,31629,44339,48532,48532,46451,46451,44339,38065,59228,57148,57148,50744,42324,38066,50745,25389,14856,29614,38065,25454,48697,48697,48697,25487,31793,23276,33905,57084,31728,40179,46485,40147,27469,40146,29612,23273,16967,19112,23240,27432,25288,23207,23176,25290,29516,19049,14823,16904,25388,29614,29518,19147,21228,19148,23342,23341,23373,27535,25453,27534,29679,40244,29680,108},
{32122,32122,34170,34170,32090,32058,32058,30010,30009,32058,32090,32090,32090,32090,32090,32090,32058,32090,32090,32154,30106,30106,34170,36250,36218,34169,34137,34137,30009,29976,32057,29976,29976,29976,32056,34104,34104,32056,32024,29976,29944,29944,27896,27863,27863,27863,27863,27863,27831,27863,27862,29910,29910,29878,27830,29910,29910,29910,29878,29878,29878,29909,29877,29877,29877,29877,29877,29845,29845,29845,29876,31892,44438,52824,50711,50711,36018,44438,46550,46550,40179,40147,48630,57083,40211,44405,50712,61276,59163,40179,40211,46550,61276,61276,57082,33807,21130,16969,21228,19114,12872,10728,6535,42390,48729,40212,10794,6600,25519,25422,8745,15019,40212,8583,12906,23565,42160,54806,37934,44208,44273,50645,46420,50744,65503,57115,46517,46484,44340,19115,17002,21228,33840,31727,38099,40244,40244,40212,40178,38065,50646,33807,14857,48564,40179,42390,38033,42292,44470,38066,36052,23473,17036,2276,6438,8551,8551,6471,10729,25421,48500,44307,21131,21195,31760,31760,23307,31759,31727,29614,40178,35985,35920,25388,29614,25323,29451,33709,46419,48532,48532,46483,46419,44339,38065,59228,52825,46517,44371,44339,38066,48664,21195,8518,27566,50777,25455,44503,42326,46617,27600,44471,23148,40213,52890,44438,42324,50711,44405,38001,40145,44339,31693,27531,21159,21127,23207,23176,31595,35887,40145,38001,29614,29581,21130,23307,29614,38034,21228,21260,19148,27600,23309,25422,25486,23309,25486,29680,40245,29680,105},
{32090,32122,34170,34170,32090,32058,30010,30010,30009,32058,32090,32090,32090,32090,32090,32090,32090,32090,32122,32154,30107,30106,32154,36250,34202,34137,34169,32089,30009,29976,30009,29976,29976,29976,32056,34104,32024,32024,29976,29943,29943,29944,27895,27863,27863,27863,27863,27863,27863,27863,27862,27830,29910,27830,29910,29910,29910,29910,29910,29878,29877,29910,29877,29877,29877,29877,29877,29877,27797,29845,31956,40277,52825,50743,44405,33906,27600,44470,52857,48631,54937,57082,54970,48598,50711,38066,50744,59195,44404,46518,40146,46517,61276,59195,40211,25485,21260,19082,36018,21260,8647,6535,8648,38229,46584,40212,25454,21260,29647,31728,17035,19180,40179,12776,10727,23597,48499,52725,39982,44241,44273,50612,42129,54937,63455,52857,46485,46452,38098,25454,21261,25389,38034,35953,25389,42325,40212,40211,42324,44339,48565,42194,33775,48564,44404,44404,46485,40179,42292,40211,31827,23441,14956,2245,8616,6471,8551,4390,10729,14889,21195,31727,23212,29647,31727,31727,19114,25388,29614,33807,40178,40210,38066,21130,29582,23242,27370,31629,46420,48532,46484,46451,46419,44339,38033,50743,46485,44372,44339,44339,40114,48664,19083,6470,33938,42325,46551,40212,31791,38131,29680,48665,25196,46552,48729,44405,44405,50711,46453,35919,40113,44371,42258,40145,35918,37965,37965,37933,42192,42193,42226,44501,40113,37870,27338,25290,31695,35889,21196,21228,21228,25454,21260,23374,21261,21261,25454,29647,27567,29647,109},
{32058,32090,34170,34170,32090,32090,30010,29977,29977,32058,32090,32090,32090,32090,32090,32089,32090,32090,32122,32155,32155,32154,34202,34202,34170,32089,34137,32089,29976,29976,30009,29976,29976,29976,32024,32056,32024,32024,32024,29943,29944,27896,27895,27863,27863,27863,27863,25815,25815,27863,27862,27862,27862,27830,27830,29910,29910,29910,29910,29910,29877,29909,29877,29877,29877,29877,29877,29845,29845,29844,29811,44470,50744,52824,44438,40212,31858,29745,46584,44438,50744,48631,50679,48533,59163,63389,57115,54970,48598,42259,40211,54969,54937,42324,19146,12841,10696,14922,29680,19180,8680,4454,8615,38197,46616,40212,21326,21358,33938,31760,25552,27664,42292,25551,29743,27791,46386,48532,42096,48466,44273,48499,42128,59163,63389,50711,46452,44340,27534,23374,21196,25421,33807,42292,33873,42292,40212,40211,40178,40178,50678,35920,25453,50678,40211,44502,46452,40178,46485,44372,25553,21361,12843,2245,6503,6439,6471,6503,6503,6470,12809,44471,25390,21195,31728,33840,42325,33872,31727,33807,40178,40146,38033,25420,29614,31662,27372,14791,46484,48532,48532,46452,46451,46419,35952,48630,44405,48598,44371,44339,38066,46583,19083,8518,23373,33904,36017,31726,25482,38131,38132,50778,25196,27503,52890,35986,44405,44437,40146,40146,38065,44404,42291,42259,40179,40179,42226,40179,40179,40179,40147,42356,46517,38032,18984,14823,31727,33744,16970,17002,27534,23374,23341,23309,23341,23309,25486,29680,35987,29648,108},
{32058,34170,34170,34170,34170,32090,32058,30009,30009,30010,32090,32090,32090,32090,32090,32090,32090,32090,32122,32154,32155,32155,34202,36250,36250,34137,32057,32057,30009,29976,30009,29976,29976,29976,29976,32024,32024,32024,29976,29976,29976,27896,27896,27863,27863,27863,25815,25815,25815,27862,27862,27862,29910,27830,27830,29910,29910,29910,29878,29878,29878,29910,29877,29877,29877,29877,29877,29845,29845,31925,33940,52824,54937,52824,46518,48663,25520,23375,31793,46518,46486,46517,50711,50744,57050,59228,57083,50744,54970,52824,42292,59196,61309,38098,19114,14889,8615,23341,29680,12841,8680,6567,6534,36083,46583,42325,17099,8746,23374,27567,10826,10761,42292,10663,12938,21549,42193,48467,44273,50612,48499,48499,44274,63389,61276,48630,46517,50744,40212,25422,23309,31695,38033,38066,40179,40244,40212,40211,40146,35888,50710,35952,25486,50677,36017,33938,48564,38065,44437,44372,23473,21393,10697,2244,2245,6439,6503,6471,4390,6471,12842,17003,16971,25454,29679,35986,38099,33841,31727,33808,40146,44372,42259,29581,35953,27436,21196,23276,46452,48532,48532,46452,46419,44339,35953,50679,46485,44437,44371,44339,38065,46551,35985,25356,16969,21193,23306,27532,23337,40244,31826,52956,27471,31535,57116,38067,40147,40179,25356,35920,40179,44404,44372,42259,40179,40179,40179,40179,40211,40179,38066,46485,42259,21162,21130,16904,31728,33841,25520,14954,27567,27567,21261,21228,23341,21261,27567,29680,42357,29648,108},
{32090,34170,34170,36218,34170,32090,32058,32090,30009,30010,32122,32090,32058,32090,32090,32090,32090,32090,32122,30106,32155,32154,34202,38330,36250,34137,32057,32057,30009,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,29975,29976,29944,27863,27863,27863,27863,25815,25815,25782,27862,27862,29878,27862,27862,27862,29910,29910,29910,29878,29878,29878,29878,29877,29877,29877,29877,29877,29845,29877,34005,38100,46518,52889,50743,44405,38100,23407,23439,31760,50743,52856,59130,48630,57050,61276,59196,59163,50776,59196,54970,46518,48630,33872,21194,19114,19146,14921,29647,29744,12841,8647,8712,8647,36084,44503,40244,25421,19147,27502,31760,17035,14954,42260,10663,8680,23695,44274,44273,46353,52693,48532,46387,46419,65503,59163,46517,48598,44405,23341,21228,21228,29646,42227,40178,40211,40244,40212,40211,40211,42292,48597,40146,38131,50677,35985,31825,48597,35920,40146,42226,27633,21393,8649,2212,2244,4390,6471,4390,6503,10696,8551,8551,10697,21196,29711,25453,38066,29647,31728,33840,38098,42259,42291,40114,38000,27436,21164,19050,46452,48532,48597,50645,46451,46419,35952,48630,44372,44372,44371,44339,38001,38067,52856,46453,38033,29581,33869,31724,29643,33872,27632,50778,23244,39856,52792,42260,40179,44372,40179,38001,40146,44404,44371,42259,40211,40179,42259,40179,42227,40146,40146,40146,35920,18985,16872,16937,23341,23341,25585,19148,21260,23341,21261,21228,21293,21293,27567,29680,31760,29647,105},
{32090,36250,36218,36218,34170,32090,32090,32090,32058,32090,32122,30009,30009,32090,32090,32090,30010,32090,32122,30106,32154,32154,34202,38330,36249,36217,34105,32057,30009,27928,29976,29976,29976,29976,30008,32024,29976,29976,29976,29976,29944,27896,27863,27863,27863,25815,25815,25815,25815,27862,27862,27830,29910,27862,27862,29910,29910,29910,29910,29878,29878,29878,29877,29877,29877,29877,29877,29845,29877,34037,42326,46518,57049,50711,52824,38132,21327,29745,33873,38066,42324,54937,52856,52791,57050,44373,46518,57050,61309,59196,42260,52889,23340,19146,19146,17033,27630,29711,23437,23373,10759,10792,6534,36083,46616,44405,25518,21390,31858,33873,27665,27632,44373,27567,25583,21614,42258,44274,46387,52725,50580,46419,50646,63390,52857,44404,48598,52825,23309,21196,23373,35953,42292,44372,38131,40212,40212,42292,40211,42325,48597,44371,50744,48564,38098,33970,46451,40178,46484,44339,48697,21360,8617,164,2244,4390,4390,4422,6535,4390,6438,8583,12777,19116,27631,23373,31728,21163,33840,35921,33873,33840,35920,40146,38033,38001,33874,40180,46484,48564,48597,46484,46452,46451,35952,48598,46485,44371,44339,42258,40146,29615,48598,54937,54969,42292,52855,44371,33806,25419,40244,52890,25389,42097,42227,40179,40211,48631,46453,35919,35887,44372,44371,42291,40211,42259,42259,42259,40179,40147,40146,40146,48531,50644,37969,21130,16970,19213,27665,21261,21228,19148,23341,21228,21261,21293,27567,29680,29680,29647,109},
{32090,34170,36218,34170,34138,32090,32090,32090,32090,32090,32122,30009,30010,32058,32089,32090,32090,32090,30074,30106,32154,32154,36250,38298,38298,36217,34137,32089,29976,29976,29976,29976,29976,29976,32024,32024,29976,29976,32024,29976,29944,27895,25815,25815,27863,25815,25815,25815,25783,27830,27862,29910,27862,27862,27862,29910,29910,29910,29910,29878,29878,29910,29877,29877,29877,29877,29877,27797,29877,29844,40245,48566,48597,52824,46518,31761,29713,21326,31761,35986,25453,54969,59195,50743,54969,50744,52857,54970,57083,52889,48631,36017,23307,19114,19114,14953,21228,19147,27630,25549,25549,25549,23404,40277,48664,42325,17132,6632,19180,31727,10858,6632,42292,10696,10793,21613,44339,44338,48499,54806,50612,48499,46484,54937,50711,44372,44372,35953,31760,25422,31727,35953,40179,42357,38131,40211,40244,42292,40211,42357,48597,42323,40211,48564,35953,33873,48565,40179,44372,44371,54971,31860,10729,2277,2245,4358,4390,6535,14955,4357,6471,4325,6471,10729,23341,36019,44406,38066,33872,35920,27566,55003,61276,44340,42226,35986,42358,35986,46484,48532,46484,46484,46452,46452,33839,48598,42291,42291,44339,42291,40146,31858,50711,54937,57115,42325,59228,44404,31758,27564,31791,38066,38001,38033,42259,40146,40179,46550,46517,40113,44339,44372,44371,42291,40211,42259,42259,40211,42259,40179,40147,40211,46517,46516,33774,33808,25390,21294,27698,17035,21228,19148,21261,21228,21261,21261,27535,29712,44438,31760,131},
{32090,34170,36218,34170,32090,32090,32090,32090,32090,32090,32090,30009,30009,30010,32090,32090,32090,32090,30074,30106,32154,34202,34202,36250,38298,36217,34137,34137,29976,29976,29976,29976,29976,29976,32024,32056,29976,29976,29976,29976,29943,27896,27863,25815,25815,27863,25815,25815,27863,27862,27862,27862,29910,27862,29910,29910,29910,29910,29910,29878,29878,29910,29877,29910,29877,29877,29877,29877,29877,34037,40245,52824,50743,50711,38132,29713,27633,25520,33841,27534,31792,61275,61243,54937,48598,63422,59196,48663,63389,61309,52857,25485,21226,19146,17034,19148,25422,27502,21260,10792,6567,8647,17001,48696,50841,44437,25454,14922,23308,33841,12841,10793,40147,12744,8647,23662,46418,46354,48532,52725,48499,46387,35854,44339,48630,44371,48663,40212,36020,33874,40212,42325,44437,44470,42390,42357,44437,44470,42357,42293,46518,44372,40146,46420,33841,31792,46484,36018,38130,38066,50810,46551,36020,12778,6503,4390,6503,8616,33906,6471,4325,6470,6438,8551,29583,50842,55003,46518,35985,33872,40212,46551,38099,33840,33840,21163,21196,19050,46452,46484,48597,54904,44372,48565,33840,46484,44372,44372,44372,40113,38066,29777,50711,52856,57083,50809,57115,48565,31725,21192,25452,40146,29614,16970,35953,40146,40179,42292,23275,38065,35953,40178,44371,42259,40211,42291,42291,40179,40211,40179,40179,38098,48598,46517,23275,19016,25421,27665,40245,23374,21229,19148,21228,19180,21293,23374,25486,31760,36019,33906,123},
{34170,34170,34170,34170,34170,32058,30010,32090,32090,32090,32090,32058,30009,30009,32090,32090,32090,32090,32122,32122,32154,32154,36250,38330,36217,36217,34137,34137,29976,29976,29976,29944,29976,29976,32024,32056,29976,29976,32056,29976,29976,27896,27863,27863,25815,25815,27863,27863,27831,27830,29910,29910,29910,29910,29878,29910,29910,29910,29878,29878,29878,29878,29877,29877,29877,29877,27829,27797,29845,34005,31793,50711,46582,44470,29745,36019,27698,23472,31793,29680,31825,59194,59163,52856,50678,50710,46517,54937,50744,52889,31759,23307,23307,21259,14921,21228,21228,23308,19147,10759,8647,8679,29712,46583,48696,42325,29712,25616,33938,38066,31858,31858,44373,29680,27631,19501,46386,44305,50612,50645,46419,48532,37902,48533,46517,44372,46584,44504,12842,8616,8648,10729,8616,10762,17100,19278,23439,27633,27633,27633,29713,27633,27633,27600,25520,25552,25585,33907,23374,21228,25487,25454,21261,12809,8583,8583,19051,17002,35953,19017,12679,8486,16969,21195,31727,40245,44470,38066,29582,38066,38098,35986,29648,33840,31727,19050,14825,31760,46452,48532,52791,50743,48565,46484,31759,44404,42292,44404,44372,38000,38066,29777,52856,52856,52889,57082,59162,48597,40144,27499,33871,46517,19148,19116,35953,40146,40179,40147,23308,23276,16937,23276,44371,42259,42259,42259,42259,40211,42259,40211,40179,40211,42324,33840,21195,19048,27533,25454,31793,21196,21261,21228,21261,21260,25454,25454,25486,31760,27567,38164,112},
{32090,34170,34170,34170,32090,32058,30010,30010,30009,32090,34138,32058,30010,32090,32090,32090,32090,32122,30074,32154,32122,32154,36250,38330,36249,36217,34137,32089,29976,29976,29976,29976,29976,29976,32056,32056,29976,29976,32024,29976,29976,27896,27863,27863,27895,25815,25815,25783,27863,25782,27862,29910,29910,29910,27830,29910,29910,29910,29878,29910,29877,29877,29877,29877,29877,27829,27829,27829,27796,36085,36051,50744,52857,44373,29680,36052,21326,21326,27600,29713,29680,52856,46485,46485,50677,52758,46484,48631,46518,54937,25421,19146,23307,23372,19114,21228,21196,27566,21260,14986,14986,17066,33905,46550,46616,42325,19180,6633,19148,33873,6632,8713,40212,14889,4454,19534,44306,42193,50612,50613,44306,50645,35854,52823,46517,46485,50809,52956,12842,8616,19149,12810,8616,10729,15020,19214,19214,23407,23440,21262,23407,21261,17035,14922,10697,8616,10664,21228,10696,8615,8583,6535,6503,6470,8583,6438,21162,25388,42291,33774,16904,6373,10663,23341,25422,27600,31728,35954,23308,57116,44437,29680,38164,46551,31793,25389,12776,17002,46452,46452,46484,46517,46484,46452,31727,44404,48566,44404,44372,42259,40146,36019,40212,50743,52857,57115,59163,48598,40112,23274,10663,38131,12875,19115,35921,40146,44372,48598,46518,40179,40179,31727,38065,42259,42259,42227,42259,42259,42259,40179,40179,40179,38066,31727,12711,21162,27501,31727,31793,21196,21261,19148,23341,23373,25486,25454,27535,31760,40245,38100,111},
{34170,34170,34170,34170,32090,30010,30010,30009,30009,32090,34170,32090,32090,32090,32090,32090,32090,30074,30074,30106,30107,32154,36250,36250,36249,36217,34137,32056,29976,29976,29976,29976,29976,29976,32056,32024,29975,32024,32056,29976,29976,27895,27863,27863,27863,27863,25815,25815,25815,27862,27862,29910,29910,29878,27862,29910,29910,29910,29878,29878,29877,29910,29877,29877,29877,29877,27797,29845,27796,33940,33971,38164,44471,44470,31794,23440,21294,23440,23375,25519,31793,57082,63388,61243,50710,54904,35953,40179,48631,52856,35985,17066,21259,19179,21259,25453,27534,25421,19115,12840,8647,12873,44405,48696,48696,42357,23373,10761,17035,35953,8615,10664,38034,14922,6535,21614,42193,44273,48499,48500,42096,39983,33645,52824,46517,44404,48729,52923,12842,17068,25520,10761,8648,10762,15020,12939,15020,19214,17100,19181,21294,23439,19213,14923,10729,8616,8616,21228,10696,8583,8551,6503,6503,8551,8583,12744,40113,33775,40179,14824,6405,4325,4325,10696,14922,25422,23308,35954,59164,57083,44438,33873,57116,44568,25454,19018,21229,27502,46452,46452,48565,50743,46452,46484,33807,46452,44372,44405,44371,44339,40146,54970,48631,54969,54970,59196,59195,48598,40146,12743,17035,27567,21294,21196,35921,40179,46550,48631,46550,46518,38098,33808,44339,44340,42259,42259,42259,40179,40179,40179,40179,42291,48598,35953,10598,12711,25356,31695,29550,21196,21228,21228,23374,25422,25454,27502,27534,31760,42358,27599,109},
{34170,34170,34170,34170,34138,30010,30010,30010,30010,32090,34170,32090,32090,32090,32090,32090,30042,30074,30106,30106,30106,32154,36282,36250,36249,36217,34137,32057,29976,27896,29976,29976,29976,29976,32056,32056,32024,29975,32056,29976,29976,27896,27896,25815,27863,25815,25815,25815,25815,27863,27862,29910,27862,27862,29910,29910,29910,29910,29910,29877,29877,29877,29877,29877,29877,29877,29877,29845,27797,36052,33906,40212,46550,46518,44438,27633,23472,21326,23407,25519,27600,38165,38165,38165,46517,50678,42259,46485,50744,50711,40211,19179,19147,21292,17099,19115,21130,25356,23308,12872,8647,14953,42324,50809,50777,44438,31793,31826,36051,40212,33938,36051,42292,29679,25551,19501,35951,46386,46419,46387,46419,42193,42194,50711,46485,44405,48729,50875,12842,8616,27633,14954,10729,12842,25520,21327,23375,25519,25455,21294,14987,23407,19213,14955,10697,8616,8616,21228,10696,10663,8584,6503,6503,6503,8551,14792,44307,35888,23308,6405,6405,6373,4324,10664,12745,19115,19115,42292,40179,50745,52857,33905,57084,50842,27502,29583,21262,40147,35887,44339,52856,54937,48565,48532,31727,44404,46485,46485,44339,42291,40145,52889,54969,54969,57050,52825,59163,46517,31695,6503,36019,36052,36052,27567,33840,42259,44372,46550,48598,50744,50744,38033,46453,44372,42292,42259,40146,40146,40179,40179,40179,44404,44437,33872,21195,19049,23308,31695,33808,14890,21261,19116,19148,25422,27502,25454,27502,31792,27534,27599,109},
{34170,34170,34170,34170,32090,30042,30042,30010,30010,32090,34138,32090,32090,32090,32090,32090,30042,30074,30075,30106,30106,32154,34202,36250,36217,34137,34105,32056,29976,29976,29976,29976,29976,29976,32056,32056,32024,32056,32056,29976,29976,27896,27864,25816,25815,25815,25815,25815,25815,25815,27862,29910,29910,27862,29910,29910,29910,29910,29910,29878,29910,29877,29877,29877,29877,29877,29877,27797,27796,38133,38100,48631,50711,50679,48631,25553,19278,29778,27600,23374,29745,25617,19247,25520,44438,50743,48598,44405,38066,25454,23406,21227,29712,21260,19083,19050,21194,25355,25387,14921,10727,12840,36051,48728,52922,46518,21261,8745,17197,38067,8745,15084,36050,17035,4487,19534,23307,44306,44274,40081,48500,42161,48532,46517,44405,42291,50810,50843,10761,8648,25487,14988,8681,12842,23439,23407,17101,44505,44406,29746,21326,14987,19181,14955,10729,8616,8616,19180,10664,8583,10664,8551,8583,6503,6470,12744,35888,33775,12712,4292,4292,4324,6438,10664,6502,17035,48632,52857,48663,61342,50744,38099,50777,52890,38067,31728,23342,25389,46484,46452,48565,48598,48533,48565,33839,46485,44372,44372,44339,42259,38066,36082,54970,54937,57115,40179,57115,46485,27470,8584,40245,33874,33906,31793,38066,42259,42259,42292,44372,31727,16937,23243,44404,44405,42292,40179,38066,38033,40146,40179,40178,44405,46550,44405,31695,14856,14856,29647,40180,29681,23341,25487,17002,25421,27502,25454,27534,31760,33906,27599,111},
{34170,34170,34170,34170,32090,30042,32090,32058,32090,32090,32122,32122,32090,32090,32090,32090,30042,30074,30074,30106,30106,32154,34202,36250,36217,34137,32057,29976,29976,29976,29976,29976,29976,29976,32056,32056,32024,32056,32024,29975,29976,27895,27863,25815,25815,27863,25815,25783,25815,25815,27862,27862,27862,29910,27862,29910,29910,29910,29910,29910,29877,29910,29877,29877,29877,29877,29877,27797,29845,31892,31859,48599,48599,48598,42259,25552,25520,25520,25487,21294,27665,21360,17135,25552,44437,42259,40178,48630,27533,19146,12873,14987,19148,19180,23309,21130,27501,27500,19114,12776,14953,21260,23373,38164,52922,44438,23309,10761,14986,40147,10761,12907,33970,19083,6535,10954,29646,44274,40146,40179,44306,37902,44306,54937,52856,40179,50810,50843,10729,10697,23375,14955,8649,12874,19213,27633,23407,29746,33971,31826,19181,15020,21359,17068,10696,8616,8616,19181,8616,8583,8616,8583,8583,6503,6470,12744,21162,44372,16840,18887,23048,18855,8453,27502,23276,25454,52889,35985,48663,54970,50744,48664,57148,59229,50777,38131,14857,25324,46484,46452,48565,48598,46485,48565,35985,46485,46485,46485,44339,42258,40114,29744,54969,52889,61277,29679,59163,44371,25388,8616,36019,42358,42325,42259,42227,42227,42259,42292,38066,38033,40178,31727,40146,44404,44372,40179,38066,38033,38066,40146,40179,42291,38066,38066,25388,10630,10663,23340,48632,44471,21262,33940,19115,23341,25454,25454,25486,31760,44470,27599,111},
{32090,34170,34170,32122,32090,32090,32122,32090,32090,32122,32090,32090,32090,32090,32090,30042,30042,30074,30074,30106,30106,32154,34202,36250,34169,34137,32056,30008,29976,29976,29944,29976,29976,29976,29976,32056,32056,32056,32056,29976,29944,27864,25816,25815,25815,25815,25783,25815,25815,25814,27862,29910,27862,27830,29910,29910,29910,29910,29910,29878,29877,29878,29877,29877,27829,29877,29877,29845,27764,33940,36052,48664,48598,50678,50710,31794,27633,25552,21294,19181,25552,23473,17135,14989,40277,50711,50678,40178,21162,19114,12841,14955,25519,23374,23309,27435,21192,19080,23304,19080,12841,17067,23341,38132,48696,44470,33937,34003,38131,42292,33906,36019,42260,38034,29613,44469,33807,42161,44339,35920,40080,37934,44306,59130,50711,23276,48730,50843,10729,8616,23439,14987,8616,10794,23472,29714,12874,38198,48762,48762,19214,21326,23505,14987,8648,8584,6535,19148,8616,6503,8583,6503,8583,6503,6470,14824,35919,48598,37871,27209,27241,27273,18920,35953,25421,35986,48631,40210,42193,37968,48598,57083,59229,57148,52890,35953,19050,29581,46452,48532,48598,48598,46452,48597,31759,46517,48598,46485,42259,42258,40146,31857,54969,52856,57115,42357,57050,40178,27533,12777,17002,40146,42259,42291,40179,40179,42259,42292,42292,44405,38066,31727,46517,44372,42292,40146,38066,38033,35953,40114,38132,36051,29648,33873,19082,27567,25389,31728,38132,52955,25488,29778,23341,25454,17034,25454,25486,31792,33905,27567,106},
{32122,32122,34170,32090,32090,32090,34170,34170,32090,34138,32090,32090,32090,32090,32090,32090,30042,30042,30074,30106,30106,32154,34202,36250,34137,34137,32056,30008,29976,29976,29976,29976,29976,29976,32056,32056,32056,32056,32056,29976,29944,27896,27864,25815,27863,25815,27863,25815,25783,25814,27862,27862,27862,27862,29878,29910,29910,29910,29878,29877,29878,29877,29877,29877,29877,29877,29877,27797,29844,40180,42260,46518,52791,48565,50678,31793,19279,21327,17133,17165,31858,23473,17102,15053,12875,15019,19115,21162,21130,19114,17034,19116,23373,23405,23342,27532,25385,19047,31689,25352,4423,6536,19148,36019,48696,44438,23373,6665,10890,40212,6568,8746,33938,35921,38099,38164,29680,42193,59195,50678,46419,35822,50613,57016,57049,52825,48665,50810,10697,10697,19213,14955,6568,10794,15052,21327,21326,27665,27698,25552,17068,19214,17132,12875,8616,6536,6536,19148,8584,8584,6535,6535,6535,6503,6503,14857,44339,42259,40048,25193,27241,23047,16839,25454,27567,50777,54970,52857,40015,46452,50711,57083,59261,55035,48598,31694,16905,27501,46484,46484,57050,54969,48565,50646,31727,46485,48598,46485,44339,44307,38033,33905,52889,54970,59164,35986,46550,40145,27499,16969,23308,42291,44339,42259,40179,40211,42292,44404,46517,52824,50744,38065,46517,44372,42292,40179,38066,35953,35953,36085,30010,27962,27994,29976,19215,17002,19050,46551,40245,48730,27633,31891,23341,25422,17002,23341,27567,31825,27567,27567,105},
{32122,32122,34170,34170,32090,32090,34170,32122,32090,32122,32090,32090,32090,30009,30042,32090,30042,30042,30074,30074,30074,32154,34170,36250,34137,32089,32056,29976,29976,29976,27928,29976,29976,29976,32056,32056,32056,32056,32056,29976,29976,27895,27863,27863,25815,25815,27863,27863,27863,27863,27863,27862,27862,27862,27830,29910,29910,29910,29878,29878,29878,29878,29877,29877,29878,29877,29877,27797,25684,42358,46551,50712,48598,52758,52791,40277,19214,25488,21327,23375,31826,19248,17135,12908,12875,14955,17034,19081,17001,21227,19115,19148,25520,19181,25487,23372,25385,16966,35915,33736,8615,10729,23341,42358,48729,44470,23341,8713,12906,42260,6600,12874,31825,35953,42292,36018,21196,42193,44306,48467,37967,37935,57050,57017,57017,52858,48730,48730,10697,19181,23374,6568,8649,12940,15053,19214,29778,46552,38067,25487,27600,21294,27598,12842,8584,6503,6503,19148,8616,8616,8583,8583,6535,8583,6503,16937,42193,40147,42227,20967,25160,23047,27403,29582,50743,52921,54970,46485,42096,46452,50712,52890,55003,42292,46550,40179,40244,40147,46484,48532,48598,48598,48533,48565,33807,46452,44404,46452,44339,44339,42259,57083,54969,57082,59228,27533,35951,35983,21257,23337,40178,44371,42292,42259,40179,42259,42291,44404,46517,42259,42291,46517,46485,42291,42259,40179,38098,35985,35953,36085,34103,32023,29911,31926,21196,14856,17001,48664,46519,46552,21262,33972,21196,25421,25454,27600,31825,33905,44438,27567,113},
{34170,34170,34170,34138,32090,32058,32090,32122,32090,32090,30009,32090,32090,30042,32090,30042,30042,30042,30074,30106,30074,32154,32122,34170,34137,34137,32024,32056,29976,29976,29976,30008,29976,32056,32056,32024,32056,32056,32056,29976,27928,27895,27895,27863,27863,27863,27863,27863,27863,27863,27863,29910,29910,27862,27862,29910,29878,29910,29878,29878,27830,27830,29877,29877,29877,29877,29877,27797,29845,27764,29746,46518,50678,48565,50678,36018,27601,29680,48664,52858,33972,21361,17135,15055,10861,8682,17067,21194,19114,19148,31793,23439,23374,21294,21326,25420,27465,19079,33768,44262,10696,17035,27535,48696,50777,46551,35954,35987,35987,44373,33873,35986,40179,38099,40179,31727,29614,42193,42193,48467,40047,46452,61308,59130,46549,23308,46617,48697,10697,19149,19213,8648,8649,12939,17133,17133,21359,27633,17100,19278,15052,19214,14987,8616,2277,229,2310,19148,8648,8584,8584,8583,8583,8583,8583,12711,37968,38034,44340,21064,25225,31499,37870,37838,48565,54970,48631,54937,42096,40113,52857,54971,55003,38131,44438,40212,46551,42324,46452,46452,48598,48598,48533,48533,31727,46485,48598,44372,44339,44339,42259,55002,54970,57115,59196,31759,46581,29643,21224,25418,44372,44340,42259,40211,40179,42259,42259,44437,44339,35888,27501,40211,44404,42291,42259,40179,40211,38065,38034,40179,38066,38066,40147,38066,29614,25355,23243,36018,46551,54971,38165,44439,21164,27535,25454,33906,31826,33905,40212,27534,93},
{34170,34170,34170,32090,32090,30010,32090,32090,32090,32090,30009,30009,30042,30010,30042,32090,30042,27994,28026,30074,30074,32154,32154,34170,32089,32089,32056,32056,30008,29976,29976,29976,29976,29976,32056,32056,32024,32024,32056,29976,27896,27895,27863,25815,25816,25815,27863,27863,25815,27863,27863,29910,29910,27830,27862,29910,29910,29910,29910,27830,29878,29878,29877,27829,27829,29877,29877,29877,27797,27764,29779,52857,52824,52791,52791,48631,21294,44438,44438,57083,36085,17135,15054,12941,8715,6602,10762,19115,14922,21261,21229,19213,21228,21228,27599,17001,27497,16966,31687,37959,10695,17003,21195,48697,50810,48631,27567,14954,17067,42293,19180,17035,31760,29647,29647,38066,38067,25356,42193,50612,46387,54937,61211,59130,46517,21195,42391,42391,31826,31794,31793,31793,31826,33939,36052,36019,36084,38165,33939,33938,33938,33938,25486,8615,10696,8616,12809,19116,10696,6470,6470,10663,10534,12679,14727,12679,21065,21098,27404,21000,21000,27371,48598,48630,54970,57116,50777,61309,44209,46485,52890,52890,48697,46551,44372,42259,33840,35920,46452,46484,48598,48566,46484,48565,33807,46485,46485,46485,44339,44307,40178,57115,55002,57082,59163,38065,38095,27498,33838,38033,42292,42292,40211,40179,40179,40179,42292,44372,42258,33807,10631,23340,42324,42291,42259,40211,40179,40146,40146,40179,38066,38066,38066,35952,29613,16936,14855,29646,42325,33906,27535,38132,25422,31760,27534,27599,27567,29680,8582,19147,112},
{34170,34202,34170,32122,32090,30042,30041,30042,32090,32122,30009,30009,32090,30041,32090,30042,30042,30074,30074,30074,30074,32154,32122,32121,32089,32089,32024,32056,32057,29976,29976,32056,32024,32024,32056,32024,34104,34104,34104,32024,29976,27896,27863,25815,25815,27863,25815,25815,25815,27863,27863,29911,29910,27830,27862,29910,29910,29910,29910,27862,29910,29878,29910,29878,29878,29877,29877,29877,29845,29844,36052,52825,50743,48598,46550,46518,27632,38131,48631,59164,40246,27600,36020,23407,23374,6602,12941,21262,19180,23341,25520,23374,25421,27436,25421,23340,29578,14885,31656,31656,14888,17035,29647,48664,50809,44503,29712,40212,31761,44437,35954,40179,40244,33905,35986,35953,48632,21163,38033,42193,48500,63356,61244,59130,54969,46517,46485,42259,40179,38099,36018,27566,31759,35921,35921,29647,23309,27502,27535,23341,25454,21260,17065,19081,21162,19115,25454,27534,25356,27534,27501,23178,18920,21000,21033,21033,23081,23113,25194,23113,23145,33742,61276,59293,57148,57116,50744,59229,46355,44307,48631,42358,42325,42325,44339,42258,40113,40113,46452,46452,57050,59131,46485,48598,35953,46485,48631,46485,44339,44339,42259,50744,59195,59195,59195,59195,42324,35952,40145,38066,42259,42259,40179,40179,40179,42227,42292,44404,44404,33905,25518,25388,38065,42291,42259,40211,40179,40179,40179,40179,38066,38065,38065,31693,21193,16968,19016,27500,31630,29582,25454,25454,25454,29648,25454,29648,27599,29647,42358,23310,58},
{34138,34170,34170,32090,32090,30010,30010,32090,30042,32090,32090,30042,30009,32090,32090,30042,30042,30074,30074,30074,32122,32154,32122,34170,32089,32089,29976,30008,32056,29976,29976,32056,32024,32024,32056,32024,32024,34104,34104,32024,29976,27896,25815,25815,25847,25815,27863,25815,27863,27863,27895,29911,29910,27862,29910,29910,29910,29910,29910,29910,29942,29910,29910,29910,29878,29877,29877,29877,29877,29845,42358,50776,52824,44373,40180,38132,25520,36051,42357,42228,50615,50615,46422,21165,38132,19246,10827,19213,23407,27600,23373,21261,25356,25421,31760,10695,27530,12772,29608,31656,12775,12810,31761,48697,50841,46551,35953,40212,38066,44405,35954,44373,38066,35986,29646,35986,48664,35954,44372,35887,42226,59130,61243,59130,46485,21195,48631,23406,38099,36018,38099,46583,40146,48566,48630,48630,52791,52791,52823,52823,52791,44403,25451,25353,40111,35887,44339,46484,44371,44339,44339,40048,31661,35823,33742,31597,25323,27436,35855,21098,18985,29549,55002,59294,57149,57083,46550,52824,40080,40015,44340,21163,21163,35920,33806,40145,42259,40145,46452,46452,52791,50678,48597,48565,31727,46484,46485,46485,44339,44339,42227,48631,54936,61275,59195,52857,29615,27468,40178,40146,42259,42259,40179,40179,40179,42259,42259,44372,46453,36083,25519,29581,38000,42259,40179,40211,40179,40179,40179,40179,38098,35985,35984,23306,21193,16935,14790,27533,33840,25389,21228,23276,29648,29648,27567,31793,33938,38132,55003,31728,160},
{34170,34170,34170,32090,32090,30010,30010,32090,32090,32090,32122,30009,30009,32090,32090,30042,30042,30074,30074,30074,32154,32154,32122,36250,34137,34137,30008,30008,32056,29976,30008,30008,32024,32024,32056,34104,32024,34104,34104,32056,29976,29944,27864,25815,25815,27863,27895,27863,27863,27863,27863,29911,29910,29910,29910,29910,29910,29910,29845,23473,13006,15151,13038,17232,15087,17265,15152,19312,17232,19345,23538,17199,31859,23439,36085,38165,17134,25552,17133,23407,27600,33777,44309,29551,23440,19051,16905,21164,19019,27404,37903,29484,27501,29615,27599,10663,29610,19046,31656,31656,19114,12809,12809,40212,46616,44503,25486,10923,12939,42325,14954,15084,34003,38066,42292,40179,35986,44405,44437,38000,50678,59163,61244,59130,54936,44340,54937,38034,46518,44373,44438,44470,48663,48598,48598,50710,50678,50710,52822,52854,52790,50678,46517,48597,50742,46549,46484,48532,46484,46484,46451,46451,46419,46419,46451,46452,23308,25487,57083,27567,19148,50776,57050,59294,59229,55035,44372,48532,42193,42161,38034,25389,31695,35888,40080,40113,40179,42259,40113,44339,50678,48630,48565,46485,29646,46452,48598,44372,44306,44339,40178,42259,61243,63356,57082,48632,42325,31727,40178,40179,42260,42259,40179,40179,40179,40211,42259,44372,44372,46550,46485,35920,35887,40146,40179,42259,38098,40146,40146,40146,38066,35953,35952,23306,19048,16967,16968,25388,38001,29614,23308,25454,25421,25486,25422,27534,38164,33906,52922,35986,128},
{34170,34170,34170,32090,32090,32058,32058,32090,32090,32122,34202,32090,32057,32090,32090,30042,30042,30042,30074,30074,32122,32154,32122,34170,34137,32089,30009,30008,32024,32056,32056,30008,29976,32024,32056,34104,32024,32056,32056,29976,29976,27896,27896,27895,25815,27863,27863,27863,27895,27863,27863,29910,29910,27862,29910,29910,29910,29910,34070,42359,27633,31891,21359,27698,27666,29844,23505,27699,21425,15087,10894,10894,12974,12941,12974,12941,12974,10829,12942,10861,10893,10828,19182,31794,27341,27177,31498,33578,33579,37771,37770,35690,25226,14824,25386,19081,23240,29576,27464,25352,17002,21229,19181,40245,48664,44470,27566,8810,6665,42325,12809,10826,29744,38066,50777,50744,42259,46518,44372,42258,48597,57017,59163,59130,44372,17002,48664,19147,40179,35986,44471,42390,48696,52857,52888,44403,35952,52821,46384,52721,54869,44373,46551,38067,19082,44438,48566,44372,48564,52758,48532,50710,48565,40179,33841,29615,23342,23374,40245,40211,42292,63422,57148,59294,57181,57115,38000,52726,46419,44306,35921,21163,27437,33775,40080,40113,40178,48664,44371,46419,50679,48565,48565,46452,33872,46452,46518,44404,44306,44306,44339,61276,63389,63421,63422,48599,50712,33840,42259,40179,42259,40179,40179,40147,40179,40211,42259,40146,27404,33839,40113,42226,42226,42259,40211,42259,40179,46518,40179,40179,38098,38033,38065,29613,21194,19048,14855,27500,31694,29582,25421,33841,25421,27534,23341,25454,38196,40245,52890,35954,123},
{34170,34170,34170,32090,32090,32090,32090,32122,32090,32122,34170,32090,32058,32058,32090,30042,30042,30042,30074,30074,32122,32154,32122,32122,32089,32057,30008,29976,29976,32056,32056,32024,29976,32024,32056,34104,34104,32056,32056,32024,29976,27895,27895,27895,25847,27896,27895,27895,27863,27863,27862,29943,29910,29910,29910,29910,29910,29910,38230,50777,33972,33972,31827,34005,29746,34004,27666,23538,10926,10926,8813,10894,10894,10861,10893,8781,10893,8748,10861,10861,12908,19247,21392,6635,19247,6538,8650,12843,14923,25421,25421,10631,19050,14825,27466,16999,23271,29575,21159,10599,14858,14890,6471,38132,48696,46583,33841,23341,25421,42260,25421,23373,31760,35953,25421,31792,48598,46485,35952,52823,46517,57050,59195,57082,52856,42291,54905,38033,48566,40179,38066,44437,29615,50744,33807,48598,38000,42226,44305,50676,42260,44405,31727,33841,19147,25454,44406,44340,44338,50645,50645,52790,42258,42291,42293,40212,40180,46518,48631,50710,54937,63422,59261,61342,57181,57148,33775,46420,40080,46420,35953,23341,21163,25356,42226,40113,38098,46486,48532,46452,54937,52856,46485,46452,31759,46452,46518,46452,44306,44306,40178,59261,59229,59229,59229,50744,40179,42259,40179,42259,40179,40179,40147,40147,40179,42259,42259,35921,27373,27501,46517,42226,42258,42291,42259,42259,40179,40211,40179,40179,38066,35952,35887,23242,21096,16936,12775,21194,31727,31695,25389,27567,25421,23341,21196,25422,29680,23374,59197,25390,116},
{34170,34203,34170,34170,32090,32122,34170,34170,32122,32090,34170,32090,32089,32057,32090,30042,30042,30042,30074,30074,32122,32154,32122,32089,32089,32057,29976,29976,32056,32056,32056,32056,32024,32056,32056,34104,32024,32056,32056,32056,29976,27895,27895,25847,27895,27895,27863,27863,27895,27863,27894,29943,29911,29910,29910,29910,29910,31990,40311,55002,38165,34005,38198,38230,29876,31957,29844,15152,10926,10926,10926,10926,10894,10861,10893,8813,8813,8812,12941,12876,17005,12974,8748,6668,6667,4522,4586,6602,19180,27534,25389,10631,16970,10664,27466,19079,23303,29576,21160,10664,17035,17035,12810,40245,46616,46583,29647,19277,21325,44405,25486,27632,36051,38099,31760,29679,50711,44404,38065,57082,46517,42292,59163,59130,44405,21260,50712,23406,42292,29680,33873,42325,27567,50744,31793,50711,46453,57082,61243,35921,35986,44406,35954,40180,33873,50809,48599,33742,40047,42127,42160,37935,42226,54937,42259,50678,59195,61276,54969,50710,54969,61341,61374,61342,57181,44438,27469,27469,42193,44405,31760,29647,25389,23243,35921,35920,38098,40147,48532,46452,50743,48630,46517,44372,35953,46452,48631,44339,44306,44306,38033,25421,29679,35985,42259,42323,44371,40178,40179,42292,40179,40147,40147,40147,40179,42259,46485,50744,46387,25355,44339,42226,42291,42291,42259,42259,40179,40179,40179,40179,38098,35920,33774,27403,18983,18984,14791,19081,31759,19050,14824,25421,25421,23341,25454,19148,27534,46583,57116,31728,123},
{32122,34170,34170,34170,32090,32090,34138,34170,32122,32090,34170,34170,32090,32090,32090,30042,30042,30042,32122,32122,32122,34202,32122,32089,32089,32057,29976,29976,32024,32056,29976,30008,32056,32056,34104,34136,32056,32056,32056,32056,29976,27895,27895,25847,25847,25847,27863,27895,27863,29911,29942,29943,29910,29910,29910,29910,29910,29910,38263,57083,42358,31858,33939,34004,8845,8845,6733,8781,4652,4652,8781,10861,8781,6700,8748,8748,8780,8748,17006,12844,10828,8747,8748,8716,8715,4521,4521,6602,14988,27567,21195,10663,14825,8551,25419,23272,27464,19078,21192,4358,10696,10729,12809,38099,48664,46551,40114,38066,38066,42260,35953,33905,35985,38098,29647,31760,40147,38033,40178,57082,52823,50646,61211,59130,52856,42292,54905,38066,48630,40179,40179,46453,29582,50712,29550,48566,33678,50646,54905,46485,33808,35986,33841,27534,33873,59229,46486,48533,52757,52757,52758,48531,48597,46453,31694,40179,46485,59195,52824,50678,54969,59196,61374,61342,57116,61309,54937,31760,42227,44470,40277,35987,44406,40180,38067,25389,19083,25389,46484,46452,50678,48630,46484,44372,35953,46452,48598,44372,42258,44306,35888,8550,8518,10630,38098,42291,42259,38065,33807,40146,40146,40146,38066,38066,40179,42227,44372,50744,40049,21098,40114,42226,44339,42291,42259,42259,40179,40146,40179,40179,38066,33839,33774,27435,21064,16871,16903,19016,25388,12646,21163,25389,23341,23341,25454,23309,25454,36018,42293,29647,120},
{32122,34170,34170,32122,32090,32090,32090,34170,34170,32090,34138,34170,32090,32090,30009,30042,30074,30074,30074,32122,32122,32154,32122,32089,32089,32057,32024,32024,32056,32056,29976,32056,32056,32056,34104,34104,32056,29976,32056,32056,29975,27895,25847,25847,27895,25815,25847,27863,27863,27862,29942,29943,29910,29910,29910,29910,29910,29942,38262,57083,42358,42358,38132,38198,12974,8814,8781,8781,8748,6700,8748,8748,8780,6699,8748,10796,17135,19280,10828,6667,8747,6667,8715,6667,6635,4521,2473,6602,12875,17035,19115,10631,14889,10632,25388,19049,31658,14821,25352,23272,14856,8616,10697,46551,63487,59196,38131,25648,29809,52857,27599,27664,42357,48663,33873,31727,46518,31694,42291,52824,61243,57050,54937,59130,44372,23341,50777,29680,44438,33841,36018,44405,29680,52825,31760,52792,40081,54937,52824,48565,33808,25454,35954,33841,38099,59229,50744,59163,50711,50711,52791,48598,50744,48598,48630,63389,54937,57083,54969,50710,57049,59196,61309,61374,57116,61342,54970,23308,40179,44406,44471,44536,44536,25519,44438,35954,25422,27501,46452,46452,50710,48630,46484,44339,35953,44372,50711,46484,44306,42258,35887,6437,6405,10598,25421,42291,42226,31694,29614,40146,40146,40146,38066,40146,40179,42227,46518,42292,35824,25324,12711,40146,44339,42291,42291,42259,40147,40178,40179,40179,38066,38000,33741,25289,19015,18983,21064,23210,19017,12646,12711,23308,23341,23341,25421,27534,25421,33905,38132,21228,121},
{32122,34170,34170,34170,32090,30042,32090,34170,34170,34138,34138,34170,34170,32090,30041,30042,30074,30074,30074,32122,34202,32154,32122,32089,32089,32057,29976,29976,32056,32056,32056,32056,32056,32056,34104,32056,32056,29976,29976,29976,29976,27895,27895,27895,25847,27895,27863,27863,27863,27895,29942,29942,29942,29910,29910,29910,29910,31958,36183,50842,36084,40278,36019,38132,29746,19279,21327,15086,21295,21360,19246,10828,8780,6667,10860,21392,15086,6635,6667,6635,6635,6667,6635,6635,4554,4521,6634,6602,12874,12841,21195,10664,10664,12777,19115,23211,29577,16967,25384,29608,19080,8649,14955,55035,63455,54970,40244,36083,42356,50744,33872,33937,46518,50744,29614,40179,52857,31727,44437,50743,59195,57050,57050,57082,52824,44340,54969,42259,52791,44307,44340,46517,33807,52824,31694,52791,31597,42292,35920,48566,31727,27502,31825,29615,42358,61342,44438,61374,55002,46518,48598,31695,42324,46485,52857,61308,52856,57050,52857,50743,54936,57115,61342,48697,46583,48631,52857,29614,42292,42325,31759,17035,12809,10664,21293,50745,29615,48598,46419,46419,52824,48598,46484,44371,38066,44404,48598,44372,42258,42258,33807,6436,8517,10598,14791,42291,42259,35920,35920,40146,38098,40146,38066,40146,40179,42259,42260,29614,33776,19115,21228,27469,42259,42259,42291,42259,40146,40178,40179,40146,38034,38001,35887,23209,16934,12709,14823,23210,10598,12614,12678,23308,23308,23341,25421,25454,25454,33873,27567,16970,113},
{32122,34170,34170,34170,32090,30010,30042,32122,34202,34170,32090,32090,32090,32058,32058,30042,30074,30074,30074,32122,34170,34202,32122,32089,32089,32057,32056,29976,32056,32056,32056,32056,32056,32056,34104,32056,32056,29976,29976,29976,29976,27896,27895,25815,27895,27895,27895,27863,27863,27862,29943,31991,29942,29910,29910,29910,29910,29910,29910,31957,31957,36118,29746,40310,29745,31858,31858,29843,29745,31858,31892,13006,8780,15021,21360,10861,6732,6667,8780,6700,8748,8748,6667,8748,12747,12714,8618,8617,12842,16970,25357,10664,12777,10696,10729,19083,31658,16966,25351,29609,19112,6503,12842,48664,50810,44438,36019,38131,42357,46518,38131,40212,50776,46550,33873,40179,52889,42324,48598,50743,57017,46485,57083,59130,59162,59162,59130,57082,57082,57082,57082,57081,57049,59130,57049,59130,57049,57050,50744,46485,31760,25454,35954,33873,50777,59294,44438,61374,57115,44437,44373,31727,46485,40146,48630,61276,52856,57050,50743,50743,54937,52857,46583,50809,50809,52889,61277,31759,44438,31726,21128,19049,21228,12776,19049,44372,63389,54970,46452,46452,54937,50743,48565,44372,35953,46452,48630,44371,44306,42258,38000,40146,38033,29517,31630,38033,42258,40113,40114,38066,40146,38098,38033,40146,40179,40211,44405,40211,44405,42357,31793,31694,42226,42259,42259,40179,40146,40178,40146,40146,38033,38066,38033,23274,16935,12709,10629,21129,12614,12614,16937,19115,21228,19050,25421,25454,27534,29648,55003,35987,86},
{32122,32122,34170,34170,32090,30042,30042,32090,34170,34138,32090,32057,32090,32090,30009,30042,30074,30075,30074,30074,34202,32154,32122,32089,32089,32089,30008,32024,34137,32089,32056,30008,32056,32056,34104,34104,32024,32024,32056,29976,29976,27895,25847,27863,25847,27895,27863,27863,27863,27895,29943,31991,29942,29942,29910,29910,29910,29910,31957,31989,31957,36117,31858,40310,31826,44471,33971,36085,33971,36052,33940,15021,14924,12876,8812,8813,8780,6700,8748,8748,8748,8748,6635,4587,24938,37065,37130,29034,23146,23113,16807,10631,12744,14922,10729,14922,25385,25321,29544,14853,21128,6471,8649,12842,14890,19116,10631,14890,10631,12777,14890,14954,10728,19115,42292,48598,59163,57050,48598,50743,57082,59163,54970,61309,52857,46550,57083,46583,57083,55002,55002,54970,52889,57082,54970,57082,57050,54969,57082,42227,29647,27502,40212,48599,52857,61342,46584,61342,50777,31727,40115,40244,35986,46550,46485,57083,54969,54937,54937,54937,61244,54970,57115,50809,52857,52857,40179,40146,40212,25354,25288,12677,21195,21195,25354,50743,63422,59163,46452,46452,50711,48630,46485,44372,33840,44372,48631,46452,44306,42226,35952,50744,54872,29420,23211,31694,44339,42226,40146,38066,38066,38066,38066,40146,40147,40211,40179,36018,44470,48631,35986,35953,42291,42291,40179,40179,40146,38098,38098,38065,38033,38066,31726,25354,16935,12774,12742,19049,16807,31435,25324,25519,27599,27502,25454,29615,29648,38100,48665,31761,46},
{32090,34170,34170,34170,32090,32090,32090,32090,34170,34138,32090,32057,32090,32090,32057,30042,30074,30074,30074,32122,34170,34202,32122,32089,32089,32057,30008,32056,34137,32089,30008,32056,32056,32056,34104,34104,32024,32056,32056,29976,27928,27895,27895,25815,27895,27895,25815,27863,27895,29943,29943,31991,29910,29910,29910,29910,29910,31957,50810,55036,55003,38198,31891,40310,42391,31827,29779,25618,38197,38101,33810,23343,12973,13006,13038,12974,13006,15054,12974,17135,12941,12974,8748,12812,18923,16843,18825,27309,31599,27337,23144,12744,12711,12809,10729,17002,25388,14759,29610,14854,25384,27497,17001,10729,14922,23342,14922,21228,19116,21228,23341,21261,17035,25422,40212,52857,59196,52824,38098,48663,57050,54970,57050,61309,38034,25389,44438,21261,48632,27567,38099,29680,19115,42325,16970,46518,48630,52791,48598,31695,29647,27534,61309,44373,52857,61342,59261,59229,48631,33808,40114,31760,12711,46518,46517,57050,52824,44373,61309,63422,54937,57083,55003,44471,48664,61276,42260,40147,38067,33807,27368,23208,12709,12678,25354,54970,63422,57082,46419,46420,50711,48598,46484,46420,40179,46452,46485,44372,44306,42258,42292,54970,52824,46486,48664,38099,27500,33774,40178,38034,38066,38066,38066,40114,40146,40179,40211,42291,40178,42324,19082,23275,44339,42259,40179,40146,38098,38066,35953,31727,35953,35985,29613,23306,14855,14887,16935,25323,23210,23145,23275,21228,23309,23341,25454,25454,29680,48729,42325,33874,26},
{34170,34170,34170,34170,32090,32122,32090,32090,32122,34170,32090,32058,32058,32058,32057,30042,30074,30074,30074,32122,34202,34202,34170,32089,32057,32057,30008,30008,32089,34137,32056,32056,32056,32056,34104,34104,32056,32056,32056,29975,29976,27895,27895,25815,27895,27895,25815,27863,27895,29943,29943,31991,31990,29910,29942,29910,29910,31957,55003,52923,40278,33971,29778,40310,42391,40278,17167,17199,38165,36085,29746,21327,15086,13006,13006,10893,12974,13006,10860,12974,15054,12908,15021,14956,25165,25099,25035,20970,25422,16938,12744,12776,21130,14857,21228,25453,33807,12678,27465,16934,25384,29576,19080,10697,12745,21164,8518,16970,8551,17003,14857,16970,12777,8551,23406,36018,54970,44405,40146,50744,54969,48631,63389,59196,40147,50776,40179,44405,48664,38131,50777,25388,44373,29614,31728,44437,48630,48630,44405,16937,29647,46551,63422,42292,54938,59261,59196,61277,46518,33808,38066,33516,35176,48436,46485,46517,57083,52857,54938,55003,42292,50711,46551,40244,46551,59164,57083,46519,27503,27468,29513,31562,14790,12678,27500,61276,63422,54970,46419,46420,50743,46517,48565,46452,40179,46452,48663,44371,44306,42258,44372,57083,57115,44405,42357,19082,19017,33774,40178,38033,38034,38066,38066,38066,40146,40147,40146,40146,44372,38065,27501,27501,35952,42259,42227,35953,38066,35986,33840,35953,35953,35952,25420,14854,12774,12741,12742,25355,31695,29582,25356,23308,21227,23373,25421,25454,31792,50777,42325,19149,29},
{34170,34170,34170,34170,32090,32122,32090,32090,32122,34170,32090,32090,32058,32090,32089,30042,30074,30074,30074,32122,34202,34202,34170,32057,32057,32057,30008,29976,32056,32057,29976,30008,32056,29976,34104,34136,32056,32056,32056,32056,29976,27895,27895,25848,27895,25815,25815,27895,27895,29943,29943,31991,29910,29910,29910,29910,29910,34070,52922,44503,36052,33972,31924,38230,42358,42423,17101,23473,15054,31892,19312,17166,19312,17232,15086,17166,17167,17166,15054,15021,17134,15021,15053,15021,12940,19084,25164,23018,27502,17002,16937,23178,29484,19050,50776,48663,42227,14759,25322,27465,33769,31689,23241,19051,16970,23244,10599,21164,10599,21196,16905,17002,16970,12744,25487,31761,50744,42259,42292,61308,59163,63421,63389,63389,63389,63356,63389,63389,63389,63389,63389,61309,61276,61277,54970,52824,52824,46485,40211,19050,27534,55003,55002,44404,54938,59229,42292,61309,52824,48598,29779,20937,39337,46323,50711,50711,57083,61309,48664,35986,36018,44405,42325,29712,46583,61309,59196,38035,21132,44340,37901,25256,12645,21096,27500,63454,65502,54937,46451,46452,57017,52824,48565,46420,35985,46420,48598,44372,40145,40081,42292,54970,50744,46485,52792,55003,54970,48631,40179,38033,35953,38034,38034,38066,38066,40146,40146,42259,44405,40211,31759,29582,40146,42259,40179,38066,35985,35953,33840,35921,35985,38033,25387,12774,12774,12741,14854,23274,35921,29615,25356,25357,21195,25421,25421,25454,27567,31826,44471,8617,43},
{34170,34170,34170,34170,32090,32122,32122,32090,32090,34138,32090,34138,32058,32058,32090,30042,30074,30074,30074,32122,34170,34202,34169,32057,32057,32057,30008,29976,32056,32056,32056,32056,32056,32056,32056,34104,32056,32056,32056,29976,29976,27895,25847,25815,27863,27895,27895,27895,27895,29943,29943,31991,31990,29942,29910,29910,31958,36183,46649,48729,38165,33972,31923,34069,40310,48697,31859,34037,25618,21425,15054,15022,15087,15087,13006,15086,17167,17134,17199,19247,19311,19279,21295,23343,25584,29745,40212,42261,38067,19115,16938,33677,37870,25324,50743,48631,40110,44235,50570,50570,42187,33776,40179,40115,38034,42228,31728,42227,33808,35953,33840,33872,35953,31760,44337,31662,50744,40211,54970,59195,48663,63389,63389,63389,63389,63357,61308,61309,61308,61309,61308,61276,61276,54970,46517,50711,44438,52857,40179,48631,35921,57116,42259,48630,55002,63422,59228,57115,46485,46453,29779,8616,10599,21358,23340,33873,61277,50744,44471,40245,46583,52857,57116,44438,44438,61309,59196,35922,20905,38002,44240,27402,23176,25289,27500,59228,57116,44438,40146,42258,50710,48597,48532,44339,38098,46452,50744,44404,42226,37968,44405,52857,40179,40179,57083,57116,61341,52857,42259,35953,35953,35953,38066,38066,38066,40146,40146,42291,46485,44437,40211,31694,42259,42259,40146,35985,33905,31792,33872,33873,38033,38065,31726,23242,19048,14822,14822,27468,29582,29582,23308,23276,21228,10696,21228,25454,27534,31793,48665,14891,35},
{34171,34170,34170,32090,32090,32090,32090,30010,30042,32090,34138,34170,32090,32089,32090,30042,30074,32122,32122,32122,34170,34170,32121,32057,32089,32057,32025,29976,32056,32056,32056,32056,32088,32056,34104,34104,32056,32056,32056,32056,29976,27896,27895,25815,27895,27895,27895,27895,27895,29943,31991,31991,29910,29910,29942,29942,31958,36183,44503,46616,38197,33939,31891,29811,40310,46584,29746,19247,21393,19280,19247,17134,19247,17166,17102,15021,17134,19214,21327,19149,23181,23149,21133,21100,27601,21327,14988,14955,17034,16970,19017,31629,33644,25389,46583,48630,54863,50569,54855,54823,40072,23082,31663,35759,19018,42163,14759,40082,23243,31663,35856,31727,27534,21228,42257,31694,50777,46517,61308,54970,57115,61277,38067,48632,48632,48664,52858,46519,57083,44470,55003,46518,54970,48598,48598,27534,42292,46550,59131,57082,42292,44437,40178,50711,55035,61341,57115,57050,40179,42291,35988,6504,21129,19049,21292,33873,52890,52857,46551,42357,44438,44470,63422,50777,44438,55003,57083,40181,31371,40048,33676,27370,23209,29547,31726,63422,61341,52824,48565,46452,48630,48597,48564,44339,36017,44372,46485,40178,44339,40113,42292,57115,50679,42227,57116,52825,54905,48630,40179,31727,35920,35953,38066,38066,38066,40146,40146,40178,35952,33872,27534,21163,33840,40179,38066,35920,33873,35953,35953,33840,35986,33873,38065,23275,23242,21129,21129,25388,31695,27502,19083,14889,21228,19115,19148,25486,27534,42325,59229,31826,68},
{34170,34170,34170,32090,32090,32090,32090,32090,32090,32090,32090,34170,32089,30009,32090,30042,30074,30074,32122,32122,34202,34202,32121,30009,32089,32057,29976,29976,32056,32056,32024,32056,32056,32056,32056,34104,32056,32056,32056,32056,29944,27895,27895,25848,25847,25847,25815,25847,27895,29943,31991,31991,29942,29943,29942,29910,29910,31957,55036,44536,38198,33971,33939,33939,42391,50810,31826,14989,19247,25586,21295,19214,19214,25456,27504,33972,38165,33939,29778,27601,21295,19214,17166,19214,25585,25520,19214,23407,19180,14922,19050,29516,37838,25356,40179,40146,52684,48488,54823,54823,40042,35791,35954,44275,33776,44340,35856,42228,40082,38034,46420,40211,31792,33905,46485,52825,40211,57050,61276,50744,63390,44438,25421,48632,23341,48632,29680,42358,33841,31760,38067,25389,48663,50743,25421,25453,52889,42292,57050,52857,44405,42259,42259,52791,52922,61341,59195,50711,38001,42259,38067,8617,29582,31662,25356,23308,48696,52857,46518,38132,23340,36018,40244,23341,21261,40244,46518,40245,42195,46419,31628,27402,29547,27467,38065,61341,57083,46453,46419,46419,50711,46517,48533,44371,38098,44371,48630,42259,42258,42258,40146,38066,44404,40114,46454,46421,48469,42195,38033,35953,33840,35953,38066,38066,38066,40146,42227,42259,29549,16937,10631,10631,19083,40179,38066,35953,35953,35985,35985,35953,35953,33937,23339,21161,16935,16903,16903,25387,35953,25422,6406,12776,21228,23373,19115,29680,29680,35986,33873,19148,33},
{34138,34138,34170,32122,32090,32090,32090,32090,32090,32090,32090,34170,32090,32089,32090,30010,30042,30074,32122,32122,34202,34170,32121,32089,32089,32056,29976,29976,32056,32056,32056,32056,32056,32056,32088,32056,32056,32056,32056,32056,29976,27896,27896,25847,27895,27895,27895,25815,27895,29943,31991,31991,29942,29910,29942,29942,29910,31958,34070,34070,34037,54971,46552,33939,50809,52922,50744,44373,40148,33744,33744,42163,44308,42228,42195,42293,40213,35987,33906,36019,31793,33873,33874,29647,33906,27567,27568,27567,21260,14921,19083,23243,35790,19115,25519,33807,54796,48487,54823,54791,33803,40017,29582,44307,25292,38034,29517,31695,40082,23210,42227,33808,19082,35985,46551,42292,48598,61309,52824,59196,59196,27566,44438,29648,31760,33906,29647,44438,21195,44406,19115,44437,50744,40179,19082,44437,54937,40211,40179,52857,44405,42324,42259,50743,38099,55002,48663,46550,35920,42259,31761,23309,35953,38131,25486,23341,48729,50777,44438,46518,27566,33905,55003,23375,12841,57116,42098,42195,46421,46452,33741,27402,27434,27402,40145,42323,44405,44404,46451,46452,57050,52856,48533,44339,40178,44339,48598,44339,42225,42226,38098,40179,42292,40146,42162,31597,33710,33775,31694,35953,35888,38034,38066,40114,40146,42259,42259,40179,29680,31728,6470,10696,10663,35954,38034,35953,35985,35953,35985,35953,33872,33872,19113,16935,12774,12742,14822,25387,33807,16938,8519,19082,23308,25421,25422,21294,35986,33873,44406,10632,61},
{32090,32090,34170,34170,32090,32090,32090,32090,32090,30009,32090,34170,32090,32058,32090,30009,30074,30074,32122,32122,34170,34202,32089,32057,32089,32057,29976,29976,32057,32056,32056,30008,32056,32056,32088,32088,32056,32056,32056,32024,29976,27896,27896,27896,25815,25847,25815,27863,27863,29943,31991,31991,29910,29910,29942,29910,29910,31958,31958,31958,29877,55003,46584,36019,46583,54970,48566,48567,50712,42326,42358,42357,42357,42357,40244,46550,50711,48598,50711,48598,50744,48631,48598,50744,48696,48663,48663,48631,31857,10761,10761,10729,19083,10697,10696,25354,52682,52678,54789,54790,23240,42162,21131,44340,31630,38067,44275,46518,48631,46550,42324,25422,25487,40211,57082,50743,57083,61309,54937,63389,46518,42324,50744,40180,50712,29615,48631,29647,42325,27501,35953,52824,50743,19114,38098,44405,54969,40179,42292,52857,40212,42357,40146,50710,42291,59261,46551,50777,40147,42259,46517,42259,44405,35921,33776,25389,46584,52857,46583,44470,40244,48663,23342,12810,8583,36052,27308,14857,38066,46420,35854,25289,25320,27368,25352,29576,29578,33806,46451,46452,50711,48630,48532,44371,40211,44339,46485,44339,42225,42225,42259,54938,40114,40114,39920,23049,29323,38001,31662,35953,35921,38066,40114,40146,40178,40147,35953,29679,27599,38099,14857,6405,8551,14889,38034,35953,35953,35985,38033,35953,27534,16968,12742,12742,12741,12741,14854,27468,31695,12744,4358,17002,23341,25453,29680,8616,29680,33873,29647,12776,96},
{32090,34170,34170,34170,32090,32090,30042,32122,34170,30009,32090,32090,32090,32058,32090,30010,30074,30042,30042,32122,34170,34169,32089,32057,32057,32057,29976,29976,32056,30008,29976,32056,32056,32024,32056,32056,32056,32056,32056,32024,29976,27895,27895,27863,25815,25815,25815,27863,27863,29943,31991,29943,29910,29910,29942,29910,29910,29910,29910,31958,29877,55036,46584,38132,38197,55003,55003,52890,52890,46584,44470,42390,42390,40309,42357,50711,48663,46550,48630,48630,46517,48597,40179,21261,17068,17067,17067,23439,17067,12809,10761,10761,14922,14890,16970,31660,50568,54789,54789,54823,21063,40017,25227,38033,31598,31663,42130,27631,31825,33938,29680,8550,19050,40145,44304,48631,48631,57050,61276,63356,63389,63389,63389,63389,63357,63389,63389,61309,61276,59195,57083,54969,54970,48598,42324,44404,52889,40212,48663,59163,29614,42292,42258,44338,46517,52857,48631,50744,44405,44372,44339,48598,46517,40180,29647,21228,31825,52890,44470,33938,29680,17035,12842,10761,10697,8584,12777,17198,31728,42259,33773,25288,23207,27432,25318,29544,27497,40113,46451,46452,50678,48597,48532,44339,35953,44339,50711,42226,42193,42225,42259,61309,44275,48598,46356,27308,25324,42260,31630,38001,35921,38034,40146,42226,42227,29615,23341,31727,12776,10728,14954,4357,8551,8584,35953,35953,38033,38033,38033,35986,10631,16967,14822,12774,12774,12741,12741,25387,21195,8552,4358,17002,27534,25421,29647,12810,23342,29647,25389,21228,126},
{32090,34170,34170,32122,32090,30042,30009,32090,34170,32090,30009,32090,32090,32089,30009,30042,27994,27994,30074,32122,34170,34169,32057,32057,32057,32056,29976,29976,32056,30008,30008,32056,32056,32024,32056,32056,32056,32056,32056,29976,29976,27895,27895,25815,25815,25847,25815,27863,27863,29943,29943,29943,29911,29910,29910,29910,29910,29910,29910,31957,29909,31957,31957,31957,31957,34005,36118,36118,38230,46551,46551,42292,48631,42358,36051,38164,50744,46551,36085,36084,38132,48664,33937,52890,36019,36052,38197,36052,14988,10729,8616,8649,10697,8616,27371,42159,50606,52678,48423,46343,23243,35953,35824,38002,44243,33809,46421,38002,40147,38034,33873,27535,33808,42224,44239,44339,42324,54970,61308,61308,61308,61308,61308,61308,61276,61308,61308,61308,61308,52824,57083,44372,57083,46550,44372,44372,54970,31727,48664,48631,35921,40178,40145,40113,40146,55003,48631,50777,42292,44339,44371,50710,44437,38131,25422,23341,27599,50777,33938,31793,12777,10664,10697,8616,6471,6438,6470,12907,29680,40146,33741,27401,27432,27432,27400,29513,21095,25322,46451,46451,48630,46452,48532,42259,38034,42258,46517,42258,42225,42225,40113,52857,48533,50711,40245,34004,23407,19147,16905,35921,33873,33873,40146,40179,44273,33577,16839,19082,8551,8551,6470,6438,6503,8551,29647,38033,38033,35985,35953,31760,38131,19081,16935,14887,12774,12741,16935,27468,19083,6472,4358,16969,27599,25421,27567,27535,23341,31760,31760,23308,107},
{32090,34170,34202,32122,32090,30042,30009,34170,34170,32090,30009,30009,32090,30041,30009,30042,30042,30042,30074,30074,34170,34170,32057,29976,32057,30008,29976,29976,30008,30008,30008,29976,29976,29976,32056,32056,32024,32056,32056,29976,27896,27895,25847,25847,25815,25815,25815,27863,27863,29943,29943,29911,29910,29910,29910,29910,29910,29910,29910,29909,29909,29877,29877,29877,29877,29877,29877,29845,29811,50744,48631,44373,44373,33939,25487,25519,44470,50744,33874,31793,25453,44438,38164,27665,17166,17166,19279,19214,8649,6503,6536,8616,6535,6503,6470,38001,50642,50598,46312,33736,14791,29550,31597,25324,42130,25291,42163,31565,29582,35888,16970,14857,29582,44304,46354,46420,44437,61309,54938,35953,54938,46486,52890,42325,54938,44405,52890,44405,52857,52856,57083,57083,63389,52824,42324,42324,54938,33873,48632,44438,33839,35887,42193,40146,46517,61341,44438,50809,42292,42226,42258,40145,44405,40277,33906,23309,21228,31793,25487,8584,6503,6503,8583,6438,4325,4358,8551,6503,23341,40179,35855,29548,31659,27433,23176,29515,31660,33741,46451,46452,57082,52856,46484,42259,33840,42258,44404,42258,42225,44306,35887,31727,46484,48630,33873,14922,27632,38100,10598,25421,33873,23276,29647,42227,48365,52358,16772,10631,6405,6438,4358,2245,4390,6471,23340,38033,33840,35953,38066,46583,25485,12742,12774,12806,12806,12806,21161,29613,23212,12746,12778,25421,27534,25454,27567,29648,23309,31728,42293,31761,143},
{32090,34170,34203,32122,32090,32090,30009,32090,34170,32090,30009,30009,30042,30042,30010,30010,30042,30042,30042,30042,34170,34169,32056,29976,32056,29976,29976,27928,30008,30008,30008,29976,29976,29976,32056,32056,32024,32024,30008,29976,27896,27895,25847,25815,25815,27863,27863,27863,27863,29943,29943,29943,29910,29942,29910,29910,29910,29909,29910,29910,29877,29877,29877,29877,29877,29877,29845,27796,34004,40212,33874,40179,40147,42325,29680,29647,46550,59196,59228,59196,57083,40244,42358,8616,4456,4423,2343,2343,6504,4391,6536,8584,8616,6535,6503,40074,46376,52678,52681,40074,33743,35953,44340,31728,42259,37937,44308,44275,38002,44340,31761,29647,35920,42259,50582,44372,57050,59196,29648,38067,33841,44438,35986,38067,38067,31760,38099,27566,48663,61309,50744,59228,63422,52857,42324,42324,52889,33905,48664,27567,42258,44306,42193,40113,46518,55035,46551,52922,42357,44339,42225,48597,44405,25551,25486,14922,12809,25454,14955,6503,4390,6470,6503,8551,8584,10697,4358,8584,12842,25487,40146,29548,29580,27434,25288,23209,23274,33871,46452,46452,50710,48598,48533,44306,33840,42258,44372,44306,42193,42225,38065,52857,59163,54937,31825,38132,19148,48697,10663,19082,33841,29614,29614,40146,42029,20964,6404,8583,2244,4358,4390,2245,4390,6503,10728,35953,25388,29614,42325,46584,29646,19113,10694,12774,12774,14854,21129,31694,38066,25422,23276,29614,14857,12808,27567,27535,23309,25421,40245,31793,144},
{32090,34170,34170,32090,32090,30042,30009,32090,34170,32090,30009,30009,32090,30042,30009,30009,30042,27994,30042,32089,34169,34169,32056,29976,32056,32056,29976,27928,29976,29976,30008,29976,29976,29976,32056,32056,32024,29976,32056,29976,27896,25847,25847,25815,27895,27863,27863,27863,27895,29943,29943,29910,29910,27862,29910,29910,29910,29910,29877,29909,29909,29909,29909,29877,27829,29877,29844,29844,29812,40245,44405,44373,44341,21262,21262,23407,25455,31761,31825,27632,29745,31826,25487,25455,27535,27535,25422,23309,21261,6471,4423,6535,6536,6503,10663,44298,44263,50599,50600,33769,29549,19115,37936,18953,38001,31663,33807,42227,23243,42227,21163,23308,29614,42160,46257,44405,59229,40180,23341,42325,31728,48631,35986,50744,31792,44438,21227,42357,57083,63422,48664,63390,61276,52857,42324,42324,52857,16937,23309,40244,42226,42258,40113,42194,38066,50809,46551,48729,44470,42258,42193,50710,42292,23373,17034,23568,25487,17034,6503,6471,6470,6438,4390,2277,8617,8584,6471,6536,12875,21327,44340,29548,29515,27402,27400,25288,25387,33839,40113,44306,50743,48598,46484,44306,46486,44339,48565,44339,42226,42258,35919,38067,52790,54904,46486,48631,42325,40179,27600,29680,52857,48631,29614,44372,37934,12709,4356,2244,2277,10762,4390,2245,4390,6471,6470,23276,23276,23308,35986,27534,25420,21226,16968,19113,21194,21193,25419,35985,38066,33841,31760,27567,27534,16970,27632,27567,31761,27534,40245,33841,142},
{32090,34170,34170,34170,32090,32090,30009,32090,34170,32090,32090,32058,32058,30042,30009,27961,27993,27994,30042,32089,34169,34169,29976,29976,32056,32056,27928,27928,29976,29976,29976,29976,29976,29976,32056,32056,32056,32056,32056,29976,27896,25848,25815,25815,25847,27863,27863,27863,27863,29943,29943,29910,29910,27862,27862,27862,29910,29910,29909,29909,29877,29909,29877,29877,27796,29877,29876,29844,27764,33971,40212,40148,38100,6504,6536,8681,23407,14923,8649,8648,8616,6568,6536,6568,8617,8649,10762,12842,12810,6536,6535,6568,6535,10696,16970,50570,46376,50572,50573,33805,35953,31695,38034,29615,40147,40114,35920,42260,31662,44372,46550,48598,40145,39980,48500,50744,52922,33906,42325,23341,44406,38131,46551,38066,44405,27534,31792,38130,63422,61341,61341,59196,59196,48631,42324,44437,52889,35952,19115,46551,42258,48532,42193,38032,46485,61309,46584,42293,19082,42226,42225,50710,42260,31825,27599,23373,17068,10696,6503,6503,4390,4357,4390,4390,2277,6439,6472,8617,17134,31826,42226,29516,27467,29482,29480,27432,27467,38065,48533,44339,50711,46485,46484,44339,44372,42258,46452,42258,42225,42258,38033,52922,54969,48631,44405,55002,54971,50744,55003,46518,50777,46518,31727,42292,14823,4356,2211,2211,8616,12908,4358,2244,4358,4390,4357,12744,16969,21195,19114,21226,23306,16935,16935,16935,19016,19048,23275,31759,27469,25389,25421,27567,16970,19115,25421,19082,21196,25389,38099,31760,133},
{32090,32090,34170,34170,32090,32090,32090,30042,32090,32090,32090,32090,32057,32058,30009,30009,27993,27994,30042,32121,34169,32121,29976,32056,32056,29976,29976,29976,30009,29976,29976,29976,32056,29976,32056,32056,34104,32056,32056,29976,27896,27895,25847,25815,25815,25815,25815,27863,27863,27895,29910,27862,27862,27862,29910,29910,29910,29910,29909,29910,29909,29909,29877,29877,27797,29877,27796,27796,34037,44373,42260,33841,29713,6569,8649,10729,23375,14955,8649,8617,8616,8616,6535,6536,4455,6503,8616,8616,4455,6536,6536,8648,6536,4391,6471,23337,16935,19047,19080,10696,6471,6471,4423,6536,10729,12809,12809,21261,23374,19148,19181,35822,44174,44174,37935,29647,38260,42518,44404,29615,23308,35952,35952,55002,52824,42324,27565,46583,61374,61374,63422,57083,57050,44437,55002,46550,54937,48597,29647,44438,50678,38033,40113,40113,46518,57116,44471,27534,10631,42226,42225,52856,46550,12841,33938,25454,14923,8584,6471,4422,2309,2309,4390,4358,4391,6471,6504,8682,15054,21294,37935,27370,25321,29514,29513,23239,27467,27467,46451,46484,50743,48598,46484,44307,38034,42259,46518,44339,42225,44306,33839,57148,61309,44437,44405,52857,44341,57083,59229,40180,50809,48566,38033,42292,25355,8484,4323,130,8583,8616,4325,2212,2276,4389,4357,4325,12776,12776,19114,14887,16967,19048,19048,16967,21128,23274,29614,33808,25388,25389,25421,25421,23341,25421,29680,27567,31760,27534,38099,27535,118},
{32090,34170,34170,34170,34170,34138,32090,30010,30009,32089,32090,32058,30009,30009,30009,30009,27962,30042,30041,32090,34169,34137,29976,32024,32056,29976,29976,29976,29977,29976,29976,30008,30008,29976,32056,32056,34104,32056,32056,29976,27896,27896,25815,25815,25815,25847,25815,27863,27863,27862,29943,27862,27862,27862,29910,29910,29910,29910,29909,29909,29877,29877,29877,29877,27797,27796,27796,27796,31892,44439,44470,44405,36051,6569,8617,12810,14955,8649,6504,6536,6568,8649,8649,10729,8616,8616,8617,8617,6503,6536,4488,6568,4455,4391,4391,6503,8583,6503,6470,4422,4390,4390,4391,4390,8616,8584,8584,14922,23309,8584,6471,14824,37934,44143,42226,38066,34099,38292,40178,27535,27566,40146,40146,54969,42291,38065,21226,55035,63422,63421,63422,57083,52889,50744,61309,44437,55034,46518,33938,46518,44405,42226,40113,42259,38098,48664,44438,27502,10598,42258,42258,54937,48663,44405,48631,21229,12842,10762,10729,6471,4423,4423,4358,6439,6471,6503,23276,31793,40245,40180,37966,29514,31628,31659,29577,23238,19046,27434,46419,46452,50743,48597,46452,42258,35953,44307,50679,44339,31726,42226,29581,29647,52857,21195,21162,33872,31533,44373,59196,35953,25389,19049,38065,44339,27467,8516,4357,2275,2243,4390,4357,4325,4357,4357,4357,4325,4325,16969,23339,16967,16935,12742,14790,16967,16967,23273,33775,19050,23276,25421,25421,27502,25389,27534,29647,31728,33906,31695,42357,27567,115},
{32090,34170,34170,34170,34170,34138,32090,30009,30009,30009,30010,30041,30009,30009,30009,27961,27962,30042,32089,32090,34169,34169,29976,29976,32057,29976,29976,29976,30008,29976,29976,30008,29976,29976,32056,32056,32056,32024,29976,29976,27928,27895,25815,25815,25815,25815,25815,25815,27863,27862,29943,29910,29910,29910,29910,29910,29910,29909,29877,29909,29877,29877,29877,29877,27829,27796,27796,27796,25651,23538,27634,42292,40212,6536,6536,12842,4423,4455,4456,6504,6536,6536,4455,4455,4423,4423,4423,2375,4423,4423,2375,4423,4455,6503,4390,6471,6503,6503,6503,6471,4423,6503,6471,4422,14922,14890,12809,17003,17003,12777,14954,16969,37835,44111,31727,38034,36277,42518,33840,21228,31760,38001,44437,54969,46517,35952,25452,63422,63422,63389,57115,52856,42324,61309,59228,46518,57083,48697,38164,42325,50679,42258,40113,35953,48566,59261,40245,23308,19049,42226,38065,46550,46583,50744,46550,42325,25487,17035,8649,6471,6503,4390,6471,6471,6503,12810,48729,57148,59261,61309,44177,33709,29582,33872,29543,27399,23238,29579,46451,46419,50678,48565,46452,42259,38034,42259,48598,35985,19083,25421,33807,40212,44372,50711,48565,38001,25227,42325,44470,33840,29581,29581,25355,35887,23240,14854,8549,12774,14791,10597,10630,12711,12743,12743,12711,12711,14791,16936,23240,21159,19015,14789,12741,14789,14822,21096,23275,25389,27502,27501,27502,27534,25454,29679,27535,23341,29647,33841,44438,27534,118},
{34170,34202,34170,34170,34170,32090,34170,32090,30009,30009,30009,30041,30009,30009,30009,30009,30010,30010,30042,32089,34169,34137,29976,29976,32057,32056,29976,29976,29976,29976,29976,30008,29976,29976,32056,32056,29976,29976,29976,29976,27896,25847,25815,25815,25815,25815,25815,27863,27863,27862,29942,27862,29910,29910,29910,29910,29910,29910,29910,29909,29877,29877,29877,27829,29877,31989,36150,29876,27763,27731,31859,40245,38164,14988,4488,15020,4423,4455,4455,6504,6536,6535,4455,4423,4423,4423,4423,4423,4455,4423,4423,4423,6503,6503,4423,4422,6470,6503,6471,6471,4422,6536,6535,6503,21229,23341,23309,21228,21228,10664,6470,10696,35820,46288,40113,35920,38196,48630,27534,21261,35986,35920,46485,52856,42291,35984,25452,61374,61373,61309,52824,44405,54970,59196,57115,42324,38131,25486,31858,40179,44339,44372,35920,35888,48598,59197,42293,27469,40179,35921,38164,42423,33939,46519,42324,44405,50777,40212,12809,8616,4423,6504,6503,6504,6503,8616,23341,52955,59229,48761,25292,29485,31793,44470,35916,25349,23205,31660,46451,46419,50678,48565,46484,44339,38066,42259,48631,25389,21195,25421,42259,44470,54936,59130,50678,31727,12744,29745,10663,23339,31757,35951,29548,27467,21128,12773,8548,16935,8516,4324,2244,2244,4324,8550,6372,4292,4324,10597,21160,21127,19047,23240,27466,29514,29482,35886,31662,25356,25421,29647,27534,27534,29647,27567,27535,19083,27501,42292,35986,27534,116},
{34170,34202,34170,34170,34138,32090,34170,32090,32057,29977,30009,32090,30009,30009,30009,30009,30010,30042,32090,32089,34169,32089,29976,32056,32057,29976,29976,29976,29976,29976,29976,30008,29976,29976,29976,32024,29976,29976,29976,29976,27896,25847,25815,25815,25815,25815,25847,25815,27863,27863,29943,27862,29942,27862,29910,29910,29910,29910,29910,29910,29910,29909,29877,29877,27796,31957,40408,25585,25649,23536,27697,25617,19246,17068,4488,12875,2375,2375,4423,4423,4455,6503,4423,4423,4456,4456,4456,4455,6504,4423,4423,6536,6535,6535,6503,6503,6503,6471,6503,6503,4423,8616,6536,6535,21196,21228,23276,21196,21196,12777,12809,14890,29484,50513,40013,23308,36148,46517,23309,21261,40179,31726,54937,54969,40178,23339,33937,48696,59261,48663,48598,27469,38067,46518,42324,42292,40179,21228,27632,38099,46452,48565,42193,44274,48566,52857,44406,27502,25454,23441,21394,27634,46551,38262,36150,36117,36117,34004,21326,17134,21327,25454,23243,19050,19083,19050,33840,46616,59229,29811,12744,21034,42260,38035,31627,23205,23270,33871,46419,46451,48598,46452,46484,42259,35985,42226,46517,29615,23276,27501,35953,31727,46484,44437,31728,35953,12776,8616,14920,21225,25386,19048,12709,21194,8548,10693,6435,8516,4324,2179,2179,2211,6436,10761,10728,10662,2211,12742,19047,19079,21160,21128,23241,23241,27467,33839,35920,27469,23308,33841,27502,29680,29679,27534,40180,16969,25421,31793,31760,27534,116},
{34170,34202,34170,34170,32090,32090,34170,34170,32057,30009,30009,32090,32057,30009,27961,30009,30042,30042,30042,32089,34169,32089,32056,32056,32056,29976,27928,29976,29976,29976,29976,29976,30008,32056,32056,32056,32056,29976,29976,27928,27896,25847,27896,25815,25847,25815,25815,25815,27863,27895,29943,27894,27894,27862,29942,29910,29910,29910,29910,29910,29910,29877,29877,29877,31957,29811,40375,19311,21424,19246,23505,19343,12940,4488,4488,19214,4488,4456,4488,4456,4455,4455,4455,4423,4456,4455,4455,4455,4455,4423,4423,8616,6503,6503,4423,6503,6503,6503,6503,6535,4423,10762,6536,8583,42227,48501,46421,46421,46421,17002,21261,25389,23211,46354,31628,21227,34099,42291,21261,25422,40146,35920,54969,50710,27532,19145,36017,33971,48696,35986,46517,14857,16970,21227,42357,42324,52889,38099,33938,35986,46452,44338,42192,44273,48598,59197,42293,33841,25585,27732,25554,38067,54970,36117,34135,36216,36248,36183,42457,36215,36150,46451,52658,48498,48631,50744,46451,46583,33841,14760,10566,18953,33646,35790,31560,27398,21126,33872,46451,48564,52824,50710,48532,42226,38066,44307,48598,29615,21195,25389,38033,44405,27533,21228,10728,12775,12809,12777,23372,14888,14854,8548,10597,10629,8549,6436,8516,8549,2211,2179,2179,17032,14854,12775,10662,12742,2211,16967,19079,16999,14822,10662,16935,16967,31694,31694,35921,27534,23308,42325,27567,31793,29680,31793,27566,27502,25421,29615,33841,27502,117},
{34170,34170,34170,34170,32090,32090,34170,34170,32090,30009,30009,32090,32090,30009,27961,30009,27993,30042,30041,32089,34169,32089,29976,29976,32056,29976,27896,29976,29976,30008,29976,29976,32024,32056,32056,32056,32024,32024,29976,27928,27895,25847,25815,25815,25815,25847,25815,25815,27863,27862,29942,29910,29943,29942,29942,29910,31958,31958,29910,29910,29909,29877,29877,29877,46585,31891,38230,21359,25617,25585,27698,25617,23505,4520,4521,19214,4456,4488,4456,6536,6536,6568,6536,6536,6568,6536,4488,4488,6504,6568,8616,8648,6536,4455,4423,4423,4423,4423,4423,4423,4423,6568,10696,14857,48598,31727,44373,42260,35921,19116,17003,16970,21163,44239,31629,36146,38390,38098,21228,29679,35952,42259,52856,42291,33839,19113,38131,42325,36019,23406,42291,12711,10631,35986,42324,42292,54970,44437,33938,36018,42291,38032,42225,44305,48566,57083,50744,54970,52825,27567,33906,54938,48598,38133,34103,36248,36248,34070,40408,36215,36150,46451,54770,48498,44503,48696,46419,38066,42325,46518,46486,33743,33678,35889,29513,23237,19045,29645,46451,46451,48565,48532,48532,42226,38098,44307,50679,25421,21228,25453,42260,57115,52857,29648,21228,17035,10664,8584,14855,14887,12742,6403,19048,10661,10629,10661,12806,10693,4356,4291,2210,12773,12741,14919,10629,6437,12742,16999,19048,14888,10597,19114,23274,25387,31727,29614,31695,29614,27534,40212,31792,31760,29680,29680,27502,27502,25421,27534,29615,27502,109},
{34170,34170,34170,34170,34138,32090,34138,34170,34170,32057,30009,32057,32090,30009,30009,27961,27993,30041,32089,34137,34169,32089,29976,29976,32056,29976,27896,29976,29976,29976,29976,32024,32056,32056,32056,32056,32024,32024,29976,27896,27895,27895,25815,25815,25847,25815,25815,25815,25847,27895,29943,29910,29942,40311,38263,34102,36183,34038,31957,34038,31957,31957,31957,29811,50778,38165,44569,25617,29778,25552,31891,29810,23440,4521,4488,21327,4456,4456,6536,4488,4488,6569,6536,6568,6568,4487,4487,4456,6536,6536,8616,8616,8616,6504,6504,4455,6503,8648,6536,6503,6503,6536,29615,46420,48565,48566,50646,50646,46485,35986,40179,33873,25388,29481,35852,46581,46550,33906,19148,33873,33840,46517,57050,44436,29613,21161,38130,46550,31825,25487,42324,19082,29615,35953,42324,40211,54970,48663,36018,36018,46419,44306,42193,42225,48630,57083,50809,55035,44405,42324,46518,52824,44340,38133,34135,36248,36248,36150,42489,38328,36182,44403,54803,52692,44503,48696,40112,46550,48663,48664,48664,48632,31793,50809,29546,21125,19078,29613,46419,46419,46452,46484,48532,40113,33807,42258,48598,29615,21196,25421,35985,44470,50809,29615,10664,10664,8583,8616,14888,12774,12741,8548,14886,14854,16999,10661,14886,6468,8549,4291,6436,17000,27532,21194,8516,6372,12774,21161,21128,21161,19048,16968,19015,33839,31694,19082,27469,29647,33873,33873,33873,31760,29647,25421,29582,29614,27534,29614,29614,29582,110},
{32090,34170,34170,34170,32090,32090,32090,34170,34170,32090,30009,30009,32090,30009,27961,27961,27993,30009,32089,34137,36217,32057,29976,30008,30008,29976,27896,27928,29976,29976,32056,32056,32056,32056,32056,32056,32024,29976,29976,27895,27895,27863,27863,27895,25815,25847,25847,25815,25815,27895,29943,29910,29910,48664,40342,38230,46649,21358,27633,31859,25487,31859,29713,31858,36019,27665,36116,31826,29713,40245,27665,29745,23439,4489,4488,23406,4456,4456,6569,6536,6568,6601,4488,6536,6536,4487,4487,4455,6536,6536,8616,8649,8649,8616,6536,6536,6536,8648,6536,6503,8617,8584,33808,46485,48566,42227,40114,21261,31825,40212,21196,19148,23211,29614,38031,40405,44502,27567,21196,38098,29614,52857,57050,40146,12743,35920,42291,48663,29681,21326,33938,52922,63422,50744,40244,42324,55002,44405,25421,31727,48532,44306,46484,57050,59228,33874,10729,17002,40179,35986,33873,29582,16971,36118,36216,36248,36248,34102,38264,38328,38263,46484,54803,52692,46649,50810,38000,48630,44437,46616,46617,46617,36051,31760,23238,19012,16965,31693,46451,46451,46484,46484,46452,40113,31694,44306,50711,25389,19115,25421,33840,35986,54970,31695,25388,21195,8551,8551,12807,14854,12741,16967,12741,8548,14854,6435,14886,10629,10661,16967,4291,8549,10629,17033,23274,14856,19080,21192,23273,21128,23274,27467,31725,38032,31662,27502,10630,25421,29680,33841,33873,29680,27502,25421,27469,27501,25421,27501,27501,35985,152},
{34170,34170,34170,34170,34170,32058,32057,34170,34170,32089,30009,30009,30009,30009,27929,27961,27961,30009,32089,34137,36217,32057,29976,29976,30008,29976,27896,27928,29976,30008,32056,32056,32056,32056,32056,32056,32024,29976,29976,27895,27895,27895,27863,25815,25815,27863,25815,25847,27895,27895,29942,29910,29910,48696,38262,38262,48794,27665,29746,31859,27665,38164,27632,25552,42358,31826,33939,33939,27632,36019,31826,36052,29745,23374,29745,21294,4456,4488,4488,4488,6536,4488,4488,4488,6568,4488,6536,6536,6536,6536,8649,8649,8616,8616,8649,8616,6568,8616,6536,6536,6536,8616,35953,44340,48598,42259,31793,10762,29745,40212,25356,29647,37969,37966,31661,40438,44469,23308,23373,38098,31759,54970,52856,25421,10630,19082,42325,50711,25487,19214,19213,44471,48696,44405,38131,44437,52857,35953,23340,31694,42225,42193,42258,44405,12809,6503,10697,25389,46484,34003,27728,23212,14923,34135,36216,12908,36216,36248,38329,29877,31957,40146,40080,38066,44471,44503,35920,48598,44405,42390,44471,42391,23276,19082,25350,23237,19015,38065,46451,46484,48532,46484,46484,40145,38033,42258,48631,29615,21196,27502,40178,44405,48630,29581,21129,19080,10629,19080,19047,16999,14886,14886,16966,8516,6468,10628,14821,14886,12741,8516,2178,10661,8516,10629,14854,8516,16967,21160,21128,19047,21128,19016,38032,27533,12743,19050,25453,29647,23308,33841,31793,27566,25421,25421,25421,25421,25389,25453,25421,38099,108},
{32090,34170,34170,34170,34170,32058,32057,34138,34170,32090,30009,30009,30009,30009,30009,27961,27961,30009,32089,34137,34169,32057,29976,30008,32056,29976,27928,27928,29976,30008,32024,32088,32056,32056,32056,32056,29976,29976,27928,27895,27895,27895,25847,25815,25815,27895,27863,27863,27895,27895,29943,29943,29910,48696,38230,42423,50875,25585,29713,29746,25552,34004,23439,25552,38197,29745,33939,33971,25552,31859,25552,36019,27632,25519,36051,19181,2375,2408,2375,4456,4456,4456,4455,4455,4455,4455,4455,4455,4456,4456,6568,6536,6568,6536,6536,6536,6536,6568,6504,6503,6536,6536,25487,31760,31792,42259,33906,12842,33939,42292,31728,38163,31792,33738,27694,38454,42357,25422,29647,35953,38066,52856,44404,31727,38066,35953,38098,46550,25487,17068,12907,42390,48696,40211,35986,48631,57083,57050,59163,50710,40145,48630,52824,31793,12842,8583,23340,42258,42226,38034,31696,21163,14923,32054,36216,4424,36183,38329,25619,12941,10861,19083,21162,23341,29778,31858,17034,42324,46550,50809,50745,40147,23243,16936,25351,25350,23272,35919,46451,46451,48532,46484,46452,40113,35985,42290,50744,31695,21195,27501,44372,50711,44339,29581,16969,12743,23305,19111,14886,10693,12741,14854,10661,16966,8580,8548,10661,10628,10661,10693,6403,10628,14886,16967,10660,14854,21160,21160,23241,21128,16935,21062,21162,14889,8518,12744,14856,31728,25389,29615,31760,25421,25389,25421,27534,25421,25421,29647,35986,29679,103},
{34138,34170,34170,34170,32090,32090,32057,32089,32122,32090,30042,30009,30009,30009,30009,27961,30009,30041,32089,34169,36217,32056,29976,30008,32056,29976,27928,29976,29976,29976,29976,32056,29976,32056,32056,32056,32024,29976,29976,27895,27895,25847,25847,27863,27895,27895,27895,27895,27895,27895,29943,29943,29910,48729,38197,29843,34036,23439,40278,29778,25584,38197,23504,27697,34003,31890,33939,31858,36052,31891,27633,38164,29745,27632,40278,17100,4488,4456,4488,4488,4488,6568,4456,4456,6568,6568,6536,4456,4456,4455,6568,6536,6503,6536,6504,6568,4455,6503,6503,4423,6503,6503,19148,21196,23276,38033,25454,12842,36051,42292,12712,29679,27599,48597,38228,44566,46583,25486,35953,38098,48663,50744,48631,35920,38033,27501,35986,46518,27568,12843,14988,36117,57083,42324,44405,57115,59163,33873,52825,50743,46517,42323,29680,10696,8616,21228,35920,44339,46452,33712,29615,19050,12875,32022,36183,4423,36216,25651,10893,10861,10828,14922,21162,21228,25520,29713,12777,27632,55003,57115,55035,42194,31598,33808,19046,19012,16901,33806,46451,46419,48532,46452,46452,40113,40146,40145,46550,27534,21196,25421,42259,44405,46485,44339,33807,16969,19048,21159,16999,10661,8484,10629,8548,19047,12773,8548,6467,10629,8548,12806,8516,6468,8549,6435,12741,14886,21160,21160,19047,21128,16903,31692,16970,6437,21196,16937,27534,29615,27470,33808,29615,25421,23308,27502,35986,25421,23341,27534,29647,25421,103},
{32090,34170,34170,34170,32122,32090,32057,30009,32090,32090,30041,30009,30009,30009,30009,27961,30009,30041,32089,34169,36217,32056,29976,29976,30008,29976,27928,29976,29976,29976,29976,30008,29976,32056,32056,34104,32056,29975,29976,29944,27895,27895,27895,27895,27863,27895,25847,25847,27895,27895,29943,29943,29910,50777,36117,34004,34004,25552,38132,29778,27665,33971,23471,33938,29777,29745,38132,29745,31858,33971,25551,36052,27632,27599,38164,19213,4456,4488,6536,6536,4488,6568,4488,6536,6569,6568,6536,6536,4456,6536,8649,6568,4488,8616,8617,10761,6568,8616,6536,6536,6536,8648,31728,33807,33808,44340,19083,12809,38099,44371,10664,31825,21293,27631,38293,44502,29679,25421,38066,35953,48630,59163,59163,57017,50646,35985,29680,46583,17036,14988,14956,31858,57147,48598,50743,50711,44437,38033,52856,52824,35952,21195,8551,8583,8584,27534,38065,35921,40146,42228,38231,38231,36183,34103,34135,27666,29812,10828,10828,10828,8747,14889,21130,21196,14955,14955,10696,23406,44470,44470,48664,42292,27501,36017,23238,21125,19013,29613,46419,46419,48532,46452,46452,40113,40178,44339,46518,27534,19115,25388,44372,48598,44372,48598,23242,10630,8548,16967,12773,8548,16967,14822,12774,8548,12741,14886,8548,12774,4290,10661,8548,6403,8548,8516,10629,19079,14854,21192,25353,25354,33807,29615,12809,8583,23341,25389,31728,33808,29583,40212,27534,25389,21228,25421,27502,25421,19083,19083,25421,19115,102},
{32090,34170,34170,34170,32090,32090,32090,29977,32057,32090,30041,30009,30009,30041,30009,27961,30009,30041,32089,34169,36217,32056,29976,29976,32056,29976,29976,29976,29976,30008,30008,30008,30008,32056,32056,34136,34104,32024,29976,27896,27895,27895,25847,25815,27895,27895,25847,27895,27895,29943,29943,29943,31958,50777,42358,38197,36052,33906,38132,29745,23471,31858,25519,36052,27632,23439,33939,23374,29713,33938,27600,33971,31826,33906,31826,29680,27600,23406,25455,29680,21293,23374,21293,19213,19181,19148,23341,14922,14987,10761,10794,6568,4488,6568,6568,10729,8616,8616,8584,6536,6536,14922,48566,44339,46452,46485,16937,12776,44437,42290,16970,35986,17034,17066,32051,46518,23341,31792,42292,29646,52824,54970,48664,52889,40179,42324,25454,40245,19214,14956,12907,21326,44470,54969,55002,46518,25388,57083,59196,40178,31760,12777,12842,10729,8616,27566,33808,52890,52857,46551,50874,46584,48729,38295,34102,31990,12908,10796,8747,10796,10763,10664,12711,12776,12842,12842,8616,17068,38132,38197,46649,48664,31695,38098,25351,21158,21126,31693,46419,46452,48565,48564,46452,40113,40146,44339,50712,23309,21163,25421,27533,16968,31726,27501,8517,14822,14886,12741,14854,6435,10693,14821,14887,6468,6403,8548,14854,6468,4323,4322,14919,14886,12774,8548,2210,14854,14790,25322,23209,35886,33775,19082,6470,8550,14888,27502,29615,12743,33873,40212,27502,23341,23308,25389,25421,25421,19050,10631,21228,19050,96},
{32122,34170,34170,34170,32090,34170,32090,32057,30009,32090,30041,30009,30009,30009,30009,30009,30009,30041,32089,34169,36217,32056,29976,30008,30008,29976,29976,29976,29976,30008,30008,30008,30008,32056,34104,34136,34136,32056,29976,27895,27895,27895,27895,27895,27863,27895,27895,27895,29943,29943,29943,29943,29942,38296,44569,40343,42456,42423,44472,38230,21392,19344,21424,21360,19279,19279,19279,19214,21327,23439,23374,25519,19213,21326,23407,21326,27600,23341,23374,25487,21261,27600,19245,23406,27568,21293,21261,19115,31792,12873,14986,8713,6600,8713,10761,19115,14889,21163,23276,25421,21293,17035,29647,46485,46485,48598,48598,48598,40211,33906,21260,19114,12840,17163,34196,44372,21228,27567,40147,35921,50744,52856,36018,40179,38098,40212,33906,40245,23440,12907,12843,12842,33873,33872,40211,40244,46518,50744,35920,35985,31727,12777,17068,8648,6503,27534,40211,55002,48631,36119,40376,52922,52922,38263,31956,10827,8715,8715,8682,8683,10730,8550,8550,10630,8583,8583,8584,12842,19181,21294,46649,46616,35954,50744,31694,23208,16901,29613,46451,48532,50678,48565,48500,40113,42259,44339,50711,27534,19114,19081,21161,17000,17000,14888,12742,12773,16966,16998,19079,14854,14854,10661,8548,8516,14918,6435,19112,12740,12773,14854,4290,6403,12742,12710,6371,12741,21128,19015,29580,35887,19082,31793,10663,12712,21195,33808,25389,19050,35954,29615,25421,23309,23308,23341,25421,19082,29518,8453,23308,19082,98},
{34170,34170,34170,34170,32090,32090,32089,32089,32057,32057,32089,32089,30009,30009,30009,30009,30009,30041,32089,34169,36217,32056,29976,29976,30008,29976,29976,27928,29976,30008,30008,29976,29976,29976,32056,34136,34104,32056,29976,27895,27896,27895,27895,27895,27895,27895,27895,27895,29943,27895,27895,27895,29943,29943,31991,31990,31990,31958,29909,29909,29909,29877,29877,29877,29877,29844,29844,29844,29811,42390,48631,48664,31826,25552,33939,33971,36018,42260,48533,48598,46453,40146,31760,19181,25454,21293,23373,21227,35985,12872,14986,8680,8680,12841,19115,23308,17068,31792,44405,46518,46518,44469,40211,46550,44372,38001,38065,31760,33938,14953,12872,14953,19146,21453,38292,38098,29680,33841,35888,31760,52824,61276,61308,44373,46518,46454,27600,42358,21359,12875,10730,12810,12809,17035,21293,19148,27599,31727,29614,38131,33873,21261,25520,17035,25487,40211,38131,55002,48663,61310,63423,50809,50809,36117,38099,48663,46518,44438,44405,42325,42292,40179,38066,38067,36018,35954,33906,35953,29582,23374,42325,42292,38002,33840,29581,19047,19048,29645,46451,46452,48532,48532,48532,40113,40146,44339,46485,21162,17001,14887,16967,19048,12774,6403,10597,16999,12741,14854,10661,12774,16967,8515,16967,14854,14918,8548,10661,19079,10660,12774,14886,12773,14886,8548,8516,19047,23208,29548,33806,31727,12809,50778,29648,14889,25486,31760,23244,17002,27567,27502,23341,23308,23308,23340,25389,27502,35824,27469,19083,21163,103},
{34170,34170,34170,34170,32090,32090,32090,32090,32057,32057,32089,32090,30009,30009,30009,30009,27961,30009,32089,34169,34169,32056,29976,29976,29976,29976,29976,27928,29976,32056,32056,32024,32024,30008,32056,34136,34104,32056,29976,27896,27895,27895,27895,27895,27895,27895,27895,29943,29943,29943,27895,29943,29943,29943,31991,29910,31990,29910,29909,29909,29909,29909,29877,29877,29876,29844,29844,29844,25619,25587,38132,44503,29746,38165,25553,23439,17133,29712,59130,48565,54871,46420,21293,21293,21292,27599,21294,14922,33873,25420,25387,25355,23243,19082,10664,12777,6503,10728,25486,29712,29679,33938,29712,29713,23341,16969,23243,6470,27631,17098,12873,12873,14953,19145,40178,33873,31825,40179,29582,33808,48630,54937,54937,42260,50743,59131,25422,44503,21295,10795,8649,10729,12809,17034,17034,12808,12808,23373,33906,42390,38164,36052,38165,33906,42325,44437,42324,55002,57115,63422,63422,48631,44372,40179,42259,57016,52822,56983,57016,44405,56983,54804,52724,54870,54870,52790,54871,54870,44306,33906,57083,52857,31630,35953,29580,14854,8516,29613,46419,46419,46452,48532,46452,40113,42227,44371,35984,16967,12774,12742,16967,14887,12742,10630,14822,19111,16998,12741,6404,8484,10596,8548,8581,6435,10661,6436,10661,10661,14854,8516,12773,16967,16999,14918,14886,21160,29580,31661,27501,44471,17035,50777,27567,12776,25422,23276,21099,23341,25454,25454,23341,23341,23308,23340,23341,25421,25421,23308,12776,19083,105},
{34138,34138,34170,34170,32090,32090,32090,34170,32090,32057,32057,32089,30009,30009,30009,30009,27961,30009,34137,36217,36217,32056,29976,29976,29976,29976,29977,27928,27928,30008,32056,32056,32024,30008,32056,34104,34104,32056,29976,27896,27895,27895,27895,27895,27895,27895,27895,29943,29943,27895,27895,29943,29943,32023,32023,31990,31990,29910,29910,29909,29909,29877,29877,29877,29844,29844,29844,27732,29844,34003,48632,33906,29746,33939,23407,17166,19278,29713,38067,31826,27568,27471,21132,23212,23276,17035,14955,14987,33873,23340,17034,10760,10728,14922,10696,12744,4423,10761,17035,19115,19180,21228,17035,19116,19083,19082,23242,8583,19179,17033,10760,10695,17066,19113,46484,27502,23341,42291,27469,25388,46453,63356,63389,52792,52824,57050,27503,44439,17101,10730,8649,17068,21227,17033,14888,12807,12808,27566,38163,42390,36084,36019,36084,33939,44438,38098,38098,46550,46518,44372,40179,42324,57082,38033,42259,57049,54871,57016,59096,44372,56950,54803,48467,35921,42325,48631,50776,50710,42193,25454,46518,44405,35921,38098,25386,23273,8516,35952,46419,46451,46452,46452,46452,40113,44372,44371,38065,17032,14887,14854,14886,19112,16935,12709,16999,19111,12741,19047,14854,19079,12741,14854,8580,6403,8580,6403,14886,14886,16967,16967,16999,14919,12838,19079,23273,25386,29548,27501,27469,31825,48664,42326,10696,8583,8550,23308,25421,23308,25389,25421,23341,23340,23308,23341,23373,25453,25421,23308,10631,12777,106},
{34138,34138,34138,34138,34138,32090,34170,34170,32090,32057,32057,32057,30009,29977,30009,30009,30009,30009,34137,36217,36217,32056,32024,29976,29976,29976,29976,27928,27928,30008,32056,30008,29976,32056,32056,34104,34136,34104,29976,27927,27895,27895,27895,27895,27895,27895,27895,29943,29943,29943,29943,29943,29943,32023,34071,48762,40343,29909,31924,29779,34004,29843,29811,34037,31924,29811,29811,25586,29811,36084,36084,33939,27633,25520,27665,19214,19246,29745,54938,38165,19214,17101,19180,19115,38034,21164,16906,33906,35953,19147,14921,12841,16937,21163,21163,23211,12809,21293,23373,23373,23373,27567,23308,23341,21195,21163,19049,12776,14985,14985,14953,14953,17065,21226,44436,23308,29647,40146,25356,35856,52792,61276,63357,50711,54937,59131,31761,36051,29615,25389,12810,50809,29677,21225,19113,14920,23405,42357,40179,40310,38165,36052,36084,23374,17036,38066,40147,48598,46518,44404,38033,44405,59163,40179,40179,57017,56984,54936,54903,59129,59063,54836,42160,8518,23407,44504,46681,46551,42193,19180,42390,46519,38001,29580,23273,23273,10629,42259,46452,46451,48532,48532,48532,40113,40179,46452,35952,17032,14886,14919,14951,16999,16967,21161,16935,19047,10660,12709,16999,12741,16966,16999,12741,12774,14886,14918,12805,12741,12806,19144,17031,19144,14918,25386,27467,25355,23307,12743,14856,38132,38164,21228,10695,8583,8550,27534,21196,21196,19115,25454,23340,23308,21228,23341,25389,25453,25454,19083,14857,12776,93},
{34138,34138,34170,34170,34138,32090,34138,34170,32090,32057,32089,32089,30009,29977,30009,30009,30009,30009,34169,36217,36217,32056,32024,32024,29976,27896,27928,27928,27928,30008,32056,32056,30008,32024,32056,34136,34104,32056,30008,27928,27895,27895,27895,27895,27895,27895,27895,27895,29943,29943,29943,29943,29943,32023,36151,57116,42391,27698,40246,46551,48664,44439,42326,46520,42358,38132,44471,40245,36051,42358,36019,38100,42326,33906,38100,38133,33906,38132,36051,35987,36052,31826,29745,33906,31793,31761,31761,33841,31727,25453,25518,23276,16873,19050,29549,25388,33841,46582,46583,50776,50809,50809,44470,44437,42324,29582,14856,17001,17065,14953,14920,23339,21259,27532,38066,21196,31760,33840,31695,52824,63389,61244,59163,46518,54937,59163,29680,12810,8616,10729,17166,55035,38098,17064,17033,23373,40179,42324,38098,40277,36084,36052,36117,17100,12843,31728,42260,42324,42291,48598,52857,54970,61277,42291,42259,59163,57082,54904,52855,54871,56983,54837,40079,6405,23407,44536,46681,46583,38001,21261,46616,48631,29548,29580,23273,19047,8549,31792,46452,46452,46484,46484,46452,40081,40146,42226,23306,21193,16967,12806,8580,12806,16967,12774,14854,19080,19080,16934,10661,10660,14886,16966,16999,8515,10628,10628,14918,14918,12805,10725,14950,14918,21225,25419,27468,27468,12711,14889,16905,14857,6405,6470,10727,19147,12744,21195,23276,12776,14889,23309,23308,23340,23308,23341,25389,25453,27502,27502,12809,16970,66},
{34138,34138,34170,34138,34138,32090,32090,32090,32090,32057,32089,32089,30009,30009,30009,27961,27961,30009,34137,36217,36217,32056,32024,29976,29976,29944,27928,27928,27928,29976,32056,32056,29976,30008,32056,34136,34104,34104,30008,27928,27895,27895,27895,27895,27895,27895,27895,27895,29943,29943,29943,29943,29943,32023,34103,57116,46584,36052,44471,46584,48665,44471,46552,50745,46552,46552,50777,46552,48632,50745,46552,48632,50713,46519,46552,46519,44406,46519,44406,42293,46519,42293,42325,46519,42293,44406,46519,27567,25453,14985,14921,33776,12744,19051,29581,29614,38132,46550,48631,50776,50809,50776,46550,44437,33873,25356,14856,19114,21194,17000,19081,23307,25387,33871,33840,21228,35953,25388,48598,52856,50678,44372,42259,38098,52824,59131,27502,10729,8584,10762,19247,52955,52922,31824,29679,38099,35921,40147,40179,40277,36084,36052,27633,19149,14956,33840,29614,33840,44405,50711,54970,59163,63389,42291,38098,57115,54969,57018,50710,44372,52822,54837,44273,16970,25455,36084,42423,33938,14922,21293,44438,40016,31629,21193,19080,19081,10662,38131,46452,46452,52791,50710,46452,40113,42291,33871,25419,23306,19145,17032,16999,8548,14887,12806,12773,19079,14918,16967,14886,12805,14886,16966,19144,16999,8580,10725,8580,8612,10725,12838,12805,21193,27532,27500,25420,21227,6405,21228,29680,10728,10696,10663,14857,12809,6438,12712,21196,16970,23308,23308,23341,23341,23308,25421,25421,25421,27502,27534,21196,16970,48},
{32090,34170,34170,34138,34170,32090,32090,32090,32090,32058,32089,32057,30009,30009,30009,30009,30009,32057,34137,36217,34169,32056,29976,29976,30008,29976,29976,27928,29976,29976,32056,32056,29976,32024,34104,34136,34136,32056,29976,27927,27895,27895,27895,27895,27895,27895,27895,29943,29943,29943,29943,29943,29943,29975,34135,59197,46649,40310,44471,46584,48697,44439,46552,50778,44471,46584,50778,42390,46584,48632,42293,48665,44471,40245,46584,42326,42325,48632,40245,44438,46584,42293,46552,46519,42293,48632,46519,19181,14954,17066,19147,38002,10664,21228,27501,27501,40147,40082,42260,40180,42292,52824,48566,44405,29614,29517,21195,31792,31726,31759,25420,44340,35952,42291,27501,23276,40146,21162,54970,46517,33807,42259,33840,40146,52824,59131,44373,14858,8616,10730,19214,48697,57116,44503,35986,38099,38099,35954,42259,40277,36052,40212,42326,31792,38066,40146,29582,48663,46517,50744,52857,59130,61276,33807,38033,54937,57050,57050,52790,46484,52789,54837,44274,31793,44504,48697,42358,36051,23439,21229,39985,41999,25290,10661,8517,10630,10695,38066,46452,46452,50678,48532,46452,40145,42324,38065,33871,21224,17031,17063,14886,12741,12806,14854,12774,19112,16966,10628,10661,14886,14918,12773,12805,12773,10660,8548,10661,14919,14886,17032,19145,29581,29581,27501,23276,14889,10696,21261,25487,8551,27567,27535,19050,21228,17035,40213,33841,14922,27567,25421,23341,23341,23308,25421,25389,25421,25421,27534,23309,16970,41},
{32090,34170,34170,34138,34170,32090,30009,32057,32090,32090,32057,32058,32057,30009,30009,30009,30009,32057,34169,36217,34169,32056,29976,29976,29976,29976,29976,29976,29976,30008,32056,32056,29976,32024,34104,36216,34136,32056,29976,27895,29944,27895,27895,27895,27895,27927,27895,29943,29943,29975,29943,29943,29943,29975,36183,48697,42423,38197,36084,38165,40310,36020,33971,42358,31858,31858,40277,29713,29745,36051,29680,36019,36051,31825,40245,38100,33938,42358,38132,36084,42358,38131,40277,42358,40245,44470,44438,25487,14922,21228,19114,38034,12777,14921,21130,27437,33711,31534,31697,27535,27502,35920,35986,27568,23243,29581,21163,33905,42258,44339,42226,46419,40178,42291,42258,44372,44371,44404,42291,42324,44372,44372,44372,42291,44372,46485,44372,38066,33873,33873,33905,46518,50776,33840,38099,42259,55002,40179,42324,40245,31793,42325,48664,52889,50743,42226,57018,42259,44372,50744,59163,54871,44372,35920,42259,57016,56951,56984,54903,56983,57015,52756,40146,12776,33939,50842,48697,38164,17068,10761,44373,44373,19081,10694,6468,8582,19147,33872,46484,46419,46484,44339,44307,40114,44404,44371,40178,23338,16999,14951,17064,14886,17000,16999,17064,12806,16966,12741,14854,12805,12773,14886,10693,19144,17031,14918,14886,17031,14886,14886,25419,29581,27501,25356,6404,17067,23374,25454,21228,14922,19115,8550,8518,38131,17035,50842,38067,21195,27534,27534,25453,25421,25421,25421,25421,25454,25421,29615,16970,14922,68},
{32090,34138,34170,34138,32089,32122,32058,30009,32090,32090,32090,32089,32057,30009,30009,29977,29977,32057,34137,36217,34137,32056,29976,29976,29976,29976,29976,27928,29976,32056,34104,32056,32024,32024,34136,36216,34136,32056,30008,29976,29944,27928,27927,27895,27895,27895,27895,29943,29943,29943,29943,29943,29943,32023,36183,57116,46584,38164,46552,48697,46584,42325,46552,48665,40213,48665,46616,42325,48664,46552,44406,50778,44471,44439,48664,42326,44438,48632,42293,44438,44439,40212,44438,44406,40212,36019,33939,33971,31858,21260,25421,33840,27566,40244,33841,33808,31728,25390,37967,25421,29614,44405,33906,40278,40114,27436,23308,35985,52823,50710,57017,48532,46517,46484,42258,46452,44339,44371,42291,42291,42291,44372,44372,42323,44372,44372,44372,42292,42292,42259,42291,44404,44437,42324,40179,44437,48663,42324,44373,40278,31761,33874,38099,46551,44404,52791,46485,35920,50776,52857,59196,50711,33741,35920,42227,57017,54871,54903,54903,52790,56950,46385,35888,6405,8615,44503,48664,29648,10696,12809,29647,31630,25389,14887,10661,10694,6404,23275,40081,44338,46452,44371,44339,42194,44371,42258,40145,40144,23305,17031,17031,14886,14919,14918,14918,16999,21192,19111,12773,10661,21224,16998,16998,14918,21224,14918,16998,12838,19144,27532,27500,29613,31694,25453,14889,21228,21195,25421,27567,23341,14890,38132,21260,46551,23374,44503,27567,21195,35986,33873,27566,23341,23340,25389,25389,25421,25421,23308,14792,12711,99},
{34138,34170,34138,34138,32089,32090,32090,30009,30042,30041,30009,32090,32057,30009,30009,30009,30009,32057,34137,36217,34137,32056,29976,29976,29976,29976,29976,29976,29976,32056,34137,32056,32056,32056,34136,34168,34136,34104,32024,29976,29976,27928,27895,27928,27928,29975,29943,29975,29975,29975,29943,29943,29943,32023,36215,57148,36085,23504,38164,44503,46584,44471,46551,50777,46519,48632,50778,46551,48664,48665,46551,50745,46584,44471,48664,44406,44438,48632,42293,44438,46519,42292,46519,44406,42293,31825,21261,19148,29582,27534,35921,31630,33808,35952,33774,31727,35953,27469,31596,33741,29548,44340,42260,48664,46485,27468,27534,35953,52791,50742,57049,46484,46517,44404,40146,42259,42258,42291,42259,42291,42324,44404,42324,44404,44405,44405,44404,44372,44372,44372,44372,44405,44404,35985,42357,46485,46518,42292,42324,40245,31761,40245,50745,25454,44405,57049,46485,40146,44405,42292,50744,38033,31660,38032,38001,61244,52824,57050,56984,46517,59096,59063,35921,6405,4357,29713,40180,35987,19116,14922,38132,44242,31694,12774,10661,12774,8550,21196,46420,48565,52726,54871,56984,50710,52824,61243,44371,40144,29611,16966,16967,27532,27499,16999,14919,14887,16967,16966,19112,14886,16966,19079,19111,17031,19176,14950,17063,14951,25452,29614,25388,31727,25389,10663,17067,38132,21228,17034,25487,50842,17003,48729,25421,38067,40212,29583,21228,27567,27501,44405,27534,25454,23341,23341,25389,25421,27534,23340,19050,23275,87},
{34138,34170,34138,34170,32089,34170,32090,32058,30041,30041,30042,32089,32057,30009,30009,29977,30009,32057,34169,36217,34137,32056,29976,29976,29976,29976,29976,27928,29976,32056,32056,32056,29976,32056,34104,34136,34104,34104,30008,27927,27928,27928,27895,27895,27928,29975,27895,29943,29943,29975,29975,29943,31991,32023,38263,57148,40311,40343,46584,46584,46584,42358,44471,46616,40245,42358,46551,42326,44438,42326,40180,44439,40245,40212,46551,42293,44406,46551,42293,44439,46519,42325,46551,46519,44406,29680,19212,17034,21293,21227,33840,31695,29582,29549,27468,25355,29580,29548,31661,37967,44306,44372,42195,42293,48598,40080,38098,42259,50711,48630,50710,46484,46484,46485,42227,44405,42324,42292,44372,44372,42324,42324,42292,42324,44372,42324,42291,42292,42292,42292,42324,44404,42324,44406,44373,44405,44405,38098,40179,40213,23406,21294,36019,38002,42259,57050,40146,35953,40179,46517,46551,38033,33709,35920,44340,61244,61276,59162,54903,48597,59064,56950,38000,6405,4357,8616,35954,38133,17100,14922,29713,48631,40179,14919,10725,12774,6405,14990,40345,57049,54871,48532,54936,61276,59130,54904,50677,40145,40144,21192,21160,29678,42291,31726,27500,29646,27532,27531,19111,19112,19111,25483,21224,21225,17064,12806,14919,12774,14888,23308,31728,12711,17003,21229,25487,21228,23341,14921,21228,50810,25487,44536,16970,14857,27699,23277,25421,23308,29615,44405,29615,25421,25389,25389,25389,25421,25421,27501,29615,29679,68},
{34170,34170,34170,34170,32090,34170,34138,32090,30009,30042,30042,32057,32057,32057,32057,30009,30009,32089,34169,36217,34137,32056,29976,29976,29976,29976,27928,27928,29976,29976,32056,32056,29976,32056,34104,34136,34104,32056,29976,29976,27928,29976,27928,29944,29976,29975,29975,29975,29943,29975,29943,29943,31991,32023,34070,38263,36151,31990,38231,42424,42424,42456,42456,42456,44504,44504,44439,40278,42359,46552,44471,48665,46584,48665,48697,50777,50745,48664,46552,46519,46552,44471,48665,48632,46551,25454,14987,17068,12842,10728,37936,31726,44438,27534,14921,12776,25454,23373,21324,42325,42357,44438,44471,40245,52889,48564,42324,31694,27501,38066,31727,46452,46484,44436,42259,44437,48663,42324,44372,44404,44404,42324,42324,44405,44437,44405,42292,42324,42324,44405,44405,44405,42325,38099,42325,50744,46518,48598,35888,44340,35952,42260,61309,46486,46453,54970,46453,40179,42292,44405,46518,38066,31628,38033,42227,59163,52824,61243,54871,59064,59063,54837,40080,8486,6405,4357,21164,29745,10729,10761,21294,31760,27533,14854,10725,12774,6437,14958,25753,36118,44339,57017,59195,54936,57017,50711,46484,42226,44370,31692,31692,29646,35920,25388,23339,25452,27532,25421,33905,23339,21258,19146,14920,12774,6501,6469,6469,6438,21229,19115,25389,25454,40245,25422,8550,6437,36018,19115,52923,50777,40212,31760,25422,25357,31696,31663,29615,31760,44438,35986,27534,25421,25389,25389,25389,25453,25421,27502,29614,27535,37},
{34170,34170,34170,34170,32089,34138,34170,34138,30009,32058,32090,32090,32057,32057,32057,32057,30009,32057,34169,36217,34137,32024,29976,29976,29976,27928,27928,27928,27928,30009,32056,32056,30008,32056,34104,34136,32056,32024,29976,29976,29943,29976,29976,29944,29976,29975,29975,29943,29943,32023,29975,29943,32023,32023,31990,34070,34038,31990,31990,31990,31958,36118,31957,31957,31957,31859,25488,19085,23343,25487,23343,27666,29714,25488,29681,31793,29745,19213,21327,10828,10860,15020,21261,23406,27632,19180,17035,12841,12842,10761,17066,21260,48664,31792,19114,17034,38132,31793,27599,40276,44470,42390,25521,42358,52922,50678,42259,33841,31825,35954,40146,46485,44404,46484,40146,44437,46550,42324,44405,44437,44404,44372,44404,44437,44437,44437,46518,44437,44437,44437,44437,44437,42325,33938,42325,46518,40179,35823,31597,46419,46451,52791,63422,63422,63389,46485,46485,42227,42259,38065,48598,42259,31661,35920,42259,54936,54904,54903,57016,50677,59064,56982,42194,16970,14889,12776,21164,33938,17035,8615,8615,19180,31759,14919,12773,12806,12776,27667,19284,31827,54872,52823,57049,50743,50711,50678,48565,46452,40177,54936,46452,44371,33840,33840,23275,21194,27533,25421,31727,21227,6437,4324,2244,6437,2244,2244,6471,23342,31891,40212,35987,21294,50777,21196,6438,10696,48697,27535,57116,35986,27502,29582,33776,35888,35888,35920,38066,46583,40180,29615,27502,25421,25421,25389,25421,25453,25421,27502,27534,23341,23},
{34170,34170,34170,34170,32089,34138,34170,34138,32058,32058,32057,30009,30009,32057,32057,30009,30009,32089,36217,34169,34137,30008,29976,29976,29976,27928,27928,27928,27928,30009,32056,32056,32056,32056,32056,34104,32024,32024,32024,29976,29975,29976,29976,29976,29975,29975,29975,29975,29975,29975,32023,32023,32023,32023,32023,34070,31990,31990,31990,31990,31958,34038,31957,31957,31925,31795,27536,16972,21198,21229,17036,23375,27568,23407,25520,25487,25520,8747,21327,10828,10796,12876,21293,19180,19180,17067,17067,14954,14922,14954,19147,19180,46583,42325,27598,23373,46551,21293,27664,42357,44438,42390,29845,44503,44470,52758,48532,31727,33906,38099,42324,46517,44404,46452,42226,40178,46518,44437,44469,44437,44405,44405,44437,44437,44437,44437,44437,44437,46518,44437,44437,44437,42292,38099,40311,44437,38033,33644,35757,46451,46451,52791,57083,59196,54970,44371,46485,40113,40178,44372,48631,44372,27500,38065,40146,57050,54904,57017,54935,48597,59063,56950,54871,52823,42390,36149,42455,38197,19180,8615,8614,17034,29711,14886,12806,14886,10629,4326,4392,31794,48565,48565,50678,50678,50710,50678,50678,44371,46452,46516,52791,52824,42226,35920,35985,33807,33840,31759,35953,19050,6438,4357,4325,4357,2212,2212,17069,25487,36084,48664,36019,42358,38132,10729,8616,17002,40212,46551,35986,33808,31695,31662,35920,40114,40147,42228,33841,35986,40180,27534,25421,25421,25421,23341,25421,25421,25421,25421,25453,17034,21},
{34170,34138,34170,34170,34138,34138,34138,32090,32058,32058,32057,32057,32057,30009,29977,30009,29977,34137,36217,36217,34137,30008,30008,32024,29976,27928,27928,27928,27960,30009,32057,32056,32056,32056,32056,32056,32056,32056,32024,29976,29976,29976,29976,29976,29975,29976,29975,29975,29943,29943,31991,32023,32023,32023,32023,32022,31990,31958,31958,31990,31958,31957,31957,31957,31925,31925,29682,21197,25391,23342,21230,23278,21261,23342,25455,29745,29746,21294,23310,17101,19214,19246,25519,14955,17067,17067,17067,14986,14954,19147,25486,36051,48696,40244,25454,25422,40245,27535,31793,40244,48664,42358,48697,55003,38098,52693,50645,40081,31728,35986,33873,46517,44404,46484,44339,42259,48663,46518,46518,44437,44437,44437,44437,44437,44437,44437,44437,44437,44437,44405,44405,44405,42324,36019,42391,44438,37935,39950,42128,48532,48531,48532,46517,54970,50743,40145,40113,35887,42227,46485,44405,44437,31759,35920,38034,59131,52792,59162,56983,52790,59096,59031,54870,59031,52791,40277,36084,40278,27600,10760,27597,14856,12743,14918,14918,16999,8485,4292,8585,14891,46485,46484,50678,50710,50710,50710,48597,44339,48597,50743,50710,46452,35919,38065,38001,33808,16969,33873,35920,19115,4390,4358,6471,10729,2244,2245,6439,8616,8584,8551,42325,21261,10729,10696,10663,19082,23308,27566,38098,40147,38066,40179,29582,38099,33873,31728,29647,31728,31760,27502,25421,25421,25389,25421,25389,25389,25421,25421,25453,12776,20},
{34170,34170,34170,34170,34138,34138,34138,32090,32090,32058,32057,32057,32057,29977,32057,32057,29977,34137,36217,36217,34137,30008,29976,32024,29976,29976,29976,29976,27928,30008,32057,32057,32056,32056,34104,32056,32024,32056,32056,32024,29976,29976,29976,29976,29976,32024,32023,29975,29975,32023,32023,32023,32023,32023,32022,32022,31990,31990,31990,31958,31957,34070,31957,31925,31860,31957,25488,25423,33875,36019,38100,44471,42326,42326,42391,42358,40213,25520,27568,29777,29745,29745,19115,19148,19180,19180,19147,14986,14954,19147,31825,44470,40244,40244,29615,27665,25552,23374,33874,44405,48664,40245,52890,55003,40178,50612,46386,44274,31695,35986,38034,46517,46484,46485,44371,40146,44405,46517,44437,44469,44470,44437,44405,44437,44437,44437,44437,44437,44437,44405,44405,42357,33938,36051,46551,35986,40015,35692,44273,48532,48532,48531,44339,46485,46517,42258,38032,33806,44339,50711,46485,46518,35953,29614,35920,54904,57082,59195,56983,59097,59097,59064,56950,59031,59063,59063,56984,44471,27632,14921,46550,31759,10630,19080,21193,8549,16904,8517,8517,29647,44339,48565,50678,50711,48598,46452,48597,54904,52823,44339,40113,38000,29581,35953,35953,33840,14922,21228,27567,10696,6471,12810,8649,6471,2245,2212,2277,4358,4390,2244,21229,12842,25585,25421,27502,25421,31760,40180,44373,42292,42260,38001,27469,29647,29647,29615,27534,35986,31727,27502,25421,23308,23341,19115,25389,25389,25421,25421,25454,8551,19},
{34170,34138,34170,34138,34138,34138,34106,32058,32090,32090,32057,32057,32057,32057,30009,30009,30009,32089,36217,36217,32089,32056,32056,32056,29976,29976,29976,29977,29976,30008,32056,32057,32056,32056,34136,32056,32056,32056,32056,32056,29976,29976,29976,29976,29976,29976,29975,29975,29943,29975,32023,32023,32023,32023,32022,32022,31990,31990,31990,31990,31958,34005,31957,31924,31827,27699,27666,29779,29746,40180,44373,50744,50744,40114,42260,44243,46355,35986,44404,54969,52888,48630,40179,17067,19212,19147,17067,17067,17067,17068,25552,44470,50744,40244,36051,19180,10761,12907,27632,46518,46551,44438,48696,55003,52823,54805,42161,50613,33840,35986,38066,46517,46485,44372,38033,46485,46550,48631,46582,48631,48631,46550,46518,46550,44470,44437,44437,44405,44373,42324,42357,33905,23374,33938,36051,35953,40080,33644,40112,46451,46451,48531,42226,50711,46485,44339,38033,40113,46485,42259,42292,44373,31727,29646,35888,52823,57082,59097,54936,48696,48696,48631,54870,59031,59031,59030,56983,44471,29745,21195,46583,50712,46486,46549,27500,35887,44306,25323,19050,33807,40146,46484,48565,44371,48597,52824,50710,46452,40113,38033,35920,33775,29646,29646,35953,25421,10664,8583,10664,6503,10697,14989,10795,10730,4358,4390,4390,6504,10730,14923,23440,8617,36117,25421,27534,35986,38099,42293,40147,33808,29615,31695,27502,27534,27534,27534,23308,31760,35954,27502,23309,23309,23308,23276,25421,25421,25421,25421,21228,6471,19},
{34138,34138,34170,34138,34138,32089,32057,32057,32089,32090,32057,32057,30009,32057,30009,30009,32057,34137,36217,36217,34137,32056,32024,32024,29976,29976,29976,29976,29976,32056,32056,32056,32056,32056,34136,32056,32056,32056,32056,32056,29976,29976,29976,29976,29976,29976,29976,29976,29943,29975,32023,32023,32023,32023,31990,32022,31990,31990,31990,31958,31958,31957,31957,31892,31892,31925,29844,29844,29779,40245,36019,25488,23375,31761,42292,46291,46388,35986,50711,48663,44470,40244,36084,21229,17100,14987,21325,21260,21326,14956,17069,46583,50809,42325,21292,8680,6600,8713,27631,44438,44438,40244,46616,42325,44339,52724,40048,48532,29614,35954,35953,46517,46517,44372,38000,46485,46517,48598,46550,46550,46550,46518,44437,44437,44437,44437,44405,44404,44404,42324,35986,17035,21294,29648,33938,40178,46452,46419,42258,40080,44339,48532,50645,46517,46485,44339,40146,44404,46485,33808,40146,40179,33808,38066,38033,57016,59063,59063,57016,48696,46648,48664,56983,59063,59063,59063,59096,48631,31793,27534,46615,50777,54970,48565,44338,42225,42226,29549,21130,10598,21130,40211,44371,50678,52791,42291,40113,42226,42227,42227,40146,38066,33872,35953,27566,10760,17097,10664,6471,2245,2245,6471,6504,4358,4358,4390,10730,12876,12876,14956,17069,23407,25487,31760,38099,38099,38034,31727,31727,29615,27469,27534,25421,27502,27534,27534,27534,21196,31760,27502,16970,14890,23309,23308,27469,25389,23341,25421,16970,6471,21},
{34138,34138,34138,34138,34138,34138,32089,32089,32090,32089,32057,32057,32057,32057,32057,32057,32057,34137,36217,36217,34137,32056,32024,32024,29976,29976,29976,27928,29976,30008,32056,32056,32056,32056,34136,34136,32056,32056,32056,32056,32056,29976,29976,29975,29976,29976,29975,29976,29975,32023,32023,32023,32023,32023,32023,31990,31990,31990,31990,31990,31958,31958,31957,31925,31925,31925,29844,27731,29779,38132,40212,48696,38099,27567,46486,46291,44308,35954,52857,36018,27632,25520,27633,19180,14987,14954,12874,10761,10827,12907,21425,40277,44470,33970,12873,8648,6600,10793,23373,44438,42325,38132,42423,44470,31693,52725,33710,44306,29647,35954,35953,46517,46484,44372,44339,46484,44436,44404,44405,46485,46517,46485,46485,46485,44437,44405,44405,44372,44372,38099,19148,21229,33840,44340,42259,46484,48565,46419,44339,35953,29582,44306,46451,42259,50711,48598,42259,46518,54937,38001,38001,42260,31760,33905,38065,52789,54902,56950,56950,48598,48598,48597,54870,56982,59063,59096,59064,48663,38164,44405,48697,55003,54938,48598,44371,44306,44338,35855,14791,27469,40211,46485,40146,35888,38033,38033,35953,38066,38066,38001,35888,33808,33808,27566,31824,12936,19338,8550,6470,4357,2277,2278,4391,4390,2310,2309,12876,12877,14989,14989,4358,8584,31761,35986,33840,29615,29615,29615,29582,27502,27502,27469,25389,27534,27534,27469,19082,16937,31728,29615,10664,6502,21228,25389,25389,25422,14857,25422,10697,4390,23},
{34138,34138,34170,34138,34170,34138,32090,32089,32090,32090,32057,30009,30009,32057,32057,32057,32057,34137,36217,34169,34137,32056,32056,32024,29976,29976,29976,29977,29977,30008,32056,34136,34104,34104,34136,34136,34104,32056,32056,32056,32056,29976,29976,29976,29976,29976,29976,29975,29975,29975,32023,32023,32023,32023,31991,31990,31990,31990,31990,31990,31990,31990,31958,31957,31925,31925,29844,27699,40246,42358,46486,44438,44437,38131,40212,40051,42260,46551,46518,50745,36019,19213,19213,19213,14986,14987,14954,12874,14955,19148,21392,33907,36051,21293,10793,8648,8681,17067,29615,46518,50777,48696,44535,40276,40113,50547,29484,31597,46420,44340,40179,50711,48630,50710,50678,50711,50742,52823,50743,50743,50743,50743,50710,48630,48597,48597,46484,46452,40178,17002,12744,10696,42291,54838,48566,44438,46485,48532,46453,40245,33874,46485,42258,48598,50679,46485,42259,44372,50743,38033,40146,44405,27501,48597,54902,57015,57015,59128,54837,52724,52789,50644,48531,52757,50611,48530,46418,44338,42292,33906,23440,31827,38099,42260,44339,44306,42193,42226,52856,50711,48565,44339,40146,38001,38066,38033,38033,35985,35920,33840,33841,14824,21196,21195,31824,19243,21452,8517,4357,2309,4358,6536,8584,197,2278,6536,6537,8649,4391,4391,10697,10729,17002,27566,27534,27534,27502,27502,27534,27534,27502,25421,27502,27502,27534,27534,23276,31761,27534,27502,17002,6470,8615,21195,23309,21228,12777,19115,6470,4390,23},
{34138,34138,34138,34170,34170,34138,32090,32089,32090,32090,32090,32058,30009,30009,30009,32057,32057,34137,36217,36217,34137,32056,32056,32024,29976,29976,29976,30009,29977,29977,32056,34136,34136,34104,34136,36184,34104,32056,32056,34104,32056,29976,29975,29976,29975,29975,29975,29975,29943,29943,32023,32023,32023,32023,32023,31990,31990,31990,31990,31990,31990,31990,31957,29909,31925,31925,29844,27699,42358,50776,52824,52824,50776,42293,31826,19182,33939,38099,42325,50745,29680,23406,23406,21261,17067,17067,14986,12874,14954,12809,12809,12809,27567,19147,10793,8680,8680,27600,38099,46550,57083,38264,33939,36018,44371,44306,16904,18985,42193,44405,42292,42324,40146,50710,52790,52823,46485,50711,55002,52824,50744,50678,50677,50677,50645,48564,46451,38033,14857,8583,8519,8518,44371,42258,42325,36019,40179,35953,40180,42390,36019,40179,29582,50711,50678,48598,44340,38098,52857,42291,33807,52791,52790,46451,44370,46483,46483,46451,50644,48564,50645,50677,50644,50677,50644,50612,46452,35953,38067,38099,31760,29615,25389,35921,27469,38066,50711,61275,57017,46485,42226,40146,40113,38034,38033,38033,35921,35953,29582,23308,25422,14857,29614,23340,23276,21259,17130,19147,6438,4357,10762,8650,12876,164,4358,6503,4456,4423,4455,6504,4391,4390,12777,27502,27502,27502,27502,27534,27534,27534,29583,27502,27534,29615,29647,31760,14857,16970,29615,23244,12809,6470,6503,17035,19083,10696,19115,16970,6470,4422,25},
{34170,34138,34138,34170,34170,34138,32090,32089,32090,32090,32090,32057,30009,32057,32057,30009,32056,34137,36217,34169,32057,32056,32056,32024,29976,29976,29976,30009,29977,29976,32056,34169,34136,34136,34136,34136,34104,32056,32056,34104,32056,30008,29976,30008,29976,29976,29975,29975,29975,29975,29975,32023,32023,32023,32023,32022,32022,31990,31990,31990,31990,31990,31957,29909,29877,31925,31925,27731,36052,42325,50744,40212,44470,33906,40245,19214,23407,42325,46518,59229,55003,48664,27599,23406,17100,17067,17099,17067,14954,10729,12809,14922,23373,25486,14954,14987,8681,38099,33873,46583,59196,38165,31794,35986,42194,40048,14792,14792,40178,38099,38099,27566,38066,52824,52823,54904,46485,55002,61342,48664,44406,44338,48532,48532,48531,48499,33774,12744,12777,8583,6470,8551,44372,48564,44405,36052,42358,42357,36019,42358,40244,38066,38001,52790,50711,46485,42259,42259,35953,33775,52757,52821,52757,52789,56983,52822,54870,52757,46451,44338,44306,46451,48563,46483,44339,35953,38066,50777,44438,44373,54938,29615,31695,42292,50745,44372,42292,42258,38000,27436,35920,25356,19082,35921,38033,38034,16937,27502,21229,12777,12809,17003,29647,25389,23276,16969,4357,10696,4357,2245,15021,8715,10795,2277,2310,6503,4423,4391,4391,4391,4391,4390,6503,16970,25453,27502,27502,27534,27534,29615,31728,31728,27470,25422,27534,16937,14857,23309,31728,25422,17002,8583,6503,8583,14922,8615,23341,14857,6470,6471,25},
{34138,34170,34138,34170,34138,34138,32057,32089,32090,32089,32090,32057,30009,32057,32057,30009,32057,34169,36217,36217,32057,32056,32056,29976,29976,29976,29976,29976,29976,27928,32056,34136,34104,34136,34136,34136,34104,32056,34104,34104,32056,29976,29976,32056,29976,29976,29975,29943,29975,29943,32023,32023,32023,32023,32023,32022,31990,31990,31990,31990,31990,31958,31957,31957,29877,29877,31925,29811,36020,35987,52825,50777,40245,36019,38132,25520,31793,42325,48696,46550,48664,46517,23405,17067,17067,17067,23439,19213,14922,12841,17067,21260,21260,31825,29680,25487,14987,40277,46551,50744,50745,29747,33906,50776,40145,37935,14824,16872,44373,27567,27567,21228,40179,44405,42259,40113,50678,55002,61309,52922,46550,50677,48532,50645,48564,42226,31661,21130,12744,8551,8551,6470,38034,54936,42325,33939,27567,38130,38066,40310,59196,50776,63389,56984,48565,38066,40147,54970,50710,50676,50643,52756,52821,50708,48563,50643,48563,46451,40112,42258,44306,44306,46452,35920,38066,50776,48663,35953,23276,27534,38099,27469,29615,44373,40179,35953,38066,40114,38001,35953,29614,10632,8551,23309,21195,23309,27535,21196,8552,19182,14922,21229,14922,10663,27567,23341,33939,8551,2245,2212,6504,6536,4358,6504,6471,4423,4391,4390,4390,4391,4391,4391,6471,6503,25454,27502,31696,29615,29647,29615,27534,27535,25389,27502,21196,27535,23341,23309,31792,31727,14889,6470,6503,6470,16970,14921,25421,21196,23309,21229,83},
{32090,34138,34138,34138,32090,32090,32090,32089,32090,32057,30009,30009,30009,32057,32057,30009,32057,34169,36217,36217,34105,32056,30008,29976,29976,29976,29976,30009,29976,27928,30008,32056,32056,32056,34104,34136,34104,32056,34104,34104,32056,32024,29976,32056,29976,29976,29976,29943,29975,29943,32023,32023,32023,32023,32023,32022,31990,31990,31990,31990,31990,31958,31957,31957,31957,31925,31925,34005,46519,44438,46518,44437,42325,40212,38132,25520,23406,27600,44470,52857,33938,50710,21260,19148,19180,14987,23439,17100,12874,14922,12873,21260,25486,14987,12874,31858,23439,44439,46583,48663,48729,33973,36019,44438,35920,37935,16937,25291,35953,29647,21228,19115,33874,50777,57018,46485,48597,44404,48630,46517,46484,44339,48564,48564,44371,40146,40113,46452,42226,38001,27469,23309,38100,33873,42326,25487,31793,33872,40211,40310,59196,50744,63422,52790,46517,35953,44339,50709,48564,50644,50709,52789,50709,52757,52757,50677,48532,48564,50645,48532,46452,40146,35986,50744,52857,33840,38098,33872,31760,35986,36018,33840,31727,40179,40146,40146,40179,40146,35953,29614,29614,12776,8615,17002,23374,31761,46584,27535,17003,14955,6471,25487,25519,38099,38132,25487,48697,17003,4358,4358,4390,6472,6471,4423,4391,4390,4391,4391,4391,4391,4391,4423,6503,6503,21260,29712,23341,16970,27534,27502,27502,27534,27534,25389,23341,19082,12776,19082,38160,29709,23276,10630,16970,29614,42292,52857,48598,48631,52824,50743,193},
{34138,34138,34170,34170,34138,34138,32090,32058,32058,32057,30009,32057,32057,32057,32057,32057,32057,34169,36217,34169,32089,30008,30008,29976,29976,27928,29976,30009,30009,27928,30008,32056,32056,32056,34104,34137,34136,34104,34136,34104,34104,32024,32056,32024,29976,29976,29976,29975,32023,29943,32023,32023,32023,32023,34071,32022,32022,31990,31990,31990,31990,31957,31957,31957,31925,31925,31957,33972,50744,50744,52824,50711,44438,40212,33939,31793,29681,23374,52857,23341,21261,35986,48630,31793,17067,17100,14987,12874,12874,12906,25421,29712,12873,8648,4455,14987,10761,46584,48696,50744,50810,38133,44470,46550,40146,40048,23243,29581,27502,31792,17002,14889,10631,8550,40211,46452,48597,44339,50677,44339,50677,44306,48565,42258,31694,38034,33873,48566,42194,38001,44339,38001,35986,21228,29713,40245,46518,46485,35986,40310,55035,50711,57082,50645,40113,42259,42258,44371,50677,52757,50644,48563,46451,48531,44371,46451,46451,44306,42225,40113,29614,31727,50744,52825,40146,25389,44405,38066,27502,31727,38066,27566,21195,23308,38066,38033,38033,40146,35985,23662,36050,23374,33873,25422,21326,42391,46551,23309,27535,12777,6438,35987,27600,52825,33873,33906,42325,33873,25422,6503,6471,6471,4423,4391,4391,4391,4423,4423,6471,6471,6504,6471,6503,6471,14889,25454,27567,29647,27534,27502,27534,27534,27534,25389,27534,25421,21163,23244,38098,38033,46452,46485,54905,59228,54969,46485,40179,40146,52824,50743,190},
{34170,34138,34170,34170,34137,32090,32089,32058,32090,30009,30009,32057,32057,32057,32057,30009,32057,34169,36217,34169,32057,32056,30008,29976,29976,27928,29976,30009,30008,27928,30008,32056,32056,32088,34104,34136,34104,34104,34136,34136,32056,32056,32056,32056,29976,32024,29976,29975,29975,31991,32023,32023,32023,32023,32023,32022,31990,32022,31990,31990,31990,31957,31957,31957,31957,31925,31925,27699,46518,48631,48599,42292,42358,40213,38132,40212,25487,21261,44471,48632,25454,27534,27534,29712,21261,19180,17035,14954,14954,12906,21228,21293,12841,10728,8648,17068,8648,40245,46551,42227,38133,38133,42357,50776,40146,42161,25356,31662,19083,23307,10727,17033,19082,10631,10631,27533,25388,27533,27501,40178,44242,48403,48500,38033,48599,40180,35953,46453,40081,37968,44242,35888,38066,29679,25453,46550,44372,42259,42325,42423,36051,54970,54904,44371,46516,44404,46484,52789,50676,48596,52756,50709,48531,42257,44306,44338,44339,40146,33840,29615,38067,46551,40180,29582,31727,38099,46551,42292,38066,35953,42292,42292,33840,21195,25421,38034,35921,17001,25711,23953,33970,29712,29615,19083,27535,23309,19147,14921,17034,6438,14889,25421,40244,35953,31728,31793,31760,31696,29615,8584,6471,6471,4423,4423,6471,6503,6536,6503,6504,6471,4391,4391,4390,4422,12777,23341,25454,29679,27534,29583,27534,29614,29615,29647,27502,25421,25389,42227,50711,48566,40212,38066,35986,40243,42259,40211,40146,40178,52856,52824,198},
{34138,34138,34138,34138,34137,32090,32090,32090,32058,30009,30009,32057,32057,32057,32057,32057,32057,34169,36217,34137,34105,32056,32024,29976,29976,27928,29977,29977,30008,27928,30008,32056,32056,32056,34104,34136,34104,32056,34104,34104,32056,29976,32056,32056,29976,29976,32024,32023,32023,31991,32023,32023,32023,32023,32022,32022,31990,32022,31990,31990,31990,31990,31957,31957,31957,31925,31925,27731,40213,44406,52889,48696,31761,29746,31826,44406,31825,21294,25519,29713,23373,27567,35953,29647,21293,21261,14986,14954,14954,8648,23309,17002,12873,10728,12873,12874,10761,36051,38164,29582,35954,40212,46551,46518,35888,35855,25324,33808,44373,27501,35985,38034,44373,33840,25389,35888,42292,38001,38033,46518,44209,44177,42161,46485,40147,40212,42324,46485,44307,42194,44242,35823,44307,46420,44371,52824,40146,42259,40244,42391,35987,54938,52856,42291,31726,33806,48564,48531,48563,48563,46451,42225,42258,44371,46451,40113,31727,40212,52890,42292,16970,21066,25454,31694,25388,44438,42358,29647,40147,38066,38066,38034,38066,33840,19083,27534,31761,40277,27824,28017,40180,31761,27567,8518,23309,27599,10696,33997,21258,19083,23341,29614,33841,33873,31760,29615,27534,27534,27534,17003,6504,6504,6504,6504,6504,6503,4423,27600,6471,4358,4358,4390,4390,6471,6471,14890,27534,29680,27567,29615,29647,29647,29647,29647,25421,27535,31695,57082,40213,17068,10664,8616,12841,23340,40211,42259,35985,38098,54937,52856,202},
{34170,32090,34138,34138,34138,32090,32090,32089,32090,32057,30009,32057,32057,32057,32057,32057,32057,34169,36217,34169,32056,32056,32024,29976,29976,29976,27928,27928,27928,30008,30008,32056,32056,32056,32056,34136,32056,32056,34104,34104,32056,32056,32056,32056,29976,29976,29976,29975,32023,31991,32023,32023,32023,32023,32023,32022,32022,31990,31990,31990,31990,31958,31958,31957,31957,31957,31925,36085,44438,44405,44437,44405,44406,35987,33939,38132,29745,21293,38099,36019,29712,40211,29679,25454,17002,19148,17067,14987,14954,10793,27470,19114,14921,14921,12841,14922,12841,29713,31760,35888,40115,35921,44340,42227,37936,40049,27437,38099,27566,35985,46550,42259,40146,40146,42227,44405,44372,44405,48631,40179,35791,37904,38033,35920,29582,42292,31760,42227,35889,33743,37903,33710,38001,33840,31662,44437,40211,40146,38098,42391,36051,48631,42292,46452,52822,48595,48563,50643,50676,52757,48564,46484,46452,40178,33839,42292,54970,46485,38033,27534,21099,25195,25422,31695,27469,23341,17002,12776,35953,38033,38033,38034,38098,38034,23309,16970,31793,52923,25487,23406,31793,17036,8617,10665,19115,23373,14889,38159,31759,31728,33841,33841,29647,29615,27534,27534,27567,23341,27567,23309,6504,6504,6471,4423,4423,4423,4391,10697,4358,4358,2245,4358,4390,4390,4422,6470,25454,29680,29680,29680,29647,27534,29615,35986,25389,27534,33808,48664,25488,12810,8616,8616,10728,12841,33873,40211,35985,38098,54937,52856,202},
{34170,34138,34138,34138,32090,32089,32089,32089,32090,32057,32057,30009,32057,32057,32057,32057,32089,34169,36217,34137,32056,32056,32024,29976,29976,27928,27928,29976,29977,29976,29976,32056,32056,32056,32056,34104,32056,34104,34104,34104,32024,32056,32056,32056,32024,29976,29975,29975,29975,32023,32023,32023,32023,32023,32022,32022,31990,32022,31990,31990,31990,31958,31957,31957,31957,31957,31925,40246,46551,50776,52824,50744,44405,40245,40245,29745,27665,21293,19181,19181,25486,44339,23308,14890,19148,19148,21261,19212,14954,14954,33808,21259,17001,12809,10728,14954,12809,23374,29680,38001,38165,29712,55035,31760,38066,29614,19050,29647,40212,44437,48598,44373,44373,40147,42292,42291,48631,50776,55002,52857,44307,48500,48501,48630,46452,50712,48631,54937,48598,46453,46387,40114,38066,35953,29581,33840,42259,42259,40211,42423,36019,48664,40179,48597,48564,57015,54935,54903,52822,50677,52822,48531,44371,38131,36019,40244,40145,38000,35855,25389,23244,27146,35986,29647,27502,14857,12809,12777,27535,35953,38034,38034,33873,29615,21195,12809,23341,27567,19148,21262,50810,14991,15055,12876,19115,25454,29679,33873,33841,31728,29615,27567,27534,27534,25454,19116,12809,6503,23341,19148,6471,4423,4423,4391,4423,6471,6471,17002,12712,12744,12744,14792,14792,14792,14824,14856,25355,31661,31629,31629,31629,31629,38001,40113,35953,35953,42259,27633,21295,8616,8584,8584,8616,12777,25454,40211,35985,40179,54969,52856,202},
{34138,34170,34138,34138,32090,32089,32089,34137,32089,32089,32057,32057,32057,32057,32057,32057,34137,36217,36217,34137,32056,32056,32056,32024,29976,29976,30008,29976,29976,29976,29976,32056,32056,32056,32056,32056,32056,32056,32056,34104,32056,32056,32056,32056,32056,29976,29976,29976,29975,32023,32023,32023,32023,32023,32023,32022,32022,31990,31990,31990,31990,31990,31957,31957,31957,31957,31957,31892,50745,48631,46518,52857,44438,38132,38132,29746,25520,21326,19245,17100,31792,52823,44405,14955,25454,21261,19148,19147,10761,8648,33841,29647,19146,12809,10696,17067,12809,12842,17035,35954,35955,40244,52922,16970,31760,27470,21131,29680,23341,27567,46551,52824,52857,52857,52824,50712,48663,50712,57083,54970,44340,40146,54970,65502,54970,42324,40179,48566,46453,42292,50581,46387,44339,40146,31726,40179,42292,42291,36018,42423,33938,46486,35921,40212,59164,63422,63421,61309,61309,61308,59195,61243,59195,46518,31727,33774,35855,40081,35887,35920,35920,35855,25356,14857,14857,12776,12777,10696,23308,35954,33840,31727,29614,29582,19115,12809,19115,21229,17003,29746,48730,25490,23377,27568,25421,31760,31760,27534,27567,27535,25454,25487,19180,17003,8584,8584,8584,6503,12842,21228,6471,6471,4423,4423,4423,4391,10696,6438,6470,6470,6471,6471,6471,6471,8519,8551,16970,33808,29582,27502,27469,27502,31662,29550,25356,35921,35922,21262,17035,6503,8583,8616,10696,14889,19083,42292,38130,42259,54969,54937,202},
{34138,34170,34138,34138,34138,34137,32089,32089,32089,32057,32057,32057,32057,32057,32057,32057,34137,36217,36217,34137,32056,32056,32024,32024,29976,29976,30009,29976,29976,29977,30008,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32024,29976,32023,32023,32023,32023,32023,34071,32023,32023,32022,34070,31990,31990,31990,31990,31958,31957,31957,31957,31925,29812,36085,50712,40212,46551,35987,36019,33939,25520,23472,23407,21326,17132,31759,46452,27567,17035,19148,19148,19180,19148,10761,10697,33906,29647,19114,14921,14889,17067,12841,14890,21195,33842,36019,27600,52922,21163,29647,33841,19051,29680,21261,23341,40244,54970,42292,50744,44372,50712,44372,46518,59228,31759,27501,46517,57083,65502,54937,42324,38034,46420,42194,40081,46387,42194,44371,44372,44372,46550,42291,42259,38098,42423,31825,48631,54938,61277,63422,61276,52857,48663,44437,42324,44469,44372,40179,40179,31727,33839,35887,35887,38033,40113,38065,38065,14857,10696,10696,10664,10696,10696,12744,27567,29614,31728,29615,27534,19115,10696,10664,19180,14987,29712,29648,35986,29615,29680,27567,27535,27502,25454,23341,19148,10697,8616,8616,8584,6503,6471,6471,6471,6503,23309,4423,6471,4423,4423,4423,12809,19148,4358,6437,4325,4325,4325,4325,4325,4358,4390,14857,29582,31695,31695,27502,25454,27534,40114,31597,33775,38066,40212,31695,16969,10696,10664,12777,12809,12809,40211,38130,40179,54937,54937,204},
{34138,34170,34138,34138,34138,34138,32090,32057,32090,32057,32089,32057,32057,32057,32057,32057,34137,36217,36217,34137,32056,32056,32024,29976,29976,29976,29976,29976,29976,29976,29976,32057,32056,32056,32089,32056,32056,32056,32056,32088,32056,32056,32056,32056,32056,29976,32024,32024,32023,32023,32023,32023,32023,34071,32023,32023,32022,34070,34070,31990,31990,31990,31958,31958,31957,31957,31957,27764,31859,50712,42325,42293,40244,42358,42358,36019,25488,19214,19181,19180,31728,40146,17002,25453,23341,17035,21293,14922,14922,14954,38132,27535,17068,31826,17035,19148,19115,23308,29582,29649,33906,19148,40212,33840,38034,42260,25389,23374,19148,27567,38163,54970,50711,54937,50744,52857,54905,50744,40179,19082,16936,25420,52824,59196,50743,35921,38033,46453,42162,42194,46355,37968,38001,40146,40179,46485,42292,40179,31792,42423,33971,42358,44470,25389,29647,46550,27597,19145,14920,14888,8550,31760,40212,35986,35985,33839,35920,38000,40113,35920,35920,35953,12744,8615,10696,8616,10696,10664,10664,23373,25421,29615,27534,25421,21228,14858,12777,19148,23341,23309,29680,29647,29647,27567,27534,27502,27502,21260,8584,6503,6503,6503,6503,6471,6471,4423,4422,6471,6471,17003,6471,4390,4358,4358,4358,4422,4390,4358,4325,2244,2244,2244,2277,2277,4325,4357,4357,19018,29582,31727,29615,27502,29647,33808,35855,35920,46518,44405,44373,29615,17002,10696,10696,10696,12809,35986,40179,40211,54937,52824,200},
{34138,34138,34170,34138,34170,34138,32058,32057,32090,30009,32057,32057,32057,32057,32057,32057,34137,36217,36217,34105,32057,32056,32024,32024,29944,29976,29976,30008,29976,29976,32057,32089,32056,32088,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32024,29976,29976,32023,32023,32023,32023,32023,32023,32023,32022,32022,32022,31990,31990,31990,31990,31958,31957,31957,31957,31925,29812,27666,48664,38099,38132,25520,23375,25488,25520,25488,25520,21261,19181,21261,40082,44308,44372,19115,14987,19180,14954,19147,36018,44405,25421,17035,31859,10761,12874,25421,21228,19114,31825,33906,29681,35986,12777,33873,35986,29647,21261,23341,33841,48598,52792,50711,50744,48631,50711,46485,44437,27501,17001,42325,33840,57115,63389,52856,40212,38034,48566,42194,44307,46388,42195,38066,35986,33808,31760,44405,42325,36051,36052,40310,38132,38164,40212,35986,35986,40242,21225,14888,19113,21195,31825,35921,40180,35921,35953,38065,38033,40113,35887,38033,38066,14889,8616,8583,8583,8584,8584,10664,21228,23308,27502,25421,25421,23341,14890,27568,27535,27567,21261,23341,25422,27535,27567,27567,27534,27502,21196,8584,6503,6503,6503,6471,6471,6471,4390,4390,4390,4390,8616,4390,4390,2309,4358,4357,4358,4357,4325,4293,2245,2277,2244,2245,2245,4325,2277,4357,10663,29517,29550,29614,27534,31792,27470,29517,46550,44404,42292,42292,42325,21163,21196,12776,10728,12776,31760,31792,40243,54937,52824,198},
{34138,34138,34138,34138,34170,34170,32090,32057,32090,32089,32089,32057,32057,32057,32057,32057,34137,36217,34169,32056,32056,32056,32024,29976,29944,29976,29976,29976,29976,29976,32057,32089,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,34104,32056,29976,29976,29976,29975,32023,32023,32023,32023,32023,32023,32022,32023,32022,31991,31990,31990,31990,31990,31958,31957,31957,31925,27764,27699,25553,38100,27665,19246,23407,23375,25520,21294,23439,21261,21294,21261,33808,42292,29615,21293,19148,17099,12842,19147,38131,44405,21228,17099,21359,10761,12842,17035,14954,21260,25486,36019,36019,33873,8551,6503,40179,31565,21164,23341,42259,35920,44372,42292,42292,29615,29647,29712,17034,17001,23308,61309,42324,59228,63421,50744,42292,38066,42226,44307,46485,46453,40147,38099,35953,38066,42325,33906,33939,38132,36084,33938,40213,40212,38099,40147,38034,46517,16936,12743,14889,10697,17101,35987,40018,40211,38033,35953,35920,38001,38066,38066,33873,14889,8584,8584,8583,8584,8584,8584,17002,25454,27535,27502,25421,25454,25422,21229,27567,27567,23341,21228,23309,27535,29647,27567,27502,25454,21261,8584,8584,6503,6536,10762,6471,4390,4357,2309,2309,4390,4390,4390,2309,2244,2244,2244,2277,4325,4325,2244,4358,8584,6503,2244,2212,2212,2244,2276,4357,27436,29517,25388,23340,31792,33808,42260,46583,40244,40212,42325,42357,35986,31760,33906,31859,23374,23342,12809,25453,52856,52856,204},
{34138,34138,34138,34170,34170,34137,32089,32057,32058,32089,32057,32057,32057,32057,32057,32057,34137,36217,34137,32057,32057,32056,32024,32024,29976,29976,29976,29976,29976,29976,32057,34136,32056,34136,32088,32056,29976,32056,34136,32056,32056,32056,34104,34104,32056,29976,29975,32024,29975,32023,32024,32023,32023,34071,34071,32022,32022,32022,32023,31990,31990,31990,31958,31958,31957,31925,31925,29812,25619,23473,38100,23440,21359,23440,27600,27568,21326,25520,27633,23407,25454,31760,29647,25454,21261,25486,19180,17100,15020,31793,46519,27599,31825,14987,12842,6536,12810,21261,19147,23341,29648,29680,29615,6470,2310,21229,35727,23276,29647,40178,35920,50711,46518,46485,35888,27502,33841,19082,29614,38033,38066,40179,61309,63389,48631,40212,38099,42226,44339,50679,46485,44372,40244,38131,29615,31793,36052,40245,33906,38132,36019,35986,39985,33840,21130,31694,48598,21162,19050,16970,17036,10729,21229,31662,42259,38033,33840,33840,35953,31760,27534,27534,12809,8616,8616,6503,6503,6503,6503,12809,27567,31728,29680,27534,25422,23374,17035,19148,27534,27567,27534,23341,23341,25486,27534,27534,27567,21228,8584,6503,6471,12842,14923,4390,2277,2277,2310,4390,4390,4390,4390,2277,163,163,2211,2211,2211,2212,2212,10762,8649,10730,2212,4325,2212,2179,2244,4325,21162,27468,29549,21194,25421,27502,42292,42357,38164,40212,42357,42325,40179,35953,31760,29713,29713,17035,12777,23341,52856,52824,204},
{34138,34138,34138,34170,34170,34137,32089,32057,32089,32057,32057,32057,32057,32057,32057,32057,34137,36217,34169,34105,32056,32024,32024,32024,29976,29976,29976,32057,29976,30009,32056,34137,34105,34137,32056,32056,29976,32056,34104,34104,32056,32056,34104,34104,32056,29976,29976,32024,32023,32024,32024,32023,34103,34071,32023,32022,32022,32022,31990,31990,31990,31990,31958,31957,31957,31957,31925,27764,23506,25553,42325,31858,23472,25520,31729,25520,19213,27600,23407,21261,23373,19148,19115,19147,19148,17035,21261,17068,12908,27633,38132,35986,31793,19148,17068,6503,10729,31794,14922,27535,29615,29680,27534,6471,2310,12842,23374,17003,31760,34002,23567,42454,46259,44017,22985,18856,18921,12647,38001,38033,40146,38098,46485,46518,42292,29615,46420,52759,48533,50711,48565,46420,42260,31760,38132,40213,36019,38132,40245,36051,40212,46551,46389,42032,35921,35953,42291,29581,19050,14923,10729,8616,12777,29615,38033,38033,33840,29615,27502,25421,27502,29647,12777,8584,6503,6503,6503,6503,8616,10697,27567,29680,29647,27535,23341,25454,17035,12809,25454,23341,25454,25422,19115,21261,25454,29615,29647,25389,14890,8583,8551,10697,8584,4390,6438,4357,4390,4390,2277,4357,4325,2277,2244,131,2179,2178,2179,2211,2212,8616,8584,6471,2211,2211,2211,2179,2212,6405,19050,29580,16999,21193,29646,29679,42260,31792,38131,40212,42324,42324,42260,40146,35953,25520,27600,14922,12776,23308,54937,54937,206},
{34138,34138,34138,34170,34170,34138,32090,32057,32057,32089,32057,32057,32057,32057,32057,32057,34137,36217,34169,34137,32056,32056,32024,32024,29976,29976,29976,32056,29976,29976,32056,34137,34137,32056,32056,32056,32056,32056,32056,34104,32056,32056,34104,34104,32056,32056,29976,29975,32023,32023,32023,32023,34071,34071,34071,32023,34070,32022,31990,32022,31990,31990,31957,31957,31957,31957,31957,27764,23506,27699,44438,36052,23408,23407,35987,23406,19181,27601,29713,27632,36051,21292,21228,19147,19115,19115,23373,17100,15021,29714,38067,31663,25487,21261,29680,8616,12810,36052,19148,27534,33841,29648,25454,6471,2310,14955,23407,21261,38098,23631,11310,30003,46324,31630,16743,16776,18921,16872,42227,40146,40178,35953,42292,33873,35920,48500,50450,50515,50450,50482,50483,48402,48402,44208,48531,40177,38097,40179,33841,35986,31760,25421,33873,31695,27534,35953,29614,29646,31695,10696,6535,8584,8616,19116,33841,27534,25421,25421,25422,27534,27534,27534,10664,8584,6535,6536,8584,6536,6535,6503,21229,29647,27567,27502,25454,25486,19116,8648,8616,8584,19148,25421,23308,23309,23309,29615,35953,44405,44340,33743,33743,33743,14825,17003,35953,29550,27437,8551,21228,19115,19083,10664,6438,8583,14922,16970,8518,6405,8583,14889,12777,6405,4357,4325,4324,4292,4292,4358,19082,25386,25418,25419,31694,42259,46518,27534,40244,40212,40244,42324,42259,42291,38066,23374,23407,14955,10664,23340,54969,54969,206},
{32090,34137,34138,34138,34170,34138,32090,32057,32057,32057,32057,32057,32089,32057,32057,32057,34137,36217,34137,34105,32024,32024,32024,32024,29976,29976,30008,29976,30008,29976,30008,34137,34105,32056,32056,32056,32056,32056,34104,34104,32056,32056,34104,32056,32056,32024,29976,29976,29975,32023,32056,32023,32023,34071,32023,32022,32022,32022,32022,32022,31990,31990,31958,31957,31957,31957,31957,27764,21426,27666,40212,31826,21359,25488,36019,21326,21326,29713,31794,38100,33971,23407,29745,23374,23342,23342,23341,25454,17069,29681,38099,38034,27600,21229,33841,29713,27535,40245,19148,29680,27567,29681,29647,4423,2310,14988,19149,27535,42258,21616,7149,36245,52727,46098,43791,43823,45806,48016,48500,40081,38033,35952,29614,40017,52596,52661,50483,54741,50547,54806,52628,54805,52756,50673,42225,33840,21163,35985,31760,33840,36018,42325,40180,38066,25389,33840,31727,35985,29615,6503,6503,6536,8616,10696,23374,25421,25422,25454,25422,25422,25454,27535,10696,8584,6536,6503,27666,10697,6503,6471,14922,27567,29647,27502,25421,25454,21261,8616,8616,8584,17035,27567,27534,31728,42292,50712,52856,52824,50678,48468,52693,52693,42227,48664,46518,48499,50612,31793,21293,17035,19083,17003,14955,12810,16970,17003,14922,12809,10696,17003,17002,16970,17003,14955,14923,12842,10697,8551,19050,23274,21191,23338,16968,40178,40211,38099,31760,38131,40211,42291,42292,44372,44339,25486,31825,31825,36051,40211,54970,54969,210},
{32090,34138,34170,34138,34170,34138,32089,32057,32089,32057,32057,34137,32089,32057,32057,32057,34137,36217,34169,32056,32024,32024,32024,32024,29976,29976,29976,30008,30008,29976,30008,34137,32057,32056,32056,32056,32024,32056,34104,34104,34104,32056,34104,32056,32056,32024,29976,29976,32023,32023,32023,32023,32055,34071,32023,32022,32022,34070,32022,32022,31990,31990,31957,31957,31957,31957,31957,27764,21426,27699,42324,33971,21295,27568,31761,27600,25519,31794,38100,44373,33907,12876,19215,19181,25520,23342,25453,27598,10763,14956,31761,35953,40245,25455,35953,10697,23276,40245,19148,27535,23374,31761,27534,8616,6471,14955,19181,33808,44339,27663,32051,48728,54937,46194,45839,48243,45774,48145,50516,42226,46484,38032,40179,42194,48565,50711,50645,56984,59097,48532,46419,37967,35918,44338,27566,33841,25453,29679,38099,38066,40212,42324,40114,40017,29517,27469,33808,33808,23276,8584,6536,6536,6536,8616,23374,25422,25422,25422,25421,25422,27535,27599,10696,6503,6503,6471,12810,8616,6535,6536,10697,23342,27535,25422,25422,25454,21261,8584,8616,8584,12809,29679,46518,54970,57050,52856,54904,50710,52759,52693,59000,56951,29582,21260,37969,52693,52693,25454,12874,14922,14922,12777,6503,6503,12809,16970,12745,8584,10696,14890,17035,14889,14955,12842,12843,12875,12875,10730,6438,8582,23337,16967,12774,44469,40211,40179,27502,38131,40212,42292,42324,44404,50645,29647,38131,40276,40244,35985,54970,54937,208},
{34138,34138,34170,34138,34138,34138,32089,32057,32057,32057,32057,32057,32089,32057,32057,32057,34137,34169,34169,32056,32024,32024,32024,32024,29976,29976,29976,30008,29976,29977,32056,32088,32089,34137,34104,32056,32056,32056,34104,34104,34104,34104,34104,34104,32056,32056,29976,29976,32023,32023,32056,32055,32055,34071,34071,32023,34070,34070,32022,32022,31990,31990,31958,31957,31957,31957,31925,29813,21426,29779,42325,27633,21359,27600,27600,27568,31858,31859,25487,40180,42260,21327,15022,23441,25552,21261,44405,27600,12811,25455,33841,33840,38099,17035,36018,8616,17035,42293,19148,27567,23374,27535,25454,8584,2310,10729,19116,40179,37968,40146,54904,52856,54969,46065,45806,45936,45774,45839,52597,44404,33839,29614,42259,38001,42324,52825,50711,52825,61309,42325,42387,12871,25485,29646,23308,29647,19115,25487,31792,38066,31792,29647,35921,39985,42000,33808,38034,35953,19148,6504,6536,6536,6536,8584,19148,25421,25422,25454,27534,27535,27535,25454,8616,6503,6503,6503,6536,10729,6503,6471,6471,21229,25422,27502,29647,29680,21261,8616,8616,6503,14955,35987,59229,61309,48630,50678,48630,44340,52791,48500,50645,46420,19212,21357,48565,52725,38033,10794,19147,12777,8615,4422,2277,4390,8583,8615,4357,2244,6470,10663,10663,10663,8615,8583,6502,4357,6438,4325,10695,12807,17000,12806,19113,46517,48630,44340,40179,27534,40244,40244,42292,46484,50677,38131,21260,38099,29711,38066,54969,54936,208},
{34138,34170,34170,34138,34138,32089,32057,32089,32057,32057,32089,32089,34137,32057,32057,32057,34137,34169,34137,32056,32056,32024,32024,32024,29976,32056,32056,30008,29976,29976,32057,34137,34136,34136,34104,32056,32056,32056,34104,34104,34104,34104,34104,34104,32056,32024,29976,29976,32023,32023,32056,32055,32055,32023,34071,32022,34070,32022,32022,34070,34038,34038,34038,31958,31957,31957,31925,29845,21426,29747,44406,23439,23440,23439,23407,23374,31859,29746,21327,25487,42260,40180,15021,10828,23407,12811,21294,19182,19182,29680,38067,29582,25487,21294,27535,6503,12777,42325,19148,29680,23309,29680,25422,8616,2310,10697,25454,42227,35920,46485,50743,52856,52857,41936,45838,48178,45774,46033,54808,48566,21226,14921,33905,35953,40211,50808,50711,54970,35954,38131,40307,12871,14888,14855,10662,25453,23309,29614,38034,31727,31760,29615,31728,35954,38002,35954,31728,27567,17003,6536,6536,6504,6504,6536,17035,27599,29680,29615,25454,25454,25422,25454,10696,6536,6503,4455,4423,12809,4423,10729,6471,27600,29615,33808,35954,31792,23373,8616,10696,8616,8616,27600,48664,59196,50678,50678,29648,25455,40180,23307,25485,19212,21455,25550,27598,21259,15084,8713,4325,2211,2276,2309,2277,164,2212,2276,2277,2244,2212,2244,6470,6502,6502,6502,4357,4390,6470,6437,17064,6500,10726,10661,27533,42291,54937,54969,40147,27534,38098,40211,42292,46485,50710,38066,33873,31760,10630,36018,52889,52856,208},
{34138,34138,34138,34137,34137,32089,32057,32089,32057,32089,32057,32089,34137,32057,32057,32057,34137,34169,34137,32056,32056,32024,32024,32024,29976,32056,32056,30008,29976,29977,32056,32089,34137,34136,34136,34104,32056,32056,34104,34104,34104,32056,32056,32056,32056,32056,29976,29975,32023,32024,34104,32055,32055,34071,34071,32023,34070,32023,32022,32022,34038,34038,31958,31989,31957,31957,31925,29877,21458,34005,42358,23374,21326,21359,21294,23375,29746,29714,25553,14956,29713,40147,35987,10796,23440,12876,17102,14988,12875,25422,38034,29615,25455,21261,31761,21196,21228,42325,27567,29680,27568,31728,25422,10664,4390,25455,35953,40114,33775,50743,50743,50711,54872,45968,47886,47854,45774,45806,54776,46550,12808,8647,12841,38099,44406,46518,46550,50744,38067,33938,34066,21450,17031,19113,23275,27502,35921,35921,38066,31727,29615,29615,27567,27567,27534,25454,25454,25454,14922,6536,8617,6504,6536,8616,14922,21227,25419,16969,19083,21228,12777,14922,10729,8616,8616,6568,8616,10729,14955,17068,10697,21196,23276,33711,38002,31728,23374,14955,25520,29745,33939,40245,46519,59196,50678,40179,23342,21261,29648,23341,17002,15116,25616,21325,21259,14889,8681,6568,4357,4324,4357,4390,2309,2244,2244,6438,4390,2244,2212,2276,6502,8583,8550,8550,6438,4325,6438,6502,17032,17031,19176,16967,38065,40146,54937,54937,40179,42259,25356,40211,40211,46484,50677,38066,57116,50745,35986,38099,54969,54937,209},
{34138,34138,34138,34138,34138,34137,32089,32090,32089,32057,32057,32089,32089,32057,32057,32057,34137,34137,34137,32024,32024,32024,32024,32024,29976,32056,32056,32056,29976,30009,32057,34137,34136,34104,34136,34104,32056,34104,32056,32056,32056,32056,32056,32056,32056,32056,32056,29976,32023,32023,32056,32023,34103,34071,32023,32022,34071,32022,32022,32022,31990,34038,31958,31957,31957,34005,31957,29877,23506,31892,44438,40245,29713,21294,19246,21327,31826,25553,21295,23408,19214,27503,40179,31761,25520,12876,21294,25455,17069,33842,38099,29680,25520,21293,31760,17003,21229,42326,31793,27600,29648,31728,27535,12777,6471,23308,42259,33807,40146,52823,50743,48566,50387,45870,45838,43693,43758,43758,50615,31793,12808,6502,10695,10695,33938,31793,35953,48630,59163,36018,23405,21481,27787,19080,16968,27469,31663,29647,35954,27566,27534,27535,27567,25454,25422,25454,25454,25486,17035,6504,8617,8617,14953,23337,23306,10726,12807,4356,6405,12744,6438,8584,8616,6503,6503,6471,8583,6470,4358,2277,12809,29615,19050,27436,27469,25454,17003,6471,4422,10729,12842,33938,52858,59196,52792,33809,21262,21229,27536,23341,14922,12970,19244,27599,19115,10631,6470,6470,4357,4357,4357,4422,4422,2277,6470,8615,6438,2309,4325,4357,8615,10728,12776,12776,8550,4325,6470,6502,14953,12806,19144,12806,44404,42292,59162,44405,40146,40179,29582,33873,40244,46484,50645,33873,33971,36019,35986,36018,54937,54937,206},
{34138,34138,34138,34138,34138,34138,32090,32089,32057,32057,32057,34137,34137,32089,32057,32057,34137,34137,34105,32024,32024,32056,32024,32024,29976,32056,32056,32056,30008,29976,32057,34137,34137,34137,34136,34104,32056,32056,32056,32056,32056,32056,32088,32056,32056,32056,32056,32024,32023,32023,32023,32023,34103,34071,32023,32023,32022,31990,32022,31990,31990,31958,31958,31957,34005,31957,31957,29845,21458,29811,42325,38132,25520,19213,21294,19214,29746,25488,25521,23440,17101,35987,40180,38099,27568,14923,27535,23310,17069,31760,40115,29648,42391,27567,38164,19181,14923,8616,23342,27535,31793,33873,25422,8616,6503,35953,42226,33775,46485,50678,48598,46420,63194,61082,46323,23082,18986,46422,29682,23310,10663,6502,8647,10727,29712,17067,29615,42259,48598,33872,29678,25452,27596,23274,23307,38033,38034,27567,31760,27567,27534,27502,27535,25454,25454,25454,25454,27567,17036,6536,8584,12840,8613,19112,17000,8582,4324,4356,4324,6438,6471,6471,6471,6470,6470,4390,4358,4357,4389,6470,8518,14920,12775,10695,6437,8550,12776,10729,17068,10762,36052,42390,52923,55003,35954,21262,19181,19149,23342,23276,14921,14922,14921,14888,12711,8517,8518,8583,6437,4357,4357,6535,6536,4390,8583,10728,10696,8648,4357,4357,8582,12808,12776,12744,8550,4324,6438,6470,8615,12807,31757,25419,44405,46518,57050,48598,38098,38098,40179,19082,38131,46484,48532,38131,40277,29680,35986,35986,52889,52856,202},
{34170,34138,34138,34138,34138,34137,34138,32089,32089,32089,32057,34137,32057,32057,32057,32057,34137,34137,34105,32024,32057,32056,32056,32024,32024,32056,32056,32056,30008,29977,32056,34137,34137,34136,34136,34104,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32055,32023,32056,34104,34071,34103,34071,34071,32023,34071,32022,32022,34038,34038,31990,31958,31957,34005,34005,31957,27764,21425,29811,29713,31793,23375,19246,19246,21359,29778,21456,19570,23440,19182,33874,40147,35954,29648,27535,31728,33809,19181,31761,38066,27502,29746,23342,27632,4423,14988,16971,23309,25390,33873,36019,27567,10729,12810,40146,33807,38033,50711,48598,38066,50646,61114,59033,46453,31728,17004,23343,17169,12940,6566,8646,4453,6502,25486,12808,29680,46518,57083,46486,40212,44437,38066,23308,35985,38131,40245,36051,27599,27535,25422,25389,27567,29647,27567,29680,29680,29680,12842,6503,10727,17000,12839,16968,4356,4324,4357,6438,4357,6438,6470,6470,6438,4390,4390,4358,6470,4357,6502,12808,12807,10694,8582,4324,4324,4357,4389,6471,6503,8648,40277,38131,44470,27632,23342,19148,17068,17036,21197,23342,10663,10696,14954,8550,8550,8485,6437,8550,6470,4357,4357,4422,6535,4422,2309,12841,6503,6503,4422,6437,8550,14889,12809,12777,12744,8550,8551,6470,8550,10728,35986,44372,38066,35985,50711,42259,38098,38066,38130,21163,23308,46452,42258,31793,27567,27599,33873,35953,54937,52824,205},
{34170,34138,34138,34170,34138,34138,34137,32090,32090,32089,32057,32089,32057,32057,32057,32057,34137,34137,34105,32024,32056,32024,32024,29976,29976,32056,32057,32056,30009,30009,32057,34137,34137,34136,34136,34136,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32056,32024,32056,32056,34104,32056,34103,34103,34071,34071,34071,34070,34070,34070,34038,34038,34038,34005,31957,31989,31957,29845,25586,25618,29713,31794,23375,23439,23439,27698,19246,15053,19343,23440,25521,21294,35987,27535,27535,25455,35954,42260,25455,35986,40114,27503,23472,21294,36019,17068,17035,31761,33841,25390,38034,38132,25486,4390,2310,19115,33839,42226,50743,46484,35920,61081,61146,52726,48566,38099,44406,33875,15219,12939,8646,8678,6566,8614,10727,14954,25421,35985,42292,42291,36017,27632,19148,25454,21228,23374,25454,14889,27567,29648,25454,27503,25422,40212,33841,25454,25454,29648,12842,12904,12839,17001,6469,2276,4325,8551,19083,8583,6471,4422,6470,4390,4357,4357,6438,2244,6437,23081,18985,19017,17033,17001,14889,21195,23308,23341,23341,19115,14922,17002,42390,31793,27664,38292,31922,21261,19149,17068,21261,23341,8583,6502,12841,8583,6405,6405,4356,4357,4357,4357,2276,4389,8649,8616,6503,4357,2277,4422,4422,4389,4356,12808,10631,10696,6437,6405,6405,6437,6502,8583,25421,46550,38099,31728,35954,40179,38098,38066,35953,16970,16970,42226,29582,16938,14922,17035,33906,35986,52857,52824,204},
{32090,32090,34138,34170,34170,34137,34137,32090,32089,32089,32057,32089,32089,32057,32057,32057,34137,34137,32057,32056,32024,32024,32056,32024,32024,32057,32089,32056,30008,30009,32056,34137,34137,34136,34136,34136,32024,32056,32056,32056,34104,32056,34104,32056,32056,32056,32056,32024,32056,34104,34104,34104,34103,34103,34071,34071,34071,34070,34070,34070,34038,34038,31958,34006,31957,31957,31957,29812,23474,29811,35987,27601,23407,21294,23439,34004,21359,25586,12973,19345,27667,21294,21261,31728,31761,29648,40212,40147,23342,33841,38066,19148,21295,19149,33939,35986,31760,29615,31761,23342,35953,31728,27535,21196,12810,12809,38033,48565,46485,42193,38000,61081,56920,48533,48566,40179,44405,29682,13138,12873,6598,8710,8646,8647,8614,12775,10695,16969,27567,33905,14922,19213,12874,17100,27535,10696,23374,21261,27567,31793,17035,10697,14923,23374,27534,23341,23342,25486,14922,8648,14952,4389,4357,2277,6503,10729,25454,10696,6503,6470,6471,4390,2277,2244,2244,2244,4357,33516,18952,25290,25420,36018,40277,50777,50809,61310,59262,57148,61342,59261,40277,36051,36179,40468,40436,25485,25454,14890,14890,19115,12808,6470,42325,29582,29549,27501,16937,4389,29680,25324,23276,8518,8615,35954,19018,19018,2309,6535,25454,14856,14824,12776,10663,21196,14824,10598,4324,4357,6470,6503,14922,42324,59196,29614,38066,40179,40179,38066,44405,25388,14856,25388,19082,16970,19115,17003,25454,40211,52857,52856,206},
{32090,32090,34138,34170,34170,34138,32089,32089,32057,32057,32057,32057,32089,32057,32057,34137,34137,34137,32057,32056,32024,32056,32024,29976,29976,34105,34137,32056,30008,30008,32056,34104,34137,34136,34136,34104,32024,32056,32056,32056,34104,34104,34104,32056,32056,32056,32056,29976,32056,32056,34104,34103,34103,34103,34071,34070,34070,32022,34070,31990,31990,31990,31958,34005,31957,31957,31925,27732,23506,40278,44406,27633,21358,21326,27568,21326,17101,15053,21360,15151,27634,17069,19181,27502,31728,23342,42260,42292,31761,23341,35953,21261,29713,25488,36052,27534,35954,33808,35921,23342,33808,31728,27534,21196,14890,12809,40146,50710,46452,35822,54807,61049,54807,44372,46485,35953,46453,25489,13074,10792,8678,8678,8646,8615,10662,10662,10662,19082,29679,19147,8583,10761,8616,33939,44438,21228,29712,35986,35986,31728,14955,8616,10762,21293,25486,25454,19148,29648,14923,8584,8584,8616,23471,32086,30102,25714,32086,36311,34069,17035,12874,10826,8681,6535,4389,4325,4325,29550,25226,33644,27501,33905,40244,52889,50744,44438,55035,59229,55003,54971,40277,31825,44662,40468,38323,31888,23309,29681,31826,33906,27567,23373,46518,48468,44242,48435,35856,12809,21228,46355,46322,25324,10664,25421,44274,44274,16937,8550,31727,42161,44275,16904,8550,33807,40048,40081,8518,4357,6470,6502,14889,44437,65503,46486,31695,40179,40179,38034,48598,40147,16969,19115,23276,14889,17035,17068,17068,33938,52857,52856,205},
{32090,32090,34138,34138,34170,34137,32089,32089,32058,32057,32057,32057,32089,32057,34137,34137,34137,34137,32024,29976,32024,32024,32024,29976,32024,34104,34105,32056,30008,30008,32057,34104,34136,34137,34136,34104,32024,32056,32056,32056,32056,34104,34104,32056,32056,32056,32056,29976,32056,32056,34104,34103,34103,34103,34071,34070,34070,32022,34070,31990,31990,31990,31958,31957,31957,31957,31957,29812,23506,38165,52857,29713,27601,23407,25487,21262,17101,15020,15020,21489,23407,19182,19181,21196,29615,19148,40179,44340,31728,29647,35921,27567,36051,38132,38099,25389,27535,27535,35986,27567,27534,29648,27535,23308,19083,12809,44405,50678,44306,29516,56855,54775,40049,44372,48566,40146,44340,21361,15219,10825,8646,8646,8679,12873,10728,6469,8615,19115,25389,23308,8584,8648,6536,33971,25421,31792,19116,19148,29680,27534,14955,6536,10697,17068,25487,27535,17035,27567,12777,10730,29746,46583,42389,34133,32247,21520,23601,32248,30037,4519,6567,6502,8648,12809,8583,10696,6470,8550,10599,29419,33873,31825,40212,48696,46551,36018,52923,59229,46519,44470,42358,36083,44630,40436,38323,38259,21195,33807,19146,14888,38066,19115,44437,52759,48500,52661,44242,33938,48697,48532,50580,37968,42325,52825,48500,50580,35856,48697,52824,50548,50612,33808,50745,52792,48499,50580,31695,46519,44438,42326,42325,46518,65535,38099,27501,38099,35986,38098,46518,40179,27534,14889,25421,21195,12777,14922,17035,31793,52824,52856,205},
{34138,32090,32090,34138,34170,34138,32090,32057,32057,32089,32057,32057,32057,32089,34105,34137,34137,34105,32056,32024,32024,32056,32024,29976,32024,34105,34105,32056,29976,30009,32057,34104,34136,34136,34136,34104,32024,32024,32056,32056,32056,34104,34104,32056,32056,32056,32056,32024,32056,34104,34104,34104,34103,34103,34103,34070,34071,34070,34070,31990,31990,31958,31958,31989,31957,31957,31957,27732,21426,29747,38132,27633,25488,25455,25422,19213,15020,14988,15020,12907,23439,23343,17036,21229,23342,17035,40179,44340,35986,29647,33840,31826,36019,40277,27600,31826,25455,17003,31760,29647,23308,33874,25422,19115,19116,27600,19147,25422,29680,21098,54743,35792,27470,44340,48598,38066,42259,21393,17332,12874,10792,19147,6536,19147,10760,8615,8582,17002,29647,19115,8616,6568,8648,12842,19148,33873,17003,12810,25486,25454,14890,6536,8649,14922,27534,29647,19148,25487,36084,46648,42389,40276,40243,34198,34360,21488,21520,32247,30037,4422,4389,2276,4357,6470,196,6470,6406,4357,6405,29388,38002,38099,36019,48696,46551,42325,55003,40180,42325,46551,40180,29679,44598,40436,38355,42485,25421,29582,25453,23308,35985,38066,42292,50646,48435,52660,42096,6470,10696,50613,52660,33645,6437,16937,50612,50580,27404,8518,25388,48467,50580,25291,10663,29582,50548,50580,23211,21196,23309,23341,38066,52856,63422,25422,31760,27535,35986,42324,42357,36050,38099,19114,19115,23341,17002,33841,40212,38099,52856,52856,203},
{34170,32089,34170,34138,34138,34138,32058,32057,32057,32057,32057,32089,34137,32089,34137,34105,34105,32056,32024,29976,32024,32024,32024,32024,32024,34104,32056,32056,29976,30009,32057,34104,34137,34136,34136,34104,32056,32056,32056,32056,34104,34104,34104,34136,32056,32056,32056,32024,32056,34104,32055,34104,34103,34071,34071,34070,34071,34070,34070,32022,31990,31990,34037,31957,31957,31957,31957,25651,23506,23473,23408,25520,23374,27567,27600,19181,17101,15020,12907,19213,12907,19181,14956,8649,21294,10763,25487,35953,27502,29680,38099,17068,38132,38165,29779,29779,27601,19115,33841,33841,19083,25455,31760,29648,12842,23439,14987,19148,19213,27502,35825,19052,19051,35954,44373,48599,42357,19248,19314,25422,31760,25487,8649,19181,19180,8647,6502,14922,16971,8616,8584,12842,8649,8616,14923,21196,12810,8583,25454,25454,14890,8616,8649,14923,27567,27535,17035,27600,55003,42292,42357,40276,40212,36278,34360,21488,21520,32280,30070,12971,10760,6567,17099,14922,163,8583,6470,6437,6405,6470,27600,40212,36019,44471,59229,61342,59261,27534,44470,46584,40147,25454,42453,40403,40435,44565,29646,42259,50777,50810,50712,50711,42292,50613,44274,48500,33742,8615,12841,50612,50580,33677,12809,19147,50645,50580,29484,8583,23308,52693,52661,25324,8518,23243,50548,52661,23146,8518,6470,8518,38066,59195,57051,46519,50744,29647,35953,46550,40212,33938,33938,38066,38000,42258,48598,42259,35920,42292,50776,52824,203},
{34138,34138,34138,34138,34138,34138,32090,32058,32057,32058,32057,34137,34137,32089,34137,34105,34137,32024,29976,32024,32024,32056,32024,32024,32024,34104,32056,32024,30008,30008,32057,34104,34137,34136,34136,34136,32056,32056,34104,32056,32056,34104,34104,34104,32056,32056,32056,32023,32056,32056,32056,34103,34103,34103,34103,34070,34071,34070,34070,32022,31990,34038,31989,31957,31957,31957,31925,25619,21426,25586,21360,23440,23374,27535,27535,19213,17069,17069,12940,17133,10827,12907,19149,10763,23374,12876,17101,31827,38099,23375,31794,27601,40277,36052,27666,29747,25521,21228,29712,33841,16970,17003,36019,38099,19115,17100,14987,12874,14955,12809,38002,16938,19116,33873,29647,44341,33873,14988,14988,19116,33906,19148,10698,21262,12874,8648,8648,14858,12744,12744,8583,6536,6504,12810,19116,17003,8584,6503,17067,23373,12809,8584,6503,6503,14922,17002,21228,17035,48728,40211,21261,27632,40211,36278,36441,21520,21553,34328,34360,36441,34328,34295,12874,10761,4389,12809,12874,10728,14954,8615,31858,42324,33873,52890,59229,59261,61310,31728,42325,46551,40212,25421,38195,40436,42517,42453,33936,42227,29647,31792,42324,48630,44372,52726,48565,29807,23437,15020,19278,50613,52660,35823,19343,25551,50613,52661,31630,14988,29744,52661,52660,44241,37903,40016,50547,50580,40016,31630,31629,31662,42292,63422,48632,23309,35953,42356,38066,46518,38099,33906,31825,50710,54902,52757,54936,65503,52824,48631,57050,57083,223},
{36218,34170,34138,34138,34138,32090,32057,32057,32058,32057,32089,34137,32057,34105,34137,34137,34105,32056,29976,29976,32024,32057,32024,32024,32024,34105,32056,32056,30008,29976,32057,34104,34137,34104,34104,34104,34104,34104,34104,34104,32056,34104,34136,32056,32056,32056,32056,32024,32056,32056,32055,34103,34103,34103,32054,34071,34070,34070,34070,31990,31990,34038,31989,31957,31957,31957,31925,25619,21426,25618,21360,21327,25455,31793,25454,21326,17101,17100,12907,10827,10795,8748,25488,15021,21262,6570,12843,17134,17101,27600,31793,29746,42423,25520,12810,12810,14955,23341,31793,42292,27534,29712,31760,21228,27567,33939,29680,27535,31793,25454,48469,19051,14890,10696,16970,33712,29681,17069,8649,6536,23374,21262,12842,23342,10729,10697,19050,14792,14759,12711,10566,6470,6503,17003,29713,38132,36084,36052,36084,38132,36019,33906,31826,31793,31825,33873,33906,29713,50809,35986,12875,21326,40244,36278,36441,21553,23634,34360,30070,15149,34328,32215,10826,8615,2276,8551,6502,4357,8583,6502,27599,42357,31792,52890,57116,59229,57148,29615,40277,44470,35986,29647,36083,42452,42453,42421,36082,48435,16969,19081,29614,42259,48598,52693,48500,33968,29646,15052,19277,52693,52725,35855,17230,25551,50612,52661,31630,15020,29744,50580,52628,52628,52661,52693,50580,50580,50548,40115,50580,48565,46485,65502,38067,27534,35985,42324,38098,44405,38099,33905,33905,56984,50677,57016,59130,65535,48598,52857,59195,61276,237},
{34170,34170,34138,34138,34138,32090,32090,32057,32089,32057,32089,32057,34105,34105,34137,34137,34105,32024,32024,32024,32056,32056,32056,32024,32024,34104,34104,29976,29976,29976,32057,34104,34137,32056,34104,34104,34104,32056,34104,34104,32056,34104,34104,32056,32056,32056,32056,32056,32056,32056,34104,34103,34103,34103,34102,34103,34070,34070,34070,31990,31990,31990,31989,31957,31957,31957,31925,29812,23538,23537,23440,19246,23407,27568,23439,23407,29810,27698,19246,12907,12875,6667,21492,12908,19181,2377,6569,6569,4457,14988,31761,27568,44503,27600,12810,8617,12810,21261,27600,38066,29647,33906,31825,25454,12809,25519,17068,14921,14922,6470,19082,16969,12808,10663,19148,33777,16971,6537,6569,6536,21261,25455,10664,8583,8584,12777,16903,21129,23209,21064,12711,6405,31793,46518,36051,42390,42390,42423,44471,42390,42390,42358,40310,40309,42358,42390,42358,40277,50842,35986,12875,14987,40277,38359,36473,23601,23601,34361,32118,15117,34328,32280,15117,14987,6535,15019,19245,4421,10759,12808,27567,44437,38099,48697,42358,21228,55036,27534,40212,44438,42325,25421,33905,42389,42357,42389,38162,52693,16936,27566,38098,38066,52791,52693,38129,40243,38000,23569,25617,52693,52660,37968,38359,40309,52693,52661,33743,25584,31824,52661,52661,33742,29776,38098,52660,52693,52825,44374,25356,38033,50743,63389,27470,33840,38098,40244,50711,42357,38099,33905,40179,57016,50645,59129,61276,65503,50679,52857,61276,61276,236},
{34138,34170,32090,32090,32090,32090,32090,32058,32057,32057,32057,34105,34105,34137,34137,34137,32057,32024,32024,32024,32024,32056,32056,29976,32056,32056,32024,32024,29976,30009,34105,34137,34137,32056,34104,34104,34104,34104,34136,34104,34104,34104,34136,34104,32056,32056,32056,32023,32056,34104,34104,34103,34103,34103,34070,34070,34070,34070,34070,31990,31990,31990,31958,31957,31957,31957,31925,29844,38230,42358,29714,25553,21294,23407,21294,19214,17101,15020,10827,19213,12907,6667,21492,23343,25520,12875,12875,10763,10795,19246,27633,27567,46551,33906,29680,19115,10729,21229,27535,35953,31793,36052,46583,38099,12841,25519,17099,8615,4390,196,6502,17001,12808,12840,12808,17003,29616,8617,14923,12875,25487,25519,8518,16937,18953,21065,16936,21096,19016,21129,10598,14889,44372,44372,29746,44470,42358,25486,21196,19115,19148,19180,21228,23341,23341,23373,25422,25519,48728,35986,12875,10762,38164,38327,36473,21553,21553,36441,32150,13004,36409,34328,36409,34328,32247,23601,10760,19177,19178,46648,42358,44437,33938,42391,42358,40278,55035,33874,40245,44438,40212,23308,33905,42357,40244,40212,40244,54806,40050,35986,50712,46486,52791,52693,48565,36081,29678,23504,23536,52726,52661,37968,36279,36148,52726,52661,33743,17067,29679,52661,52661,33775,27729,33937,52693,50612,52825,42357,16969,38098,57050,57051,21164,25389,40211,44405,46485,40179,38067,35954,44371,59097,46451,59130,50711,57083,48631,57050,61276,61276,236},
{34138,34138,32090,32090,32090,32090,32090,32090,32058,34105,32057,34137,34105,34137,34137,34137,32057,32024,32024,32024,32024,32024,32024,32024,32056,32056,32056,32024,29976,29976,34137,34137,34137,32056,32056,34104,34104,34104,34104,34104,34104,34104,34104,34104,32056,32056,32056,32056,32056,34104,34104,34103,34103,32023,32023,32022,34070,34070,32022,34038,34038,31958,31957,31957,31957,31957,31925,27699,40278,46551,27666,23440,21294,21327,21326,17133,17101,15053,10795,25520,17100,11022,23504,25390,27568,17068,21262,12875,10795,17069,10763,19182,42391,33939,23375,25487,21229,23374,25455,33841,27600,21229,52922,38099,12841,23406,10760,6534,6534,4422,4390,12808,12840,12840,10663,21228,29648,6536,8617,6536,21261,8518,10598,27404,10598,18984,16903,19016,19015,23209,14758,31727,48597,40179,27633,46519,42390,17034,8518,4357,4358,4357,2277,4389,6438,4325,6437,8583,46648,36050,10761,8681,29745,38359,36473,23601,23634,36441,32150,10923,36441,36409,15181,30037,32280,25714,12905,17096,19178,38098,44470,42324,33906,36052,36052,40278,48664,33873,40212,42358,42293,25421,29647,36051,42357,44470,48566,56919,56856,56857,52728,38033,52823,52726,48532,31920,29711,15052,19278,52726,54773,37969,25682,29809,52693,52693,44210,31630,33742,52693,52693,40016,25388,31662,52693,52660,54905,48631,23243,42259,63389,48599,16938,12776,42324,50744,33873,31793,36019,35953,50742,48597,54936,52791,57083,50711,52824,61276,61308,61308,236},
{32090,34138,32090,32090,32090,32090,32090,32057,32057,32057,32057,34137,34105,34137,34169,34137,32057,32024,32024,32024,32056,32056,32024,32024,32056,32056,32056,29976,29976,32057,34137,34137,34137,32056,34104,34136,34104,34104,34104,34104,34104,34104,34104,34104,32056,32056,32056,32056,32056,34104,34104,34103,34103,34103,32022,34070,34070,32022,32022,34038,34038,31958,31957,31957,31957,31957,31925,25586,29779,44439,25553,31859,23440,21359,17133,15053,15020,15053,23439,25520,10826,4554,12940,8682,23375,19181,23375,25455,21261,17069,8650,19214,31958,31958,21327,17100,21229,33938,36051,31728,23374,33841,38099,31728,12841,17067,4454,6567,6534,4454,4422,6470,12808,8615,6502,14890,14923,6536,6536,6536,10697,6470,19017,23242,6469,16936,21096,19015,21129,27500,21195,33807,48630,38099,33907,44438,42390,16970,6470,4357,4389,4357,2276,2277,4357,2244,6437,10696,33938,29647,6503,6536,23374,38359,38521,29940,30037,36441,34263,25714,36409,36441,23601,32118,34361,27892,17229,17162,14985,31727,38099,35986,33906,36084,31826,40277,44471,44438,46551,42325,46519,46519,31760,40244,31825,52890,50678,56951,56889,56889,59066,50679,52791,52694,50580,46452,38033,10859,15084,50645,52693,38000,31988,31890,52660,52693,52693,52725,52693,52692,52693,52660,52660,52692,52660,52660,50579,52628,50613,46485,65502,36019,14857,14857,38131,57083,50744,21195,35986,38065,57016,46484,59097,48565,63422,46485,54937,61308,61308,61276,234},
{34138,34138,34106,32090,32090,32058,32090,32089,32057,32057,32057,34137,34137,34137,34137,34137,34105,32024,32024,32024,32024,32056,32024,32024,32056,32056,32024,29976,29976,32057,32089,34137,34137,34104,34104,34136,34104,34104,34104,34104,34104,34104,34104,32056,32056,32056,32056,32056,34104,34104,34104,34103,34103,34071,34070,34070,34070,32022,34070,34038,34038,31958,31957,31957,31957,31957,31892,34005,34005,33971,27666,31794,21327,12909,17135,15054,15021,17166,23472,23440,6634,361,8747,23407,19181,14956,29713,25455,25520,10763,6602,19246,31991,31958,17102,23374,27600,40245,27599,27534,19148,40213,38099,40180,23406,14987,6567,6534,4486,10760,12841,12841,8615,8616,6471,19116,25487,6504,6504,6504,4390,4325,12743,14888,4389,12743,18984,21096,16904,23243,16970,12776,10631,12776,29680,40245,44471,19083,6470,4357,6438,4357,2276,4357,6438,12808,38066,46583,46615,50744,40374,36213,40245,38359,38521,23601,23666,36441,32150,17294,36409,36441,17295,21586,23796,19473,25812,34198,34165,29646,44470,42357,42324,48697,40180,35986,48697,55068,52890,33939,44471,50745,36019,40245,29647,44438,50678,56952,52695,40179,59131,50646,52759,52726,50613,31919,33936,17262,21423,52726,52692,40048,12809,19147,52693,52693,40048,29711,38065,52725,52693,42129,33872,42226,52628,52628,52759,48533,42227,50743,63422,27502,12809,25454,38131,54938,57051,31695,23341,44404,57016,48532,61275,46485,61309,48598,57082,61308,61276,61276,232},
{34138,34138,34106,34138,32090,32058,32058,32090,32057,32057,32057,34137,34137,34137,34137,34137,34105,32024,32024,32024,32024,32024,32024,32024,32024,32056,29976,29976,30008,32057,32056,34137,34137,32056,34104,34136,34104,34104,34104,34104,34104,34104,32088,32056,32056,32056,32024,32024,32024,34104,34103,34103,34103,34103,32022,34070,34070,34070,32022,31990,31958,31958,31957,31957,31957,34005,31892,38165,42390,44470,31826,33939,25586,21360,19247,23505,25521,23440,27601,21326,8714,2441,10860,23407,17101,10762,27633,23375,19149,4489,4522,17134,34071,31926,12843,29745,25487,38132,29680,23309,19148,35987,33906,31793,17035,21293,6567,6535,4454,6567,6567,6502,228,196,2277,12809,6503,4391,6536,4455,2277,4357,10695,14920,6437,8518,21128,23241,19048,21163,10631,4325,6437,12743,14889,27535,31760,14889,21229,25487,14889,17067,23373,14889,14922,31760,38066,46550,44502,50776,44567,42520,42390,38359,36473,23601,23601,36441,32118,13036,36409,36409,10956,8844,13200,8779,4553,19506,32086,39920,44405,42357,38131,50777,48664,44471,52890,55035,55003,46584,44438,36018,27566,40212,31793,36019,52791,59032,50517,40146,59099,52759,52759,52725,50645,29807,33871,19343,21423,50645,52692,42129,14857,19050,50612,52725,35888,21391,29744,54773,52693,35823,19245,31760,52628,50580,54938,48632,33840,55002,54970,21228,12776,31760,46519,57083,44340,42259,19018,50678,54903,50677,59130,65502,52824,52792,59195,61308,61276,61309,237},
{34170,34138,34138,34138,32090,32090,32057,32089,32057,32057,34105,34137,34137,34137,36185,34137,32057,32056,32024,32024,32024,32024,32024,32024,32024,32056,29976,30008,30009,32057,34104,34137,34137,34104,32056,34136,34104,34104,34104,32056,32056,32056,32056,32056,32056,32056,32024,32024,32024,34104,34104,34103,34071,34103,32022,34070,34070,34070,32022,31990,31958,34038,31989,31957,31957,34005,42359,33940,42358,46486,40212,38197,38230,40310,29810,25520,27633,27665,27665,27633,10826,2441,10827,10794,29713,19148,21261,23407,23407,4521,4489,19246,27926,29877,10730,21294,27535,31793,33841,23341,25455,29713,38132,31793,12744,19180,4454,4454,6534,4422,229,229,196,197,2310,8584,8616,2277,6471,8584,2277,4357,14921,12872,6470,6438,14856,23241,21096,21163,12809,4357,6502,12744,10663,44471,48664,17002,33938,44471,19148,25519,44471,23341,27567,44405,38066,46583,46582,50808,50808,50808,42324,40439,38521,23601,25649,36441,32118,15084,36408,32183,8810,8844,11055,8779,2440,8941,23699,29292,40180,42325,40212,48696,52890,59261,61342,61342,61342,50744,46486,40147,27566,42358,35986,33841,56984,57016,59034,61147,59099,54840,52759,50645,50612,36016,44307,17230,19343,50613,52693,44274,21130,21065,50580,52693,37936,16970,23243,54773,52692,35790,12744,21130,52725,52661,54905,40245,44405,38033,33873,25390,21196,40180,52825,57050,44373,50711,38034,52823,50645,57049,59163,65535,44372,54937,61308,61309,61309,61309,231},
{34170,34138,32090,32090,32090,32057,32057,32057,32057,32057,34137,34137,34137,36185,36185,34137,34105,32024,32024,32024,32024,32024,32024,32024,32024,32056,32056,29977,30009,32057,34137,34137,34137,34104,34104,34136,34136,32056,34104,32056,32056,32088,32056,32088,32056,32056,32024,32024,32056,34103,34104,34103,34071,34103,34070,34070,34070,34070,31990,31990,31958,34038,34037,31957,31957,34005,42358,33972,31826,42292,35954,27600,40278,36084,33972,23407,27633,27665,29778,25552,19213,2408,6633,10794,17068,21261,17036,21262,12940,8682,4488,12907,19763,36180,10794,8649,31728,29680,40212,17002,10664,25455,44438,44470,21195,10728,4421,6535,2309,2276,10760,14857,8551,6535,6535,14955,19116,12842,12841,17001,27567,29680,33873,31793,27599,27535,27502,25387,19080,25486,29680,4357,8550,10696,12776,12809,12841,4325,10696,12842,4357,10696,14987,6470,10729,33905,38098,46615,46583,52856,52856,52889,48695,40471,36441,23601,23601,36441,32150,13004,36409,34328,8810,13070,11022,6731,4520,19473,21586,36052,44438,42324,35986,52890,59229,61374,63454,61374,63422,61309,38066,31728,29615,31793,29647,33873,59065,52758,54840,61147,50679,46453,52759,50645,48532,31919,31759,15085,19310,52726,54773,52692,50548,52628,52693,52725,54773,52694,52725,52693,52692,52660,52660,52628,52628,52660,52726,54839,46453,38066,29647,23276,44406,40212,59164,57051,40146,57050,54872,40146,46452,59130,54905,52825,42292,54937,61309,61309,61341,61309,238},
{34170,34170,34170,32090,32090,32057,30009,32057,32057,32057,34137,34137,34137,36185,36185,34137,34104,32024,32024,32024,32024,32024,29976,32024,32056,32056,32057,29977,30009,30009,32089,34137,34137,34136,34104,34136,34104,32056,32056,32056,32056,32056,32088,32088,32056,32056,32055,32024,32055,32055,34104,34103,34103,34103,34070,32022,34070,32022,34038,34038,31957,34037,34037,34004,34005,36053,33972,31859,38165,46518,35987,25519,27600,44471,29714,19213,19246,21293,29680,42390,23439,4489,6602,12875,17068,19149,14923,12842,29713,14923,8648,12841,13035,27697,12874,4455,27535,38132,40212,12776,19148,27600,48696,48631,27534,12808,4486,6566,21228,31727,19114,40145,46451,44274,21194,33938,50842,52922,40178,21161,46615,50841,52922,52954,52955,52922,52922,29646,8582,31792,38099,16970,17035,17035,17035,12809,10696,8551,8583,8583,4357,6470,6438,2244,4357,29680,33904,48663,46615,52889,52889,52889,48696,40471,36441,21521,23601,36441,32150,13004,34360,27957,10956,8876,23731,17263,19310,34296,36344,42390,48469,44308,33873,55003,61374,61374,61374,61374,63422,50776,31760,31760,33873,29647,36019,42260,61145,52727,59067,63227,52792,50679,52759,50612,50645,31855,29711,19278,21390,50645,52692,48467,44372,44404,52725,54773,48468,44307,46419,52693,52693,48467,46452,48532,52660,52661,54806,52726,38065,38098,29614,23309,42325,42292,54970,54970,40146,57017,42194,21130,44339,54904,25323,35920,40146,59163,61341,61309,61309,61309,236},
{34170,34170,34138,32090,32090,30009,30009,32057,32057,32057,34105,34137,34137,36185,36185,34137,32056,32024,32024,32024,32024,32024,29976,32024,32056,32056,32057,30009,30009,30009,32089,34137,34136,34136,34104,34136,34104,34104,34104,32056,34104,32056,32056,34136,32088,32056,32056,32024,32055,32055,34103,34071,34103,34103,34070,34070,34070,34070,36118,36085,36085,36020,33906,38068,35987,33906,14955,12843,14923,14923,12842,12842,12842,12842,12843,10794,12875,12875,14956,17036,12875,8715,10795,10762,14956,12843,12907,14955,23374,21262,8681,23438,19212,21325,23405,21294,19148,31826,35953,8583,33906,33874,35953,38099,23308,19179,6567,6534,33938,40245,33906,46419,48531,46451,21227,36018,48664,50744,40211,17001,48696,46583,46550,44470,46550,48696,33937,19146,12807,40244,33906,42358,42390,42390,42390,42390,42391,42358,42358,42390,42358,42358,42390,40310,40278,42325,46615,50776,48695,52856,52889,52921,50776,40472,36441,23634,25682,36409,32150,15052,36409,36408,17230,23666,36409,27924,19407,36408,36376,27632,44308,42097,38002,50777,59293,61374,61374,61374,61342,54938,27534,36051,25454,31761,31728,42259,61177,59034,48534,61244,54872,52791,52759,52725,50613,42356,33905,27664,27632,50645,52660,42161,34132,36148,54774,54773,40017,23504,29744,52725,52693,37968,36181,38195,52692,52660,54905,40180,42258,38033,44372,21195,40212,48631,54937,46485,44404,59130,31662,23243,19017,18984,14791,23307,48631,61308,61309,61309,61309,61276,234},
{34170,34170,34170,32090,32090,32057,32057,32057,34137,34105,34105,34105,34137,36217,36185,34137,34105,32056,29976,32024,32024,32024,32024,32024,32056,32056,32056,30008,30009,30009,32089,32089,32056,32056,34104,34137,34104,34104,34104,34104,34104,32056,32088,34136,32056,32056,32056,32024,34104,34104,34104,34071,34103,34071,34070,34038,33972,40051,42260,42260,42293,40179,40212,25519,14955,25454,40180,36019,40212,31792,35954,35922,38034,35986,35986,31728,33841,33874,35954,31761,33874,31793,33874,31761,31793,33842,25422,19181,25487,23310,8649,29713,42325,50809,33971,27600,19182,31794,35954,8519,46551,31761,35921,35986,23308,44503,36083,31793,42357,42293,36018,48564,50644,42194,27566,35986,48728,35986,40212,40212,52857,48696,50777,52857,50776,50809,48663,17065,12839,31824,36019,40277,42358,42390,40310,40310,40310,40278,40278,40310,40277,40245,40277,40277,40277,40277,40244,44469,44437,50808,52889,52921,50776,40471,36441,27892,27892,36441,32150,15051,38457,38489,15181,30005,36408,32085,19440,36408,38424,6568,37969,44243,42292,50809,59229,61374,61374,61374,61374,50744,21228,27730,35987,23341,29648,54904,61146,54808,48534,63292,54872,50646,50710,50613,50613,38228,38099,10695,10663,48500,54773,42161,17100,19147,52693,54773,38001,17133,23374,52725,54773,37936,25584,29712,52693,52693,54937,35986,42258,38065,42323,25421,40180,52857,57050,42291,50678,59130,31694,25323,27468,27468,35952,42292,50711,61309,61309,61309,61309,61309,236},
{34138,34170,32090,32090,32090,32058,30009,32057,32057,32089,34137,34105,34137,36185,34137,34137,34105,32056,32024,32024,32024,32024,29976,32024,32024,32057,32057,30009,30008,30041,32089,32089,32056,32056,32056,34104,34104,34104,34104,34104,32056,32056,34136,34136,32056,32056,32024,32023,34104,34103,34103,34103,34103,34071,34070,38101,29616,42099,42228,42260,31760,33905,36018,40179,35986,35954,38066,38099,38099,35986,35953,33808,33808,33808,29582,31728,29647,33808,35921,33873,35986,33873,35986,35953,33873,31728,16971,23310,25455,23310,10729,29713,40245,55003,50776,36052,23408,29680,33873,14922,31793,27633,31728,42292,33905,38131,50809,50777,38067,42227,42260,50677,42226,27502,25420,31792,50841,21228,33906,40244,52889,50809,52889,52857,50809,50776,40210,19177,12871,14984,27567,40310,42358,42358,40277,40277,40310,42358,42358,42358,42358,40277,42390,40277,40277,40277,40245,44437,44469,52856,52889,52889,48663,40440,36473,21618,23634,36409,29939,17130,27662,31856,15019,25714,36409,32085,21423,38456,40310,23081,39758,42065,33873,52890,61374,61374,61374,63454,63455,40244,19148,25617,38165,27568,38099,61146,61147,63228,61179,57018,54840,52759,50678,52693,48499,54838,54806,52725,52693,52693,54773,52693,50548,46355,52660,52661,50548,44242,42194,52693,52693,46322,37903,37903,52693,54741,54871,50679,44339,38065,52824,38034,44373,35921,52857,38066,54903,50613,35855,25355,21129,25388,38098,44372,54970,61308,61309,61309,61309,61276,235},
{32090,32122,32090,32090,32090,32058,32058,30009,34138,34137,34137,34137,34137,36217,36217,34137,34105,32024,32024,32024,32024,32024,32024,29976,32056,34105,32057,30009,30041,30041,32089,32089,32057,32088,32056,32056,34104,34104,34104,34104,34136,32088,34136,34136,32056,32056,32024,32024,34104,34072,32055,32055,34103,34070,38133,40116,29616,33679,44308,42260,33873,33906,38099,40179,35953,38066,35986,35986,35954,33841,33841,33841,35953,38034,35889,35953,35921,38067,38034,35920,38066,40147,38066,38034,38066,29615,14858,21196,23277,21197,10697,23406,33971,38132,42358,36019,21359,27535,23373,27600,29713,21294,33841,33808,29647,42325,38131,46551,33840,50678,52758,44307,48597,27501,23340,23340,44437,21228,17002,23308,50776,54970,57115,52889,55002,55002,44502,23405,17130,17097,31825,42358,42390,29680,10663,10695,8583,12809,12776,12808,12808,14889,12808,12841,14921,17002,19082,38099,44470,52889,50744,50808,48696,40440,36473,25843,32085,36409,25712,17065,29644,33740,27403,16969,21325,27665,25616,32052,34133,29811,42065,44275,35986,52922,59229,59261,61374,63454,63455,44438,33874,29745,32085,19571,29972,27664,50646,50679,52759,52791,52791,52759,48598,50613,50547,52725,52693,46387,46419,52725,54773,52660,50580,50580,52693,52693,52660,52693,52725,54741,52692,52660,52693,54805,52692,52693,52692,50613,42226,40146,35920,40178,29614,38066,38033,40178,56952,44339,31629,35888,21162,25387,31694,44405,61276,61276,61276,59163,46485,44340,171},
{34138,34138,34138,32058,32058,32058,30010,32058,34138,34137,34137,34137,34137,36217,34169,34137,34104,32024,32024,32024,32024,32024,32024,32024,32056,32057,32057,30009,30041,30041,32089,32089,32057,32088,32088,32056,34104,34104,34104,34136,34136,34136,34136,34136,32056,32056,32024,32056,34104,34103,32055,34103,34071,34070,38101,40116,31696,35792,44276,42260,31793,31857,36018,40179,33840,35986,33873,33905,33841,33841,35921,35921,35953,38034,38034,33808,35921,38067,35921,35953,38066,40146,40179,42260,40179,40180,38035,29681,33874,31762,25455,31858,36052,23342,40278,42293,21326,17034,14922,40245,46584,21327,29648,27535,40179,48696,38099,42292,48598,52791,46484,48565,48631,25388,21162,19049,42292,36051,52890,42325,52857,48664,50776,52857,52890,50841,52889,40277,42389,33969,29745,40278,42390,31793,17002,14922,10663,10663,8583,6470,6470,6437,6438,4357,4324,6437,6470,29647,40179,33873,40179,40179,40244,40439,36441,21521,23601,36441,32019,21258,31660,33838,25355,17067,19376,17328,10988,34328,36409,19541,46519,44243,42260,52890,50777,44470,57116,61374,63423,52858,46519,19082,30168,26138,19408,19342,48566,50678,50678,52759,50678,50679,48566,50645,48499,52693,48467,21260,23372,50645,54773,46387,36180,38293,52758,52693,40081,25552,29680,52725,52693,40048,36083,38163,52661,52693,56985,44373,40113,38033,54969,42292,33873,31727,38001,46484,59097,35855,27436,31694,21097,21130,27501,44405,61276,61276,61276,50679,37967,25290,151},
{34138,34138,32090,32090,32090,32058,30009,32058,32089,32057,34137,34105,34137,34137,34137,34137,34104,32056,32024,32024,32024,32024,32024,32056,32024,32057,30009,30009,30041,30041,32089,32089,32056,32088,32056,32056,34104,34104,34104,34137,34136,34136,34137,34136,34104,32056,32056,32056,34104,34104,34103,34103,34070,34070,34070,36020,31697,42098,44276,40179,33873,33906,33873,40147,35986,38066,33873,36018,33906,29680,33873,33841,33873,35954,35954,33808,33873,35954,35921,35954,33841,38066,38066,35953,35953,29648,21229,23342,27535,29648,31761,42325,40245,29680,35987,36052,36051,17066,12809,46616,44503,23408,29616,14890,38132,44438,35986,35953,52824,52823,54969,42227,38066,21195,38066,14823,35986,35986,46551,35986,48631,29679,44438,46551,42292,48664,44373,50744,44404,27533,40277,42390,42422,42390,42390,42390,42391,42390,42390,42358,42358,40277,40277,38197,38197,38197,38132,31760,50744,42325,40244,33840,40244,38391,36441,21520,23601,34360,32118,17099,42487,40407,14987,13005,15215,17296,4617,15215,34296,23670,40178,42130,27405,50809,46551,29647,52955,61374,63422,59261,46551,25453,30103,21944,15085,17230,46550,44405,40179,40114,35921,29582,23244,48532,46322,52693,50548,19082,19115,48533,54741,44274,27696,29777,52726,52693,40049,23471,25519,52693,52660,40048,34035,36083,50580,50580,57017,44438,40113,38033,40179,40147,38066,25356,35953,50710,57016,33742,31662,27468,16903,14823,29581,52889,59195,59195,61276,44339,40080,16871,200},
{34138,34138,32090,32090,32090,32057,32057,32057,32057,32057,34105,32057,34137,34137,34137,34137,32056,32056,32024,32056,32024,32024,32024,32056,32056,29976,29976,30041,30041,30041,32089,32089,32056,32088,32056,32056,32056,34104,34137,34169,34136,34136,34137,34136,32056,32056,32024,32056,34104,34103,32055,32023,34103,32022,32022,34070,36085,42196,42260,40179,40244,38099,42292,38066,38066,33841,35954,38067,33905,33841,31760,31728,33841,29679,29614,31792,31728,31760,29647,31792,33873,35953,31760,33841,35986,29648,25454,25390,25390,25389,23309,42358,42390,38132,46518,42358,23472,25518,19211,40277,46583,33874,38100,46584,44471,46584,36019,35953,52824,52823,52824,48598,40212,23276,40147,10631,40212,38099,54970,46551,59229,38132,38196,42357,42358,42325,42325,40245,38131,25518,38131,44471,44471,44471,42391,42390,42391,42390,42358,42358,42358,40310,40277,40277,40277,40278,19180,17068,48697,46583,46484,44437,40244,38359,36441,21521,23634,34360,32150,13004,38457,27925,6633,10924,10957,15150,6633,10989,15280,13039,38033,42227,19115,40245,59229,61374,61374,61374,61374,46551,38164,29679,28022,21911,17459,15182,19115,23309,23276,25390,25357,23244,21196,50613,48467,52693,52661,40016,37936,48468,52693,48467,33775,31662,48500,50580,44274,27502,27469,50580,52660,42161,23307,23275,50547,50580,56985,46551,38033,40146,44405,38098,35953,35953,29582,54936,50645,33742,35920,21097,16936,16936,38033,59195,59195,59195,61276,35855,31661,23210,226},
{34138,34138,34138,32090,32090,32058,32057,32089,32089,32057,34105,34105,34137,34137,34169,34137,34105,32056,32024,32056,32056,32024,32024,32056,32057,30009,30009,30009,30041,30041,32089,32089,32056,32088,32056,32056,32056,34136,34136,34136,34136,34136,34137,34136,32056,32056,32024,32055,34104,34103,34103,32055,32054,32022,32022,34070,34070,36118,25585,17101,33938,38067,42260,40180,38066,38067,38002,42260,35953,35921,33840,35953,40115,33808,31759,35953,38034,35921,33840,40147,38099,40179,35921,38066,40114,48598,46485,48566,46452,48598,50744,44471,42358,21131,35953,31728,15087,31793,17066,19147,40245,35987,25455,52922,55003,50744,46550,44404,52824,52856,57049,50678,40147,23309,40178,31760,46551,23309,46486,44438,48632,23407,23374,23341,23341,25487,25487,21293,19148,25518,42358,44471,44503,44471,44471,42423,42423,42422,42390,42358,42358,42358,40277,40277,40245,23374,12810,10794,40342,48631,46549,50710,40211,38359,36441,21553,21585,34393,32182,15084,36408,25844,8746,6731,8941,8811,13101,21619,34264,32054,50842,34003,32020,46648,59229,61374,61374,61374,63422,46583,38132,38196,46648,25552,12907,14922,19115,21196,21164,25357,27502,29615,33776,50646,48532,54806,54806,54806,54774,54774,54773,52725,54774,54774,54773,52693,52693,52661,52693,52693,54741,52693,52660,52661,52628,52628,50580,52660,40049,38066,42260,35986,35986,38131,42260,50678,48532,31629,35921,19016,19016,25388,42259,61276,61244,59196,52824,31629,27435,35920,236},
{34170,34138,34138,32090,32090,32090,32057,32057,34105,34105,34105,34105,34137,34137,34169,34137,34137,32056,32056,32056,32056,32024,32056,32056,30009,29976,30009,30041,30041,32089,32089,32089,32056,32088,32056,32056,32056,34136,34136,34136,34136,34136,34136,34136,32056,32056,32023,32055,32055,32055,34103,34071,34103,34070,34070,34070,34070,34070,23505,17199,27666,25488,27600,29681,31793,38100,33939,36052,38100,42326,38100,38132,35987,35987,38067,38067,38099,40212,40212,40180,40179,38067,38034,33841,31695,33873,42260,42260,48566,46485,44405,40277,29680,29454,27503,29584,8716,25487,19147,21292,36019,40213,25422,48696,48631,46518,44405,36018,46518,50711,54905,48533,38066,21228,35952,42357,44438,23309,44406,57083,52857,44471,31824,21226,19081,19049,21162,23341,27534,21227,23340,23405,36083,33907,31826,33939,40277,44503,31793,19181,44503,23406,19213,36052,14955,6536,6568,29778,52890,44372,52855,48629,40178,38359,36441,21521,23634,34393,32182,15084,36409,32183,12939,17295,25844,21521,19375,34328,36408,34102,52988,34036,29940,48761,55003,52857,59229,59261,59261,42326,38131,38131,31826,19180,23341,29583,35921,44307,48533,52727,52759,54839,54872,54839,54839,54871,56952,54839,54838,54871,56919,54806,54806,52726,52725,52725,52725,52725,54774,50581,46387,48468,46387,46387,44307,46355,46387,46387,42194,35953,40212,35986,35986,38067,46518,61242,42258,33774,31695,19017,16903,27468,48630,61276,59196,61276,44340,27435,25355,48565,235},
{34170,34170,32090,32090,32090,32058,32057,32057,32057,32057,34105,34105,34137,34137,34137,34137,34105,32056,32056,32056,32056,32056,32056,32056,30009,30009,30009,30041,32089,32089,32121,32089,32088,34137,32056,32088,32088,34136,34136,34136,34136,32088,32088,32088,32056,32056,32024,32056,32055,32055,32055,34071,34103,34070,34070,34070,34070,34038,31957,34038,29844,25618,25618,25585,25553,27633,27698,27633,31859,31858,29778,29779,31891,33939,31826,29746,31826,33939,36019,33938,33906,31826,31826,29746,27633,27633,25520,27535,27600,21229,23309,42325,29713,16972,27536,23310,6635,23374,17034,27599,36051,48664,48664,57083,57116,59164,57115,57083,54936,52823,57016,57049,57017,42260,31726,40244,46583,31728,46518,59228,57083,48696,42324,23339,25419,31726,40178,44437,42324,21194,17064,21258,31825,19148,21294,23407,31793,46583,14922,6536,17068,8616,4423,4423,4390,2310,4391,29713,50809,44437,52887,48596,40178,38359,36441,21553,23601,36409,32150,15117,38457,38457,19278,27892,34328,25714,19342,34328,36409,25749,48827,38229,29939,46649,48664,21260,52923,59294,52890,27569,36084,40212,44373,48566,50613,50646,50646,54839,54839,52758,50678,57017,57017,54871,54871,52759,54807,52759,52758,50645,50646,52726,52693,50613,48467,46387,42161,31662,27436,21098,16872,12711,12711,10631,10598,12646,10631,10598,27534,25421,42324,35986,36018,35986,52823,54871,38000,40113,27501,25388,46485,38033,57050,61276,61276,61309,40113,40079,35886,57082,192},
{34170,34170,32090,32090,32090,32058,32057,32057,32057,32057,32057,34137,34137,34137,34137,34137,34105,32056,32056,32056,32056,32056,32056,30008,30009,30009,30009,30041,32089,32089,32089,32089,32056,32088,32056,32056,32056,34136,34136,34136,34136,34136,34136,32088,32056,32056,32024,32024,32056,32055,34103,34103,34070,34070,34070,34070,34038,34038,34038,34038,34038,31957,31957,31957,31957,31924,33940,36052,44406,38132,23407,21360,21392,21327,17133,14988,33939,44471,44471,44470,40245,31826,21294,19213,29713,21293,29713,44437,31793,17069,8616,42390,25455,12876,31794,25455,6635,19180,14921,38131,52922,33873,59196,61309,61276,61309,61276,61309,61276,57050,42259,29614,19083,21162,23306,36018,46583,42325,48631,21195,55035,44437,46518,42356,38131,44437,48599,48630,40178,27598,23339,27500,27500,29645,27533,31727,35986,36051,10696,4423,8616,4390,2310,4423,2310,2277,4423,19148,48696,46517,52854,48628,38098,38359,36473,25844,23666,36441,34230,8712,25681,38456,17197,27924,34328,25746,19342,34328,36409,36182,42456,38099,38196,46584,57116,50809,57116,55036,29746,23407,36051,38131,38099,40179,40114,39984,39984,48532,50644,44306,46452,65534,65502,50743,63356,54872,33807,44308,8453,31792,54970,40213,23342,12712,14792,14792,14760,10599,8485,8518,8518,8518,8518,8518,8518,10566,8518,8518,25421,35953,33840,38099,38067,38066,61275,40146,23275,40179,44438,40245,40211,35952,52824,54937,54970,52824,29581,27467,33774,57050,204},
{34170,32090,32090,32058,32058,32057,32057,32057,32057,32057,34105,34105,34137,34169,34137,34137,34105,34104,32056,32056,32024,32056,32056,32057,32057,30009,30009,30041,30041,32121,32121,32089,32056,32089,32056,32056,32056,32056,34136,34136,34136,34136,34136,32056,32056,32056,32024,32056,32056,32055,34071,34103,34070,34070,34070,34070,34070,34038,34038,34038,31958,31958,31957,31957,31957,31925,31892,29714,42357,40245,21327,21360,29779,23472,17133,17068,29713,44438,46551,46550,50776,46551,44471,31859,25519,25454,31826,42292,17068,10762,14923,42358,25487,12811,17037,14988,4521,14954,16969,40244,44470,38099,59066,48534,48534,54840,54872,46420,31726,23339,17001,8551,4358,10663,25420,35986,50841,52922,46518,36019,52923,52922,52890,52890,52922,52889,52921,52922,55002,40243,23403,23370,21257,17064,17033,19148,38131,25454,12842,10761,8648,8616,6536,6503,4423,4423,4423,6503,40309,46517,52887,48628,38065,38359,36473,30005,27924,36441,27858,15081,23534,23371,17001,19310,30037,25649,17197,34198,34231,31925,36149,29712,36051,40212,55003,59196,57116,31860,25521,19149,27599,38131,38066,40212,42162,39952,37839,46452,50677,46451,48532,65501,65502,50711,57049,40081,38001,35855,12711,27567,52923,40213,23343,6405,12679,12711,12679,10631,6372,8518,8550,8550,8518,8486,8518,10566,8518,8518,23308,57051,23276,21131,25421,52791,59130,29517,23276,38131,40309,38164,40180,42259,52857,52856,48566,35856,33840,29582,27468,23275,119},
{34138,32090,32058,32058,32058,32057,32057,32058,32057,34105,34105,34105,34137,34137,34137,34137,34105,34105,34104,34104,32056,32056,32056,32057,32057,30009,30009,30073,32122,32121,32121,34137,32088,34137,32056,32089,32088,34136,34136,34136,34136,34136,34136,34104,32056,32056,32024,32023,32056,34103,34071,34071,32054,32022,32022,34070,34070,34038,34038,31990,31990,31957,31957,31957,31925,31924,40246,38132,42293,42325,21327,21360,23440,19246,17101,19181,21229,27535,50809,27600,40212,19148,17068,12875,12843,23374,21260,21293,10762,10762,17068,42358,29583,12616,27536,23407,8649,19147,19082,23373,40244,31793,50679,38099,35986,42293,35889,35919,25354,14822,17000,10663,6502,2244,10663,12840,17034,19115,29647,35986,40212,40244,42325,44438,46518,48664,46551,48664,50744,48631,31825,23341,17067,38098,46584,27764,21261,14922,10729,21294,10697,6568,6536,6503,4455,4423,4423,21261,52922,44437,54935,48661,36017,38359,36441,23666,25746,36441,17228,15081,21290,23274,27435,21162,25584,25616,15019,34166,34198,29811,40375,50842,40212,33873,40180,33906,27568,19181,17036,12842,27600,33906,21162,42292,42194,42130,38033,42194,46419,40080,40112,48564,50645,42193,40145,35887,46518,35986,14889,31761,52922,40278,23376,8518,12711,12711,12679,14792,8485,10598,10598,10598,10598,10598,10566,10566,8485,8517,23276,59196,54970,35953,46485,54904,44339,33775,27534,23373,23373,35986,38066,46518,27501,40211,59196,46452,27436,31630,29582,23243,91},
{34138,34138,32058,32058,32058,32057,32058,32057,32057,32057,34137,34105,34137,34137,34137,34137,34137,34105,34104,32056,32056,32056,32056,32056,32057,30009,30009,32090,32122,32121,32121,34137,32088,34137,32056,32089,32088,34136,34136,34136,34136,32088,32088,34136,32056,32056,32024,32023,34104,34103,34071,34071,32054,32054,32022,34070,34070,32022,31990,34038,31990,31957,31957,31957,31957,31925,44439,50744,40179,44438,23473,21360,21360,19246,17101,17101,21261,19148,29712,29713,33874,29648,12810,6568,10794,14987,19147,21293,17068,4455,12842,42324,40115,18857,14760,21132,8649,19115,17002,14889,31793,48566,48599,42325,38132,36019,27599,21193,19112,19048,16968,10630,6438,2309,6470,10695,6470,4325,6406,8518,8518,8518,8518,8615,16969,12776,8583,10664,10696,10697,17002,23309,33873,36018,46584,21393,17003,14857,14956,38165,10729,8616,6536,6504,6503,4423,4423,25486,59196,44437,52855,48597,38065,38327,36441,21553,21520,36441,34230,19275,25645,33936,25421,31955,29973,13004,4488,23667,38457,36182,42489,50810,48664,29679,31793,25422,14955,12842,10729,8616,27600,19148,19082,40212,42227,42162,38033,48564,48564,42225,42192,56950,56951,44306,27500,21129,44438,31760,8517,27502,50842,48697,23375,8518,12711,12711,12679,14792,10566,10566,10566,10566,10598,12711,10631,10598,10598,12743,23308,59196,54938,35986,38098,40146,29581,33808,27534,23373,31825,44405,38034,54937,42292,57082,61308,59130,61244,54970,46453,38033,159},
{34138,32058,32090,32058,32058,32057,32057,32057,32089,32089,34105,34137,34137,34169,34137,34137,34137,34136,34104,32056,32056,32056,32056,32057,32057,30009,30041,32122,32122,32122,32121,34137,32088,34137,32088,34137,34136,34136,34136,34136,34136,34136,32056,32088,32056,32024,32056,32055,32056,34103,32023,34071,34103,32054,32022,34070,32022,34070,32022,34038,34038,31958,31957,31957,31957,31925,31859,33907,48599,42357,23440,23441,21359,19214,17133,17101,19245,17068,38099,31826,31793,19180,14922,10761,10794,17099,21293,14954,14922,14923,12875,44470,19051,20905,12680,23245,4424,8649,14889,23373,29712,29647,48664,46551,38099,44406,38066,23307,23274,19081,14888,12808,14921,25486,23341,25422,23342,21196,16970,21196,21164,16970,14857,12776,12744,8551,8551,8551,8583,6503,8583,8550,27534,40212,31826,10763,17003,31793,19181,44503,10729,8616,4423,4423,4423,6503,6503,17067,50809,46550,42323,46549,35985,38359,36441,21553,21488,36441,34231,19245,36344,38489,17230,13037,17361,8811,2407,8876,21522,29908,46715,59262,57116,33938,48664,27567,12810,10729,8648,8616,31858,25486,19148,40244,42227,42162,40146,48532,50612,44305,42192,56950,50644,27468,23275,25388,42325,31760,6437,33874,50810,50809,31761,10599,12711,12711,10598,14824,10566,8485,8485,8485,8518,27502,14824,14791,14791,14760,23276,59197,31728,23276,40179,31694,33807,27436,29679,27534,38131,31727,40179,48631,38034,59196,57050,50678,50711,46517,59196,48598,140},
{34170,32090,30009,30009,30009,32057,32058,32057,32057,34137,34137,34137,34137,34137,34137,34137,34137,34137,34105,32056,32056,32056,32056,32057,30009,30009,30041,30074,30074,30073,32121,34137,32088,34137,32088,34137,34136,34136,34136,34136,32056,34136,32088,32056,32056,32056,32024,32056,34104,32055,32023,34071,34071,32022,32022,34070,32022,31990,34038,34038,34038,31958,31957,31957,31957,31924,34005,25554,44470,40245,23473,21360,21327,19214,15053,17101,25552,17134,21327,21327,35986,21260,14954,14954,14922,25519,23406,14954,10696,8681,6537,38132,10664,12648,21002,23213,14956,12842,14954,27567,40309,36051,33906,44406,40179,44471,44471,42390,42358,40278,38165,27599,40244,52857,50712,50744,44405,44372,46517,48598,38066,19050,14824,10663,10664,8583,10663,10663,10663,21228,21261,23342,33938,44471,44503,44503,44471,19149,10729,12842,6568,6536,6504,4423,4423,4423,4423,6503,40342,44437,46517,48629,38065,38359,36441,21553,21488,36441,34231,12907,38457,36409,17197,8811,11054,6698,2407,8877,13038,13039,25650,31891,33939,36019,44438,40179,12809,8583,2277,8616,40277,35953,27566,42292,44308,40082,40146,44338,48531,42192,40079,50644,44307,44404,44405,31727,40180,23276,4292,35986,50842,50809,36019,8551,10631,12711,10598,12711,8518,6405,6437,8485,8518,27502,14824,14792,14792,14824,23308,54971,40212,38034,38066,31727,29549,33807,31727,31727,31760,31727,50744,52857,50711,61308,42259,31661,27468,50711,52824,54937,126},
{34138,32090,30010,32058,30009,32057,32057,34105,32057,34137,34137,34137,34137,34137,34137,34137,34137,34137,34105,32056,32056,32056,32056,32057,30009,30009,30041,30074,30074,30074,32121,34137,34136,34169,32088,34136,34136,34136,34136,34136,34136,34136,34136,32056,32056,32023,32056,32056,34103,32055,32023,34071,32055,32022,32022,31990,31990,31990,34038,34038,31990,31990,31957,31957,31957,31924,29811,23506,33939,36019,25553,23440,21327,23472,17133,15020,36051,27600,33906,29713,44406,23406,17035,17035,17035,12810,8584,12777,12841,6600,2376,12876,8616,14890,27600,33939,27632,29745,25487,40277,48696,42358,19180,35987,25454,52955,52955,52955,48729,52922,44503,23406,50744,52857,52824,54937,54937,54904,50711,35920,16969,12776,12744,8550,17002,10663,12776,14921,27533,44372,42324,42358,44503,42390,42390,42423,21261,10729,8648,8649,12874,17068,17068,17036,14955,14923,10697,17068,52922,42324,52887,50774,38065,38359,36441,23601,21520,36473,34231,10891,38457,36408,17165,19343,23764,13036,15085,27892,27762,17198,27697,29778,29907,29712,40245,38034,14889,6503,2309,8648,31760,29516,35953,42324,42227,42162,40146,46451,50645,44305,33741,37967,38000,42291,44372,29615,35986,29615,4292,29647,52922,50810,36019,10664,10631,12679,10598,10630,8485,6405,6405,4324,6404,29615,14856,8453,8485,8518,21163,55003,48664,44405,38066,29581,33807,33807,31661,31694,35952,40178,57115,61309,61308,61308,38001,35920,31661,59163,61276,57083,121},
{32090,32090,32090,32058,32057,32057,32057,34105,32089,34137,34137,34137,34137,34137,34137,34137,34137,34137,34105,34105,32056,32056,32056,32056,32089,30041,30041,30073,32122,32121,32121,34137,34137,34169,32088,34136,34136,34136,34136,34136,34136,34136,34136,34104,32056,32056,32056,32055,34103,32055,32023,34071,34071,34071,34070,34070,34070,32022,34038,34038,31990,31990,31957,31957,31925,31925,27731,27667,44438,31858,23440,25553,19247,19279,17133,17101,36084,21327,31794,27601,38132,25487,25487,23374,23244,21196,21163,19083,17003,6601,4521,8715,6601,6569,12842,27633,19246,6503,19148,36018,52890,44406,19148,29680,17068,44536,46584,44536,46616,46616,40277,25454,52857,52824,52856,54937,54905,54905,33839,19049,14856,12744,8551,6437,12809,8583,10695,25421,59163,42258,42291,40277,48696,42390,42390,33939,25519,31858,48729,50777,48631,42389,40310,40310,42390,40310,29745,25519,57148,46549,52855,50709,38033,38359,36473,21585,21488,36441,34263,13004,36409,36408,17165,15051,25648,15050,12841,32182,29875,8680,12841,8713,15312,29647,42325,33840,12809,4390,2309,6503,33873,40179,35986,42324,40081,35888,33807,46451,46483,40080,44339,52824,46517,42324,40211,29647,31792,35954,19083,33873,50745,52858,40180,23277,12776,12776,14857,14889,14857,12744,12744,12712,10631,8518,6438,8486,6405,6405,16969,48631,33840,21195,38098,38066,35888,31629,29581,35919,42258,44372,61308,61308,61276,54937,31693,29580,35887,61276,61276,57050,96},
{32090,32090,32090,32058,32057,32025,32057,34105,34137,34137,34137,34137,34137,34169,34137,34137,34137,34137,34105,34104,32056,32056,32056,32057,32057,30041,30074,30074,32122,32122,34169,34137,34137,34169,34136,34136,34136,34137,34137,34136,34136,34136,34136,34104,32056,32056,32056,32055,32055,32055,34071,34071,32055,34071,34070,34070,34070,32022,31990,31990,31990,31958,31957,31957,31957,34005,27699,23538,48632,38165,21359,27633,19279,19246,17165,21326,10794,8779,12875,12843,12875,12875,10794,8746,8714,8681,8713,10730,19181,10859,6667,8747,12941,10763,10762,23604,17264,10729,14954,14921,29712,46551,19116,29680,21261,42423,33971,31826,21261,38196,36019,46551,48663,54937,46518,42292,44405,48631,44437,40244,40212,38164,19181,12874,12875,12842,8615,23340,52889,42258,40211,40277,52890,42390,42390,33938,23406,8583,46616,52825,44372,40244,40309,38164,21261,40310,29745,21261,52922,46582,54935,52822,38065,38359,36473,23601,21456,36473,34263,12939,38457,38489,17131,12904,12937,25516,12839,23503,23438,10695,25291,19115,19376,33873,23340,27534,12841,10728,12841,14955,29777,44470,40212,40244,40049,35823,33774,48532,48564,42193,38000,52790,44372,46517,40179,33873,46584,46551,23341,23308,27502,27534,27567,23342,19116,6438,8583,4325,6438,6405,4324,4324,4292,4292,2244,2243,2212,4292,14856,55003,35954,33873,33841,40179,38066,38065,40113,44372,40146,46517,61308,61308,61308,44372,31693,31693,46452,61308,61276,46518,99},
{32090,32090,32090,32058,32057,32057,32058,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34104,32056,32056,32057,30009,30009,30041,30074,30074,32122,32122,34169,34137,34137,34137,34137,34137,34136,34136,34137,34136,34136,34136,34136,34136,32056,32023,32056,32055,34103,34071,32023,32023,34103,32023,32023,32022,34070,31990,32022,31990,31990,31957,31957,31957,31957,34038,25651,27699,46551,33971,19311,25553,27666,17134,17133,23439,17198,6666,6666,8682,8714,6634,2440,4489,4521,4521,6601,6601,21294,10795,6667,8779,17166,10794,14954,19279,21391,17066,19115,14921,46551,54970,31826,31826,19149,46649,40310,42423,42390,40245,29680,25485,23307,25453,21228,27567,46583,52889,48663,52889,50744,31793,17069,17133,14988,14954,10727,25421,57082,42258,38098,31825,52890,42390,42390,33938,23374,6503,48696,50711,44405,42325,40309,38164,19115,40310,29745,6503,48729,46550,55000,52822,38065,38327,36473,23633,21455,36441,34263,10858,27794,34066,17032,14984,19275,19145,16966,23242,29614,31759,35920,35953,40178,42291,42324,44372,42325,38132,40179,42291,33937,40245,44438,40212,40244,42292,42227,48565,48564,42225,37999,52790,46517,54969,54936,35986,46616,46583,36019,29615,19050,27600,31826,17265,15021,8584,12842,2244,6438,4358,4357,4325,4324,4324,4292,2212,2244,4292,6405,54971,42326,25389,12744,19083,17035,17035,21163,19050,14856,52889,61276,61276,50711,54937,52824,52791,54969,59195,48598,50679,169},
{32090,32058,32058,30009,32058,32057,32058,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34105,32056,32057,32057,30009,30041,30041,30074,30074,32122,32122,32121,32089,32089,34137,34136,34137,34136,34136,34136,34136,34136,34136,34136,34136,32056,32024,32024,32055,32055,34071,34071,32055,32022,32023,32023,32022,34070,34038,34038,31990,31958,31957,31957,31957,31957,31925,25619,21426,40245,38197,21392,19311,27665,19214,29745,29713,12972,8746,8714,6634,8747,4521,328,328,328,2408,2408,6601,12907,10827,10795,12875,14891,12842,25552,23505,17099,21260,14953,17034,44405,54970,38132,33939,14923,46617,48729,48697,48729,46584,40244,19147,10727,14921,27632,31826,44503,52889,54970,52889,48664,23375,19214,17133,6568,6501,10727,25486,50776,42290,42291,50744,44470,42390,44438,31793,23406,6503,46584,50744,44405,42357,42357,42358,40309,40309,29713,8714,48697,46518,54968,52822,35985,38326,38521,27924,30005,36441,34296,25713,19275,25483,25418,14951,14983,17031,25449,27500,42291,44338,42225,42257,42225,42225,42258,40146,42325,40180,29713,36019,35986,48696,44438,40179,42292,42324,42259,48597,48564,46419,44338,52725,46517,54969,54936,36018,46616,44471,38132,21196,17002,23406,21391,12974,10762,10762,8584,4325,6503,6438,4357,4325,4325,4357,4292,4292,4292,4292,6404,38132,19115,14825,10631,6438,6470,6438,8551,10664,21195,59162,61275,61276,61276,61276,59163,59163,59195,61276,59195,59163,225},
{32090,32090,32090,32090,30009,32058,32057,34137,34137,34137,34137,34137,34137,34137,34137,34137,36217,36185,34137,32056,32056,32057,32057,32057,30009,30041,30074,32122,32122,32122,32121,32089,34137,34137,34136,34137,34136,34136,34136,34136,34136,34136,34136,34136,32056,32024,32055,32023,34103,34071,34071,34103,32054,32023,32022,34070,34070,34038,34038,34038,34038,31957,31957,31957,31957,34037,21458,21425,34004,31891,21392,27633,57148,25552,36019,38067,15053,4586,6634,4521,6601,4521,2408,328,328,328,327,10795,12907,10795,10763,14891,20971,10730,23374,23505,15020,17034,17002,14889,31727,48599,23341,31793,21262,44504,17067,33938,36019,40277,31857,27631,23438,25453,35954,52922,50744,48663,52857,48664,40245,19214,17133,15020,6535,4421,8614,25486,52857,42290,42291,46517,42292,46583,38132,38164,21293,23374,46584,50744,44405,44405,33873,33906,33906,33939,25487,8681,40245,46550,50742,48597,38065,38327,36473,23633,21520,36473,34263,15116,29905,25614,29644,27629,23468,25483,12807,25551,42290,42224,40079,40177,40144,40144,42225,40080,42324,40178,23341,27632,31760,42292,40212,40179,40212,42324,40178,46451,46451,42161,37967,52790,44436,54969,54969,36018,46584,44503,33874,23341,19148,25454,23407,19312,12874,14923,6503,4357,6503,6470,6438,6406,6405,6405,4325,4324,4324,4357,6372,40212,25421,27567,27599,29647,31728,33873,35986,35954,42292,59195,59195,59195,59163,59195,59163,59163,59195,59227,59195,57114,228},
{32090,32090,32090,32090,30010,30009,32090,34137,34137,34137,34137,34137,34137,34169,34137,34137,36217,36217,34137,32056,34104,32089,32057,32089,30041,30041,30074,32122,32122,32122,32121,32089,34137,34137,34136,34136,34136,32088,34136,34136,34136,34136,34136,34136,32056,32024,32056,34071,34103,32055,32023,34103,34103,34071,34070,34071,34070,34070,34038,34038,34038,31958,31957,31957,31957,31957,21458,21426,25553,25553,21392,21359,46616,27633,33906,29680,12907,10859,10859,8746,10827,8714,8649,4489,2376,2376,328,14955,44405,29648,21261,21099,22953,21002,27244,27667,17101,8583,14922,14889,27502,35986,19180,42260,29681,46649,25552,36052,29745,40310,38165,25487,27599,25453,33840,50777,59196,52922,48631,50809,33906,17165,15020,15020,8648,8647,10695,25486,48598,42291,42291,42323,42291,33904,31793,40277,12809,8616,23341,46518,46518,40212,6437,8518,8518,4357,4325,8583,40213,46550,44404,50710,38098,40407,36473,23633,21488,36473,34263,17164,36343,38456,27630,23535,10989,10891,4390,17197,42257,42192,40047,40145,38064,38064,40112,37999,42325,31859,19213,38132,21228,33906,40179,40179,40212,42356,40178,46451,46451,42193,37935,52790,46485,54936,54969,36018,46583,44471,31793,19148,21228,29680,35987,31859,25618,14955,8584,6406,10696,10664,6438,6406,8518,6405,6438,6437,6437,8518,6405,33808,40179,42227,40179,42260,42292,46453,48566,42292,44404,46485,48566,44405,44372,44405,44438,46518,44373,46550,48599,48631,194},
{32090,32090,32090,32090,32058,32057,34138,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,36217,34137,32056,32089,32089,32057,32089,30041,30041,30074,32121,32122,32122,32121,32089,32089,34137,32088,34136,34136,34136,34136,34136,34136,34136,34136,34136,32056,32056,32056,34071,32023,34071,32023,34071,34103,34071,32023,32023,32022,34038,34038,34038,31958,31957,31957,31957,31957,31925,23538,29779,33972,31891,19312,21359,36085,21392,27600,31793,31793,31793,31793,31793,31793,33873,31793,27632,8682,4521,4521,14956,44307,21261,6602,21002,31242,29324,14761,40344,17101,8583,17001,14857,19148,8648,14922,46421,25390,55068,52955,52956,52955,52987,52955,46584,37969,25421,31760,52889,50744,57148,57148,52857,50842,44503,31923,17133,12874,8614,12873,29680,40178,42291,44371,42259,40211,25421,29745,21293,36084,17035,14922,33873,21196,23341,6405,6405,6437,4357,6470,38099,38164,44437,50742,52822,38066,40407,38522,23634,23536,36441,34263,10859,36409,36441,19278,19342,13102,6698,2374,17197,42257,42192,37998,40146,38065,38065,38065,35951,42325,33940,17134,31957,21261,31891,36018,40244,40244,42324,42291,44371,48564,40080,35854,54838,44372,50776,50744,42390,48664,42325,38099,35954,31760,27567,42325,25455,19116,12810,17003,36019,36052,36019,31826,31858,31793,29680,27567,25487,23374,27599,27535,29582,23275,23275,21162,19017,19050,19050,25356,18984,23242,27500,29549,16938,14856,25421,44471,31793,19051,25389,23308,19083,40},
{34138,34138,32090,32090,32058,32090,32089,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32056,32056,32057,32057,30009,30041,30074,30074,32122,32122,32122,32121,32089,32089,34137,32056,34136,34136,34136,34137,34136,34136,34136,34136,34136,32056,32023,32056,34103,34103,34103,34071,34103,34103,34103,34071,32023,32022,34070,34038,34038,31958,31957,31957,31957,31957,34005,34005,40277,50809,31891,17199,19246,31892,19279,21326,23406,21293,23342,23374,23374,23406,23374,23374,23406,12907,4456,6634,12908,27566,17004,8715,16841,29162,25099,23083,40278,19148,10696,17002,12808,34037,21358,31825,44341,35954,55036,50842,52955,50810,50810,50842,36019,44373,29680,31761,48664,57083,57116,55035,46583,52922,50841,44470,17100,12907,6535,14986,33907,54937,42291,44404,48631,48696,48696,36051,27599,44503,48664,46551,44438,33906,27567,14922,14890,12809,10728,14922,31761,36083,44437,54968,52822,38066,38359,38522,23666,21488,36441,34263,10891,36376,36441,19278,17229,11022,19376,15052,29907,42258,42224,37966,40114,38066,38066,38098,38033,40211,38066,31760,36019,14857,21131,38034,42324,40244,42324,44404,48596,50677,44306,46386,50677,52857,61342,52955,44503,48729,57084,59197,54971,52857,50777,57116,44471,33906,31793,10696,36084,42391,42423,42423,42423,42423,42423,42423,42423,42390,42390,44470,40179,35953,29614,31695,29615,44438,44438,38002,25291,21129,29549,23276,14922,17002,25389,25421,50777,23276,23276,23276,21196,44},
{32090,34138,32090,32090,32058,32058,32089,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32056,32057,30009,30009,30041,30041,30074,30074,30074,32154,32122,32089,32089,34137,34137,32089,34136,32088,34136,34137,34136,34136,34136,34136,34136,32056,32023,32056,34104,34103,34071,34071,34103,34103,34071,32022,34071,32022,34070,34038,34038,31958,31957,31957,31957,31957,31925,27731,34004,40245,31827,17199,19279,29811,21360,19181,19213,21293,19148,29744,44438,40244,40245,48697,25552,31825,17101,10795,17068,27600,14923,6634,10665,29161,29162,29227,27568,14954,12809,16969,19115,27502,12743,33873,35954,38131,46583,46518,38131,38099,33906,36019,27567,40082,33841,33906,42292,48664,40212,44437,40211,42325,42357,38164,25487,21294,17068,12842,14955,42259,42258,42291,40211,48697,50809,40244,33938,46583,55003,55002,55002,52857,48696,46616,46616,46616,48664,48697,46648,46584,50743,55000,52822,40146,38326,38522,23666,21488,36441,34264,10859,36376,36441,19278,25714,25876,25811,15084,38391,42258,42224,37966,40113,38066,38034,40146,38033,38100,36019,38131,42357,25422,19049,33840,42324,42259,42259,40211,48532,50644,42193,37999,52757,50777,59294,55003,46616,48697,48697,61342,59229,57116,55036,57116,59262,59262,57149,27568,38165,42423,42423,42455,42455,44504,44504,44504,44536,44503,46584,46584,44406,38034,29615,27501,29582,38131,42357,35921,29516,14759,29549,21163,12777,12744,29680,25454,52923,27535,21163,21195,25421,64},
{32090,32090,32090,32090,32058,32058,34138,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32056,32057,30009,30009,30041,30074,30074,32122,30074,32154,32122,32121,32089,34137,34137,32056,32088,34136,34136,34137,34136,34136,34136,34136,34136,32056,32023,32056,34104,34103,34103,34071,34071,34103,34071,32022,34070,34070,34070,34038,31958,31957,31957,31957,31957,31957,29876,25651,23538,38165,29778,17167,19279,27731,23473,17101,19181,19181,19180,31792,50679,25259,27567,40212,23439,23374,17134,10828,29681,29680,17036,8715,12746,39564,18791,19083,29745,14890,12777,14890,25421,19115,12776,38131,38131,23374,44438,38066,19049,17001,6438,4358,29647,42228,33873,31793,44437,50777,35985,35985,55035,50744,52922,50744,55002,50809,36051,33874,27599,44404,42259,35920,23276,21228,31793,48663,50776,46551,46583,48663,52857,29647,23341,21261,23341,17035,19181,21294,42358,40212,50744,55000,50742,38097,38326,38521,23634,21456,36441,34264,10859,36376,38489,19310,25746,34360,27891,17132,38391,42291,44241,37966,40113,40081,40081,40113,37999,35987,33939,40245,44471,29680,38131,38099,42324,42292,42292,40178,46451,46483,42225,35918,52790,50777,61342,52955,50842,52922,50777,61342,59197,57117,55068,57149,61342,61342,59229,29681,38197,38230,42423,42391,44536,44504,44504,44536,46584,46584,46616,46616,40212,40212,38099,29549,35921,29647,42325,33743,31597,14759,21131,14825,12842,12776,44471,25422,50777,23309,16937,14824,25389,74},
{32058,32090,32090,32058,32057,32058,34138,34137,34137,34137,34105,34137,34137,34169,34137,34137,34137,34137,32057,32057,32057,30009,30009,30041,30041,30074,32122,32154,32154,32121,32121,32089,34137,34137,32089,34136,34136,34136,34137,34136,34136,34136,34136,34104,32056,32056,32056,34104,34104,34103,34103,34103,34103,34071,34070,34071,34070,34038,34038,34038,31957,31957,31957,31957,31957,31957,23538,23506,38132,27698,17199,17199,27699,21360,17133,19181,21293,19148,48631,52857,33743,29680,33874,31825,21293,14988,4521,27600,33873,27567,8715,8683,25196,25034,23018,17003,6503,10729,14889,23309,23373,10695,33905,27599,21228,44438,14889,14921,14856,8583,6471,29679,46551,29648,25487,29745,36019,25422,23309,55003,46583,52889,50744,52889,50777,40245,40244,40212,42292,29582,42291,33840,25422,31825,48631,48696,48696,55002,55002,48663,42325,42325,40245,33938,8616,14955,17133,36051,38099,48663,55000,50742,38065,38327,36473,23666,19375,36441,34296,12939,36376,36343,19278,25714,34328,27892,15084,38391,42259,42193,37998,40112,40080,40112,42160,37999,38033,40211,42292,46518,29615,31793,35986,40211,42292,42259,40178,46451,48531,42225,40079,52725,48631,59294,52955,50810,50842,50809,61342,59229,57149,55068,44473,29717,33941,55003,23441,38165,40278,44504,34004,44536,44536,44536,46584,46584,46551,46616,44470,40212,40212,42260,29549,33840,38066,33808,14791,16904,12679,10631,10631,6503,8583,36051,25454,46551,25422,31793,19082,17002,73},
{34138,32090,32090,32058,32057,32089,32089,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32057,32057,30009,30009,30041,30009,30041,30041,30074,30074,32154,30074,32121,32121,32089,32089,34137,32056,34136,32088,34136,34136,34136,34136,34136,32056,32056,32056,32023,32056,32055,32055,34071,34103,34103,34103,34071,34070,34070,34070,34070,34038,34006,31958,31957,31957,31957,31957,34037,25619,23538,40277,29746,21360,23473,23505,17167,15086,17100,21293,21260,48631,57050,42324,48664,42357,33938,25520,17100,2408,25487,42293,21294,10796,6602,12843,10763,10762,14955,6536,17035,17034,46551,33905,10728,29679,29712,36019,40276,25452,21226,14921,6470,6438,19148,23374,21261,19378,21458,27567,38099,44471,55003,50776,50809,50809,52857,50744,44470,46551,44438,54970,44372,50741,48629,50842,46551,44438,50744,48631,55002,55002,46583,46550,48696,50744,35985,35953,27600,23341,44471,48663,48663,54968,52822,35985,34069,30070,23731,32150,34360,34328,29972,21454,29808,31662,19115,23569,13134,21553,38391,42291,42193,37966,40112,40112,37999,38064,33872,36018,40212,36050,42390,35987,23374,27600,40244,40211,40211,40146,48564,46483,44306,42225,50613,50744,59261,55003,50842,50842,50810,59262,59229,57149,55068,42360,33942,23346,50745,23442,33939,38197,33971,31859,44536,46616,46616,46616,46616,46583,48664,48664,44438,44438,42292,33808,38066,40179,35953,33808,31662,29549,17002,10696,6503,19115,33971,42391,42293,29680,44439,19050,21195,112},
{34138,32090,32090,32058,32058,32089,32089,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32057,32057,30009,30041,30041,30009,30041,30074,30074,30074,30074,30074,32121,32121,32089,34137,32089,32088,34137,34137,34136,34136,34136,34136,34136,32056,32056,32056,32056,32055,34104,34104,34103,34071,34103,34103,34070,34070,34071,32022,34070,34038,31990,31989,31957,31957,31957,31957,31925,27667,27699,44406,29713,19312,23505,19312,17199,15054,19181,21261,21260,40244,23373,38067,46550,44437,25552,17101,19213,328,23406,52824,23374,8715,6569,10763,10763,25520,19181,4423,10696,19147,50776,33905,10696,19180,42326,35986,40244,40211,19179,14921,6470,6438,23374,46551,27600,19411,23505,21228,36018,50841,55003,38131,48696,40244,48696,42357,50777,48664,38131,54970,40146,50677,46549,48664,46616,42325,50777,48696,55035,59228,46518,46518,48695,48695,38131,40212,42357,42357,48696,50743,42291,40210,42258,40309,32118,34328,30038,23699,25811,27924,27859,17162,21257,27402,16904,19049,19147,19244,29842,42259,42193,37966,40112,40079,35951,38097,33808,33906,40245,40213,35986,31793,25454,31825,40244,38131,40179,40178,46451,48564,42225,35919,52758,42325,50810,48729,48696,42390,50777,59229,59229,57149,55068,44440,31829,29748,48730,27667,33972,40310,42456,34004,44536,46584,46616,46616,46616,46584,46551,46519,44406,38099,31727,40179,38099,31727,42357,42324,40244,42259,33905,19148,8649,12810,29745,31761,50777,31761,33938,27534,25421,134},
{34138,34138,32090,32090,32090,32089,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32057,32057,30009,30009,30041,30009,30042,30042,30074,30074,30106,30074,32122,32121,32089,32089,32089,32056,32089,34137,34136,34136,34136,34136,34136,34136,32056,32056,32056,32056,34104,34104,34103,34071,34071,34103,34070,32022,34070,32022,32022,31990,31990,31957,31957,31957,31957,31957,36085,23506,27698,46551,23472,17199,19279,21393,19247,15053,17101,21261,17067,27632,23374,35986,42260,35986,31826,6601,4488,328,25487,48598,19148,8682,4489,8682,14988,10762,8649,4423,4423,17067,46518,33873,17068,27600,29647,25454,25421,25454,21260,21228,21228,21196,23406,44438,44471,44470,38098,25454,35986,52890,55002,55002,55002,55003,55035,52890,48663,46551,50744,57050,48630,50709,42324,42357,31760,27599,44503,48631,55035,57115,48663,46550,50744,48696,55002,55068,44471,36084,44534,50775,40178,29646,25453,29842,32118,30102,32118,326,4454,8680,4421,17001,25353,35821,19016,8615,8680,4454,8712,40179,42193,37966,40112,38064,35985,40114,35822,38065,44372,42291,38033,23309,17003,10696,19116,23374,25486,29712,44339,46483,42193,35887,50645,42292,50743,48696,52890,61342,57115,55003,57181,57116,55036,40246,33909,36021,52923,25555,33940,36052,44471,38165,46584,31825,42325,46616,46616,46551,35986,38099,35986,19082,21227,40211,29581,29614,42357,42292,44502,46550,38066,21130,14858,23374,27600,29680,27567,31728,40212,35986,25421,134},
{34138,32090,32090,30041,30009,32089,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32057,30041,30041,30041,30041,30009,30042,30042,30074,30074,30074,30074,32121,32121,32089,32089,32089,32057,32089,34137,34136,34136,34136,34136,34136,34136,32056,32056,32056,34104,34104,34104,34103,34071,34071,34103,32022,32022,32022,32022,32022,31990,31990,31957,31957,31957,31925,31925,38165,38165,40245,36020,21360,17167,17167,29779,15021,12908,12907,17068,14922,21293,33971,27600,31826,27600,21294,10794,6602,6602,12842,48598,19116,10762,8682,8682,10730,10762,8648,4423,12810,19115,44437,50744,50810,44470,25486,48729,48729,46616,48697,46616,44503,48664,36050,29711,35986,29679,33839,23341,17003,33906,54970,57083,52889,59196,59229,55035,48664,44438,40147,50744,50678,48629,40212,40277,38164,36051,46518,50711,55002,57115,50776,44437,48695,46550,35920,40180,44470,36084,44535,48662,44404,44403,46549,38229,30005,32150,32086,2406,4455,15019,12906,10728,10728,14921,10760,6567,6632,15083,12873,42260,42193,35918,38065,38065,38033,38001,27435,36018,36018,38066,38098,29680,19083,14857,10697,10696,10729,19148,46451,46451,42193,40113,52758,44372,52855,50743,52889,63455,50810,50810,57149,57116,55036,40278,36086,38134,57052,36085,33972,42423,44504,40278,46617,33906,48664,44503,46648,44471,31793,33873,42292,40179,19082,42260,29614,38033,40244,40147,44340,44404,40081,18985,10631,27600,29713,29713,31793,38099,42390,33874,10663,126},
{34138,32090,32090,32090,30042,32089,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32089,30009,30041,27961,30041,30042,30042,30042,30074,30074,30074,32154,32122,32121,32089,32089,32089,32089,32056,32088,34137,34137,34136,34136,34136,34136,34136,32056,32056,32056,32056,32056,34104,34103,34103,34103,34103,32023,34071,34070,32022,31990,31990,31990,31957,31957,31957,31957,34005,38165,40245,46551,31859,23441,17167,17167,36052,17101,12973,10893,10860,10859,10827,10827,10794,8714,8714,8714,8682,8714,6634,6602,31760,23342,10730,10795,10795,6569,12810,19148,14857,12809,23373,38131,31761,48696,57116,50777,42390,44471,38132,40277,38132,38132,42357,33905,25453,33905,33904,33872,38132,46583,50809,55002,54970,50744,57115,57116,54970,48696,33873,29647,44372,38033,31726,40212,50777,46551,33873,55003,44405,52857,59196,52857,40211,44405,42259,27436,29647,44470,34003,44535,46549,44403,52823,48629,36116,32118,27924,32118,2439,4487,17100,19212,10728,8647,19212,12938,6632,17132,19212,14921,42293,42226,35951,38098,38033,38000,33807,27534,38132,42325,40178,46484,48597,42259,42259,46518,44373,38066,38131,46452,44339,42258,40146,46484,40178,50743,48663,52889,63455,54971,31793,48665,57149,55068,42359,27668,21330,52858,29715,33972,40310,33939,36052,46616,33873,44503,44503,46616,44471,33841,31792,40211,40179,25421,40212,29582,38067,38131,37969,29451,35854,35822,31661,19018,31858,33939,27599,27534,42325,48632,21228,6405,89},
{34138,34138,32090,32090,32090,32089,34137,34137,34169,34137,34137,34137,34137,34137,34137,34137,32089,30009,30041,30041,30042,30042,30042,30041,30074,30074,30074,30074,32122,32121,32089,32057,34137,32089,32057,32089,34137,34137,34136,34136,34136,34136,34136,34136,32056,32056,32056,32056,32056,34103,34103,34103,34103,32023,32023,32022,31990,31990,31990,31990,31957,31957,31957,31957,34005,33972,38198,38196,29779,23442,19281,19313,19281,15054,21393,21361,21295,21328,21393,21393,15021,15021,15021,15053,15054,15021,12941,10795,29680,44372,27535,35954,35986,10794,10729,27535,19050,21195,27567,38164,38196,46583,52890,50744,31858,38262,31891,29778,36084,29810,31891,29778,29712,36018,29647,23308,42358,55003,46551,50777,48664,44438,44470,44471,46551,46551,38132,29680,42259,42291,44372,42324,46551,42325,33906,50776,48631,50777,57083,61342,42357,42325,29647,35986,29679,44438,34003,44535,44403,46549,54968,48597,38261,27924,32182,30070,4487,4487,10825,8712,8647,8615,10761,12809,6599,15051,12970,12873,42292,42226,35951,38098,38033,37999,31694,29582,42292,44372,40145,46452,48532,42226,33775,46485,46484,44371,48663,59196,54970,57083,59228,50743,46517,52888,50743,50777,61374,59197,42326,40245,57116,55036,42359,27636,17170,52923,29748,33972,42391,40311,42423,42391,31793,42358,44503,46649,44471,35987,38099,40244,40179,21195,40244,27534,40212,38132,35921,31564,33710,29484,27436,21228,25454,33906,33906,25454,31793,50777,17002,6500,30},
{34138,32090,32090,30009,32090,32090,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32089,30041,30041,30042,30042,30042,30042,30041,30042,30074,30074,32122,32122,32121,32121,32089,34137,34104,34104,34137,34136,34137,34136,34136,34136,34136,34136,32056,32056,32056,32056,32056,32056,32055,34103,34103,34103,32022,32023,34070,31990,31990,31990,31958,31957,31957,31957,31957,31925,34005,23473,42390,38165,12908,8716,10893,10893,10893,10861,10828,10828,12973,15054,12941,8780,10828,10828,10795,10827,8748,8747,8715,27567,46453,17036,25487,33873,17067,17068,17035,10760,14922,23406,48663,42357,55003,55003,55002,36019,59229,46584,46616,48696,44503,42390,40277,38067,31825,23308,17035,21228,25421,23341,27567,25519,23407,25520,25552,25520,25520,23406,29648,38099,44372,42292,46584,42358,48631,44405,40212,50777,57083,59229,63455,42390,42358,36019,33906,33938,44471,33971,42389,44403,52887,54967,46548,36148,32150,25876,30037,4487,6535,17099,15052,10728,8615,15019,14986,4454,15019,17132,10695,42292,42226,38032,40114,40081,35952,33807,33774,42292,40244,40146,48564,48532,46484,35888,44372,46452,44371,48630,57083,54970,55002,59196,50711,46517,52856,48663,50777,61374,52923,48697,46551,57149,57116,42326,31892,23475,57149,29748,31891,31858,42456,36084,46584,33906,46551,44503,46616,44471,35986,36019,40211,40147,16969,38099,25389,38164,40212,35888,31564,35823,29516,21130,12744,14889,25454,25421,23308,31793,38098,14886,10724,42},
{32090,32090,32058,32058,32090,32089,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32089,30041,30042,27993,30074,30042,30042,30042,30074,30074,30074,32154,32122,32121,32121,32089,32089,34105,34137,34137,34136,34136,34136,34136,34136,34136,34136,34136,32056,32056,32056,32056,34104,32055,34103,34103,34103,32023,32023,34070,34070,31990,31990,31990,31957,31957,31957,31957,31925,29812,23506,40278,34004,29811,27699,12941,8780,6700,6699,6699,6699,6667,6667,6667,6667,6667,6634,6634,6602,4554,4554,4554,23341,46421,23341,27567,27567,23374,29681,19181,12842,10761,14955,50744,42325,50809,55003,52888,38131,55035,48664,48664,48664,48696,42390,29679,19115,16969,23243,29680,23373,19049,19050,29615,40277,40245,40277,40277,38197,38165,36052,31858,27600,44404,40179,50809,52858,38066,38066,50776,52857,52857,57148,63423,42358,42357,25486,36018,29679,46551,38196,42389,52855,54968,54967,48596,38228,30102,32183,30069,2374,2309,10728,15018,14953,10760,17099,14954,6535,15051,17100,10696,40244,42226,35952,40146,40113,35953,33775,33774,42293,42326,40146,48564,48564,48532,40113,29614,46452,44371,48597,55002,54970,52889,59163,48663,46517,54936,48630,46583,57148,46551,44438,52923,55036,48665,42391,40278,27636,57149,29780,31891,38165,40310,40245,44471,31793,42390,42423,46616,44438,33841,29647,40212,40147,16969,38131,21195,38099,38132,25356,31629,33775,27436,14824,4325,8550,21228,19115,21195,19115,17032,14917,10692,45},
{32058,32090,32058,32058,32090,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32089,30041,27994,28026,28026,27993,30041,30041,30074,30074,30074,32122,32122,32121,32121,32088,32089,34105,34137,34137,34136,34136,34136,34136,34136,34136,34136,34136,32056,32056,34104,34104,34104,34103,34103,34103,34103,32023,32022,34070,34070,34038,34038,31958,31957,31957,31957,31957,29876,31924,33971,52890,36084,38197,38165,15086,10893,8780,8748,8748,6667,6635,6602,8683,6635,4554,6602,6634,4554,4522,6602,6634,12810,44340,19148,23309,27534,17035,29648,19148,10729,8616,10696,46551,42293,50777,54970,54936,46583,55068,50842,52955,52955,50842,55036,40245,27598,17001,14856,40212,36019,19017,40080,48564,48663,55035,44405,55035,50776,55002,50744,44470,25519,44404,42292,48664,50777,48631,42292,44405,29646,40147,38132,63422,42390,42358,31793,33938,35986,48631,40244,46582,54967,54967,54935,48596,36083,32150,30102,30037,2342,2277,4421,8646,14951,10726,8614,4357,4390,2309,4389,8583,40212,42258,38033,42225,40112,35920,31727,35920,42292,44372,40177,48532,48597,48565,46452,29582,44339,44339,44372,57083,54970,42259,46485,40211,42291,50743,48630,46583,57116,44438,38132,42390,55036,25520,42391,44504,21330,57181,29780,31891,38198,40311,38165,44504,38164,46616,42390,46584,46584,40212,42325,42292,38099,16969,40179,29647,38131,27534,16937,27403,33774,29516,19113,17065,2243,19115,19115,21195,19115,16999,14949,10724,42},
{32090,32090,32090,32058,32057,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32089,30042,30074,27994,28025,30041,30041,30042,30074,30074,30074,32154,32122,32122,32121,32089,34137,34137,34137,34137,34137,34137,34137,34136,34136,34136,34136,34136,34104,32056,32056,34104,34104,34103,34103,34103,34071,32023,34071,34070,34070,34070,34038,34006,31957,31957,31957,31925,31892,33972,27666,48696,42358,44504,42391,21425,21392,17199,15054,15054,15021,12909,12909,12908,10828,10828,10828,10795,8715,8715,10796,10795,14923,38098,21228,25454,42260,38133,27567,17002,10696,8616,10696,38099,40212,48663,52857,54936,40244,59229,46583,52922,50842,46616,59229,46583,27598,12775,19050,38132,33906,48696,54970,50744,52857,54970,52889,54970,52889,55002,54938,46551,27600,44404,31695,46551,46583,42357,40212,44438,38097,46518,46616,63422,42358,42357,40277,38164,35986,52857,54970,52855,54967,54935,52887,46516,36147,36408,32215,32183,19408,19375,19341,12904,19177,16967,23209,23242,14922,8583,8583,14922,40244,40146,38098,40177,40113,35855,31759,38001,42324,42358,40179,48564,48597,48565,48533,35920,35920,44371,42291,54970,54970,50711,59228,48663,44372,52856,48630,48697,59229,57149,25422,44438,38132,29712,44472,38166,25491,57149,29780,29779,40310,42391,40310,46584,42358,55003,42358,44471,46583,42325,52890,44405,40179,19018,42292,44405,42292,27534,19050,21130,33775,27468,17032,32010,8581,17034,19115,21228,19147,19112,14950,8644,37},
{34138,32090,32058,32058,32090,34137,34137,34137,34137,34137,34137,34137,34137,34169,34137,34137,32089,30042,28026,30074,30074,30041,30041,30042,30074,30074,32154,32122,32122,32122,32089,32089,32089,32089,34137,34137,34136,34137,34136,34136,34136,34136,34136,34136,34104,32056,32056,34104,34104,34103,34103,34103,34103,34071,34071,34071,31990,34070,34038,31989,31989,31957,31957,34005,38198,33972,29746,46519,44503,44504,42391,34004,31859,29779,29779,29746,29746,31794,29746,29746,29714,27633,27633,27633,27633,27601,27633,27633,25487,35986,19181,29647,42261,25488,17002,25455,27567,23341,23342,42325,46518,50776,54937,54937,42325,57116,44470,50777,48729,46583,42325,38163,27565,12807,23308,29713,23440,33906,40180,48663,52825,52889,50744,52857,50711,52889,48664,44438,27632,35985,12776,46551,35986,38131,42358,42357,52856,50642,48597,61342,46583,46583,46551,46583,44438,52857,54969,54968,52887,52855,52855,46516,38065,38327,36473,34328,34360,34328,34328,27793,21453,27663,31890,23569,32118,34263,34198,34133,23374,21260,19082,33808,25388,31662,27566,35855,38132,42326,38033,48564,48564,48565,48532,38001,35920,44339,44339,52857,55003,48663,59196,48630,44404,52824,48598,48696,59261,59229,35986,48729,55036,55003,46552,35956,27570,57084,27634,29779,40310,42391,44536,46616,31825,44438,52824,50711,50711,46485,44372,42259,40179,35953,35921,33840,31728,25356,23244,19082,21163,21163,17034,21321,17033,17034,19050,19115,19050,17033,14919,12806,41},
{34138,32090,32058,32090,32057,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,30041,30041,28026,28026,30074,30074,30042,30042,30074,30074,30106,32154,32154,32122,32089,32089,32089,32089,34136,34137,34136,34136,34136,34136,34136,34136,34136,34136,32056,32056,32056,34104,34104,34103,34103,34103,34103,34071,34071,34071,34070,31990,31989,34037,31989,31957,31957,31957,33973,36085,25553,40277,40310,33939,36085,17199,17167,17166,17199,19247,17134,17134,17166,17166,17134,19214,19246,19247,19247,21359,25520,25553,19246,14955,4456,12810,17068,14956,17003,8615,14889,14889,33939,46551,48631,48631,57083,52857,54971,55035,46518,52857,46551,44405,48696,29646,14887,12808,12808,23407,19214,25455,33873,42357,46551,52857,48631,50776,48631,52857,48631,46584,29713,19115,8583,21293,31760,36018,42358,40277,54937,52721,48564,59229,48664,48696,48696,48696,38131,48631,54937,54968,54967,54967,54935,48596,38065,38294,36441,25779,21521,34360,36408,15083,34165,34166,10891,6698,23764,32150,15149,36376,38424,25519,25421,21195,27534,25421,25389,19083,27535,14922,25453,48531,48564,48597,50645,42226,29549,38033,44339,50744,57083,48631,59163,48631,46485,52823,48630,44503,61342,48664,38099,46552,55068,57149,46552,31471,41936,52630,29747,31859,42423,42227,42390,46616,33874,44437,63389,65502,52790,52790,50677,50677,50710,50678,50678,52759,50678,50678,48598,48631,50711,48598,46485,46485,42324,42324,42324,40211,40179,38098,38098,35985,134},
{32090,32090,32058,32057,34105,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,30041,30041,30074,30074,30074,30042,30042,30042,30074,30074,30106,32154,32122,32122,32121,32089,34137,32089,32089,34137,34136,34137,34136,34136,34136,34136,34136,34136,32056,32056,32056,34104,34103,34103,34103,34103,34103,34071,34070,34070,34070,34070,34038,31957,31957,31957,31957,31957,31924,31860,29779,36052,33971,31826,29746,19246,19247,17166,19247,17166,17166,17134,19247,19215,21392,17134,17134,17134,17166,17166,19214,19247,12940,8682,6537,8649,8650,12843,31761,21261,12841,14921,38100,40212,46551,55002,52857,42292,54970,52890,52857,50809,48696,48695,48696,42356,17034,17034,21260,19180,23374,18985,38066,50744,40212,46551,42292,48631,44437,40179,44438,44438,17035,10663,23374,23342,12841,23374,42358,42358,50809,46452,48599,57116,46616,46551,38132,46584,42292,42292,54937,54968,54968,54935,52887,48596,38098,38326,36473,25779,21456,36409,38457,15149,25779,13135,10923,6665,13134,13069,10858,36376,36376,23406,23308,23308,33840,31597,18919,19113,19113,35952,38033,46484,46452,44339,42226,42259,35888,29582,42259,48598,57051,40179,48631,42325,42292,48630,46485,44503,57149,57116,50777,57116,55036,50810,48633,31439,39760,54678,29747,31892,44373,44274,44406,44503,31793,44405,61308,65502,54871,50677,50645,50644,50645,50645,50613,50645,50645,50678,50678,50678,54904,54904,52855,52823,52823,52823,50743,50743,52791,50711,50678,50678,198},
{32090,32090,32058,32057,34105,34137,34137,34169,34137,34137,34137,34137,34137,34137,34137,34137,30009,30042,30074,30074,30074,30041,30042,30041,30074,32122,32154,32122,32122,32122,32121,34137,34137,32089,32057,34137,34136,36217,34136,34136,34136,36185,36184,34104,32056,32056,34104,34104,34103,34103,34103,34103,34103,34070,34070,34070,32022,34038,34037,34037,31957,31957,31957,31925,31925,33972,42358,40278,33939,31859,29779,17133,17133,15085,15085,15085,15053,17134,19279,17134,19246,17133,15085,17165,17133,17165,17165,17166,12875,6569,4456,6601,8682,10794,29681,21261,12808,21066,27534,52858,52922,46551,46518,40179,42291,46452,48598,33873,25453,31825,27567,29680,29712,23373,14922,8583,14792,14759,35986,52889,52889,54970,52922,55003,52889,55035,52857,48696,27600,6470,14955,14988,23407,29713,38132,36051,48664,44372,48598,52922,48664,44503,38132,46584,33873,42325,54937,54936,54968,54968,54935,48629,38065,38326,36473,25779,21456,36409,36409,15116,13134,11054,10924,6665,8908,8811,6633,19472,21618,17099,8549,12678,27501,35856,21097,19113,25484,44404,38065,54937,29581,19017,23276,33873,42259,27501,27501,33840,33775,31695,27436,25323,31662,33775,35921,46551,57116,44471,38133,40279,38166,19183,36053,33551,37646,52532,31796,29778,44471,46354,44307,42325,33906,42324,61244,65501,52790,50677,48532,48564,48564,48565,48532,48532,48565,48565,48565,48565,52855,52823,52791,52791,52823,31727,31760,36018,44437,50743,48630,46485,135},
{34138,32058,32058,32057,34105,34137,34137,34137,34137,34137,34137,34137,34137,36185,34137,34137,32089,30042,28026,30074,30074,30041,30041,30041,30074,30074,32122,32154,32122,32121,32089,34137,34137,32089,32089,34137,34136,36217,34136,34136,34136,34136,34136,34136,34104,32056,34104,34104,34103,34103,34103,34103,34103,34070,34071,34070,32022,34038,34037,34037,31989,31957,31957,31957,31925,34005,38164,25455,36052,31859,29779,19246,17166,21392,17134,17166,17134,17166,17166,17198,17166,19214,17166,19214,17133,19214,17166,17133,12940,6569,4456,6601,6569,10762,14923,10697,19115,21163,50777,52857,54970,46518,46550,48598,42226,42258,36018,10663,8551,6503,6471,8551,14857,17002,14890,10631,19115,19017,14856,14889,17001,17001,16969,19081,21194,17032,19145,14952,6502,8583,6536,12908,17069,12842,8583,8583,40212,19115,38131,50777,48664,38132,31793,52922,31793,42325,54969,54937,54968,54967,54935,48629,40178,38326,36473,25779,21456,36441,36408,17197,10988,13135,13036,8875,19473,21553,13004,36344,32150,21325,27566,17001,29614,31597,25128,21194,27566,44405,42291,38000,31661,21097,21261,23472,38098,35952,27501,25388,21162,16969,14857,12776,10631,12744,19147,44470,57148,50777,19215,19281,21296,14957,36085,29391,35566,54775,40247,31826,42423,44307,44242,38132,29713,33905,59163,65501,52758,35920,33808,38001,38066,42227,44340,48533,48565,48532,46484,46452,52823,52823,52823,52823,52791,21229,19181,21294,36050,52823,52791,46421,72},
{32090,32058,32058,32057,34105,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32089,30042,30074,30074,30074,30041,30041,30041,30074,32122,32122,32122,32122,32121,32121,32089,34137,32089,32089,34137,34137,34169,34137,34136,34136,34136,34136,34136,32056,34104,34104,34104,34104,34103,34103,34103,34103,34070,34070,34070,31990,34038,34038,31957,31957,31957,31957,31957,31925,31925,31827,27601,31827,31827,29747,19214,19246,17166,19214,19214,19214,17166,17166,19214,17166,17166,17166,17166,17166,17166,19214,17133,23407,6569,4488,6569,4489,19149,19149,21294,21260,36051,52890,46551,46551,38099,42325,46518,35953,46452,31760,8583,6470,6470,4390,4358,21294,14922,35954,42227,23308,21196,12743,23276,12744,12776,10663,16936,14919,12839,12839,12775,4389,2244,8616,10697,8584,6471,6503,6470,27567,23341,29680,46616,46616,48664,48696,48696,38099,44405,54969,54937,54968,54935,54935,48596,40211,38326,38521,25811,19407,36441,36408,10858,21553,10989,17230,23633,34295,34198,13068,36408,36408,23439,25356,21163,33872,35920,29128,27469,29679,50776,42292,40145,48564,50644,48565,29713,38132,42259,35920,21130,14856,21196,19148,19083,12744,14890,25422,42390,55003,50777,23409,21361,17168,19248,42359,29391,37679,54645,36020,29746,40213,46355,48468,21229,23407,33905,59195,65469,50710,33808,27567,25454,23374,23374,23341,23309,27503,29680,33873,38034,52823,52823,52823,52823,52824,14987,12907,19149,33937,52856,52824,46453,31},
{32090,32058,32058,34138,34137,34137,34137,34137,34137,34105,34137,34137,34137,36217,34137,34137,32089,30042,28026,30074,30074,30041,30041,30073,30074,32122,32122,32122,32122,32121,32089,32089,34137,32089,32089,32089,34137,34169,34136,34136,34136,34136,34136,34104,34104,34104,34104,34104,34104,34104,34103,34103,34103,34071,34070,34070,34070,34038,34038,31957,31957,31957,31957,31957,31925,31892,23472,29746,25488,23439,23440,21327,19246,19214,19214,21327,21294,19214,19246,17166,17134,17134,17101,17101,17101,17133,17133,17100,29713,6569,4456,4521,2376,10730,21229,12809,38132,33874,52890,40212,46519,48663,42325,50776,38099,31694,10695,10728,10664,6470,8551,31825,21262,23406,27502,44373,19148,17002,12744,17002,19148,17067,19180,14791,19080,10726,12839,10662,2244,2244,6470,2244,2244,2244,12809,6438,6405,14856,27599,40277,40277,40277,40245,42357,40244,48598,54937,54937,54969,50710,48629,48597,40178,38326,38521,27827,21423,36441,36408,10858,27892,30005,25617,27761,38456,36311,23503,40504,38424,25584,25454,23341,35986,40114,31371,27501,46550,59164,57083,50710,52757,50644,46484,35985,17036,27535,12744,14858,8551,19148,19148,19083,12777,16937,16970,44438,59261,50810,25554,21427,21362,23474,46552,33552,39890,54613,33843,19149,27633,44242,27404,17036,17068,33906,59163,63421,50710,33808,27567,25454,23342,23342,23342,23309,23310,23310,21261,25454,52856,52823,52855,52823,54904,17068,12908,19181,33873,50744,52824,46518,38},
{32057,32057,32058,34137,34137,34137,34137,34137,34137,34105,34137,34137,34137,34137,34137,34137,30041,30074,30074,30074,30074,30041,30041,30074,32122,32122,32154,32122,32122,32122,32089,32089,32089,32057,32089,32089,34136,34136,34136,34136,34136,36184,34136,34104,34104,34104,34104,34104,34103,34103,34103,34103,34071,34070,34070,34070,34070,34038,34038,31957,31957,31957,31957,31925,31892,36085,38164,40245,42325,46551,42326,21359,38198,48697,27243,6537,6601,19246,10892,10828,10827,8747,8715,8715,8747,8747,10827,12908,29681,6569,4456,6569,6568,8616,12841,19147,33971,33906,42293,52858,44470,29647,27567,23276,23308,19115,17034,14922,14922,14889,14922,33938,23375,23406,31793,42293,36019,40245,36019,27599,27600,29745,27469,44240,33708,27369,29515,27369,29482,25257,25289,23177,25257,21063,25290,23177,25289,25289,27402,25322,27434,25289,23144,23176,29548,40048,46386,46419,48499,44371,40080,40112,38064,38163,38358,27697,23438,40439,27728,10791,31987,40439,25616,25648,40504,34198,15051,40504,40537,23439,19147,19082,35920,35888,18952,27533,57083,63422,63389,52790,52790,52790,52758,35888,21196,17003,10664,17036,17036,17035,14922,16938,10598,14823,19049,44470,57149,50810,29714,21329,21394,21361,42359,29390,37614,48306,33843,17101,23407,19018,25357,14922,14955,31760,59163,63388,50678,33808,27535,25422,35986,35954,31760,25422,23342,21229,21229,25422,52824,52856,52823,52791,52823,31728,27502,27534,33872,50711,50743,48598,77},
{32057,32057,32057,32057,34137,34137,34169,34137,34137,34137,34137,34137,34137,34137,34137,32089,30042,30074,28026,30074,30074,30041,30041,30074,30073,30074,32122,32154,32122,32122,32089,32089,34137,32057,32089,32089,34136,34136,34136,34136,34136,36184,34136,32056,34104,34104,34104,34104,34103,34103,34103,34103,34103,34070,34070,34070,34070,34038,31989,31957,31957,31957,31957,31924,34005,42358,46486,48631,46518,46485,42228,21261,40277,48697,22986,4456,2376,19213,6634,6602,4522,4489,4489,4489,4489,4521,6602,10828,21262,4488,2375,8616,10729,6503,10728,12841,36019,40212,40212,46551,27599,14954,44438,46583,46551,44470,42357,42325,42325,38099,27632,38164,42326,48665,44406,52858,50744,52857,31727,40179,40244,35921,37933,46352,29515,29516,21129,23210,29515,23274,21161,21096,27337,16838,21064,23177,29418,18918,25290,23209,29450,23145,16838,23111,29385,16806,38000,46419,44272,44306,42225,42192,42160,42258,44534,42323,40177,29612,33805,33739,37933,31627,33871,36016,42323,40243,33871,38129,40242,33806,29580,31627,35821,31628,25322,35853,48565,48565,50646,46386,40112,44305,44273,33741,27436,25355,25355,27502,21196,10664,8519,12711,18984,42259,38098,42357,57116,50810,42391,19248,21329,19248,31891,33876,33812,50778,31762,12810,14956,14890,12712,12777,14923,29679,59162,63356,50677,31760,25455,23342,33841,46485,46485,40115,29616,27535,23375,23341,52823,52855,52855,54903,52855,52856,52823,50710,48630,48630,50743,52791,201},
{32089,32057,32057,32057,34137,34137,34137,36185,34137,34137,34137,34137,34169,34137,34137,34137,30041,30074,28026,28026,30074,30041,30041,30074,32122,32122,32122,32122,32122,32122,32089,32089,34137,32089,32089,32057,34136,34136,34136,34136,34136,36184,34136,32056,32056,34104,34104,34103,34103,34103,34071,34071,34103,34070,34070,34070,34038,34038,31957,31957,31957,31957,31957,31892,33972,36053,35987,33809,38035,33841,35889,29550,33906,44471,16842,4457,4456,19246,4489,4521,4554,4554,4457,2441,2441,4521,6634,10795,35987,12907,10762,19180,12841,14921,14889,14922,33938,29550,38099,54970,44438,31793,40212,36019,31793,31826,29713,27632,29713,29680,27534,36051,38099,33873,48599,52825,52825,46452,33741,27435,44372,48630,37869,37869,27435,23177,25322,25354,29515,25322,29580,25322,25257,18983,25224,23112,27305,18951,27338,23177,23176,23208,21063,23144,21063,18919,33774,44338,42225,44339,40144,42225,42225,40276,42487,36018,33839,27499,29612,25352,29514,27402,27468,31726,40244,31857,31823,36017,38162,29646,29580,33839,37967,29548,33872,35855,48598,44372,44372,33807,21129,33773,33806,27469,21163,21162,16905,21163,14890,10664,8551,12679,10598,27534,21227,42357,48729,38165,38165,27667,19248,17103,31859,52923,52955,52922,29649,14923,14988,8552,6374,8583,12874,29712,57082,63388,50678,31760,25454,21261,25454,46485,46453,42259,21229,19149,19149,21229,50743,52823,52823,52855,52823,38099,33905,40212,44437,50743,52823,52791,154},
{32057,32057,32057,32057,34137,34137,34137,36217,34137,34137,34137,34137,34137,34136,34137,34137,30041,30074,28026,30074,30074,30041,30041,30074,30074,32122,32122,32122,32122,32122,32089,34137,32089,32057,32089,32089,34137,34136,34136,34136,34136,34136,34136,32056,34104,34104,34104,34103,34103,34103,34103,34071,34071,34070,34070,34070,34038,31990,31957,31957,31957,31957,31925,31925,31892,29779,25553,46518,44373,35986,35922,31630,33873,31793,10697,6537,19214,17166,4489,4489,2473,4554,4489,2408,4456,4521,6602,12908,29680,12842,8649,17003,17034,14953,14953,12873,33938,27567,48631,33873,35986,44471,38132,33906,42390,52923,44503,52923,46584,46583,46584,25453,33971,25454,52791,50646,48565,44275,37935,31629,31760,27534,29582,48565,12678,27501,44437,38131,29647,23341,25421,27502,25389,23275,33840,38000,27436,27468,27500,25323,27436,27370,25288,25321,25289,25289,35886,44273,40047,42225,40112,40047,40079,40145,40308,35919,33774,40210,33772,33707,38031,40210,33839,29614,40308,42324,33839,38097,40275,37999,33675,33773,38096,42193,35886,35822,44306,46484,40113,31693,29515,33708,31661,29549,21130,16937,14857,10631,12745,14857,10631,10598,6404,17002,14889,42325,25487,21261,25422,17036,14955,10762,29746,57149,57149,57148,29648,10729,10729,6471,12679,10664,12810,29679,57082,63356,50678,31728,25422,21229,21196,31760,40147,42260,19181,17068,17068,19148,50710,52791,52823,54903,52856,19116,8616,19214,31825,52824,52856,52824,53},
{32058,32057,32057,32089,34137,34137,34137,34137,34137,34137,34137,34105,34137,34137,34137,32089,30041,30074,30074,30074,30074,30041,30042,30074,30074,32122,32122,32122,32122,32121,32089,34105,32089,32057,32089,32057,34104,34136,34136,34136,34136,34136,34104,32056,32056,34104,34104,34103,34071,34103,34103,34071,34070,34070,34070,34070,31990,31989,31957,31957,31957,31957,31925,31925,33972,31859,33971,38132,38067,25455,27568,25358,44406,42358,10665,6505,36052,31860,12941,10861,8780,8780,8747,6667,8715,8715,6602,10860,6537,6569,4456,8617,17035,14953,14954,12873,31858,23341,48664,44470,44470,46616,46584,33939,44503,57148,46583,55036,40245,38132,52890,29679,23372,29647,44340,50646,50678,44372,40113,38098,21423,21325,25485,42291,31727,6470,44503,29746,38197,31826,17035,10697,10761,14955,33938,52890,40244,29712,21228,21260,21228,21195,21228,21195,19147,19114,27663,48630,40179,46518,40211,42324,40243,38326,38522,27827,21358,38489,38391,17131,36278,38489,23569,23568,40537,36279,15051,38423,40536,27599,23308,29873,38228,40145,31727,29613,48630,61308,44438,25486,27469,35920,21130,29615,27535,12776,21262,8584,14891,10664,10632,8551,8518,10696,17002,23341,21196,17002,8583,8518,12744,12744,29680,57148,57148,55003,27567,8518,8550,8550,10663,10631,10729,27566,48631,54937,46452,31727,25454,23309,19148,19148,19115,42259,19149,17068,17036,19115,50710,50710,54903,54904,52823,21196,6535,19181,29745,52824,52856,52856,55},
{32058,32057,32058,34138,34137,34137,34137,34137,34137,34105,34137,34137,36185,34137,34137,32057,30041,30074,30074,30074,30042,30041,30042,30074,30074,30074,32122,32122,32122,32121,32089,34137,34137,32057,32057,32057,34104,34136,34136,34104,34136,34136,34104,32056,32056,34104,34104,34104,34103,34103,34103,34071,34070,34070,34070,31990,31990,31989,31957,31957,31957,31957,31925,34005,44471,33939,46584,46518,33906,27633,33939,38165,50810,52858,16842,4456,40277,27698,10860,10796,8715,8715,8715,6634,6634,6634,6635,10829,4489,6602,4456,6536,14922,17002,10696,10728,38197,38131,46583,54969,48664,42422,44503,33938,42390,50777,40277,42358,42358,38164,55003,27566,12808,16970,21131,42195,29614,29550,31597,31760,30037,8713,19115,21162,12808,4357,31794,17069,31859,44503,42391,40278,33971,29746,46584,42325,38099,50777,44438,40245,35986,38099,36050,33873,33938,31792,38131,48663,40211,44437,40243,42292,40243,40374,38522,25779,19310,36441,36441,10891,34198,36441,23601,23601,38489,36312,15084,36376,36408,27729,21228,27826,34132,33840,29679,27566,44438,63390,57083,25552,17035,31826,12777,17068,10729,2309,14988,12876,14955,12810,10697,10632,6438,6438,8518,25389,40212,46551,38132,42326,36019,27567,27600,42390,40244,35986,17001,8581,10694,17000,10694,10695,14922,31792,44372,46485,48598,42324,40179,35986,29680,21196,19115,33873,19116,17036,14955,17003,50710,52790,52823,52790,52791,29648,21293,23407,27631,50711,52856,52824,75},
{32057,32057,32058,34138,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,32089,30042,30074,30074,27994,30041,30041,30074,30074,32122,32122,32122,30073,32122,32121,32089,34105,32089,32057,32057,32056,32056,34136,34104,34104,34136,34136,34104,32056,34104,34104,34104,34103,34103,34103,32022,32022,34070,32022,34070,34038,34038,31957,31957,31957,31957,31957,31925,33972,36085,29779,27600,42325,38100,38132,27600,31826,46583,50777,12714,10698,46551,40278,15087,13006,12974,12941,12941,12941,10860,10861,8715,10861,2409,8650,2343,4424,10697,19147,14889,12808,50809,52889,59164,54970,46550,36018,29712,42358,42423,55035,40277,27600,42357,40245,52955,31825,19115,19115,12809,12744,10663,14857,18984,14824,17100,17068,14889,12807,12841,6503,25520,12908,31859,42390,44471,38230,36052,31826,46616,48664,46551,42358,38131,50808,27534,36018,48696,21228,46615,46550,31889,48663,40211,44437,40211,42324,40243,38326,38521,27827,19310,36441,38489,10858,32150,36441,23601,23601,36441,34295,13036,36375,34328,25616,27631,27793,34100,31727,19114,27534,42325,50711,50712,31857,21228,40245,8583,8583,8584,10763,10828,8650,6503,8616,8616,6503,8616,10729,10664,6438,27535,23276,10631,12777,10631,27567,29680,38099,29711,19145,19146,14887,14854,12806,10693,16999,23275,33872,44404,46518,57082,42324,44405,35953,40147,46485,44373,36018,29712,27502,23341,23309,46453,52823,52823,52791,50710,33808,21228,23373,27534,50710,52856,52824,72},
{32057,32057,32057,34137,34137,34137,34137,34105,34137,34137,34137,34137,34137,34137,34137,32089,30042,30074,30074,30074,30074,30074,30074,30074,32154,32122,32122,30074,32122,32121,32089,34137,32089,32089,32089,32056,32056,34136,34104,34104,34136,34136,34104,32055,34103,34104,34103,34071,34071,34103,34070,32022,32022,32022,34070,34038,31990,34038,34005,31957,31957,31957,31925,31924,33972,33939,36052,40180,29746,29714,21327,25487,23309,55003,40148,18986,33939,27665,10860,8747,8715,6635,6634,6602,6602,6570,6635,15054,17134,14957,10795,8649,10729,23308,27566,12841,29679,44438,44405,52889,50776,38131,36051,44471,42422,44535,42423,46616,46616,46616,46583,27599,21260,25422,25454,19115,29648,33905,38001,21163,44471,42358,42357,21227,44471,31826,36117,34004,44503,42325,44471,38230,33939,27632,46584,48663,42390,46551,40244,52857,29648,38099,48696,23308,48695,46583,34003,42325,33873,38131,40211,42324,40244,38326,38521,27827,19310,36441,38489,10858,34198,36441,23601,21488,36441,36344,13036,36343,34328,23568,21293,29906,36180,29615,29647,31696,27469,29614,29549,40178,35920,40213,8583,4390,8551,10730,12908,8650,4358,6471,6471,6503,8584,8616,10729,10664,17035,8583,12777,10664,4325,6438,12744,12776,17033,23403,19145,14919,17032,14886,16999,25384,19015,31661,42259,48663,55002,46583,31825,17034,16970,19050,23243,29614,48598,52824,52856,54969,54936,54904,52855,54904,52823,52823,52823,52791,52791,50678,52823,52823,203},
{32089,32057,32057,34138,34137,34137,34137,34137,34137,34105,34137,34137,36185,34137,34137,32057,30042,30074,30074,30074,30074,30042,30074,30074,30074,30074,30074,32122,32122,32121,32057,34137,32089,32057,32057,32056,32056,34104,34104,34104,34136,34136,34104,34104,34103,34103,34103,34071,34103,34103,34071,34070,34070,34070,34038,31990,31990,34038,31957,31957,31957,31957,31957,31957,34005,38165,46583,44438,38133,31827,19214,23375,25422,44438,35922,27438,48664,40246,17167,15054,12973,12973,12973,12941,10860,10861,10828,10861,21360,15054,8714,4456,8584,17002,23340,14921,10663,46583,55003,54970,48631,23339,33938,48697,48697,21681,23665,52923,33906,29680,42358,25487,48664,44406,40212,46519,31728,38034,40180,33873,50744,52857,52857,38099,40180,38132,52923,46584,52955,42357,40310,38198,36020,40084,46616,44438,46583,46583,48631,50808,29615,21228,25519,14954,25486,25519,27632,17067,10729,14922,25486,36018,40212,38294,38521,25779,19278,36441,38522,10891,34231,36441,23601,23536,36441,36376,13004,34231,34328,25649,14954,27794,36212,29679,31825,36019,44405,35887,33775,46419,46451,38000,40179,19082,8551,4358,17135,8682,2277,6438,6438,6470,6438,4390,4390,6438,12809,14923,12777,10696,6470,8551,6470,8550,21257,23402,16999,16999,16999,16998,16999,19079,19015,33806,42291,48631,44405,27534,17034,14889,19115,27534,29615,29615,42259,52823,52856,52856,54936,54968,52856,54904,52855,52856,52791,50743,50743,52856,52823,52791,198},
{32090,32057,34105,34138,34137,34137,34137,34137,34137,34105,34105,34137,34137,34137,34137,32057,30042,30074,30074,30074,30042,30042,30074,30074,30074,30074,30073,32122,32121,32089,32057,34105,32089,32056,32056,32056,32056,32056,34104,34136,36184,34136,34104,32055,34103,34103,34071,34071,34071,34071,34071,34070,34070,34038,34038,31990,31990,31957,31957,34038,34038,34038,34070,34038,34005,33940,36020,42293,40245,44439,21359,25520,27568,21326,19214,17133,21293,27632,29746,27601,38132,25520,25487,23439,31859,33971,12941,8748,17134,12941,8650,12842,14921,14888,16969,14953,10727,23340,25485,31759,48663,25419,36050,48729,48729,15505,23600,48729,38132,36051,48664,33971,35987,42358,42293,33905,33905,40179,48697,46551,44438,48664,40211,35953,29680,19181,52923,36019,52923,42357,40277,38230,33907,35922,46616,48696,42357,44470,46551,50776,23308,12776,14954,14954,17066,17067,8680,23406,23407,17100,10761,10729,12875,29908,38489,25779,17230,36441,38522,17262,32118,36441,27827,21488,36441,36408,17262,30069,34360,25682,19180,27794,25648,27534,29647,36051,52824,33743,21097,42258,42225,42226,44404,27534,8616,8648,8584,6504,4358,6470,4390,6470,4325,2244,2244,6503,4358,6438,10664,14889,4325,4357,6470,14889,21227,23402,19176,14885,17030,16997,25417,21127,19016,29613,44437,38066,27534,27534,25453,23341,25389,14791,19017,38067,42324,52824,52856,54969,54969,54936,52855,54936,54936,54936,52856,52824,50743,52856,52856,52823,204},
{32057,32057,32057,34137,34137,34137,34137,34137,34137,34105,34105,34137,34137,34136,34137,32057,30042,28026,30074,30074,30042,30042,30074,30074,30074,30074,30074,30073,32121,32089,32057,34137,32056,32057,32056,32056,34104,32056,34104,34104,34136,34136,34104,32055,32023,34103,34071,32023,34071,34071,32023,32022,31990,31990,31990,29844,31925,34038,34038,15053,8747,8715,8715,10795,10795,8747,12908,14956,14988,17068,10828,10795,14924,12908,12908,10762,14988,17133,21327,17133,17133,14988,12939,15020,15052,17101,8715,8715,10795,8715,10827,19147,19114,16969,14888,14888,12840,12840,12808,25421,46517,29645,36050,48729,48729,15473,27826,48729,50777,46551,25487,46583,48696,55002,48696,46551,50776,44438,48632,44471,27632,31826,33840,31727,38132,31826,46616,42390,48729,44471,42358,38230,35987,42131,48664,50809,44438,44438,42325,50776,38131,29712,38098,36018,33873,33905,35953,33872,35985,33937,36084,25520,15020,23634,38522,32118,29973,36441,36473,36441,36441,36441,36441,36441,36473,36441,34295,34360,34360,23601,21325,17261,25680,25421,29680,25454,38066,31663,16904,40080,40080,42258,27502,17068,8648,10729,10729,8616,8551,6438,4357,6438,4390,4325,4357,10730,14989,10665,8551,10664,2211,2211,2179,4324,4324,14888,14919,21256,21288,19143,25418,44369,40210,33840,46550,48696,48729,31760,25486,29680,25421,31695,29615,38066,42324,52856,52856,52856,54968,54968,54904,54936,54936,54936,54936,54936,52824,52824,54904,54904,200},
{32089,32057,32057,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34137,34105,30009,27994,28026,28026,27993,27994,30042,30074,30074,30074,30106,30074,30074,32121,32089,32057,32057,32057,32089,30009,32056,34104,34104,34104,34104,34136,34104,34104,34071,34071,32023,32023,32023,32023,32023,32022,32022,31990,29909,31957,27699,27699,19279,8747,4522,2474,4554,4522,4554,4586,4554,4554,6602,6602,6602,10795,12941,6569,8682,6602,4521,8682,8714,8682,6602,6601,4521,4521,4554,6601,6602,6570,6602,6602,6602,6569,12842,17001,17001,17001,12808,10728,8615,19114,35985,50710,50677,52790,50711,46551,32052,36181,48729,46616,44535,23341,36051,38132,55067,55002,48664,52857,44437,38132,36084,27632,21294,25420,31792,46616,40245,55035,52922,50777,44438,42390,40310,36020,38035,48664,50776,40244,44470,44470,52921,38098,25453,44469,35985,33904,25420,31792,31759,29678,33904,23307,27534,31857,38262,46746,36148,31891,38262,34068,27730,27827,25714,32020,36214,36213,31988,27794,29907,34133,23439,19277,25583,33938,19115,31825,38132,38132,54970,50711,44306,40112,44372,23342,14923,8649,8616,8616,8616,8584,6471,4357,4390,6471,6503,8616,4390,6536,19150,4292,6470,2212,2211,2211,2211,4292,4292,4324,6469,8582,14855,25355,27435,31694,42260,48664,46583,46616,29647,23341,25486,33873,16937,16937,38034,38099,52856,52823,54904,52823,52823,52791,52791,52823,52823,52823,52823,52791,52791,52823,52823,199},
{32089,32057,34105,34137,34137,34137,34137,34137,34137,34105,34137,34137,34137,34137,34137,30009,27994,27994,27994,28026,27994,28026,28025,28026,30074,30074,30074,30073,30073,30041,32056,32057,32057,32089,30008,32056,34104,34104,34104,34104,34104,34104,34104,32023,32023,32023,32023,32023,32023,32023,32022,31990,27732,6668,6668,8748,10828,12876,10860,8780,8780,8747,6667,6667,6635,6635,6634,6634,8650,6602,4489,4489,6602,8682,10860,6634,10828,10828,6570,6634,4521,4489,4521,4521,6569,4521,6602,8715,8682,4521,6602,12874,14921,17033,17001,14920,12808,14985,19146,38097,50710,54870,50645,54871,52791,48598,48631,40179,35985,40277,25487,42390,48729,52955,46583,42292,50777,42324,29713,36084,31794,27600,38131,38132,48696,40245,55003,48696,36052,25486,42390,40278,38068,44244,31727,48664,48664,42358,44503,52921,35986,23308,38131,31791,33872,25452,33872,35985,23307,35985,29679,17034,31759,40211,50743,44371,33939,27567,25486,27600,31858,27632,14889,25422,33906,29680,19115,14922,19180,23406,29615,21293,27535,12744,27567,36019,33873,44437,48597,44404,42291,48598,23341,12842,8649,6536,6536,8616,8584,4390,2277,4390,2212,2277,6536,4358,4390,4423,4390,6471,2179,2179,2179,2179,4292,6438,6438,2276,6438,14922,6405,14823,25420,38033,50809,52923,48729,48697,35953,44404,44405,31695,25389,35986,38098,54937,54904,54936,54904,54904,52856,54904,50710,50710,52823,52823,52855,52823,52823,52823,203},
{32089,32057,34106,34137,34137,34137,34137,34137,34105,34105,34137,34105,34137,34137,34105,30009,27994,27994,27994,30042,30042,30042,27993,30074,30074,30074,30041,30042,30041,30041,30008,32056,32056,32089,30008,32056,34104,34104,34104,34104,34136,34104,34104,32023,32023,32023,32023,32023,32023,32022,32022,31990,25619,6668,6668,6668,27600,59229,57148,57148,59261,57149,59229,57181,59229,57148,59229,57181,59229,57149,57181,57181,55068,57149,57116,55068,55036,55036,55036,52955,52923,50842,42423,42423,50810,48730,48697,48697,46649,46617,44536,42358,25486,19114,19081,19081,12840,14920,17033,17033,33839,46419,44371,50678,48631,40179,38131,29646,21227,31792,17068,44471,61342,57148,50809,31760,36018,31792,25420,36019,42325,57115,52857,40212,48696,42390,50842,50842,46551,29713,31858,40278,38035,29551,17035,46583,46551,46584,44502,52921,35986,12776,21259,27566,16969,27566,25485,21227,29679,21195,17034,29679,12775,29646,50743,35855,21295,48696,44372,33777,48697,42358,14954,8583,33906,29648,14889,8615,8616,14954,42261,19115,21228,12809,23309,27633,36018,46485,48631,48598,46517,46517,25454,12843,10697,6503,6503,6536,6503,4390,2277,4390,2244,163,6503,4358,6536,4423,6503,6471,4325,2179,2179,2179,4357,4390,6503,4357,4357,21261,17002,10695,40244,29614,48630,46550,46583,42325,35985,46517,48597,44404,44373,38066,38098,54937,54937,52856,54936,54936,54904,52823,50742,48630,48630,50743,52791,52823,52823,52823,203},
{32057,32057,34137,34137,34105,34137,34137,34137,34137,34105,34105,34104,34105,34137,32057,30009,27993,27994,27994,27994,30042,30042,30041,30074,30074,30073,30074,32090,32089,30041,30009,32057,32056,32057,29976,32056,32056,34104,34104,34104,34104,34104,34104,32023,32023,32023,32023,32023,32023,32023,29910,29910,21458,6700,4620,4587,31794,40245,42325,44470,44471,42390,44471,44471,44471,44438,44438,44471,44471,44471,44470,46551,44471,48696,46583,46551,44470,46551,48697,44471,48696,46584,42325,40245,46583,48664,46518,48632,44406,46583,46583,46583,33937,17001,17001,19114,19113,14953,12840,12775,27534,44371,46484,52758,52855,50678,40178,25420,23307,38131,29648,48664,48696,57116,52955,25454,17067,17033,21194,31727,38099,48599,48664,40212,48696,44471,48697,48729,50810,44439,38035,43920,37646,27309,23373,46583,40245,44438,44470,50776,35953,14889,23243,23308,10695,23340,17001,17034,27566,10663,10695,23308,4356,27535,29647,8582,31826,48697,36052,19116,50745,29778,21294,12777,12809,12777,10696,10696,14922,25552,31857,23373,19148,10696,14890,36052,33905,46518,48598,48598,46517,46485,25454,14923,8616,6503,6535,6503,6470,4390,4357,4390,4325,2212,2277,2277,4455,6535,6568,8616,6503,4325,2179,2179,2212,2244,8616,4357,4325,4325,4324,6437,31727,40211,48631,42259,44405,42292,40211,48662,57049,52823,46485,31727,35985,52856,54904,52856,54936,54937,54904,52791,52790,42259,31726,35920,52791,50743,52791,52791,158},
{32057,32057,34137,34137,34137,34137,34137,34137,34137,34105,34105,34105,34105,34137,32057,30009,27993,27994,27994,28026,30042,30042,27993,30074,30073,30073,30073,30073,32089,32057,32057,32057,32056,32056,29977,32056,32056,32056,34104,34104,34104,34104,34104,32023,32023,32023,32023,32023,32023,32023,31990,29942,23571,21393,21393,21392,42294,36019,38132,40245,38132,38132,38132,40180,38164,40180,38132,38164,38131,40212,38132,38164,40212,44471,42390,42390,42390,44438,44471,44438,48664,46584,42325,42325,46519,46551,40245,40245,40212,40245,42325,42293,27599,17033,10663,12808,19114,17033,17066,10663,12744,40179,48565,50645,46419,48532,35919,21226,14888,38099,38164,25453,42357,57116,57116,27567,27567,29711,29711,29613,38066,33840,38067,40245,46583,38132,46648,48729,44503,29680,38067,33453,21002,17003,23373,48664,42293,44470,46583,50808,50777,52922,48663,25453,46615,46550,31792,46615,46615,46551,42325,42324,42292,44404,44405,25421,21294,21293,17068,8583,12874,17036,10696,6503,19148,17068,14954,12809,14954,44471,42261,35822,19115,14890,12777,40245,36051,44437,46550,48598,46550,46485,27535,12842,8616,6503,6503,6470,4390,4325,4357,4389,4325,2212,2244,2244,4423,6471,6503,8584,6503,6471,4389,4357,4357,2244,4357,4357,4357,4357,4357,6437,27502,29582,48631,40179,42292,46517,48630,48662,54968,52823,38067,29647,35986,54937,54904,52823,52823,54904,52823,52823,52790,52791,50710,50742,50710,48597,50710,50710,201},
{32057,32057,34137,34137,34137,34137,34137,34137,34137,34105,34105,34104,34105,34137,32057,30009,27993,27994,27994,30042,30042,30074,30042,30074,30074,30073,30073,30073,30041,30009,32057,32056,30008,32057,29976,32024,32056,34072,32024,34104,34104,34104,34104,32023,32023,32023,32023,32023,32023,32023,31990,31990,36150,38230,40278,36085,35954,40212,40212,42357,42325,40245,40244,40212,40212,40212,40212,40212,40180,40212,38132,38164,40212,44439,46551,44471,46551,44471,46551,44503,46584,44471,38099,42390,44438,46583,42293,42325,42358,42325,40212,42357,29679,17001,12808,10663,10695,14889,27566,29614,31728,33873,40245,23340,10727,12775,14920,12840,8582,36051,21228,10695,19146,33872,36018,42325,36051,44470,48664,35953,31662,31695,33906,46551,50777,31793,42390,23373,40310,46616,36019,31631,46583,46615,46615,46583,42325,40212,46583,50809,42324,44437,48663,25486,48696,46550,48696,46582,48663,48728,44405,50743,40178,46484,50678,40146,14890,17035,17068,19148,10728,6535,4389,6470,10761,19213,8648,10696,4389,14890,25421,12776,12841,19115,14890,33938,40244,46550,48598,46518,46517,46485,27535,12843,8616,6503,6471,6471,4390,2277,4325,4325,2244,131,6438,2244,4423,6503,6471,4423,6471,6471,6438,6438,4357,6470,4358,4390,4357,2244,4324,4356,23308,31727,44372,44372,44372,46485,46518,48630,54968,40179,31793,33873,38099,54904,54936,54904,52855,54903,52790,52790,52790,52823,50710,50742,52790,52758,50710,50678,200},
{32090,32057,34137,34137,34137,34137,34137,34137,34137,34105,34105,34104,34105,34137,32057,27961,27994,27994,27993,27994,30074,30074,27994,27994,28026,30073,30041,30041,30041,30009,30008,32056,30008,32056,30008,29976,32056,34104,34104,34104,34104,34104,34104,32024,32023,32023,32023,32023,32023,32023,31990,34038,36150,40278,40311,29746,31761,52858,46584,46584,48697,48632,50745,48664,48697,46584,48664,48664,46584,48697,48664,50809,46616,50745,46584,46551,46584,44438,48697,46551,46551,46551,36019,44439,44438,44438,44470,46551,42358,44471,46519,44471,46583,29679,17001,12840,19082,23373,35953,44371,46518,31793,42390,40244,14954,8614,14952,12840,10760,29712,21260,12841,14921,14921,17002,29647,48664,55002,54970,50744,46485,40179,55003,40277,44471,31825,42423,27566,40277,44536,31825,38132,48664,50808,50776,50744,40245,40212,40244,50809,42324,40243,48728,25486,46615,44502,52889,46583,46550,52857,46517,38065,33808,42291,44404,38066,12777,8616,10696,6470,17035,23374,25455,19213,14986,10728,6470,6502,2244,10631,12744,8615,10664,21196,21196,31793,42325,42325,31793,46518,44405,46485,27568,10730,6535,8584,6503,6471,6470,4325,2244,2244,2212,2211,6438,4325,6471,6503,6471,6471,6471,6471,4390,4358,4357,4390,6470,6503,4390,2244,4324,4324,14856,35888,44372,38033,35921,38066,44405,48631,44372,25422,25487,17003,31792,54936,54936,54936,52823,54904,52823,52790,52790,54904,52791,52791,54936,52823,50710,50677,198},
{32057,32057,34137,34137,34137,34137,34137,34105,34105,34105,34105,34105,34105,34105,32057,30041,27994,27993,27994,27994,27993,27993,27993,27993,27993,30041,30041,30041,30041,30009,32024,32024,30008,32089,17363,23637,21524,21524,21524,21491,21491,25685,25685,23604,23604,23572,23539,25684,27765,29845,25684,17200,19280,21425,21360,15053,12876,23408,25488,27568,27568,27568,27601,27600,29681,29713,31794,31794,31793,33874,33874,33906,33874,33874,35987,36019,38099,40180,40180,40180,40180,40180,33873,38099,40180,40212,40180,38131,40180,40245,40180,40180,38067,27567,27566,23308,19050,33873,38033,44404,44405,35987,21228,21260,10728,14921,17065,12840,14986,31858,33971,40278,40277,40245,42358,23309,48664,55003,55003,57083,44373,40146,57116,44471,46584,40244,42390,25454,40277,44536,25519,44503,48728,50776,50776,50744,42357,40212,44470,50809,44437,35985,48695,23341,48663,44502,48695,46582,46518,42324,44437,25388,19114,42291,44371,38033,10664,10696,12777,6438,10664,10632,6438,17002,17099,14954,14954,8615,4389,6534,8615,10663,8583,16970,29615,25487,42325,38099,14922,42260,44372,46517,29648,10697,6536,6503,6503,4358,4325,2244,2244,2244,2211,2211,4357,4357,4358,4358,4358,4358,4390,4390,4358,4358,4357,4390,6470,6471,4390,4325,4357,2244,10663,33775,40049,44340,44275,44340,42227,40179,25421,16969,17003,12841,25421,52823,52855,54904,50742,52790,54903,52823,50710,52791,52791,52824,52823,52791,50742,50710,199},
{32057,32089,34137,34137,34137,34137,34137,34137,34105,34105,34105,34137,34137,34137,32057,27993,27994,27994,27994,27993,27993,27993,27993,27993,27993,30041,30041,30041,27896,30009,29911,32057,27863,29976,11024,8879,8879,8879,8878,8879,8846,8879,8878,10927,8878,10894,8846,8846,10894,10926,6733,2507,2507,2507,459,459,2506,6635,6635,6603,8715,6635,6635,6635,8683,6635,6634,6634,6602,6602,8650,10763,10762,10762,10730,10698,10729,12810,14890,12842,12810,10697,8616,6536,8617,8617,8617,8649,8617,8617,8616,8616,8617,4455,4391,14825,23276,40212,44372,46517,44438,33905,40244,21228,17002,17034,19146,14953,17034,36084,36019,57148,50809,44405,44406,25454,48664,52890,55035,52857,44373,33807,54970,44438,48696,40212,44438,17002,38165,44536,25454,44471,48728,46550,42357,48696,33906,31761,29647,50809,48696,48728,48696,38099,48663,46582,48663,46583,44470,42356,46550,38131,38099,42291,46484,35953,10631,10664,10664,6503,10632,12745,4390,8583,10696,10696,8615,17035,19148,19148,14954,12776,8583,10631,33841,19116,44438,33874,12777,31760,42292,46517,29680,10697,8584,6503,6470,4324,2244,2212,2277,2244,2244,2211,2244,2212,2244,4358,4390,4358,4358,4390,4358,4358,4357,2277,6470,6470,6438,6405,6470,4325,6438,23114,41740,42064,35823,29452,31565,42194,46387,50549,42227,25422,31695,52823,54936,54936,52823,52823,52791,52791,50710,52823,52824,50743,52824,50710,50710,48597,192},
{32058,32057,34137,34137,34137,34137,34137,34105,34105,34105,34137,34105,34137,34105,32057,27993,28026,27994,27994,27994,27994,27993,27993,30041,30041,27993,27961,30009,21524,29911,21426,25685,17265,15153,6766,6766,6766,6700,6733,8813,8781,8813,8781,6668,8781,8813,8813,8813,8813,6733,4620,2507,2507,2474,2474,2507,2506,4555,4554,4554,6635,6635,6634,6634,6635,6602,6602,6602,4554,4554,4522,10795,10794,10794,10762,8681,12842,14955,14955,14955,12875,8648,6536,4456,6536,6536,6536,6536,6536,6504,6504,4423,6536,4455,2343,8584,29615,35953,40081,42292,44372,38131,38164,21228,17034,17035,17035,12809,21293,36019,44437,46551,35986,31760,31727,31760,50809,52890,55003,57083,48598,42259,50777,48631,40179,42325,46616,46616,44504,44504,25454,42390,46583,48663,23341,33907,27600,36052,31825,50776,50776,50809,48695,21260,46583,44470,50808,44502,46583,50809,44469,40178,38098,42259,40146,35920,10631,8551,8583,6471,6470,8583,6503,10696,12842,12809,10728,12744,17002,21228,17003,17067,19147,21033,38034,31761,33906,23309,19050,38034,44405,46517,33841,10729,6503,6471,4390,4358,4325,2212,4357,4357,2244,2211,2179,2179,131,4325,4325,2244,2277,4357,4357,4357,4357,2277,4357,6470,6406,4292,6373,6373,8518,14759,35499,44144,35887,31565,31532,25291,23178,45966,48112,37839,44307,50678,50711,52856,52856,54937,52824,52791,52792,52857,54937,52857,54937,50743,50710,50742,194},
{32057,32057,34137,34137,34137,34137,34137,34137,34105,34105,34137,34137,34137,34137,32057,30041,27994,27993,27993,27994,27993,27993,27993,27993,30041,27993,27896,15153,8814,4555,8716,12974,394,2474,8846,8814,6766,6733,6701,6733,6733,8813,8781,4588,4620,8781,6733,6733,6733,2507,2507,426,2474,2474,2474,2474,2474,2474,2474,4554,4554,4522,4522,4522,4522,4522,8716,8780,6668,6668,6635,8683,8682,12908,10828,10828,12908,10794,10794,12875,12875,12842,10762,8649,6569,6536,4456,4455,6536,4455,4423,4455,4455,8649,8649,6568,25422,38033,25388,46485,46485,46551,55036,44405,44371,44404,42291,40179,23406,33906,48630,52857,29648,29647,35953,23308,42325,54970,48631,40212,38034,33840,40179,36051,27502,33841,46551,46616,46616,44536,23374,44471,29647,38164,27599,14923,12842,19148,38164,50776,33905,42324,48695,27566,46583,44502,52954,44502,40276,36018,44469,46517,33807,40146,23307,27534,10663,10664,8616,6503,6471,8583,8583,8616,10729,10728,10761,10729,12809,12809,12841,17002,33872,38001,38066,29680,23342,12809,23308,42259,35953,40178,35953,10761,6503,6503,4390,4358,2212,2211,4357,4357,2244,2244,2244,2212,2244,4357,4357,4357,4325,4325,4357,4357,4325,4325,4357,6438,6438,4292,6340,6341,6405,10599,18790,39887,46485,35855,31532,18920,21000,22983,37612,27209,29387,21097,35953,48664,50777,54970,40212,29582,29582,25389,46551,38067,50745,33873,23306,21192,76},
{32057,34105,34137,34137,34137,34137,34137,34137,34105,34104,34137,34137,34137,34137,32057,30041,27993,27994,27994,30042,27994,27993,27993,27993,30041,27961,23605,2474,6668,6668,2441,8781,2442,6700,10959,8879,6799,8847,6830,6799,6831,6798,6831,6830,6766,6766,6766,8847,8878,8846,2539,8781,10894,6668,12974,10861,2442,12974,8813,8748,6635,4522,8714,6602,4521,4489,15054,17136,10862,15055,8716,8651,12877,17168,10829,10829,15055,6569,6537,4489,6536,6569,6537,6537,4488,4456,4456,4456,4456,6504,6536,8617,6536,6536,6536,6536,21228,44372,19050,38001,38065,46551,55035,44339,48565,48532,44371,40147,23374,35987,57083,54970,33873,25453,31727,21228,38131,55003,40212,35954,35954,33873,33841,29712,19082,36019,46584,50810,46616,44536,25454,44503,29712,29680,17068,10762,10697,12842,21293,48695,29679,40244,48663,23341,46583,44469,48696,44502,40211,33873,44437,52856,46485,29614,19114,19082,14922,12809,12874,10729,6470,12841,14922,12874,12841,10696,10728,10696,10664,10696,10729,19148,38034,38066,38066,35954,19116,14857,19082,31695,29615,27533,35953,14954,8583,8616,4390,2277,131,2212,4357,4389,2244,4325,2244,4325,2244,4325,2277,2277,2277,2277,2277,2277,4325,4325,4358,6470,6438,6340,6340,6340,6405,10566,21032,35757,40016,37903,37903,23081,20935,27177,18887,18952,23145,25226,33840,52890,50842,52890,42260,29582,31695,25389,23308,29679,38099,21260,21257,16997,57},
{32057,34105,34137,34137,34137,34137,34137,34137,34105,34104,34105,34104,34137,34137,32057,30041,27993,27994,30042,30042,30042,27993,27993,27993,27993,27961,19346,4555,4587,6667,2409,8813,4555,10959,8879,10992,11024,6798,10992,6798,10992,6798,10992,10991,8846,8846,6798,10992,10991,8846,2539,4555,8846,4555,4587,2506,2441,8813,8748,4555,10861,4554,8682,6570,4489,4489,15088,17201,13008,15088,15055,8684,14990,19281,15023,10830,17200,4489,4489,4488,6537,6537,4488,4456,4456,4456,4456,4456,6537,6537,6537,6504,6504,4424,2375,2375,8648,40211,33840,44372,38066,48696,46584,46451,46451,46419,44307,21163,29680,36019,59196,52858,35954,48664,44373,19148,33938,57083,50744,52922,52857,50777,44438,25486,19147,38132,44471,50810,44536,44536,23341,44471,29680,27600,14955,10729,8616,10729,25487,50744,35953,42357,48663,29647,44470,44437,44470,44469,38098,31760,44437,52888,54904,52856,50711,27501,17002,14954,14987,14955,10729,8583,10761,10696,14955,14954,12809,8615,8551,6535,8648,19147,38099,38033,40146,33841,21164,16905,25357,23341,40213,33872,38033,25421,10696,8616,4390,4390,2244,2211,4357,4357,2244,4325,2212,4292,2244,4357,4325,4325,4325,4325,4390,4358,4325,4324,4357,6470,6438,6373,6341,6340,6372,10631,16742,37838,44112,41902,46225,40016,33548,22983,25161,27274,33612,31532,33808,52890,50842,50810,42292,29582,33841,25388,23341,21228,29647,27567,16999,17029,53},
{32057,34137,34137,34105,34105,34137,34137,34137,34105,34105,34105,34137,34137,34105,32057,30042,28026,27993,27994,27994,27994,30041,27993,27993,27961,27993,13039,8780,2441,8780,4521,17199,29908,21490,8911,15217,15185,11024,15152,8846,13104,15217,15217,15217,13071,13039,10991,15184,15184,10958,4587,2442,393,4554,4554,4522,4522,4522,2474,2441,4489,4489,2409,2409,2408,2408,12942,15087,12909,10861,8683,6602,6602,15087,12941,12909,12973,2376,2376,2408,2408,2376,2376,2376,2344,2375,4423,4424,4424,2376,2376,4423,2375,2376,2376,2376,12874,29647,33807,46452,46484,44372,33906,46452,46419,46419,21195,19148,23374,38099,57148,52890,31792,50777,52890,48632,40277,57083,50777,54971,54971,46551,35954,12744,8550,21260,44503,50809,46584,44504,23406,44503,29712,25519,12875,10729,8584,8584,8583,12809,14889,14857,16970,14921,17001,19050,19082,19082,19115,21163,21227,48630,54904,52855,52823,52791,50743,31824,21358,17132,17132,17100,12841,8583,8615,12841,12841,8583,10664,8616,8616,12809,29615,33840,42292,44406,40180,38099,40180,31760,42325,38066,38065,25454,6503,6503,4390,6438,2244,2211,4324,4324,4325,4325,2244,2244,2179,2211,2212,2211,2212,2212,4325,4325,2244,4325,4357,6471,6470,6405,6373,6340,6373,10663,12614,37968,44242,39886,44242,46453,31467,37709,31467,29484,29419,33678,27468,52922,50809,52890,42292,29582,31695,23308,25421,21227,33938,36019,16999,16997,50},
{32057,34105,34137,34137,34105,34137,34137,34105,34105,34137,34105,34137,34137,34105,32057,30042,27994,27994,27994,30042,27993,30042,27993,27993,27993,30041,10991,13006,4522,12941,6602,15086,15054,21425,8780,15087,17200,17232,19280,10861,15087,19313,19280,19313,17167,8780,19280,19345,19312,13006,23571,8780,15087,19280,10828,19248,17200,15054,12974,19280,19280,10861,17200,17200,17232,10828,17199,17200,15022,15054,17167,19312,10828,17135,17134,17167,8747,15054,15054,12941,10828,12941,15053,10795,12940,12940,15021,8682,12941,12908,12940,10795,12908,14988,12908,12940,17067,21227,25388,46419,44339,48499,46354,46386,44274,25324,14825,10664,10664,21260,40180,40180,27599,50777,46518,48664,50777,57116,54970,42325,54970,38164,35986,10631,6437,12776,48729,42325,46616,44504,25454,44503,29648,29681,12875,8616,8617,6536,6504,8584,8584,8551,8551,6471,6471,6503,6470,6470,6438,8518,8550,46550,52856,52856,52823,52790,50709,52791,42324,27632,21326,19180,14987,17067,10728,8583,8583,4357,4390,10762,12842,8583,14857,33808,40179,48630,46452,46419,54970,33873,40212,40114,38032,21195,6503,4422,4390,4390,4293,2179,2244,4292,4292,2244,2244,2244,2211,2179,2179,2179,2179,2211,2212,4357,2179,4324,4357,6470,6470,8583,6470,6340,6373,10631,12549,29387,37936,42032,42194,44405,40081,42129,27306,27306,29355,31597,23243,52922,52922,55003,44373,29615,23211,23308,25421,21228,31825,38165,17031,14949,54},
{32057,34137,34137,34137,34137,34137,34137,34105,34105,34105,34105,34137,34137,34137,32057,30042,27994,27994,27994,27994,27993,27993,27993,27993,30041,30073,10959,17200,4587,17134,6602,21360,8683,25619,10861,17232,17264,17232,19280,10861,10926,17232,19280,19312,15119,10828,15119,19280,19313,6700,13039,6667,6700,6732,6667,8780,6732,8780,6700,6700,6700,6667,8748,8748,8748,4586,8780,8780,8715,6667,8748,8780,6602,10796,10828,10828,6634,10828,10828,8748,8747,10860,12908,8682,10828,12908,12908,6601,12908,10828,10795,6602,12876,12908,6601,10763,8648,19113,31726,44371,44371,48532,46353,42160,18985,12744,10631,8551,6470,6470,8583,8583,14954,44405,46551,48696,52857,57116,52889,46583,38132,33939,38164,12776,10696,17035,46584,50810,46584,44504,25486,42390,27567,25487,12842,8616,10697,6503,8616,6536,10697,4358,8583,8583,8584,6470,4390,6470,6438,6438,14825,48598,52855,52855,52823,52790,52758,50677,52790,52855,46550,23406,19213,17100,15019,12874,8616,4422,2277,2309,6503,19051,25422,31695,35888,31597,37870,38033,40212,29614,46486,29484,33742,21227,6471,6438,4390,6471,2244,2179,2212,2212,99,99,98,2179,2211,2211,2179,131,131,2211,2211,2179,2244,2211,4357,6438,4357,4260,6405,6340,6340,10631,10469,21033,25227,41967,44210,46485,42259,37936,33613,31468,27307,33710,21097,48664,52890,52890,42325,27470,10631,19115,23373,21195,27567,50810,27597,14949,44},
{32057,34137,34137,34137,34137,36185,34137,34137,34137,34137,34105,34137,34137,34137,32057,27993,27993,27994,27994,27994,27994,30042,30041,27993,30041,32122,10958,19312,6667,19215,2344,17167,264,21457,19410,23604,27829,25717,27797,19346,17330,25716,27764,27764,25684,19377,23571,27764,27764,19410,6733,2506,2506,2506,2506,4554,2506,2506,2474,2474,2474,4522,4586,4586,2473,2441,4521,2473,4489,2441,2441,4489,4489,4457,4489,4489,4489,4489,4489,6570,4489,4489,4457,4489,4489,4489,4489,4489,4489,2408,4489,4456,4456,4488,6569,8682,12874,17033,23308,40146,44372,48500,42193,21065,14825,10664,8583,8551,6471,6470,8584,8551,6470,8550,38067,48664,57116,57116,52857,52890,23406,27665,34003,21196,31826,27600,27568,36019,44471,44504,36018,42325,31793,23342,12810,10729,12810,6471,6503,8584,4390,4390,6502,4357,4390,6470,6405,6406,6438,6405,10631,46517,52823,52823,52823,52791,52790,50678,40178,35953,36018,29712,27632,27664,23439,17068,14988,12842,8648,6535,21229,19051,14889,23211,23146,21033,27437,29614,14824,10630,23243,27403,27435,27501,6470,6438,8583,8519,6373,8486,4227,4259,4259,2146,98,2211,4292,2211,2179,2179,2179,2211,2211,99,99,131,2276,6438,6405,4292,4325,8519,6373,10599,10437,10598,10566,35661,44242,42292,44307,35791,37903,29452,29452,31597,18984,46584,52922,52922,42325,23373,8551,12776,23308,21163,25389,21195,25484,14884,45},
{32057,34137,34137,34137,34137,34137,34137,34105,34105,34137,34137,34137,34137,32057,30009,27993,27993,27994,27994,27993,27993,30042,30042,30042,30042,32089,13039,21457,6667,19280,6601,19280,4522,25651,23668,23668,32055,29974,29974,17395,21588,29942,23636,29909,31990,25683,25716,31990,31957,23636,6798,2571,2571,4619,4587,4587,4587,4587,4554,4554,4554,4554,6634,4586,2441,2441,2441,2441,4521,2441,2441,2409,4457,2377,2409,2409,2376,2409,4457,4457,4457,4457,2376,2376,2376,2376,2408,2376,2408,2408,2408,2408,2376,2375,2375,4423,12873,19178,29680,40146,31727,54937,50744,27568,21229,14955,14987,12842,12842,14923,16971,21229,27568,23374,27535,23276,29646,50776,48663,48696,33906,23407,36052,23309,21228,46551,27568,17036,27601,21262,40245,38099,44470,36019,23374,14922,10697,17036,8616,10664,12841,8551,8583,21260,14922,10631,21228,12744,6405,8550,10663,46517,52855,52855,52823,52823,52823,44339,50743,48631,48664,48664,48664,50745,44439,25553,17101,15021,12842,10663,27567,16905,16904,14759,23210,21065,40114,25389,6404,4259,8485,21064,31597,16904,6372,8485,14759,25291,8486,8486,6372,6372,8518,2179,98,2211,4292,4292,4325,2212,2179,2179,2211,99,2179,2179,2211,2179,4292,4325,4390,12809,6471,8550,10599,10631,10599,27372,42162,29517,27436,23178,27436,33742,29452,33710,18984,48664,50809,52922,42325,19148,10663,17002,21195,19082,21195,16969,17032,12836,40},
{32057,34137,34137,34137,34137,34137,34137,34105,34137,34105,34137,34137,34137,32057,30009,27993,27993,30074,30042,30042,30042,30074,30074,30042,30042,32122,13104,23538,6635,23440,25488,29909,19410,34103,29975,29942,27894,27829,34167,23701,32087,38361,27861,34135,36216,29909,29942,29910,36183,27862,8911,6733,6732,6732,6732,8780,6700,6700,6700,6667,8747,6667,6700,6667,6635,6635,6634,6635,6602,4554,4554,4554,4522,4522,4522,4522,6570,6570,4489,4521,4521,4489,4521,4489,4489,4489,4489,4488,4488,4521,4489,4456,4456,4456,4456,8649,17034,16969,35985,40179,38066,35986,46584,44503,29713,31826,27632,25519,27632,25552,27600,33874,38131,29679,25420,29615,27500,42259,44405,25452,19115,17003,40245,27502,21130,40180,14955,14956,27601,19181,27632,29680,38099,46551,33938,33938,27567,27600,31793,25454,25486,29680,21260,23373,27566,21195,23308,25421,17002,12808,14889,46517,54904,54903,52855,52823,52823,35953,48598,46551,48664,42293,40148,46552,31761,27601,17068,17036,14889,14824,16872,14759,12678,12646,16838,21032,37969,31695,6340,4292,10565,29451,16904,12646,25321,10533,14694,29484,6340,4292,6404,14759,12711,2211,2179,2211,2212,4292,4324,2211,2211,2179,2211,2212,2211,2244,4325,66,2211,4357,4357,4325,4293,6405,6438,10664,10631,31564,42096,33646,8452,23210,33710,33710,29484,25258,23178,55035,50809,48697,40212,14922,10695,10630,21163,19082,21195,17002,14951,12836,47},
{34105,34137,34137,34137,36185,34137,34137,34137,34105,34105,34137,34137,34136,32057,30009,27993,27993,30042,27994,30042,30042,30042,30042,30042,30041,32122,13104,25618,8748,25585,17101,31990,29876,42521,36249,34168,34168,38361,40409,38361,55100,53021,42554,40408,38328,36248,36248,32055,40344,34104,13104,10926,8878,8878,10926,10926,8845,10893,10893,8813,8812,8812,12974,10893,8780,8747,8748,6667,6667,6635,6667,6667,8715,4522,6667,6634,6602,6634,6634,6634,6569,4521,6601,6569,4489,6569,6569,6569,6601,6569,4489,4488,4488,6536,4488,6568,17066,16969,29679,46485,52824,46518,38164,50745,50808,44437,48663,50776,50776,52824,48598,48663,38065,25420,17000,29646,42292,48598,27533,19146,17034,10663,25388,23210,21065,27599,19149,17036,23375,23407,17068,19116,31696,46551,46583,48695,48599,40147,48664,48631,40180,50744,48631,42293,50744,50712,44438,50744,48663,48663,48631,50743,52855,52855,52823,52823,52823,52823,46485,46519,48632,46551,44438,48665,42358,16970,23342,16971,14824,16872,16904,14791,14791,16872,16839,18951,37904,35920,12678,8485,14759,27274,12679,12712,16839,10565,16904,29517,6372,4292,4292,6404,12711,4292,2211,2211,2212,2244,4356,2211,2179,2178,2179,4357,4325,4324,4389,4325,4357,2276,2211,4260,2179,4292,4324,8518,8518,27306,44080,42065,8452,16871,35888,35855,29484,31532,27436,55035,52890,48697,35987,10664,8583,14824,21195,19082,17002,8550,14886,12804,42},
{34105,34137,34137,34137,34137,34137,34137,34105,34105,34105,34105,34137,34105,34105,30009,30041,27994,27994,27993,30042,30041,30042,30074,30042,30074,30041,13104,27699,10828,27633,17134,29876,19378,38295,34135,31990,38296,40376,40408,38296,40343,42456,40408,40375,36183,36183,34102,34102,40376,34135,15184,10958,10959,10958,10958,10958,10926,8845,8813,8813,8812,6732,8813,6732,6700,6699,6667,6667,6667,13006,6699,6667,6667,6602,6667,6634,6602,6602,6602,4521,6602,4521,6569,6601,4521,6569,6569,4521,6601,6569,4488,4488,4488,6569,6569,8649,17067,29679,35985,44339,54904,52824,29680,38132,52824,52856,52824,52824,52824,50744,48631,46518,57017,40178,46517,48630,57050,52824,29678,10727,8550,14888,29517,21032,21065,29648,21262,8584,21294,21294,14955,14890,12744,46550,46517,50743,42292,12744,48663,44405,10631,50776,44438,12744,48663,46518,14888,48663,48630,46550,46518,50775,52823,52823,52823,52823,52823,52823,44372,44406,48632,48632,46551,48664,48664,14890,25455,23342,14792,14857,10631,10631,10631,12711,19049,21097,35855,40016,16871,10598,18920,21033,14792,12744,14856,19017,6372,18984,8452,4292,6405,10630,10663,4324,2211,2211,2211,4292,4325,2179,98,2179,2179,8518,6405,4357,4357,4357,4357,2244,2211,2179,2179,2179,4292,6437,6405,18952,33645,42129,12678,10565,21162,40146,33710,44210,42129,55034,52922,46583,33906,12777,8550,12744,21195,21228,17003,10664,16967,14917,41},
{34105,34137,34137,34137,36185,34137,34137,34137,34137,34105,34105,34137,34137,34105,30009,30041,27993,30042,27994,30042,30074,30042,30074,30042,30074,30073,13136,27731,8683,25553,19214,29876,21458,36182,27763,23537,29844,34005,34037,31956,25618,27763,31956,34037,36117,27731,23537,29843,34036,23570,17265,10958,8910,13039,8878,10958,10958,10926,10926,8877,10926,10926,13039,10893,6732,6732,4619,6667,4554,8715,6667,6667,8780,4521,6667,6635,8683,8683,8715,6666,4521,4521,4521,4521,10795,6633,4521,10795,8714,8714,12908,6569,6569,6569,6601,12875,27632,42325,40146,40113,52791,46517,40212,31728,52824,52889,52856,52824,52824,52824,50744,42324,46485,48598,59195,61276,61276,59163,46517,23372,29678,25453,16839,21064,8485,14956,19182,10729,27600,31793,38131,36018,33873,48631,46550,50711,42260,14857,48663,44373,10663,50776,44405,12744,48663,46518,19050,50712,48598,14856,16937,46485,52856,54904,52823,52855,54903,52823,27501,44406,48664,42293,33873,50744,38035,17035,19180,12744,21132,8550,6470,8550,8518,8453,19017,21065,35791,35822,18919,6404,10630,6372,6373,14824,14857,25323,14693,8453,6404,4292,8485,6405,8518,4324,2211,4292,4324,4357,6405,2211,2244,4292,4356,4292,4325,4389,4389,2244,2212,4324,2244,2211,2211,2211,4324,4357,4324,19050,31662,33645,14791,16872,38033,33840,31597,31532,31565,52922,50809,46584,40180,12809,8583,16970,25421,25454,23407,23342,16967,12836,35},
{34105,34137,34137,34137,36185,34137,34137,34137,34137,34105,34137,34137,34137,34105,30009,30041,27994,30074,30042,30042,30041,30074,30074,30074,30042,30041,13136,25650,8715,27634,25554,29908,19409,38230,31956,27731,34069,36150,36182,36150,29811,31923,36117,38198,38230,31924,27730,34004,36150,17264,13071,10991,10959,8878,10958,10958,10958,10958,10958,8877,8845,10926,13006,8845,8780,6700,6699,4619,4554,4522,6634,6634,6634,4489,6635,6634,8683,6602,6634,8747,4489,6601,4521,4521,12940,8714,4521,6601,8682,6634,10827,6569,6569,6569,6601,8649,27632,38099,31695,35953,52824,48598,52856,52824,50776,52889,52857,52857,52857,52857,48664,42292,54970,48630,38098,42292,52857,46517,46550,44404,46517,42292,16871,29450,12679,8616,12908,6568,31859,31793,48663,50743,50744,52824,48631,50711,40180,14922,48663,42292,10696,50744,44405,21228,48631,46485,31727,48599,46518,16937,14825,42324,52855,52855,52822,52822,52823,52790,48598,42292,50744,50777,50712,25421,21163,21228,35986,38067,14889,8518,6502,10663,10598,8518,12711,21065,29419,33742,16773,12646,8485,4259,4292,6372,14824,31532,33546,14725,6372,8485,4291,6405,6373,10598,8453,4324,4389,4357,6437,4292,2179,4324,6437,6469,6437,10664,6437,6405,6437,6437,14922,27502,4292,4325,2212,4292,4292,8550,23308,18984,12678,12613,31694,29581,27371,29419,23144,50809,52922,48664,42357,25454,12776,25422,25389,29648,14955,19148,10725,12836,41},
{34105,34137,34137,34137,36185,34137,34137,34137,34137,34105,34105,34137,34105,34105,30009,30042,30042,27993,30042,30042,30042,30042,30074,30074,32090,32089,21589,27829,13006,27698,23539,42456,17264,38262,63455,57181,57149,50875,46682,44602,38296,46682,46649,44569,44601,42489,40344,50875,46682,38426,17265,10991,10991,8910,10991,10990,10990,13039,10990,10990,10990,13039,8845,8845,6732,8813,6699,4586,2506,4554,6602,6602,6602,2441,6634,6602,6634,6602,6602,6634,4521,6569,6569,6569,10827,6601,4521,8682,8682,8714,10827,6537,6569,6569,8617,14988,23405,38098,33873,38131,29646,57050,52856,57082,54970,38033,29646,31727,29646,29614,38066,48631,54904,52824,44405,40244,52825,44437,46517,46517,52824,31694,27436,27370,14792,10697,12875,10827,19246,29680,48631,50711,50744,50744,48663,50711,40179,17002,48631,42260,12776,50744,44373,19050,46485,48501,38001,48631,44373,19050,14890,44404,52823,52855,52855,52823,52823,52822,48565,44372,46485,42292,35889,21131,12744,38132,42325,38067,14824,10598,12711,14792,16872,14792,21065,25258,29419,33742,16806,12613,14759,12646,6340,6340,21130,33612,33709,23144,6372,23242,8452,4292,4324,8517,10630,2243,4292,4357,6470,8518,6437,2211,4324,6470,8518,8518,4324,4357,6470,6470,10663,14791,12678,8550,4324,4325,8518,12776,12743,27468,27468,18952,33807,33839,31694,18919,21031,50838,50869,52954,46551,27502,29518,25453,23308,21229,16971,22856,12677,14916,33},
{34137,34137,34137,34137,34137,34137,34137,34137,34105,34105,34137,34105,34105,34105,30009,30041,30042,30042,30042,30042,30042,30074,30074,30074,32121,32089,30041,34136,27830,34038,29910,29845,25717,38295,42521,38295,52988,44601,46714,44602,36215,36248,38296,44601,46682,38263,38263,48762,44601,36281,15185,13071,10959,10958,13071,15119,13071,10990,10990,10958,10957,10958,10958,10958,10958,8813,6700,4587,2474,4554,6602,6634,6634,4521,6634,6634,6602,6602,6634,6602,4521,6602,6569,6602,10827,6601,6569,6569,8682,6601,8649,6569,4521,6569,6569,8649,29679,46485,57083,40211,42259,52824,52856,57083,38033,29614,29582,27501,27533,27501,31759,50711,23243,35921,52857,48696,59196,52857,44372,46485,46518,42226,19016,31597,16872,12810,21326,14988,12875,31760,48631,50711,48631,50744,48631,50711,38099,19051,48663,42260,14889,50744,42292,19050,50646,40081,35888,33775,35953,19115,14889,42324,52823,52823,52823,52823,52823,52823,48630,42259,38034,38001,29452,19115,10630,14824,19049,21162,18984,18952,25259,23146,23113,23145,27306,18952,25226,31564,18918,16806,16838,14725,21097,21098,38001,44243,33645,27338,14725,27403,21096,6404,8517,12742,31692,10630,2244,4325,8550,21228,14856,2243,2243,6437,10631,10663,4292,6404,4324,6437,14889,12711,14791,16936,17002,23308,23307,33872,25420,35953,42291,42226,44372,38033,25421,14758,18951,50807,52920,46551,46551,29614,29517,33872,21195,21261,12777,24807,14725,14916,44},
{34105,34137,34137,34105,34137,34137,34137,34104,34137,34105,34137,34137,34105,34105,30009,27993,30042,30042,30042,30074,30074,30074,30074,30073,30074,30073,32089,36282,32057,36217,32056,34169,34104,44504,42489,36183,48762,46682,46681,42521,38295,40344,42489,48762,48762,44569,36182,44569,44601,38328,13103,10991,10958,10958,10958,10958,10958,10958,10958,10925,8877,8877,10958,8845,8845,8812,6667,6635,4586,4554,6602,6602,6570,4521,4489,6634,6634,6602,4553,4521,6602,4521,4521,6569,10827,6601,6569,6569,6601,8650,8714,6634,4488,6569,6569,14988,17068,33872,40146,48631,38066,35953,52889,46485,27501,27533,29582,27533,25420,23307,27469,44373,23308,25421,33841,35986,44373,40179,25389,33808,44406,31695,14759,31565,16839,8648,27600,42357,44470,46550,48630,48663,44405,44372,46518,50711,38066,19083,48663,40179,14889,50744,42260,21163,48533,44210,29549,50744,48598,42324,42260,50710,54904,54904,52823,52823,52822,52823,54936,33807,40179,31760,33841,21195,12775,19017,21065,29484,29484,25226,23146,21033,25226,29484,29517,19017,29420,29483,23111,20999,23177,21096,18952,14759,42227,44145,31467,16839,16806,23145,14791,10598,14823,12743,27467,21161,6372,4324,12776,21130,21130,8517,2211,6437,14824,19114,14856,25387,4356,6470,19114,29581,21130,10565,8550,10598,21227,31792,21162,33807,42259,44405,44404,33840,21163,14824,18984,48696,52955,44470,44471,29615,35921,27534,16970,19181,19051,35436,26950,16933,51},
{34137,34137,34137,34137,36185,36185,34137,34137,34104,34137,34105,34137,34105,34105,30009,27961,30042,30042,30042,30074,30074,30074,32122,30074,30074,30042,32089,32089,32089,32056,32056,32056,32056,42424,38296,31989,38263,40344,40344,40376,34037,34037,38295,40343,40376,38263,31957,36150,42456,40408,42489,34103,38328,38328,38296,34134,40376,38296,34070,36215,36183,36183,31924,34069,34037,27730,23504,27730,27698,19246,25553,25617,25585,17100,25617,25585,21392,17166,25585,23505,17166,19279,21391,23439,19246,21391,21359,21359,17101,23439,21359,17133,15053,21294,23375,21327,23440,29712,50710,46550,23275,40146,52856,54937,48631,48631,48598,46518,31759,21226,25421,42260,50711,42292,57115,46518,33873,38099,29680,31793,27567,31728,27503,23211,21131,10729,25487,46518,50744,50776,48630,48630,46485,38098,40179,50711,38034,19115,48631,40147,16969,50711,40179,21131,46453,44178,29582,50744,50711,48630,48598,52791,54904,54904,52823,52822,52822,52790,52824,35953,19081,19082,14856,16969,21195,18984,27371,25258,23145,18920,25227,25226,25226,23145,27274,16904,27371,33710,23111,16838,23243,25323,21065,18952,42162,39919,27339,16871,16872,21032,16903,10630,21162,23307,29515,23274,12711,6437,12711,25258,33677,23210,6372,10630,19049,21129,27501,21129,10630,10662,21162,35920,33807,18985,10598,8517,10695,29679,16969,29614,38098,40179,40017,25227,14889,14824,19049,50809,52955,46551,46551,29614,40180,23341,10696,17067,36051,33874,31436,29386,60},
{34137,34137,34137,34137,34137,34137,34137,34105,34105,34105,34137,34137,34105,32057,30009,27993,30042,30042,30074,30074,30074,30074,32122,32122,32122,32122,32089,32089,32057,32056,32056,32089,34136,34136,34104,34104,36184,36184,36184,36184,34103,34103,36151,36183,36151,36151,34103,34135,36183,34070,36183,34070,36183,36150,34070,34070,36150,36150,34037,36118,36118,36118,40278,42391,44438,48631,48664,48631,40245,36117,25520,23407,21294,17134,19214,19214,19181,17069,21294,21294,17101,19214,21327,21359,17100,25552,44503,42358,38164,35986,27566,25420,23308,35921,38099,31663,19018,19147,31760,35921,44372,50645,52824,54970,52857,54937,52889,52856,50711,36017,42324,44372,50743,46517,57082,48599,44438,31793,25454,23374,21293,23342,19148,31793,31794,12842,27567,46550,48663,52857,48631,46517,42324,46518,48663,48663,35953,21195,48631,40147,21195,48663,40179,19082,48501,39951,21162,50711,48630,48630,48598,52823,52823,52823,52791,52790,52790,50710,52823,42324,23275,14888,16937,14888,16969,19017,31597,25258,21065,18953,23114,21001,25194,21033,23146,23178,31564,35822,23143,14791,25454,23308,21097,16839,40081,33612,44241,18984,14759,33678,14726,8550,14889,29646,44303,27436,10630,10630,21195,31629,37936,25291,10598,10630,23243,33806,31726,23210,8550,10630,21162,31726,31694,19050,2211,4291,8582,31792,17034,27566,42259,42292,40049,29355,14759,12743,16936,48663,52922,46551,44470,29615,31630,23341,10696,10696,6437,27567,19050,25421,75},
{34137,34137,34137,34136,34137,34137,34137,34105,34105,34105,34105,34105,34105,32057,30009,30042,30042,30042,30042,30074,30074,30074,30074,32122,32090,32122,32089,32089,32057,32057,32056,32088,34136,34104,36184,34104,34104,34104,34104,34104,34104,34104,34104,34104,34103,34071,34103,34103,34103,34071,34070,34070,34070,34070,34038,34038,34038,34038,34037,34037,34005,34005,33972,31859,36052,44470,33907,40213,50777,46584,17036,14988,17102,12908,19214,19182,21230,21262,19181,21229,21229,19181,19149,21229,19181,21261,23341,21261,31760,33904,17033,23307,33807,23308,44405,33743,21131,14922,27502,21163,23276,27437,33808,44373,52857,54937,52824,52824,52856,52856,52856,52824,52856,48630,31760,42292,52857,44438,54970,54970,54969,52889,50776,36052,36053,14988,25487,44470,46518,48598,48631,33840,16969,40179,52856,48631,35953,23308,48631,40179,25389,50711,40147,23275,48566,37904,23243,50711,48630,48598,48598,50743,54904,52823,52823,52790,52790,50710,52823,48598,33840,33840,31727,21130,16936,29452,33677,27339,21033,18952,23146,23146,25258,23210,23210,29483,33677,35789,23111,12711,25421,23276,23210,27370,42193,44112,50614,21097,14758,25225,12646,6437,8582,25420,48465,29515,12678,10631,16969,31727,40081,29484,10630,14856,23307,35952,33839,21064,10565,12711,19114,31726,33807,18985,6404,8517,16969,27566,19114,25453,35953,27501,35856,31630,12678,10631,14856,50809,55068,52890,48631,29647,29582,21228,14922,10663,8518,10663,21227,19082,46},
{34137,34137,36185,34137,34137,34137,34137,34105,34104,34105,34105,34137,34105,32057,30009,30041,30042,30042,30074,30074,30074,30074,32122,32122,32122,32089,32121,32121,32089,32057,32056,34137,34137,34104,36184,34104,36152,34104,34104,34104,34103,34103,34104,34104,34103,34103,34103,34103,34103,34071,34071,34103,34070,34070,34038,34038,34038,34038,34037,34037,34037,34005,42359,36020,38165,33971,27633,34004,12907,10762,10828,10795,10795,21327,19084,23342,19083,27502,21196,21164,19083,19051,19051,19051,21163,19115,21228,29648,33840,42259,38097,38000,35888,27502,35855,29549,14857,19115,35954,25389,23308,23308,25357,40147,52824,50743,50711,48598,50743,50744,48630,48598,38033,38033,29680,50777,57115,44405,54970,54969,52856,52824,46518,29713,29746,19149,19148,31794,31794,40212,48663,33841,16937,38099,52856,48631,35921,23308,48631,40147,27502,50711,40114,27437,46388,37969,23341,48663,44437,35953,35953,48630,52855,54904,52823,52790,52790,52790,52824,42324,46518,35953,23275,25421,35921,31532,16839,12614,12614,14759,23178,21065,23145,21032,16904,23210,33644,35855,21064,12743,25421,23308,25324,23145,48565,46257,42064,23145,14758,27370,18951,8550,10663,27533,44272,33741,14823,10631,16937,33775,37936,27436,10630,10695,29614,40113,31694,21097,14824,16969,19082,29613,38000,16937,6372,10663,10663,27534,19082,21195,18984,16839,18984,31662,16904,8517,14791,50842,55068,50841,46551,29647,40212,10696,17068,12809,23341,21195,21228,14856,26},
{34105,34105,36185,36185,34137,34137,34105,34105,34105,34104,34105,34137,34105,32057,30009,30041,30042,30074,30042,30074,30074,30074,30074,32122,32122,32122,32089,32121,32089,32057,32057,34137,34137,34104,34104,34104,36152,34104,36152,34104,34104,34104,34104,34104,34104,34103,34103,34103,34103,34103,34103,34071,34070,34038,34038,34038,34038,34038,34037,34005,34005,34005,38165,33939,40245,42390,27665,36117,15054,15054,12909,12908,21360,25520,19084,21196,21196,25422,23309,19116,19083,19083,17002,19083,19083,19083,21228,29615,40179,42194,42226,40146,21196,38100,33742,27436,10695,31825,46486,48532,48532,50678,42193,48565,48598,50678,46485,38033,40114,35953,33839,35920,29647,31728,27600,50776,54969,40179,54937,50711,48631,48631,42292,21327,29778,21229,14923,21262,19181,33873,50711,31760,16969,38066,50711,48631,33840,21195,48631,38099,27534,48663,38066,29517,46421,37969,23374,48631,40147,19050,16970,42292,52823,52823,52823,52823,52791,52790,52824,44405,38066,44405,29615,27501,27501,23211,10565,14759,18953,18952,23178,25291,27339,23146,25258,27403,39951,29484,18918,21098,23309,25389,23243,23113,42161,44177,33580,27338,14759,23177,16871,8582,8582,25419,44271,31661,6404,8582,14888,31661,33742,27403,8485,14856,23275,35919,37967,23177,14824,17001,35920,38033,33808,16936,10630,16969,12776,25388,19082,14856,23211,18984,16839,19017,23242,14791,12678,48697,55068,52922,46551,29647,38002,10697,12842,6503,21195,17002,21228,10661,27},
{34137,34105,36185,36185,34105,34137,34137,34105,34105,34105,34105,34137,34137,32057,30009,30042,30042,30042,30042,30074,30074,30074,30074,30074,32122,32122,32090,32121,32089,32057,32089,34137,34136,34104,34104,34104,34136,34136,34136,34136,34104,34104,34104,34104,34104,34103,34103,34103,34103,34103,34103,34071,34070,34070,34038,34038,34038,34038,34038,34037,34005,34005,31891,31826,44406,40213,23472,25553,25521,21327,21360,19279,17069,42325,21228,21164,19083,25422,23341,19084,17003,19083,17003,19083,19083,19083,23276,31728,35986,40114,40081,29615,10729,29680,33710,33774,21227,38067,52791,48500,31662,37968,27436,21162,19049,19114,14889,14889,12808,12808,12775,21164,33906,25390,21228,55002,57082,42325,42260,50776,40179,50777,19148,19181,29713,25390,12843,19215,19182,29680,44406,27534,14921,40244,48663,48663,33840,23308,48631,38066,25421,48663,40147,40180,48566,38001,21196,48631,38066,19082,19083,44372,52823,52823,52791,52791,52790,52790,52823,48631,44373,50711,46551,44405,27468,23178,23145,25323,31597,29485,25291,23178,31565,27339,29484,25226,35823,27371,16805,12645,19082,25389,23276,21032,46420,42032,27306,18984,18984,33579,14725,6469,10662,29612,44272,31661,10663,4357,12743,35952,33807,27436,10566,12744,25421,44339,40080,23145,10598,12776,29614,33775,38001,21097,8517,14856,29647,31727,19082,17002,21131,19017,16871,23243,25388,18984,14790,48696,55068,52922,46551,31695,31566,19115,12842,8519,21195,17002,21227,12773,39},
{34105,34137,34137,34137,34105,34137,34137,34105,34105,34104,34105,34137,34105,32057,30009,30042,30042,30074,30074,30074,32122,32122,32122,32122,32122,32121,32121,32121,32089,32057,32089,32089,34137,34104,34136,34104,34136,36152,36184,34136,34104,34104,34104,34104,34103,34104,34103,34103,34103,34103,34070,34070,36119,34070,34038,34038,34038,34037,34037,34037,34005,36053,44439,40245,40245,31794,29714,29713,46616,31793,46519,44471,29583,44309,21196,21196,23277,25390,25390,19084,19084,17003,17003,19083,19116,19116,23341,33841,35954,52791,50678,31760,12842,17035,33710,40081,52791,46486,54904,52790,46420,46485,40178,33807,33807,40179,16969,8550,12776,12840,23341,17003,29681,29681,23309,46518,38098,38099,52857,54938,54938,19116,14955,14923,25390,27470,10762,19214,19182,21262,25487,23374,25487,38099,50743,48631,33840,27502,48631,35953,23341,48631,38098,44373,54905,38034,21228,48631,38066,16970,19083,42292,52823,52823,52823,52822,52823,52790,52823,48598,46486,48599,29614,48631,44405,31597,27339,27306,23146,10598,18953,23146,23145,27274,25258,23113,31532,33677,18919,21064,25323,25421,23276,21033,44275,44145,37870,18984,12645,37740,14726,10630,10630,27532,40079,27403,8517,8582,12775,33807,33807,27436,8518,8517,16936,42226,37967,18951,10630,12711,25388,33807,40146,16904,10630,12775,21195,23307,21195,21195,23276,19017,18985,12678,23210,21097,16936,48664,52922,52922,46551,31727,27469,17035,23407,21196,23308,17002,19114,12804,34},
{34137,34137,34137,34137,34137,34137,34137,34105,34105,34104,34105,34137,32057,32057,30009,30042,30042,30074,30074,30074,30042,30042,32122,30042,32122,32121,32121,32121,32089,32057,32089,34137,34137,34104,34136,34136,34136,36184,36152,36184,34104,34104,34104,34104,34104,34104,34104,34103,34103,34103,34103,34070,34070,34070,34038,34038,34038,34038,34037,34005,34005,33973,33939,38132,44406,44438,48696,46551,48697,40245,48664,46551,44275,33841,21164,19115,21163,19115,19148,19115,19084,19116,19083,19116,19116,19116,23309,33841,46486,50679,38066,38066,17036,17004,23179,40081,54839,50646,54904,50612,50644,46386,46386,46387,46452,31726,14856,10728,12808,21292,38164,38133,35987,40147,29615,40179,38034,40212,52889,50744,38067,14923,12843,12843,25390,27470,14923,19182,19181,19181,21294,19214,25455,36018,50711,48631,33840,29614,48631,35986,27534,48631,48598,44373,50711,38066,23373,48631,40147,21195,19050,42259,52823,52823,52823,52823,52822,52758,52791,44372,46517,52889,54937,44373,27501,23178,23113,23145,16872,8518,16872,21065,21032,25194,25226,21032,31565,29419,21000,23112,27436,23340,23243,23178,44274,44081,33644,23177,12710,37805,21064,12775,21227,31758,42160,42161,14823,14889,27566,31726,33775,27501,12775,23340,38065,44339,37935,18952,10663,27566,31759,33839,40146,19049,12743,19114,33873,29613,21195,19115,27469,14823,21162,12645,27468,18984,14823,46583,55067,52922,48696,31728,21228,12744,25422,19050,23276,17002,17032,12804,39},
{34105,34137,34137,34137,34137,34137,36185,34137,34105,34104,34105,34105,32057,32057,30009,30042,30042,30074,30074,30074,30041,32122,30042,32122,32121,32121,32121,32121,32089,32057,32089,34169,34137,34104,34136,34136,36184,36184,36184,36184,36152,34104,34104,34136,34136,34104,34103,34103,34103,34103,34103,34070,34070,34070,34070,34038,34038,34038,34037,34037,34005,36053,44439,42293,44438,48631,48664,44438,48664,31793,46551,48599,42163,44341,21228,19115,19115,19050,19083,19083,19115,21196,21262,23374,21261,21294,23374,27600,35955,27568,17068,14955,12843,17036,16938,31695,48599,48533,48598,50645,50612,50645,50645,52725,46452,25388,14857,10663,10695,25486,40212,42326,21196,21229,29647,46453,54937,44373,57115,52825,38100,12811,12810,14956,23310,25357,8551,4423,8584,14956,21294,19148,23374,38098,48598,48631,33808,27534,48631,35953,27534,48599,57050,42261,50712,35953,25422,48598,48598,48631,48598,48597,52791,52823,52823,52823,52822,52790,52790,46517,46518,54937,54938,29647,33840,25291,21033,16839,16839,23146,25323,25291,31565,31532,35823,25290,37871,27274,21000,23145,21097,23275,19049,25258,50645,48338,35757,21064,14758,37838,16903,12776,21227,38097,35855,37967,12710,12776,21227,33807,33775,23308,10663,14889,29646,40145,33774,16871,12743,19082,29646,35952,42259,16904,8550,17001,31727,29613,21227,19082,21163,14758,21162,12678,27436,18984,14823,44470,55035,52890,48729,31728,27600,27600,21228,19115,21163,19050,14919,12804,37},
{34105,34137,34137,34137,34105,34105,34137,34105,34105,34105,34105,34105,32057,32057,30009,30042,30074,30042,30074,30074,30041,32122,32122,32122,32121,32121,32121,34169,32089,32057,32089,34137,34137,34136,36184,36184,36184,36184,36184,36184,36152,36152,34104,34136,34136,34104,34103,34103,34103,34103,34103,34103,34070,34070,34070,34070,34038,34038,34037,34037,34005,36053,38100,36020,42358,38132,33906,33906,50810,42326,50777,50777,40147,44340,21163,19115,19083,19083,19083,19083,19116,25455,21262,21229,21229,19148,21261,23407,27633,19149,10762,14988,10730,17069,17036,29680,42358,46518,25389,31759,29646,31728,40082,46453,44373,27469,40147,12744,6502,23406,31761,42358,21131,19116,23309,46484,42291,44405,52890,35986,31794,10730,10762,12843,19116,25357,14858,4391,4455,12907,21294,19148,21196,38131,48631,48631,29614,25421,48599,31760,25389,48598,54937,40115,48566,35953,25454,48598,48598,48598,35954,40179,52822,52791,52823,52823,52790,52790,50678,44372,44404,46518,27533,44405,35921,18985,21065,25323,31630,37936,31565,21065,25259,21066,21130,46420,35758,31532,21064,23145,21032,21097,19049,21065,44307,46257,33611,21031,12678,35757,16903,8517,10630,38065,35887,42160,10565,10695,21195,35855,40113,19049,10695,16969,31759,42226,40113,19049,12776,23340,29613,33839,42226,18952,8485,16936,27533,29581,23276,19115,19115,16871,21129,18951,21130,16903,12710,42357,55035,52922,50777,29680,29713,33906,23308,19115,19115,19082,14952,12804,37},
{34137,34105,34137,34137,34137,34105,34137,34105,34105,34104,34104,34105,34105,32057,30009,30042,30042,30074,30074,30074,30074,32122,32122,32122,32122,32121,32121,32121,34137,32057,34137,34137,36185,34104,36184,36184,36184,36184,36184,36184,34136,36184,36184,34136,34136,34136,34136,34103,34103,34103,36151,36119,34070,34070,34038,34070,34038,34038,34037,34005,34005,36085,36052,35987,42293,40213,33874,33873,48664,31760,46583,48632,42260,50679,21196,19115,19083,19083,21164,19083,21163,27600,19182,17036,19214,14956,14956,14988,14956,17069,14923,17036,12843,17004,17036,29713,38164,44405,31760,48664,23342,21230,19117,17068,17068,17035,8583,4357,4357,19116,33874,38100,21229,23342,25454,46485,38067,48663,48631,38166,17069,12876,8617,8649,19149,29648,19148,6503,8649,10794,19214,21326,25455,27568,46583,48631,29615,29615,48630,33840,25453,50711,48599,46486,48598,33873,25422,48598,48598,40179,17068,40179,52790,52791,52791,52823,52790,52790,46484,46485,46517,54937,44405,35985,35953,29582,31630,27404,21065,14824,16904,14759,8518,12744,14825,40113,31597,29419,21031,23178,25291,25421,21195,18920,46387,48370,35692,20999,16871,35854,16936,14856,14856,33872,35854,37934,10598,10663,21130,35952,40179,27468,12776,12776,31759,44371,38000,16904,14856,25452,33839,33839,40114,16904,14823,25421,38065,35952,27533,17002,23341,35920,29614,23243,27468,25290,16871,42357,55067,52922,52890,25487,25487,31793,25421,19147,21195,17002,12807,12804,41},
{34137,34137,34137,34137,34137,34137,34137,34104,34137,34104,34105,34105,34105,32057,30009,30041,30074,30074,30074,30074,30074,32122,32122,32122,32121,32121,32121,34169,34137,34137,34137,34137,36217,34104,36152,36184,36184,36184,36184,36217,34104,34136,36184,34136,34136,34136,34135,34103,34103,34103,34103,36119,34070,34070,34070,34038,34038,34038,34037,34037,34005,36117,40278,36020,46518,42293,38100,31793,35987,33907,38100,33906,31761,35954,21164,19083,19083,23276,23309,21163,19115,29681,12876,4424,14956,2376,6569,10730,6504,14955,8584,12875,6504,8617,14955,33938,48696,44438,46517,52858,31794,19149,17068,17003,14955,17003,12777,4390,6503,21294,29681,31566,31761,29680,27502,48598,31825,44438,46551,38231,12876,10730,4456,6569,12842,25487,23374,6504,6536,10762,17101,19214,27632,25455,46518,48631,29647,29680,48598,33808,29647,48663,46453,48599,46485,33873,27534,46518,44373,21228,17036,38098,50710,52790,52823,52790,52790,50742,50743,46485,48598,52857,46550,48565,40112,23243,16936,10631,12679,4324,8550,12711,10599,14889,12711,29516,31597,27338,23145,23210,23243,23309,25324,23178,46387,44176,35693,16838,16904,31661,14758,10662,12775,33872,35854,40046,12646,14856,23210,33806,31694,19049,12776,14921,33839,40113,33774,14791,12743,23340,33871,35952,35888,14823,10662,23275,33839,33840,25453,21227,23340,36017,31695,25388,31662,27436,48631,48696,52922,55003,52890,33874,38165,29713,25454,19147,23308,17034,12807,12836,53},
{34137,34137,34137,34137,34137,34137,34137,34104,34105,34105,34105,34105,32057,32057,30009,30042,30074,30074,32122,32122,32122,32122,32122,32090,32121,32121,34170,34170,34137,34137,34137,34169,36217,34136,36184,36184,36184,36184,36185,36217,36184,34136,36184,34136,34104,34136,36183,36151,34103,34103,34103,36119,34070,34070,34070,34038,34038,34038,34037,34005,34005,36085,29746,33907,48599,50711,48566,52825,36052,33906,46518,27600,21262,33906,19115,23277,21196,19083,25389,19083,19083,31794,17101,10763,14956,8649,10730,12843,8649,14956,10730,14924,8649,10698,14923,33906,50744,50712,48598,46518,21196,17003,14858,14890,14857,12777,12777,8584,8584,12809,40212,33646,35986,48632,23275,44371,29680,42325,59229,38199,17102,8683,6537,15055,12875,27633,25455,10729,8649,8682,12875,10795,25487,25455,44437,48631,35987,36051,48630,33873,31760,50743,46486,44372,48598,33840,27534,46518,29647,10664,8583,38066,50678,50742,52791,52791,52790,50742,46485,35953,35985,44405,31759,38032,44305,21129,10630,10598,10631,4324,8485,10631,8518,8550,10631,23243,35790,27339,25258,18952,16937,23308,23243,23146,42193,39950,37806,18887,23210,35855,14791,10663,12743,35920,31596,42157,12645,16904,25355,35887,35952,23275,12776,14921,33839,42226,35920,14823,12743,19114,35920,38065,35920,14823,8549,17001,31727,33839,23340,16969,21228,38098,31727,27501,35790,29516,52954,50809,55003,57083,52890,38100,44472,27567,19147,19115,19115,14856,14920,14917,50},
{34137,34137,34137,34137,34137,34137,34137,34104,34137,34105,34105,34105,32057,32057,30041,30074,30042,30074,30074,32122,32122,32122,32122,32121,32121,32121,32121,34169,34137,34137,34137,34169,36217,34136,36184,36184,36184,36184,38233,36217,36184,36184,36184,34136,34136,34136,36183,34103,34103,34103,36151,36151,36118,34070,34070,34070,34038,34037,34038,34038,34005,38165,38165,31826,48598,42293,40180,38067,38100,29681,42325,29649,21262,25455,21229,23342,25422,27502,29583,27535,23277,33874,14988,21327,17069,17069,17069,19214,17101,19214,14989,17069,15021,14988,17101,31826,46519,50744,50711,42227,27403,25388,23341,23309,16970,14890,12777,10697,8616,10665,25389,33613,38067,36019,27469,42325,29712,42325,59229,36086,17069,10763,8682,17135,10762,27601,27568,10762,8617,4326,6439,10795,25519,31826,44470,48631,29647,31793,48598,27534,29615,48598,40049,42195,48598,31760,29582,46518,29615,8551,6438,40178,50677,50710,52790,52823,52790,52790,50711,44405,35953,35953,27500,33840,33774,23274,6404,10598,8518,4324,8453,12679,8485,6438,8583,21162,39984,27371,21000,25291,54970,23308,21163,18985,42226,39983,35661,18919,21032,31629,12613,10598,10695,35887,29484,35787,10662,17001,23242,37967,31694,19049,12743,12743,38033,44339,33775,12711,14856,25453,38065,40145,33807,16904,10662,23307,33872,35952,23308,14889,19115,36018,31760,27501,33678,29484,52922,48729,48696,57083,52890,33808,48632,33906,12809,10664,12777,6437,17033,8611,49},
{34137,34137,34137,34137,34137,36185,34137,34137,34137,34105,34105,34105,32057,32057,32057,30042,30042,30074,30074,30074,32122,32122,32121,32121,32121,32121,34169,34169,34137,34137,34169,34169,36216,34136,36184,36184,36185,36184,38265,36217,36184,36184,36184,36184,36184,34104,36183,34103,36151,36151,36151,36151,36118,34070,34070,34070,34038,34038,34038,34038,34037,42358,40213,33906,46518,42293,31761,48697,40213,23407,23440,23407,23407,23374,19181,19148,35986,33840,31761,27535,21230,38133,12875,23375,6505,6537,6504,17037,8584,14956,6504,10730,8681,6504,17068,33874,46551,40179,42260,35856,23145,44372,57115,44438,17068,19181,12810,10697,12842,10665,21196,35694,44373,44405,48597,42292,29712,42325,50777,21328,12908,12876,8682,15022,8584,25487,31695,10697,8616,4357,6471,10762,25519,21229,42325,48598,29614,31792,48598,25421,35953,48533,39919,42097,46550,31760,29582,46485,29582,8519,8486,35952,50645,50678,50710,52790,52790,52790,52855,48631,44372,46518,33774,35920,21162,21130,23243,17002,10598,8486,8485,12711,8453,6437,6470,14824,35823,21032,18887,29517,61277,25389,21066,23146,48468,39951,29387,18952,23210,29515,12645,10630,16969,37936,31661,31595,10598,10631,23275,38000,35887,16936,10695,14856,40146,42258,35888,14791,12775,23307,35919,40145,33742,18984,10663,25388,31759,35953,27469,14889,27534,31792,31760,25421,29452,27404,52922,50809,46616,55003,52857,31728,42260,25455,14925,6504,17100,8584,12774,10724,38},
{34137,36185,34137,34137,34137,36185,34137,34105,34105,34105,34105,32057,32057,32057,30009,30042,30074,32122,30042,32122,32122,32122,32121,32121,32121,32121,34170,34169,34137,34137,34137,34169,36217,36184,36184,36184,38232,38232,38265,36217,36184,36184,36184,36184,36184,36184,34135,36151,36151,36151,36151,36151,36151,36118,34070,34070,34038,36086,34037,34037,34005,34005,36052,40245,42293,33907,29713,38132,35987,33972,25488,23375,25488,23406,19181,25487,40179,37968,37936,33775,21229,25423,17037,21294,10730,12843,10762,19181,10763,17069,8682,10795,12875,8649,15021,31793,42325,44405,31760,31663,18952,44339,52856,42260,14955,17069,10729,8584,12810,8584,6438,25260,42260,44405,50678,50711,23406,40245,52857,17037,12876,8682,10730,17102,6471,19051,29550,8551,4390,4325,6471,12875,31793,19116,42292,46550,29582,31792,48566,29550,38001,46453,39984,37936,46518,31727,29614,46485,29583,8486,6406,33839,48597,50678,50710,50742,52790,52790,52791,44372,46518,46485,35887,27436,19017,23243,21130,25356,23276,19083,12744,14824,10630,6470,6438,6438,31695,25257,18886,23146,42260,25389,19018,18985,46387,44176,35756,14725,21129,29515,10532,8518,12678,37936,33774,31596,10630,14856,27500,35887,40113,16904,6469,10630,38066,40146,33807,14791,12743,19081,38032,40113,27403,14791,14856,25420,29614,33872,27501,19049,35920,36018,31760,25388,29452,25291,52922,50842,52922,52922,52857,31695,31598,25455,13042,10796,23407,23374,12839,14949,30},
{34137,34137,34137,34137,36185,36185,34137,34105,34105,34105,32057,32057,32057,32057,30009,30042,30042,30074,30042,32122,32122,32121,32121,32121,32121,32121,34170,34169,34137,34137,34169,34169,36217,34136,36184,36184,38233,38232,38233,36216,36184,36184,36184,36184,36184,36184,34135,34103,36151,36183,36151,36151,36151,34070,34070,34070,36086,34038,34037,34037,34005,34005,38133,46551,46486,42326,31794,44471,46551,36019,23375,25488,23407,23374,19181,21261,38034,42194,40049,40081,33841,23407,19214,42325,46584,46584,46584,40246,19247,17167,17166,17167,17167,19215,19182,31793,46551,50712,54970,33809,21129,44339,46452,44338,31760,8616,8584,8584,6535,6503,4358,8551,40180,44405,50645,42194,40212,38132,44406,17102,12843,8650,8617,15054,6503,16971,27470,14890,4325,2245,4358,8616,33939,21229,40212,46518,27534,31760,46518,33776,29614,46420,39984,35855,46518,31727,27534,46485,35921,10664,8616,35952,48597,50678,50710,52791,50710,50710,50710,44404,46453,42195,21097,27404,29582,23210,18985,14824,23178,19049,21162,29615,35986,33873,27567,27631,27501,31597,23177,21162,27502,25389,19050,23211,46388,37839,31564,14694,25323,27402,10565,8517,14824,42162,31629,27434,8485,12776,19081,33741,31694,14791,8550,8518,29646,40146,35855,10597,10695,12775,40177,42194,31629,10597,12743,16968,23307,35952,27501,16936,29582,27534,33873,25356,29452,25291,52922,52922,50777,55003,52890,31695,33744,21230,12977,12910,25520,38165,40277,19111,37},
{34137,34137,34137,34137,36185,34137,34137,34105,34105,34105,32057,32057,32057,32057,30041,30074,30041,30042,32122,32122,32122,32121,32121,32121,34169,32121,34170,34169,34137,34137,34169,34168,36217,34136,36184,38232,38233,38232,38265,38232,36184,36184,36184,36184,36184,34136,34135,34103,36151,36151,36151,34103,34071,36118,34070,34038,34038,34038,34038,34037,34037,34005,36052,44406,35987,31826,29681,33906,42293,29713,21295,21294,21262,23375,21261,21261,29647,35888,42194,42194,31695,35987,33906,40244,31760,23342,25455,14923,8650,6569,6569,4489,6537,6569,8617,14889,48599,48663,50744,29583,25388,42259,44339,42258,40179,10664,8584,8616,6503,8583,10664,8551,8518,42259,48531,46418,44339,35987,25521,23474,10795,6504,8650,12908,10729,14923,23277,16970,2277,2245,4390,4390,31793,27567,42292,44405,35986,38131,46485,40114,40147,46388,42064,40114,46485,38034,35953,44405,44373,17036,14988,40179,48597,50677,50710,50710,52790,50710,50678,40146,46356,44308,29517,29582,19017,21097,12646,12711,21066,12678,19050,38099,44469,44502,42357,33906,27469,33710,23144,18984,25454,25389,21098,25258,50549,37838,35822,16903,25420,27467,10565,12711,12710,38001,31694,29515,10630,10662,16904,37967,37968,12678,10663,12711,31694,35919,35887,10662,17002,21227,38065,35887,31662,12743,12743,19114,29613,35952,29647,19050,31760,35985,33873,27502,29517,29549,55003,55035,50744,55035,52890,31727,40213,17104,12943,17103,25552,33939,52923,19081,39},
{34137,34137,34137,34105,34137,34137,34136,34105,34105,32057,32057,32057,32057,32057,30009,30042,30074,30074,32122,32122,32122,32122,32121,32121,34170,34170,34169,34169,34169,34137,34168,34168,36217,36184,36184,38232,38264,38232,38265,38264,36184,36184,36184,36184,36184,36184,36183,36183,36151,36151,36151,34103,34071,36118,34070,34070,34038,34038,34037,34037,34005,34005,33972,38133,38132,38132,36051,35987,31761,27633,23375,21327,23375,21262,21229,19149,27535,31760,38034,31663,35986,48664,33906,42325,35986,27534,19181,10795,10730,8682,8650,6569,4456,6569,8617,12777,44438,44437,42292,35986,23277,21164,21196,19018,19051,19116,17003,19083,14890,14890,12744,6470,4357,35920,50580,52725,46451,31794,25521,23474,10795,4424,8649,12908,10730,12810,25357,25356,4390,4358,2310,2277,8583,4390,19050,23308,25422,25357,33743,27470,29582,39952,39951,31663,31695,31663,31695,31728,33841,12810,12875,38099,48565,48597,48597,50678,50710,50710,52791,44372,42129,42195,14727,40114,31629,21065,21098,19050,10533,8517,16937,38067,42357,46550,40277,38131,27469,31564,23177,19017,23308,25389,19050,21065,50581,37839,40016,16839,35920,33740,12743,23308,29614,37969,29581,29482,14856,29646,31694,37967,38000,14856,17002,31792,38000,35855,25387,12776,17002,29646,40178,40081,31662,12743,12776,19114,31759,38033,31759,23308,35985,44405,35953,29680,33775,27534,50809,50777,50744,50777,48632,31760,33841,21362,17234,27634,27633,23374,42390,35987,75},
{34137,34137,34137,34137,34137,34137,34136,34137,34105,32056,32057,34105,34105,32057,30041,30042,30042,30042,32122,32122,32122,32122,34169,34169,34170,34169,34169,34169,34137,34136,34168,34169,36217,36184,36184,38264,38264,38264,38265,38264,36184,36184,36184,36184,36184,36184,36183,36183,36151,34103,34103,36119,36118,34070,34070,34038,34038,36086,36085,36086,34005,34005,38165,38100,40180,44406,31761,31826,33874,27601,19182,19149,19149,23375,17068,19149,19182,25488,17068,21229,33906,48599,38067,38099,38067,21228,12811,10763,10763,8682,8650,6569,4456,8650,8650,10697,29648,33840,38033,46486,38067,19051,19051,14889,14889,16970,16938,23341,19148,19115,17003,17003,14857,40146,48531,54838,48499,44339,44405,25522,10828,6537,8650,10763,8682,12778,31631,27404,12745,6504,4390,4358,2277,2277,4357,8518,10696,4357,16873,8518,16840,31436,33548,8519,6470,8551,6439,8551,8584,6503,10730,29680,48565,48597,48597,50677,50678,50678,50678,44339,42129,31532,27307,35920,33742,25291,21065,18985,21130,23243,16937,35986,46551,48664,44470,40212,27469,29483,23112,21129,29583,23243,19017,29517,48468,37903,33677,19016,29548,31627,10630,14889,29582,38001,33775,19016,10630,12775,33775,38000,38066,12711,10631,16969,46420,40113,31629,14824,12744,25420,35919,40081,27468,10662,8582,17001,31791,40178,33840,33872,35985,44405,33873,27502,25324,25421,52922,54970,52890,54970,48664,29647,29518,19284,15123,29781,27665,33972,40245,42358,67},
{34137,34137,34105,34137,34137,34137,34136,34137,34105,32057,32056,34105,34105,32057,30041,30042,30042,30042,30074,32090,32122,32121,32121,34169,34170,34169,34169,34169,34169,34137,34168,36217,36217,36184,36184,38232,38264,38232,38265,38264,36184,36184,36184,36184,36184,36184,36183,34103,36151,36151,36151,36118,36118,34070,34038,34038,34038,36086,36086,36085,34037,34005,38165,42358,42325,48631,21262,19182,19149,19149,19181,19182,17101,17069,14988,14956,17036,17036,17068,23374,27568,44438,36018,40212,44406,29713,23439,23440,12908,10763,10730,12843,8617,8650,10763,12843,17069,21262,31825,40211,40147,38033,38033,40114,38033,31727,19050,19116,23341,23341,21260,21228,21196,31662,42128,50612,50547,50579,54872,25619,10828,6537,6537,12973,4423,17069,25390,25292,14793,4358,4390,4358,6471,4357,4325,6438,6438,6438,16937,6438,18985,35726,25226,6438,6405,6438,4358,6471,6471,6471,10762,19213,48598,48597,48565,50710,50710,50742,52823,46485,44274,46323,33678,23308,25355,23210,23145,10533,10566,19018,21163,35953,42324,48663,46486,38067,27437,21032,21031,23210,31760,25421,16937,23243,50613,39918,27371,14726,29581,31628,8517,10630,23275,35887,31694,25323,10663,12840,33807,35887,38066,10662,12808,21194,42226,35920,27436,14824,14889,23339,40145,42226,29581,10630,12775,17001,27565,35952,33840,36017,29646,40179,31760,27502,18985,23308,50841,52922,52890,35954,19017,29615,38066,25489,23574,27667,48729,36052,40245,42358,76},
{34137,34137,34105,34105,34137,34137,34137,34137,34105,32057,32056,32056,32056,32057,30009,30042,30042,30042,32090,32122,32121,32121,32121,34170,34170,34170,34169,34169,34169,34136,34168,36217,36184,36184,36184,38232,38264,38264,38265,38264,36184,36184,36184,36184,36184,36184,34103,34103,36151,36151,36151,34071,34070,36118,34070,36118,36086,36086,36086,34037,36085,36053,36052,40213,42325,40245,23375,21295,21294,21294,21262,21294,19214,17101,17101,17102,19181,19181,19213,27568,35987,35954,33873,38067,44406,35954,29648,19181,10763,6505,4456,8650,4456,4456,6504,10730,17101,19149,29681,33840,46485,50645,48564,52725,52725,50678,21196,16970,14890,17002,16970,17002,19050,21098,31597,40080,40048,42128,50679,29845,10763,6504,4456,17102,4423,14923,12810,10698,8584,4391,6471,4390,8649,4357,2244,4357,4325,14889,8518,4325,33840,42195,21066,6471,8519,6438,4326,6471,6471,6438,10697,12842,36018,46517,50678,50710,52790,52790,52824,40146,44242,46258,29484,21162,27469,25389,8485,4259,4259,4259,25389,38099,44405,25291,29291,25162,25258,25225,27370,21130,23276,19050,16936,23178,48533,37805,31563,14791,33775,31661,10630,14856,29614,35888,33775,21162,10663,12808,33775,38000,40179,14888,12776,21227,40178,38000,25322,12711,14889,25420,33839,33839,27468,10662,12776,21194,29646,35952,35984,38066,23275,29646,31760,27469,16840,21163,50809,52922,52922,52890,50744,31695,40147,25389,25422,23374,44439,38165,44471,40245,62},
{34105,34137,34105,34137,34137,34137,34137,34105,34105,34105,34104,32057,32057,32057,30009,30041,30041,30041,32090,32122,32122,32121,32121,34169,34170,34169,34169,34169,34169,34137,34169,34168,36184,36184,38232,38232,38264,38264,38264,38264,36184,36184,36184,36184,36152,36152,36183,34103,36151,36151,34103,36119,34070,34070,34070,36086,36086,36086,36086,36085,36085,36085,38165,46584,44503,29681,23375,21294,21294,21294,21294,21294,19181,19149,19149,19149,19149,19181,21262,23342,35986,21228,35922,40147,38099,38034,33873,23342,14988,10762,8650,8650,8617,6537,6537,8682,12875,21228,40245,42260,44339,48532,50645,50645,52725,48598,29615,21228,23308,19082,17002,17002,19082,19050,19050,21098,19050,19017,35954,34006,17103,6537,8650,10795,6504,8617,8585,8649,8616,6503,4423,6471,8616,4390,2276,4357,4357,17002,6405,4325,33809,44340,14792,6470,8551,6471,6438,6471,8584,6503,8617,10729,21261,31760,48598,50710,50742,52790,52791,48565,46355,52564,29388,23308,25356,23275,21163,12711,12679,10566,16969,38098,42259,23210,16839,14726,18886,20999,20999,21097,25356,23211,19017,25290,48500,35725,31498,21097,25387,27402,10630,14856,31695,35855,33775,19017,12776,16969,33839,33806,35921,10630,12776,21227,42258,35919,29581,10630,12743,23339,35919,33806,27403,12710,14856,19049,29614,33839,35952,27468,14824,14856,31728,23276,12646,16969,46583,50841,52890,57115,52890,31727,29517,25421,23341,21196,29680,46584,55068,21229,73},
{34105,34105,34105,34137,34137,34137,34137,34137,34105,34105,32056,32024,32056,32057,30041,30041,30041,32090,32122,32122,32122,32121,32121,34169,34169,34169,34169,34169,34169,34137,36217,36217,36184,36184,38232,38232,38232,38264,38264,38232,36184,36184,36184,36184,36152,36184,36151,34103,36151,34103,34071,34070,34070,34070,34038,34038,34038,36086,36086,36085,36085,36053,38165,42358,31826,27600,29713,27568,25455,25488,27600,27600,25487,23407,23374,23375,23374,23407,25487,25455,52890,50809,40147,42293,40212,40212,33906,25454,14988,10763,8682,8682,8682,10730,8682,19246,19181,31825,40278,21196,31663,48532,46419,50612,48532,46518,42292,44437,44437,38098,38098,38066,38033,35985,38033,35984,35920,33807,42325,36151,15054,10828,10762,12843,8617,10730,10697,8649,10697,8616,6503,8584,8649,6471,2244,4324,10663,12776,4325,6405,40213,42260,8518,8519,8551,8551,6471,6503,6503,6471,8649,10664,17003,21228,44405,46550,50710,52790,50710,44372,46356,52662,27404,25388,42292,31727,21163,19050,19082,16904,16969,35953,42292,36018,38033,29548,25257,23144,21064,21098,25421,25389,21098,29549,52758,33644,33643,27435,38000,29515,8517,10598,27436,37935,33775,21130,10695,14856,31694,38000,38066,12710,12743,27533,44372,35855,25356,16904,14888,27533,40112,33774,23177,10630,17001,23308,29613,33904,38065,27501,21195,14856,29615,21130,12679,14824,48664,52922,52890,55035,52922,31727,33743,25389,23341,21195,21228,31793,50810,38132,68},
{34105,34105,34105,34137,34137,34137,34104,34105,34105,34105,34105,32056,32057,32057,30041,30041,30041,32122,32121,32090,32121,32121,34169,34169,34169,34169,34169,34169,34169,34136,34168,36217,36184,36184,36184,38232,38232,38264,38264,38232,36184,36152,36184,36184,36152,34103,36151,34103,34103,34103,34071,34070,34070,34070,34038,34038,34037,34038,34037,34037,36085,36053,38133,40245,29713,42358,35987,31793,29648,29681,31826,31826,29713,27600,27600,27568,27567,29648,31761,29713,46551,44373,42260,40148,44374,29713,19182,17036,12843,10795,12876,10795,10763,10730,12907,38197,33907,40212,40212,17002,25389,44274,42226,46387,46419,27469,46517,48630,52856,50743,52823,54903,52823,52822,52790,52790,52823,52790,52824,40312,14989,17134,12843,12876,8682,8649,10730,10729,8616,6503,10729,19116,6503,4390,4357,2244,16970,4325,2212,12744,35986,40147,4324,4292,4358,6438,4390,8616,2278,6471,6471,8616,14890,21261,42325,42325,48598,52790,52823,42259,46420,50646,35790,31662,42226,29549,31695,25355,21129,19017,21162,31760,42325,35953,31727,25388,21065,18951,16838,16969,25389,21163,21098,33677,50612,42062,27337,23144,29548,31562,10598,10630,31694,37935,29581,19082,10662,23308,33807,38001,33840,8549,14823,31727,44404,38032,25355,12743,14889,29613,35887,31726,25323,10630,14921,25420,33839,35920,38097,27534,21195,19115,33841,27469,10598,12743,46583,52922,52922,55003,52890,29679,40179,25421,23341,21163,21196,21163,23341,25453,75},
{34137,34137,34137,34137,34137,34104,34104,34105,34105,34105,34105,34105,34105,32057,30041,30041,30042,30041,32122,32121,32122,32122,34169,34170,34169,34169,34169,34169,34169,34136,34169,36217,36184,36152,36184,38232,38232,38232,38264,36184,36152,36152,36152,36184,34104,34103,34103,34103,34103,34103,34071,34070,34070,34038,34038,34038,34038,34038,34037,34037,36085,36053,42326,46519,46519,54938,40212,33939,33906,36019,33939,33939,31826,31794,31794,31793,31793,33874,33907,35987,40212,48632,48632,42325,19116,17069,14989,12876,14989,10795,12908,12875,10795,12876,14987,31858,25488,40179,40179,33840,40179,42259,38066,54970,40179,25421,48597,54969,54969,54937,46518,27631,42292,46454,44372,42259,40178,38065,46518,42457,15054,15021,14989,12909,10762,8617,10762,10730,8649,6504,17003,33808,6470,6503,6471,4358,19148,2245,4292,21228,40147,33776,4324,4292,4325,4390,4390,8617,8617,6504,6471,8616,12810,19149,21229,23375,42259,52791,52791,38033,46388,52727,40015,38000,42192,44141,35823,29516,14790,16903,23210,33840,40146,33775,29614,25355,18984,16872,18984,19049,27534,21130,23178,37903,52693,39983,27337,27403,35854,25289,12710,19082,31661,37935,29581,19082,10663,14888,29613,31661,31695,12679,12775,31759,44404,33741,23177,10630,21195,29613,42291,38032,31629,10662,17001,27500,33807,33839,38065,21195,21163,19115,33841,25356,19050,12711,44503,52922,52890,52890,52857,29647,35889,25421,25421,19147,21227,19115,19147,19114,68},
{34137,34137,34105,34137,34137,34137,34105,34105,34105,34105,32056,32057,32057,32057,30041,30042,32090,32090,32089,32121,34170,34170,34169,34169,34169,34169,34169,36217,36217,34136,34168,36217,36184,36184,36184,38232,38232,38232,38264,36184,36184,36184,36184,36184,36152,36152,36151,34103,34071,34071,34070,34070,34070,34038,34038,34037,34037,34037,34005,34037,34037,34004,44406,44438,40212,40212,40245,23375,14956,15021,17069,19182,19182,23375,25455,27535,27568,31729,33841,33841,31728,31761,38067,40180,29648,29648,21261,19149,29713,17068,14956,14988,10730,12875,31826,29746,25488,40211,50646,33840,25421,46453,40211,52824,46518,33872,48597,50710,48565,48631,46486,25388,42292,29615,27533,23275,21163,19083,12744,36085,15054,15054,15021,14989,12876,8617,12843,12810,10697,10730,12842,16970,8551,6471,8617,12745,8584,2277,4292,23308,38034,25324,6405,4292,4325,2244,4390,8649,6537,6471,6504,8584,12842,21262,23375,19148,40179,50711,52823,48631,44372,50646,35853,42225,46353,50414,35822,27436,14791,21162,29581,33807,27436,21129,31661,25290,18919,16839,16871,19049,27501,21130,25291,40016,52661,39951,29450,23209,29548,29548,12710,12775,29612,35854,33741,16936,10663,16969,31628,33775,25356,12678,14856,33807,44371,37902,21063,14856,29581,31694,44404,40081,25355,8549,19113,29646,31758,38032,38098,17001,19115,19115,33841,25356,27502,19017,44470,52890,52890,50777,52857,29615,29518,25389,25453,19147,23308,21195,19147,19082,71},
{34137,34137,34105,34137,34137,34105,34105,34105,34105,34104,32056,32056,32057,32057,30041,32090,32090,32090,32090,32122,32122,32121,32121,34169,34169,34169,34169,34169,34169,34136,34168,36217,36185,25619,25587,25555,25587,25587,25587,23507,23507,25587,23507,23539,23506,23506,23506,23506,23506,21458,21426,21426,21425,23474,21425,21425,23441,23473,23441,23441,23473,23440,31762,27568,27601,27601,19182,12909,10828,12908,12876,12876,10795,10795,10795,10763,17036,23277,23245,23244,23244,21164,21197,21164,14890,12809,12809,12777,36051,17068,14988,12810,4423,8617,35987,27633,27600,38098,48598,35920,42226,48597,50678,52791,48631,31727,48597,54904,54904,50743,54904,54904,54936,40179,29647,23276,6438,4357,6438,10729,12908,12908,12843,10795,10762,10762,12810,10762,10730,10730,10697,10729,6471,4390,10697,14793,6471,2277,4357,29615,35954,19018,4325,4292,2244,2244,2309,6536,8585,2310,6503,8584,10762,14923,33874,33808,35953,35921,46517,52791,40114,29549,37966,35886,42127,48301,33741,27403,23242,21129,31661,33774,29548,23144,31596,23177,18919,14693,12645,16936,23276,19049,25356,44274,50581,39983,18918,23242,29580,25290,10598,14888,35919,40048,40145,12710,10598,16936,37998,35887,29582,10598,14856,27501,48532,42128,18919,12742,27501,33839,44339,38000,27435,10630,27598,33872,33872,38033,38065,19082,19082,17002,25389,21130,27469,19050,44470,50841,52890,55003,54970,31695,38034,25421,25454,21195,23341,19115,23308,19114,71},
{34105,34137,34105,34137,34137,34105,34105,34105,34105,34104,34072,32057,32057,32057,30041,32090,30042,32090,32122,32122,32121,32121,34169,32121,34169,34169,34169,34169,34169,34137,34136,34168,34136,15022,15022,15022,15022,14990,15022,14990,14990,12942,12942,10861,10861,10861,10861,10829,10862,10861,10861,10861,10829,10829,10829,10861,8780,10828,10861,10861,10829,12876,12877,12909,12877,12877,12909,12909,12877,14989,15021,14988,10763,8715,8715,10763,10763,10731,10731,14891,19052,14923,14923,14923,14923,14890,19116,23309,31826,12842,8649,4391,4390,4391,31826,27568,29746,50744,59196,31727,25356,50711,54970,59163,50743,63388,57016,54936,54936,23275,31761,21228,21228,23277,19116,19116,19149,14923,21262,14956,14988,14956,10730,12875,10730,10730,10730,8649,8617,6537,6504,6536,6503,4423,8584,10665,4391,4423,8617,40115,40050,12679,4325,4325,2211,2212,2277,4390,4423,4391,6503,8584,10762,14922,33906,38099,33808,38032,21097,19016,16936,16904,27468,42193,46352,48366,35887,27468,19016,25190,29414,33542,35654,33577,29483,23177,16806,20999,25291,23308,23340,21130,23210,44274,50549,39983,25224,21096,31661,21096,10598,14791,33806,35757,38032,10597,10630,19081,42224,33774,25323,10565,16936,25388,44307,37967,23177,14790,14855,33872,44339,35887,29483,10630,23307,31694,33872,35952,38033,18984,16937,19115,21228,14857,16937,19050,44438,50809,50744,55035,54970,31727,40147,25421,27502,21195,23340,19114,23308,19114,70},
{34137,34137,34105,34137,34137,34105,34105,34105,34105,34105,34105,32057,32025,30009,30041,30042,30042,32090,32122,32122,32121,32121,34169,34169,34169,32121,32121,34169,34137,34136,34136,34136,34104,15022,15022,15022,15022,14990,14990,12942,12942,12942,12942,10862,10862,10862,10829,10861,10861,10829,10829,10829,10829,8781,10861,8781,8781,10828,8781,10829,10828,10828,12876,12909,10828,10828,12877,12909,12908,15021,15021,15021,10763,10763,8683,10763,8682,19149,23375,19182,23310,23375,19149,21294,19214,19181,21294,19149,23407,19182,17069,21262,14988,19149,25520,27600,25520,38132,36019,25487,23309,33841,38132,40213,38067,40147,38034,40180,40147,33873,19116,19149,12777,12777,12777,10729,12778,10729,10729,10730,8617,8649,8616,8584,8584,6536,8584,6504,6504,6504,4423,4423,4423,4391,12745,6407,4391,4391,6439,35694,37839,8518,6438,4358,2212,2212,4390,2277,4390,4423,6503,8616,12811,17036,33939,27535,31760,48598,35952,23274,21194,14823,14888,25387,42127,46286,33774,23275,23177,46152,46216,41993,42093,33675,29484,25258,18951,18887,25356,27534,25389,21130,25258,50646,50549,41998,14725,25290,40112,18984,12678,10598,27435,35724,35854,12678,16936,27468,42257,33807,23243,16936,31694,40146,42226,40048,18984,21226,33840,38065,40146,33742,29515,25356,35985,35920,35920,35920,38065,19017,12711,17002,14921,12776,19018,19050,44438,52955,48728,55003,55003,31727,27501,25421,23341,19147,23340,19082,23308,19082,66},
{34137,34137,34105,34136,34137,34104,34105,34105,34105,34105,34105,32057,32057,30009,30041,30041,30042,32090,32122,32122,32122,32121,34169,34169,32056,32089,32089,34137,34137,34137,34137,32023,23506,15022,14990,15022,15022,15022,15022,12974,12942,12942,12942,10861,10861,10829,10829,10861,8781,8781,10829,10829,10828,8781,8780,8780,8781,10828,8780,8780,8780,10829,10861,10828,10828,12909,12908,14957,14957,14989,17101,15021,12876,10795,17102,23440,8650,12810,19181,19117,17069,21262,17036,19149,19149,17068,19181,17036,17069,19149,17036,19181,17069,19149,19181,17068,19181,19181,19149,19181,17036,21229,19181,19149,21294,19149,19149,21262,19181,21262,19116,14923,12745,10697,12778,10697,10729,10697,10697,10730,6536,8617,6536,6536,8584,6504,6536,6472,4423,4423,2343,4423,4391,4358,12777,4326,4358,4358,10599,31435,33581,6438,4325,2245,2244,2212,2244,164,4390,4423,6503,8616,10730,12875,19181,14890,27567,46550,46518,46550,38065,19146,14888,10630,23210,42094,46419,29549,27469,25323,29548,29548,33710,27371,23112,23144,18951,14758,27469,21195,21130,21066,23145,52726,46290,41998,16838,27370,38000,16968,25421,35953,40047,37902,33773,14791,38066,31726,35854,31726,19049,19081,38033,44404,40145,37935,18984,35920,42258,42258,42258,38000,29514,33839,38065,42291,40146,38097,38065,19049,19017,21195,17002,12712,16969,19050,42390,52923,42390,48696,55035,31695,31695,25421,23340,19147,21260,19082,23308,16969,70},
{34137,34137,34137,34136,36185,34137,34105,34105,34105,34105,34105,32056,32057,30009,30041,30042,30041,32090,32090,32122,32122,32121,32121,34169,23571,21459,21427,23540,23539,23539,23539,23539,21426,19215,19215,19247,19215,19247,19247,17167,17167,17167,17167,17167,17167,17167,15087,17167,15086,17134,17167,17167,17167,15086,15086,17134,15087,15087,17167,17135,17135,19183,17134,19214,17134,19215,19215,19215,19182,19247,19215,21263,19214,19215,21295,31858,19182,17035,19181,19149,17068,21230,19149,19149,19181,17069,19149,17101,17068,19181,17068,19149,19181,19149,19149,19149,19117,19149,19149,19149,19181,19149,19149,19181,19149,17100,19148,19181,19149,19149,17068,12842,10729,10697,10697,10664,8617,10697,10697,10697,8584,6536,6536,6536,6536,6504,6504,6472,4423,4423,4423,4423,4390,14922,8552,4358,4358,4358,23178,31467,27275,6439,4325,2277,2277,2244,2244,6503,4423,6536,6504,6536,8617,10729,21262,12842,25487,48631,46518,48696,46551,36050,29647,14889,14856,25258,40046,29514,14791,23274,23275,27338,39781,37703,37768,31463,21031,18952,25388,23276,23243,21033,25258,50613,41999,41933,12613,31628,31662,10598,14855,27467,42127,29482,23242,8517,21162,14790,29547,31661,19049,29646,38065,44404,35919,40113,21129,35952,42225,38065,42226,31694,31660,38065,38065,40211,40146,40145,38065,35920,29614,19082,19050,14824,19017,19082,44470,52955,46551,50777,55003,31695,40211,25421,23308,19147,19147,19114,21227,17034,71},
{34137,34105,34137,34136,36185,34137,34105,34105,34105,34105,32057,32024,32024,30009,30041,30042,32090,32089,32122,32122,32122,32089,32121,32056,19313,10861,10861,12975,10862,10862,10894,10862,10829,8781,8748,10796,10796,10797,8716,8748,8716,8716,6668,8716,6668,6668,6668,6668,6635,6668,6668,6635,6636,4587,6635,6635,6635,6667,8748,6635,6635,6603,8683,8683,6634,8683,8650,10763,8650,10795,10763,10763,10763,12843,12908,42358,23407,17036,25487,25520,21262,25520,23375,21294,27601,19182,23375,23407,19181,25520,19181,19182,25488,19182,21295,21294,17101,23440,19181,19213,21326,19149,23375,21262,19181,23407,17068,23342,21262,19181,23407,19181,17036,10729,6503,8584,6503,6536,6503,6503,6536,6503,6503,6536,6504,6504,4423,4423,4423,4423,4391,4423,4391,16971,4390,4358,4358,8551,35758,31467,18953,6439,4390,2277,2244,164,164,8616,8649,10729,6536,6536,8617,10697,38100,23440,27567,46518,44373,48664,46583,40180,31695,23341,12809,12745,29483,27369,6438,12678,29548,33709,33577,27273,23178,23177,18952,16839,25421,23373,21163,18985,37936,52726,37709,41933,16805,35789,27436,10533,21064,23209,46322,31564,16936,14791,16903,16935,35886,35887,16904,33807,40211,42258,37967,35887,27468,33839,38033,38032,35920,29581,44306,42324,38098,40146,38097,42258,40146,40178,29615,16970,14889,14857,16905,16937,44438,52922,46551,55068,57116,31727,35889,29647,23340,21195,21227,19114,19115,19082,68},
{34137,34105,34105,36185,36152,34137,34105,34105,34105,34105,32024,32024,32025,30009,30041,30042,32090,32090,32122,32122,32122,32121,32121,29976,17232,12910,12974,13007,10926,10894,10894,10894,12942,10894,10861,12941,10861,12910,8781,10861,10829,10861,8780,10861,10829,10829,8781,8781,8781,8781,8749,6700,8781,8748,8780,8748,8780,6667,6700,8748,8780,8780,10861,12941,12908,10763,12843,12843,10795,17069,15021,8682,6536,6536,10763,36052,12778,12777,48632,50777,44471,55035,44471,44471,48697,14988,40310,31891,17101,50810,19214,25488,44504,12907,42423,29713,12842,46649,14955,25519,42359,14955,44504,29746,21326,50810,17068,31793,38165,14956,25488,15020,19214,19149,17004,17036,17036,14955,14955,14956,14923,14923,12843,12843,12843,10762,8714,10730,8649,8649,6568,6568,12810,14923,6536,6536,6504,21196,27307,33580,16905,6471,4390,4358,4357,2277,2245,2277,6503,8649,8584,6536,8617,10697,27568,14922,25421,44437,40211,44438,46518,40212,31760,27502,14922,12777,12777,12744,8551,6405,37871,29484,21031,25225,18918,23144,21096,27403,23308,23341,21163,21066,37935,52629,41902,31467,18952,33773,27436,8484,25290,31596,37870,33710,21097,19049,35853,37966,31596,31629,18952,23275,40179,40177,33806,37999,29581,31694,35887,38000,33806,31661,21162,19082,35953,38065,40146,40178,40178,33775,29550,21163,14857,14857,16969,14889,42358,50809,46583,55003,57083,31727,29485,31695,27566,23341,27534,21195,19115,19115,124},
{34137,34137,34137,36185,36153,34105,34105,34105,34105,34105,32024,32024,32057,30009,30041,30042,30042,32090,32122,32122,32122,32121,32121,29976,21426,17167,21425,15120,13039,10926,13039,13007,15087,12974,13007,12974,13007,12974,10926,10926,13007,12942,10894,10893,12942,10893,10894,10893,10861,10894,8781,10861,8748,10861,6700,8813,8715,10861,8748,8781,8780,8748,15086,12940,19214,10730,10730,12811,10763,10763,8617,4424,4391,4423,4391,23407,10729,10665,42293,44471,40180,55003,48632,33971,25553,17101,36053,19215,19182,27601,19214,17101,23408,15021,25553,19181,14955,27633,14923,17068,21327,17068,25520,19182,21294,29714,17068,21294,21295,17069,21327,12940,17133,23407,14956,14955,14956,14956,14956,14956,14956,14923,14956,12875,12875,12843,10763,10762,10730,8682,6601,6601,21294,8617,6537,6537,17101,8584,31500,35661,12680,6504,4423,4390,4357,4325,4325,2277,4325,6503,8584,6504,10697,14988,14956,12778,23309,46485,42292,44372,46518,44405,35986,29615,23373,12776,12809,12744,10631,14856,37935,37871,12614,21097,42125,39980,33676,31629,23276,19115,19017,23113,40016,52596,37643,29354,25225,31628,27435,14791,21162,25323,35822,33742,21129,33838,46450,35822,31596,33742,27500,35920,40178,40145,37967,33741,21161,23242,35855,33774,35855,23178,14824,14856,23340,40146,40145,40178,42258,38033,27371,16904,14857,21098,29517,21195,42390,50841,50809,50841,52922,31727,38067,29615,31728,29679,31793,25486,23373,38131,146},
{34137,34137,34137,36185,34104,34137,34072,32024,32025,34105,32024,32025,32056,30009,30042,30042,30042,32090,32090,32089,32121,32089,32121,29944,25651,25554,31925,17297,17298,17265,19410,17265,19378,17232,19345,17297,19345,19345,17265,17265,19345,17200,17232,17232,19312,17199,17232,17232,15087,17200,15087,17200,15087,17200,15087,17199,15054,17199,17134,17167,15119,15054,23473,19247,27633,36052,33939,38132,36020,36052,36052,36052,36052,36020,31826,42358,27567,12842,33939,27633,40246,57083,48664,29746,23408,21328,23408,21328,23375,23375,23375,23375,21327,21327,21327,21327,21327,21327,21327,21327,21327,21327,21327,21327,21327,21327,21327,21327,19279,21327,21327,19247,15021,12908,19247,21327,19279,19247,19247,19215,19215,19215,17134,17134,17134,17101,17102,15021,15021,14989,14989,14988,23408,12876,12876,19181,17101,14923,37937,37872,14858,12810,10729,8616,4325,4324,2244,2244,4324,2277,2277,6503,8649,10697,12843,10697,23309,44372,44405,44405,48598,48599,40180,25421,21228,19115,14857,21227,31792,42259,40016,42097,12646,19017,29548,42125,33642,27371,12744,14890,19050,27404,42193,48273,35531,35725,35789,29515,33675,25291,19017,21064,37968,29548,25355,38032,38000,37967,29451,29581,21162,27501,35920,37967,35854,25322,19048,19049,14791,33774,37968,21097,12776,10663,14888,38065,40178,42291,42259,35953,29516,12613,12580,25193,25193,27468,44437,50842,50777,52922,52890,31727,40180,33873,29647,27534,29647,29647,31760,38131,88},
{34137,34137,34137,36185,34136,34137,32024,32024,34105,32025,32057,32024,29976,30009,32090,30042,30042,32090,32090,32121,32121,32121,32121,31991,36150,31892,46617,27796,25717,25684,27797,25716,27764,25684,25716,25716,27797,27797,25683,25716,27797,25684,25684,25683,27764,27731,27731,27764,25650,27731,27731,27699,25651,25650,25651,27699,23537,25618,25618,25618,25650,23505,31924,25586,31859,40245,40245,42293,40213,40213,40245,42325,42293,42293,40180,46519,31760,10730,19214,23440,40278,50777,29746,21328,21295,21263,19214,19215,21295,21294,21294,21294,21294,21263,21262,19214,21262,21262,19214,19214,19214,19214,19214,19214,19214,19215,19215,19214,19214,17134,17134,17101,15021,15021,19215,10762,10763,14956,10762,14988,10795,12876,12908,10763,10795,14988,12875,14988,10763,10763,14956,21262,12843,14923,14988,19213,12842,19181,46486,42195,12810,12842,10730,10794,4357,2244,2244,2244,2244,4325,4358,6503,8616,10697,10730,12876,19116,35888,42227,40082,37968,29517,19050,14824,16937,27534,44438,50744,44405,37936,42129,40016,16904,14823,23275,33742,31563,23144,14857,14825,18985,23145,44210,39821,27273,29450,39982,31595,31660,31662,27501,40081,35855,31662,23209,35887,40112,42192,37968,29548,25355,27435,31596,35919,35855,21194,12775,19081,21129,31693,33775,14791,8550,8582,10695,38065,42291,44404,42324,35952,29581,16838,14725,27338,25161,27435,42357,52922,50809,52922,52890,31728,31598,31695,31793,25421,25453,23373,19115,23308,67},
{34105,34105,36153,36185,34136,34104,34105,34105,34105,32025,32025,32025,32025,30009,30042,30042,30074,32090,32090,32122,32089,32121,34169,29878,44471,36085,52923,36184,34103,34103,34071,36216,34103,34103,34103,34136,34136,36216,31990,36248,36183,40377,34038,34070,34070,34102,34070,36150,36150,34070,34070,34070,34070,31989,36150,34037,31957,31957,31924,31957,34037,27698,36117,27666,33907,40213,40213,40245,40245,40245,40245,40245,40212,40213,31793,46519,31826,17068,19182,17134,33907,33939,14956,17101,17101,12843,19149,14955,14956,19181,10762,17068,17068,14923,19149,14923,14988,19181,10762,17068,17036,14923,17101,14923,14956,17101,10762,17101,14988,12876,17069,12876,12875,15021,19247,12876,12876,15021,10827,14988,10827,10860,12940,8682,8682,12908,8714,10827,8649,8617,10762,19148,8616,12810,21293,8649,10730,21261,44373,38001,8551,8584,4423,10762,8584,6470,2244,2212,163,4390,8616,6503,8616,8616,10729,14924,12842,21229,40147,37936,35855,25290,14856,12711,12743,21162,40146,35823,29420,27404,44242,40016,14791,17001,23275,37903,20998,18853,16806,16871,18984,25225,35725,31434,39918,27402,37870,31595,27403,14823,35887,38001,38033,25323,14791,27468,35886,37935,33774,25387,27435,31596,31628,35887,27403,14856,8550,12711,21161,29548,27404,8485,6405,12743,12776,25388,42259,42324,42291,38065,33807,21064,23210,29451,23080,31662,42357,52922,52890,55003,55035,33808,31663,25389,27567,19115,25454,21228,6405,16970,53},
{36153,34105,36153,34136,34136,34104,34104,32056,32024,32025,32024,32024,32057,30009,30042,30042,30074,32090,32090,32122,32089,32121,32121,27797,42391,36053,52890,31958,29910,29910,31989,34070,31990,29877,29910,29877,31990,27797,29877,29877,31990,27764,29877,29877,31957,29877,29844,31957,29844,31924,29844,31925,29844,31925,29811,29844,25650,25683,25651,23537,27731,23472,34004,25553,33939,40245,40245,40245,40245,40213,40245,38100,38132,40277,33939,31858,21229,12875,19150,19214,21327,27665,17068,10730,25520,10729,17036,19148,10762,19181,14922,12874,21294,12777,14923,17100,12842,19181,12842,10729,21261,12810,17035,17068,10729,19149,12842,12842,19182,8616,14956,12876,8682,12908,23441,23441,23441,23441,21393,21393,21393,21393,21393,21360,21360,21360,21360,19280,19247,19247,23407,23375,19215,25488,19182,17134,17134,33938,44438,33841,17101,17069,15021,14988,10794,10794,8649,8649,6471,6503,10697,6503,8616,8616,10697,8649,10698,14957,33841,38001,25324,25323,21098,16969,23243,14792,16840,27339,21097,23243,42162,44241,14824,19049,23275,33741,33676,20966,18918,18886,21065,27306,35692,37804,46288,31595,37869,40014,16871,29580,29548,27436,40146,21162,12743,16936,27403,35886,21130,6371,8484,23242,37999,29516,14758,8485,8517,8518,10566,14791,10565,2211,2211,4292,6405,12744,40179,40179,42259,29647,35953,27468,27468,37936,27242,31662,42357,52890,50777,55003,55003,29550,38033,23275,21195,6437,4325,12744,6437,14889,67},
{36153,36153,36153,36185,34136,34104,34104,32056,32057,32025,32025,29977,32025,30009,30042,30074,30042,32122,32090,32090,32089,32121,32121,27830,34037,27731,42424,23571,25716,21491,25684,23538,27765,23506,27764,23506,27764,21458,27764,23538,29812,25586,29812,23538,27731,25683,25618,27764,21425,27699,21392,27699,23473,27731,23505,27731,21392,23537,23505,21392,27667,19279,36118,25553,36019,42325,42325,42325,40245,31794,17036,10697,17036,21262,17068,19181,14923,10762,17036,14955,14955,40245,27535,19180,50810,8616,19148,42358,8584,31793,25422,10728,48697,6503,19148,44471,8583,36051,25455,6503,42358,12776,12809,38165,10696,29712,19181,8616,21327,2277,10697,19247,4456,12876,23441,23441,23473,23473,23473,23473,21393,21393,21393,21360,21360,21360,21360,21328,19279,19247,25488,19215,25456,21295,19214,17134,17134,36019,44438,27567,17101,15021,14988,14988,12907,12875,10827,10795,10762,10730,8648,6503,8649,12842,10697,8616,10730,12908,23310,42260,19050,27436,21097,16872,19050,14825,6373,10566,6372,14791,40016,37935,16936,19082,23308,23243,33807,21162,21162,21097,25291,27307,37773,39885,42094,29482,40048,29548,21130,19082,16937,25388,40113,16936,8518,8550,8550,29645,10565,4291,6371,4291,10565,6339,6372,6372,6405,6437,6404,6404,6372,2211,2211,2179,2211,6437,23308,12743,27501,16871,31727,35920,38001,42162,42097,33743,36018,42293,35954,44373,46518,31662,29613,25355,23275,16872,14824,14824,19082,29647,111},
{36153,34105,36153,36185,36185,34104,34104,34105,32057,32025,32025,32025,29977,30009,30042,30041,30042,32090,32090,32090,32089,32089,32089,32121,29975,34137,34137,34169,34169,34169,34137,34136,36184,36217,36184,36216,36216,36216,36184,36184,38232,36184,36184,36184,36183,36183,36151,36151,36151,36151,36119,34070,34070,34070,34038,34037,34037,34037,34037,19312,10861,8715,8715,8715,29713,40212,42325,42293,33939,10763,6569,6537,4456,4456,12875,21262,19149,17069,17101,14988,10794,31825,29680,14890,46617,10696,19148,38132,6503,29745,23341,8583,46552,8616,12809,29745,8584,29680,17068,4358,44503,12777,12777,40245,8584,31793,23374,4390,21359,4423,6536,23440,4456,10795,23408,23441,23441,23441,21392,23441,21393,21393,21393,21360,21392,21360,21328,19248,19280,23408,25520,25488,21327,19214,19182,17134,17134,40213,46518,23342,15053,15021,14988,14988,12908,12875,12875,12875,12843,12778,10697,6471,8616,8649,10697,8617,10730,14988,14858,29582,16873,23211,23177,25226,12711,16938,6372,14727,8550,19017,37903,35823,23243,23340,21228,23308,27534,27535,21228,21163,23276,18952,39853,46222,37867,19016,40145,23275,12743,10630,6469,8517,29547,6405,4292,4324,4324,6437,6404,4259,8484,4291,4291,2211,4291,6372,4324,4324,6404,4291,6404,2179,2178,2179,2179,4324,10696,19017,14725,18951,38066,27501,42259,39984,37838,33775,40244,48664,42325,48631,48631,29581,29581,21162,23275,29582,25421,23308,21162,38099,143},
{36153,36153,36153,36185,36185,34105,34104,32025,32057,32025,29976,29976,29977,30009,30042,30042,30042,30042,32090,32090,32089,32089,32122,32089,32121,32089,32121,34169,34169,34137,34137,36184,36152,36184,36184,36184,36184,36184,36184,36184,36184,36184,36184,36152,34103,34103,34103,34103,34071,34103,34070,34038,34038,34038,34037,34037,34037,34037,34005,21393,10894,8813,10861,8748,31794,40245,40245,38133,12811,10730,8650,8617,8650,2343,6536,33939,33874,21229,27601,17069,12843,33906,27535,12842,38165,17036,12875,23407,8616,23374,17036,6503,36085,8584,12842,27633,8616,23374,14955,6503,36052,12809,17035,31826,8583,25487,19181,6503,21359,6471,8617,19247,6536,12908,19247,17134,21393,19247,21328,23441,17135,21393,19280,17134,21360,19215,19247,21328,17134,27633,23407,23440,19247,15021,17101,17134,17068,48664,46518,19116,17101,14956,14988,14988,27600,14891,23310,21229,14858,10729,8583,6470,10664,10632,10664,14923,10697,12809,14824,14856,16937,14791,29420,27306,18952,8518,14889,19082,19114,17001,25356,27469,21227,21260,21228,19115,21228,23341,21196,21163,21195,31500,41933,44109,27370,14856,25420,14856,10630,6437,6405,4324,4324,4324,4292,4292,4292,6405,8485,6372,6371,4291,4291,4291,4291,6372,4292,4324,6404,4324,4356,2179,2179,2179,2211,4357,8550,27469,18950,16903,33807,16936,40147,29420,29452,37968,40244,52922,52857,52890,54970,31760,25388,12711,21196,8518,19115,10664,6502,16969,76},
{36153,36153,36184,36184,36185,34105,34105,34105,32057,32024,29976,29977,30009,30041,30042,30041,30042,30042,32090,32090,32090,32089,32089,32089,32089,34169,34169,34169,34169,34137,34136,34136,34104,36184,36184,36184,36184,36184,36152,36184,36184,36152,36184,34104,34103,34103,34103,34103,34071,34071,34070,34038,34038,34038,34037,34037,34005,34005,34005,17167,8780,6668,8780,21327,38132,42293,40213,14923,8585,8617,6537,6504,19182,2278,2342,36019,31793,19083,21294,12843,4423,33906,21228,6471,25521,2310,8649,17101,4358,19149,10697,2310,25488,4358,8649,19181,2277,14955,10729,2310,25488,2310,6503,19181,4390,17068,12810,4358,21327,4391,8649,17134,4423,10794,19247,19215,21360,19247,21328,21360,17102,21393,19247,15054,19280,17134,19215,21328,14989,27633,25520,14989,19215,12875,14988,21327,27600,46518,44373,19116,29713,19181,14955,17036,23374,14825,21130,16905,19050,19049,16937,14791,19017,21130,19018,14824,19018,16905,14824,12744,10630,10598,29452,37806,39918,19050,21195,23308,25454,23341,25454,23373,25454,23373,27567,23373,25421,25421,25486,23341,25454,25389,31630,37901,25421,21228,19114,10630,8582,6405,4357,4324,4357,4324,4324,2244,4292,4292,6405,4291,6404,4291,2211,2211,2211,6405,4356,6404,6372,4324,6405,131,2179,2211,2244,4357,8518,12711,21064,19016,19049,19016,25323,27274,18887,33742,42325,52954,50841,55003,50745,25421,19082,10696,38197,31826,17002,12810,6470,4389,61},
{36153,36185,36185,36185,34105,34105,34105,34105,32025,29977,29977,30008,30009,30041,30041,30041,30074,30042,32090,32090,32089,32089,32089,32089,32089,32089,34169,34169,34169,34169,34137,34136,34104,36184,36184,36184,36184,36184,36184,36184,36184,36184,34104,34104,34103,34103,34103,34103,34071,34070,34070,34038,34038,34038,34037,34005,34005,34005,33973,31957,23505,8812,15054,25553,44438,46584,27600,14923,12842,12810,14955,14923,19181,6503,8649,40213,29648,17003,31794,25487,4390,35954,19116,4390,23440,229,8617,17068,2277,19149,6536,2245,23408,2277,8649,17101,2245,14923,10697,2310,23407,2277,6535,17101,2310,14955,10697,4357,19247,2310,8649,15021,2310,10794,12908,10795,17102,8682,6537,23506,8682,12908,12843,8617,17102,12875,10697,14988,21229,25487,17069,12842,23408,29713,31826,33939,27567,40211,46518,33905,23374,21261,14922,29680,21196,19049,19049,16904,12711,14791,16904,12646,16871,16840,18953,14759,16871,14759,14824,14824,10598,12678,18953,27274,37805,19049,21195,17034,23308,21228,25421,21260,23405,21260,25421,23373,21260,21260,25453,21195,23340,19147,21260,23307,23340,17034,21195,10631,10631,4324,4357,4324,4357,4356,2244,2244,4292,4324,4292,2211,2211,2211,2211,4259,4259,4292,4356,4324,4291,4324,4357,131,2178,2179,2244,2309,4422,6438,12679,23242,29549,21129,16871,20999,18854,31661,42357,52922,52890,55003,54970,33841,23276,21261,38198,38100,19147,19149,10664,8615,84},
{34137,34105,36184,36185,36185,34105,34105,34105,32025,29977,29977,29977,30009,27961,30041,30041,30042,30042,32090,32089,32089,32089,32089,32089,32089,32121,34169,34137,32089,32089,34137,34136,34136,36184,36184,36184,36184,36184,36152,36184,36184,34104,34104,34104,34103,34103,34071,34071,34070,34070,34038,34038,34038,34038,34005,34005,34005,34005,33973,34005,31859,29844,27666,29714,48697,55003,40245,36052,36051,33939,31794,19182,33939,19180,8584,40213,25454,12777,40212,35987,4390,29713,16970,6471,21327,4390,8617,17068,4391,17068,10665,2310,23374,4390,10697,17068,4358,14955,10729,4390,21327,4390,6536,17101,4390,14923,8616,2245,19214,4390,8649,14988,2310,10762,10795,10730,17101,6537,4391,23473,4424,10795,8650,4423,17069,10730,6471,17003,48664,25454,31859,36019,36052,38100,35987,38100,40180,46486,54938,33873,12809,12777,27567,52857,46486,33710,27404,27338,31563,33644,27370,27370,31563,33742,29516,29483,35822,25259,23243,21130,12679,8517,14792,25161,35724,21163,21228,21228,23373,23373,27534,25486,27566,27534,27534,25453,27566,27534,25486,25453,25453,25454,23405,23373,23373,21260,23373,10663,8550,6405,4357,4357,4324,2276,2244,4324,2244,4357,4324,2179,2179,2179,2179,2211,2179,4292,6404,4291,2211,4291,4324,2179,2179,2178,2179,4357,4390,4390,6470,8518,21098,23178,14791,20935,18854,33806,44470,52890,50842,55003,54971,29648,19115,19148,23406,48665,14857,8519,19050,12776,84},
{34137,34137,34136,34137,36185,34105,34105,32025,29976,29977,29977,29977,30009,30041,30042,30042,30042,32090,32090,32090,32090,32089,32089,32089,32089,34137,32089,32089,32089,32089,32089,34104,34104,36184,36184,36184,36184,36184,36184,36184,36184,34104,34104,34104,34103,34103,34071,34071,34070,34070,34038,34038,34038,34006,34037,34005,34005,34005,33973,33973,31924,34005,29811,27633,42358,46551,54970,52825,48664,46552,25488,17069,40245,31793,44438,52825,52857,36019,48632,40212,21228,31793,36019,17004,21261,19148,17068,21229,14955,17036,19148,17035,21261,21261,14922,21229,17003,19148,25455,17003,19181,17068,17036,25455,17035,17068,17036,12874,17068,12875,12843,17036,12842,12875,12875,12842,17101,12875,6537,23473,8649,12908,12843,6536,14989,12843,10729,33874,48632,35987,33939,25455,21196,8584,6439,19182,29615,44437,46486,52890,46616,44471,16971,23309,48664,29452,27306,31531,23113,27339,27339,27338,23145,23210,31597,29517,25291,18952,23243,19017,12743,12712,12679,27273,35756,19083,14889,17002,21228,17034,21260,21195,23341,21227,23340,21227,23340,21292,23340,21259,23341,19147,21259,19147,21228,19082,19147,10662,10663,6469,4357,4357,4357,4357,2244,4357,2211,2211,6405,4292,2211,2179,2211,4292,2179,2211,4356,4324,2211,4292,4324,2179,2179,2179,2179,4390,4390,4390,4390,4390,12744,29517,16871,18919,21064,29581,44470,52922,50841,52922,55003,25422,14922,40278,27633,50778,19148,12809,16970,19082,94},
{34136,34104,36184,36185,36185,34105,34105,32025,29977,29977,29977,29977,30009,30041,30042,30042,30041,32090,32090,32090,32090,32089,32089,32089,32089,32089,32089,32089,32089,32089,34136,34104,36152,36184,36184,36152,36184,36184,34104,36152,36184,34104,34104,34104,34103,34103,34103,34071,34071,34038,34038,34038,34038,34038,34038,34005,34005,34005,34005,33973,31925,31924,27699,25553,48696,59196,46551,27535,25487,23408,12875,12940,42358,27568,19083,48631,48599,31761,48664,38067,17035,27567,23342,10664,27567,12809,14922,27600,10696,19148,23309,10696,25454,16970,8583,27567,12777,19115,23341,10696,21261,19115,14954,23406,12810,14955,21229,10697,14955,14955,4423,17100,10729,8649,17068,8649,17101,10762,12810,12875,10762,15021,14956,12843,12875,12810,12843,14956,19149,27568,14955,12810,14955,8616,8616,14988,33875,44438,42292,44373,48632,40245,12810,10762,8584,8584,19051,8551,6438,8617,12811,8617,6471,4390,8616,12778,10664,10663,19082,14824,12776,12776,21065,25161,37805,17002,19147,19179,23373,21260,23373,21260,23340,21260,23373,23373,25486,23373,25486,23372,25453,21260,21260,19147,21228,19114,23308,12776,8550,6437,4357,6437,4325,4356,4324,4357,2211,2244,4325,4389,4324,2211,2178,2211,2211,2211,2211,2211,2211,4291,4324,2211,2179,2179,2179,4390,4422,4390,4390,4390,6503,38066,38032,29581,23274,23243,40244,46551,50744,50809,52890,27502,15020,42424,27600,46617,21261,29713,21196,19083,98},
{34104,36184,36185,36185,34137,34104,32025,32025,29977,29977,29977,29977,30009,27961,30042,30042,30042,32090,32090,32090,32090,32089,32089,32089,32089,32089,32089,32089,32089,32089,34137,34104,36152,36152,36184,36152,36184,36184,34104,34104,34136,34104,34104,34103,34103,34103,34103,34071,34071,34070,34038,34038,34038,34038,34005,34005,34005,34005,33973,33973,31924,31924,27699,27633,50744,59196,46551,44438,42390,38197,14955,12908,44471,27567,25454,50776,50744,35953,46583,33906,14922,38099,33841,10697,48664,19083,21228,52922,10696,27535,38164,14921,40244,25422,21228,52824,12744,21228,44404,10696,31727,31695,21229,25487,8616,8649,21294,6471,10761,14955,2309,14987,8616,4358,21294,4358,14988,4391,6504,6504,4423,10762,8650,6471,6504,6471,6471,12875,4390,10664,21196,10697,14955,2309,6471,15021,29747,46551,44373,31761,10632,10665,6504,10697,4390,6406,4325,2277,4358,4390,8649,10730,6471,4358,8584,6503,8583,10663,17002,16969,14856,12744,16872,18952,37804,25421,19115,25453,25486,25518,23405,25486,25486,23405,27566,25486,27599,29679,27566,27566,27534,25454,23405,23340,23373,21260,21260,10728,10631,8551,6470,6470,8550,6437,6405,6405,2244,2244,4357,4390,4357,4292,2211,2178,2211,2211,2211,2211,2211,2211,4292,2244,2211,2212,2244,4390,4455,4390,4390,4390,6470,31792,38032,35919,42291,59228,57083,42325,52889,50809,55003,29647,33906,36084,36052,38165,19148,40278,23341,19082,91},
{36153,36185,36185,36185,34137,34072,34073,29977,29977,29977,29977,29977,30009,30009,30042,30042,30042,32090,32090,32089,32089,32089,32089,32089,32089,32089,32089,32089,32089,34137,34137,34104,34104,36152,36184,36184,36184,34136,34104,34104,34104,34104,34104,34103,34103,34103,34103,34103,34071,34070,34038,34038,34038,34038,34038,34005,34005,34005,34005,33973,31924,31925,29746,31827,52858,63389,59196,61276,57115,57148,21326,21326,46584,35953,57115,57082,59163,50777,50777,33874,17003,35986,33874,12841,50777,21228,25421,52890,12777,31760,38132,14889,42325,25422,21196,52791,14889,25454,44339,12777,35920,33807,21261,29681,10697,6503,21262,6503,10729,14955,4390,17036,10697,6503,21294,4358,10729,4390,4391,4391,4390,8617,6504,4358,4391,4391,4391,10697,2278,4390,6471,17002,17100,2277,4358,19149,38094,42256,37969,25423,6438,4358,4390,8584,4358,4325,4325,4325,4325,4357,4358,8617,6536,4390,6438,6470,8551,10663,16969,14856,16937,14824,21066,23048,37805,19115,21228,23373,23373,21260,25486,23341,25518,23340,25485,25486,25421,27566,23405,23373,25453,21227,23373,19114,21227,19114,21228,10695,10663,8583,8550,6470,8583,8550,6470,6469,2244,2276,4357,4390,6470,2244,4324,4291,4259,4259,4259,4259,4292,2243,2244,4325,2277,2277,2277,2342,4455,4390,4390,4390,6471,25454,35952,35887,42291,55002,52890,40211,52890,48696,52890,31727,36019,29712,50842,33907,25487,40278,21228,19115,90},
{36153,36185,36185,36185,34104,34072,32025,29977,29977,27929,27929,29977,30009,30010,30042,30042,30042,30042,30041,32090,32090,32089,32089,32089,32089,32089,32121,32089,32089,32089,34137,34104,36152,36152,36184,34136,34136,34136,34104,34104,34104,34104,34136,34103,34103,34103,34103,34103,34071,34070,34038,34038,34038,34037,34038,34037,34005,34005,34005,34005,34005,33973,29747,27666,52890,63389,59196,50777,48663,55003,21261,19181,46583,35953,59163,59131,59163,48631,42325,29680,21261,38132,33841,21195,54970,19082,19114,52890,16970,33906,48664,17002,44406,27534,21196,52856,33841,25454,42258,14889,35953,44372,21261,29680,8583,6471,21261,8551,10729,14955,4422,17036,8584,4390,21261,6503,10697,8616,8584,8681,8617,8617,6537,6536,6536,6504,6504,8584,6504,6503,6471,8584,17068,14923,8584,16872,54757,44201,44178,16971,6471,4390,4390,6471,6503,6471,4390,4390,4390,6438,4422,4391,6471,6438,4357,2277,6438,10696,17002,16937,14856,14824,23146,29419,33644,23373,23373,27566,27567,25486,29712,27599,29712,27566,27599,25518,29712,27631,27567,27566,27566,27566,25453,21260,23373,23340,23373,14889,10695,8550,6470,6470,8550,6470,6437,6405,4356,4357,4357,2309,4293,2244,4324,4324,4324,2211,4291,4259,4324,4324,2244,4357,4357,2309,2309,2277,2342,6471,4390,4390,6470,17002,35920,35920,42291,52890,50776,42325,52889,48664,52922,29647,38034,27502,42293,38133,29681,40310,19147,19147,90},
{36153,36185,36185,36185,34105,32025,32025,29977,27929,27929,27929,29977,27961,27993,30042,30042,30074,30042,32090,32090,32090,32089,32089,32089,32089,32089,32089,32089,32089,32089,34136,34104,36184,36184,36184,36184,36184,36184,34136,36152,36184,34104,34136,34103,34103,34103,34071,34071,34070,34038,34038,34038,34038,34038,34037,34005,34005,34005,34005,34005,34005,33973,29779,27666,54970,63389,59196,52890,33873,57083,25487,14954,46584,35954,57083,57050,57083,46519,40180,38131,27535,46551,33808,21163,59164,21163,23373,52890,12776,38131,42325,19115,50777,25421,19115,54904,21196,27534,44307,16937,44339,29614,17002,27567,6470,8615,21229,2277,10729,14955,6503,17068,6503,4390,21294,2277,17068,4423,4358,19214,4390,10697,12875,2310,14988,6471,4358,17069,4358,8584,14923,4390,14955,14922,14889,33774,54762,35722,37773,8518,10730,6471,4357,6470,10697,12843,10730,8649,10730,10730,8616,8584,4390,6471,4358,2277,4357,10663,14889,16937,14856,12776,21033,25096,35758,21228,17034,21260,21260,21292,21292,23405,25486,23373,23373,23405,25518,27632,23373,25486,25421,23373,25453,23340,23340,21227,23340,10696,19147,10696,14922,10663,17002,10695,8550,8550,8550,8550,4357,4325,4325,2244,4325,4292,4324,6405,4291,2178,4292,6405,4357,2244,2276,4357,4357,4325,2244,4325,4325,4390,6470,6502,27566,35952,42324,50809,48696,40212,52922,48696,54970,29647,27469,25357,25422,31826,36020,40213,17034,21195,95},
{36185,36185,36184,36185,34104,32057,32024,29977,27929,27961,27961,27929,27961,27993,30042,30042,30042,30042,32090,32090,32089,32089,32089,32089,32089,32089,32089,32089,32089,32089,34136,34104,36184,36184,36184,36184,36184,36184,34136,36184,36184,36152,34136,34104,34103,34103,34071,34071,34038,34038,34038,34038,34038,34037,34037,34005,34005,34005,34005,34005,34005,34005,27699,25554,48632,38066,59164,46486,38132,57051,40212,31793,50777,31761,42325,57083,52825,33874,42293,38099,33873,42292,33873,38066,59196,23308,31792,52890,27501,46551,38067,23309,50745,27470,35953,52823,16937,33840,44339,33775,48565,29582,12809,42325,12744,19115,19148,4390,10729,14955,6503,19116,6471,4358,21262,2277,17100,6503,4358,19214,2310,8649,12843,2278,17069,4423,2278,17069,4358,8584,12842,2245,12875,8551,16969,38001,48403,25129,33548,6405,12810,6503,2244,2244,2245,10730,12843,10698,10730,10762,10730,8617,6503,4390,6471,4390,6438,10663,12776,14856,12744,10663,23113,29355,35791,21260,23373,25486,25486,25486,27599,27599,25518,25518,25486,27566,29712,27599,29679,27534,27566,25486,25486,25421,23340,23341,25421,19147,19180,17002,19115,17034,19147,14921,8582,8550,12808,6470,4325,4325,4325,4292,4324,4292,4292,4324,4357,4357,2211,2179,2211,4324,4324,4357,4357,4357,2277,2276,2244,4357,6470,6470,19114,33840,42356,50809,48696,40179,55003,52890,54970,31727,29582,25389,25454,27535,59196,38132,21228,19082,92},
{34137,36185,36185,34137,34105,32025,29977,27929,27961,27961,27961,27929,27961,27993,30042,30042,30042,30042,32090,32090,32089,32089,32089,32089,32089,32089,32089,32089,32089,32089,32088,34104,36152,36152,36184,36184,36152,34136,34136,34136,34104,34104,36152,34104,34103,34103,34071,34070,34071,34038,31926,34038,34038,34038,34038,34038,34038,36086,36086,36086,34037,34038,29780,27666,44405,46518,59164,27535,23341,54970,59163,55002,55003,21261,19117,23375,19084,25454,36019,42325,31760,42292,44406,40179,59196,31793,29680,57083,38066,46551,46519,23341,46551,38131,35986,52824,29679,33873,48598,38033,48565,40146,25454,36018,31761,29648,19149,8584,12810,14955,8617,19149,8584,4390,21262,2277,19181,4390,2277,19214,2245,8585,12810,2277,15021,4391,2278,17069,2245,6536,12842,196,12842,10599,8486,37969,41967,27242,27307,6373,10697,6471,2277,2277,2244,4358,12843,12843,8649,10763,10762,8682,6568,6536,4358,6471,6471,10631,12744,12776,12744,10664,21033,25096,35758,21227,19082,21260,23405,23340,25421,23373,23373,19180,21293,21260,25486,23405,25454,25453,25421,23340,21292,19147,21260,19147,21228,14921,17034,10695,17002,12776,14921,10695,10663,10631,8550,6470,4325,4325,4324,4292,4324,2244,4292,4292,4324,6437,6438,2211,131,2179,2244,4357,2244,4389,4389,4357,4325,4325,4390,6438,14889,33905,35985,35985,50776,35953,54970,57115,57116,31760,33840,25389,25421,23276,25422,33873,21228,19115,90},
{34137,36184,36184,36185,32057,32025,29976,27929,27929,27961,27961,27929,30009,30041,30041,30042,30041,30041,32090,32089,30041,32089,32089,32089,32089,32089,32089,32089,32089,32089,34137,34104,36152,36184,36184,36152,36152,34136,34104,34104,34104,34104,34104,34103,34103,34071,34070,23538,8715,6667,8715,12941,15021,12973,10828,10828,8747,10795,10795,12908,12941,12908,12876,12811,52857,61276,63390,29647,27567,57051,59163,55002,44438,23406,21229,23374,21229,31793,38132,42325,40212,46519,44438,42358,50777,42293,40212,50745,42358,46551,44471,38132,46616,44438,44438,48664,42325,42357,48664,42325,46518,46551,40277,38132,44503,27600,17069,12811,12875,14988,10795,17069,10762,8649,14955,4390,17069,4423,4391,17102,4391,8617,12810,2310,14988,4423,2310,17069,2277,6504,12810,2277,16938,14760,14824,27340,39887,23048,25260,4358,10697,4390,2278,2310,4390,2244,6536,12843,12843,10698,12843,10795,10730,8617,6471,4390,6438,10631,12744,12743,10663,12744,27274,33548,31500,19147,21228,25454,21292,25453,25421,25454,23340,23373,21260,23340,23341,21292,21260,23373,21260,23340,21260,21227,17066,21292,21227,23340,19147,21195,19114,19147,19115,17001,12840,14856,6405,2244,4325,4325,4325,4324,4357,6405,4325,4292,4292,4325,4325,4357,4292,2179,2244,4422,6502,2309,4357,4389,4357,4357,4325,4325,6470,29679,35952,36018,44470,19082,46551,48664,48631,29647,31728,23308,23308,23308,21228,23276,21228,19147,84},
{34137,36184,36185,34137,32025,29977,29977,27929,25881,27961,27961,27929,27961,27961,30042,30009,30041,30042,30041,32090,32089,32089,32089,32089,32089,32089,32089,32089,32089,32089,32088,34104,36152,36184,36184,36152,36152,34136,34104,34104,34104,34104,34104,34103,34103,34071,34071,21393,6667,6635,10796,12941,13006,12941,12941,12973,10860,10860,10860,15021,15021,15021,17102,14988,40245,46486,63422,50744,46518,57083,59163,57050,59228,31761,25455,40245,38100,40180,40180,42325,44406,42325,44406,40245,40245,40245,40244,44438,42325,40277,42357,42390,42358,42390,42326,42358,44438,42325,44503,44438,42358,44471,44438,44471,31859,14956,12875,12843,12843,12876,12843,12843,10763,10763,10730,10762,10730,10730,8650,8650,10730,10698,8649,8649,8617,8617,8617,8617,6537,6536,6536,6536,19019,18953,16743,16677,31435,29354,18954,8584,6504,6504,4391,2310,4390,2212,2309,10730,14923,12811,10763,12876,12843,10698,8617,4358,6438,10631,12744,10663,12743,12776,23049,35596,35790,23373,21292,23373,25518,25454,25453,25486,25486,27567,25454,23373,27567,23405,25485,25454,25486,25454,27566,23373,23373,23373,23340,25453,23373,25453,23340,23340,21260,23308,17034,17034,8582,4292,4325,4325,4325,2211,2244,6405,6405,4324,4292,4325,4324,4325,4357,4357,2277,4390,6503,6503,4357,4324,6438,6405,4325,4324,6438,12777,35953,40211,25454,19082,50777,50777,50777,29615,21196,12776,23308,23308,23244,21228,21228,19147,87},
{36152,36184,36185,34105,32025,29977,27929,27961,27961,25913,27961,27961,27961,30009,30042,30042,30042,30009,30042,30041,30041,32089,32089,32089,32089,32089,32089,32089,32089,32089,32089,34104,36152,36152,36184,36152,36152,36184,34104,34104,34104,34104,34104,34103,34103,34071,34071,19280,4619,6634,10795,10828,10828,10828,8747,8715,8715,10795,10795,12908,12908,12876,31859,48762,50810,50810,63422,44438,57083,57083,59196,54970,59228,29648,27567,40213,38132,40180,40212,40213,42326,40245,36019,40212,40212,38099,40244,40212,40212,40245,36019,40245,40245,35987,42326,40213,38100,40245,40212,42325,42358,36019,42358,42326,21262,14924,12843,12843,12843,10763,10763,10763,10730,10763,10730,10730,10730,8650,8650,8650,8617,8617,8617,8617,8617,6537,8617,8617,8617,6536,6504,6504,8584,12744,39853,46156,31370,35693,10599,6504,6504,4423,2343,2310,2342,2244,2244,4358,12875,14924,10730,12843,12843,10763,8617,8584,4358,8583,10664,10663,10663,12744,23049,35563,31597,21260,21227,25486,21292,23341,21260,23340,23341,25453,23373,23373,25486,21260,21259,23372,23340,23340,23373,23373,21260,23373,17067,21260,19147,23340,21227,21228,19114,19114,14889,10662,4357,4324,4325,4325,4357,2179,2179,4324,4357,6406,4324,4292,4324,4324,4324,4357,8583,6470,4390,4390,4357,6470,6405,8550,6438,4357,4325,6471,14921,31760,16970,16970,50842,50841,52922,29615,16938,8518,21196,21195,21195,21228,19115,12809,87},
{36152,36152,36185,34105,32024,29977,27929,27929,27961,25913,27961,27961,30009,30042,30041,30041,30041,30041,30041,30041,30041,32089,32057,32089,32089,32089,32089,32089,32089,32089,32056,34104,36152,36152,36152,34104,36152,36152,34104,34104,34104,34104,34104,34103,34103,34103,31990,19279,8747,12973,12941,12973,12973,12941,10796,8682,10828,10828,12909,14989,12909,10795,33940,50810,52923,55036,55036,44439,40244,57082,52857,52857,59197,33906,38100,40213,38132,40180,36019,31761,31793,31793,31761,31761,31793,31761,31793,31793,31793,31825,31793,31793,31793,31825,31793,31793,31793,31761,33874,33906,33874,33906,31826,29713,17036,12811,12811,10730,10730,10763,10730,10730,8682,8650,8650,8649,8649,8650,8617,8617,8617,6537,6537,6537,6504,6537,6504,6536,6504,6504,6504,4455,6504,16905,33515,25095,31434,31500,8519,6471,4391,4391,4358,2310,2310,2212,2211,2212,8583,12876,12876,10698,12843,12843,10730,8584,6438,8551,10664,10663,12744,12776,27210,35563,33645,23340,25486,25518,27599,27599,27631,25518,27567,27599,25486,25454,27599,27599,25518,25485,27534,25453,25486,27566,25454,27567,23405,27566,25485,27566,23373,23373,23373,21227,17034,8518,4292,4324,4292,2244,2211,2179,2179,4357,6471,6438,4325,4325,4292,4324,2276,2244,4357,8584,8616,2277,4325,6471,8584,14857,8551,8551,6437,6437,12808,10663,12776,14889,50777,50809,52890,29615,17003,8583,16969,21195,23275,23308,16970,6502,82},
{36153,34137,34137,34104,29976,29977,27929,27929,27961,27961,27961,27961,30010,30009,30009,30041,30041,30009,30041,30041,30041,30009,32089,32089,32089,32089,32089,32089,32089,32089,32088,34104,34104,36152,36184,34104,34104,34104,34104,34104,34104,34104,34104,34103,34103,34103,29877,27732,25683,23506,21458,15151,15151,15151,17199,15086,15086,15086,13006,12941,12941,10860,36052,50810,52955,55036,55036,31858,36051,57115,46518,48631,59197,42358,42359,42391,42391,42326,40245,36019,38164,42358,35987,40213,40245,21228,36019,38100,38099,42358,33874,25454,42326,33906,35987,38132,36019,42358,33938,23341,42390,31793,29713,33907,14923,14923,12843,6504,14956,10763,12843,12843,10763,10763,10763,10762,12843,8617,10763,10763,8650,10698,10730,8617,10730,8617,6569,6569,6536,6536,6536,4455,6536,8519,46225,48371,46355,33743,8551,6471,6471,4391,2310,2310,2277,131,2179,2212,4357,10729,14956,12811,8649,8649,8617,8584,6438,10631,10663,10631,12744,12743,29290,35660,27371,21227,23341,25454,25486,25519,27567,27567,27566,27631,25454,23373,29680,27632,27534,25485,25453,23373,25486,25453,23341,23341,21260,23405,23373,23373,21260,23340,21227,14856,8550,4292,2212,4325,4292,2244,2211,2179,2211,6470,6503,8616,2276,4325,4325,4324,2244,2244,2244,4325,6471,6503,4293,6438,8584,8616,8551,8614,12807,10662,10663,6437,6437,12776,48729,50810,52858,29615,10665,14890,17002,21163,21227,23308,17068,12809,69},
{34105,34137,34137,32024,29976,27929,27961,27961,27961,27961,27961,27961,30010,27961,27961,30041,30041,30041,30041,30009,30041,30009,30009,32089,32057,32089,32089,32089,32089,32089,32056,34104,36152,36152,36184,36152,34104,34104,34104,34104,34104,34104,34104,34103,34103,34103,34103,21458,15217,15216,21490,17297,17296,15216,15184,15184,15184,17232,15151,13038,13038,15086,36118,50843,52924,52956,55036,25552,29713,46552,52923,46584,57116,42358,42359,44471,44471,40245,38132,40245,42358,40277,42325,42326,42326,42358,42326,42358,42358,42358,42358,42358,42390,42325,42358,44406,44438,42358,44439,44439,44406,44439,44438,42358,23343,14924,12876,12843,12843,12875,12843,10795,12843,12843,12843,12843,12811,12811,10762,10730,10730,10730,10730,10698,8649,8649,6569,6569,6568,6536,6536,6536,8584,10664,33646,46291,40082,19052,8584,6503,6471,6471,4391,2310,2277,2211,2211,2211,4357,6438,12842,14923,12875,12843,12810,8617,8584,10631,10663,10663,12744,12712,25193,33547,27436,19114,27566,25486,27567,25486,29680,27632,27632,25486,27599,27566,38196,21228,19115,27566,27534,12809,17002,25486,23373,12809,12809,23405,25453,19147,12776,17034,21260,10631,6438,6405,2244,4324,4292,4324,4324,4292,4324,4325,4357,6503,2244,2277,4357,2244,2244,4292,4292,2212,4292,4357,10696,6471,4390,6503,4324,6533,12806,14919,8582,2211,4292,10630,48729,50810,52890,21164,8616,6471,17002,21163,21196,21228,19181,19180,72},
{34105,34137,34137,32025,29977,27929,27929,27961,27961,27961,27929,27961,27961,27961,30041,30041,30041,30041,30041,30041,30009,30009,30009,32057,32057,32089,32089,32089,32089,32057,34104,34104,36152,36184,36152,36152,34104,34104,34104,34104,34104,34104,34103,34103,34103,34103,34071,21491,15185,15184,17199,13006,10925,8845,8845,8845,10925,13006,10860,8780,8780,8780,10828,14989,17069,17101,25488,23439,25520,29714,31859,31859,29745,27600,27600,42391,44471,42358,42358,42358,42326,40278,42358,44438,44471,38132,42326,42358,42358,40277,40277,42357,42326,42325,40245,42325,42358,40213,27600,29680,31760,31760,31727,31728,27502,23308,23276,23308,21195,23275,21195,29679,35986,42325,38164,40245,40245,38132,19149,31825,29712,29680,27599,19181,17036,14955,14955,10729,8617,8616,8616,8617,8617,8616,12777,18986,10632,8584,8584,6503,6503,6471,6504,4391,4390,2277,2244,2276,6438,8584,6471,6504,6504,6503,6503,8584,6470,10631,10663,12744,16969,18986,29354,37740,33710,19179,27599,29712,31858,25519,29712,29712,27599,25454,25519,23373,44405,48664,21164,17003,12809,4357,10696,14922,23374,10696,6470,10728,19148,12809,6438,8583,12777,6470,4325,6470,6437,4292,4292,2244,4292,2244,4324,4324,4325,2276,2244,4324,6405,2211,2244,4292,4292,2211,2211,2244,4325,10696,8583,2277,4324,14919,14919,17065,40276,27567,8518,14857,48664,50745,52858,16971,8617,6503,16970,21163,21195,19083,14987,21325,72},
{34105,34105,34105,29976,27897,29977,27929,27961,27961,27962,27961,27961,27961,30009,30009,30041,27961,30009,30042,30009,30009,32057,32089,32089,32057,32057,32089,32089,32057,32057,32056,34072,34104,36152,36152,34104,34104,34104,34104,34104,34104,34104,34104,34103,34103,34071,34103,21490,15185,15184,17199,13038,13103,13103,15151,13070,15151,13038,13006,10893,10893,10893,12909,12940,12941,15021,19213,23407,27601,25520,25520,25520,23407,23407,23407,25487,25487,25455,25487,25487,25487,25455,27535,27567,31761,36020,38100,38100,38100,38100,38100,38100,38132,38132,38100,38132,38100,21228,46519,36019,31761,50678,52759,50677,50645,48564,48564,48564,48532,48597,48564,48565,46486,52825,50744,48631,50711,42292,17003,40212,50711,50711,50744,36019,17068,14923,14923,14923,14923,12875,12875,12843,12810,4390,6503,12810,10664,4390,8616,10729,6471,4390,10697,10697,4390,6471,8616,6535,4390,6503,8584,4423,4390,6504,8583,10696,10663,10631,10631,10663,12744,12744,25129,37708,33678,17002,19212,27599,29712,31826,31793,35987,27599,29647,25486,33905,40147,50743,44438,19116,14890,6470,6502,10696,12809,8584,4357,6471,10696,8551,4325,4357,8583,8519,4325,4293,6470,8518,4324,4324,4324,4324,4324,4324,4324,2244,2276,4325,4324,2179,2211,4324,4292,4292,2244,2211,2211,2244,8583,10696,4325,14919,14951,19178,38130,48728,23309,31664,37970,40051,42164,19051,10698,4359,14889,19115,19115,12776,21293,25487,66},
{34105,34105,34105,32025,27929,27929,27961,25913,25913,27929,27929,27961,27961,30009,30009,30009,30009,30009,30009,30009,30009,32089,30009,32057,32057,32089,32089,32089,32057,32057,34104,34104,34104,36152,36152,34104,34104,34104,34104,34104,34104,34104,34104,34103,34103,34071,34071,21490,15184,15184,17199,19377,19377,19409,21490,21457,21457,19376,19344,17232,17199,15119,15086,17166,15086,36085,46617,42391,44471,44471,44504,44471,44471,42391,42391,42391,44471,42391,44439,44471,44471,44471,44439,42390,44439,44439,42390,42390,42390,42390,42390,44438,42390,42390,42358,42390,35987,40213,50745,36019,23309,50645,54871,52758,52791,52758,52758,52758,50612,52758,48564,50645,42259,52857,52824,52824,50776,42260,31793,33873,52857,52824,44373,44438,21229,17036,14956,14955,14955,14956,14923,12875,12843,6503,4358,12777,14890,4358,4390,10729,8648,2245,6471,10697,6503,2309,6536,8648,4390,2310,8584,6536,4358,4391,6503,10729,12744,10631,8583,10631,12712,14792,25161,35660,31597,19115,23373,17035,33938,31825,35987,31793,31793,29680,31825,31793,40211,54937,50744,44438,19148,14890,6503,4357,10664,12809,8616,6438,6470,12777,8583,4357,4357,8584,8551,6438,4325,6438,8551,4357,4325,4324,2244,2244,2244,2211,2211,4324,2211,98,2211,4357,2244,2244,4292,2244,2211,2211,2244,6503,12809,8550,8614,10728,23308,46582,38067,21034,37906,44180,42164,25422,12876,8617,14889,14922,14857,14987,23471,27631,77},
{34105,34105,34105,29977,27929,25881,25881,27961,25913,27961,27929,27961,27961,27961,27961,30009,30009,30009,30009,30009,32089,32089,32057,32057,32089,32089,32089,32089,32089,32057,34104,34104,34104,36152,36152,34104,34104,34104,34104,34104,34104,34104,34103,34103,34103,34071,34071,21490,13137,15185,17167,19376,21489,21490,21457,19409,19377,19377,19345,17231,15119,17167,15054,17134,15086,21392,36085,27633,36052,38165,40278,40245,40245,40213,40213,40213,40213,38165,40213,40245,40245,40213,40245,40213,40245,42326,40245,40245,40245,40277,40245,40245,40245,40213,38100,40212,31793,42358,40180,27567,10664,50613,52726,48532,50677,50645,50677,52725,50645,52758,50645,48532,46420,48565,44339,44307,44340,31760,33873,35986,54938,40179,27502,40180,33939,17068,14923,12875,14923,12875,12843,12843,14923,12842,8551,8584,14890,12778,8551,8616,12810,8616,6471,8616,10729,6503,6471,8649,8616,4390,6503,8616,6504,4391,6503,6503,8615,10598,8550,12711,16937,16937,25128,37773,31565,17002,21228,14889,23406,31793,25519,19115,29647,27599,31825,29679,38098,54937,48663,50745,42358,21196,14922,6470,4325,10664,12842,10664,4325,6438,10696,8583,2244,4292,8583,8584,4357,2244,6438,8583,4357,4325,4324,2211,2212,2179,98,2178,98,99,4325,8551,4292,4292,4292,4260,2211,4292,2211,4292,6437,10728,10663,12776,12841,42357,42293,23179,27437,33744,33809,29582,15055,17069,21163,4390,10728,23439,27600,27600,73},
{34105,34105,34105,29945,27929,25881,25881,27962,25881,25881,27961,27961,27961,27961,27961,30009,30009,30009,30009,30009,30009,30009,32057,32057,32057,32089,32089,32088,32057,32056,32056,34104,34104,34104,36152,34104,34072,34104,34104,34104,34104,34104,34103,34103,34071,34071,34071,21459,13137,13137,15054,19344,19344,19377,19344,17296,17264,17264,17264,17199,15119,13006,12973,15054,12941,12941,19247,23439,31859,33907,42391,40246,40246,40213,40246,40246,38165,38132,40213,40245,40245,38165,38132,40213,48697,55004,52923,52923,52923,52923,52923,52891,50842,44439,27567,44471,35986,44405,35954,33841,23277,50645,50645,50678,52759,50678,52758,52758,50645,52790,50645,48500,48532,44307,40114,27502,23308,48664,50745,38099,59229,33873,33841,38131,44471,19181,17036,14988,14955,14955,14956,14923,14923,14923,8584,4390,10729,14955,4390,4358,10729,12810,2245,4358,10729,8648,2277,6471,10697,6503,2310,6504,8616,4390,2310,6503,6503,8518,8517,8550,14889,14889,29354,39886,27403,16969,19147,14889,21260,17003,23341,17002,23373,29680,29713,23341,33872,52856,48631,48664,48631,40244,21196,16970,8583,4357,10664,12810,8551,4357,6470,10729,8615,2212,2212,8583,8616,4324,2211,4357,8583,4357,2244,2211,2212,98,98,98,98,2179,6437,8550,4324,2211,2211,4260,2212,4292,2244,2211,4292,6437,14857,8583,10696,38099,48599,31534,29615,33841,35954,31695,23277,16970,16938,4357,12969,25584,31826,27599,69},
{34105,34137,32057,27897,27929,25881,25913,25882,25913,27961,27961,27961,27929,27961,27961,27961,27961,30009,30009,30009,30009,30009,30009,30009,30009,32089,32089,32089,32057,32057,32056,34072,34072,34104,34104,34072,34104,34104,34104,34103,34104,34104,34103,34103,34071,34071,34071,19410,15185,13136,19312,19312,15184,17296,17264,15183,15183,15183,19344,19344,15087,13006,12973,12973,15054,12908,17134,19214,25553,34005,40279,40279,38133,38133,42359,52956,52923,52923,52923,52923,52923,52955,52923,52923,50810,35987,36019,38100,38132,38164,38132,38099,36052,21229,40245,46551,35986,42260,40147,38099,29680,42227,35920,46485,54937,44339,46452,50646,42226,42193,40145,42194,42226,14824,6373,6373,14856,46518,38099,40179,46551,48664,23341,38164,44438,31858,17068,14923,12843,14955,14923,12843,12843,14923,14923,8551,8616,14923,10761,6503,8616,12810,8616,6438,8616,10761,6503,4390,8649,10697,6471,4422,8616,8616,6470,6470,8550,10630,10566,8550,10696,17002,23048,37741,23145,8550,6469,6469,8550,12776,10598,8550,8583,19115,23341,21196,29614,50744,48599,48632,48632,48632,42358,21164,14923,10664,4358,10664,25454,12809,10664,10696,12776,12744,10696,8583,10664,10696,10631,8583,8583,10631,8583,8583,8551,8518,8518,8518,6437,6470,4357,4357,8550,6470,4324,4292,2244,4324,4324,4324,4292,2244,12744,25486,8551,25486,50712,29453,29615,33841,33906,31695,25388,23276,8551,6535,15048,31951,52890,23374,67},
{34104,34137,32057,27929,25881,25914,25914,25881,27961,27961,27961,27961,27961,27961,27961,27961,27961,30009,30009,30009,30009,30009,30009,30009,30009,32057,32089,32057,32057,32056,32056,34072,34072,34104,34104,34072,34072,34104,34071,34071,34104,34103,34103,34071,34071,34071,34039,19378,15184,15184,19312,19377,17329,17329,17296,15216,17264,19344,17231,13071,13039,13038,15086,15054,15053,10860,17167,21295,27666,27699,31860,33972,29714,31859,29714,31891,31826,29714,29746,31827,31827,31827,31794,31794,46552,48729,50778,50778,50778,50810,50778,50810,42391,19149,42357,50777,42325,42292,46485,46486,27535,40178,31696,50711,57051,38033,35920,52856,48598,42226,42258,42227,44372,8550,6405,4325,6405,29614,31728,42292,50712,50744,27567,40245,44471,42391,19149,14956,14923,12843,14955,12842,12843,14923,14923,10697,4390,8616,12875,8584,4358,10697,12842,4390,4357,10697,10729,4390,4390,10697,8649,2309,4423,8648,8583,4358,8518,12711,12711,8518,6437,6470,20968,31435,23210,6469,4356,4356,8485,25357,8485,6405,6437,8518,8551,8583,27469,50743,50712,48631,50712,48664,48664,44406,19149,14923,8616,4390,25453,38098,14857,10728,12776,10664,10696,10728,10696,10696,10663,10663,10696,10663,10631,10631,10663,10631,10631,8550,8518,8550,8551,6438,4357,10599,10631,6405,4357,4324,4324,4292,4357,8615,8550,16969,12809,16970,50777,31599,29615,33776,33873,33840,25356,21163,6503,10793,17161,25644,40277,29712,67},
{34105,34137,32025,27929,25881,25881,25881,25881,25881,27961,27929,27961,27929,27961,27961,27961,27961,30009,30009,30009,30009,30009,30009,30009,30009,32057,30009,32057,30008,32056,32024,34104,34071,34104,34104,34072,34104,32056,34104,34071,34071,34103,34071,34071,34071,34071,34071,31990,31958,31958,29845,25684,25716,25683,25683,23603,21522,23570,23602,21458,21457,19345,17199,15086,17166,19279,21360,17101,15021,19247,21327,21263,19214,17102,14989,17101,10730,10730,12843,10731,8650,8650,8650,19149,38132,27535,27568,23407,14923,19117,17037,19149,17035,23342,44406,46551,44438,48631,44405,54970,33906,46485,46453,48532,48597,44339,44339,44371,44339,44339,44371,42259,42227,31695,12744,6405,8550,19081,35954,40212,48566,44373,23309,40245,44438,44471,31794,17068,14988,17004,19149,12809,21261,25488,12810,6536,21294,21327,6503,6503,6471,6471,6503,6471,6471,6471,6503,6471,6471,6471,6471,4423,4391,6471,4423,6438,4390,8551,10663,10631,6405,12581,16774,27242,31532,21162,6437,8485,10566,25389,8518,4357,4357,8518,12744,8582,25420,48663,48631,50744,50744,48664,48664,48632,40245,17036,12842,4390,23340,44372,35986,12808,10664,8583,6470,10696,12776,10663,8550,8583,10696,10696,10631,6470,8550,12744,10664,8518,6405,8550,6470,6438,4389,6405,10599,12744,6437,4357,4325,4324,4357,4324,27470,14792,10631,14857,44373,29485,31566,31534,29550,31695,27502,17034,6535,19338,17192,23627,23405,23405,68},
{34105,34105,29977,25881,25881,25881,25881,25881,25881,25881,27929,27961,27961,27961,27961,27961,27961,30009,30009,27961,30009,30009,30009,30008,30008,30008,32057,32057,30008,30008,29976,32023,34071,34072,34072,32023,32023,32023,32023,32023,32023,32023,34071,32023,34071,34071,34039,31991,31990,31990,31958,29877,29909,29909,31925,29877,29845,29844,29844,29812,29812,27731,23505,21392,19280,27633,25488,25521,21295,23407,33874,23440,14957,14989,14956,12908,12876,10763,12876,10795,10763,8650,6537,12810,42358,48697,27665,21327,19181,19182,19182,19149,17068,29713,46551,42325,44439,44406,46551,44438,33841,46485,59164,35889,35888,40082,42259,42195,38001,48566,38066,42260,23243,10566,10631,4325,8550,14856,33841,48631,48599,29615,27534,40244,44470,44439,42391,36052,34004,36052,33972,19181,19181,29713,10697,8648,21262,8617,4390,4358,4358,4358,4357,2277,4357,4357,2277,2277,4325,4358,4326,2277,4357,4357,4325,4325,4357,4389,12809,12744,10630,10565,16775,27209,33514,29582,8550,8550,10566,23211,10631,6437,8550,10630,23308,10631,25420,48630,48599,46551,48631,50777,48631,50745,48664,36019,17036,12842,25453,46517,38098,29647,14889,10696,12809,12809,12776,12776,10728,10728,12776,12776,12744,12744,10664,12744,12712,10664,10663,10663,8615,8551,8551,8518,6437,10631,14857,10630,8518,8550,8550,6437,10598,21098,31728,31760,35985,31695,27437,25388,23308,8518,23276,17035,10858,21484,21450,23626,17033,19114,66},
{34137,34105,29977,25881,25881,25881,25881,25881,25913,25881,27929,27929,27929,27961,27961,27961,27961,27961,30009,27961,30009,30009,30009,30009,30008,30009,30009,30008,30008,30008,29976,32024,34072,34072,34072,32024,32023,32023,32023,32023,32023,32023,32023,32023,34071,34039,34039,31990,31990,31990,31990,31958,31957,31957,34005,34005,31957,31925,31925,31893,31892,29812,23538,21425,19312,25553,27633,21295,27601,33906,40212,14988,12876,12876,12876,12843,10795,10795,10795,10763,10763,6569,6537,17036,31793,50810,25553,23407,17101,17069,19149,17036,17069,17069,21230,21230,19150,23343,23374,33939,46518,46452,42324,25356,23275,25324,25324,23243,19018,12712,12744,14857,12744,8486,6372,6405,6405,8550,21228,42260,25389,44372,27501,44438,42390,42423,44471,42423,42423,42391,38133,27633,10729,14923,12842,29746,10729,4391,4358,2278,4326,4325,2245,2212,2244,2244,2244,2244,2244,2245,2245,2244,2244,2244,2244,2212,2244,2276,8518,10598,10598,16839,18887,33450,37740,31663,14889,17002,18984,27372,27469,8486,12679,10630,31727,10663,25388,48630,46486,50712,44438,50712,48664,48696,50745,48664,21262,17004,27567,46550,44405,42292,38132,14922,10696,10663,8550,8583,12809,12776,10696,12744,10696,10663,10631,8550,6437,8550,10631,10663,6437,4357,8518,8583,6470,8549,16969,21194,19082,10598,29647,12711,12743,12775,25453,23341,29646,31727,31694,31694,29582,6438,12776,17035,6601,23565,38352,21514,19114,19082,69},
{34105,34105,29977,25881,25881,25881,25881,27929,25881,25881,27961,27961,27961,27961,27961,27929,27961,27961,27961,30009,30009,29977,30009,30009,30009,29976,30009,29976,30008,29976,29976,32023,34071,34072,32024,32023,32023,32023,32023,32023,32023,32023,32023,34071,34071,34039,31990,31990,31990,31990,31958,31958,34006,34005,31957,31957,31925,31925,31925,31893,31892,27731,23538,21393,19280,23473,27568,27568,23407,33874,27535,12843,12811,10763,10763,10731,8683,8683,8682,8682,8650,6537,4456,12875,19181,23375,25488,19181,14988,17069,14988,12875,10762,10762,10730,10730,10730,12810,12810,27568,54938,48565,50678,48565,48533,33808,19050,16937,16937,23276,23341,29647,23309,10598,16969,14889,8518,4324,6437,38099,29647,42259,25389,31728,40212,42391,44504,44471,44471,42423,38165,23342,21262,10729,29746,19116,4390,4358,4390,4390,4390,4390,4358,4358,2277,4325,4325,4357,4357,4357,4357,4390,4357,4357,4325,4325,4357,2276,6437,8486,8486,8486,14695,33418,33547,31663,14922,44437,42260,37969,29582,10631,10630,14888,31694,19050,25420,46485,48631,48599,48599,46519,50744,48664,48696,50745,25488,17101,29647,48598,42292,40212,44438,31793,16970,12776,14856,14856,14889,16937,14824,14856,14857,14857,14856,10695,12776,14824,14856,14824,12711,12744,12776,12776,12743,10630,12743,29613,48564,14823,31663,19016,16871,23275,31792,21228,23275,31694,31694,31694,29549,4389,8550,17035,12973,12973,38128,29837,21259,21195,70},
{34137,32057,27929,25881,25881,27929,25881,25881,25881,27929,27929,25881,25881,27961,27929,27961,27961,27961,27961,27961,27961,27929,27929,30009,30009,30009,30009,30008,30008,30008,29976,32024,32023,32024,32023,32024,32024,32024,32023,32023,32023,32023,32023,32023,32023,31991,34039,34039,31990,31990,31990,31957,31957,31957,31925,31957,31925,31925,31892,31925,31892,29812,23538,21393,19312,25586,23440,25553,25520,27535,31761,27568,27535,25487,25487,12876,10796,10796,10795,10763,8682,6537,6537,12875,14988,14924,14956,14988,8649,14988,6504,12875,8649,6472,8682,12843,8617,14955,8616,21197,29614,50645,52791,44307,40082,19050,16937,14857,14857,40179,29648,27535,31760,25421,19083,10663,10662,6405,8550,23309,21228,44404,31727,21261,36019,44439,44471,42391,42391,44471,38165,19213,27601,17035,23407,4390,4358,2309,4390,6470,6503,4390,2277,2245,2244,4357,2212,2244,2244,4357,2309,2309,2309,4390,2277,4325,4358,4357,6437,6405,6373,6437,12743,31467,31466,27469,6470,27534,46453,48501,29647,14889,29614,25355,35920,29647,25420,48630,50744,44437,48631,33841,29615,29648,29680,33906,23375,14956,27599,40211,40212,38099,40179,38099,36019,35986,35986,40146,40145,40113,38065,38065,38097,38097,38065,38065,38065,38097,40145,40145,40113,40113,40114,38065,38097,40145,35951,42258,46484,21097,35887,38098,40178,40113,21162,16969,21194,29646,29614,29614,25388,6405,6438,12842,17264,17232,21196,21195,23308,21195,80},
{34105,32057,27929,25881,25881,25881,25881,25881,25881,27929,27929,25881,25881,27929,27929,27961,27961,27961,27961,27961,27961,27961,27929,29977,30009,30009,30009,30008,30008,29976,29976,31992,32024,32024,32024,31991,31991,32024,32023,32023,32023,32023,32023,32023,34071,31991,31991,31990,31990,31990,31990,31957,31957,31957,31925,31925,31925,31925,31925,31892,31893,27732,25586,21393,21393,21328,31794,31859,33939,31760,35987,33842,31697,42293,33906,10796,10795,8715,8715,8683,8650,6569,8650,10730,17069,14956,17036,17037,8617,14988,6504,14955,6536,4391,8649,8616,10697,10729,6471,10697,33873,50645,52791,29518,23276,25422,23244,23244,23276,48630,38066,21196,40179,31727,21228,12776,33905,14889,4325,4324,14856,42227,25421,19181,33906,44471,44471,42359,42391,44472,36084,21294,23407,19181,6503,4325,2277,2245,4325,8551,10729,6503,4390,2244,2244,2309,4325,2212,2212,2277,4358,4325,4357,4390,4358,2277,4325,6437,8517,8485,6372,6372,10630,29451,37837,29550,8550,12776,23178,35791,29647,12809,42292,44405,42291,31695,27501,48598,52857,48631,48599,38066,25389,21196,21196,19116,19084,17003,14858,21196,44405,40180,40179,38131,42325,40212,42292,42259,42259,44371,46484,46451,42323,48630,46484,46483,46484,40145,40146,46451,46483,46451,40113,35921,44371,46483,46419,46451,46484,19017,19049,25387,31693,40113,31727,16969,31727,31727,29614,29582,25421,6405,4389,8583,21261,19247,19083,19147,21227,21163,81},
{34105,29977,27929,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,25881,27929,27929,27961,27929,27961,27929,27961,30009,29977,29977,30009,30009,30009,30008,30008,29976,29976,32024,32024,32024,31991,32024,31991,32024,32023,32023,32023,32023,32023,32023,32023,32023,31991,31990,34038,31990,31990,31958,34006,31957,31957,31957,31925,31925,31925,31893,29845,29812,27699,23506,21360,19280,29746,23407,36019,38067,21262,6569,6537,6537,4489,4489,4489,4489,4489,6570,10828,10828,12908,8617,17101,10730,14956,12843,8649,12843,17037,17101,12843,14955,14955,12875,17036,14923,14955,14923,25453,50677,38001,33807,42226,40146,42226,42259,31662,48598,46518,27534,23309,23309,23341,21195,19050,16937,12743,10631,10663,29647,31728,17036,31794,44471,44471,42359,42391,42391,40278,19181,21294,25488,23407,2245,2245,2244,2244,2245,12842,10729,12907,8681,10729,8681,10794,6600,8648,8681,10762,8617,8616,8648,10697,6438,6405,8549,12678,12645,12645,12645,12645,29386,37837,29549,10663,23341,33743,40016,31728,12776,33906,42259,44372,31695,27502,50711,54938,52825,48664,40146,40145,27468,25421,25388,25421,25454,25454,25454,46518,40180,46551,42325,42325,40212,42292,40179,46583,46517,46484,46451,40244,50841,48597,46483,48532,33872,27534,44403,42226,44339,31727,17002,44403,46483,46451,46483,38062,16937,12743,12743,16903,25355,21130,6404,27534,31727,31694,29614,25421,4357,4389,6470,21195,21163,21195,19114,21227,19050,76},
{34105,29977,25881,25882,25881,25881,25881,25881,25881,25881,25881,27961,27929,27929,27929,27929,27961,27929,30009,27961,27929,30009,29977,29977,30009,30009,30009,30008,30008,32024,29976,32024,32024,32024,31991,31991,32023,32023,32023,32023,32023,32023,32023,32023,32023,31991,31991,31990,31991,31990,31990,31990,31957,31957,31957,31925,31925,31925,31925,31893,31893,29812,25554,25619,19280,19279,19279,21327,31826,46551,21230,6570,4489,4489,2441,4489,4489,4489,4489,4489,8715,6602,14956,12842,23407,10762,17069,12811,10794,19149,31794,31826,31826,31826,29746,29745,31826,31794,31794,29745,33906,44373,31565,35921,50678,52758,52791,52823,40147,44405,52857,33808,23308,31761,25454,23309,14824,19083,14857,14889,10663,19115,25454,19181,29713,44471,44472,36052,33940,33972,33940,23374,21294,29713,29713,6438,4325,2277,2245,2212,8584,14988,23505,10762,10729,10761,21392,10762,6536,8584,17166,12843,8649,10761,17133,14824,12710,8485,10630,10564,8516,8516,8484,33677,37838,27437,4292,4324,21000,35790,29615,6437,8518,23275,44372,29582,27501,44405,52825,50745,48599,40179,38032,42226,52823,50710,50743,52857,52889,57116,44406,38099,42325,44438,42292,40212,42292,35953,36019,42324,46484,46484,40211,33873,44372,46484,48532,35920,29647,46484,27501,35920,33872,19147,44403,48531,46451,46482,44462,17000,12711,8517,10598,27501,29582,23210,19082,27534,29614,31759,27502,4357,6470,6437,19050,21163,21195,19115,21195,19082,70},
{34105,29977,25881,25881,25881,25881,25881,25881,25881,25881,25881,27929,27929,25881,27929,27929,27929,27961,27961,27961,27929,27929,27929,29977,29977,30008,30008,30008,30008,29976,29976,32024,32024,32024,32024,31991,31991,32023,32023,31991,32023,32023,32023,32023,32023,34071,31991,31990,31990,31990,31990,31958,31957,31957,31925,31925,31957,31925,31893,31893,31893,29812,25554,23506,19280,21328,25553,25488,29681,31793,12908,6570,2441,2409,2409,4457,4489,4489,6569,14989,8682,4488,17068,19115,42358,19116,33972,14890,27601,25487,44503,44503,44503,46551,46584,44503,46584,46584,46584,46584,46584,46486,27469,21163,42259,46452,48532,44371,31695,23276,52856,50744,48631,35954,17003,12809,10696,12744,6437,6437,8517,25454,27567,17068,27601,42423,44504,17035,10664,10665,10729,21294,19181,36052,31858,14890,4390,4390,4325,2212,6503,12843,23473,6568,4390,6503,21392,8616,2244,2245,15053,10729,4357,6470,17100,10696,14823,8485,10597,12677,8516,10629,16870,29418,29387,27437,4292,4292,21033,37935,33808,6373,4325,21097,42291,31695,29582,48631,46518,44406,46486,40114,29580,44371,54904,50678,50710,57050,57083,61309,42293,33873,25454,44406,44405,40212,42292,38066,42325,44404,46484,46484,40146,21163,40178,46484,48532,33808,21162,44404,44405,42290,31759,17034,44371,48532,46451,46483,44494,19080,12775,12678,10533,16968,35919,40113,35953,29582,21163,29582,27501,6372,6437,6405,12776,21163,21195,21228,19115,17034,71},
{34105,29977,25881,25881,25881,25881,25881,25881,25849,25881,25881,27961,25881,27929,27929,27929,27929,27961,27929,27961,27961,27929,27928,29977,30009,30008,29976,30008,30008,29976,29976,32024,32024,32024,32024,31991,32023,32023,32023,32023,31991,31991,32023,32023,31991,31991,32023,31991,31990,31990,31958,34006,31957,31957,31957,31925,31925,31925,31925,31893,31893,31892,25586,23473,21361,19248,27666,27633,23375,33875,12908,12941,2474,4522,4489,4522,8715,8715,8683,25552,23408,14956,29778,19083,40245,19180,42326,14922,31859,23374,44503,46584,46551,46552,46584,44503,46584,44503,46617,46584,46584,46584,35986,16970,46452,52759,54904,46453,35986,21195,52856,50743,48631,42325,29712,29646,29645,27467,16936,16935,14823,33873,29712,17101,23408,36052,44504,17003,8615,8583,10664,12777,27568,38230,27600,25421,4390,2309,2245,2212,6503,12875,21457,6536,4357,4390,21392,8584,2212,2212,15021,10697,4324,8583,14922,16970,21129,14823,8452,6371,6371,6371,6371,27274,33580,29517,4292,4324,16775,35823,33808,6373,6372,18952,44404,31695,29582,44405,46487,48632,50712,38034,33806,46484,54936,50710,50710,52823,52856,59229,42260,44438,27535,46519,46551,40212,42291,38065,35920,40113,46451,46451,35952,33872,40145,46451,46452,35919,31694,38065,50678,46484,33838,33839,42258,46419,46483,44370,46542,21193,23307,35951,35951,35952,40113,38032,38032,35920,16937,16969,19050,10598,4357,8647,8550,19082,21195,21163,19114,17034,68},
{34105,29977,25881,25881,25881,25881,27930,25881,25881,25881,25881,25881,27929,27961,27929,27961,27929,27961,27929,27961,27929,27961,27928,30009,30009,30008,30008,30008,30008,29976,29944,32024,32024,32024,32024,31991,32023,32023,32023,32023,32023,32023,32023,32023,31991,31991,31991,31990,31990,31990,31958,31957,31957,31957,31957,31925,31925,31925,31925,31892,31892,29812,25554,23474,21361,21328,27601,29714,31827,27600,14989,8748,8748,10828,8747,8715,12941,6602,8650,27666,27633,14956,14989,10697,36052,23374,44471,31826,40277,31858,31859,38133,33907,36052,33939,33939,36020,33906,36116,33906,36019,36019,33841,19017,46420,52759,54839,42227,27502,19050,50744,48598,46485,40212,36019,40114,46386,48531,46419,48499,19017,14857,31793,17069,21295,42358,44504,17003,8615,8583,8583,8583,23439,38197,29680,31728,8648,12907,10762,8648,10761,12875,21425,6503,4358,2309,19311,8584,2212,2212,15020,10729,4357,8550,14890,12711,16871,14758,8452,21031,23047,23047,23047,31499,29322,31663,8518,4325,12614,35823,33776,10663,10631,21065,42291,31727,27534,23341,25423,42325,42260,35921,38033,38033,48598,48565,46452,46452,52856,54937,40147,48631,38132,40212,44438,42292,42292,44404,44371,44371,44371,46451,46419,46451,46452,46451,46484,46451,46452,46483,46484,46484,46451,46451,46484,42193,46484,42289,46574,23274,44370,46483,48564,48564,44371,38000,35920,33872,17002,6470,8518,6502,8615,36146,23372,21195,21195,21163,19082,17002,65},
{32057,27929,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,25913,27929,27929,27961,27929,27929,27929,27929,27929,27928,27928,29977,30008,30008,29976,29976,29976,29976,29944,32024,32024,32024,32024,31992,32023,32023,32023,32023,32023,31991,32023,31991,32023,31991,31991,31990,31990,31958,31958,31958,31957,31957,31925,31925,31893,31925,31893,29844,29812,29812,25554,23506,23441,19248,31859,23408,27601,33874,21295,10796,10828,12876,10795,10796,12908,10731,8714,27665,8682,4488,10827,6504,38165,14890,44471,33939,42391,33972,23342,14956,17101,14956,23375,25520,25520,25552,27665,29714,29714,27600,40212,42259,48500,52726,52758,42227,33873,42292,52824,52824,50776,46551,31826,33807,35855,35887,23210,21162,4324,4324,12777,17069,19214,44504,44471,27600,12777,12777,12776,10696,25487,36085,31826,44340,8649,17101,17134,25683,17198,14988,21457,6536,6470,4390,21392,8616,4325,2245,17101,10761,4389,6502,17003,14824,16806,14726,10501,8452,8452,10532,8452,27306,27242,33744,10663,6405,12646,37968,33809,14857,14857,23178,42227,33808,21228,19115,21229,25423,48632,38066,33807,29581,54872,50678,50677,50710,52823,50711,35953,42325,50777,35986,42357,40212,42292,42259,48630,46517,48564,48532,44371,50808,48630,48564,48564,40113,31726,44371,48564,48564,40146,31727,42226,23241,46484,48596,44462,23273,38059,46508,48621,46511,38064,38032,33840,33840,19083,8551,2212,6470,6437,42390,29647,23276,21227,21195,19114,14921,63},
{32057,27929,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,27929,27929,27961,27929,27929,27929,30009,27929,27928,27928,29976,29976,30008,29976,29976,29976,29976,29976,29944,32024,32024,31992,32024,32023,32023,32023,32023,32023,32023,32023,32023,31991,31991,31991,31990,31990,31958,31958,34006,31957,31957,31925,31925,23473,19247,19247,21328,19280,19248,17135,15054,15022,12973,17102,14989,17134,17069,14989,8715,10795,8715,8715,10795,14956,12843,10762,33971,14988,6569,6602,4423,17069,12810,42326,23341,33939,10697,6504,6471,6504,6503,6504,6536,8584,8616,8616,6535,10729,8584,35954,40146,46451,54904,54871,38002,31728,50743,52856,40147,33873,33906,33906,33872,31759,21195,19179,10727,4292,10663,21196,12875,21295,44504,42423,40310,14922,14890,14922,21261,27633,38197,33906,42259,8649,17101,17101,21457,17133,14988,23505,8649,8616,8648,21359,8616,2245,2244,17100,10729,2277,8615,17035,25227,23081,14726,10565,10533,8485,10565,10565,29354,27242,33776,10631,6438,14727,35855,35857,12809,12809,16937,27469,31727,14921,31793,23309,21262,27568,40180,23275,29582,54969,52791,50678,50678,52823,52824,33840,33873,42325,38099,42325,40212,42292,38098,46582,48597,48564,48532,42259,50841,50711,48596,48564,38033,21163,42258,48564,48564,38099,14922,46485,35919,48564,48628,48655,23272,33897,42311,44427,42286,37999,35952,33839,33840,25389,8551,2211,4356,21228,46616,29648,23308,21196,21195,21196,14889,64},
{32057,25881,25881,25881,25881,25882,25881,25881,27929,25881,25881,25881,25881,27929,27929,27961,27929,27929,27929,27929,27928,27928,27928,27928,30008,30008,29976,29976,29976,29976,29976,29976,32024,32024,31992,32024,32023,32024,29975,32024,32023,32023,32023,31991,32023,31991,31991,31990,31958,31958,31958,31958,31957,31957,31925,31892,21327,12876,10828,12909,10861,10861,10828,10828,8715,8748,10795,10796,10795,10827,12941,10828,10828,10828,10828,12908,19150,21327,21294,36052,23375,10795,14988,8617,17036,17036,44471,40213,31794,12842,12810,12810,8649,12842,10697,12842,14890,10729,17035,10729,14922,14922,31761,52824,46452,50678,52791,35954,35954,52824,52857,31727,25421,25389,35987,38098,50743,44404,38066,42259,29614,8550,8550,12875,21295,44504,42423,44472,25455,17002,19115,33907,27632,38197,33906,42259,10697,17133,15020,17166,14988,14955,21457,8617,6535,6535,21359,8616,2244,2244,15020,10697,2277,6470,17035,25227,29419,12613,10565,8452,8452,8485,10532,29451,33645,29550,10631,6438,12646,35823,35889,12776,12776,19082,31694,31760,12776,38131,38098,21196,19117,27601,23244,31727,54969,52856,52823,50742,52856,54937,38033,38098,40180,40212,46518,40212,40244,40146,44470,46485,48532,46484,40212,48729,48598,48564,48564,38033,25388,42291,48564,48532,38066,16970,44372,48565,48564,44370,46577,25386,31753,40231,42315,42319,40145,35920,33872,33839,35986,6503,2212,8582,42422,38164,14857,12743,21228,21195,21227,14889,66},
{32057,25849,25882,25881,25881,25881,27930,25881,25881,25881,25881,27929,27929,27961,27929,27929,27929,27929,27929,27929,27928,27928,27928,27928,30008,29976,29976,29976,30008,29976,29976,29976,32024,32024,31992,32024,32024,32024,32024,32023,32023,32023,32023,32023,32023,31991,31991,31990,31958,31958,31958,31957,34005,31924,31892,21360,19214,21327,8715,10796,8748,8748,8748,8715,6602,8683,8683,8683,6634,8715,8715,8715,8715,8747,10795,10795,14956,14988,14956,38132,44471,25487,40245,27600,44439,40278,44438,42358,48665,25454,29713,29713,19180,25454,12777,21261,21293,14922,25487,12777,23342,21228,27567,38066,40113,42258,44339,29583,31760,50743,52824,46549,46517,46485,31793,38065,50645,50645,46453,40146,46453,23243,14824,14924,21295,44504,44471,44472,38133,21229,19148,19148,29713,38197,31793,40179,8616,15053,17134,25651,19214,15020,23505,6536,4423,6503,23472,8616,2244,2244,17101,10729,2277,6470,17003,18921,23113,14661,10532,8485,8484,10532,10533,27307,37871,31727,8551,6438,14726,23178,33808,10664,8583,21163,35920,31695,8518,38099,46453,33775,19083,19116,23309,33840,54937,52823,52791,52791,52823,57017,44405,46486,44373,42292,46583,40212,40244,33807,25519,35953,46452,46484,33840,29713,38098,46484,48532,29614,19082,35920,46452,46451,31727,10599,35952,46451,48532,44338,44434,23306,21159,25382,27526,29577,35920,35920,33839,35952,23309,8584,2244,31857,48729,21228,6405,6437,19115,21195,19147,16969,68},
{30009,25849,25881,25881,25881,25881,25881,25881,25881,27929,27929,27929,27961,27961,25881,27929,27929,27929,27929,27929,27929,27929,27928,27928,29976,30008,30008,29976,30008,29976,29976,29943,32024,32024,32024,32024,32024,32056,32023,29975,32023,32023,32023,32023,31991,31991,31990,31990,31958,31958,31892,29715,27537,31795,29714,10795,10763,10763,6602,8683,8715,8715,8715,6635,6667,6635,8715,8715,10796,10796,8715,8715,6634,6667,8683,8682,8650,8682,10795,25487,46551,27567,42358,23374,44438,42358,44439,44438,48697,31761,31794,31826,21228,23309,10696,19148,19148,12809,23341,10696,21228,17035,21228,38099,46452,46452,50678,35954,35954,38066,52824,50743,50743,52824,40180,42226,48532,46452,23309,21163,21196,19116,19083,14924,21295,44504,44471,44504,42391,44504,44472,40278,36084,36116,31793,35921,8616,15053,15085,21425,17166,14988,23505,8649,4455,8681,19311,8616,2244,2244,15053,10730,4390,8550,17002,14824,21000,21000,12645,10533,10598,12614,10597,29451,31597,31695,10599,8486,18984,21000,33808,12744,10696,16969,27501,29647,10630,38098,44405,44307,29582,17003,12810,29647,46518,48566,46453,42226,42227,44404,46518,40147,44405,40180,46519,27534,25388,25388,23275,23275,25355,27468,25387,25388,25355,25355,27468,25388,23275,23275,25355,27467,25388,27436,25387,27436,29548,35887,46483,25323,23307,25387,27467,27435,33839,38000,33775,31727,17035,10697,2276,40277,48729,42325,14857,6437,14921,19147,19115,14921,61},
{29977,25881,25881,25881,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,25881,25881,27929,27929,27929,27929,27929,27928,27928,27928,29976,30008,29976,30008,29976,29976,29976,29976,32024,32024,32024,32024,32024,32024,32024,29975,32023,32023,32023,32023,32023,31990,31990,31990,31958,31958,31957,33973,31957,29844,21327,10762,12843,12843,10730,10795,10795,10796,10796,10795,10828,10795,10827,10827,12908,12876,10796,10795,8747,8747,10763,8683,17101,36084,33971,27568,46584,38164,44439,35987,42358,42326,42326,38100,48665,38100,44471,46584,31793,38132,33939,36019,36052,31793,38165,33938,38132,36019,36019,38099,46452,50678,52758,38066,35954,27501,48630,52824,52791,40146,31696,42194,42193,31727,25422,16938,17002,23309,16970,12875,21295,44504,42423,44472,36052,46584,44472,29681,29713,36117,23374,44405,8649,17134,15021,17199,15053,15020,23505,8682,6600,10891,19279,8616,2244,2244,15085,15053,17165,17133,17068,14824,16807,18920,12646,10532,10565,10565,10565,29484,35822,31727,8518,8518,23210,25259,33808,10664,8550,21131,38098,29614,6502,38098,46485,44308,33840,23308,14857,14858,27568,46551,40179,40146,38034,38066,40147,38034,35953,42325,42325,40212,40244,46484,46484,46484,46484,46484,46484,46484,46484,46483,46483,46452,46484,46483,46484,46484,48564,48532,48564,48532,48564,48597,46484,27436,48565,48597,46484,46452,42291,35920,33807,40244,48729,27568,10728,42422,50809,46583,27535,12744,12743,19115,19147,17002,60},
{29977,25881,25881,23833,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,27929,27929,27929,27929,27928,27929,29977,29976,29976,29976,29976,29976,29976,29976,29976,32024,32024,32024,32024,32056,32056,32056,32023,32023,32023,32023,31991,31991,31991,31990,31990,31958,31958,31957,31925,31925,31859,25553,21262,21262,19085,12843,12843,12843,12843,12876,12876,12875,12843,12875,10827,12875,12843,12875,12843,12843,12843,12843,12843,19181,33906,31826,19181,33907,33875,33874,33874,31762,33906,29649,29681,31761,29680,31793,33874,25422,31826,31794,31794,33875,25487,35987,31794,33939,33906,33874,31761,33809,40147,40115,33841,27535,25356,40115,40179,38067,29615,31728,33873,25389,21228,21229,17003,16970,16970,14890,12842,21295,44504,44471,42359,19116,46584,44472,27568,27600,38197,23342,42260,8617,15053,15085,21457,21360,19214,23505,6536,4422,4390,19279,8616,2244,4293,17133,15086,17198,12842,17100,16905,14758,12613,10565,10565,10565,10565,10565,29452,33710,33808,8550,8517,23211,23113,33841,12776,14856,16937,35953,29647,8518,38066,44404,44340,33808,19049,21195,10696,14857,42293,42292,42260,40179,42260,42292,42325,40147,40212,44406,42292,42292,40146,44404,46484,48564,48564,44371,44372,44371,48564,48564,38032,25388,42258,48564,48564,35888,25420,44339,46419,31693,44339,40278,19116,35954,33908,29582,48565,40113,35952,33807,40276,52955,29681,10664,44503,50809,42390,25487,23309,19082,19147,19147,19082,55},
{27929,25881,25881,23833,25881,25881,25881,25881,27929,27929,27929,27929,27929,27961,27929,27929,27929,27929,27929,27929,27929,29977,29977,29977,30009,29976,29976,30008,29976,29976,29976,29944,32024,29976,32024,32024,32056,32056,32023,32023,32023,31991,32023,32023,31991,31991,31990,31990,31958,31958,31957,31957,31924,33874,31793,31793,21262,12811,10762,10762,10762,10762,12843,12843,12842,10730,12811,12810,10762,12843,12810,12842,12842,12810,12843,12842,12842,14955,12842,14923,14923,14890,14955,14922,14923,14955,14890,17003,17003,14890,16971,14922,14923,16971,14890,16971,14890,14890,14923,14858,16970,14922,14889,16970,14890,16970,17002,14890,14889,16938,16970,17003,14889,14857,14857,14857,14889,12777,12777,12777,10664,12744,10729,10762,21295,44504,44504,44472,23342,44536,44472,27600,29680,38197,23342,38034,8584,17101,15085,21457,19279,17134,23473,8616,6503,6503,19278,8583,2244,2244,17101,19344,25683,14955,19148,21131,23210,10532,10532,10532,10533,10533,10565,27338,33612,31727,10631,8518,21098,23146,33776,14856,21162,23243,31727,31727,16937,38066,44372,44340,33808,21162,27501,25454,23276,25422,35954,38099,29647,40212,44405,46551,44405,44406,44406,42292,42292,38033,40212,44372,33807,42225,40211,38131,42292,48564,48564,33872,19082,40178,48564,48564,31727,14921,42258,50645,48564,46484,48731,21229,31827,27765,19050,50678,40145,35952,33775,38132,55003,27600,4357,42358,38164,29744,21228,27567,14857,21195,21195,19082,53},
{27929,25881,23801,23833,25881,25881,25881,25881,27929,25881,27929,27929,27929,27929,27929,27929,27961,27929,27929,27929,27928,27929,29976,29976,29976,30008,29976,29976,29976,29976,29976,29976,29976,29976,32024,32024,32024,32056,32024,32023,32023,29943,32023,32023,31991,31991,31991,31958,31958,31958,31957,31957,33972,31793,31793,27568,12843,10762,10762,8681,10762,10762,12810,10762,10762,6504,12810,10697,10697,12843,8616,12778,12810,10697,12842,10729,12777,14923,8616,14923,12842,10762,14922,12809,14922,14923,8584,17003,14890,10729,14922,10664,12810,14890,10664,14890,12777,10696,14890,8583,12809,12809,8583,16970,12744,14857,14890,10631,14857,12777,12744,14890,10631,12777,12777,10631,14857,10696,10696,12809,6470,10696,8616,8649,14989,27699,27666,27634,29681,46584,44472,27600,27600,38197,21262,25421,8584,17133,15053,19311,17166,15021,23505,8584,6471,4422,19246,6503,2244,2244,15053,17231,21457,12810,19115,35789,31628,14790,10532,10500,12613,12613,10532,23080,29322,33776,14857,25486,29550,31533,31695,12744,27567,33873,25421,29647,12743,42292,44405,44404,29647,12711,14824,19050,19049,17002,16970,35954,25389,35986,40211,48599,50777,46551,44405,42292,42292,38034,33841,40179,29679,38032,42324,50842,50711,48564,48532,35920,21162,38065,48564,48564,35986,19083,42258,48564,48564,46517,46650,21262,31826,29878,19115,50744,44372,35920,31727,36018,55003,29648,4357,42358,36084,19147,14922,21228,10631,16970,19147,17002,51},
{27929,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,27961,29977,27929,27928,27928,27928,29977,30009,30009,29976,30008,29976,29976,29976,29976,29976,29976,29976,32024,32024,32024,32056,32056,32023,32023,32023,32023,31991,31991,31991,31991,31958,29845,29877,31957,31925,33972,33874,29681,12843,12843,14956,14956,12875,14988,14988,14923,14988,14956,12842,12843,12875,12843,14923,14923,14923,14955,14923,14923,14955,17003,14923,14955,14955,17068,17036,17035,17035,17035,17067,19116,19083,19116,17035,17035,17035,17035,17035,17035,17003,17003,17003,17035,17003,17003,17003,14922,17002,17002,17002,17002,16970,16970,17003,17003,17003,16970,17002,17002,14890,16970,17003,16970,14890,14890,14857,12809,10729,10795,19247,19215,17070,31794,44536,44504,38133,33971,36084,31858,35986,6536,15053,15086,21457,19311,13005,21457,12972,12907,6471,21359,8583,2244,2244,17134,15086,17166,10761,19115,29452,25225,18951,10500,10500,16806,14661,16709,31467,31532,33808,8518,14824,27404,25258,31663,6405,4324,21098,46518,29647,8550,38098,44405,42292,29646,10631,10630,8550,31760,19115,17003,21196,29616,35986,42260,48664,48664,50744,48599,42292,42292,33840,29615,40147,48663,46451,38033,35986,42291,48564,48532,33775,21162,38032,48564,48532,31694,16969,38033,48564,48532,44404,48763,21261,31761,25521,23373,42292,33872,35920,31759,33873,44503,29680,12777,40245,33971,23374,19148,23341,8550,10696,21228,21196,99},
{27897,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,27928,27928,27928,29977,29977,30009,29976,29976,30008,29976,29976,29976,29976,29976,32024,32024,32024,32024,32024,32023,32023,32023,32023,32023,32023,31991,29910,23571,27797,21425,27731,31925,33973,33972,31826,17069,8650,10795,17069,14988,12907,14988,14988,14956,14956,14956,14988,14988,17036,17036,17068,17068,17068,17068,17068,17036,17068,19116,17036,17068,17068,19148,19148,19148,19148,19148,19148,19148,19148,21228,21196,19148,19116,19148,19148,19116,19116,19116,19148,19148,19116,19116,19116,19115,19115,19148,19116,19116,19115,19083,19115,19115,19115,19116,19115,19083,17035,17035,19083,17035,17003,16970,14922,14922,14922,12843,19215,17134,17069,33906,44504,44504,44471,42358,38165,40277,44406,8616,17133,15053,19312,15086,15086,19376,21457,23634,17198,19311,8583,4325,2244,17133,17199,15117,17067,19115,23211,25193,16805,12580,8451,10532,12580,16806,31532,31467,33808,10663,10631,25291,25291,35921,12776,10696,19018,33873,31760,8550,38066,44404,42291,31695,12743,12775,10662,21195,21195,21228,19116,25422,42358,46518,44405,48632,50744,50712,42292,42292,42226,38033,42226,44339,44371,42226,40178,42258,44338,44371,42193,42226,42226,44338,46419,42226,42258,42291,44339,46451,44372,48731,21197,35953,44372,42291,42291,40146,35953,31759,35986,48664,27567,4357,40277,42390,25422,17035,19148,12776,17034,14921,8549,44},
{25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,27961,27929,27929,27929,27929,27928,27928,27928,27928,29977,30009,29976,30008,30008,29976,29976,29976,29944,31992,31992,32024,32024,32024,32056,32055,29975,32023,32023,32023,32023,31991,31958,31991,25684,19344,23505,31860,31828,31827,23407,14956,12843,12876,17101,17101,10762,19149,17101,14988,17101,17069,14988,14956,17069,17036,17036,17068,17036,19149,17068,17068,17068,17068,19148,19148,19116,19148,19148,19148,21196,19148,21196,21229,19148,21228,21196,21196,21228,21228,19148,19148,19148,19148,21196,21196,21197,21229,21228,19148,19148,21196,21228,21228,21228,19115,19148,19148,21196,21196,19115,19115,19083,19083,19083,17035,17003,17003,14954,14922,17003,14956,19215,17102,17036,36052,44471,44504,44471,40310,38165,40278,44406,8616,17133,15053,19312,17134,15053,21425,17231,19344,19279,19312,8584,2244,2244,15053,19279,12939,10696,19083,18985,27273,25225,18983,33737,35818,33704,33737,35789,31499,35888,10696,8550,23211,25291,35953,12808,12776,19050,40212,31728,8550,38099,44404,44340,31695,19082,42324,42324,33807,25388,31728,23309,21229,27535,46551,46486,48599,48632,50712,42292,42292,46484,48564,48564,48564,48564,46452,48565,46484,48597,48564,46484,48564,48564,48596,48564,46484,48532,48597,46452,35919,44437,46650,17037,23309,48565,48533,46484,42259,35985,33839,33873,46616,19115,4357,36019,50810,23310,17003,27600,23374,21228,14953,12808,59},
{25881,23833,25881,25881,25881,25881,25881,25881,25881,25881,27929,27929,25881,25881,27929,27929,27929,27929,27929,27929,27928,27928,29977,29976,29976,29976,29976,29976,29976,29976,29976,29944,32024,32024,32024,32024,32056,32056,32056,32023,32023,32023,32023,31991,31958,31991,31991,29877,21458,27699,31892,31892,29746,14988,23375,19215,19214,23408,21262,12843,23440,23408,12843,19149,25456,23343,25521,25456,23407,25520,23407,25488,25520,23375,25520,25520,25455,27600,25487,25487,27600,25487,27568,27568,25454,27567,27567,25487,27600,27567,27568,29648,25422,29648,27568,25487,27600,27567,27567,27600,27535,27568,27568,25454,27567,27535,25454,27567,25422,25487,27535,25422,27535,25454,23373,25486,25422,25422,25454,23341,25422,23341,23309,23342,19149,19215,14989,12810,31825,44471,44503,44471,42358,36117,40278,44406,8616,17101,17134,19376,17133,19246,23538,4423,2277,4390,21360,10729,6535,6535,17166,17198,10826,10695,19017,25194,31435,27306,23209,33705,35785,31624,31625,35886,37871,35921,10696,6438,16839,33742,35953,12809,10728,18985,42195,31728,8551,38098,44404,44372,31727,16904,25388,35920,42258,40211,42325,29647,25422,23309,48632,48599,50744,48599,48599,42292,42292,42258,46451,46484,48564,48564,42291,52921,50711,48564,48564,38000,48598,46517,48564,48564,38033,29647,44371,50678,40114,38067,48731,14989,23342,21196,21227,35953,38033,35920,33872,27534,50777,27600,4357,17003,21229,21293,12810,12810,21229,21261,23373,23374,89},
{25881,23801,25881,25881,25849,25849,25881,25881,25881,25881,25881,27929,25881,25881,27929,27929,27929,27929,27929,27929,27928,27928,29977,29976,29976,29976,29976,29976,29976,29976,29976,29944,32024,32024,32024,32024,32024,32056,32024,32023,32023,32023,32023,31991,31991,31991,29877,29845,31958,31957,31925,33973,27634,21295,23407,21295,19150,19149,4391,10762,27568,27600,25488,25520,27600,25487,27600,27600,27567,29648,27568,27600,29648,27568,27600,27600,27535,29680,27567,27567,29680,27567,29680,29680,27535,29680,29680,29648,29680,29648,29680,29680,29647,29680,31728,29647,29680,29679,27599,29679,29647,29648,29680,27535,29647,29615,27567,29647,27567,27567,29647,27534,27566,27534,25486,27567,27534,27534,27567,25486,27534,27534,25421,25454,25454,23439,19148,14890,33906,44471,44504,42358,38164,36084,36019,33842,8617,17134,17134,21457,17166,15020,21457,15118,17198,15085,19312,15085,15053,6536,17166,19279,12940,12841,19083,33613,35596,27273,23209,33737,33703,33704,33705,42192,42129,37969,10696,8550,16839,44242,33808,12776,10696,16936,40113,27566,8550,38098,46485,44404,31727,16937,19049,31726,42258,33873,38131,33840,29647,25454,44438,48599,50745,48632,48632,42292,42292,40145,42258,44371,48532,48532,40211,46616,48631,48564,48564,35920,25454,40113,48564,48564,38034,25454,42291,48597,48597,48631,48763,17069,23439,27568,25454,36018,21163,14856,35921,23340,44471,27600,6470,42390,27599,25486,23374,14923,12810,19115,23374,25454,100},
{25849,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,29976,27928,27961,29977,29976,29976,30008,29976,29976,29976,29976,29976,32024,32024,32024,32024,32056,32023,32023,32023,32023,32023,31991,31990,29845,29877,31958,31958,31957,31925,31925,38165,40277,46551,46583,46551,48632,10664,14890,52858,54970,54970,54938,54970,54970,52890,54970,54938,54938,54938,54938,52890,52889,52889,52889,54937,52889,52889,52889,52889,52890,52889,52889,54970,54970,54970,54970,54970,54970,54937,54970,54970,54937,54937,54938,54937,54938,52889,54937,54937,52889,52889,52857,52857,52889,52889,52889,52857,52857,52857,52857,52857,52857,54937,52889,52889,54937,54937,52857,52857,52856,52857,52856,52856,52856,42325,14922,33874,44471,44471,42390,42358,36052,40278,44406,6568,17134,17133,19279,14988,15020,23505,19377,21489,19376,19344,15085,15053,8681,17198,17166,12939,10793,16970,23178,37676,29386,19048,31624,33703,31624,33672,42160,44275,35889,12776,8583,16872,44241,35889,12744,8615,16936,40113,31728,8551,38066,44372,42291,31728,14889,17002,29646,42259,23308,23341,29615,38099,35952,48598,48664,48664,50745,50712,42292,42292,40145,46484,46484,48532,46484,42291,48729,48630,48532,48532,35920,19115,35952,48532,48564,33808,27567,42291,48564,48564,44437,48795,17069,21359,21261,21261,29648,16969,8518,29582,23308,42422,29680,8583,46584,38132,31792,31825,25487,17069,8552,14922,23406,100},
{25881,25881,25882,25881,25849,25881,25881,25881,25881,25881,25881,25881,25881,27929,25881,27929,27929,27929,27929,27928,27929,27928,27928,27961,29977,29976,29976,30008,29976,29976,29976,29976,29976,32024,32024,32024,32024,32056,32023,32023,32023,31991,31991,31991,27797,29910,31990,31958,31958,31957,31925,31925,50777,54970,54970,54970,57051,57083,23212,25357,52857,52889,50712,52825,52857,50712,54937,52857,52857,54937,50712,52825,54938,50712,52889,52857,50712,52857,50744,50744,54937,50712,52824,52857,48631,52857,52825,50711,52857,52793,50744,54905,50712,52825,54937,48631,54937,52825,50744,54937,50744,52824,54937,48631,52856,52857,48631,52889,50744,50744,52857,52792,52857,54937,48631,52857,52824,50711,52857,50712,50744,52856,48598,52824,52824,48598,52857,25422,23374,44471,44503,44439,42358,38132,40278,44438,6568,17166,21391,25617,19246,19214,23506,19344,21457,15085,21360,15053,12907,8649,19279,15020,12842,12809,17035,18985,37709,33580,23177,33705,35815,33704,31624,37934,44242,35888,12776,10663,14791,42129,35889,10664,8583,14823,27435,31760,12809,36018,44405,44340,33808,14857,16970,29646,40211,23276,14857,42293,57050,46452,52856,52825,52857,50744,50712,40212,42259,31694,23210,29549,42226,42258,33840,25454,31727,40113,42193,27501,14857,25388,40081,42194,25356,16970,29581,40081,42226,42292,44570,14957,23407,12776,10696,10663,10695,6404,23308,27566,46551,27599,4357,46616,38131,36019,33873,31825,23407,17036,14890,17003,82},
{25849,25849,25881,25849,25849,25881,25881,25881,25881,25881,27929,25881,25881,27929,27929,27929,27929,27929,27928,27928,27928,27929,27928,29976,29977,29976,29976,29976,29976,29976,29976,29976,29976,32024,32024,32024,32024,32023,32023,32023,32023,31991,31990,27797,29910,31991,31958,31958,31958,31926,31925,31893,50745,54970,54970,54970,57050,59164,19051,19051,52857,52857,16969,48599,50744,38034,54970,44406,44406,54970,38067,48599,55002,33840,52857,50744,38099,54937,44405,42292,54938,40146,48631,54938,33840,52857,50712,40179,52889,42292,42292,54937,40147,50744,54938,31695,55002,48599,38066,54938,42259,44405,54970,35921,50776,52824,31695,54937,46485,42292,54937,42260,50711,54970,29581,54970,48599,40146,54937,42292,44405,52857,35952,50744,50744,38066,52857,40244,19148,44471,44503,42358,40278,36084,42358,44438,8616,19246,25553,23505,17165,17133,21425,17264,19344,6568,21360,15053,12940,12940,17199,10762,4325,4357,19148,19050,35563,33612,25290,31560,33703,31592,29480,37966,42194,35888,12776,10664,14759,40049,31696,10663,8550,14823,23242,29615,12776,36019,42292,42260,31728,14889,14889,27534,38066,21195,16937,23309,42292,46485,50711,46519,52890,50712,40180,27535,25421,29581,29581,29614,29614,31693,31693,31694,31726,31726,31694,31694,31759,31726,33807,33807,33807,33807,33839,35887,35887,40146,48730,19149,33905,29678,27566,14824,21227,14856,16969,33872,46584,27567,6405,46616,38132,33873,33841,25486,27599,25455,33938,33906,144},
{25881,23801,25881,25881,25849,25881,25881,25881,25881,25849,25881,25881,25881,27929,27929,27929,27929,27929,27928,27928,27928,27929,27928,30008,30008,29976,29976,29976,29976,29976,29976,29976,29976,32024,32024,29975,32023,32023,32023,32023,32023,29910,27797,31991,31991,31990,31990,31958,31926,31926,31925,31925,50777,54970,57051,54970,57050,59164,16938,19083,54970,50777,19082,46518,50744,38067,54970,44406,42292,54971,40179,50712,54970,38034,54970,50744,35986,54937,46518,46518,54938,42292,50712,54938,33808,52857,50711,44372,54937,44405,44405,54970,40180,52824,54937,40147,54970,48631,44405,54938,44405,48599,54970,40147,52824,52856,40179,54937,46486,44373,54938,44373,52792,54938,38066,54970,50711,42324,54937,46517,48598,52857,40146,52824,50744,35921,54937,46518,25487,44471,44471,42390,40245,36084,36052,38067,8617,17134,19214,23473,17133,17101,21457,21457,23602,6601,21360,17166,17199,17231,17199,10762,2276,4325,19148,21164,33483,27339,25321,33672,35816,31624,33737,42160,42161,38001,12744,10663,14759,40016,33776,10663,8549,33708,29515,31727,31793,23309,23276,19083,19083,21228,19115,17002,19082,17002,19115,19082,14824,35953,46518,40179,50777,50777,46518,40180,40212,46484,46484,46484,46484,46484,46484,46516,46452,44371,44371,46452,46484,46484,46484,46451,44403,46484,46484,46484,46419,46452,44405,25389,40113,40178,38033,12775,23308,23276,27501,35953,48664,27567,6405,46584,38131,33938,36051,33906,21228,25486,55035,52954,212},
{23801,25881,25881,25881,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,27929,27928,27928,27928,27928,29976,29976,29976,29976,29976,29976,32024,29976,29976,29976,32024,29944,29944,29976,32024,32023,32023,32023,25717,27798,29943,31991,31991,31991,31958,31958,31958,31958,31925,31925,50777,46519,57051,54970,55002,59164,16937,23244,52890,54938,44405,50744,52858,46551,54970,50745,50712,55003,48632,52857,55003,46551,57050,52857,46551,54938,50744,50712,54970,48631,52824,54970,44405,54937,52824,48631,52857,50744,50744,54937,48599,52857,54937,48598,54938,52824,50711,54970,50711,50744,54937,46518,52857,52857,46518,54937,50712,48631,52889,48598,52824,54937,46485,54969,52824,48631,54937,50711,50711,52857,48598,52824,52857,48630,52889,50744,40212,44503,44503,40245,38133,36052,40245,35986,8584,14988,10795,10795,10762,15020,23473,17264,19408,10859,21360,12973,15053,12972,17199,10762,4357,4357,19181,19149,16775,16872,23177,31560,35783,33640,33673,42193,42161,35823,4357,6502,12679,42129,35888,8517,16903,46254,33708,31695,40212,35986,23373,21228,17002,17002,21229,21196,16970,19051,21163,40114,31565,33807,44405,40147,48631,50745,50745,42293,42292,40211,46517,46517,48597,48564,40178,50743,50743,31694,40145,33807,48630,46484,48564,46419,35952,29581,42258,48564,46483,44338,52890,31760,23308,21129,23274,19114,21195,29614,23275,33840,46584,27535,10631,46584,33906,40244,31793,21260,23341,25421,55003,55003,215},
{25881,25849,23801,25881,25881,25881,25881,25849,25881,25881,27929,27929,27929,27929,27929,27929,27929,27929,27929,27929,27928,27928,29976,29976,29976,29976,29976,29976,29976,29976,29944,29976,29976,29976,32024,32024,32024,32024,32056,21523,25685,31991,29943,31991,31991,31991,31958,31958,31958,31957,31957,31925,31826,40180,57051,57051,57083,57083,21130,35921,54938,54970,54938,46519,31728,52858,54970,54970,54970,54970,55003,54970,54970,54938,54938,54938,52889,52857,54938,54937,54938,54970,54938,54937,54970,54970,54938,54970,54938,54938,54938,54938,54938,54937,54938,54938,54937,54937,54938,54937,54937,54905,52857,52857,52857,54937,52857,52857,52857,52857,52857,52857,52857,52857,52857,52857,52857,54937,54937,52825,52857,52824,52857,52857,54937,52857,52856,52824,44471,42392,42359,40278,40246,36084,38132,40212,6536,4456,4456,6568,8649,10795,23473,19344,21489,12875,19311,17166,8649,4423,17166,10794,4357,4357,19181,19214,12678,12645,25256,33672,35784,33640,33705,42192,40016,37936,10599,8583,12679,40081,35921,10631,16935,44206,31595,31727,40212,44470,29712,25422,19148,12809,19148,21229,21195,16970,14889,35921,25291,40114,31760,46551,48664,50777,50777,42260,42292,38098,42324,46517,48565,48564,40178,48663,50744,50679,46484,38033,46485,46485,48564,48532,35952,27566,42258,48596,48564,44371,50810,33873,27565,27500,25420,33905,23275,31694,25356,33840,23309,21196,8583,17067,25487,29680,31792,23341,25454,25453,55002,55035,211},
{25849,23801,25881,25881,25881,25881,25881,25849,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,29977,27929,27928,27928,29976,29976,29976,30008,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,32024,32024,32024,25749,29943,29942,31991,31991,31990,31958,31958,31958,31958,31925,31925,31925,44471,52825,57051,57050,57083,57083,35888,42292,54970,50680,33874,44374,35954,44406,54938,33809,33808,35954,29583,42293,48631,27502,38034,29614,25421,52857,33808,31663,35921,25388,44373,48599,31695,38034,29614,29615,54938,31728,35953,33808,25356,42325,46486,27534,35920,29549,29614,54938,33808,38033,33807,27501,44373,44405,27501,33840,27469,33808,54938,31727,35888,31695,29582,48599,40211,31694,33807,23243,35953,52857,27502,35920,27501,27469,52792,35953,21230,15022,23408,10697,31793,38165,33841,40179,6503,4391,4423,4423,6536,12843,23505,21489,23635,21392,21392,17199,8649,4358,21295,10794,2276,4357,17035,19148,12647,12679,27338,33739,40045,40012,37933,44306,35757,40114,35953,29647,33775,40114,35921,14856,19048,42093,33740,29679,40245,44438,46584,27600,25487,19148,21228,23341,23341,21228,19115,21196,27436,31662,19084,33874,44438,48632,50777,42260,42292,40147,48631,48630,48565,48564,40178,48696,48663,48597,48564,38033,48631,48565,48564,48532,33840,29679,42291,44371,29516,46452,52890,31793,27533,35952,35952,48630,42259,31694,25323,33840,14857,25454,14922,31761,33905,27599,29679,27599,21228,23341,55035,57083,221},
{23801,25881,25881,25881,25881,25881,25881,25881,27897,27929,27929,25881,25881,25881,27929,27929,27929,27929,27929,27928,27928,29976,27928,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,32024,32024,29975,32023,31991,31991,31991,31991,31991,31990,31958,31958,31958,31925,31925,31925,54970,57050,57051,57051,57083,57051,52857,57051,54938,44405,25422,44439,35986,38099,52890,29615,27502,48631,14824,38099,46486,16937,40147,31760,19050,52857,25389,23275,48598,16937,42260,44405,21130,40147,33808,23276,54938,23308,27469,46518,16969,42260,42292,21162,42292,31727,25356,54937,25356,31727,46485,21162,44372,40179,19049,44437,29614,29614,54938,19018,35888,42259,23275,46518,38034,25388,48631,23211,33840,52824,19049,38033,38033,23243,52824,29615,10730,12876,19247,8616,29713,38133,25454,33873,4423,4358,4358,4390,6535,12843,23473,17199,19312,10794,21392,12940,6536,2277,19214,10729,4389,4325,17068,19181,6470,6405,25292,25324,33775,42194,33807,44307,35725,42292,31727,40179,46453,42194,44307,31726,23306,44174,35853,42291,44438,44471,46617,42391,25487,23341,10663,12744,23341,23309,14922,8582,23308,23243,19084,19116,29681,48664,50745,42260,42292,33872,29646,38032,48532,48532,35920,31759,38065,48564,48532,35887,31727,38033,48532,46483,31727,23308,38033,50678,40178,44339,50777,33841,23307,27532,29581,48630,42291,31694,25323,33840,14792,27502,8550,23308,29680,27599,31792,27567,31792,23374,55035,57083,220},
{23801,25881,25881,25881,25881,25881,25881,27929,27929,27897,27929,27929,27929,27929,27929,27929,27929,27929,27929,27928,27928,27928,27928,29976,29976,29976,29976,27928,29976,29976,29976,29976,29976,29976,29976,32024,32024,32024,32023,31991,31991,31991,31991,31991,31991,31991,31958,31958,31957,31925,31925,33972,54938,57083,57083,57050,57051,57051,52857,59164,40147,31727,21196,42293,33841,21131,33809,31695,27534,54970,23276,38034,44373,29647,44373,40147,27501,46485,29647,27501,54970,25420,40146,42292,33840,44372,40146,29614,46486,33840,29679,52824,25388,40146,42227,31727,46518,38033,29614,46518,33840,38033,50744,29646,42259,40179,23307,50743,35953,33840,46517,27501,40147,48598,31727,44372,40146,31727,52857,31727,35920,44372,35953,40178,44372,35920,46452,38066,12810,10763,12908,8584,21261,38132,19116,14825,4390,4358,4390,4390,6471,8649,17134,10795,10794,2310,15020,6568,4358,2277,12875,8552,4357,4357,12810,12842,6438,8453,16872,21001,23178,21065,23243,40114,35791,44373,40212,42324,46485,40082,46420,46452,25324,31531,37967,40178,46583,44471,46616,46617,38100,25454,23374,17035,21261,23373,23341,27567,35985,29614,27567,17036,19149,44439,48664,40212,42292,44372,44371,44339,44339,44339,44339,44339,42259,44339,44371,44339,44339,44371,44338,44339,42258,42226,44339,44339,44371,42291,48632,31728,35921,29646,29614,48631,42259,29581,14856,27501,8485,19083,12776,21228,29679,31793,35986,29712,25486,25486,55035,57115,225},
{23801,25881,25881,25849,25881,25881,25881,27897,27897,27929,27929,27929,27929,27929,27929,27929,27929,27929,27929,27928,27928,27928,27928,27928,29976,29976,29976,27928,29976,29976,29944,29976,29976,29976,29976,32024,32024,32024,32024,32023,32023,31991,31991,31991,31991,31991,31958,29877,31957,29845,31925,34004,54970,57083,57050,57050,57051,57019,52825,57083,38034,33841,14857,42293,33841,12744,19050,46518,40211,54970,48663,50711,50744,50712,48631,52792,50679,50711,48631,44373,54970,48631,48631,50711,48631,50711,50744,48598,50711,50711,46518,52857,48598,48631,50679,46485,50711,50711,48566,50711,50711,46518,52857,48630,48630,50711,46550,52824,50711,48630,48631,48631,48631,52856,48630,48630,50711,46486,52856,50711,48598,48631,48631,48631,50743,48630,48630,50711,23375,10763,14956,10729,21228,33939,21229,16970,6503,6503,6471,6471,6471,6503,6536,4423,4422,4390,6535,6503,4390,4390,6503,6471,4390,4390,6471,8583,8583,10630,12679,16872,18952,29549,19017,33776,33742,38098,25356,38099,42227,42195,44275,38001,27501,27371,31661,31727,40212,38132,46616,44536,46617,31858,36052,27535,25486,25454,25422,25422,29615,21163,27534,27534,23342,21262,36019,42293,42292,46484,48565,48564,48564,48564,46452,48565,48532,48564,48532,44371,48565,48564,48564,48532,44339,44371,46419,48564,48532,46484,46584,31728,19083,10630,19082,48631,42291,27469,21195,21162,14824,25421,19050,16969,29647,29712,25486,31825,25486,25453,55035,57116,222},
{23801,25881,25881,25849,25849,25849,25849,25881,27929,27929,27929,27929,27929,27929,27929,27929,27929,27929,27929,27929,27928,27928,30008,29976,29976,29976,29976,29976,29976,29976,29976,29976,32024,32024,32024,32024,32024,32024,32023,32023,32023,31991,31991,31991,31991,31990,31958,31958,31957,31925,31925,42391,55003,52857,57083,57050,59163,57050,52824,57083,42260,31696,12776,44438,33841,10664,19083,35953,42260,52857,46486,46519,46519,46519,46486,46454,44373,46485,46486,46486,46518,44405,46486,46486,46518,46518,44373,44373,46453,44373,44405,44373,42260,44405,44405,44405,44405,42292,44340,44373,44373,44405,44405,44341,44405,44405,44373,46453,44373,44405,46485,46485,44405,44373,44372,44405,44405,44405,44372,42260,44373,44373,44373,44373,42259,42260,42292,42292,29649,10764,17069,10696,8583,36052,31826,31826,29745,29746,29746,29746,29746,29746,29778,29778,29778,29778,29778,29778,29778,31826,29778,29778,29778,29778,29778,31826,31826,31826,31858,31858,31760,19017,14726,42228,33678,40114,23275,25356,29582,33743,40114,29582,40211,40145,40113,29614,40212,40212,44536,44536,46585,44472,44504,31761,29648,33906,31793,31793,33873,33873,31792,35954,21195,17035,23342,42325,42292,40178,46550,48598,48532,48532,40113,50743,50711,48532,48532,40113,48630,48597,48564,48532,38033,25388,40178,48564,48532,48630,46584,33873,10598,10598,12743,46486,44372,29613,27693,17226,10792,14888,10630,14856,29680,31825,25453,31793,25453,27631,55003,57148,227},
{25849,25849,25849,25849,25881,25881,25881,25881,27929,27929,27929,27929,27929,25881,27929,27929,27929,27929,27929,27929,27928,29976,29976,27928,29976,29976,29976,29976,29976,29976,29976,29976,29976,32024,32024,32024,32024,32023,29975,32023,29943,31991,31991,31991,31990,31990,31958,31958,31925,31926,31925,38230,54971,50712,57115,57051,57083,57050,52824,50712,44405,38067,27600,46551,31728,10631,23276,42292,44405,31728,17002,38034,44405,31728,21163,19050,25356,44405,42260,38034,23308,19050,40179,44405,40211,31695,23243,27502,46486,40146,33808,23308,19050,42292,44405,42259,29614,23243,31695,44405,40179,31727,25356,21195,44405,44373,40179,27501,23275,35921,46518,40179,29582,23275,25356,48598,44373,31727,23275,21162,40179,46485,42259,29646,23243,29614,46517,44372,33906,12811,14955,17035,6470,17035,25454,33906,42390,42391,40278,31924,36084,42391,42391,38165,36020,40246,42391,38197,33939,38165,40310,38198,33972,38165,42391,40278,38230,40278,42391,40245,33906,19050,16839,25194,25161,37969,14857,31760,40114,42162,46453,54871,38033,40112,42259,31760,38132,31793,44472,44536,44536,42391,40278,42358,50777,50809,50809,50777,50745,50777,50777,48631,25421,21228,21261,42325,42292,40146,42357,44437,38230,42324,40146,46583,48599,42226,46452,40113,46485,46485,46484,46484,38033,27534,40178,48564,48532,50678,52890,33874,19082,21194,23242,27468,33807,27500,25678,21452,8841,29743,10566,14857,29647,31825,25454,27567,27567,27566,55003,57116,228},
{23801,25881,25849,25849,25849,25849,25881,25849,27929,27929,27929,27929,27929,27929,27929,27961,27929,27929,27928,27929,27928,29976,29976,29977,30008,29976,29976,29976,29976,29976,29976,29976,29976,32024,32024,32024,32024,32023,32023,32023,31991,31991,31991,31991,31991,31958,31958,31958,31957,31957,31925,36085,54971,50744,57083,57050,57050,57050,48599,44406,42260,27535,14921,50712,31761,10664,19050,25390,35954,46518,46519,50712,52825,48631,44438,46518,46550,52792,50744,46518,46518,48598,50744,50776,50711,46518,46518,48599,50744,48631,46518,46518,46486,48631,50711,48630,46518,46518,48598,50711,50711,46518,46518,48631,50711,50711,48631,46550,48598,50711,50744,50744,48599,48599,48598,50744,50743,48631,48630,48630,50711,50744,50744,48631,48663,50711,52792,50743,48663,25487,33906,36019,8551,10631,19114,27598,44503,44471,40310,23504,31891,44504,42391,27568,14923,31794,42391,27601,12777,29680,42391,31826,17132,29713,42423,36052,23439,31826,35987,31728,23341,27534,38067,35791,37871,37904,8518,21162,35823,37968,44339,42227,19017,31726,33775,33841,38099,29713,42358,44536,44536,44504,44504,46584,48664,48664,50744,50777,50744,48664,48696,31760,25389,29680,25421,42325,42324,40147,46551,46583,42489,40244,40178,50809,48631,27501,44371,40146,46550,46517,46484,46451,38033,25421,40146,48565,48532,48598,52890,40180,33775,33775,40048,46453,40179,29549,29871,29840,27599,29680,10695,16970,25454,27566,27534,27534,25453,25486,52955,55035,224},
{23801,25849,25849,25881,25849,25849,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,27929,27929,27928,27928,27928,27928,29976,29976,29976,30009,29976,29976,29976,29976,29976,29976,29976,32024,32024,32023,32023,32023,32023,32023,32023,32023,32023,31991,31990,31990,31958,31958,31957,31925,31925,34005,54970,54970,57083,57050,57050,57050,40179,40212,25422,19083,12809,46486,31728,8518,14857,8518,27535,52857,38067,42293,42260,42325,48664,44406,40212,40179,35922,42260,52825,35921,42260,38067,35954,50744,44373,40212,38034,33841,44405,52857,38002,40147,35889,35921,50744,40180,38002,38034,33808,44373,48631,38066,40147,35921,35921,52824,40179,40179,35953,31728,44437,48599,33808,38066,40146,46453,54905,35921,38034,35888,35889,48598,44405,35921,35953,31727,40147,42357,19116,31760,12744,8551,14888,25485,44504,44471,40278,23504,31891,44504,44471,27568,10664,29746,42359,27600,8583,27600,42391,31858,15052,27600,44471,36084,17100,25487,44471,40213,27632,25519,44438,35824,29387,37969,14824,16936,35888,44306,46452,33775,16937,31727,27469,21196,38099,38132,38165,44536,44536,44504,42391,44503,50744,50777,50712,48664,48631,48664,33906,25422,21229,23309,21228,42357,42325,35953,25422,31793,42489,42325,33872,25421,33873,44405,42226,31694,25389,31726,42258,44306,29581,19050,29614,42258,44306,44372,54938,44406,23211,16904,14823,14824,12711,16937,31919,42420,38131,33971,33906,33906,31792,27599,25454,25454,25454,25453,52922,57115,226},
{25850,25849,25849,23801,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,27929,27929,27929,27928,27928,27928,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,32024,32024,32023,32023,32023,32023,31991,31991,32023,31991,31991,31990,31958,31958,31958,31925,31925,33973,57051,54970,57083,57051,57050,54905,38099,31728,44406,35987,25487,48632,33841,27502,31761,10631,29647,50712,16938,31728,29647,31728,46519,31728,23276,35986,12744,29647,48663,19050,31727,27502,19017,48599,33808,23276,38066,16937,38066,50712,19050,33840,27502,19017,48631,29615,25388,40146,14856,35921,44373,21163,35953,27501,21130,52824,29582,31695,38066,16905,38066,42324,23275,38066,31727,40179,54937,23275,31727,35921,21130,44373,38066,23243,40179,21130,31695,50777,16970,33840,27534,8583,10727,23371,44471,44471,40278,21424,31891,44504,44504,27633,12842,29778,42391,27600,8616,27632,42391,31826,17068,27632,44503,36084,17068,25487,44504,38165,25519,23406,44471,33809,21000,37937,29614,25421,38033,52758,50646,25356,10631,29615,40212,23308,33906,31793,38132,44536,44536,44504,42456,46584,50777,52857,52857,50777,50745,42261,25422,23309,21196,19083,25454,38131,38099,42259,38098,40178,40211,40178,42259,40178,42226,42291,42226,42258,42291,44339,44339,44307,44339,42258,42258,42258,42259,42292,50745,46518,42291,42291,42259,35920,21162,38097,40209,44403,44437,38131,14856,25421,29679,31760,25486,25421,23341,25486,52890,57116,225},
{25849,25881,25881,23801,25881,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,27929,27929,27928,27928,29976,29976,29976,29976,30008,29976,29976,29976,29944,29976,29976,29976,32024,32024,32024,32023,32023,32023,32023,31991,31991,31991,31991,31990,31958,31958,31926,31925,31925,33973,52890,54970,57083,57050,57050,54937,42292,40212,52825,40213,25455,50745,38034,52792,52857,10696,25389,40147,14824,38066,40147,19017,38034,23276,19082,55003,14824,23308,38066,14856,40147,38066,14791,38034,25388,21163,52889,16904,27469,38066,19017,44405,38034,14856,38034,23243,25388,54938,16904,27533,33872,19017,44372,35921,19049,38099,21163,31728,50744,16904,29614,31727,19050,48598,31694,23210,40147,16937,35953,46518,16936,31727,29582,21098,52857,25388,23243,35921,16970,35953,40179,14890,6566,21291,44471,44471,40278,23537,31924,44536,44504,33972,27763,29778,42391,27600,12809,29713,42391,31826,14987,29713,44471,36084,17068,27632,44504,38165,23406,27632,42391,27600,12776,25421,35986,27534,35921,46485,44340,14857,10630,27534,44470,35954,38067,23308,36052,44504,44536,42456,42423,44503,50777,50777,50744,52825,46487,25390,21228,19148,21228,19115,19083,23341,23308,23308,25421,23308,23275,21195,23243,21163,21130,21130,19049,16969,16904,14791,12711,16872,19049,31792,44437,48631,48695,50777,31728,40180,42292,40244,42325,35953,25388,59128,48596,48597,42325,40179,14856,25388,27567,27566,25454,23341,25454,25486,50809,57083,228},
{25881,25849,25881,25881,25881,25881,25849,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,27961,29977,27929,27928,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,29976,32023,32024,32024,32023,32023,29975,31991,31991,31991,31991,31991,31991,31990,31958,31925,31925,31925,33973,44439,54970,57083,57051,57018,57050,42292,33873,29648,27502,25454,52857,44405,52857,54938,27534,29615,31760,35986,44406,42292,27469,29647,29615,31727,57051,29582,29614,31727,29647,44405,42292,29614,31727,31695,33840,54937,29614,31727,33807,33807,46518,44372,31727,33840,33808,35985,54937,31695,31759,33840,33840,48598,42292,33808,33840,33808,38098,52824,33807,35953,35953,33872,50711,40211,35921,38033,35921,42291,50679,35953,38033,38033,35953,52857,40146,35953,38033,35953,44437,46485,23309,10695,21291,44503,44472,40278,23537,31924,44536,46584,36085,29876,33972,42423,27600,10664,29680,42391,31826,14955,29713,44504,38133,19246,25520,44504,38165,21326,23374,42391,27535,8518,25389,42260,31695,14792,14824,40113,31727,19114,31728,50777,38034,40180,35954,36051,38165,44504,44504,40278,44471,52858,52857,52825,48664,27567,21229,19115,19115,21228,19115,19082,19082,17002,17002,19082,19082,16970,14921,16969,14857,14856,14856,12744,10630,10631,10598,8517,8550,10631,19083,46518,50744,50776,52890,35954,40212,46551,44503,42325,27436,23275,48628,46451,46484,44405,40180,16969,25356,27534,25421,23373,25453,25421,25454,50777,55035,226},
{23801,25849,25881,25881,23801,25881,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,27929,27928,27928,29976,29976,29976,29976,29976,29976,29976,29976,29944,29976,29976,32024,32024,32024,32024,32023,32023,32023,31991,31991,31991,31991,31990,31990,31990,31958,31926,31925,31925,31925,33972,52825,57083,57050,57050,57083,48599,46486,54938,54970,54971,59131,54970,54938,54938,54938,54938,54970,57051,54938,54970,54970,54970,57050,55002,54938,54937,54937,54938,54970,54937,54938,54937,54937,54937,54938,52857,54937,54937,54937,54970,54970,54937,54937,54937,54969,54937,54937,54937,54937,52889,52857,52857,52857,52857,52857,52857,52857,52857,52824,52824,52824,52856,52857,52857,52857,52824,52824,52856,52856,52857,52857,54937,52857,52857,52824,50744,52824,52824,52824,50743,40212,12874,17067,44503,44472,40310,23440,31891,44536,46584,34004,29779,38165,44471,29681,10696,29713,44471,31858,12842,29681,44504,38133,25618,29745,44504,38165,25519,25519,42391,25454,6405,21195,19049,19017,12711,12711,23308,35920,38066,35986,48599,31760,40244,42293,38165,42423,40310,42456,44472,40277,52825,50744,52889,42292,25454,23341,19083,19050,21196,21195,19082,14889,14889,17001,19050,16937,12776,14856,14888,14857,12776,12711,10631,12711,10630,8550,8518,8518,8550,10663,40212,50777,50777,52890,33842,40180,38164,44536,40179,31695,23243,59128,40112,46484,40245,40244,27469,25389,27566,25453,23341,25421,25454,25454,48728,55067,228},
{25849,25881,25881,23801,25849,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,27928,27928,27929,27928,27929,29977,27928,29976,29976,29976,29976,29976,29944,29944,29976,29944,32024,32024,32024,32024,32023,32023,32023,31991,31991,31991,31991,31991,31990,31990,31958,31958,31925,31925,31925,34005,52891,55002,57050,57051,57083,48599,52825,54970,52825,48631,54938,50744,50744,54938,48631,52857,54938,46519,54970,52857,48631,54937,50744,52792,54970,46518,52857,54938,46486,54938,52825,46518,54937,50744,50744,54937,48599,52857,54938,46518,54970,52824,48631,54938,50679,50711,54937,46486,52857,52857,44405,54905,50712,48599,54937,48598,50744,54937,44372,52857,52792,44405,52857,48598,48598,52857,44405,50744,52857,42259,54937,50679,44405,52857,46486,48599,52856,42291,52824,50711,40147,23341,17002,44471,44471,40278,23439,31891,46585,46585,31859,12874,33907,44504,29681,12809,29713,44472,33906,14922,29713,44536,38133,21359,27632,44504,38165,17100,25487,44471,29713,21228,23308,21195,16969,14792,10598,10599,12744,33872,33873,25390,27535,42325,36019,40278,42423,44471,40245,46552,40212,50744,50712,50776,50776,33906,27535,23309,17034,21228,23309,21228,16969,12808,17002,19083,19083,14856,10695,14888,14888,14856,12743,10630,10631,10598,10598,10630,8518,8550,10663,31760,50744,48664,52890,35955,38132,46551,46616,38099,42293,25356,59128,33772,46484,44535,31826,12712,19082,25486,29679,25453,23308,23308,25421,48728,55035,225},
{25849,25849,25849,23801,25881,25881,25881,25881,25849,25881,25881,25881,27929,27929,27929,27929,27929,27928,27929,27929,27928,27928,27929,29976,30008,29976,29976,29976,29976,29976,29944,29976,29944,29976,32024,32024,32024,32023,32023,32023,31991,31991,31991,31991,31991,31990,31958,31958,31958,31925,31925,31925,31925,33973,33972,36020,50778,50713,54938,50712,54970,42293,29680,54938,46518,48599,54970,40180,52825,54938,35953,57051,50712,40179,54937,46486,46518,57018,38066,50712,54938,35953,54937,50711,38066,54905,46485,46485,57018,42259,50744,54970,40146,54970,50711,44405,54937,44437,48598,54970,42259,52857,52857,40179,54905,48631,46485,54937,44373,48631,54937,40147,52857,50744,44372,54937,46518,46518,54905,44340,50711,54905,40179,54937,48631,44372,54937,46485,48631,52856,42292,52824,52792,42324,38100,19180,44471,44504,40278,25585,33972,44537,44536,44536,44504,44472,44472,44472,42391,42391,42424,42424,42391,42359,44472,42424,42424,40310,42391,42391,42423,40278,40310,40278,40277,38165,33906,19050,14889,12744,8551,10664,23341,23342,29713,16970,40212,19148,38133,42359,44504,40245,42326,40212,48696,48663,52824,50744,40245,31761,29680,19115,12808,23341,27535,23341,14889,10695,19082,21195,23276,12775,10662,14888,16969,16969,6437,6405,10631,10598,10630,10566,8518,10630,29615,48632,48729,52890,35955,40180,44470,48761,40212,50745,23276,16904,29579,42290,44470,31760,12744,16970,23406,29680,25421,23341,23341,23341,48697,57083,221},
{23801,25849,25881,25881,25849,25849,25881,25881,25881,25849,27929,27961,25881,27929,27929,27929,27928,27929,27929,27929,27928,27928,27929,29976,30008,29976,29976,29976,29976,29944,29944,29976,29976,29976,32023,32024,32024,32023,32023,31991,31991,31991,31991,31991,31991,31990,31990,31958,31958,31957,31957,31925,34005,33973,31924,31924,33940,36020,36020,42293,54970,42293,29615,54971,44405,48599,54971,31696,52825,52857,33840,57051,48599,38034,54970,44405,46485,54970,38066,50712,54937,35920,54937,48566,35921,52857,44405,44405,54970,38033,52792,54937,33840,54970,48598,42259,54937,42259,46453,54937,35920,52824,52792,35921,54905,46486,42259,54905,40146,48598,54937,33808,52857,48631,38066,54905,42292,42292,54905,35953,48598,52857,31694,54905,46485,35953,54905,40179,46485,54937,31727,52824,50711,38001,50776,23374,44472,44504,42359,27633,29811,27763,27763,27763,25552,12874,29746,21262,12809,8649,29811,23440,19181,12745,29779,25650,23472,10729,27665,27633,25552,12842,23342,19116,10697,10697,12810,10696,10664,10696,14922,10696,8616,8616,8648,10696,38132,40213,40278,42391,44504,44504,44504,42325,48664,48663,52857,50777,42358,40245,29680,27600,21228,19083,25454,27567,27534,19115,17002,21228,23276,23340,16969,14889,21195,19114,17002,12776,10663,10598,10598,12646,10598,10631,27534,48664,48697,52890,35955,38067,46583,50842,46551,44405,35921,10630,23208,42290,48630,46485,19082,19082,27567,31825,25486,23340,21260,21261,48696,55035,220},
{23833,25849,25881,23801,25881,25881,25881,25881,25881,25881,25881,27929,27929,27929,27929,27929,27929,27928,27929,27929,27929,29977,29977,27928,29976,29976,29976,29976,29976,29944,29976,29976,29976,29976,32024,32024,32023,32023,29975,31991,31991,31991,31991,31991,31991,31990,31990,31958,31958,31958,31957,31925,33973,33973,31925,33972,31827,29649,27568,40213,54970,52858,50712,54938,52857,52858,54970,50745,52889,54970,50744,54970,52857,50744,54937,52825,52825,54938,50711,52825,54905,48631,52857,52825,50712,54905,52824,50744,54937,50712,52857,54905,50712,54937,52857,52824,54937,52824,52856,52857,50744,52857,52857,50711,52824,52857,52792,52824,50744,52824,52857,50712,52857,52824,52792,52857,52824,52824,52857,52824,52792,52824,52824,52824,52824,52856,52824,52792,52824,52792,52856,52824,52824,52824,52824,33906,44472,44504,44472,25487,27665,27731,29811,27698,23440,17068,27698,19181,12810,8648,29811,25618,21359,10762,27731,25521,19214,12809,27698,25585,21390,17035,8518,14890,10697,10697,12842,12842,10696,10697,19115,17002,8584,8616,8616,10729,38164,48696,38165,38197,44504,44504,44504,40277,48696,50744,52857,50777,46551,46616,33938,31793,25487,14889,19148,29647,27599,25454,16969,19115,27534,27534,23308,16969,19082,23308,21227,17002,12776,12743,12743,10598,12679,14856,27502,46551,48664,50810,33842,35987,44438,48697,42390,38099,50712,29581,27467,44371,50710,48630,12743,16969,27534,25486,23340,23341,21260,19115,46616,55035,217},
{23801,25849,25881,23801,25881,25881,25881,25849,25881,25881,25881,27929,27929,27929,27929,27929,27929,27929,27928,27929,27928,29976,29976,29976,27928,29976,29976,29976,29976,29976,29944,29976,29976,29944,32024,32024,32023,32023,32023,31991,31991,31991,31991,31991,31991,31990,31958,31990,31958,31958,31957,31925,34005,33973,33973,33973,25488,27535,27535,40180,54970,54970,54938,54938,54938,54938,54906,54938,54970,54938,54971,54938,54938,54938,54937,54970,54970,54970,54937,54905,54905,54905,54937,54905,52857,52857,52857,52857,52857,52857,52825,52825,52857,54905,52825,52857,52825,52825,52857,52825,52825,52824,52824,52824,52824,52825,52824,52824,50744,50744,50744,50712,50711,50711,50711,50712,50712,50679,50711,50679,50679,50711,50679,48631,48599,48599,48598,48566,48566,46486,46486,46486,46518,46485,46453,40212,44471,44439,40278,29745,29778,27730,27731,29811,14987,14922,27731,21294,12874,10761,29811,25553,19214,12842,29779,25553,21359,12842,25617,25585,25552,19181,6470,6470,10664,14923,10729,10729,8616,10697,10697,12777,6503,8616,10697,12777,36051,31761,29713,40278,42391,42391,42391,42358,50776,50744,50744,50744,44406,46616,44536,31825,31760,23342,16970,23276,33873,31728,23341,14889,21228,27501,27534,21195,14889,19082,25453,25421,16969,8550,10630,14792,12743,14824,25422,46584,46616,52858,38068,36019,48697,50875,46648,40244,46486,44339,42293,46485,48597,38032,31727,23308,23373,25454,23340,23341,23373,21260,46583,55002,213}
};

const uint16_t Montreal PROGMEM [320][241] = {
{53053,53054,53054,51005,51005,51005,51005,51005,51005,51005,51005,51005,50973,50973,50973,50973,50973,51005,51005,50973,53021,53021,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52987,52987,52955,52955,52955,52955,52955,52955,52923,52923,52923,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50810,50810,17231,4684,2636,2636,2636,2604,2636,4684,4684,4652,4652,2604,4652,4652,4652,4619,4651,4651,4619,4651,4651,4651,4651,4651,4651,4651,4619,4618,4619,4619,2571,2571,2571,2538,2538,2538,2538,2538,2538,2538,2506,2506,4554,4553,4554,4554,4553,4553,4554,4553,4521,4521,4521,4521,4521,4521,4521,4520,4520,4521,6535,6535,4521,6503,6568,6503,6536,6503,6536,6503,6502,6502,6502,6535,6470,8518,8518,6503,8518,4552,2602,586,586,6568,6503,6503,6535,6503,6535,36049,40178,42324,46516,56950,42224,42224,52789,56950,42223,44337,54902,56950,50611,8516,8517,42258,52756,12709,12742,44402,56949,18950,8517,23340,54870,48693,23273,14888,54870,42158,10628,10661,50741,50740,44466,10596,44402,54836,18982,10628,38096,54836,38093,18981,27532,54837,29578,10597,19113,52789,44303,14918,16999,52789,44239,10532,10564,48596,48496,8484,8484,40209,52722,12676,33839,31562,18984,44503,63321,63255,63255,63255,63255,63255,65303,63255,248},
{53054,53054,53053,51005,51005,51005,51005,51005,51005,51005,51005,51005,51005,50973,50973,50973,50973,50973,50973,51005,53021,53021,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52987,52955,52955,52955,52955,52955,52923,52923,52923,52923,50875,50875,50875,50875,50874,50842,50842,50842,50842,50842,46714,42520,42488,10827,6829,4781,2733,4749,2668,588,523,4521,4488,4488,4488,4488,6536,6536,6504,6504,6503,6504,6536,6504,6536,6503,6503,6504,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,6633,6633,6503,6503,6503,6503,6503,6503,6503,6502,6471,6503,6502,6503,6470,6502,6502,8518,6568,6536,6535,4553,6535,6569,6535,6568,6503,6536,6503,6503,6503,6503,6536,6502,6502,4488,6536,8518,4520,522,586,587,4520,6536,6535,6535,6503,6535,33969,40178,38130,57016,50611,40078,42191,57015,50610,40110,35886,54870,54869,50675,40207,25385,42290,52756,40142,33900,46482,54837,27401,19048,29645,54837,48627,29612,23275,52789,44271,21160,21193,50676,50676,44466,21128,44370,52723,23240,19080,38064,52756,40206,42319,35950,52756,33804,19080,25419,52756,46449,35916,27530,50676,44303,21224,17000,46515,48464,16935,16903,40176,52691,21128,35985,27369,27567,63355,63256,63223,63255,63255,65303,65303,65304,63255,248},
{53053,53054,53053,51005,51005,51005,51005,51005,51005,51005,51005,51005,51005,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52956,52955,52955,52955,52955,50907,52955,52955,52923,52923,50875,50875,50875,50875,50875,50874,50842,50842,50842,46682,27763,8779,6601,21326,31727,35953,35985,33937,33904,27663,27631,27598,27566,29614,29582,29581,29613,29581,27533,29613,29613,29614,31662,29614,29581,29581,29581,27533,29613,29614,29614,29614,29614,31661,31661,31661,29613,29613,29613,31661,31662,29614,31661,29613,25388,15019,12907,14987,19115,19115,19115,19115,19115,19115,19114,19115,17035,19083,19083,17002,12777,6568,6568,6569,6568,6536,4521,6536,6569,6535,6568,6536,6536,6536,6503,6503,6503,6536,6503,6503,6536,6536,6502,6502,2505,587,619,4520,6535,6535,6536,6536,6503,36050,40178,40210,59095,44271,40110,48563,59095,44271,42159,23307,52822,54870,52756,42352,25353,44338,54804,42255,37997,42257,54869,29482,21161,31693,54837,48595,33740,25355,52789,44303,23240,25353,50676,52724,44466,25289,44370,52723,27369,23241,40144,52756,42222,33804,35886,52756,37932,23241,29579,52756,48562,42223,31691,50676,46351,25289,23241,46482,48497,23208,23209,42224,52691,27369,35920,16969,59195,63288,63223,63223,63223,63223,63223,63223,63255,63255,246},
{53054,53054,53053,51005,51005,51006,51005,51005,51005,51005,51005,51005,51005,50973,50973,50973,50973,50973,51005,50973,50973,53021,53021,50973,53021,53021,53021,53021,55069,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52987,52955,52955,52955,52955,50907,50907,52955,52923,52923,50875,50875,50875,50875,50874,50874,50842,46681,25682,8746,12874,21293,21391,25551,44338,29744,17295,17230,31791,36016,21324,40178,23534,42323,23501,35984,21259,29710,27533,23372,33839,21260,38097,15018,42258,21421,36016,21259,27598,27565,21325,31791,17098,40112,17098,38096,15018,33839,25484,29678,33968,19211,31726,14953,23373,6536,6699,4617,8746,6633,8681,6633,6600,6698,4585,8746,4487,8714,4520,6634,6535,6536,4488,6535,6536,6503,6568,6470,6536,6470,6536,6503,6503,6503,8518,8518,8518,6504,8518,8518,6503,6503,8518,8518,6470,554,619,6535,6536,6536,6536,6536,8616,38097,35952,48629,59095,59063,57014,59063,59095,59095,52757,21228,57015,59063,59063,59063,59063,59063,59063,59063,59063,59063,57015,57015,57015,57015,56983,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56982,56950,56950,56982,56949,56949,56949,56949,56949,56917,54869,54837,52756,52789,54869,54869,54837,54837,54837,50610,40081,46551,63321,61110,52691,52691,61142,63223,63190,63223,63255,63255,245},
{55101,53054,53053,51005,51006,51006,51006,51005,51005,51005,51005,51005,50973,50973,50973,50973,50973,50973,51005,51005,51005,53021,53021,53021,53021,53021,53021,53021,55069,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52987,52955,52955,52955,52955,50907,50907,50907,52923,52923,50875,50875,50875,50875,50874,46681,25650,8746,12874,19245,19375,19408,17360,19408,44339,21357,8877,10989,29679,42257,35952,42258,33904,46419,38129,42257,35984,40145,40177,40145,42225,33872,46418,33903,44338,38194,44370,38064,40177,40113,38032,42193,33839,44338,31758,44338,31758,40145,35952,38064,38064,35919,42192,31726,23405,6666,6699,6665,6699,8746,8746,8746,6666,6666,8811,6666,8778,6666,6666,6634,6633,6633,6633,6601,6601,6633,6569,8648,6569,8616,6568,6600,6568,6601,6535,6503,6503,6536,6503,6503,6503,6504,6503,8518,8518,4520,587,6535,6536,6536,6536,6536,6536,33937,35984,59096,59128,59128,59095,59095,59095,59095,38000,38065,59095,59095,52788,46547,48595,59063,52756,46449,44402,54934,56949,48595,48595,54901,57014,48595,44368,50676,57014,44336,40143,46482,57015,50708,46546,46482,57014,52788,48594,44368,56982,52788,44368,44336,54869,54836,40110,38029,52756,56949,44336,42255,48595,52723,21128,12775,10760,14987,14855,12743,12776,12906,14823,12710,27697,57081,52756,52724,59030,56949,56917,54836,54869,54868,56917,54870,232},
{55102,53054,53054,51005,51006,51006,51006,51005,51005,51005,51005,51005,51005,50973,50973,50973,50973,50973,50973,51005,50973,53021,50973,53021,53021,53021,53021,53021,55069,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52987,52987,52987,52955,52955,52955,52955,52955,50907,50875,52923,50875,50875,50875,50875,46649,23570,8714,14923,19245,17295,17328,17360,17360,17328,19407,44339,27566,27566,27599,35920,38032,29646,38000,27565,38032,29743,38032,27597,33807,33839,31759,35952,25453,40145,23404,40145,27629,38064,27565,31759,33806,29646,38064,29645,40144,25387,40112,23308,33807,29614,31727,33806,27533,35919,21259,17132,8746,6666,4618,6698,8811,8779,8746,6633,6633,6666,6666,6665,6633,6698,6634,8681,6633,6633,6601,6601,6600,6569,8648,6568,8648,6600,6600,6568,6601,6535,6503,6503,6536,6503,6503,6503,6503,6503,8518,8518,8550,2537,6502,6503,6503,6503,6503,6502,29808,44436,59096,59096,59095,59095,59095,59095,56983,19115,54903,59095,59063,46514,44401,48628,59063,46449,42255,46450,59095,48562,48595,48627,57015,52788,42255,40143,56982,52788,42223,40143,52821,54901,42288,37997,50708,56982,48594,46481,50675,56982,46416,40077,44369,56982,46416,40141,42256,56982,50609,40174,42223,42225,21161,8517,8517,6601,8583,8517,8517,6535,6535,8517,10564,8582,8615,8517,8517,44436,35919,29579,27434,29514,29612,33774,23407,230},
{55102,53054,53054,51005,51006,51006,51005,51005,51005,51005,50973,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53053,55069,55069,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50875,52923,50875,50875,44601,23537,8714,14922,19278,19440,17393,17360,17360,17360,17360,15280,19408,31856,33871,31823,33871,42226,42192,21325,21293,25453,44306,25647,40210,33968,35984,38129,31823,38097,25550,44370,23436,42290,23437,38065,29678,33904,33903,29711,38064,19244,42290,21324,42258,23372,33872,31920,31791,31758,25517,38096,23436,15020,10826,6698,6698,8779,4715,6731,6666,6665,6666,8745,6634,8777,6634,8745,6633,8746,6665,6665,6632,6601,8681,6601,8713,6601,8713,6600,6600,6568,6568,6535,8518,6470,6536,8518,8518,6503,6503,8518,8518,8550,8550,8550,8517,8517,8517,8517,8517,8517,25486,52855,59096,59095,61208,61176,59095,59095,46418,27566,59095,59095,52788,42288,42256,54934,54869,42288,44368,52821,56949,42256,48627,54869,57015,48594,46514,50708,57014,44336,38062,44370,59062,50675,44368,44369,56982,50675,44400,46449,56982,52723,42222,40110,54869,52723,40077,38029,52788,54836,33836,25418,29677,14953,8517,8517,8583,6568,8517,8517,8550,8584,8518,10564,10565,8517,8517,10565,8550,44436,33805,27466,25321,27467,31726,17067,48729,247},
{55102,53054,53054,51006,51005,51006,51005,51005,51005,51005,51005,51005,51005,50973,50973,50973,50973,50973,50973,51005,53021,53021,53021,53021,53021,53021,53053,55069,55069,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,52955,50907,50875,50875,42521,21457,8682,14955,19278,19408,17361,19473,17360,17360,17360,17328,19408,15248,17328,13102,11021,13134,10989,31759,35919,6600,6568,12841,42225,31823,40177,36016,38065,38065,35952,40113,33839,44305,33871,42257,33839,40113,38000,40113,40145,40112,42192,31758,42257,31758,38032,25420,33807,35984,35951,40112,35919,40144,27597,12907,8746,6666,6731,6698,6666,8746,6666,6666,6666,6665,6634,6665,6634,6633,6633,6633,6633,6601,6632,6601,6600,6569,6568,6568,6568,6568,6568,6568,6535,6503,6503,6503,6536,6503,6503,6503,6503,6470,8518,8518,8550,8550,8517,8517,8517,8517,8517,8517,27599,59096,59095,57015,56982,56982,57014,59063,25420,46484,59095,59095,44336,40175,44402,57015,48529,42256,44369,57015,48562,42288,48595,57015,54901,46481,42223,54902,54836,40110,40111,52821,54869,46481,40110,52757,56981,46514,44401,52756,56949,44368,42222,48562,56982,44335,40142,44336,56949,46448,31723,27467,19115,8582,8517,8518,6569,8550,8517,8517,10663,25224,8518,10566,10566,10565,10565,10565,8518,44372,31660,25353,21161,29612,21228,29811,63388,250},
{55101,53053,53054,51005,51005,51005,51005,51005,50973,50973,51005,51005,51005,50973,50973,50973,50973,50973,50973,50973,53021,53021,50973,50973,53021,53021,53053,53021,55069,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,52955,50907,42489,19376,6666,14955,19310,21521,17361,17393,17361,19441,17360,17360,17328,17328,17360,15280,11119,6861,4651,6764,6763,31727,35887,19082,19082,21195,42193,14953,31759,23405,27533,27501,19179,33806,17034,38064,14985,38064,17034,31726,23307,27501,29581,21259,33806,17033,38032,14921,31694,14856,27501,25549,25420,33838,25451,33838,17066,10827,10826,6666,8843,8778,6633,8778,6666,6698,6666,6665,6634,8713,6666,6632,6633,6665,6633,6633,8681,6601,8648,6568,8648,6568,6600,8615,6567,6535,6535,6502,6502,6502,6536,6470,8518,8518,6503,8518,8518,8518,8518,8550,8517,8517,8517,8517,8517,8550,31693,44337,54771,54771,52690,46385,46516,44305,19148,57048,59095,54901,44369,42256,52789,56949,40110,40143,50708,56982,44336,44401,52788,59063,52820,46481,48562,59062,46481,42288,46481,57014,46416,40142,42288,56982,50642,42320,44369,56982,50642,40142,40175,54901,50674,40109,40142,54869,52755,42222,40142,42192,12776,8517,8517,6536,6536,8517,8517,8550,10663,29449,12646,10566,10565,10565,10565,10565,8550,42291,29579,23241,25387,27500,17166,59227,65469,252},
{55102,53054,53054,51005,51005,51005,51005,51005,51005,51005,51005,51005,51005,50973,50973,50973,50973,50973,50973,50973,53021,53021,50973,50973,53021,53021,53053,53021,55069,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,50907,36214,31989,19344,8682,17035,21358,21521,17393,21586,17393,19473,17361,19441,17360,19408,17360,17328,17360,13200,15312,11119,11022,10989,8908,27598,40145,38065,38129,38097,38097,40178,40178,40178,40177,38097,42258,40177,31856,48532,31791,46451,33904,44338,35984,38097,40177,36081,44403,33903,46483,29710,38033,23340,38065,40274,35984,40177,33903,42257,23437,8779,8713,6666,8778,6731,6731,8778,6666,6698,6666,8746,6666,8713,6666,6665,6665,6665,6633,6601,8713,6601,8681,6536,8680,6568,8648,8615,6567,8648,6568,8583,6502,6502,6536,6503,8518,8518,6503,6470,8517,8517,8518,8550,8517,8517,8517,8517,8517,10631,33773,33609,52558,54705,52527,29417,33904,29483,36051,59095,59063,50642,44368,42289,57015,50610,40143,42256,57015,50643,42223,42321,56982,54869,44368,42223,54902,54868,40142,42223,54869,54836,40077,37997,52756,54901,44400,42287,50708,56950,46482,42288,48563,56949,42190,37997,46482,56949,44336,38062,42224,25355,8517,8517,8550,6568,8518,8517,8518,14825,18951,39915,20998,10565,10565,10565,10565,10598,8550,42292,29515,23273,27500,12875,48761,65469,65502,252},
{55101,53053,53053,51006,51005,51005,51005,51005,51005,51005,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,50973,53021,53021,53021,53053,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,48794,44601,17165,12874,17068,21423,21521,19473,21585,17361,21553,17393,17393,17361,17361,17360,15312,15280,15280,15312,11119,13200,11119,11021,6764,6828,10989,13036,12971,12971,15019,15019,15051,12971,15051,12971,8713,27534,40144,31790,42258,31694,42193,33839,40145,35919,35984,38031,31791,40177,33807,42226,29614,33807,27533,35952,36016,35919,38064,33838,40080,19212,6698,6633,4618,6698,6666,6698,6666,6634,6666,6633,6665,4586,6665,4586,6633,6633,6601,6601,6601,6633,6601,6600,6568,6633,6568,6568,6568,6535,6568,6568,6535,6503,6503,4488,6503,6503,6503,6503,8518,8518,8517,8518,6502,8517,8517,8517,8517,8517,8517,10597,21128,23242,23242,21161,12711,33839,25322,52855,59095,56982,44369,42289,50676,56982,42288,42255,48595,56982,42256,42288,48595,57015,46417,40142,46482,59062,46449,40143,44369,56982,44271,35884,42224,56982,48594,42288,44336,56982,48562,40142,42223,56950,48529,27466,19145,38097,31726,19081,19081,23341,10631,8517,8517,6536,8551,8517,8517,10728,29582,42194,44108,25159,10565,10598,10565,10565,10566,17101,46550,27434,27500,19147,34003,63388,65501,65502,246},
{55102,53054,53054,53054,51005,51005,51005,50973,50973,51005,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,50973,53021,53021,53021,53053,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,40276,27565,25420,25388,21423,19473,19473,19441,21554,15313,21554,17361,19441,17360,17360,17360,15312,15280,15280,15280,13200,15280,13167,15215,8941,4683,4715,6861,10989,8909,6796,6698,6666,4650,4650,4617,4520,23405,36016,31985,35985,12841,33872,12841,27566,21227,23372,25388,14921,31791,12906,33872,10695,23373,12938,25453,25582,19178,25485,19178,29646,10826,6699,8810,4618,8810,6666,6698,8778,6666,8778,6633,8746,6634,8713,6666,6665,6633,6698,8713,8648,8746,6601,8648,6535,6697,6568,6600,8583,8615,6568,6536,8518,8518,6470,6536,6502,6503,4488,4488,6502,6502,8518,8517,6502,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10630,35951,35919,57015,59063,52756,44369,44369,54935,50675,40176,40176,54902,50676,40143,40176,54902,52789,44369,42288,54902,52788,40110,38095,52789,52756,37997,38030,50709,54869,44400,40208,50709,52788,38029,35949,48596,54869,33772,10598,6601,6633,8550,8517,6536,4521,8517,8517,6503,6536,8517,8517,8550,25354,27336,31563,44076,29319,12645,10598,10597,10598,12842,42392,52758,31660,31727,19214,59260,65501,65502,65469,229},
{55102,53053,53054,53053,51005,51005,51005,51005,51005,51005,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,50973,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,40309,29678,29614,27533,23438,21423,21423,21391,21423,19311,21391,19343,19311,19311,19310,19310,19310,19278,19278,19246,19278,19278,17197,19245,15117,15052,15020,15020,12972,8844,6763,6666,6633,4553,4520,4521,4488,12874,21292,23373,23373,23309,23341,23341,21292,21292,21293,23308,21260,21260,21260,23308,21196,17035,19115,21227,21259,21228,21228,21228,19115,10761,6601,6569,6601,6601,6569,6666,8746,6665,8778,6697,8746,8779,8746,8746,6698,6698,8713,8713,8713,6633,6633,6568,8745,6568,6633,8616,8681,6535,6503,6503,6503,6503,6503,6503,6502,6503,6503,6503,6503,6502,8518,8518,6502,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8550,8549,33774,48597,57015,57015,46482,38063,44435,54902,40143,35982,42322,52790,31725,23404,29743,42356,31791,27629,29710,42324,29677,23307,25517,40243,27532,21226,25485,40243,31823,25451,25485,38162,29613,21193,23372,38130,29612,14855,6502,4618,6535,8517,8518,4521,8550,8517,8518,6536,8518,8517,8517,6535,31595,35657,31530,42029,44043,23079,10598,10566,12809,40311,50809,54871,35919,19214,48729,65469,65502,65501,61274,224},
{55101,53054,53053,53054,51005,51005,51005,50973,50973,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52955,55068,33971,10761,8681,14922,23471,21488,21489,21456,23569,19408,23569,19376,17360,17328,17328,19376,15247,17295,17295,17295,17295,17295,17263,15183,13135,11022,10989,13102,10892,8909,6731,4585,6601,6568,4488,4488,6536,6601,6666,6666,6666,6666,6666,6666,8714,6633,8714,8681,8681,6633,6633,8681,6601,6601,6601,8649,6601,6601,6601,8649,6601,8746,6633,6601,6568,6666,6633,8746,8778,6698,8810,8778,8778,6730,8778,8778,8778,8778,8746,8778,8746,6730,6633,6665,8681,6665,6633,8713,8681,6503,6503,6503,6503,6503,6503,6503,6502,6503,6503,8518,6503,6503,6502,6502,6502,8518,8518,8584,8550,8517,8518,8517,8517,8517,8550,8582,8550,27534,57015,57015,54869,38063,17034,21359,21293,10760,8713,19278,19213,8648,6600,13038,15053,6665,8778,15053,15053,8615,8583,12940,12940,8583,8583,10892,10892,6633,8583,8779,8811,6568,8583,8747,8747,8550,8550,4553,6601,8550,8518,6536,6536,8518,8518,6503,6503,8517,8517,8518,8550,19081,35723,31531,42061,50414,29384,12743,19115,50777,59163,61275,56983,25421,31956,63388,65501,65501,63355,59129,214},
{55102,53054,53054,53054,51005,50973,50973,50973,50973,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,61308,40210,19147,19115,19115,19245,19278,19278,19278,21358,19278,21326,17197,17198,17198,17198,17165,17166,17166,17166,17165,15117,15117,15085,15085,13004,10892,10892,8812,10859,8649,6601,4553,4520,4488,4488,4488,4488,6666,4748,2798,8845,4553,8812,4553,6699,6633,6666,6699,6601,6699,6568,8779,6536,4619,4553,6634,2635,4553,6634,6568,6634,6536,4586,6503,6602,6536,6666,8811,8746,6634,8778,4618,8746,4586,8810,6633,8746,6633,6536,6503,6503,6503,4521,6503,6502,6503,6536,6503,6470,6502,6503,6535,8518,8518,8518,6536,8518,6503,6535,6503,6503,8518,6503,2637,6601,8551,8518,6536,8583,8518,8517,8517,8517,8517,8549,8550,8550,17034,27664,29679,27631,25486,25389,12907,12875,12874,12874,12842,12842,12842,10761,10794,10827,12875,12875,10827,10794,10794,10827,10794,10762,10794,10762,8714,8714,8681,8649,8649,8681,8681,8648,6601,6601,8616,6568,6536,6535,8551,6535,8551,8550,8550,8518,8518,8518,8518,8518,8518,8550,12776,33643,27370,42061,52495,31464,10597,10663,42358,63389,63388,56951,21326,59228,65501,65502,65468,59161,57049,229},
{53054,53054,53054,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,59195,44403,27501,25388,23275,21391,17393,17425,17393,21553,17361,21553,15312,15312,15280,15312,15280,13232,15280,13232,13200,13167,13167,13167,13167,9007,4878,2766,653,4781,8811,8778,6698,4586,2505,2505,2505,4520,6666,6764,4749,8812,6699,8780,6666,6699,6699,6699,6700,6666,6699,6634,6699,6601,6667,4619,4586,4619,4586,6667,6601,6666,4553,4619,6601,4586,6601,6634,6634,6633,6601,6601,4586,6633,4586,6665,6634,6601,6569,4521,4520,6568,6536,4521,6536,6536,6536,4488,6536,6503,6503,6535,6536,6503,6503,6503,6536,6535,6503,6503,6503,6503,6503,6470,4750,8681,8551,8518,6535,8649,4488,8518,8518,8517,8517,8517,8549,8517,8551,4553,6503,6536,6600,21162,10663,10663,10631,8582,8582,8583,8518,8518,8551,8616,8583,8616,8615,8583,8583,8616,8584,8583,6536,6536,8583,8583,8551,8550,8550,6535,6503,8551,6535,6503,6502,8550,8550,8518,8517,8518,8518,8517,8517,8518,8517,8517,8517,8517,8517,8518,10631,27402,27336,39981,52462,33544,12646,10565,10631,36117,61341,54872,52889,65469,65502,65501,61242,57049,57048,238},
{53054,53054,53054,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,57082,48597,35952,31758,29613,25518,19408,17393,17393,21553,17361,21553,15312,15312,15280,15312,15280,13232,15280,15280,15280,13199,11119,11119,11120,6991,4878,13167,17263,17165,17035,17034,12874,6601,4553,4521,4553,4553,6667,6667,4521,6699,4520,6699,4521,4586,6601,6601,6634,4520,6667,4520,4619,6536,4619,522,4554,2538,4521,6602,6536,6634,6536,4586,6536,4554,6536,6634,6633,6633,6601,6568,4553,6665,4586,6665,6666,6633,4520,6536,6536,4520,6503,4521,6503,6503,6503,6536,6503,6503,6503,6503,6536,6502,8550,8550,6536,6503,6470,8518,6503,6536,6536,6535,6537,6536,8518,6503,6503,6601,4488,8518,8518,8517,8517,8517,8549,8517,6568,556,2537,6536,8778,29548,18984,8550,8550,8550,8550,6535,4488,8518,8550,8616,8583,8616,8583,8551,8583,8616,8583,8583,8616,8616,8583,8583,8551,8550,8550,6535,6503,8550,6503,6503,8550,8550,8518,8517,8517,8518,8518,8517,8517,8518,8517,8517,8517,8517,8517,8517,8550,25322,29416,39948,50414,44011,20998,10566,10597,10631,42423,61243,61276,61276,61243,59194,59129,57049,61175,244},
{53054,53054,53054,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,57115,52888,40243,29613,27468,29549,29646,25550,21424,21553,17361,21553,17360,17393,15312,17361,17360,17360,17360,15280,15280,13199,13200,9039,6991,6991,4911,23503,33839,23470,23535,23471,25486,10859,8812,4814,4749,6764,6699,6764,4651,6764,4618,6732,6634,6667,6634,6666,6699,6634,6699,6601,6732,6601,6699,4553,6634,4586,6601,4651,4553,6666,6568,4619,6536,4586,4521,4553,6569,6568,4553,6536,4554,6535,4554,6536,4521,6536,6536,4520,6503,6568,6503,4521,6535,4521,6503,6536,6503,6503,6503,6503,6536,8550,8550,8518,6536,8518,8518,8518,6503,6503,6502,6536,6503,4488,6503,8518,6503,4521,8551,8518,8517,8517,8517,8517,8517,8517,4553,653,490,2506,23503,44239,37899,31691,19049,8550,8550,4553,523,6503,8550,8616,8583,8583,8583,8583,8583,8616,8584,8583,8583,8584,8583,6535,8551,8550,8550,6535,8551,8550,6503,6503,6503,6503,8518,8517,8517,8518,8518,8517,8517,8517,8517,8517,8518,8517,8517,8517,8517,19049,33642,42061,52495,52462,39785,12645,10597,10597,12809,52955,61243,59163,57082,57049,57049,48499,33839,145},
{53054,53054,53054,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53020,53021,53021,53020,53021,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52956,55035,54969,40243,35985,33871,29581,25323,27468,31694,27566,21423,23601,17361,17393,17361,17361,17360,17328,17360,15280,15280,11119,13200,9040,2863,2863,782,13070,12939,555,2733,620,10860,6731,8812,4781,6797,6764,6731,6764,6699,6732,6666,6732,6667,6699,6667,6667,4651,6634,6699,6634,6699,4554,4619,6602,6634,4586,4586,4619,4553,4619,6601,4586,6569,4586,4553,4553,4553,6569,4554,4521,4554,6568,4554,4520,4521,6568,4520,4520,6536,4520,6535,4521,6536,4521,6535,4488,6536,6536,6536,6536,6536,6503,6503,6503,6536,6503,6503,6503,6503,6503,6503,6503,6503,6503,8518,8517,8518,6569,8518,8517,8517,10565,12646,8517,8517,8518,2570,620,2538,4619,19179,44304,39980,29610,31628,23177,14759,8517,6536,6503,8551,8616,8583,8583,8583,8551,8583,8616,8616,8583,8583,6568,8583,8551,8551,8550,8518,6535,8551,8551,6535,6503,6503,6503,8550,8518,8517,8518,8518,8517,8517,8517,8517,8517,8518,8517,8517,8517,8517,14889,33643,39949,50446,52494,48203,18885,14725,10597,10598,29778,57115,59163,57082,57049,59097,42095,19114,99},
{53054,53054,51006,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,48761,38130,42324,40211,38196,33905,27501,25291,27436,29613,27566,19343,17360,17360,17360,17360,17360,17328,15280,15280,13200,15248,11087,11119,11087,6894,13037,8747,587,653,620,6731,2538,6699,4813,4684,4716,4553,6699,4553,6732,4521,4619,4521,4618,6601,4553,4619,4520,4619,4488,4619,4455,4586,6536,4553,4586,4488,6634,6503,6634,6503,4554,6503,4553,6536,6536,6536,6535,2538,2473,6569,6470,4521,4488,6536,6503,6503,6535,6503,6536,6470,4520,6503,4521,6503,6536,6503,6503,6536,6502,6536,8518,8518,8518,6504,8518,8518,8518,6470,6503,8518,8518,8517,8550,8518,8517,8518,6536,8518,8517,8517,8517,12678,8517,8518,8518,6502,4521,4488,6699,14889,44304,37867,12709,27467,25387,25355,18984,6503,6535,8550,8616,8615,8583,8616,8551,8551,8616,8583,8583,6536,6536,6535,8551,8551,8550,8550,6503,8551,8551,6503,6503,8550,8550,8518,8518,8517,8518,8517,8517,8517,8518,8518,8517,8517,8517,8517,8517,8517,10695,27370,35820,50414,52527,52428,31399,20998,12645,10565,12777,52923,57083,57082,57049,59096,54804,37967,142},
{53054,53054,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53020,53020,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,50842,42356,38130,35952,36018,36116,36084,33906,29581,25290,25355,29581,25486,19342,17360,17360,17360,17328,15280,17360,13200,15247,42291,42258,38130,33937,21326,6764,4749,6797,4684,6732,4684,6732,6732,6732,6765,6732,8812,4651,6732,6699,6732,4619,6699,6699,4651,4651,4586,6700,4586,6699,6634,4619,4586,4619,6666,4586,6634,4553,4619,4586,4619,2538,4586,4553,4553,4553,6569,4554,4520,4554,6536,4554,6568,4521,6536,6568,6569,4488,4521,6536,4521,6536,4521,6535,6536,6535,6536,6536,6503,6536,6503,6503,6503,6536,6503,6502,6503,6503,6503,8550,8518,8517,8550,8518,8517,8518,8518,8518,8517,8517,8517,8550,8550,8550,8517,8550,6569,6535,8648,10696,42225,44174,21128,21129,8584,6634,6568,6536,6535,8583,8583,8583,8615,8615,8583,8583,8615,8583,8583,8583,8551,8551,8583,8551,8550,8550,8550,8550,8550,8550,6503,8550,8550,8518,8518,8517,8518,8517,8517,8517,8518,8518,8517,8517,8517,8517,8517,8517,8550,19017,33741,50446,52495,52494,43977,33511,14725,10565,10566,36084,57083,57082,57049,63323,63387,57015,215},
{53054,53054,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53020,53020,53021,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,50874,46582,44436,40211,38131,33873,33938,36084,36084,33938,27533,23242,27371,29549,25486,19310,17360,17360,15280,15312,15248,17328,42258,25550,6926,4846,4781,6731,4716,6764,4651,6732,4651,4683,4683,4651,4716,4683,6732,4618,6732,4618,6699,4586,4651,4618,4618,6667,4553,6699,4553,4619,6601,4586,4553,4586,4586,4554,4586,6569,4618,4554,4586,2538,4554,4553,4521,4521,6568,4554,4520,4553,6536,4521,6536,4521,6536,6536,4521,4488,4521,6535,4521,6503,4521,6535,6536,6535,6536,6536,6503,6536,6503,6503,6503,6536,6503,6503,6503,6503,6503,8550,8518,8517,8517,8518,8518,8517,8518,8518,8517,8517,8517,8517,8550,8550,8517,8517,8550,8517,8517,8582,42192,42029,25289,23209,25388,35954,31759,10696,8583,8583,8550,8550,8583,8518,8518,8550,8550,8518,8518,8550,8517,8517,8550,8517,8517,8517,8518,8550,8550,8550,8551,8550,8518,8518,8517,8517,8518,8517,8517,8517,8518,8517,8517,8517,8517,8517,10565,8517,8550,10630,31661,54640,54575,54574,52429,41929,14725,10565,10565,17100,55003,57051,61275,59129,59129,57016,196},
{53054,53054,51006,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,55069,55069,53021,53021,53020,53021,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,48696,42356,40309,38164,36019,29744,33970,36116,36084,33970,29582,25290,25323,29549,27533,19310,17328,17360,15248,19408,44338,23470,8974,6959,6829,6699,4683,6764,4651,6764,4651,4683,4813,2635,6732,6634,6732,4586,6732,4521,6699,2571,4651,6634,4553,6699,4553,6699,4521,4651,6536,4586,2538,4554,4586,4521,2571,2506,2571,4553,4586,6535,4553,2538,4520,4521,6503,4553,6503,4553,6503,6569,6503,4488,6536,6536,2505,2473,4488,6470,4521,6470,4488,6503,6536,6503,6503,6536,6502,6536,8518,8518,6502,6535,8518,8518,8518,6470,6503,8517,8517,8517,8517,8550,8518,8518,8518,8518,8518,8518,8518,8518,8550,19081,25355,25355,25421,25322,25323,23243,40112,39981,19015,19048,25354,33740,37934,12776,8518,8518,8517,8517,8518,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8550,8550,8550,8551,8550,8518,8518,8517,8517,8518,8517,8517,8517,8518,8518,8517,8517,8485,8518,8485,8517,10598,16936,52592,56720,46253,44205,50479,54541,23110,12677,10565,8551,44569,57115,44208,12743,23307,31792,169},
{53054,53054,51005,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,55069,55069,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,52955,50874,44536,38197,36084,34003,31857,33970,36084,36084,33970,29614,25322,25322,29581,27565,19310,19311,23503,46451,42226,38098,19311,10893,8812,8845,8845,8844,8845,8844,8844,8877,8844,8813,8812,8812,8812,8812,8779,8780,6732,8780,8748,8747,6699,6699,6732,6699,6700,6667,6699,6667,6666,6667,4619,4619,4619,4619,4586,4587,6634,4586,4586,4586,4554,6601,4554,6569,4553,6569,4521,6569,6569,4521,4521,4521,4521,4489,4488,4521,6536,4521,4488,4488,6536,6536,4488,6536,4488,6503,6503,6503,6536,6503,6503,6503,6503,6503,6503,8550,8550,8517,8518,8518,8518,8550,8517,8517,8518,8518,8518,8582,50776,63355,63355,63356,63356,63356,61242,44305,42093,14790,12709,23241,31627,37966,16937,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8550,8550,8550,8551,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8615,42092,46187,41995,39817,31495,39948,56719,54606,50446,52559,56719,54508,39848,23144,10598,8550,36149,61307,48466,23275,23243,23275,109},
{53054,53054,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53020,53020,53021,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50875,50874,46616,38197,36116,34003,31825,31857,36051,36084,33971,29647,25355,25323,29613,27500,50677,33806,23437,15084,6665,4553,2505,4586,2505,2505,2473,2473,2473,2473,4520,2473,4488,4488,4488,4488,4488,6503,6536,4488,4488,4488,4488,4488,4488,4488,4487,6504,6503,6503,6503,6503,6503,6503,6503,6503,6503,6471,6503,6502,6502,6470,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,6503,4521,6503,4488,6503,6536,6535,6503,6536,6503,6536,8550,6502,8518,6503,6503,8518,6502,6503,6503,8518,8518,8518,8517,8517,8517,8517,10598,8549,8517,8518,8518,8518,17034,50743,63387,63355,63388,63388,63388,61242,46385,44206,35884,35851,33707,35820,29613,19082,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8517,8550,8550,8550,8550,8550,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,14920,50445,50381,42060,44238,46318,50413,54607,50414,48398,48332,54606,46154,37900,14889,6536,6569,10860,19278,25453,8714,8615,8517,68},
{53054,53054,51006,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50875,50875,50875,50874,46616,38229,36084,36051,31825,31825,36051,36084,36051,29679,25388,23275,25356,27468,21390,8941,2700,653,620,588,555,523,490,490,490,523,458,490,2505,4521,4488,2505,4520,6535,4488,4488,4521,4521,2441,4488,2473,4488,4488,4488,6503,6503,6503,6503,6503,6503,6503,6503,6503,6502,6503,6502,6502,6502,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,6503,4521,6503,4520,6503,4520,6536,6503,6536,6503,6536,8518,8550,6503,6536,6503,6470,8518,6502,6503,8518,8518,8518,8518,8517,8517,8517,10598,8549,8517,8518,8518,8518,17067,48663,63356,63388,63356,63388,63388,63322,46385,44174,29578,29610,31659,31627,21227,23243,8550,8517,8517,8517,8517,10564,10565,8517,8517,8517,8517,8517,10564,10564,8517,10565,10565,8518,8518,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,10565,10565,8517,10565,8517,10728,21095,48366,46188,25320,27498,42158,52462,52494,46318,44303,42124,54606,41994,27467,6535,6536,6569,4553,4586,4586,6699,17099,10630,81},
{53053,53053,51005,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,52923,50875,50875,50875,50875,50874,46616,40277,36084,36051,31857,31825,34003,36084,36051,31792,29647,27436,17034,8713,4521,4521,6601,4586,4520,4520,6536,4488,4488,4520,4488,4488,4488,6536,6535,6536,6503,6503,6503,6503,6503,6536,6503,6503,6503,6503,6503,6503,6502,6503,6502,6502,6502,6502,6502,6470,6503,6470,8518,6470,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,6471,4521,6536,4521,6536,6536,6536,6536,6536,6503,6536,6503,6503,6503,6536,6503,6503,6503,6503,6503,6503,6503,4488,6503,8549,8517,8517,10598,8549,8517,8518,8517,8583,21196,46550,63356,61242,54936,59162,63355,63322,46385,42125,23304,23272,25320,29482,37934,16937,8616,8550,8518,8550,8583,8517,8518,8583,8517,8517,8550,8550,8517,8518,8550,8517,8517,8550,8517,8517,8518,8517,8517,8517,8517,8517,10565,8517,10565,10565,10565,8517,10533,8582,14824,21095,46285,46188,12645,19080,37900,18886,31529,48399,44304,25256,16838,19047,21258,6535,6536,4553,4553,2538,2538,2538,6536,8485,90},
{53054,51006,50973,51005,51005,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50875,50875,50875,50875,50875,50875,50874,50842,46616,40277,36084,36051,31857,31824,36051,33906,29680,40113,44306,33872,10893,6764,6764,6764,6764,8812,8780,6732,6732,6732,6732,6732,6699,6699,8780,6699,6699,6731,6731,6699,6699,6699,4651,4619,6667,6667,6667,4619,4618,4619,4619,4619,4587,4587,4586,4586,4586,4586,4554,4586,4554,4554,4553,4553,4553,4553,4553,4521,6569,4521,4553,4521,4521,4521,4521,4521,4521,4521,4521,4521,4521,4520,6568,6568,4520,4488,6536,8518,8518,8518,6503,6502,8518,8518,8518,6503,8518,4455,425,6503,8550,8550,8517,10598,8517,8518,8550,8518,6569,14890,44437,63388,59096,44338,54936,63356,63322,46385,46319,23240,21129,29547,33740,33740,12776,12775,10630,10598,12711,10630,10598,10630,10662,10597,8550,8583,8517,8518,8550,8518,8517,8518,8518,8517,8517,8582,8517,8517,8517,8517,10565,8517,8517,10565,10565,10565,8517,8517,31793,33708,23175,46253,48268,16870,21128,33675,12613,23176,39882,25353,10564,12645,12645,12678,6535,6503,4521,4554,2538,2539,6535,8517,8516,59},
{51006,51005,51005,51005,50973,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,52955,50907,50907,50875,50875,50875,50875,50875,50875,50874,50874,50842,50842,46648,40277,36084,36083,33905,31694,25388,46451,48531,38033,15053,13006,13038,15118,17167,15086,15119,13038,15118,15086,13006,13038,13006,13038,12974,13006,10925,10958,10925,10925,10893,8845,8845,8812,8845,6764,8812,6700,6732,6700,6732,6699,4651,4651,4619,6699,4587,4619,4586,4618,4554,4586,4554,4554,4553,4553,4553,4553,4553,4521,4521,4521,4521,4520,4521,4521,4488,6536,6536,6536,6536,6536,6536,6536,6536,6536,6536,6503,6503,6503,6536,6503,8550,8550,6503,6504,8518,4488,457,6503,6503,8518,8517,8517,8550,8550,8582,23373,4553,12842,40244,63355,59128,44337,52856,63355,63322,46385,44206,25289,23209,27434,27434,27434,42191,44173,40012,42125,46319,40012,40013,44271,42093,42093,31627,10630,8517,8518,8551,8517,8517,8550,8517,8517,8550,8517,8517,8517,8517,8517,10565,8517,10565,10565,10565,10533,8517,8616,16937,19016,23240,46286,54607,52593,52592,46221,10564,18983,42026,35722,23078,10565,16902,21096,6503,6503,6568,4586,4554,6535,10564,10565,10533,37},
{51006,51005,51005,51005,50973,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50907,52923,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,46648,40309,36084,33970,33840,50613,50612,46419,21326,15086,15118,17167,17199,15086,15119,13006,15086,13006,13038,13038,13006,13038,13005,13006,10957,10957,8845,10925,8813,8813,6764,6732,6764,6732,6732,4652,6732,4651,4651,4619,4651,4619,4587,4587,4586,4586,4586,4586,4553,4554,4521,4521,4521,4521,4521,4521,6569,6568,6536,6536,6536,6536,6536,6536,6536,6536,6536,6536,6536,6536,6503,6503,6503,6535,6536,6503,6503,6503,6536,6503,6503,6502,6502,6503,6502,6503,2473,6535,6503,8550,8517,8517,8550,8550,10663,31954,2538,10761,36051,63356,61209,46451,54936,63356,63355,48465,44207,23176,21096,23273,25321,35853,50544,44205,44237,46383,46285,44205,44238,48399,44205,42093,21096,8517,8517,8550,8517,8517,8550,8518,8517,8517,8550,8517,8517,8517,8517,8517,8517,10565,10565,10565,10565,8517,8550,8681,8485,10597,19047,44141,50414,23273,19016,18983,10565,16870,35624,12710,10597,8516,12677,8518,8551,6535,4521,4586,4554,8518,10565,10565,10565,38},
{51005,50973,51005,51005,51005,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,52988,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50907,52923,50875,50875,50875,50875,50875,50874,50842,50842,50842,50842,50842,50842,48697,44503,44438,50677,48563,46419,21326,15086,15118,17166,15118,15086,15118,13006,13038,13006,15086,13006,13006,13006,10925,13038,10925,10925,10893,10925,8813,8813,8812,8813,6764,6732,6732,4651,6732,4619,4652,4619,4619,4619,4619,4587,4586,4586,4554,4554,4554,4553,4521,4521,4521,4521,4521,6568,6568,6568,6536,6536,6536,6536,6536,6536,6536,6536,6536,6536,6504,6503,6503,6503,6503,6536,6536,6536,6536,6536,6536,6536,6504,6536,6536,6504,6503,6503,6503,6503,6503,8518,8517,8517,8615,8616,10696,13101,15052,19115,33906,63355,63356,63388,63356,63356,63355,48465,46319,16870,12677,16935,19079,23241,25321,23208,29579,50577,50544,52657,52657,52689,29450,21096,42060,37900,37867,31593,12677,8518,8550,8517,8517,8518,8517,8517,8517,8517,10565,8517,8517,10565,10565,10565,10565,8517,8583,8550,8484,8485,16935,42060,52462,23176,23306,31660,31463,14757,31398,8517,10565,10532,10565,8517,6503,6536,4521,4554,4554,8517,10565,10565,10564,35},
{51005,50973,51005,51005,51005,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50874,50874,50874,50842,50842,50842,50842,50842,50842,50810,50809,52758,50612,46419,23406,17166,17166,17166,17199,15086,15118,13006,15118,13038,15086,15086,15054,15054,13005,15086,12973,15086,10925,13006,10893,10925,8812,10893,8780,6732,8812,6732,6764,4651,6732,4619,4684,4619,4651,4619,4619,4586,4586,4554,4553,4553,4521,4521,4520,6536,6536,6536,6536,6536,6536,6536,6536,6536,8649,10761,10729,8648,8649,10728,10729,10696,10729,12809,10696,6536,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,8550,6502,6536,8550,8518,8550,8746,13004,19310,34035,46549,48628,44404,63356,63356,63356,63356,63356,63355,48465,46319,27402,25322,27467,29515,29579,29546,29546,31659,37965,40045,37933,42125,40014,14759,16936,46319,46318,46318,42092,12678,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,10564,10533,6569,10729,8517,10564,14822,42028,52461,16837,19048,27499,33608,12645,18885,8517,8517,10564,10565,8550,6601,6601,6634,4619,6634,8550,8517,8583,8649,29},
{50973,51005,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,52955,50907,50907,50907,52923,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,50810,50776,48663,48631,42358,36117,36117,36117,36085,34004,34004,33972,33972,33939,33939,36019,40212,36018,33938,35985,31825,33938,29745,31793,31825,38065,40144,42192,38032,33905,33971,36117,38230,38230,38198,29811,29843,23505,36182,38294,38262,31793,31760,21324,12939,10859,25617,23635,25584,29745,589,8845,25453,21259,21292,8681,14987,35953,33774,23340,14889,8648,8616,8648,10728,12808,14889,14857,10728,10631,8518,8518,8518,6503,6503,6503,6503,6503,6503,6503,6502,6470,8550,8518,6503,8550,8517,8517,6535,8583,8583,27598,46484,46645,50807,63356,63388,63356,63355,63355,63355,48465,46319,27369,25289,27466,27434,23209,27402,29483,25290,29483,31595,25321,25126,12776,19049,10630,12645,12677,12677,10596,10564,10564,10564,10564,10564,10564,10565,10564,10564,10564,10564,10564,10564,10564,10564,10565,18983,27368,27435,29483,27336,29352,18950,39947,52494,33609,33641,35787,21062,10565,10565,8517,10565,10564,10564,8550,6700,8748,6700,6732,6634,6568,6569,6568,6569,26},
{50974,51005,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48729,46681,46649,44536,42456,42424,40343,42359,40310,40310,40310,42325,40277,46486,46485,36051,38131,42324,42324,42324,42324,40243,42323,48498,48497,40145,36018,40310,42455,44568,46648,44568,34004,34004,29778,42456,44601,44568,38132,33872,29678,29677,19243,25650,34102,42356,46419,12973,25484,25450,23371,19178,14921,19179,40080,31661,42259,33806,12908,8650,6536,10825,33970,46549,42323,14921,12808,10696,10696,10664,10664,10696,10728,10728,10696,10663,10663,10663,10663,8583,8551,8550,8517,8517,8517,8517,8517,8517,19244,44371,46645,50807,63356,63356,63356,63355,63355,63355,48465,44272,16838,12677,16903,18951,12709,16935,19015,12709,16903,21095,14790,18885,8583,8551,12775,16838,16871,16870,12645,8517,8517,8517,10565,10565,10565,10533,8484,8517,10564,10564,10564,10565,10564,10564,10630,33674,50380,50315,50380,50380,52428,25158,39947,52494,46319,46287,39916,12645,8517,8485,8517,10565,10564,10565,8517,8583,6601,12973,8583,4488,656,2605,8518,8517,35},
{50974,51005,50973,51005,50973,50973,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50874,50842,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48730,46681,46649,44536,42456,42423,40311,40343,38230,40278,38229,42325,38165,44438,48630,40244,38131,36083,36083,38163,38099,33970,36018,42291,44337,42257,42258,40342,42456,44569,46649,46617,31923,34037,27730,40375,44601,44536,31924,21359,19311,35983,21356,23470,33937,46484,46451,25485,31723,31756,29610,19145,23338,19178,17002,19148,25519,17100,8714,8649,6536,6601,40212,48630,58998,16937,8616,8615,8615,8583,8616,8616,10663,8583,10664,10663,10663,10631,8583,10664,8551,8550,8517,10630,10630,8517,8517,8518,12906,46516,50774,52822,63355,61209,46451,50710,63355,63355,48498,48465,23208,19048,25354,25354,25353,25354,27434,23241,25353,27466,27530,10565,6536,8615,14823,12678,12710,10597,10565,10565,10565,10565,10565,10533,8517,21194,23110,23111,25191,27304,25223,27336,33577,33577,19016,33674,50379,50379,50412,50412,52429,27206,39915,52494,25321,16903,10597,8517,8517,10565,10565,10564,10564,10565,10564,10565,6535,13038,8518,6504,657,2606,8550,8517,35},
{50974,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50874,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48730,46682,46649,44536,42456,42424,40343,42391,40310,40278,40277,42357,40245,44470,46518,40212,36084,36083,36051,38099,29810,40211,38130,44370,48465,48466,44338,42390,42456,44569,46617,46649,33972,34004,29778,38295,44601,44503,31825,13037,19311,29744,19276,17230,27696,40275,48596,29776,31788,31756,33835,27531,19113,25355,14890,10795,15118,10828,8779,6601,6568,29613,42259,54871,61144,17002,8583,8583,8583,8583,8551,6502,6470,6502,6502,8518,8518,8518,8518,6503,6503,8518,8517,8549,8517,8517,6634,10893,27598,50677,46483,44338,63356,61209,48531,50710,63355,63355,48530,48433,31594,29546,31626,29514,23241,25386,29546,25289,27434,31594,25289,16806,6535,12840,12710,12710,12710,12677,8484,10565,10565,10565,8517,21162,31824,46418,37801,37801,35721,35721,37769,37769,37801,33576,21096,31560,48267,48267,50380,50413,52461,27238,37866,52494,18917,10597,8517,8485,10565,10565,10565,10564,10564,10564,10565,8550,6568,6601,6536,4554,624,2605,8551,8518,35},
{50974,51005,51005,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,55069,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50875,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48730,46681,46649,44536,42456,42423,42391,40343,40310,40310,40277,42357,40277,44470,46485,42292,34003,38163,36051,33971,29810,31890,38131,44403,46386,46386,42291,42391,42488,44569,46649,46649,38164,34037,34004,38262,46649,42292,38000,27665,19344,31858,31826,10925,19245,36018,44403,13036,31755,35949,33835,29643,29579,29578,12841,6667,15086,10860,6765,6602,21195,50449,50710,61176,56885,38033,44404,40178,38033,46516,21195,6503,6503,6536,6503,6503,6503,8518,6503,6503,8550,8518,8517,8517,8517,6502,6797,11023,40210,48629,46450,44338,63356,61209,44338,46516,63355,63355,48530,48432,27465,25417,27434,25321,21097,25290,29482,21128,23241,27434,23209,16838,8615,19048,16903,16871,14790,14790,10565,10565,10565,10565,10533,12710,14889,42290,33641,31496,29448,31496,31496,31496,33674,29482,21161,29513,50380,50347,52460,50413,52461,27238,37834,52494,18917,10597,8517,8484,10565,10565,10565,10564,10565,10564,10564,6503,4520,6601,6569,4521,625,590,8551,8518,35},
{50973,50973,50973,50974,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,55069,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50875,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48730,46681,46649,44536,42456,42424,42391,40343,40310,40278,38229,42325,44470,50776,48630,38164,36083,40244,38131,31859,27730,27698,36018,42290,46385,44305,42258,42391,42456,44569,46648,46649,42390,38262,36084,25683,36084,40082,38099,33938,21391,29777,40146,17166,33905,38131,44403,13070,27629,35982,37932,29547,19145,29642,17034,8780,17134,8747,6700,21196,42063,44208,44436,61144,54642,46386,38065,42193,40113,42290,19115,6536,6503,6536,6503,6470,6503,6503,6470,6503,8518,8517,8517,8517,8517,6568,11055,19408,29809,42388,42289,42258,61307,61274,54936,57016,63355,61274,50578,50545,23175,14822,19047,21095,12709,16903,23208,14790,16902,21160,14854,16871,14823,12742,12742,12710,10597,10565,10565,10565,10565,10565,10564,10564,8615,12842,10663,10663,10663,10565,8517,8550,8550,8517,19048,27434,50379,50347,52460,52461,52461,29286,35722,52494,44107,39914,27303,21031,10565,10565,10533,10564,10564,10565,10565,8518,8550,8550,8550,6503,625,591,8550,8518,33},
{50974,50974,50974,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,55069,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50875,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48729,48729,46649,44536,42456,42424,40343,40311,40310,40278,38230,42325,48630,57016,46550,42324,38132,38131,38131,33971,31858,36050,40210,48497,50545,48497,44338,42391,44504,44568,46649,46649,42455,34069,36116,15151,29743,38097,37967,48565,27665,36051,35986,27632,33970,38064,38031,23437,21423,36051,25388,31563,8682,21258,33872,25552,15086,12875,8780,27501,37868,23273,33905,54838,52561,42225,35951,46257,44110,44207,19083,6536,6470,6536,4488,4488,6503,6470,6502,6503,8518,8517,8517,8517,8517,6633,4749,2603,12972,50676,48498,42225,63355,63355,63355,63355,63355,63355,50611,50545,31562,25322,27466,29514,25289,25386,29547,23273,25386,29579,25322,19048,16871,14758,10597,10565,10533,10564,10564,10564,10565,10565,10565,10565,8681,8649,8615,8583,8582,8517,8516,8550,8518,8517,16904,25321,50379,52427,50348,52395,52363,29318,35722,52526,52526,50446,41994,39882,16805,10565,10565,10564,10565,10565,10565,8518,10565,10565,10564,6470,625,2605,8517,8518,33},
{50974,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,50907,52955,52955,50907,50907,50907,50907,50875,52923,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48730,46681,46649,44536,42456,42424,40343,40343,40310,38230,38197,40245,48631,50711,44470,42324,38131,36051,36051,36051,29778,36018,42290,48529,54672,50545,46386,42390,44536,44568,46648,46649,38262,36150,34036,15118,25550,31790,31890,29778,25650,31824,21228,14988,25585,38030,35918,31757,25484,31791,33741,27500,4553,19147,46451,42226,17134,6634,19213,33904,31661,29547,33807,52661,52660,40177,35985,40113,37999,35854,27469,10762,6503,6503,4488,4488,6471,6503,6503,6503,8518,8517,8517,8517,8517,8681,13071,6700,8845,10794,23275,42193,63355,63355,63355,63355,63355,63355,50611,48465,27401,25289,27434,27466,25354,27435,31595,27402,27434,29547,25321,25386,31659,25353,10565,8517,8517,8517,8517,8517,8517,8517,10565,8517,6601,8648,10762,8550,8518,8517,10564,10565,8517,8517,10630,21160,50347,50347,52363,52266,52331,31399,33642,52559,42059,33707,31627,39947,14725,10565,10565,10565,8485,8484,10565,8584,6601,6601,8518,6471,592,4587,8518,6535,34},
{50973,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50874,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,48762,48762,48761,48730,46681,46649,44536,42456,42456,40343,42391,40310,38198,38197,42325,46486,50711,36084,36084,40212,36019,33971,36051,33970,40178,38130,46418,50545,46385,42258,42391,44536,44569,46649,46649,36149,31892,36084,31924,36114,29742,17231,25617,19279,21327,25487,12908,42161,40046,33836,29643,29643,29611,29643,12873,4553,8714,46451,44339,15053,8682,25454,23373,31727,23308,31727,54741,50351,46451,31759,29581,29548,29451,29516,25521,6503,6503,4521,4488,6503,6503,6503,8550,8518,8517,8517,8517,8517,14987,17265,15120,13007,6536,17001,40145,61307,61307,63355,63355,63355,63355,50643,48465,20998,12677,16870,19015,12709,14822,21127,14854,14854,21127,12709,14790,23208,12677,8517,8517,8517,10565,8517,10565,10565,10533,10565,10565,6535,8616,8550,8616,8615,10630,10565,10565,8583,8550,8518,19015,50315,50347,50315,52233,52331,31399,31562,52559,33576,16935,12742,33610,10565,10565,10565,10565,10565,10565,10565,8583,8779,10860,8551,6470,592,4521,8551,8551,38},
{50973,50973,50973,50973,51005,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48730,46682,46649,44536,42456,44504,40343,42391,40310,40278,40278,40245,42325,46486,42325,42292,38131,33971,36051,36051,36018,40211,42258,44370,42258,42289,40211,40311,42488,44568,46649,44536,34069,27764,31956,23636,29843,25584,21457,29778,42324,27599,23505,13005,35887,40144,38029,37963,35883,31755,29643,12906,6634,8714,46452,33938,17199,6667,10892,23342,29647,25322,23243,54741,48402,38032,40210,33873,35952,35984,35985,17134,6536,6536,4488,4521,6503,6503,6503,8518,8518,8517,8517,8550,31790,31824,10959,15185,10959,6536,14889,40178,63388,65436,65436,65436,65436,65435,52658,50578,31594,25353,27466,29514,21096,23241,29546,23273,23305,29546,21128,23209,29546,19047,8517,8517,10565,10565,8517,8517,10565,8517,10565,10565,8583,8616,8517,8616,8615,10631,10564,8517,8583,8583,8517,14855,48202,48267,50315,50315,50316,31366,27369,52527,33577,12645,12678,12677,8484,10565,10565,10565,16838,14725,10565,10565,14824,10631,8518,8517,4522,8551,8551,8518,36},
{50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,52988,53020,52988,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48729,46681,46649,44536,42456,44504,42391,42391,40310,40278,40310,40245,44405,52759,40244,44340,40212,33971,36051,38131,38098,38130,40178,46450,44305,48498,44339,40342,44536,44569,46649,40342,31956,31924,29875,27796,42423,38130,33905,33971,38196,44371,31759,29614,27600,36049,44336,35916,31690,29642,29643,17065,6633,10794,31727,10926,8910,6733,8846,10926,19246,35920,27468,54741,46190,44273,40145,44403,38000,38065,38065,19247,6667,6536,6503,6470,6535,6535,6535,6470,8518,8517,10760,44500,38063,25421,10926,15185,6667,4520,10728,50743,65468,65468,65435,65435,65435,63321,44239,48465,31562,25289,27433,29546,23273,25321,29515,25322,27402,31627,27401,27466,31658,18983,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8583,8616,8517,8616,10728,10728,10630,8550,8517,10598,8517,10598,23110,25190,25223,27271,27304,16805,21096,46253,48301,25191,21031,18918,10565,10565,10565,10533,10565,10565,10565,8517,8517,8517,8517,8517,8550,8551,8550,8518,36},
{50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50907,52923,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48761,48729,46681,46649,44536,42456,42424,42391,42391,40310,38230,42326,40245,48533,52726,48565,46453,42292,40277,38132,38099,42291,42290,44338,48530,46418,42258,40211,40310,44536,44569,46649,46616,38197,29811,34004,31924,40375,38097,38097,27599,38132,40276,36018,33937,31826,42356,48531,42192,40075,35883,25450,21226,8681,10794,31727,10795,6601,6569,6569,6569,8746,44469,40113,54741,50220,41932,31596,52724,41965,48368,48335,23211,6700,6536,6535,8518,6503,6470,6503,6503,8550,8550,23536,50806,35917,29613,10893,15152,4619,4552,10762,38196,48631,50743,44437,48695,42355,35919,42126,48465,29449,19016,21161,25353,23241,21129,27401,21096,21160,27434,21160,23273,29514,14790,8517,8517,8517,8517,8517,8517,10565,8517,10565,10565,8583,8616,8517,8583,12808,12808,10728,8517,10532,10631,8517,8517,10565,10565,10565,10565,10598,10565,19016,54639,54607,41929,37868,27304,10565,8517,10565,10565,10565,10565,10565,8517,8517,8583,10663,8518,8551,8518,8518,8550,27},
{50974,50973,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,52988,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50874,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,48762,48762,48761,48729,46681,46649,44536,42456,42456,42423,42391,40310,40278,40278,38197,46486,52694,44373,42260,38132,38164,38100,38099,44372,46483,50610,50610,48465,42258,42258,42390,44503,46616,46648,46648,38197,33971,33971,29778,31857,33904,44338,38064,42291,38095,31758,21325,19245,40243,42291,29645,40043,35882,29578,19146,10729,12874,38066,12876,6569,6569,6569,6569,6569,21294,33840,52693,46256,37967,31662,52691,39917,46255,48401,25355,4521,4488,4488,6536,6536,6536,6536,6503,6503,8616,13166,48725,35917,29614,8813,15152,6797,4619,6732,15184,19345,19312,17265,15152,13006,17001,40013,48497,27336,12677,16902,23208,16902,21159,25321,14822,14822,23241,16966,19047,23240,12645,8517,8517,10565,10564,8517,10565,10564,10565,10564,10564,8582,8616,8517,10728,12808,23306,29384,27206,31529,25062,10598,8517,10565,10565,10565,10597,10597,10565,16935,54639,54607,50247,48268,31432,8517,10565,10565,10565,10565,10565,10565,8517,8517,8813,12875,8616,8551,8551,8550,8517,36},
{50974,50973,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53020,53020,53020,53020,52988,52988,52988,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50907,52923,50875,50875,50875,50875,50874,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48729,46681,46649,44536,42456,42424,42391,42359,40310,40278,42326,40245,40277,50679,40212,40212,38132,38164,40179,42324,48596,46516,50644,54869,50611,52692,50645,50743,50744,52889,52921,52921,44469,42291,40212,29712,36016,40145,46419,48532,48596,46483,31726,15052,8844,40244,44405,29743,46315,40042,35883,27530,12841,17002,40113,10763,6601,6569,6569,6569,6569,14988,29647,52660,48141,46093,37870,50579,48434,44306,44338,23274,4521,4488,6536,8518,8518,8518,8518,8518,8517,10826,21520,48725,35917,27533,8813,15153,15120,4521,8943,17298,17265,15185,15152,13040,8682,12808,37933,46385,48432,48432,48464,48465,48465,50545,48497,48465,50546,50578,50546,50578,48497,19018,8649,8616,6569,6569,6568,6536,6536,6536,6535,6503,6568,8648,8517,8616,21260,35919,48236,48171,52527,46059,27536,8517,8517,10565,10565,10565,10565,8485,14823,54640,54542,46118,31528,25191,8517,8517,10565,10565,10565,10565,10565,10565,8517,8844,21392,12907,8551,6568,6568,8485,39},
{50974,50973,50974,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,52988,52988,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50907,52923,52923,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48730,46681,46649,44536,42456,42424,42391,40311,40310,38230,42358,42357,42325,50679,42325,36084,46550,46550,40243,33970,33970,38131,44371,50610,48498,44306,42258,44438,42456,44568,46649,46649,40310,36084,36117,29777,38129,48695,48532,48499,48597,48564,31694,12972,8812,40211,48630,38196,42125,33803,33771,29579,12841,12874,40114,14955,6569,6569,6601,12777,10761,8681,21261,52660,48238,42028,42159,48499,44176,38064,42290,27500,4521,4488,6536,6503,6503,6535,6503,6503,8518,12940,25485,50773,37998,29549,8780,15153,15152,6765,13137,17266,15218,13104,13072,13007,8551,10663,31660,50610,54770,56851,56819,54771,56883,56851,54802,56883,56851,56851,56916,50545,40013,12744,8616,8583,6536,6535,6503,6535,6503,8550,6503,8551,6535,8616,8517,8648,14987,29580,43977,46123,54771,31728,8550,8518,8517,8517,10565,10565,8517,8485,12710,52494,39883,16870,14790,12710,8517,8517,8517,10565,10565,10565,10565,10565,8517,8583,8583,8583,8583,6536,6537,8517,39},
{50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53020,53020,53020,53020,52988,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48730,46681,46649,44536,42456,42424,42391,40311,40310,42358,44406,42325,42325,50743,42325,36052,42357,44437,42324,38098,40211,40211,44371,46451,42258,40178,40211,40342,44536,44569,46649,46649,40342,38230,38262,29778,44404,44371,46419,48467,50645,54870,40210,8811,6731,40275,46386,35952,33772,29610,27498,21194,12841,10762,21262,12875,6569,6569,17067,39917,31759,25388,23372,52661,41997,46516,54836,48466,46027,46125,46158,27273,4489,6634,6634,6569,6536,6536,6536,6503,8550,21259,50643,52853,38030,29614,12974,17232,17232,10992,15250,15218,15218,15185,10992,8715,8518,8583,33838,52657,54770,54770,52657,54770,54803,52624,54770,56850,52657,54802,56851,54705,44206,14823,10598,8550,8550,8518,8518,8550,8518,8518,8518,8517,8550,8648,8518,8650,8583,19081,44043,43978,50448,37805,10598,8517,8517,8517,8517,8517,8518,15021,10630,46188,25191,8517,8517,8583,8517,8485,10565,10565,10565,10565,10565,10565,10565,8485,8485,8485,8517,6503,4489,8517,39},
{50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53020,53020,53020,53020,53020,53020,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50907,52923,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48730,46681,46649,44536,42456,42424,40343,40311,40278,42358,46486,40245,40245,54904,52758,38164,38164,42357,40244,33970,36018,40179,46451,48563,42258,42258,40211,42390,44536,44569,46649,46649,40310,36117,34037,27698,36083,27599,29614,29646,42259,38097,17133,6699,4651,27697,31727,27469,25418,23338,19145,17034,14954,8649,12908,6634,6569,6536,12874,27501,33841,23242,17034,40081,37642,40015,42258,40113,39787,39787,39786,25193,4488,6667,6569,6536,6536,6503,6503,8550,8550,21390,42355,50773,38031,29614,21393,21393,17232,6700,6733,4586,4554,4619,4554,8550,8518,8583,35918,33740,44271,35820,35853,42126,31660,35853,35853,33740,35885,33772,35853,33740,31627,35853,27435,16904,8518,8517,8518,8518,8517,8517,8517,8517,8550,8648,6470,6635,14856,37900,43913,48237,50479,46157,10565,8517,8517,8518,8485,8517,8517,8583,8583,35721,16870,8517,8517,6535,8517,8485,10565,10565,10565,10565,10565,10565,10565,10565,10565,8517,8517,6536,4521,8517,39},
{50974,50973,50973,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53020,53020,53020,53020,53020,52988,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,50907,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48761,48730,46681,46649,44536,42456,42424,40343,40311,40310,40310,44406,40245,40277,52856,52758,42292,40244,42357,44372,40178,38099,42324,44403,48530,46418,46418,44339,44470,44536,46617,46649,46649,42423,38197,38262,29809,21358,6601,6601,6601,6601,6569,4521,6568,4521,6536,6569,6568,6568,6536,6568,6536,6536,6536,6536,6536,6536,6536,6536,6536,6536,6536,6535,6536,6536,6536,6535,6536,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,8550,8550,2570,10988,48758,40078,31694,23407,8747,8780,4554,4652,4554,4521,4554,6536,8518,8518,8616,8550,8550,8582,8550,8550,8550,8518,8550,8550,8550,8550,8550,8517,8518,8518,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8518,8583,8649,8518,6503,10630,40078,48106,52463,52624,44239,10597,8550,10598,10598,10598,8550,10598,8518,8550,33609,16838,10598,8517,8583,8648,8517,10565,10565,10565,10565,10565,10565,10565,12645,14725,8517,8517,4554,4521,8517,36},
{50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53020,53020,53020,53020,52988,52988,52988,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50874,50842,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48730,46681,46649,44536,42456,42424,40343,40343,42358,40310,40277,42325,40277,50744,48598,38164,44437,44437,42292,40179,40146,40178,40210,46450,42290,44338,42259,44470,44536,44569,46649,46649,44536,36117,42423,27698,10957,4553,4553,4553,6569,6503,6536,6502,6536,6502,6536,6502,6535,6502,6503,6503,6503,6503,6503,6503,6503,6503,6502,6503,8518,6503,8518,6503,8518,6503,8517,6503,8517,6503,8518,6503,8518,6502,8518,8518,8518,8518,8518,8518,8518,2537,4779,38226,42288,29612,23340,4521,6536,6569,4619,4554,4554,4587,6569,8518,6502,8584,8517,8550,8518,8518,8518,8518,8517,8518,8517,8517,8518,8517,8517,8517,8518,25453,14791,8517,8517,8517,8517,8517,8517,8517,8518,10696,12743,14824,12710,10598,8550,42094,46058,54738,50512,40078,12711,10631,10630,10599,10631,10631,8518,8583,8517,27336,18919,10565,8517,8583,12711,8485,10597,10565,10565,10565,10565,10565,10565,10565,10565,8517,8517,4554,4521,8518,37},
{50974,50973,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,50972,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,52923,50875,50875,50875,50875,50875,50875,50874,50842,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48730,46681,46649,44536,42456,42424,40343,42391,42391,40310,42326,42325,42325,50775,46517,38164,42357,50710,42324,36051,38099,38131,38130,40210,44338,46418,44339,44470,44536,44569,46649,46649,46649,40310,38262,29811,8779,6601,6601,6601,6569,6503,6536,6503,6536,6503,6536,6502,6503,6470,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,8518,6503,8518,6503,8518,6503,8518,6503,8518,6503,8518,6503,8518,8550,8518,8550,8518,8518,8518,8518,8550,6666,2602,21357,44434,31660,27468,4521,8583,6536,4587,4554,4554,4587,6536,8550,6536,8518,8517,8518,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,12842,48662,35690,10630,8517,8485,8485,8517,8517,8517,6536,8681,8550,8616,8616,8550,6502,35885,50219,52560,48367,27402,12710,8550,8550,8583,10729,10631,8551,8517,8517,10597,12678,10565,8516,8550,8582,8517,10565,10565,10565,10565,10565,10565,8485,10565,10565,8517,8517,4554,6503,8518,36},
{50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53021,53020,53020,53020,52988,52988,52988,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,50907,50907,50907,50907,50875,52923,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48761,48730,46681,46649,44537,42456,42424,42391,42391,42391,38230,40278,38197,40245,50743,42324,42324,44437,46518,36052,31859,33938,36050,44403,44370,48466,46418,44371,44470,44536,46617,46649,46649,46617,34037,36182,31891,6667,6601,4521,4553,6569,6503,6536,6503,6536,6503,6536,6470,6536,6502,6535,6503,6503,6503,6503,6503,6503,6503,6503,6503,8518,6503,8518,6503,8518,6503,8518,6503,8518,6503,8518,6503,8518,8550,8518,8518,8518,8518,8518,12743,12710,17099,15019,25518,46514,31660,29581,8649,6503,6536,4586,6536,6535,6535,6503,6503,6568,8517,8518,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,31891,57049,35755,10565,8517,8485,8485,10532,8485,8550,6601,10630,8550,8616,8583,6503,12743,42126,43848,46091,48399,44141,33578,12810,8517,8517,6535,8616,8550,8517,8517,8517,10598,10565,10564,6503,8517,8517,12646,10565,10565,10565,16805,14725,8485,8485,10565,8517,8517,6535,8518,8518,37},
{50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,53020,53020,53020,52988,52988,52988,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48761,48729,46681,46649,44536,42424,42424,42391,42391,42391,38230,42326,38197,38197,48630,38164,38164,38164,40244,38100,36019,42227,38099,44404,46483,42258,44339,42291,44470,44568,46649,46649,46649,44568,27731,38230,33971,6634,6601,6569,4553,6569,6503,6536,6503,6535,6502,6536,6502,6536,6502,6503,6502,6503,6503,6503,6503,6503,6503,8551,6503,6470,6503,8518,6503,8518,6503,8518,6503,8518,6503,8517,6503,8518,6502,8518,8550,8518,8518,10695,50774,52886,52886,50806,36081,44434,31692,33805,23307,6503,8650,10763,8485,8517,8517,8518,6568,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,14988,48728,59031,31595,8485,8485,8485,8485,12710,10565,8551,6569,8583,10533,8485,10533,8518,8551,35853,50186,50349,54705,40045,23308,8748,10565,8517,6502,8682,8682,8517,8517,10566,10630,10663,10598,6503,8517,8517,12677,10565,8517,8485,10565,12645,10565,8485,8485,8517,8517,8517,8550,8518,38},
{50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53021,50972,53020,53020,52988,52988,52988,52988,53020,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,52955,50907,50907,50907,50875,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48761,48729,46681,46649,44536,42424,42424,42391,40311,40311,36150,40277,36117,36085,48631,44372,38164,42357,42357,38164,38099,38098,40211,42259,44371,46418,42258,42291,44470,44568,46649,46649,46649,44568,34004,38165,31858,6634,6601,6569,6601,6569,6503,6536,6503,6536,6502,6535,8518,6503,6502,6503,6503,6503,6503,6503,6503,6503,6503,8551,6503,6502,6503,8518,6503,8518,6503,8518,6503,8518,6503,8517,6503,8518,6502,8518,8550,8518,8518,8614,44597,46678,46677,46645,38162,46546,33740,38030,25420,6503,6568,8551,8485,8518,8517,8518,6568,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,21458,48696,48433,29515,8484,10533,8485,8485,12710,10663,8583,8617,8551,10533,10565,10565,8485,6503,35820,37706,14790,14790,12678,10597,8517,8517,10565,8518,8518,8517,8517,8517,8517,10566,8517,10564,8550,8518,8517,10565,10565,10565,10565,10565,10565,10565,10533,8485,8517,8517,8518,8518,8517,43},
{50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53020,53020,53020,53020,52988,52988,53020,50940,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,50907,52955,52955,50907,50907,50907,50875,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48729,46681,46649,44536,42456,42424,42391,40311,42391,40310,42358,40245,34005,44470,46485,40212,42390,34004,40212,38099,38131,42259,42291,44371,42258,42258,42291,44470,44568,44601,46649,46649,46649,36117,40310,6634,4586,6569,4521,4521,4521,6503,6535,6503,6535,6502,6503,8518,6503,6502,6503,6502,6503,6503,6503,6503,6503,6503,6503,6503,6503,6503,8518,6503,8518,6503,8518,6503,8518,6503,8518,6503,8517,6502,8518,8550,8518,8518,10760,44565,46677,46677,46677,46612,46546,33740,27499,27467,6536,6667,6535,8518,8518,8518,6503,8616,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8485,8517,8517,8517,17298,15053,44305,29515,8484,8484,8485,8517,12842,19312,17035,8517,10696,10565,10565,10533,8485,6503,19049,21032,10565,8485,8517,8484,8517,8550,8516,8518,8518,8518,8517,8517,8517,8517,8517,10564,8518,8550,8485,10565,10565,10565,10565,10565,10565,10565,10533,10565,8517,8517,8517,8517,8517,39},
{50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53020,53020,53020,53020,52988,52988,53020,53020,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,50907,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48730,46681,46649,44536,42456,42424,42391,42359,42358,40310,40278,38197,38165,42358,42324,38132,38132,31923,42324,40178,38131,40211,48630,44371,42258,42258,42292,44471,44568,44601,46649,46649,46649,34005,34004,8682,6634,6569,6569,6569,6568,6503,6535,6503,6535,6502,6503,8518,6503,8550,6503,6502,6503,6502,6503,6503,6503,6503,6503,6503,6502,6503,8550,6503,8518,6503,8518,6503,8518,6502,8517,6502,8517,6502,8518,8550,8518,8518,25485,44564,46677,46677,44532,46613,48627,33740,31693,29581,10893,13039,10828,8715,6667,6667,6700,10827,8616,8648,8616,8649,10762,8616,8616,8584,8649,8681,8583,8584,8583,8649,8649,10926,8779,35952,29515,8485,8517,10663,21393,23539,21491,17069,8517,10697,10565,10565,10565,10565,8550,8583,10565,8517,10663,10630,8517,8517,8518,10564,8517,8518,8518,8517,8517,8550,8550,8517,10564,8549,8550,8517,10565,10565,10565,10565,10565,10565,10565,10533,8485,8485,8485,8517,10565,10565,40},
{50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53020,50972,50972,53020,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,50907,52955,52955,50907,50907,50907,50875,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48730,48730,46681,46649,44536,42424,42424,42391,42359,42358,40310,40278,44438,40245,40278,40245,38164,40277,36083,36051,36019,40147,38066,46418,44370,46417,44338,42291,44470,44536,44568,46649,46649,46649,34004,40245,8714,8682,6568,6568,6568,6568,6503,6535,6503,6503,6503,6503,6470,6503,8518,6503,6502,6503,6535,6503,6503,6503,6503,6502,6503,8550,6503,8550,6503,8518,6503,8518,6503,8518,6503,8518,6503,8517,6502,8518,8550,8518,6502,27598,40372,21551,21519,19438,19308,44466,33740,23307,21359,19313,19312,10664,8649,8616,8616,8649,8583,8615,8583,8583,8616,8582,8583,8583,8583,8583,8550,8551,8550,8550,8551,8518,6536,8780,23372,29515,8485,8485,8517,10631,17167,23572,19149,10565,10565,10565,10533,10533,8517,8518,8551,10565,8518,10663,8517,8517,8517,8518,10565,8517,8517,8517,8517,8517,8551,8551,8551,8550,8551,8518,8517,10565,10565,10565,8517,10565,10565,10565,8517,8517,8517,8517,10532,10565,10565,40},
{50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,53020,50972,50972,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52988,52988,52955,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48730,48730,46681,46649,44536,42424,42424,42391,40311,44471,40310,38197,40245,38197,36085,38165,40244,48630,44437,36051,38131,40179,38066,48531,50610,48498,40178,44339,46518,44568,44568,46649,46649,46649,34004,38197,10762,6666,6568,6568,6568,6568,6503,6535,6503,6503,8615,6503,6503,6535,8616,6535,6503,6503,8551,6503,6503,6503,6503,6502,6503,6502,6503,8550,6503,8551,8551,10663,8551,8518,6503,8518,6503,8517,6502,8517,8550,8518,8550,12807,44532,46678,46678,46677,44531,44466,33772,29613,23440,21393,19214,10795,10762,8682,8714,8714,8616,8649,8616,8681,8682,8616,8616,8616,8681,8616,8583,8584,8583,8681,8584,8551,6536,8780,14954,25355,8516,10533,10565,8485,8485,8747,14988,8551,12809,14857,10664,8583,8551,8518,8550,8485,8615,10696,8517,8517,8517,8518,10532,8518,8518,8518,8518,6634,4652,6601,6535,6503,6536,6502,8517,8517,10565,10565,8485,8517,8485,8517,8517,8485,10533,8485,10532,8485,8485,40},
{50973,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,50972,50972,53020,52988,52988,52988,52988,50940,52988,52988,52988,52988,52988,52988,52988,52988,52955,52988,52955,52955,52955,50907,50907,50907,50907,50875,50875,50875,50875,50875,50874,50874,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48729,46681,46649,44536,42424,42424,42391,40311,44471,42390,40278,38197,50711,44437,38164,36116,40277,36116,36083,40211,38163,31858,40243,44371,40178,40211,40212,44470,44536,44568,46617,46649,46649,36085,38132,10794,8714,6568,6568,6569,6568,8583,6536,6503,6535,8583,6535,8583,6535,8648,6535,8615,6535,8583,6535,6503,6503,8550,8551,6503,8550,8551,8550,8551,8550,8551,8518,8550,8550,8550,8518,8551,8518,8550,8518,8550,8518,8550,14856,44532,46677,46677,46677,44531,46514,35852,33741,40112,40146,38000,35985,35952,31726,23374,19115,10697,14890,12809,10729,10697,10664,10664,8616,10697,8584,10632,8583,8583,8616,10631,8584,6568,10860,8681,25388,8484,8485,10565,8485,10533,8550,8551,6536,15020,14922,12842,6601,6569,8518,8551,10566,8518,10696,10565,8517,8518,8550,8517,8517,8518,8518,6568,4652,6634,8518,6503,6536,6503,8518,8517,6634,10795,10565,10565,6569,10794,10598,8517,8550,8681,10599,8517,8517,6503,31},
{50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,53021,53021,50972,50972,53020,52988,52988,52988,50940,50940,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50874,46550,46550,44469,38131,33938,36050,36082,36082,38130,38130,38098,35985,33937,33905,31857,29777,33937,38130,38130,36017,38131,36018,33905,36050,31857,31857,31856,31824,38098,33872,33904,35985,33872,36017,33904,29711,36049,31791,33872,33904,31726,33905,31759,31759,36017,29710,31824,33936,31791,29711,31759,31824,33969,29710,33937,29678,27533,27565,27533,27565,27598,27533,31824,29646,27565,33936,27565,31758,29678,27533,31823,29645,29646,31856,27565,31758,31725,29613,33871,29613,29613,29613,29613,29645,29678,27468,27500,31758,36016,31693,31758,31757,31726,35984,31758,29677,31757,29645,44338,48563,54868,56917,56916,56916,56916,56884,50578,56851,40079,44305,56852,54803,42125,52724,42126,21194,29645,25387,12711,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8812,6568,23276,10565,8517,10565,10565,10565,8485,8584,8714,15020,12809,10729,6568,6536,8518,8583,8517,8485,8615,10598,8517,10631,8517,8517,8518,8518,8551,6699,4620,6569,6535,6536,8584,8517,8517,8517,4684,10926,8517,8517,4587,10926,10630,8517,6503,6666,14856,14790,14790,14890,58},
{50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,53021,50972,50972,53020,53020,53020,52988,52988,52988,50940,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50907,50842,44437,46517,46484,35984,19212,31791,38129,38097,40177,40145,40145,33774,33806,35919,29710,19244,33871,42323,29678,38097,33936,33904,35984,31856,38065,29743,29775,38097,46614,40242,40242,33968,38097,46516,44403,38128,40307,33903,42323,31790,38096,44501,29710,38129,36017,35984,33968,31823,35984,19146,31758,33871,19276,35984,44468,29710,35983,21258,35919,35951,29710,35983,42421,38129,40177,48726,36016,40275,31823,35984,23372,27597,38097,42388,35983,21259,23372,35951,40274,38031,38032,38031,35983,38063,38063,31725,31725,38097,44435,38031,38096,38096,33903,23404,31758,35983,35983,40145,46451,50578,56916,56916,56916,58964,58964,58964,52691,56851,23208,31725,56851,33707,37967,52757,54771,25322,31758,29579,12743,8550,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8518,8812,8649,29582,10565,10533,8485,10565,10565,8517,8551,8551,14987,10761,8649,8649,8649,8550,8517,8550,8485,10566,10565,8550,8517,8517,8517,8518,8518,8714,8649,8550,8517,10564,8518,8517,8517,10565,8517,8681,12809,8551,6503,6503,10663,8550,8550,8649,33839,52592,52591,52559,52559,199},
{50974,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,53021,53021,50972,53020,53020,53020,52988,50940,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,52955,50907,50907,50907,50874,44437,48597,46517,35984,19212,31791,38097,38096,38096,38064,38064,31693,31726,33871,31791,21292,31791,42355,33904,38065,38162,36017,38064,31921,36016,31791,29743,38097,46614,40242,40242,33969,38097,44468,44435,38129,44500,36016,42323,33936,40112,46581,33936,40177,38195,36049,42387,44468,38064,29678,35984,38097,33969,36016,44468,36016,38031,29678,38032,38064,36016,38032,44469,40242,40144,46614,38096,40307,40274,38064,33936,33904,38162,44534,35984,31790,31790,38064,40242,38063,38064,38064,38096,38096,38096,31693,31758,38096,44435,38063,38128,38128,35983,29710,33902,40209,42322,35952,50611,56884,56916,58964,56916,58964,54771,54771,50578,56883,27401,35886,56851,44336,44238,52724,54738,50578,48497,48465,21130,8616,4553,4553,6535,8551,8551,8551,8551,8550,8518,8518,8518,6470,10860,38097,31596,10565,8485,10533,10565,10533,10565,8517,8517,12940,8616,8584,6568,6568,6568,8681,8518,8485,10695,10597,8517,10565,8517,8517,8518,6601,10730,10564,10565,10564,10565,8518,8518,8517,10565,10564,8517,8518,8518,10565,10564,8517,8517,8517,19278,48433,52624,52592,52624,52592,204},
{50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,53021,53021,50972,53020,52988,52988,52988,50940,50940,52988,52988,52988,52988,52988,52988,52988,52987,52955,52955,52955,52955,52955,50907,50907,50907,50874,44437,48597,46517,35984,19180,31791,38096,40177,38096,38096,35984,27500,29581,31790,31758,21292,31791,40275,31823,38096,38130,35984,38065,29744,36016,27663,27630,38097,44469,40210,42323,46581,40177,31791,27598,38129,38161,36016,42355,31823,40145,44468,33871,40209,40275,38096,31790,29645,40145,23339,33871,38129,36082,38064,44468,29710,38064,23339,33903,38064,29710,38064,46516,29678,35983,19114,35951,42323,25452,38097,27630,33903,38162,44468,36016,31822,33903,38064,35951,33871,38064,40144,40144,38096,38096,31725,33838,40177,44435,38064,35951,33871,38096,33870,35983,33838,25517,35984,44370,52691,52723,56884,56884,56884,58964,56884,52691,56851,46385,46385,56851,50610,50577,50643,50512,35852,29645,27467,10565,8518,4488,2473,8517,8517,8517,8517,8518,8517,8517,10565,8517,8518,8812,8583,19050,8518,8518,8517,8518,8517,8517,8517,8517,10697,8583,8780,11024,11025,13137,10958,8583,10565,10663,12678,10564,8517,8485,8517,8616,8715,8517,10564,10564,10564,10564,10565,10564,10565,10564,10564,8517,8518,8517,10564,10565,8517,8583,19246,48630,48400,52624,52592,52624,52592,205},
{50974,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,53020,52988,52988,52988,52988,50940,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,52955,50907,50907,50874,44437,48597,46516,35984,19180,31759,38096,38096,40177,40177,38064,27500,29548,33806,31758,21260,33871,42355,31856,40145,40243,36017,38097,29744,38064,31856,29744,40177,31856,33904,42323,40210,40177,33904,29710,40177,42355,35984,42355,27598,40177,42355,33904,40209,42388,38096,33839,29678,40177,27532,33871,38097,27663,38096,42421,33935,38096,25484,35984,38032,33904,40144,46548,36016,38096,31759,35984,42355,33871,40177,38194,38096,40210,44468,38129,40274,40242,38064,42321,40177,40144,40177,40177,40144,40144,33837,33870,40176,42355,40144,42290,42322,40176,40274,38096,35983,33839,37999,50611,56884,56916,56884,56884,58964,58964,56916,54803,56851,21031,25355,54803,29579,35951,50644,50544,37965,31757,25419,8517,8518,4521,2538,8517,8517,8517,8517,8518,8518,8517,8517,19082,31792,36051,35985,29549,8517,8517,8517,8517,8517,8517,8517,8517,8517,8583,6733,13138,15186,15153,10860,8648,10565,10598,12678,10565,10631,10565,10565,8550,4521,8517,10564,10564,10564,10564,10564,10564,10564,10565,8517,8518,8518,10564,10564,8517,8549,25519,46549,52789,42191,52624,52592,52592,52591,206},
{50974,50974,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53021,50972,53020,53020,52988,52988,52988,50940,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,50907,50907,50874,52922,44437,48597,46516,35984,21325,33871,38097,38064,38096,38064,38064,29613,29613,35919,33903,21357,33903,40242,36017,40177,38130,33904,40178,36116,38064,36082,33969,40177,46614,42290,42323,46581,40177,44468,36049,40210,42355,35984,42388,38162,40145,46581,31823,40177,27565,38064,44435,44468,38096,29710,33904,35984,23404,36016,42420,36017,38097,23339,35951,40145,23372,40145,48596,31726,40209,48662,38064,29677,31791,40145,46582,42323,38097,21258,38064,31790,31790,40144,23306,33871,40144,40144,40176,40177,40176,33837,33870,35951,29710,38064,42290,42322,40144,29645,29645,40176,48596,40112,33773,46385,50577,44304,56884,50545,50545,56916,48432,50545,23208,23242,52691,25354,27499,48563,48432,46384,50610,35820,12842,12809,8616,8649,8616,8583,8583,8583,8551,8583,8550,8583,42323,61209,61241,61241,44273,8484,10565,8517,10565,10565,10565,10564,10564,10564,10564,12876,12843,10631,8517,8583,8550,8517,10697,8517,8518,8649,8517,10565,8518,491,4488,10564,10564,10565,10564,10564,10565,10564,8517,8517,8517,10564,10565,10533,8517,21261,40244,50710,48531,40046,52592,52592,52592,52591,205},
{50974,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,53020,53020,52988,52988,50940,50940,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,52955,50907,50907,50874,54969,44405,48597,46516,35984,21292,33871,38096,38096,40177,40177,38032,31693,31693,35951,33871,21292,33903,40242,31856,40177,36082,33871,40145,33970,38097,36017,31856,40209,48694,42323,42322,48662,40210,44468,38161,40209,42388,36016,44436,48694,40177,46581,33936,42322,38162,38097,44435,46548,40209,31790,33903,38097,27630,38096,42420,36082,38096,27565,36016,40177,33903,40177,46516,33936,40242,48694,40177,33871,31791,40177,44501,42322,40210,27662,38096,40241,42290,38096,27499,35984,40177,42257,42257,40177,40176,33837,33870,38095,33936,40144,38064,29711,27630,25551,21325,19245,19213,19179,12710,21194,19048,14823,21194,19016,18984,23274,16935,21161,21161,12710,21162,14856,12742,19049,14822,16903,12775,12710,14856,12743,10565,8550,8518,8549,12775,10598,10630,10662,8517,8551,42323,59161,61241,61241,48466,10565,8485,8517,10565,10565,8517,8517,10564,10564,10564,6536,8649,6568,8649,8550,10565,10598,10631,8485,8550,8518,8517,8517,8517,491,2473,8517,10565,10565,10564,10564,10565,10565,8517,8518,10565,10564,10564,8517,8648,13039,25551,21294,42290,37933,52592,52592,52592,52592,205},
{50974,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,53020,53020,52988,52988,50940,52988,52988,52988,52988,52988,52988,52988,52988,52956,52955,52955,50907,52955,52955,50907,50907,50874,46550,44437,48597,46517,35984,23437,35952,40177,40144,40177,40177,40144,31693,31693,37999,33936,25518,33936,38162,36017,40177,38130,38097,40210,40276,38129,36017,27663,40209,48662,42290,44371,48662,40209,44468,36048,40242,48694,42323,42356,36049,40178,44469,42355,40210,29711,38097,44403,46548,40177,31822,38064,38096,27630,38129,36081,29710,40177,25452,33904,38097,36049,40177,46549,36016,42290,44468,38031,42355,27565,40177,31823,38097,40242,46647,38129,33871,31790,40176,25419,35951,40177,42257,42257,40176,40177,33838,33870,35950,23307,33871,10793,6568,8747,8583,17001,27565,25388,27532,27468,27435,27500,29548,29548,29580,29548,27500,31661,29580,27500,29580,29580,29580,31628,29547,29548,29580,29580,29581,29548,29580,29548,23274,14889,10728,12808,14889,12809,10696,8648,8649,42323,61241,61241,61241,46386,10565,10533,10533,10565,10564,10564,10565,10564,10564,10564,6536,6536,6536,8583,8517,8550,10631,8517,8517,8517,8517,8517,8517,8517,492,2539,6503,8517,10564,10564,10564,10565,8517,8518,8517,10564,8517,8517,8551,4717,4619,4587,6699,36017,31627,50544,52592,52591,52591,205},
{50974,50974,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,53021,50972,50972,53020,53020,52988,52988,50940,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,52955,50907,50907,50875,50842,44437,48597,46549,35984,21292,33871,40209,40177,40209,42257,40144,31726,31694,35951,33903,21357,33904,38162,29710,42290,29775,33904,40210,36083,38097,36082,29743,40210,46614,42322,44371,48662,40209,44468,31855,40209,48694,40274,42355,31823,40210,31791,27597,40177,21259,36016,44436,46516,40177,23307,38032,38096,23371,38096,36016,27597,40177,21259,33839,40209,40275,40177,44501,44435,40210,44468,35951,42323,27564,40176,40275,40242,42290,46614,40209,40274,40241,40177,25419,35951,40177,40177,42257,40209,40177,35886,33838,35951,25484,25420,6601,6569,6667,8714,31791,50643,54902,56950,56950,54869,54870,54837,56918,54869,56918,54870,52756,54804,54837,52757,54869,54869,54869,54869,54869,54837,54837,52756,54837,54837,52757,42193,19148,15021,15020,17132,15020,8779,8780,8747,42291,61241,61241,57016,33872,10565,10533,10533,8485,10564,10564,10564,10564,10564,10564,6535,6536,6536,8518,8518,8551,8550,8517,8517,8485,8517,8517,8517,8517,492,458,6503,8517,10564,10564,8517,8517,8517,10565,10564,8517,8517,8518,8878,4652,6667,2474,12874,27630,33740,50511,52591,52591,52591,205},
{50974,48925,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50973,50972,50972,50972,53020,53020,52988,50940,50940,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,50907,50907,52955,52955,50907,50907,50875,52954,44437,48597,46549,35984,21325,33903,40209,40177,40209,42225,40145,31725,31726,38032,33904,23470,35984,40210,40275,40177,42388,38097,40210,40342,38129,38163,36082,40177,46614,42290,44371,48694,40177,44436,31823,42290,42355,35984,38097,31824,40210,33904,33904,38097,25549,36016,35984,33935,40145,29678,31758,40177,23437,38097,44436,29645,40177,23340,35984,38096,27597,38097,44501,33936,40210,46549,35983,40210,29613,40177,29710,35984,40242,44501,40209,31758,29678,40177,31757,35983,40177,40144,40209,42257,40209,35918,33838,38096,40210,17002,8616,4553,6666,8616,12776,44305,46450,38031,48564,42191,33773,50644,37934,37934,54837,44338,44337,52692,38030,42225,46385,31725,48563,44305,40144,52724,38031,35951,46417,29580,44305,29483,8583,10860,10729,10794,14987,8583,6634,8649,42258,61241,61241,56984,27534,10565,8485,10533,10565,10565,10565,10564,10565,10565,10565,6503,6536,8649,6536,8550,8518,8518,8518,8682,8484,8517,8517,8485,8485,491,458,6502,10564,10564,10565,8517,8517,10565,10564,10565,8517,8517,6732,6667,4652,2506,8714,33938,40177,35853,48431,52591,52591,52591,204},
{48925,48925,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,53020,53021,50972,50972,53020,50972,53020,53020,53020,52988,52988,50940,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,50907,50907,52955,52955,50907,50907,50907,50842,44437,48597,46549,35985,21292,31791,40177,40177,40209,40177,40145,33774,31725,35952,33871,19212,35951,35984,31824,42257,40275,35984,40178,31922,38097,33937,27631,40209,46614,40210,42290,48662,40177,46516,29678,40209,48694,42322,33871,19211,40177,25485,25517,38129,36017,33935,31791,23404,40177,29677,29645,40209,19179,38064,44436,27565,40177,19146,31791,35984,21292,38097,44469,36017,40209,44501,35951,31758,23372,42257,25484,29678,40209,46581,40177,29677,27533,40177,25451,33871,40177,40177,42257,40209,40209,35918,33837,40144,36048,16969,6503,4553,6634,8551,14856,46417,40015,21161,44338,33676,21161,50644,27337,23210,54837,21064,33806,50611,16870,29645,33741,8517,31759,25420,17099,38033,19146,17034,29613,8550,25421,19016,8517,10860,8517,10695,17099,8517,6601,8649,40178,59161,61241,59063,42095,10565,8485,10565,10565,10565,10565,8517,10533,10565,10565,8551,6535,8649,6568,8518,8517,8518,8518,8550,8517,8518,8517,8517,8517,492,458,8518,10564,10533,10565,8517,10565,10565,10564,8517,8648,10597,6601,6569,4521,6536,17101,48532,35951,33740,46318,52592,52592,52592,205},
{48925,48925,50974,50973,50973,50973,50973,50973,50973,50973,50973,48795,46747,46715,44634,48762,52988,57149,57149,57148,57148,57148,57148,57180,57181,57180,57180,57180,57181,57181,57181,57181,57181,59229,59228,59229,57181,57181,59228,59228,59228,59229,59229,59228,59228,59228,59228,59229,59229,59228,59228,59228,59228,59228,46583,44503,44405,48597,46549,36016,25485,33903,40177,40177,42258,42258,40145,33806,33774,35984,35984,23470,36016,40177,40243,42290,36081,40274,40210,40308,38129,33969,29711,40209,42356,35984,42323,31823,40177,38161,33903,42290,48694,42323,40242,33904,40177,33936,33936,38097,31823,38097,36017,33871,40177,33903,31790,38096,29711,38097,44468,35984,40209,29678,35984,38129,27630,40177,33903,35984,40210,44533,38064,35984,29678,40177,29743,33871,40210,48727,38129,36016,38161,40177,38161,38096,42257,40177,42257,40209,42257,42224,21324,17165,15019,12874,8714,6601,6601,8550,10598,33806,37967,25355,38065,33741,25387,38032,29548,25387,38032,25387,29612,42225,25386,23307,17002,8583,10794,8714,8681,10827,10663,8616,10762,8583,8681,8616,8550,8746,8615,10761,17099,8550,6569,8616,38098,59161,61209,59096,25388,14758,8517,10663,10566,8485,8517,8517,8517,8517,8517,8550,6503,6535,6568,8518,8517,8518,8518,8517,10565,8518,8517,8517,8485,492,459,4652,10663,8517,10565,8517,10564,8517,8517,8583,8582,8517,8584,6503,6502,8517,19278,40014,25485,40080,40112,56917,56884,46286,190},
{48925,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50940,50940,55101,55133,53053,53053,53053,53053,53053,53053,53053,53053,53021,53021,53021,53020,53020,53020,53020,53020,53020,55068,53020,53020,52988,53020,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,52955,52955,52955,52955,52955,50907,50842,44437,48597,48565,35984,21292,33903,38129,40177,42257,42258,40177,33806,33774,35984,33871,17067,33871,35984,31824,42258,29743,31823,38097,21292,36017,27597,23372,40209,46582,44403,42323,25485,38097,36016,25452,40209,46548,42290,33871,21259,40177,25420,25484,40209,17033,33904,38097,23372,40210,33904,29645,38096,17066,38097,44468,29677,40209,23307,31758,40177,33936,38097,25484,25453,38129,40275,33871,38129,31823,42225,25485,29646,42322,48727,40176,31790,40241,40176,38129,31790,40177,42257,42257,40209,44370,40144,12940,8779,8746,25453,27566,25453,25453,25421,25421,27468,25453,27468,27534,27501,27501,29646,29613,29613,31759,31727,29678,29646,27533,17067,10795,10762,10795,8714,8714,8714,8681,8649,8714,8715,8714,8714,8682,6666,8714,12907,17132,10794,6602,8682,44438,63355,57016,38131,31693,14790,10565,10565,8517,10533,10565,8485,8485,6503,8550,8518,8518,8518,6568,8518,8517,8518,8518,8517,10565,8583,10598,6504,8517,491,459,6601,8517,8517,8485,8517,10533,8517,8648,8517,10565,8517,8550,8551,14955,27567,33906,35920,17066,27467,40179,59128,61143,29416,75},
{48925,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50972,50972,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,50940,52988,52955,52955,52955,50907,50907,50907,50907,50907,50875,50842,44437,48597,46517,35984,21325,33904,42290,42290,42290,42258,40177,33806,33774,38064,33903,19211,35984,42290,42356,42258,42323,38065,38097,25485,38097,33871,29678,40210,33936,35951,40210,27598,40209,44468,38129,42290,33903,38064,38064,31823,40177,46581,42356,40210,31888,38129,35984,31758,40177,36017,33871,42323,44501,38097,44468,36016,40177,42387,40241,42289,38129,38096,35984,31791,40241,29743,36016,42323,31791,42257,33936,35984,38064,31790,38064,36015,36015,40177,35950,38064,42257,42289,42289,42322,40144,35886,12940,10859,33904,56950,56983,59031,54869,54804,40144,46450,48563,52756,52724,50643,50676,52724,54869,54805,54804,54869,54902,56918,54805,29614,15086,15021,12973,12940,10860,12940,8714,8747,10893,12908,10861,10860,10795,8812,8779,10859,17197,12907,6699,6699,21293,29712,21327,10827,27598,18919,10565,8485,8517,10565,10565,8517,8517,8518,8518,4521,4521,8519,6568,8518,8517,8518,8518,8518,8517,10663,8485,8583,8485,2507,491,8615,8517,10565,10565,8517,8517,8616,8517,8517,10565,8549,8517,8616,40342,46518,42292,35954,10729,29580,38098,61176,61176,25190,55},
{48925,50974,50974,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,50940,52988,52955,52955,50907,50907,52955,50907,50875,50842,50842,50809,44437,48597,48597,38065,21292,33871,40209,40177,42257,42257,40177,33806,33806,38064,35984,19147,33904,36049,29744,42258,36049,31856,38097,17131,38097,29678,23372,42258,27598,31758,42323,48695,40209,44468,31855,42290,48694,40145,35984,21259,40177,42323,27598,40209,21324,35984,31758,21227,40178,36017,27565,42291,38162,38096,44436,31823,40177,36049,31758,40209,38096,38128,27565,23404,40209,21227,33871,42323,19178,40177,25452,29678,38097,19211,35984,40208,40241,40177,42322,33903,42257,40209,42289,42290,19114,27566,14987,42258,56982,54837,56982,54837,54837,52691,19048,31725,35885,21161,33805,33740,23241,42224,37901,29515,50611,40111,33806,54869,31595,12841,12940,8550,8747,12777,8518,10860,8583,8583,10860,8517,6601,10729,8517,8747,8583,8551,15052,8517,6536,6633,8517,6634,6764,6569,14954,18983,10532,8485,8485,8485,8517,8517,8517,8518,6503,4488,6503,8518,6568,8518,8517,8518,8518,8518,8517,8518,8648,8518,8518,459,491,8616,8516,10564,8517,8516,8648,8550,8517,10565,8517,10662,12711,8583,17100,27535,17134,23407,10729,29613,36050,61176,61176,25191,63},
{48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52988,52955,52955,50907,50907,52955,50875,50875,50875,50874,50842,44437,48630,48597,38065,23405,33871,40177,40177,42257,42257,40177,33806,33806,37999,35984,25453,33903,42355,46614,40177,48662,46516,42258,42356,38129,36017,29743,40209,42388,38064,42290,36017,38129,44469,46549,40210,38129,40177,40178,33936,42258,36016,38162,40242,44533,38129,36016,33904,40209,42355,31823,40210,33904,40177,40242,35983,40209,33871,35983,40177,38129,40177,33936,33904,40209,36016,38096,40242,31790,40177,33871,33903,40177,29678,38096,33903,29710,40176,38063,38064,42257,42289,42289,44403,29646,25485,48564,59031,59030,56950,54869,54870,52789,52724,16968,33805,37965,25354,31692,31627,19016,38096,46320,37998,50611,35820,19082,52757,19016,8648,12940,8517,8714,10761,8551,8812,8615,6535,10828,8517,6601,10762,8517,6667,8583,8518,10827,8485,6535,8649,8517,6601,6765,6568,10761,18984,8485,10533,10533,8485,8517,8517,8517,8518,4586,6503,8518,6535,8616,8518,8517,8550,8518,8518,8518,6470,8681,8518,8518,2539,6569,8517,8517,10565,8517,8583,8550,8517,10565,10565,8517,12775,16969,14857,8550,6503,6700,19344,10728,25453,36083,59128,61176,23111,40},
{48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50940,52988,52988,52988,52988,50940,52988,52988,52988,52988,52988,52988,52988,52956,52956,52955,50907,50907,52955,50907,50907,50907,50907,50842,44437,48597,46517,38065,23373,33903,40177,40177,40177,40177,40177,33806,33806,35983,33903,19211,36016,44468,46614,42258,46582,44404,40145,23405,38065,31758,25453,42290,33871,31726,42290,46614,40242,44501,44501,40242,23404,33936,33871,19114,40177,33871,25485,42258,50775,42322,31758,19146,42290,40243,27565,40242,46581,38097,44468,31823,40177,21226,31758,40209,33936,38129,27565,25485,40210,46646,35951,40242,19146,40209,44501,44436,40209,19211,35984,42354,44467,40177,33838,31726,42289,42290,42322,50742,46483,52790,59031,56982,56982,59030,56917,29579,33871,54837,46418,48563,50644,48531,50611,48531,46386,50611,44272,35886,50611,46417,35984,48564,29613,8746,12941,8648,8747,10794,8615,6732,8648,8616,8780,8649,6666,10795,8616,6699,8682,6601,8747,6601,6601,6634,8583,6634,8846,8649,8682,16936,8485,10565,10533,10565,10533,8517,8484,8517,8550,6601,8518,6535,8617,8550,8517,8518,8518,8550,8518,8518,8517,8518,8518,6568,8550,8517,8517,8517,10598,14923,10598,10565,10565,8517,8517,10597,8517,10664,8551,6503,4652,13072,8616,21260,33970,59128,61176,29383,44},
{48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50940,52988,52988,52988,52988,50940,52988,52988,52988,52988,52988,52988,52988,52956,52956,52955,50907,50907,50907,50907,50907,50907,50907,50842,44437,48597,46517,36017,21292,33871,40177,40144,40177,40177,40144,31726,31726,35951,33903,21227,33903,42356,48694,42258,44469,38097,38064,27565,38064,33839,29645,42257,46581,40210,42290,38129,40210,44469,44501,40209,46614,38096,38097,29711,40177,33904,33936,40209,48727,42290,33871,27597,42257,40243,29710,40210,40307,38097,42388,38129,40177,29613,35951,42290,44468,38096,36016,33936,40177,48694,40209,40242,44502,40177,36016,38097,40209,36017,38096,40242,42355,40209,38096,35983,42257,42257,42289,50709,54838,56982,56983,56982,56982,59030,54837,48595,50676,56918,56950,56950,56950,56950,56950,54869,54869,52756,54869,54837,52724,54869,54837,54805,44338,15085,13006,12941,12974,10893,8813,8813,10860,8780,8812,8780,8813,8780,8747,6732,6667,6667,6700,6667,6667,6667,6635,4587,6765,6569,12907,16936,8485,8484,10533,10565,10565,10565,10565,10565,8517,8518,8518,8517,8616,8551,8517,8518,8518,8550,8518,8518,8518,8518,8518,8518,8518,8517,8517,8517,17035,16969,10565,10564,12776,14824,10565,8517,8518,10664,8551,8518,4620,11024,8550,17035,31825,59129,61176,44042,133},
{48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50972,52988,52988,52988,52988,50940,52988,52988,52988,52988,52988,52988,52988,52988,52956,52955,52955,50907,50907,50907,50907,50907,50907,50907,50842,44470,48630,48597,36016,21260,33871,40177,42257,42257,42257,42225,33806,33806,35983,33903,19179,33935,44435,36049,42258,40243,27630,38097,17065,35984,29645,23340,42257,48629,42355,42290,27597,38097,42323,27597,40209,48694,33903,33871,17066,40177,25517,25485,40242,50775,42290,31758,21226,40177,36049,33936,42290,36081,36016,40210,25517,40177,21194,29678,42323,50774,40209,29645,25485,40177,48727,42355,42323,44501,40210,27597,25549,40209,40308,40209,44403,46515,42257,33870,29678,42289,42289,42289,42258,29678,54837,54870,59031,59063,59063,54837,31692,35886,56950,48530,42192,46353,33773,40112,46450,48563,50611,42159,29547,46450,44304,29645,52757,35886,8747,12973,8518,6634,8747,2505,6765,10631,6470,10860,8518,6634,10762,8517,6634,6601,4488,6667,6503,6536,6666,4488,4554,6733,6536,6568,12776,8484,10565,8517,8485,10564,10565,8485,10565,10565,8518,8517,8517,10696,10599,8517,8518,8518,8518,10565,10564,8517,8518,8518,8518,6503,8517,8517,8517,8484,8485,10532,10532,10565,10564,8484,8615,12908,10795,10729,8518,4652,2637,8550,8649,29712,61177,61176,50380,166},
{48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50972,52988,52988,52988,52988,50940,50940,52988,52988,52988,52988,52988,52988,52988,52956,52956,50907,50907,52955,52955,52955,50907,50907,50875,48729,44469,48630,48597,36017,21324,33903,40145,40145,40177,42225,42225,33774,33774,35951,35983,23372,35952,44436,46581,40209,42355,36081,38129,33936,38096,29678,23372,40177,40242,38064,42258,46581,40177,42355,42388,40177,38161,38097,40177,34001,40177,40243,36049,40177,36049,40177,38031,31790,40177,38161,40307,40210,40275,38097,42387,36016,40177,33903,38064,40209,46516,40145,40243,33936,40177,48694,42322,42322,46614,42322,36048,36016,40209,38194,40209,40242,42355,42257,40176,33870,42257,42257,42289,40209,48597,56884,54836,54870,59031,54837,56950,54837,48531,54837,25321,31692,40078,23274,31693,35917,27500,38096,46385,37966,46450,37933,16936,50644,31693,6698,12973,8517,6601,8747,2505,6733,10663,8518,8812,8517,6568,8714,8517,6634,6601,2473,6667,6503,6536,6634,6535,4522,4685,6536,6503,10663,8484,8484,8517,8485,10564,10565,10565,10565,10565,10565,10565,8516,14889,14824,8517,8518,8551,8518,10565,10564,8517,8518,8518,8518,8518,8517,8583,10598,10532,8485,8517,8517,10565,10565,8517,27697,44470,44405,38098,16936,524,4652,8518,8550,25584,59161,61176,44140,115},
{48925,48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50972,52988,52988,50940,50940,52988,52988,52988,52988,52988,52988,50940,52956,52956,52956,52955,50907,50907,50907,50907,50907,50842,46583,44470,42357,46549,46517,36016,21260,31823,40177,40177,42257,42257,42258,33806,33806,38064,35951,17034,33871,44403,48694,40177,31823,29710,38064,19146,35984,29645,21260,40177,42388,40210,42290,48662,40177,44436,44468,40209,23339,31791,38097,17066,38097,36017,23404,38097,19147,35984,31790,19114,40177,33903,27629,40209,21227,33903,42355,23307,40177,27565,31823,40177,36081,38096,35984,21227,40177,48694,42290,38097,38195,40210,27630,29711,38129,29776,36049,35984,23371,40209,42322,33838,42257,42257,44370,42322,52823,56883,56883,56884,56917,56950,56983,56982,56950,56885,35917,38030,40110,38062,37998,40111,33838,44338,46417,35951,46450,44304,29613,50676,31661,6699,10893,8616,6666,8747,4553,6732,8681,8616,8812,6666,4651,6699,4553,4619,6634,4554,6667,6568,4521,4586,6569,4554,6700,8584,8550,10631,8484,8484,10565,10565,10565,10565,10565,10565,10565,10565,10565,10564,12776,14791,8517,8519,8551,8518,10565,10564,8517,6470,8518,8518,8518,8518,10663,10565,8517,8517,8518,10533,10565,10564,10598,29777,44438,40146,25421,8550,6732,10827,38132,29580,19147,59129,59128,37802,68},
{48925,48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50972,50972,52988,52988,52988,50940,52988,52988,52988,52988,52988,52988,50940,52956,52956,52956,50907,50907,50907,50907,50907,50907,48762,50809,46583,42357,48597,46517,35984,21292,31791,40177,40145,40177,40177,40209,35952,35919,38064,33903,19147,33871,44403,44403,40177,42323,38097,38064,25517,35984,31791,27598,40177,46581,44403,40210,40275,38096,44436,44468,40209,40275,35984,40210,31888,38097,40242,31856,40177,38130,36016,38032,29646,40145,35983,31855,38128,29709,36016,38129,31790,38097,29678,33903,40210,44468,40209,42387,31855,40176,48694,40209,40210,42388,40209,40275,40275,40209,44501,40210,38031,29709,40176,40144,38063,40209,42289,42323,44404,52823,54803,56851,56883,56883,54836,56982,56983,56950,56918,54869,54837,54837,54837,54869,54837,52789,52724,56918,56950,54837,56950,56950,56918,33840,13038,13006,10926,10925,10925,10925,8845,10926,8845,8813,8845,8813,8813,6732,6732,6700,6700,6667,4652,4652,6700,4652,4652,33742,33480,23046,10565,10565,10565,10565,10565,10565,8485,8485,10565,8485,8484,8516,10532,10728,14856,8517,8518,6503,8518,10565,10564,8517,8518,8518,8518,8518,10631,8485,8516,8517,8517,8517,8517,10533,8517,19279,48631,48598,44372,25486,12743,12842,12776,29582,14791,16969,59129,59096,46220,111},
{48925,48925,48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50972,50940,52988,52988,50940,50940,52988,52988,52988,52988,52988,52988,50940,50940,52956,52955,50907,50907,50907,50907,50907,50907,50809,48696,42325,44437,48630,46549,38065,21292,33872,40177,42257,42290,42290,42290,38065,35984,36016,33871,17034,33839,44403,29710,40177,46549,33903,36016,17066,33903,29645,23339,40177,29743,31823,40209,27662,36016,44436,46549,40209,46549,38129,40177,19178,36016,36081,21292,38129,36017,33871,33838,21194,38096,35951,25484,38096,23340,31790,35951,21258,38097,23371,29678,38097,31855,36016,42355,25485,38096,29710,31790,38097,38162,38129,40275,40307,40209,36082,38129,33838,21291,40177,35951,25484,40177,44435,48564,48630,52824,54803,54770,56851,56851,56851,59029,61176,59062,61143,56949,48497,54836,54836,52756,56949,54804,54869,54869,52788,56917,46385,38031,44273,19115,8747,10828,10729,8715,8779,6634,8747,8682,6633,6732,4619,4619,6634,6602,4619,6602,4553,6634,6568,6569,4554,4521,27697,46190,41768,29287,10565,10565,10565,10565,10565,10565,10565,10565,8485,8485,8485,10533,8485,10664,10631,8517,8518,6503,8518,10565,10565,8517,8518,8518,8518,8550,8517,8517,8484,8517,8485,8517,25321,25222,25190,10630,16904,16903,14823,12743,10565,10565,10565,10596,8517,14856,57048,59096,50511,164},
{48925,48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50972,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,50940,50940,52956,52956,52955,50907,50907,50907,50907,50907,52922,48728,46518,44437,48629,46549,38064,23372,33904,40177,42225,42257,42290,42258,38065,35984,36016,35984,23372,33872,44403,38194,40177,46548,38162,38097,36050,38096,29678,23372,40145,36082,36016,40210,38194,38097,42388,46548,40177,46613,42322,40210,38162,38097,40307,34001,38129,46549,38129,35984,33904,38097,40210,36016,40177,35984,36016,40176,40209,38064,31823,33903,38097,36049,36016,42355,38129,38096,36016,36016,38128,38162,40177,40274,40274,40209,40275,38096,38063,31789,40145,42258,38097,40177,44435,50709,50710,52856,54803,54770,54770,56850,56883,59029,56917,52723,56949,56949,46352,52723,31724,40177,50643,37998,48531,46352,33805,48531,27403,10664,12874,8746,6666,10827,8583,8681,8747,4553,6699,8616,6535,6732,6536,6569,6666,6569,4619,4586,2506,4586,8550,6535,6634,12940,37967,39884,27174,10597,10565,10533,8517,8517,8485,10565,8517,8517,10565,10565,8485,8517,8517,8616,12776,8517,8518,6503,8550,8517,10564,8517,8518,8518,8518,8518,8517,8517,10565,8517,10565,12775,33706,29414,25222,19015,18950,18918,16870,14790,12710,14790,19016,19049,14823,27533,57016,61177,48431,118},
{48925,48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50972,50972,50940,52988,50940,50940,50940,50940,52988,52988,52988,52988,52988,50940,52988,50940,52956,52955,50907,50907,50907,50907,50907,52955,50809,48663,44437,46549,46517,35984,23373,33871,40177,40177,40177,42257,40209,38065,35985,36016,33871,16969,31759,42323,44436,38097,38162,36017,38064,21226,35951,29645,23339,38097,40275,27630,40210,40242,38097,42356,46548,40145,46614,42290,42290,42388,38064,42355,31855,38097,46614,33871,33838,17033,38064,36016,23372,38064,14952,31758,40208,40208,38064,25452,29678,38064,31855,33936,40275,29645,40145,21259,29646,35983,29743,38096,40307,42388,40177,31823,33871,35951,27564,38129,27532,25484,38096,44436,50709,50742,59194,54803,54770,54770,54738,58964,65303,61076,52723,56949,54869,44304,50610,14855,33839,46417,27499,42225,40013,16903,42225,21097,8518,8682,6633,4585,10828,8517,6600,8747,4521,6667,8616,6502,6732,8517,6535,6666,6536,4587,4586,2506,4586,8518,10631,10795,42291,21031,39916,35527,20966,10533,8485,8485,8485,8485,8485,8517,8517,8517,8517,8485,8517,8517,8616,10663,8517,8518,8518,8518,8517,10564,6601,8551,8518,8518,8518,8518,8518,8517,8517,8517,14856,37867,31495,29514,35820,37899,33673,33674,31626,31660,27467,29612,25355,10598,17034,57080,61176,50545,123},
{48925,48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50972,50972,50940,52988,52988,50940,50940,50940,52988,52988,52988,52988,52988,50940,50940,50940,50908,50907,50907,50907,50907,50907,50907,50875,50809,46615,44437,48630,46549,36016,21292,33871,42225,42225,42258,42258,40209,38065,36017,36016,33871,19114,33839,42323,46549,38097,36049,33903,38096,27598,38064,33871,29646,40145,42323,31823,38161,46614,40177,42388,46548,40145,44403,40177,40210,44469,38096,44436,38161,38097,46614,38097,35984,29678,38064,38129,33903,36016,29678,33903,40176,38096,38064,27630,31791,38096,40242,36016,42355,40242,40177,36016,35983,38064,36049,38096,42388,44500,40177,44435,38128,35983,33902,38064,38129,36016,40144,42322,50709,52855,54935,52691,54738,54770,54738,58964,63190,61044,52690,50546,35885,42159,52724,31692,40144,50611,37999,46450,48530,38096,44370,14954,6633,8714,6633,4553,8812,8583,6633,8747,4521,6667,8649,6536,6732,2538,2538,6667,6601,6699,4586,4554,4586,6535,23242,33741,40013,14790,37868,31334,14725,8517,8485,8517,8517,8485,8517,10565,8517,10565,10565,8517,10564,10564,8583,10664,8517,8518,8519,8518,8517,10564,6601,8550,8518,8518,8518,8518,8518,8517,8517,8583,14888,35820,31594,40112,35820,37965,35885,33772,33772,33741,31692,33806,27500,10598,10695,57048,61176,56917,203},
{48925,48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50940,50972,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,52988,50940,50940,50940,50908,50907,50907,50907,50907,50907,50907,50907,50875,50809,42389,46549,46517,36017,21292,35952,42258,42257,42258,42258,40177,38065,36017,38064,33871,19179,33807,42323,29710,38097,36017,31823,38097,31856,35984,33871,33903,40145,35984,29645,38162,44533,36016,38162,36017,38064,27532,31790,38065,29678,35984,44436,29678,38096,46582,36016,38097,17033,38097,40242,27597,38064,27564,33838,38063,29644,38064,25484,27597,36016,33936,36016,42355,29677,38064,25484,27565,35983,23405,36016,40242,40275,38096,36017,29677,36016,38129,38096,40210,31791,40144,42290,42322,40209,44304,50578,54770,54770,54770,58964,63190,58964,52691,50611,38063,46385,54870,52789,54870,54870,54870,54837,50612,40178,35985,15054,12973,8780,8844,8844,10893,10925,10893,8845,10893,8813,10894,10927,8846,6765,6732,6733,6797,8813,6700,4652,4619,6700,12973,25487,33675,12710,37868,33382,12645,10565,8485,8517,8517,8485,8517,10565,8485,10564,10564,10533,8484,10533,8583,8648,8485,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8583,6503,2572,2539,23471,42291,37901,37866,35786,35787,33739,33739,35753,35719,14724,8517,14856,54967,59096,52626,137},
{48925,48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50972,50972,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,52988,50940,50940,50940,50908,50907,50907,50907,50907,50907,50907,50907,50875,50842,44437,46549,46517,36017,23340,33871,40177,40177,42258,40209,40145,35984,35952,35984,33871,21227,33871,40242,33969,38097,44469,38161,38064,33969,40176,31823,31791,38064,38130,31823,38129,44469,35984,36016,29743,38064,33936,36016,40209,44501,36016,42388,36081,36016,44501,33904,38097,31824,38064,40242,33936,38096,31823,35951,40176,42289,38031,31790,33903,38064,33968,36016,40275,38130,38064,36049,35984,38032,33937,36016,40243,40307,38096,42387,38097,38128,40274,38064,40210,36048,38096,38064,38096,38096,40079,50642,54738,52690,54738,58964,63190,58964,50578,52724,48531,54836,59030,56917,48432,46416,56916,40046,27402,14889,10695,10696,10729,8583,8616,8648,8583,8648,8615,8583,8616,8583,8583,8616,8550,8583,8551,8550,8583,8550,8550,8550,8518,8550,8550,17002,29515,12710,37836,39688,25127,10565,8485,8517,10565,8485,6503,8517,10532,8517,10728,12808,14888,14888,12808,10696,8746,15084,14921,10794,17232,15052,14856,10794,12973,12874,10663,8681,4619,8648,8583,524,524,492,21326,42291,37933,37800,39913,42026,44073,44040,41992,39879,12612,8517,12775,54903,59096,50480,89},
{48893,48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50940,52956,50907,50907,50907,50907,50907,50907,50907,50875,50842,42357,46517,46516,35984,23340,33839,40145,40145,40177,40177,38097,35984,35952,35984,33871,21227,31790,42290,38130,38097,44468,36049,36017,33904,38032,33871,33871,38096,44469,40242,38129,48694,38129,38129,31790,38065,46582,33903,38097,33904,38064,44436,33904,38096,42323,31791,33871,21259,35984,38194,40242,38064,27531,33838,40176,27531,38031,27532,27565,35983,38130,33903,40275,31757,38031,42322,35983,35983,31856,38064,42356,42420,38096,33902,33870,36015,31855,35984,40210,40209,38064,36016,38096,38096,44337,56916,58931,56883,56883,63255,65335,46352,50579,52691,46449,52723,52690,46449,35818,40142,54835,18983,12678,8583,8517,8550,8583,8517,8550,8551,8517,6502,8550,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,14888,31596,12710,35756,31334,10565,8485,8485,8517,8517,8517,4521,6502,8517,8518,8746,10696,12807,14920,10662,8550,6633,17165,12743,8713,13104,15052,10663,8681,15151,14987,10631,6601,2572,6569,4521,557,525,492,23439,42291,40014,42125,54771,54771,50380,44072,44072,39848,10564,8517,10662,52823,59096,52594,106},
{48893,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50972,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50940,52955,50907,50907,50907,50907,50907,50907,50907,50875,50842,42325,46484,44436,35984,23340,33839,38065,38064,38065,38064,35984,33839,33807,33839,31759,19115,31726,40210,33904,33904,40275,31824,33904,31791,35919,29678,35984,33871,40178,33904,33904,42355,33904,35985,38098,33839,40243,33935,33903,38097,31758,36016,25517,31758,35984,29645,31726,17066,29678,31791,31791,29677,19146,23372,31725,23339,29645,19146,21227,29613,25517,25484,31791,23340,27565,29677,25484,27532,23372,27564,29710,31791,27565,33871,31693,27565,25452,27564,29678,29677,29645,27532,29612,29645,46450,56883,56882,56817,54770,52724,44306,35854,48531,50578,38031,46450,48497,38031,31594,40078,48562,18983,10663,8615,8517,8583,8615,8517,8615,8551,8518,6568,8518,8550,8551,8517,8550,8518,8517,8550,8517,8518,8550,8517,8550,8518,8517,8518,8517,12808,31596,12678,33676,43880,29255,8518,8517,8517,6503,10859,15019,14986,12874,8550,6633,8616,10761,15018,12906,8616,4552,10859,8517,8615,13071,12907,8517,8616,19344,14955,8517,6536,2702,6601,6504,557,525,460,19246,42291,40014,39980,50545,52592,48300,41927,35721,35623,10532,8517,17000,52790,59128,56819,136},
{48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50940,50972,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,50940,50940,50940,52956,50907,50907,50907,50907,50907,50907,50907,50907,50907,50874,50742,50710,50677,44403,44371,42258,42290,42257,44338,42290,42258,42225,42225,42225,40145,40145,40145,42257,42258,42257,42257,42225,40177,40145,40145,40145,40145,40177,40177,40145,42257,42258,42257,42225,42225,42225,42225,42225,42257,42258,42257,42257,42257,42257,42258,42257,44338,42257,42257,42257,42257,42257,42257,42225,42257,42257,42257,42257,42225,44338,44338,44370,44370,42257,44337,42257,44305,44337,44337,44337,44370,44337,44338,44338,44337,44337,44337,44337,42257,44305,44337,46385,46418,46450,46417,48465,48497,50609,50610,40079,23308,50644,54869,59030,59030,59063,59095,59063,59062,59030,56950,46419,12940,12940,10860,10795,8747,8682,8714,8714,6666,6634,6634,6601,6601,6601,6568,6569,6536,6568,6568,6536,6536,6536,6536,6536,6536,6536,6535,10762,31596,10597,35789,45993,31400,10664,8485,8517,6503,15052,23372,21291,17067,8550,8812,12907,8680,8517,8518,6535,4552,10925,12744,10761,15184,15052,12744,10762,15216,12907,10631,6536,524,6601,4520,525,524,460,17165,40211,40047,37768,39880,39912,44040,41993,37966,31529,10532,8517,10662,48630,59128,58964,168},
{48893,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50940,50972,50972,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,50940,50940,50940,50908,50907,50907,50907,50907,50907,50907,50907,50907,50907,52923,61209,59129,59128,54870,50677,50677,52789,52757,52757,52757,50676,50644,50644,50644,50644,48564,48563,50676,50677,50644,50676,50644,48563,48564,48564,48531,48531,48563,50644,48563,48563,48564,48531,48563,48563,48531,48596,48531,48563,48563,48531,48563,48563,48563,48563,48531,48531,48531,48531,48531,46483,46450,46451,46450,48498,46450,46450,46450,46450,46450,46418,46418,46450,46418,46450,46450,46418,46418,46417,46417,46450,46417,46417,44305,46417,44337,44337,46385,44337,44337,46417,46417,46417,46417,46417,44272,44305,46385,44337,42160,29613,54837,59030,59062,59063,61143,61110,61143,59062,56949,48530,25453,10860,10827,8714,6634,6633,6601,6634,6634,6601,6601,6601,6569,6569,6568,6568,6568,6536,6536,6536,6503,6504,6503,6503,6503,6503,6503,6503,6568,19082,19115,48171,48106,44143,12777,8518,8682,6503,15084,27629,27597,21260,8550,10859,12907,8648,8517,8550,6600,4520,8844,12711,10729,17264,17101,12743,8681,13103,15020,10630,6535,491,6569,6535,525,492,459,17132,40179,40080,35687,37799,39912,44040,41993,44304,37900,8485,6470,10695,46549,59096,56884,120},
{48925,48925,50973,50973,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50940,50940,50972,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50940,50940,50908,50907,50907,50907,50907,50907,50907,50907,50907,50875,59162,61209,59128,54870,50709,50677,52757,50677,52757,50676,50644,48564,48564,48564,50644,48564,50644,50676,50677,50644,50676,50676,48564,48564,48564,48563,48564,48564,48564,48563,48564,48596,48563,48596,48564,48563,48564,48531,48563,48564,48531,48564,48531,48563,48596,48531,48531,46483,48531,46483,38064,27566,25486,25453,25453,25453,23372,23372,23340,25420,23340,23340,25420,23307,23340,23307,23275,21259,21227,21227,21195,21195,19147,19114,21194,19114,19114,19146,19146,19114,19114,21162,21162,19114,19081,16969,19049,17001,17001,17002,33839,59063,54837,44338,44338,44337,42225,31694,27501,25420,17002,10730,8681,8649,8616,6601,6601,6569,6601,6569,6568,6569,6536,6536,6536,6503,6535,6503,6503,6503,6503,6503,6503,8518,6503,8550,8518,8550,8550,14856,21260,46223,52332,50383,42225,16969,8649,8551,6470,12972,27597,29677,25485,8616,10859,15020,8648,8517,10565,6567,4520,10892,8517,8550,15184,12907,8517,8582,13104,15019,8517,6502,524,4520,6502,525,492,459,12940,40179,40112,31494,35687,37800,41960,39848,44338,42094,10729,23372,31725,50677,59096,56917,92},
{48925,48925,50973,50973,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50940,50940,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,59162,61209,59161,54903,50709,50677,56820,56820,52628,50677,50644,48564,48564,48564,50645,48596,48596,50644,50677,48596,48596,48596,48564,48564,48564,48532,48532,48564,48596,48564,48596,48564,48564,48596,48564,48564,48564,48563,48564,48564,48564,48596,48563,48596,48564,48531,48531,46483,46483,46451,27566,10924,8811,6666,6666,6634,6601,6601,6568,6601,6568,6568,6600,6568,8616,6568,6535,6568,6567,6567,6568,8615,6568,8615,8615,8616,8583,6535,6567,6535,8615,8615,8583,8615,8583,8615,8615,8615,8615,8648,21326,21359,12940,10828,10827,8681,8714,8682,6600,6633,6601,6568,6536,6536,6535,6568,6536,6535,6568,6503,6536,6536,8551,6536,8551,8550,6503,8518,8550,6503,8518,8551,8518,8518,8550,8518,8518,8518,14823,31595,50579,45993,23079,25356,23308,25323,8517,8518,8682,15052,25484,31822,25485,17231,10893,17101,6568,8517,10565,6535,4519,8845,10696,10761,15184,10860,10695,10696,11023,10860,10631,6535,524,4553,6535,524,492,459,4489,38164,44273,35720,39913,41994,39913,41994,42027,35821,33939,29580,35983,50644,59096,58997,119},
{48925,48925,48925,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50940,50908,50908,50907,50907,50907,50907,50907,50907,50907,50907,50875,52954,52922,52922,52890,50842,50842,50809,50809,50810,50809,50809,50809,50777,48729,48729,48729,48729,48729,48729,46649,44536,44536,44536,44536,44471,42423,44471,42358,42358,42358,42358,40277,38197,42325,38164,38131,38132,36051,40244,42324,40243,44404,46483,46418,40178,42259,44470,46615,46616,44536,44569,44601,36147,620,621,2701,4520,6502,6535,6535,8583,8550,10630,10663,12743,8550,8550,8550,8582,10630,8582,10662,10630,10630,10630,10630,10630,8550,10630,10630,10630,10630,10630,10630,10630,10630,10630,10630,10630,8681,10892,13006,10893,10893,8780,8747,8779,8714,6634,6667,6602,6602,6633,6601,6569,8681,8648,6633,8648,6568,8681,10664,10664,14857,8583,6600,8584,6503,6536,8551,8551,6503,8551,8583,8550,6503,6503,8550,8583,8582,19017,14856,8551,8550,12776,27469,6666,8615,8584,12939,21259,27597,23372,8780,8780,12940,6568,8517,10565,6535,6535,6764,10598,8615,11024,8845,10630,8583,10991,8780,10630,8583,4684,14987,23275,4651,2572,4586,8616,38098,42093,44108,44108,44108,46220,46220,37834,23308,38164,31660,27564,48564,59128,59030,172},
{48893,48925,50973,50973,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50940,50940,50972,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,50940,50940,50940,52956,50908,50907,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48729,46649,44537,44537,44537,44536,42424,42424,42391,40310,40278,42423,40278,42325,38197,40245,38165,38132,40277,40277,42324,46485,44404,44437,46483,48498,48564,48565,48662,52856,50776,48696,48729,48729,36148,685,653,621,4520,8550,8550,8518,8550,8518,8550,8550,8550,8550,8518,8517,8550,8550,8550,8550,8550,8550,8550,8550,10598,8550,8550,8550,8550,8549,8549,8517,8517,8549,8550,8549,8550,8550,8550,8550,8550,8550,8518,8518,8517,8517,8517,8517,8517,8517,10759,21388,21323,25580,23402,25515,23434,19177,31821,39982,35789,42127,37934,40113,10827,10696,12776,10663,8518,8517,10630,8517,8517,8517,8517,8517,8517,8517,8518,8550,6535,8649,14889,29615,8911,10761,8682,12940,14987,17067,12907,8551,8714,17133,6568,8517,8517,6535,4487,4684,8517,8518,6831,6732,8517,8518,4783,8813,38032,29646,46382,50543,52623,50511,50511,50511,50511,44207,50579,44241,48499,12775,27533,46451,31694,25585,38196,46483,42257,50644,59128,61111,160},
{48893,48925,50973,50973,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50972,50972,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,50940,50940,50940,50908,50908,50907,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,44569,42489,44537,44537,44504,42424,42424,42391,40310,40278,44503,42390,38197,36085,40245,38197,38132,38164,38163,38132,44372,40211,42389,48531,52626,48499,48564,48630,50776,50776,46616,48697,48696,36083,2733,653,653,4520,8518,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,11050,27660,27692,27660,29740,29707,29740,31820,42190,33740,21194,37966,46385,44240,17068,19147,19115,12745,8550,8485,8517,8517,8517,8517,8517,8517,8517,8517,8518,8518,6535,10762,31629,31661,8583,6503,8714,12907,10826,10762,8616,8517,10762,19246,6600,8517,8517,6567,6535,17231,10761,6601,6700,6667,8649,6601,4619,6733,33970,46484,46350,50543,52590,52590,52590,52590,50542,42126,52791,56950,42159,21162,33905,46451,27502,38198,44468,46449,42191,46483,59128,59063,140},
{48893,48925,50973,50973,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50940,50908,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,44569,42489,44537,44537,44536,42424,42424,40343,40311,38230,44503,44470,42325,40245,42326,38197,40212,36115,36052,36019,40211,38131,42357,44371,48466,44338,44372,44503,46616,44568,42423,38263,42390,31890,2732,653,620,4520,8518,8518,8518,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,17097,21258,21258,21290,21257,23338,23338,25418,27466,12775,19081,31693,29548,31629,42225,42192,37836,14824,8517,8485,8550,8517,8517,8517,8517,8517,8517,8517,8550,6569,6568,17068,31629,29549,6536,6699,10762,10761,8714,8550,8517,8517,8681,21294,6568,8517,8517,8615,14953,48563,21163,23308,23308,23340,23341,23341,23373,12973,19245,33969,44301,50542,50542,50542,50542,50542,50542,44206,54871,46386,42257,40079,38064,44403,36019,36051,27468,21160,33773,44371,59128,59063,193},
{48893,48925,50973,53021,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50908,52956,50907,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48762,48729,44537,44537,44537,44537,44536,42424,42424,40311,40311,40278,44503,42358,40277,38197,40277,40277,36052,40245,40245,40212,38163,38131,42390,46451,50578,46386,46452,46583,46616,46616,31923,23537,34003,27729,9103,2928,587,4487,8518,8518,8518,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8485,8517,8517,8485,10565,8517,8517,10565,10533,8517,8517,8517,8517,38129,33837,41964,18985,8550,8485,8517,8485,8485,8517,8485,8517,8517,8485,8517,8517,6535,19181,27500,14856,8714,12874,12809,6536,6536,8518,8517,8517,8649,21326,6568,8517,8517,8583,44337,52626,48498,52724,54805,54837,56950,56950,52757,27469,8714,25584,44269,50542,50542,50542,50542,50542,48397,42060,37868,23208,40210,40112,33871,44403,38131,29613,31891,44404,37834,38065,59128,59096,194},
{48893,48925,50973,53021,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50973,50973,50973,50973,50973,50973,50941,50972,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50940,52956,50907,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48730,46649,44537,44537,44537,44537,44536,42424,42424,40311,40279,40278,44471,44438,42325,42357,44438,38165,36020,36084,33971,33939,34003,36051,40310,46451,48498,42259,44404,46583,46616,46616,40277,40244,57049,40275,29971,9168,4910,6600,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,10565,8517,10565,10565,8517,10565,10565,10565,10565,8517,10565,8517,38097,33805,39916,14923,10631,10566,10566,10598,10598,10598,10566,8518,10566,8518,8518,8518,6502,36050,31660,14889,16871,8485,8516,8485,8517,8518,8517,8518,6569,17101,6536,8517,10662,10663,44336,52658,42126,52724,54837,54869,56918,52691,50576,46318,40078,40145,44269,50542,50542,50542,50542,50510,33608,29546,27369,12710,31857,31627,35950,44403,50677,42291,42259,29579,33610,38066,59128,61176,165},
{48925,48925,50973,53021,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50908,50908,50907,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48762,46649,44537,44537,44537,44537,44536,42456,42424,42391,42359,40278,42359,44471,40245,38197,42326,38132,38100,38164,31923,38132,38130,38163,42390,46451,50578,44339,46485,46583,46616,46617,44502,52790,56950,44468,50904,40405,38324,8680,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,8517,33871,40176,39852,25291,23111,23079,25192,25192,25192,25192,25192,25192,23144,12646,8518,8518,6503,31824,27434,37935,27174,10565,8517,8517,8517,8517,8517,8518,6601,19213,6536,10630,42257,17002,46416,52658,40078,52724,54805,54837,50611,46384,52591,52591,50543,48530,44269,50542,50542,50542,50542,50478,25255,19080,16871,8550,23471,33805,33806,44405,48630,54936,46353,16902,37931,42226,61209,61176,154},
{48925,48925,50973,53021,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50973,50973,50973,50973,50973,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,50940,50940,50940,50908,50908,50907,50907,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48730,46649,44537,44537,44537,44537,44536,42424,42424,42391,40311,40278,38198,44438,40277,38165,50679,40276,34004,38164,33971,31891,40243,38195,42389,44403,46386,44339,44404,44503,46616,46617,50775,54902,59063,44467,57177,59257,57177,12840,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,8517,38131,19082,39786,41865,39850,35592,44010,37672,37704,44074,33446,39752,18917,10533,8485,6634,4488,17165,29581,31595,10532,10564,8517,8517,8517,8518,8517,8551,6536,19246,8649,40144,52658,44272,52690,37933,35918,52724,52756,54837,54837,50643,52624,50511,50543,48498,42156,50510,50509,48429,50509,48397,37866,19015,10630,10630,21326,33871,31692,44471,50711,48597,44305,25355,16902,33840,59128,61176,147},
{48925,48925,50973,53021,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,52988,50940,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48762,46681,44537,44537,44537,44537,44504,42424,42424,42391,40311,40311,38230,42358,40245,40245,42293,42390,36084,38132,38164,31891,36083,38163,42422,44371,48466,44338,44405,46583,46616,46649,48663,54870,59095,44500,57177,52951,57176,14953,8550,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,10565,10565,10565,8517,8517,8517,8517,10565,10565,10565,8517,31793,14922,46157,46156,48204,48171,48204,46123,41930,41931,41898,37673,12646,8518,8518,8682,4488,8780,23308,29548,10532,10564,10564,10564,8518,8518,8517,8518,6536,12972,35984,54706,27402,19048,31692,27434,31725,40080,33807,35888,35920,35952,37966,23208,31626,42225,37963,44236,46316,46284,46316,46284,46284,31528,14790,12710,21294,29712,27467,35919,44438,46518,46452,38066,21096,31792,59128,61176,195},
{48925,48925,50973,53021,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50973,50973,50973,50973,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,50940,50940,50940,50908,52956,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48730,48730,44569,44537,44537,44537,44536,42424,42424,42391,40311,40278,40310,42358,40245,40277,38197,38197,36084,36083,36051,33939,38131,38163,44437,46451,44338,44339,44405,46583,46616,46649,46616,50742,52822,44436,57177,55096,55096,14985,8550,8518,8518,8518,8518,8518,8517,8518,8518,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,8517,23440,37968,50383,48237,50383,52431,31465,14759,10598,10598,10598,10566,8583,10631,8551,10827,10794,13005,17034,38032,29253,12612,8485,8485,8517,8517,8517,8518,6503,25584,52658,23144,8583,8583,19145,25386,29548,21130,8616,8617,8682,10762,29645,29481,21193,40176,42157,50510,50510,50510,50542,50510,50542,39946,27368,19015,21294,23471,40078,35821,42357,48663,48663,48597,35953,36018,59128,59128,207},
{48925,48925,50973,53021,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50973,50973,50973,50973,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,52988,52988,50940,50940,52988,52956,52956,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48730,48730,46681,44537,44537,44537,44504,44504,42424,42391,40311,40278,40310,42391,42325,38197,38262,38164,38164,36084,34003,31891,40243,40211,44404,42291,44307,44339,42357,44535,44568,44569,44569,44568,38196,38195,55096,55096,55064,17066,8518,8518,8518,8517,8517,8518,8517,8517,8518,8517,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8485,8517,8517,8517,8517,8517,8517,31760,50383,50317,50317,48237,39851,29418,21065,16903,16904,12711,10598,8551,8551,10664,33840,44404,40179,23342,33773,33381,14661,8485,8517,8517,8517,8517,8517,10696,27597,48399,12677,8550,8582,12808,25387,23274,14823,8583,8583,8648,10728,19245,31628,21161,40144,42189,50542,50542,50542,50542,50542,50542,46252,37866,35852,21293,19246,44272,48367,42159,35919,42423,39982,27599,46483,52756,54869,222},
{48893,48925,50973,53021,53053,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50973,50973,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,52956,52956,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48730,48730,48729,44569,44537,44537,44504,42424,42424,42391,40311,38230,40278,40310,42325,38165,40310,40277,36084,34003,34004,34003,36083,38163,42357,40210,40178,42227,42357,44536,44568,44569,44569,46649,52922,54968,36210,40371,44435,21194,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10630,19114,17001,14921,14857,8616,8583,10663,8517,8518,8518,8517,8518,8517,8517,8518,8517,8517,8517,8517,8518,8517,8517,8517,8517,12678,14758,14758,14726,12678,12677,12678,12677,12710,21129,12678,8517,8518,8517,10696,40145,57015,44306,21195,29580,29253,14693,8484,8517,8517,8517,8517,19081,40079,29645,48399,12678,8550,10663,14921,25419,23307,46451,50676,52757,52789,52789,52691,12644,10565,25453,42189,50542,50542,50542,50542,50542,50542,46317,39979,35788,23374,12973,38031,54704,37867,33806,38197,40113,40243,46450,48366,33769,183},
{48893,48893,50973,53021,53053,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50973,50973,50973,50941,50973,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,52956,52956,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50843,50875,50843,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48762,48730,48730,46649,44537,44537,44536,42424,42424,40343,40311,40278,40310,38197,42357,36117,38197,42325,36052,38132,36052,40212,42291,40243,44437,42290,42290,42259,42357,44535,44568,44569,44569,44601,46617,48695,36210,40371,54935,19114,8550,8518,8518,8517,8517,8517,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8551,6634,10761,10828,8681,8649,6634,6601,8550,8517,8550,8518,8517,8518,8517,8517,8518,8517,8517,8518,8517,8517,8517,8517,8517,8517,8485,8517,8517,8517,8517,8485,8517,8517,8517,8517,8517,8517,8517,8485,8550,44371,61209,59096,27436,23372,33414,18853,8517,8517,8517,8517,12743,48530,46352,52691,42094,8550,8550,8550,10728,25387,23242,44338,52756,54837,52691,54803,54705,18918,19049,23406,42190,50542,50542,50542,50542,52590,52591,48398,44172,37900,25486,6732,40209,58963,39980,35919,34004,33873,42323,46450,50512,27466,151},
{48925,48925,50973,53021,53053,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50973,50973,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48730,48730,48729,46681,44537,44537,44504,42424,42392,40311,40311,38230,38230,38230,40277,38165,40277,48663,38164,36084,36052,33971,36051,38132,42357,46418,48498,44339,46518,46616,46617,44601,44601,44601,46649,44568,42486,42453,44468,17066,6535,6503,6502,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8517,8518,8517,8517,8517,8517,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,6569,6634,6535,6700,8616,35919,44339,42291,44338,21261,12941,10893,10861,10893,10860,8780,8812,8748,8715,8812,8780,6732,6732,6700,6732,6700,6699,4652,4652,4652,6700,6668,4619,4619,4587,4619,4619,4587,4587,4587,6667,42291,59129,59160,46452,29614,29253,25062,10565,8517,8517,8517,12808,48464,50512,37868,18983,8518,8550,8550,10695,23306,21162,44338,54805,52626,37736,50411,50410,37832,35852,44404,40141,50542,50542,50542,50542,52590,52591,48366,42060,40013,25454,6732,38064,50578,46385,37966,33972,29680,42324,44370,46254,10564,89},
{48893,48925,50973,53021,53053,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50973,50973,50973,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48730,48729,48729,44569,44537,44536,42424,42424,40311,40311,38230,40278,40278,38197,42357,38164,44405,40212,40211,38100,44437,59097,48597,46517,42291,44338,42291,44438,44568,44568,44601,44569,44569,46649,44569,44601,46616,52790,21324,12841,12809,12808,8648,8616,8648,8615,8615,8615,8583,8647,8583,10663,8583,8583,10695,8582,10663,8582,10663,8647,10663,10695,10630,10663,8582,8582,10695,8582,10663,10630,10663,10630,6503,6503,6536,6732,10762,31726,50644,48498,42193,19247,10925,8845,10991,8780,6764,10926,8649,8649,6667,4652,4717,8715,8616,4652,6634,6601,6732,8616,6634,6667,8616,6667,4586,2538,4619,6569,4521,4586,6535,4521,6601,42258,59128,48531,40145,37966,31334,23078,10565,8517,8517,6470,15052,50577,50544,33739,16936,8518,8518,8550,8583,21194,14824,42258,52756,52659,39816,50376,50410,52458,37767,31760,35949,50542,50542,50542,50542,50543,52590,48398,40012,33739,17068,15020,25453,27534,23373,35919,33971,23341,42291,44338,48432,25354,121},
{48893,48893,50973,53021,53053,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50973,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48730,48729,48729,46649,44537,44536,42424,42424,42391,40311,40278,40310,40310,38197,38197,38165,40244,36052,38164,36083,40212,52693,48564,44371,44338,42259,40211,42357,44503,44568,44569,44569,44569,44601,44601,44569,46649,44436,38064,38030,38030,35918,21291,19211,23404,17098,23372,19178,19211,23339,17066,21291,19146,19210,19146,17033,21291,19113,21291,19145,19113,21226,19113,21259,17033,19145,17033,16968,19146,14920,21227,10695,8551,6503,6536,6699,8550,14856,48563,42224,27533,15086,6764,2636,8878,6601,4554,8845,8583,6536,4618,2603,4684,8681,8550,4619,8584,6503,6700,8550,6568,6634,8550,4554,6601,4521,4587,8550,6503,6602,8518,6536,6568,40178,57016,46451,46418,29385,14660,12612,10532,10532,8518,8682,42224,54738,42125,31659,16936,8518,15019,19082,8550,19113,10630,38065,52756,54837,54804,56917,56884,46286,21031,8813,31821,50510,50542,50510,50510,50542,52590,48398,42157,37900,42191,27402,17001,23340,23372,38032,36019,27534,42323,44338,48367,12709,74},
{48893,48925,50973,53021,53053,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50973,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,52988,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,48729,46649,44537,44536,42424,42424,42391,40311,40278,40310,40310,40277,38165,40277,42357,42357,42325,44437,38100,33972,42324,44403,42258,40146,40179,42357,44535,44568,44569,44569,44569,44569,44569,44601,46649,42389,38031,40078,37965,31724,21258,23404,21291,19146,25452,19146,25452,21194,19114,21259,19146,23371,19113,21226,19145,17032,21258,17065,21258,17065,19145,21226,19113,23339,17065,23371,19081,12874,12939,8681,8616,8649,8551,6666,6535,19146,48531,33773,12841,10925,4683,555,8877,6536,4488,8812,8517,8551,4586,2571,2604,8649,8517,6635,8550,8518,6667,8517,6536,6601,8517,4553,6536,6503,4586,8518,8550,4553,8517,6535,6536,33872,57015,59064,46417,42159,33414,31335,18886,25062,10565,8517,40078,50544,35754,31627,19017,8518,23406,42291,27599,38065,12776,35952,52724,52756,54837,54869,54837,33741,10696,27697,46449,54769,54769,54769,56817,54769,54736,48430,46318,40013,46286,23176,14888,19114,17034,35951,33939,44371,42323,44338,56885,40079,132},
{48893,48925,50973,53021,53053,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46681,46649,44537,44504,42424,42423,42391,40311,40278,40310,38197,42325,38197,40277,40245,42292,38164,40244,46485,50743,42324,46451,44338,44371,44339,44437,44535,44568,44569,44568,44569,44601,44569,44601,46649,42356,52821,52756,52756,52756,46482,48563,48563,50611,52756,52756,52756,52724,48563,48563,50675,52820,48530,48497,52756,50676,46417,35919,15019,27662,36048,23404,10793,12841,12809,38130,29582,10860,8779,8682,10794,8681,8550,6633,4586,6666,29614,27500,16969,8780,4618,4586,4684,6569,6569,6667,8550,6503,4553,2506,4586,6601,8550,4554,8583,8550,6634,8550,6535,6569,8518,6569,6537,4488,4521,6503,6503,4521,8518,6503,6536,25552,52822,54870,52822,52757,37607,35625,39981,37769,31334,8517,40078,50512,35755,29515,16937,8518,8583,23308,23340,44470,46485,44337,52724,52757,54869,56918,56917,25323,27598,48562,56849,56849,56817,54738,50544,50544,46252,46318,44239,42159,46286,21128,12841,19213,17099,40144,23439,27533,42291,42290,56885,29514,71},
{48893,48893,50973,53021,53053,53053,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50973,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50940,50940,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46681,46649,44537,44504,42424,42424,42391,40343,40278,40278,38198,40277,38165,42357,46486,40244,36051,40243,38131,44339,38163,44372,42290,42258,42259,44438,44535,44568,44569,44569,44569,44569,44601,44601,44503,40242,56950,59030,56918,61143,54836,54804,54837,54772,52692,52724,58965,58964,58997,59062,54804,52756,56950,50610,35886,35951,46353,38033,8682,12940,17099,38161,40112,35886,35920,33839,25453,10860,8779,12906,40178,40178,38097,38097,40178,38098,38065,40145,25519,12974,10893,8845,6732,8813,8813,6667,8780,8780,8780,6732,4652,6732,6700,6700,6700,6700,4652,6700,6668,6668,6668,6668,4587,4587,4587,4587,4554,4554,4554,4586,4554,4587,6602,6699,10696,40210,37607,33382,33414,37607,37574,27239,44173,48431,31561,27403,18985,8518,8517,8517,8550,19179,21260,42323,52724,52756,54837,56917,54869,37999,50610,56882,56849,56849,42060,27401,27467,12677,31659,40046,42094,33740,46318,21129,10794,19377,21260,38064,21195,23307,42292,42323,56885,27401,52},
{48893,48925,50973,53021,53053,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50973,50973,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50940,50908,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46681,46649,44537,44536,42456,42423,42391,38231,36150,40278,42358,40245,40213,44437,44437,38164,42293,40244,46484,50644,46516,44371,44338,44338,40211,44437,44535,46583,46550,44502,44470,38098,36017,36017,33871,31758,33838,40144,38000,46385,35886,33806,31693,29613,29581,29613,33773,37902,40047,42191,35886,35919,42160,42192,35886,33806,33806,33774,29580,29613,31725,35951,37966,37902,29581,25452,19180,10860,17132,50677,59063,59063,57015,56950,50579,31725,42192,42159,19050,10827,8780,6765,8812,10860,8780,8748,10795,8714,10894,8714,6667,8780,8682,6700,8714,6568,6700,8649,6601,6700,8616,6634,4586,6569,4586,6569,6568,4586,6535,6536,6601,8550,4521,4619,6535,25421,29351,18853,20933,20901,20933,20965,40013,48431,31562,23210,19016,8518,8517,8517,10761,19212,27666,48565,52724,52756,54837,54869,54869,50644,56884,54803,54770,54737,48333,44206,23209,10631,16936,12677,29578,25387,46319,23210,10761,10958,14954,33904,21064,12842,42324,42323,56982,46451,150},
{48893,48893,50973,53021,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50973,50973,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,52988,50908,50908,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48729,48729,46649,46649,44569,44536,44504,42424,40343,38231,36150,38230,44438,42293,38165,40245,44470,38164,33971,33971,40211,46419,44404,44370,44338,42258,40211,44437,46615,50611,48498,48498,46385,48466,48498,48498,48530,48530,50578,50578,50579,50578,50578,50578,50546,50578,50546,50578,52659,50611,50578,50578,50578,50578,50578,50610,50611,50610,50610,50610,50610,52658,52691,52658,50610,50578,42192,23339,15019,23406,54903,56983,54870,56983,54870,54870,46385,19016,31692,31692,8616,6634,6666,2636,6764,6633,4521,6700,10631,8518,10893,8517,6536,8780,8518,4586,8616,8517,6700,8550,8518,8747,8485,6568,6601,6502,4554,8551,8517,4586,8517,8550,6569,8484,4488,4619,6535,17100,27271,8484,8484,10533,10532,8484,33740,48431,27336,21129,21129,8550,16936,17001,21323,15018,12809,35952,52692,52756,54837,54869,56917,56917,56918,54771,48432,44304,39980,39883,29581,19017,23339,21128,33739,27501,44271,25290,14921,19179,21292,31758,10696,4619,40244,48630,52725,42226,157},
{48893,48925,50973,53021,53053,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50973,50973,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50940,50940,50940,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46649,46649,44569,44536,42424,42424,40343,40311,38230,38198,40310,42325,38197,40277,44405,40244,38131,36051,38163,44339,44372,44371,46418,46418,44339,44470,48695,50643,48498,40113,38065,42224,48498,48498,48530,48530,50579,50578,50611,50611,50611,50611,50578,50578,50578,50578,52659,52659,50578,50578,50610,50610,50578,50578,50611,50611,50611,50610,50610,52659,52691,52659,52724,48531,46517,50710,31759,57016,59063,59063,59096,56950,37901,40080,48465,19145,29645,31724,8583,6601,4586,2603,6764,6666,6601,6667,8615,8518,8845,8517,6535,6699,8550,4586,6634,6536,6700,8550,8518,8747,8485,6535,6601,4487,4554,8551,8517,4586,8517,8550,6569,8485,6535,4619,6535,23373,16805,10565,10565,10564,10564,10564,31627,44141,10597,17001,19181,16969,50577,52657,52657,33707,8518,27534,50644,52724,54837,54869,54869,54869,54869,54771,46385,42257,40046,42029,50710,44305,42257,31594,37965,29614,40111,25290,14921,17066,19146,25420,4586,21424,48498,42061,27401,21128,66},
{48893,48893,50973,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50973,50973,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48729,48729,46649,46649,46617,44536,42456,42424,42391,40311,40278,40278,40310,40277,36117,38165,40245,40244,38196,40276,42324,46485,44437,44372,42226,44339,42292,44471,50808,50643,46418,46452,50677,48531,48530,48498,48498,48530,50611,50611,50579,50611,50610,50610,50578,50578,50578,50578,52659,50611,50610,50578,50611,50611,50611,50611,50611,50611,50610,50610,52658,52658,52659,52691,50709,46484,46485,56983,59096,61176,59063,59063,59095,52692,23209,40112,56983,54870,54871,54870,23440,15152,15119,13039,10893,8813,6733,6700,10827,8714,8846,8682,6634,6732,6634,6668,6667,4586,6700,8714,6634,6700,8681,6667,6635,4553,4586,4553,4553,4587,6569,6569,4554,4521,4521,4619,6536,21293,12644,10532,10532,10564,10564,8516,35918,44206,14790,14889,36050,23241,50578,54738,54705,35754,8518,25453,52724,52756,54837,56917,56917,56917,56950,54804,52756,52790,35952,52757,54903,46418,42290,35885,37998,33806,37998,25290,14921,17034,17099,21260,25585,44306,59062,59029,56851,44204,103},
{48893,48925,50973,53021,53021,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50973,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,50940,50940,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46649,46649,46617,44536,44504,42424,42391,40311,38230,38230,42358,42325,38165,36117,38165,38164,33971,36084,36019,44340,42292,42291,42290,44339,42292,44471,50776,50611,48498,46451,48531,48530,50578,50578,50578,48498,50611,50579,50579,50578,50578,50578,50578,50578,50579,50610,50611,50611,50611,50611,52659,52659,52659,52659,52659,52658,52658,52658,52691,52691,52658,52659,50709,50709,48597,56951,59063,59096,59096,59096,59063,52724,33773,44338,59096,59096,59064,54870,25488,17232,17200,15120,10893,8845,8812,6700,10828,8747,8846,8747,8747,6765,6668,6733,6667,6602,6700,8714,6601,4652,4619,4619,6667,6569,4586,4587,4587,4587,6568,6536,4553,6536,4489,4620,6601,15020,37933,27141,29254,31334,31301,33478,46286,50544,29481,12808,42356,27434,46417,52658,52592,35788,21163,23372,52724,52756,54837,54869,54869,54869,54870,54804,52724,52789,29711,52823,52790,44371,42291,42159,42191,38000,33773,25322,14921,23308,27566,38033,40145,46419,56982,56981,54803,52690,110},
{48893,48893,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50973,50941,50941,50941,50940,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,50940,50940,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48730,48729,46649,46649,46617,44536,44504,42423,40343,40311,38230,38198,46551,40245,38197,38164,44470,34004,31892,34003,38131,38164,40276,44371,44370,42258,40211,44470,48696,50643,48498,48498,48530,50578,50578,50578,48530,50578,50611,50610,50578,50578,50578,50578,50578,50578,50578,50610,50611,50611,50611,50611,50611,52659,52659,52659,52659,50611,52691,52659,52691,52691,52658,52691,52757,50677,48597,52757,54837,59031,59096,59096,59096,54837,23274,33774,54902,48628,48563,35853,10695,8681,8616,8583,8747,6666,4553,6635,8615,8518,8845,8518,6535,8748,8518,6601,6601,6502,6667,8582,8517,4652,490,2539,6634,8517,4521,2506,2506,4587,8517,8518,6569,8518,6503,4652,6536,8616,37901,33381,35461,35493,35461,39686,44206,50577,31530,14857,40244,33741,46385,52625,50545,38000,46484,31759,52789,54837,54869,56917,56917,56950,56950,54836,48563,46484,44305,42061,40014,40113,35951,40144,42160,35887,19049,17034,14921,12841,12841,14953,40210,44338,56982,56982,56981,54803,139},
{48893,48893,48893,50973,50973,50973,50973,50973,48893,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48730,48730,48729,48729,46649,46649,46617,44536,44504,42424,42391,40311,38230,38230,50776,42325,42358,38197,36116,36052,33972,33971,38163,38131,40211,40210,44370,42259,42292,44503,48696,50611,48466,48498,48498,48498,48498,48498,48498,48498,50578,50578,50578,50578,50578,50578,50578,48530,50578,50578,50578,50578,50579,50579,50611,50611,50579,50578,50546,50578,50578,50579,50611,50611,50578,50578,50709,50710,52855,59063,56884,54804,56983,59063,54870,56983,54870,50677,54902,44435,44369,33739,16968,8616,8583,8518,6666,6699,6634,6634,6666,2537,6797,8550,6535,8780,8517,4553,6601,6502,4587,8583,8517,4620,2505,2505,6601,8517,4553,2506,458,4586,10598,8517,6601,8518,6504,4653,6504,6536,31628,33381,35461,37542,37542,37606,44206,48432,29481,16936,38130,29514,48497,54706,50580,42259,48597,33775,52789,54837,56917,56949,56917,56917,56918,54804,50545,46385,48465,50512,50512,42193,44371,44371,42225,19115,10728,14986,14921,12841,14922,14954,38130,44338,56950,58997,39981,54672,211},
{48893,48893,48925,50973,50973,50973,50941,48893,48893,48893,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50973,50973,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48730,48730,48729,46681,46649,46649,46649,44536,44504,42424,40343,42359,40310,40343,48696,42325,54937,42357,36084,36084,42292,38163,40211,36051,42324,42258,40178,40146,38163,42390,38197,46450,48498,48498,42126,37933,46353,40046,31627,33740,33740,31660,35853,31692,31660,33740,33739,33740,31595,33740,31627,31660,35852,35853,35820,35853,31627,33740,31627,31660,29482,31692,33739,33740,33739,33772,37933,46483,40178,42257,54770,52658,54771,56950,59063,59063,56983,56950,54870,40242,42257,33804,29612,8712,6567,8550,6666,8747,6634,6666,6731,4651,6765,8681,6601,6732,4553,4619,4586,4521,4587,6601,6568,4619,6568,6569,6635,6568,4554,4586,2506,4586,6536,6536,4553,6536,4489,2605,6536,6536,31628,33381,35462,37542,35494,37574,44173,48432,29449,19016,33871,29514,48497,54706,44373,42324,48597,44371,59063,61143,61175,63256,63256,63256,63256,52657,58930,58930,52690,50610,52657,44305,38032,44370,25387,29712,42224,31595,17035,17132,17132,14921,36017,44338,56950,54869,27434,54672,215},
{48893,48893,48893,48893,50941,50941,50941,48893,48893,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50973,50973,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48730,48730,48729,46681,46649,46649,46649,44536,44504,42424,42391,42391,40278,42423,46583,40277,57049,44469,40244,38164,40243,38163,36083,33971,40211,44371,38098,40179,40212,42423,38229,48531,48465,48465,25321,14823,44305,46384,23208,29579,35820,31659,42158,35885,29546,31660,31627,33708,37965,37965,31626,33740,40044,40078,40044,42126,33707,35853,35819,35853,25321,35853,37932,37933,40013,40078,42125,42191,48498,25355,54770,54770,56850,56883,56917,56983,57015,56950,59096,59096,59063,56983,27632,17232,15152,15152,10893,10926,10926,8748,10893,10893,8813,8846,6797,6765,6765,6733,4652,4652,4619,6732,6700,4652,6700,6668,6668,4619,4619,4587,4587,4587,6602,6601,6601,6601,6569,2604,6535,6536,29548,27173,27173,31334,31334,31333,42093,46385,35886,29548,33839,31628,48465,50545,50578,33741,29614,40178,54836,56917,59029,59062,59062,61110,61143,48431,56850,56850,54771,52724,50543,42192,23406,29613,33936,59029,65269,54737,46449,48497,35820,23208,38066,42258,52758,46550,46483,52591,209},
{48893,48893,48893,48893,48925,50973,50973,50973,48893,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50973,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46649,46649,46649,46617,44536,44504,42424,42391,42391,38230,42391,44503,38197,55001,44437,38164,40244,42228,40212,42356,33971,42292,42291,40178,38098,40212,44502,50743,52691,48498,48498,25289,12743,44272,46384,44239,44272,46352,44304,46352,46352,46352,46352,46384,46352,46384,46352,46384,46384,46384,46384,46352,46352,46352,46384,46352,44304,46384,46384,46351,46384,46352,46352,50578,54770,54771,35918,54771,54770,56818,56883,56883,61176,61143,59062,61143,56949,50578,52658,42224,12908,12875,10729,10828,8747,6699,8780,6666,6601,6634,8584,6568,6634,8583,6569,6601,6536,6602,8583,8551,4586,6536,6535,6569,8550,6536,6569,6536,6569,8518,8518,6536,8518,6503,2572,6536,6504,29580,31301,31334,33414,35462,35494,42028,42259,44437,27403,33839,42192,56884,54706,44174,12678,6503,42324,54804,56917,56949,59030,59030,59062,59062,48399,56817,56817,52690,52724,50511,42094,33774,42258,61076,63123,60944,56783,31496,18982,40078,33609,33905,42258,52758,40113,40144,54671,213},
{48893,48893,48893,48893,48893,50973,50973,50973,48893,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50940,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48730,48730,46681,46649,46649,46649,46649,44536,42424,42424,42423,42391,40310,44503,54969,38197,54936,46550,38196,40244,38164,40244,46484,40243,44403,46451,44370,44306,31825,23406,19147,46418,48498,48498,25322,14823,44272,46385,40012,37965,42158,40045,42125,35820,42125,40013,44206,40045,42125,37900,40013,31627,42125,40045,37932,31627,35787,31660,40044,35820,40012,37933,37931,33772,39979,35852,27369,46385,52822,54870,54771,54770,54770,54770,58996,61142,56884,54837,59030,56917,50578,48498,42157,14988,12842,8615,8812,8747,4619,8845,4619,4585,6634,8583,6536,6666,8550,6569,6602,6535,4554,8583,8550,4619,2506,6536,6601,6503,4521,6536,8518,4521,8518,8518,6569,6502,6503,2572,6536,6504,27500,31333,35462,35494,35494,35494,42060,46352,19016,23209,40111,56852,48465,48497,23209,10597,8518,31792,52756,54837,56917,56949,56950,56950,56982,48399,54769,56785,52658,50643,48463,37967,44403,54837,58931,60946,27173,18852,10532,10597,27433,31561,31792,42258,52724,35789,48597,54737,219},
{48893,48893,48893,48893,48893,48893,48893,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50843,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46649,46649,46649,46649,44536,42424,42424,44471,40343,40278,44471,57049,38197,57049,44437,36116,38196,40244,38132,44403,44403,44404,46451,48466,44306,25552,8779,6633,44338,48498,48498,40079,35886,42192,50579,42191,42192,46384,46352,46352,44272,46352,46352,46384,46385,46384,46385,46384,44337,46384,46385,46384,44305,46384,46385,46384,46417,46384,46417,46352,46385,46352,46385,44304,46418,52822,54869,54738,54738,54738,54738,61077,63190,56851,54837,52756,48596,48498,46418,44236,12874,12810,8517,8747,6699,523,6765,4651,4585,6634,8550,6503,8747,8518,6569,6601,6503,4586,8583,8518,4619,2538,6536,6602,6503,4521,6536,8517,4553,8518,8518,4521,6502,6503,2572,6536,6536,21260,22981,23013,23013,25061,25061,35754,42126,36019,44339,37902,40112,42292,50677,42226,16936,8550,29678,52756,54837,56917,56950,56950,56950,59030,48399,54737,56785,52658,50611,48463,50546,54870,54871,56916,58899,16805,10565,10564,10597,21063,33706,29678,42258,50612,50676,37901,54803,224},
{48893,48893,48893,48893,48893,48893,50973,50973,48893,48893,50973,50973,50973,50941,50973,50973,50973,50973,50941,50941,50941,50941,50941,50973,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50940,50940,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48729,48729,46649,46649,46649,46649,44536,42424,42424,44471,40310,40278,40278,52856,38197,46551,44437,42389,40244,38132,36051,48597,48531,42324,42259,42192,33839,27599,27632,27632,46419,48498,48498,48498,50578,48466,42159,40079,40079,42127,42127,42127,42127,42127,42127,40046,40046,40078,40047,40079,40079,40079,40079,42159,40079,40046,37998,40046,40047,40046,40014,40046,40079,40046,40046,37933,37966,42225,44336,54739,54770,54738,54738,61077,63158,56818,54837,46514,40209,44305,52757,35951,10925,12875,8616,8812,6731,4619,6765,6699,4586,6634,8583,6536,6699,6503,6569,6634,6535,4586,8616,8550,4619,4521,6536,6602,6503,4521,4553,4521,4554,6503,8550,4521,6503,6503,2571,6536,6536,6601,14823,10631,8550,8518,8550,29547,40013,23308,25356,10631,19050,14856,14758,10598,12842,10664,27566,52724,54804,54837,56917,56917,56949,56950,48432,52656,54737,54738,52724,48463,54705,54838,54871,56918,58899,18885,10564,10565,10565,16870,33707,31726,40178,52790,57015,54805,42224,159},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50973,50941,50941,50973,50973,50973,50941,50941,50941,50941,50941,50973,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48762,48730,48729,46649,46649,46649,46649,44536,42456,42423,44471,40311,38230,38230,48631,40245,40277,40245,36116,38164,36051,36083,48630,42292,42358,42323,44370,52791,54903,57017,56984,50644,48497,48498,37900,33707,46385,46385,25289,29515,42158,37900,42158,33740,40045,29547,42125,31627,42158,40013,29514,29548,35820,29579,29514,31628,42125,40045,37932,31660,29482,31660,40012,31660,37932,31660,42125,40078,33707,33837,52690,54770,54770,54738,61077,63158,54770,54837,46482,42290,48498,56951,48564,19279,17200,15152,13039,13039,10959,10926,13007,8878,8748,8780,8781,8781,6732,6700,6732,6732,6700,6700,6700,6700,4652,4652,4620,4620,4619,4619,4652,4619,6602,6602,4554,4554,4554,2571,6536,4554,17198,42224,10729,6568,6569,6569,29612,39981,10565,8518,8518,6502,8551,10598,8583,21326,21294,27632,52724,54804,54837,54837,54869,56917,56950,48432,52592,54738,54738,52724,48463,54705,54805,54871,54870,58899,20998,10564,10564,10565,12645,31595,33774,48564,54870,50578,25387,14922,42},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50973,50973,50941,50941,50941,50973,50941,50973,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48730,48729,46681,46649,46649,46649,46617,44536,42424,42391,44471,40311,40278,40278,44439,40277,40310,42325,38164,36084,36084,36083,42293,48532,44437,46451,46451,52823,57016,57016,57016,48531,48465,48465,35820,31594,44272,46352,35853,40047,44271,42191,44271,44239,44239,40078,44271,42159,44271,44271,40047,40111,44272,42191,40079,42159,46351,44271,46320,44272,44271,44272,46320,44272,44272,44272,44239,44271,44272,42258,56850,54770,54770,54770,63223,63190,50513,54837,54870,54837,56917,59063,56917,25323,25485,21260,10664,10696,10761,8583,10664,10729,8551,8583,8583,8550,8583,8550,8550,8583,8550,8550,8550,8518,8550,8550,8517,8550,8518,8518,8550,8517,8517,8517,8517,8517,8517,6568,4521,4718,42323,50612,21227,21260,21292,23341,35853,44174,12677,10631,12743,16936,10630,10565,8550,12972,33840,25486,52756,54837,54837,54869,56917,56917,56950,50513,50511,54738,54738,52724,48463,54705,54771,54871,54871,56852,23046,10565,8517,10631,10565,23209,19016,14888,14888,10663,10696,10664,59},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50973,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,46681,46681,46681,46649,46649,44536,42424,42424,42423,40311,38230,40278,46519,42325,40277,48598,40244,36084,38196,38099,44438,50677,42357,44339,46451,54904,57017,57016,52791,44370,48498,48498,35820,31627,44272,44271,42093,40045,44271,42125,44207,37901,44271,42125,44270,42125,44271,42093,44238,40046,44270,42158,42157,40045,44206,40045,44238,42126,44238,42125,42125,40045,37900,33740,42157,40045,42126,52691,58931,58930,58931,61109,65368,54739,35853,52724,50643,46482,54837,48431,35819,25256,21161,23241,8517,8518,8550,8517,8517,8550,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,8517,4488,2638,6602,38031,54903,52756,52757,46386,33807,35820,10598,8517,12776,29580,40078,25290,10598,8518,10827,23243,42355,59095,61175,61175,61208,61208,63256,61175,52724,54803,54770,44239,52724,52690,52657,42093,52823,54871,54804,31430,10565,10696,12810,10598,16904,19016,12840,12808,8582,10696,10663,60},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50973,50973,50973,50973,50973,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48730,48730,48729,46681,46681,46649,46649,46617,44504,42424,42424,42423,40311,38230,40278,46519,40245,40245,40245,40244,36052,38164,36084,48630,52725,44437,46419,48498,54904,57016,57016,48598,44371,48498,48498,35820,27434,40047,46385,37933,33739,42190,33707,37901,29612,44238,37933,44270,40077,44238,35852,42126,31692,44271,37965,37965,33772,44238,40078,35853,35853,42158,42126,44238,40078,40013,35885,42158,37966,42126,42126,40045,44238,50512,46384,50513,33676,40144,52757,50644,44402,52789,48531,40144,31627,35886,25387,8583,8616,8648,8550,6535,8584,8518,8583,10664,12743,8551,14857,10631,6503,8550,8550,8550,8550,8551,8551,8550,6503,8551,8550,8551,8550,8550,8550,8518,8550,8550,4653,6634,8550,38030,50545,50611,52724,35887,10860,19082,8550,8517,8517,23274,42126,12677,8517,8518,8518,25421,59030,61110,61142,54738,48432,48464,48465,52757,52724,56850,52494,25322,50644,48496,39946,21030,50710,54871,52625,33447,10565,14889,12744,10598,10630,19016,23339,14888,8582,10728,8518,35},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50973,50973,50941,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,46681,46681,46649,46649,46649,44536,42424,42424,42391,40311,40278,40278,42326,40245,38165,38165,38164,36084,38164,38132,50711,48532,44437,42291,46517,54903,57016,57016,52790,48531,48498,48498,48498,48498,48498,50546,50578,48530,50578,50579,50611,50611,50611,50611,50579,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50579,50578,50578,50579,50578,48530,48498,50578,50546,50578,50578,48466,48465,48465,48465,46353,44273,54837,59031,61143,59095,61176,61143,61143,61143,59063,52757,46517,44436,33905,21359,17134,17166,17133,17133,21359,17134,17134,21327,15053,19246,17133,12940,17167,12940,10892,15086,10860,12973,12973,8747,12973,10827,10795,12940,10795,10828,8715,6667,8714,27631,54771,54771,50611,52724,29549,8747,19180,29548,10565,8517,21194,39980,10565,8517,8518,8518,23308,54837,56917,58997,50480,40045,42159,44271,50676,52724,56818,46155,23273,50644,52691,50413,12645,48596,52560,29446,16869,14857,44404,40048,44338,10564,14823,23274,16936,8583,8681,10696,50},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50973,50941,50941,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48762,48730,48729,46681,46681,46649,46649,46649,44536,42424,42424,40343,40311,40310,40278,44438,40277,42325,38196,38164,34004,38132,38132,44405,50612,46452,46419,46419,52790,54903,57016,50677,48563,48466,48498,44271,44207,46384,40046,33739,31659,37933,31627,37965,31627,35853,29547,33740,29515,35853,31595,35852,29547,35852,29546,37900,33740,35820,29547,35852,31627,33740,29515,33707,31595,35820,29547,35819,29546,37868,31627,33739,29547,31627,29515,42127,42257,59095,59063,61111,61143,59063,59062,59063,59062,59062,59030,61143,59095,59030,50644,44338,46484,46483,48532,50644,48564,50677,48564,48564,50677,42225,23373,19213,21293,19245,21293,21293,21261,21261,19213,17133,17099,10827,6568,8714,8714,8583,8649,8681,40112,52625,44174,44273,52692,21228,6699,15052,42159,14823,8550,27532,33707,8583,27665,17101,17166,25486,56950,56949,56982,56884,52724,52756,54804,54804,54770,56850,46253,48596,61143,56883,50446,42257,61175,54705,35845,16963,17033,50677,48532,48531,14757,19048,31627,16968,10663,10664,10597,44},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50973,50941,50941,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48762,48730,48729,46681,46681,46649,46649,46649,44536,42456,42392,42391,40311,40311,40278,44406,40245,38197,40245,38164,34004,38100,44373,50645,54902,46484,48531,46451,52822,48630,57016,56951,50611,48497,48498,35852,31627,44239,46352,33706,33739,42158,35819,44239,35884,42126,31659,33772,27467,40046,31660,42126,33739,42158,35820,42158,40045,40045,31627,42158,35852,37932,31659,31628,31628,42157,37932,42125,35852,44205,40045,42125,40045,40045,37933,42160,57016,63322,59063,56950,52757,54869,56950,52757,54837,54869,59063,56918,50580,44339,33807,38066,35920,31759,38066,31759,33872,38033,31694,35953,33808,29615,33874,25486,27599,31760,25421,29712,29614,21260,25519,17067,14987,15020,6568,8714,8714,8551,8648,8681,37998,50545,27370,35952,52692,14955,6667,12971,44372,38033,12842,33837,29449,12809,44372,27600,25552,52855,63256,63256,63288,63289,63321,63321,61143,56950,57015,56885,46254,48465,56983,56918,44174,46287,54870,50579,31559,14756,14856,50710,50612,40079,23144,16935,40013,29546,23209,8582,8582,36},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50973,50973,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50908,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48762,48730,48730,46681,46681,46681,46649,46649,44536,44504,40343,40311,40279,38230,40278,48567,40277,42390,40277,40277,38294,40180,42292,50581,48565,44405,44339,44371,52822,54936,54904,48598,46418,48465,48498,35820,29546,42159,46352,44239,44239,46352,46352,46384,44272,46385,46352,46384,44271,48465,46352,48464,46352,46352,44272,48432,46352,46384,46384,46384,46352,48464,46352,48433,46384,44272,44239,48432,46351,48464,46352,48432,46351,46352,46319,50610,56950,58998,56917,56949,56917,54804,56917,56917,52691,56884,54771,48498,46418,33807,27599,31758,23308,29646,27598,31760,36017,31759,35952,29614,29647,27566,21228,27501,25421,21260,25486,29581,23341,23308,12842,12875,19213,17036,6536,8714,10762,8551,8648,8681,33837,50545,19017,38097,50546,10762,6667,12971,40080,33808,14923,31725,33839,25519,12875,33905,54903,61175,63255,63256,63256,63288,63256,59029,56949,57015,54804,48302,46287,37967,29581,21064,14758,23244,21196,12710,18983,18981,12677,37999,27401,31562,18950,10630,12677,16935,14855,8615,8583,38},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50973,50941,50973,50973,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48730,48730,48729,46681,46649,46649,46649,46649,44536,42456,40344,40311,40311,40278,40310,48599,42357,42358,40277,40277,46615,42260,36052,40212,48565,42325,42291,48498,46419,46485,46584,31923,42258,48465,48498,35820,29514,37933,44206,40012,37964,44271,31659,44239,33739,44239,29514,44238,33707,46319,37932,44270,37964,35820,23242,44206,31626,44206,31594,42093,29514,42158,29514,37868,27434,31627,27467,42125,31595,44270,35819,35787,31659,42125,31659,44173,40012,37932,31595,42092,37900,40012,37932,42092,37964,39979,31627,44204,40044,37899,33707,35754,33707,35754,31627,50545,50545,50545,50545,35852,31627,27434,16904,18983,31594,33707,35787,37835,44305,37934,23373,21196,12842,10762,4488,8714,8714,8551,8648,8649,31692,50512,31628,35952,46353,8714,4586,10858,27369,8517,10728,29644,37967,19050,8551,44403,56917,59029,59062,61110,61142,61142,61110,56818,56850,56850,54705,54673,44141,12710,10598,8549,8550,8550,8550,8517,16935,18980,10596,10596,12645,27369,14758,8517,8516,8517,8550,8517,8517,38},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,48762,50810,48762,48762,48762,48762,48730,48729,48729,46681,46649,46649,46649,46649,44536,42424,42391,40343,40311,38230,40310,44438,42390,44470,44470,42390,54904,34003,44405,48565,50645,44405,40178,44338,44339,44405,44536,29843,42291,50578,50578,44240,40046,42191,48465,44271,44271,48465,40111,48433,42191,46385,42159,48464,44239,48465,44304,48497,46384,46352,42224,48465,44272,48465,44272,48465,44304,48465,44304,48497,46385,46385,46352,48465,46385,48465,46385,46416,46385,48465,46385,50545,48497,48465,46417,48464,46417,48432,46416,48464,48465,46352,46384,48497,48497,48432,48465,48432,48465,46383,48465,52658,52658,52658,52626,35852,31627,27434,16903,21128,31626,31626,35819,39948,46353,29385,21130,10630,14955,10794,6568,8714,8714,8583,8649,8616,29612,48464,42127,29678,40047,6666,4586,10827,23209,8517,12808,27499,14758,8517,19146,54935,61175,61208,63256,63288,63288,63288,58997,56785,56785,48399,54738,44140,25224,10630,8616,8550,8517,8518,8583,14889,35850,19013,12677,12677,10565,25353,10598,8517,10564,12742,16935,8518,8518,36},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46649,46649,46649,46649,46617,44536,42424,42424,40343,40311,38230,40278,42390,40310,44438,46518,40278,40277,38164,44373,52790,46485,44404,42291,44306,42259,42325,44535,29843,42226,48465,48465,48498,48465,48498,48498,48497,48498,50578,50578,50611,50610,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50610,50578,50578,50610,50610,50578,50610,50610,50610,52658,52658,52658,50610,50578,52658,52658,52658,52658,50610,52659,50611,50610,50578,50578,50610,52658,50578,50578,50610,50578,50578,50578,50578,50610,50578,50545,50578,50545,50545,50545,37900,31627,27434,19016,25321,31626,31626,33739,39980,44272,33643,33677,14888,23308,12841,6568,10762,10794,8584,8583,6569,35919,50577,42062,27598,33742,6634,4554,8714,31694,10597,10597,19016,10565,25485,57048,61176,63256,63288,63288,63289,63288,56950,50578,27501,33773,40176,56949,33773,10696,8583,10762,6633,8616,8518,6601,10761,29513,12677,12645,14821,16868,29449,10598,8517,10564,18983,23241,10826,2505,32},
{48893,48893,48893,48893,50973,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46681,46649,46649,46649,46617,44536,42424,42424,42391,40310,38230,40278,42390,36117,40245,42357,29844,31925,42325,38165,46517,42356,46517,46419,48466,44339,46453,46584,27730,40145,46384,48433,48465,48465,48498,48498,48465,48498,48498,48498,50610,50610,50578,50578,50546,50546,50578,50578,50578,48498,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,50578,52658,50578,50610,50610,50610,52659,52691,52691,52690,52691,52658,52659,52659,52658,50578,50578,50610,50610,50610,50578,52658,52658,52658,52658,52658,52658,52658,52658,52658,50610,50577,50545,40013,31627,31594,35787,35754,33641,33642,35787,39980,44240,35788,42061,35722,35886,29612,12776,10762,10794,8616,8583,4521,35918,50512,40014,27533,25388,6602,4521,6633,42258,16838,14856,40112,35953,59128,61208,63256,63288,63288,63288,61175,50513,35757,21163,10762,16969,33774,27436,14857,8518,8681,10729,8617,8518,8583,8648,10729,27367,10565,10532,10533,29253,37834,10598,8517,10564,12677,16935,8681,4520,33},
{48893,48893,48893,48893,50941,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46681,46649,46649,46649,46649,44536,42424,42423,42391,40311,38198,40311,42358,36117,42358,42325,31892,33973,42325,40212,44405,48500,42324,44306,48466,44339,46517,48696,33938,50644,56690,56820,56885,56917,54836,54836,50610,52658,52658,52691,52658,52658,52658,52658,52658,52690,52658,52690,52658,52723,52658,52691,52626,54739,52658,54739,50577,52690,50576,52690,50545,52690,50544,54738,50545,54738,50544,54770,50544,52690,50512,52690,50512,54738,50480,52690,48431,52690,48431,52690,48399,52658,48399,52657,48399,52625,48399,52625,46286,52625,48398,52625,33674,25223,23143,29449,37803,29547,29546,33739,33739,31593,33641,35787,37900,44240,44272,50513,48400,48498,40112,14922,12874,12906,10696,8615,4521,33838,48464,39949,25453,17003,6569,6536,6536,23309,19081,50774,61143,61208,63256,63288,63288,63256,59030,50580,54804,20998,10598,8550,10729,10631,8517,8550,8617,8550,8517,8517,6536,8550,8485,8517,6536,12742,10597,10630,19015,10565,35852,12678,8517,10564,12742,23208,10760,6535,32},
{48893,48893,48893,50973,50941,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46681,46681,46649,46649,46617,44536,42424,42424,44471,40311,38198,40278,46616,40245,44438,40245,33972,29844,38164,36052,44437,46485,42324,40210,52626,44339,46517,50743,44403,56852,60267,58770,56917,56917,54804,54771,46351,50610,46318,52658,46319,52658,46318,52658,44238,50578,44238,52626,46318,52625,46319,52658,44238,52658,44238,52657,44238,52657,44238,52658,44238,52625,46318,50577,46318,50577,46351,50544,46319,50544,46351,50544,46351,50512,46383,50512,48432,48431,48432,48431,48464,48399,50512,48398,50544,48399,48496,46318,50544,46318,50545,42060,29514,23177,12710,25256,37835,31595,29547,35819,35787,33642,31626,33740,37965,46320,37933,48367,37769,37868,38032,42258,44305,42159,35886,27435,6602,33806,50545,42127,23340,10729,6536,6536,6536,10729,40145,44338,48499,46384,44272,50611,59030,48465,35821,17035,29613,10597,8550,10795,15021,10598,8550,8682,10828,10598,8518,6601,8714,8517,8517,6568,8682,16935,10630,12710,25288,8517,35820,12678,8517,10564,10565,14855,12775,8583,36},
{48893,48893,48893,48925,50973,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48762,48730,48730,48729,46681,46649,46649,46649,46649,44536,42424,42424,44503,40311,38198,40278,50776,40277,44437,40213,33972,27764,42357,38132,44372,40179,42291,46418,44306,42259,44437,48597,52789,52658,54674,52691,54771,52691,52690,52658,48497,52658,48497,52658,48464,52626,48497,52626,50512,50577,48497,50578,48497,50610,50545,52626,50545,52658,50578,52658,50577,52625,50577,50577,50577,50578,50578,52626,52626,52625,52658,52625,52658,50577,50577,50544,50577,52625,52658,52625,52658,50577,52658,50577,52657,50577,50577,50545,50577,50545,50577,50545,50545,50545,48432,25288,18983,14790,10597,37803,37771,29547,29514,35819,35787,31593,33674,35819,40012,48529,40013,48335,21160,42159,37966,12776,8550,27565,37966,23209,6602,31726,50577,44241,19147,25518,42258,42290,42290,44339,44370,44371,46451,46319,46318,44239,39948,10630,8583,14988,12842,8517,8550,10762,10696,8517,8550,10729,10631,8517,8550,8648,8550,8485,8550,8648,19049,21128,10597,10565,10564,10597,27466,10565,8517,10564,10565,23241,16936,8583,36},
{48893,48893,48893,48893,50973,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48730,48730,48729,46681,46649,46649,46649,46649,44536,42424,42456,44471,40311,40311,40311,52856,40277,38197,38133,34004,29844,38165,42291,46483,38099,31890,42258,44306,42259,44438,52823,59064,50545,50545,50545,50545,50545,50578,50578,50610,50610,52658,50610,50578,50578,52658,50578,50578,50578,50578,52658,52658,50610,52658,52658,52658,52658,52658,52658,52658,52658,52658,52658,52658,52658,52658,52658,52658,52658,52658,52690,52658,52658,52658,52657,52657,52690,52658,52658,52626,52658,52658,52658,50577,52625,52625,52626,52625,52625,52625,52657,52625,52625,50545,25257,12742,18983,10565,37771,37771,29547,29515,33740,35788,29546,29546,35819,37933,48497,40013,46286,21128,27305,14759,8649,8550,25420,37934,19017,6569,29613,50611,33875,19345,44404,52756,54837,54837,54837,54837,54837,54804,54738,54705,42061,37802,10630,8550,8550,8518,8518,8518,10630,10597,8517,8517,10565,8517,8517,8517,10565,8517,8517,8517,10565,21095,29480,14820,10564,10564,16903,27434,10597,8517,10565,8517,8582,2505,458,24},
{48893,48893,48893,50941,50941,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50908,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48730,48729,46681,46649,46649,46649,46649,44536,42424,42424,44471,40279,42391,40311,46583,38165,38165,42357,36052,31924,34004,46451,48499,38131,36051,46418,48466,42259,46518,21293,38065,50545,50545,50545,50545,50545,50578,46287,39980,48432,44205,48431,42157,48464,44238,48432,44237,48464,44205,48431,44238,48431,44205,48431,46286,46319,42126,46318,44173,46317,42125,48431,46318,44205,44206,37899,44206,44173,46286,37899,46286,35754,46318,42059,46318,44172,46350,42091,46318,42059,48398,42090,48398,42058,46318,37898,48398,44140,46318,39978,46253,37834,50545,40012,25257,18982,10597,35723,37771,29514,29482,33707,35787,29514,29546,35820,40013,48497,37933,46319,39850,18919,10631,12809,27631,35918,37966,12743,6569,29580,50677,38034,8616,48695,52756,52757,48466,33708,25289,25322,27467,46319,46286,25257,25224,8550,8550,8518,8518,8518,8517,10598,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8516,12676,16932,12741,12677,10597,21161,37933,12645,8517,10564,8517,8550,6535,6700,28},
{48893,48893,50941,50941,50941,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48762,48730,48730,46681,46681,46649,46649,46649,46649,44536,42424,42423,44471,40278,44471,42423,46551,40245,36117,36084,36052,38100,36084,42324,42291,33938,31890,42258,46418,44339,44438,6699,33937,50545,50545,50546,50578,50578,50578,42125,25321,46352,31626,46351,31658,46384,35819,46351,33771,48432,35787,46319,37933,46351,35819,46318,35819,40013,33740,44238,37900,44206,37933,37964,40045,42093,40078,31659,42158,39980,42158,33707,44239,29514,44271,39979,46351,42156,46351,37899,46351,35819,48432,42060,48432,40044,46351,40012,48431,37964,48399,40012,46286,40012,50545,42159,25257,12742,16933,35722,35658,27434,27434,29514,31594,31593,31594,35819,40012,48432,29612,48465,50480,25355,19115,36084,36050,37999,35886,10631,8715,29580,48465,48498,10728,29679,50676,52756,33742,10696,8518,8518,6503,8615,10630,8550,8550,8550,8518,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8485,27499,25255,14922,8517,23307,33674,12645,8517,10533,8517,8517,10564,6568,26},
{48893,48893,50941,50941,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48730,48729,46681,46681,46649,46649,46649,46617,44536,42456,42424,44471,40278,40310,42391,44471,36117,38197,40213,38132,38132,38132,38164,46451,31858,36018,44338,52594,44339,44372,4554,29777,52723,54771,52723,50578,52658,52691,52658,50545,52658,50545,52658,48498,52658,50577,50610,50545,50578,50577,50545,50578,50545,50578,50513,50578,48464,52658,48432,50610,48432,52658,46352,52690,46351,52690,44239,52690,46318,52690,46286,52658,46287,52625,46319,52625,46319,50545,44239,50512,46287,50512,46351,48432,46319,48399,48400,46351,48432,46286,48432,46286,48464,50512,33707,19082,14855,14791,31530,37836,37967,56918,50611,50644,50611,52692,52724,52757,54837,52757,56917,52724,48564,40080,33938,36051,37999,35854,14987,6667,27500,48432,50545,29778,48597,50644,52756,25389,8747,6667,6634,6634,8616,8583,8550,8550,8550,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,8484,8485,10663,8518,19147,31562,12645,8517,8517,6502,6535,8550,6568,15},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,48794,50875,50875,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48730,48697,48697,46649,46617,46617,46617,44569,44536,42424,40343,40311,42359,38198,38198,38197,44438,36052,38165,36052,36051,33971,31891,31891,46419,19246,36051,17133,14955,15053,12940,6764,21294,29614,33840,29613,25485,27533,44273,40079,33805,40080,31726,37966,35854,37934,35854,37934,33773,35886,33805,33773,37999,33772,40080,31628,37966,33708,37966,29612,37966,27500,38031,29548,40079,29548,40079,29548,40014,31660,37934,27499,35886,25354,31726,25355,29515,27500,29483,27435,27434,23274,27435,25354,25322,27499,31660,33839,31694,21196,21196,25388,21195,21228,21228,14923,12843,19083,19083,17003,21195,29580,58998,59030,61078,61111,61111,61110,61110,61111,61078,61110,61078,58997,52692,25421,8681,25420,35853,33807,6667,25419,46351,48465,27600,52823,50611,50579,14987,6699,6667,4586,4586,8616,8551,8550,8550,8550,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,10565,8518,8518,14922,14758,8517,8517,8615,4652,655,2539,6536,16},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50875,50875,50875,50843,50842,42424,46681,46649,48762,50810,50810,48762,48762,40343,48730,46617,44536,48729,42391,48697,46617,44472,38230,31891,33972,31924,31891,31891,29811,29843,29811,29778,27698,27698,27666,25585,23505,23505,23504,23472,23440,23440,23407,23408,21327,21326,23407,19278,21326,19246,17198,19279,21326,23374,21294,21294,21327,21326,21326,21326,21294,21294,21326,21294,17165,17133,17133,19180,19180,19116,17035,17068,19148,23373,25420,23339,23308,23276,23275,19147,14921,14986,17034,14954,17067,14953,12874,12841,16969,14922,14889,14889,12808,10728,12809,10728,8648,8648,8583,8680,8615,8583,8615,8550,8615,8583,8583,12808,21163,25355,29484,14988,17100,21260,17101,15020,19213,17101,14988,21293,19213,17068,27566,35886,52659,29450,23144,42161,35789,27435,38000,31563,16903,31693,33644,14790,27468,35954,29712,31758,37966,38000,12940,25387,46351,48465,21262,14955,25388,10697,6601,6601,6602,6602,4586,8583,8550,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,8485,8518,8517,8583,8517,8517,6600,8583,4619,655,2605,4718,19},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,52988,55068,55068,55068,55068,55068,55068,55068,55068,55068,55068,57116,55068,57116,57116,57148,57148,57148,57148,57148,57148,57116,59196,57115,57083,55035,50777,48696,46616,46648,38230,29877,21490,21458,19377,13038,17199,15119,13005,15119,10893,13038,13038,10893,8812,6765,2603,2635,2635,2538,4684,4586,2538,4684,4553,4683,4618,4586,4683,2538,2635,4586,4553,2604,4521,4619,6699,4521,2604,10860,6731,15118,12972,4619,6568,4553,4554,4488,2539,4520,4521,2506,6536,2505,4488,4488,4521,6504,4521,6536,6535,4521,6503,4488,6535,6536,6633,8615,8616,6567,8583,6600,6567,6568,6567,6535,6600,8583,8615,8615,8583,8616,8583,8615,8615,8583,8615,8583,8615,8615,8583,8615,8583,8615,10696,8550,8615,8615,27664,52725,48500,31792,38000,17102,38033,50546,44274,29582,50546,44274,27501,48434,42160,27534,46386,44208,23276,12744,10664,14923,12842,10696,12874,12810,10728,12842,12810,10696,17068,31859,36018,37999,35885,42193,29647,25387,46384,50546,16938,8551,25453,21227,8649,6601,6601,6569,4521,6536,8550,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,10565,10565,10565,10565,8518,8517,8518,8584,15152,15086,8615,4586,623,524,4751,20},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50908,50908,50907,50907,50907,50907,50907,50907,50907,50907,52923,52923,52923,50875,44537,36118,29813,34005,36117,40310,44502,42389,42356,40276,40243,42324,40276,40244,40308,40276,42356,40276,42324,42356,42356,44404,44436,44404,44404,44404,44404,40243,29712,42292,44404,44404,44436,44404,44436,44436,44436,13038,4684,4684,4652,4619,4651,6699,6700,6699,8747,4619,4619,4619,4619,4587,4587,4586,4586,4586,4554,4554,4554,4553,4553,4521,4553,4521,4521,4521,6536,6536,6536,6568,10826,12906,8713,10793,12841,10793,10793,8680,10825,10761,12873,10760,12841,12873,10696,10728,8680,12841,10760,10728,10728,10760,10695,10760,12808,10728,10696,10663,10696,10760,8615,12875,25453,61079,50547,31792,35919,38098,33807,33806,40145,31759,35919,40178,33807,38032,42225,35887,38000,40079,54870,52757,46419,31694,42258,46386,31694,40145,50676,35887,35952,52725,38000,31827,36018,38000,35853,42193,38000,31725,44272,37869,10894,8616,36017,48434,10763,6699,6667,4619,4586,6601,8518,8517,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,8517,8485,10564,10564,10565,10565,10564,10565,8518,8517,6699,19345,14922,8517,8615,6536,591,492,4488,6},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,31958,23604,23572,17199,42454,40438,50807,42421,40275,48695,25453,46581,42324,29646,52855,25453,40211,40242,27598,52887,27533,36050,42356,23308,48662,29678,27631,40276,11021,44501,33904,27598,48726,21292,40275,36017,17099,15183,6569,6699,4683,4520,2669,4553,4586,4684,6568,4684,6601,4553,2636,6536,2571,4553,6536,2571,6503,4553,4521,6503,2538,6503,4488,2473,6502,4553,6503,6535,6536,6502,8745,10696,6535,8778,8550,10826,8712,6535,10890,8550,12905,10792,8550,10890,8550,8680,10792,8550,12937,8550,8615,8712,8518,8713,8582,8582,10760,8518,17099,8648,10696,8615,23308,59030,44241,33873,38064,59063,46287,54707,50513,46352,52626,48465,44272,50545,50514,46385,52658,35820,54870,48432,40014,44240,37933,35723,37966,39948,35820,46353,37900,35787,46354,46485,48597,40112,40046,44306,46419,42159,46352,37966,10664,8517,35952,42128,8715,6667,6634,4586,4554,6601,8517,8517,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,10565,10565,8517,8485,10564,10564,8517,10565,10564,10564,6601,6634,15053,12678,8517,8517,8615,6536,558,492,4488,20},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50908,50908,50908,50907,50908,50907,50907,50907,50907,50907,50875,50875,50875,48794,29877,23572,21491,12940,44535,42486,48759,40309,29776,48694,17099,42421,40307,29872,50807,19212,36082,38162,17132,50807,21325,31889,40307,15051,46614,27663,23503,36115,11054,42453,29776,21422,48694,17131,38195,44501,19277,23536,6601,6731,8844,8713,8909,8746,6731,8876,8746,6829,10826,8746,8876,10761,8843,10826,10761,6796,6568,4586,4553,6536,2538,6536,4521,4521,6535,4553,6536,6568,4521,6535,6698,6600,6600,6698,6535,8714,8680,6568,8778,6535,8713,8713,6535,10858,6535,8680,8680,6503,8745,6535,8648,8681,6535,8649,6535,6567,8680,6503,27599,6568,8616,8550,21227,59030,37968,29615,35919,61111,31594,31660,42192,40078,31628,42192,40046,31628,42160,44272,21096,33773,54837,42126,25387,46385,33771,33870,46385,33675,21161,44305,33740,21161,48466,44238,42192,33838,42225,44306,46451,44271,46352,35952,33838,33740,42192,35854,6634,6634,6602,6602,4554,6568,8517,8517,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,8517,10565,10565,8517,10565,10564,10564,8517,10565,10564,10564,6503,8517,10565,10564,8518,8517,8616,6535,557,2506,6502,27},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,44536,50842,44568,42456,42423,38229,46583,38229,50743,48629,46549,50678,46550,48629,48629,46549,50709,46549,48629,48629,48597,50677,48629,48629,50677,48629,50677,48629,42324,36018,42356,48629,48629,48629,48597,48597,48596,48564,46549,46582,23471,44404,50677,50677,50710,50677,50678,50677,50677,50677,50677,50677,50677,50677,50677,50677,48596,42323,12973,8813,8813,6732,6732,6732,6699,4684,6699,6699,4652,6667,4651,4619,4619,4651,4586,4619,4619,6634,4651,6634,4586,4586,6601,4618,4586,4553,4586,4521,4586,4553,6569,2538,4520,4553,4553,6568,4586,6568,6568,4553,27566,23439,42258,42258,46483,59030,29647,27535,35919,59063,27369,23242,33774,40046,21097,33741,40079,18984,31693,42192,21129,42160,54837,56949,54805,44240,42159,54772,46320,42127,54739,46352,42126,54771,48433,44206,56884,31695,44371,44306,46419,44271,46352,44404,52691,44142,37999,29549,6602,6569,6569,6569,6569,6568,8517,8517,8517,8517,8517,8517,8517,8485,8517,8517,8517,10565,8485,10532,8517,10533,10565,10565,10565,10565,10564,10565,10565,10564,10564,6503,10793,21193,19048,8517,8517,8615,6535,492,426,8518,23},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,48697,50776,48663,50743,48663,50743,48695,42518,48759,40243,31889,50775,27566,44436,38130,25518,52855,23372,40276,38162,21260,50807,23373,33937,42356,19147,48662,29776,19277,27729,17034,42388,33904,25486,50775,19147,40243,33937,29679,46582,15116,25616,46614,19179,40308,33936,19276,50807,21259,34002,40242,17066,46614,25485,25550,44501,17098,40372,21325,4586,8975,6536,4684,4684,4487,2766,8648,6634,4749,6536,2669,6633,4520,2701,6503,2603,4618,6503,2669,6535,4585,2603,6503,587,6568,4488,2635,8550,2570,6601,6503,2636,6502,4553,4585,8518,2571,6535,6536,2603,29744,40146,59031,59031,59031,58998,25389,25487,33807,59063,29482,23273,40047,40079,29580,40079,40079,23273,35918,42224,19016,37934,52757,44239,19048,33773,31594,16871,31660,35819,16870,29579,37932,16870,29546,44174,21095,31727,46451,44338,46451,44272,44337,44372,50611,14791,31759,23211,6569,6536,6536,6536,6536,6536,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8485,8517,8517,8485,10532,8517,8485,10565,10564,8517,10564,10565,10565,10564,6502,458,10956,48498,40014,8517,8518,12776,8648,2474,4489,6470,26},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50908,50908,50908,50907,50908,50907,50907,50907,50907,50907,50875,48696,54970,50776,46550,46550,46550,48663,48695,42486,48727,34002,29874,48727,21454,42356,33969,23406,50807,19212,36082,36114,17196,48759,21325,31857,42388,17132,46582,27631,12972,27729,17164,42388,31824,23503,48694,19277,38163,36147,23503,48694,23632,23567,46581,15019,38195,33904,21292,48726,19179,31889,40210,19147,46581,27598,27630,44468,19179,42420,23373,6601,8942,6568,4684,6732,4521,4782,6601,4586,4716,4520,4684,6634,4553,4716,6568,4619,4618,6536,2636,6568,4586,4618,6568,2571,6569,4520,2603,6535,4586,4553,4488,2571,6536,4553,4553,6503,2538,6568,6568,6666,38066,33807,59031,59031,59031,54805,21230,21229,29613,59031,35853,35918,42224,44305,42192,44305,44272,37999,44272,46385,29580,35853,52756,52691,38032,40111,42094,37966,40078,42126,37966,40111,44239,40078,42093,46287,44304,44337,44338,42258,46451,44272,46385,35986,50676,46548,42191,16937,6536,6503,6503,6535,6535,6536,21227,10564,8517,8583,8681,8517,8517,6535,8616,8517,10565,6535,8583,8485,10565,8550,8550,10564,10565,8518,8518,10564,10565,8518,4488,458,6633,25387,21097,8518,15118,38099,8550,6503,6503,6470,32},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50874,52889,54969,52856,50776,50776,50776,50808,50776,42357,50775,50710,50710,50710,48662,50710,50710,50742,50710,50710,50710,50710,50710,50709,50710,50710,50709,50710,50677,42324,21425,29745,42291,42324,42323,42291,40179,42291,42291,31857,33971,50743,50710,50710,50709,52758,50710,50709,50678,50742,50709,50710,50709,50677,50709,50677,50645,50709,48564,48597,21326,8845,6797,6732,4717,4716,6700,4717,6700,4651,4684,4619,2604,4651,4619,4684,4619,4619,4619,4586,4651,4586,4619,4619,4553,2571,6634,4554,2571,6569,4586,4553,4521,2570,6569,4553,4554,6536,2538,6568,6601,27664,48531,46418,59031,59031,59031,50579,17102,14988,25421,56951,52627,58965,54739,52659,56917,54772,52658,56884,54739,50578,54771,48433,52756,56884,52691,48400,44206,44239,44271,46286,44206,44240,46319,44206,46319,48367,46318,46352,44306,40178,46484,44272,40079,44437,50676,59030,52692,10696,8583,6503,6503,6503,6503,4521,21325,10564,8516,8583,8714,8517,8484,6535,8649,8517,10564,6535,6568,10564,10564,6503,8551,10564,10565,8518,8518,10565,10565,8518,4488,8550,10661,8517,8517,8517,23537,42259,14856,6536,6503,6503,31},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50908,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50842,52889,52889,50776,48663,50711,50711,50743,48728,44599,48727,40244,31954,48727,21292,42356,38195,27598,52887,21260,38162,40211,21228,50807,23437,33969,42356,19147,48662,23438,10925,19409,10761,27665,19310,8811,25682,10794,23536,19440,19310,46646,27630,27631,48694,17098,40275,35984,21260,50839,19179,33969,40275,17066,46614,27533,27566,46581,14954,38163,15019,4553,9008,4585,4717,6764,4520,4815,6634,4618,6797,4520,2701,6666,4553,4782,4488,2603,4619,6536,2701,6536,4585,2636,6503,2636,6601,4520,2668,6503,2570,4586,6503,2668,6535,4619,4618,6502,2603,6536,6536,4683,35951,40144,59031,59031,59031,40082,15021,14956,23308,56983,39949,35788,44240,44240,40014,46352,44240,31627,40046,46385,25289,29515,50676,50545,33772,42159,42060,21160,35918,44206,21160,33741,46319,21096,31692,48400,33705,37965,44338,38033,46484,44305,27533,48598,52692,54772,50580,23373,29678,29679,31759,31791,31791,31824,38097,31627,12678,8583,10664,8550,8550,8551,8583,8517,8517,8550,8550,8517,8517,8517,8517,8517,10565,8517,8517,10565,10565,8517,10565,16837,14756,8517,8517,8779,29843,35985,10630,6569,12809,14889,64},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50940,50908,50908,50908,50907,50907,50907,50907,50907,50875,50842,38164,54970,50776,46550,46550,48598,48663,48695,42486,48727,38228,34034,48727,19213,40276,36050,21325,50807,21455,36114,38162,17099,48727,25550,31824,42356,17132,46549,19245,10892,19376,10826,31890,23535,8876,29843,12874,25616,15182,19278,46582,27630,25518,46613,17099,38163,33904,21325,48694,21292,33937,40242,17099,44501,27566,27598,46516,19147,33937,10859,6634,8943,2636,4749,4684,4553,4782,4586,4587,6765,4553,2668,4619,4554,4749,4521,4619,4619,6569,2636,6569,4554,2603,6568,2603,4553,4520,2603,6536,2538,4586,4521,4651,6568,6634,6634,6536,4586,6568,6568,17133,40211,52790,59031,59031,59031,33775,12875,10730,19115,56950,29482,21161,31660,42128,14855,31661,42160,14822,29579,44272,12677,29547,50644,58965,54837,48530,48464,54804,50579,50512,54804,46287,48432,54772,48530,48497,56883,52723,46386,31759,46484,44305,29614,48598,50676,54706,52691,52756,52724,54804,54837,54869,54869,54869,54837,52593,16871,10663,10631,12743,10630,8582,8550,8550,8550,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10564,10532,8517,8518,19246,38131,38066,12809,19408,29613,27567,182},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50842,50744,52889,52856,50808,52857,50808,52856,50776,42357,50775,50742,50742,50710,50742,50710,50742,50743,50710,50710,50742,50710,52790,50742,50710,50742,50742,50742,48629,38131,25552,33971,46484,46517,46484,44436,46549,44436,42324,17231,33939,50711,46484,50677,50710,50677,50710,50709,50677,48629,48597,50677,46452,38065,40178,38033,38065,38098,35985,29680,8845,8813,6830,6732,6765,6764,4652,4717,4652,4652,4684,4619,4684,4651,4619,4684,4586,2604,4619,6634,4651,4554,4586,4619,4553,4619,6633,4553,2571,6568,4586,2538,2538,2571,6601,8714,8747,10761,6667,6601,6568,6699,42291,46418,59031,59031,59031,29614,23438,25454,29613,56950,46320,44305,44305,46385,42160,44305,46385,40079,44273,44305,42126,35787,50611,52690,37997,52692,46286,27402,50611,48431,23175,42159,50512,29547,29516,29418,21031,21032,42226,27534,46484,42192,29613,50742,54772,48434,31760,35952,52724,50644,52692,52725,52724,52724,52724,54673,16871,10663,10631,16904,10631,8550,8550,8550,8550,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10564,8517,8518,19212,38131,38066,29678,44371,33741,27599,190},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50907,50842,50743,52889,50776,48631,48630,48631,50711,48695,42551,50808,42454,34067,48727,23405,42388,36082,23405,50840,21292,38130,40243,19244,50807,25648,31825,44533,29775,44501,21325,10892,29875,15019,40276,31824,21356,50807,17099,27696,25683,33971,42356,17132,25518,48694,19179,40275,36017,17067,27730,19147,31857,31857,8681,19376,10827,8747,19376,6601,6862,6732,4521,8975,6601,6732,13070,4520,4814,6633,2571,4749,6536,2668,10859,6568,2734,6568,2603,4651,6503,2669,6568,4553,2636,6503,2604,6601,4488,2668,6502,2538,523,457,588,8616,10762,8747,10663,4651,6568,6535,25617,46517,38032,59031,59031,59031,52724,56917,48497,48529,56950,50512,42060,46320,48497,39980,44272,50546,37899,42158,48497,29449,29514,48531,52691,35919,52692,48465,35853,50611,52658,35918,48498,52690,33840,14922,12710,12678,10630,38098,31726,46484,44305,44272,52790,35888,36084,42258,35920,48498,17003,14955,12907,12875,12875,14956,19017,10663,8583,10631,12775,12711,8550,8550,8550,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8517,8518,17099,38131,38099,33904,42258,33806,27534,191},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50842,50743,55002,50776,46550,46550,46583,48663,48695,42454,48727,40309,34002,48695,21325,40276,33969,23437,50775,19276,36082,40243,21357,48727,23438,31825,42388,19212,36115,19277,13005,40308,19148,40275,31889,23502,48694,21292,23503,23570,31892,38164,12972,23470,48630,19212,38163,36049,15051,19376,10826,17198,19246,6634,11023,8779,8747,10991,4553,6797,6764,4586,6862,6601,4684,6764,6601,6797,4586,4618,4716,4553,2636,6667,4553,4717,6601,4619,4651,4521,4684,4553,4554,4651,6568,2571,4554,4553,2603,6536,4586,4586,4521,2571,6536,4521,4554,6536,4586,6569,6568,8714,37968,37999,59031,59031,59031,50578,56949,50609,46416,56950,54836,52723,50644,54837,52724,52724,56885,54772,52724,52756,54837,48433,52756,61111,61143,61111,61110,61143,61110,61110,61111,61110,61110,61143,48531,44337,42225,40112,42258,35919,46484,46418,46351,25290,14923,27501,21065,25453,44241,10795,8747,8747,8747,8715,8747,10664,10631,8583,8550,10631,12711,8550,8550,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,8485,8550,8518,15052,38099,38131,25452,35887,33806,27468,190},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,52988,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,52922,57114,52888,52857,52857,52857,52856,50776,48695,50776,50742,50710,50742,50742,50742,50742,50710,50742,50677,50710,50710,50677,52790,50710,50710,50710,48629,44404,38099,46485,50742,48629,50710,48629,48564,50742,46484,40211,15151,25553,38164,21359,46517,50710,48564,50677,50677,40146,17231,33938,36082,17198,13005,10958,10958,8845,10958,8812,6797,4749,4716,4749,4651,6764,6764,6699,4717,6699,4619,4684,4586,4652,4651,4618,2636,4587,4619,4651,4554,4652,4586,4586,4651,6569,2571,4586,4553,2571,6568,4586,4586,4553,2571,6568,4521,4586,6536,4586,6568,6568,6699,25421,33839,59031,59031,59031,50546,59030,52690,46449,61176,61143,56885,59030,63256,59062,59063,63224,56852,56885,63223,54739,50578,54837,61045,56851,59029,63158,54706,56885,63191,54706,54772,63223,52593,52658,63191,54673,52593,46451,40178,46484,48563,48432,14824,10795,10663,10663,23340,37935,8682,8649,6634,6634,6634,8715,8616,8551,8550,8550,8550,8550,8550,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,10533,6503,8681,8518,14987,36019,40179,21194,23275,31726,27501,188},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,55002,52889,54969,54969,54969,54936,48695,44599,50807,42389,31921,50807,21325,42421,36146,23437,50840,21357,36082,40275,27630,50807,23437,31857,42420,21388,40341,23535,25583,48694,19180,42355,33937,21389,50807,17066,17263,11022,15053,27730,10924,23405,48727,19179,38163,36017,10794,19409,17132,27664,15150,4553,8975,6763,4619,15216,6568,6829,4749,2538,8975,4553,6764,6797,4520,2766,4586,4586,4782,4488,2636,4618,4520,2734,4520,4619,6699,6503,4717,6601,4553,2668,6503,2604,4586,6535,621,6503,2538,523,2472,588,6535,6568,2570,8518,4619,6568,6536,23504,44404,48564,59031,59031,59031,50579,61175,52723,46514,63289,56884,33740,46450,63224,48594,50708,63256,27433,35918,65337,27401,40112,54804,50481,16870,35952,54739,16838,33774,58965,23208,29580,61110,21030,21161,61111,37867,31691,46419,42193,46484,44305,44239,10729,40342,48564,44404,52823,29549,8649,6569,6601,8617,8617,8649,8583,8550,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,8517,8518,8550,6470,6535,23505,38131,21129,19082,27598,25388,192},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,52954,54969,54969,57049,57049,54969,48695,40373,48727,42389,34002,48695,21422,44501,40340,25518,50775,29808,38227,38195,25518,48694,25680,31954,44468,25614,36083,29841,31856,46614,34066,44501,36017,25583,48662,23470,13070,10990,8877,19410,10892,23470,48661,19179,36050,36017,10826,17296,21422,17198,10958,6634,8910,6732,4651,8943,4586,6797,4781,2668,6862,6634,4684,4716,4554,2701,4618,4586,4717,6601,4684,4619,4586,4717,4553,4619,4619,4521,2636,4586,4554,4652,6568,4619,4586,4521,2571,4520,4554,2538,2505,2571,4520,4521,4586,6536,4586,6569,8616,12745,31726,40048,59031,59030,59031,48498,63288,50674,54869,65402,59030,42256,50676,63288,48595,52789,65370,44336,44370,65402,37901,44338,54804,58998,52789,56950,61143,54870,56951,61176,59063,56983,61176,56950,56950,61143,63223,61142,50612,40145,46451,40111,37965,8616,40277,59030,58997,44272,23242,8583,8583,8583,8584,8616,8616,8551,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8485,8485,8485,10565,8485,8485,8484,10565,10565,10565,8517,10565,8518,525,2506,17199,38099,19081,27467,31759,25421,189},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,52922,52890,52922,52922,52922,54969,50775,52856,52822,52791,52790,52823,52790,52790,52823,52790,52822,52822,52790,52822,52790,52822,52822,52790,46517,36018,50710,52790,52790,52790,50742,52790,52790,50710,48598,23505,15183,17264,23570,17199,38066,40178,33872,31792,38098,19246,15151,42226,21359,10958,10861,10926,8845,8845,6830,4684,4749,6764,6700,6797,6699,4684,6829,6699,4717,6699,6667,4716,6634,4652,4651,6634,2669,4586,4619,4651,6601,2604,4586,4586,4652,6569,4619,4586,6569,2604,6568,4554,2571,4553,2571,6568,4553,4586,6536,4586,6569,6536,4651,29744,40146,56951,58998,59030,56950,63224,54804,48531,65435,56917,33772,48532,61143,37933,44370,63288,37964,42257,65402,31627,46451,54804,61143,63224,63224,63257,63256,63257,63289,63289,63257,63257,63256,63256,63256,65304,63256,52692,38032,46451,40144,33707,10696,40243,48564,46451,38097,29613,27566,27598,29679,29679,29679,29711,29646,12743,10663,10631,10631,10663,8583,8550,10598,10598,8550,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8518,6568,6535,17231,33906,19147,27500,35920,25356,189},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50843,50842,52857,54936,54968,54936,54936,54904,54936,54935,46581,31791,55000,38129,44468,48661,42386,54967,36049,40275,44435,29678,48662,40145,40210,52822,31823,46581,38161,36049,52887,34000,38195,31986,10925,15281,12972,21325,48727,15051,27697,34034,4651,15249,10826,6764,10991,4553,8975,6732,4618,9007,4520,4781,4716,4521,6927,2602,2668,6926,6600,2734,6666,6601,4814,4552,2669,4651,6536,2734,4521,4619,4684,6503,2669,4585,2505,2669,6535,587,4586,6535,2669,6535,4586,555,2505,588,6568,4520,2603,8550,4619,6601,6535,15150,40112,48531,44274,40081,40081,54805,61111,59030,54869,65435,63256,56982,59128,63289,56982,59128,65369,56982,59096,65402,59062,61143,63289,65336,65336,65402,65369,65369,65401,65369,65336,65369,65369,65336,65369,65336,63288,65368,50612,40113,46451,42192,27434,29711,52756,48564,52725,46386,40145,54837,54869,56950,56950,56950,56950,50611,21097,10630,8582,8517,6568,10991,10762,8517,8517,8550,8550,8517,8517,10565,8517,10564,10565,8517,8517,10564,8517,8517,8517,10565,10565,8517,6503,8518,15184,44470,44305,29547,35920,27468,190},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50842,52889,54936,54968,54936,54936,54936,54968,54936,52822,50742,52855,50742,52823,52822,52855,52822,52823,52823,46516,44469,52822,52822,52855,52822,52855,52822,50710,52823,50709,46517,50710,31889,17232,19410,13005,25551,48662,21357,27664,33969,4716,13135,8811,6829,8910,4619,8910,6732,6667,8911,6634,6765,6764,4586,6862,4618,4652,4749,6634,4717,6666,6634,6829,6698,4684,4651,4553,2637,4586,4619,4651,6601,4652,6634,4586,4652,4553,2571,4586,4521,4619,6569,4554,4586,4553,2570,4521,4521,4586,6568,4554,6601,6569,19246,38000,46451,23342,21262,21262,48532,61111,61143,61143,65336,65369,65369,65369,65369,65369,65369,65336,65336,65369,65336,65336,65304,65304,65304,65336,65336,65336,65336,65303,65271,65271,65303,65271,65270,63223,63190,63223,63223,56919,46418,46451,37934,31757,52756,50642,38064,29614,19082,25486,52757,54837,54869,56918,56918,56950,52692,35821,12645,8550,8484,6568,15185,12875,8516,10565,10564,8517,10565,10565,10564,8517,10564,10565,8517,8517,10564,8517,6503,8517,10565,10565,8517,6503,6502,15151,42324,37933,23306,33839,33872,204},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50874,50843,50842,50841,52890,52889,52889,50809,50809,57017,54968,57016,54935,54936,54936,54935,54936,54936,54936,54903,50710,52855,54903,54935,54936,54903,54936,54903,54903,52855,52823,52855,54903,52823,42389,23538,21392,35986,35986,27633,23440,27632,13039,15118,33938,15118,8878,8910,8878,8878,8878,8846,8846,8845,8845,6797,6797,6765,6765,6765,6765,6765,6732,6732,6732,6732,6732,6700,4684,6700,6700,6700,4651,4651,6667,6699,6699,4619,4619,4619,4619,4619,4619,4619,6667,4587,4619,4586,4618,4586,4586,4586,4586,4586,4586,6634,33872,44305,31793,33873,33873,52758,61176,63224,61175,65369,65402,65402,65402,65402,65402,65402,65370,65370,65402,65369,65369,65369,65369,65369,65369,65369,65369,65370,65369,65369,65369,65369,65336,65336,65336,63288,65336,65336,52692,44306,46484,40046,25482,36013,38127,44371,40179,19148,31759,54837,54837,56917,56950,56950,56950,56917,48400,10532,8550,8485,6568,17330,12908,8516,8517,10564,8517,10564,10565,10564,8517,10564,8517,8517,8517,10564,6502,2506,8518,10565,10565,8517,8517,8518,17264,50777,37933,14823,27500,33839,204},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,48893,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50908,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,50842,50842,50842,50810,54969,54936,54936,54936,54936,54936,54903,54936,54936,54936,54936,54904,54936,54903,54936,54936,54903,54936,54903,54903,54903,54903,54903,54903,54903,42292,21392,17264,17232,17231,17232,17232,15151,15152,15151,15119,17199,13039,11023,13071,13039,13039,10958,10958,13039,10926,10958,10958,8877,8878,6797,8877,8845,8845,8877,8844,8845,8844,8812,8845,8812,8845,8812,6764,8812,8780,8812,8812,6732,8780,6731,6732,6731,6699,6731,6699,6666,4586,6569,6601,6601,6569,4553,6568,6569,6569,6569,31824,48564,63289,65369,65402,65402,65402,65369,65337,65434,46385,52757,65467,54738,50643,65435,50545,48499,65435,56884,50676,65435,61176,50676,63322,63256,52756,63289,63255,42093,59096,63288,46352,59063,65369,44206,54870,65368,46385,44306,46484,40079,27595,42287,40208,42257,42258,44501,52757,52756,54837,54837,54869,56917,56917,54869,50513,10564,8583,8485,6535,11057,12908,8516,8517,8517,8550,10564,10565,10565,8518,10564,8517,8517,8550,10564,8518,4521,8518,10565,10564,8517,8517,8518,25651,63388,44142,12710,17033,33839,200},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,50941,50941,50941,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50940,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,50842,50842,50842,50810,52889,52889,52857,52857,52857,52857,50809,50809,50809,50777,50776,50776,50776,50776,50776,50744,48664,48664,48631,46583,46551,46518,46518,48599,46518,50743,48662,44470,38196,46550,46550,46549,42357,42389,42389,38196,46550,44502,40308,38196,38196,40244,38196,38196,38196,40244,40244,38196,38196,38163,38196,38195,38195,38195,40243,40243,40244,40244,40276,40275,40276,40276,42356,40276,42323,42356,42356,42356,42356,42356,42356,42356,44436,42388,44436,42290,17068,4554,6503,6536,6536,6503,6568,6503,6535,6535,6535,31792,52789,65369,65369,65434,65434,65434,65402,61176,65402,63289,61176,61175,63321,61176,61143,63321,61176,61143,65369,61176,59030,65369,63288,59029,63256,63289,61078,63223,65369,59030,63191,65368,58997,61110,65336,56819,58965,50612,42225,46484,40079,16999,16999,25484,42290,48695,59097,52725,52724,54837,54869,54869,56918,56918,56917,52593,12776,8681,8484,6503,10959,10827,10532,8517,6536,8583,10564,8517,6503,8550,10564,8518,6503,8550,10564,8518,4489,6634,8616,10564,8517,8517,6502,27763,57048,29417,12710,16969,29679,199},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,48893,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48762,48762,48730,48730,46681,46681,46649,46649,46649,46649,46617,44536,44504,42391,40311,40311,40278,38198,44439,38197,48630,48630,48630,48630,48598,50710,50678,48598,50678,52757,50677,50710,50743,50744,50743,50776,48695,48695,50743,50743,50743,50743,48597,46484,46450,44369,44305,44338,46386,46386,50611,50611,56885,48466,50513,48466,50645,42259,48598,46484,48565,50613,48564,48596,48564,46515,46515,40178,31759,35855,33807,17133,6699,6634,6634,4585,4586,6633,6601,6633,6601,6601,33840,48531,65369,65402,65434,65434,65434,65434,65402,65369,46352,44240,50609,44238,42191,50609,46319,42158,50578,54836,50611,50611,59029,52756,48497,56916,57046,52723,50577,44239,48465,52722,50610,48530,52755,56982,48497,46352,38032,44306,46451,40079,16967,14919,25452,42225,50710,40048,27533,52724,54837,54837,54869,54869,54869,54869,54706,21196,17068,10565,8550,10795,10761,10532,8550,8650,8584,10564,8550,6568,8551,10564,8518,8583,8518,10564,6503,4686,15218,8616,10564,8517,8517,6502,34069,56983,18951,18983,12743,27598,203},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,48893,50941,50941,50941,50941,50940,50940,50940,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50875,50875,50875,50842,50842,50842,50842,48794,50842,48794,48762,48762,48762,48762,48762,48762,48762,48730,48730,46681,46681,46681,46649,46649,46649,46617,44536,42456,42424,40311,40279,40278,36150,40278,36117,38165,36116,36084,40244,36084,40213,40213,31924,44437,50546,44339,46517,46616,46617,46617,44569,44569,44569,44569,44601,44569,44568,38229,38196,35919,21324,25452,33806,35854,35820,37933,42158,46288,37967,46254,40079,54871,46518,48630,38129,46548,46483,44467,44500,44468,40275,42324,23472,13039,15086,40276,48629,44404,42324,42356,42323,40243,42323,42323,42323,42323,42323,44338,46484,65369,65370,65434,65434,65434,65434,63289,61111,42160,42193,46320,42159,40079,44271,42159,42224,50578,50610,48531,52659,52691,46418,50578,52691,50675,50643,50578,44337,50611,52691,50611,50611,52691,48531,50611,48465,42257,48498,48563,48498,38031,38031,40144,23307,19179,12776,21260,52757,54837,54869,56950,56950,56950,56950,52593,10564,12744,12645,8517,8518,8551,8485,10565,8517,8518,10564,8517,8517,8517,10565,10565,10564,8517,10564,6503,6865,15153,8583,10564,10564,10565,6535,40342,46321,10597,19015,10598,25485,201},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,48893,50941,50941,50941,50941,50941,50940,50940,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50843,50875,50875,50842,50842,50842,50842,50842,48794,48794,48762,48762,48762,48762,48762,48762,48762,48730,48729,46681,46681,46681,46649,46649,46649,46649,44536,44504,42391,40311,40279,38198,38198,40278,38229,42357,38164,36116,40244,36084,42357,40212,31924,42324,42226,42227,42357,44536,44569,44569,44569,44569,44569,44569,44569,44569,40342,27663,27599,35920,25517,29678,33806,35885,40012,29611,31692,52658,54577,50383,37999,52790,46452,50743,42226,44436,44405,44372,46420,46452,46452,46452,48597,46484,31857,31759,29646,29614,27566,27566,27469,25356,25356,23276,23276,23243,19083,33871,42259,65337,65369,65434,65434,65434,63289,54707,16937,12745,10664,10664,10664,10696,10664,16970,46418,54738,50513,46320,44239,48433,54706,48367,44175,44143,50481,54674,44207,42094,40046,50545,52626,42062,39981,39949,52593,52560,37901,37869,39982,52626,48399,37966,44370,46450,16903,19147,52724,54837,54837,56917,56918,56918,56917,54706,12612,10663,10565,8517,6503,6536,8485,10533,8517,8517,10565,8517,8550,8517,10565,10565,10564,8517,10564,6503,9010,13040,8551,10565,10564,10565,8615,42357,12711,8517,19080,10629,23340,199},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,48893,50941,50941,50941,50941,50941,50940,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50875,50874,50842,50842,50842,50842,50842,50842,48794,48794,48762,48762,48762,48762,48762,48762,48762,48730,48730,46681,46681,46649,46649,46649,46649,46649,44536,42424,42423,42391,40279,38198,36118,42390,40278,40213,44373,42325,44373,44405,48598,42325,38131,48498,46418,44339,44405,44536,46616,44569,44569,44569,44569,44569,44569,44601,38229,27662,19309,31662,25517,29581,35885,44205,37931,31691,31692,40045,50481,35821,29645,50710,54969,52824,40177,46517,48598,48598,48598,48630,48597,48597,46418,42257,52757,48563,46450,46515,38031,48564,12809,8518,8518,8518,8518,10664,15020,33905,38065,63256,63289,65401,65369,59030,44273,42161,12810,8616,8550,8549,8549,8517,8517,14823,25484,46255,29450,8484,8484,23241,44174,23144,8517,10565,29515,44142,18919,8517,8549,35853,44174,12677,10597,10597,40014,39981,10629,10630,14823,42127,37836,21226,46450,44337,14889,19082,52756,54837,54837,54869,56918,56950,56918,54706,12677,8616,8485,8517,6503,8616,8485,8484,8517,8518,10533,8517,8550,8517,10565,10565,10564,8517,10564,6503,13171,10926,8583,10565,10564,10564,6535,23374,8551,8484,16935,12710,21259,200},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,48893,48893,50941,50941,50941,50940,50940,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50843,50875,50842,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48762,48730,48730,48730,48729,46681,46649,46649,46649,46649,46617,44536,42424,42423,42391,40311,38230,36118,42358,40245,40245,36084,38164,44373,36084,40245,33972,33971,46418,46418,42259,44405,46584,46616,46617,44569,44569,44569,44569,44569,44569,44536,36048,31823,33774,29581,31661,35918,46317,42156,35916,35916,37996,46418,33804,35950,52855,48662,48630,44404,44437,44437,44437,46518,44404,25519,31758,46419,52756,46418,42159,37966,48498,44240,42193,19115,8518,8518,8518,8518,6503,4554,23439,46550,65337,65402,65434,63289,23210,12808,25355,44306,44338,44273,46386,46418,46418,44338,46418,48531,50579,50611,50611,50612,50611,52659,52692,52725,52757,52724,54805,54805,54837,54805,52724,54772,54773,54805,54805,54772,54739,52691,52691,52659,52724,54804,52724,48563,48596,25321,14889,52757,54837,54837,56917,56918,56918,56918,54738,14725,14921,12645,8517,6503,8616,8485,8517,6503,6536,10533,8517,8550,8517,10565,10565,10565,8517,10564,6601,15251,8747,8551,10565,10564,10564,8550,8846,17068,10631,14823,14855,21324,201},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,48893,50941,50941,50941,50940,50940,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50875,50875,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,46681,46649,46649,46649,46649,46617,44536,44504,44504,42359,40311,38198,36118,40278,44406,38165,36084,36084,42292,36020,46518,33972,31891,46484,50578,44339,46518,46616,46617,46617,44569,44569,44569,44569,44601,44569,44568,40309,33970,35855,31629,27533,31726,42159,40044,35948,37996,40109,35917,25387,31758,54902,50743,54936,40210,38033,29743,29776,31856,33970,14987,36081,35983,31726,33805,33805,31692,29515,31725,33839,25389,8518,8518,12776,12809,12875,10892,31825,54838,63224,63191,63191,58966,10631,8518,23242,35918,48367,42062,37901,37934,33708,31595,31595,31595,40014,33642,18886,16838,29581,48434,27369,14757,14757,33740,50448,25192,14725,14693,37901,46254,18918,12645,12612,42127,46287,12676,12645,12710,46352,44110,27566,48630,54904,46420,19049,52724,54805,54837,54837,54837,54869,54869,54739,27535,29745,12613,8517,8780,8747,8485,8518,6667,8650,8517,8518,6602,8584,10564,8518,6536,8551,10564,8813,17266,6601,19213,29612,40110,14855,6568,13105,31794,10729,12710,19048,19212,197},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,50941,48893,48893,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50940,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50875,50875,50842,50842,50842,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,46649,46649,46649,46649,46649,44536,44504,44471,42391,42359,38198,36118,42326,40245,46486,38132,38132,42293,40245,40245,40245,36084,44339,48466,44339,44437,44536,44569,44569,44569,44569,44569,44569,46649,44569,44569,44569,44503,33743,29549,27533,33806,40080,35950,31756,33804,38032,37998,31692,33838,54837,52757,54936,44436,42323,40243,40242,36017,31856,23374,38292,46548,46450,44272,42094,42127,48531,54837,52756,23307,8518,8518,10631,10663,12809,12875,12875,14988,19084,19084,48500,58966,10598,8550,21195,31790,46320,44207,40079,40047,33741,31627,29547,29547,42159,40014,29581,29581,35919,50513,37935,29613,29613,37967,44240,35886,31693,31693,42127,44272,35886,33774,33806,42192,44273,35919,35919,35951,44305,44273,42290,48564,40179,52856,52790,52756,54837,54837,54869,56917,56950,56950,54771,37966,44272,16838,8517,8551,8583,8485,8517,8517,8518,8516,8517,8517,8517,10565,8516,8517,8517,8517,8911,8845,13070,44403,46352,58898,16904,8616,10828,10696,10662,8517,10533,17034,200},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50941,48893,50941,50941,50941,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,50908,50908,50908,50908,50908,50908,50907,50875,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,46649,46649,46649,46649,46649,44536,42424,44472,40311,40311,40246,36118,42358,42293,46485,36084,36052,40180,38164,54871,50710,36051,46450,48498,44339,46518,44568,44569,44569,44569,44569,44569,44569,44601,44569,44569,44536,44438,35759,23276,27565,35918,37967,38000,29612,25453,40179,42256,31691,35919,59031,52758,52856,42259,40211,38162,36050,27630,31857,48564,42289,42256,44173,42093,42093,42061,44238,54804,54836,31693,14856,10760,14954,14986,12906,4586,4586,2539,4587,4619,46483,50579,10631,10696,17001,46450,52626,48401,46320,46288,40013,37868,37836,35788,44175,44175,37901,37869,42095,50513,44142,35788,35756,44207,52560,42029,35724,33676,46320,52626,35787,31595,31563,48401,52593,33610,31530,33611,50513,50448,35853,44371,50678,52856,54870,52724,54837,54837,54869,54869,54870,56918,54771,35755,33773,27271,10533,8517,8518,8517,10533,8517,8518,8516,8517,8517,8517,12743,10565,8517,16969,14889,2539,4488,6503,27566,44272,54705,12743,8551,8583,19279,42358,29613,21226,25452,198},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,48893,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,50842,50842,48794,48794,48794,50810,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,46649,46649,46649,46649,46649,44536,42456,44504,44471,40279,40278,38198,40278,42325,40245,38164,36084,38132,36052,44437,38132,38132,46418,46418,44371,44470,44536,44569,44569,44569,44569,44569,44569,44569,44569,44568,44470,44372,31630,29613,25484,35918,33774,33775,27468,21293,38098,42192,31723,33837,52757,52757,48565,35984,38130,33937,33969,31823,33904,50643,48432,42225,50578,35788,35821,39949,35756,48464,52756,50545,31660,27564,31823,33870,29678,4586,2506,2506,4587,4587,44371,56885,16969,8518,10695,23436,46288,42127,37967,37934,31627,27434,27434,27434,37934,31594,10597,10597,21194,50546,31562,12710,12710,27435,44142,25257,12710,12710,33741,46255,21096,14823,14823,35853,44174,16935,16936,16936,42127,42062,27533,44371,50678,54904,50612,52724,54837,54869,56918,56950,56950,56950,54771,20998,17001,27239,10532,10564,8517,8517,8517,8518,8518,8517,8517,10597,8517,23373,14725,8517,17001,23275,15118,25551,23405,10794,25355,33676,8649,8681,6569,23505,57048,35820,27499,27500,197},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,52988,52988,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,48794,48794,48794,50810,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,46649,46649,46649,46649,46649,44536,44504,44504,44471,40279,40278,38198,44471,44438,42358,42325,36084,36084,36020,44470,50743,44404,46451,46419,44372,44470,44568,44569,44569,44569,44569,44569,44569,44569,44569,44535,44403,40144,31694,35886,31757,33838,38000,33775,23374,23439,38066,46451,44239,29677,38098,33873,44405,42291,42388,36017,38098,42290,46516,50676,48400,44306,44142,42093,42061,42029,42093,52691,48531,52626,35886,29645,29645,29645,31759,4587,2506,2506,2538,4587,42291,54740,12711,8517,14824,50644,52691,52691,52692,52692,52724,52724,54804,52724,54772,54772,54772,54804,54804,54772,54772,54773,54773,54772,54772,54740,54771,54772,54772,54772,52691,52659,52627,54772,54739,50546,50514,48465,54739,54772,50546,46451,50677,52724,56984,50676,52756,54837,54837,54869,54869,56917,54771,23144,19114,27272,10532,8517,8518,8517,6503,8551,8518,12711,19049,37901,8583,19180,14725,8517,14888,25323,23538,44404,38001,8648,10761,10794,12907,15020,12875,27697,50677,27368,27467,27467,195},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,50842,48794,48794,48762,50810,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,46649,46649,46649,46649,46617,44536,44504,42423,42391,40279,40246,38198,42358,40278,42325,40244,36084,40244,33972,46517,48565,44372,42258,44338,42292,42390,44536,44568,44569,44569,44569,44569,44569,44569,44569,44568,44536,40244,23438,17132,21292,33806,40080,31662,31759,40146,54903,63354,44111,27469,42259,27566,38131,42290,40210,40211,46516,54837,52757,52724,40145,42225,23209,25354,25322,19017,21129,25322,27435,29515,46419,25355,19113,17033,29678,4587,2506,2506,2507,4586,40211,56918,14856,8517,10598,31758,46319,42095,37966,40014,35789,35789,33676,35789,44207,33642,12645,12612,23209,50546,31563,10565,10565,29482,50415,27304,10564,12612,33675,46221,21030,10532,10532,37934,50481,14725,10532,10564,42159,48335,21194,44403,44402,37966,48564,52756,54837,54870,56950,56950,56950,56982,54771,31627,27500,33642,12677,8550,8551,8517,8517,8517,8550,12678,19016,31562,8551,14953,14757,8517,12808,27403,15217,36018,31726,29613,52757,52757,50677,52757,52757,48531,42258,31593,29547,27467,195},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,48794,48794,48794,50810,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,46649,46649,46649,46649,46617,44536,44504,42391,44471,40279,38230,38166,42358,38165,40277,40277,40244,40244,36084,38164,42324,40243,42291,44339,42292,42390,44536,44568,44569,44569,44569,44569,44569,44601,44569,42422,38196,38131,25485,15084,15117,38098,44404,42323,42258,31760,23405,42159,37936,25422,50644,54869,54870,46581,36082,34034,40242,54837,52757,52757,52757,50579,35788,35788,35788,35755,35853,44338,46450,46417,48498,23274,21291,23307,25452,6634,2506,2506,491,4521,40211,61143,18984,8517,10630,35984,44272,44240,42192,42160,40015,37935,35854,37967,46385,42192,33806,33839,38032,50513,44240,35886,35919,40112,46385,42192,38032,38032,42225,44304,42192,40112,40145,46386,48466,42192,42192,42224,46385,46418,46418,44370,44370,37998,33774,56982,59063,59095,61143,61175,61175,61110,56851,56850,56850,52592,27402,10631,10598,8518,8517,8517,8517,8485,14855,21064,8518,12808,16870,6470,8714,21261,23603,36018,16968,29580,52757,54805,52757,52757,54805,50644,40113,23241,27466,25387,191},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48892,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50842,50842,50842,50842,48794,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48730,48729,48730,48729,48729,46649,46649,46649,46649,46617,44536,44504,42391,44471,40279,38230,38198,40310,38198,46486,38164,36084,50776,36084,46518,54870,46452,42291,42259,42325,44503,44569,46649,46649,44569,44601,44569,44569,44601,44569,44535,38129,35983,35918,33805,33837,44272,46289,46289,35920,21360,19312,27567,52627,29614,52757,50579,50645,48629,31953,31888,40275,50677,52725,54837,52757,44305,46319,31563,29547,31627,44305,54804,54804,44272,40078,23274,38064,38064,31726,6634,2506,2474,2539,6634,40211,58998,14790,8517,10598,44305,50513,46288,44176,44175,37901,37901,37869,39950,46255,44110,33643,31595,35853,50545,42094,31562,31530,39981,52528,39948,29418,29418,42062,52594,35755,27337,27337,42127,52561,29417,25224,25224,46320,50512,31628,44370,44370,37934,38032,54836,56917,59029,59062,61110,61110,58997,58898,58897,58865,58865,41963,10533,8517,8517,8517,10565,8517,8517,12710,25354,44304,44304,31627,4488,2540,11024,36019,14890,10630,27499,52757,52756,52725,54805,54805,52692,40145,29612,31627,25354,188},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48892,48892,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,48794,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48730,48729,48697,48729,48729,46649,46649,46649,46649,46617,44536,42424,42391,42391,40279,38198,38166,36118,38197,44405,36084,34004,46550,36052,40212,42292,40212,42291,42258,42325,44503,44569,46617,44569,44569,44569,44569,44569,44601,44569,44568,33968,35982,31725,23372,27565,46353,54480,54447,54838,29746,27665,31760,46256,31727,54870,56916,54838,42323,46581,46581,46581,46580,50677,52757,52724,44338,39916,42029,42029,46352,42093,50610,46352,40014,46418,27467,40144,40176,33774,6634,4554,2474,2539,8551,16936,27435,14823,8517,8517,27597,46287,44208,40079,42128,35853,33741,31692,33773,44239,37869,14823,14823,21194,48466,35821,14888,14888,25387,44174,29515,19016,19049,31661,44208,27435,19049,19081,33806,44207,25322,21194,23242,40047,46256,29613,44370,44370,35821,33839,54836,56917,56949,59030,59030,59030,58997,58898,58865,56720,39851,16805,10565,8517,8517,8517,8517,8517,8517,10598,29547,54771,54770,44174,21227,19212,25651,29615,8517,8517,25387,52756,54805,54837,54837,54805,52724,40145,14888,29547,25354,184},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48762,48729,52758,48663,48729,48729,46681,46649,46649,46649,46617,44536,42456,40343,42391,40279,38198,38166,42391,42358,42325,36052,38132,38132,31924,40276,50677,42323,44370,46419,42325,44503,44568,44569,44569,44569,44569,44569,44569,44569,44568,44568,36114,33902,23405,17164,17132,27566,37967,31724,33870,61273,61241,44371,38000,35919,52789,50611,52725,44435,44435,44403,44436,44436,48596,52757,52724,40080,21096,23274,23242,31725,36016,33838,33773,25258,33741,23275,40176,40177,31759,4586,4554,6634,8714,10695,10695,17001,14856,10662,12743,48563,52723,54772,54805,56885,54805,54772,54740,52692,56852,54772,52691,52691,54739,54707,54707,52627,52627,52659,54739,52659,50545,50513,52627,54739,50513,48368,48369,52626,54739,48401,46320,46320,52658,54739,46352,44370,44370,27435,29646,54804,54869,56949,56949,56949,56950,56917,56818,56817,23078,10565,8517,8517,8517,8517,8517,8517,8517,8517,8518,27467,54770,50448,33709,27469,38130,31988,10925,8518,8517,19114,52756,52757,54805,54837,54837,52724,31662,14887,29645,27434,181},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48730,52693,50679,48729,48729,48729,46649,46649,46649,46617,44537,42424,40343,42423,40279,38230,40278,42358,42326,40245,34004,40212,38164,31924,40244,48597,44404,40145,40146,36019,36117,36149,36149,36149,33971,33939,33939,33939,33907,31859,31858,27598,23372,19147,15019,14986,19147,25387,25450,25419,42225,37935,27501,29483,23307,33742,39982,39950,39950,33677,23209,14824,14857,23307,31693,29580,23275,19016,19017,18984,19017,19017,23275,25388,25355,25355,16969,19082,19081,16937,6535,10729,42288,44108,41995,42061,42027,42061,40043,42158,31822,46352,42094,37934,37934,33740,31660,29515,31595,42127,35755,10532,10532,16936,52660,37869,10532,10532,23177,48335,31530,8484,10532,27402,48367,25223,10532,8484,31628,48368,18918,8484,8484,35853,46287,19016,42225,33774,19049,27566,54804,54869,56949,56949,56950,56950,56917,56818,56817,18950,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,25354,54770,48498,46582,50710,42356,34134,10860,8518,10564,14856,52756,54805,54805,54837,54805,52724,25356,21159,27565,27467,175},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50940,48860,50940,48860,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48762,48730,50678,50679,48729,48729,48729,46649,46649,46649,46617,44537,42456,44438,50645,42292,44273,52692,44406,42358,46486,40277,36084,40212,44307,46451,33839,40080,21195,17098,19211,14954,15018,12873,17066,8681,4586,6536,6535,6535,6535,6535,8550,8551,8550,6503,8550,8583,6535,8681,6535,8583,6503,8615,8550,8583,8551,8583,8551,8551,8550,6502,8550,8550,8518,8550,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8517,8518,8517,12807,44368,50479,50445,52561,50446,50545,50478,35786,40177,46352,46320,42160,42192,42160,42160,40080,40112,48466,44305,40112,40112,40145,50546,48466,42225,42225,44273,48466,46386,44338,44338,46418,48498,46418,46419,48499,48499,48498,46418,46418,48466,48498,50578,48531,33806,6568,6535,25485,52756,54837,54869,56917,56917,56917,54837,56850,56818,23110,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,21161,54771,54771,54772,54804,48497,44305,17099,15116,14920,14823,50676,52724,52724,54805,52756,52724,35953,38129,27499,27500,171},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,48860,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48762,48730,52693,50678,48729,48729,48729,46649,46649,46649,46617,44537,42456,42358,48531,38197,38164,48628,46518,44438,42357,42357,42324,38196,42226,44338,35852,33741,17066,12841,14954,8746,12874,10891,21292,8648,6536,6503,6503,6535,8550,6502,6502,6503,8518,8550,8550,8615,8550,8615,6502,6503,8550,8680,8550,8550,8550,6502,6502,8551,8551,8550,8518,6502,8518,8550,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8517,8517,8517,8517,8517,12775,44336,50511,50478,50545,50511,50546,50478,37932,40144,48400,42062,35789,35789,35756,33708,33675,33708,42095,42029,29385,27338,29483,48433,42094,25288,25256,31595,50480,41996,25192,25192,35788,50481,35723,21031,21031,37901,52560,29417,20998,18918,40013,50480,27369,27501,6536,17101,38130,52756,54869,56949,56950,56950,56950,56917,56851,54672,16838,8517,8517,10565,10565,10565,10565,10565,10565,8517,8517,16969,50610,50512,54770,54738,52625,50480,23274,50741,52724,46450,50610,52691,52724,52756,52756,52724,48630,48466,29449,27467,168},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,48860,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48762,48729,52725,52726,48729,48729,48729,46649,46649,46649,46617,44536,44504,40246,44306,38099,63388,59096,29613,38000,40112,42128,46321,33971,48565,50612,48499,33741,31758,23405,23405,25583,21292,27728,23405,12907,6601,6601,8648,6600,14987,8681,12874,10729,8616,6568,6568,12841,8616,19115,8648,14954,8648,17034,8616,12841,8616,8616,8616,8648,8648,6535,6535,8616,6535,6536,6536,8551,8551,8551,6535,8583,6503,6502,6503,6503,6503,8550,8551,8550,8551,8550,10727,44335,50446,50446,50512,50445,50511,50445,46318,31757,46288,44240,37934,37902,35821,33773,31661,31661,44240,42094,19049,19049,21194,48434,42094,19081,19081,25387,44175,35789,23242,23274,31661,46288,33709,23307,25355,33838,44272,31628,25387,25420,37999,46288,33741,21163,6503,10631,27533,52724,54837,54869,56917,56917,56917,54837,56851,50479,10532,8517,8485,8517,10565,10565,10565,10565,10565,8517,14921,31857,29678,40013,52657,52657,52657,52656,27402,50676,52724,50578,52691,52691,52724,52724,52724,52724,50677,39979,21095,23306,159},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,50941,48860,48860,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50940,50908,50908,50908,50907,50907,50907,50875,50907,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50842,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48729,52758,52693,48697,48729,48729,46649,46649,46649,46617,44536,42456,44439,40048,35987,33808,50612,27598,23439,31859,29679,40113,27665,40146,48499,42128,35854,33774,12808,27565,21227,23308,29645,19082,21260,6568,6633,8583,6601,8582,6536,8616,8584,8583,6503,8583,8550,8583,10598,10696,8550,8615,8550,8615,8550,8615,8518,8583,8518,8583,8518,6535,8518,6503,8518,6535,8518,6503,8518,6503,8517,8551,8517,6503,8518,8550,8517,8550,8518,8550,8518,10663,42288,48398,50445,50479,50412,50480,50446,48300,46385,54771,54771,54772,54740,52659,52659,50547,50546,54739,54707,48465,48465,50513,50546,52561,48368,50481,50481,54739,50546,44207,44207,46352,54706,48432,44175,44175,48433,54706,46287,42062,39982,48433,52626,44207,16936,8518,8518,21260,52756,54869,56917,56950,56950,56950,56917,56818,52527,10532,8517,8517,8517,10565,10565,10565,10565,10565,8517,21161,42224,27533,37965,52657,52657,52657,52657,40079,46385,42159,31594,48563,52724,52724,52724,52756,52724,46452,37866,16870,23241,154},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48892,48892,48860,48860,50940,50940,50940,50940,50908,50908,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50842,50842,48794,50842,50842,48794,48762,48762,48762,48762,48762,48762,48762,48730,48730,52758,52660,48697,48729,48729,46649,46649,46649,46617,44537,44504,44438,40146,31990,15414,48566,31824,36116,38196,25518,40080,38197,44372,40145,25453,31760,31662,19147,23341,17099,21292,21291,17099,14987,6568,6633,8616,8649,12874,8649,10761,8616,8616,6568,8616,12808,10696,14889,8648,17001,8648,14889,8615,12809,8615,8616,8583,8583,6535,6535,6536,8616,8584,8583,6535,6535,6535,8551,8551,6535,6535,8583,6535,8550,6503,8550,6503,8550,6502,8550,8615,35982,48397,50445,50511,48429,50512,50511,44172,27596,46352,40047,37935,37934,33773,33708,29515,29515,40014,37869,10532,8484,12710,52724,44207,8484,10532,18984,48368,35723,8484,8484,21161,46288,29449,8484,8484,25322,46255,23144,10565,8517,29580,46223,21064,21227,19146,10663,33937,57015,59063,59095,59095,59095,61143,56917,56850,48365,10532,8517,8517,8485,8517,8517,19081,31691,31691,33771,33804,37998,25486,35885,52657,52657,52657,52657,52657,37933,29580,25321,48596,52757,52757,52757,52757,52756,44371,31626,12710,21129,146},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48892,48893,48893,48893,48860,48860,48860,50940,50940,50940,50940,50940,50940,50940,50908,50940,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,48794,50842,50810,48762,48762,48762,48762,48762,48762,48762,48762,48762,48729,52758,52628,48697,48729,48729,46649,46649,46649,46617,44537,44503,42259,42357,42423,38395,50776,33774,38065,35789,40015,42192,35822,52658,42192,33677,42029,21194,12808,19114,12776,14888,10728,12776,8616,6535,8583,6503,6503,8551,6502,8583,8550,8550,8518,8550,8550,8518,8583,8582,17001,8550,8550,8518,8550,8518,8550,8518,8550,8518,8518,8518,8518,8518,8518,8518,8517,8518,8517,8517,8518,8518,8518,8518,8517,8517,8517,8518,8517,8517,8517,8518,17033,48366,50446,50446,50445,50446,50478,44171,35984,44304,46353,44273,44273,44306,44338,44306,44305,48498,48498,44370,44370,46418,50579,50579,48531,48531,48531,50611,50579,48563,50612,50612,50611,50579,50611,50644,52692,52692,52692,52724,52724,52724,52692,54772,50643,44305,12841,54869,59029,59030,59062,61110,61142,61142,54803,56817,44105,8517,8517,10630,27435,44306,42159,42059,48331,50477,50477,48365,37899,31759,33804,52657,52657,52656,52656,52656,33675,25354,23176,48596,52757,52757,52757,52757,54805,42259,29514,10630,21161,131},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48892,48860,48860,48860,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50842,50842,48794,50842,50842,48794,48762,48762,48762,48762,48762,48762,48762,48762,48730,52725,50612,44471,48729,48729,48729,46649,46649,46617,44536,44471,44307,54969,59195,52955,54904,27468,38098,23274,29450,35820,29581,50546,40079,23177,39915,17033,8615,10793,8583,19017,8616,6535,6568,6535,6503,6502,6503,6535,6503,6502,8550,8550,8518,8518,8518,8518,8583,8550,8550,8518,8550,8518,8582,8518,8518,8518,8550,8518,8518,8518,8518,8518,8518,8518,8517,8517,8517,8518,8517,8518,8517,8517,8517,8517,8517,8518,8517,8518,8517,8518,8680,46383,50478,50544,52591,50545,50479,42028,31725,48400,42062,40014,39982,37934,37901,33708,33708,42126,44142,27370,25258,25225,46418,44207,21031,18982,25289,50481,42061,20998,18918,29450,50448,35723,16838,16870,31595,48368,31497,14725,14725,33707,50415,25159,31693,42192,10696,52789,56950,59029,59062,59062,59062,59062,54804,56785,44172,12678,10630,16871,52594,56917,56852,42126,50544,52625,54705,52559,39947,38032,33772,52657,52657,52657,52656,52656,29482,23241,21063,46484,54837,52757,52757,52757,54837,40178,21064,8550,19114,112},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48892,48860,48860,48860,50940,50940,50940,50940,50940,50940,50940,50908,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50842,50842,48794,48794,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48730,48730,50613,39983,33773,52758,50612,54806,52757,52823,52758,52725,52724,56885,59030,61143,58998,54739,27401,52493,50345,50380,44173,37799,48333,38031,27271,35623,29612,25452,31759,27501,29678,27532,27500,21292,8649,12907,14889,23406,21195,19147,23372,12809,10729,8584,17002,16969,23340,21130,21227,19114,19178,19114,19146,25355,10728,10696,8616,8583,8616,8583,6568,8583,6568,8551,6568,8550,8616,8551,6568,8550,8583,8550,6503,6502,6503,8550,8551,8551,6503,8551,8550,12937,48430,50445,50510,50509,52591,48398,48497,31725,44239,44207,42127,42127,40015,37967,31693,31693,44272,46320,23340,23340,23307,46386,46288,25387,25387,27533,42159,37966,27468,27500,31726,46321,39983,29581,29581,33839,44240,35854,29581,29613,37966,46320,35854,40145,37967,17099,54902,59062,59095,61175,61175,61175,61110,52594,46254,40048,23373,14856,10597,35756,56917,56885,38068,29779,33938,50577,52689,52689,52689,37932,52657,52657,52657,52656,52656,29449,19080,19015,46483,54837,52757,52757,52757,54805,42259,23241,10630,19114,120},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48892,48893,48893,48892,48860,48860,48860,48860,50940,50940,50940,50940,50940,50940,50940,50908,50908,50940,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,48794,50842,50842,50810,50810,50810,48762,48762,48762,48762,48762,48762,48730,48730,48532,48335,54604,56820,56820,56852,58900,58998,56819,56820,56754,60945,61045,63159,58932,54739,29482,50380,52327,48203,48301,46055,52494,40079,37702,46058,27467,10631,33775,10631,19147,19049,10631,23308,6503,14922,8550,21261,10630,12874,17002,10696,14889,8550,12809,8550,21194,8550,21194,8518,21162,8517,21162,8550,10729,8517,8616,8517,8583,8518,8648,8517,6535,8517,8583,8517,8583,8518,8583,8517,6503,8518,6503,8518,6503,8518,8518,8550,8518,10631,10630,17033,42027,44107,46188,46220,46221,42059,29481,38032,52691,52658,48367,46320,50513,50481,48336,46288,52594,52626,44207,44207,44239,50513,50481,44174,44142,46255,52626,50513,42030,42094,44207,52626,48400,39948,39916,44175,52626,46255,37837,35789,44207,52626,42061,40079,31693,40210,56917,58997,59062,61110,61110,52593,48335,50481,35756,33709,29548,16936,8550,19147,56885,56917,36021,29812,33937,50577,52689,52689,54737,37932,52657,52657,52657,52656,52656,29417,16936,19015,44403,52757,52757,52757,52757,52725,44307,25353,8517,19114,123},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48860,48860,48860,50940,50940,50940,50940,50940,50908,50940,50908,50940,50940,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,48794,50842,50842,50810,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48630,54675,52691,50745,52759,52693,54806,54806,54806,54806,56886,54641,59096,61243,50744,56951,37999,46418,42126,44208,46320,33742,50546,44338,33774,46386,40080,27598,35953,29613,33839,31759,31694,23405,10762,23341,17067,33873,23275,29646,25485,23340,21292,12841,31726,19115,33839,21195,35887,21194,33838,21194,33806,21162,12809,8648,8681,6568,8682,8648,8714,6568,6601,6535,6569,6535,6568,6535,6568,6535,6568,8551,8583,8551,8583,8551,8551,8551,8551,10631,8550,14921,46317,50445,50511,52558,50479,37802,27401,23338,44272,42094,29515,27467,31693,33740,29514,29514,40014,42062,10565,8517,8517,48499,46320,10564,8517,12743,44142,35755,8517,10565,16936,44175,31563,10597,10597,21161,44142,29449,10663,12711,27435,44142,25257,27533,27435,38097,54869,56917,59029,59030,58997,54739,56819,44240,25487,27534,19049,25585,27665,19147,56885,56885,36020,21427,21426,29681,38033,46320,46385,37932,52657,52657,52657,52657,52656,29449,14888,19015,40177,52659,52724,52724,52757,52724,44307,23273,8517,14921,125},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48860,48860,48860,50940,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50842,50843,50843,48794,48794,50842,50842,48794,48794,48762,48762,48762,48762,48762,48762,48762,48730,48730,50711,52693,46453,48729,48697,48697,48697,46649,46617,44536,44471,52693,59097,59195,52889,50710,25322,46385,29450,56984,50579,16871,44273,40079,29547,56915,37933,8582,23308,10630,17002,17001,12808,17034,6503,14922,8550,17035,10631,10663,17002,8583,14889,8518,19081,8518,19049,8518,21194,8518,21162,8550,21162,8517,8616,8517,8615,8517,8616,8518,8616,8517,8583,8517,8583,8517,6503,8517,8583,8518,8551,8518,8550,8550,8518,8550,8518,8550,8518,8583,8518,6567,46252,50412,50478,50445,48333,27336,16935,35951,46417,48498,46418,46451,48499,48531,46418,46450,50578,50611,50611,50612,50611,52691,52691,52692,52724,52724,52692,52724,52724,52724,52724,52724,54772,54772,54805,54805,54772,54771,52723,52691,52724,54804,52691,52627,21195,36050,54804,54869,56949,56949,58997,56852,58932,44274,31794,29647,25355,38163,44338,16970,56918,56885,38002,23341,23473,34037,38195,40144,40178,37933,52657,52657,52657,48432,52625,33642,21226,21064,40177,52724,52725,52757,52757,52692,44339,33937,23209,17001,120},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48892,48860,48860,48860,50940,48860,50908,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,48794,50842,50842,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48730,48697,44470,42390,48729,48729,46649,46649,46649,46617,44536,44503,50646,57049,59195,59195,59128,25419,44338,21161,46420,44272,10662,40080,37966,23177,37836,35789,16969,19148,14953,19114,19082,17002,12808,10696,12842,12777,14922,10696,12841,12809,10761,8648,10696,14889,10728,16969,12744,16969,12744,17001,12808,16969,10663,8583,8583,8583,8551,8583,8551,8583,8550,8551,8550,8551,8550,8550,8550,8551,8550,8550,8550,8550,8550,8550,8518,8518,8518,8550,8550,8550,6567,44237,50444,50478,50478,46254,27336,14823,27532,46352,42094,31660,31660,35854,37901,35788,33708,42127,44207,20999,18919,21097,46353,46320,16838,16838,18951,48433,44174,14725,14693,21064,48400,39916,12645,12613,23242,48400,33610,12612,10564,27402,48400,29417,25355,23374,42323,54869,56950,59030,59062,59030,56819,58899,38000,23408,27535,29516,31728,29581,17034,56950,56852,42227,38065,44404,63388,59096,42225,40177,35852,52657,52657,52625,33807,52691,44271,31692,23208,40177,52757,52757,52725,50612,52724,46452,42160,23144,25355,114},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48892,48893,48893,48893,48892,48892,48892,48860,48860,48860,50940,50940,50940,50908,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50842,50842,50842,48794,50842,50842,48794,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,42357,42357,48729,48697,48697,46649,46649,46617,44536,46551,50646,57050,59196,44700,50809,29612,42160,16904,48631,38032,10728,37999,35886,21162,33741,23210,8615,15018,8583,8615,8615,8550,8550,8550,8583,8551,8583,8551,8583,8583,6535,8583,6502,8551,10663,8583,8550,8582,8550,8583,10695,10695,8518,8518,8550,8550,8518,8518,8518,8518,8517,8518,8517,8517,8517,8517,8517,8550,8517,8517,8517,8517,8517,8518,8517,8517,8517,8517,8517,8518,10760,46286,50511,50543,39979,44271,23208,16936,25419,42192,46288,33709,33709,37935,37934,31661,31661,44240,48368,29548,27500,27500,46354,48433,29613,29613,31694,44273,42160,31694,31726,33839,46353,44208,33806,33807,35952,46321,40080,33839,35918,38063,46353,42192,40144,21227,54869,58997,59062,61142,61143,61142,37772,25226,23374,4620,23439,33906,42357,50710,48532,56918,56884,42227,46484,48597,63388,61176,42258,40145,31659,52657,52656,52625,31694,52724,46352,33773,23241,38097,52757,52757,52724,50645,52724,33872,27403,10565,14921,104},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48892,48893,48893,48892,48892,48860,48860,48860,48860,50940,50940,50940,50940,50940,50940,50940,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50842,50842,50842,48794,48794,48794,50810,50810,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,46418,56982,50776,48697,46649,46649,46649,46617,44536,48599,46519,54969,57115,34332,46616,25355,31563,14889,50743,31824,14922,33708,40112,14791,29483,25387,8615,8712,8615,6567,8615,6503,6502,8551,8615,8615,8680,6567,8615,8680,6535,6503,8615,10728,10696,10728,8615,10728,8615,10728,17065,17065,8583,6502,6535,6534,8582,8582,8582,8583,8550,8550,8518,8518,8550,8550,8550,6503,8518,8550,8550,8550,8550,8550,8550,8550,8518,8550,8518,10663,27532,44206,50445,50478,44107,50511,33673,19048,40079,52626,50512,48401,48400,48368,46287,42062,42030,48368,50481,39981,39949,39949,48433,50480,39916,39884,39949,52626,52561,37836,35756,37869,52626,48400,33675,33643,37901,52594,44174,33611,31530,39950,50513,39949,31563,25518,56950,59030,59062,61143,61143,61142,31596,29647,29713,12810,31726,37934,27402,21031,16904,56918,56885,42259,46484,48565,63388,61209,40081,17069,23339,52657,52657,52657,40144,52690,48465,35886,25289,38032,56984,46321,57081,61242,56885,33775,19017,8517,10631,98},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48861,48893,48893,48893,48893,48892,48860,48860,48860,48860,50940,50940,50940,50940,50940,50940,50908,50940,50940,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48762,48762,48730,50777,52791,57016,48729,48729,46649,46649,46649,46617,46584,46519,46551,59130,59163,55068,54936,33806,44369,46482,44370,42192,38031,48499,46417,48531,48596,42192,27500,33904,25419,27533,31693,27468,23308,25388,31759,25419,31726,25419,29679,31725,17002,10761,25388,31693,25355,31693,23275,33774,23274,33773,23274,31693,21161,25453,25386,31693,23242,31693,21194,27500,10662,8583,8518,6535,8518,8551,8518,8551,8518,6503,8550,8551,8518,8550,8550,8518,8550,8518,8550,8518,25289,48464,50576,50545,33675,46353,40014,40146,35984,42192,42126,37934,40014,35853,35821,29547,29483,40014,44174,12677,8517,8517,42192,48400,12645,10597,12710,42127,42030,10629,10630,14855,42094,35788,12742,12742,19081,42127,33643,14855,14855,23275,44175,29483,17001,29548,37934,46319,37869,31596,31595,40014,52659,52692,44274,19016,29580,33707,21096,8517,8714,54902,58998,44339,44273,42225,63355,61177,21229,8845,27499,52657,52657,52656,52656,52657,48465,35853,21096,35952,54902,48433,52790,57016,52659,31628,14791,10565,6503,91},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48861,48893,48892,48893,48893,48892,48860,48860,48860,48860,50940,48892,50940,50940,50940,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50842,50810,50810,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,48729,46649,46649,46649,46617,48631,44471,42423,48631,48599,46486,54805,31758,35984,31759,38098,25420,40178,48466,44305,33807,44272,37966,8583,21260,12840,17099,19081,6502,19147,10631,23308,10630,17002,12776,10728,19114,8615,10728,8647,19114,8583,21162,8583,21195,10663,21194,10663,19146,10663,21226,29644,29579,10727,21194,12776,21227,8550,6535,8550,8583,8550,8551,8550,8583,8550,8551,8550,8583,8550,8551,8551,8551,8583,8615,21064,29448,27401,50544,52626,52626,33741,37966,31628,23307,44305,48530,50578,50579,50611,50643,52692,50644,50644,52692,52691,52724,52724,52724,54772,54739,54772,54772,52724,54772,54772,54772,54772,52691,54772,54772,54740,54740,54772,54772,54772,52659,52659,52691,54771,52658,52562,52561,52626,54739,52594,48401,48433,52626,52658,44142,12679,14792,27467,33675,42193,23439,17100,54903,61111,59063,44174,52659,58900,56819,19116,19311,29612,52689,52689,52689,52689,52657,48498,33773,21128,35952,50578,14758,14823,52791,52724,40144,29515,14790,10761,104},
{48893,48893,48861,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48861,48893,48892,48892,48892,48892,48860,48860,48860,48860,48860,48860,50940,50908,50940,50940,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,50744,48599,46617,48664,48631,44504,42391,42423,40311,42457,57016,33838,33905,36051,38098,40178,38163,44208,50644,54936,50643,33774,17067,17066,21357,23404,23372,23372,19114,21195,19115,21195,17034,19147,19179,21227,17034,21227,21162,19114,19082,19081,21162,19114,21226,21194,19113,21194,23242,25451,38030,27467,23275,25387,19146,21226,8550,8550,8550,8551,8550,8518,8518,8550,8518,8518,8517,8518,8518,8518,8518,8518,8518,8518,16936,25223,27369,50545,50578,52626,29548,50579,27436,38032,37999,44304,44142,31595,31596,35821,35821,33708,31628,42062,44174,20966,14725,14693,40112,46352,14790,12678,12677,46288,46287,12612,12580,14725,46320,42029,10532,10532,16871,48368,39916,10597,10532,21128,48368,33578,8484,10532,25322,48367,27336,8484,8484,29515,37804,21065,17002,23339,29514,33774,42292,33807,25289,50611,58965,58965,58965,61045,61045,58932,40080,46450,56884,58963,56851,56884,54771,46319,46418,31693,19049,33871,52724,48629,48564,48564,50611,46287,27599,25387,44370,200},
{48893,48861,48861,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48861,48892,48892,48892,48892,48892,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50940,50908,50908,50908,50908,50908,50908,50907,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50843,50842,50810,48762,48762,50810,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,50744,50711,46616,48664,46584,42424,40343,42391,40312,36445,54936,48598,40244,44372,44372,48597,40342,46387,48564,38099,44338,27598,12971,12906,13133,10793,19244,23371,10760,14986,12808,12905,8647,12873,12808,14921,10760,12840,12873,10760,14921,12840,10792,12873,14985,14953,12840,14985,19146,19145,19145,19146,23436,21291,21323,14953,8550,8518,8518,8518,8517,8518,8518,8518,8517,8518,8517,8518,8517,8517,8517,8518,8517,8517,16904,31530,35788,48398,50544,50512,29580,44207,25388,12744,25452,42192,48368,33708,33709,37966,37967,33741,31693,42159,48368,33773,31693,31693,44241,48433,33806,31759,33839,44273,44272,33838,35919,35951,44273,44240,38000,38032,38064,44305,44273,38032,38032,40112,46353,44305,42225,42257,44338,48433,44305,42225,44305,44305,25291,16970,33905,37901,33675,31595,29482,10565,10565,48531,58965,58965,58997,58997,58965,58964,41963,44271,54738,54705,52658,50677,38032,31661,35887,25388,25420,44305,48531,46386,46352,46320,46417,46353,46518,37967,52757,205},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48861,48861,48892,48893,48892,48892,48892,48892,48860,48860,48860,48860,50908,50908,50908,50908,50940,50908,50908,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,48762,48762,50810,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,48729,50744,50711,48664,48664,44537,42424,40343,40311,40311,40441,57016,46550,42325,42292,42324,40178,40277,44273,52757,50610,50446,31725,17131,17099,15084,17164,10760,17001,14953,10793,10825,10793,10825,12905,17066,12873,21292,14921,14954,14953,19082,23404,38096,25516,38095,25516,36015,25484,40176,29644,40176,29676,42354,31789,40273,25484,6535,6503,6535,6535,6503,6503,6503,6503,6502,6503,6502,6503,8518,6503,8550,6535,6470,6503,8550,8615,29579,44204,50545,50513,33740,46385,42225,14824,29613,50578,50448,42029,39949,42062,44143,39949,39917,44175,48335,37836,35756,35755,46320,50480,37803,35723,35723,50513,52593,33643,31563,33643,52561,50448,31529,31498,33643,50513,46222,29385,29417,35756,52593,42061,25256,25256,33708,50480,37835,25191,23111,35788,50546,29419,21162,31627,33707,31562,14758,10598,10565,48499,58965,58965,58965,58965,58965,58932,44075,44272,54706,54705,52691,52789,38032,31726,25420,27467,25354,27434,35919,17068,23176,23208,38031,44339,44438,40080,52757,202},
{48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48893,48861,48861,48892,48892,48892,48892,48892,48860,48860,48860,48860,50940,50908,50940,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50843,50843,50842,50842,50842,48762,48762,50810,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,48729,48697,50711,50679,48664,44536,42424,40343,40311,40279,42326,59031,54935,52855,52823,54903,56951,56983,56917,59063,59096,59063,46451,38130,36049,36017,36017,33905,35986,33937,33938,33937,33970,33970,33937,33937,35952,35951,33871,33871,33871,33871,38032,42192,38064,40112,38032,40144,40145,44273,40112,40080,38000,40144,40112,38031,27533,17034,14954,14954,12842,10794,12809,14889,23340,27501,23340,17034,12809,10696,8584,6536,6536,6535,6536,6535,6535,29579,44272,52659,50546,44273,37934,33741,14824,17033,37998,44174,29515,27435,33740,33773,31595,29515,37901,44207,16871,10597,10597,35886,48400,16870,12742,12743,39982,44110,14790,12743,14855,40047,39982,14855,14855,19048,42126,37836,16968,16968,23275,44207,35788,19081,19081,25419,42126,31660,21162,21194,31693,48433,18985,17001,31659,33707,33675,25322,8517,10565,46418,58965,58965,58965,58965,58965,58932,44043,44239,54705,54705,52658,48564,38065,31758,27500,31693,31693,27402,27467,14954,21096,19081,44370,40177,46484,35854,52789,211},
{48893,48893,48893,48893,48893,48893,48893,48861,48893,48893,48893,48893,48861,48861,48892,48892,48892,48892,48892,48860,48860,48860,48860,50908,50940,50940,50908,50908,50940,50940,50908,50908,50908,50908,50908,50908,50908,50908,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,48794,50842,50842,48762,48762,50810,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,48729,48697,50711,50678,46617,44536,42424,40343,40311,40278,42326,46518,40277,40245,40213,42260,44437,42325,40179,40211,38099,48532,40179,38100,38165,44471,44503,44503,44503,44504,42456,44504,44536,44536,44536,46550,54903,44370,35919,25485,31693,35853,33773,33741,29582,44371,59128,56852,52723,52757,54837,54805,52723,46350,33770,27499,27467,16937,21193,23274,12809,19179,41933,43980,43948,41900,39820,14726,8550,8518,8517,8518,8551,8518,8550,8518,8550,31627,42125,50577,50513,44239,42224,35854,12711,29646,50611,52659,50611,52691,52724,52724,52691,52691,52723,54804,54739,54739,54772,54772,54772,54772,54772,52691,54772,54772,52659,52659,52626,54771,54739,52626,52595,50546,54771,52658,48432,48432,50513,54739,52626,48368,48336,48433,54706,50481,44207,44175,48400,48401,14792,19049,37900,35821,33708,18984,10663,10663,46386,58965,58933,58933,58933,56884,56884,44076,42159,54705,54705,50544,44305,44403,35919,31660,31661,33774,29515,33774,21161,16903,19048,33806,35918,33773,29581,52757,209},
{48893,48893,48893,48893,48893,48861,48893,48893,48893,48893,48893,48893,48861,48861,48893,48892,48892,48892,48892,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,48794,50842,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,48697,54805,50679,46617,44536,42424,40343,40311,40278,40278,40278,42391,40246,38133,38132,40309,38197,34004,36051,44372,46452,33939,29780,34005,42456,42456,44536,42488,42489,42489,42488,44536,44568,44568,46583,57016,59095,40144,23469,31757,37965,25452,21228,29550,42259,59161,54609,37869,46419,59128,50644,54804,46381,37994,37962,31690,21098,27433,21192,14920,17001,37837,37837,37869,39982,19116,6536,8583,8517,8518,6502,6503,6503,8550,8551,8551,31693,40045,50546,48465,46320,44273,40144,19049,29677,44272,44174,33676,31596,35789,35821,33676,31595,37934,44174,20998,12612,10564,35919,50546,16837,10564,10564,42127,50448,14692,10564,10564,44207,46255,10532,10532,12677,48368,44077,10532,8484,16871,48367,37803,8484,8484,19048,46287,31529,8485,8484,23241,39950,12744,21195,35919,38032,33742,17002,29680,42390,59195,63324,61275,61275,61242,59031,52658,39981,42158,54705,54705,52625,52658,40177,33839,31693,29614,31661,29547,35951,27468,12677,14790,25387,38064,35919,29613,52757,208},
{48893,50941,50941,48893,48861,48861,48893,48893,48861,48893,48893,48893,48861,48893,48861,48892,48892,48892,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50810,50810,50810,50810,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,48729,48696,54773,48631,46617,44536,42424,40343,40311,40278,40278,40278,40246,40213,40213,36052,40277,36084,31892,36019,42260,44373,38100,27731,38198,42456,42456,42488,42488,42488,42488,42489,44569,44568,42389,46518,59064,61176,44305,27597,35949,37965,31725,25388,33646,42226,61210,54545,48369,48499,54870,52757,52723,46417,44173,33772,27532,21129,19080,23272,25353,16968,37901,37933,37900,37901,27502,10762,10631,8517,8517,8518,8518,8518,8518,8550,8518,27501,40078,50578,48465,50546,44305,40144,25354,33838,40144,46320,35821,33741,37967,35919,33806,33773,40112,46352,37967,33839,33839,42225,48466,37999,35919,35919,44305,46353,38032,38032,38064,44273,46353,40145,40145,40177,46386,46385,40177,40177,42258,48466,48466,44338,46418,46450,48498,48466,46418,46450,46450,33644,12744,19115,33905,42291,48563,54805,56917,59029,58997,63125,54738,46352,44174,39981,37901,35787,37966,54738,54705,54705,52658,40145,31759,31726,31694,31662,29548,35952,33806,12710,16935,19016,29547,38000,31726,52757,205},
{50941,50941,50941,48893,48861,48861,48861,48893,48861,48893,48893,48893,48893,48893,48861,48892,48892,48892,48892,48892,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,50842,50842,50810,50810,50810,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48729,48729,48729,48697,50743,54773,48631,46617,44536,42424,40343,40311,40279,40278,40246,40278,40246,40245,34004,36116,34004,27764,38131,48532,44372,44339,33939,42358,42423,42456,42456,42488,42488,42488,42488,44536,44536,40342,42324,61176,61175,40046,25453,31725,37997,33805,35789,33678,38098,61209,56625,56723,44273,52725,48499,50676,48498,23341,38098,21260,19050,16968,19112,16968,14824,16968,14823,31626,29677,17199,17100,10663,12841,17035,10794,8616,8617,6568,6569,4521,17133,40045,52626,46386,50577,46385,40145,21161,33838,50578,48367,39916,37836,39949,39950,37869,35788,42030,46223,37868,33675,33675,44239,50513,35723,33578,33578,48400,52594,31530,29417,27369,48400,50512,29417,27336,29417,50480,50415,27272,25224,27370,50513,44174,23111,23111,29482,50480,39915,21031,23079,31563,37869,40146,48497,42156,27399,23237,16997,19044,21093,23206,54538,46220,31593,40046,40013,37933,35788,35853,54738,54706,52625,52658,40178,33839,33839,27501,27469,29548,38000,35952,12710,16935,23306,23306,33904,31694,52757,203},
{50941,50941,50941,48893,48861,48861,48861,48861,48861,48893,48893,48893,48893,48861,48892,48892,48892,48892,48892,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,48794,48762,48762,50810,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,48729,52791,54806,48663,46617,44536,42424,40343,40311,40279,40246,40278,40278,38165,36085,34004,34036,36117,27732,31924,48631,52824,48598,31859,44470,46616,44536,44536,44536,42488,42488,44536,44536,44568,42391,33906,54871,59063,42160,21325,27597,27564,25484,27533,31726,35985,59161,54513,54643,50547,52757,52757,56949,46418,14988,23374,17067,23146,19113,21192,12840,12807,25385,21192,21224,25484,46518,31761,25454,29646,23340,12842,10697,8747,6568,4521,4554,38163,42094,50514,44273,48433,44240,40145,23273,27564,35918,44207,31660,29579,33773,35821,31628,31595,37934,44207,23209,10694,10694,31725,50513,23176,14855,14856,35853,46223,19016,16936,16936,37934,42095,19048,17001,19081,42126,42062,21129,21129,23242,44240,37934,21194,21226,25452,44207,37901,23307,25387,29613,44175,35821,21161,16903,12678,12710,14823,27400,31560,29479,54539,42027,33740,40014,37901,35853,35788,33773,54738,54738,52657,54706,40145,33807,29580,23274,25355,31661,33839,35951,14823,16935,25452,33871,36017,31726,52757,204},
{50941,50941,50941,48893,48861,48861,48861,48861,48861,48893,48893,48893,48893,48861,48892,48892,48892,48892,48892,48892,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,48794,48794,48762,48794,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,48697,54838,52758,48631,46617,44504,42424,40311,44406,40278,38198,38165,38198,36117,36085,38164,40309,38229,29844,34004,59129,52693,50645,31827,44470,48663,46583,44536,44535,44503,44536,44536,44536,44568,48696,42357,52790,48531,31790,27597,42157,33772,31692,33773,33806,44371,61241,52464,41998,44273,52757,52757,58997,48498,15021,8748,23274,25194,19048,21224,19208,23336,35785,35816,25351,23370,38066,44275,35920,29646,21195,12842,12680,10730,6536,4521,10892,46517,42124,46351,44305,46352,42191,40112,29579,38031,50611,52691,52692,52723,54772,54772,54772,54772,54804,54804,54707,54707,54707,52659,52691,52659,52659,50578,52691,54804,52594,50513,52594,52659,54739,50514,48433,48401,54739,54707,46320,46287,46319,52658,52593,46223,46223,46288,52626,50480,42094,42094,44239,52626,48367,33643,16903,21097,20999,19081,23207,14757,21094,54539,46318,44240,37933,35853,35820,33708,31692,54738,54738,54705,52658,40178,38097,38065,38065,38064,38031,35952,38032,21130,16936,27565,38097,38033,29581,52757,202},
{50941,50941,50941,48893,48861,48861,48861,48861,48861,48893,48893,48861,48893,48861,48892,48892,48892,48892,48892,48892,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50907,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50842,50842,50842,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,48697,50745,52759,50679,46617,44536,42424,40311,40311,40278,40246,38198,38198,34037,36117,40245,42422,40309,31924,36116,59161,61109,58998,36019,46550,52856,48696,46616,46583,48598,44504,44536,44536,44569,44536,42357,57048,59063,29710,23437,33805,35852,35852,37933,29581,29647,61209,50384,33613,42161,52725,52724,56885,48498,17101,8747,25322,29354,23242,17096,17096,37962,42089,37896,27431,33804,33872,50711,42259,31759,29679,12842,10599,8649,6503,4521,6569,35920,44172,50544,46385,48465,44304,42225,31692,33773,40111,44142,33643,31563,33708,35789,31628,29515,35821,44142,23143,10532,10532,31694,52659,21031,10532,10532,35853,50448,16838,10532,10564,40014,48335,12645,8484,10564,42127,46222,10532,8484,12677,46287,41996,8484,8484,14822,46255,35723,8484,8484,16935,44174,29449,10630,12744,29548,25257,14855,29481,21063,16869,52426,46318,42159,35820,29482,25257,16871,23307,54738,54737,54705,52658,42226,38097,38097,40145,40113,38032,31629,14725,12711,14856,23340,19148,10828,15019,50709,200},
{50941,50941,50941,48893,48861,48861,48861,48861,48861,48861,48861,48893,48893,48861,48892,48892,48892,48892,48892,48892,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50842,50842,50842,50842,50842,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48729,48729,48729,48729,48729,48729,48697,50711,50679,46617,44536,42424,40311,40311,40278,40246,40246,42359,38165,36117,40245,42454,40341,34004,38196,59161,57016,56885,48499,50678,50743,46616,46616,44568,44536,44536,44536,44536,44568,44568,48696,57016,59063,33871,23437,33869,35917,33772,29580,33710,23341,61209,56560,33677,33808,52757,50644,50610,48466,21293,19212,29482,35627,29579,21288,14952,31657,40010,35816,27464,23337,42324,48598,15021,29647,33742,12875,10729,8551,6503,4521,6470,23242,33739,50513,48400,46352,46352,44305,40046,38031,40112,46320,37934,33805,37967,37999,35919,35887,40112,48400,40080,35951,35952,42225,48466,42193,40145,40145,44305,46385,40144,40112,40144,44305,48434,44305,42258,44338,46386,46353,44338,44338,46418,48498,48466,46418,46418,46418,48498,48530,48531,48563,48531,50578,50578,37869,14856,31661,23209,12743,21095,12677,16869,50378,44205,37965,16904,12743,12711,10631,21226,54738,54705,54705,54738,42226,38098,40146,40145,33774,21196,10795,12907,4521,4554,12908,4620,4620,10859,50677,200},
{50941,50941,50941,48893,48861,48861,48861,48861,48861,48861,48861,48893,48893,48861,48861,48892,48892,48892,48892,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50842,50842,50842,50842,50810,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48729,48729,48729,48729,48729,50777,48697,50743,50679,46617,44536,42424,40343,40311,40278,40246,40246,38198,36085,36085,36084,42389,42422,36149,36052,44373,46485,46484,44340,52726,56951,54871,48630,44536,42488,44536,44536,44536,44568,44568,44568,52822,54805,35919,25517,31757,31692,35884,35853,29613,21293,52790,48336,31629,33873,54870,54837,42124,42124,33771,25386,23209,23178,29547,25513,14952,14951,31657,35848,33736,29545,48661,46386,29548,31693,31660,21194,29580,35951,8615,4488,8518,16904,33804,52624,50544,50577,50545,46386,50512,42224,46417,48367,39982,37869,39950,39949,37869,35788,39982,44175,35755,31497,29449,40014,50513,35723,29385,29385,44175,52593,33578,25224,25224,44207,50481,29416,25192,25192,46287,48399,23111,21031,21032,48433,48334,18917,18917,21063,48400,42028,18885,18854,23176,48367,37835,14823,12743,27468,23177,12743,16903,10597,29416,52426,44106,18982,12743,12711,10631,10631,23339,52690,52624,52592,52625,42226,40146,38065,33775,8715,4587,4620,8747,2539,2506,12907,4619,6765,12972,50677,199},
{50941,50941,48893,48893,48861,48861,48861,48861,48861,48893,48893,48893,48893,48893,48861,48892,48892,48892,48892,48892,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48729,48729,48729,48729,48729,50777,48697,50744,50679,46617,44536,42424,40343,40311,40278,38198,40278,40278,34037,34005,36052,36084,44502,38229,33972,38132,42325,42260,38100,42326,46551,50678,50711,50679,48631,46583,44536,44536,44568,44568,44536,52791,48497,31724,25485,35949,31757,23340,27404,29549,25486,59096,59096,40145,31728,54870,50610,42157,40075,42090,33706,31595,25226,19081,21320,23400,23336,29544,40009,35817,31625,50742,42127,25321,37966,33805,31725,25387,42160,10696,6536,8518,19016,35819,39916,39916,46254,46320,46386,35787,40144,33837,44206,35885,33741,35886,35854,35789,33676,37902,44208,27402,12775,14855,27564,48432,27434,16968,16968,33741,44207,25322,19049,17033,33806,42127,25355,27531,21194,40046,44175,23275,23275,23340,44240,42095,23339,23339,25420,42159,40014,27468,27500,29613,44208,40015,23242,12776,31693,23177,12711,16870,10565,23143,52427,48333,14757,10631,10631,10598,8550,40144,56817,50480,48465,56916,42258,35953,21196,8682,4554,6602,6732,8715,8846,15184,29810,31988,35953,12775,46581,197},
{50941,50941,48893,48861,48861,48861,48861,48861,48861,48861,48893,48893,48861,48861,48861,48860,48892,48892,48892,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,48729,50777,46649,50743,50646,46617,44536,42424,40311,40311,40278,38198,38198,40278,36117,36085,38196,38164,46582,44501,42390,44437,33971,38132,36020,36085,40343,44471,44504,46584,46583,50679,50679,48631,48663,44568,44503,44404,33871,33806,29678,31757,29677,23405,25357,31694,29677,48695,48662,40113,35921,54837,46349,37963,35851,44170,33673,27434,29354,14921,10792,19176,27496,31592,33673,31625,35818,52822,42225,31529,23210,8518,8518,8518,8518,6503,6504,8617,8517,8518,6503,8551,8518,19147,40145,25322,40209,52724,54772,54740,54772,54772,54772,54739,52659,52691,54771,54707,52627,50547,52627,52626,50546,50513,50545,52658,54771,50513,48400,50481,52626,54739,48368,46255,44175,52626,54706,46255,44174,44174,52658,52593,44110,42062,42062,52626,50513,39949,39948,42061,52593,48367,29417,12743,31661,27370,12678,23241,10564,12708,50378,48300,14693,10598,10598,10566,8518,42224,56785,48465,44206,27434,29647,17101,13005,17198,25616,36050,42323,44436,50708,52690,48334,39916,29384,10630,33904,45},
{50941,50941,48893,48861,48861,48861,48861,48861,48861,48861,48893,48893,48893,48861,48861,48892,48860,48892,48892,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,48729,48697,48697,50744,50612,46616,44536,42424,40311,40311,40278,38198,38198,42391,36117,36117,38164,40244,46550,44502,31891,31859,40244,46517,48565,31924,40310,42423,42456,42456,42488,44536,46584,48632,50711,50711,48597,38064,36050,35919,21389,33902,38097,44403,38032,35952,31758,42323,33839,40048,42128,48564,46382,42155,40011,37930,33674,25354,31370,14825,10760,10728,10759,19081,23208,37930,37962,46548,38064,14855,19081,8518,8518,8518,8518,6503,8518,8583,8518,8518,6503,8550,8518,31791,46483,25354,40242,38031,44207,35756,31595,33709,33741,31596,29483,33708,42094,27336,10532,10532,25420,52659,27337,10565,10565,31628,48400,23111,10532,10532,33740,48335,16838,8484,8484,37934,46255,12645,8516,8516,40046,44141,10564,8517,10597,44142,39916,10565,8517,12742,42028,33610,10630,14888,33741,27369,19016,27403,31627,29480,50346,44140,10565,10598,8517,8517,8517,42159,54738,48465,48431,54737,40145,40210,48531,46385,46286,44141,40078,48529,41963,25223,14725,27369,29353,21194,33709,42},
{50941,48893,48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48893,48861,48860,48892,48892,48892,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50842,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,50777,48697,48697,50744,52726,46616,44536,42424,40343,40311,40278,38198,38198,42358,36117,38165,34004,36084,42389,50808,31891,33971,46550,46550,44405,34004,42423,44535,44568,44568,44536,42488,44536,44568,46616,48664,48630,48433,48335,46286,31757,31756,33805,44338,31725,37933,29676,31757,21228,31694,54545,54871,50609,42154,40075,35851,27435,23242,29322,12712,10792,10760,8648,8615,10663,21127,37959,34129,42355,33643,27338,8518,8518,8518,8518,8649,8517,8518,8518,8518,8550,8518,8518,19017,37935,23241,42323,42257,46353,40080,35919,38064,40112,40112,38064,42225,48401,44273,40145,40145,42258,48466,44305,42225,42225,44305,48433,44305,44305,44305,46386,48499,46418,46450,46451,48499,48498,48466,48499,48531,50578,50546,50611,48531,48563,50579,50578,50611,50644,52692,52691,52659,50514,23241,50578,50513,18950,8517,16871,27304,46185,39914,10565,10630,12775,16968,21194,44304,54738,48433,48464,54737,48334,46385,52658,44337,31562,35885,37965,46482,35722,14757,12677,10630,10631,12743,14856,79},
{48893,48893,48861,48861,48861,48861,48861,48861,48861,48861,48893,48861,48861,48861,48861,48860,48892,48892,48892,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,50777,48697,46649,50744,52693,46452,44536,44470,40310,40311,40278,40246,38198,38198,36085,38165,33971,38164,36084,52856,31891,36052,46485,44372,46453,34004,44471,44503,42423,44536,44536,42423,44536,44536,44568,44568,44535,44405,39983,46286,46284,48333,42092,44272,33838,33837,29644,29678,25452,35887,54545,54806,50608,44235,40042,37931,29515,29483,31466,25161,21227,12840,10728,12808,16936,23209,29511,46580,42289,21096,14823,8518,8518,8518,10729,10664,8517,8583,8517,8518,8518,8518,8517,14921,33970,35951,40209,44305,46254,39949,35756,37869,37869,35788,33708,37901,46255,37836,27337,27337,35788,50481,37771,27272,25224,37869,50480,33610,23111,23111,39981,50480,29384,20998,20998,42094,50448,23111,18886,16838,46288,50447,18885,16805,16838,46320,46254,16805,14693,16838,46254,41963,12644,16903,27369,46222,23144,10630,16903,16903,46186,35722,16936,40079,44304,48465,52658,48431,48399,48400,46352,52658,42093,46417,50545,52691,48432,42093,33739,46450,37737,29449,27370,27534,27532,29743,50709,168},
{48893,48893,48861,48861,48861,48861,48861,48861,48861,48893,48893,48861,48861,48861,48861,48860,48860,48860,48892,48892,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50908,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,50744,48697,46649,50712,52660,46387,35986,40146,44340,44405,46453,42260,42293,42325,42227,40146,33709,42193,42225,42192,40144,42225,50547,50578,46451,31825,36051,38229,31858,31859,36020,25454,27632,36084,36019,38164,27567,23341,19115,29580,29513,35754,41963,29482,25290,25290,25289,25355,23275,23177,37708,39917,35755,31593,25320,23143,18983,21032,18919,18919,16871,14791,10695,12743,14791,14823,14790,25354,29482,23079,18919,8518,8518,8517,12711,10566,8517,8518,8517,8550,8518,8517,8517,23210,33937,38063,40242,31790,46287,40014,33741,35886,37934,35821,33709,37934,46320,33740,17001,16969,27532,48432,33708,19082,19114,31661,44175,29548,21194,21194,33773,44207,29515,23307,23339,37934,44207,27468,25388,25420,40112,44208,27468,25420,27500,42159,44175,29613,29613,29645,42127,40047,29645,29645,31726,40047,29547,31725,38064,44305,37867,31595,35918,54770,54738,48498,54739,46351,44304,52723,52723,52625,44239,52724,50675,52658,48367,44271,37933,37967,42225,44402,48595,52723,44337,48562,56883,163},
{48893,48893,48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48892,48860,48860,48892,48860,48860,48860,48860,48860,50908,50908,50908,50908,48860,48860,50908,50908,50908,50908,50908,50908,50908,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48762,48730,48730,48729,48729,48729,52791,52659,50677,50646,50646,54739,50677,46615,46220,52527,50548,50548,48434,46420,46387,48269,48367,48269,52396,52528,50448,52559,50447,52527,52559,40047,29614,29614,29647,25453,21195,25421,14824,16872,23243,21162,19082,10630,10630,10630,12712,10630,12711,12678,10598,10598,10598,10598,12777,12874,10598,10597,10597,10597,10597,10598,10565,10598,10565,10597,10565,10597,10565,10565,8517,10565,8517,8517,8517,10565,10565,8517,8517,8517,10565,8517,8517,8517,8517,8517,8518,8518,8517,8517,8550,25486,29645,42258,52692,54772,54739,52691,52692,52691,54707,52626,52627,54739,54707,50513,48433,50481,48401,48400,48400,48368,50546,54739,50513,48336,46256,50513,54706,48335,44142,48399,50513,52626,44174,42029,42029,50545,52626,42029,39949,37869,50513,50512,37835,37803,37836,50512,50447,35691,35691,35756,50447,46254,35789,39981,33676,44207,50577,48399,35755,46254,52690,52592,31561,42094,52658,54705,33610,37933,50545,54673,33642,35788,48432,48464,39916,35756,48433,46188,39915,39981,48400,50480,131},
{48893,48893,48861,48861,48861,48861,48861,48861,48861,48893,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,48860,50908,48860,50875,50875,48859,50875,50875,50875,48827,48827,50875,50875,50810,48795,50843,50875,48795,50843,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48729,48729,48729,48729,48729,50744,48696,50744,48664,54806,56951,54937,52660,48500,44406,44373,44372,40213,40115,31531,33676,31531,33642,35755,33675,35787,35820,31530,35756,23209,19116,19049,21097,16968,14890,14923,16936,27176,25193,19049,16937,10598,10631,8550,10631,10566,10631,8517,8517,8518,10598,10598,10631,17069,10597,10597,10597,10597,10565,10598,10598,10598,10565,10598,10565,10565,10565,10565,10565,10565,10565,10565,8517,10565,10565,10565,8517,8517,10565,8517,8517,8517,8517,8517,8518,8518,8517,8517,27436,12710,12808,27533,27596,44174,35789,29515,31628,33708,31627,29515,31627,42126,31563,14757,14854,23274,52691,31562,8484,8484,25322,48367,27337,8517,8517,29515,48335,23111,8517,21192,31627,46254,16870,8516,8517,35821,46254,12677,10565,10597,37869,42029,10597,10597,10630,37932,39916,12678,12710,14855,39916,35723,12775,14855,16968,37868,31594,18983,12742,10598,16806,14725,8517,8517,14725,12645,8517,8517,10565,10565,8485,8517,10565,10565,8485,8517,10565,10533,8517,10565,10565,10565,43},
{48893,48893,48861,48861,48861,48861,48861,48861,48861,48893,48893,48861,48861,48861,48861,48860,48860,48860,48860,48892,48860,48860,48860,48860,48860,50908,50908,50908,48860,48860,50908,50908,50711,50809,50875,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50842,50842,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48729,48729,48729,48729,48729,48729,48697,48697,46649,52823,52791,44536,42424,40311,40311,40278,40246,38166,21196,19082,23210,23242,23241,31595,27402,31595,31562,31595,29514,27434,4620,10664,21032,19049,16970,14890,16872,25031,25161,16871,21064,10631,10598,14759,10599,10566,10599,8517,8517,8517,8518,10598,10566,10664,10566,10566,10565,10565,10566,8518,8517,10598,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8517,8517,8517,8517,8517,8517,8485,8517,8518,8551,6502,6502,6502,8550,10631,14953,29613,38064,46353,42192,38032,40112,42225,42225,40177,42257,48466,46386,40177,42226,44338,48498,46418,44338,44370,46418,48498,48499,46451,46483,48531,48498,48531,48564,50644,50611,50578,50644,50644,52724,52692,52724,52724,52724,52724,52692,52692,52724,54772,54804,54772,54739,52691,52659,52691,52691,54739,52626,52626,52626,52658,52626,50513,39917,10631,8551,8551,8550,8550,8550,8550,8518,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,40},
{48893,48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,48860,50908,50908,50908,50908,48729,52791,52759,52824,52857,50842,50874,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,50842,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,48697,48697,48664,48664,52790,54839,44536,42424,40343,42358,33874,40246,31826,14954,14889,14824,19049,19081,29515,23208,33675,25256,33674,31595,29483,36051,25420,25584,27861,16938,14857,14921,23113,23178,12808,23210,10664,16937,12809,12744,19082,10696,19082,8616,19083,10728,16970,19115,23309,19148,19148,19180,17034,23341,17002,14889,19147,23341,21260,21260,23405,19148,23406,17067,23406,21260,27533,23340,21228,25518,21227,27566,23405,29711,23372,29744,21260,29711,23404,25582,27598,23437,31824,27598,14856,8550,31726,56885,58900,39915,35755,42061,39980,44142,37837,42127,50480,44141,27336,27304,33643,50448,41964,25192,23112,33708,50480,37804,21096,21063,37868,50480,33610,25224,27369,42094,50447,27272,16805,16805,40013,50415,23078,14725,14693,44174,48367,16805,16901,12612,44174,46189,12612,10564,12677,46254,41996,10564,10532,14822,46254,35690,10532,10532,8517,8517,8517,10564,10564,10565,8517,10565,10564,10564,10565,8517,10564,10564,10565,10565,8485,10564,10565,10565,10564,10532,10565,42},
{48893,48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,48860,48860,48860,50908,50908,50907,50875,52890,52856,52792,52791,52856,52857,50841,50874,50875,50875,50875,50875,50843,50843,50843,50843,50843,50842,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48729,48729,48729,48729,48729,48697,42325,42259,48565,50480,52627,42293,42391,46551,46517,21394,25620,29649,23440,23441,21361,29550,37934,46418,37998,44337,40078,46351,37933,19049,48598,31629,27664,25814,21392,29613,25387,39917,31596,16969,33742,8583,14889,23307,10728,31693,12776,35822,12776,31661,14920,19115,23242,12744,27435,8550,25420,8517,29581,12808,19081,40045,44206,42126,44271,25387,8648,29613,6567,29678,8647,29482,25387,8680,29613,8647,31726,8680,29711,10760,27598,19082,29679,31693,27565,37999,23404,40145,31791,33773,25388,38032,56917,56884,33805,29613,31693,31693,31725,31726,29677,42159,35821,19114,23339,25420,44175,33709,21194,21194,29613,42159,33741,23340,23307,33806,44239,33741,29677,31757,37934,44207,29580,25387,25419,37967,44240,31693,29613,29613,42127,44239,31725,33838,31693,40079,42127,31693,31725,33773,42159,42159,33806,33838,35886,42159,42127,35886,19016,8550,8551,8550,8518,8518,8518,8518,8518,8518,8518,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,39},
{48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,48860,48860,48860,48860,50908,50908,50908,50908,50908,48859,50875,50809,52857,52857,52857,52856,52824,52856,52856,50809,50842,50842,50843,50843,50843,50842,50842,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,48729,48697,44339,39983,44145,54836,56884,44403,46517,46517,44372,31861,34005,29649,25588,25588,23540,27503,25191,42094,19016,35788,33676,44175,42095,33773,50612,44305,40309,25781,19378,42225,40080,48499,42160,40113,40080,33806,46483,40144,46516,44337,46418,50611,42193,50676,40112,50612,40079,48564,37999,42290,38031,42258,44306,35919,27532,25386,29547,31627,23306,29644,25484,38031,25516,40080,23371,37965,31791,19376,37999,29644,42192,27596,42224,33933,44369,38030,44369,44304,46417,48498,46449,46450,35983,27468,14954,33872,56884,56884,56885,56917,56917,56917,56917,56918,54805,56885,56885,56917,56917,56885,56885,56917,56885,56884,56884,56884,56884,56884,56885,56884,56852,56884,56885,56884,56884,56884,56852,56852,56852,56820,56852,56820,56820,56787,54707,54675,52594,52561,50481,50448,48336,46255,46255,44143,42030,41997,37804,37804,37804,35756,35724,35691,16839,8550,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10564,10565,8517,40},
{48893,48893,48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50875,50875,48827,48795,48795,50842,52890,52889,52824,54872,52824,52824,52856,52857,50842,50842,50842,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,48729,48697,50712,46452,40080,54674,56686,40014,44338,40145,40049,19347,19347,23343,23441,21363,21363,25326,14790,31627,27303,42061,21030,44174,48465,37933,50677,33709,31825,25814,19345,37999,31627,46385,37933,25355,42160,16871,46353,31595,37966,46319,27369,50545,19015,48466,23208,40014,25322,29547,31628,21161,35885,16936,44240,14855,29676,37835,27435,39916,23209,35821,37771,44207,39882,40047,35691,44109,33807,17198,38030,35689,42127,33577,40014,35658,39982,35690,35789,39948,33611,42126,29449,42193,23371,48498,50579,54772,54739,54771,54739,54739,56852,54772,56819,56819,56819,56820,56787,54675,52594,52626,54706,52658,52755,52756,52756,52723,52723,52723,52723,50610,52691,50512,48398,54673,54739,52659,52626,50514,50579,42159,42224,40144,31659,33806,29677,29645,23305,17000,12842,10631,10565,10565,10565,8550,8550,8517,8517,8518,8517,8517,8517,8518,8517,8517,8517,8517,8517,10565,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,8517,10565,10564,10565,8517,10564,8517,8517,40},
{48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50907,50907,50875,50875,50875,50875,48795,50842,50843,50875,52890,52857,54905,52824,54872,54871,52791,52824,52857,50809,50810,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,48697,44243,42228,48566,46386,44175,48399,40079,40177,44338,40146,25653,29846,25554,21395,17300,17332,27601,25355,35886,27500,37998,25452,48433,50546,44272,56951,33741,27632,25814,17298,33773,21129,42160,35852,25388,33709,14823,48400,31530,35853,46351,27370,50545,19015,46418,23241,42224,27434,35919,33675,25420,37965,21194,46353,16936,31692,12743,19049,21162,8680,27533,14954,19082,8744,19146,14889,21162,21130,14921,19082,14889,19082,14922,19082,14921,17002,12743,10695,14888,10727,14888,10728,17001,10793,17001,12809,16969,14888,14888,16969,14954,17034,15114,17000,16936,14791,14824,14824,10663,10598,19179,29516,27597,36210,36210,36210,34129,34097,36178,34129,27758,34129,35982,40046,44109,35822,31695,27469,21326,48564,38096,48627,48562,27627,44467,46515,46579,38094,23402,19212,10696,12774,8549,8517,8550,10597,8517,8517,8518,8517,6502,6503,6502,8517,10564,10564,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10564,10564,8517,10564,8517,8517,39},
{48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50842,50842,52857,54872,52791,52791,52823,54871,52791,50776,50809,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48730,48729,48729,48697,33548,35856,40080,41966,35724,31531,29450,42192,46386,46323,23508,27701,25490,21427,19348,19349,27471,31465,50415,29449,46286,31593,50544,50578,48465,56918,33774,27632,25814,21393,35853,19048,37967,35820,23442,35888,14823,46287,27369,31660,46287,25354,50513,21096,50514,25289,46288,27434,37999,33675,25420,37965,23274,46353,21161,31724,14887,23339,21194,12874,27500,8647,8615,8680,6599,8616,8615,8648,8648,8648,8648,8615,8680,6567,10761,8615,10793,8648,10760,10728,10793,10793,10825,13067,10891,13004,8713,14986,10825,12971,10858,6632,6695,8679,8615,10793,10760,10728,14921,10598,14954,10728,17229,34130,34162,34162,34129,29903,32016,31984,23501,34129,34129,35984,25453,19149,17134,19247,25585,44371,42257,44434,46515,44435,44403,44403,44435,44435,33968,31824,25387,8517,8517,8517,8550,8517,8517,8517,8518,8517,6503,6535,6503,8517,10564,8517,8517,8517,8517,10565,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10564,10564,8517,10532,8517,8517,39},
{48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50908,50875,50875,50875,50875,50875,50875,50875,50875,48795,50843,50843,50843,50843,50842,50842,50842,50810,50809,52824,52791,52791,52791,52758,50711,40244,48729,48729,48761,48730,48729,48729,48729,48729,48729,48729,48663,35692,37903,40047,48367,40079,44174,40078,52723,37901,37838,23508,25653,29715,21459,19348,19381,25391,23079,48302,29383,50414,31496,52495,48400,50610,52757,35855,27632,25814,21360,29580,25387,35757,33742,23474,35824,16968,35821,29612,29581,40079,27435,42127,25322,37967,29547,35853,29612,25484,33773,21291,33805,23307,40079,27532,29612,44173,31691,23307,15018,40111,35854,23439,25487,17229,23471,19245,27664,25583,25551,25551,27631,29712,25551,27632,23503,27664,27631,14953,12840,12873,17001,12841,17002,10793,14987,10728,17034,12841,38063,40143,12841,10793,10793,8647,14986,12840,10760,16969,8517,10761,23599,34033,27727,29903,27758,25549,29872,29904,27759,17228,29936,32016,31921,31826,33972,36052,36051,38098,35953,48499,46418,44305,40111,48629,54968,50709,44273,29580,27631,38031,10661,10629,10565,8550,10597,10564,10564,8517,8517,6502,6502,6502,8517,10565,10565,10565,8517,10565,10565,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10564,10564,8517,10532,8517,8517,39},
{48893,48861,48893,48861,48861,48861,48861,48861,48861,48861,48860,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,50908,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50842,50842,50843,50842,50842,48794,48762,48762,48762,48762,50809,50777,50678,52791,54936,48399,52659,54773,54806,54806,54806,54773,54838,54805,54772,54771,60684,58735,54574,56654,56654,56621,56622,61011,50349,41998,25588,29813,23474,21428,17300,19348,23375,27402,52494,29449,46254,35722,48333,37868,40014,50645,42224,38130,25846,25552,42160,54837,54707,50578,46420,44242,44207,52593,50448,54674,50448,50513,52593,44207,50481,42094,48497,42126,42159,39981,44142,40046,37966,44239,33805,29612,33740,27466,35853,25419,35919,31629,23373,21261,19115,23341,21228,27502,23341,23373,21325,23341,25486,19147,23438,19147,27534,23340,21227,19179,17066,19180,14986,21422,15083,19277,8712,19244,12971,29613,27500,10728,8648,8648,8680,8615,8680,8680,8582,8517,10598,14920,29679,36114,32081,38291,42256,31952,38258,31855,10890,12969,21485,34129,42256,48269,44108,44140,48269,48466,46221,41963,18951,16966,44501,63387,52789,23176,12775,25486,40079,14887,10629,8517,8518,10630,10597,10565,8517,8517,8517,8517,6502,8517,8517,10565,8517,8517,10564,10565,8517,8517,8517,8517,8517,8517,10564,8517,8517,8517,10565,10564,10564,8517,10564,8517,8517,39},
{48893,48861,48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,48859,50908,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50842,50810,50842,50842,50842,48794,48762,48762,48762,48762,48762,50744,50710,54705,48332,52659,52725,54805,54805,56885,54773,56820,54707,52691,54739,56622,54607,54608,58833,58832,58800,58832,58931,44142,39951,19316,25652,40211,40212,40244,38163,38162,42322,44272,44305,46451,38065,44339,48564,46483,50677,35886,27631,25879,31794,44207,39981,44141,42061,33776,38002,33609,46156,35690,48334,46287,42029,44174,35787,40079,39949,42126,35820,25386,39981,25192,33707,16967,35853,23306,29579,25289,21161,29514,14856,29516,12711,10663,8808,8647,10695,10696,8648,10793,8615,8648,6567,8648,6568,8616,6568,10695,10728,10695,12808,10728,12873,10760,12938,10857,10858,8777,10825,10891,8778,10923,8648,6664,6632,8680,8648,8712,8680,8615,8517,8517,8517,16969,38162,29969,29936,25646,27857,36178,23468,11020,27889,27888,34129,42255,52364,50282,50250,50283,50382,50449,33579,16903,21259,52888,63355,52789,19114,12906,23471,42257,29742,27597,19243,10792,8549,8549,8517,8550,14953,25549,21291,10760,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8517,8517,10565,8517,8517,8517,8517,8518,8518,8517,10565,8517,8517,39},
{48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,48859,48859,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,48794,50809,50810,48794,48762,48762,48762,48762,50743,52725,44339,46320,48466,50777,50777,50777,48729,48729,48729,50745,48696,48695,48695,48630,50646,52758,56950,58999,56951,54871,52790,50644,50612,25621,33974,25424,29683,35856,31763,33744,50546,54739,52658,52723,52723,54771,54836,50611,50677,35919,27664,25879,31826,42127,50645,48368,46319,35888,38034,37868,42060,35659,46255,46255,37803,35820,42062,46288,37934,40013,29580,29547,40013,18951,35821,19048,33805,21161,33773,16936,16969,25355,8615,27501,8648,8582,2695,6599,6567,8615,6535,12906,8648,8745,6632,8809,8777,10825,12938,10760,15018,10760,15018,10792,15083,10825,21421,15083,13100,15083,10858,15115,15051,19341,10793,9067,10922,12938,12938,13002,12906,8615,8550,8517,8518,8583,10695,27565,35952,31986,25451,19081,14920,17293,27887,29968,34129,40174,50283,52330,52330,52331,50283,52561,44107,27433,33937,44405,46485,42292,33938,31761,31793,42194,38032,38032,38032,38065,35952,33872,29646,29646,29548,29516,16970,8617,6503,6503,6536,6536,6503,8550,6503,6503,8518,8518,8518,8517,8517,8517,8516,8517,8517,6503,6503,6503,8551,8517,8517,8517,37},
{48893,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,48859,50907,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50810,48762,48762,50810,48761,52791,52758,52824,50777,48696,40179,44307,48762,48730,48730,48730,48730,48729,48729,48729,48729,48729,48697,46649,46649,46617,46617,46617,44536,46584,48499,46126,19315,19316,23313,25456,19183,17268,27471,50512,52722,54770,54737,54769,52658,56850,44240,48597,33806,27534,30039,38099,42095,54838,46385,46353,44174,37934,42094,39981,35691,46320,46320,37869,37966,46320,46320,40078,40046,35950,23338,33708,14823,33740,12743,31693,19048,33772,19049,17033,25419,8712,29614,17034,25485,8904,25517,12841,19146,25419,12905,27598,14888,27566,14888,27598,17001,21292,21226,14985,29613,12872,33904,23371,38128,56720,52692,17130,27695,19210,21421,29678,19276,29678,12840,10760,10728,14921,14953,33806,10630,6568,8550,8517,4553,8517,25485,29678,17196,12775,12776,10695,19373,32049,32049,34129,40207,50283,50250,52330,52330,50250,50250,50250,50284,46188,48236,48268,46155,31464,18951,18984,18951,18919,12678,14824,14693,12645,12645,10565,10533,8485,8517,8485,8518,6503,6535,8550,8550,8517,8484,8516,10564,10533,10533,10533,10564,8517,10565,10564,10533,10565,8517,10565,10564,10564,10564,10564,10564,42},
{50941,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,48859,48859,50875,50875,50875,48827,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50842,50842,48762,48696,54738,50711,48729,48729,48729,52791,33806,44274,48762,48730,48730,48730,48730,48729,48729,48729,48729,48729,48697,46649,46649,46617,46617,46617,44536,44503,52724,52692,25620,31926,19218,21395,19348,17300,27602,54804,56949,56884,56917,56917,54804,56949,48499,50645,35919,19213,30071,40147,40014,50513,46353,48466,37869,33708,48336,23144,46288,44272,46320,42160,48433,44239,46320,40079,44240,42192,23274,31627,19049,40079,19114,35918,23371,38030,46286,46319,46351,46352,44272,46286,44272,44270,44304,37965,50479,46221,48268,44239,46253,46352,50545,52626,46286,50577,46350,52625,50480,52559,48497,52592,50578,37998,46451,31822,40243,31789,36081,40112,33968,33807,10858,10761,17034,29646,21259,38032,12808,4618,8550,6470,4587,8517,23274,25355,10923,8583,8550,8550,10890,19275,23630,32049,38159,50283,50250,50250,50250,50217,50249,50250,50250,50250,50250,50250,43977,20966,10566,8551,8551,8583,8517,10598,10565,10565,10565,10565,8517,8517,8518,8583,4554,10794,19147,25454,40046,40079,31594,19016,18984,8517,10533,10533,10565,10565,10565,10565,10565,10564,10564,10564,10564,10564,10564,10564,10564,42},
{50941,48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,50908,48859,48859,50875,50875,50875,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,48794,48794,50810,48761,48730,48730,46649,46649,48665,58996,54870,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48697,46649,46649,46617,46617,46617,44536,42423,52725,54642,29781,27765,27667,25618,23507,23475,35823,52626,54804,56885,54837,56917,56916,54837,38197,50645,38065,36083,27926,31825,46418,48531,54805,50611,52692,48399,46353,52724,50578,54837,54837,56950,54804,56917,54869,54869,56917,50611,46483,48497,44402,50578,44338,48498,46417,40143,31627,31628,35884,23305,33838,23306,40080,21192,42159,21160,31626,35852,21258,40080,23338,38064,23338,37999,27499,33838,29611,27564,38063,27596,42225,25483,42290,25581,42322,27597,33936,29677,31888,38032,27630,40113,14986,14986,8582,10728,8647,33871,8647,6634,8550,8518,6634,8549,21130,27435,10696,10663,12743,16936,23307,21518,21550,32049,38159,50283,50283,50250,50217,50185,50217,50217,50250,50218,50217,50217,41832,16805,10565,10565,10565,10565,10565,10565,8517,8518,8551,8517,8518,8518,10598,18951,8517,10565,8517,10598,12645,12645,10565,8517,8517,8517,10533,10533,10565,10533,10533,10565,10565,10564,10564,10564,10564,10564,10564,10564,10564,42},
{50941,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,48859,48859,48859,48827,48827,48827,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50842,50843,50842,50842,48794,48794,48762,48762,48762,48762,48762,48762,48730,48762,48730,48730,48730,48730,48729,48730,48730,48730,48729,48729,48697,46649,46649,46617,46617,44569,44536,42423,50710,52628,40181,44374,46324,40310,38199,38167,44047,37837,50611,59030,56917,61110,56917,58997,48563,54838,42192,33839,32119,48563,48466,52756,50578,50546,37836,41996,48465,44174,48401,44175,50545,46288,48433,44208,46289,46288,44208,29516,25355,23210,23307,18984,14791,14856,14888,16936,14921,17066,13032,10986,17161,15178,33903,8968,38063,23501,29644,38030,21388,33904,25516,35984,17228,31856,31790,31823,36015,29742,40209,29742,42290,27694,40242,29806,38162,36048,36017,38129,36081,42322,33936,44371,21325,15116,14986,17131,17131,36081,17098,6601,8583,8550,6568,8550,12842,21227,35789,39982,39949,42127,44208,36081,33936,31822,29547,33513,31401,29352,29352,27272,27239,27206,27207,27174,25126,25094,18853,10565,8517,8485,8517,8518,8518,8518,8518,8518,8550,6569,4651,8649,10565,8485,8517,8485,8517,10564,8517,8518,8517,8518,8517,8517,10532,10564,10565,10532,10533,10565,10565,10565,10564,10564,10564,10565,10564,10564,10564,40},
{48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,50908,50908,48859,48859,48859,48827,48827,48827,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48730,48730,48729,48729,48697,48697,46649,46649,46617,46617,46617,44536,42424,48597,50517,38232,38264,44276,40278,38167,40149,41805,33647,46419,54837,54805,56820,54772,52724,42193,46485,44338,38131,32087,44372,48565,50710,48597,40081,27535,29615,42319,42095,42063,39982,42095,42094,40046,37934,44273,42062,37968,40211,37968,29614,33937,38098,37968,33742,35855,37935,40080,33774,31791,33904,35953,38065,37968,38032,35821,31694,33774,40145,46517,44370,44305,33709,29614,31629,31628,27500,27467,27500,38000,35919,33806,33806,35854,23275,27468,31661,27435,23275,27468,16936,14888,16936,27435,50676,33838,29646,31823,25452,15019,8616,6503,6503,6503,6470,6502,8518,10565,12678,10630,52822,33901,14921,8582,8681,6699,8714,4554,4587,4554,4521,6536,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10565,10564,4521,4587,8681,8517,8517,8517,8517,8517,10564,8485,8517,10663,8518,8485,8517,10564,10564,8517,10565,10565,10533,10533,10565,10564,10564,10564,10564,10565,10565,10564,41},
{48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,48860,48859,48859,48859,48827,48827,48827,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48729,48729,48729,48729,48697,48697,46649,46649,46617,46617,46617,44536,42424,42359,42262,40280,42360,42229,36118,38133,40051,38067,38067,36019,38132,38131,38131,31859,38067,42291,44404,46485,42423,38295,42456,44504,42456,42423,40180,29780,33907,54968,44369,40110,33870,33870,35949,35949,35918,54903,40080,33906,44371,59064,27598,48695,61209,54577,35789,23371,37965,46386,42127,21163,25356,29516,23243,19146,33739,23240,25353,27530,40079,56983,48564,40080,21261,17132,17001,16969,14920,16969,29613,38032,35887,33742,29548,27436,23243,23243,25355,21162,21162,21130,14889,14922,10664,25322,50577,14791,17034,48499,12744,21228,12810,6535,4553,6503,6503,6536,8519,8519,4521,6536,14856,10597,8517,8550,10695,10598,8517,8517,8517,8517,8517,10565,10533,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,8517,8517,8517,8517,10565,8517,8550,8517,8517,8517,8550,8517,8485,8517,8517,8517,8517,8518,8517,8517,10564,10564,8517,8517,8517,8517,8550,8518,8517,8517,8517,10565,8583,8517,8517,35},
{48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48860,48859,48859,48827,48827,48827,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48730,48730,48730,48729,48697,48697,46649,46649,46617,46617,44569,44536,42424,40343,40311,40278,40246,38198,34037,36085,35923,40115,48468,50646,48630,36020,42291,31859,52758,56918,54838,46517,46519,44504,44536,42488,42456,44536,31893,25588,29747,59096,50577,46318,33837,31726,33806,38029,40143,50644,29613,29713,38066,56918,29678,46517,61177,50416,23306,27531,35853,27468,29581,23243,21196,25356,19148,23340,21130,23243,19082,27531,33835,50612,35952,38000,48597,40145,44207,37642,33611,10598,12679,23210,23145,16774,12679,12679,10565,10565,10565,10565,8517,8517,8517,19114,12842,8649,14759,8517,8517,12646,8550,8517,8518,8518,8517,8517,8517,8517,8518,8517,8518,8517,8518,8518,8517,8517,8517,8517,8517,8517,8517,10565,10565,10565,10565,8517,8517,8517,8517,8517,8517,8615,8517,10564,10564,8517,10565,10565,10564,10565,10564,10564,10564,8517,8517,8517,8517,8517,8517,8517,10598,8550,8517,8517,8517,8517,8517,8518,8517,8517,8518,8551,8518,8517,8517,8517,10564,8517,10565,10565,40},
{50941,50941,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,48828,48828,48860,48859,48859,48827,48827,48827,50875,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48729,48729,48729,48729,48697,48697,46649,46649,46617,46617,44569,44536,42424,40343,40311,40278,40246,38166,36117,40278,44244,44275,52759,44373,40213,33939,40179,33972,31859,40180,42391,38230,42423,42456,42488,42456,42488,42488,36149,31990,40311,57081,42192,46351,40078,35886,31757,35949,38030,54870,35985,33906,33905,61241,48662,33874,48532,52465,29580,23372,31628,25486,27533,21228,21228,21228,17099,12875,12842,12809,14955,17034,29578,42193,35920,25518,42193,38000,52658,33579,29418,8518,6536,8551,8518,8518,6503,8551,8517,8517,8517,8517,8517,8517,8517,16969,14922,6569,8518,8517,8517,8517,8517,8517,8517,8518,8518,8517,8517,8517,8518,6470,8517,8517,8517,8517,8517,8517,10565,10565,8517,8517,8517,10565,10565,10565,10565,10565,8517,8517,8517,8517,10565,8517,8517,10565,8517,8517,8518,8517,8517,8517,8517,8551,8583,8518,8518,8517,8517,8517,8517,8518,8681,6668,8517,8518,8518,8518,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,40},
{50941,50941,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,48828,48828,48859,48859,48859,48827,48827,48827,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,50843,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48729,48730,48730,48730,48729,48697,48697,46649,46649,46617,46617,44569,44536,42424,40311,40311,40278,38198,38166,36085,40278,36084,36084,33972,36052,40245,31859,42291,31827,27731,33972,42358,38165,42423,42456,42456,42456,42456,42488,42488,44536,44568,52888,42257,46287,42126,35919,31790,35918,37934,56819,44272,33809,29647,52562,42096,19247,33938,52464,40080,35984,35854,35886,27501,33773,27435,27500,27436,10794,10762,8681,12809,8681,25386,37998,40079,38065,37967,42194,46256,42030,33709,14759,16970,10599,8551,8584,6504,6536,8518,8518,8517,8517,8517,8517,8549,12744,10860,6764,6568,10696,8550,8518,8517,8517,8517,8517,8518,8517,8485,8517,6470,19410,12711,10565,8517,8517,8517,8517,8517,10565,10565,8516,8517,8517,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8583,10762,8517,8518,8518,8518,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,40},
{48893,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48859,48859,48859,50875,48827,48827,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,50843,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48729,48730,48730,48729,48729,48729,48697,46649,46649,46617,46617,44569,44536,42424,40311,40311,40279,40246,38166,36118,38133,36052,36084,36052,34004,44438,33971,42291,31859,38164,46550,48598,38099,42358,42424,42456,42456,42456,42488,44536,44536,42455,52856,54772,46319,42126,37999,33903,37998,40079,42159,25421,29648,25454,27502,23439,17200,27632,52497,33774,23308,48368,29516,31693,37901,25388,42062,19180,12842,17100,14955,14987,14955,21227,33739,31658,37934,21130,29646,31759,35920,37870,41867,29386,21098,10697,10632,6504,6503,8518,8518,8518,8517,8517,8517,10597,10597,12776,10663,8551,8583,8550,8518,8517,8517,8519,8518,8517,8550,6503,17133,8517,19214,12679,10598,10565,8517,8517,8517,8517,10565,10565,8517,8517,8517,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,8518,8517,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8518,8518,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,39},
{48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48827,48859,48859,50875,48827,48827,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48697,48697,46649,46649,46617,46617,44569,44536,42424,40311,40311,40279,40246,38166,36085,40246,34004,38132,38132,36052,44405,33939,52855,33939,38132,38100,46583,46518,42423,42456,42456,42456,42456,42456,42488,44536,38229,44436,48499,38031,40077,37966,33838,38030,40046,37966,27566,25520,25486,25453,40276,19312,25489,33680,21229,8616,31628,23340,35886,25420,37966,23275,31694,14890,16970,16971,17003,14922,17067,19115,16937,33807,25454,31759,27567,35953,35789,47945,39560,12646,8550,12646,8551,4488,4488,6537,6504,6503,6503,8518,8518,8517,8517,8518,8551,8584,8551,8518,8517,8518,6504,8517,8517,8518,10696,29778,8485,10598,8517,8518,8518,8517,8517,8517,8517,8517,10565,10565,8517,10565,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,6535,10598,8517,8517,8517,8517,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8518,8518,8518,14663,8517,10565,8517,8517,8517,8517,8517,8517,10565,10565,10565,40},
{48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48860,48859,48859,48827,48827,48827,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48697,48697,46649,46649,46649,46617,44569,44536,42424,40311,38231,40278,40246,38198,36085,38165,36052,36052,34004,38132,33971,27731,38099,31859,29779,34005,42423,44438,42423,42456,42456,42456,42456,42488,42488,44536,44536,40310,31727,44207,42095,37999,40177,38031,37999,37967,35951,38065,25454,21260,52888,27697,23440,50384,40113,8616,33741,29613,35789,31693,31563,29580,27501,17003,17036,14923,14923,14923,17036,19116,14956,33971,36018,29613,27566,44404,27404,43850,35529,10566,8550,6502,6503,6503,4489,6504,6503,6503,6503,6503,8550,8518,8517,6503,6536,8648,12743,8518,8517,8517,6536,8518,8517,14824,8551,8583,8518,8518,8518,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,10564,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,10565,8517,8517,8517,10565,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8518,8518,6536,8584,10565,10565,10565,10565,8517,8517,8517,8517,8517,8517,10565,40},
{48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48827,48827,48827,48827,48827,48827,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48697,46649,46649,46649,46649,46617,44569,44536,42424,40311,40311,40278,38198,38166,34038,36117,36084,36084,34004,38132,36051,31891,36083,33971,29780,38132,44503,44438,42358,40375,42456,42456,42456,42488,42488,44536,38197,31858,29647,35888,35855,40079,31757,31789,33805,33806,35887,38033,27567,23373,50710,29713,23439,52562,37935,17035,42094,35789,31629,37869,27468,42094,23308,14890,14956,14891,14890,14890,14923,17003,12810,35856,37935,29613,29614,46484,17035,39625,31305,10599,8551,6601,4521,6536,4489,4488,6503,6503,8551,6504,8550,8518,8550,8616,8584,10696,12679,8518,8517,8517,6503,8518,8517,10598,6471,6471,6471,8518,8518,8518,8518,8517,8518,8517,8517,10565,8517,10596,8517,10565,10565,10564,10564,10565,8517,8517,10565,10565,8517,8517,8517,10565,8517,8517,8517,8517,8517,8517,10565,10565,10565,8517,8518,8518,8550,6503,8518,8517,8518,8485,8517,8517,8517,8518,8518,8517,8551,10564,10565,8517,10565,10565,10565,10565,10565,10565,10565,10564,40},
{48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48827,50875,48827,48827,48827,48827,50875,50875,50875,50875,50875,50875,50843,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48729,48729,48729,48729,48697,46649,46649,46649,46649,44569,44569,44536,42424,40311,40311,38198,38198,38198,36118,36117,38133,36052,34004,36052,40211,40211,42291,34003,31891,50710,52791,48631,44438,42424,42456,42456,42456,42456,42488,44536,40342,33905,37999,35951,37998,37998,33838,35918,40078,35886,33775,25520,25455,23373,46485,29778,29744,52529,29581,21261,35854,25355,25355,25356,25355,25355,25453,14922,14923,14891,14890,14858,12810,12842,12843,8681,29678,29614,29614,44272,8650,8616,10697,6537,6569,6570,4554,6536,6503,8518,8518,8518,8518,8518,8518,8518,8518,8551,6536,8584,10631,8518,8517,8517,6535,8518,10533,8518,6471,8518,6503,8518,8518,8518,8518,8517,8518,8517,10565,8516,12677,10596,10564,10565,10565,10564,10532,10564,10565,10565,10564,10565,10565,8517,10565,10565,8517,8517,8517,8517,8517,8517,10564,10564,10564,8517,8518,8518,8517,8517,8517,8517,8518,8517,10564,8517,8517,8518,8518,8518,8518,8517,6503,6503,8517,8517,8517,8517,8550,10631,8550,8517,34},
{48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,50875,50875,50875,50875,50875,50875,50843,50843,50843,48794,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48729,48729,48729,48697,46649,46649,46649,46649,44569,44569,44536,42424,40311,38231,40278,38198,38198,38166,38165,34004,38132,34005,31892,44372,46419,46451,40212,42357,54871,46583,46648,42456,42456,42456,42456,42456,42456,42488,44536,44536,40277,35920,35952,37968,40112,33871,37998,37998,33773,33839,25552,21294,23309,42291,57081,40177,52561,29548,38163,42192,40079,48401,35886,48433,33774,44240,21164,19116,19116,19084,19084,19116,17035,8650,8649,27533,29614,27502,37967,6667,6666,8649,6536,8682,10729,10730,8518,8518,8517,8517,8517,8517,8517,8517,8517,8518,8550,8584,8616,10663,8518,8517,8517,8517,8517,8517,8517,8518,8518,8518,8518,8517,8517,8518,8517,8518,8517,8517,10565,10564,10564,10565,10565,10564,10564,10564,10564,10564,10564,10564,10564,10565,10565,10565,10565,8517,8517,8517,8517,10565,8517,10565,10564,8517,8550,8518,8517,10597,10565,10565,8517,6470,6471,8518,10565,8517,8518,8517,8517,8550,8518,6601,6504,8517,8518,8583,8518,6503,10598,8550,8517,36},
{48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48859,48859,48827,48827,48827,48827,48827,50875,50875,50875,50875,50843,50843,50843,50843,48795,48794,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48729,48697,46649,46649,46649,46649,44569,44569,44536,42424,40311,38231,40279,40246,38198,36085,38166,34004,36052,34005,29812,38100,50579,44339,38132,40245,46517,44470,50711,44471,42424,42456,42456,40408,42456,42488,44536,44536,40244,37968,40113,35823,40112,31757,33869,40046,35886,33840,23473,23374,21228,44370,61143,52659,50287,35789,29614,52626,46320,46320,44240,46353,44272,44273,27469,27502,27469,27469,27469,27469,25357,8681,12940,27501,29582,25453,31760,8715,8616,8616,8584,8617,8550,8551,8517,8517,8517,8517,8517,8517,8517,8517,8679,14887,21093,10630,8583,8582,8517,8517,8517,8517,8485,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10597,8517,10565,10565,10564,10564,10564,10564,10564,10564,10564,10564,10565,10565,10565,10565,10565,10564,10564,10565,8517,10565,10565,10565,8550,10630,8517,10565,10564,10565,10564,8517,10565,8484,8517,6503,6503,6503,10565,10564,8517,8517,8517,8517,10564,8517,10533,10533,10564,10533,10565,10564,10564,10564,10564,40},
{48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48859,48859,48827,48827,48827,48827,48827,50875,50875,50875,50875,50843,50843,50843,50843,48795,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48729,48729,48729,48697,46649,46649,46649,46617,46617,44569,44536,42424,40311,38231,40279,40246,38198,36118,36117,38197,40277,36085,29813,31860,44339,40178,38164,36052,46485,52758,50678,44405,42423,42456,42456,42456,42456,42488,44536,42390,29711,37999,46418,37934,40079,35950,37998,35854,19117,15021,23473,21326,17100,40177,56918,56853,52530,35789,29612,44143,40014,39982,40014,40014,39982,35821,29517,27437,27437,27437,35855,31661,33774,25388,25486,25453,29614,25453,31760,6700,10861,14759,8551,6568,8550,8518,8517,8517,8517,8517,8517,8517,8549,8517,10695,14821,23172,10596,8517,8517,8517,10565,10565,8485,8517,8517,8517,8517,8517,8517,8517,8550,8517,8517,8517,8517,10629,10565,10565,10564,10564,10564,10533,10564,10564,10564,10564,10564,10565,10564,10565,8517,8550,8582,10564,10565,8517,8517,10565,10564,10533,10565,10564,10564,10565,6470,8550,10598,10565,8485,10565,10565,8517,8550,8517,10533,10565,8517,8517,10564,10564,10564,10564,10565,10565,10565,10565,10564,10564,10564,10564,34},
{48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,48828,48828,48828,48828,48859,48827,48827,48827,48827,48827,50875,50875,50875,50875,50875,50843,50843,50843,48794,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48729,48729,48729,48697,46649,46649,46649,46617,46617,44569,44536,42424,40311,40311,40278,38198,40246,38165,38166,36084,38132,34005,29812,29812,40180,48499,42259,33972,48597,46519,44503,42423,42424,42456,42456,40408,42456,42488,44536,42423,31922,35951,46450,35917,38030,38029,35885,25453,29648,33873,35954,23439,21229,27567,56886,56820,54675,31693,21259,14889,17002,19115,21227,21227,23275,23308,19147,17066,19147,14921,35756,25387,31628,12743,25420,25421,29646,25453,29680,4718,8748,16872,10630,8551,8517,8517,8518,8517,8517,8517,8517,10629,14821,10565,8517,10597,8517,10597,10565,8517,8517,10533,10565,8517,8485,8485,10565,8517,8517,8517,8517,8517,8517,8518,8517,8517,10597,10532,10565,10565,10565,10564,10533,10564,10564,10564,10564,10564,10565,10564,10564,10565,10565,10532,10564,10565,10565,8517,10564,10564,10533,10565,10564,10564,8517,6471,8517,8517,8517,10565,10565,10565,8516,10564,10564,10565,8517,8517,8517,8517,8517,10564,10564,10565,10565,10565,10564,10564,8517,8517,10565,40},
{46813,46813,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,48828,48828,48828,48828,48859,48827,48827,48827,48827,48827,50875,50875,50875,50875,50843,50843,50843,50843,48794,48794,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48697,48697,46649,46649,46649,46617,46617,44569,44537,42424,40311,40311,40279,40246,38198,40246,36085,36085,38164,36150,27765,27764,27732,40211,42259,31891,40245,40310,42423,40343,40376,42456,40408,40408,42456,42488,44536,44536,44503,50742,50610,38029,40110,40046,35918,25454,42096,48305,37969,21359,17100,25553,46452,56787,56722,48531,21259,21292,44338,46451,44305,46385,46385,44337,46416,40144,29614,21195,19050,14921,17002,12776,23308,25389,29582,27534,31695,6733,6635,8583,8551,8551,6535,8550,8518,8518,8518,8518,8550,21126,12709,10630,8517,8517,8517,12678,14791,10565,10565,8517,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,10533,10630,10696,6502,8583,8551,6567,6732,6699,6601,6568,10761,10696,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8517,8517,8517,8517,10598,10662,10598,8517,8517,10565,8517,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,40},
{46813,46813,46813,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,50875,50875,50875,50875,50875,50843,50843,50843,48794,50843,50843,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48697,48697,46649,46649,46649,46617,46617,44569,44537,42424,40311,40311,40279,38198,38198,36117,36085,34037,44438,38165,29813,27765,27732,36052,38132,31859,54903,54968,50711,46583,44535,42456,42456,42456,42456,44536,44536,44568,44470,54935,44305,37966,33774,27501,25453,17101,39951,50385,38065,25519,21261,27599,50645,54772,46321,29647,19245,17099,27532,25420,27499,27500,25452,31692,25419,27532,29678,19179,17001,17066,8550,8550,8550,10598,10598,10566,8518,8518,8518,8518,8518,8517,8517,8518,8518,8518,8517,8517,8517,8517,8517,8583,6601,6633,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10533,19180,36050,8745,4619,2669,4750,8878,13006,12973,12973,12973,10827,8779,8747,8779,8746,8681,6568,8682,6666,8714,8714,8648,8648,8649,6536,8583,8550,8615,25485,23308,12841,12809,12776,8550,8518,8517,21259,23275,10598,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8485,8517,39},
{46813,46813,46813,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,50875,50875,50875,50875,50843,50843,50843,50843,48794,50843,50843,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48697,48697,46649,46649,46649,46617,44569,44569,44536,42424,40311,40311,40279,40246,38198,38166,34037,34005,36052,38198,38230,38230,36150,42324,38131,31859,59063,61144,52823,48663,46583,44504,42324,42259,40146,38098,42291,44340,40113,42161,27533,25421,23373,25355,23242,19115,29484,31499,31597,16970,14922,25453,44240,42094,31595,10761,17132,14954,17099,19211,21324,25550,21324,27629,23436,25484,25451,27565,25484,19179,10760,8648,17099,10761,15019,19179,19212,19212,21227,19212,21325,21325,19179,21260,21260,21292,21292,21259,23373,25518,25486,23405,21389,19245,6535,6503,6503,6503,6503,6502,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,19147,36017,12808,2506,557,590,4718,6602,10827,12875,10729,10664,10696,10631,10631,10631,10598,10598,10598,10598,10565,8517,8517,8517,10565,10565,10565,12840,27598,27598,27631,27663,27630,15051,12971,8648,8583,10662,12777,8583,8615,8615,8615,8615,10663,10663,8615,10663,10663,10663,10663,10695,10695,10663,10663,10695,10695,10695,43},
{46813,46813,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,50875,50875,50875,50875,50843,50843,50843,50843,48794,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48729,48697,48697,46649,46649,46649,46617,44569,44569,44536,42424,40343,40279,40278,40246,38166,38166,34037,36085,36084,54936,61209,61241,61208,48565,40244,33971,44405,42358,44536,42423,42456,42456,29582,31629,29548,23307,21260,19083,19050,21260,21292,23341,23373,25420,25453,25420,25420,23372,17034,12874,17165,31889,31888,25681,23600,19342,25615,15050,19308,10857,19244,14953,8712,12808,10728,8680,10728,10630,10760,23470,27696,29776,38130,27728,40210,35919,25452,35952,23274,33839,33936,40177,29645,27533,31725,29645,33774,23371,35919,31693,35951,17001,17002,23372,6503,6503,6470,6503,6470,10827,4586,4554,12907,10663,8518,8550,8550,8550,8550,10597,14853,29675,38098,12808,2474,589,655,4750,6635,10827,10827,10827,10762,8517,10565,8517,10565,10565,10565,10565,10565,12710,14822,18982,23143,27336,27368,29449,35951,59094,57014,48693,46645,46645,44565,44532,13003,25647,23534,21519,38096,38129,40274,42322,44402,42354,42354,44403,44402,44435,46515,46515,46515,46515,46515,48596,48628,48628,48628,194},
{46813,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,50875,50875,50875,50875,50843,50843,50843,50843,48794,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48697,48697,46649,46649,46649,46617,44569,44537,44536,42424,40343,38231,40278,38198,38166,36118,36085,36117,38164,59096,61241,61177,59128,48597,44437,29811,50710,48598,44503,42391,42423,44535,27598,23211,23243,19082,17034,16937,16808,19082,17002,19180,19082,14856,19050,14954,19180,21325,23405,27468,23243,25387,40243,40373,40373,40341,40243,40340,27728,23568,23568,21454,10890,17229,23535,25680,25615,23437,25518,36017,25549,21389,42259,44403,31660,25355,23307,27468,25420,27468,25419,25387,25387,25387,25387,25387,25387,23306,25355,23274,23274,23274,21194,19146,14953,14953,14920,14920,17001,31856,10762,10730,23243,35951,38031,38097,42225,27436,12841,25350,21156,29642,36017,12841,2474,590,655,2703,6797,10794,10827,12875,12874,8517,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,8516,10532,12645,16838,29482,44239,50610,48628,46581,46580,46580,44467,44435,52755,50675,42354,56949,59062,61142,61142,63222,63222,63222,63222,63222,63222,61141,63222,63222,63222,63222,63222,63222,63222,63189,237},
{46813,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48795,50875,50875,50843,50843,50843,50843,48794,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48729,48697,48697,48697,46649,46649,46649,46617,44569,44569,44536,42424,40343,40311,40278,38198,38166,36118,36085,36117,38132,48566,52758,52726,52726,50612,54904,29811,42292,52757,52758,44405,42390,44439,23276,10761,12809,10729,10729,10729,14728,10696,12842,17132,17002,17034,19146,25453,46582,42323,40178,33903,31790,31726,44404,61307,63388,59194,29746,27600,23439,19180,17100,27630,17163,25452,40178,35984,38097,38064,35982,42323,44436,42323,21292,36050,29743,29678,31790,33839,29580,31726,33872,33871,27565,29613,31791,31790,33871,33903,31758,33838,31791,27500,25420,29678,23274,25386,25354,23274,25387,40081,8682,8551,8518,35918,46352,46451,46483,23309,14889,31589,27428,29641,38098,14889,2474,590,622,6831,13039,10795,10795,12907,10794,8517,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10597,12710,10630,17000,31757,42224,44467,44532,44532,44532,42387,48563,54803,52755,48563,33804,56884,58997,56884,56850,56850,56850,56850,58898,56850,56850,56850,56882,56850,56850,56850,56850,56817,56817,218},
{46813,46813,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,50875,50875,50875,50843,50843,50843,50843,48795,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48697,46649,46649,46649,46649,46617,46617,44569,44536,42424,40343,40311,40278,38198,38166,36118,36085,38165,40212,36052,36052,36085,38132,42292,46484,29779,46517,46485,46584,42456,42423,42423,27566,8616,8616,8616,6503,6503,8583,8551,8583,6698,8518,8550,8517,8550,23177,40080,44404,33970,21390,19245,40177,44405,57017,50744,29844,25650,17230,12939,10891,25550,17131,17066,38065,21292,31824,38064,25452,35953,33773,33871,27532,33935,44435,48628,44402,48596,40144,46483,48694,50742,40176,44370,48595,46515,48563,44434,48595,42257,50676,42159,38032,46451,25354,27467,25354,23274,25387,33807,8551,8518,8518,35918,48432,48465,48465,27435,16936,27397,23203,25481,38098,14921,2473,589,655,8813,6634,10795,10795,10827,10794,8517,10565,10565,10565,10565,10564,10565,10564,10564,8517,12710,16936,21095,14823,19145,33805,56883,56851,48595,44499,46580,46547,44467,48563,56851,52722,37964,25418,36048,56917,54803,54737,56818,56817,56818,56850,56818,56818,56818,56850,56850,56850,56818,56817,56817,56817,216},
{46813,46813,46813,48861,48861,48861,46813,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,48828,48828,48828,48827,48827,48827,48827,48827,48827,50875,50875,50875,50875,50875,50843,50843,50843,48795,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48729,48729,48697,46649,46649,46649,46649,46617,44569,44569,44504,42424,40343,40311,40278,38198,36118,36085,36086,36085,38133,33972,36052,36052,40213,38132,44404,29811,48630,46550,44504,40311,42424,44503,29614,10696,10696,8584,8616,4553,6634,6601,8584,6731,8550,8551,8550,8583,8550,8550,12711,25290,33709,35887,38032,29779,40311,40278,34036,34003,27696,12874,12873,17133,14954,17067,38065,23340,36050,42291,31923,31891,35920,35919,33806,31823,46549,52789,48596,44304,40079,42224,52854,48595,35885,40111,42255,44401,44369,46482,40111,40144,42191,42192,44272,40047,25386,27467,25354,23274,31727,31857,31857,31857,31824,42224,48465,50545,48465,31595,23205,23236,25347,23336,38130,14889,4521,557,2670,6503,6536,10827,10795,10827,10794,10565,10565,10565,10565,10565,10564,10564,10565,14823,21096,23208,23176,21160,21160,23273,33805,56884,54770,48464,40111,44369,48529,48594,46482,56851,52755,31724,33870,54901,56949,54836,54671,39881,31529,54770,56850,44074,31463,40078,56850,56818,54737,54704,54737,56785,54703,211},
{46813,48861,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,48828,48828,48828,48827,48827,48827,48827,48827,48827,50875,50875,50875,50875,50875,50843,50843,50843,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48729,48697,48697,46649,46649,46649,46617,44569,44569,44536,42424,40311,40279,38230,38198,38166,36118,36085,36085,36085,33972,33972,36052,40245,40212,42292,33939,40212,48533,44438,34004,42358,44503,29646,6503,6503,6536,10761,10860,10893,10827,10729,15052,10663,10728,10760,10760,10727,10695,8616,12808,10727,12776,33905,44568,46648,40374,34036,31923,27730,19279,15150,8845,6666,12906,38065,15019,27632,38000,29582,33873,42291,29680,29646,27501,40178,50643,40112,33773,42290,40143,48660,46450,33773,31725,38096,42256,33805,38000,31693,40112,35853,37966,42224,40112,25387,27467,25387,23274,31726,44436,54902,56950,54870,48465,48465,50545,48465,29481,27460,27427,21187,25512,38098,12809,2473,557,2670,8681,17230,25583,17067,10762,10729,8517,10564,8517,10565,8517,10629,16903,21160,21127,21095,21095,23175,23208,21160,23240,33772,48432,42158,46449,48497,35884,54803,48594,46450,54738,52691,29546,23241,33936,56917,54804,52592,35852,33805,52690,56850,46317,35885,42223,54770,56785,42060,31529,46416,54737,48332,139},
{46813,46813,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,50908,50908,48828,48828,48828,48827,48827,48827,48827,48827,48827,50875,50875,50875,50875,50875,50843,50843,50843,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48729,48729,48697,48697,46649,46649,46649,46649,46617,44569,44569,44536,42424,40343,40279,38230,38198,38198,38166,34037,36085,36052,36052,38132,36020,40212,40212,40180,31859,40212,52757,54903,52791,42391,44535,31791,6568,6536,6536,8583,15021,15086,15021,6536,6699,6633,8583,6535,6535,8551,6632,6567,8551,10825,6535,23406,57114,57082,44470,38230,36149,38262,34037,36149,17296,13103,15215,25617,15150,23504,35984,17034,27631,37968,12940,21325,8648,19245,38000,37806,31628,33871,33806,38097,40144,23307,25387,33871,33871,25419,19147,25452,29646,31725,25419,31726,42192,25354,25387,25419,25420,38032,46386,48467,46452,46451,48531,50611,50644,46386,19148,33704,31683,29570,31719,38098,12809,4488,524,622,19246,48563,54804,27435,8617,8583,8517,10564,10565,12710,16967,23208,21128,21127,23208,23208,23208,23208,23240,23241,23273,31724,46352,37965,48498,50577,37931,37933,44303,44370,54770,48465,14823,17131,52821,56917,56884,54672,50413,29416,52722,54770,41994,25158,33772,56850,56817,37932,33804,46416,56817,50479,132},
{46813,46813,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,50875,50875,50875,50843,50843,50843,50843,50843,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48729,48729,48729,48697,46649,46649,46649,46649,46617,44569,44537,44536,42424,40343,40311,40278,38198,38198,36118,36117,38165,34004,31924,36052,38100,40245,40211,42324,33971,40277,42325,42390,38132,42423,46616,33872,6568,6568,6568,6568,17133,21327,19214,6568,8648,21291,6568,6535,6535,6535,15018,6567,6535,12970,8681,42323,57049,57049,54969,54969,54969,52921,52889,52889,52889,52889,52889,52889,38164,21327,33807,12841,27599,31694,8713,17131,6568,15019,33774,33548,37772,27533,31726,31790,38031,14855,21259,27596,27532,25387,12841,23339,23275,29612,19016,21194,35918,25387,27499,29645,40145,38064,42291,42357,54935,54870,52822,56983,56983,44372,21360,21295,27530,29572,29639,38098,12809,4488,524,589,15117,48530,54771,21097,8583,8517,10597,14855,19080,23240,23208,21031,14725,18983,23208,23208,23240,23208,21128,19080,16935,31692,54836,54836,54803,52690,40110,40110,21225,38033,37836,29385,10631,12775,33937,56949,56884,52657,35885,31757,50642,56850,48399,33805,38063,54770,56818,56818,56785,56785,56785,56785,215},
{46813,46813,48861,48861,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,50875,48795,50843,50843,50843,50843,50843,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48729,48729,48729,48697,46649,46649,46649,46649,46617,44569,44569,44536,42424,40343,40311,40310,38198,38198,36118,36085,36085,36084,34004,38132,38132,40213,40211,44371,38099,40212,46453,44470,46485,42423,44471,33872,6633,6601,6601,10761,21326,27600,25519,12874,10695,23403,8680,10760,8648,8648,12938,6632,8648,12971,10826,46517,54936,54936,54937,54969,54969,54969,54969,54937,54937,54937,54937,54937,34004,15216,17199,10957,21424,23471,13069,21391,15117,17198,19311,21358,21260,17229,25452,19309,27663,19309,27663,25517,23502,29710,21357,27630,25616,33904,23503,21488,25615,29612,29580,35919,42258,35854,42258,40342,57015,54902,52822,56983,56983,40179,17232,15152,17199,25450,27530,38098,14857,4456,491,557,15117,48530,52625,12677,12742,17000,21193,23240,23208,23208,23208,10597,14920,25484,25386,23208,21127,21127,21128,16968,23338,29644,54835,56883,56851,54803,50610,42158,17000,23340,12744,8518,10728,14985,46483,50447,46287,39916,37868,37836,35787,37835,35787,37835,35787,37835,35787,35755,35754,35754,35723,33675,136},
{46813,46813,48861,48861,46813,48861,48861,48860,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,50843,48794,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48697,48697,46649,46649,46649,46649,46617,44569,44569,44536,42424,40343,40311,40311,38198,38198,36118,36085,36085,33972,31924,36052,33972,36052,42292,40211,33907,33972,50743,59096,50612,46550,44471,33905,10859,10826,12939,12906,25519,31793,29712,17067,12808,19146,19179,19243,10793,10793,15051,12971,10890,10890,12971,38097,54937,50743,54937,54969,54969,54937,54937,54937,54937,54937,54937,54937,57016,50612,56983,56983,52725,52757,48531,48466,52757,50676,54805,44305,31790,46321,27499,48628,48498,42160,48531,42127,46385,40111,50579,42191,46353,44272,42062,46255,44207,31660,31661,23275,27502,35920,38130,42422,57016,54902,52822,56983,56951,35986,17232,15120,13040,15119,23438,38098,12776,6503,2474,524,13004,46417,52690,38030,27466,23208,23208,23240,23208,23208,23208,23240,23273,25289,25321,23208,21128,21160,23208,23273,25387,29611,54803,56851,56851,56883,54803,50577,29678,40145,8517,8583,10793,34001,46385,48530,46354,40082,38002,38002,38002,40082,38034,38002,37970,38002,40082,40050,35889,35889,35889,35857,135},
{46813,46813,48861,46813,46813,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48795,48795,48827,50875,50843,50843,50843,48794,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48697,48697,46649,46649,46649,46649,46617,44569,44569,44536,42424,40343,40311,40278,38198,38166,38166,36085,36085,31924,31924,38132,36020,36020,40212,44371,40211,29811,44405,50711,46551,44471,42390,36017,12939,10859,10859,10858,29744,36018,35986,15051,12906,21259,15019,21356,12906,12938,12938,10858,10858,8680,12971,25583,54937,54937,54969,54969,54969,54937,54937,54937,54937,54937,54936,52889,52856,25485,44436,44467,36016,40177,27533,27532,42386,42354,35952,27598,27597,21259,21194,36016,40112,19114,35952,25355,33838,29612,31726,29645,25485,35886,25289,21064,27468,27467,29547,19082,14954,35854,40178,38262,56983,54902,52822,56983,54870,31793,15120,13072,10991,8911,13039,36018,12841,4488,6503,4554,14922,48498,54771,56884,56916,48465,33739,23207,21095,23176,23176,23176,23208,23208,23208,23208,23208,23208,23208,21193,23273,29579,54803,54739,54771,54803,54803,54770,27469,10696,8517,10859,8843,38161,54803,54802,42162,17202,4623,4623,11026,17235,13009,4688,2607,13107,17202,8848,2575,6801,15154,13041,8},
{46813,46813,46813,46813,48861,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,50843,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48729,48697,46649,46649,46649,46649,46649,46617,44569,44569,44536,42424,40343,40311,40278,38198,38166,38166,36118,38165,34036,34004,36052,33971,36084,40212,38099,36019,33972,50678,44536,44536,42423,40343,40211,13004,13004,12972,12972,31857,40211,40211,17164,10760,8647,15084,19308,17131,12906,8648,12971,15051,8680,17165,27696,54937,52791,57049,59162,59162,59162,59162,59130,50709,35887,31629,46484,44403,29645,44403,44436,40177,38065,27566,23307,40177,40177,25420,29645,23339,17034,19114,12873,23372,12808,19147,16968,19082,23242,14889,25420,12809,23339,10565,8518,23242,25387,25354,27500,38032,44371,44403,42423,56984,54870,52790,56983,54870,27601,15152,13072,10992,8911,8911,31825,12841,6535,6634,6667,12841,46450,54804,56916,56916,56916,58996,54803,40077,27368,21063,21063,21095,21095,21095,21095,21095,21095,21128,21160,21192,27498,46384,42190,46449,54771,54770,56818,19049,8584,8615,40274,52788,56850,56851,54770,44274,19282,4655,2607,8946,15187,13042,2672,2673,11058,15154,8881,4720,6833,17202,13074,19},
{46813,46813,46813,46813,48861,48861,48860,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,50843,50843,50843,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48697,48697,46649,46649,46649,46649,46617,44569,44569,44537,44536,42424,40343,40311,40278,38198,38166,36118,34037,34005,38164,36052,36052,38132,38132,40212,42291,38131,31891,40180,42390,42423,40342,40343,40178,17164,15084,15084,15084,36018,44404,44404,23437,12938,12906,19277,21389,17196,15051,10760,19277,17164,8615,23405,29744,48630,57082,57082,59129,59129,57048,52660,48434,46451,27533,19179,40211,38162,38065,42323,42322,35984,42291,44403,29515,42290,42257,19179,19179,21259,19113,17034,21356,23373,25516,31855,23371,19179,21194,14985,23371,19211,21227,12743,19081,25387,25387,27467,27467,29483,33773,38066,42422,54935,54838,52822,56983,52758,23440,15152,13072,10991,8911,6831,27665,12842,6536,13005,8616,12776,44273,52659,56820,54739,54706,54706,54706,54706,52592,27270,25255,21063,21063,21127,23175,21095,21095,21127,21128,21192,27466,48432,42126,44369,54771,54738,42060,31757,29742,50741,58997,56883,56883,56851,54770,44242,6770,4657,4625,2609,2577,4625,4625,2577,2578,2578,2544,2512,465,497,497,4},
{46813,46813,46813,46813,46813,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,50843,50843,50843,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48729,48697,48697,46649,46649,46649,46649,46617,44569,44569,44569,44536,42424,40343,40311,40278,38198,38198,38166,36085,36085,40277,38164,36084,33972,40212,40180,42292,40211,29779,46484,54870,52758,44438,40310,42291,19277,17229,17197,17197,40211,48597,48565,27598,17066,19147,19309,19341,17163,19244,8582,19244,21356,10727,21357,38130,38164,59130,57082,57049,57016,54871,42258,38097,46516,40243,31791,40210,38162,42291,44404,44403,35952,44403,46516,42160,44402,31693,17033,17066,17066,12840,17034,17099,19180,21290,29742,19114,15018,19114,14953,17066,14985,19114,12776,25451,23372,27467,27467,27499,27467,33806,38033,42389,54903,52790,54903,56951,48565,17199,13072,10991,8911,8879,6830,17133,10794,8649,17100,8518,6536,10861,14890,14922,17002,17003,17003,17003,17003,21129,27336,27336,25191,18885,23110,21030,21095,23176,23176,21160,23273,25385,48400,39981,42224,54771,52657,33675,40209,52658,56884,58964,56884,56883,54803,54770,42162,498,464,464,529,562,561,464,464,594,2642,528,496,529,2642,2641,4},
{46813,46813,46813,48861,46813,46812,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48729,48729,48729,48697,46649,46649,46649,46649,46617,44569,44569,44569,44536,42392,40343,38231,38230,38198,38198,38166,38166,36085,44406,38164,33972,38197,38164,42325,38132,38067,31891,48630,44503,44536,42423,40310,42291,21390,21358,19310,19309,42356,50710,50710,29743,17034,10728,23470,25582,17196,23437,6567,21357,23502,23469,25550,29711,29810,50743,54904,54904,52823,50677,44403,48596,50741,46450,33839,38130,46581,44436,44403,40242,29711,35985,40243,42258,35951,21259,21324,23404,19276,21291,21324,25583,25583,25517,27597,25550,25582,23372,29580,23372,19178,21325,27598,37998,29612,27532,27467,27467,25387,25323,35952,42423,52790,50677,54903,54870,44306,13006,13039,8879,8879,6831,6798,12973,10827,6634,12907,6568,6634,6699,8779,10794,12907,12907,12875,12907,12907,17034,23175,23175,23142,20998,20965,23045,20997,16837,18950,21095,21160,25320,50512,46319,46449,48497,48464,33707,38128,54836,52723,52691,54770,54770,54770,54770,46386,13106,10994,10994,13106,17234,15154,13042,10994,15186,17234,15089,12977,10929,15122,15122,32},
{46813,46813,46813,46813,46813,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48729,48729,48697,46649,46649,46649,46649,46649,46617,44569,44569,44537,44536,42424,40343,38263,38198,38198,38166,36118,36085,36085,38165,36084,36116,48598,42325,44405,38164,38099,31859,44372,42358,42456,42423,40310,42323,23470,23470,23470,21422,46517,54903,54903,31856,17099,10793,27663,12905,17163,29776,31887,27662,29743,31823,29744,38163,48663,48630,50743,50743,52823,52823,50775,50742,52822,44338,35952,38130,46549,36049,44404,44468,31823,40276,42421,44403,42387,29742,29580,21194,27468,21194,27435,23307,23307,27499,23306,27468,25354,27467,23274,25387,21226,25387,29547,25387,27500,27467,27467,27500,25387,23308,35984,42421,50710,52757,50645,54870,38033,13006,10959,8878,6830,6798,4750,10893,8747,6536,10893,10860,12907,12908,10827,10859,12907,14988,15020,14987,14987,14889,14758,21030,23110,23110,16805,21030,29449,27401,23240,23143,23143,23175,25223,25256,38225,38159,46384,33707,38128,56884,56916,52755,46481,48464,52657,54770,48434,19282,2510,430,6833,17234,13041,430,463,13106,17234,6703,462,2575,15122,15122,3},
{46813,46813,46813,46813,46813,48861,48861,48861,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48697,48697,46649,46649,46649,46649,46617,44569,44569,44537,44536,42424,40343,38231,38198,38198,38166,36118,38166,38165,36084,34004,33972,48565,42292,40277,40244,40211,29779,44405,46550,46518,42358,40310,46517,29679,17067,19115,17066,27403,29484,27371,33969,21324,14986,31856,17195,21356,31888,29774,29744,19211,12938,29808,40177,46419,48532,48563,44338,50709,57015,50709,52789,54870,42257,44338,50775,52920,50839,50806,48694,29678,44436,46549,44403,44468,29645,27500,29645,29613,31758,29613,31726,29678,29645,27565,29645,29678,29645,29612,29612,29612,29580,31693,29580,27467,29613,29645,29645,33806,35984,48564,52790,50709,48564,42258,54805,33776,10925,10926,8846,8846,6765,6765,8780,8747,6536,12973,12907,12907,12908,12907,10827,12907,14988,15020,15020,14987,14889,8485,10565,14758,21030,23078,37770,52592,52592,44140,42256,33641,23078,23110,23208,36079,40240,44335,33707,36015,56883,56883,56884,46352,38129,52756,50545,44240,23506,15217,15217,19345,23506,23538,19377,19409,25650,27698,25617,23537,23569,27697,27697,92},
{46813,46813,46813,46813,46813,48861,48861,48860,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,48794,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48729,48729,48697,46649,46649,46649,46617,44569,44569,44569,44537,44536,42424,40343,38231,38198,38198,38198,38166,38166,38165,36084,36084,34004,44373,42325,40212,40244,42259,27731,40245,52823,52791,46518,40310,48597,27534,8747,10794,10794,10762,10762,10761,33969,21292,14954,36049,29807,27695,36081,38129,36049,31888,31888,36049,33968,21324,38097,38161,31855,44403,52757,44403,48564,54870,40144,40177,54967,55000,57015,54967,54967,48661,44436,42356,46549,44436,27630,38097,29645,35952,40243,35984,42291,42323,38097,31855,40209,42289,42290,31790,35951,23339,31791,42258,29709,29775,40177,42225,31726,33937,38065,40112,52758,50612,46386,31562,27305,12712,8616,8584,8584,8584,6503,6536,6732,8779,8584,10925,12908,12908,12908,12907,10827,10859,12939,15020,17100,17100,17002,10565,8484,10564,10565,18918,16805,10532,12612,12677,46482,50675,44368,48530,50675,52787,54835,50641,38030,42322,58996,56851,46221,21129,29711,52788,54803,52723,52690,54770,54770,54802,54770,54770,54770,54770,54770,54770,56818,56818,54737,54769,54737,214},
{46813,46813,46813,46813,46813,46812,48861,48860,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48729,46681,48697,48697,46649,46649,46649,46617,46617,44569,44569,44569,44536,42424,40343,38231,38198,38198,38166,36118,36117,38198,34004,31924,36052,36052,42325,42292,38164,36019,29811,44405,46550,44536,42456,40310,40243,29776,29809,29841,27728,29809,29809,29808,40243,23469,17164,40242,36048,35984,40274,36016,38162,36048,29742,38162,38161,29710,40242,40241,40274,40242,40241,42323,35950,46384,40145,31823,33806,31726,25289,31693,31693,37965,29580,31660,29547,31693,27597,31790,31758,31757,29645,27533,23274,29645,31661,27435,29612,25355,23274,29612,21161,25387,25419,25451,37966,37966,44338,44338,44370,44338,38000,33839,50677,46419,38001,18984,12776,8616,8616,8616,6536,6536,6536,6536,4684,8813,8616,10893,10827,10859,12908,10859,10827,10827,12940,17068,17068,19082,21259,29580,25419,21226,23306,19047,21030,14790,12645,10597,48563,56916,54803,54607,52527,56851,56883,54737,42126,44435,58964,58996,48464,29612,12776,27500,52788,56916,54738,56818,56850,56850,56818,56818,56817,54769,56818,56818,56817,56817,54737,54769,54737,215},
{46813,46813,46813,46813,46812,46812,48861,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,48794,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,48697,46649,46649,46649,46649,46617,44569,44569,44569,44537,44504,42424,40343,38231,38231,38198,38166,36118,38166,36117,34004,34004,36052,44438,38132,42292,44404,44339,31891,40212,40310,42488,42424,38263,33937,33969,36050,36050,31856,33969,33969,31889,42323,31855,31790,40242,31693,38096,40209,33935,40242,35983,36015,42290,38128,21356,42290,38064,40177,40242,38128,40242,35982,40144,33904,23469,27630,21357,23469,19243,19276,27629,19211,14953,10695,19179,17098,19178,17131,19210,12873,15050,15018,12938,17130,10759,17098,14921,10695,17066,10662,12840,17131,27596,31660,29612,42192,44305,42192,42225,40112,35920,27371,18985,14824,10696,10696,8649,8616,6568,6568,6536,6536,6536,4652,6699,6601,10893,10827,12907,12908,10859,10827,10827,10859,14988,17068,19017,31660,52560,50479,50414,50415,39914,23110,23175,14725,8549,46482,56916,54803,29579,25419,52755,56883,48431,27532,42354,54802,58964,52625,21129,6535,32018,52788,56885,52722,48366,39882,48465,56850,54672,37736,35722,52690,56850,52624,44140,42061,54737,56785,174},
{46813,46813,46813,46813,46812,46812,46812,48860,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,48794,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48697,48697,46649,46649,46649,46649,46617,44569,44569,44569,44537,44504,42424,42391,38231,38198,38198,38166,36086,38166,34005,31924,31924,33972,44438,31892,36052,40212,38131,31892,38132,48565,50710,44437,38230,21293,19180,17100,17099,17067,27663,31889,31888,40243,36113,33903,38162,19211,29710,25452,27630,27630,25517,27663,27566,25517,27630,25517,10825,27630,25517,23436,27597,21259,23404,31856,25614,31855,25550,19146,21324,23437,31822,25517,23437,27629,23404,17001,14921,19276,25548,27629,19243,25516,19146,23372,14920,21291,23404,12807,17131,10694,14953,19178,17001,29580,25386,29613,38064,29645,38064,31822,29679,8616,8551,8551,8616,8617,8616,8617,6568,6536,6536,6536,6536,4652,8715,6535,10893,10827,12907,19181,14988,10827,10827,10828,12940,14988,17068,19147,25255,25255,18982,14757,12645,14757,18950,10532,23339,50578,54706,48399,48400,52561,52561,52528,52561,50480,44239,40208,56851,48368,10696,8582,29645,54837,56917,54771,37834,16935,33870,56851,54672,25288,23241,50642,54770,46253,27433,29612,54770,54737,122},
{46813,46813,46813,46813,46813,46812,46812,46812,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,48794,48794,48794,48794,48762,48762,48762,48762,48762,48730,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48729,48729,48697,46649,46649,46649,46617,46617,44569,44569,44569,44537,44504,42424,40343,38231,38198,38198,38198,38166,38198,34005,29844,31924,31924,50743,33972,40212,38131,38131,31892,33972,52758,52791,46518,40310,25519,29711,29679,29743,29808,31889,31857,29808,38162,38161,23405,36081,25549,19212,17131,10695,19212,14986,12776,19277,14888,14921,21389,23436,19212,21389,12937,19309,25582,31855,29743,21388,23403,21357,23436,25581,17196,27629,23372,19244,19113,19211,12840,14889,17163,23404,19210,17099,23435,21323,19211,17033,17099,12807,12775,15050,21355,17033,14985,12743,23242,25387,29645,35918,31725,35918,31692,27533,8812,6601,6503,8584,8616,8616,6568,6568,6536,6536,6536,6536,4620,6667,6503,10925,10795,12907,15020,12907,8747,10828,10828,12908,12940,15020,17067,18983,18950,18950,16838,8516,8517,8517,10727,12840,21259,27500,25452,27532,29645,31757,33838,33838,33838,23371,23436,25419,21129,10630,8517,21356,52691,56820,52560,48335,48399,50448,50448,50415,48399,48367,48335,48302,48334,46254,46253,46220,46188,175},
{46813,46813,46813,46813,46813,46812,46812,46812,48861,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48729,48729,48697,48697,46649,46649,46617,46617,44569,44569,44537,44537,44536,42424,42391,38231,38198,38198,40246,38165,38165,36117,36085,36084,36084,56919,38100,40179,40211,40211,33972,33972,40245,44536,44470,40343,21358,19342,19310,23503,31889,31889,31889,31889,36049,46547,33903,36049,34000,25550,15051,8582,19309,15051,8648,19277,21389,19309,21357,21291,19212,21389,31985,25614,23501,29742,29711,21356,10727,19244,23468,27630,17163,27629,21323,19244,14888,17131,14888,10663,17131,23403,14920,17098,23403,12873,19211,14888,17067,12776,8582,15018,21355,14952,14986,8549,17000,23241,21160,23273,23241,21096,21194,19048,21260,14988,15053,12906,14953,10663,8550,8550,8550,8518,8518,8519,2571,6667,6503,8845,10795,10827,10860,10827,8747,10827,10827,12908,12908,14988,19148,23274,21194,23242,21161,19081,21161,31595,27662,44402,48627,54869,54901,54901,56982,59062,59095,61143,54869,44468,50675,48498,25419,10857,10858,14921,21162,21194,23274,21195,23275,23339,25420,25420,25420,23372,25452,25484,27532,27565,27565,29645,27565,110},
{46813,46813,46813,46812,46812,46812,46812,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48729,48729,48697,46649,46649,46649,46649,46617,44569,44569,44569,44537,44536,42424,42391,38230,38198,38198,40246,36117,36085,36085,33972,36052,33972,44373,40180,38068,40180,38099,36085,36084,46550,46583,48598,40343,25616,31889,31889,29809,29841,29841,31889,31889,34001,34000,29808,33969,27630,19211,17196,19276,23437,19277,17131,21357,23469,23469,21357,23501,23470,21357,33968,29743,23501,27694,29743,21388,12840,19277,23404,25517,17164,25484,19211,19211,17065,17131,12808,8615,17131,21323,14986,17098,21290,15018,17130,19210,14986,10727,12775,15018,10662,14887,14985,15014,14984,12840,8517,14953,19146,10727,14985,10597,10662,10662,17066,29579,31627,10630,10629,10597,8517,10565,8517,8518,2572,8813,6503,8813,10795,10827,12908,10827,8747,10827,10860,10860,12908,12940,14988,17068,17035,14987,12907,21227,14758,12677,25354,42126,50447,52592,58963,61011,61011,61011,61010,61010,58930,58963,40079,50806,52853,50773,40339,50676,52788,54869,54869,54869,54869,54901,54901,54902,56982,56982,57014,59062,59062,59030,59062,59094,57014,229},
{46813,46813,46781,46813,46813,46812,46812,48861,48860,48861,48860,48860,48860,48860,48860,48828,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48729,48729,48697,48697,46649,46649,46649,46617,46617,44569,44569,44569,44537,44536,42424,40343,38231,38198,38198,38198,38198,36085,36085,33972,38132,33972,33940,31891,33972,38165,38099,36085,33907,46452,50677,46485,40278,29777,31889,31889,29809,29841,29841,29841,29809,31921,34033,34001,31889,27760,25615,23437,27630,27598,25549,25485,25518,27630,25517,23502,27630,23470,25647,21323,25582,27694,33935,31823,21388,12840,21357,17033,21227,21324,25484,21259,21324,21291,19244,19211,19113,21324,25517,21291,21259,25516,19179,21292,25483,19211,14953,12840,17163,12808,14953,19146,19111,17033,17033,12775,21259,23403,21225,19178,10662,10727,12840,17033,17066,29580,12775,12807,12840,12841,10662,8517,8518,492,6765,8583,8813,8714,10827,10828,10827,10795,10827,10860,10827,12908,12940,12940,14988,14987,12907,12907,14955,17003,17003,16970,19082,27338,27338,25225,23080,20999,20998,20966,18885,16805,16902,27662,54901,63255,65335,56981,61077,63157,61077,61045,54738,61044,58930,56818,56818,56818,61044,58931,58833,50479,58931,58898,54705,48269,174},
{46813,46813,46813,46813,46813,46812,46812,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,50843,50843,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48730,48729,48729,48697,48697,46649,46649,46649,46617,46617,44569,44569,44537,44537,44536,42424,40343,38230,38198,38198,40246,38166,34037,36117,33972,33972,34004,29811,38099,38099,38132,36051,38165,33971,50645,50679,44471,40310,31891,29809,29809,29809,29841,29809,29776,29776,31857,27663,25582,29744,23501,29743,17131,19179,23470,21356,25549,21357,25517,25582,21356,29775,34000,21357,23403,27630,25549,29742,29710,21324,15017,19245,17163,21356,19244,25516,17099,17099,17032,21324,14953,17130,19244,14856,12872,17099,23435,19211,17131,21323,17098,19210,23402,17098,19178,21258,15017,25576,19209,14953,17000,17065,19178,14985,14985,23273,33643,35788,35788,37804,31595,29450,29450,31563,31563,16903,8517,8518,2572,6797,8583,8813,10795,10827,10828,10827,10827,10828,10860,10860,12908,12940,12907,14988,12907,12907,10827,12875,12875,10827,10794,10762,8681,8681,8649,8551,8517,8517,8517,8517,8517,10630,27434,31562,31528,31463,27271,20966,20966,23143,33740,33739,33641,29480,31463,21031,31692,33642,37966,16869,21129,25320,23175,29547,10565,40},
{46813,46813,46813,46813,46812,46812,46812,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,50843,50843,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48762,48730,48730,48730,48730,48730,48730,48730,48729,46681,46649,46649,46649,46649,46649,46617,46617,44569,44569,44569,44537,44504,42424,40343,38231,38230,38198,40246,38166,36085,36053,31924,31892,34004,31891,36084,42291,38163,40211,38197,33972,40244,42424,42456,38263,38231,36150,36182,38262,38262,38263,36150,36117,35985,33774,29647,21327,25617,31856,21389,21388,23437,23469,27630,25518,27598,27630,21357,27694,31855,23502,23469,29743,27662,31887,27630,23501,25582,23501,25549,25517,23405,23437,21324,19212,14953,23469,19276,25549,21324,19244,23437,21260,19113,19179,21292,23403,19212,21323,25450,19211,23371,23338,19210,38184,25546,19146,12808,19179,23404,23403,19210,8680,8518,14888,19081,16935,19016,8485,12743,14855,12742,21129,8517,8550,2604,8911,8551,10894,10794,10827,10860,10828,10827,10828,10860,10860,10860,12908,12907,12908,10827,10827,10827,10827,10827,10795,10794,8714,8681,8681,8682,6503,8517,8517,8517,8517,8517,10565,10565,10565,10565,10565,10565,10533,8484,12677,21095,25256,27369,25288,25256,21128,21127,25256,25289,25321,21129,21128,21096,18983,10597,39},
{46813,46813,46813,46813,46813,46812,46812,46812,48860,48860,48860,48860,48860,48860,48860,48828,48828,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,48697,48697,48697,46649,46649,46649,46617,46617,44569,44569,44537,44537,44504,42424,42456,38231,40278,38198,38198,38166,34005,36053,36084,34004,36019,38131,38132,40179,40179,40211,40277,31859,46453,44503,42456,40311,40376,42456,44536,44536,46616,44536,42423,44471,48564,46353,35888,19247,27633,31760,25551,31727,27567,25486,31792,29679,33937,27665,21358,29679,27630,25549,31791,33870,29677,31693,27598,27565,29710,27566,31790,29678,21325,23405,25582,25581,25548,25484,29809,33969,29679,29711,33871,35984,29678,29645,27468,33904,31627,35820,42029,33643,33674,29545,29608,31687,27464,27529,25385,21160,16967,19048,14856,19410,4651,10891,21323,10565,12678,8517,10597,8517,8517,10565,8517,8517,2571,4815,8813,8747,8714,10795,10827,10827,10795,10827,10860,10860,10860,10860,10827,10827,10827,10795,10795,10795,10827,10795,8714,8714,8681,8682,8682,4652,6470,8517,8517,8517,8517,10565,10565,10565,10565,10565,10565,8516,8517,10565,8517,8484,10565,8516,10532,10532,10565,10564,10533,8485,8517,10533,10532,10564,10565,40},
{46813,46781,46813,46813,46813,46812,46812,46812,48860,48860,48860,48860,48860,48828,48828,48828,48828,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,48729,48697,48697,46649,46649,46649,46617,46617,44569,44569,44569,44537,44504,42424,42391,40279,40278,38198,38198,38166,36085,36085,38132,33972,25651,31892,36052,38131,38131,40211,48631,29779,40244,50677,48565,40277,40343,40376,42424,44536,44536,42424,36183,38230,42226,35920,29680,19247,21327,19150,21262,19214,17134,19181,19150,23341,31760,19313,10861,6700,8812,17099,19210,29610,17032,14920,10760,8680,10760,10792,23371,21227,10858,12907,33839,27594,23435,15051,40309,44372,40145,33839,46418,50448,44012,33709,27305,25388,35594,37869,42095,33578,27368,16901,21157,21157,16900,25317,16933,12741,8517,10630,10826,23375,8648,12841,12710,10565,10597,8517,10565,8517,10565,8517,8517,8517,2539,6863,8780,8715,8747,10827,10860,10860,10827,10827,10828,10827,10827,10828,10827,10827,10795,10795,10795,10827,10827,10795,8714,8714,8681,8681,8649,2506,4488,6470,8517,8517,8517,8517,8484,8484,8484,10565,10565,10565,10565,10565,10565,8484,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10564,10564,10564,40},
{46813,46781,46781,46781,46813,46813,46812,46812,48860,48860,48860,48860,48860,48828,48828,48828,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,50843,50843,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48730,46681,46649,48729,48697,48697,46649,46649,46649,46617,46617,44569,44569,44569,44537,44536,42424,40311,38231,38230,36150,38198,38198,36085,36085,38132,31924,27764,27731,42292,42259,36051,38099,46551,29779,44373,48631,48631,40311,40343,40376,40376,44504,44536,40376,36183,33970,35952,37999,38000,21359,27665,19150,15021,23375,25455,17101,19279,23309,33840,19345,10894,10892,17131,15018,14954,29580,19178,14986,12841,6600,8713,6632,23372,21195,6633,12938,21194,27596,27693,21293,48663,46484,38032,27566,37967,42029,46093,25323,31628,42194,37707,44044,41898,35561,29384,16933,10661,14884,25348,27429,21125,27428,10596,16968,14921,10565,10565,18887,8517,8517,8517,8517,8517,8517,8517,6470,8518,8517,2539,6831,6699,8714,10795,10827,10828,10860,10827,10827,10860,10827,10827,10827,10827,10827,8747,10795,10795,10795,10827,10762,10696,10631,10598,8517,8583,8583,8550,8550,8517,8517,8517,8517,8484,8484,8517,8517,8517,8517,8517,8517,10565,10565,8517,10565,10564,10564,10564,10565,10565,10565,10564,10564,10564,10564,10564,41},
{46813,46781,46781,46813,46813,46813,46812,46812,48860,48860,48860,48860,48828,48828,48828,48828,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,50843,50843,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46682,46681,46681,48729,48697,46649,46649,46649,46649,46617,44569,44569,44569,44569,44537,44504,42424,40343,38231,38230,36150,36118,36117,36085,36085,38132,36052,36052,33972,38132,42291,40211,40179,48631,29811,44469,42423,42456,40343,40343,40376,42424,42488,44536,40376,36183,36150,38098,35951,33807,21295,23407,21262,17102,23375,27536,21327,21425,25454,36018,21393,13039,17163,23370,19113,14986,19180,14921,21228,21162,12841,12874,8779,8715,12941,8746,17066,17131,27628,27466,10760,50744,38098,27567,38098,40112,44077,48336,27403,33546,35953,23178,25192,29515,29516,25289,21158,19077,35778,27395,16901,19013,29444,10596,17000,16968,8517,8517,12646,8517,8517,8517,8517,8517,8517,6470,2539,4521,8517,2506,4718,4554,8715,8747,10795,10828,10860,10827,10795,10828,10827,10827,10827,10827,10827,8747,10762,10631,10565,8518,8517,8550,8582,8550,8518,8582,8517,8517,12743,25452,25484,25452,19048,14856,12775,14856,12743,10695,10663,10663,10630,8550,8550,8550,8550,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,40},
{46813,46781,46781,46813,46813,46780,46780,48860,48860,48860,48860,48860,48860,48828,48828,48828,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,50843,50843,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48730,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,46649,46649,46649,46649,46649,46649,46617,46617,44569,44569,44569,44537,44536,42424,40343,38231,38198,38198,36118,36117,36085,36085,38132,33972,31924,29812,31924,38131,33971,38131,42390,29779,40212,40343,42423,40311,40343,40344,40376,42456,42456,40344,36151,38198,40146,35952,31759,19247,17166,19182,19182,23375,25520,25553,31924,23373,44306,23473,15183,21388,21290,19178,19179,12841,33935,27533,21194,12840,16969,10795,6667,10925,6634,12873,35918,29611,21224,12904,48663,33939,27632,38065,42225,46157,50384,31532,31433,17035,31434,21064,27370,27370,21064,21158,23237,33666,25252,16901,16901,18981,10597,19016,17000,8517,8551,8517,8518,8517,8517,8517,8517,10533,8518,8584,8550,8517,2506,2637,2571,8747,8747,10827,10827,10828,10827,8747,10860,10827,10827,10827,10827,10827,8747,10795,8616,8517,8518,8518,8550,12711,16902,21063,25224,27336,29449,33837,59061,59062,59062,35854,19180,17067,14987,14954,12842,10761,12809,10728,10696,10696,8616,8616,8583,8517,8518,8550,8550,8518,8517,8517,8517,8517,8517,38},
{46781,46813,46781,46781,46813,46812,46812,48860,48860,48860,48860,48860,48860,48828,48828,48828,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48795,48827,48827,48795,48795,48795,48795,48795,48795,50843,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,46649,46649,46649,46649,46649,46649,46617,44569,44569,44569,44569,44537,44536,42424,40343,38231,38230,38198,38198,36085,34037,34005,36052,33972,34004,29811,42261,38131,40211,38132,40342,27699,44469,54935,38131,42358,40343,40376,42456,42456,44536,40376,36183,38198,40146,35920,31727,19215,17167,19182,17101,25455,31761,33841,33873,19180,33808,25553,23506,25550,25483,25515,21226,12841,40047,23373,14954,12808,12776,8648,14986,10793,8681,10761,19147,27564,21323,10760,46550,35987,27567,44469,44370,48303,48173,35789,35594,21261,39787,31433,39852,35723,23143,12774,19013,33667,19012,27363,29538,23204,12644,10597,14920,8550,8518,8518,8518,8517,8551,8517,8517,8517,8518,6471,8518,8517,4554,6798,8780,8747,10827,10827,10828,10860,10827,10827,10860,10828,10860,10828,10827,10827,8747,10827,10827,8747,8747,8747,8714,10598,10564,10564,10532,10564,14758,25256,40012,50577,58996,42223,37966,14889,12776,12776,10696,10663,10663,10630,10630,10598,8550,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,40},
{46813,46813,46781,46781,46813,46780,46780,48860,48860,48860,48860,48860,48860,48828,48828,48828,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,50843,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,48729,48697,46649,46649,46649,46649,46617,44569,44569,44569,44569,44537,44536,42424,40343,38231,38230,36118,38198,38165,36085,34005,31892,29844,34004,33972,38068,38131,40179,40179,42391,27699,42324,46550,40278,40310,40343,40344,42424,42456,42456,40344,36151,36118,40146,40113,33839,19247,19247,19182,17101,23407,25454,21260,21293,25421,36050,31793,21425,27697,25484,38063,21225,19178,27498,19178,14985,14920,14856,12777,21259,19114,8649,8648,29680,21291,31791,17100,46550,23310,21294,44339,48531,50448,48141,33644,41867,29548,31498,29450,29449,18982,10597,8581,12773,39970,39905,33634,33604,33576,16968,8550,12841,8517,8517,8517,8517,8517,8518,8517,10565,8517,8517,8518,8518,8517,8583,12908,4554,8747,10828,10827,10828,10828,10827,10828,10860,10828,10828,10828,10827,10827,10795,10827,10827,10827,10794,10663,8517,8517,10533,10565,10565,8517,10597,12711,19243,38061,50610,42126,14823,10663,10695,10663,10663,8582,10630,10598,10598,8517,8550,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,39},
{46813,46813,46813,46781,46781,46780,46812,48860,48860,48860,48860,48860,48860,48828,48828,48828,48828,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,50843,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,48697,48697,46649,46649,46649,46649,46617,44569,44569,44569,44537,44537,44504,42424,40311,38230,38198,38198,38198,36085,34037,36085,31924,29844,34004,31859,31859,36051,40211,40179,42391,27731,48597,44471,48664,40343,38295,40344,40376,42456,42488,40376,36151,36118,38066,37968,33840,19247,17134,17134,17102,27568,27533,19180,17100,25421,44274,38066,25618,34002,27564,29611,27562,23369,21226,14985,17162,17129,10825,8680,21259,19114,6568,6601,19180,23371,16968,8680,44405,10827,8780,29679,46385,48368,46321,25290,39755,31531,46190,41931,31464,16902,8517,8582,8582,14821,21061,18948,14725,29451,27403,8681,17001,8517,8550,8517,8518,8517,8517,8517,8517,8517,8518,8518,8518,8485,8517,10762,6700,8747,10860,10827,10827,10827,10827,10827,10860,10860,10827,10827,10827,10827,8747,10827,10795,10664,10565,10533,8517,10597,8517,10565,10565,10565,8517,8582,17065,35982,50609,39947,12678,8549,10663,10662,10630,8517,10598,10598,10598,8517,8517,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,42},
{46813,46781,46781,46781,46781,46812,46812,46812,48860,48860,48860,48860,48860,48828,48828,48828,48828,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,48729,48729,48697,48697,46649,46649,46649,46649,46649,44569,44569,44569,44537,44537,44504,42424,40311,38231,38198,38198,38198,36117,36085,36085,33972,27764,33972,33972,31923,44339,38066,40179,42391,29844,33939,40278,46551,40311,38263,40344,40376,42456,42456,38296,36151,36118,35986,33873,38033,17166,17167,15054,17101,27568,31727,23373,21293,25453,35920,35953,40179,44338,25451,23403,25418,27530,21325,10892,10924,13004,10859,4651,10859,8746,4651,6666,8714,10827,8714,8714,17036,6666,6699,10827,12842,14922,14922,10761,12809,10761,12809,12809,10729,8681,6633,6633,6633,6633,6600,6600,6600,8648,8648,8648,12809,8517,8550,8517,8517,8517,8517,8517,8517,8517,8517,8518,8517,8518,8517,12809,19182,10697,12810,10697,10696,10696,10696,10696,10696,10696,10664,10664,10762,10664,8616,10663,10565,8517,8517,12710,16871,16870,10533,10565,10565,8516,8517,14888,25451,31724,44206,29481,12743,10695,10662,10630,10630,10630,10598,10597,10597,10598,8550,8550,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10630,43},
{46781,46781,46781,46781,46781,46780,46780,46812,46812,48860,48860,48860,48828,48828,48828,48828,48828,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48730,46681,48729,48729,48697,48697,46649,46649,46649,46649,46649,44569,44569,44569,44537,44537,44536,42424,40311,38231,38198,36118,38198,36118,36085,36085,34004,29844,29844,31892,34003,40211,38131,38131,40343,31892,31891,52855,50677,44373,40311,40344,42424,42456,42488,38296,36151,34037,38066,35953,31727,23407,23439,23343,19182,25422,42258,38065,31760,27566,33807,35953,35986,31857,23503,21390,19277,10924,8779,8746,17197,21358,25647,19309,19342,21519,17197,23470,27599,17164,27631,23437,17164,25583,19277,17196,21389,15084,23470,21357,15019,25582,21357,15019,23405,17131,17196,23501,12938,19243,25581,10825,25549,23436,12937,23371,12808,8517,8518,8517,8517,8517,8517,8517,10565,8517,8518,8518,8518,8485,8485,8485,8485,8485,10565,10565,8517,10566,8551,8583,8518,6470,8518,8550,6568,8517,8485,8517,10597,14790,18951,18951,18918,16806,10532,10564,10597,14888,27629,46450,54770,50479,54738,40013,17000,10630,12743,12743,10663,10598,10598,10630,10598,10565,8517,8582,8517,8517,8517,8517,10565,8517,8517,8517,8517,8517,8517,44},
{46781,46781,46781,46781,46781,46780,46780,46780,46812,46780,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48730,48730,48729,48729,48697,48697,46649,46649,46649,46649,46649,44569,44569,44537,44537,44537,44536,42424,40311,38231,38198,38198,38198,36118,34037,36085,34004,34004,36052,27731,27731,33971,40179,40179,42391,31924,31891,50678,48599,42326,38263,40343,40376,42456,42456,38263,36183,40310,38098,31760,27632,23407,19247,21262,17069,21262,42258,35953,15087,23439,29744,38065,31825,40210,31857,25583,23470,25615,23503,21454,23469,19243,12840,19212,21356,23404,19308,15018,15018,17163,14920,19146,19211,19146,23437,19244,17066,19276,23436,21356,15082,25549,25549,17066,12807,21227,21323,23339,19210,17000,29676,15049,25451,31724,17097,12807,12841,8518,8518,8517,8517,8517,8517,8517,8517,8517,8518,8518,8518,8518,8517,8517,8518,8485,8518,8517,8517,8518,6471,6569,6503,6536,6503,8518,8517,8517,12710,16903,21031,20967,16806,18886,18886,14725,10533,8485,12775,38128,56882,56850,56818,54770,54802,54770,48562,27467,12776,12743,10695,10663,10630,10598,10598,10598,8550,8550,8517,8517,10565,10565,8517,10565,8517,8517,8517,8517,8517,39},
{46813,46780,46781,46781,46780,46780,46780,46780,46780,46780,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48730,48729,46681,46649,46649,48697,46649,46649,46649,46649,46617,44569,44537,44537,44537,44536,44536,42424,40311,38231,38230,38198,38198,38166,36085,36117,33972,31892,40245,40244,29811,36051,44307,42227,42423,34005,31859,44438,42391,31925,38231,38296,40376,42456,42456,38263,36183,44438,38066,33873,31759,25487,23407,19150,19149,23310,42226,44372,17265,23439,42258,44306,44371,42258,35984,17163,15115,17228,17196,17228,33871,19210,29807,21324,19243,23403,17130,21324,29775,17097,29741,29710,17098,19177,21323,17163,17163,17163,19275,25581,12905,25548,25484,12904,25581,17130,17130,19145,14953,17097,25515,12904,29741,19177,12904,31822,12808,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8518,8517,8485,8517,8485,8485,10533,8485,8485,10565,8485,8518,6568,10631,8615,8583,10630,14823,18983,20999,20999,18919,10565,12710,21064,20999,14693,10695,31855,54901,56883,56850,56818,54835,56883,56883,56818,56851,37901,14857,12744,12744,10630,10631,10630,10598,10598,10597,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,39},
{46813,46813,46781,46781,46780,46780,46780,46780,46780,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,48697,48697,46649,46649,46649,46649,46617,44569,44569,44537,44537,44537,44536,42424,40311,40279,38230,38198,38198,36118,36085,36117,34004,31925,36052,33972,31924,36051,40179,38131,42391,34005,29779,38197,40278,31892,36183,38295,40376,42456,42456,38263,36151,38131,31825,35954,29647,23407,17134,19182,19215,31826,40146,50613,25553,19247,35953,33840,44404,46419,40177,21453,23501,21421,25615,42355,38097,15083,14855,19179,19276,21324,17195,15018,19081,19179,14855,21227,19211,19211,23469,19276,25517,21324,21356,25581,17098,21258,25484,17066,16968,23340,21292,23435,17130,14984,27499,17098,25451,23338,17065,21161,10695,8517,8518,8517,8518,8517,8517,8517,8517,8517,8517,8518,8517,8517,8517,8517,8517,10565,10565,8485,10565,10533,8518,6504,8517,8583,14792,18951,20999,20999,18918,18919,18919,18951,20998,23110,23079,18951,44239,56851,58996,58963,56883,56851,56851,35786,37932,29481,50610,39981,12776,8550,10663,10631,10630,10565,8550,8550,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,40},
{46780,46813,46781,46781,46780,46780,46780,46780,48860,46780,46780,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48795,48795,48795,48795,48827,48795,48795,48795,48795,48795,48795,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46681,48729,48697,48697,46649,46649,46649,46649,46617,44569,44537,44537,44537,44536,44536,42424,40343,40279,38230,38198,38198,38166,40246,36085,31924,31925,29877,29877,29812,36051,36019,38099,40311,34005,27699,50742,50742,38164,40310,40343,40375,42456,42456,38263,36183,40277,31793,35954,29583,23407,17134,23375,25520,42292,33873,17102,15120,19312,40210,35920,50677,42226,38064,19211,17163,17163,29743,48565,35984,17098,8550,17066,21323,23403,19211,21258,14856,17098,10630,14953,17130,25484,23371,17130,19113,17131,17130,19211,12905,10630,17001,12872,8549,14953,17130,21258,12905,12807,12808,12872,12775,12807,12872,21161,12776,8517,8517,8517,8517,8485,8517,8517,8517,8517,8517,8518,8517,8517,10565,10565,8485,8484,10565,10565,10565,10533,8518,6503,8518,6503,6503,8517,10697,16840,18919,18886,18918,18886,18886,18886,18854,16806,18854,27271,44141,54705,56850,56850,56850,54835,54803,54770,46222,25257,14856,12776,12743,10663,10630,10630,10598,10598,10598,8550,8517,8517,10565,10565,8517,8517,8517,8485,10565,10565,10565,39},
{46813,46780,46781,46781,46780,46780,46780,46780,48860,48860,48860,48860,48860,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46649,46649,48697,48697,46649,46649,46649,46649,46617,44569,44569,44537,44537,44537,44504,42424,40343,40279,38230,38198,38198,36118,38165,36085,40277,48663,50743,48663,44438,42291,40179,38131,40343,36085,29811,52758,52791,48533,40310,40343,42424,44504,42456,38263,36151,42358,33906,33906,29680,23375,17166,19214,27633,42292,35954,10959,21426,25488,44306,35920,44339,42291,38097,17163,15083,15083,27663,46517,35984,15050,8518,14986,17195,21323,15083,12808,12776,15018,10630,14953,15082,23371,19243,12905,10760,17131,14985,12873,12905,12840,15050,12872,10727,19243,15049,19145,15017,10792,12808,10857,10727,19178,10760,12808,10761,8550,8551,8517,8518,8517,10533,10565,10565,10565,10565,8517,8517,8517,10565,10565,10565,10565,10565,10564,10564,8516,6503,6470,8517,6503,6504,6502,6504,6503,8518,16839,18918,16838,16838,16806,14726,14726,16806,16806,18918,27401,54835,56850,56850,54639,48334,31530,18984,12678,10663,12743,12711,10597,10598,10598,10598,8517,8517,8550,8517,8485,10565,10565,8517,10565,8517,8517,8517,8517,10533,40},
{46780,46781,46780,46781,46780,46780,46780,46780,46780,48860,48860,48860,48860,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,46649,46649,48697,48697,46649,46649,46649,46649,46617,44569,44569,44537,44537,44537,44536,42456,40343,38231,40278,38198,38198,36118,36117,36085,46518,57016,59096,52791,42324,44405,40179,40179,42391,38198,29779,42293,42424,42326,38263,38295,40376,42456,42456,38263,34103,38164,31825,25519,35952,27567,17166,21295,29680,42324,36018,13007,19313,21295,33808,33808,42324,42258,35984,15083,15083,15083,27663,48565,35984,15082,8549,14986,17195,21324,17163,12808,12776,15018,10727,15018,15050,8549,15018,15050,14888,17131,15018,15018,12937,23436,25516,12873,8550,14985,12905,8614,12905,12904,15017,12873,12742,10728,10792,10598,10697,8518,8517,8485,8517,8517,8517,10565,10565,10565,10565,8517,8517,8517,10565,10565,10565,10565,10565,10565,10564,8517,6503,4488,6503,6503,6503,6503,6504,6503,8518,8518,14856,21161,18918,18918,16806,14726,12613,12645,14726,18918,33576,54673,54705,50511,21096,16969,14888,12808,12776,12711,10663,10663,10630,10598,10598,10598,10597,8550,8517,8485,8517,8517,8517,8517,8517,8517,8517,8517,10565,40},
{46813,46780,46780,46781,46780,46780,46780,46780,46780,46780,46780,46780,48860,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48795,48795,48827,48795,48795,48795,48795,48795,48795,48795,48794,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,48697,46649,46649,46649,46649,46649,46617,44569,44569,44537,44537,44537,44536,44504,42391,38231,38230,38166,38198,36118,36085,31892,38133,59096,59096,54935,44501,46484,40179,38099,42423,38198,25618,38197,42455,48663,38263,38295,40375,42456,40376,36183,34103,29809,27665,31858,31695,29615,19247,25520,27568,44372,38098,4750,17266,25487,33840,46516,52855,44370,40210,19276,19276,19276,29744,46517,35984,15082,8582,12938,17163,14985,15083,10792,12809,15050,10727,15082,15050,8647,13003,12937,8680,17131,17163,21388,17130,23436,25517,15018,12841,17131,15147,14953,15050,14952,23338,14985,14953,15018,17066,17001,10631,10566,8517,8485,8517,8517,8517,10565,10565,10565,8517,8517,8517,8517,10565,10565,10565,10565,8517,8517,8517,8518,6470,4488,491,2474,4488,8518,6470,6503,8518,8550,10663,12777,12744,14825,12678,16806,16838,10565,8484,10565,12645,18951,21160,33836,14954,12809,12776,12744,10696,12743,10663,10598,10598,10598,10598,8517,8517,8518,8517,8517,8517,8517,8517,10565,10565,8517,8517,8517,10565,40},
{46781,46781,46781,46781,46781,46781,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48795,48795,48827,48795,48795,48795,48795,48795,48795,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,48730,46681,46649,46649,48697,46649,46649,46649,46649,46617,44569,44569,44569,44537,44537,44537,44536,42424,42391,38230,38230,38198,38198,36118,38165,33973,31892,44341,46421,44341,42228,42259,40243,38099,40343,38198,27699,46452,48597,48565,40277,38263,40375,42456,40376,36183,34103,31891,25553,33873,36017,27567,19247,29680,23407,42292,38098,6798,17233,29648,38001,37968,50644,42258,38064,19243,17163,19243,27663,46517,38097,21324,14920,21324,25517,33903,25549,29775,36081,19243,15050,21388,19275,29741,29677,21323,27629,23404,19211,14953,17162,25549,27629,15017,27693,23404,19178,17065,17098,14985,25516,14952,25483,25451,12873,12841,8550,8518,8518,8517,8517,8517,10565,10565,10565,10565,10565,8517,8517,8517,8517,10565,10565,10565,10565,8518,8517,8518,6471,4488,2474,459,2474,8518,6503,6503,6536,8551,8518,6569,6536,6601,8550,8517,16838,14758,10565,10565,10533,10598,12840,23370,17100,12873,12808,10663,10663,12743,10663,8550,8550,10598,10598,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,39},
{46781,46781,46781,46781,46781,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48795,48795,48795,48827,48795,48795,48795,48795,48795,48795,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,46681,48730,46681,46681,46681,46649,46649,46649,46649,46649,46649,46617,46617,44569,44569,44537,44537,44537,44536,42424,42391,38198,38230,36118,38198,38166,38165,38197,29812,29812,29812,29812,29844,36019,38099,38131,40343,38198,27699,50677,52759,48630,42358,40311,42423,42456,40376,36183,34037,25552,23504,33906,33807,33873,19247,38066,29681,42292,38099,4718,15185,21295,38131,35920,44371,42290,38065,17164,17196,19276,27663,46517,38064,15083,8647,15018,12905,23338,19243,15017,21259,15018,17033,25419,15082,17032,19210,12904,23274,21259,17163,19210,15050,23404,25517,10857,19080,19178,12937,14920,19178,19145,23240,17065,21258,21194,10728,8616,8518,8517,8517,8517,8517,8517,8517,10565,10565,10565,8517,8517,8517,8517,8517,8517,8517,10533,8517,8517,10565,8517,8518,6536,2473,459,459,6503,6503,8518,8517,6503,8518,8582,33902,46515,46482,46515,44369,18982,10597,18982,19015,12774,14887,21225,17034,14921,10695,8550,10631,12743,10663,10598,8550,10598,10598,10597,8517,8517,8518,8485,10565,8517,8517,8517,8516,10565,8517,10565,10565,40},
{46780,46781,46781,46781,46781,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48795,48795,48827,48827,48795,48795,48795,48795,48795,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,44569,44569,44569,44537,44537,44537,44536,42424,42391,38230,38230,38198,38198,36118,38166,34005,29844,29812,29812,31924,36051,38131,36051,36051,40311,38230,29843,42260,42423,40310,38231,38263,40375,42456,40344,36183,36151,25584,21360,33874,31791,31825,17134,25520,25455,42292,40211,6798,13072,25520,42193,35888,46484,42258,40145,19276,19244,19276,27663,46517,38097,21356,14888,19179,19179,19081,23404,19147,14986,21324,16968,21226,21355,29774,31823,19146,14823,23372,23468,21259,17130,17000,33903,17097,12710,17066,19242,25483,19114,16968,12711,17033,17000,14856,10597,8485,8517,8517,10565,8517,8517,10565,10565,10565,10565,10565,10565,6470,4489,6471,8518,8517,8485,6568,8518,8517,10565,8551,6536,8518,6470,458,459,6502,4521,8518,8517,8518,10631,16904,42127,54706,52593,52593,54673,37835,16838,18950,37834,23110,14888,14953,12808,12776,12808,17001,12776,12744,12711,14856,14855,10631,10630,10630,10598,10598,10598,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,39},
{46780,46780,46780,46780,46781,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48795,48795,48795,48827,48827,48795,48795,48795,48795,48795,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,46649,48697,46649,46649,46649,46649,46649,44569,44569,44569,44569,44537,44537,44536,42424,42391,38230,38230,38166,38198,38166,36117,38230,38262,38229,38230,44437,46451,44371,42259,40179,40343,40310,27731,33972,42423,52888,42455,38263,42424,42456,40344,36151,34037,29713,25585,33906,29711,33906,17167,27600,25455,42292,40211,6798,13072,27601,27534,35920,52823,44404,38097,15083,15083,15115,25583,46517,36016,15083,10825,15051,12905,10695,17163,14953,10728,15083,10695,14921,15050,10727,17066,15050,8647,17098,12840,17065,17130,12775,14920,12872,10630,14953,15050,12742,14953,10759,8582,12905,31824,35722,31529,21064,10533,8517,10598,8517,8517,8517,8517,8517,10565,10565,10564,6503,492,6536,6503,8518,8517,6634,8551,8517,8517,6536,6537,6503,6503,2506,2507,8517,8517,8517,8517,8518,8518,8518,10630,14759,14693,12613,16806,21063,21063,21096,21096,21128,44434,54934,50708,29710,19114,21163,14922,12809,12776,12743,12743,10631,10598,10598,10598,10598,10598,10565,8517,10565,10565,10565,10565,10565,10565,10565,10564,40},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46681,46649,46649,48697,46649,46649,46649,46649,44569,44569,44569,44569,44537,44537,44536,42424,42391,38230,38230,38198,38198,38166,40246,48598,56918,56951,58999,56918,56884,48531,42259,40179,42391,40343,25651,38132,44470,50710,40309,38263,40375,42456,40344,34103,36150,31792,25520,31793,31727,40179,17167,23473,23407,44372,42292,6798,11024,21328,25454,35888,48531,44371,40178,15115,15147,17196,25550,46517,36016,15083,10727,15050,15050,8615,17131,12872,10792,15050,10695,14921,15050,21421,27727,17163,21421,21323,17130,12776,15050,10662,12840,12872,10630,19178,15050,19210,17066,17162,10759,15017,37902,37641,39657,25063,18855,10599,8551,6601,8550,8517,8517,8517,8517,8517,8517,4489,524,6503,6503,8518,8517,6503,8518,8517,8550,4586,8551,8517,8517,8517,8517,8517,8517,8517,8517,8518,8518,8518,4521,6568,8517,8517,8484,8518,8550,16870,27303,27303,46287,48334,46189,44076,23144,10630,10598,10598,8517,10565,8517,8517,10565,8517,8517,8517,8517,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,42},
{46780,46780,46780,46780,46781,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44537,44537,44537,44536,42424,42391,38230,38230,38198,38198,38166,38166,44438,61079,61111,61111,59030,52724,46451,40211,40179,42423,42423,25618,40212,52790,56951,44405,38263,42423,42424,40344,34103,34103,29843,27665,29713,27566,42259,19247,25520,21294,42292,42292,10926,29778,25521,25519,33872,44403,42291,42291,15083,15116,15083,21389,46517,38097,15083,10760,15050,15050,8615,17099,12872,10760,15083,10695,14953,15050,14888,21258,15017,14952,19211,12840,8582,13002,10727,12841,10824,8549,14920,14985,10630,14985,17098,19178,19211,40112,41931,39689,41737,33416,12711,10599,6536,8583,8517,8517,8517,8517,8517,8517,4489,525,6503,6503,6503,6503,6470,8518,8518,6535,8518,8550,8517,8517,8517,8517,10565,8517,10565,8485,8518,8518,8518,6601,8551,8517,8517,8516,6470,4489,12710,27270,10564,10728,12776,10598,8517,8517,8517,8517,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,42},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48762,48763,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,46649,48697,46649,46649,46649,46649,46649,46617,44569,44569,44537,44537,44537,44536,42424,42391,38230,38230,38198,36118,36118,38166,40245,46453,46420,48500,50581,54837,46517,38131,38132,42424,42423,25619,33939,42358,44503,38231,38263,40343,40376,38296,34103,36183,33938,27599,29712,25486,40147,21327,25553,19214,42292,42292,15085,12908,27567,33807,40079,35886,42258,42290,21388,21388,19341,25583,46517,40210,21356,17098,21356,19276,12872,21324,17098,17065,23404,17033,19211,21292,14921,25550,19243,12840,19275,19308,25581,19308,23469,27694,19146,12808,17098,19211,23371,19179,21356,29742,25517,44337,44043,41802,39656,39624,31433,21194,8682,8584,8517,8517,8517,8517,8517,8485,4488,557,4487,6503,6470,6503,8518,8518,8518,10793,25287,16870,16870,10565,10565,10565,8517,8517,10565,8517,8518,8550,8518,8518,6470,8583,8517,10565,8517,6536,12710,14725,8484,8615,10631,8517,8517,8517,10565,8517,10564,10564,10565,10565,10565,10564,10565,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,42},
{46780,46780,46780,46780,46780,46781,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48828,48828,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48762,48763,48763,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44537,44537,44537,44536,42424,42391,38231,38230,38198,38198,38166,36085,38165,38164,38132,36052,40180,54772,46452,38099,36051,40376,42456,25651,40245,42391,42456,38263,38263,40344,40376,38264,34103,36151,34004,25583,25583,27631,40179,19214,25520,23374,40211,40179,23437,17164,15052,21260,31724,31724,42258,40177,15083,15083,13035,21357,46517,40177,17163,17162,21323,17163,10695,17131,14985,12841,17163,21291,31789,15082,14888,23339,17162,23339,17195,17195,23469,17162,14887,19114,14985,25451,21258,15017,17033,14985,15050,23435,23372,44273,44043,44109,37805,35821,35658,31563,10794,8584,8517,8517,8517,8517,8517,8517,4489,557,4488,6503,8518,6471,6504,8518,6503,10793,37865,21094,19014,10565,8517,10565,8517,8517,8516,8517,8518,8518,8517,4554,4717,8583,8517,10565,8518,10696,12677,10532,10565,8615,8550,8517,8517,10565,10565,10565,10565,10565,10564,10565,10564,10564,10565,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,42},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48763,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44537,44537,44537,44504,42424,42424,38230,38230,38198,38198,36118,36117,36085,36052,36052,38133,36052,42325,46485,44339,40212,42424,42456,27699,31859,40278,40343,38230,38231,40343,38296,38263,34103,34103,29745,23407,25519,40179,40210,17134,23440,21262,36019,29747,21357,21357,19245,21292,31724,29676,42258,40145,21388,21356,19308,25550,46517,40209,21356,15050,21324,17228,12808,21324,19211,14888,23437,19081,23307,21324,17066,23502,21356,12840,19275,21388,21324,21356,17001,17034,19179,25549,27597,19211,23403,17130,19275,27629,21291,35723,41898,33642,35853,42224,39981,37935,17067,8616,8518,6470,6471,6471,8518,8518,4489,557,4489,4456,8518,6503,6471,8518,4586,8648,31527,27238,20965,10565,8517,10565,10564,10564,10564,10564,8518,6503,8518,8518,6634,8550,8517,8517,8517,10695,10597,10565,8517,8518,8518,8517,8485,10565,10565,10565,10564,10564,10564,10565,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,42},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48828,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48763,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,46681,48730,46681,46681,46649,46649,46649,48697,46649,46649,46649,46649,46649,46617,44569,44569,44537,44537,44537,44536,42424,42424,38231,38230,38198,38198,36118,36117,38165,36052,33972,44373,44373,40244,40244,38099,38132,42424,42456,27699,29779,46517,48597,40212,38230,40311,38296,38263,34071,36151,25681,21390,23470,31823,40178,12974,21392,19182,38132,29811,19247,19279,27567,23341,31791,29676,38032,44338,17163,17163,17163,23437,46517,40209,17163,17163,19276,15018,8615,17163,12937,10759,19275,12840,14953,15050,17195,27597,17098,17033,21323,17098,12775,17131,15050,17163,12905,17000,25484,12969,21387,19210,15049,12774,14985,23338,27499,12840,14855,35887,29548,25355,12809,8616,8550,6471,6471,6471,8518,6470,4488,557,426,4489,6471,6503,6503,8517,6503,4488,8517,8517,8517,8517,10565,10565,10565,10565,10564,10565,8518,6502,6536,8517,6569,6536,8517,8518,8518,10663,8615,8550,8517,10598,8517,8517,8517,10565,10565,10565,10564,10564,10565,10565,10564,10564,10564,10565,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,42},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44537,44537,44537,44536,42424,42391,38230,38230,38198,38198,38166,38198,38165,36052,31892,42292,42291,42291,40212,38067,38100,40343,42456,27731,29779,42326,46551,40310,38230,38263,38264,38263,34071,36151,29810,23502,27664,31856,42322,13006,25552,21230,40212,40212,17167,10991,17199,21261,46419,38096,31759,38000,17163,15115,17163,23437,46517,40210,15083,27661,27629,17131,29840,25581,21388,34000,19307,12840,14953,15050,23468,19276,17130,10630,19211,14985,8615,17195,14953,19211,12937,8582,17034,12969,19177,17033,15050,12872,12873,12840,14856,12840,10662,19082,21130,33741,17035,8584,8517,8517,8517,8517,8517,8517,8517,6503,458,2441,6471,6471,6471,8518,6503,2505,8517,8517,8517,10565,10565,10565,8517,8517,8517,10564,8518,6503,8518,8517,6503,8682,8517,8517,6471,4488,6536,15215,23339,8517,8517,8517,8516,10565,10565,10565,10565,10564,10565,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,10564,41},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48828,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,44569,44569,44569,44569,44537,44537,44536,42424,42391,38230,38198,38198,38198,38166,42358,38165,33972,31924,44373,44340,44339,40211,42291,40180,42423,42456,29811,27667,40278,42424,38295,38230,38230,38264,36183,34103,34103,36084,25520,29745,31758,48499,17134,25521,23342,40180,38099,19246,23472,23440,21229,31727,54935,29646,40177,17163,15115,17163,21389,46517,46516,44371,31854,29710,17195,44434,27661,27694,46547,21355,10792,14953,15050,21388,23469,15050,8550,15018,15017,10760,21323,10727,12873,12905,10695,19146,12937,17000,15018,15050,21323,15017,10695,10663,12840,10630,12808,14921,14889,12777,6536,8517,8517,8517,8517,8517,8517,8517,8485,4489,4488,4456,6471,8518,8518,6503,6503,8517,8517,8517,8517,8517,10565,8517,8517,8517,10565,8517,4488,8518,8517,8517,8518,8517,8516,4455,15214,19244,21487,29710,8648,12776,12711,10565,10565,10597,10565,10565,8517,10565,8517,10565,8517,8517,10565,8517,8517,8517,8485,10565,8485,8484,8484,8484,8484,8484,8484,39},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48763,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44569,44537,44537,44537,44536,42424,42423,38230,38198,38198,38166,38166,40278,38165,38132,36052,46421,44373,38132,40211,40179,38131,40343,42456,38230,25586,38166,40376,40343,38230,44471,38263,36183,34103,34103,29811,25584,29713,31758,44306,17134,23408,19182,42292,44372,23439,23440,29681,25422,40244,54870,33905,33807,23436,23404,23404,25517,38000,40177,48531,29611,27500,25485,46417,29677,29742,44336,23501,15018,27630,19276,25614,27662,19275,14888,21324,19243,21324,23404,17163,19276,15083,14953,21227,15115,19275,21323,19211,23435,21291,17001,19179,17034,12808,12840,12840,12711,8616,6568,8517,8517,8517,8517,8517,8517,8485,8485,8517,8518,6503,8518,8518,8518,4488,6503,8518,8517,8485,8517,10564,10564,10565,8517,8517,10565,8517,6503,8518,8517,8518,8518,10564,8518,6601,38259,52853,46644,21389,35984,14856,12743,10598,10630,10598,10597,10565,10565,10565,10565,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8485,8485,8517,8485,8517,40},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48828,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44569,44569,44537,44537,44536,42424,42391,38230,38198,38198,38198,38166,42359,40278,36085,38165,34004,33972,42325,44371,40179,40180,40343,42456,44405,25586,42389,50743,44469,40278,42391,38264,36183,34103,34103,40310,31760,27664,27630,35985,17166,25553,19182,40179,44404,38164,38164,21393,23310,44469,35985,33938,25551,17163,15083,17131,17163,15115,23437,44338,25484,23340,19211,19211,21388,19275,23469,23403,19275,31822,17130,29709,29709,17130,14985,21323,19243,27629,21323,19307,27661,14985,19242,25516,12937,25484,21291,15050,23403,17000,8550,12743,10630,8550,8614,8549,8550,8550,8551,8517,8517,8517,8517,8517,8517,8517,8485,8517,8517,8517,8518,8518,8518,8650,6569,8518,8517,8485,10565,10564,10564,8485,8517,8517,8517,8518,8518,8517,10564,8517,8518,8516,6535,6569,14824,33838,46545,25321,21326,25582,25485,17099,21227,14953,21194,21129,12710,14823,16871,10630,10630,10565,10597,10565,10597,8517,10565,10565,10565,10565,8517,10565,10565,12677,8485,40},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,46681,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44569,44569,44537,44537,44536,42424,40343,38230,38230,38198,38198,38166,40278,36085,33972,36020,36084,31892,36051,38131,36019,38099,40343,42456,44438,25618,48566,54871,48598,42358,40343,38264,36183,34103,36150,36018,27567,27632,25486,38033,19246,23440,17134,36019,44372,46452,44340,23473,23342,42388,48565,33874,27568,31824,23405,23471,29841,25550,27663,21291,27565,23437,19308,29807,25613,27726,42321,21355,23436,29644,15050,17065,17098,15050,25614,23436,19211,21258,19178,17097,17129,12905,17065,27499,12937,25516,21258,14952,21226,12775,8517,8550,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,6503,2473,4489,8518,8518,10566,8617,8517,8517,10565,10565,10564,10565,8517,8517,8517,8550,8518,8518,8517,10564,8517,8518,8517,6503,6569,6536,12970,40208,19114,23665,46580,46483,36049,15148,36016,23177,19017,12678,12710,12678,10565,10565,10597,12677,10565,10565,10565,10565,8517,10597,8516,10565,10565,8517,8517,8517,40},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44569,44569,44537,44537,44536,42424,40343,38230,38230,38198,38198,38198,40310,36117,34004,33940,36020,34037,36051,38163,38131,36084,40343,42456,42293,25586,38197,42424,40343,38230,38263,38263,36151,34005,23570,31793,19246,23440,19245,38001,19247,23472,19181,33906,46453,44372,44307,31761,23342,46485,35954,17200,15053,48241,23307,29810,48597,23275,25420,29742,23241,25452,25549,25386,25517,23273,27467,21323,21291,23370,15082,12873,12937,17163,27662,21323,17130,8550,12905,23370,23338,12905,17000,14952,12969,19145,10760,10760,10662,8582,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,6503,6504,6471,8518,8517,8518,6537,8517,10533,8517,8517,10564,10565,8517,8517,8518,8550,8550,8518,8517,10564,8518,8518,8517,8517,6568,6602,10826,6697,15311,40111,33675,37934,40014,29645,33676,19049,17002,14889,14888,14888,14889,14889,14921,16969,16969,14889,14889,14921,14921,17033,17001,14920,14920,17066,14953,14953,63},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48828,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,48730,48730,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46649,46649,44569,44569,44569,44569,44537,44537,44536,42424,42391,38230,38230,38198,38198,38165,40246,38165,36052,33972,33972,40213,36051,40211,38131,36083,40343,42456,44437,25618,36085,42423,40342,40310,40343,38263,34103,42357,29843,36084,25617,27697,25486,31760,19279,23440,19214,31761,44372,38000,27501,21229,19148,29647,33872,23471,17165,33838,23405,38164,50678,17002,15051,21324,10695,19244,23469,17097,21356,14921,19146,21323,14953,21258,15082,25581,23501,15083,27597,19243,14985,12840,15017,21226,10727,10857,17065,17033,12905,19081,21128,10727,10663,8518,8517,8517,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,10565,10565,8517,8517,8517,8517,8518,8518,8518,8518,8517,8518,6503,6502,8517,8517,8517,10565,10565,8518,8517,8518,8550,8550,8518,8518,8517,8518,8518,10564,8583,6503,6568,8714,11118,46481,38063,31920,40210,42290,40242,42322,42322,44402,42322,44402,44434,44435,46515,46515,46515,46515,48595,48596,48628,50676,50676,50676,50676,50676,48595,50675,50675,199},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,46780,48828,48828,48828,48828,48827,48827,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46649,46649,44569,44569,44569,44569,44537,44537,44536,42424,42391,38230,38230,38198,38198,38166,38165,40246,36052,33972,29812,31860,42291,38131,38099,38132,40344,42456,40245,27699,31891,40277,46453,38197,38263,38264,36151,40244,52855,54903,33971,31858,27598,25551,19279,23440,19182,21294,44372,33806,23340,17067,14987,25454,42290,35950,25419,25452,42455,46549,46516,19180,15083,21421,21486,23501,23469,25549,21356,23567,29840,21355,12937,19178,15050,23435,33903,15050,17065,12905,12905,10663,12905,21257,27564,12905,23338,10727,10857,23403,21225,8647,10663,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,8517,8517,8518,8518,8517,8517,8517,8517,8518,2473,8517,8517,8518,8517,8517,8517,8517,8517,8550,8550,8518,8550,8517,6503,6470,8516,8551,6503,6536,4520,23600,39882,27629,50708,56981,56981,56949,56949,56949,59029,59029,59061,59062,59062,59062,59094,61142,61142,61142,61142,61142,61142,61142,61142,61174,61174,56981,54901,59094,234},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,46780,46780,48828,48828,48828,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48795,48795,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,48730,46682,46682,46649,46649,46649,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44568,44537,44537,44536,42424,42423,38230,38230,38198,38198,38166,38198,38198,34004,31892,31892,31892,42259,38099,36019,36051,40343,42456,38165,27699,36019,48597,46485,38165,38263,38263,34071,40244,52823,52790,29746,33905,31727,27631,23440,23473,19182,19181,44340,35984,25484,21260,19179,25486,52823,40242,25548,27597,56951,50548,48532,23341,17099,25582,42354,27629,23469,27629,21356,36113,44466,21356,15017,12873,15050,14984,12840,15050,27596,10825,12905,8550,10825,19145,14887,10857,21258,23436,12905,23371,14920,12969,14920,8518,8517,8550,8517,8517,8517,8517,8517,8517,8517,10565,8517,8517,8517,10565,8517,8517,8517,8517,8518,8518,8518,8518,8517,8517,8518,6535,8517,8517,6502,6503,8550,8518,8517,8517,8550,8550,8550,8518,8517,6700,8583,10565,8517,6535,8713,15115,23533,33706,29775,52722,56851,46255,44076,44044,44011,44141,41964,54836,61109,61077,58995,61076,61076,61076,61076,61043,61076,61043,61043,61043,61076,61044,59028,59061,61044,235},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,46780,46780,46780,48828,48828,48828,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48795,48795,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,46682,46682,46682,46682,46649,46649,46649,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44569,44537,44537,44504,42424,42423,40279,38230,38198,38198,38166,38165,38166,40310,36149,31989,31924,36084,40179,40211,38132,40343,42456,36117,27699,36052,42423,44471,38230,38263,38264,34103,40245,54903,48597,29746,40212,27598,23470,21392,25553,19182,12908,23374,48629,40242,27597,21260,27566,54837,48433,33804,31757,42357,50710,40113,38163,21358,27662,48660,27629,23436,31855,21388,40305,48660,23436,31919,29806,13034,17162,10857,15114,15018,10922,12937,6632,10857,23500,15050,10857,29742,25451,10857,23436,17097,12937,12840,8518,8518,6535,8517,8518,8550,8517,8518,8518,8517,8517,8517,8517,8517,10565,8517,8517,8517,8517,8518,8518,8518,8517,8517,8517,8518,6503,8485,8517,8518,6503,8550,8518,8517,8517,8550,8550,8550,8518,8517,6700,6536,8517,6503,4520,21356,33934,52689,50609,48563,40078,25354,10826,12743,12710,10630,19179,18983,21291,52788,56884,54770,56883,58931,58931,56883,56850,58931,58898,56850,56850,58931,58930,56916,56948,56850,222},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48828,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48794,48795,48795,48794,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,48730,46682,46682,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46649,46649,46617,46617,44569,44569,44537,44536,44536,42424,42423,38230,38230,38198,38198,38165,38198,38198,50711,50807,52887,44436,42324,38131,38099,38131,40343,42456,40277,31827,33940,40343,42424,38230,38295,38263,34103,40245,52822,42356,19345,38099,35887,25518,23472,23473,23375,14989,21294,52660,54740,52724,27438,33808,27468,14923,19180,27565,50645,54642,27599,31792,19246,17132,29549,19147,17034,21195,14954,17034,25355,17131,25516,27531,15115,36080,27628,15114,38192,29741,17195,27662,15050,27596,35950,12905,27563,27499,10857,17130,10825,12969,12808,8517,8518,8518,8517,8518,8517,8517,8517,8517,8517,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,6569,8517,10533,8518,6502,8518,8517,8517,8517,8518,8518,8518,8550,8518,6569,6569,6535,11053,30002,36015,33902,52689,54705,42093,44271,35983,10728,12710,10630,10597,12808,19048,40307,54836,56884,52689,52722,44075,50513,56883,54770,52559,39979,56818,56817,39849,35656,50577,56916,52558,127},
{46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,46780,48828,48828,48827,48827,48827,48827,48827,48827,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48795,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48762,48730,48730,48730,48730,46681,46681,46681,46681,46649,46649,46649,46649,46649,46649,46649,46649,46649,46649,46649,46617,44569,44569,44569,44537,44536,44536,42424,42423,38230,38198,38198,38198,38198,38198,38198,42357,52791,52790,46516,44372,40179,40211,38132,40343,42456,38230,27699,27731,40343,40376,38230,40311,38263,34103,40277,54903,50710,34002,33970,33839,33872,25553,23473,21262,15021,17069,29550,27502,42128,27470,21164,19147,10762,12875,19148,35759,44048,27599,21229,8747,8714,10762,10794,10729,8649,12907,8779,19377,13035,27727,25581,13002,33967,23468,15050,29709,27596,19211,27662,17130,27694,29774,12905,36081,31757,12872,25548,19243,12937,12873,8518,8518,8550,8517,8518,8518,8517,8517,8517,8517,8517,8517,8517,8517,8517,8517,8518,8518,8517,8517,8517,10565,10565,8517,8517,8517,8517,6503,4553,4488,6503,8518,8518,8518,8518,8518,8518,8517,8518,8517,4455,15018,25678,38257,44499,27596,31790,52689,54705,50610,52658,29579,31919,33870,33870,31822,33935,23208,21226,50675,54803,54738,42125,23208,42223,56850,50511,35787,25418,54769,56817,29481,25353,44336,56883,48334,92}
};

const uint16_t Salt_Lake_City PROGMEM [320][241] = {
{57084,52891,50811,55004,57084,57084,57084,55004,52891,52859,48730,48698,48698,48698,48666,48698,48698,48698,46618,46618,44538,42458,44538,44538,44538,40345,31828,27666,25553,25586,23505,25553,25586,25585,25585,23505,23505,23505,23473,21425,21425,21457,21425,21457,21457,23473,23505,23505,27666,25585,23504,23504,25553,27665,27665,27665,27633,25585,25553,27633,25521,25521,25553,25552,25521,25553,25553,21392,19312,19280,19344,19312,19312,17231,21392,25553,25520,21294,19213,19214,17133,21262,25488,17068,23407,21262,21229,25487,17068,25455,21262,21262,25487,17068,23407,21261,21229,25455,19148,25455,21261,21261,23407,17068,25455,21261,21261,23375,17035,23375,21229,21229,23342,17035,23342,19116,19149,21229,48632,19084,21229,19117,14857,14890,17003,14890,10696,12809,12809,10663,10597,10630,8581,12838,25582,48696,50777,35922,25324,48631,40209,31785,27592,19144,19112,12741,14949,17094,14917,10692,14917,17062,17094,19174,19142,17029,14885,14886,14885,14950,16967,29353,27469,12711,21288,29615,31728,38066,35791,14792,16937,50711,35983,19143,14885,10660,8547,8612,8580,8516,6436,4291,4291,8517,12742,16936,14822,10597,4291,6404,4323,4323,4291,6404,6404,6404,4291,16870,23047,29320,42095,33772,25449,21222,19175,12900,12836,14982,12868,10756,8611,10724,8611,10755,12868,10692,12805,17031,27597,46486,29678,21225,25420,48599,17002,8614,21254,21222,50614,14792,33873,48632,54905,61244,46388,23113,25194,27209,31370,31402,116},
{55004,50811,50779,55004,55004,57052,55004,55003,54971,54971,52891,48698,48698,50779,48698,48698,48698,48698,48698,46618,44538,42490,44538,44538,44538,40344,31828,27667,25553,25586,25586,25553,25586,25585,25585,23537,23537,23505,21425,23473,23473,21425,23505,21457,21457,23505,23505,21393,23505,27665,23504,23504,25553,25585,27665,27665,27633,27633,25553,27665,25553,25553,27601,25520,25553,25553,25553,25552,21392,17232,17232,17232,17232,19312,19344,23440,27568,14890,6502,8680,8616,12842,25488,19116,25455,21294,21294,25487,19116,25487,21294,21262,25487,19148,25455,21261,21262,25455,19116,25455,21261,21229,23407,17068,25455,21229,21261,23375,17068,23407,21229,21229,23342,17036,23342,19116,19149,21262,23342,19084,21229,19149,12777,14890,17035,12777,10631,10697,10697,14823,12742,12710,12774,17063,27662,44470,48665,48632,48664,44469,33935,23367,23402,19146,16999,8548,12901,12868,10692,8548,10724,10724,14949,19142,14948,21287,14917,14950,17062,17030,21160,31402,29648,12776,23402,29615,31728,35986,35758,16903,14822,31757,19143,19142,14917,10692,10692,8644,6499,4355,6436,4323,4323,8516,6436,8548,6435,4323,4291,6436,8548,4291,4291,6468,4323,6404,4291,18918,27239,27239,44208,27562,23368,23335,27532,21258,19142,14980,10819,10755,8643,10755,12900,10787,10787,17030,21223,23273,23307,38067,27566,23340,29647,40211,10694,8582,21254,25384,48533,19017,25454,44406,52824,59163,46323,31402,31402,31370,31370,31434,119},
{54971,52859,52859,54971,54971,54971,55003,54971,54971,54971,52891,50778,50778,52859,50811,50778,48698,48698,48698,46618,44538,44538,44538,46618,46618,42457,29780,29748,29746,27666,25586,25586,25585,25586,25585,23505,23505,23505,23473,23505,23505,23505,21425,21425,21425,23505,23505,23505,23505,25585,23473,23472,23505,23504,25585,25585,25585,27665,27633,27633,27601,27601,27601,25521,25553,25553,27601,27633,23472,17232,19280,19312,19312,19312,19312,23472,27601,25487,23407,23407,23407,25455,25487,21229,25487,23342,23342,25488,19149,25487,21262,23310,25487,19116,25455,21261,21262,25455,19116,25455,21229,21229,23407,17035,25455,21229,21229,23375,17003,23375,19181,21229,23375,17036,23342,19116,19117,21262,21229,19084,21229,21197,14858,14890,17036,19084,21197,19116,19117,19115,14855,14824,14887,12806,29743,50712,50745,52825,50777,38098,23338,21256,23339,21225,16998,8612,10692,15013,14949,17062,10692,12805,14917,14949,17062,14918,21288,17062,19142,17029,33872,42195,29616,21228,23370,29582,31760,33839,27466,19111,19079,23368,21223,19110,17030,12804,8547,8548,6468,8548,10629,8516,6403,10692,8579,6532,6435,2211,4291,8580,6468,4323,2211,4323,2243,6436,6437,16838,25094,27175,42127,31723,25480,21287,27534,21225,19143,17095,15012,12899,8675,12932,15012,8707,10819,12836,21255,27465,25451,35921,27565,23371,31760,31759,10694,10663,17029,25384,46420,27501,19082,38132,50712,57050,48403,42064,37773,31402,29322,31402,117},
{54971,54971,54971,54939,55003,54972,54971,54971,54971,52891,52891,50811,50778,50811,50811,50811,50778,48698,48698,46618,44538,44538,46618,46618,46618,42425,29748,29748,27666,27666,27634,25586,25586,25553,25585,23505,23505,23505,23473,23537,23537,23505,23505,23473,21425,21425,23505,23505,23505,25585,25553,23472,21424,23504,23504,25553,25553,25553,27665,27633,29713,27601,27633,27601,27601,27601,27601,27601,21392,17232,19312,19280,17232,19312,21392,25553,27568,12809,6503,8680,6503,14890,25487,21262,25488,23342,23374,25487,21229,25487,23342,23342,25455,21229,25487,21262,21294,25455,19149,25455,21262,21261,25455,19116,25455,21229,21229,23375,17036,23375,21229,21229,23342,17003,23342,19117,19149,25455,25390,33841,38067,40180,29647,21164,38035,38067,42261,42261,42261,42261,38034,29549,21225,6469,25550,48664,50777,50777,48632,48631,33902,31788,36017,19146,19143,12773,10660,14917,17062,12804,17030,14950,17062,12805,12805,12805,21259,14886,21255,17062,25388,46453,35921,27501,10662,10566,25421,29676,21191,21224,31692,31724,17030,19142,17029,12836,10660,12774,10693,14886,12741,8548,4355,6499,8612,4387,6435,4291,6468,4355,4323,4291,4259,4323,4291,4291,4291,16870,27175,29256,44208,31723,25481,23368,29614,25422,27564,14980,14979,10755,12899,12899,12931,10787,12868,12836,19175,29578,25452,38067,27566,25452,31792,29645,10726,14922,19109,29642,46420,40114,12711,38132,42292,57050,44242,44274,37838,31370,31402,31402,119},
{52891,54939,55004,55003,54971,54971,57052,57052,54971,54971,52891,50811,50779,50779,50778,50811,50779,50778,48698,46618,44538,46618,48698,48698,48698,36087,29748,29748,27699,27666,27666,25586,25586,25553,25585,23537,25585,23505,23473,23473,23505,23505,23505,21425,21425,23473,23473,23505,23505,23505,27633,23505,23504,23504,23504,23504,25521,25553,27633,27633,29714,27633,27601,27601,23472,27633,27633,25553,21392,17232,19312,19312,19312,19312,21392,42358,27568,23374,21261,23374,21262,23374,27535,21262,27536,23375,23375,25487,21262,27535,23375,25423,25487,23342,25487,23375,23375,25487,23342,25487,25423,25422,25455,23342,25455,23375,23375,25455,19148,25455,21261,21229,23375,19116,23342,21229,23342,35954,40148,48599,54938,54970,35954,21196,50745,50745,54938,54938,54938,54938,54938,52857,40179,8582,25550,48696,50777,29583,12679,48632,50777,42324,40244,17068,19112,12773,8580,17095,19174,12837,12773,12773,12837,14917,14917,14917,12806,14918,21288,19144,29582,46486,33874,27503,33648,16743,23274,23336,19110,25353,31725,35951,16998,14949,17030,8548,6435,6403,12773,10661,12773,10692,6403,4323,6435,6435,6467,8580,8580,6436,6436,6436,8548,8516,8516,4323,4291,16838,25095,27240,42127,29610,29642,29611,29615,25422,25450,12899,10787,10787,10723,8611,15011,12899,12835,19142,19143,31723,27467,31695,21162,17000,27502,27469,12742,12711,16934,27498,46419,50711,10598,31728,38067,59130,39984,31435,33547,33580,35661,35725,137},
{52859,52891,57052,57084,57052,57084,59196,59164,59164,57084,54971,52891,50779,50779,48730,48730,50778,50778,50778,48698,46618,48698,50778,50778,48698,33974,29748,29748,29747,27666,27666,25586,27666,25586,25585,25585,25553,23505,23505,23505,23505,23505,23505,21425,23505,23505,21425,23473,23505,23505,25585,25553,23504,23504,23472,23504,23473,25521,25585,25553,27633,27633,27633,27633,27633,23440,21424,19311,19344,19312,21392,19312,19280,19312,19344,31859,27600,12809,8616,10761,10696,17036,25487,23374,27536,25423,25455,25487,21229,25487,25455,25455,25487,23342,25487,25455,25423,25455,23342,25455,25423,25423,25455,23342,25455,25423,23375,25455,23310,25455,23342,23342,23407,21262,23375,21262,25455,35987,40180,48631,54938,54970,38002,23244,50745,50745,54938,54938,54938,54938,54938,54938,52858,12808,25550,46584,50777,50745,50712,50777,46549,27530,38129,17036,19145,14950,12773,17030,17062,14917,12837,14949,14917,14918,10693,14950,17031,19142,17062,14888,19082,42260,35954,27470,18922,21064,23337,19046,23272,27466,27498,21159,14885,12805,10692,10693,8580,10661,10692,8580,14885,12741,4355,4323,4323,8580,6531,8644,8579,8580,8612,8580,8612,8612,8580,4323,4291,16838,29320,27239,37902,29579,27530,29644,29615,25422,21255,15012,8611,8611,17095,14949,14979,12899,14948,6499,10693,12741,21161,31663,25420,23339,40114,31694,16968,16936,21160,33740,44339,57050,10598,25389,36019,59130,39919,31402,31402,31402,31402,31435,121},
{54972,57052,57084,59132,59164,59132,59164,59164,59164,59164,57084,57052,52891,50811,50778,50778,48730,50779,50779,48698,46618,50778,50811,50778,48698,36087,29748,29748,29748,27666,29746,27634,25585,25585,25553,25553,23505,23537,23505,25553,25553,23505,23505,23505,23505,23473,23473,21425,23473,23473,25585,27633,23504,23504,23472,23472,23472,23473,25553,25553,25553,29714,27633,27633,27633,25521,19312,17264,19312,19312,17232,17200,17231,19312,19344,25585,27568,19116,19116,19181,19148,23374,25487,23374,27535,23407,25455,25487,25423,27536,23375,25455,25455,23342,25487,23374,25455,25455,23342,25487,23374,23375,25455,23342,25455,25423,25455,25455,23342,25455,23375,23374,25455,23310,23375,21262,33841,59163,57083,48599,54938,54970,35921,21164,50745,42325,33841,38067,35986,31793,35987,38099,40245,6437,29744,48599,48632,48664,48664,48631,40174,27559,40241,19116,17036,19146,17064,17094,12837,14949,10660,10660,12805,10660,12806,10661,14950,19111,14917,16967,16936,29582,12679,21131,14793,14726,25450,23336,19079,31725,27531,21190,19142,14885,14917,14885,14917,14853,8580,12836,16997,10660,8516,6468,6436,8580,8580,8611,8676,8580,8612,8644,10724,8580,8644,6500,6436,6372,14886,23335,35886,37998,29579,33806,29615,27502,31758,12867,16999,6467,10724,12837,17093,17062,16999,8580,12806,23307,33773,35922,23307,23371,42292,35921,14921,12776,21192,31724,44307,50679,8485,12711,33907,54970,37838,31435,31435,31435,29354,31467,124},
{52891,57084,59164,59132,59132,59132,57084,57084,57084,57084,57052,57084,57052,54971,52891,52891,50811,52859,50779,50778,48698,48730,50778,48698,46618,36119,27668,29780,29748,29747,27666,27666,25585,25585,25585,25553,25553,23505,25553,23505,23505,23505,23505,23505,21425,23505,23473,21425,23473,21425,23473,29714,25553,23504,23504,23504,23472,23472,25552,23504,25553,27665,27633,27633,27633,25552,23472,19344,17264,17232,17232,17232,19312,19279,19279,23440,27601,14922,12842,14987,12809,21196,25455,19116,27536,21229,23342,25487,23342,27536,21229,23310,25455,19084,27535,21229,21261,25455,19084,25487,21229,21261,25455,14923,25455,21229,23342,25455,21261,23310,23342,23342,25455,21262,23375,25455,31761,52890,50745,48632,54970,54938,35921,21164,50745,31728,8518,14857,10664,12712,14890,19083,29680,12744,27631,50745,50777,50745,50745,46551,36012,31751,44401,19149,10697,8583,14919,17062,14949,14949,12837,12773,10693,8581,8549,12805,19143,21256,21257,19112,27502,40148,31696,25389,16936,14693,25418,23337,19078,19047,16934,19142,14916,16997,16933,16966,14886,14853,12773,19110,14885,10660,8580,8580,10661,10693,8580,8612,8579,6500,8579,10756,8579,6435,4323,6468,4324,6469,17064,19206,31724,44306,27466,29580,29648,27535,23305,12867,12805,8547,12836,10723,8579,8547,8548,6435,10693,21258,31657,40179,25452,25419,48566,40147,17002,10662,25387,35950,44339,25356,10630,10630,33939,52792,35725,31435,31434,31435,31402,31435,120},
{50811,55004,57084,57084,57052,57084,55003,57052,57084,57052,55003,55003,55003,52923,54939,54971,52891,50811,50778,50778,48698,48698,48698,48698,46650,44505,27700,29748,29748,27699,29714,29746,25585,25585,25585,23505,25585,25553,23505,23505,23505,23505,23505,25553,23505,23505,23505,23473,23505,21425,21425,27633,25553,23472,23504,21424,23472,23472,23504,23472,23504,27633,27665,27633,25553,25553,21424,17264,17264,17200,17200,25585,29681,31761,31793,31761,27601,14890,14890,17035,14857,21229,23407,19148,27536,21229,23342,25455,19149,27536,21229,23342,25455,19116,27536,21229,23310,25455,19084,25487,21261,21261,25422,12777,21294,19148,23375,25487,19083,17003,21229,23341,25422,25390,27535,27535,27568,27567,29648,31761,33874,33841,27567,25422,29680,27535,23341,23309,23341,23341,25390,27502,25454,27535,31857,48664,50745,31761,16970,46486,46516,36008,44434,21229,17003,17036,21226,17030,17030,19174,14949,14917,14885,10693,10660,12837,14950,16999,16998,19113,27501,40148,38068,25421,16903,18983,21191,21191,23240,19047,19078,23303,21191,21223,19110,19078,16966,19110,21190,19142,19109,14853,12773,10660,10660,8548,8580,10692,8611,10692,8580,10692,8644,6435,4355,8580,8615,12874,12902,19206,31725,42226,21224,25386,27567,21258,12867,10787,10723,10660,6499,10724,8579,8580,6435,8580,10693,19145,33771,35952,31499,37418,48436,38002,19082,8582,31792,40145,42258,19017,8518,10631,29680,54905,35693,33515,31435,33483,31435,31435,117},
{50779,52859,54971,54971,54971,52923,52891,52891,55003,57084,54971,52891,52891,54971,57052,54971,50811,50778,50778,50778,48698,48698,48698,46618,48698,48698,31861,27668,29748,27668,29715,29747,27666,25586,25585,23537,25585,25585,25554,25553,25553,23505,23505,25585,23505,23505,23505,23473,25553,23505,23473,25553,29714,25553,23504,21424,23472,21392,23472,23472,23472,23505,27665,27633,25585,25553,23505,17264,17232,17232,17232,27665,31793,33874,33874,31794,27600,17068,17068,19148,17035,23342,23374,19149,27568,19149,23374,25455,19148,27536,21229,23342,25455,19116,27536,21229,23342,25455,19116,25488,21261,21262,25390,14857,10696,12776,25488,25455,19083,21228,23309,25389,25422,25455,25422,25455,25455,25454,25454,25454,25422,25454,25422,23374,25422,23342,23341,25390,23341,21261,23309,33841,42293,42326,31857,48696,50777,46486,40148,52825,50777,38093,44434,21262,17036,19116,21257,19175,21223,19207,17029,17062,14949,14885,14949,17061,16999,14918,17031,21226,29614,31761,33842,25420,19048,16998,19111,25353,21127,19047,23272,21191,23303,21223,19046,21191,21191,21191,21255,21254,19142,17029,16998,14853,14822,10661,10661,10660,8580,10693,8612,10692,8612,8612,6468,6501,10826,10793,10725,19206,35919,40145,19143,25419,27535,27534,14948,12899,10755,8579,10723,8611,6499,6467,10724,6467,8516,14919,25449,29612,27370,35305,44308,33808,16937,8550,25421,38034,42258,29550,8485,12711,27534,50679,46322,48468,50581,33580,31435,31435,120},
{50811,50779,48730,50811,50779,48730,48698,46618,52891,57052,54971,52891,52891,52891,55003,54971,52891,50810,50778,50778,48698,48698,48698,46618,48698,48698,38232,27668,29748,27700,27667,29714,27666,25585,25585,25585,25585,25585,25585,27666,25585,23505,23505,23505,23537,25553,23505,23473,25553,23473,23473,23505,25585,27633,23504,23472,23472,23472,23472,23504,23472,23472,25585,27665,27633,23505,21392,17263,17232,17232,19280,25553,31793,33906,33874,29681,27568,8583,8615,10761,8583,21196,23374,21229,27536,19149,23374,23375,19148,27568,21261,23374,25423,19116,25488,21229,23310,25455,19116,27535,21261,21261,25390,14857,17035,19148,25455,25455,19115,19115,21261,25389,21229,21229,25454,23341,21196,25455,25422,21228,25422,25422,21196,23341,25422,21196,21229,25422,21261,21229,25422,21229,21196,21228,31793,46518,48631,48632,48632,44438,42325,38097,42355,25455,17036,17068,19147,21228,21224,19144,19143,17062,19174,14916,14949,17061,21256,14949,12837,14886,27565,40212,33840,21192,21224,19079,16966,21160,19047,25386,21160,16966,21191,21158,19078,19046,16998,12773,14853,12804,12741,14854,14853,10628,8516,10661,10629,8580,10693,10661,8612,8580,6468,10597,8516,4324,6469,6469,10693,19206,37999,42226,19111,29645,25422,27534,12836,10723,17030,8579,8675,10723,8579,8612,10723,8611,8611,10756,21256,27531,29483,35499,50614,33744,16905,8549,21227,42195,44339,40115,8485,10631,23340,57017,54872,56952,52759,31467,31435,31402,119},
{50811,50779,48730,48698,48731,48699,46650,46618,50779,52891,54971,52891,52859,50811,52891,52891,52891,52859,50811,48730,48698,48698,48698,48698,48698,48698,44538,29748,29748,27700,27667,29747,27666,25586,25585,25618,27666,25585,25585,25585,25585,23505,25585,25553,25585,23505,23505,23505,21393,25553,23505,21392,23505,27665,25585,25553,23504,23472,23472,23472,21424,21424,21392,27633,25553,21392,19312,19344,17200,19280,19312,27601,33874,31793,31761,29681,27568,23374,23374,23374,23342,25455,23374,21229,27568,19181,25423,25423,19149,27568,21229,23374,25455,19149,27536,21229,23342,25455,19149,27536,21229,21229,27470,14857,10696,12777,25488,25455,21163,25422,25454,25389,27503,27535,27567,27503,25455,25455,25455,25455,25455,25455,25422,25422,25422,25422,25422,25422,25422,25422,25455,25422,25390,23342,27534,50712,50777,50777,50745,48632,42325,44437,44437,27568,17036,17068,17068,17068,19147,21289,17062,17029,19206,17061,17029,14981,17093,14981,17062,14917,23338,38066,27498,21224,19111,19079,19079,21160,19048,19048,19048,16966,21158,16965,16966,14854,12741,12742,10629,8516,8548,10629,8516,6468,10693,10660,8548,8580,10661,8548,10692,12774,10693,14823,16935,10629,8549,10662,8612,19206,35919,38000,17030,27500,25422,25452,12867,12867,10691,8579,8643,8643,8611,10724,10756,8643,10787,12867,12837,17000,19144,23371,52792,33743,18986,6469,17032,42227,44339,48566,8453,12711,25356,54872,54872,54872,54872,31467,31435,31402,124},
{50811,50811,48730,48698,48698,46618,46618,46650,50779,54972,57052,55003,54971,52891,52891,50811,50811,50778,50778,48698,48698,48698,48698,48698,48698,48698,48698,34006,29748,27700,27668,27667,29714,27666,25586,25585,27666,25585,25585,25585,25585,25585,25585,25553,25585,21424,23505,23505,21425,23473,23473,21393,23473,25553,27665,27633,23504,23472,23472,21424,21424,21392,21392,23505,25552,21424,21424,23472,19280,19312,21424,27633,33874,33906,31761,29681,25455,6471,8583,10728,6471,21229,25455,23374,27568,23342,25455,25455,23342,27568,23342,25455,25455,21261,27536,23342,25423,25455,21261,27536,23342,25390,27503,14889,12841,17002,25488,25455,19083,25422,25455,25389,27535,27535,27535,27535,27503,25455,25455,25455,25487,25455,25454,25454,25422,25422,25422,25422,25422,25422,25422,33841,44406,44406,42293,50713,50777,50777,50745,42357,33905,25421,40179,29681,17004,17068,17068,17068,17036,21227,19113,19174,10756,10724,17061,16998,19142,17061,17029,19174,17030,29677,21223,21160,19111,19079,16966,19047,19015,19016,16935,21160,21191,19143,19078,19079,16966,14886,12774,12741,10661,10629,10661,6467,6467,6468,8580,10693,10661,10693,10693,14886,10693,19080,21128,16935,10630,10662,8548,14949,31693,35952,14917,23307,27503,29647,19175,12867,10787,10723,8643,8643,8611,10723,10756,10819,12836,12867,10692,17000,8516,10629,33808,25291,14824,10630,12838,42258,46420,54905,8485,10630,27469,56984,54871,54872,50581,31403,31435,31467,126},
{52891,50811,48731,48698,46650,46618,46618,46618,52891,54971,54971,57051,57051,54971,52923,52891,50811,50778,48730,48698,48698,48698,48698,48698,48698,48698,48698,44473,29748,29748,27668,27635,29747,27666,25585,25585,27666,25585,25585,25585,25617,27665,25585,25585,25585,23505,23473,23473,21425,21393,23473,21425,23473,25553,29713,29713,25553,23504,23472,21424,19344,21392,21392,23472,25585,23505,23505,23472,21360,21360,23440,33907,38067,40180,42293,31794,27568,23342,23342,23374,23342,25487,25455,23374,27568,23342,25455,25455,23342,27568,23342,25455,25455,23342,27568,23342,25455,25487,23342,27568,23342,27503,27470,14825,14922,14923,31761,27567,21228,17035,19116,25389,21229,21197,25422,23341,21229,25454,25422,19180,23341,25422,21196,21261,25422,21196,21196,23342,21196,19116,23309,21163,29648,25454,23308,23309,19115,17035,14920,19174,17031,14886,38066,31794,17003,17068,17068,17036,19084,21229,14890,19177,8581,10693,23109,33447,17030,14981,17062,17061,21287,23335,23335,21224,21224,21192,21224,23241,19079,23273,21160,21192,23304,21224,23305,19111,16966,12741,19111,14918,14886,14886,12773,8612,6468,8516,10628,10660,8580,10692,17031,29612,14853,23273,33773,25354,14856,10597,4323,10661,27500,35887,14949,23307,27535,31696,25451,10755,10819,12899,10723,8611,17062,10724,10787,10723,10755,10755,10755,19144,12806,16968,38001,23210,16872,12775,12838,44339,48630,59130,14759,10598,14824,50581,42129,44242,46355,33515,35628,35661,138},
{50811,50811,50779,48699,48698,46650,48698,50779,54972,54971,50811,52891,54971,54971,52891,52891,50811,50778,48730,48698,48698,48698,48698,48698,48698,48698,48698,48698,31893,29748,27668,27667,33940,27666,27666,25585,25585,25586,27666,25585,25585,27666,27633,25585,25585,25553,21425,21425,21393,21425,23505,25553,23473,25553,29713,29713,29713,23504,25553,21424,21424,21392,21392,21424,25585,25553,25553,23472,27633,23504,23472,35987,42292,48534,48566,35987,23375,10696,10729,10761,10664,25422,25455,23310,27568,21229,25455,25455,21261,27568,21261,25455,25455,21261,27568,23310,25455,25455,21229,27568,23342,27471,27470,14824,12745,14890,38100,31761,23309,21196,27502,27502,27502,27535,27567,27535,27535,27535,27535,27535,27535,27535,27503,27503,27502,27502,27503,27503,27503,27503,27535,27535,27535,23374,21228,21228,19148,19083,21225,12837,12837,17030,38098,33875,16971,17036,17036,17036,17003,19116,14922,19147,10662,6436,12612,14660,10565,29613,10662,17030,25450,23335,25448,25385,25418,27498,27499,23273,21160,21160,21160,21224,23272,21224,21192,12740,14886,19080,19079,19111,12773,16999,16998,8580,6435,6403,10661,12773,14854,14854,19111,29548,14789,19047,19048,21129,8517,6436,4355,6468,27533,35918,23271,27533,27567,31728,27531,14980,10723,10755,8643,12804,19144,12805,8611,8579,12804,10819,12899,21257,17031,29677,52824,25386,23178,23274,14950,44371,48630,59098,23211,8517,14824,56952,31402,31370,29322,31435,31435,33483,126},
{50811,50811,50779,48731,48698,48731,50811,52891,50811,50811,48730,50779,52891,52859,50811,50811,50811,50778,48730,48698,48698,48698,48698,48698,48698,48698,48698,48698,40280,29748,27668,27667,33908,29746,29746,27634,27634,25586,25553,25585,27633,27634,27634,27633,25585,25553,23473,23473,21425,21425,23505,21392,23473,25553,27633,29713,29713,25553,27633,25553,23505,21424,21424,21424,27633,25553,21392,21360,27666,21392,29778,59197,63389,63389,63389,42293,27535,17003,19083,19148,19051,31761,27535,23342,29648,21196,27535,25422,23309,29616,23277,27503,27503,23309,29648,25390,27503,27535,23277,31728,25390,31664,27470,19083,36019,52858,59164,52825,31728,31760,46486,42228,46486,52793,50745,50745,52793,52793,52825,52825,52825,52825,52825,52825,52825,54905,54905,52857,54905,54938,54938,52857,35954,23374,21196,21228,21228,19147,19177,14886,10661,19111,40178,36020,17003,17068,17068,17036,17036,17066,10726,8613,8581,10693,6468,6372,10565,29614,8551,12777,23307,25447,23367,21223,23336,23337,25418,21160,19079,19079,21192,19079,21192,21192,23273,16934,12741,16966,21224,19079,14790,10596,8548,10693,6403,6436,12709,19046,21127,21128,19143,27435,14917,16997,16965,14822,4323,8548,4323,6404,27468,37999,29578,31727,29680,33841,27532,15012,10787,10755,8643,10723,10723,6467,8611,10723,10756,12931,8611,25450,12773,29646,52824,27498,25324,23275,17063,46452,50678,57017,31630,8485,19017,54872,29322,31402,31402,31402,33515,31435,125},
{50811,50811,50811,50778,50779,52891,52859,50778,50779,50779,50779,50778,50778,50778,48730,50779,50779,48730,48698,48698,48698,48698,48698,48698,48698,48698,48698,48698,44505,29748,27700,27667,33940,29746,27634,27634,25554,27634,27634,25585,25585,25585,27634,27666,25585,27634,23505,23473,19344,23505,23505,21393,21425,23473,27633,29713,29713,31826,29713,27633,23440,25553,21424,25553,27601,23440,19311,19312,19312,21392,27698,55003,63357,63357,63389,59164,57083,57083,55003,57051,57083,57083,57083,57084,57083,57083,57083,57083,59164,59131,57116,59163,59164,57116,59164,57115,59163,59164,57083,59164,46519,29583,23244,14922,21423,31922,61277,52857,44373,35953,52792,54937,54970,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59164,59164,59164,59164,59164,59164,59164,59164,57083,42293,25422,21195,19178,17064,14950,14950,14886,6469,14854,40178,38100,17004,17068,17068,17068,14921,14951,8613,8613,10725,10725,6468,8581,10566,27469,8616,21295,29646,25446,19141,19109,17062,23336,21224,23273,14854,21224,23305,21160,19079,19079,21192,19047,16967,16998,19112,16903,12742,10629,16934,14854,8548,8484,14790,21063,21127,29547,19078,19048,12772,17060,14885,21160,8515,6435,4323,6436,27501,33837,21191,33807,31728,33841,27564,19174,12932,12899,8675,12867,12899,12836,8643,8611,8643,12898,10691,21225,8548,29678,48566,23338,25356,23243,14983,48565,48630,56985,40082,8453,21130,52759,29321,31402,31370,31402,31435,31402,127},
{50811,50811,50811,50811,50811,50811,48730,48698,50779,50778,50779,50779,50779,48730,48698,48698,48698,48698,48698,48698,48698,48698,48698,48730,48698,48698,48698,48698,46586,29748,27668,27667,31860,29747,29714,25586,25553,27634,27666,25586,25585,25585,25585,27634,25585,29714,25586,23473,21425,23473,25553,23473,23473,23473,25521,31826,29713,29713,29713,29713,25553,25553,23472,25553,25521,21360,21392,19344,19312,21392,23440,52857,63357,63389,63389,61277,50777,23567,29872,31921,14857,54937,40180,33808,59196,25518,52857,48696,27502,59164,34067,50809,48664,23276,59163,33970,44405,50744,21131,59196,40115,25325,25292,14857,14987,23406,61309,52857,38034,31760,48599,59164,44438,44406,59196,50744,38132,59164,55002,46583,55002,59196,40179,44437,59196,40179,42260,57083,44406,33873,57083,50712,27502,25454,19177,17063,19143,17063,17062,17062,14886,19110,40243,40213,17036,17068,19116,23339,14918,10725,10725,12805,14950,10693,8580,6436,10565,19017,10632,16938,29613,27558,17060,21285,17061,19143,23304,19111,14917,19143,21192,23305,23273,21192,21192,19079,14886,16998,19047,14822,14854,19048,21128,16967,12773,10596,21063,21031,18950,27433,21126,18983,12740,15012,10724,10660,10692,4355,8580,8548,27468,27563,14949,35985,29648,33809,33840,19174,12899,15043,12963,15011,12899,10723,6499,8579,10755,12931,15043,16967,19014,35985,52792,23337,27467,23211,14887,50678,48597,54904,48501,8485,23211,50613,31402,31370,31370,31402,31434,31434,127},
{52923,52859,52891,52891,50811,50778,50811,48731,50779,52891,50778,50779,50811,50779,50778,48698,48698,48698,48698,48698,48730,48698,48698,48698,48730,48698,48698,48698,48699,29780,27668,27667,33973,31859,29715,29715,25586,25585,25585,27666,27634,23505,25553,25585,27634,29714,27634,23505,23473,25553,29682,23473,23505,23505,23505,29714,29745,29713,29746,29714,29681,27633,25553,25553,25553,21392,23472,21392,23472,23472,27633,54970,63357,63389,63389,61276,50777,19406,25679,29872,10696,50744,59164,59164,59164,59196,59196,59196,59196,59196,59228,59196,59196,61277,59196,59196,59196,59196,61245,59196,27437,25325,25292,14857,14890,21164,61277,50776,29582,31728,48566,59196,46486,46486,59196,52792,42260,59163,57051,48663,55002,59196,46485,50711,59196,48566,48631,59163,50744,48566,59163,54938,35986,25422,21226,19176,19144,12870,14982,19142,19175,16998,42291,42326,17036,17036,19116,19116,19179,19176,12838,8581,14951,10757,8612,8549,12678,10598,8519,14824,31694,21254,19142,16996,21254,14917,19142,17029,14853,19079,21192,23273,25354,21160,21192,19111,14918,21224,23240,25321,18983,21095,21127,18950,19015,16902,23241,25257,19015,21095,23175,18982,21221,14948,12868,8547,6467,8612,8580,8613,21257,25418,12869,33873,27535,33809,33808,29644,23399,17092,12899,12899,12867,10755,8612,6499,10723,14981,17093,16999,21126,35952,50711,23337,27499,27404,21161,52759,50711,54904,52791,40017,29517,48436,31434,31402,31402,31402,31435,31402,127},
{54971,55004,57052,54971,54972,50811,54971,54971,54971,55004,50778,50811,50811,50811,50778,50779,48698,48730,48698,48698,48698,48698,48698,48730,48730,48698,48698,48698,48698,34006,27668,27668,33940,31827,29747,27666,27634,25585,25585,27634,27634,27666,25553,25554,25554,27634,29714,27634,27634,27602,27602,23473,23473,23473,23473,25553,29713,29713,29713,31762,29714,27633,25585,25552,27601,25553,25553,23472,23472,27633,25553,52890,61309,63389,63389,61277,52858,10761,19244,31953,10696,50744,48728,33873,59196,31857,50744,50809,38357,59163,27535,48598,50744,23276,59196,36115,44373,52857,19018,61244,25324,25325,25324,14857,8551,14825,61277,52825,31695,50712,57018,54938,57018,54970,57018,57018,56986,54906,54938,54938,54906,54905,54905,54937,54905,54905,54905,54905,54905,54905,54873,52824,40180,25454,21227,19146,21227,19144,14919,17031,21223,21159,42292,42358,17036,17036,17068,19116,19116,19115,19177,10693,10693,10693,10661,8581,8485,10599,16971,12744,35921,25481,21255,17029,19142,12804,14917,17030,25386,23241,23274,21161,21160,23304,19143,14950,14886,23305,23207,25321,23209,23241,29514,27369,27370,27370,27435,29483,27370,27435,27401,27433,21222,17028,12899,10723,10724,6467,8644,12837,21256,21224,14917,29615,25422,33808,33776,38066,27594,19238,17125,10723,10755,8579,6499,8579,12836,14918,12806,14920,12773,37999,42292,19111,23338,25324,23307,50678,52791,54904,54904,52694,42162,48468,54839,48436,31402,31434,31435,29354,127},
{57052,57084,57052,57084,57052,54971,57084,57084,57084,57084,54971,54971,50811,50811,50779,48730,48698,48731,48730,48730,48698,48730,48730,50779,48730,48730,48730,48698,48698,36086,27700,27668,31860,36020,31827,27666,27634,25586,27633,27634,25585,25585,25553,25585,25554,25554,29714,31794,27634,27634,27634,23473,23441,23473,23505,21392,29714,29745,29745,29714,31762,27633,25585,23504,23472,25553,25520,23472,25552,29714,27601,48664,61309,63357,63357,61277,59131,50680,50712,52824,52793,57051,57050,54905,59163,54938,57083,57083,55002,59163,57018,59131,59131,57051,59163,57083,59163,59163,59131,59164,27405,25357,25292,17002,21391,27729,61309,54938,38066,61244,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59164,59164,59164,59164,59164,59163,59163,59163,59163,59163,44406,25422,21259,19145,17000,14921,14920,17000,19080,19143,42324,44471,17068,17036,17068,19116,19116,19116,17033,10693,10725,6436,6468,6404,8550,21131,21230,14890,31727,25449,19141,17061,19142,14885,10661,10661,25355,25355,23242,19048,16935,16998,17029,19142,21191,27434,27466,25386,25353,19047,21127,27369,19015,21064,25290,25257,14758,10597,19046,23238,17028,14916,14948,10691,6499,12836,4355,12837,21320,25449,21191,27534,25422,33841,33776,40179,25481,17125,12932,8643,10723,8611,6467,8579,12837,8580,12805,17032,12805,31627,29580,21256,25482,27404,27469,48565,54904,54872,54872,50678,56984,44209,31434,31434,29354,31435,33515,33515,127},
{57084,57084,57084,57084,59164,57052,57084,57084,57084,57084,57084,57052,54971,50778,48730,48730,48698,50779,50778,48730,48730,48730,50778,50779,50778,50779,48730,48698,48698,36119,27636,25587,27667,33908,31860,29746,25586,25617,25585,25553,25585,25585,23505,25585,23505,23505,25553,29714,29714,27634,29714,25553,23473,23473,25553,25553,29714,29713,29745,25552,27633,27633,25552,23504,23472,25553,23472,23472,23472,25553,31794,48664,61309,61309,61309,61276,59164,35986,38131,46614,38260,52890,50777,40147,59164,38035,52792,52922,44567,59163,35986,48599,52857,29615,59163,42486,48728,55002,36211,59196,27405,27438,25325,27502,40212,44405,59131,52889,38067,61277,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59164,59164,59164,59163,59163,59163,59163,59163,48631,23374,21291,17031,17001,16969,19115,19113,17032,19143,42292,44471,17068,17036,19116,19116,19116,19116,17032,12837,8548,6436,6436,4324,8485,23179,33940,14857,23274,27562,21223,19111,12805,12805,12805,14885,21128,25355,16936,14790,10661,14885,14916,14916,14885,19014,23273,23273,19079,16966,16902,23176,21031,23209,16838,10564,10565,21162,21191,21031,14979,14915,19140,10723,8579,4323,4323,10693,21287,23368,21192,27566,25422,33841,33808,40179,21352,14980,17125,12867,10755,10691,8547,10691,8579,8580,10755,17064,10724,25354,35952,23401,25482,27436,31630,46452,54969,54872,52791,50646,56984,42096,31434,31434,31402,31402,33515,31434,125},
{57084,57084,57084,57084,59164,59164,57084,57084,57084,57084,57084,57084,57052,52891,50811,50811,48730,50778,50778,48730,48698,48730,50778,50810,50811,50779,50779,50778,48730,36054,27668,25588,25555,36020,33908,29747,25585,25617,25617,25585,25585,25553,25553,25585,23505,23473,25553,25553,27666,27634,29714,27602,25553,25553,23505,25553,25553,29713,29745,27665,23472,27665,25584,25584,25585,25553,25520,25553,23472,23472,27601,48632,61309,61309,61309,61277,57083,17131,19244,34065,15115,48663,54938,46486,59164,46486,54938,55002,50809,59163,50680,54938,57051,48567,59163,52889,55002,57083,50809,59196,27437,27437,25357,38034,61277,61277,61309,52889,38066,61277,59228,59196,61276,61276,59196,59196,61276,59196,59196,59196,59196,59196,59196,59163,59163,59164,57083,59163,57083,57083,59131,57083,44406,23406,19177,14918,14920,17033,19178,19145,17063,19144,42356,42326,17036,17004,19116,19116,19148,19081,12839,8548,6468,6468,6468,4291,8518,19083,46520,33809,25356,35952,19174,17030,14852,14917,12837,14885,12709,25387,12742,10661,8548,12772,12836,10724,14950,19079,19079,23305,19079,23241,23177,27402,29450,25289,18982,8484,14823,29615,27530,27305,12836,14916,19174,25449,17030,8549,4324,10694,25513,19207,25385,29615,27502,33841,33809,40211,17061,14981,12867,12867,12899,10723,8611,10819,8643,8611,12836,17031,10756,23273,31693,21256,25482,25322,35823,46387,57050,54904,52759,52758,57017,39951,31402,33482,31434,31434,33515,31435,129},
{57052,57084,57084,57084,57084,59164,59164,59164,59132,57084,57084,57084,57084,55003,52891,50811,50779,50778,50778,48730,48730,50778,50810,50811,50811,50811,50811,50778,50779,31893,27668,25588,25555,33940,31827,29714,25618,23505,25585,25585,23505,25585,23505,23505,23505,23505,25553,25585,25585,27634,29714,27634,25553,25553,27601,27634,25553,29746,29746,29713,23473,23504,25617,23536,25585,25585,27633,25553,23472,23472,23504,46584,61309,61309,61309,61276,57116,19341,19244,34097,17260,48631,54938,46486,59163,44438,52857,55002,44502,59163,44406,52792,57051,38099,59163,46486,48631,57083,38034,59196,27437,25357,25389,35954,61277,61277,61309,54937,38067,23309,35953,59163,35953,31727,59196,42325,23243,57083,52825,21098,48599,59164,23211,40114,59163,29550,31728,57051,35921,25389,57083,44373,23276,25389,19145,10725,12806,10693,14919,17031,23372,23341,29680,29714,17068,17003,19116,19148,16999,14918,12805,6468,8581,8548,6436,6404,10631,19017,48664,42292,21194,27564,17030,19111,16998,19110,17030,10660,12773,14885,14822,10596,10661,10693,10724,10724,17062,19111,19111,25386,25354,25354,27402,27369,29449,27369,16902,10597,16967,27469,29612,27402,14949,19173,27563,40113,29677,16969,4324,8582,25481,19207,14950,31696,27535,33841,33808,40179,17093,15013,12932,12932,12867,12867,8611,12900,10787,8643,10755,17063,12836,31693,27532,17063,19143,19079,46388,31564,59131,54904,52758,52758,59097,37806,31402,31402,29322,31434,31435,29354,118},
{57052,57084,57084,57084,57084,57084,59132,57084,57084,57052,57052,57052,57052,57052,54971,50811,50811,50778,50778,48730,48730,50778,50811,50811,52859,50811,50811,50778,50779,31861,27668,25588,25587,31828,29779,27666,25586,23505,25585,25585,23505,25585,23505,25585,23505,23505,25585,25585,23537,23505,25586,27634,25553,25586,27633,29714,25553,29714,29746,29713,29713,25585,25617,25584,25552,25585,25553,25553,25553,27633,25553,44439,61309,61309,59196,59131,59164,36018,35986,44437,36018,52824,52825,40115,59163,40180,52792,54970,42389,59163,44405,50712,57051,40083,59163,46486,50647,59131,40115,59164,27437,27405,25357,35922,61277,61277,61309,52857,38067,54938,57051,59163,57083,57050,59196,57083,57018,59163,59163,57050,57083,59163,57018,57050,59163,57018,57018,57083,57051,57018,57051,57050,44340,25388,21226,10725,10725,8613,12807,29614,38067,25422,23310,23375,17068,14955,19116,23308,14887,12805,10693,6468,6404,14789,12676,6437,14857,14791,29614,12743,17031,19142,17061,17062,16997,14885,10692,14917,10660,12773,12741,12773,8548,6468,10692,12804,17029,14949,17030,23274,21128,27403,27466,27433,25320,21127,21096,12677,14918,14823,8453,16968,10628,8548,19113,33743,31630,19017,6437,12743,19144,14949,14918,27535,25357,27502,35922,42292,21318,19237,17125,12868,8643,10755,10691,8611,10755,10756,8643,17064,12835,21225,10660,17062,19207,19143,37935,42161,59163,54904,50678,50678,59097,35725,39919,44242,50580,54806,52694,52727,197},
{57052,57084,57084,57084,57084,57084,57084,57084,57084,57084,57052,57052,57052,57051,54971,50811,50811,50811,50779,50779,48730,50779,52859,52891,52859,50811,50811,50778,50811,29780,27668,27668,27700,31860,31827,29747,27666,25585,25585,25553,25585,23505,25553,23505,23473,23505,23505,23536,23504,23505,25553,27634,25553,25554,25585,25585,27633,27633,29714,31762,29713,25553,23504,23536,23504,25585,25585,25553,25553,27601,23473,27633,33939,33939,46519,54970,61244,54970,54970,55034,52857,57051,57051,52825,59163,52857,57018,57083,50809,59163,50744,54937,57083,46518,59163,52922,54970,59163,46551,59164,29486,25325,25325,35921,61277,61277,38099,27599,33874,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59196,59164,59164,59164,59164,59163,59163,59131,59163,59163,57083,57083,38033,17000,17001,17032,12807,8581,12775,46486,38100,25423,27535,25423,17068,14923,21197,21162,16998,10725,10693,6468,4323,14724,10564,8517,19050,12742,19081,8548,19142,19143,17061,17061,14917,10660,17062,17061,12804,14949,14853,14854,10661,6435,8580,14917,14917,17061,10660,10629,10661,19015,21159,19078,19142,14885,10564,23177,12741,19081,16937,16967,8484,6404,23275,42195,38034,27502,6437,21258,23401,14981,17063,31760,31696,33809,33841,42292,21286,15044,12868,10691,8643,10787,10723,8643,10691,12900,10755,19176,10723,23370,12772,12836,12805,14918,33742,42258,61244,54904,50646,52726,56984,50613,48533,42096,42162,44274,31467,25161,87},
{57052,57052,57084,57084,57084,57084,57084,57084,59132,57084,57084,57052,57052,57052,52923,50811,50811,50811,50779,50779,48730,50811,52859,52859,52859,50811,50811,50811,50779,27668,27668,27668,27667,27667,33940,29747,29747,25585,25585,25585,25586,23505,27666,23505,23505,21457,23505,23505,23505,23505,25553,25553,25586,25553,25585,25585,25585,25553,29714,31794,29714,25553,23504,23504,21424,23504,27633,27633,25552,27633,25553,27633,27633,23472,42326,54938,61277,19309,14889,38259,14953,44405,50744,31631,59164,35922,46486,54970,36147,59163,40147,48566,57018,27502,59163,46615,46550,57083,31760,59131,29518,25325,25325,31728,59164,59164,40246,25454,21228,27502,35955,42260,48664,50744,50744,50712,48664,48664,48632,48631,48631,48599,48599,46518,46519,46518,46518,44438,44438,44406,44405,42293,29615,23307,21162,12774,8549,8549,33840,46518,40148,27568,27568,29649,19149,17003,19148,21194,14853,12837,8548,6468,8517,14856,6436,6469,6436,6468,8581,14950,14949,19175,19110,19174,14980,14917,19110,19174,14917,17061,12837,12774,10629,8516,8580,14885,12772,17061,8548,6468,6436,8516,12773,10692,14949,14917,12741,12710,8516,14887,12743,14887,6404,8517,21130,40114,40082,33808,6405,21290,21288,21319,21224,33841,33776,33809,33809,40178,17125,17157,10788,10788,8611,10787,10755,10755,8579,8611,10724,19208,10755,12837,10660,10693,14886,16998,33806,44306,61243,54872,50613,50646,46453,50614,44307,42129,37806,31467,31402,31435,121},
{57052,57052,57084,57084,57084,57052,57084,57084,59132,59132,57084,57052,57051,57052,54971,50811,50811,50779,48698,48698,50779,50810,50811,50811,52859,52859,50811,50810,46618,27700,27668,27668,27668,25587,27667,25618,27666,29746,29714,27634,27634,25553,25586,23505,23505,23505,23505,23505,23505,23505,23505,21425,23473,25553,25553,25585,25585,25585,29714,31794,29746,25585,25584,23536,23504,23504,27665,25553,27633,25553,25553,27601,27633,29681,42359,57051,61277,19341,12841,36210,19373,42389,59163,59163,59163,59163,59163,59163,57083,59163,57083,57083,59163,57018,59163,57083,57083,59163,55002,59163,29550,25325,23244,14889,17067,17068,25455,25454,19050,21196,23309,23309,25455,27535,27567,27535,27535,27535,27535,27535,25455,25455,25455,25455,25455,25422,25422,25422,25422,25422,25422,23342,21229,19114,12806,14918,17033,12743,50680,48567,42260,29648,29616,31761,21229,17004,21229,23276,14920,12805,6500,6404,8549,23339,14950,6468,12774,6436,8548,14950,12868,19142,19175,12805,10661,8580,25450,19142,14949,17029,14917,10661,8580,14886,12805,12773,8581,8580,10693,6435,8516,8516,6436,10661,10660,16997,12741,14822,8484,12678,12711,14791,16903,14790,12743,35921,38001,35921,6437,19112,23369,23367,27499,35921,33841,33809,35921,38064,12899,14980,10788,12836,10723,10755,10755,12867,8643,10724,10691,19208,10691,12804,10660,12805,19207,19142,38064,44339,61243,54839,48565,50712,46454,54905,48565,35725,31434,31467,31435,31435,129},
{57052,57052,57084,57084,57084,57084,57084,57084,57084,59132,57084,57051,57051,57051,54971,50811,50811,50778,48730,48698,50779,50779,50811,50811,52859,52859,52859,50811,46585,29748,27668,25588,25555,25587,29747,29779,29746,29714,29714,27666,27633,25585,25585,25553,23473,23473,23473,23505,23505,25553,23505,23505,23473,23505,25521,25553,25585,25585,27665,27633,29713,27633,23504,25616,23536,23472,29746,23504,27633,27633,23440,23472,25553,29681,27569,40180,61276,21389,17098,36146,14921,40211,52824,19050,59196,31728,42228,55002,32051,59164,36019,40147,57051,18985,57051,38067,38002,57083,25518,57050,31631,25325,25325,14890,19181,19181,25488,31729,38035,38067,19116,17003,19116,17035,17036,19116,19116,19084,19084,19084,19084,17036,17036,17036,19084,17035,17035,17003,17003,17003,17036,17003,19114,12807,8613,12838,14856,35921,48631,48599,42293,31761,31761,33874,23342,14955,19148,17001,12807,10692,6500,6436,10693,21223,8612,6468,12870,6468,8580,6435,14917,17061,12836,10693,8548,12774,17063,19110,19141,17061,8580,10693,10661,19142,14885,10661,6436,6403,6436,8549,6468,10629,10628,14886,12774,10596,16903,12742,16937,21131,10598,16967,10597,10630,10598,31663,42195,40114,10598,10631,14887,19142,25385,33841,33841,33808,35953,40178,15012,10787,10755,12900,10787,12867,12899,10787,8611,10724,10756,14951,10787,17062,10660,14981,21352,21287,40176,46452,61243,54839,48598,57117,61277,54905,50646,52661,59001,33515,31435,31467,129},
{57052,57052,57084,57084,57084,57084,57084,57084,57084,57084,57052,57051,55003,57051,54971,52891,50811,50810,50778,50778,48730,50778,50811,52891,52859,52859,52859,50811,48666,29748,27668,25588,25620,25587,27667,33940,31827,29714,27634,25585,25585,27634,25586,25586,25553,23505,23505,25553,23505,23505,25553,25553,23505,25553,23473,25521,23505,25585,27665,27666,25585,25585,23504,23536,23536,27665,31827,25585,25585,27633,23440,23440,21392,25521,27633,40213,61276,59196,61244,59196,61244,59196,59196,61244,59163,61244,61244,59196,59228,59196,59196,59196,59164,61244,59163,59196,59196,59163,59196,59196,31599,25292,25357,14857,14955,17068,23375,23374,27503,23342,17036,17036,17036,16971,17036,17036,19116,17036,19116,19116,17036,17003,17035,17035,17036,17036,17035,17035,17036,17035,17001,14954,17065,10694,12806,12807,29647,50712,48631,48631,42293,35987,33874,38067,25455,14921,12839,10726,8613,10693,6468,6436,14982,12837,10693,8613,6468,10757,8548,8580,14949,10724,8580,10693,12741,14854,10693,10692,17061,17062,8580,12805,12773,16997,12741,10693,10661,8548,6436,8517,6403,8548,14887,23274,12742,12709,12710,21129,10630,10664,10631,16935,12742,10630,14857,16937,29549,29549,25356,21163,19050,12774,21193,33808,33808,33776,35921,33903,17092,15044,12963,12867,10691,10756,8643,8643,10724,10692,10755,19144,14980,12868,17029,17062,21288,21255,40209,44339,59163,52791,50646,59229,61276,57017,54872,54872,59033,31402,31402,31434,126},
{57052,57052,57084,57084,57084,57084,57052,57084,57084,57052,57051,54971,52923,55003,54971,52891,52859,50811,52859,50811,50811,50811,50811,52891,52859,52891,52859,52859,50811,29780,27668,27668,27668,25588,27667,33940,29747,29714,29746,27665,25585,25585,23505,25586,25553,25553,25553,25553,25553,23505,25553,27634,25585,25553,25553,23505,23505,23505,25585,27633,27633,25553,23504,23536,23536,33907,33907,25585,23504,25552,23440,25521,23472,23472,29681,38132,61276,33970,25518,44533,23373,42292,54970,23276,59163,35954,40147,57083,32051,57115,38067,40147,57115,19050,57050,44534,35953,59164,16937,54938,31631,25325,25325,23277,29648,21261,17035,17035,17003,14923,12777,14890,17036,17003,17068,19148,14922,14922,14955,14955,17003,17003,14922,14922,17003,14955,14955,17003,14889,14887,10694,12808,14919,10661,12838,8581,16969,52857,48631,48631,44405,40180,33874,40180,27600,14985,10693,10693,6500,10725,10693,12805,10693,8644,12837,12870,6436,8581,8612,12837,12804,10692,10692,10692,10693,14885,10692,14917,8547,12837,12805,17029,19141,17030,16998,12773,10660,10628,10629,12742,6436,19049,19082,25356,14823,10629,10629,21127,8517,12777,14824,21224,8516,4291,6405,14856,38000,38001,35888,31695,27469,19048,19048,33776,33808,31728,33841,40210,19270,19237,17123,12931,17095,10787,12931,10787,10755,10692,12869,21256,12867,21287,21221,17029,21287,21288,42322,44372,59130,54839,48533,57083,61276,57017,54904,46387,50581,31434,31435,31435,128},
{57052,57052,57052,57084,57084,57084,57052,57052,57052,54971,54971,50811,50779,54971,54971,54971,52891,52891,52891,50811,50811,50811,50811,52859,52891,52891,52859,52859,52891,33973,27668,27668,27668,25555,27667,27667,31827,35988,29714,27666,27665,25585,25553,25554,25553,25586,25553,25553,25553,23505,27634,29714,27634,27633,25553,23505,23505,21424,23505,27633,27665,23504,23504,23504,23536,33939,29714,25552,23504,25553,23472,27633,25553,23440,27633,33939,61277,23535,19373,38290,12938,38099,59131,54937,59163,57051,57051,59163,57083,59163,59163,59163,59163,59131,59163,59163,59196,59196,59164,59163,38002,38067,27503,19083,19116,17036,23310,19084,27535,27568,19148,21229,21229,23277,23309,23309,23309,23341,23341,23341,23342,23341,25389,25390,25422,25390,23341,21226,14952,10694,10662,10662,12807,14951,14919,8581,27533,50712,48631,50711,44406,44374,33874,42293,29713,14985,12871,10693,8580,10693,12837,17126,14949,12869,14982,10693,10693,8580,12837,14981,14916,14981,10660,14885,12836,17061,12772,12772,10628,10628,10628,17029,19110,14917,12773,12741,10693,8580,12774,10661,8516,25387,19050,27501,14759,10597,12742,27436,10630,12744,14857,19081,10630,8518,8486,10630,35855,37969,35888,33743,27469,19049,21194,31695,35889,33776,35953,38129,19270,17189,17189,12963,17095,8611,10723,12899,12867,8643,10756,21289,8579,21288,21221,17030,19176,21224,44338,46420,59130,54871,48500,54872,57050,54904,57017,33547,33547,33515,33515,33515,130},
{57052,57052,57052,57052,57084,57084,57084,57052,54971,52859,52859,50778,50746,54971,54971,54971,54971,54939,54939,52859,50778,48730,50811,52891,52891,52891,52891,52859,52859,42360,27700,27668,27668,27668,27668,29748,33940,33940,29746,27634,27666,25585,25585,25553,23505,23505,23505,25553,23505,25553,27634,27601,25553,27634,25553,25553,25553,23473,23505,27633,27665,25552,23504,23504,25584,29778,25553,23504,27665,25553,25520,25553,25552,27601,29681,33939,61277,25648,19405,36242,15019,36018,57083,38292,59163,42293,42325,57115,36245,57083,46615,46615,59163,23340,54970,46647,44566,57115,25422,40180,25422,23309,23309,27567,31761,35922,38002,44341,52793,44374,40147,46486,50712,48567,52792,52857,52825,52825,52825,52825,52825,50712,52825,52825,46486,40180,33904,19176,17064,14887,10661,6469,6469,12807,12806,14919,36017,48632,50712,50711,46486,44406,33809,40148,31794,14920,10726,12838,8580,10693,6468,14982,19239,12869,17095,12838,10725,6500,10725,8612,14981,17061,12836,19142,17061,14885,17029,14949,16998,10660,12772,17029,19110,12773,10628,14885,12740,14885,8548,10693,10628,27500,16969,31662,14791,8452,6404,12711,6437,8485,8518,8485,6437,6437,6437,10631,29549,33710,31630,33743,33743,29549,21129,29582,35889,33808,38001,38097,19270,17190,12931,10819,12837,10724,10756,10787,12868,12931,12867,21257,10692,17029,17061,14917,17095,19207,44371,46484,59098,54839,46420,52791,52791,54872,56952,46355,41999,37773,33548,33515,128},
{57052,57052,57052,57052,57084,57084,57084,57052,54971,50811,50811,50811,50810,54971,54971,54971,54971,54971,54971,52891,50778,48698,50778,52891,52891,52891,52891,52891,52891,50778,29748,27668,27700,27668,27668,27668,31860,31859,29747,27666,25586,25553,27634,27666,25553,25585,25553,25553,23473,25553,25553,27634,27634,27633,25553,25553,25553,25553,23537,23504,25585,23536,23504,23504,23504,29746,27666,23472,25553,27601,25520,25585,25520,29681,29681,35987,61277,48631,44470,48695,46486,52825,59164,48728,59164,52890,54938,59196,50841,59163,54970,52922,52890,46519,50712,44438,38099,38067,40148,44406,50745,52857,46551,48632,54938,54905,50680,50744,52825,46486,44308,48599,50712,50679,52825,52824,52792,50744,50712,50744,50679,46453,48566,27503,21229,21228,21225,19144,12838,10694,8581,10661,10662,8550,8581,17064,40211,48631,50712,50712,48599,46519,35954,42325,35955,17035,17001,8549,6436,8581,8581,8548,14950,17062,14950,10725,10661,8580,12805,10692,14980,14916,14917,19109,17061,12804,12805,14885,14949,10660,14949,17060,19141,14917,12804,14981,12805,10692,10628,8516,10628,35952,16937,29614,14791,6404,4291,6372,8517,4324,6404,6404,6404,6404,6405,8518,27501,40081,35856,33775,35888,27435,19081,27469,35889,33841,35953,38096,19270,19270,17156,10787,14982,12836,12868,10788,8643,12899,12899,19176,12837,12772,10723,10659,12804,19207,44370,48597,57017,52759,46420,52791,52759,54904,56985,54839,50646,50646,50614,48468,184},
{57052,57052,57052,57052,57084,57084,57052,57084,57052,50779,50779,50811,52859,54971,54971,55003,54971,54971,54971,50811,50778,48698,50778,52891,52891,52891,52891,52891,52891,52891,42327,29748,29748,27667,27668,27668,29780,33908,31827,27666,27634,25585,25553,25585,23505,23505,25553,25585,25585,23505,25585,27666,29714,27633,25553,25553,25553,25553,23537,23537,23537,23504,23504,23504,25585,31827,25553,27633,31794,29681,29681,27633,25553,27633,29713,25553,31859,31891,33939,33939,35987,29778,29746,31827,27666,27666,27665,27633,23504,25553,35923,25488,23440,27633,29746,38164,40277,50745,54938,54938,52858,54970,46518,48631,52857,54938,46486,50712,50712,54938,52857,48599,52857,50712,52824,54970,52857,38002,27502,25357,29583,31696,21163,17035,17003,19146,21257,17062,12806,10661,12806,14950,12806,6469,8549,25519,38100,48599,50711,52824,48599,48600,38067,46551,38100,17036,17034,12806,6468,10726,8549,10694,17065,14982,14950,14950,12773,10564,12710,12677,14885,14917,19141,21254,17061,14884,12804,17061,14949,17029,14884,14948,14916,14885,12740,12836,10660,8516,8580,10660,8580,42291,14856,31662,14759,10566,8517,8517,14824,8518,10598,10598,8518,8518,8518,8518,19050,35888,35855,33743,33775,27468,14855,23308,35921,33809,35954,38096,19270,19237,19237,17157,17125,15012,17125,15013,17093,15044,8611,19176,10692,16934,8611,12772,10692,21287,46483,48598,57017,52726,46420,52791,52791,54937,59130,42129,27274,27274,27274,27274,134},
{57052,57052,57052,57052,57052,57052,57052,57084,55004,50778,48698,50811,52859,54971,55003,55003,55003,55003,54971,52859,48698,48698,50811,52891,52891,52891,52891,52891,52891,52891,52859,31828,27668,27668,27668,27667,27667,33908,31827,29714,27666,25585,25585,25585,25553,25585,25553,25553,25553,25553,23505,23473,27634,27666,25553,27601,27634,25585,23504,23537,23537,23536,23536,23536,25585,25553,25553,31827,29681,27633,25553,27633,27601,23440,25520,21392,21392,21392,25488,23440,25488,23472,23408,36020,23473,21360,21328,21360,19279,21392,29649,21328,17167,21391,21359,23439,21326,35955,42260,38099,35954,40180,40179,42292,46486,48567,44373,46453,46518,50744,50679,44373,52824,50711,52824,54970,57050,50712,48599,38034,27535,21196,21196,19083,19148,23338,17063,14950,10693,10693,10693,12838,12839,8517,6404,44406,48600,48599,50712,52792,50679,50745,42260,50712,40181,17004,17002,10693,10727,12807,12841,14953,17065,14950,12773,8516,10468,10468,12614,14694,12581,8452,14854,19110,14948,19141,14917,14949,12772,19173,19173,19174,14949,14916,17029,12772,12773,12806,8581,8548,8580,46517,19050,31695,14791,8517,8485,10598,14824,10598,12711,10598,8518,10598,8518,8486,12679,35855,33775,33775,31663,35920,27533,27534,38034,35954,38099,38097,27626,25546,25578,27627,25578,25547,27627,29740,21288,10692,10660,21193,10724,12772,19143,10724,8580,23338,48565,48565,56984,50646,46452,52759,52791,57017,59130,39984,27274,27242,27242,27274,113},
{57052,57052,57052,57052,57052,57052,57052,57084,55003,52891,48730,52891,54971,54971,54971,54971,54971,54971,54971,52891,50778,48698,50778,54939,54939,54939,52891,52891,52891,52891,52891,38199,27668,25619,27667,25587,25587,29715,31859,31827,27634,27633,25585,25585,25553,23505,23505,25553,25553,23505,23537,23505,23505,25553,27633,29714,29714,25553,23505,23504,23504,23536,23536,23472,27666,25521,31827,31795,29714,29714,27633,27601,25552,23472,23440,23473,23472,21392,25488,25521,21360,23440,23408,29747,21360,21360,23440,27666,19279,21392,29649,25488,19279,21360,21359,23439,21358,35955,44308,33938,31728,40148,42228,38002,38034,42228,46453,48599,48599,38034,31630,42228,46486,44373,46453,50711,48631,44405,42260,23309,21229,21196,14890,12809,17034,19176,17031,12838,10661,8581,6469,10694,10661,8581,10661,48664,52858,46519,50712,52824,50712,50745,42260,50712,42294,17036,17003,17002,17035,14953,17035,19146,12838,17063,14822,10468,10468,10468,12614,14694,14662,12613,16902,16997,17029,17028,17029,14949,14884,19109,19141,17029,19141,14917,17029,10692,12837,12773,10693,10660,10692,50742,25357,29582,16904,10565,6404,6404,10598,10598,10631,8485,6405,8485,8485,6405,10598,31662,35888,35888,25291,27532,23370,21162,35889,33841,38034,33902,19239,19206,19238,19206,17158,19207,19239,25580,17094,8611,8644,19080,10756,10723,6499,6467,6435,16998,50645,50678,54872,50645,46420,50678,54839,57017,59130,37903,27274,27274,27274,29323,111},
{57052,57052,57052,57052,57052,57052,57052,57052,55003,52891,50778,50779,52891,52891,52859,54971,54971,54971,54971,54971,52891,50810,50778,54939,54939,54939,54939,54939,54939,52891,52891,42392,27668,25619,27635,25587,25587,25587,29747,31827,27633,27666,25585,25585,25553,23505,23473,23505,23505,23537,23505,23505,23505,25585,25553,29682,27601,25585,23505,23505,23504,23504,23504,25553,33907,29682,29714,31794,25553,29714,31794,25520,29713,23472,23472,23440,23472,23472,25521,23440,21392,25521,23440,25586,17199,19247,15119,17167,21359,25585,29649,25488,21327,21359,21359,25552,21325,33874,44308,25454,25325,27406,25325,23276,38035,44373,50712,50712,50712,52793,50679,54906,54938,50744,54970,54970,54938,40180,42228,33808,23342,17035,17068,17068,19148,19209,14950,10757,8613,8549,8549,6469,10661,12838,12806,46583,46551,46519,50712,52825,50712,52858,42261,50745,44407,17036,17004,17036,17036,17035,17036,19115,17064,14918,16871,10468,10468,10468,12613,14694,14694,10533,10565,12741,14885,14949,12741,14917,17029,17061,19141,17061,17093,16997,12804,12772,17029,14885,12805,12805,10692,50743,31696,27469,21163,12711,10598,10566,10598,14824,10598,10598,10598,10598,10566,8518,10566,23275,33775,33775,16905,23274,21225,19145,33808,33808,40147,33869,19270,19238,17189,17189,17157,19238,19206,23433,14980,12868,8612,16999,8612,12805,6467,6467,8516,14918,50678,50711,52759,50613,46420,50646,54904,57017,59131,37839,27274,27274,29322,29322,112},
{57052,57052,57052,57052,57052,54971,57052,57052,54971,50779,48698,48730,50778,50778,52891,54971,54971,54971,54971,54971,54971,52891,52859,54971,54971,54939,54971,52891,52891,52891,52891,34006,27700,27668,27668,25587,25619,25587,29747,31859,27634,27666,25553,27666,25586,25553,23505,23505,23505,23505,23505,23505,23505,23505,23505,25553,27633,23505,23505,21424,21456,25585,23472,25585,29714,31794,25553,29746,25585,25521,29681,25520,27633,23472,23472,25553,23440,23472,25488,23472,21392,25521,29714,25585,19279,19247,19248,21360,19279,25585,27569,27537,17167,19215,19246,21358,19213,25552,19213,21229,25324,25390,25325,12712,6405,10631,59164,48567,38034,40115,29550,38002,44373,46486,44373,48598,48566,44405,44405,25422,17003,17036,12810,10697,17066,19144,14982,10757,10661,8581,8549,8549,8549,6468,8614,17032,31759,46519,50712,52825,50712,54906,44406,52857,46519,17036,17004,17036,17036,17036,17004,17003,19147,17095,16839,10468,10468,10468,12581,14694,14694,8420,8453,8453,8452,12709,12741,10564,12804,14948,14949,12804,14884,14917,10660,10660,12773,12741,12772,17062,10660,52856,38034,25356,25356,12711,8517,10599,10663,19083,16937,16937,12711,12712,14857,16905,14825,18985,29517,29550,23244,25389,27533,27532,35921,33808,40147,31821,19270,19238,17189,17157,17157,19238,19238,19205,12868,12804,8547,8547,12869,8579,6499,6435,8548,10661,50646,50743,52726,50613,46388,50646,54904,57017,59131,35726,27242,27274,29322,29322,113},
{57052,57052,57052,57052,55004,54971,55003,55003,50811,48698,48698,48698,48698,50778,52859,54971,54971,55003,54971,54971,54971,54971,52891,52891,54971,54971,54971,52890,54939,52891,52891,34006,27700,27700,27668,25587,25619,25619,27667,31827,29746,29714,27634,27634,27634,25586,23505,23505,23505,21425,23505,23505,23505,23505,23505,23505,23505,23505,21424,21425,23505,25585,23472,29746,31795,29714,25521,29714,25520,27633,29714,29713,29713,27601,25552,27633,23440,21360,23473,21360,21392,21392,21360,19279,27634,23473,21360,25585,27633,25553,23472,29617,21359,21359,19246,21358,17165,23471,17164,21261,25292,25390,25324,12744,17002,27534,46519,27535,29615,35954,33841,31760,33841,35921,35921,35921,44373,38002,35921,27535,23342,17003,17035,17068,17068,21288,17062,12837,10725,10661,10662,12838,12806,6436,8582,14984,27597,46486,50712,54905,52792,54938,44373,52858,48665,17036,17004,17068,17068,17036,17036,17004,17003,21225,23081,10469,10468,10468,12581,14695,14695,12614,14694,14694,14694,14694,14790,14822,16934,16997,12804,19110,14949,17030,12773,12804,14917,14885,10660,17030,14885,54969,42260,23243,27469,14791,8517,10598,10631,21164,19050,14857,8518,10631,10663,12744,10631,18985,35888,37969,19017,21226,25452,29646,44340,33809,42228,31788,19270,19238,17189,19237,19205,19205,21318,19173,12804,12868,8547,8547,8579,8612,8580,8547,8516,12742,48533,54904,50646,50645,46388,46453,54904,57018,59130,33645,27242,27242,27242,27242,113},
{57052,54971,52859,52891,52891,52891,55003,54971,48730,48698,48698,48698,48699,50778,52859,54971,54971,55003,54971,54971,54971,54971,54971,54939,54971,54971,54971,54971,54939,54939,50778,34006,29748,27668,25587,25588,25587,25619,27667,31827,29746,29714,29714,25586,27634,25585,23505,23505,23505,23473,23473,23505,23505,23505,23505,23505,23505,23505,23505,23473,27665,25585,29746,35988,31795,29714,29714,27601,31795,31827,31794,27633,27601,31826,29713,27633,25520,25520,21392,25521,23440,23440,21360,19311,17199,21393,19248,31891,27634,17199,23439,29552,25553,19311,19246,21326,17165,27730,17132,23374,25324,27438,27470,19083,16970,23277,36019,27567,19116,16970,16970,16970,19050,19050,19050,19018,25389,40180,38067,29648,19116,17035,14890,10697,14922,17065,12902,15014,14918,10726,10694,8581,8549,6469,10661,10757,27596,46518,50712,52825,52792,54971,46486,54906,52826,17068,17004,19116,17068,17068,17036,17036,17003,17003,27243,10469,10469,10468,12581,14694,14727,12614,14694,14694,14694,14694,14694,14694,16870,19142,21223,14949,10692,14885,10660,12772,14949,14949,12804,14885,12740,59163,48567,19082,29614,14824,12711,10630,12711,19083,19050,16937,10598,12743,12743,12744,10631,12678,35855,40082,23211,27532,31758,29581,42227,33809,40179,29707,19270,19238,19270,21285,19205,19141,19140,14916,10692,17029,12836,19174,14949,8580,6467,6435,6435,14855,46420,54937,48565,50613,44275,35921,56985,57050,59098,31532,27274,27242,29322,27242,113},
{54971,52891,50811,50779,50779,50811,52891,50779,48698,48698,46650,48698,48698,52859,54971,54971,55003,55003,54971,54971,54971,54971,54971,52891,54939,54939,54939,52891,54939,52891,38200,27701,29748,27668,25587,25587,23507,25587,25587,36053,29746,27666,27666,25553,25553,27634,25553,23505,23505,23505,23505,23505,23505,23505,23505,21457,23505,23505,23505,25553,27665,27633,33907,33875,29714,27634,27633,31795,33875,31827,27633,33907,25552,27601,31794,25552,25552,25553,23440,23440,25520,29681,19279,23440,19280,17167,21361,19247,17167,21360,23439,37971,23407,25520,27633,21359,19277,29746,23438,23374,25390,31696,25390,17003,16970,17035,17003,14890,14858,16970,19083,19050,19082,21163,16937,19050,27470,31728,29583,27503,23374,17035,17003,17035,19146,14983,12870,12902,10725,10694,12870,8581,6469,8613,10790,12870,14950,46551,50712,52825,50744,54938,46519,54938,52891,19116,14923,17036,19117,19116,19116,19116,19084,17003,35759,10468,10468,10468,12581,14694,14694,10533,10533,10533,10533,10533,10533,10533,14789,14885,16997,19206,17029,12741,12772,12836,10724,14917,14948,17030,12773,59195,52825,19018,31695,14824,10565,12711,14823,19050,16937,16905,14791,10597,10630,8581,10662,12679,29517,38001,18985,23211,35984,29580,42260,35921,42260,27561,19270,19238,19238,23398,19173,19173,17028,14885,12772,16997,10692,19174,17029,8579,10660,6467,4355,16968,44307,57017,48533,48533,42194,48631,57050,59098,57017,31500,27242,27242,27242,27274,112},
{52891,52891,52891,52891,52859,50811,50811,48730,48730,48698,46650,48730,52891,54971,54971,55003,55003,55003,54971,54971,54971,52891,52859,52891,54939,54971,52891,50778,50810,50810,38200,29749,29748,27668,25588,25587,25587,25587,25555,31860,29779,29714,27666,25585,27634,25553,25553,23505,23505,23505,21457,23473,23473,23505,23505,23505,23505,23505,23505,27665,27665,27665,31827,33907,29714,21360,29714,31794,29714,25553,23440,27633,29681,27601,29713,29713,23472,25552,23472,21391,23472,29681,17200,21360,21360,17167,29715,19248,19247,23472,23472,27601,29746,25584,23504,21358,21358,33939,36019,25455,19116,21196,19116,21229,17003,19115,25390,21197,19083,19083,19115,19083,21195,21196,21196,21163,31696,44373,44438,31761,17035,17003,14922,12842,17033,17063,17063,12870,10693,8581,10757,6468,6501,8613,8613,6500,12869,40210,48631,52824,50712,54971,48599,54938,55003,19116,17036,19149,19149,19149,19149,21197,21197,21196,42066,10436,10468,10468,12549,14695,14694,10501,8453,8453,8453,10501,10501,8453,12709,14853,14917,17029,14916,14885,16997,17061,17125,19174,17061,12772,10660,59196,57051,19017,33775,16872,8517,10597,12773,19111,16968,16936,8517,8549,8516,8516,10630,16905,25356,33743,19050,19114,31791,29645,42260,35921,44341,25480,19270,19238,19238,19205,19109,19173,17029,19046,14885,14885,12772,14917,8579,8579,6467,6467,6467,25418,44274,57050,48500,48533,44307,52857,57017,59131,56985,31468,27274,27242,27242,27242,112},
{54971,52891,54971,54971,52891,52891,50811,48730,48731,48698,48698,50779,54939,54971,55003,55003,55003,55003,54971,54971,54971,52891,52891,52859,52891,54971,52891,50811,50810,52859,34007,27668,27668,25587,25587,25587,27668,25587,23475,29747,29746,27666,27634,25585,27634,27634,25553,23505,23537,21457,21457,21425,23473,21425,21424,23505,21457,23505,23472,27665,27633,27633,29714,35988,29682,27601,33875,29682,27666,21360,23473,23472,33939,29681,31762,29713,27665,23472,25552,21392,25553,25585,21393,21361,21360,19279,27666,33972,23472,19279,19279,19279,33939,21391,19278,21326,21261,21294,21229,25422,19116,21196,23277,21197,23277,31728,46487,40180,35954,40148,40147,38067,42260,42260,42260,42260,42260,27470,23309,21196,19148,17068,17003,14922,17035,19146,15015,14950,12838,8613,8613,10693,12870,10757,10693,6468,10693,27596,50711,52824,50712,54971,48599,54938,57116,21197,17068,21229,21229,21229,21229,23310,23309,23309,46324,10469,10469,10469,10501,14694,14694,14694,14694,14694,14694,14694,14694,14694,14694,14693,19078,19110,16965,14821,12709,12773,17061,17061,17061,12836,14917,61309,61309,19018,33775,16904,14822,14853,14884,19174,12804,12805,12741,10661,6468,8548,16969,14792,16937,35856,19050,19112,25451,31758,40147,35921,42324,23399,19270,19238,19173,17061,19173,21254,10692,8515,14885,12836,8612,8611,17029,8547,12837,10660,12836,29675,42194,59131,48500,48500,46420,52792,54904,59131,56952,29387,27242,29322,27242,27242,113},
{54971,52891,54971,54971,54971,50811,50779,50779,48698,48698,48698,50778,52891,54971,57051,55003,54971,54971,55003,55003,54971,54971,52891,52891,54939,52891,50778,50810,52891,52891,46618,27700,25588,25555,25587,25588,25587,25555,25555,27699,29779,27666,27666,25553,25586,29714,23505,23505,23505,21425,21425,21425,23473,23473,21425,23473,21424,21457,25585,27633,27601,25585,25552,35988,33875,27634,33875,33875,27634,21392,23472,23472,23472,25553,27633,21424,25552,25553,23472,23472,27666,29746,21328,19247,23440,29811,27667,27666,21359,21327,21295,21262,23375,21229,21229,23310,27535,35922,38067,35922,42261,50712,52793,50712,52825,50679,50680,40147,35888,44373,52792,48566,50712,52824,52824,52824,54937,50744,48631,40148,23309,17003,17003,14890,14955,17065,14950,14982,12869,12870,12837,8677,10725,14982,10758,6469,6468,14982,46518,50744,50712,54938,48599,54938,57116,21262,17068,21229,21229,21229,21230,23310,23342,25422,42163,10469,10468,10468,10501,14694,14694,12614,10533,12581,10533,10565,12613,10533,10533,10533,10533,10565,10565,8485,8485,14949,12804,17093,19173,12836,12773,63389,63422,21131,33808,16967,14884,14916,17028,17061,12804,12772,12804,10693,8580,10629,12775,12678,12678,37936,27437,27469,29678,33839,40115,35922,44405,25479,21318,12836,15012,19205,17092,10724,8612,6435,12836,14948,8548,10660,10724,12836,14948,8515,12836,33868,42226,59130,46420,46420,46452,44307,52791,57050,54840,29355,27242,27274,27242,27242,113},
{55003,52891,54971,54971,54971,52859,50811,48730,48698,48698,48698,50778,52891,55003,57051,55003,54971,54971,55003,57051,57051,54971,54971,52891,54939,52891,50778,52891,50810,57084,59197,27668,23507,23508,25587,25587,25587,25587,27668,25587,29746,25586,25586,23505,25585,27666,23505,23505,23505,21425,21393,21425,21425,23473,21425,21425,21425,23537,27633,27633,25520,25553,25552,31826,33907,33908,31795,33875,25554,21360,23505,21392,21392,23472,33940,23472,23472,23472,25552,27633,27633,38133,38066,42223,35852,35919,38100,38100,42358,44438,46486,48632,50745,52793,52858,54938,54938,54906,52857,42260,44373,48632,50744,46486,48631,44405,54938,48566,48566,50679,54905,50744,52857,54937,54938,54937,54937,35921,33906,19115,19116,17036,14890,12810,14922,17098,17094,12869,12838,10726,12805,10789,10757,12902,10725,8549,8581,17094,29644,52792,50712,54938,48599,52825,57084,27536,21229,25422,25422,25423,27503,27535,29616,31729,48599,10501,10469,10468,10501,14694,14726,10533,8452,8452,8452,8452,8452,8453,8453,8452,8452,8453,8453,8452,6372,10628,12836,14948,14916,12804,17030,63421,65471,25357,31695,19049,12709,14820,17060,19173,14916,10692,12740,8548,10628,8516,8581,10629,10629,29517,21098,23210,33807,35984,38002,35921,42259,23366,21318,23366,19173,17029,10691,14981,6499,8580,10724,12804,10724,12804,6403,8580,10724,8548,10724,33869,40081,57050,46420,46420,48533,42227,52759,57017,54839,27274,27274,27274,27274,27242,111},
{55004,52891,52891,52891,52891,52891,50811,50779,48698,46618,48698,50811,54939,54971,54971,54971,54971,54971,57051,57051,57051,54971,54971,52891,52891,52891,50811,50778,50811,50746,55036,29748,25588,25588,25587,25587,25555,25587,25619,23507,31859,25586,25586,25585,25585,25585,25553,23473,23473,23473,23473,21425,21425,21425,21425,21425,23505,25553,27633,27601,23472,25553,25552,29714,31794,33907,35923,33875,21393,19312,23472,21392,19312,19344,23472,29746,29746,21424,23472,25552,23440,27666,25488,25551,27631,29713,23440,25521,27666,42358,46519,48664,54938,52858,52857,54938,54970,54938,52857,46486,48631,52857,54938,50679,48599,44405,54906,52857,50679,42260,50679,46453,48599,52824,48631,38066,44373,44341,44373,35986,25455,17003,17003,14923,14955,14986,14950,10725,10661,12838,12902,10789,10789,12869,8549,6436,10693,17094,19175,44372,50712,52825,46486,54906,54970,50777,50712,52858,52858,52858,52858,52858,52858,54906,46486,12614,10468,10468,10469,14694,16775,14694,14694,14726,14694,14694,14726,14694,14694,14694,14694,12646,12614,12614,10565,10725,8580,4355,6403,8612,17030,63389,63422,29615,27501,21163,16904,16935,17029,19141,14884,14884,14884,10692,10693,10661,10661,12709,14791,23243,29582,29582,31759,42259,38034,38034,42292,25511,21285,19141,14916,10691,6467,8579,10724,8547,10724,8547,8612,10660,6403,6468,4355,12772,12836,31757,40113,54937,46420,44339,50613,37968,52726,57017,52759,27242,27242,27242,27242,27242,111},
{57052,54971,52891,52891,52891,52891,50811,50779,48698,46618,46650,50811,54971,54971,57051,57051,57051,57051,57051,57051,57051,54971,54971,52891,50811,50810,50778,48698,48698,42360,38166,27700,25588,25588,25588,25555,25555,25587,25587,23475,29779,27634,27634,25586,25553,25553,25553,25553,25553,23504,23473,21425,23473,21425,21425,21425,21425,25553,29714,25553,23472,23505,23472,27633,29714,33875,44374,38100,21392,17232,19280,23505,21392,21392,19344,19312,29714,23472,21424,21392,23504,19280,17200,21360,36052,25521,19247,19279,17199,23440,23472,17135,19247,29778,36019,40277,46519,50745,52825,44341,48631,52857,54938,52793,50744,46486,54970,54970,54905,46453,54937,48631,52792,54938,42292,21131,23276,33873,42389,25454,17035,17036,14890,10664,12809,17066,15016,14983,14950,10725,10789,10757,10757,10789,10725,6437,10661,14918,19143,23369,48631,50712,40180,52858,57051,50777,48632,52858,52858,52858,52858,52825,52858,54906,48567,14727,10469,10468,10469,14694,16743,14694,14694,16775,14727,14726,14727,14727,14727,14727,14694,14694,12646,12646,12773,10756,12837,8580,6403,8548,16967,63357,63422,35921,25356,25356,14791,16966,16996,17060,14916,14917,17028,12805,8548,8516,10628,12710,12711,19017,29550,31695,38066,42226,38002,38002,42291,21318,19270,21318,14980,14948,10724,10692,6467,6435,8548,6435,6403,8580,8548,6403,4291,19207,14980,36015,40081,54871,46420,44339,48565,37968,50646,56985,52662,27242,27242,27274,29322,27242,108},
{55004,57052,57052,54971,52859,50811,50779,48698,48698,46618,46618,48698,50778,52859,57051,57051,57052,57052,57052,57051,57051,54971,52891,52859,50778,48730,48698,48698,46585,52891,40279,29813,27636,25556,25587,25587,25555,25619,23507,21427,29780,29747,27698,25586,25553,25553,25553,23505,23505,23505,23505,21425,21425,21425,21393,21425,21425,23473,29714,23504,23504,23504,23504,25552,29746,31794,44342,40213,21393,17232,17200,21392,23440,27633,23505,23472,23472,23505,19311,23472,21424,19343,19279,19280,31859,25521,19280,17199,19279,21392,19311,19279,17167,21359,21327,21326,23406,31793,35986,42293,46518,50712,52793,48599,48631,46518,54970,52857,52792,48598,54937,50679,52824,54970,46518,31663,33776,38002,31760,27535,25422,14923,14890,14955,14955,14923,14890,17001,14950,10693,10757,10725,10693,8581,8582,8517,6469,14950,12838,12837,21257,29678,23339,29677,54938,50777,46519,52825,52858,52858,52858,52825,52858,54906,48599,16840,10469,10469,10469,14694,16743,14662,8453,8453,8453,8453,8453,8453,8453,8453,8452,8452,8485,12805,10756,12868,8580,8612,8612,6436,14855,63357,63422,40147,23243,27500,14853,14948,19140,16995,14916,14885,14916,14884,8548,8548,10628,10596,10598,25323,25324,27435,31725,42258,35953,35921,42291,19270,19237,19238,19205,14916,10724,8579,6467,6468,6435,4323,4355,6435,4323,4323,6435,12836,14948,38063,40081,54871,46420,44307,48500,40081,50646,56985,50581,27242,27242,27242,29290,29290,109},
{55004,55004,57052,54972,52891,50779,50779,48698,48698,46618,46618,48698,50779,54971,54971,57051,57051,57051,57052,57052,57052,54971,52891,50811,48730,48698,48698,48698,42360,57084,44505,36087,25588,25556,25588,23475,27668,27667,27668,23507,25587,29779,27698,25585,25585,25553,25585,23505,23505,23505,21457,21425,21393,21393,21425,21425,21425,23473,27666,25553,23504,23504,23504,21392,25585,29714,29714,29714,19312,17232,17232,21392,21360,23440,25520,25520,21392,19312,19344,19311,23504,21391,29779,17200,21360,21360,17167,19279,17199,25552,19279,19311,36019,35987,38100,27600,27600,31794,35987,35987,33809,38002,38035,40147,48599,44373,50712,35921,33808,42227,50744,46486,50712,50776,50744,50712,35921,40212,44437,25422,17035,17003,12810,10632,10697,17003,16970,19113,14950,8580,12805,12838,10693,10758,8614,6404,6405,14918,12837,12838,14983,12773,17031,25483,52890,50777,46519,48599,52858,52858,52858,52858,52858,54906,54840,27404,10469,10469,10468,14694,14694,14694,10533,10533,10533,10533,10533,10533,10533,10533,10533,10533,10565,10788,8612,6499,8612,6500,8580,6500,14855,63389,63422,44373,21131,29614,17030,17027,19141,17028,16964,14916,14916,12804,10660,10596,12741,14821,12678,31628,31596,31661,29612,42258,35921,35921,40178,19270,19237,17157,19237,12867,14916,8547,6403,6435,4323,6435,10628,8548,6403,6467,6467,8547,8547,40144,31628,40014,46420,44340,46420,40082,50645,54904,48436,27242,27242,27242,27242,27242,111},
{54971,54971,54971,54971,52891,50779,48698,48698,48698,46618,46618,48698,50779,54971,54971,57051,57051,57051,57052,57052,57051,54971,54939,52859,50778,50778,48698,46650,50746,50746,40279,31893,27668,25587,25555,25588,27667,25619,25588,23475,23507,29779,27666,25585,25585,27633,25585,23537,23505,23473,21457,21457,21425,21393,21425,21393,21425,23505,25553,25553,23504,23504,23504,23504,23472,27633,27633,25522,19280,17232,17232,21392,21392,19312,21392,21392,21392,21424,21424,19343,23472,31859,33940,21393,19279,25521,21360,21360,23473,21391,21391,21391,42292,42228,31728,25422,23375,23375,25455,23375,27438,35922,40180,38067,40180,42292,54938,44341,44373,48599,48631,40148,31793,31793,31793,29648,25422,19148,23342,17035,17036,17036,17036,17003,17036,17036,19115,19111,14886,12774,14886,8581,8517,8549,8613,6469,8517,17031,17063,14918,19144,14919,10693,21226,50777,50745,46518,33839,50744,52858,52825,52858,52858,54906,46453,25292,10469,10469,10468,14694,14694,14726,14694,14694,14726,14726,14694,14727,14694,14694,14726,14694,12741,10756,6435,4323,8644,6468,8612,8580,14887,63389,63390,48632,21130,33775,16999,19173,21253,19076,16997,14916,14916,12772,12709,10628,12709,14791,12679,27436,33710,29581,27532,40178,35921,35921,38097,21350,19238,15013,17061,14949,12804,12804,6435,6435,8548,8579,10660,10724,10692,8548,6403,8580,8580,33806,21129,37901,46387,44275,44339,40082,48533,54872,46387,27242,27242,27274,29322,27242,116},
{54971,54971,55003,54971,50811,48730,48699,48698,48698,46650,46618,48698,52891,54971,54971,57051,57051,57051,57051,57052,57052,54971,52891,50811,50811,50778,48698,48698,55004,48633,50778,33973,27668,27668,25587,25587,27667,23507,21395,23507,21427,27699,27666,27698,25553,25553,23505,23505,21425,21425,23473,23473,21425,21425,21392,23473,23505,23473,23473,27665,23472,23504,23504,23472,23472,23472,29746,31827,23473,21360,19312,21392,21392,21392,19312,21424,23472,23472,23472,21424,21392,38165,33940,19280,19247,25521,21328,21328,23441,23472,21359,21359,40213,35954,21164,19083,21229,23310,23342,23310,25325,27438,25325,16937,10664,23276,52825,38035,33841,44373,40180,23342,14923,14955,17003,17003,17035,17036,17036,17036,17036,17036,14923,17003,17035,17036,19178,17062,12805,14918,10693,8581,6436,8517,10661,8549,6436,10661,14918,14950,17063,19144,19144,16999,44470,29646,40212,27533,35953,46551,31759,31759,50745,52858,46486,23179,10469,10469,10468,14662,16742,14694,10533,10533,10533,10533,10533,10533,10533,10533,10534,10533,10693,12868,8612,6468,12868,10724,8644,8644,21193,63389,61309,54906,19082,33807,19111,19141,19139,16964,16933,17028,16996,12772,14884,17029,12741,16936,14825,27469,31597,29581,31693,38033,35921,35953,38129,21350,19141,10691,10692,10724,10724,17029,6467,6435,12805,14949,14949,14981,10724,10692,10660,12837,8580,27468,19017,40114,44339,42194,42195,44340,46452,54872,44242,27274,27242,27242,27242,29290,122},
{55003,55004,55003,52891,50778,48730,48698,48698,48698,48698,46618,48698,52891,54971,54971,54971,54971,57051,57051,57051,57051,54971,52891,52859,50811,50778,50778,48698,46552,48633,55004,27667,27668,25587,25555,25587,25619,23475,23475,21427,21427,27699,27666,27698,27634,25553,23473,23505,23473,23473,23473,23473,21425,21424,23473,25553,23473,23505,23505,23504,23472,23472,23504,23472,23472,23472,27633,27601,27633,27634,27602,23472,21392,21424,21392,21424,23472,23440,21391,21392,19343,40245,25618,19312,19279,19279,19248,17135,21360,25553,21391,21391,29713,33842,29648,23277,21229,23374,21294,21229,21294,25422,25325,16938,14857,31696,42293,23277,21229,21229,25455,14923,12810,12842,17036,17003,17003,19116,19116,19149,19116,17068,14922,14890,14922,14922,17033,16998,14950,14950,10693,8549,6436,6469,8613,8549,6437,6405,6469,12805,12806,12773,21255,14982,21194,19113,21225,14855,23307,16968,25452,23339,27534,52858,46486,25292,10469,10469,10468,14662,16774,14727,8452,8453,8453,8453,8453,8453,8453,8453,8453,8453,10597,12900,8612,6499,12900,10788,10756,6467,23337,63390,61309,38035,19050,31662,16936,16998,19107,19075,19076,19045,16965,16965,12709,16997,14822,16871,14791,25324,33710,33743,29644,25451,29582,33807,31757,12836,17093,10723,12804,8611,10724,12836,8579,8515,10660,10660,10660,12804,12900,14949,10660,12836,14916,23274,23178,38067,44340,42162,44374,44405,48500,54872,42129,27274,27242,27242,27242,29290,133},
{52891,52891,50811,50779,48698,50779,48698,48698,48698,46650,46618,48698,52891,54971,54971,54971,54971,57051,57051,54971,54971,54939,52891,52891,52859,50811,50778,48666,46552,50713,48665,29747,27668,27635,25555,27668,25587,25587,23507,21427,23507,25618,27666,27666,27666,25553,23505,23473,23505,23505,23473,23473,23473,23504,23505,23473,21425,21425,21425,21424,25553,23472,23504,23472,23504,23472,23473,25553,29746,27601,25553,21392,21392,21392,21424,21424,23440,23440,23472,21391,21392,29778,23505,21360,19279,21392,19279,21360,21360,29713,29713,27600,31761,31697,31729,25390,23342,25390,23342,23342,21326,29681,33777,27503,12777,17003,21229,17003,21229,17068,23309,17003,14923,17003,17068,17003,17036,19116,19116,19116,17068,17036,14955,17003,14955,14954,19145,17030,14950,12805,10725,8516,6468,6469,6436,4324,4324,4324,4324,4356,6436,12773,19142,17062,21226,19144,17031,14918,14854,17031,17000,25388,44406,52857,46486,27470,10469,10469,10468,12613,16774,16774,14694,14694,14694,14694,14694,14694,14694,14694,14694,14694,12646,12804,10755,12900,8676,8644,10788,12836,23402,63389,61309,42228,16937,31728,35921,19049,23336,23302,21158,25351,23273,16935,19016,21097,23177,23145,23177,27436,40080,40113,29612,16965,23338,21255,14916,12804,10723,14948,17061,12868,12836,8547,10660,12772,12773,8548,10661,6467,14949,12772,10660,12772,10628,19047,29549,29648,46420,46486,54971,52825,48533,54904,40016,27242,27242,27242,27242,27242,138},
{50811,50779,48730,50779,50779,50779,50779,48698,48698,46618,46618,48698,52859,54939,54939,54971,54971,54971,57019,54971,54939,52891,52891,52891,52859,50778,50810,50778,54971,46552,38134,27700,27668,27636,25555,25587,25587,23475,21427,21427,23507,25619,27666,27666,25585,25553,25553,23473,21393,21393,23473,23505,23505,21425,23505,21425,23505,21425,21457,21425,27633,25552,23472,23472,25552,23472,23472,29746,31827,25521,25553,19344,19311,21424,21424,21392,21392,21392,23472,23472,23440,23504,21392,21361,25585,29714,21360,23440,36020,42293,31728,31663,46518,40148,40148,33842,48567,46487,46519,46519,31729,29713,27567,29648,17003,17035,21229,21197,23309,31728,46486,38067,31761,35954,35954,35954,40148,40147,40148,38067,40180,40148,40180,40180,42293,42260,25418,17031,19143,14918,12773,12806,10693,8549,8549,6436,4356,4356,4292,4324,4324,6436,12837,17062,19112,21225,19143,14950,14950,10725,14886,19081,44373,52825,44405,29550,10469,10469,10469,12581,16774,14694,12614,12614,12614,12614,12613,12614,12614,12614,10533,10533,10533,12837,8579,15012,10756,10788,10788,12836,27562,63389,59196,48599,44307,52824,44373,21162,19080,23302,31756,27495,21191,19046,25321,37935,40081,42193,42194,42161,42225,21223,21223,25419,12773,19173,17125,12836,10659,14917,17061,14980,12804,8547,12772,12804,10659,10692,6467,12804,10723,17093,12772,12773,12773,14885,35855,27502,44340,48566,59197,59098,50646,56984,39951,27242,27242,27242,27242,27242,150},
{50779,50779,48730,50779,50779,50779,48731,48698,46618,46618,46618,46618,50779,52859,52859,52891,54971,54971,54971,54971,54971,54939,54939,54939,52891,50778,50778,50778,54971,44472,34005,27668,27668,25588,21427,25555,25555,23507,23507,21427,25587,25619,27666,27666,25553,25585,23505,21393,21393,23473,23473,23505,23505,21425,23473,23505,21425,21457,21457,21425,23473,29681,25553,23504,23504,23472,23472,29746,35988,27634,21392,19344,19312,19344,21392,21392,21359,21392,23440,23440,21360,21392,21392,17167,23472,27633,21392,31826,50712,52824,44373,33808,50679,46486,46486,44341,46486,46519,38035,29648,25390,31729,38068,35922,40148,44406,50745,48632,50712,50712,50712,40115,35922,44373,50712,48566,50680,52825,52825,52825,52792,54906,44373,52793,52792,50744,27563,21256,17030,17030,16999,12806,14950,10693,10661,8613,6404,6436,6436,4356,4324,6468,17063,14950,19144,23337,19111,14950,14950,14917,14854,38066,48664,46487,52727,44341,10469,10469,10469,12581,14694,14694,8453,8420,8420,8420,8420,8420,8420,8420,8420,8420,8452,10629,10724,10724,12836,12868,17092,14980,27562,63389,54906,31696,52791,56984,42194,23242,21162,31726,42193,35917,25383,23270,23334,33772,40080,42226,44306,42194,33869,14949,14949,10692,19142,14948,17061,10724,8579,8611,12868,14916,12836,12772,12836,14949,10692,10692,10660,8579,10691,14948,12772,12805,12772,8580,33775,25421,40147,44308,59197,57050,52759,59065,37839,27242,27242,27274,27242,27242,157},
{50779,50779,48730,48698,50779,50779,50778,50779,46650,46618,46618,48698,50778,52891,52891,52891,54939,54971,54971,54971,54971,54971,54971,54939,52891,48698,50778,46585,42360,34006,29780,27636,25556,23475,23475,25555,25587,23507,23475,23507,25587,27667,27666,27634,25554,25586,23473,23473,23473,25553,25553,23505,23505,21425,21425,23505,23473,23505,23505,21425,23473,27633,27633,23472,23504,23504,21424,23472,31827,23440,19312,19312,19312,19344,21392,21360,21360,21392,21424,21360,23440,21424,21360,19248,19279,21392,27633,57083,59131,57050,54937,52792,54937,52824,46486,44373,48599,52825,52858,54938,57051,57050,54970,52792,46518,48599,54970,54938,44373,48631,54938,50712,50712,52793,52857,50712,54938,57018,54970,54970,54938,57051,50679,54906,54938,54938,46550,25450,17030,17030,14918,12838,12837,10693,8613,8613,6468,6404,6469,6404,10693,14950,16998,12805,16998,23336,19110,16999,14918,14918,19113,25389,29615,21164,52728,52793,10468,10469,10469,12549,16742,14694,12614,12614,12614,12614,12614,12614,12614,12614,12646,12614,12614,12613,12773,10661,12709,12836,14980,10724,29676,63422,61309,63357,52791,56984,40049,27468,27436,44275,46420,42193,37966,37997,33802,40111,44306,44274,44274,44274,35918,16997,10660,10692,19205,17093,17094,10692,6467,8547,6467,12804,14948,12804,10660,10692,14917,14917,10692,10692,10691,12868,12804,10692,8548,6435,29581,23276,31761,27502,48599,57018,54839,59065,35726,27242,27242,27242,27242,27242,164},
{50779,50779,48730,50811,52891,52891,50811,50811,46650,46618,46618,46618,48698,50811,50778,52859,52891,54971,54971,54971,54971,54971,54971,54971,52859,48698,46618,42360,44472,31893,27668,27636,23507,21427,23507,23507,25587,23507,23507,25587,25555,27667,29746,25586,25553,25553,23473,23505,23473,23473,25553,23505,23505,23505,21425,23473,23473,23505,21457,21425,23473,25585,23504,21424,23472,23504,21392,21392,21392,21392,19311,19279,19344,21392,21392,21360,23440,23472,23472,23440,23472,21360,31763,40149,35891,29714,42326,63390,61309,59164,50712,57050,46453,54937,48599,52858,52825,54970,54938,54938,54905,52857,52825,50712,46518,48598,54970,54938,46486,48631,52857,54938,54905,44373,54970,48599,50712,54970,57019,57019,54970,57051,48566,42292,48598,48598,50678,52856,31757,17063,12870,12837,12869,12870,10725,8581,6469,6437,4356,6404,12838,14950,16998,19143,19111,14854,12773,12742,14854,12742,10694,23308,29680,27503,42228,33743,10469,10469,10468,10501,16742,16742,12614,12614,12614,12614,14694,12614,12614,12614,12614,12614,10566,10565,10565,10597,14916,12868,10724,10660,29612,63422,61309,57051,52758,54904,40081,16871,14726,40016,44144,46289,42064,37870,44176,39950,39984,39950,35822,39982,19110,10692,8580,10660,19174,14980,10691,10692,8548,10692,10660,12804,12772,8579,10724,12804,10691,14917,17030,8579,14884,12772,14885,10692,6403,6435,25388,21098,19051,16970,25356,48565,54871,57017,31532,27242,27242,27242,27242,27274,174},
{50811,52859,52859,54971,54971,54971,52891,50778,48698,46618,46618,46618,48698,50811,50778,52859,52891,54971,57051,57051,57051,55003,57051,54971,52891,50778,44505,48666,46585,31861,29748,25556,23508,23508,25556,25555,25587,21427,23507,23507,23507,29747,29746,25586,25585,23473,23505,25553,23505,23505,23473,23473,23473,21425,21425,21425,25553,23505,21425,19344,23505,23472,25585,21392,23472,23504,21424,21424,19344,19312,17231,19344,19311,21392,21392,23440,21360,21360,21359,21391,21359,21392,29714,37972,37940,40117,29681,63422,63422,59196,50647,50679,48599,57083,42227,33841,31728,40180,44373,44405,44341,48631,46518,44373,46518,46486,50712,50744,46453,48631,52825,48599,48566,46486,54970,48631,50712,54971,54970,54970,54970,57051,48599,38066,46485,48598,50679,54905,44437,14950,10693,10725,12805,10725,12870,10693,8549,6469,10630,10629,21256,17031,14918,21255,19143,14918,8581,10629,10661,8581,8581,10662,14888,21195,44405,35856,10468,10469,10469,10501,14694,14694,10533,6372,6372,6372,8420,8420,8420,8420,6372,6372,6372,6372,6372,12772,14980,12836,6499,8548,31725,63422,59164,44340,52759,54904,40081,10565,14758,40015,46289,42064,39983,42096,39951,42096,39951,31658,14884,16997,14916,14916,10660,8547,12900,14980,14981,8547,8547,10692,10692,10659,17093,10724,14917,14948,14981,12836,14917,8515,12804,8515,10660,10660,10660,8516,21130,23179,31729,16938,16937,48533,52791,59065,31532,27242,27242,27242,27242,27274,181},
{52891,54971,54971,52891,52891,54971,54971,50779,48698,46618,46618,48698,48698,50779,50778,52859,54971,54971,57019,57051,57051,55003,54971,54971,54971,52891,36087,50778,44440,27668,27636,25556,23507,21395,25588,23507,25587,25555,23507,21395,23507,27699,29747,27634,25585,23505,25585,23505,23473,23505,23473,21425,23473,21457,23505,21424,21425,23473,21424,21425,23473,21424,25585,21424,21424,21424,21424,21424,19311,19312,19311,21392,17068,12842,12842,12810,12842,12810,12810,12810,12810,12810,12810,12778,12745,14858,12777,33809,63357,59196,52761,54873,52792,57051,54938,48599,52792,57018,52825,44406,33744,40083,44405,46486,46518,48599,46486,44341,46421,48566,48599,38002,35954,42260,50679,48534,50744,52825,52825,52825,52825,54905,42260,50745,50744,50744,52825,52825,48598,27595,14982,10725,10757,10693,10693,12838,10725,10725,10726,16999,14950,12806,17063,21287,19143,17030,10693,10661,10661,12773,12805,12837,12774,21064,44373,38034,10501,10469,10468,10501,14695,14694,12614,10533,12613,12614,12613,12614,12613,12613,12581,10565,10565,10565,12741,17028,14948,12836,10692,10725,29613,63422,63422,61309,52759,54904,38001,10533,12646,35822,40016,46322,42064,37838,42064,42096,42096,27432,17028,17061,10723,17061,12772,12772,19173,14980,12868,10692,14917,10692,14917,14916,14981,14949,14917,12836,12772,14949,12806,6403,6467,6403,6403,6467,6435,4323,10598,25357,48665,46519,27437,48533,50646,52726,37871,39984,27242,27242,27242,31500,165},
{54971,54971,54971,52923,52859,54971,52891,50779,48698,46618,46618,48698,48698,48698,48698,50779,54971,54971,57051,57051,57051,55003,54971,54971,54971,42425,27734,46585,44472,27668,27668,25588,23507,23475,25587,25587,25587,25587,21427,21427,23507,27667,29747,27666,25553,23505,23505,25553,25553,23473,23505,21457,21457,21457,21424,21425,21425,23473,23473,23473,23473,23473,23472,23472,23504,21423,21424,21424,19311,19280,19344,21392,12810,8519,8454,8486,10567,8486,6438,6438,8486,8486,8486,8486,8486,8486,8486,8486,27437,57018,54938,59131,50679,54905,40115,31695,46518,57051,50745,42293,29486,27373,25292,25292,31728,42260,48631,50712,50712,54905,52825,44373,42260,48631,54938,50712,52857,54970,54970,54970,57018,57051,50712,54938,54970,54938,54970,57019,52792,33871,19143,17031,12838,14951,10758,8645,8613,8613,10790,12806,19111,12773,14918,19175,19143,14950,17030,14918,12837,12805,14917,17030,12805,25194,38067,38068,12581,10468,10468,10468,14694,14695,14694,12646,14694,14694,14694,14694,12646,12646,12646,12646,12646,14790,12709,14948,12868,10692,8579,8548,31758,63422,63390,61277,52791,54871,38001,12645,6404,8452,23178,44209,42128,44209,42096,42097,33645,25351,19141,17093,17029,17029,14885,14884,12804,10723,14948,10724,12805,10692,14916,10691,12836,14916,12804,8547,10660,6435,6436,6403,6467,6435,4323,4355,6404,6404,8517,27437,48665,50778,33743,48565,54839,54807,52726,52726,25161,27210,39984,56920,196},
{54971,54971,54971,52859,50779,52891,52891,50778,48698,46618,46618,48698,48730,48698,48698,48698,54971,54971,57051,57051,57051,57051,54971,54939,52891,27766,27734,44440,36054,27668,25588,23507,25556,25555,25555,25555,25555,25587,23507,21427,27667,27667,27699,27634,25586,23505,23505,23505,23505,23505,23473,21457,21424,23505,21424,23473,21425,21425,23473,23473,23473,23473,23473,21359,19181,17133,19213,17133,17101,17069,17101,17068,12745,10566,8486,8518,10599,8518,8486,8486,8486,8486,8486,8518,10566,8518,8518,8518,12712,35921,42260,48631,54938,52825,40115,27469,31663,40147,40179,52858,29551,27405,25292,21098,25422,48567,54938,50744,44406,40115,48599,40148,35922,44373,44405,44373,44406,46486,46486,46485,46486,48566,46486,46519,46518,46518,48566,48599,48566,40178,19176,17031,10757,10694,12806,10725,8645,10725,12805,14918,16999,12741,14918,14886,14918,12741,10661,10629,10693,10693,12805,12837,16934,29517,38067,42261,16775,10469,10468,10469,14694,14694,12613,6372,6372,6372,6372,8420,6372,6372,6372,6372,6404,12772,10723,12836,12867,12836,8612,8580,35951,59131,63390,61276,52791,54871,37968,18951,23210,31662,35855,40016,37968,37935,37903,35758,29484,21063,21158,19142,19174,21255,17061,16998,14885,14885,17093,12835,12836,8579,14949,10723,14885,10691,12804,12837,8548,6403,6467,6435,6435,6403,4323,4291,4291,4323,10629,31630,48697,50777,33808,48533,54871,52759,50581,50581,27242,25161,37871,54872,212},
{54971,54971,52891,50779,50779,52891,50811,50779,48698,46618,46618,48698,50779,48698,48698,50811,54971,57019,57051,57051,57051,57051,57019,54971,42393,25686,29815,44505,31828,27636,23507,25588,25588,23507,25587,25588,25587,23507,21427,23507,27667,27667,27699,27666,25586,23505,23473,23505,23505,23473,23473,21425,21425,23505,23504,23473,23473,23505,23473,23505,25521,23473,21360,12745,12680,12680,10632,10598,10598,8518,8518,8485,8518,8518,8486,8486,8486,6438,6437,8485,8485,8485,8485,8486,10598,6438,6438,6438,8518,8518,8550,8517,33841,48600,50680,54905,52825,59131,54938,57116,37938,27405,25292,21099,19084,29616,59164,50745,38067,23277,23309,27567,25422,25389,23309,23341,23276,31695,42293,52825,52825,57018,40147,21163,27502,54905,50712,54905,44340,16937,17000,17094,14982,10758,10726,6500,8645,19175,19175,12805,16998,14949,14917,17030,10661,10629,10693,10694,6436,8516,10693,14950,16837,35857,38067,54938,21033,10469,10469,10468,12614,14694,12614,10533,10533,10533,10533,10533,10533,10533,10533,10533,10565,17029,14980,14980,12835,12836,8612,10724,23338,37998,54937,57018,52791,52790,40080,25323,19017,25356,40080,52791,29516,18952,23114,39887,35693,23112,18983,16997,14948,17061,17029,21255,23337,12804,12804,12836,14948,12836,14916,12804,14917,10691,14916,12804,12773,6403,6435,6435,6403,4323,4323,4323,4291,6435,16935,25388,46584,50745,29582,48533,54872,56952,29387,27242,27242,27209,27210,29387,187},
{54971,54971,54971,50811,52891,54971,50811,50778,48698,46618,46618,48698,52859,48698,48698,52859,54971,57051,57051,57051,57051,57051,57019,52859,27766,25686,27766,36119,29748,27636,25588,25556,27636,27635,27636,23507,21427,23507,23507,25555,25587,27667,27667,29746,27666,25586,23505,23473,23505,23505,23473,25553,25553,23505,23505,23473,23473,25553,23505,23473,23473,21327,12713,10599,10599,10598,10598,10598,10566,10598,8518,8518,8518,8518,8486,8486,8518,8517,8518,8518,8518,8517,8517,8485,8485,6437,8486,8486,8517,8517,8518,8517,29582,46486,50712,50712,48631,57051,52825,46519,37970,29486,31599,27502,27535,17003,17003,17003,21197,14922,12777,12777,14857,12777,14857,14889,14857,14890,19115,23276,23276,25422,23276,21228,23277,25357,23276,23276,21196,21164,21195,19177,15014,12837,8613,6468,12870,8612,19143,12838,21255,21255,19142,19079,12741,10661,8581,10629,12741,12741,12741,12773,27177,44374,52825,57051,25259,10469,10469,10469,12614,14694,14694,12646,14694,14694,14694,14694,12646,12646,12646,12646,12679,14789,10692,14948,14980,12836,12836,12836,21256,40080,52792,52792,52791,52758,38000,16871,18985,31630,44339,52758,18952,14726,25259,39919,39918,31596,35855,37998,21254,25481,17093,16997,16998,12741,12805,12772,17061,14916,12804,17061,10691,12836,17029,6467,8515,8580,6435,8548,8548,4323,6467,6435,6403,6435,33742,16904,46616,48664,35888,50646,52726,56952,27274,25161,27210,27209,27210,27242,178},
{55003,54971,54971,54971,54971,54971,52891,50811,48698,46618,46618,48698,52859,48698,48698,52859,54971,57019,57051,57051,57051,57051,57019,44538,25686,25654,31927,36086,29748,27668,25588,25555,25587,27636,25587,21427,21427,21427,23507,25587,25587,25619,25619,31827,29746,25618,23505,23505,25553,25553,23473,23473,23505,23505,23505,23473,23473,23472,21424,21392,21424,27406,12680,10599,10598,10630,10598,10566,10566,10598,8518,8518,10566,8518,8486,8486,8486,8517,8517,8518,8485,8486,8517,8485,8518,6437,8485,8518,8518,8518,8518,8518,37969,42227,44340,38034,50680,54906,52793,50712,35889,23245,19084,17003,12777,14890,14922,16970,17003,14858,12777,16970,17003,16938,16970,16937,16937,14857,16970,17002,19051,19050,16938,19083,19083,19083,19018,19050,19018,19050,19050,19114,14950,14982,10757,8613,10725,6500,6468,12806,17030,21223,21223,16998,12774,10661,12806,12806,14918,14918,14918,19047,37677,48599,59164,59131,29485,10501,10469,10468,12614,14694,14694,8452,8453,8453,8453,8452,8420,8452,8452,8452,8452,10660,12868,12867,10724,12804,17028,17028,21192,46485,54937,52791,52791,50678,37968,29613,38033,61308,61243,35822,12710,19016,31695,46420,42226,50646,54872,54905,40177,29707,19142,14917,12804,12773,8547,10692,14948,14884,8611,12836,14981,14916,17029,6435,8612,8548,6435,6403,8580,8580,10692,6467,8548,6403,29582,16871,21163,42358,42194,50678,48500,56920,27242,27209,27209,25161,27210,25161,185},
{54971,54971,54971,54971,54971,54971,54971,52891,48698,46618,48698,48698,50779,50778,46650,50811,57019,57051,57051,57051,57051,57019,55003,34007,25686,29912,29814,29781,29748,27668,25587,25588,25587,25587,25587,23507,23475,23507,23475,25587,25587,25587,25619,27699,29746,27698,25553,25553,25553,25553,25553,23473,23505,23473,21425,21392,21425,21424,23505,23472,23472,27536,31404,12648,12647,12648,10599,12647,10598,12646,10566,10566,8518,8486,8486,8486,8485,8517,8485,8486,8485,8517,8485,8485,8485,8486,8485,8485,8485,8485,8518,8518,38002,48599,46485,46454,50712,59196,57051,46487,33874,17036,17035,19116,17003,23309,17003,19083,35954,16970,14890,16970,16970,19051,19050,19083,21131,21131,21163,23243,23244,23243,21163,23243,23276,23244,25324,23276,23276,23244,23244,23275,19176,19143,17031,12838,8613,6501,8517,12741,19111,21255,17029,19111,14854,10693,12774,14886,14918,16999,14886,14918,33483,48599,50712,57051,33711,10501,10469,10468,12614,14694,14694,8452,8453,8453,8453,8453,10501,10501,8485,8453,8453,10565,14916,14916,12804,16997,19077,16997,16934,35888,21097,54904,52791,50645,35888,21162,25388,40179,52759,21032,14790,12678,16937,31695,46420,52824,52824,44403,31756,19142,14948,12804,12772,14852,17029,17029,10691,8579,6467,12804,12804,10724,8547,6467,6467,8516,6435,8580,12836,8611,10724,8547,6435,8516,27468,18984,10565,23342,40114,52759,44274,54839,27241,27209,27209,25161,27209,25161,188},
{54971,54971,54971,54971,54971,54971,54971,52859,48698,46618,48698,50779,50779,50779,48666,52859,57019,57051,57051,57051,57051,57019,50779,27734,27767,31992,27734,29748,27668,25588,25555,25587,25588,25555,25587,25587,25587,25587,25619,25555,25587,25587,25619,27699,29746,27666,25585,25553,25521,23473,23505,23473,23473,23505,21425,21425,21425,23505,25585,23473,23505,25488,37548,31372,21066,21067,21066,16873,16840,16872,16872,16872,16905,16905,16905,16905,18985,18985,18986,18986,19018,21066,21066,18986,14792,12679,12680,14825,16873,16873,16873,16905,35921,54938,46518,38100,33906,27568,21229,21229,23310,17035,23309,27535,29584,27470,33841,42293,48632,42293,42260,48599,48599,44373,50680,50712,50712,50712,50712,50744,50744,52792,50744,52825,52825,52793,52793,52825,52825,52825,52825,50744,44404,35983,21257,12838,10725,6436,12838,19111,19175,19207,19142,14885,12806,12741,12806,10693,14885,12741,14886,14918,33483,48599,54938,52792,31630,10501,10469,10468,12581,14694,14694,12614,12646,12646,14694,12614,12614,12646,12646,12646,12646,14885,12804,12836,10691,16965,10660,14917,16965,46485,40179,50678,54839,50613,35888,16871,10597,12678,38001,21032,16903,8452,8485,10566,40081,50679,46484,21287,14948,10724,8547,12805,14917,14917,19174,19205,12804,10692,8547,12803,17061,12836,10692,8548,8548,8516,6403,6467,12805,12804,10692,6467,6435,6435,23274,23146,10565,21228,40179,52792,42161,52694,25161,27209,27210,25161,25161,27241,192},
{54971,54971,54971,54971,54971,54971,54971,50811,48698,46618,48698,50778,50811,48698,48698,50811,57019,57051,57051,57051,57051,57019,42425,27734,31960,29847,27701,29748,27668,27636,25555,25556,23507,25587,25587,25555,25587,27668,23507,23507,25555,25587,25587,25618,29779,27634,25585,23505,25521,23505,23473,23473,23473,23473,21425,21425,21425,23473,23473,21393,23472,23440,41806,37580,39887,39952,39984,31566,27405,25325,25357,19052,10599,10567,10534,10566,10566,8486,8486,8453,8453,8453,6405,6373,6373,6405,6373,10566,10599,10631,14857,19051,23277,25455,27536,29648,31761,33874,40180,46487,44406,46519,50680,54906,52825,48567,50680,52792,48566,38002,37969,48599,50711,48534,50744,52825,52825,52792,52792,52793,52792,52825,44340,50712,52792,50712,50712,50712,50712,50712,48599,44372,33871,38032,27532,21289,14918,10662,8613,19206,12868,12869,14982,17030,12773,8581,8580,17063,17062,14886,10661,12773,33451,50648,59164,57083,40082,10501,10469,10468,12581,14694,14694,12614,12646,12646,12646,12646,12646,12646,12646,12646,12646,14948,10724,8579,14916,14852,14885,10660,8548,42292,12711,35888,52791,48565,38000,25289,31629,42226,44306,35789,37968,40049,38001,42194,42161,44306,35919,12836,17093,14980,12804,12805,12804,21286,19141,19173,14916,10692,14917,12868,12836,12868,8547,6435,6403,8516,4323,8548,10693,8580,8548,6435,6435,6467,14855,27404,10565,19083,38099,52792,42161,48500,27210,27210,27210,25161,27210,27274,198},
{54971,54971,54971,54971,54971,54971,54971,50779,48698,48698,48698,48698,50811,48730,48730,50779,54971,57051,57051,57051,57051,57051,40312,34040,27766,27767,27701,27668,25588,27636,25588,25588,25556,25555,25555,25587,25555,25587,23475,23475,25587,25619,25587,25587,31827,27634,25585,23505,23473,23505,23473,23473,23473,23505,21425,23473,23473,21393,23473,23473,23473,23440,43919,43854,39919,39984,42097,29452,25292,23244,27373,21100,10599,10599,10599,10599,10567,8518,6373,6373,6373,6373,6373,8551,8518,10664,12744,40213,40148,52826,54971,54938,54938,57051,54970,54970,54938,54970,54938,54938,46519,48631,48566,54938,54938,48599,50712,50680,54970,50744,48566,52792,54905,50712,52857,54938,54938,54938,54938,54938,54938,57051,50712,54937,54938,54905,54938,54938,54938,54938,48663,29676,25483,25451,21193,21225,40178,33870,19208,19174,14949,14982,12805,12805,10660,10661,12805,14950,14981,17030,14854,14886,33484,48600,59164,57083,42228,10501,10469,10469,12581,14694,14694,8452,6372,6372,6372,6372,6372,6372,6372,6340,6372,10660,10756,10692,17029,14852,16965,16965,10693,31694,33808,46453,48565,40114,40146,21097,29549,42259,50711,54904,54904,56985,52791,54937,50678,48533,16967,19141,17093,17029,14980,10659,17061,17061,14916,8579,10692,6467,10692,14948,12868,14916,12772,6435,8515,8547,6435,6467,6499,6467,6467,6435,6467,6435,8548,35856,12613,14856,33873,52759,44242,46355,27209,27209,27209,25161,27209,27274,199},
{54971,54971,54971,54971,54971,54971,54971,50811,48699,48731,48698,48698,52859,50811,48730,50778,54971,57051,57051,57051,57019,55003,42425,34072,29814,27734,27733,25588,25556,23507,23475,25588,23475,27668,25588,25587,25587,25587,23507,23475,25555,25619,25587,25587,29747,27634,25586,25585,25553,25585,23473,21425,23473,21425,21425,23473,21425,21393,23473,23473,23473,25520,43919,43854,39887,39952,39952,35759,37839,35726,35759,35791,10566,8518,8518,10567,10566,8518,8518,8486,8486,8518,8518,8518,8518,8518,8518,8486,10599,10599,19018,27470,35954,44373,50744,52857,52857,52857,54938,54938,48567,48599,48566,54970,54970,46486,48599,50680,54938,54970,48599,52825,54970,50711,52825,54938,54970,54938,54938,54938,54938,57051,48631,52825,50711,50679,50679,50711,50679,48630,38031,27562,21256,21225,17063,14887,19081,27531,21288,14917,14949,19143,12869,14982,17030,12805,12805,17030,14950,19143,19111,12741,33452,50712,59132,59131,48534,10501,10469,10468,10501,14694,14694,10533,10533,10533,10533,10533,10533,10533,10533,10533,10533,10628,14948,14980,16997,14852,16997,12740,10660,21161,35920,46485,31694,25356,23275,14791,25356,44372,46485,42259,46452,46453,42259,50678,40114,31630,12678,16999,17061,17028,12868,12868,14949,14949,10723,14916,8547,8547,8547,8579,10724,12836,10692,10692,10691,12837,6467,8547,6467,6467,6403,8516,6435,6467,6467,35888,14726,10630,31761,50613,46355,44242,27209,25129,27210,25129,27209,29387,196},
{54971,54971,54971,54971,54971,54971,52891,52859,50779,48730,48698,50779,52859,50811,50811,52859,57019,57051,57051,57051,57019,57019,46618,29846,40345,27766,27701,25556,23508,21427,23508,25556,25588,25587,25587,27667,25619,25587,23475,23475,25587,25619,25587,25619,31859,29746,27666,27666,25553,23473,23473,23473,23473,21425,21425,23473,21425,21393,21393,23473,23473,23472,43919,45966,44145,44178,44210,33711,33711,33711,35792,31631,10567,8518,8486,10567,10599,8486,8486,8486,8486,8486,8486,8486,8486,8486,8486,8486,8486,8486,8486,8486,8518,8486,10566,16905,25357,33841,40147,48599,48567,50680,48566,54970,54970,46518,48631,50712,54970,54970,48599,52825,54970,50712,52825,54970,54970,54938,54970,54938,54970,57083,50679,40147,44373,44372,44372,48565,46485,46485,42290,27563,23337,27531,19112,12838,12742,19113,14918,14918,12837,19175,19175,17094,12837,14885,19143,17030,12805,14918,19111,14822,33484,50712,59164,59164,52760,10469,10469,10469,10501,14694,14694,12614,12646,12614,12646,12614,12614,12646,12646,12614,12646,14853,14949,17029,12804,14820,12805,14885,10628,14887,29581,42227,25356,29550,19049,14791,23276,31695,35953,33808,29582,33775,27469,29582,27469,23274,16966,17030,12868,12836,10723,14916,19173,14916,12836,14948,10692,8548,8612,12772,17093,12804,10724,12868,12868,12837,6435,12805,8580,8548,6435,8548,6467,6467,6403,29581,16872,10565,29681,46420,50581,44210,27209,25161,25129,25129,27209,31532,194},
{54971,54971,54971,54971,54971,54971,50779,48730,48730,50779,48730,48730,50778,50779,50779,52891,54971,57019,57051,54971,54939,54971,50779,29814,27766,27734,25556,25556,21427,23475,23508,23475,23507,25588,25587,25587,25587,23507,23507,25587,25587,25587,25587,25619,29779,29746,27666,27634,25553,23473,23505,25553,25521,23473,23473,23505,21425,21393,21393,25521,25521,23473,41871,45966,44113,46258,46291,19019,19019,19019,21132,16971,10599,8486,8486,10567,10567,8486,8486,8486,8486,8486,8486,8486,8486,8486,8485,8486,8486,8486,8454,8486,8486,8486,8486,8486,8486,8486,19051,29583,38035,46486,46454,50744,50744,46486,48631,50712,52857,46518,44309,52792,54970,50680,52825,54970,54970,54970,54970,54970,54970,57083,50679,54906,40147,46518,48599,48598,48598,46518,33870,25449,23304,19112,19111,14886,12870,12773,14950,12805,19143,19175,14885,19175,19143,16998,17030,19110,14918,14950,16999,19049,35597,50712,59164,59132,52825,10469,10469,10469,10501,12646,14694,10533,8453,8453,8453,8453,8453,8453,8453,8485,10597,12708,17061,16997,12805,8516,8548,12805,10692,16967,33807,42259,27436,25356,21130,10597,12743,14856,29614,38001,29549,10598,10598,10598,25388,23336,19109,21223,21190,14884,17029,17093,17093,12836,8579,14948,10692,8579,8547,12804,12836,8611,8611,8611,10724,12804,8579,10724,8547,6467,8548,6467,6403,6371,4323,25388,21065,10598,25455,42162,52759,42097,27209,25129,25129,25161,25161,33678,190},
{54971,54971,54971,54971,54971,54971,50778,48698,48730,50779,50779,48730,50778,52859,50811,52891,54971,52891,54971,52859,52859,54971,48698,38232,31960,25654,23476,23508,21395,25556,25588,23475,25556,25588,25587,25587,25587,23507,25555,23507,25587,25587,25587,25586,27667,27666,27666,27634,25586,23505,25553,25553,25553,23505,23505,23472,23473,21392,23441,25521,25521,23473,41871,45966,44113,44178,44210,19019,19052,19020,19084,16939,10599,8518,8486,10567,10599,8486,8486,8486,8486,8486,8518,8486,8486,8486,8486,8486,8486,8486,8486,8486,8518,8518,8518,8486,8486,8486,8486,12679,21262,31664,33711,33711,33841,42260,46518,48567,48599,38034,37970,46486,50679,48566,50712,52793,52825,52793,52825,52793,52825,52857,44340,52792,50744,50744,50744,50744,52792,52792,38097,29611,25386,25419,21256,17030,14982,14981,19142,19143,21287,19142,14918,16998,14886,10661,17030,19175,17030,12805,12741,14854,33483,35856,61212,54905,50647,10501,10469,10468,10501,12614,14694,10533,8452,8452,8452,8452,8452,8452,8452,14821,16966,12708,19109,14852,14917,12837,14885,12772,10692,14822,29581,42259,23243,25388,21162,12678,12710,14823,19082,27468,25323,10598,10598,10630,16966,14885,16965,16933,16933,21222,19141,12868,19172,14948,17060,12836,8547,8548,10660,14916,12804,8580,8579,12804,10723,12836,8579,10692,8547,8580,6403,6403,4323,4323,4323,23275,23178,12646,23374,42194,54839,39984,25129,27209,25129,25129,27209,37839,180},
{54971,54971,54971,54971,54971,54971,52859,50779,48730,50778,52891,52891,52891,54939,52859,50778,52859,52859,52859,52859,54939,54971,44505,38200,31927,27734,23508,21427,21427,23508,25556,23508,25555,25587,25587,25555,23507,25587,23507,25587,25619,25619,25587,25587,27667,27666,27666,25585,25585,25553,25585,25553,25553,23505,21425,23505,23473,23473,23441,23473,23473,23472,41871,45966,44113,46258,46291,21131,19052,19052,21132,16939,10599,8486,8454,10599,10567,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8486,8486,8486,8486,8486,8486,8486,8486,8486,8518,14858,27470,29551,29583,31826,33906,48599,54970,50745,44341,42228,50744,54937,50712,52857,54970,54938,54938,54938,54938,54938,57018,50679,54906,54938,54938,54938,54906,52857,52857,50712,46517,31725,23305,19143,19111,14982,17126,17126,17062,21255,19143,19143,16999,14854,10629,12805,19143,16998,12805,12741,14822,16902,23209,59131,59131,59196,14662,10469,10468,10469,12646,14694,12614,12614,12646,12646,12614,12614,12646,16902,17029,17030,10660,12740,10692,12836,17029,14949,10692,8548,12742,21160,33807,21130,21130,27468,14855,19049,21130,23243,25388,31695,33775,25388,25387,19142,19142,21254,19142,19141,19174,21318,17060,19173,16996,14883,16997,10628,8580,12804,17028,12835,12804,10692,14917,8611,12804,14949,12804,8515,6435,8548,6403,6371,6403,6435,19048,29484,12646,21196,40147,54872,35758,25161,27209,25129,25161,27209,39952,177},
{54971,54971,54971,54971,54971,54971,52923,50779,50779,50811,54939,52891,54971,54971,52859,50778,50779,50811,52891,54971,54971,54971,38232,29846,40345,36087,21427,23508,21395,25588,21395,25556,25555,27667,25619,23475,23507,25587,25587,25587,25587,25587,25587,25587,25587,27666,27666,25586,25586,25554,25585,23505,23505,23505,23473,25553,23505,23473,23473,25553,23505,23472,39823,45966,44080,44178,46258,21132,19052,19052,21164,23245,25260,23179,25259,25259,25260,25292,25292,27372,27372,27372,27340,27373,27405,27405,29453,29453,29453,29453,29453,29453,29453,29453,29453,29453,29485,29485,29485,27405,10730,23244,27438,21099,17068,25455,46519,54938,48632,38034,38002,42260,46453,42228,46486,46486,46486,46454,46453,46486,46486,46486,46486,48566,46518,46486,46486,48566,48566,48567,48566,42292,25417,21256,19175,27563,31758,10725,15013,17094,23400,25449,31724,17031,12806,14887,12773,14982,17094,14950,19110,19111,19078,33839,40146,57050,59131,16807,10469,10468,10468,12614,12614,12614,12614,14694,14694,14694,12646,19079,19111,19110,12836,12805,14885,14852,12836,12836,14916,14949,8580,10660,10660,29582,21193,14854,14822,10629,16936,19049,16904,21097,21097,27501,25356,25387,19143,17030,17061,12804,14916,16997,23334,14948,17060,17060,12804,14852,12708,10628,12836,17060,17060,12835,17093,10692,10692,14917,10692,12805,10724,10692,12804,8547,6435,12741,6435,12741,35790,18919,21130,40180,54904,33645,27209,25161,25129,25129,25161,42097,165},
{54971,54971,54971,54971,54971,54971,52859,50779,50778,52859,54971,54971,52891,52891,50778,50778,50811,52859,54971,54971,54939,52891,31927,36119,46585,31926,23508,21427,21428,25556,25556,25556,25556,25555,25587,25587,23507,25587,25587,25587,25587,25587,25587,25587,25587,27666,27666,25586,25585,25586,23505,23505,23473,23473,23473,23473,23472,23473,23473,21392,21392,23472,39823,48014,44080,46258,46290,23212,19052,19052,21132,27406,33646,29453,27405,35791,33678,27373,25324,27372,25292,25292,25292,25292,25292,25260,25260,25260,23212,25260,23212,23212,25292,23212,23211,23212,25260,25292,23211,21132,12974,19051,27405,21131,23342,40245,59164,52858,38099,33841,48599,29583,25357,23309,23276,21196,21196,27502,42293,44405,44373,50744,40180,27502,31728,48631,46485,46518,42260,25389,25387,21256,17031,21255,25481,23337,27533,31790,19175,17061,23368,23336,19079,14886,12774,14854,14918,14950,12805,14918,14917,19111,19079,35984,42259,59163,59196,21066,10469,10468,10468,12613,12614,12614,10533,10533,10565,10533,10565,16998,19110,17030,12740,10693,12773,12772,14917,14917,14949,14949,12773,10693,19081,14823,12773,12837,12837,10661,10629,10630,10630,12710,14823,16904,16937,19017,19079,14949,12835,12836,17029,21223,23367,15012,17028,12803,14884,14916,16996,14884,10691,14948,19173,12836,14948,8611,10660,10660,12805,12836,12868,14885,14949,14917,8580,10660,8580,6403,46387,33710,18953,38067,56952,31532,25161,25161,25129,25129,25129,42097,151},
{54971,54971,54971,54971,54971,52891,50811,50779,50778,52891,54971,54971,52891,54971,52891,50778,50778,52859,52891,54971,52891,52891,36087,38232,44537,23475,23508,21395,21427,23475,25556,25555,25555,27667,25619,25587,23507,23539,25587,25587,25587,25587,23507,23507,25555,27699,27666,27634,27666,25585,23505,21425,23473,23473,23473,23505,23473,23473,23473,23473,23473,23472,37775,48047,44048,46258,46290,23245,16971,19019,19084,27438,39952,27405,18986,39952,33711,19019,16939,16970,16938,16938,19019,16971,16938,16938,16938,16938,16938,18986,16906,16906,16938,16938,16938,16938,16938,16938,16938,16939,13006,21132,27405,21099,14988,25520,38100,25422,21164,16970,16938,14857,10664,12744,16937,14889,14825,14857,14825,16970,21196,23244,21131,21228,21196,21196,21163,21163,21163,21195,23306,21287,17063,19143,19143,21192,19081,19050,21161,23304,21256,23304,19143,17031,12741,16966,14950,17062,12838,12837,14918,16967,17000,29646,42260,57083,59196,25259,10469,10468,10468,10533,12614,12614,6372,6372,6372,6372,6372,16965,19142,16997,14918,12805,10661,12805,14853,12740,14917,14917,12773,8580,17000,14855,12806,12836,12900,10692,4323,8516,8516,10693,14823,16936,14823,12711,21224,19141,17061,14884,21255,21254,17060,12867,17060,12804,14916,8547,14980,10723,10724,8611,19205,12803,17029,14948,10692,6467,12772,10692,10724,10724,8579,12804,12805,8547,10660,10661,10628,50646,23178,33874,54904,29420,25129,25161,25129,25129,25161,42129,144},
{54971,54971,54971,54971,52859,50779,50811,50811,50810,54939,54971,54971,54939,54971,50811,50778,50778,50778,50811,54971,52859,50811,42425,38232,38167,23508,21427,23508,23508,23475,25556,25588,27635,25619,25619,23507,23507,23507,25555,25555,25587,25587,25587,25587,25587,27666,29714,27634,25586,25586,23505,23505,21425,23473,25553,25553,25553,23505,23473,21392,21425,23472,37775,48015,44080,46258,46291,25325,19051,19051,19084,25358,35758,27405,18987,37904,35791,19019,18987,18986,16938,16938,19019,16938,16938,16938,16970,16970,16938,16938,16906,16906,16938,16938,16938,18986,16938,16938,16938,16939,12974,25358,33841,35954,23374,19148,23309,23309,25390,16970,23277,23277,16938,16970,19018,19018,19050,21131,19050,21131,23211,21163,19050,21098,21131,21131,21131,21131,21131,21131,21162,23337,21224,21192,23337,21161,23243,25323,23243,25387,21192,16967,16966,17031,14886,19079,17031,17030,14950,14918,14950,16968,6436,31727,40115,35954,46486,23146,10469,10468,10468,10533,12614,12614,10533,10533,10565,10565,12614,16966,16998,17029,19110,12837,10660,10660,14885,14853,14885,12837,10724,12741,12741,12773,10661,8612,12868,12836,6468,8548,10724,12837,10661,4259,4292,6372,19017,19110,14980,14949,19110,14917,19173,19172,12868,10691,10691,12804,8579,10691,10724,10692,14980,17061,12804,12836,12772,8548,10660,14885,12836,12804,14949,12804,14916,14949,6435,8547,10693,21192,29549,27535,54904,29419,25129,25129,25129,25129,25129,44210,141},
{52891,52891,50811,50779,50779,50779,50811,54971,54971,54971,54971,54971,54971,54939,50811,50779,50778,50779,50811,52859,50778,50778,44505,44537,34006,23508,23475,23508,23508,23508,25556,25588,25587,25619,25555,23507,25587,25587,25587,25587,25587,25587,25587,25587,23507,25586,29747,27634,25586,25586,25585,23505,23473,23473,23473,25553,27633,23473,23473,21425,21425,21424,37775,48015,44080,46290,46290,25357,21132,19051,21164,25358,35758,27405,19019,37871,37872,16938,16939,19019,16938,16938,19019,16938,16938,16938,16938,16970,16938,16938,16938,16938,16938,16938,16938,16938,16938,16938,16938,16939,12974,23309,25389,23244,23277,19083,21196,25390,27503,33809,46486,46486,38067,42293,42292,44340,42260,44373,44373,46454,46454,46486,46486,46486,46486,46486,46518,48567,46518,48567,48599,46485,48598,48598,46485,50711,50679,50679,48631,44371,25450,25386,16998,17031,17030,14917,14885,14950,12838,14885,12741,10662,8516,31693,50679,50679,54905,39855,18791,10468,10468,10533,12614,12614,12613,12614,12646,12646,14727,14790,16966,19142,16997,14917,12773,10661,17061,14981,12804,12773,10692,10628,17000,17030,12773,10661,8548,8548,8516,10756,10756,12869,10693,8485,8485,10565,14759,14822,16997,19141,19174,14917,21319,19172,14948,12835,14948,10692,8547,6499,10691,12771,14948,14949,14916,12804,8579,6467,6467,14885,14885,10724,14916,10724,14916,12836,12772,10724,12804,10692,35856,25389,54872,27306,25129,25129,25129,25129,25129,44242,135},
{50779,50811,48731,48698,50779,50811,52891,54971,54971,54939,54971,54971,54971,54971,52859,50778,50779,50778,50778,48698,48698,50779,48666,44473,38199,25556,23508,21395,25556,25556,25556,25555,25587,27667,25587,25619,23507,25587,25587,25587,25587,25587,25587,25587,23475,25555,29714,27666,27666,25585,25585,23505,23505,23473,23473,23505,25553,23473,23473,23473,21424,23472,35727,48047,44048,46258,46258,27406,21132,21164,23213,25325,35791,27405,16939,35759,35792,18987,16938,16971,16938,16938,19019,18987,18987,18986,18986,18986,16938,16938,16938,16906,18986,16938,16938,19019,19019,21164,23309,27503,29681,35987,44373,44406,44406,48632,54938,52825,50712,52793,52825,44309,37970,44373,50680,50712,48599,52825,52825,52825,52793,54938,44341,52792,50744,50744,50744,50744,50712,52792,50744,52792,50744,52824,50679,52760,50711,50679,50712,46517,40177,27530,23305,14886,17031,10725,12837,14918,10693,12774,10629,10662,10661,23208,59131,59131,57083,46097,39564,29161,12517,10533,12613,12646,8452,8453,8453,8452,8452,8452,10564,19142,17029,16997,10692,12837,10660,17029,14917,10629,12741,12741,14854,10692,14917,12806,8549,10629,8549,10756,12868,6436,6404,6404,8518,10598,14758,18984,18984,16998,25449,14917,14916,17060,14916,15012,12803,12804,8579,10724,12836,10755,14948,12804,10723,10724,8579,8547,8547,8547,10660,10692,10724,12772,10692,19142,17062,12772,10692,8515,33775,25389,52760,27274,25129,25129,25129,25129,25129,44210,129},
{50779,48731,48698,48698,48698,50811,52923,54971,54971,54971,54971,54971,54971,52891,50778,50811,52859,50811,50778,48698,48698,52859,48698,36119,33974,25588,23508,23476,25556,25556,25588,25587,27668,25587,25587,25587,23539,23539,25587,25587,25587,25587,23507,23475,21427,23475,29747,27634,27634,27666,25585,23537,23473,23473,23473,27634,25553,21424,23472,21425,21424,21424,35727,48015,44048,46258,46258,33679,33711,23244,25325,25325,37839,27373,16906,37872,39985,16938,16938,19019,16938,16938,19019,16938,16938,16938,16938,16938,16938,16938,18986,16938,19051,19083,19051,33809,38035,48632,52858,52858,52890,54938,54938,54906,44373,44405,54938,54938,48599,50712,50744,52857,50744,48599,52857,52792,50712,54970,54970,52857,54938,57018,50712,54906,52857,54938,54938,54938,54937,54938,54938,54938,54905,52825,52793,52793,50648,52792,42260,33841,33872,21257,12773,12806,10693,12838,10725,14918,12806,10629,8549,6436,12839,33871,52857,54970,38035,41839,39532,39532,37484,18790,18855,18855,18822,18823,16774,16774,18822,18823,18886,21190,17029,17030,12804,10692,12805,17029,17029,10660,10628,8548,12773,12805,12836,12837,10628,8484,6435,10724,12868,10692,6404,6372,8485,18952,27306,27371,18984,21127,27497,19110,17062,14949,17061,12867,14948,12804,12804,19206,14948,12868,12836,8611,10756,12804,10692,14949,10660,10692,14980,10724,17029,12772,6435,8579,10724,8547,10628,8515,29581,25356,52727,27242,25129,25129,25129,25129,25129,44210,134},
{50779,48730,48731,48698,48730,50811,52859,52891,54971,54971,54939,52891,50778,52859,52891,52859,50811,50811,50778,48698,50778,50778,46618,31926,36119,25588,23476,23508,25556,23508,25587,27667,25587,25619,25587,25587,23507,25587,23507,25555,25587,23539,23474,21426,23475,23474,27666,27634,27666,27666,25585,25585,23505,23473,23473,25553,23505,21425,23504,21425,21425,21424,33679,48015,44016,44145,44145,27372,27373,23212,25325,23212,37872,29485,18987,35791,40017,16906,16906,16906,16906,14858,16906,16906,16938,16906,16938,16906,14857,16906,16906,16906,16938,16906,16906,16906,16938,18986,16906,23212,25554,40148,44406,48599,44373,48567,54938,57019,46454,50680,50712,54970,54938,48599,54938,50744,50679,54970,54970,54970,54938,57083,48566,42227,50679,50711,50743,57050,50712,40147,48566,50678,52791,57050,52857,40114,46453,48565,44405,35987,29678,19175,17063,10693,8581,14953,14954,12841,14922,12842,12842,12842,19115,38066,42292,40147,44471,37646,39532,37451,33225,27112,29354,29321,31402,33515,33515,33547,33547,33547,33547,25352,21223,19142,14981,17062,12836,14981,12836,12805,8548,8580,14918,17031,14917,12804,10724,6436,6436,12836,15012,12804,8516,12646,16839,27338,35790,31597,29548,25320,27529,19110,23368,17093,12836,14948,17028,12772,14884,17061,14980,14916,10724,10692,10724,10724,12836,14948,12772,12740,19141,10660,10755,12804,12772,10692,8612,12804,12836,6435,27500,23242,52759,25162,25129,25129,25129,25129,25097,44242,120},
{50779,50811,50779,50779,50779,50779,50811,50811,50811,50779,52859,50778,50779,52859,52891,50811,50778,50778,50778,50778,50778,48698,42425,36119,40280,23508,21395,23508,23475,23475,25587,25619,25587,25555,25587,25587,25587,25587,25587,25587,25587,23507,23507,23507,25587,25587,27666,27634,29714,27634,23505,25585,23505,21425,21425,25553,23505,21425,21392,21392,21392,21424,33679,48015,44015,44145,44145,33646,29486,23213,19052,21132,37839,29518,19019,35759,40017,16906,16938,16971,16938,16906,16938,16906,16906,14858,16906,14825,14825,14857,14825,14825,16905,16905,14825,16874,16906,16906,16906,16906,12974,25423,35987,40180,46519,46486,50680,50712,46486,50712,50712,46453,38035,44341,52857,52792,50679,57018,54970,54970,54970,57083,48599,44373,46485,50679,50679,52857,54905,46453,46453,48630,48630,54937,54937,46518,42260,44405,38100,33872,23339,14951,14950,10694,8581,12808,14955,12842,14923,14923,14922,14890,19149,35986,42292,38099,42423,29324,35338,33258,16678,12549,12581,12581,12581,12581,12581,12581,12581,12581,12581,16870,16934,19109,17061,14980,14949,19141,10724,10628,10660,12836,16998,14886,12869,17061,10692,8580,10692,12836,14980,15012,14853,21096,21000,33645,37838,39983,33741,25352,33804,27529,19141,14915,17061,17028,14948,12836,19141,19205,14916,17028,14948,14949,12772,17094,14853,16965,14884,12772,12836,12900,12836,10723,14916,12804,10692,12869,10756,8579,27499,25290,52726,25161,25129,25097,25097,25129,23048,46355,110},
{50811,52891,50811,50779,50779,48731,50779,48730,48730,48698,48698,48698,50779,50811,50811,50778,48731,48698,50746,48698,48698,48698,38264,34039,34006,23508,23476,23475,23508,23475,27700,25619,25587,25555,25587,25555,23507,25587,25587,25587,23539,23506,23475,25555,25587,25555,27666,31795,27666,27666,25585,25553,25554,21425,19312,25553,23505,21425,21392,21392,23505,23504,31664,48015,44015,44177,44177,33679,23212,21132,19052,21132,37839,31566,19051,35727,42065,19019,19019,19052,19019,19019,19051,19019,16939,16971,19019,16938,16938,16938,16906,16906,16938,16938,16938,16906,16906,16906,16906,16906,10894,23310,29615,21131,27535,38035,44373,46486,44405,48599,52792,44374,42260,48599,50712,50680,48631,50744,52793,52825,50712,54905,44340,50712,50712,50712,50712,50712,50712,50712,50712,50712,50744,52825,48534,50679,46486,40181,35986,21258,14950,12774,12773,8581,12773,17033,14955,12842,14955,14923,14923,17036,17036,38099,40114,35986,44536,27308,35306,37451,27016,33547,37773,35628,37676,37741,37740,37740,37741,37741,37741,37740,33643,31626,25351,17061,12836,19173,14917,10628,8580,10692,12773,14854,10692,12804,10692,8548,10597,10661,17092,15012,16965,27401,33645,35725,37902,33709,25321,23240,38032,31691,23400,17060,17029,10691,10691,10691,12868,17060,14948,14948,14916,12835,12804,12772,10660,17029,12804,17062,14884,12835,14916,17061,12868,12804,12836,12836,12804,17061,21225,31629,52694,25161,25129,25129,25096,25129,25129,48468,97},
{52891,54971,52891,52859,50811,48730,48730,48698,48730,48698,46650,48698,48698,48730,50778,50778,50778,50778,50779,48698,48698,44538,36152,34071,23508,23476,23508,25588,23475,21427,25619,25619,25587,25555,25587,25587,25587,25587,25587,23507,25587,23475,23507,23475,25587,25555,27666,29747,31859,27698,25585,25553,25553,21425,21425,23473,21456,21457,21392,21425,23504,23504,31664,48015,46063,46258,46226,31566,23212,21132,25325,21099,42098,33679,19051,33679,42065,18986,19019,19051,19019,19051,19052,19051,19051,19019,19051,19019,19019,19019,19019,18987,19018,19018,18986,19018,19018,16938,16938,16938,12876,23310,29616,21131,23374,40212,55003,57083,42260,35921,52825,44374,37970,42228,48566,50680,44373,50712,50712,50712,50680,52792,46453,48631,48631,48599,50679,48599,48599,48631,48599,48631,48599,50712,48566,48631,42326,40180,31824,19176,14950,14950,10726,10694,10693,19082,17003,14890,14955,14923,14923,19116,14955,29583,52824,48632,46584,39824,41612,41612,29032,27209,29321,27209,27208,27241,27241,27241,27241,27241,27241,27241,25160,21158,19142,12836,14949,17061,12836,10660,14917,17093,10724,10628,14855,19080,12804,10756,16966,27468,29643,21223,37935,40048,37936,40049,40049,42161,40080,27466,40146,35920,27562,19173,17061,17093,17060,19141,12836,14884,16996,19141,12804,12804,12772,10692,8579,17061,12804,12772,10660,14980,14948,14980,14948,10660,12836,14917,12804,12868,14917,37968,50613,25129,25129,25129,25097,25129,25161,46355,105},
{54971,54971,54939,54971,52891,50811,50779,48698,48698,48698,46650,48698,48731,48698,48730,50779,48730,48698,48698,46650,46618,36152,36152,27733,21396,25620,19380,23508,25556,23507,27699,27667,23507,23475,23539,25587,23507,25587,23539,23507,25619,25587,23506,23507,25587,25555,29747,27666,27698,27666,25553,23505,23473,23473,23473,23472,23504,23472,23505,21424,23537,23504,29615,48015,46031,46258,46258,29485,25324,29485,29518,21132,42097,33711,19019,33679,42097,18986,19019,19019,16939,16971,19019,19019,19019,19019,19019,19019,16939,19019,18987,19019,19019,19019,19019,19051,19051,19019,16971,19051,19051,27503,29616,21131,14988,17101,40212,59196,42292,31728,42261,48599,44341,46486,50680,48632,44374,48599,50712,48631,46486,52825,50679,46486,50712,50679,44405,44372,44372,44405,44405,48566,42292,44404,46518,44438,44406,27565,23370,21288,14917,14918,14918,12806,12774,19116,17003,14922,14955,14955,14955,19149,14955,29551,46486,44373,50712,48211,41613,43660,37419,22951,25096,25128,23015,23048,25096,23048,25128,25128,25128,25128,25128,21190,19141,17093,12804,10724,14981,12868,12836,14948,14948,19047,38000,27467,19047,17062,21256,33807,29613,27532,31759,35920,38063,44339,44275,46420,44275,33709,40113,38033,31789,19206,17061,19141,17092,14916,12771,19174,14884,10692,14948,14916,8579,8547,12836,17060,14948,12804,8579,12836,12868,17061,14916,12804,14916,12805,8547,17029,16998,40113,48501,25129,25129,25129,25129,25097,27274,39952,98},
{54971,52891,52891,52891,52891,50811,50811,48698,48698,48698,48698,48731,48698,48698,48698,48698,48698,48698,46650,46618,44538,38264,42458,31926,21428,21428,19347,25588,23508,23475,25619,25587,25587,25587,23507,23507,23507,23507,21427,23539,25619,25619,25587,25587,25555,25587,27667,27634,25618,25585,25553,25553,25585,25553,23505,23504,23504,21424,23473,21424,23505,23504,29616,48015,46031,46290,46290,33711,29485,29518,29518,21132,42130,33711,19019,31631,42098,18986,16938,16971,16938,18987,18987,16938,16971,16938,19019,19019,19018,16938,18987,16938,16970,16970,16938,19018,19018,19018,19019,19019,16971,25422,29616,21131,14890,12842,38066,59164,40180,25389,31695,52792,52793,38002,50679,52825,40148,46454,57019,48599,40180,57051,50712,40179,46452,40113,29580,25418,25418,25386,27435,31693,21192,16999,19112,42326,42358,29710,17031,19144,16998,19143,19143,14950,12742,40213,17003,14922,14955,14955,14956,17068,17036,38099,38034,40147,54938,48210,43661,43660,41612,37515,41934,39853,27274,23080,23113,21032,23113,21000,21032,21032,21065,16870,19109,17029,16997,14917,14916,14980,14917,10724,12804,33869,50645,27466,21162,19146,17030,12775,10630,10663,10630,8517,17093,31693,35888,50712,33743,35855,42194,42260,40178,23335,19174,17028,17060,16996,12836,10723,10692,10660,19205,14916,10692,12804,17061,17061,14948,12836,17029,14916,14980,14948,12772,12805,17061,12805,12773,16998,16933,40113,46323,25129,23081,25129,25097,27242,33613,35758,100},
{52891,48698,46650,48698,50811,50779,50779,48698,48698,48698,48698,48698,48698,46650,46618,46618,46618,46618,46618,46618,42425,36152,40345,25588,19347,25653,21427,23507,25620,25587,27667,25587,25587,25555,25555,23507,23539,23507,23539,25587,25587,25587,25587,25587,25587,25587,27667,27666,25586,25618,25585,25553,25585,25553,25553,23505,23505,21424,23505,21424,23505,23504,29616,45967,46031,46290,46290,37872,31630,25325,23244,21131,40017,35759,19019,31598,42097,18986,16938,16971,16970,19018,16970,16938,16938,16938,19019,16938,16938,16938,16938,16938,18986,18986,16938,16938,16938,16938,16938,16938,14956,25422,29616,21131,16970,17067,40212,59196,40180,31696,48567,57050,52825,57051,59196,59196,57051,59164,59197,57051,57083,59197,57084,57051,57083,50712,42226,35919,27499,25354,31726,33839,38064,27499,33905,46551,40245,23370,12838,14919,17031,16998,17031,12774,10694,38100,21196,17003,14955,14956,14988,14956,17036,38100,40147,40179,46421,43984,43661,41580,41612,37547,41934,41901,16806,6340,8485,6372,8485,6372,6372,6372,8485,6404,14885,16997,17029,19174,17061,19141,14916,12772,10692,29645,50678,38000,10697,14921,17062,12775,10631,10631,10631,8517,17029,27498,40081,35921,33776,38000,40146,44340,35984,23336,23368,17060,17060,14980,14948,17029,12836,10659,19173,10692,10692,12836,14948,12836,12868,14980,14948,10723,14980,14949,10660,10660,12804,12836,14949,16934,16997,40112,42162,25129,23048,25129,23049,56952,50548,50581,192},
{46618,46618,46618,46618,48699,50779,48698,48698,48698,46650,46650,46650,48698,46650,46618,46618,46618,46618,46618,44538,40377,36152,29846,23508,21428,21396,29781,23475,23540,25619,27667,25587,25587,23507,23507,23507,23507,23507,23539,25619,25587,25587,25555,25587,25587,23507,25587,27667,27634,27666,25586,23505,25585,25553,25585,23505,23505,23505,23505,21424,21424,23505,27536,48015,46031,46290,46323,35792,25293,21132,21132,21132,35824,33646,19019,29485,42065,19019,16938,19019,16938,18986,16938,16938,19018,16939,19019,16970,16938,16938,16906,16938,16938,16938,16906,16906,16938,16938,16938,16938,14989,21262,29648,21131,23309,40180,59131,59196,40180,29648,48599,57050,52825,57051,59196,59197,57051,59164,59197,57084,57084,59197,59164,59132,57051,52824,46485,42259,40146,29580,25419,33839,38065,44405,46551,48600,36018,21224,17064,17064,23339,21256,14918,12773,8581,23308,21261,17035,17036,17036,17036,17036,17036,40213,38067,40147,50679,48178,41612,43660,41613,37515,41902,39821,25128,31467,31434,31467,33547,33515,33547,33547,35627,35660,35691,35659,19014,19109,14916,19173,17061,16997,14885,23306,12743,8517,8649,21229,10630,12712,12711,10598,12711,8518,12837,25384,40082,35889,33776,35856,40082,40114,27531,23368,19141,12835,17028,14916,12868,12835,10723,12804,14917,8547,8547,12804,12868,8611,8611,12836,19173,14948,12804,8547,8580,10660,12805,8547,14916,14885,14852,38096,42097,25161,23081,23048,25129,48468,50646,50613,182},
{46651,46618,46618,46618,48699,48699,48698,48698,48698,46650,46618,46618,46618,46618,44570,44538,44570,46618,44570,44538,40345,34039,31926,21427,21428,21428,21395,23508,23507,25587,25587,21395,23475,21427,21427,21459,23507,23507,25587,25587,25587,25587,25587,25587,23539,23507,25587,29747,27666,29747,25586,25585,23505,25553,25585,23505,21425,21425,23505,23473,23473,23473,27536,45967,46031,46258,46290,33679,23212,21164,23245,23245,37904,33711,19019,29486,42065,21067,18986,19019,19019,19019,18987,16938,16938,16938,19018,16970,16938,18986,16938,16938,16938,16938,16906,16938,16938,18986,16938,18986,16971,19149,31728,21164,16970,14890,38035,59196,40180,25422,33841,52824,52793,42260,50680,52825,42293,48664,57084,46551,40213,59132,52858,44438,52857,50711,38064,31693,31725,25418,25419,25386,21161,35985,52825,46520,42325,31824,21258,12839,14919,17062,17062,12773,10662,14922,17036,17036,17036,17036,17036,17036,17036,42261,38066,38066,46453,46097,43661,43660,41580,39596,41902,41901,20935,41934,41901,39853,41934,41934,39854,41902,41901,41934,41934,39853,35691,23271,14884,19108,17028,17028,8580,23339,10630,12775,14891,33809,12679,12712,12711,10630,12711,10630,12805,21222,40082,37969,33808,35888,35920,33870,33839,23337,17061,8611,14948,17093,14948,17093,12868,10691,17062,10692,10660,12772,12804,12804,10724,14917,12868,10692,10660,21128,10628,10660,12741,8548,12804,12837,10596,19048,33710,21032,16807,23113,25161,29452,37839,31565,128},
{46618,46618,46618,46650,48698,48698,46618,46618,46618,46618,46618,46618,46618,44570,44538,44538,44538,44538,44538,44538,42425,38232,23507,21427,21395,23508,21428,23507,25587,25555,25587,23507,25587,21395,21395,23539,25619,27699,27667,25587,25587,23507,23507,25587,23506,21458,23506,29747,27634,27666,27634,25585,25585,25585,25585,21424,23473,23473,23473,23505,21425,21393,25488,48015,45999,46258,46290,37905,23245,21164,21164,19019,37872,37872,16938,29486,42098,21099,16938,19019,16939,16939,18987,18986,18986,16938,18986,16938,16938,16938,16938,16938,16938,16938,16906,16938,16938,16938,16938,16938,16938,21132,29616,21164,14890,12809,38099,59196,40212,23308,21164,48599,50712,27470,44373,50712,31761,38067,54970,40147,29615,57051,46551,33905,44404,35951,27498,31725,35919,31693,27467,27499,25387,52825,52858,48600,44406,25484,21258,21225,19143,17030,17063,12806,17000,38100,23342,17036,17036,17068,17036,17036,19148,31631,59163,59163,52792,46098,43661,43661,41580,39596,41902,41902,20967,41934,41934,39853,41934,41934,41902,39853,41901,41934,41933,39853,35692,27400,14852,17029,17028,14917,6467,25452,8581,8548,17002,17003,12711,12712,12711,10598,12711,10598,12773,21223,33775,35889,33776,35920,35952,23271,21224,19173,14916,10660,12836,14916,14980,17093,17061,17093,14980,12836,10692,10660,10660,12804,8579,14916,12804,8547,14821,16870,12772,10692,10629,6371,8548,10692,8516,8484,10564,10629,6403,19049,31564,38034,48500,23113,90},
{44538,44538,46618,46650,46618,46618,46618,44538,44538,44570,46618,46618,44570,44538,44538,42490,42490,44538,44538,42458,38232,31958,25588,21395,21428,29781,23475,23475,27699,23507,25555,25555,23507,21395,21427,25587,25651,25619,25587,25587,25587,23507,21395,23539,23506,23506,23506,27667,27666,27698,27698,27666,23505,25585,25553,23505,21392,23505,23473,23473,21425,21393,25488,45967,45999,46290,46323,37937,21100,19084,21164,16971,39985,40017,16938,27405,44146,21131,18986,19019,16939,16939,16939,18986,18987,16938,16938,19019,16938,16938,16938,16938,16938,16938,16906,16938,16938,16938,16938,16938,16938,19019,27503,21164,14857,14858,35954,59228,42260,29648,48631,57018,52825,61244,59196,59132,59197,59197,59196,59197,59196,59164,59196,57083,42258,38031,42258,40178,42291,38097,40145,42291,38065,59164,52858,48600,44407,35986,33939,25452,17031,19176,19176,14950,19145,50745,25487,17068,17004,17036,17036,17068,27600,50711,52857,48599,44373,46033,43661,43628,41580,39628,41934,39821,20935,35660,35627,35628,35595,35628,33547,33515,33482,31434,31434,25128,16838,23238,19077,14916,17028,10660,6436,6436,4291,6436,10696,14988,12711,12711,12712,10598,12711,10630,8549,21190,33742,40082,35921,33808,42259,33838,19110,17061,14948,12804,10660,12804,12836,12772,8579,17061,10659,10660,10756,12836,14916,12804,10692,14916,10692,10661,8548,10628,8579,10628,8516,6435,8548,6436,8548,6436,6404,8516,8516,10629,37935,33808,46387,23081,88},
{44538,44538,44538,44570,44570,44538,44570,44538,44538,44570,44570,46618,44570,44538,44538,42490,42458,42490,44538,42458,40345,27733,21428,21427,21396,25588,21427,25587,25619,25587,25587,25555,23507,23507,25587,25619,27699,25619,25587,25587,23539,23538,23507,23506,23539,23507,23506,27699,29747,27666,27666,27698,25585,23537,23505,23505,21425,23505,23505,23473,21392,21393,23408,45967,45966,46258,46290,37872,23212,21132,19052,16972,33711,37904,16905,27372,42097,21131,16938,16938,16938,16938,16906,16938,16938,16938,16938,18987,16938,16938,16938,16906,18986,16938,16938,16938,16938,16938,16938,16938,18986,16938,25357,21131,23309,46519,59132,59229,42292,31696,44373,54938,52825,54873,59196,59197,54906,57083,59197,54971,57019,59197,57051,54938,54970,42290,35984,35919,35952,27532,29645,27499,46486,61277,54938,48600,44407,40246,33940,29680,29679,19145,19175,16998,17031,35955,23310,19116,17004,17068,17068,17068,31761,59163,48599,35987,42260,43952,43661,41580,41580,39628,41934,39853,20967,14693,16806,16774,16807,14726,16807,16839,16774,16774,21033,12646,21062,23237,21189,16997,19109,10692,10660,8548,8580,6468,14857,21262,16937,12743,12744,10598,12711,10630,8517,19142,29547,40082,35889,29613,23337,14885,16966,12836,10724,12804,8515,12804,10691,12836,12772,12869,10724,17061,8611,12836,12868,10724,12804,12804,8547,8580,10660,8547,10692,12773,8548,6467,8548,6436,8516,6436,6403,6436,6403,8516,37967,31597,46355,23049,89},
{44538,44538,44538,44538,44538,44538,44538,46618,44538,44538,44538,44538,44538,44538,44538,44538,44538,44538,42490,42458,36119,23508,21428,21427,23508,23475,21427,25587,25619,23507,25555,23507,23507,23507,25619,25586,25586,25587,25587,23507,23506,25587,23539,23539,23539,23506,25586,27666,29746,27666,25617,25585,25585,23537,23537,23505,23505,23505,23505,21425,21392,19312,21328,45967,45966,46258,46323,37937,23179,21164,19052,19052,33744,37936,16906,27373,42097,23179,16906,16938,16906,16906,16906,16906,14826,14826,16906,16906,16906,16906,16906,16906,16906,16938,16906,16906,16938,16938,16906,16938,16938,16906,21132,21131,17002,21196,42260,59196,42292,27535,42260,54905,52825,50745,54938,54938,50745,52857,59164,52858,48632,59196,54971,48567,57050,48630,29644,35952,40178,31725,29612,23306,54938,61277,54938,46520,44407,40214,36021,31860,27633,23371,19144,12805,16967,50810,29648,19116,17036,17068,17068,17068,23375,50647,46551,31793,33809,37549,43661,43661,41580,39660,41934,41901,12613,6340,8453,6372,8485,6340,6340,8453,6340,6340,10533,12645,14757,23270,21189,19109,17029,14884,8580,10660,12805,8548,25454,21229,16937,12711,12744,10630,12711,10598,8517,19110,23304,42194,33776,33807,25449,19141,14917,17062,14949,17061,10659,14949,14916,17029,14949,12836,12804,10692,10660,12836,14948,14916,10724,10692,10660,10724,12772,12773,8515,10660,10660,6435,6403,6403,6403,6403,6371,6403,8516,6435,31661,31596,44242,23049,88},
{44570,44538,44538,44538,44538,44538,44570,44570,44538,44538,44538,44538,44538,44538,44570,44570,44538,44538,44538,40345,29846,23475,25588,25620,21427,21427,23475,27668,25587,23475,25587,23539,23507,25587,25587,27667,27667,27699,25587,25555,25587,23506,23539,25587,25587,23539,25587,25586,29779,27666,27666,27666,25618,25585,25585,23505,23473,23505,23505,21425,21424,19344,21360,45967,45966,44177,46258,40017,21132,19052,19052,19052,31630,37904,18986,27405,44210,25292,19018,16971,16938,16938,16906,16938,16938,16906,16906,16906,14858,16906,16874,16906,16906,16906,14857,16905,16906,16906,16905,16905,16906,16905,19018,21131,14857,12841,31760,59229,42292,21196,10663,44405,50712,21131,38035,50680,25422,33873,54938,35922,23276,57051,44405,23276,52824,42290,29644,35919,35951,25385,19080,29645,54970,61309,54938,46552,44407,40214,36021,31859,23405,17031,14950,16998,12806,27600,19149,19116,17036,17068,19116,19116,17068,33712,48664,38132,40116,37549,41613,43661,41580,37547,39789,39756,23048,35628,35628,35660,35660,35660,35660,37741,35692,27241,20967,16838,14758,21126,14884,25414,14884,12804,10692,14917,10692,8580,19050,27469,19018,12711,12743,10631,12711,10631,10630,16966,21190,40081,33808,33840,29675,21287,17029,14917,17029,19173,21286,14948,17061,14948,12836,17061,10724,8612,10692,14916,12804,12803,10755,14916,10723,14916,12836,10692,6435,8580,6435,8515,10628,6403,6468,6403,4323,6403,6435,6435,27500,35822,42097,23049,86},
{44538,44538,44570,44570,44538,44538,44538,44538,44538,44538,44538,42490,44538,44538,46618,46618,44538,44538,42490,38232,25588,25588,38167,21427,21427,19347,23507,27635,25587,25555,25587,25619,25555,27635,27667,27635,27667,27699,25619,25587,23539,23539,23539,23538,25587,25587,27667,27667,29747,29779,27666,25586,25553,25585,25585,23473,23473,23472,23472,23472,23440,21360,23440,43919,45966,44145,46226,42130,18986,21132,21100,23180,31598,37839,19019,27373,44177,25325,16971,19019,19019,19051,19019,19051,19019,16939,19019,19018,16938,16938,16906,16906,16906,16906,16906,16906,16905,14825,14825,14857,14825,14825,16873,21099,12809,12809,29680,61277,40212,29647,46486,54938,52793,57051,59164,54970,57083,57083,57084,57084,57083,59132,59164,57084,59132,57051,46518,50679,48598,38064,35951,42259,54938,61309,54906,46520,44407,40214,36021,31826,23371,19176,14950,14918,12773,31760,38068,35987,31761,33907,35955,35987,35987,44244,52825,44471,42358,39662,43661,43661,41580,39660,41902,41901,20967,41934,39853,41934,41901,41934,41902,39853,39853,41934,39853,35660,14726,18981,19077,19077,16965,8516,10660,10692,12805,10660,23276,21163,21098,12711,12711,10631,10631,10631,12743,16934,19142,35855,35889,33808,35952,23335,17029,17029,12804,12804,17060,19174,14980,17093,14916,12772,10660,8580,10660,12868,19173,17028,14949,12836,14916,14980,10723,10692,8580,12772,10692,10692,12805,10661,12773,10660,12773,8580,8548,6435,23275,40016,39984,23049,87},
{44570,44570,44570,44570,44570,44538,44538,44538,44538,44538,42490,42490,42490,44538,44538,44538,44538,44538,42458,38232,25588,23476,23507,25588,23507,21427,27668,25587,25555,25587,25587,25587,25587,25587,27635,27700,27699,27699,27667,25587,25587,23507,23506,23506,25587,25619,27667,27667,27698,25618,25585,25585,25585,25585,25553,23473,23473,27601,25455,25455,25455,25455,27471,43886,45934,44145,46258,44178,39984,42065,42065,42065,42130,44178,19019,25293,42065,27373,19019,19019,19019,19019,19019,19019,19051,19019,19051,19051,19019,18986,16938,16938,16938,19018,16938,16938,16938,16906,16906,16906,16906,16906,16906,19018,19083,33874,48664,59196,42325,42293,40147,50744,46453,54970,59197,59197,59197,59197,59197,59197,59197,59196,59197,59197,59196,59196,59164,59164,48631,35918,31724,44372,52890,61277,54906,48600,42359,42327,35988,31827,25485,19175,14918,16998,12805,36017,48664,54937,50744,54938,54938,54970,54938,54776,52890,44503,44439,37516,43661,43660,41548,39660,41902,41902,20968,39854,39853,41901,41901,41902,41902,39853,39853,41934,39853,39853,31434,21126,16933,21157,19109,17029,16997,10660,10660,6436,23276,19050,25389,10631,12744,10663,10630,10630,12711,12709,21222,33742,40114,38002,31692,23367,19141,19173,19173,12804,14949,17093,12836,12868,12804,10692,12773,6467,8548,10724,14948,17029,12836,12803,17061,14916,12804,14916,12804,12804,10660,12773,12740,12741,12773,8547,10692,6435,8612,8580,19048,44274,37871,23048,89},
{44570,44570,46618,44570,44570,44570,44538,44538,42490,42490,42490,44538,42490,42490,44538,44538,44538,42490,42490,31926,25588,23508,23508,23507,23475,25587,27668,25587,25587,25587,25587,27667,27667,25587,27668,27699,27667,27699,27699,27699,25619,25555,23506,23506,25587,25619,27667,27667,29747,27666,25586,25585,25585,25553,23505,23473,25618,27568,23310,23212,20840,21002,27438,39693,41741,44145,44177,44177,37872,35791,35791,37904,39984,40017,18987,25260,39952,27373,19019,19019,19019,19019,19019,19019,19051,19019,19051,19051,19051,19051,19019,19019,19051,19051,19019,16939,19019,19019,18986,19018,19018,18986,16938,16938,16906,21132,29583,57051,44405,46519,38035,50745,42228,40115,40115,46486,44406,44373,40180,40147,38067,38099,38067,40148,38035,38067,38034,35921,29647,29643,21192,46518,54938,61244,57019,48632,44407,42294,29712,27567,23339,21257,16999,16999,14886,27565,48664,54937,50712,54970,54938,54938,54937,54775,61276,61276,59164,41774,43661,41580,41548,39692,41934,41902,18854,31402,31402,31402,31402,31402,31434,31402,31402,31402,31402,31434,29321,21062,16901,19077,19109,17029,12804,17061,12836,8580,33776,27437,27535,10631,12744,12711,10630,10630,12743,10597,19109,29548,40114,40114,31726,23304,29644,19141,21286,17060,19141,19205,17061,17028,12804,12772,12804,14884,10660,14853,12772,12836,14948,17061,14916,14884,12772,12836,14948,12804,14884,14820,16965,16998,12804,10724,10692,8580,6467,8548,10661,50614,33677,23081,90},
{46618,46618,46618,46618,44570,46618,46618,44570,44538,44538,42490,44538,42490,42490,44538,44538,42490,42458,38264,23540,21428,23508,23507,21427,23475,27700,27668,25555,25587,25587,27667,27700,27699,27667,27699,27699,27667,27699,27699,27699,25619,25619,27699,25619,25619,25618,25619,27667,29747,29747,25586,25585,25585,25553,23505,21425,23505,25455,25422,25358,20840,23180,25390,35533,39661,39887,42032,42097,21066,21067,21099,25260,31631,37872,19019,23212,42033,27373,16938,18986,19018,16939,16938,19019,19019,16938,19019,19019,19019,19018,16938,18986,19019,19019,16938,16938,19018,18986,18986,19019,18986,16938,19019,18986,16938,19018,21099,52825,42293,23277,16905,16873,16938,16905,23212,27438,25357,23277,16970,14792,14792,14824,14824,16938,14792,14857,14824,14825,14857,21192,21192,48664,52858,59164,57051,50713,44407,42294,31792,25453,19145,19112,19112,12806,19144,27565,48632,54905,50744,54938,54938,54938,54938,52695,57050,54905,54905,41774,43661,43660,41580,39660,41934,41902,16806,14726,16807,14726,14758,12646,12645,14726,12646,12646,14726,12646,10533,12645,14821,16965,19141,14916,12805,12772,10692,6435,38066,50744,36052,14857,12744,12744,10599,12743,10598,12678,16965,27433,42195,40146,29580,44372,35953,25449,14884,14916,14980,12867,17028,14916,12804,14949,10692,17029,10692,10628,14885,10692,12836,10723,14916,10659,17061,14916,14916,17060,16997,16966,19109,10627,10724,10756,14949,12836,8580,6435,8548,52759,31565,23049,86},
{48699,48698,46650,46650,48731,50811,50779,48698,46650,46618,44538,44538,44538,44538,42458,44538,42458,42458,38232,23508,23508,23476,21428,21395,25587,27668,25587,25587,25587,27635,27667,27700,29780,27699,27700,27699,27699,27699,27699,27699,27667,25619,27667,25619,25619,25619,25619,27667,27667,27634,25585,25585,27633,25585,23505,21393,23505,25456,25390,25390,20937,21002,27406,31274,39628,39887,42032,44177,25227,23147,21131,23180,31599,37871,21067,23179,42065,27405,16938,16938,16938,16938,16938,18986,16938,16938,16970,16970,16938,19018,16938,19018,18986,16938,16938,16906,16938,16938,16938,16938,18986,19019,19019,19019,19019,19019,23244,33809,25357,19051,19018,19019,21099,19051,21164,21197,21164,19084,14890,14825,14792,12744,12744,16938,12744,12744,12744,12745,14825,19049,19049,48632,52858,61244,57051,48632,44407,42294,33873,25452,21225,21256,21191,16967,14886,27565,48632,54937,50744,54938,54938,54938,54938,48501,52791,52824,54905,43854,43661,43660,41548,39692,41902,39853,8420,6339,8420,8420,8453,8420,8452,10533,8420,8452,8453,10533,8484,16901,14820,14852,16965,14916,14917,19141,8515,12773,38098,52825,33906,16970,12744,12743,10631,14857,10630,12678,14821,25319,38033,38001,29612,42259,35920,23399,14916,17061,17060,17028,19141,14916,10659,12836,10692,12772,10628,12836,14916,12836,14885,14949,12804,10692,17028,17028,17060,21253,17029,17061,14948,12836,10692,12836,14916,12804,8547,8580,8580,46485,27339,16775,86},
{48731,48699,48731,50779,52859,52891,52891,52891,50811,48699,46618,44538,44538,44538,44538,42458,42458,42458,36184,23508,23508,21427,21395,23507,27636,27636,25587,25587,25555,27668,27667,27699,27732,27731,27699,27667,27699,27699,27699,27667,27667,27699,25619,27667,25587,27667,27667,27667,27667,27634,25585,25585,25585,25553,23473,21425,23505,27536,23342,12777,10632,14825,25358,33485,35435,41967,42064,44210,27372,23147,19019,19019,31598,37872,19019,21099,42033,27405,19019,12778,12745,12745,12745,12745,10664,10664,10664,10664,10664,10664,10664,10664,10664,10664,10664,10664,10664,10664,10664,10632,10664,10632,10631,10632,10664,10631,10631,10632,10631,10631,16970,14857,12745,12745,12745,12745,10632,8519,10600,10632,10632,10697,10632,8518,8519,8551,8518,8551,14890,19116,17003,46519,52826,61245,59131,50713,44439,40246,33939,31792,21223,21223,19175,14950,14919,27566,50712,54905,50744,54938,54938,54970,54938,48502,50711,52792,50744,41774,43661,43661,41548,39692,41934,41902,25161,39821,39853,41934,39853,39886,39886,41902,39853,41934,39821,39885,31529,29481,25319,19045,14884,12772,16996,16996,12772,8548,38098,50712,29713,25357,12744,12744,10663,12776,12743,16937,14855,25384,35856,35921,29613,46453,35920,25416,19173,17124,17092,17060,12868,10660,10660,8547,8580,10660,10660,10692,12836,12804,12804,10756,8579,10692,14916,12804,19141,14916,19109,14916,14916,10723,12772,12772,14916,19109,10660,10660,12805,40209,23275,14759,81},
{50779,50811,52891,54971,55004,57052,55003,52891,48698,48698,46618,46618,44570,44538,44538,42490,44538,42425,42458,25620,23508,21428,23475,27668,25587,25588,25555,27635,25587,27668,27668,27700,27699,27731,27731,27667,27699,27699,27699,27667,25586,25619,25619,27667,25587,27667,27667,27667,27667,27666,25585,25585,25553,25585,21393,23473,25585,29681,25390,21262,10632,14858,23342,19019,31307,39919,42065,44178,27340,25292,23212,23180,33711,39952,27405,29453,42065,33679,27373,10665,8551,8551,10663,8551,8551,8518,8551,8583,8551,8583,8551,8583,10696,8551,8551,8519,8551,8551,8583,8551,8551,8519,6438,6470,8551,6470,10631,8551,8551,12744,21131,14825,12745,16938,12778,10665,6438,6406,8519,8551,8584,10697,8551,6438,6438,8519,6438,8519,25520,29713,31761,48664,52858,59196,57051,50745,44407,40246,36053,23337,21256,21224,21255,12806,19081,29647,44373,52825,50744,54938,54906,54938,54938,48502,50711,50711,46486,41742,43661,41612,41580,39692,41902,41902,21000,39853,41934,41934,39853,41934,41902,41901,39853,41901,41934,39853,39853,39853,33642,19045,19077,16964,12804,14852,16997,12773,38065,44373,40245,29583,12744,12744,14857,12776,12744,12743,12742,21254,35888,33808,29613,44405,35921,38065,23399,19172,19173,12835,10755,14981,12836,10692,8547,8579,12772,12836,12868,8579,14916,10692,14980,12804,14917,14917,10660,10692,12804,10692,8579,14980,10691,10628,8579,17029,12805,8547,10660,12773,10661,10660,56},
{52891,52891,54971,55004,57084,57052,52891,50811,48698,48698,46618,46618,46618,44570,44538,44538,46618,42425,31926,25588,23507,21427,25588,27668,25588,25587,23475,25587,25587,27635,27668,27700,27700,27731,27699,27699,27700,27699,27699,27699,27667,27667,25619,27667,27667,25587,27667,25587,27666,29746,25553,23505,25553,25585,25553,23505,25521,27568,23342,21197,8519,12777,21196,17003,12712,16905,21131,14792,16905,21131,12744,14792,21164,14824,14792,19051,10599,16905,21132,10664,8552,8551,8551,8583,8584,8583,10632,10632,8583,10631,10664,10664,10664,10696,10664,10664,10664,12745,10664,10664,12745,10696,10696,10696,12745,12744,12744,12777,12777,14825,16938,14826,14858,14858,19084,16971,14858,12745,12777,14890,14891,16971,14890,12777,12777,14857,12777,12777,31955,46584,42326,50777,52826,61245,57051,50713,44439,40246,29679,25484,21256,19143,19143,17031,21225,33905,46519,52825,50744,54938,54938,54938,54905,48502,50679,50711,46454,41710,41613,41612,39500,39692,39821,39853,20968,39853,41934,41934,39853,41934,41901,41901,41902,41901,41934,39853,39853,39853,33610,25350,16932,16997,6435,14885,14853,12773,31695,23310,27568,23244,12744,12744,14824,12776,14857,12711,12710,21254,33839,23276,23274,46453,35921,42292,25481,19237,17060,14980,12836,21286,14948,10724,8579,12804,6467,12804,14916,10692,14949,17061,14916,12804,17028,17061,8579,8579,12869,6467,6435,10724,12836,8547,12772,14980,10660,8612,10660,10629,12741,10692,42},
{52891,54971,54971,54971,55004,52891,50811,50811,50779,48698,48698,46618,46618,44538,44538,44538,46618,34007,27668,23507,23507,23507,27668,27668,25555,25587,25587,25587,25555,27635,29716,27667,27699,27699,27667,27667,29748,27667,27667,27699,25619,27667,27667,25619,27635,27635,25619,25619,25586,29779,25553,23505,25585,27633,25553,23473,23472,25423,23342,8584,8551,14890,23310,17003,12744,14858,17003,12744,14857,16971,12745,12777,17003,12744,12745,16971,14825,19051,19051,21197,17036,16971,14923,14923,14923,14923,14891,23342,17003,16970,19116,17003,14890,14890,14890,14858,17003,19084,14922,14858,14858,14890,14858,17003,19083,12809,12777,12777,12777,14825,12777,16971,16971,14858,17003,14890,12712,12744,16938,14857,12745,12810,12777,10632,12744,14857,10664,10664,31923,48665,44439,50745,52826,59196,59131,50745,44439,42326,36018,27565,19111,17031,19178,14951,12839,31759,50712,52825,50744,54938,54938,54938,54938,48566,52760,48631,50647,39564,41613,41580,39499,39725,41934,41934,20935,31434,31402,31434,31434,31434,31402,31402,31402,31402,31434,31402,31402,27304,23142,19013,16965,14884,12804,12804,10660,10693,21294,27602,17003,23341,12744,14889,12712,12776,16938,12711,12678,21222,33838,25421,25386,44372,35953,29613,14949,14948,12804,12803,12804,10692,14916,14981,8611,8579,12804,10692,14917,12805,19110,17061,12836,10724,19206,12836,10724,6467,6467,6467,10692,12836,17029,14916,17029,17061,10691,14949,8516,10693,8548,10693,33},
{54971,54971,52891,52923,52891,50811,50779,50811,50779,50779,50779,48698,46618,44570,44538,44538,40345,27636,27636,25588,25588,27668,27668,27668,27635,27635,25587,25555,25587,27668,27668,27668,27699,27700,27667,27667,29748,27699,27699,27667,27667,27667,27667,27667,27635,27635,25619,25618,27666,29747,25585,25585,25553,25553,27634,25553,25553,25423,25390,17004,8552,16971,23342,16971,12744,14857,16970,12712,14857,16939,10664,12777,16971,10664,12745,16938,14825,19019,16970,16971,10697,8584,8584,10664,10664,10632,8584,23309,12777,12745,17003,14857,10664,10664,10664,10632,12809,16970,12745,10632,10664,8584,10632,12777,16971,8583,8551,8583,8551,8518,8518,14889,14857,10632,16971,16939,8551,8551,14857,12777,10664,12777,10665,8551,10664,14825,10632,8551,31923,48665,46551,50745,52826,59164,57051,50745,46487,44374,33871,23369,17031,14951,12774,10694,10726,27566,50712,50712,52792,54938,54938,54938,54938,50679,52760,59196,61244,39564,41612,41580,39499,39724,41934,41934,10500,10565,10533,8452,10565,8452,8452,10532,8452,6372,10598,10532,12645,18981,21157,19077,10628,10628,10660,8547,10660,12838,17200,19085,27568,40180,12744,16938,12712,12744,17003,14791,16872,14821,29611,31694,25418,42260,35921,21224,14916,10692,8579,12868,10692,12772,12836,10691,8547,10660,10692,10660,8580,10692,12804,10691,17060,17093,14948,17061,14948,12804,6467,8579,10723,17029,10691,10724,12804,14916,14949,10724,12805,10660,8548,6467,38},
{54971,54971,52891,52859,50811,50779,48698,48730,52859,52891,50811,50811,50779,48698,44570,42458,31861,29748,27668,25556,23507,27636,27668,27668,27635,27668,25587,25555,25587,27667,27668,27700,27667,27668,27667,27667,27699,27699,27699,27699,27699,27667,27667,25619,25587,25619,25619,25618,27667,33940,27666,25553,23473,23473,25521,27666,25553,25423,25390,17004,8552,19051,23342,16970,12744,14858,14890,12744,14857,14891,12744,14825,16970,12712,12777,14858,14858,16971,16971,16971,10665,10632,10632,10664,10664,8583,10632,23309,10696,12745,17035,14890,10664,12745,12745,10664,14890,19083,14857,12745,12777,12745,12744,14890,17003,12777,12809,12809,12809,12777,12777,19051,19051,14890,14890,14858,14858,14857,17003,16970,12777,14891,14858,12745,14857,16970,14825,12777,25552,40278,33841,50713,50778,59132,59132,52793,52793,54906,48565,25450,19111,17032,12742,12806,14920,29647,50712,52825,52824,54938,54938,54938,54937,52792,48534,59163,59131,37516,41612,41580,39499,39692,41902,39854,12581,14726,12613,12613,14694,12646,12646,14726,12646,14758,14759,18951,21031,25319,25350,16932,14852,16965,10692,6435,10660,10759,17102,42294,23310,42325,12744,14825,12712,12744,17002,16937,16937,14823,23306,33839,27499,44340,33839,19142,10723,10692,10724,14948,10724,12868,15012,12836,10692,8579,8547,8547,10692,8547,6467,12804,17028,17092,14979,17060,12868,14948,10691,15013,14948,12900,10756,12837,10692,8547,12804,14948,14885,10660,8548,8580,39},
{54972,54971,52891,52891,50811,50779,50779,50779,52891,54971,54971,54939,52891,50811,46618,38199,29748,27668,25620,27668,25588,27635,27668,27635,27667,27668,27635,25555,23475,25587,27667,27668,27667,27667,27667,27667,27667,27667,27667,27699,27667,27667,27667,25619,25619,25587,25619,25618,27667,31827,27698,25585,23473,23473,23473,27634,27601,25423,23343,12745,6438,19084,23342,12744,6470,12745,12777,8518,12745,12745,6438,10664,12777,8518,12745,16972,16971,16971,16971,16971,12810,12777,12777,12777,12778,12777,12745,25390,17003,17003,21196,19083,16970,16971,16971,14922,19051,19116,16970,14890,14890,14890,14890,17003,19084,14890,14890,14858,14858,14857,12777,17003,16971,14857,12778,14858,12777,12777,16971,14890,12777,12810,12810,10664,12745,14857,12744,12745,17003,33907,27503,48665,50777,57051,59164,57051,57019,52793,44404,21255,19110,16998,14918,19112,17000,31791,50712,52825,52825,54938,54938,54938,54937,54873,48534,59196,61244,39532,41612,41547,39563,39756,39853,39821,41869,41901,39853,39853,41901,41901,39853,39853,39853,33548,35660,33580,31499,29450,23205,23270,16996,12772,17028,8548,14886,14920,25520,57083,46519,50712,14857,12776,12776,12744,17002,14856,14824,10565,21224,33871,27531,42292,33872,21223,14916,14948,14948,8612,10659,14916,12835,12836,8579,8579,10660,8579,12804,10692,8580,8547,17093,19140,19205,14979,14948,12803,17029,12804,12868,14981,8579,8612,10692,10692,14981,14981,12804,14950,10628,12805,48},
{55003,54971,54939,52891,52891,50811,50811,50811,50811,52859,52891,52891,52891,48698,46618,31893,29748,25588,25588,25588,25587,27635,27667,27668,27667,27667,25587,25587,23507,25587,25587,27668,27668,29716,27667,29748,27699,27699,27699,27699,27667,27667,27667,27667,25619,25587,25619,25619,25619,29779,29746,25585,23505,23473,23473,23505,29681,25423,25423,23310,23278,23310,23342,14858,12744,16938,14858,10664,14890,14858,10664,14857,14858,10631,14923,14924,16971,16971,16970,16971,10664,8584,8584,10664,10632,8583,10632,25422,10664,12777,17003,14857,10663,10697,10697,10664,14890,17003,12777,10664,10697,10664,8583,14857,16971,10664,10631,10664,10631,8551,8551,14858,14857,10599,8552,8584,12842,14890,16938,14857,8584,10697,10664,8518,8551,12777,10631,8519,8583,17003,25455,48632,50745,59164,61244,57019,46519,44439,35984,23368,21255,19175,12838,12838,17031,29678,50712,52825,52825,54905,54906,52857,52857,54905,46421,59163,59163,37483,41612,39467,39724,41902,39821,41901,41901,41901,39821,41901,41901,41902,41901,39821,37741,39821,37708,39853,27274,10500,16805,27399,19109,14884,19109,16965,19078,14856,27568,52858,35890,44373,14825,10631,12712,12712,16938,14824,14792,10630,14791,31726,29644,42227,35920,25449,19173,17093,17061,14948,17060,17060,14915,14948,12836,10724,10660,10660,8579,12741,8547,12740,14916,14981,12836,12836,12836,12772,19109,21189,17029,12804,8580,8579,12772,10660,12804,10691,12805,10660,12773,14949,52},
{55004,55004,55004,54971,52891,52891,52891,50811,50811,52859,52891,50811,50779,48698,44538,27668,27668,25588,25588,25588,25588,25587,25587,27668,29748,27635,25619,25587,23507,25587,25619,25619,27667,27668,27667,27667,27667,27667,27699,29747,27699,27667,27667,27667,25619,25587,25619,25619,25619,27698,27666,25585,23505,23473,23505,23505,25488,25423,25423,25423,25423,25423,23342,14857,12744,16971,14858,10696,14922,14858,12745,14890,14858,12745,16972,16972,16971,16971,16971,16971,10664,8584,8583,8584,8584,8551,10632,25422,10632,12744,17003,12777,10664,10664,10665,10664,14858,17003,12777,10664,10696,10664,10631,14857,17003,10664,10696,10696,12745,12745,12744,17003,16971,12777,12777,12778,14858,14858,17003,16938,12745,14858,12810,12744,12777,16938,14825,12744,12777,19084,25455,46551,50777,59132,57083,46487,44407,42358,31821,31789,27594,21255,17062,17064,19143,31758,50744,52857,52825,54906,54905,54938,52857,44372,42130,48599,46453,37451,41613,37386,33418,33515,31434,31402,33482,31434,31434,31434,31402,33482,31402,31434,29354,29354,18887,20967,12613,8484,14758,18982,16997,12708,16933,19014,21160,21130,35922,46454,33776,46421,23212,14824,14857,14857,19082,19017,16937,16904,19083,27501,31757,40115,38002,42259,21254,17093,14948,12867,19172,17092,17060,14948,10692,8580,6467,8548,12804,14917,10692,12772,12804,12804,10724,10724,6403,8580,14916,21286,19077,12804,8612,8579,14950,10691,17062,14917,12837,12773,8516,16870,55},
{57052,57052,57084,55004,54971,54971,52891,52891,52891,52891,52891,50811,50779,48730,40280,27668,27668,25588,25556,25588,27636,25555,23507,25555,27635,27635,27635,25587,25587,25587,25587,25619,25587,27635,25587,27667,27667,27667,27699,27699,29747,27667,27667,27667,25619,25619,25618,25618,25618,27666,25618,25585,23505,23505,23505,25553,23440,25423,25423,25423,25423,25423,25390,12777,12744,16971,12809,10696,16971,12777,10664,14890,14857,12778,14924,16972,16971,16971,16938,16971,12810,12777,12777,12777,12777,12777,12745,25422,17003,19116,21197,19116,17003,17003,17003,17003,19116,21196,17035,17003,17003,17003,16970,19083,19116,14890,14890,14890,14890,14858,14890,19051,19051,14857,12777,12810,14923,17003,19051,16971,12809,14890,14858,12744,12777,16970,12777,12744,12745,17003,27535,46519,50778,57051,46520,42326,40278,42326,27594,21256,17031,21287,17062,19207,21255,31791,50712,52825,52857,54970,54938,54970,52857,44404,40049,48599,44341,37419,41612,41580,22855,14629,16709,16709,16677,18757,16709,16741,18789,18790,14661,8452,10533,10533,10532,10565,8452,6404,4324,4291,4259,6372,8452,10564,29580,23243,40180,59164,50712,59164,19018,16938,14889,12744,19050,16937,16937,16905,14856,31727,38032,38034,35921,40178,21286,19205,12868,14948,14980,14980,14915,12836,8611,8612,10660,12772,17029,16997,12804,12837,8548,6467,12772,8515,6467,14949,12804,17061,16997,12805,10692,6467,8547,12805,14948,12804,14885,10660,10628,12709,34},
{57052,57084,57084,57052,55004,54971,54971,52891,52891,52891,52891,50811,50779,48731,33974,29748,27668,25588,25588,27668,27668,27668,25556,23508,25555,27668,27668,25587,25587,25587,25587,27667,25587,25555,25587,27668,27667,27667,27667,27699,27699,27699,27667,27667,27667,25618,25618,25618,25618,25618,27698,25585,23505,23505,25585,25585,25488,25455,25423,25455,25423,25423,25423,10664,10631,17003,12777,10632,16971,12745,10632,16938,12777,12810,16972,16972,16972,16971,16971,16939,10664,8583,8584,10664,10664,8584,10632,25422,10632,12745,17003,12777,10664,10697,10664,10632,14890,17003,12777,10664,10697,10664,10631,14857,17003,10664,10664,10697,10697,10664,10663,16970,16938,10631,10664,10697,10664,10632,16938,14857,10697,12778,12745,8551,10631,14857,12745,10664,10665,14923,25455,44439,50745,50713,42326,40246,40246,38165,29676,23337,23337,19111,21192,14918,14885,31791,50712,52792,54905,54970,54970,54970,48631,38065,37969,42260,40082,39499,41580,39500,35306,37676,41934,39821,31466,31467,29419,29386,29386,29354,23080,10565,6372,10533,10565,8485,8485,6404,6404,6404,8452,6404,6404,6371,19080,14823,33809,14825,19018,33874,16970,14857,14857,12744,19083,19050,19018,16970,14823,29646,42258,38034,35954,40178,21318,19269,17124,17092,19205,14980,12835,10692,10692,14917,12837,19142,14915,17028,12804,10660,8548,10660,10660,8580,10692,14916,10660,14948,14916,10660,6467,8580,8579,12805,8611,14917,8580,10693,8548,14917,34},
{57084,57084,57084,57084,57052,55004,54971,52891,52891,50811,50811,50779,50811,46618,29716,29748,27668,25556,25556,25556,25588,27668,25588,23475,23507,25587,27667,27667,25587,25587,27667,27667,27635,27667,25587,27635,27667,27667,27667,27699,27699,27699,27667,27667,25619,25618,25618,25619,25618,25618,27731,25585,23505,23473,23505,25585,25521,25455,25455,25455,25455,25423,25390,8551,8551,17003,10631,8551,16971,10632,8518,14890,10632,14924,16972,16972,16971,16971,16971,14923,10632,8584,8584,10632,10632,8583,10632,25422,10631,12777,17003,12777,10632,10665,10664,10664,14890,17003,12745,10664,10664,10664,10631,14857,17003,10663,10663,10664,10664,10663,10631,16970,16938,10631,10696,12745,12777,12777,16970,16970,16971,17035,14923,10664,14857,19051,19052,19084,17036,21229,25455,44406,50745,42326,40246,38166,38133,27631,23369,21257,19112,17032,21192,16999,10661,29646,50712,50712,52857,54938,54938,54938,50744,33839,44275,57083,59131,31209,37354,35273,33192,39724,41934,37741,4259,6340,6372,6372,8452,8452,8484,8485,8452,8485,10565,8452,6404,8485,4291,6404,4323,4324,6437,10694,8516,12775,29583,29518,16938,29648,16970,14857,14857,12712,19051,14824,16905,16970,23243,38033,42258,38002,35921,40177,21318,19238,19206,12867,14980,14948,14916,14917,10692,12772,12837,19173,17028,14981,12836,10660,10660,8547,12804,10692,12804,17029,10659,12836,10692,10660,8548,10724,14917,10691,17030,10724,8547,10692,8580,14981,53},
{59132,59132,59164,57084,57052,55003,54971,52859,50811,50811,50778,48730,48730,36054,27668,27668,29748,27668,25588,25588,25587,27668,27668,23475,25588,25587,27667,27668,25555,25587,25587,27667,27667,27635,27667,25587,27667,27667,27667,27699,27699,27667,27667,27667,25618,25618,25619,25618,25619,25618,27666,25585,23505,23473,23505,23505,25488,25423,25423,25423,25423,25423,23342,12744,12744,19116,12744,12744,19084,12745,10664,17003,12778,14859,12811,14891,16939,16939,16971,17003,12778,12777,12777,12777,12777,12777,12777,25422,16970,19083,21229,19084,17003,17003,17035,17003,19148,21229,19084,17003,17035,17003,17003,19116,21196,17003,17003,17003,17036,16971,16970,19116,19116,16970,16938,16971,17036,17003,21164,21164,19116,21197,19084,14890,16970,19083,19116,19116,19084,21229,25455,44374,48632,38165,38166,38133,36085,31824,19144,21289,21257,19176,16998,12806,8549,29615,50712,50712,52825,54938,54906,54938,52825,42292,46485,59196,59196,27048,35306,33193,33193,39756,41934,37709,18854,27208,27208,27176,27176,23047,14726,14726,20967,25160,14726,10565,8452,8485,10629,12708,6404,8548,6436,4355,6404,14823,35889,42260,23244,31761,12777,12776,14857,12744,19083,16905,16905,19050,16969,23308,42258,35921,35921,40177,21318,21384,19206,17125,17060,17093,17093,17093,10724,10724,12803,17027,17028,12836,12772,10660,10692,12804,10691,14917,10724,14917,17061,19142,8579,8548,6435,6435,8579,14916,14981,10660,12837,8612,12741,17061,48},
{59164,59164,57084,57052,55003,54971,52891,50779,50811,50779,50779,50779,48698,33942,27668,27668,27668,27668,27668,25588,25555,25588,27667,25588,25556,25587,27700,27667,25587,25587,25587,25587,25587,25587,27667,25587,27635,29747,27667,27667,27699,27699,27667,27699,27667,25619,25619,25619,25619,25619,27666,25585,23505,23473,23505,23505,25488,25423,25423,25423,25423,25423,23310,12744,12745,21164,10664,10664,19084,12712,10664,17003,12778,10633,10665,12778,16939,16971,16971,16971,8584,8584,8584,10632,8584,8584,10664,25422,10631,14857,17003,12777,10664,10697,10696,10664,14890,17003,12777,12777,10697,10696,10664,14890,17003,10664,10664,10697,12778,10631,10631,16970,16970,10631,10664,12745,14923,14890,19051,19083,17036,19149,17003,10664,12777,16970,17003,14923,12777,14923,25455,42326,44439,38134,36021,33973,36085,33972,33938,29710,25484,19144,12838,12806,10629,29647,50712,50744,54905,54938,54906,52857,50744,33871,42258,57083,57083,27081,33225,33193,33225,39757,41934,39821,39788,39788,39756,39788,39788,20934,16806,14726,18854,25160,10565,8452,10565,8484,12741,19077,19077,10628,6403,4323,4323,12776,38068,35954,16938,12874,14857,12776,14857,12744,19083,19082,19083,23276,19082,17033,40145,35921,35921,40177,21318,21351,14948,17060,12835,12803,17028,17060,14949,14948,14980,17093,17061,14948,12836,12804,10691,12868,10692,8579,8579,10660,12804,14916,6467,8548,8548,10692,10693,12837,17029,10724,10756,14949,12836,14916,56},
{59164,57084,54971,55004,54971,50811,50811,50811,50779,50778,50811,50811,50779,36087,27668,27668,27668,27668,27668,25588,25588,25588,25587,27636,23475,23507,27635,27668,25587,25587,25587,25587,25555,25587,27667,27635,27635,27667,27667,25587,27667,27699,27699,27699,27667,25619,25619,25619,25618,25618,27666,25586,23505,23505,23505,23505,23408,25390,25423,25391,25423,25390,21229,12744,12744,21197,10664,12744,19116,12712,12744,19084,12778,10633,12745,12778,16939,16939,16971,16971,8584,8583,8584,10632,8584,8583,10664,25422,10631,12809,17003,12777,10664,10697,10665,10664,14890,17003,12777,12745,10697,10664,10664,14890,17003,10664,10664,10697,12777,12712,10631,16938,16938,10631,10664,12777,14923,14890,19051,17003,17003,19084,14923,10664,12777,16970,14825,10632,10664,14858,23342,38100,42327,36053,33940,33972,33972,34004,25484,27564,23338,19111,17031,10661,8581,29647,50712,50744,54938,54938,54905,52857,48566,31757,44371,59196,59163,35468,33225,33225,33225,39757,41934,39821,41901,41901,39853,39853,39788,31402,20967,14661,23080,23113,8485,8452,14823,19048,23272,16967,21192,14855,12775,12775,12743,14890,27503,29616,19116,8551,12777,12712,14857,12776,16970,16937,16937,19083,19114,19114,42259,35889,33841,36015,19238,21319,17093,17124,17093,17029,21255,17060,17092,19141,12835,17060,17028,16964,14916,19141,12868,12836,17093,12772,12837,14885,14885,10660,14917,10660,12836,10660,8548,10660,12836,14916,8579,8611,14980,12868,66},
{57084,57052,55004,54972,52891,50811,50811,52891,50811,50811,52859,52859,50811,44537,29748,27668,27668,27668,27668,27636,25556,25588,27668,27668,25555,23507,25587,27635,25587,25555,23507,25587,25587,25587,27667,27667,27635,25619,25587,25587,27667,29747,27699,27699,27699,27699,27667,25618,25618,25586,25618,27666,23505,23505,25585,23537,25455,25423,25423,25455,25423,25423,21197,8486,6438,19084,8486,6438,19084,8519,8518,17003,12779,12745,10632,14826,16939,16971,16971,17004,12778,12777,12777,12778,12777,12777,14826,25422,17003,19116,21229,19116,17003,17003,17003,16970,21164,21229,19083,17035,17035,17003,16971,19116,21229,17003,17003,17003,17003,17003,16970,21164,21196,16938,16970,19116,21229,23310,23276,21196,21197,21229,19084,16970,19083,21164,19051,17003,19083,25488,12778,29648,38133,36053,31859,33973,36085,36053,31791,23402,25451,21193,16999,12807,12775,29647,50680,50712,52857,54905,54905,54937,50744,44372,37969,52825,52825,35435,33225,33225,33225,37676,41902,37708,37708,37740,37740,37740,37707,35627,27240,31466,20999,12645,8485,8452,12742,12773,12772,14852,16966,8516,4323,6436,4291,12808,35987,29616,21197,6470,19116,12711,14857,14857,16970,16937,16937,21130,19082,17001,40178,35921,35921,38096,19238,21352,19239,17061,14980,14916,14948,14948,17060,19141,17060,14948,14948,14916,14916,19173,14948,10692,12804,8579,14917,14916,12836,12805,8611,8580,10724,6435,6435,6467,10724,12836,10660,12772,10724,17062,49},
{55004,54971,54971,52891,52891,52891,52891,52891,52891,54971,52891,52891,52891,52891,33974,27668,27636,27636,27668,29716,27668,25556,25555,25587,25588,23475,25555,25587,25587,25587,25587,25587,27668,27667,27667,27667,27667,25619,25619,25587,27667,27699,27667,27699,27699,27699,25651,25618,25618,25618,25618,25618,23505,25553,25585,23505,25456,25423,25423,25423,25423,25423,21197,12745,12744,21197,12745,12744,19116,12712,10696,21229,14924,12745,10600,14859,16971,16971,16970,14890,8552,8584,8584,10632,10632,10632,12745,25390,10664,14857,19051,12777,10664,10697,10664,10664,14890,17003,12777,12744,10697,10697,10664,14890,17003,10664,10696,10697,12745,10664,10631,16938,16970,10663,10664,12777,19148,21229,21131,21164,17003,19116,14923,10664,14825,16970,14890,14890,17035,36085,31826,40245,38133,33940,31892,33940,36053,38198,36052,31823,25450,23371,21195,21229,21229,16971,50712,50712,52857,54905,54905,54937,52825,44404,42227,46485,46453,33290,33225,33225,33225,39757,41902,33547,4259,4259,4291,6339,6372,6372,6404,6404,8452,8485,12677,12709,14821,12773,10628,19077,12740,6436,4323,12773,12773,12743,52890,38035,17003,8551,35922,12744,14857,14857,16970,16937,16937,19050,19081,17001,38098,35889,35921,38096,19238,21353,19239,17094,19174,17029,14916,17060,19141,17061,17061,17029,12804,10659,10692,14916,14884,14917,8579,10628,10691,17028,14916,8579,10692,10692,12772,6499,6467,8548,12837,12804,10692,10692,10628,10628,45},
{54971,54971,52891,52891,54971,54971,54972,54971,57052,57052,57052,54971,52891,52859,38167,27668,25588,27668,27636,27636,27668,27668,25588,25555,25588,23507,23507,25588,25555,25555,25587,27668,27635,27635,29748,27667,27667,25619,25587,25619,25555,27635,27667,27667,27699,27699,27698,27698,25618,25618,27666,27698,23505,25585,25585,23505,27536,25423,25423,25455,25455,25423,21197,12744,12744,21197,12745,12744,21164,12744,10664,19117,14924,12745,10632,14858,16971,16971,19019,14891,8584,8583,8584,8584,10632,8551,12745,25389,10664,14857,17003,12777,10664,10697,10664,10632,14890,19083,12777,12745,10697,10697,10664,14890,19051,12809,10696,12777,12745,12712,10664,16970,16970,10663,10664,10697,19116,21294,19084,21164,16971,19084,14923,10664,12809,16970,14857,10664,23374,44536,44471,46519,38133,33940,33940,33973,36085,38198,38198,27629,19175,17063,21228,25423,25423,19084,48632,50744,52825,52825,52825,52825,48631,42291,50743,59163,57083,33322,33225,33193,33225,39757,41902,31467,4291,4292,4324,6372,4324,4292,6372,6404,6404,12709,21158,16933,14853,10660,8548,14853,16965,8548,4323,14885,10628,25452,48632,44373,14857,17003,27502,14857,14825,14857,16937,16937,16937,16969,21227,21227,42260,35921,38002,35983,19238,21385,21319,21287,17061,14916,21255,17060,17060,19141,17029,17029,16965,14853,17029,14916,14917,12836,14916,14916,10724,12803,14949,12836,19174,14917,10692,8611,8580,10756,17093,12868,10756,12805,14853,14949,47},
{55003,54971,54971,54971,55004,55004,57084,57084,59132,59164,57084,54971,52891,50811,40280,27668,27636,27636,27636,27636,27668,27668,25587,21395,23507,25555,25555,23475,25555,25587,25555,25587,27668,27667,27667,25587,25587,25587,25619,25587,25555,27667,27667,27667,27699,27731,27699,27698,27698,27698,25618,25618,25585,25585,23505,27666,29649,27535,29584,29584,27536,27536,23309,16938,16938,25390,16970,16970,25390,19051,19051,25390,14892,10665,10632,14859,16971,21132,23277,21197,17003,16970,16971,17003,17003,17003,19084,25422,17003,19116,21229,19116,19083,17036,17003,19083,21229,23277,19116,19149,19117,19116,19116,21197,23277,19083,19116,19116,19116,19084,19083,21196,21196,19083,17035,19083,21229,23310,21229,23277,19116,21197,21196,19051,19083,21164,21131,19051,27600,44536,42391,44439,36053,31860,31860,33973,36085,38198,42359,27629,23368,17031,23340,25455,27503,21164,46487,50712,52857,52857,52857,52857,52825,50712,50711,57050,54938,33290,33193,33193,33193,39757,41934,31435,4259,4291,4292,4324,6372,4291,4291,6372,8484,16933,19045,19045,23270,14820,10628,10660,8548,6435,12805,19077,14917,38066,52825,48632,33874,33841,35921,16970,14825,14890,14889,16937,16937,19051,21196,23341,42292,33809,35921,33902,19238,21385,19271,21319,21286,19174,21286,17061,19141,19141,19109,21222,14916,19142,17029,14948,16997,12804,12836,14980,14948,14948,17061,14948,12836,10756,10724,10724,14949,10724,14949,17061,19206,17125,12836,14917,101},
{57052,54971,54971,54971,57052,57084,57084,59164,59164,59164,57052,54971,52891,50811,42360,27636,27636,27636,29716,29716,29748,27668,27668,27668,27668,25588,25587,25587,25587,25555,25587,25587,25587,25555,25587,25587,25619,25587,25587,27667,25587,25555,25587,27667,27667,27699,27698,27698,27698,27698,27666,29779,25585,25585,23505,27634,52825,52793,52825,52825,52825,52792,50712,52760,50712,50679,52760,52760,50679,52825,52825,38100,16972,12713,10600,14858,16971,21164,27503,17003,10664,10664,10664,14858,12744,8583,10665,16938,8551,8584,14890,10632,8551,10664,12777,8518,8551,16971,10632,8584,12778,14858,10697,8584,16971,8551,8518,12745,14825,8519,8518,10599,16970,6470,8518,16906,21197,23407,23310,19051,17036,19149,19051,10632,10697,14890,14825,14856,25550,44536,44439,46552,33973,31860,31860,33973,38133,40246,42359,27596,23368,21289,27567,25455,27503,21197,46486,50712,54938,54938,54938,54938,54906,50712,52825,57115,54970,33290,33225,33193,33225,39789,41934,31435,6340,6372,8420,8452,8452,8452,8484,8452,14757,16932,14852,14852,19077,16933,12740,12740,8548,6436,10660,14885,10692,27533,50744,52858,27503,33842,50712,19018,14857,16970,16938,19050,16970,19083,21196,21227,44405,35889,38034,31821,19238,21385,19240,19208,19207,21319,14948,12868,17061,19141,19077,19110,17029,19174,14980,14980,12805,12836,14980,17060,17029,12836,14949,15012,10724,12772,14916,10724,12900,12836,10660,14948,17061,19206,17029,23402,84},
{57052,57052,55004,55004,57052,57084,59132,59164,59164,57084,57052,54971,54971,52891,44473,27636,27636,27636,27668,27668,29716,27668,27668,25587,23507,25587,27668,25587,25587,25555,25555,25555,25587,25555,25587,25619,25555,27635,27667,27667,25619,25587,25587,27667,27667,25619,25619,27698,27698,27698,27698,29778,25585,25585,23505,25553,57083,57050,57050,59098,59098,57050,48599,29614,29550,50712,27502,25357,52793,23310,25357,33842,17004,12746,10632,14858,16971,16939,17003,14923,8551,8551,8551,12778,10664,8518,10632,14890,6438,6471,14890,10631,6470,10664,12777,6438,8551,16971,10697,10665,12745,14825,6471,8519,16971,10631,10697,14858,14825,8518,8518,10631,16970,6470,8518,16905,21229,25488,23342,19083,19116,21230,19051,12744,12777,14923,16938,17032,17096,44536,44439,42358,33973,31828,31860,33940,38165,40246,38197,27629,19143,17030,25453,25455,27503,21197,46487,50712,54938,54938,54938,54938,54906,52857,46518,57050,52728,31144,33225,33193,33225,39789,41902,31435,25128,29321,29288,31369,29288,29320,31369,25159,25287,19077,19109,16997,19077,21157,16965,16997,12772,8548,6436,8580,6404,27469,59164,35954,33874,19084,44341,31695,35953,40147,31728,42260,35953,33841,31760,27533,44405,35889,40115,31788,19238,21320,19239,21320,21352,19173,17061,14948,19173,19140,21222,17028,17028,17029,19173,10723,8579,14916,14916,8579,10659,12804,10660,17061,10691,10692,10724,12804,12868,12805,12740,12772,17062,19141,14949,14886,123},
{57084,57084,57084,57084,57084,57084,59164,59164,59164,57084,54971,54971,54971,54971,46585,29748,27636,29749,27636,27636,29748,27668,27635,27668,25555,25555,25587,25555,25587,25555,23475,25555,25588,25555,27635,27667,27667,27667,27667,27667,27667,25587,25587,25619,25587,25587,25619,27667,27699,27730,27698,27698,25585,23505,25553,25553,57051,59131,59131,59099,59098,59098,50679,29582,29550,52760,29583,27502,52793,27503,27502,27536,14924,12745,10664,19084,19084,21164,21165,19116,12810,14857,14857,19084,16970,14857,16971,19116,14858,14890,21165,16971,14922,17003,19083,14890,17003,23277,17004,17004,21164,21197,19116,19116,23277,19116,19116,21197,21196,19083,19083,19116,23309,19083,19115,23277,25390,25455,25390,23309,23277,23310,23277,21164,21196,21196,23276,17032,12838,27566,33938,36052,33940,31860,31827,33940,36021,38166,36020,21257,19111,14950,25420,27503,27503,21165,48599,48631,54906,54938,54938,54938,52825,44404,19113,29583,37937,31145,33225,33193,33193,39789,41902,37708,37740,39788,39788,39788,39788,39788,35658,16965,19077,16965,19077,14820,16997,8515,16997,8547,12805,8516,10693,10660,10660,29581,50712,38132,40148,29584,27535,38034,44372,48599,40147,54905,54937,52824,46485,33806,46453,33808,40115,29708,19271,21320,19271,21321,21352,21319,23400,23399,17060,19140,21190,19108,14980,14916,12836,8611,8547,12804,8579,8548,10660,10724,6467,10724,12804,8611,12804,10692,14949,12773,17029,19174,17029,10659,8612,6467,45},
{59132,59164,59164,59132,59164,59164,59164,59164,59164,57084,54971,55003,54971,54971,50779,29748,27636,29781,27636,27636,27636,27668,27668,25588,25555,25588,27636,25588,27668,25555,25587,25555,25555,23507,27668,27635,25587,25587,29747,29748,25587,25587,25587,25587,25587,23539,23539,23538,27699,27699,27698,27698,25585,25585,25585,25553,50712,54872,59131,59098,59098,57050,52792,33809,31663,52792,33808,29550,54905,29615,27503,21198,14891,12680,16938,19116,19116,17036,19084,19083,19083,14858,17003,19116,14922,14858,19116,19084,14890,17003,19116,14890,14858,19116,17003,14857,19051,21164,14890,14890,19116,17003,14890,17003,19083,14890,14890,19083,16970,12744,12712,16938,17003,10663,12776,17002,19116,23342,21164,21164,17003,19084,19051,14825,12777,19051,19050,19175,10757,8548,10693,21260,31860,36053,33972,36053,38133,38165,40277,29677,19111,17031,25453,27503,27535,21197,48599,50712,54938,54938,54938,54905,52857,25451,14886,33776,46388,31144,33225,33193,33225,41902,41934,39821,39821,41901,39821,41901,41901,39853,33610,19014,12772,17029,16965,14852,10660,10660,14917,8516,8516,8516,10692,17029,16997,29645,44405,36019,35923,12679,19115,38067,42227,44373,38066,54905,52792,50678,42226,27499,46485,33808,38067,27627,21319,19239,19272,19272,21353,21320,17094,21287,17093,17060,14916,17028,14980,10723,8579,12836,10692,10628,10660,10628,8580,12772,10692,12837,8611,12772,12772,14916,19141,12772,17061,19141,17092,17029,10660,8547,44},
{59164,59164,59164,59164,59164,59196,59196,59196,59196,59164,57051,52891,54939,54939,52891,36086,27668,27636,27636,27636,27636,27668,25587,27668,23475,25555,25555,25587,27635,25587,25587,27668,27635,25555,25555,25587,25555,27635,29715,29747,27635,25587,25587,25619,25587,25619,23539,23538,23538,25618,25650,27699,25586,25585,25585,25553,48632,57018,59099,59098,59098,57018,52760,27502,27405,52760,31663,27437,52793,29583,25422,17004,17004,21164,21164,12810,12842,12810,14858,14825,14858,10632,14857,17003,10664,8551,17003,14890,8551,12809,16971,10631,8551,16971,12777,8551,14890,17003,10664,10632,17003,14890,12777,16938,19083,10632,10664,17003,14890,10663,10631,16970,19051,10631,12777,19051,21229,23375,21196,23277,19116,21197,19051,14857,14890,19051,19114,17095,12805,12870,12805,19179,31860,33972,33940,36053,38198,40278,31825,29646,21224,19144,25420,27535,29616,25390,48632,50712,54938,54938,54937,52857,38098,25484,14919,31696,46356,37451,31112,33193,33193,41902,41934,35627,31434,31434,31434,31402,29353,29353,19014,12805,10692,19077,19077,12740,10660,8580,12740,14853,10628,8548,12772,10660,19109,31758,46551,33841,21165,14793,19083,33776,40114,42195,33743,46420,48501,46421,42259,27467,38034,31728,40179,25513,21351,19239,19272,19240,21353,23435,19239,17094,14981,17060,17060,19173,19173,12837,10660,14948,12836,8547,12741,14885,10660,12772,10724,12804,12836,12804,12772,14981,10659,17029,12772,19141,19205,17060,10756,10660,39},
{59164,59164,59196,59164,59196,61244,61244,61244,61244,59164,57052,52891,52891,52891,52891,42424,29748,27668,27636,27636,27636,27668,25588,23507,21427,21395,23475,25555,29748,27700,27635,27667,27667,27635,25587,27667,27635,27667,29747,29747,27667,25555,25587,25587,23506,23539,23539,23539,23539,23538,25651,25618,25618,25585,25553,25553,44406,52792,59098,59098,59098,59098,54905,35921,33776,52792,38002,33776,52792,35889,29648,17037,14924,23277,23342,14890,12842,12810,14858,14858,14890,12745,16970,19084,14857,12777,19116,17003,12777,17003,19116,14890,14890,21196,19083,16971,19116,21197,16971,16971,21196,19083,17003,21164,21196,19083,19051,21196,19083,14857,14858,19083,21164,14857,16970,21196,21229,25455,23277,23309,21197,23277,21196,19083,17003,21196,21225,17094,14982,14949,17030,19177,33908,36053,31859,36053,38198,42358,31824,31791,23305,25451,33807,52825,52858,52825,50712,50712,54938,54970,54938,46517,21257,19144,12774,29583,50582,39500,39467,33192,33193,41902,41902,27274,4259,4291,6340,6372,6340,6372,12740,12836,8547,19077,14852,14853,14885,10660,10660,12772,10660,8580,14885,10692,14917,33806,50712,40179,21262,8518,25390,40115,46485,48566,42195,52824,52824,52792,48566,37999,44372,35888,42292,23433,21352,19207,19240,19240,23434,23401,21287,15013,12868,12836,14980,17061,17060,10691,10724,14948,17093,12804,12837,14916,10659,12772,12772,10724,12772,12836,14981,17060,17029,12836,6467,8547,19141,12836,10692,8612,36},
{59197,59164,61245,61244,61277,61245,59196,61244,59196,59164,57052,54971,52891,52859,52891,52924,31861,27668,27636,27636,27636,27668,27636,23475,23475,21427,21395,23475,31828,29748,27699,27635,25587,25555,25555,25555,25587,27667,27667,25587,25587,25587,25587,25587,25587,23539,23539,23539,23539,23539,25618,25618,27698,25553,25586,25553,48632,57018,59098,59098,59098,59098,54905,25389,23244,50679,27469,25325,50679,27470,23245,17004,14924,16971,19084,21229,21261,21229,23277,21229,21229,19116,21196,21229,19084,19083,21229,21196,17003,19148,21229,17003,17003,21197,19084,16971,19116,21197,17003,17003,21196,19116,17035,21164,21196,16971,19051,21196,19083,14890,14858,19051,21164,14857,16970,19116,21229,25423,23245,21196,19116,21196,21164,16938,14890,19051,21256,17062,17094,14917,12805,17030,36053,33940,31860,33940,36085,42359,38099,33904,27531,23305,33807,57051,57051,54906,48632,50712,54970,54970,54938,44404,29677,23338,17000,27438,46356,39500,41613,35273,31112,41902,41902,29354,10500,10533,10533,12613,12613,14694,14725,16965,10724,19077,16964,16996,10628,14884,12804,16997,12772,8580,10660,8580,8516,33807,50711,54971,21294,8584,12744,40115,44372,48566,44340,50711,48599,48598,46485,38031,44308,35888,40211,21352,21320,17159,19272,21320,21385,25546,21353,25547,19175,21288,17093,17092,17060,14948,17061,14948,15012,12836,14981,17060,12804,10692,10723,12836,14948,10724,14981,17060,14916,8612,8547,10724,14949,12837,10756,14917,71},
{59164,59164,59196,59196,61244,61244,59164,59164,59164,59132,57052,52891,52891,52859,54971,52891,38200,29748,27668,27636,25556,27636,29716,25587,25588,21428,21395,23475,29748,33941,27667,25555,25587,23507,25555,25555,25555,25555,27635,25587,25555,25587,25587,25587,25587,23539,23539,23539,23539,23539,23539,23539,27699,27666,31860,29714,48599,59131,59099,52178,52533,57050,56986,38067,38002,54873,35922,33809,52760,38034,35922,17037,14924,25357,21229,12810,12874,12810,14858,14858,14857,10696,14890,19083,12745,12744,19083,14923,10664,16970,19084,12745,12744,19084,14890,12777,17003,19116,12778,12778,19084,14922,14858,19083,19116,14825,12712,19051,14890,10631,10632,16938,17003,8551,12712,17003,19148,21262,21164,21164,16971,21164,19051,14857,10697,19050,21224,12869,17093,17062,12869,12869,33939,33973,31860,31860,36118,38198,44439,36051,29646,38065,40148,54970,54970,54906,50712,50679,54937,54937,52857,38098,19145,21226,19081,23244,46324,39499,41612,39499,35370,41869,39821,39821,41934,41934,41934,41901,41901,41934,41901,39885,14885,12804,19141,16997,12772,14884,12804,12804,14885,10660,10661,10693,10693,23307,29615,42326,25488,10765,12778,40147,46453,46485,44340,52824,52792,52792,48598,38031,42227,33808,42292,19238,21320,19207,19272,21321,21385,25547,23434,25548,25580,21256,23401,17093,14948,17029,16964,17061,19173,17060,19205,14916,17029,12836,14948,14980,14948,14916,19205,14948,10692,6467,10692,14948,17061,10660,8579,8612,103},
{57084,57084,57084,59164,59164,59164,57084,57052,54971,55003,54971,52891,52859,50811,52859,50811,44505,31893,29748,27636,27636,27636,29748,29716,25555,23475,25588,25555,27635,29748,29716,25555,25587,23507,23539,23507,23507,25555,25587,25587,25587,25587,25587,25587,25587,23539,23539,23538,23539,23539,23507,23538,27667,31827,44504,44407,46486,59098,59098,50000,56759,57018,56985,23276,21164,50647,25357,23244,48534,27502,23244,17004,14891,25390,21229,12842,12875,12842,14858,14858,14858,10696,14922,19083,12712,12745,19083,14890,10664,16970,19083,12744,12744,19083,14858,12745,19051,21196,14858,12777,19083,16970,14890,19083,21196,21131,19018,21164,16971,12744,12745,19051,19083,12745,16938,21163,23277,21229,21164,21196,19051,21197,21164,16970,16971,23275,21224,12838,12837,17094,14949,17030,27567,33973,33940,33940,34005,38198,40279,44471,38132,54971,46486,54905,54937,46519,48599,48631,54905,54905,52825,40179,27532,16999,16999,21131,44276,39500,41612,37418,41902,41902,39788,41901,41901,41934,41934,41901,41901,41934,41901,39853,31561,14916,12836,14948,19077,10692,19110,19109,17029,12804,8548,8517,12774,10663,12745,42358,27635,4622,14890,42227,46453,46485,44340,52824,50744,48631,29614,31725,40114,33776,42292,19238,19272,19239,19240,21321,21320,23434,23402,25548,23401,12837,14981,14949,16997,19109,21222,19109,21222,19173,14980,14916,19077,19109,14948,12804,14980,12803,17061,8611,8579,8580,10660,12772,14980,14949,10756,8580,151},
{57052,55004,57052,57084,59164,57084,57052,54971,54971,54971,54939,52891,52859,50811,50778,52891,42425,44538,31893,27668,29781,27668,27668,27668,25587,25588,25555,25555,27635,27668,29748,25555,25555,23507,23539,23507,23507,25587,27667,25587,25587,25587,25619,25587,25587,25587,23507,23506,23539,23538,25587,25587,25586,38133,52858,44439,50712,59131,59131,54388,50162,59098,57050,40147,38035,52792,42228,38035,50680,42260,40115,40148,42229,21099,31696,33841,33873,33873,35954,35922,35954,35922,38035,38067,38067,40147,40148,40148,40180,42228,42228,42228,42260,42260,42261,44341,42292,44341,44373,44341,44341,44373,44373,44373,44373,44405,46454,46454,46454,46486,46486,46486,48566,48566,48566,48566,48566,48566,46454,31696,27535,27503,27503,27503,27503,27502,21257,12837,12837,12869,10660,8581,21259,34005,31860,34005,38166,38166,42359,46585,48665,59197,46519,54938,54970,52825,50712,50711,54905,52857,52857,50744,29678,14951,12773,21098,46324,39500,41580,35338,31402,31434,29321,29354,31402,31402,31434,31434,31402,31434,31401,31401,21093,10724,8612,10659,12836,8580,14920,19079,16999,19112,12775,12776,14887,16937,19083,29681,17069,8652,14857,40114,46453,46485,44373,52792,50711,48598,33872,23274,33776,33808,42292,19238,19239,17094,17127,21289,21321,21322,21322,23404,25516,25515,21288,16998,35951,21158,23270,29545,31625,23334,17060,19141,12804,14916,14980,14980,14948,14948,14980,10692,14917,10756,8547,8547,12804,17093,12868,14918,177},
{54971,55004,54971,57052,59164,57084,55003,54971,54971,54971,52891,52891,52859,50810,50779,50811,40312,44537,42425,29748,31861,29748,27636,27668,25555,25588,27636,25587,25588,25587,29716,25587,25555,21427,23507,23507,23475,25587,27667,25587,25587,25587,25587,25587,25587,25587,23539,25586,23538,23539,23539,25586,23506,29747,29714,31827,48600,59131,59131,40115,46454,57018,57018,25389,23244,46421,27470,19083,44308,31695,23244,48567,46486,48567,46453,52824,52857,52857,50647,54905,52792,42228,50712,54937,42260,42260,54905,50679,35954,48599,54905,44373,40147,54937,48566,38034,50679,54905,40147,40147,52824,48566,31696,46486,52824,38034,38034,52792,46453,27502,27470,46486,52824,29583,33808,50711,40147,25325,44373,27503,10664,12777,19116,14890,10631,17001,19143,14949,12837,14949,12869,8613,8549,38198,31892,33972,36085,38166,42359,46585,52923,55003,42293,54970,54970,52857,50712,44373,38067,33841,27502,21228,14919,12806,12839,23179,44211,39500,41580,37419,20774,16709,16709,14661,14661,16709,16709,16709,16709,16709,16709,16741,18949,10724,10724,12804,12836,12772,16999,12773,10660,14885,21194,10599,10630,21098,21196,48665,19181,10665,12679,40146,44340,46453,46485,52792,48598,44372,35920,31758,33808,35921,40210,19238,19271,19272,19272,21320,19240,19241,21353,23402,25483,23434,25515,35984,50679,29513,31561,33674,31593,35787,21189,12836,12804,14916,14949,12836,14949,12804,10692,8547,17061,12868,10692,12805,6467,12805,10724,27596,174},
{55004,55004,55004,57052,57084,57084,57052,54971,54971,54971,52891,52891,52891,50811,50779,50811,42425,40344,46618,38167,29748,31861,29748,27668,27636,27636,25587,25587,27636,25587,27668,27668,25587,25587,25587,25555,25587,25555,25555,25587,25587,25587,25555,25587,25587,25587,25587,25587,25586,23538,23539,25587,23506,27667,27666,25553,42294,59163,59163,44341,35889,59098,57050,44308,40147,50679,44340,40147,50647,44373,38035,48567,48567,48535,42358,40277,42455,44503,31663,46421,40148,10632,35922,48598,21196,16938,48598,38066,12712,35921,48566,25390,19018,50679,35954,21163,42227,50647,25357,25357,48598,40147,14857,38034,48566,25422,25389,48598,40147,19116,19084,42260,50679,27502,31695,48566,35954,21196,40179,25390,10664,12777,17003,12810,10631,14890,19175,14949,14949,12869,10789,14950,8549,31826,33972,33973,36086,38198,40311,46552,42358,29645,33840,52824,48566,31696,29615,23276,23309,25389,23309,23309,21195,17032,14887,27339,44211,39500,41580,41580,37451,41934,39854,35628,31466,31467,29386,31467,29386,29387,29354,29419,25256,17061,12836,8611,12772,21190,14820,14885,8548,6403,25420,8518,6404,12744,16970,52858,35988,29617,23245,40147,44372,46485,44372,50679,42259,38033,38033,31726,33808,38001,40178,19238,19271,19272,19272,21320,19240,19240,23402,23435,25515,25515,29709,40178,50679,37837,37804,42029,39949,35787,21221,17093,14948,17093,14948,12836,10723,14917,10724,8580,14948,14948,12836,12837,6467,6435,14950,33936,176},
{55004,55003,55003,57084,57084,57084,57052,54971,54971,54971,52891,52891,52859,50811,52859,57084,46618,44538,40312,46618,33974,33973,29780,27635,27668,27668,27668,29748,25587,25587,25555,27668,25587,25587,25555,23507,23507,25587,25587,25555,23506,25587,25587,25587,25587,25587,25587,25555,23506,23538,23539,23507,23539,23506,29779,25553,44406,59131,59131,42293,38003,57018,59098,29550,23244,42260,31695,21164,40083,31696,19051,48535,48567,46454,40212,36116,38262,40309,33744,42195,40116,23408,40181,50711,35955,29681,50712,44406,27569,42261,50679,33874,31729,50712,42261,25488,44373,50679,31729,29616,48631,42293,19182,38067,48599,27536,25423,48598,42260,23342,23278,42228,48631,19149,23342,48566,33841,12811,38067,25390,8618,10698,17003,12778,8552,12810,19143,17062,12901,14981,14949,12901,10757,21259,36085,34005,36086,38198,42391,44504,44438,29711,33840,50679,46518,35954,25389,25422,25422,27502,27502,27502,25422,23308,17032,29387,44211,41580,41580,41580,37451,41934,41901,25161,6340,8485,6404,8518,6372,6372,6372,8485,10660,10691,8611,14980,12868,16964,21157,16965,8516,4323,21195,8518,6437,10663,16937,52858,33907,10697,17036,40147,46453,40114,40146,48630,46518,50711,48598,29581,31696,38001,38130,19238,19271,19272,19272,23435,19240,19239,23434,23434,23435,23434,27629,40178,52760,39918,44078,46192,46192,37868,17060,12836,14948,14948,10691,10724,14917,10692,10692,6467,6467,8612,12773,8580,6435,10660,10692,40244,178},
{55003,55003,55003,57052,57084,57052,57052,54971,54971,54971,52891,52891,52891,57052,59164,59196,46618,36152,36119,48698,44473,33941,33941,27668,27668,27668,25587,27635,25587,27668,25555,27635,27668,25555,23507,25587,23507,25587,25587,25555,23507,23507,23539,25587,25587,25619,25587,25555,23507,23539,23539,23538,23506,25554,29779,27633,44374,59131,59131,44373,33777,54905,57018,42260,40115,50647,44373,40115,48534,46453,40147,48567,48567,48534,40245,36149,40342,42358,29550,42228,40148,17070,38067,48599,31794,27568,50679,44406,25456,40180,48599,33874,29649,50679,42293,25488,44373,50679,31794,29649,48631,44374,27569,40212,48599,31794,29649,50647,44373,29649,27568,44341,50679,29616,31761,48599,38100,25423,42260,40147,12778,12778,17004,14891,10697,14890,19146,17062,14949,12869,15013,12901,14981,10725,38165,36085,38166,38198,40311,44504,50810,48664,42260,54938,50712,23309,23276,23341,25389,25454,25422,25454,25390,21228,21227,31403,46292,41580,41580,41580,37451,41934,39853,29321,25193,27273,27274,27274,27306,27306,29354,29386,21158,14948,10756,10692,21190,12740,16997,14852,12772,8516,21195,8517,6437,10663,16937,52825,31794,14826,25390,42260,50679,44372,19017,10598,10598,12679,35953,25419,31695,37968,40145,19238,19271,19272,19272,23435,19239,19272,23434,23434,23435,23434,29678,38066,52793,37773,46159,46192,46192,37868,19173,14916,12804,14948,14885,8547,6435,6467,6467,8548,6435,6467,8547,8580,6435,8612,12805,44438,176},
{55003,55003,55003,55004,55003,57052,57052,54971,54971,52891,52891,52891,52891,57052,59196,59164,46618,31927,38232,46585,48699,40312,38167,29780,27636,27636,25588,25588,27667,25587,25587,25555,27667,25587,25555,23507,23475,25555,25555,25555,25587,23507,23539,25555,25587,23539,25587,23507,23507,23506,25587,23506,25587,25586,31827,29746,42326,59131,59131,29583,31663,59098,57018,29583,21164,40115,33809,23276,35889,33841,21131,46422,48567,48534,42293,38197,40310,42358,35889,46389,42261,19150,38035,50679,33874,27568,50647,44373,17038,35987,48598,31729,25455,48599,40148,19150,40180,48566,29648,23310,48566,40180,12844,35922,48566,25455,23310,48566,40180,12811,10731,38035,48599,23342,27535,48534,33842,10666,40114,46453,8551,10632,14923,12777,8519,12777,19113,17062,17061,14981,14981,12869,12869,10725,31791,38166,38198,40279,40311,44504,48730,55036,46519,54938,48599,23309,21196,23309,23309,25390,23342,23341,23309,23309,21228,25227,46291,39500,41580,41580,37451,41966,41901,27209,35692,41934,41934,41934,39885,41934,41934,31530,14853,14948,14948,14884,23238,16933,14852,16964,23301,14885,25388,10630,12807,16937,23276,46519,29649,40084,33842,42227,48598,44372,19018,10598,10598,8485,40145,29610,29582,38001,40177,19238,19239,19208,19272,21321,19239,21320,23434,21354,21354,23369,27597,40178,54873,35693,46159,46159,37868,19140,14948,12836,17061,16996,8579,6467,6435,8548,8547,8612,6467,8580,8580,8612,8516,8580,14886,46519,175},
{55003,54971,57052,55003,54971,54971,54971,54971,54971,54971,54939,54971,54971,57052,57051,54939,50778,36184,34007,44538,44505,44538,40247,36054,27668,27668,25587,25588,25587,25587,25587,27636,25587,27667,27667,25587,23507,25587,27635,25587,25587,23506,23539,23507,25587,23539,23539,23506,23507,25555,25587,23506,25587,25586,29746,29746,40213,59131,59098,48599,31664,59098,57018,44340,40115,48534,46453,40179,46453,46486,40115,48535,48567,48567,46519,44471,46583,48632,44276,50679,50712,31827,44406,52792,44374,40213,52792,50712,36020,46487,52792,44406,42293,52824,50712,42293,50712,52824,46486,42325,54873,50744,42294,50680,52825,46519,46486,54872,50744,44406,44406,52792,54905,48599,48631,54905,50744,46487,52824,50712,23309,21229,21262,21229,21197,21229,21260,19207,17126,17126,14981,17062,15013,12837,17031,42358,38198,40279,42359,44472,48698,55036,48632,31695,33809,23341,21164,23309,21229,23309,23309,23309,21229,21229,21196,21099,46259,41548,41580,41580,37483,41967,39853,25161,37741,41934,41934,39853,41934,41934,41934,25287,16966,16997,16996,16997,23270,19077,21158,12772,16964,12772,23339,12743,8612,8550,10631,25357,23245,21197,29584,42260,48566,42259,19050,12711,12744,12711,38065,25450,27501,38001,38129,19238,17158,21354,21322,19176,17127,19176,21290,19210,21290,21290,29678,38001,46486,31500,46159,46192,44110,25383,19141,17060,14948,14980,10660,6435,6435,12805,12836,10691,12772,10692,8548,10693,10692,10660,19145,46519,178},
{55004,55004,57052,57052,55003,54971,54972,54971,54971,54971,54971,54971,57052,55003,52891,52891,52859,42425,36184,38232,46618,48666,55036,40214,31861,27668,27668,25588,27668,25587,27668,27668,27668,27667,27699,27635,25555,25587,25555,25555,25587,21427,25587,25587,25587,25587,23539,23506,23539,25586,25587,25586,25587,25586,29747,31859,36052,59131,59131,48599,38002,59098,57018,31728,23277,37969,33841,23277,33808,38002,23244,44309,48567,48567,48567,56986,56986,54938,54905,52793,46519,42293,44406,52825,46454,40213,48600,50712,44406,44406,52792,44406,42325,44439,48567,38100,42293,52792,40180,35955,44406,46487,33875,35987,50680,38068,35987,42326,46454,31794,31762,35987,50680,35954,33907,44341,40148,27568,33874,46454,14825,12744,14891,14858,10632,10664,21194,19143,17062,17062,12869,12869,14982,15014,12805,27534,42359,40311,42391,44472,48698,55004,52858,29616,33841,35987,27535,23309,21229,23310,23309,23309,21229,21228,21196,19050,44179,41548,41580,41580,37483,41934,41901,25161,31434,33515,33515,31434,31434,31434,31466,25224,19078,19046,12772,14821,19013,23239,16932,17029,14884,12772,23307,10662,10661,8486,18856,8486,16906,12745,25390,44340,48533,44340,19017,10598,12679,12711,38065,25449,27468,37969,38096,19238,17190,21353,23435,21321,19240,21289,23403,23436,23436,25516,29711,42260,46486,33677,44079,44078,37836,19077,10755,12868,12836,14981,8547,6435,8580,12836,15012,14948,14916,17029,16966,14885,8580,10660,21290,44438,178},
{57052,57052,57052,57052,55003,55003,54971,54971,54971,54971,54971,54971,54971,54939,52891,54939,52891,42393,31927,38265,48730,55004,55004,38166,38134,31828,29748,27636,27636,27636,27668,27667,27667,27667,27667,27667,25555,23507,25555,23507,23507,23539,25587,25587,25587,23507,23507,23506,23506,23539,25587,25587,25587,25555,31859,31827,36020,59131,59098,44373,29551,59098,57018,44373,38067,46454,46454,40115,46421,46454,40147,46486,50648,48567,48535,50647,50647,48567,46454,40181,23474,12877,19215,44342,27569,14990,31795,40180,23376,19215,44341,27569,25488,31795,35923,17102,23376,46454,23376,14990,31795,38035,12909,19215,44342,23343,19182,31794,38003,17069,17037,21295,46454,12844,14957,35923,29617,10731,21263,46421,14857,10632,12778,12745,8518,14888,21224,17094,15014,12869,10757,12869,10757,12869,12869,14950,44438,42359,42391,44504,48697,52890,44406,27535,31728,50745,38067,23309,14890,14857,14857,12777,12809,12809,12776,14760,42066,41580,41580,41580,37483,41934,39853,27241,14726,16839,16806,18887,14758,14759,14758,16934,12805,8580,10660,8580,16965,21126,10628,16997,19077,14853,23307,12742,8580,14858,27471,33842,16938,38068,42229,42260,46453,42260,16937,12711,12712,14824,38097,23401,25419,37969,38096,19238,17190,19272,23436,21322,19240,19241,23403,21354,21322,23435,29678,42259,44341,33710,39917,35755,19109,17093,12867,14948,8579,8579,8547,10628,8580,14981,14916,12900,14948,12804,17029,17029,10724,10692,25515,44438,178},
{57052,57052,57052,57052,54971,54971,54971,54971,54971,57052,54971,54971,54971,54971,54971,54971,52891,48731,38265,40313,48666,54971,50778,38167,33941,36054,31860,27668,25587,27636,27635,27635,27635,27668,27635,27667,27667,25555,25587,25555,25587,25587,25587,25619,25587,23507,23474,23507,23538,23539,23506,25587,25587,25587,29746,29778,33939,59131,59098,54905,46421,57018,57017,33841,23277,35857,35954,23277,33744,40115,21196,42228,50680,50648,48567,50648,50648,46422,35889,50712,42358,40213,42294,50712,46487,42293,46519,50712,46486,46454,52793,48599,46519,48632,50744,46487,46519,54873,48599,46519,50712,52825,46519,48599,52857,50680,48599,50744,52825,48600,50680,50712,54938,50712,50712,52825,52793,50712,50744,54937,50712,42261,25423,21229,23309,25419,21256,19207,17094,14950,12837,10725,8581,6500,6436,10725,21226,48665,44440,46552,50778,54970,48599,27471,29616,50777,40213,21261,19148,21229,21196,21197,19148,19148,19116,19051,39920,41580,41612,41580,37483,41934,41934,23080,6340,8452,6372,8453,8420,8420,8452,10628,10660,10628,10628,8580,8580,10692,10692,14884,16997,16997,25420,14886,14886,12778,33907,52793,18986,48600,44373,44340,46453,44373,14824,10598,10599,12776,29646,23369,25354,35888,35983,19238,19238,19239,23435,23403,19240,19241,21323,21322,23402,23435,29711,40115,35889,31630,31530,23270,19140,17060,14980,14981,12804,8580,10692,8548,10628,12804,17029,14916,10756,10692,14948,14917,17062,12868,27660,46519,179},
{57052,55003,55003,57052,55003,54971,54971,54971,54971,54971,57051,57052,57052,54971,54971,54971,54971,46618,31991,38200,50778,57117,40279,46618,29748,33941,38134,29780,25555,27636,27635,27667,25587,25587,27668,27667,27667,25587,25587,25587,23507,23507,25587,25587,25587,23506,23539,23506,25555,23538,23539,25587,25555,27667,29779,31827,33907,59164,57051,57051,57051,57018,54938,48631,46487,50647,50712,48599,50712,50712,46519,48567,48535,48535,48534,48567,50648,48600,29551,46552,33941,23409,27602,31860,27634,23408,36020,50712,21295,35987,50679,31794,19182,27667,25554,23343,42261,50679,31761,17102,27634,25554,19182,35987,48599,27536,14957,23441,23441,14957,12844,38067,50679,23343,21230,21328,19183,10763,40115,50679,21163,19083,8584,8519,6438,21225,19143,15014,17126,14950,15014,12870,10725,8581,8549,6436,8581,35921,48665,46584,50810,61277,50712,27503,29648,50745,38100,19148,19116,19181,19148,19148,19148,19116,17068,19083,37743,41580,41580,41580,37483,41934,39853,29321,37773,39853,39854,39854,41966,41934,41934,39886,33643,29481,21126,21126,16933,21191,19014,31595,19046,14854,27532,14950,14919,21197,54971,46519,29616,35954,44406,44340,46453,42260,10598,6404,6405,10631,14888,21320,21225,35888,33902,19238,19238,17190,23435,23435,19208,19273,21322,21322,23403,29710,31791,40212,29583,35856,31530,27463,19140,17028,14916,14917,10660,8515,10661,10628,8516,10628,10692,8547,10692,8579,17061,10692,12772,12836,21288,55003,204},
{57052,54971,54971,54971,54971,54971,57052,57052,57052,57052,57052,57052,57052,54971,54971,57051,54971,50811,38200,46618,46585,61342,54971,61310,29780,29748,38134,31828,25587,27636,25555,25587,25587,25587,25587,27668,29748,27667,25587,25555,25555,25587,25587,25587,25587,25587,23539,23538,23506,23507,23506,23538,23539,27667,29778,31827,29779,29714,27633,27633,27633,25553,25552,25552,25553,25553,23472,25552,25553,25552,25552,42293,48599,48567,48535,50648,50648,48599,27470,42326,29747,21296,25522,29780,27634,23440,36020,50679,25456,35987,50679,35987,27569,31827,29747,25488,42260,48598,33907,25488,29811,29747,23408,38067,48598,33875,25488,29747,29714,29649,27568,42228,48598,29616,27536,29714,29681,25455,42228,48566,25357,23244,12810,12745,10664,19112,19175,17094,17094,12901,14950,12805,10693,6468,8581,6436,6436,8549,48566,54971,57084,59164,48632,27503,29616,31761,29680,21196,19115,19181,19149,19149,19148,19116,19116,19083,35598,41580,41580,41580,37483,41934,41901,25128,39853,41901,41902,41934,41966,41934,41934,41934,41934,35691,33611,23207,21126,23207,23143,19046,19015,12741,10693,15014,8551,21197,33874,27568,31697,33841,40180,38034,31728,21163,8518,8485,8485,12744,16969,25481,25417,33775,33901,21319,19238,17189,21354,23436,19240,21321,21323,21322,21355,27630,29679,38067,27535,29581,25352,25383,21253,17029,10692,8547,8515,8516,8515,8580,6467,8516,8580,8548,6467,6467,6467,8547,17061,14948,17062,31822,163},
{57052,57052,55004,54971,55003,54971,57052,57052,57052,57051,57052,57052,57052,57051,57051,54971,54971,52891,42360,46617,54971,54939,63390,61342,38199,29748,42327,38166,25587,25555,25588,25588,23475,25555,25555,25555,27635,29747,27667,25555,25587,25587,25587,25587,25587,23539,23506,25587,25586,25587,23506,23507,25555,27635,29746,27666,27666,29778,27666,25585,25585,23505,25553,23505,23505,23505,23473,23473,25553,23505,23505,40181,48567,48567,48535,48599,50648,50647,27470,42293,23474,12877,17103,23473,21296,19150,29714,50679,14957,29681,48599,31762,21230,21361,21328,14957,35922,48566,27568,19182,21360,21361,10764,29648,48566,25456,17070,21360,19248,21230,19150,38034,48566,17070,14957,21360,17102,10731,38035,48566,12712,12745,12843,12810,8518,25419,19143,17094,17127,14982,12838,10661,8581,8581,10725,8549,6436,6436,10662,44405,54970,50777,44406,40148,42261,42293,35922,19148,19116,21229,19181,19181,19148,19148,19116,19116,33485,41580,41580,41580,37483,41934,41901,25128,39854,41934,41934,41934,41966,41934,41934,41934,41934,39853,39885,27369,19013,25256,23271,14886,19049,19016,16967,14981,14950,21195,25391,42262,31697,21099,33841,14857,16938,12744,8518,8518,8517,14856,17001,25480,27562,31694,31821,19270,19238,19238,21321,23436,21258,14887,17033,14953,21259,23341,29647,31761,25422,23305,21190,21254,19173,12836,10660,8548,10660,12740,8548,8483,6435,8516,8580,6467,8548,6435,10693,6467,12836,14980,17093,23369,132},
{57052,57052,54971,54971,55003,57052,57084,57084,57052,57052,57052,57052,57051,57051,57051,54971,54971,54939,50811,55036,59197,61277,52891,57084,46618,29748,31861,33941,25587,27636,27635,25555,25555,25555,25555,25555,25587,27667,29747,27667,25619,25587,25587,25587,23506,23539,25587,23539,23539,23539,23506,23507,25587,25586,29778,29746,27666,27665,27666,27666,25585,23505,23505,25553,23505,23505,23505,23505,23505,23505,23505,40181,48567,48567,48535,48567,50648,50648,31631,48599,38166,31828,33972,38198,38133,38100,44439,52760,38101,44439,52825,46519,42294,42359,42359,40245,48631,54872,46519,40246,44439,44439,42326,48599,54905,48599,44407,46552,46552,46551,46551,52824,54937,48632,48600,48664,48632,46519,52825,54937,48599,48599,48632,48632,48599,50678,25450,17094,14982,14950,12902,12870,8613,10725,10725,8581,8549,8549,8581,10694,21195,31761,40181,52825,57019,57051,40213,19148,19116,21229,21229,19181,19149,19148,19148,19116,31372,41580,41580,41580,37483,41934,39853,27241,27241,29289,29289,27241,29289,27241,27208,25128,27208,20967,19046,10660,12772,14916,10660,8548,12710,16871,25354,19111,19176,25388,29616,42228,25325,18985,23244,19050,16937,14857,12679,12678,12711,21130,21163,23338,23370,29646,29708,19270,19238,19238,21320,23403,27566,10663,10631,8550,12777,19083,31728,27535,25390,25452,14947,17093,12836,17028,10724,6467,12772,12804,8547,8548,6435,8516,6435,6468,8548,10660,8548,12805,12868,17062,12773,42324,176},
{57052,57052,57052,57051,57084,59164,59164,57084,57084,57084,57052,57084,57084,57084,57052,57051,54971,54971,52891,63390,63390,61277,59197,57116,52956,36086,31829,29781,25588,25588,27635,25587,25587,25587,23507,23507,25555,25587,27667,29747,27667,25587,25587,25587,25587,25587,25587,23538,23539,23539,23507,23506,25587,25618,29778,29746,29747,25585,27666,25585,25585,23505,23505,23505,23505,23505,23505,23505,23505,23505,23505,23473,25521,29714,33875,38068,50680,48599,31664,50680,38133,31827,33908,36053,33908,29714,38133,52760,29714,38068,52792,40181,25488,31860,31827,25456,40180,50712,40181,31826,31827,29779,21295,35955,50711,35987,25455,27634,25521,25456,17037,40115,48598,21262,17069,25489,21263,14924,40115,48599,19050,19051,21294,17036,12712,33807,25450,17094,14950,15014,12901,12869,12870,8580,12870,10661,12774,6469,6468,10693,10662,12744,27568,52825,57019,57019,42293,21196,19116,19181,19149,19149,19149,19148,19116,19116,29259,41580,41580,41580,37483,41902,39853,18887,6339,8452,6372,8485,8420,12646,10598,12773,6436,14885,12836,8547,10692,14981,10724,12807,6404,12743,19113,14949,17094,23339,27502,42293,31632,29259,25357,21164,23277,21164,16937,19018,19082,27502,27502,25421,25420,29646,27627,19270,19270,17190,19240,23467,21227,8550,8550,10631,12744,14922,25422,25422,25421,19112,27561,12836,14980,12868,12836,10692,14917,14948,10660,8516,8515,8548,8548,6468,6435,10692,10724,14917,10724,8580,14885,36049,176},
{57052,57052,59132,59132,59164,59196,59196,59164,59132,59164,57052,57084,57084,57084,57084,54971,54971,54971,52891,61309,61277,59164,61277,52891,52891,48730,38166,31829,27668,25555,25587,27668,25588,25555,23507,23507,25587,25587,25587,29748,29747,25619,25587,25587,23506,25587,25587,25586,23538,23538,23507,23539,25587,27666,29778,29746,29778,25617,27633,27634,25585,23505,23505,23505,23505,23505,23505,23505,25553,25553,23505,23473,23473,25521,27633,29681,50712,50648,29616,44341,25554,19216,21360,27634,25522,21328,31827,50712,25488,33875,50679,38100,23440,29747,29747,23408,38100,48599,38068,29681,31827,29747,25488,35987,50647,38035,27568,29747,29747,27568,25456,40180,48599,29648,25456,29714,27601,25391,42228,48598,29583,27535,29649,27567,25389,31726,23337,19143,17126,17094,15014,12869,12870,12838,8613,10694,8581,8549,10694,12838,10694,6437,21163,52858,57051,57051,44374,21196,19116,19181,19181,19149,19149,19148,19148,19116,29226,41580,41580,41580,37483,41934,39853,25161,21000,21032,21000,23080,21032,23081,23113,25320,21158,12805,12836,10660,10661,8579,14949,16968,16936,18985,21064,17063,23401,25418,25422,40181,10534,22953,12777,12777,12809,10631,6372,6404,6404,17002,14889,17001,27628,31726,27594,19270,19270,19238,19239,23402,25453,10631,10696,10696,12744,19148,29615,23340,21256,14918,16998,12804,17061,12803,10723,14949,12804,12836,12772,12741,10692,8547,8580,8548,6467,10660,8580,14916,12772,12804,14949,25515,189},
{57052,57052,59164,59164,59164,59196,59164,59164,59164,59164,57084,57084,59132,57052,57084,54971,52859,50811,50778,61277,63390,61309,59197,55004,50811,50811,46618,38134,33941,25555,25587,27668,25587,23507,23507,23507,25587,23507,23507,25587,29747,27667,25587,25587,23506,25587,25587,23506,23538,23506,23506,23506,25587,27699,29778,29746,29746,27665,25617,27666,25585,25585,23505,23537,23505,23505,23505,23505,25553,25553,25553,25553,25521,25553,29682,27569,50680,50648,31696,44374,27634,21295,21296,27634,23441,17102,29682,50712,27536,33842,50646,38067,21263,23441,23409,19150,33874,48566,29681,10796,23408,21361,19149,31761,48566,29681,17069,21360,21328,12844,12844,35954,46486,16971,12779,21295,17102,8551,35921,48566,21163,12744,19181,17068,19050,35952,23306,14886,14918,17126,17062,10693,17030,17031,10693,10661,8549,6469,8517,12806,10661,12806,23308,52857,57019,57051,46487,23277,21164,23309,23341,23342,23341,23342,25390,25390,31340,41580,41580,41580,37515,41934,41902,23016,41966,41966,41933,41934,41934,41934,41934,41966,29480,23271,19110,10724,12804,8547,14981,10628,16998,14726,10564,10820,23400,19175,25421,42294,20775,21002,8551,12777,12809,8583,4292,8452,6404,17002,14889,12840,25547,31758,27594,19270,19270,19238,19271,23402,27567,8518,8551,10631,8551,21196,31728,23372,19110,12772,17029,10724,14980,14980,12804,12803,17061,12804,10691,10724,10692,10660,8579,10660,16999,10660,8547,10724,10660,10628,12805,14950,198},
{57019,57084,59164,59164,59164,59164,59164,59164,59164,59164,59132,59164,57084,52891,50811,50811,50811,50779,48698,50746,61309,61277,50778,52924,52923,48698,46650,46585,36086,25555,25587,25587,25587,23507,23475,21427,23475,23507,25587,23507,27667,29779,25587,25587,25587,25587,25587,25586,23539,23506,23538,23506,25554,27666,29746,29746,29746,27666,27666,27666,25586,25585,23505,25585,23505,21425,23505,21425,25553,27634,25553,27601,29682,27601,27601,25521,35988,38100,31729,42294,29747,25554,27634,31828,29747,25521,33907,50712,31794,35987,50679,40213,29681,31795,29747,25521,38100,48599,40148,27569,31794,29747,27569,38035,48598,38067,29649,31794,29714,27568,27536,40148,48566,29615,25455,29714,27601,23309,42228,48566,29583,27535,27568,27535,25389,42259,25419,14919,19114,23370,17127,14950,17062,14950,12806,12774,8549,6437,6469,8581,10662,10662,25420,54906,57051,57019,44341,54938,50712,54970,54970,54970,54970,54938,54970,54938,46130,41580,41580,41580,37515,41902,41902,23016,41966,41934,41934,41934,41934,41934,41934,35723,21189,16997,12804,10691,12868,8579,8612,6499,14884,14726,14726,10724,23433,17063,25421,31729,10535,16972,6437,10697,12777,8518,4259,8517,8485,17002,14889,12807,23434,27564,25480,19238,23432,25481,25481,27563,25487,10631,8551,10631,8551,23341,31759,21288,19110,14948,12868,10755,14948,17092,12835,12835,14948,17061,17061,10724,12772,8579,10724,12804,12805,10692,12805,8548,8515,8580,8580,17061,171},
{54971,57052,59132,59164,59164,59164,59164,59196,61244,59196,59164,59164,57052,52891,52859,50811,50811,50779,48698,48698,59164,55004,44473,55004,52924,42392,42393,48698,36054,25587,23507,23475,25587,23475,21427,19347,21427,21395,25587,27668,25587,27699,27667,25587,25587,25587,25586,25587,23539,23506,23538,25586,25586,27666,27666,29746,27698,27666,27666,27666,27698,25585,23505,25553,23505,23505,21425,21425,25553,27601,25521,27601,29649,29649,25521,25553,25553,23440,33809,42294,23441,14990,17070,21361,21328,14989,23408,52760,25456,29681,48599,38067,19150,23409,21361,17037,33842,48566,31794,12909,23409,21361,12876,29616,48566,31762,21295,23441,21328,19182,21230,38002,48566,19019,12778,21328,17102,10632,38002,48599,23276,21196,19182,17068,14889,40114,35887,12773,12774,19175,17030,12805,12838,14886,14854,12774,10662,6436,6404,6436,8517,12774,23307,54938,57018,57019,48567,54938,50712,54970,54970,54938,54938,54938,54970,54938,52598,41613,41580,41580,37515,41934,41902,23015,39853,39821,37741,37741,37740,35660,35660,35692,23238,16996,17061,12868,10660,10660,10693,10692,8548,10565,6404,8612,23400,17032,25389,23309,14825,25422,8518,10664,10664,6438,2211,8485,8485,14825,12743,10694,21321,23434,19142,19238,25481,33870,29708,27466,12809,14857,12744,12744,10663,23341,23306,14980,17029,19174,17061,12868,14948,17092,19205,12867,17061,12868,10723,10659,12773,12804,14948,14980,14916,14917,10660,8515,10660,10660,12804,14981,112},
{54971,57052,57084,59196,59196,59164,59164,59196,59196,59164,59132,57084,54971,54971,52891,52891,50811,50779,48730,48698,48698,46650,48698,50811,48731,48698,48698,48698,42392,29781,21427,21427,23507,21427,19347,19347,23475,23507,25587,27667,25619,27635,27667,25619,25587,25554,25587,25587,23506,23538,25587,25586,23538,25618,27666,27698,27666,27666,27666,27666,27698,25585,23537,25585,25585,25585,23505,23505,27633,27601,25521,29681,27633,29681,27601,25553,23473,27568,38067,48600,42359,40213,40213,42391,42391,42294,44439,52792,46487,48599,54905,50712,44407,44471,44472,44439,50712,54905,50712,44439,46552,48632,46519,50744,54937,52792,48632,50712,48664,50680,50712,54873,54937,52760,50712,50745,50744,50712,54937,54938,52824,52825,52857,52857,54905,54937,40145,16998,17063,19175,12773,10693,8581,10693,14919,16999,10629,8517,8517,4356,8517,14887,21161,54938,57018,57019,48599,52857,50712,54938,54938,54938,54938,54938,54970,54938,57018,46033,41612,41580,37547,41934,41902,20967,14693,18822,16774,16806,20999,20999,16806,16774,19013,14916,17061,12899,12868,10724,8579,10692,6468,4291,4291,8611,21288,12871,25454,31695,21131,19083,8518,10631,8551,6437,2211,6405,6404,10598,8549,8516,19207,21352,14949,19206,21319,27594,38096,38032,6438,6405,4292,8486,10631,19083,23340,19144,14949,17029,19141,14949,14980,17093,14979,14948,17092,14980,14916,14885,10692,12805,10660,17029,17060,10692,10660,10724,12805,8580,10660,8580,32},
{54971,54971,57084,59164,61245,61245,59164,59196,59164,57084,59164,57084,54971,54971,54971,52891,50811,50778,48730,48698,42425,40312,48731,48698,44506,42393,48698,48698,44473,31861,23508,21427,23507,21427,21395,23507,23507,27668,27635,29748,25587,25619,27667,25587,25587,25587,25587,25586,23538,25587,25587,25587,23506,25618,27698,27698,27666,27666,27666,27666,27698,25585,23505,23505,25585,21425,23505,21425,23473,29714,25553,27601,27601,27633,27601,27633,25521,27601,38100,38132,38100,38132,36052,36019,36019,33971,38132,38100,40180,38100,38100,38132,38099,38067,35987,29746,31762,31762,29745,31793,29680,27599,25454,23406,31760,46453,44373,40147,44372,56985,56985,56985,40115,42195,46485,31663,54937,21099,42292,33841,54905,25291,23211,33776,33776,59131,33839,21257,19143,14918,14886,14886,10693,12774,12741,16999,10661,10662,8549,6436,8549,16968,21161,54938,57019,57051,46487,52857,50679,48631,46518,48567,48632,48599,48599,46518,48599,48437,41580,41613,37547,41934,41902,12614,6372,8453,6372,8485,10565,12645,8453,6371,14916,10659,12836,12835,15012,14949,10724,8579,10660,4291,4291,4387,17095,10694,19148,29583,12582,29421,14825,8551,8551,6405,2179,6405,6372,10598,8517,8517,17095,23401,16998,19206,19238,19270,19238,23306,6438,6405,4324,6372,10663,19083,19148,19082,17000,19142,17060,14980,10755,14948,19173,14980,17092,12835,17093,8579,8579,12773,10660,17061,17028,17061,14916,14917,12773,8548,8548,8580,36},
{54971,55004,57052,59164,61244,61245,61244,59196,57084,57052,57052,54939,54971,54971,54971,52891,52859,50778,48698,48698,48731,42425,46618,46618,46650,40313,42425,48698,46618,36054,25588,21427,23507,21427,21427,25555,23475,29748,27668,27635,27668,25587,27667,27667,25587,25587,25587,23506,23506,25587,25587,25587,25587,25586,27698,27698,27666,27666,29746,27666,25585,27665,23505,21425,25585,23505,23505,21425,19344,27634,27601,25553,25521,25553,25553,27633,25553,25553,21392,21393,23473,25585,25552,23472,23473,21328,27665,29681,31761,29713,29713,29745,29713,29681,27601,23408,25456,23441,19279,23472,21261,21293,19181,17100,23374,42228,42293,27567,40179,54936,54936,54936,38002,40114,46453,29550,54905,18985,44405,25422,42260,23211,21131,31663,31695,59131,42259,25449,21255,16998,12773,12741,10661,12774,10661,12774,14887,12806,12774,6437,8581,10693,14887,54938,57019,54970,46454,54906,46486,31696,27502,31664,35955,35955,33874,35889,35889,40147,41774,39532,37579,41934,41934,35660,37741,37741,37773,35693,37773,39821,37773,37772,23206,25351,23302,14948,14948,10660,8612,8516,6435,4291,2211,4291,4291,6437,16970,19051,12615,23244,19050,8551,8550,6405,4292,8486,10566,10630,8549,8549,14983,23433,17031,17158,19238,19238,19237,12806,6438,6405,4324,4324,10631,19083,19116,21196,21195,17093,14980,14980,17060,14980,14981,12803,12803,10723,14948,14948,14916,12772,12836,17029,12836,17029,17029,12804,8547,10693,8580,8580,36},
{57052,57052,57052,57084,59164,61277,61245,57084,55003,54971,52891,52891,54971,54971,54971,52891,52859,50811,48698,48698,48730,46618,44505,46650,48731,38232,42425,48698,42457,42425,27668,23508,21395,21427,25588,25587,29748,31829,27700,27668,29780,25587,25587,27667,25619,25586,25619,23506,23506,25587,25587,25587,23506,23538,27666,27698,27666,27666,27666,27666,25617,25617,23505,25586,25586,21425,23505,21425,19345,27633,27601,25553,25521,25553,25553,25585,29714,25553,19312,21360,21392,25553,25585,23504,23504,21392,29746,29713,31761,29713,29713,29713,29713,29713,27633,25521,29681,27666,19247,19279,21326,19181,17068,19213,27567,42260,44341,42293,40179,54904,54936,54936,48566,48598,52759,46420,54904,42163,48631,27599,21163,21131,21163,31663,31663,59131,48565,27564,23336,19143,16998,14886,12774,8549,12742,14886,12774,12774,14822,8549,16999,19111,21224,54938,54970,54938,48599,54905,46486,38067,33808,33841,40213,40213,38067,38034,38035,40180,48437,39758,39821,39886,37806,44112,44080,41805,39725,37773,35660,35692,35660,33611,31594,21158,16965,12804,12836,6435,6404,6436,8516,6436,4291,2211,4291,4323,8517,17004,8551,12712,10632,8518,8518,6405,6405,8518,8485,8517,6436,8549,14918,23432,17063,17125,19237,19238,19238,10662,6405,6405,4292,4292,10599,19050,10663,21196,19081,14916,10692,8579,14981,12804,12836,12837,8579,10724,12804,17029,14980,14916,14948,14916,16965,17029,14948,10692,8612,8612,12805,8612,40},
{57052,57052,57051,57052,59196,61277,61245,57052,54971,54939,52891,52891,54971,54971,54939,54939,54939,52859,50778,48698,48698,46618,44537,44537,46650,46586,48698,46650,36120,34007,31861,25587,21427,23507,25587,25587,27635,29781,27668,29748,31861,27700,23506,25587,25587,25587,25587,25587,25587,25587,25587,25587,23538,23538,25586,29746,27698,27666,27666,27666,25585,25585,23505,27666,23505,21425,21425,21425,21425,23505,25553,25553,25553,25553,25521,25553,27601,25553,19312,21393,21392,23505,23504,23504,23505,25584,31826,29713,29713,29713,29713,31761,44341,29745,27665,25521,27633,21328,21391,21359,21357,17068,17100,17101,27599,42260,44341,46518,40179,54936,56984,54936,38001,37969,48566,27437,54905,21099,46550,31792,44372,23243,21131,44340,52792,59131,46485,27499,19143,17063,12838,10661,8581,12774,10661,14886,16998,14886,14822,14854,10661,14982,17031,54970,57018,54938,46519,54906,48566,42293,40180,42261,44406,46487,46487,44406,44341,38002,50744,38067,21130,25357,29550,52857,48534,35307,18629,10599,14826,21098,21162,23305,21191,16997,10692,10692,10660,8516,6404,4323,6371,8516,6404,4323,4291,8516,4291,12809,10731,12779,6438,8518,8518,4324,2211,4324,6404,8517,8485,8550,12806,21352,19144,17061,17157,19238,19238,8550,4324,8485,4292,4292,6372,16937,6404,8453,14821,12804,10692,10692,14916,12836,14916,12836,8579,10660,8580,12804,17029,12804,10660,14917,12772,10660,10724,12868,12836,12836,12804,10692,40},
{57052,57052,57052,57084,59164,59132,57084,54971,54971,54971,54939,54939,54971,54971,54971,54971,54971,54971,52859,50778,48698,48698,44505,44538,38232,44505,46650,46618,36152,29846,25621,23508,21427,25555,25587,25555,25555,25587,27668,29748,29748,27667,23507,23475,25619,25586,25587,25587,25587,23538,25586,25587,23506,23538,25586,29746,29746,27666,27698,27666,25585,25585,25585,25618,23505,23505,23505,23505,25585,23473,25553,25553,25553,25553,25521,23473,25520,29714,21392,21392,21392,23504,23472,23472,23440,23504,31826,31859,29713,29713,29713,29713,33874,29745,27665,29779,25521,25520,25520,19214,19277,17132,15020,17068,21261,40180,40147,35954,33872,48566,48566,48566,31728,31663,40179,21163,42260,21099,31696,21163,46453,19018,16937,38034,48567,44373,35888,25418,19143,17062,14918,10693,12774,10693,10661,8549,10693,12773,10629,8549,10662,8581,19144,54970,57019,54938,35889,54905,44373,33874,46583,40212,29648,29648,27503,19018,27470,48632,50680,31663,18985,23276,33776,46453,40082,27244,14859,10664,14858,14824,14824,19015,14821,12773,12742,12741,10661,10629,8549,8517,8549,8517,6404,6404,6436,10629,8517,17003,10699,12811,6405,8518,8518,6372,4291,4324,10598,12678,8550,8550,10694,21352,21257,17030,17158,17190,19174,8551,4292,12711,4292,4324,6372,16937,6372,6372,12772,12804,14884,17029,17028,14916,12804,12835,14917,10756,10692,8515,8547,10724,14916,19141,14949,12804,14949,12868,14980,14917,12804,12773,41},
{54971,54971,57052,59132,57084,55003,54971,54971,55003,54971,54971,54971,54971,54971,57052,57052,57052,57051,54971,50811,50778,48698,46585,40312,42425,40345,42425,38232,29879,25653,25621,23507,23507,25555,25587,23507,25555,25555,27668,29748,27635,27668,25587,21427,25587,25619,25587,25587,23506,23538,25587,25587,23538,23538,23506,27666,29779,29746,27666,27698,29779,27666,25585,25618,25585,23537,23473,25553,42392,33972,27569,17069,17036,17004,17004,14955,17004,17004,14956,17036,19117,17036,12777,14890,19084,19116,12745,12777,12777,12745,12745,12745,12777,12777,12777,12745,12777,12777,12777,12777,14857,14858,14858,14858,14857,14857,14857,14890,16971,14858,14857,14857,14858,14858,14857,16906,14858,14890,16938,16938,14890,14890,16938,16938,16938,16938,14889,19112,21256,17062,14950,12838,12774,10693,10693,14886,8549,8549,8549,10629,10661,10693,12774,40179,40179,42292,38067,54905,50680,50745,50712,48632,50745,50745,48632,46519,50712,48632,50712,29615,21163,25389,29583,52792,46453,31631,8552,12744,14890,14791,16938,21161,12741,8517,12711,12710,10661,6436,8518,8550,14791,12743,6372,10598,8485,10630,12711,23276,12777,8518,8518,12711,12743,12711,12711,12711,14791,12711,12711,12711,10661,21320,21288,14918,17158,19238,14982,8551,4291,19050,4292,4324,4324,16937,6372,6372,12772,12772,12804,14916,14916,14916,14948,12868,10724,10692,10660,10660,8548,8579,12772,17029,14948,12772,10692,10691,12868,14917,10692,8580,41},
{54971,57052,57052,57052,57084,59164,55004,54971,54971,54971,54971,54971,54971,57051,57084,57084,57084,57052,54971,54939,50811,48698,46618,38200,31959,34039,36120,34072,34040,27701,27733,25587,25587,25588,25587,25555,25555,25555,27635,29748,29748,27668,25587,23507,25619,25587,25587,25587,23506,23538,25619,25587,23506,23506,23539,25586,31827,29746,27666,29746,31859,27634,25553,25586,25585,25585,25585,25585,21294,8551,31793,19116,10697,10665,10664,10664,10664,10664,10664,14890,14923,14890,10664,12777,16971,16971,10696,12745,16971,17003,12777,12745,16971,17003,14858,12744,14858,17003,14890,12745,14858,17003,16971,12744,12745,17003,17003,12777,12745,16971,17003,14858,12745,14890,17003,16938,12745,14858,17003,19051,12777,14825,19051,19051,14825,12777,19114,21224,19175,17094,17062,17062,12773,16998,17031,17031,12742,12742,14823,8549,10693,12806,10726,12806,14918,12806,29648,54938,44373,31696,27437,27470,33874,33874,33809,29550,33809,44341,50712,38035,21163,25422,29615,54938,46454,16938,14890,8518,14857,12743,14824,19050,10598,8485,12711,12710,10597,8517,6404,6405,14856,10630,8518,14857,8551,8518,14791,10631,8519,8486,6437,12743,14791,12711,12711,12711,14791,14791,14791,12743,10630,19208,23401,12806,17157,17157,14918,8518,4291,23243,10598,6372,4292,16937,6372,6340,4259,6371,12804,16996,17029,14948,17092,12867,10724,10692,14917,12772,12773,10660,8580,8579,17061,10755,14917,12804,12836,12772,10692,6435,38},
{54971,57052,57052,57052,57052,57051,55004,54971,54971,55004,54971,54971,57052,57052,57084,59132,57084,57084,57052,54971,50810,48698,46618,38232,29847,27767,25686,27734,34040,29814,25620,25587,27636,25587,25587,25555,25587,25587,27635,29748,25555,27668,27635,27667,25587,25587,25587,25619,23538,25586,25587,25587,25587,25587,23539,25554,29746,29778,29746,27666,29746,25553,27666,25585,27666,25553,25585,23473,21262,8551,10697,17003,14891,17003,17036,17036,17036,17036,17003,17036,19116,17036,17003,17004,19117,19117,16971,16971,19084,19084,16971,16971,19116,19116,17003,14890,17003,19084,17003,14890,16971,19116,19116,14858,14890,19084,19084,14890,14858,19084,19116,14923,12777,17003,19084,16971,12777,14890,21164,19116,12777,14857,19083,19083,14858,12745,19051,19176,17061,17062,10725,12837,14950,17094,19175,19111,14886,12806,12742,10694,12806,10725,10661,10661,8548,10662,38100,54938,46519,40180,35922,35922,42293,42261,38067,33841,40147,50712,48632,40115,27469,35921,42228,48566,40148,18985,19017,16937,10566,10597,12678,12679,10566,6405,10630,12678,10597,10597,8517,8485,14824,8485,6372,4324,4292,8485,12710,8517,8517,8485,8485,12744,12743,12743,12711,12711,14791,14791,14791,14824,8582,17128,23433,14886,17126,17158,12774,6438,4292,23308,16905,4324,4292,16937,6372,8388,2179,4259,12772,14884,14884,19141,19205,14980,12804,12804,17029,14949,17030,12740,14885,8611,17061,12836,14949,12868,14949,10692,6467,6468,30},
{57084,57084,57052,57052,57052,54971,54971,57052,57052,57052,57052,57051,57084,57084,57051,57084,59132,57084,57052,54971,52859,48698,46618,44538,29879,29847,27734,27734,27734,29814,27701,23475,25556,23507,25587,25555,25587,25555,25587,25587,27667,29780,27668,27668,25619,25587,25587,25619,25618,23538,25587,25587,23506,23539,23507,23538,25586,29778,29746,27698,27698,23505,27666,25553,27666,25585,25586,29746,21262,6471,10632,17036,19084,8519,6438,8518,6438,6470,6438,17036,21230,14955,6405,12744,19084,19084,6406,10632,19116,19117,8551,6438,19084,19116,12745,6438,16970,19116,14922,6438,12744,19116,19084,8518,10631,19116,19116,10664,12712,19051,19116,12777,8519,16970,19116,16971,10599,14825,21164,19116,10632,12777,19084,19084,12745,10664,17003,19115,19143,17062,12805,17062,12805,17062,19174,19111,16998,16998,16999,8581,10693,10789,12806,8613,8549,8613,29616,54937,48567,42228,40147,42228,44406,44406,44406,42293,46518,48567,33776,40148,31695,44373,50744,50711,50712,19017,19017,16937,12710,14791,14791,12711,10630,10598,12711,12711,10565,10565,10598,8485,12743,10598,6404,6404,8452,8485,12711,8485,8485,8485,8485,10630,12678,12678,10630,10630,12678,12678,12678,12711,10597,14982,23401,16967,17093,17158,10695,10631,10599,29582,21195,4292,6373,14856,6372,8388,4259,4259,8580,14885,14884,16996,17061,19205,14948,19141,17029,12836,17029,17029,10724,12868,14917,12804,10724,12772,14885,10692,10725,6467,40},
{59132,57084,57084,57084,57084,57052,54971,57051,57052,57084,57084,57084,57052,57084,54971,57051,57084,57084,57051,54971,50811,48698,46618,44538,34072,36120,36152,31959,29879,27701,31894,25588,25556,23475,23507,25587,25587,25587,25555,27668,25587,25555,27668,27667,27667,25619,27635,25587,25618,25587,25587,25587,23506,23539,23506,23506,25586,27698,29746,27698,27698,25553,29747,23473,27634,27633,27634,27666,19182,6470,10664,19084,19116,17003,17003,17003,17003,17003,17035,19116,19149,19117,17036,19116,19116,19116,19116,19116,19116,19116,19084,19116,19116,19116,19084,19116,19116,19116,19116,19116,19116,19116,19116,19116,19116,19084,19116,19084,19084,19116,21164,19084,17003,19084,19084,19084,17003,19051,21164,19116,16970,16971,19084,19084,16971,16938,19083,21164,17034,14951,17030,19175,14917,17030,17062,17063,16998,14950,14886,12806,10757,12902,10693,8581,6468,8581,29648,54905,42293,29583,29615,29583,29649,29648,27503,29551,35922,48599,25356,25357,23309,42260,50744,50679,52792,25356,21130,21130,21130,19049,19017,21130,21130,21098,16936,12646,12645,12645,12646,12646,12646,12613,12645,12646,10597,12645,10597,10565,10597,10597,10565,10565,10565,10597,10565,10565,10598,12710,14759,12710,25388,25453,29677,27533,29709,17126,23309,6437,6405,6372,16969,4292,8518,12711,6372,8388,6307,4291,4291,6371,12740,16996,14916,10691,12836,14916,17093,14980,14980,14916,12836,12868,17125,14981,14916,17029,19078,10660,8515,12805,44},
{57084,57084,57084,57084,59132,57052,57051,57084,57084,59132,59132,57052,57052,57052,54971,54971,57051,57051,54971,52891,50778,48698,46618,44538,42457,36119,44538,38232,36120,29814,36087,25588,23507,25555,25555,25555,25587,27635,27700,27635,25555,25555,31829,27668,27667,27667,25587,25587,25619,25619,25587,25587,23506,23538,23538,23506,23506,25586,31827,29746,27666,25585,29746,25585,27634,27634,29746,31794,21230,10632,12777,17003,19084,8584,8584,8583,8551,8551,8551,17004,19117,14891,8518,12777,19084,17003,6470,10664,19084,19116,8551,6438,19083,19116,12745,8519,14922,19116,16971,8519,12745,19116,19116,6438,10632,19116,19116,17003,12778,19051,21164,16970,10664,16970,19116,17003,14858,16970,21164,21164,14890,16971,19116,19116,16971,16971,17003,21164,16971,14889,23337,19144,19111,17030,12805,10693,17063,17063,16998,14918,12837,12902,10693,8613,6469,6404,27503,54905,50712,50745,50745,50713,50745,50713,50712,50712,52825,48567,25324,21131,23309,42260,52825,50712,50679,29582,21162,21130,21130,21130,19017,16937,16937,19017,18984,12646,10598,10598,10598,12646,12646,12678,10630,10630,10565,12645,12645,10598,10598,10598,10565,10565,10565,10565,10565,10565,10598,10598,12646,12678,29614,17001,21257,16968,21225,14982,25389,6372,4292,4259,8453,6405,12711,8518,6373,8388,6339,6340,6307,4259,8516,16965,19109,16996,12804,12804,17060,17092,17092,14852,10692,10724,19205,17093,14948,12772,10660,16997,10628,14949,45},
{57084,57084,59132,59164,59164,57084,57051,57084,57084,57052,57052,54971,54971,54971,54971,52891,54939,54971,52891,50811,48698,46618,46618,44538,44538,38232,40345,38232,34039,27733,33974,27636,23475,25588,25555,25555,25555,25587,25555,25555,25587,29748,31861,29780,25587,27667,25587,25619,25619,25587,25587,25587,23506,25586,23538,23538,23539,23506,29746,29778,27666,25618,25585,31859,29746,29714,40246,33907,46617,10665,14890,19084,19116,12777,12777,12777,12777,12745,12745,19117,21229,17036,12777,16971,19116,19084,14858,16971,21197,21197,14891,14890,19116,19117,17003,17003,19116,21197,19116,17003,19084,19116,19116,16971,17003,19149,19148,19116,19084,19116,21164,19116,19116,19116,21196,19116,19116,19116,21196,21164,21164,19116,19116,21164,19116,19116,19084,19116,19083,19084,19114,19112,21256,19111,17030,17030,19143,17031,17062,12837,12901,12869,10757,10725,8516,6469,29615,52857,44341,31728,31728,31729,33874,33874,33809,31728,40147,46486,23243,21163,25389,42292,52825,52792,52792,35921,21162,21130,21130,21130,12743,6436,8517,12743,19017,12646,8549,8517,12743,12678,12646,12711,12841,10760,10598,12645,10597,12776,12905,12841,10630,10565,10565,10565,19114,19180,12744,12646,12645,10533,25421,21226,21320,23370,23306,17093,23309,6372,4324,4291,4259,10630,12744,8485,6373,6339,8388,6372,6339,6307,4259,8484,29545,21189,14884,14884,17093,14980,17092,16997,10692,10660,12835,17094,14884,10692,10660,10724,10660,10692,42},
{57084,59132,59164,59196,59196,59196,57052,57052,54971,54971,55004,55004,54971,54971,54939,52891,52891,52891,52891,50778,48698,46650,46618,44538,44538,42425,36119,42425,36152,36120,33974,25587,23475,25588,25587,25555,25555,25588,23507,23475,23475,23507,25587,27667,25587,25619,25619,25619,25619,25587,25586,25586,23538,25619,25618,23538,23538,23506,27666,31826,27698,27666,25585,27634,29714,40245,44406,33907,33907,6406,21196,38067,38100,35955,29680,23310,17004,14890,14890,17004,17036,14891,12745,14890,17003,16971,10697,12745,12745,10697,10664,10664,10664,10664,12745,12745,10696,10664,10664,10664,12745,10632,10632,10632,8551,8584,8552,10664,12777,10664,8551,10632,12712,10664,10664,12745,12745,10632,12745,12745,10664,12745,12745,10664,12777,12777,10632,10632,10664,8583,10631,10599,10630,12775,14952,14950,19111,10662,14952,12902,14982,14982,10757,19175,14886,8549,27502,54905,46486,40180,42293,40212,40213,40213,38067,35954,42261,40115,21131,19083,23277,40179,50679,48598,50711,40115,23210,21130,21130,21130,16936,12743,12743,16904,19017,12645,10597,10630,19050,12678,12645,12646,10663,10663,10630,10597,10597,10630,12743,12743,12678,10565,10565,10565,14823,14888,14856,12678,10598,10597,16969,12775,27628,27596,19145,17094,23276,6372,4324,4292,4259,10630,12744,8485,6372,6340,6372,6372,6339,6307,4259,6372,39885,29513,19142,19110,12836,14948,17028,10659,12773,12836,12868,8611,10660,10692,8579,12804,12869,8547,37},
{59132,59164,59196,61277,61245,59197,57084,57052,54971,55003,57084,57052,54971,54971,54971,54971,52891,52891,52891,50810,48698,46650,46618,44570,44538,44538,27734,27734,29814,29846,27669,23508,23508,25556,25587,23507,23507,23507,21395,21395,21427,23508,25587,23475,21426,25587,25619,25619,25619,25619,25618,25618,23538,25619,25618,23538,23506,23538,23506,31859,29746,27666,27666,29714,40213,44406,44439,29713,21294,6471,14890,27503,19084,12745,10664,10632,8584,8551,6471,19149,19182,12810,6405,14858,19084,16971,6405,6438,8519,8519,6438,6438,8518,6470,8551,10632,8551,10632,8583,8552,10665,10664,8551,8551,8551,10631,10632,12777,14890,10665,10632,10632,14858,12745,10664,14858,14858,10664,14890,14858,12745,14890,16938,14890,14858,14890,14857,14890,14890,12777,14857,14857,14857,14890,14890,14857,19113,10694,12807,14950,17094,12837,12837,14918,12742,8549,29647,52857,46486,42261,42261,42293,44406,44406,44374,44373,48599,48599,23244,21163,23276,44373,52824,52792,52824,44340,23210,21130,21130,21130,21130,21130,21130,21130,19049,12645,12645,12645,12645,12645,12645,10597,10597,12645,10597,10597,10597,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,12678,12678,12678,21163,10662,21288,23402,21226,17094,21195,6372,6372,6372,4292,10598,14857,6372,4292,6340,6372,6372,8420,8355,4227,6405,35692,33673,27496,23367,19109,19173,12803,17030,17061,12900,10691,12804,8611,12804,14884,12868,12804,8580,43},
{59164,59164,61244,61277,61245,61245,57084,57052,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52859,50810,48698,48698,46618,46618,46618,44538,31927,25654,27767,27734,36086,23508,23475,23475,23507,23475,23475,23475,23475,23508,21395,21427,23507,23507,23539,23506,25619,25619,25619,25619,25586,25618,25587,25618,27698,25618,23506,23538,23538,27666,29778,27666,29746,38133,44406,44438,46487,31762,21230,8519,14890,10664,8519,8551,12777,21229,16971,14923,14923,17003,17036,16971,14923,17003,19084,17003,17003,16971,17003,16971,17003,16971,16971,16971,16971,14922,14922,14890,14890,14890,14890,14890,14890,14858,14858,14858,14890,14890,14890,14890,14858,14890,14890,14890,14890,14890,14890,14890,14890,14890,14858,14890,14890,14858,14858,14858,14857,14857,12809,12777,12777,12777,12745,12745,12776,12743,12743,8581,8549,17062,14981,17030,12805,10661,10629,8581,25453,52857,40180,27470,27502,25390,29648,29616,25390,21163,33841,46519,23211,19083,23244,44373,50744,52792,54873,46486,21162,21130,21130,19049,16936,21129,21130,19049,19049,10597,12645,12645,12645,10597,10597,10565,10565,10597,10565,10565,10565,10565,10565,10565,10565,10565,10565,10565,12711,17002,12744,12678,10597,10565,14824,12710,19175,23401,19113,19207,19082,4324,6372,6405,6372,10630,14857,4292,4292,6372,6372,14759,14727,14694,12646,12678,33580,31625,25448,25448,21254,19173,17028,16964,21319,17093,17093,19174,14949,14853,17093,12868,12804,10660,46},
{59164,61244,61245,61244,61244,59196,57084,57084,54971,54971,54971,57052,54971,57052,54971,54971,52891,52859,50811,50779,50746,48698,48698,46618,46618,46618,36184,25654,29847,36152,46585,25587,23507,23507,23507,23475,21427,23475,25588,21395,21395,21395,25587,23539,23539,25587,27667,25619,25618,25619,25618,25586,25586,25618,27699,27666,25586,25586,23538,25586,29778,27698,27666,42358,44439,46519,46519,27601,19149,6470,14890,8584,6470,6438,8518,12745,12745,10665,10632,19182,21262,12810,6406,14890,19116,14923,6406,6471,8519,6470,6470,8551,8551,8519,8551,8551,10632,8551,8551,8551,10631,8551,8519,8519,8519,8519,8551,12778,12778,10632,8519,10664,12745,10697,10664,12777,12810,12777,14825,12777,12777,14858,14858,10664,14858,14858,12745,12777,14858,14858,14858,14858,12744,14921,14855,10661,10662,12806,12870,17062,14950,12805,10661,12807,8581,6468,27534,52857,48631,48632,48632,48632,50712,48664,48632,48599,50745,46551,23211,21131,23309,44373,52792,50712,48567,48631,23210,21162,21130,21130,21130,21130,19050,12711,10598,8485,10565,10597,10565,8452,6404,8485,10565,10597,8485,6372,6404,8485,10565,10565,6404,6372,8452,10565,12710,17002,16969,12743,10597,10565,8517,17001,19177,23433,19081,23369,16970,6372,6372,6405,6405,12678,14857,4292,6405,8518,10598,31630,31662,33743,33743,21098,29386,31593,25416,25481,21286,21287,21285,25447,21255,14981,14980,12836,17030,14853,14916,10692,12868,14949,45},
{59164,59196,61244,61244,59196,59196,57084,59132,57052,54971,55003,57052,57052,57051,57052,54971,52859,50811,50811,50778,50778,50778,48730,48698,48698,46618,42458,27766,27766,34007,40279,27700,23507,21427,23475,23475,21427,29781,23475,21427,21395,23475,23507,23507,25587,25587,27667,25619,25619,25619,25619,25586,25586,25586,27699,27666,25618,25586,25586,25586,27666,29746,27601,46519,46519,46487,44439,27568,19117,6438,14890,8551,6471,6438,8519,10664,10697,12745,12745,19117,19117,14923,12777,17003,19116,17036,14890,14890,14890,14890,14890,14890,14891,14890,16971,16971,16971,16938,16971,16971,16971,16971,16971,16971,16971,16971,16971,16971,19052,17003,16971,17003,19051,17035,19084,19051,19084,19084,19083,19083,17003,19083,19051,17003,17003,17003,16971,16971,16971,16970,16970,16970,17000,12838,10725,10693,10694,12838,14917,17030,12837,12741,10661,10661,8581,8549,27533,54938,42260,31728,33776,33809,35954,35955,33809,33777,42260,46486,23243,21164,27503,48599,52857,48599,46486,52825,23210,21130,21130,21130,21130,16936,10663,8550,8485,8453,10565,12711,25357,12744,8518,8485,10597,8485,6405,6372,6372,8485,14856,12776,10631,8550,6404,10565,10565,10565,10565,12678,12678,12679,12678,29615,27565,31823,31759,31758,14857,6372,6372,6405,8485,12711,14857,4292,8518,10664,14792,37969,40081,38001,40114,27437,27306,29545,25416,25480,21287,21254,23333,29640,23401,17125,12836,10724,25450,16998,14948,10724,17061,17060,58},
{59164,59164,59164,59164,57084,59164,57084,57084,57084,54971,54971,57052,57084,57084,57052,54971,50811,50778,50778,50778,50778,52859,50811,50778,48698,46618,42490,38265,40313,36119,40279,34006,21427,21427,23507,25587,25587,25555,25555,23475,21395,21395,21395,25587,27667,27667,27667,27667,25619,25587,25587,25587,25586,25586,25618,27699,27666,25618,25618,25586,25586,29778,29746,46552,46519,44439,44439,25488,19116,6470,14923,8551,8551,6470,12777,21197,14890,12778,12778,17036,17036,12777,8583,14890,17003,14858,10600,10632,10632,8551,8551,10697,10664,8552,10664,10665,10697,10664,12777,12777,12777,10632,8519,8519,8519,6471,6438,10632,12777,8584,6470,10632,10664,10632,12745,12745,12745,10664,12745,12745,10632,12745,12777,10632,12777,12777,10697,12744,12745,10632,12745,14856,12838,12773,8613,6468,6468,12838,17094,19174,19175,12773,12773,10693,12806,8581,29645,54938,50744,54938,54938,54938,54938,54938,54970,54938,57018,46486,44373,50712,54938,54938,54905,50744,48632,57018,23210,21130,21130,21130,21130,16904,6469,6437,6405,6404,10565,10598,12711,6405,6405,8452,10597,6437,6405,6372,6372,8485,8517,8517,6405,6405,6372,10565,10565,10565,10565,12678,10565,10565,10565,23243,17000,23369,23306,27565,12744,6372,6372,6372,10566,14791,16905,4292,6437,8550,12711,29582,35856,33775,33743,27404,25258,27464,25448,25480,21287,19206,25445,29607,25481,19206,19173,17030,21257,21288,21255,12772,14948,17092,52},
{55003,57084,57084,57084,57051,57084,57052,57052,55003,54971,54971,57051,57052,57084,57051,54971,52859,50811,50778,50778,50778,52859,52859,50778,48730,46618,44506,34072,34039,36152,34006,38199,23507,21395,25587,25587,25555,23507,21395,23507,21427,21395,23507,29780,29780,25555,25587,27667,27667,25619,25587,25587,25586,25586,25586,25586,27699,25618,25618,25618,23538,29746,31826,46552,46519,44438,44439,25455,17036,8519,17003,6503,6471,6438,6471,8519,8519,8519,8551,21230,21230,12777,6438,16971,19084,14891,10632,12713,10632,10664,16971,17003,14890,12777,14858,14922,14858,12777,14890,17036,14923,12777,10664,10664,10664,12745,12778,16971,16971,14858,12777,14922,14922,14890,16971,16971,17003,17003,16971,16971,17003,17003,17003,17003,17003,17003,17003,17003,16971,16971,16970,16968,14951,10693,10693,8613,8549,8613,17094,19207,17062,14886,12806,14886,17030,17062,25451,54938,52792,54938,54937,54938,54938,54938,54970,54938,54938,48631,54905,54937,54937,54937,52825,52792,44341,59164,21162,21130,21130,21130,21130,19049,16969,16937,16937,14791,14759,14791,12743,12710,12710,12710,14791,12743,12711,12711,12711,12711,12711,12711,12711,12711,12710,12743,12711,12711,12711,14791,12711,12711,12679,21195,19113,25482,25450,27502,10631,6372,6372,4292,16904,14792,14824,4292,4324,10663,12744,14825,16970,16937,19050,16970,23210,25351,25416,23400,21287,21254,25446,29609,23369,23400,21286,17029,23339,12837,14885,14917,14948,15012,67},
{55003,54971,54971,54971,57052,57052,57052,54971,54971,55003,55004,54971,54971,54971,54971,54971,52891,52891,50811,50778,50778,50778,52859,50778,48698,46618,44538,38233,31959,29847,34040,34007,23507,23475,25587,25555,23475,23475,21395,23507,23508,21395,25587,29748,29748,25587,25587,25587,27667,27667,25619,25587,25587,25586,25587,25586,27666,27698,25618,25618,23538,27666,31794,48632,44439,44438,44439,25455,17004,6470,16971,6438,6470,6438,14890,21262,17003,17003,16971,17004,17036,14923,14890,17003,19084,16971,14858,14890,14890,16971,16971,16971,16971,14890,14890,14923,14890,14858,14922,14923,14858,12777,12777,12777,12777,12777,12777,12810,14858,12777,12745,12777,14857,12777,12809,14857,14858,12777,12777,12777,12777,14826,12777,12745,12777,12777,12745,12745,12745,12777,14888,12838,10725,10757,10757,10725,8581,14918,19144,14982,14917,19111,17063,19207,17095,17034,33872,54970,52825,57018,54938,54938,54938,54970,54970,54970,54970,54938,54938,54937,54937,54937,52825,52792,50647,61309,21162,21130,19049,19049,19049,19050,19082,19049,19049,19050,19017,19050,19049,19049,19017,19017,16969,19017,19017,19017,16969,16936,16937,16937,16937,16937,16936,16969,16937,16937,16937,14889,16937,16937,16937,14824,12807,23434,25483,27502,12711,10631,12711,12711,33808,40179,14824,4292,4292,10631,12744,25389,29614,16937,19083,19083,29582,23239,25416,21287,23368,21286,25446,29608,23402,23400,17093,12837,12804,8612,12837,14980,14980,12803,63},
{57052,57052,57052,57052,57052,57052,57052,57052,57052,57052,57052,54971,54971,54971,54939,52891,52891,52891,52859,50778,50778,50778,50778,48698,48698,46618,44506,42458,36152,31959,29847,25621,23475,23507,25555,23507,23507,21395,21427,23475,21395,25555,25587,27668,27667,29780,27667,25555,25587,27667,25586,27667,25586,25587,25587,25586,25586,27698,25618,27666,25586,25586,31827,46519,44439,44438,44439,23375,14955,6438,17004,6438,6470,6438,8551,10664,10664,8584,10665,21230,21230,10697,6438,17003,19084,14890,8518,8519,8519,14890,19116,17003,16938,12745,12713,12778,10664,8552,12745,12778,8584,8519,6438,6438,6438,6438,6438,10664,12778,10664,6438,8519,12745,10632,8519,10664,12777,10697,12777,12777,10632,12777,14890,12777,14826,14890,12777,14825,12777,10664,12808,12838,12805,12806,10693,19112,19144,19175,21257,21257,21224,14918,19142,19110,23370,23375,27567,38066,27469,29582,27501,27533,27501,27469,25356,25388,25389,38002,48599,38002,29615,29582,31727,42291,50712,65471,23243,21130,21130,21098,19050,14888,14888,12776,16937,19049,19049,12776,12808,12711,19049,19017,16936,10662,10662,12711,16969,16969,12743,10695,10631,14824,16937,16969,16969,16937,16937,14888,16937,14824,14824,14824,8548,17127,23434,29647,12711,4292,4292,4259,44340,48599,19018,4292,4291,10631,10631,31695,35921,14857,19050,19083,33840,27433,25448,21254,23400,21286,25478,29576,23401,21287,21287,14948,10755,8611,12868,19174,12835,17029,58},
{57052,57052,57052,57052,57052,57052,57084,57052,57051,57052,55003,54971,54971,52891,52891,52891,52891,52891,52891,52859,50811,50778,48698,48698,46618,44538,42458,42458,38265,27767,27766,29846,25555,25555,25556,23507,21395,23475,23475,21427,23507,27668,25555,29748,31828,31828,29716,25587,25555,27667,27667,25587,25586,25586,25587,25587,23506,23538,27698,25618,27698,25586,31859,46519,44439,44406,44406,23342,14923,6438,17004,6470,6471,6438,8519,10631,8551,14858,14890,19117,19117,14923,14890,19084,19117,17036,17003,17003,17003,19084,21197,19116,19084,19084,17003,19116,19084,19084,19084,19116,19116,19116,17036,19084,19083,19084,19116,19116,19149,19116,19116,19116,19116,19116,19116,19116,19116,21164,21164,21196,19116,19116,19084,19116,19084,19084,19083,21164,19116,19051,17034,17031,14950,12805,8581,12774,23336,17062,19144,23305,23337,21223,19174,19175,23339,21262,14921,12741,8549,10597,10629,10629,8549,6436,6404,8550,8517,16969,52793,42195,16937,16967,19176,35953,52825,65503,25356,21130,21130,19049,19082,14953,12937,10760,16937,21130,19049,12872,12905,12808,19049,19049,14824,8549,8549,10630,19017,19017,12808,10792,10695,14824,16936,16969,16969,16937,16937,14889,16937,14824,14824,14824,8516,17095,21352,27534,8517,4292,4292,4292,6372,14824,14792,4292,4291,8518,8550,8550,14857,14857,17002,19050,35920,29579,25448,21254,21320,19206,25478,27496,23401,23433,21288,12772,23336,17060,17093,12836,14916,19111,73},
{57052,57052,55003,57052,57052,57052,57052,57084,57052,55003,54971,54971,54939,52891,52891,52891,52891,54939,54971,54971,52891,50778,48698,48698,46618,44538,42458,42457,40377,38265,40345,40344,25588,23507,23507,21395,21427,23507,21395,21427,27668,23507,27635,31860,31861,31828,31860,25555,23507,25619,25618,25619,25587,25587,25586,25587,25587,23538,25618,27666,27698,25586,31827,44471,44438,44406,42326,23342,14891,6470,17036,6471,8551,6470,10632,19084,19084,10632,12745,19117,19117,12777,8583,16971,19084,14890,10632,14858,17036,19084,12777,16971,19084,19116,14890,12777,19052,19084,16971,12777,16938,19116,19084,8551,10632,19084,19084,14923,14858,17003,19116,19083,10664,17003,19116,19084,14890,14890,19116,21196,14858,14858,19084,19116,17003,14891,16970,21164,19084,16971,16971,21226,19175,14950,10693,17030,21255,14917,19111,19111,17030,19175,21255,19111,23339,19182,14954,14854,10661,12742,12742,8517,8549,6437,6436,8517,6436,12744,42293,50744,21163,19048,21225,48598,65503,65535,29549,21130,21130,21098,21130,17001,17033,17002,19049,19050,19049,17001,17001,17001,19017,19017,19017,16936,14856,14856,16969,19017,16937,14856,14856,14856,16904,16969,16969,16937,14889,14856,16937,16969,17002,16969,8549,17063,21320,29646,8485,4292,4292,4292,6372,16937,10630,4292,4291,10599,8518,8550,12808,12776,12744,14857,31727,27531,25416,19206,23401,21286,27526,27496,23433,25481,21256,23337,23305,12836,12868,12836,14948,8580,61},
{54971,54971,57052,57084,57084,57052,57052,57084,57052,54971,54971,54971,54939,54971,54971,54939,54971,54971,57051,55003,52891,50811,50778,48698,46618,44538,42458,42458,42458,36152,27734,25653,21395,23475,25587,21427,25588,21395,23475,23475,27667,27668,29780,33909,33941,31861,29780,25555,23475,27667,25586,25586,25586,25586,23538,25587,25587,25587,25586,27666,27666,27666,31859,44471,44439,44406,42294,21262,12842,6438,17036,6438,6470,6438,12745,19084,19084,10632,12777,19149,21197,12777,10664,16971,19116,14891,10664,16939,19117,19116,12745,17003,19116,19117,17003,16971,19116,21165,19084,17003,19084,21197,19116,12777,14890,19149,19117,21197,17003,19084,21164,19116,16971,19116,21164,21164,19116,19116,21164,21196,19116,19116,21164,21164,19116,19116,19084,21164,21164,19116,21196,21225,19143,17031,12837,19142,16998,19111,16967,21257,17030,21255,21255,16966,23339,21295,14954,12742,8581,10662,10661,10629,8517,6437,8550,6437,6404,6404,10598,10630,14824,19050,27501,48599,65535,65535,31695,21130,21130,21098,21098,12743,8517,8517,12743,19050,19050,8549,8517,8485,16937,19049,16936,6372,6372,8517,16969,19017,12711,4324,6405,10630,16936,16937,16969,16937,12743,12711,14824,16904,14824,14856,8549,14983,19271,29678,6437,4292,4324,6372,6404,16970,10598,4292,4292,10631,8550,10663,16970,12777,10631,12744,27468,33805,23304,17094,23369,19141,27558,25448,23433,23433,23401,23370,27498,12836,12836,14949,10660,14918,82},
{54971,54971,57052,59164,57084,57052,57052,57052,55004,54971,54971,54971,54971,54971,54971,54971,54971,57052,57052,57051,54971,52859,50811,48698,46618,44538,42458,42458,42458,34040,27734,25588,21427,25555,27636,27668,25587,31829,25588,27668,29748,29748,31860,36021,33941,33941,29748,27700,23507,25587,25586,25586,25586,25586,23538,25587,25555,25555,23538,25586,27698,27666,31891,46519,44439,44439,42326,23343,12810,6438,17036,6438,6471,6438,12777,19084,19084,16971,17003,19084,19084,16971,14890,19084,19116,17003,14890,17003,19084,19084,14858,14858,19116,19117,14891,16971,19084,19116,19084,17003,17003,19117,19084,12745,14858,19084,19084,14891,17003,19083,19116,14890,14890,16971,19084,19084,14858,12777,19116,19116,14858,16970,19083,19116,17003,14858,14858,21164,19084,17002,19113,14919,16999,21224,27563,27595,31758,25451,19112,23338,19143,23272,21257,19145,21227,21262,14890,12776,10663,10663,12808,10663,10663,10631,10663,8583,8551,10631,12777,8551,8518,16937,31695,50712,65535,65535,35888,21130,21130,21097,19049,14823,8549,8550,12743,19049,19049,10630,8517,10598,16936,19049,16969,8485,8517,10598,16969,19017,14823,8517,8518,10630,16937,16937,16937,16937,10630,8485,12711,14856,14824,14791,10598,12870,21320,27597,8485,4292,4324,6405,8453,19050,12743,4259,4292,10663,8550,10598,14857,10664,12744,12712,29614,31757,10660,12773,10660,10660,25382,25417,21257,23369,21257,23274,29580,14981,12836,10724,35951,38000,124},
{54971,54971,57051,59164,57052,57051,57052,57052,57052,54971,54971,54971,54971,57052,55003,54971,54971,57051,57051,54971,54971,52891,52891,50778,48698,44538,42458,42458,42458,34072,27734,25556,25588,27668,27668,29748,31861,31828,29780,25587,27667,27667,33941,36021,36053,31860,29748,25587,23507,25587,27667,25586,25586,25586,25586,25586,25587,25587,23506,23506,27666,27698,33972,46551,46519,46551,40246,21295,12777,6438,17004,6438,6470,6438,12777,19084,17003,6438,10664,21230,21262,10632,6406,16971,19116,12777,6438,14858,19116,19084,6438,8551,19084,19116,10664,12777,19116,19116,19084,16971,17003,19116,19084,8551,10664,19116,19116,14858,17003,17003,19116,17003,14890,16971,19116,19116,14858,14858,19116,21164,14890,16971,19116,19116,19083,16971,16971,19116,19115,19112,19144,17000,14887,14887,27563,31822,23337,19111,21224,16998,16998,21257,29680,38100,33874,25456,19149,17004,17004,14956,14923,14891,14891,14891,14891,12810,12778,14889,12774,12807,8518,12712,29581,54905,65535,50712,27437,19017,16937,16936,16936,16936,16936,16936,16936,16904,16904,16904,16904,14856,14856,14824,14823,14823,14823,14824,14823,14823,14791,14791,14791,12743,12743,12743,14823,12743,12743,12743,12744,14824,14824,14824,10663,12806,21288,25517,8485,4292,6372,8485,8517,21163,16905,4259,4292,10663,8518,10599,10599,10599,10631,12774,14886,14886,8548,6468,6468,4323,10596,14886,14951,23434,21257,23274,29613,8580,14980,12741,42225,37968,124},
{54971,54971,55003,57052,54971,54971,57052,57052,55004,54971,54971,54971,54971,57052,57084,57051,57052,57051,57051,54971,54971,54971,52891,50810,48698,44538,42458,42457,42458,34072,27734,25556,25587,27635,27635,27667,31828,33973,33941,27635,27667,29747,27667,29748,29748,29747,29748,27667,25587,25587,25587,25586,25586,25586,25586,23506,25586,25586,25586,23538,25586,27699,36052,48632,48632,48632,40245,27633,17069,12778,27633,12810,12875,12843,25423,25455,23342,17003,19051,19117,19116,19084,19116,19116,19116,19116,19116,19116,19117,19117,19116,19116,19116,19116,19116,19116,19084,19116,19116,19084,19084,19084,17036,19084,19084,19084,19084,17003,17035,17003,17003,17003,17003,16971,16971,16971,16971,16971,14923,14922,16971,16971,16938,16938,14890,14890,14890,16938,21226,19112,19111,17031,12775,12774,14982,14950,21287,17030,17030,19143,21192,25452,23277,23277,19116,16971,14923,8551,8551,8551,8519,8519,8519,8519,8519,8519,8518,10630,12837,10661,8582,10630,27533,54970,65535,33776,16904,16904,14823,14823,14823,14791,14791,14823,10598,10598,12743,12710,12711,12711,8517,10598,12743,10630,12710,10598,8485,10630,10630,10598,12678,10598,10598,12743,14856,14824,14824,14824,14824,14824,14792,12711,10630,10661,19239,25485,8485,4292,4292,8485,12678,23243,19017,10533,10436,14791,10598,12711,10663,12743,14886,14853,10660,10693,4323,6436,4291,4323,8516,8516,17031,23401,19208,23274,29612,10661,12837,10629,37966,38001,126},
{54971,55003,55004,55004,54971,54971,54971,57052,54971,54971,54971,54971,55004,57052,57052,57052,57052,57051,57051,57052,55003,54971,52891,52859,48698,44538,42458,42458,42458,34040,27734,25587,25587,27636,25587,25587,27667,27667,27635,25587,29747,27699,27667,27667,27699,29747,27667,27667,27667,27635,27667,25586,25586,25586,25586,25586,25554,25586,25586,25586,25586,27666,38165,50712,48664,48632,46552,38166,23375,14924,29747,12811,14956,14891,35954,46454,46454,50745,44406,19149,19117,8583,8519,8551,8519,8519,6438,8519,8518,8519,6438,6438,6470,6438,8518,10631,10632,10632,10665,12745,12778,14858,6470,6438,6438,6438,6406,10664,14858,10632,6470,12745,12777,10664,10697,12745,12810,10697,12777,12810,12777,14858,14858,12777,12810,14890,12777,14857,19112,19144,19111,17063,12774,12806,17063,12837,17062,14918,12837,17030,19079,31725,19084,19116,19084,14923,17004,17003,17004,17004,17004,14955,14955,14923,14891,14891,12810,14921,14918,12838,14887,14855,12775,52824,65535,35921,16904,16872,14824,14823,14791,12678,6404,8517,6437,6437,12711,8517,6437,6437,6437,6437,12775,6372,6404,6405,6437,10631,12710,6372,6405,6405,4324,12776,16969,16937,16937,14889,14857,16905,14824,14792,12710,8580,17158,25452,8485,4292,4292,8485,16904,19050,19017,25259,16548,20903,16743,18823,18823,16871,14820,14916,12804,8548,4323,8581,10628,6435,6435,6436,14919,23401,19176,23274,27564,10661,10693,10662,35886,38001,126},
{54971,57052,55004,54971,54971,54972,54971,57052,54971,54971,54971,54971,57052,57052,57052,57051,57051,57052,57052,57051,57051,54971,52891,52859,48730,44538,42458,42458,40378,29879,27766,27668,25587,25587,25555,25555,25587,25587,27635,25587,27668,27667,27667,25619,25587,27699,27667,27667,27667,25555,27635,25586,25586,25586,25586,25554,25586,25586,25586,25586,25586,25586,40246,50745,48664,50745,46552,38165,23343,12811,29747,8650,12843,10698,38003,50679,46487,12746,16971,21262,23343,8584,8519,8551,6471,8551,6470,8551,8551,8583,6438,8551,8551,8551,8551,8583,12777,10664,10665,8552,10697,14890,10664,8519,10600,10664,8584,10697,10697,14858,8551,10696,10632,10697,10664,12777,10632,10632,10665,10664,10632,8551,14890,10664,12777,8551,10664,12745,19113,19144,21225,25450,31791,33872,29678,33904,21256,16998,12838,16999,19112,33871,38067,29648,17036,14923,17004,17004,17036,17004,17004,17004,14923,14923,14891,14891,14890,12839,8613,8548,17000,14920,16968,52825,65535,38034,16904,16872,14824,14823,14823,12711,6404,8485,6437,6437,12711,10598,8485,8517,8550,10631,14823,8517,8517,8517,10631,10631,12743,8485,8517,8517,8518,14824,16937,16937,16937,16937,14856,16905,16905,14824,14824,8581,17127,23340,8485,4291,4291,10565,19050,18985,18985,21066,8388,10566,12711,16870,16933,19014,14916,12772,10660,14917,12740,14885,8548,10628,12741,6436,10693,23402,19176,23241,27564,10661,8517,10630,35950,33743,117},
{54972,55004,54971,54971,54972,57052,57084,57084,57052,57052,54971,55003,55004,57052,57052,57051,57052,57084,57052,54939,54971,52891,52891,52859,48730,44538,42458,42458,38265,29814,34007,31861,27668,27668,25555,25555,25587,23507,25555,25555,25555,25587,27635,27667,27667,27635,31828,27667,29748,27667,27667,27667,25586,25586,25586,25586,25586,25586,25586,25618,25586,25586,42326,50745,50778,50745,46584,38165,19150,6472,31827,10730,12843,10698,33842,50679,46519,14859,27536,46585,46617,19117,14891,19150,14923,14989,12843,17068,17036,17069,10698,14924,17069,12875,14956,12810,21230,12875,14988,10698,14955,17068,17036,12810,17004,17037,14923,14923,10664,21229,12778,14923,10664,12811,14923,12810,10631,12744,14890,12777,14890,10696,17036,12777,14890,8551,12777,14923,27567,31759,42323,46517,52856,54970,50744,57050,27499,21224,23305,19111,16999,33839,52825,40180,19116,14923,17004,14955,17036,17004,14924,14923,14923,14923,14922,14890,14920,10661,10693,6436,10630,17000,14887,44405,57018,29550,16904,14824,14824,14823,14823,14823,19049,19017,19017,16969,19017,16904,19017,19017,16937,16969,16969,16904,16969,16936,16937,16937,16936,14856,16937,14856,16936,16936,16969,16937,16937,14856,21163,19050,16905,14824,14824,8549,17094,23340,8518,8485,8518,14824,25356,16937,23243,27404,16548,16677,21030,12772,10628,14821,21157,14852,14852,10628,10692,8516,6468,6435,8516,8548,17032,21257,19176,23241,27563,8581,8517,12710,33838,33775,118},
{54972,54971,54971,54971,57052,59132,59164,59132,57084,57052,57052,57051,57052,57052,57052,57052,57084,57084,57052,52891,52891,52891,52859,50811,48698,44538,42458,42458,31927,34007,42392,40312,27635,27668,25587,25587,23507,23507,23507,23507,23507,27635,27635,27667,25587,27667,27667,29780,27699,27667,27667,27635,25586,25586,25586,25586,25586,25586,25618,25618,25586,25586,42327,50778,50778,50745,46552,38133,21295,12811,29747,10730,12875,12844,23342,50712,50712,50745,50745,46552,46584,19117,12745,17069,12843,14956,10730,14923,14956,14956,10698,14891,14923,12778,14891,10697,17068,10730,14891,10664,12809,14890,14890,10665,12811,12811,10698,12777,8519,17003,10665,12777,8584,10697,12777,12745,10631,10664,12776,10664,10697,8584,12810,10664,10696,8551,10664,12745,14858,12774,19175,17063,19111,17030,14950,14886,37968,25418,25450,25417,27531,35952,52792,33809,17036,14923,17003,14923,14956,14956,14923,14923,14923,14891,14922,17000,14918,10660,10660,10661,14855,14919,14982,21161,37936,29582,16904,14824,14823,14823,12711,12711,19017,16936,14791,16936,19017,14824,16969,14823,14791,16969,16937,14824,16937,14791,14823,16937,14856,14824,16936,12711,14856,16937,16937,16904,16937,14856,16937,14856,16904,16872,14824,12742,16998,25453,8518,8485,8485,19083,42260,23211,31662,33711,18855,16839,18983,19047,16935,14854,16966,16934,14887,19080,19015,16903,14790,14855,14855,16935,14855,14887,14822,18984,25321,16935,16904,21097,31629,33743,117},
{57052,57052,57052,57084,57084,59132,59164,59164,59164,57084,57052,55003,57052,54971,54971,57051,57084,57084,57052,54971,52891,52891,50811,50778,48698,46618,42458,38265,31926,50811,48698,36087,27668,27636,25555,25587,25555,25555,27636,27635,25587,27635,27667,27667,25555,25587,25587,27667,27667,29748,27667,27667,25619,25586,25586,25586,25586,25586,25586,25586,25586,25586,44471,50810,50778,50777,46584,38133,21295,10731,29747,10762,12908,12844,17037,23342,23342,23310,29616,31761,25423,10632,6471,10664,10664,8551,8519,10600,10697,10665,6471,8519,12745,12745,8551,8519,10697,12745,10632,8519,10632,12777,12745,8518,8551,12777,12777,8552,8519,12745,12777,10664,8519,10632,12777,12777,8551,8551,14825,14857,10632,10599,12745,14825,12744,10631,12712,14858,10664,8485,21192,19143,17030,14950,23402,33871,50679,54905,46484,38096,38064,46517,44374,25391,21229,21229,21229,21229,21230,17037,17004,17004,14923,14923,17001,14918,10692,10661,10693,19080,16936,12806,17062,31661,37968,33711,16903,14824,14824,14823,14823,14824,19017,19017,16904,12711,12744,14823,16936,16937,14791,12711,14824,14823,14856,14856,12711,12711,14824,12743,14824,14824,14824,14824,16937,16937,16937,14888,16937,16937,16872,14824,14792,10630,14918,23340,6405,4259,4291,12711,29582,25324,31662,31630,18790,16837,16933,14884,10628,12773,14885,14852,12708,14853,10596,10629,10660,6435,6403,6403,14886,8516,21225,21161,27531,12709,8484,23242,35886,31630,116},
{59132,59164,59164,59164,57084,57084,59164,59164,57084,57052,55003,55003,57052,54971,54971,54971,54971,54971,54971,52891,52891,52891,50811,50778,48698,46618,42458,34039,44473,50811,44473,38167,40247,27635,27668,27667,25587,23507,23507,25587,25587,27668,25587,27635,27635,25587,25587,25587,25587,27667,27667,25619,27666,25619,25586,23506,25586,25586,25586,25586,25586,25586,48665,50810,50778,50777,46584,38133,21262,8585,31827,8585,10730,8617,10698,12810,10730,8585,16971,38035,38035,35922,35954,35987,38067,38035,38067,38067,38100,38100,40180,40180,40180,40180,42261,42293,40212,42261,42293,44374,42293,42293,42293,44406,44406,44406,44406,44406,46487,44406,46486,44406,44406,46487,46519,48567,44406,44373,46486,46519,44373,42261,46486,46519,46454,40180,44341,46454,23309,6405,21098,19143,16998,19144,27596,35984,37936,38001,35919,27530,25386,29646,48632,57051,57051,50745,38067,27503,19116,14891,12842,12810,12778,12841,17063,17030,12805,12806,10661,14887,10662,12807,25419,37936,37968,33743,14824,14824,14823,14823,14823,14823,8485,8517,8518,6373,8518,12710,6405,8485,6437,10663,12744,10597,6404,8485,8518,10631,12743,8485,6404,6405,6372,8485,16937,16937,16937,16937,14856,14856,16936,17031,14856,12710,12806,21260,8517,4291,4292,6372,14824,8485,16773,23014,16774,12613,14885,14884,12772,10692,12804,12772,12773,14917,16965,10660,6435,6403,14886,6435,17030,10628,14887,14791,17000,10629,8517,23275,31692,29517,117},
{59164,59196,59196,59196,59196,59164,59164,59132,57084,57052,55004,55003,55004,54971,54971,54971,54971,54971,52891,52891,50811,50811,52859,50810,48698,46618,40345,36119,40279,40279,38167,36087,44472,33941,27668,27668,23507,21395,21427,23507,25587,25555,23507,27635,25587,25587,25587,25587,23539,25587,25586,25586,25618,27667,25586,25586,25586,23538,23506,25586,25586,25586,46552,52858,50745,50778,46584,38166,23375,10698,29779,10762,12875,10731,14923,14956,14956,12843,25391,50680,50712,25390,16971,50680,50680,35955,17037,40148,50712,48632,14826,25358,50680,50712,23342,12745,48600,50712,38100,14891,35922,50712,48632,14826,21197,50712,48632,27503,12745,46454,50712,35987,12745,31729,48599,48632,12744,16970,48599,48599,25390,12712,42228,46486,40115,10599,27470,46453,25357,6405,23146,16839,19113,21162,16903,21130,35888,38000,35919,25417,19079,27499,48632,54938,57018,46454,21229,14858,12745,10697,10697,10697,10665,10664,10728,10695,10662,10661,8580,10661,8517,14823,37968,37936,37968,35856,14823,14824,14823,14823,14823,14823,6404,6372,6373,6372,8485,12743,4292,6372,6372,6373,10630,10597,4324,6372,6372,4324,12711,8485,6372,6372,4292,6404,14856,16937,16937,14889,14856,14856,14919,12837,12805,10661,8548,23309,6405,4292,4324,6372,10630,10565,23144,27174,16839,4292,12742,14852,12741,12772,12772,10692,12837,10692,14852,8548,8548,4323,10596,12773,12805,8580,16999,19017,21225,14886,8517,19082,27467,29549,119},
{59164,59197,61245,61245,61245,59164,59164,57084,57084,57052,55003,54971,54971,54971,54971,54971,54971,54971,54971,52859,50811,50778,50810,50778,48698,42425,38199,42392,50778,46553,42360,38199,34006,29813,27700,25587,25555,21427,21427,23507,21427,21395,23507,25555,27635,25587,25587,25555,23475,21394,23507,21426,23538,25586,25586,25586,25586,23538,23506,23506,23538,25586,44471,50745,50810,48665,46552,38165,23375,10731,29747,12811,12875,12810,14891,14923,14924,12811,23310,48599,50680,21230,12778,50679,50712,35987,14957,38100,50712,48632,14859,25358,50712,50712,25423,14891,48600,50712,40180,12778,33842,50680,50680,14859,23245,50680,48600,27536,14826,44374,48632,38035,14825,31728,48567,48599,16937,21131,48566,48567,29583,19018,40147,46454,40115,19018,29583,46421,23309,6405,23146,16775,19018,21131,16840,14760,35855,38000,33805,25449,21223,31757,46551,54938,57018,48567,19116,17004,17036,17036,17036,14956,14924,14923,12842,10729,10695,10629,8580,10725,10661,35855,37936,37935,37968,37968,14791,14823,14823,14823,14823,14823,12711,12743,12711,12711,12711,14791,12711,12711,12711,12711,12711,12711,12711,12711,12711,12711,12711,12711,12710,12711,12711,12711,12711,12743,12711,12711,12742,10662,10693,10660,10725,8580,4355,23309,6404,8518,6372,6372,10630,6372,6372,20966,16806,6372,14886,16996,21225,12740,14885,10692,10692,12773,16997,10660,8515,8516,8516,14917,14886,17063,21257,25323,25482,19142,8550,16936,29517,29517,117},
{59164,61244,61277,61277,61277,59196,59132,57084,57052,55004,54971,54971,52891,52891,54971,54971,54971,54971,54971,52891,50811,50778,50778,50778,46618,40312,31926,42360,38167,40280,38167,48698,42360,31894,27700,25587,23507,21427,21395,21459,21427,21459,21427,23507,25555,23507,25587,23507,21427,21395,19346,21427,25587,25619,25586,25586,25586,23506,23538,23538,23538,25586,42391,50778,48665,50778,46552,44472,27634,14924,31827,8585,10697,8585,10697,10730,10698,8585,21197,50679,50712,46519,46487,48599,50680,46519,46487,48567,48599,48599,46487,46487,48599,48600,44439,44407,48599,48632,46519,42293,46486,48599,48632,40148,42261,50680,48632,42294,38068,46519,48600,44406,33874,42293,48599,48599,35922,35922,48566,48566,35954,29583,44373,46518,42292,27502,35921,44373,19050,6405,23146,16807,16839,23244,21099,27470,44340,38000,29610,21223,19143,19111,40212,54938,56986,44373,19116,14955,17036,17036,17004,14956,14956,14923,12843,10762,10663,10629,8548,10756,31661,37936,37936,37936,37936,37969,33808,19050,16936,16937,16936,16936,16904,14856,14856,16904,16936,16936,14824,14856,14856,14856,16936,16904,16969,12776,12775,14856,16904,14856,12775,14856,12711,14856,14824,14856,14856,14824,12775,10693,10660,10693,6436,6403,4356,23309,8517,10630,6404,6405,12711,6405,8453,8453,14759,4292,6404,19077,16933,12740,12772,10692,12804,10692,12772,8547,12740,8548,14886,10628,21256,19176,21289,27436,23305,17030,8485,23243,29517,29517,117},
{59164,61244,61277,61277,59164,59164,57084,57084,55004,54971,54971,54971,52891,52891,54971,54971,54971,54971,54971,52859,50811,50811,50779,48730,40312,31959,31926,48666,44537,36054,57116,50746,34006,27701,27700,25555,23507,21427,21395,19347,19347,19315,21395,23475,23507,23507,23507,23507,21427,21395,21427,29748,29780,25587,25619,25586,25618,23538,23538,23538,23538,25586,40278,48665,50778,48664,50745,48697,29714,12778,40246,19150,21295,17037,21263,23408,21328,19150,27568,50679,50712,23310,12778,48599,50744,38036,14859,35955,50712,50713,14891,23310,50712,50713,25423,12746,46519,50712,40213,12778,33809,50712,50713,14858,19084,50712,48632,27568,12746,44373,50712,38068,10632,29583,40115,38002,18986,19018,37937,37937,25292,16840,31598,33743,31663,19115,25422,33841,23309,21196,27502,25422,29615,33906,27600,42325,50679,42258,40177,29579,19111,19143,31760,54970,57018,44405,19084,17003,17036,17036,17036,17004,14956,14923,12843,12810,10697,8549,8581,23241,37935,37936,37936,37936,37936,37936,37936,23275,19050,19049,19049,19049,12743,6436,6436,12743,19049,19049,8517,8485,12743,14856,19017,16969,19147,23341,12743,16936,16968,14856,14888,23308,12711,14856,14856,16936,14888,12743,8485,8548,12837,8548,6468,6436,6436,21228,8517,10662,8485,8485,12711,8485,10565,10565,16839,4291,4292,12741,16964,14884,14884,12772,14885,12773,10692,12740,8547,10660,10692,17030,17031,12837,14950,25355,21224,10693,12711,29549,29517,29549,117},
{59164,59196,61245,61245,59164,59164,57084,57052,55004,54971,54971,52891,52891,52891,54971,54971,54971,54971,54971,52891,50811,50811,50778,44538,31926,31959,29879,29814,42392,42359,63422,52891,40280,31926,29781,25587,23539,23507,21395,21427,21395,23507,21427,23475,25587,23507,23507,23507,21427,25587,27667,31828,29780,23507,25618,25618,25586,25586,23538,23538,23538,25586,42359,48632,48632,50777,48697,46584,31795,12778,42391,10698,12843,6472,10731,12876,10763,8618,19085,50680,50712,25391,14859,46519,50712,38068,16939,35954,48632,48632,19084,25423,50680,50680,29616,19084,46454,48632,42293,23278,35955,48632,48632,23310,27535,48632,48599,35955,25422,44374,48632,42261,25422,31696,23146,16807,18887,18887,18887,18887,18888,23113,31695,31728,33906,33874,31729,27535,23374,25423,25423,27568,33874,44373,33841,50712,33808,25385,23273,25418,21255,19143,31760,57018,57018,52825,19084,14858,10632,10632,10632,8584,8583,8551,8551,8551,10694,10693,8580,31629,37936,37936,37936,37935,37936,37936,37936,25356,19049,19049,19049,19049,14856,10662,10663,14823,19049,19049,12710,10630,16937,16904,19017,16969,12775,17001,16969,16968,14855,14888,12775,10597,16969,14856,14824,14856,14856,10630,6404,12805,10788,10756,8580,4323,8517,19115,6436,10662,8485,8517,12711,10598,10598,10598,14824,4259,4292,8518,12740,16996,16964,16964,12740,14885,12805,10693,12740,10628,12741,10660,12805,10692,14949,8580,12773,8549,23211,29517,29517,29517,117},
{59164,61245,61277,59196,57084,57084,57084,57052,57052,54972,54971,54971,52891,52891,52891,52891,54971,54971,54971,52891,52859,50811,50779,36119,34007,27766,31927,40312,52859,63390,63390,59197,34006,31894,27700,25587,25555,23507,23539,21395,21427,23507,23507,23507,25555,25587,25587,25587,25619,27667,31828,31828,31861,23507,25619,25618,25586,23538,23538,23538,23538,25586,29779,46552,48632,46552,48697,48697,33908,12811,42327,14956,17102,12877,15021,17134,14988,14957,17070,29649,33842,38068,38035,40212,38100,40148,38035,38068,38100,38067,35987,38035,38067,35954,35987,33842,38035,35955,35955,31762,33907,33874,33842,33842,33809,35922,33809,33842,29648,33842,31729,31761,27503,31696,23146,18888,21131,16970,21131,18953,33776,46487,59196,52858,50777,52825,54971,40213,44406,48632,52857,54938,52824,48566,33873,48631,35888,25480,27529,27498,23335,23336,33873,57018,57018,54938,19084,16971,14955,14923,14923,14890,12842,12810,12778,12809,21254,12804,12804,29515,37935,37936,37936,37935,37936,37936,37968,25356,19049,19049,19049,19049,16937,14856,14856,14824,19049,19017,14856,14823,16969,14824,19017,16969,14888,16968,14822,14855,14822,16936,8581,6436,10630,12743,10629,8516,10629,6436,6436,8548,10724,10756,8612,6435,12775,17002,8581,10694,10565,10598,14792,12679,12679,12679,14792,6372,4292,8518,8485,10596,16996,19076,14885,12772,12772,8515,12772,10628,10693,10660,16997,10724,14949,10692,6435,8548,29517,29517,29517,29517,116},
{59164,59197,59197,59164,57084,59132,59132,57084,57052,57052,54971,54971,54971,52891,52891,52891,54971,54971,54971,52891,52891,50811,48698,31926,46650,29846,27766,44440,57084,59197,57116,48665,29814,27701,27701,27668,23507,25619,25619,21459,23475,23507,23507,25555,27635,25587,25587,25587,25555,25587,25619,27667,29780,23506,27667,27666,25586,25586,23538,23538,23538,23538,25586,29779,46552,48632,46552,46585,33972,14892,42327,14956,17102,12844,17069,19149,14956,14892,14957,17070,14956,14957,10698,21262,14956,19149,12811,17004,17069,14956,14923,14923,19117,14924,17004,10697,19150,14924,14957,10698,12843,14956,12810,12778,12778,14923,10730,12778,6439,14924,8584,10697,4325,19018,21098,21033,29582,35889,40148,50680,61310,59196,59196,54970,52825,50712,42261,35954,40180,44373,46518,46518,48598,50711,35953,48631,38002,33806,38031,46517,31756,23336,35954,57051,57050,57051,19116,14922,12777,12777,12809,12777,10697,10697,12809,21287,17092,17028,17060,27467,37936,37936,37936,37936,37936,37968,37968,27437,19049,19049,19049,19049,14856,10630,10630,10598,19049,19017,12711,8517,14856,14791,19017,19049,19112,21193,14886,16967,16935,14887,12808,8549,4355,4355,8580,6468,6435,4323,4323,4355,8644,8643,10724,6468,17065,14889,10725,10694,8485,10566,12711,12679,12679,12711,12679,8518,4292,8518,6437,4292,10628,16964,14917,14852,14884,14885,14885,10660,16997,12805,12804,12804,10723,12837,8548,19081,29549,29549,29517,29517,117},
{59164,59164,59164,59164,57084,59164,59164,57084,57084,57084,57051,55003,54971,54971,52891,52891,52891,54971,54971,54939,52891,52891,44505,31895,31926,29814,27766,36086,59196,59197,57116,46585,27733,29814,25621,27668,25587,23539,23539,23507,23507,23539,25555,25587,27635,25587,25587,25587,23539,23507,23507,25555,25587,23506,25619,27699,25618,25586,25586,23538,23538,25586,25586,25586,31859,46552,48632,48633,36021,8585,40246,14956,17069,12812,17037,17101,12876,14989,14990,15022,12876,14956,10698,19149,17036,17101,10731,14956,17102,14956,14924,14924,17069,14924,17004,10698,17070,14989,17069,10731,17036,17101,14955,14955,14923,19149,14956,15021,12778,17069,14891,14923,8551,21163,21098,16807,16807,14727,14727,21000,40050,61277,59164,54970,54938,54938,59164,59196,50679,50712,52825,61276,59196,63357,50744,59163,42196,42195,48566,46516,31756,21255,35954,57019,57018,59164,21197,17003,19116,17036,17036,17036,17004,17004,19145,21285,19140,19204,17092,25386,37903,37936,37936,37936,37936,37968,37936,29517,19049,19049,19017,19049,16969,12775,12743,14824,19017,19049,16904,14856,19017,16936,19081,17000,16967,16999,19111,19079,14886,14854,12774,8549,6403,4323,8548,8580,8580,6436,6436,4323,4355,6500,6500,6436,17033,12776,8644,10694,8517,10630,12711,14792,12711,14792,10598,12679,4292,8518,6405,4324,6404,6436,10629,14885,12804,14885,14884,14917,16933,14917,14884,10724,10756,10724,8580,27436,29517,29517,29549,29549,119},
{59164,57084,57084,59132,59164,59164,59164,59164,59132,57084,57051,57084,55003,54971,52891,52891,52891,52891,52891,52891,52891,52891,48698,29846,27766,29846,38232,46585,61310,57117,57084,36087,29813,34006,27701,27668,27668,25619,25587,23507,23507,23507,27635,25555,25587,25555,27635,25587,25587,23507,25555,23507,25587,25587,25586,29747,27699,25586,25586,25586,25586,25586,25586,23506,25586,31860,46584,48633,38166,10731,38166,14989,17102,10764,17037,17069,12876,15022,15022,17101,14956,17036,10698,19149,14989,17102,10731,14924,17069,12876,14924,14859,19149,14924,17036,10698,17069,14988,17069,10763,17036,17068,12875,14924,14891,17069,12875,14957,10698,17069,14891,14923,10664,21195,23146,16807,12647,6373,16970,18985,25357,31729,50648,61245,59196,54971,54938,57083,54970,57051,57083,59164,59163,59196,52857,65502,42195,35823,35855,33774,29610,23336,35986,59131,59131,59196,23277,17003,17069,17036,17036,17036,14956,17068,17067,21254,21285,17092,14980,23273,37936,37936,37936,37968,37936,37968,37936,29550,19050,19049,19049,19017,19049,19049,19049,19049,19017,19017,16937,19017,19017,19017,19080,19111,14885,12805,19111,14885,12805,14822,12709,6468,6403,6435,10692,10692,8548,6436,4355,4323,4291,4323,4291,4323,21259,12775,12807,14920,16970,12679,12711,14792,14792,14792,14759,12711,4292,8518,4292,6405,6372,4324,6405,8549,14854,16966,16965,16997,14853,14918,16997,12805,12869,12836,19048,29517,29517,29549,31597,29517,118},
{57084,57052,57084,59132,59164,59164,59164,59164,59132,57084,57084,57084,55003,54971,54971,52891,52891,52891,52891,52891,52891,52891,42425,31959,27766,31927,29846,36086,50811,31926,46585,38199,27701,36087,29781,29781,27667,25587,25587,25587,25587,25619,25587,25555,27635,25587,27635,25587,27667,25587,25555,29780,33941,27667,27667,27667,29747,27667,25618,25618,25586,25586,23506,23538,23538,23538,31860,46584,38166,12843,38166,17069,19150,12908,17102,17102,12876,15022,15022,19214,14956,17036,10698,17070,14989,17101,10762,14923,17068,14923,17003,14858,19149,14923,17037,10698,17037,14989,17069,12843,17004,17068,14891,17004,14923,17069,14923,14923,10664,17035,14890,14923,8583,21131,23178,16807,16807,14726,16872,18920,18920,18920,18920,21033,27371,31630,37969,38002,40082,42227,44308,48566,52760,59163,50744,61277,44308,35855,35855,35920,29611,23304,35986,57083,57051,61244,25390,17003,17069,17069,17036,17036,17036,14956,14923,10727,17061,8612,8580,21129,37936,37935,37936,37936,37935,37936,37968,31630,19050,19049,19049,19017,19049,19017,19049,19017,19017,19049,19017,19017,19017,19017,21192,21191,16998,17030,14885,12805,14853,12709,12774,8549,6436,8580,8612,10692,8580,8516,6403,4323,4323,6404,4291,4291,10662,8581,6436,6436,12776,12711,12679,14824,14824,14825,12712,12711,4292,6405,4291,12744,8518,8485,8517,8485,6404,14886,16997,16997,14885,19143,25514,19144,14918,10692,27404,29517,29517,29549,29517,29517,119},
{57084,57084,57084,59132,59164,59164,59164,59164,59164,59164,57084,57052,55004,54971,54971,52891,52891,52891,52891,52891,52891,52891,42425,27766,29846,46553,46618,36119,27734,31927,55004,29781,27701,27701,27733,27701,29748,27667,25587,25587,25587,27667,25587,25555,27635,25587,25587,25587,25587,25587,31860,31860,33941,27667,27699,27667,29779,29747,27666,25618,25586,25586,23506,23506,23506,23538,25586,29779,38165,25488,42359,14924,14956,8683,12877,12909,10763,12909,12876,17003,8585,10664,6438,10665,10665,10665,6439,8552,10664,8552,8551,8519,10665,8551,8584,6438,8584,8584,8584,6438,8551,8552,8519,8551,8519,8552,6439,8519,6406,8551,8519,8519,6405,19017,33678,23081,16775,16775,16775,16775,16775,16774,16775,16775,14727,14727,14727,14727,14726,14726,14726,16774,16774,21001,18952,23146,44275,44372,52792,61276,54969,31757,35986,59131,57051,61244,27503,17003,14891,14890,14890,12810,12810,12777,12871,10757,12837,12869,8612,17000,37936,37935,37936,37936,37936,37935,40016,33710,19050,19049,19049,19049,19017,12743,12711,12711,16969,19049,16937,19049,19082,12743,19143,19143,16998,17030,14917,14918,14917,10628,12773,10661,8516,8580,10756,12772,8547,8548,10693,6468,4323,6403,6435,8548,8548,6436,4355,4291,12776,12712,12711,14824,14792,14825,14792,12744,4292,4292,4260,10663,6437,6437,6405,6437,10693,10629,6468,14885,12804,14919,25483,19176,19144,19016,29517,29517,29517,29517,29517,29549,119},
{57084,59164,59164,59164,59164,59164,59164,59164,59196,59164,59164,57084,57051,54971,54971,54971,52891,52891,52891,54939,52891,55004,46618,29814,38232,55004,34007,27734,27734,27734,40248,25621,25653,27701,27734,25621,29748,27700,27667,27667,27667,25587,25587,25587,25555,25555,23507,23507,23539,25587,29780,31861,29780,27667,25618,25618,27667,29747,27666,27698,25618,25586,23506,23538,23538,23538,23538,23506,33972,27569,40279,17101,15021,10764,12941,14989,10730,14956,10731,31729,29648,29616,27535,29616,29648,29648,29648,31696,31729,31729,31761,33809,33842,33874,33842,35922,33874,35922,35954,35987,35987,38035,38035,38068,38100,38100,38100,40148,40181,40180,25390,23342,23342,33841,48468,18953,12711,12711,12744,14824,12744,14824,14824,17002,17035,14856,17002,17034,14824,12743,12776,12743,12711,12711,12711,14824,33743,35823,33775,35856,40081,27531,38098,59131,57050,61244,29616,17003,12810,14922,14923,12842,12810,17001,17062,14982,12901,12868,12932,19111,37936,37936,40016,37968,37936,39984,37936,33775,19049,19049,19049,19049,19049,8517,6404,6437,16936,19049,16969,14953,17066,12808,21192,19110,14917,12837,12837,10692,12773,8548,8548,6436,6436,8612,12836,12836,8548,10660,12836,8516,6467,8580,8515,12772,10724,6435,4323,4291,12744,12712,12711,16905,14792,14857,14792,16905,4291,4292,4259,10663,6437,6437,6437,10661,10692,6436,6436,14949,12836,17063,17030,21257,23306,29485,29517,29517,29517,29517,29517,29517,118},
{57084,59164,59164,59164,59164,59164,59164,59196,59196,59164,59164,57084,57052,55004,54971,54971,54971,54971,54971,54939,54939,57052,46585,31927,36119,42392,29846,25654,25654,31927,29814,25621,29814,34007,27733,27701,29748,27700,27667,25587,27667,25587,25587,25587,25587,25555,23475,23507,23507,23506,23539,25587,25587,27667,27699,25618,25586,25587,27699,27699,27699,25586,23538,25586,25586,23506,23538,23506,23506,27666,38166,23376,21295,17102,21295,23407,21294,23407,21262,48600,50712,38068,25358,42261,50680,46487,23343,33842,50712,50712,27503,21165,48632,48632,33842,16971,42260,50712,46519,14858,27503,48631,50680,19084,16971,48632,48632,33842,12746,40148,23342,23342,23374,33841,46387,19083,12809,10729,10697,17002,10664,10696,14954,25584,27729,21260,27761,27761,21260,10630,10630,14856,8517,8550,12776,10663,33743,35856,37968,40049,40081,38000,40180,57050,57018,57050,27535,17003,10632,10632,10632,10632,8584,8583,17030,12901,12869,6468,10788,17030,37936,37936,37936,37936,37936,37936,37936,35855,19049,19049,19049,19049,19049,12775,12775,14856,16969,19017,19017,16969,17033,17001,19111,19110,17030,17062,12836,12804,12805,8548,6436,6403,8517,8580,10755,12804,8580,8581,8580,8548,6467,12804,12804,10692,10691,12836,10692,6371,12744,12744,12712,16905,12744,14857,14792,18985,4259,4292,4292,10631,6437,8517,8549,14949,8579,6436,6403,12805,14949,14917,21288,23369,23306,31695,27404,29517,29517,29549,29549,29517,120},
{59164,57084,59132,59164,59164,59164,59197,61245,59196,59164,57084,57084,57084,57084,55004,54971,54971,54971,54971,54971,54939,44505,29846,29879,29847,31894,46650,25654,29815,29814,27734,27701,27701,31894,34006,31894,29781,27700,27668,27667,27667,23507,23507,25587,25587,25587,21427,21395,23507,23507,21394,23474,23506,23538,25586,23538,23506,21394,27667,29747,29747,27666,25586,25586,25586,23538,23538,23506,23506,27666,33907,27633,23408,23375,25456,35955,38067,38100,40148,48567,48632,33875,12779,38067,50712,46487,14956,27568,50712,50713,23343,14891,48632,50712,33842,14858,40148,50680,46519,14859,27503,50680,50713,19117,14891,50680,48632,33842,14858,40148,25423,23342,23374,33809,48468,21163,10729,8585,10729,17035,10665,12810,14922,25649,29842,21261,27761,27761,21292,8518,6470,14856,8550,10631,14889,10663,29550,31630,33775,37936,40049,40081,42260,57051,57050,61244,33842,17035,17036,17036,17068,17036,17036,17066,14982,12869,12901,12869,12900,19142,37935,37936,37935,37936,37936,37936,37936,37968,19017,19049,19049,19017,19017,12710,10630,10630,14823,19017,19017,10695,12808,12743,16999,19142,14916,12804,12837,14949,12805,8548,6435,4323,6436,6467,14916,12772,8580,6435,6403,8548,10692,12868,14980,10724,12804,12836,12804,14853,10631,12744,12744,16937,12744,14824,14825,16970,6372,4292,6372,8518,10631,14951,12837,12868,10692,10693,6435,12709,10660,12837,12869,23369,23306,31694,27404,29517,29549,29517,31597,29517,117},
{61245,59132,57084,59164,59164,59164,59164,59164,59196,59196,57084,59132,57084,57084,57052,54971,54971,54971,54971,54971,54971,38232,29847,29814,27766,31927,40345,25654,29846,38200,29814,31926,29814,29813,29814,25621,29781,29748,27668,27667,25587,25587,25587,25587,27667,25587,25587,25587,23507,23507,27668,25587,23507,23506,23538,25586,25586,25587,25619,27699,29747,29747,27666,25618,25586,25586,23538,23506,23506,23506,31859,29746,25489,25521,29649,48631,50679,42325,38068,48599,50712,46519,42326,46519,50680,48599,44374,46487,48599,48599,46487,44439,48599,48631,46519,44407,46519,48599,48567,44406,46486,48599,48599,44406,42293,48599,48599,46486,40213,46486,25422,23342,23374,33776,48468,23244,10697,8552,8551,17035,10697,8584,17002,27697,29842,21293,29842,29874,23373,6438,6437,14857,8518,8518,25422,31760,31663,27404,27404,25324,23179,33807,40147,57051,57050,59164,35954,19116,17036,17068,17036,17036,17036,14985,14983,14983,12902,14982,19237,19110,37935,37936,37936,37936,37936,37936,40016,37968,19017,19049,19049,19017,19017,10630,6436,6436,12743,19049,19017,14888,10727,10695,16968,17031,14949,12837,12772,14949,12805,12773,8548,4323,6436,6436,14916,12868,12772,10693,8548,8580,8612,10692,14980,12836,12836,12804,8547,21030,14759,14825,12744,16937,12712,14825,14792,12712,14824,6437,8485,6372,10725,17093,10788,12900,10723,12869,8515,16933,21224,10660,14886,23370,23242,27500,29517,29517,29517,29517,29517,27436,116},
{59197,59164,59132,59164,59164,59164,59196,59164,59197,59196,59196,57084,57084,57084,57084,57052,57051,55003,54971,55003,54971,31926,29846,27734,27766,27734,25654,29847,34039,31927,27701,29814,27701,25653,25621,25621,27701,27700,29748,27667,25619,25587,25555,27635,27667,27667,25587,25619,25619,25587,25555,21395,25588,27668,23507,27699,27699,25618,25619,25586,29779,29746,29715,27666,25618,25618,23538,23538,21458,23505,27666,27634,27666,25554,25553,48631,50679,31762,14924,44374,50712,35987,14859,35955,50712,48600,14892,25391,50712,48664,25423,14858,48632,50712,35923,14826,38068,50680,46519,12778,25423,50680,50712,23311,14923,48632,50680,33874,12778,38068,25423,23342,23342,31728,48468,25324,10697,8584,8583,17035,12810,8552,17002,29842,31955,23406,31987,31987,25486,8518,8550,16937,10598,14857,17003,16970,31630,27372,25324,14825,21099,27500,38098,57083,57050,61244,38035,19116,17036,17069,17068,17036,17036,14955,12874,12809,10761,8648,14950,19142,37935,37935,40016,39984,37936,37936,37936,35823,19017,19017,19017,19017,16936,14856,14856,14856,16904,16936,16936,14856,14856,14856,16968,17031,19111,19142,12836,12837,12805,8580,10660,6436,8516,8516,14948,12836,8611,10692,8548,8612,10692,8580,8611,10756,8548,6403,6435,23079,33418,14857,12744,16938,12744,16970,14824,12646,10566,17000,10661,8548,12804,10723,14980,17092,15012,14980,10724,14853,14886,17030,21289,21225,19048,21193,27403,29517,29517,27470,27600,29681,100},
{59164,59132,59164,59164,59164,59164,59164,59197,59197,59164,59164,59164,59132,57084,57084,57084,57084,57084,57084,57084,52891,31926,36152,27766,27734,25654,27767,29847,44506,31927,25621,25653,27734,25653,25621,25621,25621,27701,29748,27699,25619,25587,23507,27635,27667,25587,25619,25587,25619,29780,27635,27668,29716,27635,29780,33941,29747,25618,25586,27667,27666,27698,29747,27666,25618,25618,23538,23538,23538,23538,23538,29779,21393,25586,25553,46486,50712,40181,31761,46486,50680,38035,14859,35922,50680,48632,14891,25390,48632,48632,25423,14890,48567,50680,35955,14891,38067,48599,48600,19052,27535,48599,48632,27536,23310,48599,48599,38035,23310,40180,23342,23375,23374,27502,29484,25389,14923,17036,17068,19148,12810,8584,19051,31954,34067,25486,34100,34100,31792,27469,40147,50744,57051,54970,54938,57083,33776,35921,38034,42293,40147,44372,44405,57050,57018,61244,40148,19116,17036,17069,17069,17069,17036,14988,14955,12842,12841,10729,14920,21192,35855,37936,37968,37935,37936,37936,37968,14791,14823,14823,14823,14823,14791,14823,14791,14791,10630,10630,12711,12711,12743,12711,12774,14918,14950,17029,12836,14981,12837,10660,10693,6435,6436,8516,12772,12868,14948,12867,10724,6499,8612,10692,10660,8580,4323,6435,8516,25127,43787,23114,12712,16938,12744,14825,12744,12711,10599,14888,12869,10660,12741,17061,12805,15012,12932,14980,17125,12804,14917,14949,10724,16968,16936,21161,21097,21130,21130,19050,14824,12744,70},
{59132,59164,59164,59164,59197,61277,59196,59164,59196,59196,59196,61244,59196,59132,57051,57051,57052,59164,59164,59164,52859,29846,29846,29879,25654,27767,34040,25686,27702,27734,25621,27701,27701,25621,25621,25621,25621,25621,29780,27699,25619,25587,25587,25587,25587,25587,25587,25555,27667,25619,27667,27635,31828,33941,36021,33941,29748,27699,25618,25586,25586,25618,27699,27699,25618,25586,23538,23538,23538,23538,23506,27666,27666,29746,23505,31826,36019,38132,44374,48599,50712,48600,38100,40181,42261,40180,40180,40180,42261,40180,40180,38068,42261,38068,38068,35987,38067,38100,38068,33874,35955,35987,33874,35954,31762,35955,31729,33842,29649,29681,23342,23375,25422,25422,27404,27469,29745,36148,36148,23373,12776,8550,19050,34035,36115,27566,36180,36180,27599,10630,10630,19050,16904,25356,25356,35985,40147,35954,38034,50712,48534,48599,48599,57050,57018,61244,42293,21197,12745,10664,10664,10664,10664,10664,10631,8583,8583,8551,8550,14887,35855,37936,37936,37936,37936,37936,40049,16904,14823,14823,14823,14823,14791,10630,8517,8518,8485,6437,12711,8517,8485,10598,14885,14949,14949,14917,12837,14948,12836,12837,8548,6436,6436,6468,8548,12805,14948,14980,10691,8547,12837,12836,10724,8580,6403,6468,8515,25095,43819,25194,12744,19018,12712,14825,19083,14791,14791,17033,12836,12868,12805,12804,12805,17093,15012,15012,17157,12836,12773,21287,21289,23339,23275,29613,29549,29549,31597,29517,29517,27467,84},
{59197,59197,59164,59196,61245,61277,61277,59197,59164,59164,59164,59164,61245,61277,57084,57051,57051,57084,59196,59196,50810,29846,27766,29815,25654,29847,27766,25654,29847,27733,25653,27701,25621,25621,25653,27702,25621,25589,27700,29747,25619,25619,25587,23507,21427,21395,23507,23507,25587,25618,25619,25586,29780,33908,33908,31828,31860,29780,25618,25618,27667,25586,27666,29747,27667,25618,23538,23538,23538,23506,23506,25586,29714,29746,23505,25586,31794,29714,29714,42359,46585,42327,10698,14956,17069,14955,14891,14923,19084,14923,17004,10698,21263,14924,17037,10698,14956,17070,14989,12811,17004,17068,14923,17068,14891,21229,14923,17003,12778,14924,25455,25487,27536,27535,25323,27469,25519,38261,38261,25486,14857,8518,16937,36148,38228,27566,40342,40341,29647,10630,10663,19050,10598,10631,19050,27599,29679,12712,14792,27404,27404,31662,44373,38065,44339,61309,48599,35987,33842,36020,38068,36020,36019,38068,38100,36020,36020,36019,31761,25388,35823,37936,37936,37936,37936,37935,37968,19017,14823,14791,14791,14823,14791,10598,4292,6372,4324,6372,12711,8485,6372,6404,12805,14949,14949,12836,14916,12868,12869,10756,10725,6436,4355,8548,6436,4356,8548,19172,17093,8579,10692,12869,12837,12805,12805,8581,8548,25160,43852,29387,12712,19050,12744,10664,14922,12745,12711,17062,8548,15012,10724,17093,12837,15012,17156,17156,17061,10596,12774,19175,23369,23402,27436,29677,19049,29517,29549,29517,27467,19142,73},
{59197,61277,61245,59197,61245,61277,61277,59196,59164,59164,57084,59132,59196,61245,59164,59196,57084,59132,59196,61244,46618,27766,27767,27734,31928,29847,34040,25654,25654,27701,29814,27701,27734,27733,25653,25621,25653,25621,27668,29780,27699,25587,23507,23507,21427,19347,21395,23475,23507,25586,23506,25586,25586,31828,36021,31860,31828,29780,29747,25618,25618,25618,25586,27698,27699,25618,25586,23538,23506,23538,23538,21458,27667,29746,23505,23505,31826,27665,27633,33907,31827,29746,23472,23472,23472,23440,21392,21360,21360,25521,31827,27601,21328,21392,21393,25554,27634,21328,21328,23473,23440,23440,12842,10729,8584,8583,6471,8551,8551,8584,33940,36053,36053,38132,23179,21066,18985,12679,14759,18985,14792,16872,18985,14792,14792,18985,14791,14791,18952,14791,14791,16904,14759,16872,19050,21163,16937,10599,25324,33776,50712,61244,40210,19143,33839,63390,54905,52858,42228,40115,38034,38034,35954,35954,38034,35921,35954,33841,29615,33808,35856,37936,37936,37936,37936,37935,37936,21098,14823,14823,14823,14791,14791,12743,12711,12743,12711,12743,14824,14791,12743,12743,17031,17030,14950,14917,14981,14949,17030,10756,8548,6403,6436,6404,6436,6404,6404,10692,14980,10724,10724,10724,8547,8548,6404,4323,6403,25062,43755,31403,12712,14857,12712,14825,12777,12744,12744,17001,4291,8580,14949,15012,12805,12837,14980,10692,8484,12677,12710,19143,21289,21289,23210,21225,14854,27404,29517,29549,31662,27500,91},
{59196,61277,61277,61277,59196,61245,61277,61244,61244,59196,59164,54971,57052,59164,59196,61244,59196,59164,61244,61244,44473,27766,29815,34072,38232,42458,36152,25686,27766,25653,27734,29814,29781,29814,25653,27701,34007,34006,36086,29748,29748,25587,23507,21395,21395,19347,23507,25555,25587,25587,23507,23539,21426,21426,27667,31892,33973,31860,31892,29779,27698,25618,25618,25618,29747,27666,25618,25586,23538,23506,23538,23506,23538,31795,27634,23505,29714,27665,27633,29714,31794,27633,25585,23505,23505,23473,21393,23473,27666,23473,23473,29747,25554,21360,21360,17200,21392,19280,19311,17199,19279,17133,8616,8616,10664,8584,6471,8551,8551,8583,33940,36053,36085,40245,48435,41966,39853,41934,41966,41966,41966,44046,44046,41965,44078,44078,41966,44078,46159,48337,50548,52792,54938,59131,52825,54938,42260,33744,54906,61244,54873,57018,44404,19143,35951,63422,52857,52857,38002,33841,33841,35922,33841,33841,35954,35954,38034,38034,38034,40115,37969,37936,37936,37936,37936,37936,37968,23211,14823,14823,14791,14791,12711,12743,19017,16904,14823,16937,16937,14823,16937,14824,16935,19111,17030,12837,12837,12804,12773,10660,6468,10661,8548,6468,6436,8516,6436,8580,17061,12900,12868,10724,10692,10628,6436,4291,6371,25030,39626,33484,12712,16970,14792,16905,14857,14857,14857,17002,10724,12804,10660,14981,10693,10628,12773,12741,12742,19014,8484,14886,19176,23337,19017,12742,8549,27436,29517,29549,27467,16998,43},
{59164,61245,61309,61277,61245,59164,59197,61245,61245,59197,57084,54971,55003,59164,59164,59164,59164,59164,59164,61244,50811,31959,38232,29879,27734,27734,29847,27702,29814,25653,27734,31959,31894,29814,27734,29814,29814,25653,25620,29780,29780,27699,25587,21395,21395,21427,23475,23475,25587,23507,23507,23507,23507,21426,23507,21426,27667,29780,29780,31860,25618,25586,25586,25618,27666,27667,27666,25586,23506,23506,23506,23538,23538,27666,29714,25586,27634,29714,27665,27633,29746,25553,23440,23375,23505,21424,21392,23473,38166,33940,27666,23473,29714,27634,19280,17200,21360,19280,19311,21392,19279,19181,8616,8648,10729,8584,8551,8551,8551,6438,31859,36053,36053,42293,52661,46159,46159,48239,46126,48207,46159,46159,46159,46158,46126,46126,46126,46126,46126,46158,46158,46126,46191,50450,48402,52694,56986,54905,59132,57051,52825,52792,40147,40210,46452,61277,52792,52857,50712,54938,54938,54970,54938,54938,54938,54938,54970,54938,54970,54971,44340,37936,37936,37936,37936,37936,37968,23243,14823,14791,14791,14791,12711,12711,19017,16937,16904,16936,16937,14791,16936,16904,17000,19079,17030,14917,14917,12773,10661,10629,6468,10693,8580,8548,10661,8580,10661,10661,10692,17061,14948,12804,10692,12772,8548,4323,6371,27143,39594,35565,12712,19018,14792,16938,14792,14857,14857,14920,17062,14916,12741,10628,12741,14885,12804,10724,12773,19110,8516,6436,19144,21256,18985,8549,8549,27404,29517,29517,29547,19110,91},
{61245,61245,61277,61277,61277,59164,59196,61244,61244,59164,54971,52891,54971,57084,59132,59132,59164,59164,59164,61244,52890,36120,38200,34039,38233,38233,31927,25653,25653,27701,29814,29782,29781,31894,25621,27701,29814,27733,29781,36054,31828,29747,25587,21427,21427,23507,23475,23507,21427,21427,21394,23507,21394,23507,23475,23475,27635,27667,25554,31860,23538,23538,23538,25586,25618,27666,27666,25586,23506,23506,23506,23538,23538,23505,31795,25553,25553,31794,27665,27633,29746,25521,23375,21230,19181,19181,19181,19149,25455,19117,17036,19149,17004,12843,14988,12810,12843,14956,10730,14955,14955,10697,8616,14922,8584,8551,12810,8551,10664,10696,31859,36053,36053,42293,54741,46159,46159,37837,44176,48239,33514,37805,46191,39950,46322,46191,39982,46386,48239,31434,29353,46191,31466,27273,46192,33547,25160,46224,35725,42227,57083,57051,52825,50679,52824,63390,52857,52825,50712,54938,54938,54938,54938,54938,54938,54938,54938,54938,54971,57019,46421,37936,37936,37936,37936,37936,37968,25356,14823,14823,14791,14791,14791,14824,14824,16904,12743,10598,10663,12711,12743,14823,12710,10662,12743,12806,14917,12804,12773,10661,8580,8580,12805,8548,8580,10660,10693,8548,10662,14950,14981,12868,10660,10692,6435,6371,4291,29159,39594,35531,16906,19051,12744,14825,16938,16938,14889,14888,17032,12836,17093,8548,10629,14948,8611,10756,8580,12773,8516,10629,8580,16998,14791,8517,6403,23211,29517,29517,29517,29549,103},
{61277,59197,61245,61245,61277,61277,61277,59196,59164,57084,52891,52859,52891,55003,55003,57052,57052,57084,59164,61244,57084,31959,31959,36184,34039,31959,34040,29846,27734,25621,27733,36119,27733,27734,25621,36087,40280,27701,33942,38134,31860,29780,27699,25587,23475,21427,21427,21395,23507,21395,21395,23475,21427,23507,27668,25588,27635,25587,27635,27700,25619,23538,23538,23538,25586,25586,25586,23538,23506,23506,23506,23538,23538,23505,27666,23505,23505,31794,29713,27633,29714,27666,23375,19084,16971,16971,16971,14923,12777,10665,10664,12778,8584,8551,12777,8519,8519,12745,8584,8519,12745,8551,8551,12777,8551,6471,12777,8519,8551,10664,31859,36085,38133,42293,52660,46159,46159,23242,38097,46192,14791,25355,46224,31727,42389,46224,33807,44469,46256,18952,14791,46256,21032,12678,46224,23145,14823,46224,27306,21097,42194,50614,57051,57051,54905,63422,52857,52825,50744,54970,54938,54970,54938,54938,54970,54970,54971,54970,57051,57051,46453,37936,37936,37936,37935,37935,37968,27404,14823,14823,14791,14791,14791,12743,6404,6437,6405,6405,8518,12710,6372,6405,6405,10631,10663,10629,12837,14917,12836,10692,10693,8580,10693,8548,6468,10725,12805,12741,12709,12741,12804,12804,12837,10692,8548,6435,6371,33385,39594,31338,19051,16970,12744,14857,16938,16938,14857,14825,14889,23401,21287,14885,19145,19175,12869,12805,8547,12741,10597,10597,6403,8517,10598,8485,10597,21097,29517,29517,29517,29548,97},
{61245,61245,59164,59196,61277,61277,61277,61277,59197,55004,52891,50811,50779,50778,52891,54971,57052,57051,59132,59164,42393,31926,36152,29847,36152,29847,25654,27734,27701,25621,27733,36087,38200,27733,38200,31861,31894,27733,36086,36054,31861,31828,29747,25619,23475,23507,25555,23475,23507,19346,19346,23507,25555,27668,25587,25555,27667,33941,36021,31893,25619,23506,23538,23538,23538,25586,25586,23570,23538,23506,23506,23538,23538,23505,25586,29746,25554,29746,29713,27665,29681,31794,25488,19084,16971,16971,16971,14923,12777,10665,8551,10697,10762,8551,10697,10664,8584,12745,8616,8552,14890,10664,10665,12777,8584,8584,10665,10632,8584,12777,31859,36085,38133,42326,52660,46159,46159,25290,37935,46191,23112,29451,46224,33677,42161,46191,35789,40080,48272,27306,25225,46223,29418,25225,46224,33677,29549,48467,40081,44372,44340,48501,40082,44340,48566,63390,52857,52825,50712,54938,54938,54938,54938,54938,54970,54970,54970,54938,57018,57018,48566,37936,37936,37936,37936,37936,37936,27436,14824,14791,14791,14791,14791,14791,6372,6404,6404,6373,8485,12743,6372,6404,6372,6372,8517,12742,12773,12805,14949,10692,10660,8580,10692,12773,8549,12806,14949,10660,12741,16903,16997,12804,10724,14917,12836,10628,6404,33417,37546,31338,16970,16937,14824,14825,14857,16970,14857,14857,14890,8517,14950,16998,14948,17093,12899,14917,10660,10661,8516,8516,6436,8516,10598,8517,8484,16936,29517,29517,29548,27560,89},
{61277,59196,59164,59196,61277,61277,61277,61277,59164,52891,52859,50811,50778,48698,50811,52891,54971,54971,57084,59164,42425,46618,40345,27767,25654,25654,25654,29814,29814,29781,27734,27701,33974,34007,36054,31926,29814,29781,31893,33974,31893,31828,29780,29748,25587,23507,21427,23475,23507,23475,25587,27668,29748,27668,27667,31861,31828,27635,27667,27699,23506,25587,25586,25618,23538,25586,23538,25618,23538,23538,23506,23537,23538,23505,23505,29746,29746,29746,29713,27633,27633,29746,25456,19084,16971,14923,16971,14923,10697,8584,8552,12745,8551,8552,12777,8552,8551,12745,6471,6438,12745,8552,6438,10664,8519,6406,10697,6438,8519,10664,31859,38166,38166,42326,52661,46127,46159,46159,44078,35627,35627,41965,35659,33514,41965,35627,31434,39885,33579,27241,37772,35627,25160,37772,35660,25193,39951,37936,37969,50744,57018,56985,52792,40145,40145,63422,52825,50744,50744,54938,54938,54970,54970,54971,54971,54970,54971,54970,54970,57019,50647,37936,37936,37936,37936,37936,37936,29517,14824,14823,14791,14791,14791,14791,14791,14791,12743,12743,12711,12743,12743,12743,12711,12711,12711,14854,16998,16998,16998,14917,10628,10693,12741,10662,10662,12773,12805,12837,12773,12742,14917,14916,12805,10724,12772,12805,8484,35433,35466,31338,14857,14857,12712,14825,16937,16938,16937,14857,16970,4292,8548,12804,17092,14948,17124,19173,12805,8580,6403,8548,10660,17030,12678,4324,6436,14791,27404,29517,29517,27498,88},
{61309,61309,61245,59196,61277,61277,61277,61245,57084,52891,50811,50811,52859,50778,50811,52859,52891,55003,57052,52859,38265,34039,42425,36152,29815,27767,27734,27734,36086,31926,31894,31894,27734,40280,34007,27734,25621,29781,34007,29781,25621,29748,31828,29780,25587,23507,25588,25555,25587,25587,25587,27635,29748,27668,29748,29747,29747,29780,29748,31860,25586,25586,25586,25586,23538,25586,25586,25618,23538,23537,23537,23538,23506,21426,21458,25586,29746,29714,29746,27633,27633,29714,25456,17004,16971,14923,16971,14891,10697,8584,8551,10664,8584,10664,10697,8584,6471,10697,8584,6471,10665,8551,8551,12777,8551,6471,12777,8519,8551,12745,31859,38166,38166,42326,52661,46126,46159,23242,31759,19017,6405,14856,21130,8453,12711,21098,6373,14824,21098,6373,12776,23243,6373,12776,23275,6372,10598,21066,10565,19082,44307,46256,27338,21192,42258,63390,54906,50712,37970,33808,31696,33776,31663,31695,31695,29615,31663,29583,29582,29550,31663,37936,37935,37935,37936,37936,37936,37969,33776,16937,16937,16904,16936,16937,14856,14856,14824,14856,16937,16937,14824,12743,12743,14824,16936,16968,14887,14886,14886,14950,14919,16967,12743,12711,10662,16966,17029,14949,10693,10661,14917,14948,8580,10597,8548,14917,10628,37546,35434,31305,16937,16938,12744,14825,16937,16938,14857,14825,14857,8485,6436,17061,17125,14948,14980,17093,19174,8580,8548,6468,8580,16966,10598,6404,4355,10630,27404,29517,29517,29516,107},
{61309,61309,61309,59164,61244,61245,59197,54971,54971,54971,54971,54971,52891,52859,50810,50778,50811,54971,54971,52891,31959,29879,29879,31927,34040,29847,27766,31894,36087,31926,31926,27733,31927,29814,27734,25621,25621,23541,25621,27733,29781,27701,31828,31828,27667,25555,25555,27667,25587,23539,23507,23507,25555,27667,27667,31828,31860,29748,31860,29780,25586,23506,23506,23506,23506,25586,25586,25618,23538,23537,23537,21458,21458,21426,21426,21426,29747,29714,29746,27665,27665,29713,25423,19052,16971,14923,16971,14890,10632,8584,8519,10665,8552,8551,10632,6471,8519,10665,6471,8519,12745,10632,6438,12810,6406,6438,12745,8519,6438,12745,31859,38166,38166,42326,52661,46158,46159,19017,23308,19050,8453,14824,19049,8453,16905,19050,8420,14824,21130,6372,12711,21130,6405,12743,21163,6372,12679,19018,8485,21130,42194,42063,29580,19112,35984,63390,52825,50712,42260,42260,40180,42260,42293,42293,40180,40115,44373,44373,44406,46486,46421,37935,37935,37935,37936,37968,37968,40016,37936,19017,19049,19049,19049,19017,12840,12841,10695,12743,19017,19017,10630,6469,8518,12743,16969,16937,8549,8581,8516,12774,16936,16936,8517,6404,8485,12710,16999,14917,14886,12806,14917,12772,10661,8516,8516,12773,12773,39626,35466,27177,21164,16938,14825,14825,14857,16938,14857,14825,14825,10631,6371,14916,17061,10660,14948,19142,14917,14980,10693,12743,12806,19080,12646,4324,6404,19081,27404,29517,29517,29517,118},
{61309,61309,61309,61309,61277,59164,57084,54971,57052,57052,57052,55003,54939,52891,50811,50778,50811,54971,52891,38167,27734,29847,25654,27734,27734,27702,29847,34006,44505,27701,31894,40312,33974,25653,25621,25621,25621,23541,23541,27701,27733,25620,29748,29747,29748,25555,27668,25587,25587,23507,21427,21427,25587,29748,31828,29748,29780,31828,31860,27699,23538,23506,23506,23538,23538,25586,27666,25618,25618,23538,23538,21458,21458,23506,21425,23506,27666,29714,29746,29713,27665,27633,25423,19084,17003,16971,17003,14891,10632,10632,8551,12745,10632,8584,10664,10632,8584,12777,10632,8551,12777,10696,8551,14890,10664,10632,14890,10632,10665,14858,33940,38198,38198,42358,54773,31434,20902,16742,16807,14759,12581,16807,14759,12581,16807,16807,10533,16806,16807,10500,14726,16839,10501,14726,16872,10533,27339,40049,52759,59163,54905,48566,52857,23306,38065,63390,52825,50680,40115,35922,33841,35954,33809,33841,35922,33841,29583,29550,29550,40115,29517,37935,37936,37936,39984,40016,42129,44242,44275,19017,19049,19017,19049,19017,14888,12873,12808,14889,19017,19017,12776,10728,10695,14824,16937,16936,12743,10662,10662,12775,16936,16904,10662,10598,10630,12743,14855,14886,14855,14854,12772,8548,10629,10693,12805,12741,12709,41739,33386,27177,21131,16938,14825,14825,14857,16938,16938,14857,14824,16967,14917,17092,12868,10692,14917,17061,12772,12900,10693,17002,17033,16967,10598,4324,6436,19049,29549,29517,29517,29517,111},
{61309,61309,61277,61277,61309,61245,57052,54971,57052,57084,57052,54971,54939,52891,50811,50778,50811,52891,50778,34040,25654,27735,27767,29847,27767,27734,27702,29814,34006,27734,27701,42425,31926,27701,25621,25621,25621,23541,23541,23541,23540,25588,27668,29748,29748,25555,27635,25587,25587,23507,23507,27668,29780,27635,29716,27667,25555,29780,29780,27667,23506,23506,23506,23506,23506,23538,25618,25618,25618,25586,23538,23506,21458,23506,23538,23506,25554,31794,29746,29713,27665,27665,36052,40246,42327,40246,42359,38165,33940,29714,23440,27634,27634,23408,29714,25521,25488,27634,23375,27601,25521,23408,27634,23408,25488,25521,23375,27601,25488,27569,36085,38198,38198,42326,52725,48370,41933,41933,41933,41933,41966,41966,41965,44046,44014,41966,44046,44014,44046,44046,44046,44046,44046,44046,44046,44046,44046,44046,44014,41997,44305,33707,31724,17062,33838,61309,52825,50712,40115,38067,42260,42293,38067,40147,42228,42293,42261,42228,42260,52792,33775,37904,40016,42129,42161,44242,44242,44242,44275,19049,19049,19049,19049,19049,16937,14856,14856,14824,19017,19017,14856,12743,12743,12743,16937,16937,14823,12743,12743,12743,16904,14856,12743,12711,12711,12711,14824,14855,14887,12804,10692,10661,12743,16967,10628,8548,10596,43787,31306,25129,23212,14857,12712,12744,14824,14825,14825,14825,14857,17000,12804,19140,10787,12868,12869,12869,14917,6467,8580,12775,17000,21193,19050,4323,8517,19048,29549,29517,29517,29578,74},
{61277,61277,61277,61277,61245,61245,57052,54971,57052,57052,55003,54971,52891,52891,50778,50778,50778,50779,48698,29847,27735,27766,31959,29847,31959,25654,27701,42393,36119,25621,27702,29814,29813,25653,25621,25621,25621,25621,23541,23541,23540,25620,25588,27700,29748,27667,27667,27667,25587,25587,25555,25555,23474,29748,31828,29780,29780,33941,31860,25587,25587,23506,23506,23474,23506,23538,25586,27666,25618,25618,23538,23538,23538,23538,21458,21426,21425,31827,29714,29746,27665,27665,44439,48698,50778,48697,50810,35988,19149,14956,17004,19182,14988,14956,14956,12875,14923,12875,12843,14891,14956,14923,12843,17068,12810,12843,14922,12778,14923,17004,31859,38198,40246,42358,54806,52563,46094,33579,37805,46159,31466,33611,46158,29386,29353,46159,29353,25160,46159,31434,27305,44078,29386,25225,44078,29354,20967,44111,33579,42226,46386,37868,31626,19143,33903,61309,52826,50744,52825,54938,54938,54938,54938,54971,57019,54971,57019,54970,57019,57051,44340,42162,44242,44242,44242,44274,44274,44274,44275,21130,19049,19049,19049,19049,14856,10695,8615,10631,19017,19017,12711,6436,8517,8550,16937,16937,12743,8581,8550,10598,16936,16936,10630,8485,8517,10598,14856,14824,14854,14885,14949,12773,14822,14822,10661,12773,12708,41707,33418,23049,23179,16905,12712,14792,14825,14825,12744,14856,10629,8516,6435,14916,14980,12900,14949,14981,12837,8644,10756,14981,14919,23242,21162,6404,6404,25419,25354,29549,29517,29611,87},
{59197,61277,61277,61245,61245,61245,57052,54971,54971,54971,54971,54971,52891,50811,50778,50778,50778,50779,46585,42393,27734,29847,36152,34072,27766,31927,31894,40312,36151,27701,29846,31926,29814,27701,25621,25621,25621,25621,23573,23541,25620,27701,25621,25619,27667,27667,27667,27667,25587,25587,25555,25555,29748,31828,31828,27667,25587,29780,27667,25587,23506,23538,23506,23506,23538,23506,25586,27698,27666,25586,25586,23570,23506,23538,21458,21426,23506,29714,29714,29746,27633,27633,44439,48698,50778,48665,48730,25455,8584,10697,14890,8552,8617,14891,8519,8584,12778,8584,12843,6439,10665,14890,8551,17004,6438,8584,12810,6471,12777,12745,31827,38230,40278,42358,54806,50515,46126,16903,27435,46192,14791,21129,46191,16839,16871,46192,16872,12710,46224,21032,14856,46191,21065,14823,44111,23113,12678,42031,27371,33970,44306,37901,25385,19143,35983,63390,52857,50712,52825,54938,54938,54938,54938,54971,54971,54970,54970,54970,57018,54970,50680,29518,44242,44242,44242,44242,44242,44274,44274,23243,19049,19049,19049,19017,14888,12776,12776,12776,19017,19017,14856,12711,10663,12711,16936,16937,14823,10662,10663,12743,16936,16936,12743,12679,12711,12711,14824,14824,14855,14886,14885,12837,10629,14853,14885,14917,19015,43787,33386,23048,27404,19051,12744,14857,16938,16938,16936,14886,6467,6403,14885,12836,17124,15012,19174,19174,8579,10692,8644,10725,14920,23210,21130,4323,4324,27532,23273,29517,29549,27529,83},
{57084,59196,59197,61245,59196,57052,55004,54971,54971,54971,57052,54971,52891,52859,50779,50779,50778,50778,40344,40280,40313,27734,31959,31959,34040,29847,36087,44538,29781,42425,38200,27701,25621,25653,25621,25621,27701,25653,25589,25589,29813,25621,23508,25620,27667,27667,27667,27667,27635,25587,25587,27667,25555,29716,27668,25587,29748,27699,25587,23506,23506,23538,23506,23506,23506,23538,23506,27666,27698,25618,25618,23570,23538,23538,23506,21426,25554,27634,29714,29746,27633,27633,44439,48698,50778,42359,44471,27601,8551,10697,14923,8552,10697,17004,8584,10729,17036,12778,17069,8584,10729,17036,10697,19117,10697,12777,19116,12745,14923,17003,31859,38198,40278,42358,54838,52595,46126,21065,29516,46159,21032,25290,46192,23112,23144,46191,23145,21097,46192,25225,21097,44111,27306,21065,42031,27338,18984,41998,31499,29613,35853,37901,25321,21160,35952,63390,52825,50712,52857,54970,54938,54970,54938,54971,54971,54970,54970,54970,54970,57051,54970,33744,44242,44242,44242,44274,44242,44274,44274,25356,19050,19049,19049,18985,19017,19017,19017,19017,19017,16904,14759,14759,12711,12710,12678,12711,12711,12711,12711,12679,12678,12678,12678,12678,10630,10630,10630,10598,10630,12710,14885,12805,8516,12709,12805,14917,16869,43787,35466,23048,29550,23277,10632,14824,14857,16937,16933,14916,8548,6467,14884,17061,14980,17092,19174,17061,10756,8579,8579,12743,12744,19018,21193,4323,4323,29613,27498,27404,29517,29517,105},
{57052,59164,59196,57084,57052,55004,57052,57084,55003,54971,54971,52891,52891,52859,50811,50810,50810,50811,42393,42392,57117,36184,31927,36119,36152,27734,27733,34039,29814,25621,27701,25653,25621,25621,25621,23541,25621,27702,25621,25621,27701,23540,23540,23540,27667,27699,29747,27667,27635,25554,25555,25587,29748,29748,27668,25555,27668,27635,23474,23539,23506,23506,23538,23506,23506,23506,27667,27666,27698,27666,27666,25618,25618,25586,23506,23506,25554,23473,31794,29746,29714,27633,44472,50778,50778,44439,42359,12908,10698,12843,14989,14989,10698,14956,10665,10762,12811,8617,12843,8585,12843,8682,8617,10730,10698,8618,12843,6504,10698,8617,31827,38198,40278,42359,54806,50515,31499,21066,21195,21196,21131,16937,21195,21163,16904,21163,19082,14792,19082,19050,14759,14759,12711,12646,14759,14759,12678,10598,12646,16903,21129,23274,21194,21161,38033,63357,52825,50680,52825,54938,54938,54938,54938,54938,54938,54970,54906,50680,52857,52825,50712,42162,44242,44242,44242,44242,44242,44274,44242,27437,19049,19049,19049,18985,19017,19017,19017,19017,19017,14824,12646,12646,12646,10598,12646,10598,10598,10598,10598,10598,10598,10598,10598,10598,10597,10566,10565,10565,10565,12678,10661,10661,6437,8517,14949,12868,16901,41707,35466,23016,27404,25259,10631,14792,14857,19079,14884,14884,10660,10692,14885,12740,12804,17093,14981,12836,17156,12868,8612,12839,14889,19050,21257,4323,4291,21194,25385,27436,31597,29517,114},
{54971,57052,57084,54971,57052,57052,57052,55003,54971,54971,54971,52891,52891,52891,50811,50811,52859,52891,40279,50778,52924,40312,40345,42425,38232,27766,31927,38200,34039,27733,27701,27733,25653,25621,25621,23541,23541,23541,25621,23540,23508,23508,23508,23541,27700,27667,29780,25587,25587,23507,25587,27667,27667,27668,29748,27700,27667,25555,27668,23506,23506,23506,23506,23538,25586,25586,25586,25586,27666,29747,29714,27666,25618,25618,25586,23506,25554,23473,29746,31794,29714,29714,44439,50778,50778,44439,42359,13040,12943,15153,15121,12975,15121,15120,15088,17201,15120,15088,15120,13008,17168,15120,15121,15120,17201,17201,17201,15055,17169,15120,31893,38231,40279,42358,54838,44210,21033,16807,14727,14727,14727,14727,14694,14727,14727,14694,12678,14727,12646,12646,14727,12646,12646,12646,12646,12646,12646,12646,10598,12646,16871,16903,14823,16999,38032,63357,52825,50712,52825,54938,54938,54938,54938,54938,54970,54970,54938,54938,57019,57051,54905,35857,37937,44242,44242,44242,44242,44210,44274,29517,19049,19017,19017,19017,19017,16969,16937,16937,16937,14824,12646,12646,12646,10598,10598,10598,10598,10598,10598,10598,10598,10598,10566,10598,10598,10598,10598,10565,10630,10597,8517,8517,8517,8517,14948,14885,20999,41707,35466,20936,29453,27275,12712,14857,19050,16965,14917,8547,6435,8515,14884,14917,14885,17061,19142,17029,17188,12804,8516,8517,14889,21163,21256,4355,4324,8517,12742,29517,29549,29517,116},
{54971,54971,54971,52891,54971,55004,57052,57052,54971,55003,54971,52891,54971,54971,52891,52859,52859,50811,31894,31894,29814,38232,31927,36120,29846,31894,36119,38200,27733,25621,27701,31927,27701,25621,25621,27766,25621,27733,23541,23541,23541,23508,23541,23508,27668,27667,29747,27667,25587,23507,23507,25555,29748,29748,29780,27667,25555,25555,25555,25555,23506,23506,23538,23506,23538,25586,25586,23506,23538,27666,27666,25586,27666,27666,25586,25554,25586,23473,25554,31794,29714,33843,46520,48698,50778,44439,42392,17267,15122,15154,17267,15186,13074,19347,15122,15186,15186,13074,17267,13074,15154,17234,15122,17234,15154,15121,17266,13074,15154,17202,31925,38231,40279,42358,52726,42162,18953,18920,8453,8453,16937,14792,19018,14825,14759,12743,12744,12711,10565,8485,12678,10565,8485,10598,10566,8485,8518,10598,6437,8517,14791,19081,19049,19080,38064,63357,52825,48631,31663,27470,25389,27437,23244,23211,25389,25389,23244,21098,21098,21098,31630,33810,17006,35889,42162,44242,44242,44242,44242,29517,12743,12743,12711,12711,12711,8550,8550,8517,12711,12711,10598,6404,8452,6404,10598,10598,10598,10695,10727,8517,10566,10566,10598,10663,12808,8582,10630,10565,12711,10565,8517,10597,10597,12741,14916,10692,18854,43787,35466,20968,29485,27242,14857,16938,16970,14853,10660,8580,12772,10660,6403,12772,12772,16967,19047,8516,8547,17061,4356,10663,12744,21163,14918,4323,6404,8518,10598,29549,29517,29517,115},
{54971,55003,54971,54971,54971,54971,57052,57084,57052,57052,54971,52891,52891,54971,54939,52891,50811,44505,40344,46618,46586,46586,38232,34007,40312,42425,34006,27701,27734,29846,27733,31894,29781,25621,25621,27733,25621,23541,23541,23541,23540,23508,23508,23541,27668,27667,27667,27667,25587,23506,25587,29716,29748,31860,29780,27667,25587,25555,25555,23506,23506,23506,23506,23506,23538,23506,23538,23506,23506,25618,27666,25586,25586,27666,27666,27666,25586,25586,25554,31827,31794,33875,44407,50778,50778,44439,42424,17267,13106,15154,17299,15186,13106,17300,13106,17267,17235,13074,17267,15154,15187,17234,15154,17234,15186,15154,17234,13042,13074,17267,31925,38230,40279,42358,50645,42162,18953,16840,10566,12614,16872,14759,14727,12646,14727,14792,14824,12679,12646,12646,12678,12646,10598,12646,12646,10566,10566,12646,10566,10566,12646,10598,14791,21161,38065,63389,52825,48599,48632,48632,48632,50712,50744,52825,52857,52857,52858,52793,44373,52793,50679,31730,17072,19151,27438,42097,40049,42161,44242,31630,12743,12743,12711,12711,12711,6437,4324,6437,12679,12711,10630,6372,6404,8485,10598,10598,10598,10662,10695,10663,10598,10598,10598,12776,12841,14889,23276,16936,21161,19080,17000,17000,14885,12837,10692,12741,23047,43787,35466,20968,29452,27307,19051,14825,16937,16999,12804,12772,10692,12772,10660,16965,12741,14854,14887,10629,8548,4291,6404,8485,8518,14857,6404,4291,6372,14826,12711,29550,29517,29517,118},
{54971,54971,54971,54971,54971,54971,57052,57084,57084,57052,54971,54939,52891,52891,52891,52891,50811,46585,44440,46617,40344,31927,31927,34007,36119,27701,27701,27701,27701,25653,29782,36087,34007,25621,23541,27734,25621,23540,25589,23541,23541,23508,21460,21460,25588,27667,27667,29747,25586,25587,29748,29748,27699,31860,25587,27667,25587,23507,23506,23506,23538,23538,23538,23506,23506,23506,25586,23538,23506,25618,27666,23538,25586,27666,27666,29714,27666,25554,23473,29746,31794,29681,40214,48698,48698,44439,44472,19347,15187,15187,19380,17267,15187,19412,15187,17267,17267,15187,19348,15187,17267,17267,13106,15187,13074,10961,17267,10961,13074,17235,29813,38230,40279,40278,54806,40082,18953,18920,8485,8486,14824,14792,8518,10598,14760,12743,12776,12711,10565,8485,10598,10566,8517,10598,10566,6405,8485,12646,6405,8485,12679,10664,12776,21129,35920,61309,52825,48599,31728,29583,27469,27502,25357,25324,25389,27470,27437,21131,19050,21130,31663,31762,15024,21297,14991,29583,42162,35823,42161,31695,14791,12743,12743,12711,12743,12711,12711,12711,12711,12711,12646,12646,10598,10630,10598,12646,10598,10598,10598,10598,10598,10598,10598,10598,10598,10598,10598,12678,14822,12741,10628,8515,8548,12773,12773,12741,25127,43819,35466,23146,29420,25097,21099,12712,14824,16969,14917,12772,12804,14885,16965,12772,10628,14823,12742,8548,12742,6436,8516,6436,8485,8485,4324,6437,8518,21164,23244,29517,29549,29517,118},
{54971,54971,54971,54971,54971,54971,54971,57084,57052,57052,54971,54971,54939,52891,52891,52891,50810,40312,44537,55004,34007,27766,34039,29782,27733,27701,25653,25653,27766,27733,29814,27669,29781,25621,23541,25621,25621,25588,27701,23540,23540,23508,23541,23541,23540,27699,27667,29747,27667,27667,27668,27667,29748,27667,25587,25586,25555,23506,23506,23506,23506,23538,23538,23506,23538,23538,25586,23538,23506,23538,27666,25586,25586,25586,27666,29714,29714,25553,25554,27634,31794,29714,40246,48730,48730,44472,44472,19380,15252,17299,19412,19380,17300,19412,17300,19380,17332,15252,19380,17299,17300,19380,17299,17332,17332,13106,19380,15219,15187,17267,27765,38230,38230,40246,54839,42162,18953,16807,12646,12614,16840,14759,12646,12679,14759,14792,14824,14759,12646,12646,12646,12646,12646,12646,12646,10566,12646,12646,10566,10566,12646,12679,16937,19017,33775,57051,50744,48567,46454,46486,44406,46486,46518,48599,48632,50712,50745,50712,46486,42260,40114,31762,15057,19249,15024,21264,23244,33742,40049,33808,14791,12711,12743,12711,12743,12711,12743,12743,12711,12711,12678,12646,12646,12646,12646,12646,10598,10598,10598,10598,10598,10598,10597,10565,10566,10566,10598,10565,14853,12805,12773,10692,8612,8548,8516,8517,25127,43787,35466,25227,27340,27177,23179,12712,14824,19050,16998,10724,10692,12772,12772,14885,14854,21097,21129,16936,27436,16936,19048,21129,25356,23242,23177,21097,25389,23211,27404,31662,27404,29517,115},
{54971,54971,54971,54971,54971,54939,54972,57052,57084,57084,57052,54971,54971,54971,52891,52859,50778,46618,46586,31894,27767,29847,29846,27733,29814,27734,27701,40312,29782,27733,33974,44473,52891,40312,29814,25653,25621,33974,31894,23540,23540,23540,25589,25589,25620,27667,27667,27667,29747,27667,27635,29748,31828,27667,25586,25587,25555,25587,23506,23506,25587,23538,23538,25586,25586,23538,23506,23506,23506,23538,25586,27666,25554,25586,27666,29714,29714,25554,25554,27634,31794,31762,40213,48697,48698,42391,44472,19348,17267,17299,19412,17300,17299,19412,17267,19380,19348,19347,19379,17267,19347,19379,17267,19347,17299,17267,19380,15186,19347,17267,29812,36053,25455,25455,29583,29550,18953,14694,14694,14694,14694,12614,12581,10533,10533,10533,10533,10533,10533,10533,10501,10501,10533,10501,10501,10501,10501,8453,8453,8453,8453,16903,19048,23210,25356,54938,52793,48599,16905,16970,12712,29583,54938,54938,54938,54938,54970,57018,42228,27502,33776,29649,17105,17169,17104,19216,14990,33776,44275,40114,16872,14824,12743,12743,14791,12743,12743,12743,12711,12743,12711,12646,12678,12678,12646,12646,12711,10598,12646,10598,10598,10598,10630,10598,10598,10598,10565,10565,12774,14885,12836,10724,8612,8548,6403,8485,27176,43787,37514,23179,27339,25128,25259,12712,14792,19017,14885,10692,10660,10692,12804,14884,14852,16935,10628,8516,21227,12741,12773,8549,19115,21195,16969,19115,19149,16971,25389,35921,27404,29517,114},
{54971,55003,54971,54971,52891,52891,54971,57052,57084,57084,57084,57052,54971,52891,52891,52891,42392,36087,29814,36120,38265,36119,27734,34039,36087,27734,27733,40280,29781,31861,40280,50746,59164,57117,27733,25621,27701,38167,25620,23540,25620,29781,25589,27669,27700,27667,27667,27667,27699,27667,27667,27667,31860,25619,25586,25587,23506,25554,25586,25587,25555,23506,23538,23506,23506,23538,23538,23506,23506,25586,25586,27666,25586,25586,27666,29714,27666,27634,27634,25553,31827,31794,38165,48697,48697,42391,44504,19380,15187,15187,19412,17267,15187,19412,15187,17299,17299,15154,19380,15187,15187,19348,15155,17267,17267,15154,19380,15122,15154,15154,27732,33972,19181,19149,21197,25357,18921,14694,14694,12614,12614,10533,10566,10534,10534,12614,12646,14726,12678,12645,12646,12678,12646,10565,10565,10565,8485,8485,8453,8453,10598,14791,12678,8485,25356,52825,52825,48599,14857,17002,12744,29615,54938,54970,54970,54970,54970,57018,59131,40082,42195,31730,14959,21297,14991,19217,17104,27536,46323,44274,52693,50678,21228,10565,12646,12678,12711,10663,8518,8518,14824,8485,10598,10598,8485,8485,14792,8485,10565,10598,8517,8485,10631,8485,8517,8517,8485,8485,8550,10693,10660,12837,10724,10692,8580,8484,29256,43787,37547,18985,27340,25096,25259,10631,10599,14855,10660,12804,12804,10660,10692,16965,12804,14853,12773,10692,14885,12805,12805,8516,14856,19081,12837,10761,8618,6438,19082,35888,27404,29517,116},
{54971,54971,54971,54971,54971,54971,57052,57051,57084,57084,59132,57052,54971,54939,54971,50811,36119,31927,34007,38232,27734,29846,27734,29781,29814,27701,31927,34007,36087,50779,48665,48633,54971,36087,25621,25621,34006,36086,25588,23540,25621,25621,25588,25588,25588,27699,29747,27667,29747,29780,27667,27668,27667,23539,23506,25587,25586,25587,25587,25586,25586,23506,23506,23506,23506,23506,23506,23506,25586,27667,25586,27666,25586,25586,27666,29714,27666,25586,27634,25554,29714,31794,38133,48697,48697,44439,44504,21460,15187,15187,21460,17267,15187,21492,15187,17300,19347,15187,19412,15187,15219,19380,15187,19347,17267,13106,21427,15155,15186,17234,27700,33972,19149,19149,19149,23244,18921,14694,12614,12614,10565,10533,10566,10534,10566,12679,21065,27369,18982,14789,12677,12677,12709,12677,10629,10565,8517,8485,6405,6372,10598,10598,8485,8485,25450,52825,52825,48631,14857,14890,10663,27534,54938,54970,54970,54970,54970,54970,59131,48500,46453,29650,14991,19216,17072,21330,15024,29649,44210,42130,48467,42161,42294,10598,8485,10565,10598,10598,8518,8518,14824,8485,8485,8517,6405,6405,12743,8485,8485,8485,6437,6404,10631,8452,6404,8485,6437,6437,10663,10630,10662,10757,12869,12837,12805,8516,31369,43787,37547,21066,29453,23049,25227,12679,12744,16969,19112,16997,14884,14917,14853,12805,8547,10628,12740,8515,12804,12772,12805,12774,16970,23339,17094,14987,8618,8518,27502,35888,29485,29516,117},
{54971,54971,54971,52891,54971,57052,57084,57051,57084,57084,57084,57052,54971,54971,54971,42392,36087,36152,34040,27702,27767,29846,31926,29846,29814,29781,34039,36119,31894,31861,42360,48665,42392,25621,25621,27701,38167,29781,25621,25621,25588,23540,27669,27701,27669,27668,27667,29747,29715,31828,27635,25587,25587,23506,25555,25587,25586,25587,23506,23538,23506,23506,21458,23538,25554,25554,25586,25586,27634,27667,25618,27634,27634,25586,25586,29714,27634,25586,27634,25554,27634,31827,38133,48665,48665,42359,44504,27700,15187,17267,23540,17300,15220,23540,15187,17300,21427,15219,23508,15186,15219,21459,13074,19347,17266,13074,21427,10993,15154,17234,21394,25553,19116,19116,17068,19084,16905,12614,12614,10565,10533,10533,10566,10566,12679,12647,21065,25225,23176,18983,21063,16903,14758,12710,10565,8517,8485,8485,6405,6405,8517,10566,8485,10598,25418,52825,52792,50680,14857,14857,8550,27502,54906,54938,54938,54938,54938,54938,59130,46420,46388,27537,17072,17168,15056,19217,14992,31696,44242,40114,25225,23178,38068,21229,10565,10565,8517,8518,6437,6437,14824,10565,6404,8485,6405,6373,12776,10565,6404,6404,6437,6437,12744,8485,6372,6404,6405,8518,16937,10630,10630,10630,12838,12806,16998,19048,35562,41675,35531,16873,29452,25129,29420,16937,12744,16937,19048,14853,14885,14917,12804,12773,10628,10692,12804,6403,6371,12805,8516,10630,10663,23371,19175,14920,8553,21164,29582,35889,29517,29517,116},
{54971,54971,54971,54971,54971,55003,57084,57051,57052,57052,57052,57052,54971,54971,54971,36087,34072,31959,25686,25686,27734,31894,27734,31894,31926,31926,29814,31926,40312,33974,44440,52891,31926,25621,25621,27701,31894,29781,25621,23540,29781,25621,25620,25588,27701,29749,27668,27667,27667,29748,29748,29780,27699,27667,25587,25555,23506,23506,23506,23506,23506,23506,23506,25586,25586,27667,27634,25586,25586,25586,25586,25586,27634,25586,25586,25618,27666,25586,25586,27634,25554,29746,38165,48665,48665,42359,44504,27700,13074,15154,23507,15187,13074,23507,13041,15154,19314,13041,21394,13009,10895,21394,10896,17169,15089,8783,21394,8815,12975,17169,8783,19215,19116,19116,17036,17003,16872,12614,10565,10533,10533,10533,10566,10566,14759,12647,14758,18983,18918,21031,21064,16903,12710,10597,8517,8485,8485,8485,6404,6372,8485,10598,8485,8485,23211,50745,52857,48631,38067,38099,38099,46486,54905,54938,54938,54938,54938,54938,57050,48500,46420,35889,14959,21298,15024,19216,17104,29648,44340,44373,21097,23178,21163,40213,10631,10597,6405,6405,6373,6373,21163,10598,6404,6372,10631,10663,14857,10598,6404,6372,12743,12776,14856,14791,12711,14823,16936,21098,27469,21130,21162,21162,21162,21161,27468,27435,35595,43787,33451,14792,29420,25129,29355,14825,14825,14857,19050,19050,16966,14917,16997,14852,14884,12804,14852,6403,10628,12773,10629,17035,17035,27565,21289,25452,6440,21131,38002,33841,29517,29517,116},
{54971,52891,54971,57052,55003,54971,57084,57052,55003,57052,57084,57084,57052,54971,52923,31927,31927,27766,25686,27734,31927,31926,31894,38232,31926,34006,38199,40247,46585,38199,38134,46553,31926,25621,25621,27701,27701,33974,27733,23540,23540,25621,27669,23540,23508,25589,27636,27667,27667,29748,31860,33941,25587,23506,25586,25555,25555,23506,23506,23506,25586,25586,27634,27635,25586,25586,25586,25586,25586,25586,25586,23506,25586,25586,25586,25586,29714,27666,23505,25554,25554,25553,38100,46585,46585,42326,44472,25620,6703,6735,21394,10896,6735,21394,6702,10895,17201,6702,19281,8815,8815,21362,6702,15056,12975,6571,19215,6603,10829,15055,6603,19182,19116,17068,17036,16971,16872,10566,8453,8453,6373,8453,10566,10566,12646,12679,18952,25257,23144,23144,23176,18983,12709,12677,8517,8485,8485,6404,6372,6405,10598,10598,8485,10630,25387,52857,52825,48599,19050,16938,25422,23276,14792,14792,16905,19018,16905,33776,57050,48533,48500,35889,35857,21265,15024,21330,15024,31729,44340,46487,19017,23177,21098,38100,21229,12678,10630,12711,14791,16904,27437,23210,21098,19050,23243,25356,31694,25323,21129,21130,21130,21098,40114,23210,16871,16839,16904,12710,46420,50612,50612,50612,50580,50547,46322,44241,42030,41707,37579,16905,29387,25129,27209,19018,12744,14825,16970,19018,16970,19111,14884,17030,14916,12804,14853,6435,10628,17029,8581,14857,10631,23338,19176,33871,23274,27501,38034,35889,29582,29517,114},
{54971,52891,54971,57084,57084,57052,59132,57084,57052,57052,57084,57084,57052,57052,57116,31894,34039,31959,29847,29814,34007,34006,34039,36054,42392,36087,33974,52891,40247,44473,46585,36087,25621,25621,25621,29782,36055,36119,23540,25621,25653,25588,25621,27701,25620,23508,23508,27636,29715,29715,33908,31828,25587,25587,25586,23506,25587,25555,25586,27635,29715,29747,27667,25586,25586,25586,23506,23506,23506,23506,21426,23506,25586,25586,25586,23538,27666,29714,25554,25586,25586,25586,38101,46584,46584,42326,44472,25587,8848,8880,21394,13041,8880,21427,8848,10993,17233,8880,19314,10960,8880,19346,8880,12975,12876,6538,19182,6537,10730,14956,6537,17069,19117,17036,14956,16971,16872,10533,8453,8453,6405,8486,10566,10566,10566,12679,16839,18951,16870,16838,14726,14758,10565,10597,8517,8517,8485,8485,6405,8485,10565,10565,8485,19111,19079,52792,40180,27438,25357,25422,25390,25422,27470,27503,27535,27535,29615,27470,52759,48565,48500,25457,25359,23377,25456,27470,27503,33710,38001,42228,21098,23210,27371,33808,42294,16872,25290,25291,23242,23178,46420,21031,18984,21065,16872,23145,42194,18951,16838,16903,21065,21129,42194,25322,27403,29516,31662,33742,48532,52693,52693,52693,52693,52660,46354,48434,42128,41966,33451,18985,27307,23016,25129,21131,10631,12744,16970,16970,19018,19049,16967,14886,14852,12772,19078,14917,10660,12740,6436,10630,8518,14887,19175,29612,33869,33773,38001,33841,31695,27436,116},
{54971,54971,52891,54972,57052,57084,57084,57052,54971,57052,57052,57052,57052,57051,59164,57084,48698,29814,27734,27766,31926,36119,29814,29814,29814,27766,34007,40280,44473,34006,29781,25621,27702,25621,29814,29781,25620,25589,23573,23508,23541,25621,23508,27669,27668,27701,25620,27668,29748,27667,31828,29780,29747,27635,25587,25587,25587,27667,27667,25554,27635,27667,25586,25554,23474,23506,25587,23507,23507,21458,23506,23506,25586,25618,23506,23538,25586,29746,25586,25586,25586,25554,33940,44504,46552,40246,44439,25587,6702,6702,21394,10928,6799,21394,6670,8783,17201,6670,17168,10830,6637,19281,6669,10797,12778,4326,19117,4358,8584,14858,4358,17036,14924,12844,12843,14891,16873,10566,10566,8486,8518,10599,10566,10566,12614,12646,16871,18983,21063,12677,14758,12678,10597,12678,8484,6436,6436,6404,6404,8485,10597,10565,8485,19111,23368,48599,42261,38002,38067,40148,38067,40147,42228,42260,42293,42292,44373,44373,42195,50679,48565,27570,14991,21297,35921,33709,35789,35789,35823,31728,54872,56951,56951,56952,46518,31629,50580,50612,44241,46322,50613,50580,50580,48532,46419,48500,52693,52725,54838,54838,54838,56919,54871,54871,52726,54774,54774,54774,52693,52725,52693,52693,52693,52660,46354,46386,42128,42161,35725,31597,31598,21034,18953,21099,10631,12744,16970,16970,16970,14825,6405,8485,19047,16997,16997,14853,12773,12772,8548,10662,8517,10661,14949,14887,31757,33806,35953,33841,33808,27404,117},
{54971,54971,54971,54939,54971,57052,57052,57084,54971,54971,57052,57052,57052,57052,54971,57084,48698,38200,29847,36152,38232,36151,31926,29813,27766,29814,31926,31894,34006,42392,46618,46618,27701,25621,29781,27701,29814,27669,23541,23540,25621,25588,23508,29781,29781,27668,27668,27668,27700,25587,27667,31860,27667,25587,25587,27667,29747,27634,27667,27667,27667,25586,25586,23506,23507,25587,25555,23475,23507,23506,25586,23506,25586,25586,23538,23538,23506,27666,25586,25554,27634,23506,31859,44504,46552,40246,42391,25587,12975,17168,23506,13008,15120,25619,12975,17168,23442,17168,23474,17103,19183,23409,17201,21361,19149,17003,23343,14923,21294,19148,14955,21262,12908,12876,12844,14891,14824,10534,8453,8453,8486,8519,10566,10567,16840,14759,16871,18983,16870,16903,14790,12710,12677,12710,8517,8485,6436,8517,8517,8517,8485,8517,12678,16967,25513,44405,50712,50679,54937,54938,54938,57018,54938,57018,56986,54937,54938,54938,54873,54937,50679,25489,15024,21264,40082,35757,35789,35789,35822,33776,54871,56951,56951,56951,52726,54839,56951,56919,56919,56951,56951,56919,56919,56919,56919,56919,56919,56919,56919,56919,56919,56919,56919,56919,35693,37805,35725,37805,37837,35692,35692,33579,35692,35757,33645,33645,29419,31532,31531,29451,35823,21131,21131,23212,12712,12744,16970,16938,16970,16938,12744,12679,31662,19048,14886,16999,14854,16966,10629,10663,10662,12805,10660,8516,33838,33806,35953,33841,33808,27404,115},
{54971,54971,54971,52891,52891,55004,54971,57052,55003,54971,57052,57084,57084,57052,54971,44505,46585,34007,27734,29814,34039,29814,38199,42393,29814,36119,31894,33974,48698,36119,27701,36087,25621,25621,31926,27733,31862,38200,25621,25621,25620,25621,23540,29814,25588,27701,25620,27668,29780,27668,27635,29747,33908,27635,29715,29715,27667,27667,27667,27667,27667,25586,25587,25555,25555,25587,27635,25587,25587,25587,25586,23506,25554,25618,23538,23538,23506,25618,27666,27634,25554,23506,29779,44471,44471,40213,42359,25587,13008,15121,21426,13041,13073,23474,13008,15153,19313,15153,19313,15121,13041,21361,13041,15153,17168,8683,19182,8650,10795,14923,8650,17101,14957,12876,12876,14923,14824,10534,8453,6405,8453,8486,10566,10566,16872,14727,14759,14758,12678,14790,14726,12678,12678,12677,10565,8517,8517,10565,10565,10597,10565,8485,10566,14791,23273,44373,50712,50679,54905,54905,54937,54938,54938,52825,52825,52857,52857,52825,54905,54905,48566,25457,17104,19216,42227,35757,35789,35789,35822,33808,54871,56951,56951,56951,56951,56951,56951,56919,56951,56951,56951,56951,56919,56951,56919,56951,56919,56951,56919,56919,56919,56951,56919,56919,35757,37837,37870,37805,35725,37837,39950,39950,42063,42031,39983,37838,31564,33645,35725,35790,35855,23243,23244,23276,12712,12744,16938,16970,16970,16970,16938,14825,25324,19018,12743,16937,14824,14824,14824,14857,14855,14919,12742,14855,27436,27436,25388,23243,21195,27436,91},
{54971,54971,54971,54971,52891,54971,54971,57052,57051,57052,57051,57052,57051,57051,54971,46585,40247,34039,31926,31894,36151,34007,38167,40312,31894,27734,29813,44440,59197,48665,40312,34039,25653,27701,42392,31893,25588,25588,36087,27733,25588,25588,25588,25621,23541,25588,29780,25620,27701,25588,25587,29747,29747,31828,33908,33908,29747,27699,25586,25586,25586,25586,25587,25555,25587,21395,23507,25555,25555,23507,23506,23538,25586,25586,25586,23538,23506,25586,27666,27634,25554,23473,29779,44439,44439,40213,42326,25555,13008,15153,23507,12975,17201,23506,15055,17200,23441,17201,23441,17168,17168,23441,17168,21361,19215,14989,23407,14891,17102,19149,14955,19182,14989,12909,12876,14923,14825,10534,8453,8453,8486,8486,10566,12614,14759,12647,18952,21096,18950,18951,18983,18983,16870,19015,10629,8516,6436,8549,8516,8516,8485,8517,12711,18984,18983,42293,50680,50679,54905,54905,54937,54938,54938,54905,52857,52857,54905,54905,54937,54938,50646,25490,14991,23409,42292,35789,35789,35789,35822,35889,54871,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56919,56951,56919,56919,56919,56919,56919,56919,37837,39918,46289,50612,50547,50612,50612,50611,50579,50612,48434,48434,46321,44240,46288,44208,42129,23243,23244,25357,12679,12712,14857,16970,16970,16970,16938,14857,10599,19050,14824,16904,14824,12711,10631,14857,14887,14916,12772,10660,35952,44339,31727,21193,19081,12773,46},
{54971,54971,54971,54971,54971,54939,54971,57052,57052,54971,57052,57052,54971,54971,54971,52891,44472,36119,38199,44505,36054,31894,31894,31926,31894,46585,50811,42392,36054,33942,48698,25653,25621,27701,29813,36119,25621,27701,25588,29782,25588,27701,31894,27700,25588,25588,27701,27668,27732,23540,23507,29748,33941,31860,31795,33908,27667,25586,25586,25586,25587,25587,25587,27667,29748,25588,23475,27635,27668,25555,25587,23506,21458,25586,25586,23538,23506,23506,25618,27666,27634,25554,29747,42359,42359,38133,40246,25587,10895,12943,21426,8783,10895,19281,10862,12943,19248,12943,17168,12975,10862,21329,10830,15055,15023,8650,19182,6471,10697,14891,8552,14988,15022,14989,12908,14923,14824,8486,8453,6373,6405,8453,10534,10566,10566,14727,16839,21064,18951,14758,18951,18951,14758,14790,10597,8485,8517,8517,10597,10597,8517,8485,12678,14758,10533,42228,50679,50647,54905,54905,54905,54938,54938,54938,57018,52824,57018,57018,57018,57018,50647,23442,14991,21297,46454,35790,29484,23210,25323,29517,54871,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56919,56919,56951,56919,56919,56919,56919,54838,35692,37838,46354,52660,50579,50612,50612,50612,50612,50579,48434,46353,44208,42063,39982,37902,37936,23243,23244,27469,12712,12712,14857,16970,16970,16970,16938,14857,10631,19083,23276,25324,23276,23276,21163,27469,29646,23335,17061,14981,33837,31725,16966,19142,12804,14916,48},
{54971,54971,54971,54971,54971,54971,54971,52891,54971,54971,54971,54971,54971,54939,52891,52891,44473,42360,31926,33974,31894,29814,38200,27733,36055,59197,44472,46585,27766,25621,25621,25621,23573,29814,36087,27701,31894,27701,29813,25588,29781,29749,33941,38167,27700,25620,25588,25620,29812,27700,25588,25555,31860,33941,29747,29747,27667,27667,25586,25587,27667,23507,27635,29748,33973,29748,29748,27668,25555,25555,25555,25619,21458,23506,27634,25586,23538,23505,23538,27666,27634,27634,29746,42326,42326,38100,38165,23539,10960,8848,21394,10928,8848,17233,10928,8880,19281,8847,15088,13040,8847,19281,8847,10927,15120,8782,17134,6537,8682,14956,6569,14989,15054,14989,12908,14923,14825,8519,8453,6405,6405,8453,10534,10533,10534,10534,10534,10533,10533,10533,10533,10534,10533,10533,8485,8485,8485,8485,8485,8485,8485,8453,8485,8485,8453,46486,50679,48599,54905,54905,54937,54938,54938,54938,57018,48534,52825,52825,52857,52825,48533,23377,15056,21297,44406,33709,25323,18952,18985,25356,54871,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56919,56919,56919,56919,56919,56919,56919,54806,35725,37805,46354,50612,50579,50612,50612,50612,50612,50579,46353,48434,44240,44208,44240,42160,40048,23244,23244,25356,27470,12712,12744,16970,16970,16970,16970,14825,12712,27567,27502,31630,31695,31630,29550,35953,46452,27530,19141,14884,17060,10692,14917,12836,12836,14916,40},
{54971,54971,54971,54971,54971,54971,54971,54971,54939,52891,52891,52891,52891,52891,52891,52891,48698,36087,52891,44537,36087,38200,40280,36152,31894,44473,38167,29781,31894,27734,25653,23573,25621,29814,29781,23541,23541,27701,27733,29781,27701,33974,46553,46552,42392,40247,27701,27700,29780,29781,29749,25588,27667,33973,31860,25586,27667,25587,25587,25587,27635,31860,31828,33940,31795,29715,31828,31828,33941,31860,29780,27699,23506,23506,25586,25586,23538,23537,23505,27634,27666,27634,27634,42326,40246,38100,36052,25554,15055,10895,19281,10895,10895,17168,12943,10862,19280,8782,15055,15055,8782,19248,8749,12942,15055,8749,17102,6570,10796,14989,8617,12843,17134,15054,14989,14955,14825,8552,8453,8453,6405,6373,8485,8453,8485,8485,8485,8453,8485,8485,8485,8485,8453,8485,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,38067,48599,48599,54905,54905,54937,54938,54938,54938,54938,52825,57018,57018,57018,46389,46421,25457,17136,23408,50712,46387,46387,46419,48499,52825,54871,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56919,56919,56919,56919,56919,56919,56919,52726,42031,41999,46354,52660,50579,50580,50612,50612,50612,50579,44273,44273,44208,44208,44208,42128,37936,23244,23244,23276,27502,12744,12744,16970,16938,16905,16938,16938,14792,31793,29583,35921,40114,40146,38001,40147,35951,27498,17029,14948,14916,12836,12836,14916,10724,10692,32},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,50843,36086,36086,29814,34039,31894,29813,31894,40312,44537,46585,44505,57117,27701,25621,25621,25621,25621,27733,25621,23508,23508,27701,25589,29781,31861,38102,52891,46585,42327,31894,29781,33942,38167,29813,27669,25587,31828,33940,25587,27667,27667,27667,33941,38101,38101,31860,31860,29715,29747,27667,27635,23506,25554,25587,25555,23506,23506,25586,25618,25586,23538,23505,25586,27666,27634,27633,40246,40213,36019,36020,21360,10763,6505,17102,6537,6505,12844,8617,6473,17069,6472,10730,10763,6472,17069,6538,8617,12844,6537,17069,4391,8618,12843,6439,12843,19215,17102,14989,14956,14824,10632,6405,6373,6373,6373,8453,8453,6340,6373,8453,6373,4292,8453,8453,6340,6372,8453,6373,4292,6373,8453,6340,4292,8453,6373,6340,6373,8453,38067,48566,50679,54905,54905,54937,54938,54938,54938,54938,54938,57018,57018,57018,54938,50646,23377,21329,33907,59164,50612,50612,50612,50580,50711,54871,56919,56919,56919,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56919,56919,56919,56919,56919,56919,52693,44143,44144,48434,52692,50579,48500,48467,50612,50612,48434,37902,35790,33677,35789,35789,35822,29517,21131,21131,21098,21131,16970,12744,14857,16938,16937,16938,16970,14824,31728,33776,42260,44372,44340,42260,44372,23304,17029,17029,17092,17061,12836,19142,10692,6467,6499,37},
{54972,54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54939,52891,52891,52891,52891,50811,42392,40279,34007,31894,54972,61310,52956,44472,63390,50810,33973,52859,36119,27701,25621,27701,23541,23541,25621,25621,23540,25621,25621,25589,25620,29781,40215,46552,46553,40247,36054,40247,33974,23540,23508,25587,27667,33941,29715,31828,36021,40214,40214,40182,33908,29747,27666,27667,27667,25586,23506,23506,23507,23475,29780,25587,25586,25586,25586,25586,23538,23505,25586,27666,27634,29682,40213,40213,38100,36019,21295,14956,8683,17134,8617,10731,14956,12811,10763,17069,8618,14988,14923,10730,19182,8650,12875,14956,10763,19181,8585,12876,14956,10698,17069,19215,17102,14989,17004,16873,12712,6405,6373,6373,6373,8453,8453,6372,6373,8453,6373,4292,8453,8453,6373,6373,8453,6405,4324,6373,8453,6373,4292,6405,6373,6340,6373,8453,35921,46486,50679,54905,54905,54905,54938,54938,54938,57018,54938,57018,57018,57018,54905,46421,23377,23410,29683,52825,50612,50612,50580,50580,50679,54871,56951,56951,56951,56951,56951,56951,56951,52726,54838,56919,56951,56951,56951,56951,56951,56951,56951,56919,56919,56919,56951,56919,50580,39951,44144,48434,52692,50579,50612,50612,50612,50612,50579,48466,46353,44240,46320,46320,42160,37936,23244,23244,23277,23276,19083,14824,16937,16970,16938,16970,16970,14857,31761,38034,38001,42259,42227,40212,40178,21223,16998,14884,14948,29675,14948,17029,8579,10692,8547,40},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,46586,40279,52923,31926,36087,42392,40279,38134,48666,61310,61310,61342,61277,55036,29814,25653,27668,34007,25621,25621,25621,27733,27701,23540,25621,25621,27733,25556,44473,31860,40247,38134,33941,23508,23540,25588,27668,27667,29747,38134,42295,40182,33940,36021,33940,31860,27699,27699,27667,27667,25586,23506,23506,29748,25555,25555,25587,25586,25586,25586,23538,23538,23538,25586,27666,27634,29714,38133,38132,35987,33874,19214,14891,12843,19181,6471,12810,14923,14858,12810,19117,12778,14988,14858,10697,19117,8616,12810,14922,10697,19117,6471,12778,14923,8584,17036,25554,17134,14989,17036,16873,10534,6405,6373,6373,6373,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8453,8421,6405,6373,6373,6373,6373,6373,8453,33841,46454,50679,54905,54937,54937,54938,54938,54938,54938,54938,57018,54905,50647,52792,44340,23377,15024,23410,44374,50612,50612,50580,50580,50646,54871,56919,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56919,56919,56919,56919,56919,56919,56919,50548,37806,37837,48402,52660,50579,50612,50612,50612,50612,50547,48466,48466,42095,40015,40015,42096,35823,23244,25324,25325,23244,19116,12712,14825,16938,14857,16938,16970,14857,35987,46486,48599,48599,48599,50777,46550,19141,14917,12868,14948,19110,21286,14980,14980,14917,12773,41},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,38199,38199,46585,29814,27701,36119,38167,42392,40280,59164,63390,63390,61310,44473,25621,25653,29814,36087,31926,29781,27669,27701,27701,23541,25588,25588,31894,44473,46585,46553,44440,36054,23508,23508,27701,25620,23507,23507,27667,27666,29779,31827,27634,27666,29747,31827,31827,27699,25554,29747,25618,23538,27667,29715,31860,25587,29748,23506,23506,25586,25586,23538,23505,23538,25586,27666,29714,38100,38100,36019,31761,19214,12843,6504,14924,8585,6504,10698,10730,6504,14956,6472,10698,12810,6504,14988,6504,8617,12810,8585,14988,6471,8617,12810,6504,14924,31925,23408,19215,23342,25259,12614,6405,6373,6341,6373,8453,8453,6373,6373,8453,6373,6373,6405,8453,6373,6372,8453,6373,6340,6373,6373,6340,4292,6373,6373,6372,6373,8453,31696,48599,48599,25356,27469,27534,44405,52825,50744,52825,52825,52825,54938,54905,48565,44307,23345,15023,19249,40181,50580,50580,50579,50579,50646,54871,56951,56951,56951,56919,56919,56919,56951,56951,56951,56951,56951,56951,56951,56951,56951,56919,56919,56919,56919,56919,56919,56919,48467,37838,37805,48467,35790,46354,50612,50612,50612,50612,50547,48466,48466,44208,40015,42095,42096,35823,23244,25357,25324,8486,12777,12712,14857,16937,14857,14889,16906,14857,40212,48598,52824,50711,50712,52890,48664,25545,12899,15012,19205,17029,12772,12836,14948,12804,10660,45},
{54972,54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54939,54939,52891,52891,48698,29814,34007,31927,27734,29781,40344,36086,55004,55004,46520,61309,61277,46585,31894,27766,25621,25620,31894,27701,31894,31862,29781,36087,31894,34007,36087,40247,50745,46520,42359,36054,25620,23540,23540,23508,25588,23508,23507,27667,27667,27699,29779,29747,31827,33940,27666,27667,25586,27699,27667,25586,25586,25554,27635,27635,27635,27667,23538,23506,23538,25586,23538,21425,23538,23538,27666,27634,38100,36020,36019,36019,21262,14891,6471,12810,8584,6439,10665,10697,6504,14955,6438,8617,12777,6439,14923,6439,8552,12778,6471,14923,4326,6503,12778,6439,14955,36085,36053,36053,42293,50483,31499,8453,6373,6373,6373,8453,8453,4292,4292,8453,6373,4292,6373,8453,4292,4292,6405,6373,4292,6373,6373,4292,4292,6373,6373,4292,6341,6373,33808,48632,48599,10566,12776,21228,42293,54938,52857,52857,52858,52857,54970,54905,48533,44307,21232,17072,19217,27570,33842,33809,50580,50580,50614,54871,56951,56919,56919,56951,56919,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,54871,56919,56919,56919,56919,56919,56919,48467,39951,42096,48434,46386,48467,48466,50547,48499,48466,48434,44208,44208,39983,40015,40015,37903,31630,23244,25357,21098,12777,12777,12744,14857,16937,14825,14890,16938,14825,38100,46453,46518,44373,44405,44406,38132,36047,17093,14948,12835,14948,19174,14917,14948,14981,10724,46},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54939,52891,52891,52891,36119,31894,36119,25653,42393,50811,46618,42425,36087,40279,57084,59165,34006,29814,29846,27733,27734,25621,27701,25621,27701,27701,27701,27701,25588,27669,25588,42328,50746,50746,38199,29781,25620,25588,23508,23508,23508,21460,23476,25555,27667,25618,29747,31827,31828,27667,25618,25618,29779,29747,29779,27667,27667,25586,25554,27635,23506,25554,27635,25586,25586,25586,23506,21457,23506,23537,25586,27601,35988,36020,35987,36019,21262,14891,6504,12810,10697,6504,8617,12777,6536,14923,6471,8617,12810,6504,14956,6472,8585,12810,8585,14923,6471,8617,12810,8617,19182,36086,36085,36053,42293,50483,33547,8453,6373,6373,6341,8453,8453,4292,6340,8453,6373,4292,6373,6373,4292,4292,6373,6373,4292,6373,6373,4292,4292,6373,6373,4292,6373,6405,27502,48631,48567,8486,14857,23373,44438,54938,54938,54938,54970,54970,57018,54872,48500,44307,21264,14991,19249,17104,19217,23377,50646,52692,50614,54871,56951,56919,56919,56919,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56919,56919,56951,56919,56919,56919,56919,46387,35757,39918,39951,42063,42063,39950,39983,42064,42063,42096,39950,42031,37805,37838,35757,35791,31630,23244,27469,21163,27503,16970,12744,14825,16938,14825,14857,14890,14857,40180,48631,52824,48631,50679,52825,50712,48631,27562,14948,12868,17061,14916,12804,12804,12836,12804,47},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54939,52891,52891,52891,48730,31926,29814,29814,31959,34007,27701,29846,31926,31894,31894,48666,50843,27733,27701,29846,27701,25621,33974,34006,38199,29782,29814,25589,23541,29782,40312,44473,38167,33941,40247,29748,27700,29781,25620,23540,23508,23540,23508,23508,25556,27699,25618,25619,27667,27634,27667,29747,27667,27667,29747,31827,29747,27666,25586,25554,25554,25586,25586,27635,27667,27667,25586,23506,23538,23505,23537,25586,27634,31827,31827,33875,31762,21328,17102,12876,14956,14989,10796,12876,14989,10796,17102,8715,10796,14989,8683,17102,8683,8748,14989,8716,15021,8683,8716,14989,10828,17102,36118,36085,33972,42293,50450,31467,8453,6373,6373,6373,8421,8453,6373,6373,6373,6373,6373,6373,6373,6373,6373,6373,6373,6372,6373,6373,4292,4292,6373,6373,19050,25357,25357,31696,48599,46519,8486,10631,10664,38099,54938,52858,52858,52890,52858,57018,54905,42194,37968,21297,17072,19217,17136,19217,17104,50645,52692,50613,56919,56919,56919,56919,56919,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56919,54871,56919,56919,56919,56919,56919,56919,46354,37805,39918,37870,42030,39983,35757,35757,37837,37805,37870,37870,37838,35757,37837,37870,33677,29517,23244,25324,19083,12745,12776,14825,12744,16938,14857,14825,14857,16970,42260,48631,52824,48630,48566,50678,48598,48598,40176,17028,17029,17029,12868,12868,12836,14949,16997,41},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,42360,31926,31894,29846,25654,25653,27766,31926,36152,31959,29814,38199,31894,36087,27734,25621,25589,25621,34006,27733,25621,27733,27733,25621,23541,23508,31894,25621,25653,31893,27701,27668,25620,25620,25621,21460,23540,23508,23508,23508,23508,25555,27699,25586,29747,29747,27667,29747,29747,29779,27666,29779,29779,27666,25618,23506,23506,25586,25587,25554,25554,23506,23538,23505,25586,23506,23505,23505,27666,27634,25586,27634,27602,25489,21361,17201,19248,21361,17168,17168,19313,17200,21360,13007,13007,19280,13039,19248,12974,13006,17134,13006,17167,12974,12974,15054,10829,15053,36118,36085,36052,42293,48403,27307,8453,6406,6373,6373,6373,8453,4292,4292,6373,6340,4260,6373,6373,4292,4292,6373,6372,4292,4293,6373,6373,6373,6373,6373,25357,29615,27502,27470,48599,48599,23276,25422,27567,44373,52825,52825,52825,52857,52857,52824,54905,40114,40081,19152,19217,17136,19217,19217,19184,50645,52692,50645,56919,56951,56951,56919,54871,56919,56951,56951,56951,56951,54838,56919,56951,56919,56919,56919,54838,54871,56919,54871,56919,56919,54838,56887,54838,56887,56887,54806,56919,54839,56919,56919,54838,54806,54806,52725,46354,42096,40048,35790,27437,23244,25389,12810,12745,12744,21196,12744,14824,16937,14857,16937,16970,42293,50679,52824,48598,48598,50678,48597,48598,40176,19173,21318,19141,12772,12804,10724,12836,14885,49},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,42392,31926,29814,25654,25654,27734,31959,31926,31927,27734,29814,29847,29813,42392,29814,25653,25621,23540,27701,27734,25588,27701,23541,25589,29846,23541,23508,25621,29781,31893,27701,27669,25621,25588,21460,21460,23508,21428,21428,21428,21460,21427,27667,25618,27699,31828,29747,29715,33908,29747,27667,27666,27698,27699,25586,25586,23506,23506,23506,23506,25619,25586,25619,25586,25586,23506,21457,23505,23505,29714,25586,25554,27634,27634,25522,25522,33908,44309,33907,31763,36021,31892,29747,19281,17168,23473,21360,17199,17198,17133,19245,19213,14988,15020,17165,17100,12907,14988,36118,36085,36053,42325,52563,35660,8486,6439,6438,6405,6373,6405,6340,4292,6373,6373,4292,6373,6373,4292,4292,6373,6372,4292,6373,6373,4292,4292,6373,6373,27470,31728,31696,29615,48599,48567,25389,31728,38099,29615,25324,23276,25324,23211,23244,27469,44340,35888,42162,19152,17136,17169,17136,17104,21329,50645,52693,52693,54806,54806,54805,42128,31596,33677,37935,46354,44274,29516,29516,31597,46420,50678,42128,25290,27403,29484,48566,50646,31596,23177,23209,27371,33677,54838,54838,54838,54838,54838,54838,54838,54806,35823,33709,37903,23145,42161,27436,14759,23211,23244,21164,14923,10631,12744,23244,10631,12711,16937,16905,16905,19051,46486,50679,52824,48598,48598,50678,48565,48597,38096,17092,17093,14916,10660,8515,10724,8579,10660,44},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,48730,36119,27734,27734,27734,27766,31959,29814,27734,36119,31894,25653,25621,33974,29814,25653,25621,25621,29814,23541,23541,23509,23509,23541,23509,23540,27701,27668,42392,29781,29781,25588,25588,21460,21460,21460,21460,23508,21460,21428,21428,23508,25555,25619,25618,29779,31827,33940,29747,29747,27667,25618,27666,27698,27666,27666,23474,21426,25586,21394,21394,21426,23474,27666,25586,21457,21425,21426,21426,27634,27634,25586,25586,25554,25554,23474,25489,42228,35955,27601,25521,27666,31827,29714,25587,21392,17199,17231,19279,15052,17164,15020,17101,12940,12939,8715,8683,12908,36118,36086,36053,42293,52595,39820,23146,10730,10730,8519,6373,6405,6373,6373,6373,6373,6373,6373,6373,6373,6373,6373,6373,6372,6373,6373,4292,6372,6373,6373,29648,31696,31696,29615,48599,48567,46486,46519,48599,48599,50680,50744,50744,52792,52792,50712,54937,44308,44307,19184,19185,17104,19249,17072,21264,50645,52692,52692,52692,52692,52660,27403,12645,14758,16871,27403,37935,14726,14758,16904,21065,33775,33709,10565,10565,14759,14759,40081,23145,10565,10565,14759,16839,54839,56919,56919,54838,54773,56919,54839,46354,37935,35854,37935,25225,44307,42194,16871,23211,23244,21196,17004,16971,23276,23276,12711,14825,16938,14824,16937,21163,46518,50679,52824,48598,48598,50678,48565,48597,35982,19205,17060,12836,10724,10692,17093,14917,10660,47},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,50811,44505,38264,29846,29814,27734,27766,27734,29846,31927,29814,25621,25621,25621,25621,27701,25621,25621,23541,23540,23541,23509,23508,23541,23508,25653,27701,31893,42392,36054,36119,29814,23508,21460,21428,21460,23508,21428,21428,21428,23508,21428,21428,23508,25619,25619,25586,31827,31827,29747,27666,27667,25586,25618,27666,27698,27666,25586,23506,29748,29748,27667,25555,27667,27666,27666,23505,21425,23506,21425,25586,29714,25586,25554,25586,25586,23473,21393,38068,33875,25521,23441,25553,40246,31827,29779,21360,23473,19279,19311,12972,17133,17100,12908,12939,10827,12907,10795,15021,36118,36118,36085,42293,52595,46094,44014,29485,25357,16905,6373,6373,6372,4292,6373,6372,4292,6373,6373,4292,4292,6373,6340,4292,4292,6373,4292,4292,6373,6373,31696,29583,27503,27502,48599,46487,27437,27469,25357,25357,25324,25324,25324,23211,23244,31663,48566,46355,48501,15023,21330,14991,21297,17136,21264,50613,52692,52692,52692,52692,52660,35789,23209,23242,23177,31596,42161,27403,29516,29516,29515,37968,42128,29516,31596,29516,29515,37968,29451,27403,27435,29548,31629,40047,40047,40047,42095,42193,56986,54872,46354,44240,44272,42160,25289,33742,33743,21065,25292,23244,25357,25357,21131,25357,27469,12711,12744,16905,14824,16905,21196,46518,50679,52824,50679,50711,50744,48598,48631,35949,23398,17093,21286,12803,10724,17093,14949,12804,47},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,48698,29814,27733,31959,29846,38200,34007,29846,29846,36120,27734,25621,27734,25621,27701,31926,25621,25621,25621,23541,25653,23541,23541,23509,23509,27701,38167,42392,36054,25588,25588,23508,23508,21460,21460,23508,23508,21428,21460,21428,21428,21396,21428,21428,23507,25618,25586,29747,31859,27666,25586,27634,25586,25586,25618,27698,25618,27667,25586,27635,25555,25586,29748,31860,25587,27698,25586,23538,23505,25586,25586,29714,27634,25554,25586,27666,29682,31730,33811,35987,21360,25522,27665,27665,25553,38197,25585,27601,19279,19279,17165,17165,17133,17133,17132,12939,15020,12939,14988,36118,36118,36118,42325,52595,46094,44014,40049,46387,31564,6373,6373,4292,4292,6373,6372,4292,6373,6373,4292,4292,6373,4292,4292,4292,6373,4292,4292,6373,6373,29648,38068,38067,38035,48631,48599,46487,48567,48599,48632,50712,50744,52793,52825,52825,54938,59163,46388,48500,15024,19250,17104,19217,14991,23409,50645,52692,52692,52693,52692,52692,52692,52692,52692,52692,52692,52660,52692,52660,52660,52692,52660,50612,52692,52692,52660,52692,50612,50612,50612,50612,50612,50612,52725,50612,50612,50579,50579,50580,52628,52660,44208,42095,37935,23208,27436,27469,23146,25324,25324,25357,25389,27502,25357,27470,10631,12712,14857,14825,16937,21196,48631,50711,52824,50711,50744,50744,48598,48630,31788,19237,17093,14980,12836,14917,14981,14949,12803,65},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54971,54939,52891,52891,52891,34007,31926,40312,36119,34007,29814,31894,31926,31894,25653,29846,31926,29814,27701,34039,27701,23541,23541,23541,25621,25589,29814,25621,23541,23508,27701,29781,27733,21460,21460,23540,21460,23508,23508,23508,23508,21428,21428,21428,21428,21428,21428,21428,21428,23539,23538,25586,29747,27698,27667,25618,25618,25586,25586,25618,25618,25586,27667,27634,27635,23474,25587,27635,27667,27634,27666,25618,25586,25586,25586,25586,29746,25586,25554,27634,27634,31731,29650,38068,25553,21361,29746,25553,23440,38132,23537,21360,19311,17199,17133,17132,15020,19181,21325,17164,15051,15019,14988,36118,36086,36118,42325,52628,46126,41966,27534,38098,29451,6373,6373,6373,6373,6373,6373,6373,6373,6373,6373,6373,6373,6373,4324,6373,6373,6373,6372,6373,6373,35954,54938,54938,54906,50712,50679,54905,54938,54938,54938,54938,54938,54938,54938,54970,57018,59196,46388,48500,17105,19282,17104,21330,14991,21264,46420,52726,54806,54806,54806,54806,54806,54806,54806,54806,54806,54806,54806,54806,54838,54806,54838,54806,54806,52725,52726,54838,54838,52757,50613,50645,52693,54806,54838,54805,54773,52725,52692,52692,52725,54773,44209,44274,44306,25385,27467,29517,23211,25324,25357,25357,25389,27470,25389,27470,14792,12744,14857,14857,16970,16938,48631,52792,52824,50712,52792,52824,46518,48631,29675,19205,19173,17060,14949,14916,12836,19142,19141,58},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,54939,54939,52891,52891,52891,44537,31893,38167,34006,31926,34039,33974,36087,36120,31926,29814,29846,31894,27701,27701,25653,23541,23541,27702,25621,29814,25588,25588,25653,25653,29814,25621,23508,23508,23508,23540,23508,23540,23508,23508,21428,21428,21428,21428,19348,21428,21428,21428,23508,25619,25618,23538,27666,27698,27666,25586,25618,25618,25586,25586,25618,25618,29715,27634,27634,27667,27667,29747,31828,25586,27666,27666,25586,25586,25586,23505,27666,27634,25554,23473,25586,29682,27602,35955,33908,27666,23440,29746,21360,31827,25618,21392,19279,19311,15053,17165,15052,15084,21325,19245,19212,17100,15020,36118,36118,38166,42326,52628,46126,46126,41933,44046,35692,6373,6373,4292,4292,6373,4292,4292,6373,6373,4292,4292,6373,4292,4292,4292,6373,4292,4292,6373,6373,31760,54938,54938,54938,50679,48599,54938,54938,54938,54938,54938,54938,54938,54938,54938,57050,59196,44242,44307,15024,21330,14991,21362,17072,19183,44275,54839,56919,56951,56951,56919,29516,14758,21097,46387,56919,56951,23210,18984,21130,52758,56919,54806,8420,6371,25291,56919,56919,44242,6339,12646,27403,56919,56951,56919,56919,48499,40015,44241,54838,54838,48499,50612,46387,25354,27468,29550,23244,23244,25356,25356,25357,27437,25356,23211,16970,12744,14857,16905,16905,27535,50744,50744,52824,50711,52792,48631,44405,48566,27594,21286,25480,17061,19142,23368,19142,21255,21254,66},
{54972,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,50843,36119,34039,34006,36087,38232,34007,29781,29781,29782,27701,27733,38167,44538,31926,25621,23541,25621,25621,27701,23540,27733,31926,23540,25653,27701,29781,31926,23540,25620,29781,25621,23508,23508,21460,21428,21428,21428,21428,19380,21428,21428,21428,23508,25587,25586,25586,25586,27667,27666,27634,27666,25586,25586,25586,23506,23538,25618,29747,27634,27634,29715,29715,31860,23506,25586,29714,27666,25586,23505,23505,25585,27666,25554,23473,23505,27666,27602,31826,33907,31860,25553,38133,23472,21360,27698,25553,19247,17166,17165,21325,21325,17100,17068,12842,12874,14922,12809,33972,33972,36052,40213,48403,41966,41933,41966,41966,33580,10566,10598,6437,8518,10631,8518,8486,10599,10631,10631,6437,10598,6405,6405,10599,10599,10631,10631,8486,6405,29647,54938,54938,54938,48599,48599,54906,54938,54938,54938,54938,54938,54938,54938,54938,57018,59196,31597,42194,15024,21362,14959,21297,14991,21296,44275,54839,56951,56951,56951,56951,23242,8484,16936,46419,56919,56951,40178,44502,40243,54838,56919,52726,23275,25453,35985,54871,56919,42161,10662,16936,29517,46354,46354,44274,44241,31597,25291,29549,44241,44241,40048,42096,35790,25354,27436,27435,23276,29583,27469,27469,27437,29517,27437,25324,19083,12711,14825,14825,16905,31793,50711,52791,52791,50678,52791,52791,48598,48597,29642,19206,35950,40177,33840,19078,17030,19174,14916,55},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,52859,36119,29846,29846,34006,52923,38199,48698,38199,31926,31926,34007,31894,36054,31894,25621,23540,40280,29814,29814,29846,25621,23541,23541,29781,27733,36086,33974,27701,42392,33974,23508,23508,23508,23508,21428,21428,21428,21428,21428,21428,21428,21428,21428,23475,25618,25586,25618,21327,21196,21164,17068,17068,21229,17035,14955,17036,17003,16971,14922,14890,19116,14923,14890,17003,12777,14857,17003,10696,10696,17003,10664,12744,14857,12809,10664,10664,16970,10631,10631,14889,8551,8551,14889,6470,8518,14825,8583,8550,8550,14857,8517,8518,14857,8518,10663,14889,8550,8550,16937,10631,12711,14792,14792,10566,8518,19018,8518,8518,16937,16970,14857,19083,8550,10631,14857,10598,8550,8550,14856,8549,8549,14824,8517,8517,14792,6405,6405,21195,38002,50744,52825,50679,50679,54938,54938,54938,54938,54938,54938,54938,54938,57018,57050,59163,29484,42194,15057,19282,17071,21265,14991,19184,44275,54839,56919,56919,56951,56951,38032,31758,35952,50645,56919,56951,44306,44436,46452,54871,56919,54806,46484,48629,48597,54871,56951,50613,40145,44339,46452,56951,56951,54871,56919,50612,44274,48499,56918,54806,44241,42128,46386,19110,35887,23305,6371,14824,25324,25357,25389,27502,25357,25357,19082,10631,14825,14857,14857,31694,48462,48430,48430,50576,52657,52723,52723,42159,25416,19206,40145,44307,35921,25418,14980,12868,12836,32},
{54971,54971,54971,54971,54971,54971,54971,54971,52923,52891,52891,52891,52891,52891,42393,34039,36119,31894,31894,31894,44505,52891,40279,36119,29814,31926,29846,33974,38199,25621,29782,29814,23540,25621,25621,31894,31893,42360,46553,46585,27701,25620,40280,38167,25588,23508,21460,21428,23508,21428,21428,21428,21428,21428,21428,21460,21428,21428,21427,25619,25586,25618,19246,21130,19017,12776,12776,17002,12777,12777,16970,14824,12744,12744,14824,17002,14889,14857,16970,14856,14857,19050,14824,14824,19050,14856,14889,16970,16937,16937,14856,19050,16904,16937,19050,16937,16969,19050,16937,16937,19049,16969,19049,19017,19050,16937,19017,19050,19017,19017,19050,19017,19017,21098,19050,19082,21098,21098,23276,25356,25356,25356,25356,25356,27437,27437,27437,25388,27437,27437,27469,27437,27469,27469,27437,27469,27437,27437,27437,25389,25389,25389,25324,23244,50712,52857,50712,48631,50712,50680,48631,48631,48599,48599,48567,48566,48534,48566,54938,25291,42194,15056,21362,14991,21329,17072,19183,44307,54871,56919,56919,56951,56919,52725,50613,50645,54838,56919,56951,52758,52823,50677,54839,56919,54838,52790,52791,48532,56919,56951,52726,52823,52758,50678,56951,56951,56951,56951,56919,54838,54838,56919,54806,33710,29517,35822,21223,31662,23304,8515,10630,23244,25357,25389,27470,25357,25357,21131,10663,14825,14825,16937,27437,33578,31465,29384,31498,39983,44340,46453,31662,27531,19174,42226,42227,33808,23304,12836,10724,14981,43},
{54971,54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,38199,34006,44472,38200,36087,29846,34006,38199,42392,50811,34006,27734,27701,31894,38167,31926,25653,25621,23573,25621,23541,36054,42359,48665,38199,23508,23541,36087,31861,25588,23508,23508,21428,21428,21428,21428,21460,23508,21428,21396,21460,23508,23476,23508,23508,25587,25586,25586,19214,21130,19050,19018,19018,21131,19018,19018,21131,16937,16937,16937,21098,16937,16937,19018,19017,16937,16937,19050,14856,14824,21130,14824,14824,21098,14824,14824,19017,16937,14792,16904,19017,14824,14792,21130,12711,12711,21130,12743,12711,18985,14824,12711,16904,16937,12711,12711,21098,12711,12711,21130,10630,12679,19017,14792,31695,33808,33808,33808,33808,35856,33808,33808,33808,33808,33808,33808,33808,33808,33776,33776,33776,33776,31695,31663,31663,29583,29582,27502,25390,25357,50712,52857,50680,46454,23244,25356,25324,25389,25389,27469,29550,27502,31663,29582,52792,25291,40082,15024,21362,14991,21297,14991,21264,44307,54871,56951,56951,56951,56951,27436,16936,21097,50613,56919,56951,42324,48728,38065,54838,56919,50612,40243,42356,33807,56919,56919,46419,48695,46582,44339,56952,56951,56951,56951,56919,54838,54838,54838,52725,33743,29549,37967,21255,33775,21129,8548,14856,21164,21164,25324,27437,25357,25357,27502,14857,12744,14857,14825,27404,35757,33644,31563,33644,40080,44340,44340,31630,33838,19174,38000,40114,27532,21256,19143,17093,12836,53},
{54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,55004,38199,44505,42424,31926,29846,29846,34006,40280,46553,52891,31861,40280,33974,38167,33942,40280,25621,25621,25653,25589,29814,36054,42327,36087,23508,23508,23508,23508,21428,23508,21460,21428,21428,21428,21428,21428,21361,21295,21263,17168,17135,21262,19215,19215,21263,23309,17101,17101,17003,21162,16937,10631,10663,19050,10664,8583,21163,6438,8518,8518,19050,12712,10631,16905,14824,6470,10663,19017,8518,8517,21163,6437,8485,19050,10598,8550,16905,12743,6437,12711,16937,8517,8518,21130,8517,8518,21130,8550,8485,16937,12744,8518,14824,16905,10599,12679,21098,12711,10631,21130,12743,12743,19049,16904,27502,33808,33808,33808,33808,33808,33808,33808,33808,35856,33808,33808,33808,33776,33776,33776,33776,31696,31696,31663,29583,29582,29550,27502,25389,25325,42261,44373,46518,40147,42260,42260,42228,42228,42228,42227,40147,40115,40114,40082,48566,18985,31598,35923,21265,15024,19249,14991,21264,46355,54871,56951,56951,56951,56951,27404,21130,23275,50645,56919,56951,42356,48728,48630,54838,56919,48532,42356,44469,42291,54871,56919,46419,48694,48630,48597,56951,56951,56951,56951,56919,54838,54838,56918,52693,33743,31630,37935,21222,31662,21065,14855,16969,23309,25324,23211,25292,23211,21131,23244,19050,16905,16905,16937,25324,29484,27370,27338,25225,31564,33742,31662,23243,27468,21160,27403,27467,19111,16998,17030,25450,21222,68},
{54971,54971,54971,54971,54971,54971,54971,54971,54939,52891,52891,52891,52891,52923,48698,44504,42392,38232,34006,31958,29813,42392,38199,38232,40312,36022,44473,40279,34006,27701,25653,25653,34039,27734,27701,40279,31894,27669,23541,23508,21460,21460,23508,23508,23508,21460,21428,21428,21428,21428,23343,23243,19083,12874,10761,23244,12777,8583,14857,19050,8584,8648,14857,21098,19050,19018,19018,21130,19050,19050,21163,19050,19050,19018,21130,19050,19050,21130,21098,19050,21098,21130,21162,21130,21130,21130,21131,21130,21163,21163,21130,21130,23211,21162,21130,21130,21130,21130,21130,21130,21130,21130,21130,21098,21098,19050,21098,21098,21098,19050,21098,19018,19017,19050,19017,23211,23211,23276,31728,33808,33776,33808,33808,33808,33808,33776,33808,33808,33776,33776,33776,33776,31695,31696,31696,31663,31663,29615,29550,27502,27470,25389,25357,23277,23244,21164,21164,21131,19051,19018,19018,16938,16937,16905,14825,14825,14792,14792,29582,16905,27535,42129,40115,17071,21297,17072,21198,46387,54871,56919,56951,56919,56919,48532,48532,48532,54838,56919,56951,48564,48597,50645,54871,56919,54806,48564,50677,50677,56919,56919,52758,50645,52758,54838,56984,56919,56951,56951,56919,54644,54838,56886,52693,33775,31662,40048,19174,35888,23178,21161,8517,21131,25357,25389,27469,25357,25389,23276,23244,12744,14824,27502,33807,35855,35790,33710,33677,40049,44373,44340,40082,42259,35887,38000,33806,31726,19175,14949,25450,21254,64},
{54971,54971,54971,54971,54971,54971,54971,54939,52891,52891,52891,52891,52891,57084,46585,38231,36087,36119,38199,42392,29814,29781,38199,36119,36086,38135,33974,46553,40312,33974,33974,27701,31926,25621,29814,31894,23540,23540,23509,23508,21460,21460,23540,23442,19248,19216,19248,19216,19216,19248,21197,23243,17002,8616,8584,23243,10663,8518,14824,19018,8550,8551,14857,21130,16905,12711,12712,16938,12712,10663,16970,10598,10631,10631,14824,12744,10631,10631,16937,10598,10598,16905,8518,8518,14857,8485,8517,12776,8550,8485,8518,14824,6437,6437,14889,8583,8583,14857,6437,8518,14857,8517,6437,8518,14857,8518,8485,16937,10631,10599,16937,10631,10599,16938,12712,31793,31761,29648,48632,57083,59164,40115,29550,21131,16905,27502,21131,16938,25324,19050,16970,25357,19018,16970,25357,16938,14857,21164,16938,12777,19083,14857,12745,16906,14792,12712,12744,10631,10599,10631,10599,10599,10599,10566,10599,10598,8518,8518,21163,16937,40082,35824,44242,40082,25424,19183,23344,46388,54871,56919,56951,56919,56919,56919,56919,56919,56951,56951,54871,56951,56919,56919,56919,56919,54838,56919,56919,56919,56919,54870,54871,54871,56919,56919,52726,56919,56919,56951,56919,54838,56918,54838,50645,29549,29549,37935,25448,33808,23211,21161,4323,19017,25357,25389,27437,27437,27470,23276,23277,12744,12744,16905,29516,35822,33709,33677,31531,35822,44340,40082,33808,40114,35854,40111,33806,23305,17031,14885,16999,19142,57},
{54971,54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,50778,52891,48698,38232,40279,40279,57084,36119,27766,29814,33974,36087,40280,38167,34006,36087,40279,44505,27701,27701,25621,25620,23540,23540,21460,21428,21460,21460,25621,23508,25455,12875,10729,10729,10730,10762,10698,16970,23211,21163,21163,21163,23211,21131,21131,21163,21130,19050,16938,16969,21130,14824,8518,8518,14890,6470,6470,14890,6438,8518,6470,12776,8551,8518,8518,14857,6438,8518,14857,6470,8518,14857,8518,6470,12776,8551,6470,8551,14857,6470,8551,14922,8551,8551,14857,8550,8551,14857,10663,10664,10696,17003,12809,10697,16970,10664,10631,16970,10664,10696,14922,14890,31761,31761,29648,40148,48567,42260,33776,29615,19116,12777,27502,21196,14857,25389,21229,17003,25357,21196,16970,23309,19148,14922,21196,19083,12810,19083,16970,12777,14858,21164,12745,12744,10598,8518,10599,10598,8518,8518,10631,12744,10631,10631,10631,19050,14824,38035,14925,33809,25358,44242,42162,29584,46388,54871,56919,56951,54838,54838,56919,56951,56919,56919,56919,54838,56951,56951,56919,56919,56919,54838,56951,56919,56919,56919,54838,54871,54871,56951,56919,48467,54871,56919,56919,56919,54838,54871,56886,50613,31630,29549,35887,25385,29582,27469,19079,6403,16936,25357,25357,27405,27470,25357,25324,25390,12776,12744,16905,29516,33644,31563,31563,31466,35757,46421,40050,35856,40114,40145,37967,27531,14917,19176,10724,19174,14916,51},
{54971,54971,54971,54971,54971,54971,54971,54939,52891,52891,52891,52891,52891,54971,52859,42360,36086,57117,42425,46585,57117,29814,27733,40280,36087,33974,29813,40280,42424,36054,42360,27701,27701,23540,27766,23540,23540,23508,21428,21460,23508,25620,23475,25455,19181,19181,19181,19181,19181,17069,21131,23243,16937,12809,12744,23243,12712,12809,16937,16937,8550,8551,14857,19017,14890,12842,10762,14923,10729,10729,14955,10664,10729,10729,14890,12809,12809,12809,14922,10729,10729,14922,8681,8649,14890,10697,10697,12810,10697,8616,10664,14858,8616,8616,14890,10696,8616,12809,8616,8648,12809,8616,8584,8583,12777,8583,8551,12745,8551,8551,12744,6471,6471,10696,8551,33841,33841,31761,29680,31761,27470,29583,33841,31728,17003,29647,31760,19116,27502,31760,21196,27469,31728,23309,25357,31728,23309,23244,29615,23309,19051,25422,21196,16938,21163,19083,12777,19018,16970,12712,16905,16938,12712,14824,14857,10631,14792,14857,14857,14825,38002,17072,19184,15023,27439,42162,42162,46387,56919,56951,56951,56919,56919,56951,56951,56919,56919,56951,56919,56919,56919,56919,56919,56919,56919,56919,56951,56919,56919,56919,54871,54871,56951,56919,56984,56951,56951,56951,56919,54838,56918,54838,50612,33742,31630,37967,25418,29582,27469,23304,10659,16904,27437,25357,27502,27470,25357,23276,25422,12776,12712,14825,33742,35691,33578,33578,29320,31531,46453,35889,35889,35953,29646,23337,12772,14885,17029,19173,19172,14916,68},
{54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,44472,48698,54971,50778,34006,52859,61310,34006,29846,38199,40312,40280,38232,40280,48697,44472,33974,27701,23541,27734,23540,23508,23508,23509,21428,23441,31762,29681,27569,25422,12810,12810,12810,10729,10730,10697,19051,23211,19050,14890,16970,23243,16937,16937,19050,19050,14857,12776,10631,14759,10598,8518,8518,10664,8518,8550,12744,8486,8518,8518,10663,8550,8550,10631,10663,8518,8518,12712,8518,8518,12744,8518,8550,12712,8518,8518,10599,12711,8518,8518,12744,10631,10631,12776,8550,8550,12744,8550,8550,10598,12744,10599,8551,14857,12744,14889,14825,12712,12711,16970,14857,33874,33874,35954,35986,35954,31728,31695,29615,27470,23244,29550,27503,23277,27469,27438,21196,25389,25389,21164,25356,23276,21164,23244,23244,19083,19083,21131,19051,16970,19051,16970,12777,16938,16938,12712,16937,16938,10631,16905,14857,8518,14825,19083,12810,12777,38035,14992,21330,14991,21297,23246,44242,46355,54871,56919,56919,56919,56919,46485,48630,44339,54839,56951,52758,42323,46517,40080,56919,56919,44274,33741,37967,40048,56919,54871,46484,48662,42258,48532,56984,56951,56951,56951,46387,46354,52725,54838,50612,44307,40048,44338,25449,29582,27469,25420,12772,14822,25357,25357,29583,27502,25389,23244,27535,12744,12712,14857,33677,33578,31433,33578,31465,31531,44340,33711,38001,40309,36180,36082,23335,19206,17061,21285,19172,10723,62},
{54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,52891,50811,48698,42360,40247,44505,31926,34006,59165,61342,42425,31894,31926,29813,34006,42360,48633,54971,46585,29813,25589,23541,23540,23540,23508,23508,23508,27535,29583,31695,29583,23309,10762,10729,10729,10729,10762,10730,21131,23211,19050,19018,19017,23211,16905,16970,21131,21163,12744,12776,14857,12744,10631,10631,10631,16970,10663,10664,16938,8550,8583,10631,16970,8550,8550,12744,12744,10599,8518,16937,8518,8518,14857,8550,8551,16937,6437,8518,10631,12744,6470,8485,14857,6437,8485,14857,6437,6437,14857,8517,8517,10598,14889,12711,10664,19050,12744,12744,19050,14857,14857,19050,19050,35954,35954,35986,35986,35922,33841,31663,33841,31728,17003,29615,31761,19083,27469,31760,21196,25325,31729,21229,21196,31728,23342,19083,29615,21229,16938,25390,21197,14825,23276,19116,10664,19083,17003,10599,19018,19051,10599,16905,16970,8518,16905,19116,14858,21163,38002,15024,21297,17104,19217,14991,25390,46387,56919,56919,56919,56919,56919,40211,44470,42259,54839,56919,50645,36050,40308,38065,54871,56919,40048,31726,35951,42193,54871,54871,44404,46647,44436,50678,56984,56951,56951,56951,23178,23243,48532,56918,54805,52725,52725,48532,27529,29582,27469,27469,21193,19048,21098,27437,27502,27470,25389,23277,29615,12712,12712,16937,35822,35789,35821,35789,29386,25257,44373,29550,38001,42129,44274,44306,27498,21256,14917,19205,19172,12835,68},
{54971,54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,38199,44505,52891,50810,44537,42392,54971,54971,50778,31894,36119,40312,33974,44505,38135,46553,42392,29813,27733,23540,23540,23508,23541,23508,23507,29583,31663,33776,29583,25422,21229,21229,21228,19148,19116,17003,23211,23243,19018,14857,16937,23243,14824,14857,21130,21131,16969,17002,19050,16970,16970,17002,19050,21131,19050,19050,21130,19018,19050,19050,21130,19050,19050,19050,21098,19050,19050,21130,21130,21098,21162,21130,21130,21131,21130,21130,21130,21163,21163,21162,23211,23211,23211,23210,23243,23243,23211,23243,23243,23211,23211,23243,23211,23210,23211,23211,23211,23211,23211,23210,21130,35954,36019,35987,35986,35986,33874,29615,29615,12776,10631,25389,21196,14857,25389,21196,17003,23277,23276,16970,21196,21163,14922,19083,16905,10664,14857,17003,14858,14857,19083,14890,12745,14824,12679,10631,14857,10631,10599,14857,16971,12777,19084,21197,16938,35855,35921,17104,19216,17072,21330,14991,25424,46420,56919,56919,56919,56919,56919,40179,42324,44404,54871,56951,50645,38098,42324,42291,54871,56919,44274,33839,40112,44338,56919,56919,46452,46517,46484,52790,56984,56951,56951,56951,42161,40081,52693,56886,50580,44240,46353,44273,33804,31662,27469,25420,8484,6372,16937,25389,27437,27470,25390,23276,29615,14857,12712,21131,31596,27337,27402,27370,27370,21064,37968,18985,35889,44275,48468,33772,23273,19078,16997,19173,14948,12836,52},
{54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,42392,34006,44440,48730,33974,44440,63422,59196,44472,42425,29814,44505,42360,31894,33974,44473,52891,34039,25588,23540,23508,21460,23508,23508,25555,29583,31663,33775,29583,21196,10697,8616,8616,8616,8617,8584,23211,23211,23243,23275,23243,23243,23243,23243,21130,21130,16937,16937,23211,14824,14824,14824,19017,16937,14824,16905,19018,12744,12744,12744,21130,12711,12711,21130,14792,14824,18985,16905,12744,14824,19050,12744,12711,23211,12679,12679,21098,12711,10598,16904,14856,10598,12711,19050,8518,8518,23211,8518,8518,18985,12744,10598,14824,16937,10598,12711,21163,12712,10599,21130,14792,35954,35987,38067,38034,38067,38067,29615,29583,25390,23277,27502,25390,23277,27470,23309,21196,25389,21164,19083,23277,21131,19083,23244,19083,16970,19051,19018,16938,16938,19018,14890,14825,16970,14857,12712,16970,14857,12712,16938,19117,21262,21295,27568,31663,37936,35922,14991,21329,14991,19216,17104,23343,46388,54871,56919,56919,56919,56919,56919,56919,54871,54871,56951,54871,54871,54871,54838,56919,56919,54839,54871,52790,52758,56951,56951,50645,48532,48532,54838,56984,56951,56951,56951,54838,54838,56918,54838,48500,33677,48499,48499,29580,29550,27470,21225,4323,4291,14824,25357,27437,27438,27470,23276,25389,19051,12712,23309,48496,52654,54767,52719,54767,52688,52688,52688,44273,44307,42226,25449,17030,27562,17061,12836,8547,8547,39},
{54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,50843,36087,40312,42425,38167,36086,59165,63422,52858,54971,42392,44440,50811,31926,31926,40280,52891,44505,23540,23540,23508,23509,23508,23508,25555,29582,31663,31695,29550,23342,19181,19149,19149,19149,19149,17036,23211,25324,14825,10664,16905,23243,10599,14824,21130,19050,8518,8518,21163,8518,8518,6437,16937,12744,8550,12744,16937,8518,8518,10598,21130,8518,8550,21130,12744,12744,18985,16905,12744,14824,19050,14824,12711,23211,12711,12711,21098,14824,12711,16937,16937,12711,14824,21130,12776,12711,23211,14824,14824,21098,19018,14824,19017,21098,16905,16937,23211,16937,16937,23210,16937,33874,36019,38067,38067,38099,40148,31696,29615,14922,12809,25389,14922,12809,23277,17002,14890,21196,17003,14890,19083,19051,14890,17003,14857,10664,14825,16970,12810,12745,19083,14890,10632,14857,8550,8486,14856,8518,8486,16905,16971,19116,21196,33809,37936,37936,35922,14991,19217,17104,19216,15023,25456,46420,56919,56919,56919,56919,54838,16935,19016,27436,54871,56919,48532,48727,48728,38032,56919,56919,33742,12710,19017,40048,56919,56919,25387,19113,21162,50645,56984,56951,56951,56951,54838,54838,56886,56886,48500,35855,50547,46451,29612,27502,31695,21286,6467,4291,8518,27437,25389,25389,27470,25357,23276,29583,16970,21196,48631,50711,54872,50679,50678,50645,48565,48533,42194,44340,46452,48532,31726,21256,10691,12836,8579,10692,48},
{54971,54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,50811,48698,40312,34039,36054,42360,54939,63422,61310,48666,54971,38167,52891,44537,29814,33941,54971,42392,23540,23508,23508,21460,23541,23508,25522,29583,31663,33776,29550,21229,14890,14890,12842,12842,12842,12809,23211,23243,16970,16937,21130,25291,19017,19050,21098,21130,23243,23243,23243,23276,23244,23276,23276,23276,23276,23244,23243,23276,23276,23243,23243,23243,23243,23211,21130,21130,21130,21162,21130,21130,23178,21130,21130,23211,21130,21130,21130,21130,21098,21098,21162,21130,21098,21130,19050,19050,21130,19018,21098,19050,21130,21130,19050,21098,21098,19050,21130,16905,19018,21131,19050,46519,50745,50745,50712,50745,52857,40147,31728,27567,17003,27470,27535,17003,23276,27535,19116,21163,29615,21196,19050,27502,21229,16938,25422,21196,12776,23309,21229,12744,23309,21229,10664,21196,21196,10664,19050,21196,12744,19018,23342,25390,23310,33810,35855,40016,35889,14991,19216,15023,21329,14991,27504,46420,56919,56919,56919,54871,54806,14888,17000,31694,54871,56919,46452,48727,50808,46517,56919,56919,33775,29712,36018,46452,56951,56951,23307,19113,25355,52758,56984,56951,56919,56919,54838,54838,54838,56919,46355,29517,31629,37967,29612,27470,33808,21287,6403,4291,6372,27502,25389,27437,27470,25357,25324,25356,21131,23309,48631,50711,52792,50679,50678,48598,50678,48597,38031,48500,46452,48532,19142,19174,17092,17060,10724,12804,36},
{54971,54971,54971,54971,52923,54971,52891,52891,52891,52891,52891,52891,52891,52891,48665,42359,40279,36119,34007,36055,38167,57084,54972,34006,52891,40312,50746,61310,50811,40215,54971,38199,23541,23508,23508,23508,23540,23508,25522,29583,31663,31695,27502,17068,10762,10697,10729,10729,10729,12777,23211,25324,19018,19017,21130,25291,18985,21130,21130,19050,10631,10631,16970,8518,8550,8550,12744,12776,8550,8518,14889,8518,8517,8518,16970,10599,6470,17003,16970,16970,12777,19050,14857,14857,17002,16970,19115,19051,8550,10663,14825,10663,8550,10599,16970,10631,10663,16970,10664,12744,17002,10663,12744,12744,14857,10631,10664,16970,10631,12712,17002,8550,10663,16937,12711,59164,50745,52825,59196,57051,59164,44341,29582,27502,23309,27470,27502,23309,25389,27470,23309,25389,25389,23277,23309,23276,21196,21196,23276,21196,19083,23244,21164,16970,21196,21131,16938,21163,19083,14858,21131,21163,16937,19018,21196,19084,25423,33842,35823,40016,35922,14991,19216,15023,19217,17072,25423,46420,56919,56919,56919,56919,54838,35887,35920,42259,54871,56919,52726,48597,48597,50645,56919,56919,46452,40146,44371,50645,56951,56951,46419,44371,46452,54871,56984,52693,50612,56919,54838,56886,56919,56919,44274,27405,29582,33805,29612,29582,31727,17094,6467,4323,4292,27469,25357,25389,27470,25389,25357,27469,21164,25389,50679,50711,52792,50679,50678,48598,50678,48565,33869,48565,46453,42193,21253,17092,17028,19140,14885,19109,38},
{54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,52859,48730,40312,38232,40280,36087,36087,59197,61310,36119,27733,48666,57117,40280,40279,57052,52890,59164,44505,27669,25621,23508,23540,23508,21460,25522,29583,31695,33775,27502,25422,23309,23309,23309,23309,23309,23276,23211,25356,10631,8551,19050,23211,8518,16937,27436,21130,10631,12744,19115,8550,8583,10663,14889,12744,8583,10664,19083,8583,10664,12744,19115,8551,10663,19115,8583,8615,12776,17003,10664,10663,19115,10663,10664,21196,10696,10696,19083,14890,12777,10696,21196,10697,10696,21196,8615,10664,21228,10663,10696,14889,17035,8616,10664,23309,10729,12809,25422,10697,10729,21228,14889,57051,63390,63390,63357,63389,63389,44373,31695,27567,19083,25389,29583,21164,23244,29615,21196,21164,29583,23309,19051,27502,23342,16970,27470,23342,12777,25422,23342,10664,25389,23342,12712,23276,25422,12745,21130,25422,14825,16905,23309,14825,21164,29681,33743,40016,35922,15024,19216,17072,19217,17072,33842,48500,52725,52725,50612,52725,52725,52693,52693,52693,52692,52692,52692,52693,52693,52692,52693,52692,52693,52693,52725,52725,52725,52725,52725,52725,52725,52725,54838,50580,52692,52692,52660,52660,50612,50612,40081,27437,31630,35886,35919,29582,27502,21255,14949,10660,6436,27437,25357,25389,27470,25389,25389,23243,21163,27502,50711,50711,50744,48598,50646,48565,48598,44307,29642,48565,48533,42258,23399,19172,17028,14916,16997,10659,59},
{54971,54971,54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,52859,44505,46618,38200,36119,31926,44505,57084,55004,36119,36119,34007,31894,46585,50843,50778,42360,44440,54972,33974,23540,23540,23541,23540,23508,25521,29583,31695,33776,27502,17035,10729,10729,10729,10761,12810,14922,23243,25356,8551,8551,21130,21130,8486,27501,46387,33808,14988,21294,35986,14988,14956,14988,27535,21294,14988,17068,29680,17101,19181,19181,31793,17068,17100,31793,17068,17133,23374,27600,17101,19148,31793,17101,19213,33841,17068,17101,29680,23374,17133,19181,31793,17100,19148,33841,14988,17068,33873,17035,17068,23374,27535,17101,19181,33873,17068,19149,35986,14955,14987,31728,23309,57050,63357,61244,59163,63357,63389,46453,31728,23309,14923,25390,23309,17003,23277,23309,16971,21164,23309,17035,19051,23244,17035,16970,21196,17003,12745,23244,17003,10632,23309,19116,10664,19051,12712,8518,16937,12711,8518,14857,16970,12744,19084,23343,31630,37936,35922,15024,19249,17072,19249,17072,38036,50644,52692,52693,52693,52725,50612,46386,46386,46354,50579,44241,42128,44241,44241,44209,44241,40015,40048,42128,42128,44273,44273,37935,35854,35854,37967,46386,50645,52725,52725,52692,52692,52692,52660,52692,42129,37968,37968,38031,31694,27470,25389,38066,21287,14949,10629,27469,25357,27437,27470,25389,27470,27470,21163,29648,50711,48598,48599,44372,40113,31662,35920,35888,23336,46483,33837,29580,25450,19173,12836,19141,12803,14884,60},
{52891,54971,52923,54971,52891,52891,52891,52891,52891,52891,52891,52891,52891,52859,46618,36086,31926,33974,36087,36087,40247,61310,44537,27733,29814,40344,36119,48633,57116,38199,34007,38167,40280,25620,23540,23540,21428,23508,27569,29583,31695,33776,33808,27600,25552,27599,27600,29713,29745,35986,48500,48565,25487,29680,44372,38033,21261,38034,54806,44307,33841,33873,44340,33872,33873,33873,40146,38033,33905,35953,42259,33873,35953,35953,44307,35953,35985,44339,38033,38066,40146,42259,38066,40113,46419,40146,40146,46452,40178,42227,46420,44339,42259,44339,48533,44372,44372,48532,44372,44372,48565,46452,46452,48532,48565,48532,48532,50613,50613,50613,50645,50613,48565,50645,50613,59163,63357,61244,61244,63389,63389,46486,29582,23341,21229,25390,25389,23277,25390,25389,21229,23309,25389,23309,23277,23276,21196,21196,23276,21196,19083,23276,21196,17003,23244,21164,16971,21164,21131,16970,21131,19083,17002,19051,16970,14858,21197,36020,29550,37936,33842,17105,19217,14992,19249,15024,38035,50612,52692,52693,52693,52693,44274,12743,16937,21098,42161,46387,16839,12711,16904,20903,27242,40049,12711,12711,16937,29549,37969,31630,8452,10598,14791,25323,52758,54871,56919,54838,54838,54838,54838,54838,44274,37902,52725,40112,31759,27469,25389,23276,25420,21161,8484,25356,25357,27470,25389,27438,27438,27502,23243,29616,50679,46453,48631,46453,48565,48565,48598,46420,25448,44338,40144,16997,19173,14980,14980,19173,17029,14916,48},
{52891,54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50778,40280,33974,36087,40280,46585,50778,48698,36119,27702,31959,40312,52891,38232,34006,33974,40247,46585,34006,25588,23540,23508,23508,23508,40213,48566,48598,50646,57017,42357,42390,38197,38197,38229,42423,46551,54806,54871,25487,27632,48565,40179,19214,33841,54806,54838,54838,54871,48532,42194,46387,48500,46387,42161,42194,46387,42193,40081,46420,46387,42194,38001,42194,44274,37936,40081,44274,37936,37936,42161,38000,33775,40048,40081,31630,35823,40048,29549,31598,37968,33743,29517,35855,35823,27404,31662,37936,25324,27469,37968,25356,25291,31662,29516,21130,29485,29517,19018,25291,35822,21065,57018,63357,63389,63389,63389,63389,46518,29583,14890,14857,23277,14857,14890,21196,16938,14858,19051,16970,14858,16971,16970,12777,12777,19051,12810,10664,21163,12777,10631,19115,12810,8584,19051,10696,10599,19050,10631,12777,21163,19084,21196,25423,33842,29518,37936,35890,15024,21298,17072,19217,15056,38035,52692,52692,52725,52725,52693,44241,12645,14791,19017,18984,44339,14759,10597,14727,27404,37936,40049,12678,10597,14791,33808,44340,31597,10565,12678,14823,14791,50678,54838,56919,54839,56919,56886,54838,54838,38000,37902,52692,38031,33807,27470,25389,23308,19083,21163,6404,21130,25357,25389,25389,25390,25390,27470,25356,29680,50679,50711,50711,48598,48598,48533,48565,44371,23400,42224,40112,17029,14980,17061,17061,16997,12835,12772,35},
{54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,34006,34039,40280,33974,52891,48698,31894,25653,31927,27734,34007,44472,46586,29781,55004,44473,42392,36086,27701,27701,23540,23508,23540,48600,56984,56985,56985,57017,40245,33971,33939,33939,33971,36084,44405,52725,52725,44339,44372,50645,50612,46452,46484,54838,46420,33873,40276,40211,12777,16971,23277,37969,19051,12777,25389,25390,12778,16971,31663,31695,14825,19083,35856,14825,16938,31663,21131,14858,25389,27502,14857,23244,33776,12745,19051,31695,16938,16938,29550,25356,14857,23276,31663,14825,21131,33808,14792,16970,31663,19050,14857,23244,23244,14825,21163,25389,12744,19051,35856,16905,23211,33743,29550,23178,27437,42163,38002,31695,12776,12712,23276,14857,12745,21163,21228,16971,19051,23309,17036,16971,23277,19116,12777,23277,19148,12745,23309,21229,12712,23276,23309,12778,21163,19050,12744,19018,19050,27502,35954,19149,10698,10763,19182,27470,40016,35922,15056,19217,17105,19217,15024,40148,50644,52692,52693,52693,52693,44274,21097,23210,23210,23210,44306,25290,23210,25355,31695,44340,42194,25323,27436,27436,35855,44307,37968,29549,31661,29581,31661,50678,54839,54839,54839,56919,54838,54838,54838,48499,52693,52725,35950,31759,29550,27469,23244,10663,12805,4323,8485,25389,25389,25357,25422,27470,25389,23276,35954,50679,50711,50711,48566,48565,46452,48564,46452,25448,21191,23272,21224,10724,12836,14948,14980,10723,10692,40},
{54971,54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,40312,29846,31959,34007,46585,38199,27734,31959,31926,36087,38167,44505,36119,27701,36119,48633,46585,31926,29781,25621,23540,21460,23508,46551,57017,57017,57017,57050,46518,42325,42325,42357,42357,44438,48598,54838,54838,46452,46485,52726,50645,42194,42259,54871,48532,29646,38131,38099,12744,21130,31629,35888,21131,16937,21196,27437,16937,27436,33743,31662,21098,27469,33776,21097,25356,29550,27404,25323,29517,29549,25291,29517,33743,23211,29517,31631,27436,29549,29549,31630,29548,31630,31662,29516,31662,33743,29516,33742,29549,33742,33742,29549,29550,31662,29615,27502,33742,35855,33710,33774,37935,31662,37935,37935,33742,33775,35921,29615,29615,23309,25389,29616,25422,23277,29615,25422,21196,29583,25422,21164,27470,25390,19083,25389,25422,19051,25357,25422,19083,23276,25390,19083,21164,25357,19083,19051,23276,25357,29616,38166,36118,42392,40247,27437,37936,33810,17137,19217,15024,21329,19184,42260,50612,52692,52692,52692,52692,52725,56951,56951,54871,54871,54871,52758,56951,56951,56951,56951,54806,54806,54871,54871,56951,54871,54838,54839,56951,56951,56919,54838,56919,54839,56919,54838,54838,54838,54838,52693,54805,54806,40144,35888,27502,25357,25357,10695,17061,6403,6372,25357,25389,23309,27470,27470,27470,21163,38067,48631,50711,50711,48598,48597,44339,35950,29643,21286,19173,21223,12772,10693,17031,14948,15012,10659,10660,29},
{54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,52859,52859,50811,50811,50811,44505,36087,34007,31959,40312,31927,29814,31894,31926,38199,36119,29846,27701,31926,40280,36086,57117,29813,25621,29814,25621,23508,23508,46519,57017,57017,57017,57049,52824,50745,48696,48696,48664,50809,52824,54838,56951,27600,29713,46452,44372,23342,29648,54839,52725,52758,48565,42259,52790,54838,52726,44307,52726,54871,44307,48500,54839,54838,48532,46420,56919,50646,46388,54838,52790,46452,50645,54838,46452,48533,52758,48532,48532,54806,50613,48533,50613,50678,48500,50613,50645,46452,50645,48532,46419,50645,46452,46419,48500,46420,46419,44306,46420,44307,42162,50613,44274,42194,48532,44274,44274,44274,44339,37968,40113,44340,35921,29583,25389,19083,23277,25389,19084,21164,25389,21196,19083,25357,21196,16970,25357,21229,14857,25389,23277,14825,25357,23309,14858,21196,23309,14890,19083,23309,16970,16970,21196,17003,16970,21196,21197,21164,50778,29550,40016,33810,17072,21329,15024,19216,17104,40147,50612,52692,52693,52725,48499,50612,56951,52758,52693,54871,54839,54838,56919,50645,52726,56919,52758,54871,54839,50580,52725,56919,52693,54871,52758,48532,48532,50612,56919,54839,54838,54676,54838,54838,54838,52692,54805,54806,38032,29582,27470,25357,23276,12809,21318,12804,10597,23244,25357,23309,27470,27470,27502,23276,40213,50679,50744,50744,48566,50711,33804,27498,19110,17062,12836,17030,14853,10628,12838,10691,14947,12772,10628,42},
{54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,46618,36087,36119,29814,27766,29814,36119,38200,33974,50811,46585,29846,27766,31894,34006,36086,44440,40280,25653,36119,29781,23508,23508,44407,57017,57017,57017,57018,44438,36085,33971,36019,33971,38165,44405,54838,56951,27599,29680,44340,46452,21261,29615,54839,50645,44437,46550,54871,29615,27502,27567,46420,35888,21164,35888,44339,25389,23308,29549,52758,21164,23309,50645,29582,27502,42194,38001,27469,35888,46452,25389,29550,54871,27469,29615,50613,35888,31695,42194,42259,29647,35921,50678,29615,31727,54839,33808,33808,48500,40114,33807,42226,46452,31727,33808,54839,31662,33775,52725,35888,33775,44339,42226,29582,40081,52758,38067,31695,29648,21229,23277,29648,21229,19083,27567,21261,17003,27535,23309,14890,27502,23342,12777,27502,23342,12777,23276,23342,14858,21164,23309,14858,19083,23276,16938,16937,21163,16938,14857,21163,17003,21164,46519,27470,40016,33842,15024,19217,17136,19216,19152,42292,50612,52692,52692,52692,50580,50612,56951,54838,54806,56919,54839,52758,56919,54806,54838,56919,54806,54838,54871,54838,54871,56919,52725,54838,54871,54871,56951,56951,54871,48500,40048,44209,48467,48467,48467,44273,46386,46386,38031,29615,27470,27501,25389,8551,17030,19109,10660,23275,25389,25325,27438,27470,27502,27437,31728,40147,40147,40179,38034,40146,21223,17029,17030,10692,10659,12805,19079,12806,17061,17093,12836,12836,12772,47},
{54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,52891,52859,50811,50811,50811,50811,42425,31894,34007,34007,38232,31926,34006,36087,36119,36055,34006,29814,42392,59229,44536,46617,38167,31894,36087,29781,27668,25587,42293,56985,57017,57017,57017,44406,36019,36052,36052,36052,38165,44405,54838,52725,50677,50677,52758,52725,50613,50645,54806,52726,48631,48695,54871,38033,33873,38034,46452,42227,29615,40082,48565,29647,29582,31663,54871,25389,27470,50645,31695,27534,42194,40114,25421,33775,48533,25357,27437,54871,25356,25389,46420,31695,25357,38000,42194,23308,29582,50646,23244,25324,54806,27469,27502,44307,37968,27501,38000,44339,27469,29582,52758,25388,29549,50612,31662,29582,42194,40113,27501,35887,46420,33776,29583,19083,17003,23309,21164,19083,21196,19083,16970,19083,23277,19148,19083,23309,19148,17003,23309,21196,17035,23244,23342,21164,21196,23309,19116,21164,19051,16970,19051,19083,21164,25454,27601,29714,31827,29681,27470,39984,31729,17104,19249,15024,21297,14991,42260,50612,50644,52692,52693,52693,52725,54838,54806,52758,54806,48500,48468,50613,50580,48467,48467,44274,46354,46355,46387,44274,46354,42161,44306,46387,44274,46387,54839,54871,54838,56919,54838,54870,54838,56886,50612,52692,52693,33837,29582,27437,27469,25389,16905,6405,21225,21129,19017,25357,23277,27470,29550,27502,29582,21163,25389,23309,25390,23309,23338,14917,17060,12868,14948,12868,12804,19111,14917,8579,14981,17028,17029,10659,35},
{54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,52891,52859,50811,50811,50811,52891,52891,36119,36087,40280,34007,31894,38199,40280,38199,38199,34006,46585,48666,61309,59229,46585,34006,29781,29813,42392,27668,29682,46453,57017,59098,59098,57017,57050,55002,54970,54970,54970,57050,57018,54871,56951,40114,40211,35920,29516,23309,31695,54838,50645,44371,46451,52758,44371,44339,44372,50645,48565,46420,48532,52725,48565,48565,50645,54838,50645,50678,54838,50677,52758,52758,54838,52758,52758,54806,52758,54839,54838,54839,54871,54838,54838,52790,54838,54838,54838,54838,54806,54838,54839,54838,54838,54838,54806,54806,54806,54838,54806,52758,52758,54838,52758,52725,54838,52725,52725,54806,52758,50613,52725,54838,42195,29583,19050,16938,21196,19051,16970,21164,21164,17003,17003,21163,16970,14825,23244,16970,14857,21196,16971,12745,21164,16938,12777,19083,14857,12745,17003,14857,12744,14857,14825,19083,38133,33940,27602,31795,25488,25357,40016,31730,15024,19249,15023,19217,17104,42260,52692,52693,52725,52725,52693,37935,14791,16936,25291,29419,37968,14758,12678,12711,33776,44307,33742,8452,10565,16904,40147,48533,21065,12710,14856,19017,37968,54839,54871,54839,56919,54838,54838,54838,54838,50612,54805,54806,31756,29614,27437,27469,25389,27470,23276,23273,10597,16905,23276,23276,25389,29551,27502,27470,21131,25389,23309,23309,21226,21224,21226,19143,14980,14948,8547,12900,10723,14917,12804,14948,10691,14948,10692,36},
{54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,52859,52891,54971,42425,36086,40247,36086,40280,40312,34006,48666,40312,46618,44473,59197,36054,46553,40312,31894,31894,50778,48697,36054,27668,48567,57017,59098,59098,57017,46583,38164,33938,33938,33938,36084,40244,54871,56952,29615,31761,33873,31662,19116,25455,54806,52758,48630,52824,54871,42259,40147,40179,48500,44340,35921,40114,50678,35888,33775,33807,54871,27502,29582,50613,35920,29614,42194,44307,29550,33775,50646,27437,29550,54839,29549,27501,46387,35888,27437,37968,44307,25389,29549,52758,23243,25356,52726,27437,25421,42226,35888,25356,35888,46387,25356,27469,54871,23308,27437,50613,31662,27533,42194,40081,25324,35855,48532,33809,31696,12744,10664,19083,17003,14923,19083,19083,14890,14858,21196,14890,10696,19083,8518,8518,21196,14923,10697,21164,19116,12842,19083,19148,17003,19083,14857,10664,14825,14857,12744,16938,21164,19084,21164,33907,27471,37968,31762,15024,19217,15056,19217,17072,44340,52693,52693,52693,52693,52693,35822,10597,12678,16904,25257,40048,14758,12678,14824,23243,35920,33710,10565,10598,12711,23211,44339,18984,10565,12711,16904,23210,54807,56919,54838,56919,54838,54838,54838,56886,52692,54805,54806,33838,31662,27469,27437,27470,25357,25389,25419,8547,10598,23276,23276,25390,29551,27470,27470,23244,25389,23309,25389,21226,19143,17060,17061,17029,10691,8547,12804,17060,12836,12804,17060,14980,12803,12868,38},
{54971,54971,52891,52891,52891,52891,52891,52891,52891,52891,50811,52891,50811,50811,50811,52891,54971,52891,36086,44505,42392,48698,50843,36087,38167,38167,54971,61277,59229,42425,31894,29781,29781,36087,42392,33973,48665,36086,46486,56985,57017,57017,57018,46551,33971,36051,36084,34004,36116,42325,54871,56952,33808,31694,29582,33742,25356,29550,52726,54839,42291,46518,52758,40146,33873,33873,44340,44340,33808,40081,52726,33840,33808,35888,54871,31728,33808,48532,40114,33840,44307,46420,33840,38001,52758,33775,33808,54838,33808,33840,48500,40114,31728,42194,46420,31695,33808,54839,29550,29582,52726,31662,29582,42226,38001,27469,37968,46420,25356,27469,54839,23243,27436,50580,29549,25356,40081,37968,21162,31661,46419,31663,29615,27535,19116,19083,27567,23309,19051,27535,23342,14890,27535,25423,14858,27535,27535,14890,25390,27535,17003,21196,27568,19084,19083,27502,21197,19051,25390,21229,16938,23309,23309,16938,21196,21229,25422,42358,29648,37936,31730,15024,19249,15024,21297,17039,44373,52692,52692,52693,52692,52692,35887,18984,21129,19017,23243,40113,23177,23210,23242,25290,33774,35887,21097,21130,21130,21097,44275,27435,23243,25356,25355,31629,54806,54838,54806,54838,54806,54806,54806,54806,48467,48499,50612,31726,27502,27437,25389,27470,25390,25390,25323,12741,6372,21099,23276,25357,29583,27502,27502,21131,25389,25357,23308,23308,16999,17125,17060,12835,8611,8547,10692,10659,14948,14885,19173,17092,12804,8547,32},
{54971,52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,54971,54971,46618,40247,38167,38167,48665,38199,50843,46650,50713,63390,57116,34006,25621,27701,34006,38167,31894,44472,61277,61277,52792,57017,59098,59098,59098,57050,54970,54970,57050,57050,57050,57050,54871,54806,54838,54903,48533,52726,56951,52758,52758,54806,40178,44404,50678,40114,33841,33873,44307,44307,33841,40082,50645,33840,33840,38000,52757,35953,38033,48532,42227,40113,46419,48500,40114,44307,50645,40081,42194,50645,42193,42194,48500,44339,42194,46419,48468,42162,44274,50580,42161,44274,48500,42193,44242,46387,44274,42161,44274,46386,40113,44274,46387,40081,44274,46354,42161,42161,44241,44241,40048,42161,44242,38034,29550,25390,23277,25357,27470,23309,23277,25390,23309,21196,25390,25390,21164,27470,25390,19083,25357,25389,19083,23276,25389,21196,21196,23309,21164,19083,23276,19116,19051,21196,19116,14857,19083,19051,31729,35987,38067,37936,29649,17136,19249,14992,19249,17072,44372,50580,50580,50612,50612,50612,50612,50644,50644,50580,50612,50644,52692,50644,50644,50644,50644,50644,50644,50612,50612,50645,48532,46387,48467,48499,50580,50612,50612,50580,50580,50580,50612,52693,48467,48467,31564,25258,33676,31693,27470,27469,27437,25389,25389,25389,27371,10628,6372,19051,23244,27502,29583,29583,29583,19050,25389,25389,19143,14949,17093,12836,15011,15012,12803,14917,14917,14948,14981,12773,14949,12867,10691,8579,43},
{52891,52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,54939,55003,54972,44473,31894,42360,59197,61277,59197,55036,36022,61309,52859,40312,34039,27701,44505,36054,31829,50746,61277,61245,54905,57017,59098,59065,59066,50744,38164,36084,36084,36052,38197,40277,54871,56951,35954,29713,40114,54871,25487,31761,50613,54838,52758,54904,54903,50646,48533,48565,52726,52758,46484,42194,33708,29515,18985,23210,31597,19017,21098,21130,25356,18985,19050,27404,16905,19017,27436,14824,16937,25323,19050,16937,18985,25356,14857,16937,27436,14857,16937,29517,12776,14857,23211,21131,14857,16937,27469,14889,16970,27469,14857,14889,29582,14857,14890,21163,21195,14857,16937,27502,12744,19083,33743,35922,31696,27503,21197,19051,27503,21261,16971,27503,23342,14857,29583,25423,12777,27535,27535,14858,23309,27535,14923,21164,27503,19116,19051,27503,21229,14890,25422,23309,14857,23309,23310,12777,23309,25422,23309,35987,33840,37968,29682,15024,21297,15023,19217,15024,40147,50613,52725,52725,52725,52725,52725,54806,54806,54806,54806,54806,54838,54838,54838,54806,54806,54806,54806,54806,54806,52693,46387,50613,50581,48500,46419,46387,44274,44242,44241,44274,54872,61211,44274,44241,35790,33645,40016,31726,27470,27469,25357,25357,25389,25357,16807,19077,6404,14857,25324,35889,29583,29583,29583,23244,23276,25389,14981,12803,17092,12868,15012,17060,17092,12803,12836,17060,10723,10660,14980,19172,12836,19174,48},
{52891,52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,52891,52891,54971,54971,48698,34006,40247,59197,63422,59229,36087,40280,44472,42359,44537,38232,27733,44505,38166,48666,48633,59197,59196,52792,57017,57017,57017,57017,50744,33971,36019,36051,36052,36117,40245,54871,56951,35954,27633,38002,54871,25454,29712,48533,56951,42324,46550,50678,42260,33905,33906,44340,48533,40211,40211,50612,44371,19181,21262,35954,21294,19181,23342,33874,21262,21261,35986,21262,21294,38034,19149,21261,33809,25487,19213,23341,35986,19213,21261,35986,19213,23342,38034,19116,19149,29647,29616,21261,21229,35986,19181,21261,38034,19148,19148,35986,19148,19148,27535,29647,19115,19115,35921,14857,19083,38034,31728,29648,21228,14890,19083,23277,17036,17003,21228,16970,12744,25389,19084,12777,25357,21197,14858,21196,19083,12777,21164,21164,14890,19051,23309,17035,16971,16937,10631,12744,19083,14890,25422,27535,27601,29681,31794,27567,37968,29682,15024,19216,15023,19249,15023,35889,50646,56951,56951,56951,56951,40081,18984,25291,44274,56951,56951,27403,14758,21097,50677,56951,54871,21161,23274,29581,52759,52791,40081,10565,16904,33742,56951,54871,54871,54871,48467,44274,50612,54806,52725,50612,50645,52725,31694,25356,27469,25357,25389,27470,23147,20743,25222,8484,12711,27437,35889,29615,29615,29583,27470,21130,23306,17060,17060,15012,14980,10691,8611,17092,12771,14916,14916,10756,10692,10724,12836,14948,12804,29},
{52891,52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,52891,54971,54971,54971,54971,46618,33973,57052,63422,48730,29813,27733,29782,52923,42425,27669,31926,33974,44407,42327,46552,52891,57051,54905,57018,59098,59098,59097,57050,52857,52857,54937,52889,52889,54938,54871,54838,48532,46420,50613,54806,48532,48533,52726,56919,42259,46550,48565,40146,29647,29647,38033,46452,29615,31727,54838,48533,21229,23341,38034,23374,21229,25389,38067,23341,23342,38067,23310,23374,38099,21229,23341,33808,29647,23341,23309,38067,23309,25357,40115,23309,23309,40147,21229,21229,29615,29680,23341,23276,38066,23341,23341,38066,23309,25389,42292,23309,23309,29583,33809,23341,23276,38066,21196,25421,38034,29582,29582,23309,21196,23309,25390,23309,23277,25389,23277,21164,27470,25422,23277,25422,25422,23277,25390,25390,23277,23309,25422,23309,23309,25422,23309,23309,25389,23277,19116,23277,19084,38100,38100,38133,40213,42293,33842,37936,27569,17104,19249,14959,21362,15024,37970,50678,56951,56951,56951,56951,33775,8484,19049,44339,56951,56952,21130,8549,19049,52726,56951,54839,19081,19146,31726,56951,56951,40113,16968,23275,40113,56951,54871,54871,54871,27404,25356,44274,54838,54805,52725,54805,52725,31694,19017,23211,21131,23211,23211,25162,20807,21093,12710,12711,23243,27437,25389,25356,25324,23244,16904,21161,19110,14948,15012,17092,12867,17060,12803,12868,10723,8611,14980,8579,10693,8580,8579,8547,28},
{52891,52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,52891,54971,52891,54971,54971,52891,40312,48666,59197,33974,34007,31959,31926,27733,25621,25621,27701,36022,36054,33942,38167,40247,54939,46454,57017,57017,59066,57017,54937,36052,36051,36051,36019,38131,40212,54839,56951,46452,40179,44340,56983,42259,42292,48533,52725,52790,54871,54871,54870,54839,54839,54838,54838,52790,50678,52725,50645,25389,27502,38034,29647,27535,29582,40146,27567,31695,42227,29647,31727,42227,29615,31727,38033,35953,33840,33807,42259,31760,35888,44307,33840,35920,44339,35888,35920,40113,42226,35985,38001,44339,38033,38065,44371,38033,40113,46419,40146,40146,44307,44339,42194,42226,46452,42194,42226,46452,40114,29615,16970,14890,19083,17003,14890,14890,19083,12809,10664,23244,14857,12744,23277,14858,12745,21196,12809,12777,21164,14890,12810,17003,19083,14922,16970,19116,14922,12745,21164,12712,10663,19116,17003,25422,48600,31729,35856,29617,14991,21329,14991,19249,17104,37969,52726,56951,56951,56951,56951,44307,33807,38033,48565,56951,56951,42226,40178,42226,54839,56951,54839,35919,35984,42258,54903,56951,48565,46484,46452,48597,56952,54871,56919,54871,44306,42194,50580,54838,52725,52725,54805,50644,31726,23211,25356,23243,25389,25357,23147,20839,23143,14886,12710,27469,27437,31631,29550,27470,27437,21098,23243,21161,19143,19204,19173,19204,19173,8611,8643,12867,10691,8547,10724,6467,8547,10724,8547,37},
{52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,52891,54971,54971,54971,54971,52891,48731,33974,36086,31894,27733,27701,29814,27766,25621,27701,31862,33974,29749,29814,29781,44472,54972,31728,59130,59098,59098,59098,54970,31858,33971,36019,36051,36116,40244,52791,56951,38099,29713,35922,56951,33873,38164,46420,54838,42291,46582,46484,42227,27567,27567,35888,48565,23309,29615,54806,52693,52726,52791,54871,52758,52758,54838,52790,52758,52758,54838,52693,50645,56919,50580,48532,54839,50645,48533,52726,50678,46452,50613,52758,46420,46420,56951,44307,44339,52758,46419,44339,50613,48500,40081,44307,52758,40114,42194,54871,38001,38033,50613,42226,40081,46419,46420,38000,42194,52725,35888,31695,23309,19116,19116,25390,19116,16971,25422,21229,14890,25422,21229,14890,25390,25423,19116,23309,23341,17036,21164,25422,19149,19084,27503,23342,17003,25422,21229,12777,27502,23277,16938,23309,23342,25455,33874,31696,35823,29650,14991,19249,15024,21297,15023,40050,52726,56951,56951,56951,56951,54838,52790,52758,52790,56951,56951,50677,52758,50645,54871,56951,54839,48564,48564,48500,56919,56951,50580,44306,44339,50613,56984,54871,56919,56919,54838,54838,54838,56919,52725,52660,52692,48499,31693,23276,27469,25324,27469,25389,23244,24774,23080,12837,12774,29549,27502,29550,29615,29583,25357,23276,23276,23308,19176,17092,14916,14916,17060,10691,14948,12804,10724,10659,17028,14948,12804,14948,8548,42},
{52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,52891,54971,54971,52891,52891,50811,29846,27733,27733,31926,34039,29814,27701,27701,31894,38167,36054,40280,33974,31861,50778,36054,40148,59130,59098,59098,59098,59163,54938,54970,54970,54970,55003,57051,56952,56919,46420,38066,42227,54838,40178,42259,48532,54838,44372,50776,48597,44340,27567,27599,33840,50678,25454,29615,52726,52725,25421,27534,50646,31695,29615,44307,40146,27566,38001,48533,29615,33743,54871,27502,29647,50613,35953,33808,44339,44339,31728,40081,50613,31728,33808,54871,31695,33775,50613,38001,33808,44307,44307,29615,38001,50678,31727,35855,54871,31695,31727,48532,38001,33775,44306,44339,31695,38000,52726,38034,33809,33841,29648,27502,33873,31793,27535,35921,33873,29583,35954,35954,31728,35921,38035,33841,35921,38067,35954,35921,40147,38067,35954,40180,40180,35986,42260,42260,38099,42325,44373,44406,44438,42326,38100,42261,42326,42162,27602,17072,19249,15023,21330,15023,38001,52726,56951,56951,56951,56951,44371,42389,38098,46484,56951,56984,35920,36018,33840,54871,56951,50645,23372,27598,33775,56951,56951,35920,19113,27501,44371,56984,54871,56919,56919,54838,54838,54838,54838,52725,52725,54773,48564,29645,27502,27469,23276,25389,23276,27438,28903,20969,25356,27436,29549,29550,31663,27503,29583,27437,23243,23276,23308,17094,15013,10723,14980,17124,14947,14980,12803,8579,14884,10723,17060,17060,19109,10659,34},
{52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,52891,54939,52891,50811,50778,44506,29814,27701,34006,34039,38167,34071,31894,42360,46585,36054,36054,33974,38134,38167,25621,27635,57018,57050,59098,57017,59163,33971,33906,35987,35987,36051,38164,52726,56919,52726,48565,48533,56983,46420,42227,48532,52725,50645,50677,52758,54806,52758,52758,52758,54806,52790,50678,54806,54806,23309,27501,48533,29615,27470,42162,38033,23341,33807,46452,23308,27436,54839,31727,40179,50645,33775,27502,40113,40146,27470,33808,48533,27502,27502,54839,27469,29582,48467,35888,29614,40114,42227,35921,40082,48565,29615,31695,52726,29614,29647,46419,38000,33807,42226,44307,31727,37968,50613,48534,59130,59098,59098,59130,59130,59130,59130,59130,59130,59130,59131,59131,59131,59163,59131,59131,59131,59131,59131,59131,59131,59131,59131,59131,59131,59131,59131,59131,59131,59163,59163,59163,59163,59164,48632,48600,50777,44275,27569,15024,21330,14991,19249,17104,38001,52758,56951,56951,56951,56951,44404,46582,44437,52823,56951,56984,31727,31824,33872,54871,56951,48565,25485,29678,35920,56951,56951,35887,21194,27533,48565,56984,54870,54871,56919,54838,54838,54838,54838,52725,52725,54773,48531,29644,29550,27469,23211,25357,25325,25390,27437,27437,29486,31371,33581,27437,29615,27470,29583,27470,23276,19083,25388,19110,17093,15013,19205,15012,12803,17093,14980,12772,8579,8579,12836,17028,19141,12772,60},
{52891,52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,52891,52891,50778,48730,46650,29846,27733,31926,29781,44505,38199,40280,48633,42360,36022,38134,33941,40279,27701,25621,25588,54905,59130,59130,59098,59163,38132,38164,38196,40277,40277,42358,52791,56951,44340,27600,29648,56952,29648,21262,42227,54838,46452,50808,48597,46452,31727,31728,33808,54806,44306,44307,52725,52693,52725,54838,54838,52726,52758,54838,52726,52726,54838,52693,50645,52758,52693,50612,52693,52693,48499,50580,52693,48467,48467,50613,48467,46387,50613,48500,46355,48467,48499,44274,44307,48500,44306,42194,48467,46354,42161,46355,46419,40081,42194,46387,40048,40048,46354,42193,40113,46387,44339,46486,57018,59098,59098,59098,59098,59098,59098,59130,59130,59098,59130,59130,59130,59131,59131,59131,59131,59130,59131,59131,59131,59131,59131,59131,59131,59130,59131,59131,59131,59163,59131,59163,59098,59131,46520,40213,50745,44308,27602,15024,19250,15023,19249,15024,38001,52726,56951,56951,56951,56951,52725,50645,50677,52790,56951,56951,50645,50645,50710,54903,56919,54871,50678,52758,52790,54903,56919,54871,54871,54871,54839,48468,54871,54870,54838,48435,44177,54838,54838,52725,52725,52725,46386,35919,27405,16872,16937,23276,25357,25390,25357,25389,33582,35370,35564,25357,31663,29550,29583,27470,23244,16937,25389,23339,25420,19205,19237,12868,8579,8579,10691,8579,10692,12803,17093,17061,8579,8580,43},
{52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50778,52859,50811,50779,48698,48731,38200,29814,34007,31926,38199,44505,40279,36022,35989,38134,46585,40280,38167,25621,25621,25620,52793,57017,59097,59065,61211,48664,50777,48696,50745,50777,50809,54904,54871,48532,35953,38034,54838,40114,33873,44372,54838,46453,42389,44404,48565,35921,35986,35985,54871,33808,33841,52725,54838,31630,29550,31630,19018,14858,25324,23243,14857,21163,27469,14793,19018,31630,12745,14825,29582,16938,12745,23243,23276,12745,19051,27437,12810,21196,33808,12745,12777,29582,19050,12745,23276,25389,10664,19018,27469,12712,16938,33775,12745,12745,27436,19050,10664,23243,27502,14889,23276,33775,31727,57050,59098,59098,59098,59130,59130,59098,59130,59130,59130,59130,59130,59130,59130,59130,59131,59131,59130,59131,59131,59131,59130,59131,59131,59130,59130,59131,59130,59130,59130,59130,59131,57017,57050,44407,38100,52858,44308,29650,15024,19250,15023,21330,15023,38002,52758,56951,56951,56951,54838,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,56951,54839,56951,56951,56951,56951,54838,56919,56919,54839,54838,52758,56919,56919,54838,44307,40082,50612,54806,46387,50580,52692,42225,31694,19018,6405,14792,25358,25357,23276,23244,23277,31501,33224,33451,25357,33743,29550,29583,27502,25357,14824,23308,23309,23309,21256,17092,12836,8579,10724,8579,8579,14916,12868,14916,10691,8547,6435,29},
{52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50811,50778,48698,48731,46586,34039,34007,40280,46586,40247,38134,36054,33941,40215,36054,33974,44505,25621,36087,31862,48600,54905,54905,54873,59131,38132,31826,31826,31826,31890,33971,46517,54870,54871,46420,46420,46387,42194,40113,46452,52725,46420,42323,46452,50613,42259,42258,44339,52757,44339,44339,52726,54838,19083,21196,25422,19083,10665,21229,23277,10696,19083,29615,10664,14890,29583,12744,12777,23309,19083,10664,21164,23309,10664,17003,25357,14857,23309,31761,12744,12777,23276,21163,12744,21196,25389,12712,16970,25389,14791,19050,29582,16905,16937,21130,25323,19050,23276,29517,23210,25389,31695,40147,57017,59097,59098,59098,59130,59130,59130,59130,59130,59130,59130,59130,59131,59131,59130,59131,59131,59131,59131,59131,59131,59130,59131,59131,59130,59130,59130,59098,59098,59098,50646,50679,46485,48566,38067,44373,48632,42228,25456,17072,19282,15024,21330,17104,40115,48402,48434,50482,50482,46289,48401,48434,48434,48369,48402,46321,48402,48402,48401,48369,48369,46256,46288,46321,46289,48369,46289,44176,46288,46321,46288,50515,50515,50514,50514,37903,42097,39950,39984,33678,37838,37838,35854,29646,23276,12744,19051,27536,23309,25357,23277,25357,33646,46063,33676,25389,31695,27437,29550,27502,27502,12744,23309,25389,25389,23338,19172,12803,10691,12868,14948,12836,12804,10692,10659,6435,8580,6467,33},
{52891,52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50778,50779,48698,48731,42457,29814,27733,34039,38167,38135,36054,36022,36021,40280,27701,29813,27701,31861,46585,25588,25588,23508,25556,25555,52825,52825,50744,50744,50744,50776,50776,54904,56919,50678,23244,29615,33808,23211,17035,38033,54806,48532,46452,44404,38000,29582,35856,40081,40048,27437,29615,50645,52725,44306,42259,38034,40113,44306,35920,42194,46387,40146,42194,46387,44274,40082,46387,46419,38033,44339,48532,35953,42226,48564,42226,40081,50612,46419,40114,50613,50645,40081,48500,52726,40113,46387,52757,44339,37968,54805,50580,42193,52757,52757,42226,50612,54806,44307,48468,52758,48500,46420,52825,59098,59098,59098,59098,59098,59130,59130,59130,59130,59130,59131,59130,59131,59131,59130,59130,59130,59131,59131,59130,59131,59130,59131,59163,59131,59130,50679,46453,42194,37969,35888,35889,33808,31728,25390,42260,40180,38002,29550,21231,21330,15023,19249,17136,42195,46288,48336,48369,48369,48401,44176,48369,48369,48369,46288,46289,46289,48369,48369,46289,46256,44208,46256,46288,44176,44175,44144,39983,42095,44143,44176,48369,46256,46256,46256,40049,42129,44144,46256,42031,42030,44111,40080,25356,12778,14890,19050,23276,25357,25357,23309,25389,27438,29485,33775,25389,31663,27437,27502,29550,27470,16937,23309,25389,23309,21257,17060,14947,12803,12868,12835,17060,12868,10660,8547,8547,10660,10724,48},
{52891,52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,48730,48698,48699,44506,27733,29814,33974,38167,31861,33941,36021,38102,48633,52923,44505,25621,50778,36119,23509,23508,23508,23508,23508,50745,42358,35987,33874,33874,33906,36051,44372,54870,52758,42129,40081,38033,37968,42161,46420,54806,38066,33970,36116,31760,12843,19051,31695,38001,16971,19116,46452,54806,38001,38001,42226,40114,35888,40049,44339,31727,33775,50678,31662,31662,50613,31695,31695,42226,38001,29582,38001,44307,27469,29549,52726,27437,29517,52726,29549,27501,44306,35856,27469,38000,42226,23308,29582,52726,23243,25356,52758,25356,27436,44306,33775,25388,37968,42226,25388,31662,52694,40115,59066,59098,59098,59098,59098,59098,59098,59130,59130,59130,59130,59130,59130,59130,59130,59130,59130,59098,59130,59130,59130,59130,59130,61211,59163,59163,46453,46420,44372,44340,42260,42227,40179,38067,33808,44307,42227,48566,40081,44243,23344,17104,19249,15023,46388,46288,46288,48369,48369,48369,23177,16904,19017,31596,42160,23210,8452,10532,16839,31630,40113,12645,8420,10565,27436,35920,31629,10565,10598,14823,27370,44274,46256,46256,46256,37837,39918,46224,46223,42062,42030,42031,40048,25422,8584,6372,16937,23276,25357,25357,25357,25389,25422,27470,27437,27438,29550,29582,27470,29582,25390,31727,23276,25389,23276,19142,19173,14948,12835,14979,14948,12836,8579,6435,8548,8580,12740,14916,68},
{52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,48698,48698,46618,31926,31926,31893,33941,36022,36022,35990,46553,57084,57116,27733,27733,44473,25589,21461,23508,21428,21428,23508,40246,40278,34004,34005,34005,34005,36053,42325,54838,56951,38001,38002,48533,46452,35921,42227,54838,44274,33808,31760,33775,27436,33774,35855,35888,29548,31662,48500,54838,35921,35888,46420,42194,33808,42194,48533,33808,35888,54839,31760,33808,52758,35920,33873,46452,42227,33808,42194,48500,31695,33775,54839,31695,33808,52758,33808,33840,46420,40114,31759,42194,46419,29614,35856,52758,27501,29614,52758,29549,29549,44307,35855,27437,37968,44274,33807,40113,48500,40114,59066,59098,59098,59130,59130,59098,59098,59130,59130,59131,59131,59131,59131,59130,59130,59130,59130,59130,59131,59130,59130,59130,59131,59163,59163,57018,42260,44340,44340,42292,42260,42260,40179,40147,33808,46452,42227,48599,44242,44274,42195,19183,21329,19184,46355,46289,48336,48369,48369,48369,18919,10598,14791,16872,33807,25290,8485,10565,14824,33775,46420,12678,10565,12678,23243,35921,33774,10565,10565,12711,12710,40049,46224,46256,44144,16872,23211,46256,46223,41998,39918,39950,35887,17004,8617,6405,23211,21131,23276,23244,25389,25357,25422,27469,31663,27470,27502,29582,27470,29583,27470,23276,25389,21225,17029,14948,12867,12803,12772,12804,14916,12836,10692,10692,10692,10692,12772,12804,62},
{52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50779,48730,48698,42425,34007,46618,38167,36022,36022,33909,33941,38134,48666,31862,25621,25621,25621,21461,21461,23540,21460,21428,23508,21428,21396,21428,21396,21428,21460,21428,29748,56951,56984,25423,25455,46452,44373,25455,35953,54806,52726,54871,50678,50645,50613,48565,46452,50645,48500,46484,50645,52725,35920,35888,46387,42227,35921,42226,48532,38001,40113,52758,38033,40113,52693,42194,40146,48532,46452,42194,46420,50613,42226,44339,52758,42258,44339,52725,46387,44371,50612,48532,46419,50612,52693,48500,48532,52725,48532,50580,54806,50613,50645,52725,52725,52725,52758,52757,52725,52758,48500,50645,59065,59098,59098,59130,59130,59098,59130,59130,59130,59130,59130,59130,59130,59130,59131,59130,59098,59098,59131,59130,59131,59130,59131,59163,59130,57050,40146,42227,42260,42260,42260,42260,40179,40147,35986,44373,44341,48598,46420,44274,44274,42194,23377,25522,46289,46288,46288,48369,48369,48369,18952,16871,18984,21097,33742,25291,14758,14759,16871,18984,42194,18984,16904,19017,19016,29581,37935,19016,19016,21097,21097,40048,46224,46224,44143,31564,31596,44176,46224,42031,44111,44111,35919,16972,10632,10599,23244,23211,23276,21196,25357,25357,25422,25389,33808,27502,27502,29582,27470,29583,27470,23276,25388,23305,19174,15012,12867,10755,10755,14948,17092,12836,14949,10660,8515,10660,8547,10724,29},
{52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,48730,48698,48698,42425,48698,63390,44473,38135,44473,44473,46585,40311,38199,25621,25621,25621,25654,21461,21461,27734,21428,21428,23508,21428,21428,21428,21428,21428,21428,21428,25587,46454,46485,25455,25488,38066,40147,25520,33874,54838,48500,44437,44436,52791,33841,31793,33873,54838,35921,33873,48533,54838,48500,52726,48468,40016,40016,44274,42161,35823,40081,44274,33710,37936,40081,31629,35823,42193,33710,31629,37968,35790,29549,37936,40048,27436,33743,42161,27371,31597,40081,29516,27436,35855,31662,25291,33742,35790,23178,29517,40049,21065,25291,35855,23210,21098,31629,29517,19018,27404,31630,33743,59098,59066,59098,59098,59130,59130,59130,59130,59130,59130,59130,59130,59130,59130,59130,59130,59098,59098,59130,59130,59130,59130,59130,59131,59130,59163,57051,61244,61244,61276,61276,61309,61276,59163,61277,40212,44374,48599,48566,44275,44274,44274,44275,35955,46289,46256,46288,48369,48337,48336,46321,46321,46289,44208,46289,46321,46289,46289,46289,46321,48401,46321,48369,46321,46289,46353,48369,46288,46288,46321,46321,48434,46256,46256,46256,46223,46224,46256,46256,37838,33612,33580,31725,23310,8519,25422,23276,23244,23276,21163,23309,25357,27470,25357,33808,27502,27502,29582,25389,29583,27470,25357,25389,25356,21254,19172,19172,12900,14980,14980,12835,12803,14916,12836,8547,6403,10660,10724,24},
{52891,52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,48698,48698,46586,48666,61309,59229,31926,36086,36086,38167,50746,61278,55036,27733,25621,29782,25621,21461,23573,29814,23508,25621,21428,21428,21428,21428,21428,21428,21460,21428,21428,21428,21393,21327,23407,21327,21261,23407,31761,54838,46452,42324,42324,52791,27535,27567,29647,52726,29615,27535,46452,54838,31663,31663,33743,25357,14858,23244,29615,12777,19083,33776,12745,14890,27470,16938,12777,25357,23276,12745,19083,25389,12777,19083,29583,10664,14858,27502,16905,12712,25421,23276,12712,21131,27470,10697,16970,29615,10631,14857,29615,14825,12712,25357,23243,10663,21164,29582,14857,23276,35921,42228,59098,50420,48275,50323,50549,59130,59130,59131,59131,59131,59130,59130,59130,59130,59130,59130,59130,59098,59130,59098,59130,59130,59130,59131,59130,54905,54938,61276,61276,61276,61309,61309,61308,59163,61309,38132,42261,48631,48598,46420,44274,44274,44274,46387,46288,46256,46288,48369,48337,46288,46289,46289,46289,48337,48337,48337,48369,48369,48369,48337,48337,46289,46289,46289,46289,48369,46289,46289,46289,46321,48369,48402,46256,46256,46224,46223,46256,46256,46256,35758,23211,25258,31756,31695,29583,25389,25389,23211,16937,12744,23309,23309,27502,25389,33744,27502,27502,31695,25356,29583,27502,25389,25357,23275,17030,17060,19204,15012,14979,19205,19172,17093,8579,8547,6435,6435,8547,8579,30},
{52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,48731,48731,46585,54971,61309,42392,27701,29781,40280,44440,31894,36022,54972,27733,27701,27734,23541,21461,23541,23541,23508,21428,21428,21428,19380,21428,19380,21428,21428,21428,21428,21428,19248,21359,19214,21294,21294,25488,31793,54806,52693,52758,52790,54806,52758,54806,54806,54806,52758,52758,52726,54838,33775,29549,27437,27469,23243,27437,29582,25323,27469,31695,23275,27469,27469,27468,29581,27469,31630,29517,29549,29582,29549,31662,31695,31629,33775,31662,31662,33807,29614,35887,35887,33743,35855,35855,35855,35855,37935,37968,35856,38000,40081,33743,40049,40113,35888,38033,42193,38001,38001,44306,59098,50065,47725,54452,56888,59130,59130,59131,59131,59131,59131,59131,59131,59130,59131,59130,59098,59098,59131,59130,59130,59130,59130,59131,59130,59131,59131,61276,61276,61244,61276,61276,61276,59163,63357,42293,46519,48599,48598,48598,44274,44274,44274,46387,46256,46288,46289,48369,48369,46288,33774,33839,33742,46289,46289,44176,12645,14791,29484,46289,46289,35822,27533,29581,39983,46288,46321,33871,38129,31694,46321,48402,46224,46256,46223,46223,46256,46256,46224,33678,21131,23210,33805,33807,29615,27470,27470,21131,8453,4292,23276,25389,27502,27470,31663,27502,27470,29583,27470,29583,29551,25390,25357,23308,21257,12835,17124,12899,17061,17060,17092,10691,6435,8547,6466,8579,10724,14949,37},
{52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50779,48731,48730,44505,38167,48666,59229,34039,29781,54971,38199,40247,48698,31926,29813,31958,29846,27734,27766,27766,25589,21428,21460,21428,21428,21428,21428,21428,19348,21428,21428,21428,21428,21329,23407,23440,23440,25488,25552,29648,52726,46452,27567,31793,35986,25422,19116,19084,29551,27503,19084,42226,52725,52725,52758,44339,50613,52726,46420,48533,50613,46420,46452,50613,46452,44307,48565,48532,42194,48533,46420,42227,46452,44307,40114,44340,44339,42226,42194,46387,42226,38001,46452,42194,35920,44339,38033,35920,42226,40081,38000,40081,40114,38001,31727,44307,37969,29614,44307,48597,44404,44340,38000,59098,56727,45644,47919,48340,59130,59130,59131,59131,59131,59131,59131,59131,59131,59131,59130,59130,59098,59130,59130,59130,59130,59130,59131,59130,59098,59130,59130,59131,59131,59131,59163,59131,59163,61244,42293,42326,48599,50711,50745,46388,44274,44307,48403,46288,46288,46288,48369,48369,48369,27500,29678,29613,46321,46289,42096,6372,10630,27436,48369,46289,31661,21291,25453,40048,46289,48369,36017,42388,40211,46321,48402,46256,44143,37805,37805,41998,39917,39918,31598,21163,23210,35919,29582,29583,25389,25389,12809,8519,4260,23277,25390,25422,29583,29550,29551,27470,25389,29550,29583,29583,25390,23276,21227,21224,19173,17092,17060,17093,12868,10659,8579,6499,10756,10724,14916,14980,14948,71},
{52891,52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,48731,48731,50778,48698,31926,38167,40279,31894,29781,59197,40280,38200,27701,29782,38167,40279,34006,42425,27701,29814,27734,21428,21428,21428,21428,21428,21428,21428,19348,21428,21428,21428,23508,27700,40311,42391,42424,42424,46617,48730,48730,29746,10730,8649,8617,40246,12843,10762,29681,21295,10697,40114,54806,31728,21196,23277,38067,21229,23342,38067,19180,21261,35953,25454,21262,27502,35954,23342,23309,38099,23342,23374,38067,21261,23342,40147,23342,23341,31695,31793,21261,23309,38067,23309,23309,38067,21229,21261,38099,21228,21261,33808,29615,21228,25389,38034,21196,23277,40114,31759,33840,42260,29550,59098,59065,59001,54420,48275,59131,59130,59131,59131,59131,59131,59131,59131,59130,59131,59130,59130,59130,59131,59131,59130,59130,59130,59131,59130,59098,59130,59130,59131,59131,59163,59163,59131,59163,61244,35954,42293,48599,50712,52857,50712,44242,46388,46321,46288,46288,46288,48369,46288,48369,33742,33839,35887,46321,46289,44208,31694,33807,37967,48369,48337,37935,33774,35855,44208,46288,48369,37967,40080,42161,46321,48369,46224,46224,44111,44111,44111,44111,44111,29484,21098,23210,33838,27469,21163,21163,23276,10698,8553,16970,25389,25390,25422,33776,25389,27503,27502,25357,35856,31663,29583,25357,16937,21257,14949,14917,14949,12836,17124,10723,10724,8547,8580,14948,12835,12804,12836,6467,59},
{52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50779,48731,38199,34039,29814,29846,38232,27733,27701,42360,36086,48731,33974,40280,42392,42392,31926,31894,29782,27766,34007,21460,21428,21428,21428,21428,21460,21428,21428,21428,21428,21396,21428,27700,36117,42423,42423,42391,44504,46649,48697,31859,10762,10762,10730,38133,12876,10762,25553,21295,10698,33776,52726,31696,19149,27568,31793,19117,21262,36019,14923,14923,33874,21230,14923,25422,29681,12810,14858,36020,14924,12810,31729,19181,12810,21197,29714,10730,14858,29681,17036,10697,23310,25488,10697,17003,25488,14923,17003,21262,19182,14858,12810,29714,12745,14857,23342,17036,21196,33841,27502,19083,33808,59098,59001,47822,43467,50453,59131,59130,59131,59131,59130,59131,59130,59131,59130,59130,59130,59130,59098,59130,59130,59130,59130,59130,59131,59130,59098,59130,59130,59130,59131,59163,59131,59130,59131,61244,27535,42326,48599,50711,50745,50744,46388,46388,46256,46256,46288,46288,46288,46256,46321,46321,48435,44241,46321,46321,46256,39983,42095,42095,48337,48337,42095,42128,40047,44208,48369,48369,33644,31596,37870,46321,46289,39917,39917,46256,46224,46224,46223,46256,37838,37870,39983,31724,31728,31728,29583,14857,10697,10632,14792,21164,23243,25259,25260,23277,23375,25422,25324,29550,25389,25357,29583,35920,21223,12804,14948,8547,8611,17092,10755,10788,10692,10724,10755,10660,6499,10659,8547,64},
{52891,52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,48698,38199,36120,29814,27733,27766,27701,33974,46650,48698,48633,52859,48698,42392,33974,33974,33974,29782,25654,27701,23508,21428,21428,21428,21428,21428,21428,21428,21428,21428,19348,21428,29780,36085,44504,44536,42423,44536,46649,48697,33972,10762,10762,10730,36053,14956,10762,27568,21327,10698,10697,36053,10730,10730,27601,19182,8682,14955,31859,8617,10697,29714,14956,8617,21262,23408,8585,8552,31859,10763,8585,23408,19182,8617,8617,33972,8617,8617,27568,17069,8617,14891,29746,8584,8617,29714,14923,10730,23375,23375,8617,8617,31827,10697,8616,23407,17036,8551,12809,29615,23342,42293,59066,59001,50258,50032,48242,59130,59130,59131,59131,59130,59130,59130,59130,59130,59130,59130,59098,59098,59098,59098,59130,59130,59130,59131,59130,59098,59131,59130,59130,59131,59131,59131,59130,59131,59163,31728,46519,48631,50712,52825,52825,46453,46388,46288,46288,48369,48337,48369,48369,48369,40243,48695,38097,46289,46288,37935,19146,25453,33742,46289,46288,31661,23437,27533,42128,46288,48369,27501,29743,33775,46321,48402,44176,46288,35725,25258,33645,46224,46256,41998,44111,42096,31757,31663,29583,29582,25422,10697,6406,10598,21163,25357,29518,29486,25357,27535,27502,29550,35856,31663,31663,25388,23306,19174,12836,14948,14980,17061,12835,17092,10755,10659,12836,12900,10692,8580,8547,10660,42},
{52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,48731,42425,31926,36119,36119,40344,29814,29814,40248,38167,44472,54939,57084,48665,40214,40214,35989,33941,31862,23541,25621,23540,21428,21428,21428,19380,21428,25621,21428,21428,19348,21428,21427,27700,38165,48697,46617,42391,46584,46649,46649,36053,10763,12908,12875,33972,15021,12907,25552,21359,10795,10794,36053,10795,12875,27601,19214,12843,14988,31860,8618,8650,27634,14988,8650,21294,23408,8617,10697,31827,12908,10827,25553,19214,10698,10730,36053,10795,10892,25585,19182,10697,14891,31827,8584,10697,29714,14988,10794,21327,23407,10762,10762,31827,12810,10729,25487,17068,10729,12809,31695,19084,35921,59098,56888,50031,54258,48275,59130,59130,59130,59130,59130,59130,59130,59130,59130,59130,59130,59130,59130,59130,59098,59098,59130,59130,59130,59130,59098,59130,59130,59130,59131,59131,59130,59130,59130,59163,33874,48600,50679,50712,50744,50712,48598,46387,46288,46288,46289,48337,46289,48337,48369,40243,48695,46582,46289,46289,37902,19178,27565,33709,46289,46288,31661,25517,29678,44241,46288,46289,27500,31791,38000,46321,48369,46256,46256,33612,18985,29483,46223,44143,35692,35725,33644,35950,33743,29550,27502,25389,14890,6405,12744,21163,25389,27502,25390,25357,29551,27502,29582,33776,31663,31663,27501,23335,19141,10723,14948,14979,21285,10691,12836,8579,12836,12868,14916,8579,8515,6467,8580,57},
{52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,48731,46586,52891,57117,52923,52891,44505,29813,36086,40279,48665,50746,40247,40214,40182,40181,44440,33974,25589,23541,25589,25588,21460,21428,21428,21428,21428,23540,21428,21428,19348,19347,23508,27700,38165,50810,50842,44504,46616,48697,46649,36085,10762,10763,10795,33940,17069,10795,25488,23440,8650,8649,36053,10763,8682,25553,19214,8650,12842,33973,8650,8617,27601,17037,8617,19149,25521,8585,8617,31859,12843,8682,23440,21295,8617,8617,36053,10730,10795,25553,19214,10697,12778,33908,8552,8584,27633,14923,8584,19181,23375,8552,8552,31794,10729,6470,23342,17003,6470,8518,35954,21164,35953,59098,59001,47984,41548,50420,59098,59130,59130,59130,59098,59130,59130,59130,59130,59130,59098,59130,59130,59130,59130,59130,59098,59130,59130,59098,59098,59130,59130,59130,59130,59130,59130,59130,59130,61211,33874,46519,48599,50679,50679,50679,44372,46387,48336,46288,46289,48369,48337,46288,46288,42128,42160,44241,46321,46321,46289,44208,44241,46321,48337,46289,46289,46289,46289,46289,46289,46289,46289,46289,46321,48369,48369,46256,46256,46224,44144,44143,46224,46224,29451,23211,27403,31662,31695,27502,27502,27470,21164,12712,21131,19051,23309,27502,25422,25389,29551,25389,31695,29582,31663,31663,27502,21225,23337,12836,12836,12804,19140,8611,8579,14948,14980,14886,14919,10628,10692,10660,12772,42},
{52891,52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,44538,31894,42360,36087,42360,46585,33974,36119,40279,46618,38167,36054,44472,40182,38102,42294,36053,31893,31861,27701,23541,23540,21460,21428,23508,21428,23508,21428,21428,21428,21427,21428,21428,27700,36085,50810,46616,42423,46649,48730,48729,36118,10763,10762,10763,31859,17069,10762,23408,23408,10698,8617,33972,12843,10730,25488,21295,10730,12810,33940,8617,8617,27601,17069,8649,19116,27601,8585,10697,29746,12843,8585,23407,21263,8585,8584,33940,10698,8617,25520,17134,10730,10730,31827,8584,8617,27601,14923,8584,19149,23375,8584,8584,29746,10729,6471,23342,17035,6470,8551,40212,21196,50712,54905,52727,52597,48178,42065,48501,48501,46421,54905,54839,54872,48566,52791,52791,54904,57050,59098,54937,52792,59098,54839,48501,46388,46388,44307,42195,40113,38000,33807,31662,31662,31694,31662,44275,52791,35954,46519,48599,50712,52825,50713,42291,48467,46321,46288,48369,48369,46320,46288,46289,48369,48369,48401,48401,48369,46256,46289,46289,48369,46289,46224,46288,46288,46288,46289,44143,33612,37837,37870,37838,37838,35789,35757,35725,35692,52727,59131,33645,33644,27404,27372,31630,31693,31696,27502,27502,27470,25357,10599,21131,10598,21196,27502,27470,25389,27502,25357,33808,27437,29582,29615,27500,12804,10692,8579,10692,12803,12868,12835,6434,14947,12899,12804,19176,14854,10628,8547,10692,68},
{52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,46618,40312,31926,34039,29782,29814,34007,38199,46553,38102,46553,52858,44440,38134,33941,40182,40246,38134,40214,29749,25621,23540,21460,21460,23508,21428,21428,19348,21396,21396,21428,21428,21427,25620,34005,46552,50810,44504,46649,48729,48729,38166,14988,10730,10730,31827,17069,10730,23407,23440,10730,10698,33940,12844,10730,25520,21295,10730,10730,33972,8617,8617,25521,17069,8617,17004,27634,8584,8584,29714,12843,8585,23408,19214,8617,8584,31892,10698,8584,25455,19182,10730,10762,31860,8585,8649,27601,14956,8617,19181,23408,8616,8616,29746,10730,8583,23374,17035,8551,8583,38100,8551,35986,59164,57018,50679,23211,21098,21098,16905,19018,48567,44242,50678,44373,50678,42226,52759,57050,57082,52824,44405,37903,33677,31564,29451,27371,25290,23178,14823,10630,8549,14791,14791,8485,8517,29549,50679,35955,44406,48599,50712,50745,50745,44437,29676,42260,52692,50644,50644,50612,50579,50612,50612,50612,50612,50612,44241,37869,39950,39918,39918,39918,37837,39918,39918,44143,39886,29419,35725,37837,35692,37773,37870,39951,37805,37837,37805,35725,35725,37838,39951,29517,23244,29516,29611,31696,27502,27502,27470,19018,14825,12712,10631,21131,25422,27470,25389,25357,23244,35888,16905,27502,27500,16997,14948,14980,17092,19204,17092,14979,12836,12836,12803,17092,14948,12868,19176,8579,8580,8516,61},
{52891,52891,50843,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50779,50811,36087,34007,31926,25653,29814,40280,38134,40214,40214,40214,42294,42294,42327,44407,44375,42262,40182,38069,40247,31861,23540,21428,23508,23508,21428,19380,21428,21427,21428,21428,21428,21428,25587,34037,44504,46616,42423,46617,48697,48730,38198,14988,12843,10762,29746,17101,10730,21262,25553,8650,10698,31860,14956,10730,25488,21327,10730,8649,36053,10730,10697,27569,17069,8585,14923,29714,8584,8584,29714,14924,8585,25553,21295,8617,8584,31892,10698,8584,23407,19182,8649,12843,33940,8617,8682,27633,14988,8649,19182,25488,8584,8649,29714,12810,8583,23374,17036,8551,8583,31826,8583,35954,63390,59164,61277,29615,21098,21098,21131,21098,48567,46387,46387,33741,44306,44274,50678,57082,57082,52824,42293,33742,29483,27403,27403,27403,27371,25290,23210,21129,21129,23210,18984,10630,10598,14823,27467,35953,44439,48566,48632,50745,50745,50679,38065,42292,50612,50612,50612,50612,50612,50612,50612,50612,50612,50612,44241,39950,37903,39951,39951,42063,42064,37806,42096,42031,44176,42096,42064,39983,40015,39983,39951,37870,35757,39951,37870,35758,33645,39951,37903,23276,23244,27436,27531,31696,27470,27502,21164,8518,12744,25390,23244,19082,25390,27470,8486,8486,19051,35888,16905,25325,23275,21192,19175,17060,17124,14948,12868,14980,10723,17092,17124,17124,14916,12772,10628,10660,10724,8548,56},
{52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,48731,48731,40312,25653,27734,25654,27734,44473,40280,33942,36022,38134,38102,38069,40182,38102,40182,40182,38069,35989,36022,36022,44505,25620,21428,21428,21428,21428,21428,21428,21428,21428,21427,21427,21427,25587,36053,48697,46616,42391,46617,48729,48730,38198,17133,10762,10762,29746,19214,10762,19214,27666,8650,10698,31859,14956,10762,25488,23408,10730,10729,36086,10763,10730,25520,19150,8585,12843,31827,8584,8584,27633,14924,8585,21327,23408,8617,6504,33940,10730,8617,23408,19182,8617,8649,34005,8584,8585,27601,14988,8616,19149,25520,8584,8584,29714,12777,6503,21294,17068,8551,6503,31826,8583,14890,31761,38067,63389,29583,19018,19018,19083,16937,48599,50646,48500,37935,46420,46387,50646,57017,57018,50711,42325,35855,27403,27403,27403,27403,27403,29451,29451,29483,29483,29483,25290,23177,12711,16967,19079,33840,42326,48566,50680,50744,50744,48631,44405,44373,50612,50612,50612,50612,50612,50580,46517,50710,42226,50612,42161,31629,42291,40178,37967,39983,37870,40145,48629,38033,42096,44144,35790,48662,48662,35855,37870,35725,35758,42031,35758,37871,35757,39950,33645,23276,21131,25323,31725,33775,27470,27502,19051,19084,6438,29551,19018,12743,25390,21196,6436,6405,14857,31662,10599,27438,27469,27468,14918,10756,8643,14980,12868,17060,12836,17093,12836,14980,12835,12804,10691,10692,8579,8612,31},
{52891,52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,48731,44505,27734,25653,29847,27766,31894,46585,42392,29781,31861,38167,46520,48665,44440,40247,38102,40214,36022,33941,40247,36022,42295,36119,23508,21460,21428,21428,21428,21428,21428,21427,21427,21428,21427,23539,36053,48665,46617,42391,44504,46617,48729,38198,17101,12908,12908,29746,21295,12907,19214,29779,10763,12875,31859,14989,10730,23408,23408,8682,10730,34005,12875,10762,25520,19150,8617,12843,31860,8617,10730,27634,14988,10762,23407,23440,8585,10730,31859,12843,10795,25521,19182,10795,10827,36053,8682,10795,27633,17068,10794,19214,25521,8681,8681,29713,10729,8649,23374,17036,8648,8616,31826,8616,8648,25519,33841,63422,38034,40113,37968,37968,38001,52825,44242,50614,33808,44340,40114,40147,42227,33775,44404,42293,35888,29484,29483,29483,29483,29516,29516,29483,29515,31564,31596,29450,27370,21097,21160,19079,31792,44406,48566,50679,48598,48566,48566,46453,44373,50612,50612,50612,50612,50612,48500,44502,50808,40211,50612,40016,31661,46614,42389,40081,42096,39983,40210,48695,42323,37903,39918,35823,48760,48727,42226,39983,35725,35725,42031,16871,8452,25290,35725,31532,25258,12710,16937,29613,29583,27502,21164,12777,29616,19084,21132,10631,6436,21226,12806,6468,4323,12711,31662,8518,23276,27468,16997,14948,14949,10692,10691,12803,10724,10691,17060,14948,12899,19205,12868,12836,12772,8579,10660,49},
{52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50811,50811,50811,50779,50779,50779,46618,31926,27766,25654,25653,25653,42360,44505,36119,36087,38167,40247,42327,40215,36021,38134,35990,35989,36022,38102,50713,42327,48600,38199,23508,21428,21428,21428,21428,21428,21428,21428,21460,23476,21427,25620,33972,46616,48729,44471,44504,46617,48730,38230,17101,10730,10762,29746,19215,10763,17068,31859,10698,10730,29746,14956,8650,23375,23408,8650,8649,33972,12843,10697,25488,19182,8617,10730,33940,8585,8617,27633,14988,8617,19149,25489,8585,8585,31827,10762,8584,23407,19182,8617,8682,33973,8617,8649,27601,17036,8649,17068,25553,8584,8616,29681,12777,8584,23407,19148,8583,8584,31827,8583,6470,25455,23309,27534,33841,50612,48435,44274,44275,54873,44242,46387,21098,35920,42227,27469,40147,25324,42259,44373,37968,29484,29516,29516,29483,29516,31564,31564,31564,31596,31628,31563,29483,21065,21128,23305,33840,46519,46486,48599,48599,48599,50679,46518,44341,50612,50612,50612,50612,50612,48499,42324,46549,44404,50612,40015,33742,40210,40146,37968,39950,44176,35920,42258,42194,35758,39950,39983,42259,42259,40081,39983,37838,33645,37838,27403,21097,29484,35757,31500,27339,18984,23243,29613,29582,27470,19019,16971,21132,10632,12745,21195,8580,17093,10723,10725,6403,8549,25388,6405,21131,25420,19206,12867,10755,8579,14948,12803,12900,10723,10755,14980,14948,12835,14885,17060,12803,12836,8579,48},
{52891,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50811,50779,50779,50779,50779,46618,29814,25654,25654,25654,31959,38167,38232,29781,42392,52858,46585,44472,36086,33973,33942,31894,33942,36022,40182,44407,59197,57084,48730,23540,21428,19348,21428,23508,21428,21428,21428,21427,23508,25588,27667,36053,48665,48729,42391,46584,46616,48730,40311,19214,10762,10762,27666,21295,10730,17004,33940,10730,10730,31827,17101,10730,23375,25521,8650,8649,33940,12876,10730,25488,19182,8617,10697,33972,8585,8617,27601,14988,8617,19149,25521,8585,8585,29779,12843,8682,23440,19182,8649,8682,33972,8649,8650,27601,17069,8649,17069,27601,8584,8616,29713,12810,8584,23375,19116,8583,8584,31827,8583,6470,25422,14923,10664,35953,50547,46420,52825,48566,56986,48533,52760,44373,50711,50679,44373,40180,23211,40179,44373,38002,29549,29549,29516,29548,31596,31596,31628,31628,33676,31628,33676,33644,27403,18984,23273,29614,48632,48566,48599,48566,48599,48599,46518,42292,50612,50612,50612,50612,50612,50612,50612,50612,48532,50580,42096,39951,37903,39983,37870,42063,42063,40016,42096,40016,42096,44144,44176,42031,44176,44209,44176,35758,37806,37838,42031,37870,37838,39950,33645,33678,21128,23242,29613,27502,27470,21164,14857,19083,12745,29649,27501,12805,10723,8643,12836,8611,8549,4323,6404,19082,19082,14919,17060,17093,14948,19172,17060,17061,15013,17093,10691,14948,14948,8579,21256,15012,10691,12868,52},
{50811,50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50779,50779,50779,48731,31926,31927,27734,25622,29814,40313,34039,29814,36119,52891,54939,54906,57052,38167,36087,31862,34039,29749,27669,44473,59164,54939,40214,46553,21428,21428,19348,21460,21428,21428,21428,21428,23540,25588,27700,25619,34005,42391,46584,42423,44503,46616,48729,40311,17133,10795,10795,27633,21295,10730,12875,34005,10698,10698,29746,17069,10698,21262,27633,8650,8650,31859,12876,8617,23407,21262,8617,10665,33973,8617,8585,27601,17069,8617,17069,27601,8585,8585,29747,12875,8617,23407,19214,8617,8617,33973,8617,8617,25520,17069,8584,14956,27633,6471,6471,29681,10697,6470,23309,19084,8519,6471,31826,8584,8551,25423,14923,10664,38033,50547,46387,46388,48468,56986,44242,50646,44372,48631,42194,42260,42228,23211,42227,44341,38034,31597,31629,31629,31629,31629,33709,33709,33742,35822,35854,35822,35821,35789,27403,25354,31694,38100,48567,48566,48566,48534,50647,48566,44340,50612,50612,50612,50612,50579,50580,48499,50612,50612,50580,39983,37838,35725,37838,44144,37870,39951,39983,39951,39983,42095,44144,39951,42064,42063,44176,42096,37838,37838,37870,37837,39918,37805,37805,33644,31564,25384,25387,31726,27470,18985,16970,8616,12809,12775,17001,14949,14949,10756,8644,12900,10723,8549,6467,10692,12710,16937,6438,10691,12868,10723,17028,14947,12804,15012,15011,12836,12868,17092,8579,10724,17124,12804,14948,62},
{50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50779,50779,50779,50779,46618,34007,34039,27734,23573,48731,44505,40312,33974,40279,57084,42359,46552,52858,59229,46586,38232,29814,31894,36119,38199,46585,59165,42392,36119,23508,21396,21428,21428,21428,21428,21428,23508,23508,23508,23540,25587,36085,42391,48697,42423,44536,46617,48730,42391,21295,10795,10762,27601,21327,10730,14955,36085,8650,10698,29714,19182,10730,19214,27666,8682,10730,31827,14989,8650,23407,21295,8585,10665,34005,8617,8584,27601,17069,8617,17004,27634,8585,10697,29747,12876,8617,23407,19214,8584,8552,33972,8617,8584,25520,17069,8584,14923,27666,6439,6471,27601,10697,6438,21229,19116,6470,6438,31826,10664,8551,25423,14955,8615,40114,50579,40081,38034,42227,56985,44242,46387,27436,40113,42193,27468,42293,23243,33775,38034,54905,59162,59162,59130,59130,59162,61210,61210,61210,61210,61210,61210,61210,61210,29516,27435,31727,38067,46486,48566,48566,48566,48599,48566,46421,50612,50612,50612,42160,42096,42128,42128,44306,44306,46387,46387,33644,27371,29451,29484,39983,42096,27339,31564,33677,35822,44209,33644,25258,29451,31597,33711,33677,33612,31564,35757,33612,33612,31564,31531,33612,21190,27467,38001,23244,8518,19051,10631,21229,12869,10692,12868,10755,12868,8643,10756,6467,10725,8579,4323,4323,17002,8582,14916,12899,14980,15012,15012,17061,14949,17124,12867,10691,14948,12836,12868,10755,12835,17092,44},
{50811,50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50779,50779,50779,50779,42425,34007,34039,25654,34007,57084,46585,36119,36087,52891,59164,46585,42392,40247,48665,50778,57084,46618,40313,40280,50811,52923,54971,59197,40312,23508,21428,21428,21428,21460,19347,23508,21395,21396,21428,21427,23507,34005,46584,48730,46617,46617,46649,48729,42391,19247,10795,10730,27601,21327,10730,10762,36085,10730,8650,27633,17134,8682,17101,29779,8650,10730,31827,14989,10730,25456,21295,10698,8649,34005,10698,10730,25520,17101,10730,17069,29747,8585,8617,29714,14956,10795,23440,19247,10762,10762,33972,10762,10794,25553,17101,10827,17100,29714,8649,10761,27633,12810,10761,23407,17068,10761,10761,31826,10696,10729,23439,14923,10696,38001,48499,48500,50646,50646,56986,52727,48467,46353,48467,44273,46386,44341,27437,23276,19018,48598,61210,61210,59098,42161,59065,59162,59130,59130,59130,61210,59130,61210,61210,27469,33840,44405,40213,46454,48598,48566,48566,50679,48599,46453,52692,52692,50646,21130,14726,18984,19082,31825,40276,44469,50613,16839,14889,19147,27534,44340,37967,10598,10695,19082,31792,46452,23145,8485,12711,14856,29615,33645,33645,33645,35757,33645,33644,33677,37838,35789,25385,31725,31695,19051,12746,21163,33842,23307,17062,10755,10692,12867,10723,10756,8612,6499,12837,12836,8579,6467,19083,6437,10724,14979,17124,14980,19174,8611,12836,17124,15012,14916,17029,14916,14948,14980,10691,17060,60},
{50811,50811,50811,50811,50811,50811,50811,50779,50811,50779,50779,50779,50779,50779,50779,48698,36087,31927,25654,27702,48666,52858,54971,40312,46585,38199,52859,44472,36087,46553,42425,36054,36086,36087,40344,44505,46617,50778,54971,61277,42392,21428,21428,21460,23508,23508,21396,21428,19348,19380,21428,21428,23507,34005,46552,52923,48697,46617,48697,48762,42424,19247,12940,12875,25553,23408,12843,10795,36085,12875,12940,27666,19214,12875,17101,29779,8617,10698,29746,17037,10762,23407,21295,8649,8617,33973,10698,8617,25520,17101,8585,12842,31827,6537,8617,29746,14956,8682,21327,21295,8650,8649,33972,10730,8649,25488,17069,8617,12875,29714,6503,8583,27600,12809,8583,21294,19148,8583,8584,31826,8616,6503,25455,17003,8583,33775,48466,44307,48631,44340,48534,35921,33839,19143,12837,10725,14855,33776,35921,23276,14857,54904,59162,59162,56985,48468,54904,59162,59130,59130,59130,61210,59130,61210,61210,27469,36018,61244,42293,46486,48566,48566,48566,50679,48598,48533,50644,52692,52725,33808,12678,18951,19016,33774,35919,42258,50613,16838,14791,21097,37837,37967,37935,12646,14823,25323,35855,48565,21065,12710,19017,29517,35855,33677,35758,35725,35757,33645,35724,33644,37870,37936,33741,35919,27501,8585,10633,35954,31728,33872,17062,12836,10755,12867,8611,12836,6467,8644,8580,6435,4323,6403,8517,12711,4291,10756,15012,15012,19173,19175,14980,14981,14980,10755,14948,17092,14980,19173,17060,12835,58},
{50811,50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50779,50779,50779,50779,50778,46650,27734,25654,38232,59229,54971,59164,48698,31893,36054,40247,42360,34006,36054,48698,42360,42425,38200,34039,38167,44505,42392,50778,61309,42425,23508,23508,21428,23508,23508,21428,19348,19347,21428,21428,21427,21427,31892,44471,52923,46616,46649,48729,50810,44504,21327,12843,10730,23440,23440,10730,10698,36085,10795,10763,27633,21295,10698,17036,31860,8617,10698,29746,17037,8617,21263,21328,8617,8617,33972,10730,8584,25488,17101,8585,12810,31860,8584,8617,27634,14956,8617,21262,21295,8649,8649,31860,10730,8649,25520,17069,8617,12843,27633,8583,8616,25520,12810,8583,23342,19149,8583,8551,31794,8584,8518,23374,14923,8583,33807,48467,44274,42228,40147,31695,33841,38034,23371,15014,12837,12774,27503,40146,25356,16969,54904,59162,59162,56984,42194,56984,59130,59162,59130,59130,59130,59130,61210,59097,27469,38131,61244,44406,46486,48598,48566,48566,50679,48566,48565,50612,50612,50644,46486,12645,16839,21097,37935,46353,48532,50613,14758,16871,21065,39982,42191,35854,14726,16904,27436,33775,50613,18984,14726,21032,33643,33709,33612,33644,31564,33645,31564,31531,33644,33677,35823,35791,40113,25422,8553,6438,23341,29614,17062,12899,10788,8579,15011,10723,12868,10756,10725,6467,4323,4323,4355,8549,14856,6403,14885,17156,12836,12804,31789,21286,21286,12867,17092,17124,17092,14980,17092,14979,19205,65},
{50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50779,50779,50779,50779,50779,44505,48666,36152,29846,54972,57084,38166,55004,50843,42392,36087,36119,33974,29813,33974,34006,27734,27701,25621,25621,29814,29782,27734,36087,59164,46585,23508,21428,21460,23540,25588,23508,19347,19348,21427,21395,19347,21395,31925,42391,50778,44504,46617,46649,48762,44536,21327,10763,10730,23375,25521,10730,10698,34005,12875,10763,25553,21295,10730,14923,33973,8617,10730,29714,17069,10698,21230,23376,8585,6504,33940,10730,8617,25488,19182,8585,10730,31860,6536,8585,27634,14956,8649,21262,23375,8617,8584,31827,10698,8617,25488,17069,8616,12778,29681,6503,6503,25520,12810,8551,21229,19116,8551,6471,31794,10664,8583,23374,14955,8583,35919,48499,35824,40180,38068,29616,25422,35987,33906,21290,14982,14951,25357,35888,31695,33807,52823,59162,59162,54904,50646,54904,59130,59162,59162,59130,59130,59130,61210,59098,25226,44114,59164,46519,46486,48598,48566,48599,48599,48534,48566,50646,50646,50646,48631,33808,29581,25355,27467,19015,27467,50645,12709,16871,19049,38001,46452,37935,16903,18984,29582,46486,50613,23241,21128,19048,29416,37902,31564,35693,33709,33645,35724,37838,35692,33612,33676,31563,33742,25390,6439,6438,16937,12806,15012,12835,10755,10723,10755,12867,10787,10788,12837,6435,4323,6467,8579,8547,21194,23276,21130,6436,4355,8484,35952,23303,19206,15012,12867,19236,14979,17093,15011,17092,12867,63},
{50811,50811,50811,50811,50811,50811,50811,50779,50779,50779,50779,50779,50779,50779,50779,48666,34007,34039,38199,57084,54971,44472,57084,44537,31894,40280,36054,31926,31862,33974,31862,27734,25621,27734,27734,29847,25654,23541,25589,29781,27701,23540,23508,23508,23476,25588,23508,21428,21428,21428,19348,19380,19347,31893,40246,48665,42423,44536,46617,48762,44537,21360,10731,10730,21294,27601,10730,8682,33972,12908,10795,25520,21295,10762,12843,36053,8650,10763,29714,17101,10730,21262,23440,8585,8584,31859,10763,8617,25488,17102,8617,10697,33940,8617,8649,27633,14956,8584,21229,23408,8585,8584,31859,10730,8617,23408,19149,8617,10730,29746,8649,8583,27568,12777,8551,21196,19148,8551,6470,31794,8584,8583,23342,14923,8518,40113,50645,50581,50613,50646,50646,50678,48566,46486,50679,40211,46550,14825,31728,27469,25324,50646,61210,59162,57017,46420,57017,59162,59130,59162,61210,61210,61210,61210,61210,27242,50388,59163,48632,46486,48566,48566,48599,48598,46485,48566,48566,48598,50679,46551,33841,23210,25387,38034,23275,27500,50612,16902,12709,12709,29581,42194,37902,19015,21128,21162,25355,46419,27434,23241,25354,23242,33742,29516,29451,31564,31564,35757,31564,31532,35757,31532,23208,16935,25422,6472,8519,19114,14886,14949,12899,14979,6498,6499,14979,15012,10787,14917,6467,4355,10724,8579,10724,12775,29550,25357,6437,4355,6436,29676,19142,14980,12836,14916,17028,10691,19172,17027,17060,17124,67},
{50811,50811,50811,50811,50811,50811,50779,50779,50811,50811,50779,50779,50779,50779,50779,48699,31894,38200,48698,55004,42360,54971,57116,33974,31926,38199,40215,40312,34006,33942,33974,29782,27701,34039,31959,27766,23541,23541,23541,25589,27701,25588,21428,23508,23508,25556,23540,23507,21460,21428,19347,21428,21395,31892,40278,44536,44503,46584,46649,48730,46617,23408,10730,10763,19182,29714,10730,8650,33940,12908,10763,25521,21327,10730,10730,36053,8649,10698,27633,17102,10698,19181,25553,8584,8584,31827,10730,8617,23407,19182,8617,12843,33973,8617,8617,27601,14956,8617,19149,23408,10795,10892,31859,10730,8584,25455,17101,8585,10730,31827,8616,6503,25520,12810,8551,19149,19149,8551,6470,31794,8616,8551,23374,14923,6470,38000,48532,50613,50613,50646,37969,31728,35888,48533,48598,48598,46453,12744,23244,38034,10566,48566,59162,59162,54904,54871,59130,59129,59130,56984,61210,61210,59130,61210,59097,23242,44178,50711,50745,48631,54905,54937,54905,54937,54905,54937,57018,56986,56953,42261,44406,40147,50679,46485,27468,29515,37967,23209,23241,25354,27435,23209,23177,18983,14822,10564,6371,8483,12773,16999,14887,14887,16967,12805,8548,14854,21130,10664,35889,6340,37969,23210,14885,14948,27534,16940,14825,27534,14949,10787,12899,10787,12899,10756,12899,15011,10755,19238,8611,8579,8611,8580,10662,19082,21131,25324,6437,4323,6436,31725,23366,21317,19236,12803,17060,14980,19236,19140,12867,10724,59},
{50811,50811,50811,50811,50811,50779,50779,50779,50779,50779,50779,50811,50779,50779,50779,46651,27766,36119,52858,48665,46553,46520,57084,48697,40280,38167,42359,38134,38134,40214,40215,42359,42425,34039,27734,27766,23541,23541,23541,27669,50843,29781,23508,23508,23508,23476,25588,23508,21428,21428,19348,21396,21427,29812,36117,38198,38198,42456,44568,48729,48697,23440,10795,12908,19214,31827,10730,12875,31859,15021,12908,25553,23407,10762,10795,34005,10762,10795,27633,19182,10795,19214,25554,8617,10730,29779,12843,8649,23472,19247,10697,10762,33973,8650,10730,27666,17102,10827,19279,25489,10730,10762,31827,10730,10730,25520,19149,8584,10730,31859,8584,8681,27600,12810,8551,21294,19149,6471,8616,31826,10729,10729,25487,14955,10631,46485,46452,50646,50613,50646,48533,48598,46485,46452,48566,46453,44340,8551,27469,38066,14792,50679,59162,59162,54904,44242,56984,59130,59130,59097,61210,61210,54903,42192,27434,16999,23242,38001,57051,57083,61277,63357,61276,63357,52824,61212,61212,61179,59002,35922,42260,48599,31726,21223,14884,12708,12709,8548,16902,16870,14790,12710,14790,12677,10596,8516,6371,8484,8516,12773,14854,16968,16969,14855,8580,19080,16969,27470,10632,12743,35921,31695,16998,10724,29614,29583,25357,25389,25420,12900,15043,10787,10787,10692,10659,17092,12867,14981,10691,6467,6435,6435,12774,14887,14823,19018,8550,4291,8517,35952,33838,27561,19204,17092,21254,17028,19204,15011,14980,12835,50},
{50811,50811,50811,50811,50779,50779,50779,50779,50779,50779,50779,50779,50779,50779,50779,38232,25654,44473,59164,44472,42360,48698,59197,59197,46585,38102,44440,40246,40182,46552,46520,52858,44472,42393,36184,29847,23541,25621,27733,42425,54971,44473,23540,23508,23540,23540,23508,23508,21428,21428,21428,21396,21427,29812,34005,36150,36150,40343,44536,46649,46649,23440,12940,10795,17069,33940,8682,10762,31827,15021,10763,25553,23408,8682,10730,33973,10730,10698,27601,19182,10730,17036,27666,8617,8617,29746,12876,8617,23375,21295,8617,8617,33973,8649,8617,27601,17069,8617,17036,25521,8585,8584,31827,10762,8584,23407,19150,8585,10697,31860,6503,8551,27568,12810,8551,19116,19149,6503,6470,31794,8584,8518,23374,14923,8550,46453,44340,46420,48500,46420,48533,50581,48532,46452,48500,48500,46420,12744,29615,42292,29615,54872,59162,59162,54904,48532,56984,59162,61210,56952,57049,59097,50644,37998,19111,10662,19049,35856,57051,57051,61277,61277,61276,63357,52825,61244,61244,63292,59001,42196,44405,50744,25482,19175,19078,12773,10660,12709,14789,16902,12709,14822,12709,10596,12677,10596,8516,8484,8484,10628,12742,16969,19050,21162,12805,23306,14856,33743,23211,33775,35921,33808,16998,12805,27534,27502,25357,27469,23338,10724,12931,12931,10755,12837,12804,12900,10754,15045,12867,6499,4355,6403,8548,19112,10629,8518,12743,10596,8517,33840,33774,23334,21318,19205,17028,27563,21286,21284,17092,21318,50},
{50811,50811,50811,50811,50779,50779,50779,50779,50779,50779,50779,50779,50779,50779,48731,36152,27733,50779,50746,46585,50779,42360,48666,48665,40280,36054,40214,40214,44407,54971,61277,57116,54971,44537,36152,29879,34039,44537,44537,31829,48665,59197,31893,25588,23508,23508,25588,25588,21427,21428,21428,21428,19347,27732,31924,34037,36117,40311,46617,46617,46649,23473,8714,10763,12875,33973,8682,10762,29746,15021,10795,23408,25488,10762,10762,33972,12843,10762,27601,19247,10762,17036,29747,8585,8585,29714,14924,10665,23375,21295,8617,8617,33973,8650,8650,25553,17069,8682,17036,25553,8585,8584,31794,12810,8585,23407,19182,8585,8617,31860,8584,8584,27568,12842,8551,19148,19181,6503,6470,31794,10729,8615,25487,17036,8583,38001,38034,23244,25324,33841,38002,50711,46453,25356,29550,46421,57018,19083,31696,50678,50678,57017,59130,59130,59098,44243,59097,59162,59130,56984,59162,61210,57016,48531,44369,19143,21226,31662,50745,55003,61309,61277,61277,63357,61276,63357,63357,63357,63292,44341,46486,42292,25482,19174,17061,12805,10660,10628,18982,16870,16902,14822,14790,12709,10596,10596,12677,10596,10596,12677,21162,21196,21196,16969,12868,23338,12774,33808,23243,37936,38001,31727,17063,12805,29614,29582,25389,27470,31759,23433,10787,15012,15012,14979,12867,12836,12867,17156,10786,10756,6403,4355,6403,14887,6372,6437,12743,4290,4323,31727,31695,27530,21253,27591,23366,25450,31758,19173,19204,15011,57},
{50811,50811,50811,50779,50779,50779,50779,50779,50779,50779,50779,50779,50779,50779,48731,29846,36087,48633,46553,54939,46585,40247,36022,38102,38166,40214,44407,42294,48633,61309,63390,61309,57084,44505,40377,40345,44473,40280,29781,34006,50811,59164,52891,27700,23508,25588,25588,25588,21428,27700,23540,21428,21427,27699,31892,34004,40310,40343,42456,44537,46649,25553,10731,10763,12875,36085,10731,10795,29714,17134,10795,23408,25521,10763,12875,34004,12876,10762,27601,19215,10762,14956,31860,8585,8617,29714,14924,8617,21295,19215,8682,8649,33972,10730,8682,25521,17069,8650,14988,27634,8585,8585,29714,12811,8617,23408,19150,8585,10697,33940,8584,8616,27568,14923,8551,21229,21230,8551,8551,31794,10632,10631,25454,17036,8616,14889,29648,8583,17100,29713,42293,59131,48599,27470,35921,46518,61245,14825,27470,48598,46420,52791,59129,59130,59097,50678,59097,59097,59097,59064,59064,59064,56984,56984,56951,21065,18985,31630,50745,54971,63357,63357,63357,63357,63357,63357,63357,63357,65438,48567,48566,44372,40177,27593,17125,14981,10693,8548,12741,12709,16902,14789,14790,12709,10565,10596,12676,14789,12676,12742,23243,23276,21163,16936,12868,25450,12804,35888,23210,37968,40082,25387,14981,14918,23307,29583,25389,27470,27533,19144,10757,12839,14950,12835,12931,10692,15012,10819,12899,8611,8547,8580,6403,6436,6468,6404,10630,4291,4324,31759,31663,31694,27529,40142,42258,35953,35953,31758,29707,17092,46},
{50811,50811,50811,50779,50779,50779,50779,50779,50779,50779,50779,50779,50779,50779,44505,29814,38200,44440,57084,52858,42295,42295,44375,42295,44375,42295,48633,48633,61309,63358,63358,63358,61309,48730,42457,40345,29814,25589,27701,52891,54971,59164,52858,31893,25588,25588,27636,27668,23508,23508,21428,19347,23507,25619,36085,42424,40343,40343,44537,46649,48730,27699,10795,10828,12876,38198,8715,10763,29746,19214,10828,21327,27666,8682,10762,36053,12875,8682,25553,19182,6537,12811,33973,6504,6537,29746,12876,6504,21262,21295,6537,6537,33973,8617,6504,23440,14989,6537,12876,27634,6472,6504,29747,10763,6537,23375,17101,6504,10697,33972,10697,8616,25488,17035,6438,23374,21229,8551,14922,27535,16938,12744,23309,25454,16970,17002,33809,14889,33938,42358,52857,59164,48599,33808,46454,48598,61244,23276,29615,50679,48598,48532,48532,48532,48532,48564,48565,50613,50613,50645,50613,46386,42160,35757,42128,18952,18985,33807,50745,57051,63357,61277,61277,63357,63357,63357,63357,63357,63357,50712,46486,44372,40145,23398,17060,14980,12804,10628,10628,10661,16934,14822,12677,10597,10565,12709,12709,14789,10564,12677,23243,21162,12775,10629,12901,25482,8579,38000,18984,37968,35888,21161,14981,17031,21227,29582,25357,25388,21225,19175,8612,10692,12836,12836,10755,10691,10755,12867,12931,15011,10723,8643,12900,8547,6435,4323,6436,4291,8517,29614,31695,31727,31695,44306,37968,38002,38002,35889,35920,19173,74}
};

const bool mdb_icon PROGMEM [306][78] = {
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
};

const bool mdb_per PROGMEM [306][78] = {
{1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
{0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
{0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0}
};

void disp_centered_text(String text, int h) {
  int16_t x1;
  int16_t y1;
  uint16_t width;
  uint16_t height;

  gfx->getTextBounds(text, 0, 0, & x1, & y1, & width, & height);
  gfx->setCursor((320 - width) / 2, h);
  gfx->print(text);
}

uint32_t rnd_whitened(void){
    int k, random=0;
    int random_bit1, random_bit2;
    volatile uint32_t *rnd_reg=(uint32_t *)(ROSC_BASE + ROSC_RANDOMBIT_OFFSET);
    
    for(k=0;k<32;k++){
        while(1){
            random_bit1=0x00000001 & (*rnd_reg);
            random_bit2=0x00000001 & (*rnd_reg);
            if(random_bit1!=random_bit2) break;
        }

  random = random << 1;        
        random=random + random_bit1;

    }
    return random;
}

int chosen_lock_screen;

void setup() {
  gfx->begin();
  gfx->fillScreen(BLACK);
  gfx->setRotation(1);
  chosen_lock_screen = rnd_whitened() % 4;
  
  if (chosen_lock_screen == 0){
    for (int i = 0; i < 320; i++){
      for (int j = 0; j < 240; j++){
        gfx->drawPixel(i, j, Dallas[i][j]);
      }
    }
  }
  
  if (chosen_lock_screen == 1){
    for (int i = 0; i < 320; i++){
      for (int j = 0; j < 240; j++){
        gfx->drawPixel(i, j, Tel_Aviv[i][j]);
      }
    }
  }

  if (chosen_lock_screen == 2){
    for (int i = 0; i < 320; i++){
      for (int j = 0; j < 240; j++){
        gfx->drawPixel(i, j, Montreal[i][j]);
      }
    }
  }

  if (chosen_lock_screen == 3){
    for (int i = 0; i < 320; i++){
      for (int j = 0; j < 240; j++){
        gfx->drawPixel(i, j, Salt_Lake_City[i][j]);
      }
    }
  }

  delay(1000);

  for (int i = 0; i < 306; i++){
    for (int j = 0; j < 77; j++){
      if (mdb_per[i][j] == 1)
        gfx->drawPixel(i+7, j+87, 0xf7de);
    }
  }
  
  gfx->setTextSize(2);
  gfx->setTextColor(0xf7de);
  disp_centered_text("Press Any Key", 205);
}

unsigned int k = 0;
void loop() {
  
  if (chosen_lock_screen == 0){
    for (int i = 0; i < 306; i++){
      for (int j = 0; j < 77; j++){
        if (mdb_icon[i][j] == 1)
          gfx->drawPixel(i+7, j+87, Dallas[(i + 7 + k)%320][j+87]);
      }
    }
  }

  if (chosen_lock_screen == 1){
    for (int i = 0; i < 306; i++){
      for (int j = 0; j < 77; j++){
        if (mdb_icon[i][j] == 1)
          gfx->drawPixel(i+7, j+87, Tel_Aviv[(i + 7 + k)%320][j+87]);
      }
    }
  }

  if (chosen_lock_screen == 2){
    for (int i = 0; i < 306; i++){
      for (int j = 0; j < 77; j++){
        if (mdb_icon[i][j] == 1)
          gfx->drawPixel(i+7, j+87, Montreal[(i + 7 + k)%320][j+87]);
      }
    }
  }

  if (chosen_lock_screen == 3){
    for (int i = 0; i < 306; i++){
      for (int j = 0; j < 77; j++){
        if (mdb_icon[i][j] == 1)
          gfx->drawPixel(i+7, j+87, Salt_Lake_City[(i + 7 + k)%320][j+87]);
      }
    }
  }
  
  k++;
}
